/*
 * ctags-vala.vala
 *
 * Copyright 2008 Abderrahim Kitouni <a.kitouni@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <vala.h>
#include <gee.h>
#include <options.h>
#include <stdio.h>
#include <gobject/gvaluecollector.h>


#define TYPE_CTAGS_ENTRY (ctags_entry_get_type ())
#define CTAGS_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CTAGS_ENTRY, CTagsEntry))
#define CTAGS_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CTAGS_ENTRY, CTagsEntryClass))
#define IS_CTAGS_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CTAGS_ENTRY))
#define IS_CTAGS_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CTAGS_ENTRY))
#define CTAGS_ENTRY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CTAGS_ENTRY, CTagsEntryClass))

typedef struct _CTagsEntry CTagsEntry;
typedef struct _CTagsEntryClass CTagsEntryClass;
typedef struct _CTagsEntryPrivate CTagsEntryPrivate;
typedef struct _ParamSpecCTagsEntry ParamSpecCTagsEntry;

#define TYPE_REPORT (report_get_type ())
#define REPORT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_REPORT, Report))
#define REPORT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_REPORT, ReportClass))
#define IS_REPORT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_REPORT))
#define IS_REPORT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_REPORT))
#define REPORT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_REPORT, ReportClass))

typedef struct _Report Report;
typedef struct _ReportClass ReportClass;
typedef struct _ReportPrivate ReportPrivate;

#define TYPE_CTAGS_VISITOR (ctags_visitor_get_type ())
#define CTAGS_VISITOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CTAGS_VISITOR, CTagsVisitor))
#define CTAGS_VISITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CTAGS_VISITOR, CTagsVisitorClass))
#define IS_CTAGS_VISITOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CTAGS_VISITOR))
#define IS_CTAGS_VISITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CTAGS_VISITOR))
#define CTAGS_VISITOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CTAGS_VISITOR, CTagsVisitorClass))

typedef struct _CTagsVisitor CTagsVisitor;
typedef struct _CTagsVisitorClass CTagsVisitorClass;
typedef struct _CTagsVisitorPrivate CTagsVisitorPrivate;

struct _CTagsEntry {
	GTypeInstance parent_instance;
	volatile int ref_count;
	CTagsEntryPrivate * priv;
	gint line_number;
	char* name;
	char* kind_name;
	gchar kind;
	char* access;
	char* implementation;
	char* inheritance;
	char* signature;
	char* typeref;
};

struct _CTagsEntryClass {
	GTypeClass parent_class;
	void (*finalize) (CTagsEntry *self);
};

struct _ParamSpecCTagsEntry {
	GParamSpec parent_instance;
};

typedef void (*CTagsEntryMaker) (CTagsEntry* entry);
struct _Report {
	ValaReport parent_instance;
	ReportPrivate * priv;
};

struct _ReportClass {
	ValaReportClass parent_class;
};

struct _CTagsVisitor {
	ValaCodeVisitor parent_instance;
	CTagsVisitorPrivate * priv;
};

struct _CTagsVisitorClass {
	ValaCodeVisitorClass parent_class;
};

struct _CTagsVisitorPrivate {
	ValaParser* vala_parser;
	ValaGenieParser* genie_parser;
	GList* taglist;
};


static gpointer ctags_entry_parent_class = NULL;
static gpointer report_parent_class = NULL;
static gpointer ctags_visitor_parent_class = NULL;

gpointer ctags_entry_ref (gpointer instance);
void ctags_entry_unref (gpointer instance);
GParamSpec* param_spec_ctags_entry (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_ctags_entry (GValue* value, gpointer v_object);
gpointer value_get_ctags_entry (const GValue* value);
GType ctags_entry_get_type (void);
enum  {
	CTAGS_ENTRY_DUMMY_PROPERTY
};
CTagsEntry* ctags_entry_new (void);
CTagsEntry* ctags_entry_construct (GType object_type);
static void ctags_entry_finalize (CTagsEntry* obj);
GType report_get_type (void);
enum  {
	REPORT_DUMMY_PROPERTY
};
static void report_real_err (ValaReport* base, ValaSourceReference* source, const char* message);
static void report_real_warn (ValaReport* base, ValaSourceReference* source, const char* message);
Report* report_new (void);
Report* report_construct (GType object_type);
GType ctags_visitor_get_type (void);
#define CTAGS_VISITOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_CTAGS_VISITOR, CTagsVisitorPrivate))
enum  {
	CTAGS_VISITOR_DUMMY_PROPERTY
};
static void _g_list_free_ctags_entry_unref (GList* self);
CTagsVisitor* ctags_visitor_new (void);
CTagsVisitor* ctags_visitor_construct (GType object_type);
static char* ctags_visitor_get_access (ValaSymbol* sym);
static char* ctags_visitor_to_string (GeeIterable* seq, const char* sep);
static void _g_list_free_g_free (GList* self);
static char* ctags_visitor_implementation (ValaSymbol* sym);
static char* ctags_visitor_name (ValaSymbol* sym);
static char* ctags_visitor_signature (GeeList* parameter);
static void ctags_visitor_print_tag (CTagsEntry* en);
static void ctags_visitor_real_visit_source_file (ValaCodeVisitor* base, ValaSourceFile* source_file);
static void ctags_visitor_real_visit_class (ValaCodeVisitor* base, ValaClass* cl);
static void ctags_visitor_real_visit_struct (ValaCodeVisitor* base, ValaStruct* st);
static void ctags_visitor_real_visit_interface (ValaCodeVisitor* base, ValaInterface* iface);
static void ctags_visitor_real_visit_enum (ValaCodeVisitor* base, ValaEnum* en);
static void ctags_visitor_real_visit_error_domain (ValaCodeVisitor* base, ValaErrorDomain* edomain);
static void ctags_visitor_real_visit_enum_value (ValaCodeVisitor* base, ValaEnumValue* ev);
static void ctags_visitor_real_visit_error_code (ValaCodeVisitor* base, ValaErrorCode* ecode);
static void ctags_visitor_real_visit_delegate (ValaCodeVisitor* base, ValaDelegate* d);
static void ctags_visitor_real_visit_signal (ValaCodeVisitor* base, ValaSignal* sig);
static void ctags_visitor_real_visit_field (ValaCodeVisitor* base, ValaField* f);
static void ctags_visitor_real_visit_constant (ValaCodeVisitor* base, ValaConstant* c);
static void ctags_visitor_real_visit_property (ValaCodeVisitor* base, ValaProperty* prop);
static void ctags_visitor_real_visit_method (ValaCodeVisitor* base, ValaMethod* m);
static void ctags_visitor_real_visit_namespace (ValaCodeVisitor* base, ValaNamespace* m);
static void ctags_visitor_real_visit_creation_method (ValaCodeVisitor* base, ValaCreationMethod* m);
static void ctags_visitor_real_visit_local_variable (ValaCodeVisitor* base, ValaLocalVariable* local);
void ctags_visitor_parse_vala (CTagsVisitor* self, const char* filename, CTagsEntryMaker maker);
void ctags_visitor_parse_genie (CTagsVisitor* self, const char* filename, CTagsEntryMaker maker);
static void ctags_visitor_finalize (ValaCodeVisitor* obj);



#line 24 "ctags-vala.vala"
CTagsEntry* ctags_entry_construct (GType object_type) {
#line 182 "ctags-vala.c"
	CTagsEntry* self;
	self = (CTagsEntry*) g_type_create_instance (object_type);
	return self;
}


#line 24 "ctags-vala.vala"
CTagsEntry* ctags_entry_new (void) {
#line 24 "ctags-vala.vala"
	return ctags_entry_construct (TYPE_CTAGS_ENTRY);
#line 193 "ctags-vala.c"
}


static void value_ctags_entry_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void value_ctags_entry_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		ctags_entry_unref (value->data[0].v_pointer);
	}
}


static void value_ctags_entry_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = ctags_entry_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer value_ctags_entry_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* value_ctags_entry_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		CTagsEntry* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = ctags_entry_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* value_ctags_entry_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	CTagsEntry** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = ctags_entry_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* param_spec_ctags_entry (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ParamSpecCTagsEntry* spec;
	g_return_val_if_fail (g_type_is_a (object_type, TYPE_CTAGS_ENTRY), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer value_get_ctags_entry (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_CTAGS_ENTRY), NULL);
	return value->data[0].v_pointer;
}


void value_set_ctags_entry (GValue* value, gpointer v_object) {
	CTagsEntry* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_CTAGS_ENTRY));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TYPE_CTAGS_ENTRY));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		ctags_entry_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		ctags_entry_unref (old);
	}
}


static void ctags_entry_class_init (CTagsEntryClass * klass) {
	ctags_entry_parent_class = g_type_class_peek_parent (klass);
	CTAGS_ENTRY_CLASS (klass)->finalize = ctags_entry_finalize;
}


static void ctags_entry_instance_init (CTagsEntry * self) {
	self->ref_count = 1;
}


static void ctags_entry_finalize (CTagsEntry* obj) {
	CTagsEntry * self;
	self = CTAGS_ENTRY (obj);
	self->name = (g_free (self->name), NULL);
	self->kind_name = (g_free (self->kind_name), NULL);
	self->access = (g_free (self->access), NULL);
	self->implementation = (g_free (self->implementation), NULL);
	self->inheritance = (g_free (self->inheritance), NULL);
	self->signature = (g_free (self->signature), NULL);
	self->typeref = (g_free (self->typeref), NULL);
}


GType ctags_entry_get_type (void) {
	static GType ctags_entry_type_id = 0;
	if (ctags_entry_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { value_ctags_entry_init, value_ctags_entry_free_value, value_ctags_entry_copy_value, value_ctags_entry_peek_pointer, "p", value_ctags_entry_collect_value, "p", value_ctags_entry_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (CTagsEntryClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) ctags_entry_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (CTagsEntry), 0, (GInstanceInitFunc) ctags_entry_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		ctags_entry_type_id = g_type_register_fundamental (g_type_fundamental_next (), "CTagsEntry", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return ctags_entry_type_id;
}


gpointer ctags_entry_ref (gpointer instance) {
	CTagsEntry* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void ctags_entry_unref (gpointer instance) {
	CTagsEntry* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		CTAGS_ENTRY_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


#line 40 "ctags-vala.vala"
static void report_real_err (ValaReport* base, ValaSourceReference* source, const char* message) {
#line 346 "ctags-vala.c"
	Report * self;
	self = (Report*) base;
#line 40 "ctags-vala.vala"
	g_return_if_fail (message != NULL);
#line 351 "ctags-vala.c"
}


#line 44 "ctags-vala.vala"
static void report_real_warn (ValaReport* base, ValaSourceReference* source, const char* message) {
#line 357 "ctags-vala.c"
	Report * self;
	self = (Report*) base;
#line 44 "ctags-vala.vala"
	g_return_if_fail (message != NULL);
#line 362 "ctags-vala.c"
}


#line 38 "ctags-vala.vala"
Report* report_construct (GType object_type) {
#line 368 "ctags-vala.c"
	Report * self;
	self = (Report*) vala_report_construct (object_type);
	return self;
}


#line 38 "ctags-vala.vala"
Report* report_new (void) {
#line 38 "ctags-vala.vala"
	return report_construct (TYPE_REPORT);
#line 379 "ctags-vala.c"
}


static void report_class_init (ReportClass * klass) {
	report_parent_class = g_type_class_peek_parent (klass);
	VALA_REPORT_CLASS (klass)->err = report_real_err;
	VALA_REPORT_CLASS (klass)->warn = report_real_warn;
}


static void report_instance_init (Report * self) {
}


GType report_get_type (void) {
	static GType report_type_id = 0;
	if (report_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ReportClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) report_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Report), 0, (GInstanceInitFunc) report_instance_init, NULL };
		report_type_id = g_type_register_static (VALA_TYPE_REPORT, "Report", &g_define_type_info, 0);
	}
	return report_type_id;
}


static void _g_list_free_ctags_entry_unref (GList* self) {
	g_list_foreach (self, (GFunc) ctags_entry_unref, NULL);
	g_list_free (self);
}


#line 54 "ctags-vala.vala"
CTagsVisitor* ctags_visitor_construct (GType object_type) {
#line 412 "ctags-vala.c"
	CTagsVisitor* self;
	ValaParser* _tmp0_;
	ValaGenieParser* _tmp1_;
	self = (CTagsVisitor*) g_type_create_instance (object_type);
	_tmp0_ = NULL;
#line 55 "ctags-vala.vala"
	self->priv->vala_parser = (_tmp0_ = vala_parser_new (), (self->priv->vala_parser == NULL) ? NULL : (self->priv->vala_parser = (vala_code_visitor_unref (self->priv->vala_parser), NULL)), _tmp0_);
#line 420 "ctags-vala.c"
	_tmp1_ = NULL;
#line 56 "ctags-vala.vala"
	self->priv->genie_parser = (_tmp1_ = vala_genie_parser_new (), (self->priv->genie_parser == NULL) ? NULL : (self->priv->genie_parser = (vala_code_visitor_unref (self->priv->genie_parser), NULL)), _tmp1_);
#line 424 "ctags-vala.c"
	return self;
}


#line 54 "ctags-vala.vala"
CTagsVisitor* ctags_visitor_new (void) {
#line 54 "ctags-vala.vala"
	return ctags_visitor_construct (TYPE_CTAGS_VISITOR);
#line 433 "ctags-vala.c"
}


/* helper functions */
#line 60 "ctags-vala.vala"
static char* ctags_visitor_get_access (ValaSymbol* sym) {
#line 440 "ctags-vala.c"
	char* result;
#line 60 "ctags-vala.vala"
	g_return_val_if_fail (sym != NULL, NULL);
#line 61 "ctags-vala.vala"
	switch (vala_symbol_get_access (sym)) {
#line 446 "ctags-vala.c"
		case VALA_SYMBOL_ACCESSIBILITY_PRIVATE:
		{
			result = g_strdup ("private");
			return result;
		}
		case VALA_SYMBOL_ACCESSIBILITY_INTERNAL:
		{
			result = g_strdup ("internal");
			return result;
		}
		case VALA_SYMBOL_ACCESSIBILITY_PROTECTED:
		{
			result = g_strdup ("protected");
			return result;
		}
		case VALA_SYMBOL_ACCESSIBILITY_PUBLIC:
		{
			result = g_strdup ("public");
			return result;
		}
	}
#line 67 "ctags-vala.vala"
	g_assert_not_reached ();
#line 470 "ctags-vala.c"
}


#line 70 "ctags-vala.vala"
static char* ctags_visitor_to_string (GeeIterable* seq, const char* sep) {
#line 476 "ctags-vala.c"
	char* result;
	GString* str;
	gboolean first;
	const char* _tmp1_;
#line 70 "ctags-vala.vala"
	g_return_val_if_fail (seq != NULL, NULL);
#line 70 "ctags-vala.vala"
	g_return_val_if_fail (sep != NULL, NULL);
#line 485 "ctags-vala.c"
	str = g_string_new ("");
	first = TRUE;
	{
		GeeIterator* _type_it;
		_type_it = gee_iterable_iterator (seq);
#line 73 "ctags-vala.vala"
		while (TRUE) {
#line 493 "ctags-vala.c"
			ValaDataType* type;
			char* _tmp0_;
#line 73 "ctags-vala.vala"
			if (!gee_iterator_next (_type_it)) {
#line 73 "ctags-vala.vala"
				break;
#line 500 "ctags-vala.c"
			}
			type = (ValaDataType*) gee_iterator_get (_type_it);
#line 74 "ctags-vala.vala"
			if (first) {
#line 75 "ctags-vala.vala"
				first = FALSE;
#line 507 "ctags-vala.c"
			} else {
#line 77 "ctags-vala.vala"
				g_string_append (str, sep);
#line 511 "ctags-vala.c"
			}
#line 1015 "vala-1.0.vapi"
			_tmp0_ = NULL;
#line 79 "ctags-vala.vala"
			g_string_append (str, _tmp0_ = vala_code_node_to_string ((ValaCodeNode*) type));
#line 517 "ctags-vala.c"
			_tmp0_ = (g_free (_tmp0_), NULL);
			(type == NULL) ? NULL : (type = (vala_code_node_unref (type), NULL));
		}
		(_type_it == NULL) ? NULL : (_type_it = (gee_collection_object_unref (_type_it), NULL));
	}
#line 81 "ctags-vala.vala"
	_tmp1_ = NULL;
#line 525 "ctags-vala.c"
	result = (_tmp1_ = str->str, (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_));
	(str == NULL) ? NULL : (str = (g_string_free (str, TRUE), NULL));
	return result;
	(str == NULL) ? NULL : (str = (g_string_free (str, TRUE), NULL));
}


static void _g_list_free_g_free (GList* self) {
	g_list_foreach (self, (GFunc) g_free, NULL);
	g_list_free (self);
}


#line 84 "ctags-vala.vala"
static char* ctags_visitor_implementation (ValaSymbol* sym) {
#line 541 "ctags-vala.c"
	char* result;
	gboolean _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	gboolean _tmp4_;
	gboolean _tmp5_;
	gboolean _tmp6_;
	GList* list;
	GString* ret;
	gboolean first;
	const char* _tmp8_;
#line 84 "ctags-vala.vala"
	g_return_val_if_fail (sym != NULL, NULL);
#line 556 "ctags-vala.c"
	_tmp0_ = FALSE;
	_tmp1_ = FALSE;
	_tmp2_ = FALSE;
	_tmp3_ = FALSE;
	_tmp4_ = FALSE;
	_tmp5_ = FALSE;
	_tmp6_ = FALSE;
#line 85 "ctags-vala.vala"
	if (VALA_IS_CLASS (sym)) {
#line 85 "ctags-vala.vala"
		_tmp6_ = TRUE;
#line 568 "ctags-vala.c"
	} else {
#line 85 "ctags-vala.vala"
		_tmp6_ = VALA_IS_SIGNAL (sym);
#line 572 "ctags-vala.c"
	}
#line 85 "ctags-vala.vala"
	if (_tmp6_) {
#line 85 "ctags-vala.vala"
		_tmp5_ = TRUE;
#line 578 "ctags-vala.c"
	} else {
#line 85 "ctags-vala.vala"
		_tmp5_ = VALA_IS_PROPERTY (sym);
#line 582 "ctags-vala.c"
	}
#line 85 "ctags-vala.vala"
	if (_tmp5_) {
#line 85 "ctags-vala.vala"
		_tmp4_ = TRUE;
#line 588 "ctags-vala.c"
	} else {
#line 86 "ctags-vala.vala"
		_tmp4_ = VALA_IS_METHOD (sym);
#line 592 "ctags-vala.c"
	}
#line 85 "ctags-vala.vala"
	if (_tmp4_) {
#line 85 "ctags-vala.vala"
		_tmp3_ = TRUE;
#line 598 "ctags-vala.c"
	} else {
#line 86 "ctags-vala.vala"
		_tmp3_ = VALA_IS_DELEGATE (sym);
#line 602 "ctags-vala.c"
	}
#line 85 "ctags-vala.vala"
	if (_tmp3_) {
#line 85 "ctags-vala.vala"
		_tmp2_ = TRUE;
#line 608 "ctags-vala.c"
	} else {
#line 86 "ctags-vala.vala"
		_tmp2_ = VALA_IS_CONSTRUCTOR (sym);
#line 612 "ctags-vala.c"
	}
#line 85 "ctags-vala.vala"
	if (_tmp2_) {
#line 85 "ctags-vala.vala"
		_tmp1_ = TRUE;
#line 618 "ctags-vala.c"
	} else {
#line 87 "ctags-vala.vala"
		_tmp1_ = VALA_IS_DESTRUCTOR (sym);
#line 622 "ctags-vala.c"
	}
#line 85 "ctags-vala.vala"
	if (_tmp1_) {
#line 85 "ctags-vala.vala"
		_tmp0_ = TRUE;
#line 628 "ctags-vala.c"
	} else {
#line 87 "ctags-vala.vala"
		_tmp0_ = VALA_IS_NAMESPACE (sym);
#line 632 "ctags-vala.c"
	}
#line 85 "ctags-vala.vala"
	g_assert (_tmp0_);
#line 636 "ctags-vala.c"
	list = NULL;
	/*
	if (!(sym is Vala.Signal) && (bool)sym.is_abstract) {
	list.append("abstract");
	}
	if (!(sym is Class) && (bool)sym.is_virtual) {
	list.append("virtual");
	}
	if (list == null) {
	return null;
	}
	*/
	ret = g_string_new ("");
	first = TRUE;
	{
		GList* str_collection;
		GList* str_it;
#line 102 "ctags-vala.vala"
		str_collection = list;
#line 656 "ctags-vala.c"
		for (str_it = str_collection; str_it != NULL; str_it = str_it->next) {
			const char* _tmp7_;
			char* str;
#line 88 "ctags-vala.vala"
			_tmp7_ = NULL;
#line 102 "ctags-vala.vala"
			str = (_tmp7_ = (const char*) str_it->data, (_tmp7_ == NULL) ? NULL : g_strdup (_tmp7_));
#line 664 "ctags-vala.c"
			{
#line 103 "ctags-vala.vala"
				if (first) {
#line 104 "ctags-vala.vala"
					first = FALSE;
#line 670 "ctags-vala.c"
				} else {
#line 106 "ctags-vala.vala"
					g_string_append (ret, ",");
#line 674 "ctags-vala.c"
				}
#line 108 "ctags-vala.vala"
				g_string_append (ret, str);
#line 678 "ctags-vala.c"
				str = (g_free (str), NULL);
			}
		}
	}
#line 110 "ctags-vala.vala"
	_tmp8_ = NULL;
#line 685 "ctags-vala.c"
	result = (_tmp8_ = ret->str, (_tmp8_ == NULL) ? NULL : g_strdup (_tmp8_));
	(list == NULL) ? NULL : (list = (_g_list_free_g_free (list), NULL));
	(ret == NULL) ? NULL : (ret = (g_string_free (ret, TRUE), NULL));
	return result;
	(list == NULL) ? NULL : (list = (_g_list_free_g_free (list), NULL));
	(ret == NULL) ? NULL : (ret = (g_string_free (ret, TRUE), NULL));
}


#line 113 "ctags-vala.vala"
static char* ctags_visitor_name (ValaSymbol* sym) {
#line 697 "ctags-vala.c"
	char* result;
	char* name;
#line 113 "ctags-vala.vala"
	g_return_val_if_fail (sym != NULL, NULL);
#line 702 "ctags-vala.c"
	name = NULL;
#line 116 "ctags-vala.vala"
	if (Option.include.qualifiedTags) {
#line 706 "ctags-vala.c"
		char* _tmp0_;
		_tmp0_ = NULL;
#line 117 "ctags-vala.vala"
		name = (_tmp0_ = vala_symbol_get_full_name (sym), name = (g_free (name), NULL), _tmp0_);
#line 711 "ctags-vala.c"
	} else {
#line 119 "ctags-vala.vala"
		if (VALA_IS_CONSTRUCTOR (sym)) {
#line 715 "ctags-vala.c"
			char* _tmp1_;
			_tmp1_ = NULL;
#line 120 "ctags-vala.vala"
			name = (_tmp1_ = g_strdup ("construct"), name = (g_free (name), NULL), _tmp1_);
#line 720 "ctags-vala.c"
		} else {
#line 121 "ctags-vala.vala"
			if (VALA_IS_DESTRUCTOR (sym)) {
#line 724 "ctags-vala.c"
				char* _tmp3_;
				char* _tmp2_;
				_tmp3_ = NULL;
#line 1015 "vala-1.0.vapi"
				_tmp2_ = NULL;
#line 122 "ctags-vala.vala"
				name = (_tmp3_ = g_strconcat ("~", _tmp2_ = vala_code_node_to_string ((ValaCodeNode*) vala_formal_parameter_get_parameter_type (vala_destructor_get_this_parameter (VALA_DESTRUCTOR (sym)))), NULL), name = (g_free (name), NULL), _tmp3_);
#line 732 "ctags-vala.c"
				_tmp2_ = (g_free (_tmp2_), NULL);
			} else {
				char* _tmp5_;
				const char* _tmp4_;
				_tmp5_ = NULL;
#line 124 "ctags-vala.vala"
				_tmp4_ = NULL;
#line 124 "ctags-vala.vala"
				name = (_tmp5_ = (_tmp4_ = vala_symbol_get_name (sym), (_tmp4_ == NULL) ? NULL : g_strdup (_tmp4_)), name = (g_free (name), NULL), _tmp5_);
#line 742 "ctags-vala.c"
			}
		}
	}
	result = name;
	return result;
}


#line 130 "ctags-vala.vala"
static char* ctags_visitor_signature (GeeList* parameter) {
#line 753 "ctags-vala.c"
	char* result;
	GString* ret;
	gboolean first;
	const char* _tmp1_;
#line 130 "ctags-vala.vala"
	g_return_val_if_fail (parameter != NULL, NULL);
#line 760 "ctags-vala.c"
	ret = g_string_new ("(");
	first = TRUE;
	{
		GeeIterator* _p_it;
		_p_it = gee_iterable_iterator ((GeeIterable*) parameter);
#line 133 "ctags-vala.vala"
		while (TRUE) {
#line 768 "ctags-vala.c"
			ValaFormalParameter* p;
#line 133 "ctags-vala.vala"
			if (!gee_iterator_next (_p_it)) {
#line 133 "ctags-vala.vala"
				break;
#line 774 "ctags-vala.c"
			}
			p = (ValaFormalParameter*) gee_iterator_get (_p_it);
#line 134 "ctags-vala.vala"
			if (first) {
#line 135 "ctags-vala.vala"
				first = FALSE;
#line 781 "ctags-vala.c"
			} else {
#line 137 "ctags-vala.vala"
				g_string_append (ret, ",");
#line 785 "ctags-vala.c"
			}
#line 139 "ctags-vala.vala"
			if (vala_formal_parameter_get_ellipsis (p)) {
#line 140 "ctags-vala.vala"
				g_string_append (ret, "...");
#line 791 "ctags-vala.c"
			} else {
				char* _tmp0_;
#line 1251 "vala-1.0.vapi"
				_tmp0_ = NULL;
#line 142 "ctags-vala.vala"
				g_string_append (ret, _tmp0_ = vala_data_type_to_qualified_string (vala_formal_parameter_get_parameter_type (p), NULL));
#line 798 "ctags-vala.c"
				_tmp0_ = (g_free (_tmp0_), NULL);
#line 143 "ctags-vala.vala"
				g_string_append (ret, " ");
#line 144 "ctags-vala.vala"
				g_string_append (ret, vala_symbol_get_name ((ValaSymbol*) p));
#line 804 "ctags-vala.c"
			}
			(p == NULL) ? NULL : (p = (vala_code_node_unref (p), NULL));
		}
		(_p_it == NULL) ? NULL : (_p_it = (gee_collection_object_unref (_p_it), NULL));
	}
#line 147 "ctags-vala.vala"
	g_string_append (ret, ")");
#line 148 "ctags-vala.vala"
	_tmp1_ = NULL;
#line 814 "ctags-vala.c"
	result = (_tmp1_ = ret->str, (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_));
	(ret == NULL) ? NULL : (ret = (g_string_free (ret, TRUE), NULL));
	return result;
	(ret == NULL) ? NULL : (ret = (g_string_free (ret, TRUE), NULL));
}


#line 151 "ctags-vala.vala"
static void ctags_visitor_print_tag (CTagsEntry* en) {
#line 151 "ctags-vala.vala"
	g_return_if_fail (en != NULL);
#line 152 "ctags-vala.vala"
	fprintf (stdout, "%s: %s at %d\n", en->name, en->kind_name, en->line_number);
#line 828 "ctags-vala.c"
}


#line 155 "ctags-vala.vala"
static void ctags_visitor_real_visit_source_file (ValaCodeVisitor* base, ValaSourceFile* source_file) {
#line 834 "ctags-vala.c"
	CTagsVisitor * self;
	self = (CTagsVisitor*) base;
#line 155 "ctags-vala.vala"
	g_return_if_fail (source_file != NULL);
#line 156 "ctags-vala.vala"
	vala_source_file_accept_children (source_file, (ValaCodeVisitor*) self);
#line 841 "ctags-vala.c"
}


#line 159 "ctags-vala.vala"
static void ctags_visitor_real_visit_class (ValaCodeVisitor* base, ValaClass* cl) {
#line 847 "ctags-vala.c"
	CTagsVisitor * self;
	CTagsEntry* entry;
	char* _tmp0_;
	char* _tmp1_;
	char* _tmp2_;
	char* _tmp3_;
	CTagsEntry* _tmp7_;
	self = (CTagsVisitor*) base;
#line 159 "ctags-vala.vala"
	g_return_if_fail (cl != NULL);
#line 858 "ctags-vala.c"
	entry = ctags_entry_new ();
#line 162 "ctags-vala.vala"
	entry->line_number = vala_source_reference_get_first_line (vala_code_node_get_source_reference ((ValaCodeNode*) cl));
#line 862 "ctags-vala.c"
	_tmp0_ = NULL;
#line 163 "ctags-vala.vala"
	entry->name = (_tmp0_ = ctags_visitor_name ((ValaSymbol*) cl), entry->name = (g_free (entry->name), NULL), _tmp0_);
#line 866 "ctags-vala.c"
	_tmp1_ = NULL;
#line 164 "ctags-vala.vala"
	entry->kind_name = (_tmp1_ = g_strdup ("class"), entry->kind_name = (g_free (entry->kind_name), NULL), _tmp1_);
#line 165 "ctags-vala.vala"
	entry->kind = 'c';
#line 872 "ctags-vala.c"
	_tmp2_ = NULL;
#line 166 "ctags-vala.vala"
	entry->access = (_tmp2_ = ctags_visitor_get_access ((ValaSymbol*) cl), entry->access = (g_free (entry->access), NULL), _tmp2_);
#line 876 "ctags-vala.c"
	_tmp3_ = NULL;
#line 167 "ctags-vala.vala"
	entry->implementation = (_tmp3_ = ctags_visitor_implementation ((ValaSymbol*) cl), entry->implementation = (g_free (entry->implementation), NULL), _tmp3_);
#line 168 "ctags-vala.vala"
	if (vala_class_get_base_class (cl) != NULL) {
#line 882 "ctags-vala.c"
		char* _tmp4_;
		_tmp4_ = NULL;
#line 170 "ctags-vala.vala"
		entry->inheritance = (_tmp4_ = ctags_visitor_name ((ValaSymbol*) vala_class_get_base_class (cl)), entry->inheritance = (g_free (entry->inheritance), NULL), _tmp4_);
#line 887 "ctags-vala.c"
	} else {
		char* _tmp6_;
		GeeList* _tmp5_;
		_tmp6_ = NULL;
#line 882 "vala-1.0.vapi"
		_tmp5_ = NULL;
#line 174 "ctags-vala.vala"
		entry->inheritance = (_tmp6_ = ctags_visitor_to_string ((GeeIterable*) (_tmp5_ = vala_class_get_base_types (cl)), ","), entry->inheritance = (g_free (entry->inheritance), NULL), _tmp6_);
#line 896 "ctags-vala.c"
		(_tmp5_ == NULL) ? NULL : (_tmp5_ = (gee_collection_object_unref (_tmp5_), NULL));
	}
#line 177 "ctags-vala.vala"
	_tmp7_ = NULL;
#line 177 "ctags-vala.vala"
	self->priv->taglist = g_list_append (self->priv->taglist, (_tmp7_ = entry, (_tmp7_ == NULL) ? NULL : ctags_entry_ref (_tmp7_)));
#line 903 "ctags-vala.c"
	/*print_tag(entry);*/
#line 179 "ctags-vala.vala"
	vala_code_node_accept_children ((ValaCodeNode*) cl, (ValaCodeVisitor*) self);
#line 907 "ctags-vala.c"
	(entry == NULL) ? NULL : (entry = (ctags_entry_unref (entry), NULL));
}


#line 182 "ctags-vala.vala"
static void ctags_visitor_real_visit_struct (ValaCodeVisitor* base, ValaStruct* st) {
#line 914 "ctags-vala.c"
	CTagsVisitor * self;
	CTagsEntry* entry;
	char* _tmp0_;
	char* _tmp1_;
	char* _tmp2_;
	CTagsEntry* _tmp3_;
	self = (CTagsVisitor*) base;
#line 182 "ctags-vala.vala"
	g_return_if_fail (st != NULL);
#line 924 "ctags-vala.c"
	entry = ctags_entry_new ();
#line 184 "ctags-vala.vala"
	entry->line_number = vala_source_reference_get_first_line (vala_code_node_get_source_reference ((ValaCodeNode*) st));
#line 928 "ctags-vala.c"
	_tmp0_ = NULL;
#line 185 "ctags-vala.vala"
	entry->name = (_tmp0_ = ctags_visitor_name ((ValaSymbol*) st), entry->name = (g_free (entry->name), NULL), _tmp0_);
#line 932 "ctags-vala.c"
	_tmp1_ = NULL;
#line 186 "ctags-vala.vala"
	entry->kind_name = (_tmp1_ = g_strdup ("struct"), entry->kind_name = (g_free (entry->kind_name), NULL), _tmp1_);
#line 187 "ctags-vala.vala"
	entry->kind = 's';
#line 938 "ctags-vala.c"
	_tmp2_ = NULL;
#line 188 "ctags-vala.vala"
	entry->access = (_tmp2_ = ctags_visitor_get_access ((ValaSymbol*) st), entry->access = (g_free (entry->access), NULL), _tmp2_);
#line 190 "ctags-vala.vala"
	_tmp3_ = NULL;
#line 190 "ctags-vala.vala"
	self->priv->taglist = g_list_append (self->priv->taglist, (_tmp3_ = entry, (_tmp3_ == NULL) ? NULL : ctags_entry_ref (_tmp3_)));
#line 946 "ctags-vala.c"
	/*print_tag(entry);*/
#line 192 "ctags-vala.vala"
	vala_code_node_accept_children ((ValaCodeNode*) st, (ValaCodeVisitor*) self);
#line 950 "ctags-vala.c"
	(entry == NULL) ? NULL : (entry = (ctags_entry_unref (entry), NULL));
}


#line 195 "ctags-vala.vala"
static void ctags_visitor_real_visit_interface (ValaCodeVisitor* base, ValaInterface* iface) {
#line 957 "ctags-vala.c"
	CTagsVisitor * self;
	CTagsEntry* entry;
	char* _tmp0_;
	char* _tmp1_;
	char* _tmp2_;
	char* _tmp4_;
	GeeList* _tmp3_;
	CTagsEntry* _tmp5_;
	self = (CTagsVisitor*) base;
#line 195 "ctags-vala.vala"
	g_return_if_fail (iface != NULL);
#line 969 "ctags-vala.c"
	entry = ctags_entry_new ();
#line 198 "ctags-vala.vala"
	entry->line_number = vala_source_reference_get_first_line (vala_code_node_get_source_reference ((ValaCodeNode*) iface));
#line 973 "ctags-vala.c"
	_tmp0_ = NULL;
#line 199 "ctags-vala.vala"
	entry->name = (_tmp0_ = ctags_visitor_name ((ValaSymbol*) iface), entry->name = (g_free (entry->name), NULL), _tmp0_);
#line 977 "ctags-vala.c"
	_tmp1_ = NULL;
#line 200 "ctags-vala.vala"
	entry->kind_name = (_tmp1_ = g_strdup ("interface"), entry->kind_name = (g_free (entry->kind_name), NULL), _tmp1_);
#line 201 "ctags-vala.vala"
	entry->kind = 'i';
#line 983 "ctags-vala.c"
	_tmp2_ = NULL;
#line 202 "ctags-vala.vala"
	entry->access = (_tmp2_ = ctags_visitor_get_access ((ValaSymbol*) iface), entry->access = (g_free (entry->access), NULL), _tmp2_);
#line 987 "ctags-vala.c"
	_tmp4_ = NULL;
#line 1714 "vala-1.0.vapi"
	_tmp3_ = NULL;
#line 203 "ctags-vala.vala"
	entry->inheritance = (_tmp4_ = ctags_visitor_to_string ((GeeIterable*) (_tmp3_ = vala_interface_get_prerequisites (iface)), ","), entry->inheritance = (g_free (entry->inheritance), NULL), _tmp4_);
#line 993 "ctags-vala.c"
	(_tmp3_ == NULL) ? NULL : (_tmp3_ = (gee_collection_object_unref (_tmp3_), NULL));
#line 205 "ctags-vala.vala"
	_tmp5_ = NULL;
#line 205 "ctags-vala.vala"
	self->priv->taglist = g_list_append (self->priv->taglist, (_tmp5_ = entry, (_tmp5_ == NULL) ? NULL : ctags_entry_ref (_tmp5_)));
#line 999 "ctags-vala.c"
	/*print_tag(entry);*/
#line 207 "ctags-vala.vala"
	vala_code_node_accept_children ((ValaCodeNode*) iface, (ValaCodeVisitor*) self);
#line 1003 "ctags-vala.c"
	(entry == NULL) ? NULL : (entry = (ctags_entry_unref (entry), NULL));
}


#line 210 "ctags-vala.vala"
static void ctags_visitor_real_visit_enum (ValaCodeVisitor* base, ValaEnum* en) {
#line 1010 "ctags-vala.c"
	CTagsVisitor * self;
	CTagsEntry* entry;
	char* _tmp0_;
	char* _tmp1_;
	char* _tmp2_;
	CTagsEntry* _tmp3_;
	self = (CTagsVisitor*) base;
#line 210 "ctags-vala.vala"
	g_return_if_fail (en != NULL);
#line 1020 "ctags-vala.c"
	entry = ctags_entry_new ();
#line 213 "ctags-vala.vala"
	entry->line_number = vala_source_reference_get_first_line (vala_code_node_get_source_reference ((ValaCodeNode*) en));
#line 1024 "ctags-vala.c"
	_tmp0_ = NULL;
#line 214 "ctags-vala.vala"
	entry->name = (_tmp0_ = ctags_visitor_name ((ValaSymbol*) en), entry->name = (g_free (entry->name), NULL), _tmp0_);
#line 1028 "ctags-vala.c"
	_tmp1_ = NULL;
#line 215 "ctags-vala.vala"
	entry->kind_name = (_tmp1_ = g_strdup ("enum"), entry->kind_name = (g_free (entry->kind_name), NULL), _tmp1_);
#line 216 "ctags-vala.vala"
	entry->kind = 'e';
#line 1034 "ctags-vala.c"
	_tmp2_ = NULL;
#line 217 "ctags-vala.vala"
	entry->access = (_tmp2_ = ctags_visitor_get_access ((ValaSymbol*) en), entry->access = (g_free (entry->access), NULL), _tmp2_);
#line 219 "ctags-vala.vala"
	_tmp3_ = NULL;
#line 219 "ctags-vala.vala"
	self->priv->taglist = g_list_append (self->priv->taglist, (_tmp3_ = entry, (_tmp3_ == NULL) ? NULL : ctags_entry_ref (_tmp3_)));
#line 1042 "ctags-vala.c"
	/*print_tag(entry);*/
#line 221 "ctags-vala.vala"
	vala_code_node_accept_children ((ValaCodeNode*) en, (ValaCodeVisitor*) self);
#line 1046 "ctags-vala.c"
	(entry == NULL) ? NULL : (entry = (ctags_entry_unref (entry), NULL));
}


#line 224 "ctags-vala.vala"
static void ctags_visitor_real_visit_error_domain (ValaCodeVisitor* base, ValaErrorDomain* edomain) {
#line 1053 "ctags-vala.c"
	CTagsVisitor * self;
	CTagsEntry* entry;
	char* _tmp0_;
	char* _tmp1_;
	char* _tmp2_;
	CTagsEntry* _tmp3_;
	self = (CTagsVisitor*) base;
#line 224 "ctags-vala.vala"
	g_return_if_fail (edomain != NULL);
#line 1063 "ctags-vala.c"
	entry = ctags_entry_new ();
#line 227 "ctags-vala.vala"
	entry->line_number = vala_source_reference_get_first_line (vala_code_node_get_source_reference ((ValaCodeNode*) edomain));
#line 1067 "ctags-vala.c"
	_tmp0_ = NULL;
#line 228 "ctags-vala.vala"
	entry->name = (_tmp0_ = ctags_visitor_name ((ValaSymbol*) edomain), entry->name = (g_free (entry->name), NULL), _tmp0_);
#line 1071 "ctags-vala.c"
	_tmp1_ = NULL;
#line 229 "ctags-vala.vala"
	entry->kind_name = (_tmp1_ = g_strdup ("errordomain"), entry->kind_name = (g_free (entry->kind_name), NULL), _tmp1_);
#line 230 "ctags-vala.vala"
	entry->kind = 'E';
#line 1077 "ctags-vala.c"
	_tmp2_ = NULL;
#line 231 "ctags-vala.vala"
	entry->access = (_tmp2_ = ctags_visitor_get_access ((ValaSymbol*) edomain), entry->access = (g_free (entry->access), NULL), _tmp2_);
#line 233 "ctags-vala.vala"
	_tmp3_ = NULL;
#line 233 "ctags-vala.vala"
	self->priv->taglist = g_list_append (self->priv->taglist, (_tmp3_ = entry, (_tmp3_ == NULL) ? NULL : ctags_entry_ref (_tmp3_)));
#line 1085 "ctags-vala.c"
	/*print_tag(entry);*/
#line 235 "ctags-vala.vala"
	vala_code_node_accept_children ((ValaCodeNode*) edomain, (ValaCodeVisitor*) self);
#line 1089 "ctags-vala.c"
	(entry == NULL) ? NULL : (entry = (ctags_entry_unref (entry), NULL));
}


#line 238 "ctags-vala.vala"
static void ctags_visitor_real_visit_enum_value (ValaCodeVisitor* base, ValaEnumValue* ev) {
#line 1096 "ctags-vala.c"
	CTagsVisitor * self;
	CTagsEntry* entry;
	char* _tmp0_;
	char* _tmp1_;
	char* _tmp2_;
	CTagsEntry* _tmp3_;
	self = (CTagsVisitor*) base;
#line 238 "ctags-vala.vala"
	g_return_if_fail (ev != NULL);
#line 1106 "ctags-vala.c"
	entry = ctags_entry_new ();
#line 241 "ctags-vala.vala"
	entry->line_number = vala_source_reference_get_first_line (vala_code_node_get_source_reference ((ValaCodeNode*) ev));
#line 1110 "ctags-vala.c"
	_tmp0_ = NULL;
#line 242 "ctags-vala.vala"
	entry->name = (_tmp0_ = ctags_visitor_name ((ValaSymbol*) ev), entry->name = (g_free (entry->name), NULL), _tmp0_);
#line 1114 "ctags-vala.c"
	_tmp1_ = NULL;
#line 243 "ctags-vala.vala"
	entry->kind_name = (_tmp1_ = g_strdup ("enumvalue"), entry->kind_name = (g_free (entry->kind_name), NULL), _tmp1_);
#line 244 "ctags-vala.vala"
	entry->kind = 'v';
#line 1120 "ctags-vala.c"
	_tmp2_ = NULL;
#line 245 "ctags-vala.vala"
	entry->access = (_tmp2_ = ctags_visitor_get_access ((ValaSymbol*) ev), entry->access = (g_free (entry->access), NULL), _tmp2_);
#line 247 "ctags-vala.vala"
	_tmp3_ = NULL;
#line 247 "ctags-vala.vala"
	self->priv->taglist = g_list_append (self->priv->taglist, (_tmp3_ = entry, (_tmp3_ == NULL) ? NULL : ctags_entry_ref (_tmp3_)));
#line 1128 "ctags-vala.c"
	(entry == NULL) ? NULL : (entry = (ctags_entry_unref (entry), NULL));
}


/*print_tag(entry);*/
#line 251 "ctags-vala.vala"
static void ctags_visitor_real_visit_error_code (ValaCodeVisitor* base, ValaErrorCode* ecode) {
#line 1136 "ctags-vala.c"
	CTagsVisitor * self;
	CTagsEntry* entry;
	char* _tmp0_;
	char* _tmp1_;
	char* _tmp2_;
	CTagsEntry* _tmp3_;
	self = (CTagsVisitor*) base;
#line 251 "ctags-vala.vala"
	g_return_if_fail (ecode != NULL);
#line 1146 "ctags-vala.c"
	entry = ctags_entry_new ();
#line 254 "ctags-vala.vala"
	entry->line_number = vala_source_reference_get_first_line (vala_code_node_get_source_reference ((ValaCodeNode*) ecode));
#line 1150 "ctags-vala.c"
	_tmp0_ = NULL;
#line 255 "ctags-vala.vala"
	entry->name = (_tmp0_ = ctags_visitor_name ((ValaSymbol*) ecode), entry->name = (g_free (entry->name), NULL), _tmp0_);
#line 1154 "ctags-vala.c"
	_tmp1_ = NULL;
#line 256 "ctags-vala.vala"
	entry->kind_name = (_tmp1_ = g_strdup ("errorcode"), entry->kind_name = (g_free (entry->kind_name), NULL), _tmp1_);
#line 257 "ctags-vala.vala"
	entry->kind = 'r';
#line 1160 "ctags-vala.c"
	_tmp2_ = NULL;
#line 258 "ctags-vala.vala"
	entry->access = (_tmp2_ = ctags_visitor_get_access ((ValaSymbol*) ecode), entry->access = (g_free (entry->access), NULL), _tmp2_);
#line 260 "ctags-vala.vala"
	_tmp3_ = NULL;
#line 260 "ctags-vala.vala"
	self->priv->taglist = g_list_append (self->priv->taglist, (_tmp3_ = entry, (_tmp3_ == NULL) ? NULL : ctags_entry_ref (_tmp3_)));
#line 1168 "ctags-vala.c"
	(entry == NULL) ? NULL : (entry = (ctags_entry_unref (entry), NULL));
}


/*print_tag(entry);*/
#line 264 "ctags-vala.vala"
static void ctags_visitor_real_visit_delegate (ValaCodeVisitor* base, ValaDelegate* d) {
#line 1176 "ctags-vala.c"
	CTagsVisitor * self;
	CTagsEntry* entry;
	char* _tmp0_;
	char* _tmp1_;
	char* _tmp2_;
	char* _tmp3_;
	char* _tmp4_;
	char* _tmp6_;
	GeeList* _tmp5_;
	CTagsEntry* _tmp7_;
	self = (CTagsVisitor*) base;
#line 264 "ctags-vala.vala"
	g_return_if_fail (d != NULL);
#line 1190 "ctags-vala.c"
	entry = ctags_entry_new ();
#line 267 "ctags-vala.vala"
	entry->line_number = vala_source_reference_get_first_line (vala_code_node_get_source_reference ((ValaCodeNode*) d));
#line 1194 "ctags-vala.c"
	_tmp0_ = NULL;
#line 268 "ctags-vala.vala"
	entry->name = (_tmp0_ = ctags_visitor_name ((ValaSymbol*) d), entry->name = (g_free (entry->name), NULL), _tmp0_);
#line 1198 "ctags-vala.c"
	_tmp1_ = NULL;
#line 269 "ctags-vala.vala"
	entry->kind_name = (_tmp1_ = g_strdup ("delegate"), entry->kind_name = (g_free (entry->kind_name), NULL), _tmp1_);
#line 270 "ctags-vala.vala"
	entry->kind = 'd';
#line 1204 "ctags-vala.c"
	_tmp2_ = NULL;
#line 271 "ctags-vala.vala"
	entry->access = (_tmp2_ = ctags_visitor_get_access ((ValaSymbol*) d), entry->access = (g_free (entry->access), NULL), _tmp2_);
#line 1208 "ctags-vala.c"
	_tmp3_ = NULL;
#line 272 "ctags-vala.vala"
	entry->implementation = (_tmp3_ = ctags_visitor_implementation ((ValaSymbol*) d), entry->implementation = (g_free (entry->implementation), NULL), _tmp3_);
#line 1212 "ctags-vala.c"
	_tmp4_ = NULL;
#line 273 "ctags-vala.vala"
	entry->typeref = (_tmp4_ = vala_data_type_to_qualified_string (vala_delegate_get_return_type (d), NULL), entry->typeref = (g_free (entry->typeref), NULL), _tmp4_);
#line 1216 "ctags-vala.c"
	_tmp6_ = NULL;
#line 1280 "vala-1.0.vapi"
	_tmp5_ = NULL;
#line 274 "ctags-vala.vala"
	entry->signature = (_tmp6_ = ctags_visitor_signature (_tmp5_ = vala_delegate_get_parameters (d)), entry->signature = (g_free (entry->signature), NULL), _tmp6_);
#line 1222 "ctags-vala.c"
	(_tmp5_ == NULL) ? NULL : (_tmp5_ = (gee_collection_object_unref (_tmp5_), NULL));
#line 276 "ctags-vala.vala"
	_tmp7_ = NULL;
#line 276 "ctags-vala.vala"
	self->priv->taglist = g_list_append (self->priv->taglist, (_tmp7_ = entry, (_tmp7_ == NULL) ? NULL : ctags_entry_ref (_tmp7_)));
#line 1228 "ctags-vala.c"
	(entry == NULL) ? NULL : (entry = (ctags_entry_unref (entry), NULL));
}


/*print_tag(entry);*/
#line 280 "ctags-vala.vala"
static void ctags_visitor_real_visit_signal (ValaCodeVisitor* base, ValaSignal* sig) {
#line 1236 "ctags-vala.c"
	CTagsVisitor * self;
	CTagsEntry* entry;
	char* _tmp0_;
	char* _tmp1_;
	char* _tmp2_;
	char* _tmp3_;
	char* _tmp4_;
	char* _tmp6_;
	GeeList* _tmp5_;
	CTagsEntry* _tmp7_;
	self = (CTagsVisitor*) base;
#line 280 "ctags-vala.vala"
	g_return_if_fail (sig != NULL);
#line 1250 "ctags-vala.c"
	entry = ctags_entry_new ();
#line 283 "ctags-vala.vala"
	entry->line_number = vala_source_reference_get_first_line (vala_code_node_get_source_reference ((ValaCodeNode*) sig));
#line 1254 "ctags-vala.c"
	_tmp0_ = NULL;
#line 284 "ctags-vala.vala"
	entry->name = (_tmp0_ = ctags_visitor_name ((ValaSymbol*) sig), entry->name = (g_free (entry->name), NULL), _tmp0_);
#line 1258 "ctags-vala.c"
	_tmp1_ = NULL;
#line 285 "ctags-vala.vala"
	entry->kind_name = (_tmp1_ = g_strdup ("signal"), entry->kind_name = (g_free (entry->kind_name), NULL), _tmp1_);
#line 286 "ctags-vala.vala"
	entry->kind = 'S';
#line 1264 "ctags-vala.c"
	_tmp2_ = NULL;
#line 287 "ctags-vala.vala"
	entry->access = (_tmp2_ = ctags_visitor_get_access ((ValaSymbol*) sig), entry->access = (g_free (entry->access), NULL), _tmp2_);
#line 1268 "ctags-vala.c"
	_tmp3_ = NULL;
#line 288 "ctags-vala.vala"
	entry->implementation = (_tmp3_ = ctags_visitor_implementation ((ValaSymbol*) sig), entry->implementation = (g_free (entry->implementation), NULL), _tmp3_);
#line 1272 "ctags-vala.c"
	_tmp4_ = NULL;
#line 289 "ctags-vala.vala"
	entry->typeref = (_tmp4_ = vala_data_type_to_qualified_string (vala_signal_get_return_type (sig), NULL), entry->typeref = (g_free (entry->typeref), NULL), _tmp4_);
#line 1276 "ctags-vala.c"
	_tmp6_ = NULL;
#line 2280 "vala-1.0.vapi"
	_tmp5_ = NULL;
#line 290 "ctags-vala.vala"
	entry->signature = (_tmp6_ = ctags_visitor_signature (_tmp5_ = vala_signal_get_parameters (sig)), entry->signature = (g_free (entry->signature), NULL), _tmp6_);
#line 1282 "ctags-vala.c"
	(_tmp5_ == NULL) ? NULL : (_tmp5_ = (gee_collection_object_unref (_tmp5_), NULL));
#line 292 "ctags-vala.vala"
	_tmp7_ = NULL;
#line 292 "ctags-vala.vala"
	self->priv->taglist = g_list_append (self->priv->taglist, (_tmp7_ = entry, (_tmp7_ == NULL) ? NULL : ctags_entry_ref (_tmp7_)));
#line 1288 "ctags-vala.c"
	(entry == NULL) ? NULL : (entry = (ctags_entry_unref (entry), NULL));
}


/*print_tag(entry);*/
#line 296 "ctags-vala.vala"
static void ctags_visitor_real_visit_field (ValaCodeVisitor* base, ValaField* f) {
#line 1296 "ctags-vala.c"
	CTagsVisitor * self;
	CTagsEntry* entry;
	char* _tmp0_;
	char* _tmp1_;
	char* _tmp2_;
	char* _tmp3_;
	CTagsEntry* _tmp4_;
	self = (CTagsVisitor*) base;
#line 296 "ctags-vala.vala"
	g_return_if_fail (f != NULL);
#line 1307 "ctags-vala.c"
	entry = ctags_entry_new ();
#line 299 "ctags-vala.vala"
	entry->line_number = vala_source_reference_get_first_line (vala_code_node_get_source_reference ((ValaCodeNode*) f));
#line 1311 "ctags-vala.c"
	_tmp0_ = NULL;
#line 300 "ctags-vala.vala"
	entry->name = (_tmp0_ = ctags_visitor_name ((ValaSymbol*) f), entry->name = (g_free (entry->name), NULL), _tmp0_);
#line 1315 "ctags-vala.c"
	_tmp1_ = NULL;
#line 301 "ctags-vala.vala"
	entry->kind_name = (_tmp1_ = g_strdup ("field"), entry->kind_name = (g_free (entry->kind_name), NULL), _tmp1_);
#line 302 "ctags-vala.vala"
	entry->kind = 'f';
#line 1321 "ctags-vala.c"
	_tmp2_ = NULL;
#line 303 "ctags-vala.vala"
	entry->access = (_tmp2_ = ctags_visitor_get_access ((ValaSymbol*) f), entry->access = (g_free (entry->access), NULL), _tmp2_);
#line 1325 "ctags-vala.c"
	_tmp3_ = NULL;
#line 304 "ctags-vala.vala"
	entry->typeref = (_tmp3_ = vala_data_type_to_qualified_string (vala_field_get_field_type (f), NULL), entry->typeref = (g_free (entry->typeref), NULL), _tmp3_);
#line 306 "ctags-vala.vala"
	_tmp4_ = NULL;
#line 306 "ctags-vala.vala"
	self->priv->taglist = g_list_append (self->priv->taglist, (_tmp4_ = entry, (_tmp4_ == NULL) ? NULL : ctags_entry_ref (_tmp4_)));
#line 1333 "ctags-vala.c"
	(entry == NULL) ? NULL : (entry = (ctags_entry_unref (entry), NULL));
}


/*print_tag(entry);*/
#line 310 "ctags-vala.vala"
static void ctags_visitor_real_visit_constant (ValaCodeVisitor* base, ValaConstant* c) {
#line 1341 "ctags-vala.c"
	CTagsVisitor * self;
	CTagsEntry* entry;
	char* _tmp0_;
	char* _tmp1_;
	char* _tmp2_;
	char* _tmp3_;
	CTagsEntry* _tmp4_;
	self = (CTagsVisitor*) base;
#line 310 "ctags-vala.vala"
	g_return_if_fail (c != NULL);
#line 1352 "ctags-vala.c"
	entry = ctags_entry_new ();
#line 313 "ctags-vala.vala"
	entry->line_number = vala_source_reference_get_first_line (vala_code_node_get_source_reference ((ValaCodeNode*) c));
#line 1356 "ctags-vala.c"
	_tmp0_ = NULL;
#line 314 "ctags-vala.vala"
	entry->name = (_tmp0_ = ctags_visitor_name ((ValaSymbol*) c), entry->name = (g_free (entry->name), NULL), _tmp0_);
#line 1360 "ctags-vala.c"
	_tmp1_ = NULL;
#line 315 "ctags-vala.vala"
	entry->kind_name = (_tmp1_ = g_strdup ("field"), entry->kind_name = (g_free (entry->kind_name), NULL), _tmp1_);
#line 316 "ctags-vala.vala"
	entry->kind = 'f';
#line 1366 "ctags-vala.c"
	_tmp2_ = NULL;
#line 317 "ctags-vala.vala"
	entry->access = (_tmp2_ = ctags_visitor_get_access ((ValaSymbol*) c), entry->access = (g_free (entry->access), NULL), _tmp2_);
#line 1370 "ctags-vala.c"
	_tmp3_ = NULL;
#line 318 "ctags-vala.vala"
	entry->typeref = (_tmp3_ = vala_data_type_to_qualified_string (vala_constant_get_type_reference (c), NULL), entry->typeref = (g_free (entry->typeref), NULL), _tmp3_);
#line 320 "ctags-vala.vala"
	_tmp4_ = NULL;
#line 320 "ctags-vala.vala"
	self->priv->taglist = g_list_append (self->priv->taglist, (_tmp4_ = entry, (_tmp4_ == NULL) ? NULL : ctags_entry_ref (_tmp4_)));
#line 1378 "ctags-vala.c"
	(entry == NULL) ? NULL : (entry = (ctags_entry_unref (entry), NULL));
}


/*print_tag(entry);*/
#line 324 "ctags-vala.vala"
static void ctags_visitor_real_visit_property (ValaCodeVisitor* base, ValaProperty* prop) {
#line 1386 "ctags-vala.c"
	CTagsVisitor * self;
	CTagsEntry* entry;
	char* _tmp0_;
	char* _tmp1_;
	char* _tmp2_;
	char* _tmp3_;
	char* _tmp4_;
	CTagsEntry* _tmp5_;
	self = (CTagsVisitor*) base;
#line 324 "ctags-vala.vala"
	g_return_if_fail (prop != NULL);
#line 1398 "ctags-vala.c"
	entry = ctags_entry_new ();
#line 327 "ctags-vala.vala"
	entry->line_number = vala_source_reference_get_first_line (vala_code_node_get_source_reference ((ValaCodeNode*) prop));
#line 1402 "ctags-vala.c"
	_tmp0_ = NULL;
#line 328 "ctags-vala.vala"
	entry->name = (_tmp0_ = ctags_visitor_name ((ValaSymbol*) prop), entry->name = (g_free (entry->name), NULL), _tmp0_);
#line 1406 "ctags-vala.c"
	_tmp1_ = NULL;
#line 329 "ctags-vala.vala"
	entry->kind_name = (_tmp1_ = g_strdup ("property"), entry->kind_name = (g_free (entry->kind_name), NULL), _tmp1_);
#line 330 "ctags-vala.vala"
	entry->kind = 'p';
#line 1412 "ctags-vala.c"
	_tmp2_ = NULL;
#line 331 "ctags-vala.vala"
	entry->access = (_tmp2_ = ctags_visitor_get_access ((ValaSymbol*) prop), entry->access = (g_free (entry->access), NULL), _tmp2_);
#line 1416 "ctags-vala.c"
	_tmp3_ = NULL;
#line 332 "ctags-vala.vala"
	entry->implementation = (_tmp3_ = ctags_visitor_implementation ((ValaSymbol*) prop), entry->implementation = (g_free (entry->implementation), NULL), _tmp3_);
#line 1420 "ctags-vala.c"
	_tmp4_ = NULL;
#line 333 "ctags-vala.vala"
	entry->typeref = (_tmp4_ = vala_data_type_to_qualified_string (vala_property_get_property_type (prop), NULL), entry->typeref = (g_free (entry->typeref), NULL), _tmp4_);
#line 335 "ctags-vala.vala"
	_tmp5_ = NULL;
#line 335 "ctags-vala.vala"
	self->priv->taglist = g_list_append (self->priv->taglist, (_tmp5_ = entry, (_tmp5_ == NULL) ? NULL : ctags_entry_ref (_tmp5_)));
#line 1428 "ctags-vala.c"
	(entry == NULL) ? NULL : (entry = (ctags_entry_unref (entry), NULL));
}


/*print_tag(entry);*/
#line 339 "ctags-vala.vala"
static void ctags_visitor_real_visit_method (ValaCodeVisitor* base, ValaMethod* m) {
#line 1436 "ctags-vala.c"
	CTagsVisitor * self;
	CTagsEntry* entry;
	char* _tmp0_;
	char* _tmp1_;
	char* _tmp2_;
	char* _tmp3_;
	char* _tmp4_;
	char* _tmp6_;
	GeeList* _tmp5_;
	CTagsEntry* _tmp7_;
	self = (CTagsVisitor*) base;
#line 339 "ctags-vala.vala"
	g_return_if_fail (m != NULL);
#line 1450 "ctags-vala.c"
	entry = ctags_entry_new ();
#line 342 "ctags-vala.vala"
	entry->line_number = vala_source_reference_get_first_line (vala_code_node_get_source_reference ((ValaCodeNode*) m));
#line 1454 "ctags-vala.c"
	_tmp0_ = NULL;
#line 343 "ctags-vala.vala"
	entry->name = (_tmp0_ = ctags_visitor_name ((ValaSymbol*) m), entry->name = (g_free (entry->name), NULL), _tmp0_);
#line 1458 "ctags-vala.c"
	_tmp1_ = NULL;
#line 344 "ctags-vala.vala"
	entry->kind_name = (_tmp1_ = g_strdup ("method"), entry->kind_name = (g_free (entry->kind_name), NULL), _tmp1_);
#line 345 "ctags-vala.vala"
	entry->kind = 'm';
#line 1464 "ctags-vala.c"
	_tmp2_ = NULL;
#line 346 "ctags-vala.vala"
	entry->access = (_tmp2_ = ctags_visitor_get_access ((ValaSymbol*) m), entry->access = (g_free (entry->access), NULL), _tmp2_);
#line 1468 "ctags-vala.c"
	_tmp3_ = NULL;
#line 347 "ctags-vala.vala"
	entry->implementation = (_tmp3_ = ctags_visitor_implementation ((ValaSymbol*) m), entry->implementation = (g_free (entry->implementation), NULL), _tmp3_);
#line 1472 "ctags-vala.c"
	_tmp4_ = NULL;
#line 348 "ctags-vala.vala"
	entry->typeref = (_tmp4_ = vala_data_type_to_qualified_string (vala_method_get_return_type (m), NULL), entry->typeref = (g_free (entry->typeref), NULL), _tmp4_);
#line 1476 "ctags-vala.c"
	_tmp6_ = NULL;
#line 1853 "vala-1.0.vapi"
	_tmp5_ = NULL;
#line 349 "ctags-vala.vala"
	entry->signature = (_tmp6_ = ctags_visitor_signature (_tmp5_ = vala_method_get_parameters (m)), entry->signature = (g_free (entry->signature), NULL), _tmp6_);
#line 1482 "ctags-vala.c"
	(_tmp5_ == NULL) ? NULL : (_tmp5_ = (gee_collection_object_unref (_tmp5_), NULL));
#line 351 "ctags-vala.vala"
	_tmp7_ = NULL;
#line 351 "ctags-vala.vala"
	self->priv->taglist = g_list_append (self->priv->taglist, (_tmp7_ = entry, (_tmp7_ == NULL) ? NULL : ctags_entry_ref (_tmp7_)));
#line 1488 "ctags-vala.c"
	(entry == NULL) ? NULL : (entry = (ctags_entry_unref (entry), NULL));
}


/*print_tag(entry);*/
#line 355 "ctags-vala.vala"
static void ctags_visitor_real_visit_namespace (ValaCodeVisitor* base, ValaNamespace* m) {
#line 1496 "ctags-vala.c"
	CTagsVisitor * self;
	self = (CTagsVisitor*) base;
#line 355 "ctags-vala.vala"
	g_return_if_fail (m != NULL);
#line 356 "ctags-vala.vala"
	if (vala_symbol_get_name ((ValaSymbol*) m) != NULL) {
#line 1503 "ctags-vala.c"
		CTagsEntry* entry;
		char* _tmp0_;
		char* _tmp1_;
		char* _tmp2_;
		CTagsEntry* _tmp3_;
		entry = ctags_entry_new ();
#line 360 "ctags-vala.vala"
		entry->line_number = vala_source_reference_get_first_line (vala_code_node_get_source_reference ((ValaCodeNode*) m));
#line 1512 "ctags-vala.c"
		_tmp0_ = NULL;
#line 361 "ctags-vala.vala"
		entry->name = (_tmp0_ = ctags_visitor_name ((ValaSymbol*) m), entry->name = (g_free (entry->name), NULL), _tmp0_);
#line 1516 "ctags-vala.c"
		_tmp1_ = NULL;
#line 362 "ctags-vala.vala"
		entry->kind_name = (_tmp1_ = g_strdup ("namespace"), entry->kind_name = (g_free (entry->kind_name), NULL), _tmp1_);
#line 363 "ctags-vala.vala"
		entry->kind = 'M';
#line 1522 "ctags-vala.c"
		_tmp2_ = NULL;
#line 364 "ctags-vala.vala"
		entry->access = (_tmp2_ = ctags_visitor_get_access ((ValaSymbol*) m), entry->access = (g_free (entry->access), NULL), _tmp2_);
#line 366 "ctags-vala.vala"
		_tmp3_ = NULL;
#line 366 "ctags-vala.vala"
		self->priv->taglist = g_list_append (self->priv->taglist, (_tmp3_ = entry, (_tmp3_ == NULL) ? NULL : ctags_entry_ref (_tmp3_)));
#line 1530 "ctags-vala.c"
		(entry == NULL) ? NULL : (entry = (ctags_entry_unref (entry), NULL));
	}
}


/*  print_tag(entry);*/
#line 371 "ctags-vala.vala"
static void ctags_visitor_real_visit_creation_method (ValaCodeVisitor* base, ValaCreationMethod* m) {
#line 1539 "ctags-vala.c"
	CTagsVisitor * self;
	CTagsEntry* entry;
	char* _tmp0_;
	char* _tmp1_;
	char* _tmp2_;
	char* _tmp3_;
	char* _tmp4_;
	char* _tmp6_;
	GeeList* _tmp5_;
	CTagsEntry* _tmp7_;
	self = (CTagsVisitor*) base;
#line 371 "ctags-vala.vala"
	g_return_if_fail (m != NULL);
#line 1553 "ctags-vala.c"
	entry = ctags_entry_new ();
#line 374 "ctags-vala.vala"
	entry->line_number = vala_source_reference_get_first_line (vala_code_node_get_source_reference ((ValaCodeNode*) m));
#line 1557 "ctags-vala.c"
	_tmp0_ = NULL;
#line 375 "ctags-vala.vala"
	entry->name = (_tmp0_ = ctags_visitor_name ((ValaSymbol*) m), entry->name = (g_free (entry->name), NULL), _tmp0_);
#line 1561 "ctags-vala.c"
	_tmp1_ = NULL;
#line 376 "ctags-vala.vala"
	entry->kind_name = (_tmp1_ = g_strdup ("method"), entry->kind_name = (g_free (entry->kind_name), NULL), _tmp1_);
#line 377 "ctags-vala.vala"
	entry->kind = 'm';
#line 1567 "ctags-vala.c"
	_tmp2_ = NULL;
#line 378 "ctags-vala.vala"
	entry->access = (_tmp2_ = ctags_visitor_get_access ((ValaSymbol*) m), entry->access = (g_free (entry->access), NULL), _tmp2_);
#line 1571 "ctags-vala.c"
	_tmp3_ = NULL;
#line 379 "ctags-vala.vala"
	entry->implementation = (_tmp3_ = ctags_visitor_implementation ((ValaSymbol*) m), entry->implementation = (g_free (entry->implementation), NULL), _tmp3_);
#line 1575 "ctags-vala.c"
	_tmp4_ = NULL;
#line 380 "ctags-vala.vala"
	entry->typeref = (_tmp4_ = g_strdup (""), entry->typeref = (g_free (entry->typeref), NULL), _tmp4_);
#line 1579 "ctags-vala.c"
	_tmp6_ = NULL;
#line 1853 "vala-1.0.vapi"
	_tmp5_ = NULL;
#line 381 "ctags-vala.vala"
	entry->signature = (_tmp6_ = ctags_visitor_signature (_tmp5_ = vala_method_get_parameters ((ValaMethod*) m)), entry->signature = (g_free (entry->signature), NULL), _tmp6_);
#line 1585 "ctags-vala.c"
	(_tmp5_ == NULL) ? NULL : (_tmp5_ = (gee_collection_object_unref (_tmp5_), NULL));
#line 383 "ctags-vala.vala"
	_tmp7_ = NULL;
#line 383 "ctags-vala.vala"
	self->priv->taglist = g_list_append (self->priv->taglist, (_tmp7_ = entry, (_tmp7_ == NULL) ? NULL : ctags_entry_ref (_tmp7_)));
#line 1591 "ctags-vala.c"
	(entry == NULL) ? NULL : (entry = (ctags_entry_unref (entry), NULL));
}


/*print_tag(entry);

public override void visit_constructor (Constructor c) {
var entry = new CTagsEntry();

entry.line_number = c.source_reference.first_line;
entry.name = name (c);
entry.kind_name = "method";
entry.kind = 'm';
entry.access = get_access (c);
entry.implementation = implementation(c);
entry.typeref = "";
entry.signature = "construct";

taglist.append(entry);
//print_tag(entry);
}

public override void visit_destructor (Destructor d) {
var entry = new CTagsEntry();

entry.line_number = d.source_reference.first_line;
entry.name = name (d);
entry.kind_name = "method";
entry.kind = 'm';
entry.access = "";
entry.implementation = "";
entry.typeref = "";
entry.signature = "";

taglist.append(entry);
//print_tag(entry);
}
*/
#line 419 "ctags-vala.vala"
static void ctags_visitor_real_visit_local_variable (ValaCodeVisitor* base, ValaLocalVariable* local) {
#line 1632 "ctags-vala.c"
	CTagsVisitor * self;
	CTagsEntry* entry;
	char* _tmp0_;
	char* _tmp1_;
	char* _tmp2_;
	CTagsEntry* _tmp3_;
	self = (CTagsVisitor*) base;
#line 419 "ctags-vala.vala"
	g_return_if_fail (local != NULL);
#line 1642 "ctags-vala.c"
	entry = ctags_entry_new ();
#line 422 "ctags-vala.vala"
	entry->line_number = vala_source_reference_get_first_line (vala_code_node_get_source_reference ((ValaCodeNode*) local));
#line 1646 "ctags-vala.c"
	_tmp0_ = NULL;
#line 423 "ctags-vala.vala"
	entry->name = (_tmp0_ = ctags_visitor_name ((ValaSymbol*) local), entry->name = (g_free (entry->name), NULL), _tmp0_);
#line 1650 "ctags-vala.c"
	_tmp1_ = NULL;
#line 424 "ctags-vala.vala"
	entry->kind_name = (_tmp1_ = g_strdup ("local"), entry->kind_name = (g_free (entry->kind_name), NULL), _tmp1_);
#line 425 "ctags-vala.vala"
	entry->kind = 'l';
#line 1656 "ctags-vala.c"
	_tmp2_ = NULL;
#line 426 "ctags-vala.vala"
	entry->access = (_tmp2_ = ctags_visitor_get_access ((ValaSymbol*) local), entry->access = (g_free (entry->access), NULL), _tmp2_);
#line 428 "ctags-vala.vala"
	_tmp3_ = NULL;
#line 428 "ctags-vala.vala"
	self->priv->taglist = g_list_append (self->priv->taglist, (_tmp3_ = entry, (_tmp3_ == NULL) ? NULL : ctags_entry_ref (_tmp3_)));
#line 1664 "ctags-vala.c"
	(entry == NULL) ? NULL : (entry = (ctags_entry_unref (entry), NULL));
}


/*print_tag(entry);*/
#line 432 "ctags-vala.vala"
void ctags_visitor_parse_vala (CTagsVisitor* self, const char* filename, CTagsEntryMaker maker) {
#line 1672 "ctags-vala.c"
	ValaParser* _tmp0_;
	GList* _tmp1_;
	ValaCodeContext* context;
	Report* _tmp2_;
	ValaSourceFile* source_file;
	GList* _tmp4_;
#line 432 "ctags-vala.vala"
	g_return_if_fail (self != NULL);
#line 432 "ctags-vala.vala"
	g_return_if_fail (filename != NULL);
#line 1683 "ctags-vala.c"
	/**
	     * @bug CodeVisitor doesn't derive from GObject (r1757), construct doesn't call.
	     */
	_tmp0_ = NULL;
#line 436 "ctags-vala.vala"
	self->priv->vala_parser = (_tmp0_ = vala_parser_new (), (self->priv->vala_parser == NULL) ? NULL : (self->priv->vala_parser = (vala_code_visitor_unref (self->priv->vala_parser), NULL)), _tmp0_);
#line 1690 "ctags-vala.c"
	_tmp1_ = NULL;
#line 437 "ctags-vala.vala"
	self->priv->taglist = (_tmp1_ = NULL, (self->priv->taglist == NULL) ? NULL : (self->priv->taglist = (_g_list_free_ctags_entry_unref (self->priv->taglist), NULL)), _tmp1_);
#line 1694 "ctags-vala.c"
	/* We create a context for every source file so that Parser.parse(context)
	 * don't parse a file multiple times causing errors. Parser.parse_file(source_file)
	 * assumes that Parser.context is the same as source_file.context anyway */
	context = vala_code_context_new ();
	_tmp2_ = NULL;
#line 442 "ctags-vala.vala"
	vala_code_context_set_report (context, (ValaReport*) (_tmp2_ = report_new ()));
#line 1702 "ctags-vala.c"
	(_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL));
	source_file = vala_source_file_new (context, filename, g_str_has_suffix (filename, "vapi"), NULL);
#line 444 "ctags-vala.vala"
	vala_code_context_add_source_file (context, source_file);
#line 445 "ctags-vala.vala"
	vala_code_context_push (context);
#line 446 "ctags-vala.vala"
	vala_parser_parse (self->priv->vala_parser, context);
#line 447 "ctags-vala.vala"
	vala_code_context_accept (context, (ValaCodeVisitor*) self);
#line 1713 "ctags-vala.c"
	{
		GList* tagentry_collection;
		GList* tagentry_it;
#line 448 "ctags-vala.vala"
		tagentry_collection = self->priv->taglist;
#line 1719 "ctags-vala.c"
		for (tagentry_it = tagentry_collection; tagentry_it != NULL; tagentry_it = tagentry_it->next) {
			CTagsEntry* _tmp3_;
			CTagsEntry* tagentry;
#line 52 "ctags-vala.vala"
			_tmp3_ = NULL;
#line 448 "ctags-vala.vala"
			tagentry = (_tmp3_ = (CTagsEntry*) tagentry_it->data, (_tmp3_ == NULL) ? NULL : ctags_entry_ref (_tmp3_));
#line 1727 "ctags-vala.c"
			{
#line 449 "ctags-vala.vala"
				maker (tagentry);
#line 1731 "ctags-vala.c"
				(tagentry == NULL) ? NULL : (tagentry = (ctags_entry_unref (tagentry), NULL));
			}
		}
	}
	_tmp4_ = NULL;
#line 451 "ctags-vala.vala"
	self->priv->taglist = (_tmp4_ = NULL, (self->priv->taglist == NULL) ? NULL : (self->priv->taglist = (_g_list_free_ctags_entry_unref (self->priv->taglist), NULL)), _tmp4_);
#line 1739 "ctags-vala.c"
	(context == NULL) ? NULL : (context = (vala_code_context_unref (context), NULL));
	(source_file == NULL) ? NULL : (source_file = (vala_source_file_unref (source_file), NULL));
}


#line 454 "ctags-vala.vala"
void ctags_visitor_parse_genie (CTagsVisitor* self, const char* filename, CTagsEntryMaker maker) {
#line 1747 "ctags-vala.c"
	ValaGenieParser* _tmp0_;
	GList* _tmp1_;
	ValaCodeContext* context;
	Report* _tmp2_;
	ValaSourceFile* source_file;
	GList* _tmp4_;
#line 454 "ctags-vala.vala"
	g_return_if_fail (self != NULL);
#line 454 "ctags-vala.vala"
	g_return_if_fail (filename != NULL);
#line 1758 "ctags-vala.c"
	/**
	     * @bug CodeVisitor doesn't derive from GObject (r1757), construct doesn't call.
	     */
	_tmp0_ = NULL;
#line 458 "ctags-vala.vala"
	self->priv->genie_parser = (_tmp0_ = vala_genie_parser_new (), (self->priv->genie_parser == NULL) ? NULL : (self->priv->genie_parser = (vala_code_visitor_unref (self->priv->genie_parser), NULL)), _tmp0_);
#line 1765 "ctags-vala.c"
	_tmp1_ = NULL;
#line 459 "ctags-vala.vala"
	self->priv->taglist = (_tmp1_ = NULL, (self->priv->taglist == NULL) ? NULL : (self->priv->taglist = (_g_list_free_ctags_entry_unref (self->priv->taglist), NULL)), _tmp1_);
#line 1769 "ctags-vala.c"
	context = vala_code_context_new ();
	_tmp2_ = NULL;
#line 461 "ctags-vala.vala"
	vala_code_context_set_report (context, (ValaReport*) (_tmp2_ = report_new ()));
#line 1774 "ctags-vala.c"
	(_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL));
	source_file = vala_source_file_new (context, filename, FALSE, NULL);
#line 463 "ctags-vala.vala"
	vala_code_context_add_source_file (context, source_file);
#line 464 "ctags-vala.vala"
	vala_code_context_push (context);
#line 465 "ctags-vala.vala"
	vala_genie_parser_parse (self->priv->genie_parser, context);
#line 466 "ctags-vala.vala"
	vala_code_context_accept (context, (ValaCodeVisitor*) self);
#line 1785 "ctags-vala.c"
	{
		GList* tagentry_collection;
		GList* tagentry_it;
#line 467 "ctags-vala.vala"
		tagentry_collection = self->priv->taglist;
#line 1791 "ctags-vala.c"
		for (tagentry_it = tagentry_collection; tagentry_it != NULL; tagentry_it = tagentry_it->next) {
			CTagsEntry* _tmp3_;
			CTagsEntry* tagentry;
#line 52 "ctags-vala.vala"
			_tmp3_ = NULL;
#line 467 "ctags-vala.vala"
			tagentry = (_tmp3_ = (CTagsEntry*) tagentry_it->data, (_tmp3_ == NULL) ? NULL : ctags_entry_ref (_tmp3_));
#line 1799 "ctags-vala.c"
			{
#line 468 "ctags-vala.vala"
				maker (tagentry);
#line 1803 "ctags-vala.c"
				(tagentry == NULL) ? NULL : (tagentry = (ctags_entry_unref (tagentry), NULL));
			}
		}
	}
	_tmp4_ = NULL;
#line 470 "ctags-vala.vala"
	self->priv->taglist = (_tmp4_ = NULL, (self->priv->taglist == NULL) ? NULL : (self->priv->taglist = (_g_list_free_ctags_entry_unref (self->priv->taglist), NULL)), _tmp4_);
#line 1811 "ctags-vala.c"
	(context == NULL) ? NULL : (context = (vala_code_context_unref (context), NULL));
	(source_file == NULL) ? NULL : (source_file = (vala_source_file_unref (source_file), NULL));
}


static void ctags_visitor_class_init (CTagsVisitorClass * klass) {
	ctags_visitor_parent_class = g_type_class_peek_parent (klass);
	VALA_CODE_VISITOR_CLASS (klass)->finalize = ctags_visitor_finalize;
	g_type_class_add_private (klass, sizeof (CTagsVisitorPrivate));
	VALA_CODE_VISITOR_CLASS (klass)->visit_source_file = ctags_visitor_real_visit_source_file;
	VALA_CODE_VISITOR_CLASS (klass)->visit_class = ctags_visitor_real_visit_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_struct = ctags_visitor_real_visit_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_interface = ctags_visitor_real_visit_interface;
	VALA_CODE_VISITOR_CLASS (klass)->visit_enum = ctags_visitor_real_visit_enum;
	VALA_CODE_VISITOR_CLASS (klass)->visit_error_domain = ctags_visitor_real_visit_error_domain;
	VALA_CODE_VISITOR_CLASS (klass)->visit_enum_value = ctags_visitor_real_visit_enum_value;
	VALA_CODE_VISITOR_CLASS (klass)->visit_error_code = ctags_visitor_real_visit_error_code;
	VALA_CODE_VISITOR_CLASS (klass)->visit_delegate = ctags_visitor_real_visit_delegate;
	VALA_CODE_VISITOR_CLASS (klass)->visit_signal = ctags_visitor_real_visit_signal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_field = ctags_visitor_real_visit_field;
	VALA_CODE_VISITOR_CLASS (klass)->visit_constant = ctags_visitor_real_visit_constant;
	VALA_CODE_VISITOR_CLASS (klass)->visit_property = ctags_visitor_real_visit_property;
	VALA_CODE_VISITOR_CLASS (klass)->visit_method = ctags_visitor_real_visit_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_namespace = ctags_visitor_real_visit_namespace;
	VALA_CODE_VISITOR_CLASS (klass)->visit_creation_method = ctags_visitor_real_visit_creation_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_local_variable = ctags_visitor_real_visit_local_variable;
}


static void ctags_visitor_instance_init (CTagsVisitor * self) {
	self->priv = CTAGS_VISITOR_GET_PRIVATE (self);
}


static void ctags_visitor_finalize (ValaCodeVisitor* obj) {
	CTagsVisitor * self;
	self = CTAGS_VISITOR (obj);
	(self->priv->vala_parser == NULL) ? NULL : (self->priv->vala_parser = (vala_code_visitor_unref (self->priv->vala_parser), NULL));
	(self->priv->genie_parser == NULL) ? NULL : (self->priv->genie_parser = (vala_code_visitor_unref (self->priv->genie_parser), NULL));
	(self->priv->taglist == NULL) ? NULL : (self->priv->taglist = (_g_list_free_ctags_entry_unref (self->priv->taglist), NULL));
	VALA_CODE_VISITOR_CLASS (ctags_visitor_parent_class)->finalize (obj);
}


GType ctags_visitor_get_type (void) {
	static GType ctags_visitor_type_id = 0;
	if (ctags_visitor_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (CTagsVisitorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) ctags_visitor_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (CTagsVisitor), 0, (GInstanceInitFunc) ctags_visitor_instance_init, NULL };
		ctags_visitor_type_id = g_type_register_static (VALA_TYPE_CODE_VISITOR, "CTagsVisitor", &g_define_type_info, 0);
	}
	return ctags_visitor_type_id;
}




