/* aboutdialog.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <config.h>
#include <gio/gio.h>
#include <stdlib.h>
#include <string.h>
#include <glib/gstdio.h>
#include <gdk-pixbuf/gdk-pixdata.h>


#define VALIDE_TYPE_ABOUT_DIALOG (valide_about_dialog_get_type ())
#define VALIDE_ABOUT_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABOUT_DIALOG, ValideAboutDialog))
#define VALIDE_ABOUT_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABOUT_DIALOG, ValideAboutDialogClass))
#define VALIDE_IS_ABOUT_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABOUT_DIALOG))
#define VALIDE_IS_ABOUT_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABOUT_DIALOG))
#define VALIDE_ABOUT_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABOUT_DIALOG, ValideAboutDialogClass))

typedef struct _ValideAboutDialog ValideAboutDialog;
typedef struct _ValideAboutDialogClass ValideAboutDialogClass;
typedef struct _ValideAboutDialogPrivate ValideAboutDialogPrivate;

/**
 * The about dialog box
 */
struct _ValideAboutDialog {
	GtkAboutDialog parent_instance;
	ValideAboutDialogPrivate * priv;
};

struct _ValideAboutDialogClass {
	GtkAboutDialogClass parent_class;
};


static gpointer valide_about_dialog_parent_class = NULL;

GType valide_about_dialog_get_type (void);
enum  {
	VALIDE_ABOUT_DIALOG_DUMMY_PROPERTY
};
#define VALIDE_ABOUT_DIALOG__copyright COPYRIGHT
#define VALIDE_ABOUT_DIALOG__program_name PACKAGE_NAME
#define VALIDE_ABOUT_DIALOG__version VERSION
#define VALIDE_ABOUT_DIALOG__website WEBSITE
static void valide_about_dialog_activate_link_func (ValideAboutDialog* self, GtkAboutDialog* _self_, const char* link);
ValideAboutDialog* valide_about_dialog_new (void);
ValideAboutDialog* valide_about_dialog_construct (GType object_type);
static void _valide_about_dialog_activate_link_func_gtk_about_dialog_activate_link_func (GtkAboutDialog* about, const char* link_, gpointer self);
static GObject * valide_about_dialog_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);



#line 32 "aboutdialog.vala"
static void valide_about_dialog_activate_link_func (ValideAboutDialog* self, GtkAboutDialog* _self_, const char* link) {
#line 81 "aboutdialog.c"
	GError * _inner_error_;
#line 32 "aboutdialog.vala"
	g_return_if_fail (self != NULL);
#line 32 "aboutdialog.vala"
	g_return_if_fail (_self_ != NULL);
#line 32 "aboutdialog.vala"
	g_return_if_fail (link != NULL);
#line 89 "aboutdialog.c"
	_inner_error_ = NULL;
	{
#line 36 "aboutdialog.vala"
		g_app_info_launch_default_for_uri (link, NULL, &_inner_error_);
#line 94 "aboutdialog.c"
		if (_inner_error_ != NULL) {
			goto __catch45_g_error;
			goto __finally45;
		}
	}
	goto __finally45;
	__catch45_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 40 "aboutdialog.vala"
			g_warning (e->message);
#line 109 "aboutdialog.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally45:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
}


/**
 * The about dialog box
 */
#line 25 "aboutdialog.vala"
ValideAboutDialog* valide_about_dialog_construct (GType object_type) {
#line 127 "aboutdialog.c"
	ValideAboutDialog * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


#line 25 "aboutdialog.vala"
ValideAboutDialog* valide_about_dialog_new (void) {
#line 25 "aboutdialog.vala"
	return valide_about_dialog_construct (VALIDE_TYPE_ABOUT_DIALOG);
#line 138 "aboutdialog.c"
}


#line 32 "aboutdialog.vala"
static void _valide_about_dialog_activate_link_func_gtk_about_dialog_activate_link_func (GtkAboutDialog* about, const char* link_, gpointer self) {
#line 144 "aboutdialog.c"
	valide_about_dialog_activate_link_func (self, about, link_);
}


static GObject * valide_about_dialog_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValideAboutDialogClass * klass;
	GObjectClass * parent_class;
	ValideAboutDialog * self;
	GError * _inner_error_;
	klass = VALIDE_ABOUT_DIALOG_CLASS (g_type_class_peek (VALIDE_TYPE_ABOUT_DIALOG));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_ABOUT_DIALOG (obj);
	_inner_error_ = NULL;
	{
		char* contents;
		contents = NULL;
		{
			char* _tmp3_;
			gboolean _tmp2_;
			char* _tmp1_;
			char* _tmp0_;
			_tmp3_ = NULL;
			_tmp1_ = NULL;
#line 2034 "glib-2.0.vapi"
			_tmp0_ = NULL;
#line 50 "aboutdialog.vala"
			_tmp2_ = g_file_get_contents (_tmp0_ = g_build_filename (DATA_DIR, "COPYING", NULL), &_tmp1_, NULL, &_inner_error_);
#line 50 "aboutdialog.vala"
			contents = (_tmp3_ = _tmp1_, contents = (g_free (contents), NULL), _tmp3_);
#line 50 "aboutdialog.vala"
			_tmp2_;
#line 178 "aboutdialog.c"
			if (_inner_error_ != NULL) {
				goto __catch46_g_error;
				goto __finally46;
			}
			_tmp0_ = (g_free (_tmp0_), NULL);
#line 52 "aboutdialog.vala"
			gtk_about_dialog_set_license ((GtkAboutDialog*) self, contents);
#line 186 "aboutdialog.c"
		}
		goto __finally46;
		__catch46_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
#line 56 "aboutdialog.vala"
				g_debug (e->message);
#line 197 "aboutdialog.c"
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally46:
		if (_inner_error_ != NULL) {
			contents = (g_free (contents), NULL);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
		}
		{
			char* _tmp4_;
			GdkPixbuf* _tmp5_;
			GdkPixbuf* _tmp6_;
			GdkPixbuf* _tmp7_;
#line 2034 "glib-2.0.vapi"
			_tmp4_ = NULL;
#line 61 "aboutdialog.vala"
			_tmp5_ = NULL;
#line 216 "aboutdialog.c"
			_tmp6_ = (_tmp5_ = gdk_pixbuf_new_from_file (_tmp4_ = g_build_filename (PIXMAPS_DIR, "logo.png", NULL), &_inner_error_), _tmp4_ = (g_free (_tmp4_), NULL), _tmp5_);
			if (_inner_error_ != NULL) {
				goto __catch47_g_error;
				goto __finally47;
			}
			_tmp7_ = NULL;
#line 61 "aboutdialog.vala"
			gtk_about_dialog_set_logo ((GtkAboutDialog*) self, _tmp7_ = _tmp6_);
#line 225 "aboutdialog.c"
			(_tmp7_ == NULL) ? NULL : (_tmp7_ = (g_object_unref (_tmp7_), NULL));
		}
		goto __finally47;
		__catch47_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
#line 65 "aboutdialog.vala"
				g_debug (e->message);
#line 237 "aboutdialog.c"
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally47:
		if (_inner_error_ != NULL) {
			contents = (g_free (contents), NULL);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
		}
		{
			char* _tmp11_;
			gboolean _tmp10_;
			char* _tmp9_;
			char* _tmp8_;
			gint _tmp13__length1;
			char** _tmp13_;
			char** _tmp12_;
			_tmp11_ = NULL;
			_tmp9_ = NULL;
#line 2034 "glib-2.0.vapi"
			_tmp8_ = NULL;
#line 70 "aboutdialog.vala"
			_tmp10_ = g_file_get_contents (_tmp8_ = g_build_filename (DATA_DIR, "AUTHORS", NULL), &_tmp9_, NULL, &_inner_error_);
#line 70 "aboutdialog.vala"
			contents = (_tmp11_ = _tmp9_, contents = (g_free (contents), NULL), _tmp11_);
#line 70 "aboutdialog.vala"
			_tmp10_;
#line 265 "aboutdialog.c"
			if (_inner_error_ != NULL) {
				goto __catch48_g_error;
				goto __finally48;
			}
			_tmp8_ = (g_free (_tmp8_), NULL);
#line 831 "glib-2.0.vapi"
			_tmp13_ = NULL;
#line 273 "aboutdialog.c"
			_tmp12_ = NULL;
#line 72 "aboutdialog.vala"
			gtk_about_dialog_set_authors ((GtkAboutDialog*) self, (_tmp13_ = _tmp12_ = g_strsplit (contents, "\n", 0), _tmp13__length1 = _vala_array_length (_tmp12_), _tmp13_));
#line 277 "aboutdialog.c"
			_tmp13_ = (_vala_array_free (_tmp13_, _tmp13__length1, (GDestroyNotify) g_free), NULL);
		}
		goto __finally48;
		__catch48_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
#line 76 "aboutdialog.vala"
				g_debug (e->message);
#line 289 "aboutdialog.c"
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally48:
		if (_inner_error_ != NULL) {
			contents = (g_free (contents), NULL);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
		}
#line 78 "aboutdialog.vala"
		gtk_about_dialog_set_program_name ((GtkAboutDialog*) self, VALIDE_ABOUT_DIALOG__program_name);
#line 79 "aboutdialog.vala"
		gtk_about_dialog_set_version ((GtkAboutDialog*) self, VALIDE_ABOUT_DIALOG__version);
#line 80 "aboutdialog.vala"
		gtk_about_dialog_set_website ((GtkAboutDialog*) self, VALIDE_ABOUT_DIALOG__website);
#line 81 "aboutdialog.vala"
		gtk_about_dialog_set_copyright ((GtkAboutDialog*) self, VALIDE_ABOUT_DIALOG__copyright);
#line 82 "aboutdialog.vala"
		gtk_about_dialog_set_url_hook (_valide_about_dialog_activate_link_func_gtk_about_dialog_activate_link_func, g_object_ref (self), g_object_unref);
#line 309 "aboutdialog.c"
		contents = (g_free (contents), NULL);
	}
	return obj;
}


static void valide_about_dialog_class_init (ValideAboutDialogClass * klass) {
	valide_about_dialog_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->constructor = valide_about_dialog_constructor;
}


static void valide_about_dialog_instance_init (ValideAboutDialog * self) {
}


GType valide_about_dialog_get_type (void) {
	static GType valide_about_dialog_type_id = 0;
	if (valide_about_dialog_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideAboutDialogClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_about_dialog_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideAboutDialog), 0, (GInstanceInitFunc) valide_about_dialog_instance_init, NULL };
		valide_about_dialog_type_id = g_type_register_static (GTK_TYPE_ABOUT_DIALOG, "ValideAboutDialog", &g_define_type_info, 0);
	}
	return valide_about_dialog_type_id;
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}




