/* abstract-config-manager.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <config.h>
#include <glib/gstdio.h>
#include <stdio.h>
#include <glib/gi18n-lib.h>


#define VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER (valide_abstract_config_manager_get_type ())
#define VALIDE_ABSTRACT_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManager))
#define VALIDE_ABSTRACT_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManagerClass))
#define VALIDE_IS_ABSTRACT_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER))
#define VALIDE_IS_ABSTRACT_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER))
#define VALIDE_ABSTRACT_CONFIG_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManagerClass))

typedef struct _ValideAbstractConfigManager ValideAbstractConfigManager;
typedef struct _ValideAbstractConfigManagerClass ValideAbstractConfigManagerClass;
typedef struct _ValideAbstractConfigManagerPrivate ValideAbstractConfigManagerPrivate;

#define VALIDE_TYPE_WINDOW (valide_window_get_type ())
#define VALIDE_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_WINDOW, ValideWindow))
#define VALIDE_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_WINDOW, ValideWindowClass))
#define VALIDE_IS_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_WINDOW))
#define VALIDE_IS_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_WINDOW))
#define VALIDE_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_WINDOW, ValideWindowClass))

typedef struct _ValideWindow ValideWindow;
typedef struct _ValideWindowClass ValideWindowClass;

/**
 * Abstract class for manage configation files.
 */
struct _ValideAbstractConfigManager {
	GObject parent_instance;
	ValideAbstractConfigManagerPrivate * priv;
};

struct _ValideAbstractConfigManagerClass {
	GObjectClass parent_class;
	void (*preferences) (ValideAbstractConfigManager* self, ValideWindow* window);
	void (*first_run) (ValideAbstractConfigManager* self);
	char* (*get_config_file) (ValideAbstractConfigManager* self);
};


static char* valide_abstract_config_manager__tmp_dir;
static char* valide_abstract_config_manager__tmp_dir = NULL;
static gpointer valide_abstract_config_manager_parent_class = NULL;

GType valide_abstract_config_manager_get_type (void);
GType valide_window_get_type (void);
enum  {
	VALIDE_ABSTRACT_CONFIG_MANAGER_DUMMY_PROPERTY
};
void valide_abstract_config_manager_preferences (ValideAbstractConfigManager* self, ValideWindow* window);
static void valide_abstract_config_manager_real_preferences (ValideAbstractConfigManager* self, ValideWindow* window);
void valide_abstract_config_manager_first_run (ValideAbstractConfigManager* self);
static void valide_abstract_config_manager_real_first_run (ValideAbstractConfigManager* self);
char* valide_abstract_config_manager_get_config_file (ValideAbstractConfigManager* self);
static char* valide_abstract_config_manager_real_get_config_file (ValideAbstractConfigManager* self);
char* valide_abstract_config_manager_get_home_dir (ValideAbstractConfigManager* self);
char* valide_abstract_config_manager_get_config_dir (ValideAbstractConfigManager* self);
char* valide_abstract_config_manager_get_tmp_dir (ValideAbstractConfigManager* self);
static char* valide_abstract_config_manager_create_file (ValideAbstractConfigManager* self, const char* path, const char* contents);
char* valide_abstract_config_manager_create_preferences_file (ValideAbstractConfigManager* self, const char* filename, const char* contents);
gint valide_abstract_config_manager_get_integer (ValideAbstractConfigManager* self, const char* group, const char* key);
char* valide_abstract_config_manager_get_string (ValideAbstractConfigManager* self, const char* group, const char* key);
gboolean valide_abstract_config_manager_get_boolean (ValideAbstractConfigManager* self, const char* group, const char* key);
void valide_abstract_config_manager_set_integer (ValideAbstractConfigManager* self, const char* group, const char* key, gint val);
void valide_abstract_config_manager_set_string (ValideAbstractConfigManager* self, const char* group, const char* key, const char* val);
void valide_abstract_config_manager_set_boolean (ValideAbstractConfigManager* self, const char* group, const char* key, gboolean val);
static GObject * valide_abstract_config_manager_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void valide_abstract_config_manager_finalize (GObject* obj);



#line 34 "abstract-config-manager.vala"
static void valide_abstract_config_manager_real_preferences (ValideAbstractConfigManager* self, ValideWindow* window) {
#line 104 "abstract-config-manager.c"
	g_return_if_fail (self != NULL);
	g_critical ("Type `%s' does not implement abstract method `valide_abstract_config_manager_preferences'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return;
}


/**
   * Create and show preferences dialog
   *
   * @param window The main window of the application
   */
#line 34 "abstract-config-manager.vala"
void valide_abstract_config_manager_preferences (ValideAbstractConfigManager* self, ValideWindow* window) {
#line 34 "abstract-config-manager.vala"
	VALIDE_ABSTRACT_CONFIG_MANAGER_GET_CLASS (self)->preferences (self, window);
#line 120 "abstract-config-manager.c"
}


#line 39 "abstract-config-manager.vala"
static void valide_abstract_config_manager_real_first_run (ValideAbstractConfigManager* self) {
#line 126 "abstract-config-manager.c"
	g_return_if_fail (self != NULL);
	g_critical ("Type `%s' does not implement abstract method `valide_abstract_config_manager_first_run'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return;
}


/**
   * Function call at first launch
   */
#line 39 "abstract-config-manager.vala"
void valide_abstract_config_manager_first_run (ValideAbstractConfigManager* self) {
#line 39 "abstract-config-manager.vala"
	VALIDE_ABSTRACT_CONFIG_MANAGER_GET_CLASS (self)->first_run (self);
#line 140 "abstract-config-manager.c"
}


#line 44 "abstract-config-manager.vala"
static char* valide_abstract_config_manager_real_get_config_file (ValideAbstractConfigManager* self) {
#line 146 "abstract-config-manager.c"
	g_return_val_if_fail (self != NULL, NULL);
	g_critical ("Type `%s' does not implement abstract method `valide_abstract_config_manager_get_config_file'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return NULL;
}


/**
   * Path of the configuration file
   */
#line 44 "abstract-config-manager.vala"
char* valide_abstract_config_manager_get_config_file (ValideAbstractConfigManager* self) {
#line 44 "abstract-config-manager.vala"
	return VALIDE_ABSTRACT_CONFIG_MANAGER_GET_CLASS (self)->get_config_file (self);
#line 160 "abstract-config-manager.c"
}


/**
   * Path of the user directory.
   */
#line 49 "abstract-config-manager.vala"
char* valide_abstract_config_manager_get_home_dir (ValideAbstractConfigManager* self) {
#line 169 "abstract-config-manager.c"
	char* result;
	char* path;
	char* _tmp1_;
	char* _tmp0_;
#line 49 "abstract-config-manager.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 176 "abstract-config-manager.c"
	path = NULL;
	_tmp1_ = NULL;
#line 820 "glib-2.0.vapi"
	_tmp0_ = NULL;
#line 53 "abstract-config-manager.vala"
	path = (_tmp1_ = g_build_filename (g_get_home_dir (), _tmp0_ = g_strdup_printf (".%s", PACKAGE_NAME), NULL), path = (g_free (path), NULL), _tmp1_);
#line 183 "abstract-config-manager.c"
	_tmp0_ = (g_free (_tmp0_), NULL);
#line 55 "abstract-config-manager.vala"
	if (!g_file_test (path, G_FILE_TEST_EXISTS)) {
#line 57 "abstract-config-manager.vala"
		g_mkdir (path, 0700);
#line 189 "abstract-config-manager.c"
	}
	result = path;
	return result;
}


/**
   * Path of the user config directory.
   */
#line 65 "abstract-config-manager.vala"
char* valide_abstract_config_manager_get_config_dir (ValideAbstractConfigManager* self) {
#line 201 "abstract-config-manager.c"
	char* result;
	char* path;
	char* _tmp0_;
#line 65 "abstract-config-manager.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 207 "abstract-config-manager.c"
	path = NULL;
	_tmp0_ = NULL;
#line 69 "abstract-config-manager.vala"
	path = (_tmp0_ = g_build_filename (g_get_user_config_dir (), PACKAGE_NAME, NULL), path = (g_free (path), NULL), _tmp0_);
#line 71 "abstract-config-manager.vala"
	if (!g_file_test (path, G_FILE_TEST_EXISTS)) {
#line 73 "abstract-config-manager.vala"
		g_mkdir (path, 0700);
#line 216 "abstract-config-manager.c"
	}
	result = path;
	return result;
}


/**
   * Path of the temporary directory.
   */
#line 81 "abstract-config-manager.vala"
char* valide_abstract_config_manager_get_tmp_dir (ValideAbstractConfigManager* self) {
#line 228 "abstract-config-manager.c"
	char* result;
	const char* _tmp1_;
#line 81 "abstract-config-manager.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 83 "abstract-config-manager.vala"
	if (valide_abstract_config_manager__tmp_dir == NULL) {
#line 235 "abstract-config-manager.c"
		char* _tmp0_;
		_tmp0_ = NULL;
#line 85 "abstract-config-manager.vala"
		valide_abstract_config_manager__tmp_dir = (_tmp0_ = g_build_filename (g_get_tmp_dir (), PACKAGE_NAME, NULL), valide_abstract_config_manager__tmp_dir = (g_free (valide_abstract_config_manager__tmp_dir), NULL), _tmp0_);
#line 87 "abstract-config-manager.vala"
		g_mkdir_with_parents (valide_abstract_config_manager__tmp_dir, 0700);
#line 242 "abstract-config-manager.c"
	}
#line 89 "abstract-config-manager.vala"
	_tmp1_ = NULL;
#line 246 "abstract-config-manager.c"
	result = (_tmp1_ = valide_abstract_config_manager__tmp_dir, (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_));
	return result;
}


#line 92 "abstract-config-manager.vala"
static char* valide_abstract_config_manager_create_file (ValideAbstractConfigManager* self, const char* path, const char* contents) {
#line 254 "abstract-config-manager.c"
	char* result;
	const char* _tmp0_;
#line 92 "abstract-config-manager.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 92 "abstract-config-manager.vala"
	g_return_val_if_fail (path != NULL, NULL);
#line 92 "abstract-config-manager.vala"
	g_return_val_if_fail (contents != NULL, NULL);
#line 94 "abstract-config-manager.vala"
	if (!g_file_test (path, G_FILE_TEST_EXISTS)) {
#line 265 "abstract-config-manager.c"
		FILE* stream;
		stream = fopen (path, "w");
#line 97 "abstract-config-manager.vala"
		if (stream != NULL) {
#line 99 "abstract-config-manager.vala"
			fprintf (stream, contents);
#line 272 "abstract-config-manager.c"
		} else {
#line 103 "abstract-config-manager.vala"
			g_warning (_ ("Can't create %s"), path);
#line 276 "abstract-config-manager.c"
		}
		(stream == NULL) ? NULL : (stream = (fclose (stream), NULL));
	}
#line 106 "abstract-config-manager.vala"
	_tmp0_ = NULL;
#line 282 "abstract-config-manager.c"
	result = (_tmp0_ = path, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
	return result;
}


/**
   * Create configuration file if doesn't exist, in user configuration
   * directory #home.
   *
   * @param filename a name of file.
   * @param contents a content of file.
   *
   * @return A path of file.
   */
#line 118 "abstract-config-manager.vala"
char* valide_abstract_config_manager_create_preferences_file (ValideAbstractConfigManager* self, const char* filename, const char* contents) {
#line 299 "abstract-config-manager.c"
	char* result;
	char* _tmp0_;
	char* _tmp1_;
	char* path;
#line 118 "abstract-config-manager.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 118 "abstract-config-manager.vala"
	g_return_val_if_fail (filename != NULL, NULL);
#line 118 "abstract-config-manager.vala"
	g_return_val_if_fail (contents != NULL, NULL);
#line 65 "abstract-config-manager.vala"
	_tmp0_ = NULL;
#line 120 "abstract-config-manager.vala"
	_tmp1_ = NULL;
#line 314 "abstract-config-manager.c"
	path = (_tmp1_ = g_build_filename (_tmp0_ = valide_abstract_config_manager_get_config_dir (self), filename, NULL), _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_);
	result = valide_abstract_config_manager_create_file (self, path, contents);
	path = (g_free (path), NULL);
	return result;
}


/**
   * Return interger from configuration file.
   *
   * @param group a group name.
   * @param key a key.
   *
   * @return the value associated with the key as an integer, or 0 if the key
   *         was not found or could not be parsed.
   */
#line 143 "abstract-config-manager.vala"
gint valide_abstract_config_manager_get_integer (ValideAbstractConfigManager* self, const char* group, const char* key) {
#line 333 "abstract-config-manager.c"
	gint result;
	GError * _inner_error_;
	gint ret;
#line 143 "abstract-config-manager.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 143 "abstract-config-manager.vala"
	g_return_val_if_fail (group != NULL, 0);
#line 143 "abstract-config-manager.vala"
	g_return_val_if_fail (key != NULL, 0);
#line 343 "abstract-config-manager.c"
	_inner_error_ = NULL;
	ret = 0;
	{
		GKeyFile* key_file;
		char* _tmp0_;
		gint _tmp1_;
		key_file = g_key_file_new ();
#line 44 "abstract-config-manager.vala"
		_tmp0_ = NULL;
#line 150 "abstract-config-manager.vala"
		g_key_file_load_from_file (key_file, _tmp0_ = valide_abstract_config_manager_get_config_file (self), G_KEY_FILE_NONE, &_inner_error_);
#line 355 "abstract-config-manager.c"
		if (_inner_error_ != NULL) {
			(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
			goto __catch31_g_error;
			goto __finally31;
		}
		_tmp0_ = (g_free (_tmp0_), NULL);
		_tmp1_ = g_key_file_get_integer (key_file, group, key, &_inner_error_);
		if (_inner_error_ != NULL) {
			(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
			goto __catch31_g_error;
			goto __finally31;
		}
#line 151 "abstract-config-manager.vala"
		ret = _tmp1_;
#line 370 "abstract-config-manager.c"
		(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
	}
	goto __finally31;
	__catch31_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 155 "abstract-config-manager.vala"
			g_debug (_ ("Can't read config.ini file: %s"), e->message);
#line 382 "abstract-config-manager.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally31:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return 0;
	}
	result = ret;
	return result;
}


/**
   * Return string from configuration file.
   *
   * @param group a group name.
   * @param key a key.
   *
   * @return the value associated with the key as a string, or empty string if
   *         the key was not found or could not be parsed.
   */
#line 169 "abstract-config-manager.vala"
char* valide_abstract_config_manager_get_string (ValideAbstractConfigManager* self, const char* group, const char* key) {
#line 408 "abstract-config-manager.c"
	char* result;
	GError * _inner_error_;
	char* ret;
#line 169 "abstract-config-manager.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 169 "abstract-config-manager.vala"
	g_return_val_if_fail (group != NULL, NULL);
#line 169 "abstract-config-manager.vala"
	g_return_val_if_fail (key != NULL, NULL);
#line 418 "abstract-config-manager.c"
	_inner_error_ = NULL;
	ret = g_strdup ("");
	{
		GKeyFile* key_file;
		char* _tmp0_;
		char* _tmp1_;
		char* _tmp2_;
		key_file = g_key_file_new ();
#line 44 "abstract-config-manager.vala"
		_tmp0_ = NULL;
#line 176 "abstract-config-manager.vala"
		g_key_file_load_from_file (key_file, _tmp0_ = valide_abstract_config_manager_get_config_file (self), G_KEY_FILE_NONE, &_inner_error_);
#line 431 "abstract-config-manager.c"
		if (_inner_error_ != NULL) {
			(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
			goto __catch32_g_error;
			goto __finally32;
		}
		_tmp0_ = (g_free (_tmp0_), NULL);
		_tmp1_ = g_key_file_get_value (key_file, group, key, &_inner_error_);
		if (_inner_error_ != NULL) {
			(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
			goto __catch32_g_error;
			goto __finally32;
		}
		_tmp2_ = NULL;
#line 177 "abstract-config-manager.vala"
		ret = (_tmp2_ = _tmp1_, ret = (g_free (ret), NULL), _tmp2_);
#line 447 "abstract-config-manager.c"
		(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
	}
	goto __finally32;
	__catch32_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 181 "abstract-config-manager.vala"
			g_debug (_ ("Can't read config.ini file: %s"), e->message);
#line 459 "abstract-config-manager.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally32:
	if (_inner_error_ != NULL) {
		ret = (g_free (ret), NULL);
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
	result = ret;
	return result;
}


/**
   * Return boolean from configuration file.
   *
   * @param group a group name.
   * @param key a key.
   *
   * @return the value associated with the key as a boolean, or false if the key
   *         was not found or could not be parsed.
   */
#line 195 "abstract-config-manager.vala"
gboolean valide_abstract_config_manager_get_boolean (ValideAbstractConfigManager* self, const char* group, const char* key) {
#line 486 "abstract-config-manager.c"
	gboolean result;
	GError * _inner_error_;
	gboolean ret;
#line 195 "abstract-config-manager.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 195 "abstract-config-manager.vala"
	g_return_val_if_fail (group != NULL, FALSE);
#line 195 "abstract-config-manager.vala"
	g_return_val_if_fail (key != NULL, FALSE);
#line 496 "abstract-config-manager.c"
	_inner_error_ = NULL;
	ret = FALSE;
	{
		GKeyFile* key_file;
		char* _tmp0_;
		gboolean _tmp1_;
		key_file = g_key_file_new ();
#line 44 "abstract-config-manager.vala"
		_tmp0_ = NULL;
#line 202 "abstract-config-manager.vala"
		g_key_file_load_from_file (key_file, _tmp0_ = valide_abstract_config_manager_get_config_file (self), G_KEY_FILE_NONE, &_inner_error_);
#line 508 "abstract-config-manager.c"
		if (_inner_error_ != NULL) {
			(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
			goto __catch33_g_error;
			goto __finally33;
		}
		_tmp0_ = (g_free (_tmp0_), NULL);
		_tmp1_ = g_key_file_get_boolean (key_file, group, key, &_inner_error_);
		if (_inner_error_ != NULL) {
			(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
			goto __catch33_g_error;
			goto __finally33;
		}
#line 203 "abstract-config-manager.vala"
		ret = _tmp1_;
#line 523 "abstract-config-manager.c"
		(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
	}
	goto __finally33;
	__catch33_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 207 "abstract-config-manager.vala"
			g_debug (_ ("Can't read config.ini file: %s"), e->message);
#line 535 "abstract-config-manager.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally33:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return FALSE;
	}
	result = ret;
	return result;
}


/**
   * Associates a new integer value with #key under #group. If key cannot be
   * found then it is created.
   *
   * @param group a group name.
   * @param key a key.
   * @param val an integer value.
   */
#line 220 "abstract-config-manager.vala"
void valide_abstract_config_manager_set_integer (ValideAbstractConfigManager* self, const char* group, const char* key, gint val) {
#line 560 "abstract-config-manager.c"
	GError * _inner_error_;
#line 220 "abstract-config-manager.vala"
	g_return_if_fail (self != NULL);
#line 220 "abstract-config-manager.vala"
	g_return_if_fail (group != NULL);
#line 220 "abstract-config-manager.vala"
	g_return_if_fail (key != NULL);
#line 568 "abstract-config-manager.c"
	_inner_error_ = NULL;
	{
		GKeyFile* key_file;
		char* _tmp0_;
		char* _tmp2_;
		char* _tmp1_;
		key_file = g_key_file_new ();
#line 44 "abstract-config-manager.vala"
		_tmp0_ = NULL;
#line 226 "abstract-config-manager.vala"
		g_key_file_load_from_file (key_file, _tmp0_ = valide_abstract_config_manager_get_config_file (self), G_KEY_FILE_NONE, &_inner_error_);
#line 580 "abstract-config-manager.c"
		if (_inner_error_ != NULL) {
			(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
			goto __catch34_g_error;
			goto __finally34;
		}
		_tmp0_ = (g_free (_tmp0_), NULL);
#line 227 "abstract-config-manager.vala"
		g_key_file_set_integer (key_file, group, key, val);
#line 2793 "glib-2.0.vapi"
		_tmp2_ = NULL;
#line 44 "abstract-config-manager.vala"
		_tmp1_ = NULL;
#line 228 "abstract-config-manager.vala"
		g_file_set_contents (_tmp1_ = valide_abstract_config_manager_get_config_file (self), _tmp2_ = g_key_file_to_data (key_file, NULL, NULL), (glong) (-1), &_inner_error_);
#line 595 "abstract-config-manager.c"
		if (_inner_error_ != NULL) {
			(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
			goto __catch34_g_error;
			goto __finally34;
		}
		_tmp2_ = (g_free (_tmp2_), NULL);
		_tmp1_ = (g_free (_tmp1_), NULL);
		(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
	}
	goto __finally34;
	__catch34_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 232 "abstract-config-manager.vala"
			g_critical (_ ("Can't read config.ini file: %s"), e->message);
#line 614 "abstract-config-manager.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally34:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
}


/**
   * Associates a new string value with #key under #group. If key cannot be
   * found then it is created.
   *
   * @param group a group name.
   * @param key a key.
   * @param val a string value.
   */
#line 244 "abstract-config-manager.vala"
void valide_abstract_config_manager_set_string (ValideAbstractConfigManager* self, const char* group, const char* key, const char* val) {
#line 637 "abstract-config-manager.c"
	GError * _inner_error_;
#line 244 "abstract-config-manager.vala"
	g_return_if_fail (self != NULL);
#line 244 "abstract-config-manager.vala"
	g_return_if_fail (group != NULL);
#line 244 "abstract-config-manager.vala"
	g_return_if_fail (key != NULL);
#line 244 "abstract-config-manager.vala"
	g_return_if_fail (val != NULL);
#line 647 "abstract-config-manager.c"
	_inner_error_ = NULL;
	{
		GKeyFile* key_file;
		char* _tmp0_;
		char* _tmp2_;
		char* _tmp1_;
		key_file = g_key_file_new ();
#line 44 "abstract-config-manager.vala"
		_tmp0_ = NULL;
#line 250 "abstract-config-manager.vala"
		g_key_file_load_from_file (key_file, _tmp0_ = valide_abstract_config_manager_get_config_file (self), G_KEY_FILE_NONE, &_inner_error_);
#line 659 "abstract-config-manager.c"
		if (_inner_error_ != NULL) {
			(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
			goto __catch35_g_error;
			goto __finally35;
		}
		_tmp0_ = (g_free (_tmp0_), NULL);
#line 251 "abstract-config-manager.vala"
		g_key_file_set_string (key_file, group, key, val);
#line 2793 "glib-2.0.vapi"
		_tmp2_ = NULL;
#line 44 "abstract-config-manager.vala"
		_tmp1_ = NULL;
#line 252 "abstract-config-manager.vala"
		g_file_set_contents (_tmp1_ = valide_abstract_config_manager_get_config_file (self), _tmp2_ = g_key_file_to_data (key_file, NULL, NULL), (glong) (-1), &_inner_error_);
#line 674 "abstract-config-manager.c"
		if (_inner_error_ != NULL) {
			(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
			goto __catch35_g_error;
			goto __finally35;
		}
		_tmp2_ = (g_free (_tmp2_), NULL);
		_tmp1_ = (g_free (_tmp1_), NULL);
		(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
	}
	goto __finally35;
	__catch35_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 256 "abstract-config-manager.vala"
			g_critical (_ ("Can't read config.ini file: %s"), e->message);
#line 693 "abstract-config-manager.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally35:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
}


/**
   * Associates a new boolean value with #key under #group. If key cannot be
   * found then it is created.
   *
   * @param group a group name.
   * @param key a key.
   * @param val a boolean value.
   */
#line 268 "abstract-config-manager.vala"
void valide_abstract_config_manager_set_boolean (ValideAbstractConfigManager* self, const char* group, const char* key, gboolean val) {
#line 716 "abstract-config-manager.c"
	GError * _inner_error_;
#line 268 "abstract-config-manager.vala"
	g_return_if_fail (self != NULL);
#line 268 "abstract-config-manager.vala"
	g_return_if_fail (group != NULL);
#line 268 "abstract-config-manager.vala"
	g_return_if_fail (key != NULL);
#line 724 "abstract-config-manager.c"
	_inner_error_ = NULL;
	{
		GKeyFile* key_file;
		char* _tmp0_;
		char* _tmp2_;
		char* _tmp1_;
		key_file = g_key_file_new ();
#line 44 "abstract-config-manager.vala"
		_tmp0_ = NULL;
#line 274 "abstract-config-manager.vala"
		g_key_file_load_from_file (key_file, _tmp0_ = valide_abstract_config_manager_get_config_file (self), G_KEY_FILE_NONE, &_inner_error_);
#line 736 "abstract-config-manager.c"
		if (_inner_error_ != NULL) {
			(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
			goto __catch36_g_error;
			goto __finally36;
		}
		_tmp0_ = (g_free (_tmp0_), NULL);
#line 275 "abstract-config-manager.vala"
		g_key_file_set_boolean (key_file, group, key, val);
#line 2793 "glib-2.0.vapi"
		_tmp2_ = NULL;
#line 44 "abstract-config-manager.vala"
		_tmp1_ = NULL;
#line 276 "abstract-config-manager.vala"
		g_file_set_contents (_tmp1_ = valide_abstract_config_manager_get_config_file (self), _tmp2_ = g_key_file_to_data (key_file, NULL, NULL), (glong) (-1), &_inner_error_);
#line 751 "abstract-config-manager.c"
		if (_inner_error_ != NULL) {
			(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
			goto __catch36_g_error;
			goto __finally36;
		}
		_tmp2_ = (g_free (_tmp2_), NULL);
		_tmp1_ = (g_free (_tmp1_), NULL);
		(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
	}
	goto __finally36;
	__catch36_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 280 "abstract-config-manager.vala"
			g_critical (_ ("Can't read config.ini file: %s"), e->message);
#line 770 "abstract-config-manager.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally36:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
}


static GObject * valide_abstract_config_manager_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValideAbstractConfigManagerClass * klass;
	GObjectClass * parent_class;
	ValideAbstractConfigManager * self;
	klass = VALIDE_ABSTRACT_CONFIG_MANAGER_CLASS (g_type_class_peek (VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_ABSTRACT_CONFIG_MANAGER (obj);
	{
		char* _tmp0_;
		gboolean _tmp1_;
#line 49 "abstract-config-manager.vala"
		_tmp0_ = NULL;
#line 127 "abstract-config-manager.vala"
		if ((_tmp1_ = !g_file_test (_tmp0_ = valide_abstract_config_manager_get_home_dir (self), G_FILE_TEST_EXISTS), _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_)) {
#line 799 "abstract-config-manager.c"
			char* _tmp2_;
#line 49 "abstract-config-manager.vala"
			_tmp2_ = NULL;
#line 129 "abstract-config-manager.vala"
			g_mkdir (_tmp2_ = valide_abstract_config_manager_get_home_dir (self), 0700);
#line 805 "abstract-config-manager.c"
			_tmp2_ = (g_free (_tmp2_), NULL);
#line 130 "abstract-config-manager.vala"
			valide_abstract_config_manager_first_run (self);
#line 809 "abstract-config-manager.c"
		}
	}
	return obj;
}


static void valide_abstract_config_manager_class_init (ValideAbstractConfigManagerClass * klass) {
	valide_abstract_config_manager_parent_class = g_type_class_peek_parent (klass);
	VALIDE_ABSTRACT_CONFIG_MANAGER_CLASS (klass)->preferences = valide_abstract_config_manager_real_preferences;
	VALIDE_ABSTRACT_CONFIG_MANAGER_CLASS (klass)->first_run = valide_abstract_config_manager_real_first_run;
	VALIDE_ABSTRACT_CONFIG_MANAGER_CLASS (klass)->get_config_file = valide_abstract_config_manager_real_get_config_file;
	G_OBJECT_CLASS (klass)->constructor = valide_abstract_config_manager_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_abstract_config_manager_finalize;
}


static void valide_abstract_config_manager_instance_init (ValideAbstractConfigManager * self) {
}


static void valide_abstract_config_manager_finalize (GObject* obj) {
	ValideAbstractConfigManager * self;
	self = VALIDE_ABSTRACT_CONFIG_MANAGER (obj);
	G_OBJECT_CLASS (valide_abstract_config_manager_parent_class)->finalize (obj);
}


GType valide_abstract_config_manager_get_type (void) {
	static GType valide_abstract_config_manager_type_id = 0;
	if (valide_abstract_config_manager_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideAbstractConfigManagerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_abstract_config_manager_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideAbstractConfigManager), 0, (GInstanceInitFunc) valide_abstract_config_manager_instance_init, NULL };
		valide_abstract_config_manager_type_id = g_type_register_static (G_TYPE_OBJECT, "ValideAbstractConfigManager", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	}
	return valide_abstract_config_manager_type_id;
}




