/* abstract-document.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <glib/gi18n-lib.h>
#include <gtksourceview/gtksourceview.h>
#include <float.h>
#include <math.h>
#include <glib/gstdio.h>
#include <gio/gio.h>
#include <gdk/gdk.h>


#define VALIDE_TYPE_IDOCUMENT (valide_idocument_get_type ())
#define VALIDE_IDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_IDOCUMENT, ValideiDocument))
#define VALIDE_IS_IDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_IDOCUMENT))
#define VALIDE_IDOCUMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), VALIDE_TYPE_IDOCUMENT, ValideiDocumentIface))

typedef struct _ValideiDocument ValideiDocument;
typedef struct _ValideiDocumentIface ValideiDocumentIface;

#define VALIDE_TYPE_UI_MANAGER (valide_ui_manager_get_type ())
#define VALIDE_UI_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_UI_MANAGER, ValideUIManager))
#define VALIDE_UI_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_UI_MANAGER, ValideUIManagerClass))
#define VALIDE_IS_UI_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_UI_MANAGER))
#define VALIDE_IS_UI_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_UI_MANAGER))
#define VALIDE_UI_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_UI_MANAGER, ValideUIManagerClass))

typedef struct _ValideUIManager ValideUIManager;
typedef struct _ValideUIManagerClass ValideUIManagerClass;

#define VALIDE_TYPE_DOCUMENT_STATE (valide_document_state_get_type ())

#define VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT (valide_abstract_text_document_get_type ())
#define VALIDE_ABSTRACT_TEXT_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocument))
#define VALIDE_ABSTRACT_TEXT_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocumentClass))
#define VALIDE_IS_ABSTRACT_TEXT_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT))
#define VALIDE_IS_ABSTRACT_TEXT_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT))
#define VALIDE_ABSTRACT_TEXT_DOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocumentClass))

typedef struct _ValideAbstractTextDocument ValideAbstractTextDocument;
typedef struct _ValideAbstractTextDocumentClass ValideAbstractTextDocumentClass;
typedef struct _ValideAbstractTextDocumentPrivate ValideAbstractTextDocumentPrivate;

#define VALIDE_TYPE_SPLIT_SOURCE_VIEW (valide_split_source_view_get_type ())
#define VALIDE_SPLIT_SOURCE_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_SPLIT_SOURCE_VIEW, ValideSplitSourceView))
#define VALIDE_SPLIT_SOURCE_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_SPLIT_SOURCE_VIEW, ValideSplitSourceViewClass))
#define VALIDE_IS_SPLIT_SOURCE_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_SPLIT_SOURCE_VIEW))
#define VALIDE_IS_SPLIT_SOURCE_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_SPLIT_SOURCE_VIEW))
#define VALIDE_SPLIT_SOURCE_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_SPLIT_SOURCE_VIEW, ValideSplitSourceViewClass))

typedef struct _ValideSplitSourceView ValideSplitSourceView;
typedef struct _ValideSplitSourceViewClass ValideSplitSourceViewClass;

#define VALIDE_TYPE_SEARCHING (valide_searching_get_type ())
#define VALIDE_SEARCHING(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_SEARCHING, ValideSearching))
#define VALIDE_SEARCHING_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_SEARCHING, ValideSearchingClass))
#define VALIDE_IS_SEARCHING(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_SEARCHING))
#define VALIDE_IS_SEARCHING_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_SEARCHING))
#define VALIDE_SEARCHING_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_SEARCHING, ValideSearchingClass))

typedef struct _ValideSearching ValideSearching;
typedef struct _ValideSearchingClass ValideSearchingClass;

#define VALIDE_TYPE_DOCUMENT_MESSAGE (valide_document_message_get_type ())
#define VALIDE_DOCUMENT_MESSAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_DOCUMENT_MESSAGE, ValideDocumentMessage))
#define VALIDE_DOCUMENT_MESSAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_DOCUMENT_MESSAGE, ValideDocumentMessageClass))
#define VALIDE_IS_DOCUMENT_MESSAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_DOCUMENT_MESSAGE))
#define VALIDE_IS_DOCUMENT_MESSAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_DOCUMENT_MESSAGE))
#define VALIDE_DOCUMENT_MESSAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_DOCUMENT_MESSAGE, ValideDocumentMessageClass))

typedef struct _ValideDocumentMessage ValideDocumentMessage;
typedef struct _ValideDocumentMessageClass ValideDocumentMessageClass;

#define VALIDE_TYPE_SOURCE_BUFFER (valide_source_buffer_get_type ())
#define VALIDE_SOURCE_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_SOURCE_BUFFER, ValideSourceBuffer))
#define VALIDE_SOURCE_BUFFER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_SOURCE_BUFFER, ValideSourceBufferClass))
#define VALIDE_IS_SOURCE_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_SOURCE_BUFFER))
#define VALIDE_IS_SOURCE_BUFFER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_SOURCE_BUFFER))
#define VALIDE_SOURCE_BUFFER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_SOURCE_BUFFER, ValideSourceBufferClass))

typedef struct _ValideSourceBuffer ValideSourceBuffer;
typedef struct _ValideSourceBufferClass ValideSourceBufferClass;

#define VALIDE_TYPE_SOURCE_VIEW (valide_source_view_get_type ())
#define VALIDE_SOURCE_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_SOURCE_VIEW, ValideSourceView))
#define VALIDE_SOURCE_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_SOURCE_VIEW, ValideSourceViewClass))
#define VALIDE_IS_SOURCE_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_SOURCE_VIEW))
#define VALIDE_IS_SOURCE_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_SOURCE_VIEW))
#define VALIDE_SOURCE_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_SOURCE_VIEW, ValideSourceViewClass))

typedef struct _ValideSourceView ValideSourceView;
typedef struct _ValideSourceViewClass ValideSourceViewClass;

#define VALIDE_TYPE_POSITION (valide_position_get_type ())
typedef struct _ValidePosition ValidePosition;
typedef struct _ValideSourceBufferPrivate ValideSourceBufferPrivate;

typedef enum  {
	VALIDE_DOCUMENT_STATE_NORMAL,
	VALIDE_DOCUMENT_STATE_EXTERNALLY_MODIFIED,
	VALIDE_DOCUMENT_STATE_BAD_ENCODING,
	VALIDE_DOCUMENT_STATE_INFO
} ValideDocumentState;

/**
 * An interface for document
 */
struct _ValideiDocumentIface {
	GTypeInterface parent_iface;
	void (*open) (ValideiDocument* self, const char* path, GError** error);
	void (*save) (ValideiDocument* self, const char* current_folder);
	void (*save_as) (ValideiDocument* self, const char* current_folder);
	void (*rename) (ValideiDocument* self);
	gboolean (*close) (ValideiDocument* self);
	GdkPixbuf* (*get_icon) (ValideiDocument* self);
	char* (*get_tooltips) (ValideiDocument* self);
	void (*setup_ui) (ValideiDocument* self, ValideUIManager* ui_manager);
	ValideDocumentState (*get_state) (ValideiDocument* self);
	void (*set_state) (ValideiDocument* self, ValideDocumentState value);
	gboolean (*get_is_save) (ValideiDocument* self);
	void (*set_is_save) (ValideiDocument* self, gboolean value);
	const char* (*get_path) (ValideiDocument* self);
	void (*set_path) (ValideiDocument* self, const char* value);
	const char* (*get_title) (ValideiDocument* self);
	const char* (*get_filename) (ValideiDocument* self);
};

/**
 * The base class for text document
 */
struct _ValideAbstractTextDocument {
	GtkVBox parent_instance;
	ValideAbstractTextDocumentPrivate * priv;
	ValideSplitSourceView* split_view;
};

struct _ValideAbstractTextDocumentClass {
	GtkVBoxClass parent_class;
	ValideSplitSourceView* (*create_source_view) (ValideAbstractTextDocument* self);
	char* (*get_contents) (ValideAbstractTextDocument* self, GError** error);
	void (*set_contents) (ValideAbstractTextDocument* self, const char* new_path, GError** error);
	GdkPixbuf* (*get_icon) (ValideAbstractTextDocument* self);
	char* (*get_tooltips) (ValideAbstractTextDocument* self);
};

struct _ValideAbstractTextDocumentPrivate {
	gboolean _save;
	guint64 mtime;
	ValideSearching* searching;
	ValideDocumentState _state;
	char* _selected_text;
	char* _filename;
	char* _title;
};

typedef enum  {
	VALIDE_DOCUMENT_ERROR_UNKNOW_DOCUMENT,
	VALIDE_DOCUMENT_ERROR_BAD_URI,
	VALIDE_DOCUMENT_ERROR_NO_DOCUMENT,
	VALIDE_DOCUMENT_ERROR_MIME
} ValideDocumentError;
#define VALIDE_DOCUMENT_ERROR valide_document_error_quark ()
/**
 * A class for represente a rectangular position
 */
struct _ValidePosition {
	gint start_line;
	gint end_line;
	gint start_row;
	gint end_row;
};

struct _ValideSourceBuffer {
	GtkSourceBuffer parent_instance;
	ValideSourceBufferPrivate * priv;
	char* path;
};

struct _ValideSourceBufferClass {
	GtkSourceBufferClass parent_class;
};


static gpointer valide_abstract_text_document_parent_class = NULL;
static ValideiDocumentIface* valide_abstract_text_document_valide_idocument_parent_iface = NULL;

GType valide_ui_manager_get_type (void);
GType valide_document_state_get_type (void);
GType valide_idocument_get_type (void);
GType valide_abstract_text_document_get_type (void);
GType valide_split_source_view_get_type (void);
GType valide_searching_get_type (void);
#define VALIDE_ABSTRACT_TEXT_DOCUMENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocumentPrivate))
enum  {
	VALIDE_ABSTRACT_TEXT_DOCUMENT_DUMMY_PROPERTY,
	VALIDE_ABSTRACT_TEXT_DOCUMENT_BUFFER,
	VALIDE_ABSTRACT_TEXT_DOCUMENT_TEXT_VIEW,
	VALIDE_ABSTRACT_TEXT_DOCUMENT_IS_SAVE,
	VALIDE_ABSTRACT_TEXT_DOCUMENT_STATE,
	VALIDE_ABSTRACT_TEXT_DOCUMENT_EDITABLE,
	VALIDE_ABSTRACT_TEXT_DOCUMENT_SELECTED_TEXT,
	VALIDE_ABSTRACT_TEXT_DOCUMENT_PATH,
	VALIDE_ABSTRACT_TEXT_DOCUMENT_FILENAME,
	VALIDE_ABSTRACT_TEXT_DOCUMENT_TITLE
};
ValideSplitSourceView* valide_abstract_text_document_create_source_view (ValideAbstractTextDocument* self);
static ValideSplitSourceView* valide_abstract_text_document_real_create_source_view (ValideAbstractTextDocument* self);
const char* valide_idocument_get_path (ValideiDocument* self);
ValideDocumentState valide_idocument_get_state (ValideiDocument* self);
guint64 valide_utils_get_mtime (const char* filename, GError** error);
GType valide_document_message_get_type (void);
void valide_idocument_set_state (ValideiDocument* self, ValideDocumentState value);
ValideDocumentMessage* valide_document_message_new (ValideAbstractTextDocument* document);
ValideDocumentMessage* valide_document_message_construct (GType object_type, ValideAbstractTextDocument* document);
static void valide_abstract_text_document_check_externally_modified (ValideAbstractTextDocument* self);
GType valide_source_buffer_get_type (void);
ValideSourceBuffer* valide_abstract_text_document_get_buffer (ValideAbstractTextDocument* self);
GType valide_source_view_get_type (void);
ValideSourceView* valide_abstract_text_document_get_text_view (ValideAbstractTextDocument* self);
static void valide_abstract_text_document_scroll_to_iter (ValideAbstractTextDocument* self, const GtkTextIter* iter);
static void valide_abstract_text_document_cursor_move (ValideAbstractTextDocument* self);
gboolean valide_searching_find (ValideSearching* self, gboolean show_replace, gboolean show_dialog);
static void valide_abstract_text_document_on_action_search (ValideAbstractTextDocument* self);
gboolean valide_searching_find_next (ValideSearching* self);
static void valide_abstract_text_document_on_action_find_next (ValideAbstractTextDocument* self);
gboolean valide_searching_find_prev (ValideSearching* self);
static void valide_abstract_text_document_on_action_find_prev (ValideAbstractTextDocument* self);
void valide_searching_replace (ValideSearching* self, gboolean show_dialog);
static void valide_abstract_text_document_on_action_replace (ValideAbstractTextDocument* self);
char* valide_source_buffer_get_contents (ValideSourceBuffer* self, GError** error);
char* valide_abstract_text_document_get_contents (ValideAbstractTextDocument* self, GError** error);
static char* valide_abstract_text_document_real_get_contents (ValideAbstractTextDocument* self, GError** error);
void valide_source_buffer_set_contents (ValideSourceBuffer* self, const char* new_path, GError** error);
void valide_abstract_text_document_set_contents (ValideAbstractTextDocument* self, const char* new_path, GError** error);
static void valide_abstract_text_document_real_set_contents (ValideAbstractTextDocument* self, const char* new_path, GError** error);
void valide_abstract_text_document_grab_focus (ValideAbstractTextDocument* self);
void valide_abstract_text_document_show (ValideAbstractTextDocument* self);
static void valide_abstract_text_document_real_show_all (GtkWidget* base);
void valide_idocument_set_is_save (ValideiDocument* self, gboolean value);
void valide_abstract_text_document_reload (ValideAbstractTextDocument* self);
void valide_idocument_set_path (ValideiDocument* self, const char* value);
GQuark valide_document_error_quark (void);
static void valide_abstract_text_document_real_open (ValideiDocument* base, const char* path, GError** error);
gboolean valide_idocument_get_is_save (ValideiDocument* self);
static void valide_abstract_text_document_real_save (ValideiDocument* base, const char* current_folder);
static void valide_abstract_text_document_real_save_as (ValideiDocument* base, const char* current_folder);
static void _lambda75_ (GtkEntry* s, ValideAbstractTextDocument* self);
static void __lambda75__gtk_entry_activate (GtkEntry* _sender, gpointer self);
static void valide_abstract_text_document_real_rename (ValideiDocument* base);
void valide_idocument_save (ValideiDocument* self, const char* current_folder);
static gboolean valide_abstract_text_document_real_close (ValideiDocument* base);
void valide_abstract_text_document_select_offsets (ValideAbstractTextDocument* self, gint start, gint end);
GdkPixbuf* valide_utils_get_pixbuf_for_file (const char* filename, GtkIconSize size, GError** error);
GdkPixbuf* valide_utils_get_pixbuf_for_stock (const char* stock_id, GtkIconSize size);
GdkPixbuf* valide_abstract_text_document_get_icon (ValideAbstractTextDocument* self);
static GdkPixbuf* valide_abstract_text_document_real_get_icon (ValideAbstractTextDocument* self);
char* valide_utils_get_mime_type (const char* filename, GError** error);
const char* valide_source_buffer_get_encoding (ValideSourceBuffer* self);
char* valide_abstract_text_document_get_tooltips (ValideAbstractTextDocument* self);
static char* valide_abstract_text_document_real_get_tooltips (ValideAbstractTextDocument* self);
GtkAction* valide_ui_manager_get_action (ValideUIManager* self, const char* name);
static void valide_abstract_text_document_real_setup_ui (ValideiDocument* base, ValideUIManager* ui_manager);
GType valide_position_get_type (void);
ValidePosition* valide_position_dup (const ValidePosition* self);
void valide_position_free (ValidePosition* self);
void valide_abstract_text_document_select_range (ValideAbstractTextDocument* self, const ValidePosition* pos);
ValideSourceBuffer* valide_source_view_get_buffer (ValideSourceView* self);
ValideSourceView* valide_split_source_view_get_active_view (ValideSplitSourceView* self);
void valide_abstract_text_document_set_editable (ValideAbstractTextDocument* self, gboolean value);
gboolean valide_abstract_text_document_get_editable (ValideAbstractTextDocument* self);
const char* valide_abstract_text_document_get_selected_text (ValideAbstractTextDocument* self);
const char* valide_idocument_get_filename (ValideiDocument* self);
static void _lambda76_ (ValideAbstractTextDocument* self);
static void __lambda76__gtk_text_buffer_changed (GtkTextBuffer* _sender, gpointer self);
static void _lambda77_ (ValideAbstractTextDocument* self);
static void __lambda77__gtk_text_buffer_modified_changed (GtkTextBuffer* _sender, gpointer self);
static void _lambda78_ (ValideAbstractTextDocument* self);
static void __lambda78__gtk_text_buffer_mark_set (GtkTextBuffer* _sender, const GtkTextIter* location, GtkTextMark* mark, gpointer self);
ValideSearching* valide_searching_new (ValideAbstractTextDocument* document);
ValideSearching* valide_searching_construct (GType object_type, ValideAbstractTextDocument* document);
static gboolean _lambda79_ (ValideAbstractTextDocument* self);
static gboolean __lambda79__gtk_widget_focus_in_event (GtkWidget* _sender, const GdkEventFocus* event, gpointer self);
static void _valide_abstract_text_document_on_action_search_valide_split_source_view_search (ValideSplitSourceView* _sender, gpointer self);
static void _valide_abstract_text_document_on_action_find_next_valide_split_source_view_find_next (ValideSplitSourceView* _sender, gpointer self);
static void _valide_abstract_text_document_on_action_find_prev_valide_split_source_view_find_prev (ValideSplitSourceView* _sender, gpointer self);
static void _valide_abstract_text_document_on_action_replace_valide_split_source_view_replace (ValideSplitSourceView* _sender, gpointer self);
static GObject * valide_abstract_text_document_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void valide_abstract_text_document_finalize (GObject* obj);
static void valide_abstract_text_document_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_abstract_text_document_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static int _vala_strcmp0 (const char * str1, const char * str2);


static void g_cclosure_user_marshal_VOID__INT_INT (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data);

#line 46 "abstract-text-document.vala"
static ValideSplitSourceView* valide_abstract_text_document_real_create_source_view (ValideAbstractTextDocument* self) {
#line 323 "abstract-text-document.c"
	g_return_val_if_fail (self != NULL, NULL);
	g_critical ("Type `%s' does not implement abstract method `valide_abstract_text_document_create_source_view'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return NULL;
}


/**
   * The function create an appropriete source view for display a content of a
   * file
   */
#line 46 "abstract-text-document.vala"
ValideSplitSourceView* valide_abstract_text_document_create_source_view (ValideAbstractTextDocument* self) {
#line 46 "abstract-text-document.vala"
	return VALIDE_ABSTRACT_TEXT_DOCUMENT_GET_CLASS (self)->create_source_view (self);
#line 338 "abstract-text-document.c"
}


#line 210 "abstract-text-document.vala"
static void valide_abstract_text_document_check_externally_modified (ValideAbstractTextDocument* self) {
#line 344 "abstract-text-document.c"
	GError * _inner_error_;
	gboolean _tmp0_;
#line 210 "abstract-text-document.vala"
	g_return_if_fail (self != NULL);
#line 349 "abstract-text-document.c"
	_inner_error_ = NULL;
	_tmp0_ = FALSE;
#line 212 "abstract-text-document.vala"
	if (_vala_strcmp0 (valide_idocument_get_path ((ValideiDocument*) self), _ ("New file")) != 0) {
#line 212 "abstract-text-document.vala"
		_tmp0_ = valide_idocument_get_state ((ValideiDocument*) self) == VALIDE_DOCUMENT_STATE_NORMAL;
#line 356 "abstract-text-document.c"
	} else {
#line 212 "abstract-text-document.vala"
		_tmp0_ = FALSE;
#line 360 "abstract-text-document.c"
	}
#line 212 "abstract-text-document.vala"
	if (_tmp0_) {
#line 364 "abstract-text-document.c"
		guint64 _tmp1_;
		_tmp1_ = valide_utils_get_mtime (valide_idocument_get_path ((ValideiDocument*) self), &_inner_error_);
		if (_inner_error_ != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
#line 214 "abstract-text-document.vala"
		if (self->priv->mtime < _tmp1_) {
#line 374 "abstract-text-document.c"
			ValideDocumentMessage* msg;
			ValideDocumentMessage* _tmp2_;
			msg = NULL;
#line 218 "abstract-text-document.vala"
			valide_idocument_set_state ((ValideiDocument*) self, VALIDE_DOCUMENT_STATE_EXTERNALLY_MODIFIED);
#line 380 "abstract-text-document.c"
			_tmp2_ = NULL;
#line 219 "abstract-text-document.vala"
			msg = (_tmp2_ = g_object_ref_sink (valide_document_message_new (self)), (msg == NULL) ? NULL : (msg = (g_object_unref (msg), NULL)), _tmp2_);
#line 220 "abstract-text-document.vala"
			gtk_box_pack_start ((GtkBox*) self, (GtkWidget*) msg, FALSE, TRUE, (guint) 0);
#line 221 "abstract-text-document.vala"
			gtk_box_reorder_child ((GtkBox*) self, (GtkWidget*) msg, 0);
#line 222 "abstract-text-document.vala"
			gtk_widget_show_all ((GtkWidget*) msg);
#line 390 "abstract-text-document.c"
			(msg == NULL) ? NULL : (msg = (g_object_unref (msg), NULL));
		}
	}
}


#line 227 "abstract-text-document.vala"
static void valide_abstract_text_document_scroll_to_iter (ValideAbstractTextDocument* self, const GtkTextIter* iter) {
#line 399 "abstract-text-document.c"
	GtkTextMark* mark;
	GtkTextMark* _tmp1_;
	GtkTextMark* _tmp0_;
#line 227 "abstract-text-document.vala"
	g_return_if_fail (self != NULL);
#line 405 "abstract-text-document.c"
	mark = NULL;
	_tmp1_ = NULL;
#line 231 "abstract-text-document.vala"
	_tmp0_ = NULL;
#line 231 "abstract-text-document.vala"
	mark = (_tmp1_ = (_tmp0_ = gtk_text_buffer_create_mark ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (self), "cursor", &(*iter), TRUE), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (mark == NULL) ? NULL : (mark = (g_object_unref (mark), NULL)), _tmp1_);
#line 232 "abstract-text-document.vala"
	gtk_text_view_scroll_to_mark ((GtkTextView*) valide_abstract_text_document_get_text_view (self), mark, 0.4, FALSE, (double) 0, (double) 0);
#line 233 "abstract-text-document.vala"
	gtk_text_buffer_delete_mark ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (self), mark);
#line 416 "abstract-text-document.c"
	(mark == NULL) ? NULL : (mark = (g_object_unref (mark), NULL));
}


#line 236 "abstract-text-document.vala"
static void valide_abstract_text_document_cursor_move (ValideAbstractTextDocument* self) {
#line 423 "abstract-text-document.c"
	gint row;
	gint col;
	GtkTextIter iter = {0};
	GtkTextIter start = {0};
	guint tab_size;
#line 236 "abstract-text-document.vala"
	g_return_if_fail (self != NULL);
#line 431 "abstract-text-document.c"
	row = 0;
	col = 0;
	tab_size = 0U;
#line 244 "abstract-text-document.vala"
	gtk_text_buffer_get_iter_at_mark ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (self), &iter, gtk_text_buffer_get_insert ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (self)));
#line 246 "abstract-text-document.vala"
	row = gtk_text_iter_get_line (&iter);
#line 248 "abstract-text-document.vala"
	start = iter;
#line 249 "abstract-text-document.vala"
	gtk_text_iter_set_line_offset (&start, 0);
#line 250 "abstract-text-document.vala"
	col = 0;
#line 252 "abstract-text-document.vala"
	tab_size = gtk_source_view_get_tab_width ((GtkSourceView*) valide_abstract_text_document_get_text_view (self));
#line 254 "abstract-text-document.vala"
	while (TRUE) {
#line 254 "abstract-text-document.vala"
		if (!(!gtk_text_iter_equal (&start, &iter))) {
#line 254 "abstract-text-document.vala"
			break;
#line 453 "abstract-text-document.c"
		}
		/* FIXME: Are we Unicode compliant here? */
#line 257 "abstract-text-document.vala"
		if (gtk_text_iter_get_char (&start) == '\t') {
#line 259 "abstract-text-document.vala"
			col = col + ((gint) (tab_size - (col % tab_size)));
#line 460 "abstract-text-document.c"
		} else {
#line 263 "abstract-text-document.vala"
			col++;
#line 464 "abstract-text-document.c"
		}
#line 266 "abstract-text-document.vala"
		gtk_text_iter_forward_char (&start);
#line 468 "abstract-text-document.c"
	}
#line 269 "abstract-text-document.vala"
	g_signal_emit_by_name (self, "cursor-moved", row + 1, col + 1);
#line 472 "abstract-text-document.c"
}


#line 272 "abstract-text-document.vala"
static void valide_abstract_text_document_on_action_search (ValideAbstractTextDocument* self) {
#line 272 "abstract-text-document.vala"
	g_return_if_fail (self != NULL);
#line 274 "abstract-text-document.vala"
	valide_searching_find (self->priv->searching, FALSE, TRUE);
#line 482 "abstract-text-document.c"
}


#line 277 "abstract-text-document.vala"
static void valide_abstract_text_document_on_action_find_next (ValideAbstractTextDocument* self) {
#line 277 "abstract-text-document.vala"
	g_return_if_fail (self != NULL);
#line 279 "abstract-text-document.vala"
	valide_searching_find_next (self->priv->searching);
#line 492 "abstract-text-document.c"
}


#line 282 "abstract-text-document.vala"
static void valide_abstract_text_document_on_action_find_prev (ValideAbstractTextDocument* self) {
#line 282 "abstract-text-document.vala"
	g_return_if_fail (self != NULL);
#line 284 "abstract-text-document.vala"
	valide_searching_find_prev (self->priv->searching);
#line 502 "abstract-text-document.c"
}


#line 287 "abstract-text-document.vala"
static void valide_abstract_text_document_on_action_replace (ValideAbstractTextDocument* self) {
#line 287 "abstract-text-document.vala"
	g_return_if_fail (self != NULL);
#line 289 "abstract-text-document.vala"
	valide_searching_replace (self->priv->searching, TRUE);
#line 512 "abstract-text-document.c"
}


/**
   * Return the entire file content in UTF-8 encoding
   *
   * @return The content of the document
   */
#line 297 "abstract-text-document.vala"
static char* valide_abstract_text_document_real_get_contents (ValideAbstractTextDocument* self, GError** error) {
#line 523 "abstract-text-document.c"
	char* result;
	GError * _inner_error_;
	char* _tmp0_;
#line 297 "abstract-text-document.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 529 "abstract-text-document.c"
	_inner_error_ = NULL;
	_tmp0_ = valide_source_buffer_get_contents (valide_abstract_text_document_get_buffer (self), &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return NULL;
	}
	result = _tmp0_;
	return result;
}


#line 297 "abstract-text-document.vala"
char* valide_abstract_text_document_get_contents (ValideAbstractTextDocument* self, GError** error) {
#line 297 "abstract-text-document.vala"
	return VALIDE_ABSTRACT_TEXT_DOCUMENT_GET_CLASS (self)->get_contents (self, error);
#line 545 "abstract-text-document.c"
}


/**
   * Write the entire document content in file
   */
#line 305 "abstract-text-document.vala"
static void valide_abstract_text_document_real_set_contents (ValideAbstractTextDocument* self, const char* new_path, GError** error) {
#line 554 "abstract-text-document.c"
	GError * _inner_error_;
#line 305 "abstract-text-document.vala"
	g_return_if_fail (self != NULL);
#line 558 "abstract-text-document.c"
	_inner_error_ = NULL;
#line 307 "abstract-text-document.vala"
	valide_source_buffer_set_contents (valide_abstract_text_document_get_buffer (self), new_path, &_inner_error_);
#line 562 "abstract-text-document.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return;
	}
}


#line 305 "abstract-text-document.vala"
void valide_abstract_text_document_set_contents (ValideAbstractTextDocument* self, const char* new_path, GError** error) {
#line 305 "abstract-text-document.vala"
	VALIDE_ABSTRACT_TEXT_DOCUMENT_GET_CLASS (self)->set_contents (self, new_path, error);
#line 574 "abstract-text-document.c"
}


/**
   * @see Gtk.Widget.grab_focus
   */
#line 354 "abstract-text-document.vala"
void valide_abstract_text_document_grab_focus (ValideAbstractTextDocument* self) {
#line 354 "abstract-text-document.vala"
	g_return_if_fail (self != NULL);
#line 356 "abstract-text-document.vala"
	gtk_widget_grab_focus ((GtkWidget*) valide_abstract_text_document_get_text_view (self));
#line 587 "abstract-text-document.c"
}


/**
   * @see Gtk.Widget.show
   */
#line 362 "abstract-text-document.vala"
void valide_abstract_text_document_show (ValideAbstractTextDocument* self) {
#line 362 "abstract-text-document.vala"
	g_return_if_fail (self != NULL);
#line 364 "abstract-text-document.vala"
	gtk_widget_show_all ((GtkWidget*) self);
#line 365 "abstract-text-document.vala"
	gtk_widget_hide ((GtkWidget*) self->priv->searching);
#line 602 "abstract-text-document.c"
}


/**
   * @see Gtk.Widget.show_all
   */
#line 371 "abstract-text-document.vala"
static void valide_abstract_text_document_real_show_all (GtkWidget* base) {
#line 611 "abstract-text-document.c"
	ValideAbstractTextDocument * self;
	self = (ValideAbstractTextDocument*) base;
#line 373 "abstract-text-document.vala"
	GTK_WIDGET_CLASS (valide_abstract_text_document_parent_class)->show_all ((GtkWidget*) GTK_VBOX (self));
#line 374 "abstract-text-document.vala"
	gtk_widget_hide ((GtkWidget*) self->priv->searching);
#line 618 "abstract-text-document.c"
}


/**
   * Refresh the content of the widget with the hard disk file
   */
#line 380 "abstract-text-document.vala"
void valide_abstract_text_document_reload (ValideAbstractTextDocument* self) {
#line 627 "abstract-text-document.c"
	GError * _inner_error_;
	char* contents;
#line 380 "abstract-text-document.vala"
	g_return_if_fail (self != NULL);
#line 632 "abstract-text-document.c"
	_inner_error_ = NULL;
	contents = g_strdup ("");
	{
		char* _tmp1_;
		char* _tmp2_;
		_tmp1_ = valide_abstract_text_document_get_contents (self, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch39_g_error;
			goto __finally39;
		}
		_tmp2_ = NULL;
#line 386 "abstract-text-document.vala"
		contents = (_tmp2_ = _tmp1_, contents = (g_free (contents), NULL), _tmp2_);
#line 646 "abstract-text-document.c"
	}
	goto __finally39;
	__catch39_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			ValideDocumentMessage* msg;
			ValideDocumentMessage* _tmp3_;
			msg = NULL;
#line 392 "abstract-text-document.vala"
			valide_idocument_set_state ((ValideiDocument*) self, VALIDE_DOCUMENT_STATE_BAD_ENCODING);
#line 660 "abstract-text-document.c"
			_tmp3_ = NULL;
#line 393 "abstract-text-document.vala"
			msg = (_tmp3_ = g_object_ref_sink (valide_document_message_new (self)), (msg == NULL) ? NULL : (msg = (g_object_unref (msg), NULL)), _tmp3_);
#line 394 "abstract-text-document.vala"
			gtk_box_pack_start ((GtkBox*) self, (GtkWidget*) msg, FALSE, TRUE, (guint) 0);
#line 395 "abstract-text-document.vala"
			gtk_box_reorder_child ((GtkBox*) self, (GtkWidget*) msg, 0);
#line 396 "abstract-text-document.vala"
			gtk_widget_show_all ((GtkWidget*) msg);
#line 670 "abstract-text-document.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			(msg == NULL) ? NULL : (msg = (g_object_unref (msg), NULL));
		}
	}
	__finally39:
	{
		GtkTextIter start = {0};
		GtkTextIter end = {0};
#line 403 "abstract-text-document.vala"
		gtk_text_buffer_get_start_iter ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (self), &start);
#line 404 "abstract-text-document.vala"
		gtk_text_buffer_get_end_iter ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (self), &end);
#line 405 "abstract-text-document.vala"
		gtk_source_buffer_begin_not_undoable_action ((GtkSourceBuffer*) valide_abstract_text_document_get_buffer (self));
#line 406 "abstract-text-document.vala"
		gtk_text_buffer_delete ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (self), &start, &end);
#line 407 "abstract-text-document.vala"
		gtk_text_buffer_insert ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (self), &start, contents, -1);
#line 408 "abstract-text-document.vala"
		gtk_text_buffer_set_modified ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (self), FALSE);
#line 409 "abstract-text-document.vala"
		gtk_source_buffer_end_not_undoable_action ((GtkSourceBuffer*) valide_abstract_text_document_get_buffer (self));
#line 410 "abstract-text-document.vala"
		valide_idocument_set_is_save ((ValideiDocument*) self, TRUE);
#line 695 "abstract-text-document.c"
		{
			guint64 _tmp0_;
			_tmp0_ = valide_utils_get_mtime (valide_idocument_get_path ((ValideiDocument*) self), &_inner_error_);
			if (_inner_error_ != NULL) {
				goto __catch40_g_error;
				goto __finally40;
			}
#line 413 "abstract-text-document.vala"
			self->priv->mtime = _tmp0_;
#line 705 "abstract-text-document.c"
		}
		goto __finally40;
		__catch40_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
#line 417 "abstract-text-document.vala"
				g_debug (e->message);
#line 716 "abstract-text-document.c"
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally40:
		if (_inner_error_ != NULL) {
			contents = (g_free (contents), NULL);
			goto __finally39;
		}
	}
	if (_inner_error_ != NULL) {
		contents = (g_free (contents), NULL);
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	contents = (g_free (contents), NULL);
}


/**
   * @see Valide.iDocument.open
   */
#line 425 "abstract-text-document.vala"
static void valide_abstract_text_document_real_open (ValideiDocument* base, const char* path, GError** error) {
#line 741 "abstract-text-document.c"
	ValideAbstractTextDocument * self;
	GError * _inner_error_;
	self = (ValideAbstractTextDocument*) base;
#line 425 "abstract-text-document.vala"
	g_return_if_fail (path != NULL);
#line 747 "abstract-text-document.c"
	_inner_error_ = NULL;
#line 427 "abstract-text-document.vala"
	if (g_file_test (path, G_FILE_TEST_EXISTS)) {
#line 429 "abstract-text-document.vala"
		valide_idocument_set_path ((ValideiDocument*) self, path);
#line 430 "abstract-text-document.vala"
		valide_abstract_text_document_reload (self);
#line 755 "abstract-text-document.c"
	} else {
		_inner_error_ = g_error_new (VALIDE_DOCUMENT_ERROR, VALIDE_DOCUMENT_ERROR_BAD_URI, _ ("cannot open file '%s': No such file or directory"), path);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == VALIDE_DOCUMENT_ERROR) {
				g_propagate_error (error, _inner_error_);
				return;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return;
			}
		}
	}
}


/**
   * @see Valide.iDocument.save
   */
#line 441 "abstract-text-document.vala"
static void valide_abstract_text_document_real_save (ValideiDocument* base, const char* current_folder) {
#line 777 "abstract-text-document.c"
	ValideAbstractTextDocument * self;
	GError * _inner_error_;
	self = (ValideAbstractTextDocument*) base;
	_inner_error_ = NULL;
#line 443 "abstract-text-document.vala"
	if (!valide_idocument_get_is_save ((ValideiDocument*) self)) {
#line 784 "abstract-text-document.c"
		gint response;
		response = (gint) GTK_RESPONSE_ACCEPT;
#line 447 "abstract-text-document.vala"
		if (_vala_strcmp0 (valide_idocument_get_path ((ValideiDocument*) self), _ ("New file")) == 0) {
#line 789 "abstract-text-document.c"
			GtkFileChooserDialog* dialog;
			GtkFileChooserDialog* _tmp0_;
			dialog = NULL;
			_tmp0_ = NULL;
#line 451 "abstract-text-document.vala"
			dialog = (_tmp0_ = g_object_ref_sink ((GtkFileChooserDialog*) gtk_file_chooser_dialog_new (_ ("Save file"), NULL, GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL, NULL)), (dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL)), _tmp0_);
#line 457 "abstract-text-document.vala"
			if (current_folder != NULL) {
#line 459 "abstract-text-document.vala"
				gtk_file_chooser_set_current_folder ((GtkFileChooser*) dialog, current_folder);
#line 800 "abstract-text-document.c"
			}
#line 461 "abstract-text-document.vala"
			response = gtk_dialog_run ((GtkDialog*) dialog);
#line 462 "abstract-text-document.vala"
			if (response == GTK_RESPONSE_ACCEPT) {
#line 806 "abstract-text-document.c"
				char* _tmp1_;
#line 5545 "gtk+-2.0.vapi"
				_tmp1_ = NULL;
#line 464 "abstract-text-document.vala"
				valide_idocument_set_path ((ValideiDocument*) self, _tmp1_ = gtk_file_chooser_get_filename ((GtkFileChooser*) dialog));
#line 812 "abstract-text-document.c"
				_tmp1_ = (g_free (_tmp1_), NULL);
			}
#line 466 "abstract-text-document.vala"
			gtk_object_destroy ((GtkObject*) dialog);
#line 817 "abstract-text-document.c"
			(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
		}
#line 468 "abstract-text-document.vala"
		if (response == GTK_RESPONSE_ACCEPT) {
#line 822 "abstract-text-document.c"
			{
				guint64 _tmp2_;
#line 472 "abstract-text-document.vala"
				valide_abstract_text_document_set_contents (self, NULL, &_inner_error_);
#line 827 "abstract-text-document.c"
				if (_inner_error_ != NULL) {
					goto __catch41_g_error;
					goto __finally41;
				}
#line 473 "abstract-text-document.vala"
				gtk_text_buffer_set_modified ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (self), FALSE);
#line 834 "abstract-text-document.c"
				_tmp2_ = valide_utils_get_mtime (valide_idocument_get_path ((ValideiDocument*) self), &_inner_error_);
				if (_inner_error_ != NULL) {
					goto __catch41_g_error;
					goto __finally41;
				}
#line 474 "abstract-text-document.vala"
				self->priv->mtime = _tmp2_;
#line 475 "abstract-text-document.vala"
				valide_idocument_set_is_save ((ValideiDocument*) self, TRUE);
#line 844 "abstract-text-document.c"
			}
			goto __finally41;
			__catch41_g_error:
			{
				GError * e;
				e = _inner_error_;
				_inner_error_ = NULL;
				{
#line 479 "abstract-text-document.vala"
					g_warning (e->message);
#line 855 "abstract-text-document.c"
					(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
				}
			}
			__finally41:
			if (_inner_error_ != NULL) {
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return;
			}
		}
	}
}


/**
   * @see Valide.iDocument.save_as
   */
#line 488 "abstract-text-document.vala"
static void valide_abstract_text_document_real_save_as (ValideiDocument* base, const char* current_folder) {
#line 875 "abstract-text-document.c"
	ValideAbstractTextDocument * self;
	GError * _inner_error_;
	char* path;
	gint response;
	GtkFileChooserDialog* dialog;
	GtkFileChooserDialog* _tmp0_;
	self = (ValideAbstractTextDocument*) base;
	_inner_error_ = NULL;
	path = NULL;
	response = (gint) GTK_RESPONSE_ACCEPT;
	dialog = NULL;
	_tmp0_ = NULL;
#line 495 "abstract-text-document.vala"
	dialog = (_tmp0_ = g_object_ref_sink ((GtkFileChooserDialog*) gtk_file_chooser_dialog_new (_ ("Save as file"), NULL, GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL, NULL)), (dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL)), _tmp0_);
#line 501 "abstract-text-document.vala"
	if (current_folder != NULL) {
#line 503 "abstract-text-document.vala"
		gtk_file_chooser_set_current_folder ((GtkFileChooser*) dialog, current_folder);
#line 894 "abstract-text-document.c"
	}
#line 505 "abstract-text-document.vala"
	response = gtk_dialog_run ((GtkDialog*) dialog);
#line 506 "abstract-text-document.vala"
	if (response == GTK_RESPONSE_ACCEPT) {
#line 900 "abstract-text-document.c"
		char* _tmp1_;
		_tmp1_ = NULL;
#line 508 "abstract-text-document.vala"
		path = (_tmp1_ = gtk_file_chooser_get_filename ((GtkFileChooser*) dialog), path = (g_free (path), NULL), _tmp1_);
#line 905 "abstract-text-document.c"
	}
#line 510 "abstract-text-document.vala"
	gtk_object_destroy ((GtkObject*) dialog);
#line 512 "abstract-text-document.vala"
	if (response == GTK_RESPONSE_ACCEPT) {
#line 911 "abstract-text-document.c"
		{
			guint64 _tmp2_;
#line 516 "abstract-text-document.vala"
			valide_abstract_text_document_set_contents (self, path, &_inner_error_);
#line 916 "abstract-text-document.c"
			if (_inner_error_ != NULL) {
				goto __catch42_g_error;
				goto __finally42;
			}
#line 517 "abstract-text-document.vala"
			valide_idocument_set_path ((ValideiDocument*) self, path);
#line 518 "abstract-text-document.vala"
			gtk_text_buffer_set_modified ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (self), FALSE);
#line 925 "abstract-text-document.c"
			_tmp2_ = valide_utils_get_mtime (valide_idocument_get_path ((ValideiDocument*) self), &_inner_error_);
			if (_inner_error_ != NULL) {
				goto __catch42_g_error;
				goto __finally42;
			}
#line 519 "abstract-text-document.vala"
			self->priv->mtime = _tmp2_;
#line 520 "abstract-text-document.vala"
			valide_idocument_set_is_save ((ValideiDocument*) self, TRUE);
#line 935 "abstract-text-document.c"
		}
		goto __finally42;
		__catch42_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
#line 524 "abstract-text-document.vala"
				g_warning (e->message);
#line 946 "abstract-text-document.c"
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally42:
		if (_inner_error_ != NULL) {
			path = (g_free (path), NULL);
			(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
	path = (g_free (path), NULL);
	(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
}


static void _lambda75_ (GtkEntry* s, ValideAbstractTextDocument* self) {
	GtkDialog* dlg;
	GtkDialog* _tmp2_;
	GtkDialog* _tmp1_;
	void* _tmp0_;
	g_return_if_fail (s != NULL);
	dlg = NULL;
	_tmp2_ = NULL;
#line 551 "abstract-text-document.vala"
	_tmp1_ = NULL;
#line 551 "abstract-text-document.vala"
	_tmp0_ = NULL;
#line 551 "abstract-text-document.vala"
	dlg = (_tmp2_ = (_tmp1_ = (_tmp0_ = g_object_get_data ((GObject*) s, "dialog"), GTK_IS_DIALOG (_tmp0_) ? ((GtkDialog*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (dlg == NULL) ? NULL : (dlg = (g_object_unref (dlg), NULL)), _tmp2_);
#line 552 "abstract-text-document.vala"
	gtk_dialog_response (dlg, (gint) GTK_RESPONSE_APPLY);
#line 980 "abstract-text-document.c"
	(dlg == NULL) ? NULL : (dlg = (g_object_unref (dlg), NULL));
}


static void __lambda75__gtk_entry_activate (GtkEntry* _sender, gpointer self) {
	_lambda75_ (_sender, self);
}


/**
   * @see Valide.iDocument.rename
   */
#line 532 "abstract-text-document.vala"
static void valide_abstract_text_document_real_rename (ValideiDocument* base) {
#line 995 "abstract-text-document.c"
	ValideAbstractTextDocument * self;
	GtkEntry* entry;
	GtkDialog* dialog;
	GtkWindow* parent;
	GtkDialog* _tmp0_;
	GtkWindow* _tmp3_;
	GtkWindow* _tmp2_;
	GtkWidget* _tmp1_;
	GtkEntry* _tmp4_;
	self = (ValideAbstractTextDocument*) base;
	entry = NULL;
	dialog = NULL;
	parent = NULL;
	_tmp0_ = NULL;
#line 538 "abstract-text-document.vala"
	dialog = (_tmp0_ = g_object_ref_sink ((GtkDialog*) gtk_dialog_new ()), (dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL)), _tmp0_);
#line 539 "abstract-text-document.vala"
	gtk_window_set_title ((GtkWindow*) dialog, _ ("Rename"));
#line 1014 "abstract-text-document.c"
	_tmp3_ = NULL;
#line 540 "abstract-text-document.vala"
	_tmp2_ = NULL;
#line 540 "abstract-text-document.vala"
	_tmp1_ = NULL;
#line 540 "abstract-text-document.vala"
	parent = (_tmp3_ = (_tmp2_ = (_tmp1_ = gtk_widget_get_ancestor ((GtkWidget*) self, GTK_TYPE_WINDOW), GTK_IS_WINDOW (_tmp1_) ? ((GtkWindow*) _tmp1_) : NULL), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (parent == NULL) ? NULL : (parent = (g_object_unref (parent), NULL)), _tmp3_);
#line 541 "abstract-text-document.vala"
	gtk_window_set_transient_for ((GtkWindow*) dialog, parent);
#line 542 "abstract-text-document.vala"
	gtk_window_set_modal ((GtkWindow*) dialog, TRUE);
#line 543 "abstract-text-document.vala"
	gtk_dialog_add_button (dialog, GTK_STOCK_APPLY, (gint) GTK_RESPONSE_APPLY);
#line 544 "abstract-text-document.vala"
	gtk_dialog_add_button (dialog, GTK_STOCK_CANCEL, (gint) GTK_RESPONSE_CANCEL);
#line 1030 "abstract-text-document.c"
	_tmp4_ = NULL;
#line 546 "abstract-text-document.vala"
	entry = (_tmp4_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL)), _tmp4_);
#line 547 "abstract-text-document.vala"
	g_object_set_data ((GObject*) entry, "dialog", dialog);
#line 548 "abstract-text-document.vala"
	g_signal_connect (entry, "activate", (GCallback) __lambda75__gtk_entry_activate, self);
#line 554 "abstract-text-document.vala"
	gtk_entry_set_text (entry, valide_idocument_get_path ((ValideiDocument*) self));
#line 555 "abstract-text-document.vala"
	gtk_box_pack_start ((GtkBox*) dialog->vbox, (GtkWidget*) entry, TRUE, TRUE, (guint) 10);
#line 557 "abstract-text-document.vala"
	gtk_widget_show_all ((GtkWidget*) dialog);
#line 558 "abstract-text-document.vala"
	if (gtk_dialog_run (dialog) == GTK_RESPONSE_APPLY) {
#line 1046 "abstract-text-document.c"
		char* new_path;
		char* _tmp6_;
		const char* _tmp5_;
		new_path = NULL;
		_tmp6_ = NULL;
#line 562 "abstract-text-document.vala"
		_tmp5_ = NULL;
#line 562 "abstract-text-document.vala"
		new_path = (_tmp6_ = (_tmp5_ = gtk_entry_get_text (entry), (_tmp5_ == NULL) ? NULL : g_strdup (_tmp5_)), new_path = (g_free (new_path), NULL), _tmp6_);
#line 563 "abstract-text-document.vala"
		g_rename (valide_idocument_get_path ((ValideiDocument*) self), new_path);
#line 564 "abstract-text-document.vala"
		valide_idocument_set_path ((ValideiDocument*) self, new_path);
#line 1060 "abstract-text-document.c"
		new_path = (g_free (new_path), NULL);
	}
#line 566 "abstract-text-document.vala"
	gtk_object_destroy ((GtkObject*) dialog);
#line 1065 "abstract-text-document.c"
	(entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL));
	(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
	(parent == NULL) ? NULL : (parent = (g_object_unref (parent), NULL));
}


/**
   * @see Valide.iDocument.close
   */
#line 572 "abstract-text-document.vala"
static gboolean valide_abstract_text_document_real_close (ValideiDocument* base) {
#line 1077 "abstract-text-document.c"
	ValideAbstractTextDocument * self;
	gboolean result;
	gboolean is_close;
	self = (ValideAbstractTextDocument*) base;
	is_close = TRUE;
#line 576 "abstract-text-document.vala"
	if (!valide_idocument_get_is_save ((ValideiDocument*) self)) {
#line 1085 "abstract-text-document.c"
		GtkWindow* parent;
		GtkDialog* dialog;
		GtkLabel* label;
		gint response;
		GtkDialog* _tmp0_;
		GtkWindow* _tmp3_;
		GtkWindow* _tmp2_;
		GtkWidget* _tmp1_;
		GtkLabel* _tmp4_;
		parent = NULL;
		dialog = NULL;
		label = NULL;
		response = 0;
		_tmp0_ = NULL;
#line 583 "abstract-text-document.vala"
		dialog = (_tmp0_ = g_object_ref_sink ((GtkDialog*) gtk_dialog_new ()), (dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL)), _tmp0_);
#line 584 "abstract-text-document.vala"
		gtk_window_set_title ((GtkWindow*) dialog, _ ("Are you sure?"));
#line 585 "abstract-text-document.vala"
		gtk_dialog_add_button (dialog, GTK_STOCK_YES, (gint) GTK_RESPONSE_YES);
#line 586 "abstract-text-document.vala"
		gtk_dialog_add_button (dialog, GTK_STOCK_NO, (gint) GTK_RESPONSE_NO);
#line 587 "abstract-text-document.vala"
		gtk_dialog_add_button (dialog, GTK_STOCK_CANCEL, (gint) GTK_RESPONSE_CANCEL);
#line 588 "abstract-text-document.vala"
		gtk_window_set_modal ((GtkWindow*) dialog, TRUE);
#line 1112 "abstract-text-document.c"
		_tmp3_ = NULL;
#line 589 "abstract-text-document.vala"
		_tmp2_ = NULL;
#line 589 "abstract-text-document.vala"
		_tmp1_ = NULL;
#line 589 "abstract-text-document.vala"
		parent = (_tmp3_ = (_tmp2_ = (_tmp1_ = gtk_widget_get_ancestor ((GtkWidget*) self, GTK_TYPE_WINDOW), GTK_IS_WINDOW (_tmp1_) ? ((GtkWindow*) _tmp1_) : NULL), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (parent == NULL) ? NULL : (parent = (g_object_unref (parent), NULL)), _tmp3_);
#line 590 "abstract-text-document.vala"
		gtk_window_set_transient_for ((GtkWindow*) dialog, parent);
#line 1122 "abstract-text-document.c"
		_tmp4_ = NULL;
#line 592 "abstract-text-document.vala"
		label = (_tmp4_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("The document has unsaved changes. Save changes before closing?"))), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp4_);
#line 593 "abstract-text-document.vala"
		gtk_box_pack_start ((GtkBox*) dialog->vbox, (GtkWidget*) label, TRUE, FALSE, (guint) 10);
#line 594 "abstract-text-document.vala"
		gtk_widget_show_all ((GtkWidget*) dialog);
#line 595 "abstract-text-document.vala"
		response = gtk_dialog_run (dialog);
#line 596 "abstract-text-document.vala"
		gtk_object_destroy ((GtkObject*) dialog);
#line 597 "abstract-text-document.vala"
		switch (response) {
#line 1136 "abstract-text-document.c"
			case GTK_RESPONSE_YES:
			{
#line 600 "abstract-text-document.vala"
				valide_idocument_save ((ValideiDocument*) self, NULL);
#line 601 "abstract-text-document.vala"
				is_close = valide_idocument_get_is_save ((ValideiDocument*) self);
#line 602 "abstract-text-document.vala"
				break;
#line 1145 "abstract-text-document.c"
			}
			case GTK_RESPONSE_NO:
			{
#line 604 "abstract-text-document.vala"
				break;
#line 1151 "abstract-text-document.c"
			}
			case GTK_RESPONSE_CANCEL:
			{
#line 606 "abstract-text-document.vala"
				is_close = FALSE;
#line 607 "abstract-text-document.vala"
				break;
#line 1159 "abstract-text-document.c"
			}
		}
		(parent == NULL) ? NULL : (parent = (g_object_unref (parent), NULL));
		(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
		(label == NULL) ? NULL : (label = (g_object_unref (label), NULL));
	}
#line 610 "abstract-text-document.vala"
	if (is_close) {
#line 612 "abstract-text-document.vala"
		g_signal_emit_by_name ((ValideiDocument*) self, "closed");
#line 1170 "abstract-text-document.c"
	}
	result = is_close;
	return result;
}


/**
   * Select the specified offsets
   *
   * @param start Char offset from start of the buffer
   * @param end Char offset from start of the buffer
   *
   */
#line 624 "abstract-text-document.vala"
void valide_abstract_text_document_select_offsets (ValideAbstractTextDocument* self, gint start, gint end) {
#line 1186 "abstract-text-document.c"
	GtkTextIter end_iter = {0};
	GtkTextIter start_iter = {0};
#line 624 "abstract-text-document.vala"
	g_return_if_fail (self != NULL);
#line 629 "abstract-text-document.vala"
	gtk_text_buffer_get_iter_at_offset ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (self), &start_iter, start);
#line 630 "abstract-text-document.vala"
	gtk_text_buffer_get_iter_at_offset ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (self), &end_iter, end);
#line 631 "abstract-text-document.vala"
	gtk_text_buffer_select_range ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (self), &start_iter, &end_iter);
#line 632 "abstract-text-document.vala"
	valide_abstract_text_document_scroll_to_iter (self, &start_iter);
#line 1199 "abstract-text-document.c"
}


/**
   * @see Valide.iDocument.get_icon
   */
#line 638 "abstract-text-document.vala"
static GdkPixbuf* valide_abstract_text_document_real_get_icon (ValideAbstractTextDocument* self) {
#line 1208 "abstract-text-document.c"
	GdkPixbuf* result;
	GError * _inner_error_;
	GdkPixbuf* pixbuf;
#line 638 "abstract-text-document.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 1214 "abstract-text-document.c"
	_inner_error_ = NULL;
	pixbuf = NULL;
#line 642 "abstract-text-document.vala"
	switch (valide_idocument_get_state ((ValideiDocument*) self)) {
#line 1219 "abstract-text-document.c"
		case VALIDE_DOCUMENT_STATE_NORMAL:
		{
			{
				GdkPixbuf* _tmp0_;
				GdkPixbuf* _tmp1_;
				_tmp0_ = valide_utils_get_pixbuf_for_file (valide_idocument_get_path ((ValideiDocument*) self), GTK_ICON_SIZE_MENU, &_inner_error_);
				if (_inner_error_ != NULL) {
					goto __catch43_g_error;
					goto __finally43;
				}
				_tmp1_ = NULL;
#line 647 "abstract-text-document.vala"
				pixbuf = (_tmp1_ = _tmp0_, (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp1_);
#line 1233 "abstract-text-document.c"
			}
			goto __finally43;
			__catch43_g_error:
			{
				GError * e;
				e = _inner_error_;
				_inner_error_ = NULL;
				{
					GdkPixbuf* _tmp2_;
					_tmp2_ = NULL;
#line 651 "abstract-text-document.vala"
					pixbuf = (_tmp2_ = valide_utils_get_pixbuf_for_stock (GTK_STOCK_FILE, GTK_ICON_SIZE_MENU), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp2_);
#line 1246 "abstract-text-document.c"
					(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
				}
			}
			__finally43:
			if (_inner_error_ != NULL) {
				(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return NULL;
			}
#line 653 "abstract-text-document.vala"
			break;
#line 1259 "abstract-text-document.c"
		}
		case VALIDE_DOCUMENT_STATE_EXTERNALLY_MODIFIED:
		{
			GdkPixbuf* _tmp3_;
			_tmp3_ = NULL;
#line 655 "abstract-text-document.vala"
			pixbuf = (_tmp3_ = valide_utils_get_pixbuf_for_stock (GTK_STOCK_DIALOG_WARNING, GTK_ICON_SIZE_MENU), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp3_);
#line 656 "abstract-text-document.vala"
			break;
#line 1269 "abstract-text-document.c"
		}
		case VALIDE_DOCUMENT_STATE_BAD_ENCODING:
		{
			GdkPixbuf* _tmp4_;
			_tmp4_ = NULL;
#line 658 "abstract-text-document.vala"
			pixbuf = (_tmp4_ = valide_utils_get_pixbuf_for_stock (GTK_STOCK_DIALOG_ERROR, GTK_ICON_SIZE_MENU), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp4_);
#line 659 "abstract-text-document.vala"
			break;
#line 1279 "abstract-text-document.c"
		}
		default:
		{
			GdkPixbuf* _tmp5_;
			_tmp5_ = NULL;
#line 661 "abstract-text-document.vala"
			pixbuf = (_tmp5_ = valide_utils_get_pixbuf_for_stock (GTK_STOCK_DIALOG_QUESTION, GTK_ICON_SIZE_MENU), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp5_);
#line 662 "abstract-text-document.vala"
			break;
#line 1289 "abstract-text-document.c"
		}
	}
	result = pixbuf;
	return result;
}


#line 638 "abstract-text-document.vala"
GdkPixbuf* valide_abstract_text_document_get_icon (ValideAbstractTextDocument* self) {
#line 638 "abstract-text-document.vala"
	return VALIDE_ABSTRACT_TEXT_DOCUMENT_GET_CLASS (self)->get_icon (self);
#line 1301 "abstract-text-document.c"
}


/**
   * @see Valide.iDocument.get_tooltips
   */
#line 670 "abstract-text-document.vala"
static char* valide_abstract_text_document_real_get_tooltips (ValideAbstractTextDocument* self) {
#line 1310 "abstract-text-document.c"
	char* result;
	GError * _inner_error_;
	char* tooltips;
#line 670 "abstract-text-document.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 1316 "abstract-text-document.c"
	_inner_error_ = NULL;
	tooltips = NULL;
#line 674 "abstract-text-document.vala"
	if (_vala_strcmp0 (valide_idocument_get_path ((ValideiDocument*) self), _ ("New file")) != 0) {
#line 1321 "abstract-text-document.c"
		char* mime_type;
		char* content_type;
		char* content_description;
		char* _tmp9_;
		mime_type = NULL;
		content_type = NULL;
		content_description = NULL;
		{
			char* _tmp0_;
			char* _tmp1_;
			char* _tmp3_;
			const char* _tmp2_;
			_tmp0_ = valide_utils_get_mime_type (valide_idocument_get_path ((ValideiDocument*) self), &_inner_error_);
			if (_inner_error_ != NULL) {
				goto __catch44_g_error;
				goto __finally44;
			}
			_tmp1_ = NULL;
#line 682 "abstract-text-document.vala"
			content_type = (_tmp1_ = _tmp0_, content_type = (g_free (content_type), NULL), _tmp1_);
#line 1342 "abstract-text-document.c"
			_tmp3_ = NULL;
#line 683 "abstract-text-document.vala"
			_tmp2_ = NULL;
#line 683 "abstract-text-document.vala"
			content_description = (_tmp3_ = (_tmp2_ = g_content_type_get_description (content_type), (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)), content_description = (g_free (content_description), NULL), _tmp3_);
#line 1348 "abstract-text-document.c"
		}
		goto __finally44;
		__catch44_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
				char* _tmp5_;
				const char* _tmp4_;
#line 687 "abstract-text-document.vala"
				g_debug (e->message);
#line 1361 "abstract-text-document.c"
				_tmp5_ = NULL;
#line 688 "abstract-text-document.vala"
				_tmp4_ = NULL;
#line 688 "abstract-text-document.vala"
				content_type = (_tmp5_ = (_tmp4_ = _ ("Unknow"), (_tmp4_ == NULL) ? NULL : g_strdup (_tmp4_)), content_type = (g_free (content_type), NULL), _tmp5_);
#line 1367 "abstract-text-document.c"
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally44:
		if (_inner_error_ != NULL) {
			mime_type = (g_free (mime_type), NULL);
			content_type = (g_free (content_type), NULL);
			content_description = (g_free (content_description), NULL);
			tooltips = (g_free (tooltips), NULL);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return NULL;
		}
#line 690 "abstract-text-document.vala"
		if (content_description != NULL) {
#line 1383 "abstract-text-document.c"
			char* _tmp6_;
			_tmp6_ = NULL;
#line 692 "abstract-text-document.vala"
			mime_type = (_tmp6_ = g_strdup_printf ("%s (%s)", content_description, content_type), mime_type = (g_free (mime_type), NULL), _tmp6_);
#line 1388 "abstract-text-document.c"
		} else {
			char* _tmp8_;
			const char* _tmp7_;
			_tmp8_ = NULL;
#line 696 "abstract-text-document.vala"
			_tmp7_ = NULL;
#line 696 "abstract-text-document.vala"
			mime_type = (_tmp8_ = (_tmp7_ = content_type, (_tmp7_ == NULL) ? NULL : g_strdup (_tmp7_)), mime_type = (g_free (mime_type), NULL), _tmp8_);
#line 1397 "abstract-text-document.c"
		}
		_tmp9_ = NULL;
#line 699 "abstract-text-document.vala"
		tooltips = (_tmp9_ = g_strdup_printf ("<b>%s</b> %s\n\n<b>%s</b> %s\n<b>%s</b> %s", _ ("Name:"), valide_idocument_get_path ((ValideiDocument*) self), _ ("MIME Type:"), mime_type, _ ("Encoding:"), valide_source_buffer_get_encoding (valide_abstract_text_document_get_buffer (self))), tooltips = (g_free (tooltips), NULL), _tmp9_);
#line 1402 "abstract-text-document.c"
		mime_type = (g_free (mime_type), NULL);
		content_type = (g_free (content_type), NULL);
		content_description = (g_free (content_description), NULL);
	}
	result = tooltips;
	return result;
}


#line 670 "abstract-text-document.vala"
char* valide_abstract_text_document_get_tooltips (ValideAbstractTextDocument* self) {
#line 670 "abstract-text-document.vala"
	return VALIDE_ABSTRACT_TEXT_DOCUMENT_GET_CLASS (self)->get_tooltips (self);
#line 1416 "abstract-text-document.c"
}


/**
   * @see Valide.iDocument.setup_ui
   */
#line 709 "abstract-text-document.vala"
static void valide_abstract_text_document_real_setup_ui (ValideiDocument* base, ValideUIManager* ui_manager) {
#line 1425 "abstract-text-document.c"
	ValideAbstractTextDocument * self;
	GtkAction* _tmp1_;
	gboolean _tmp0_;
	GtkAction* _tmp3_;
	gboolean _tmp2_;
	self = (ValideAbstractTextDocument*) base;
#line 709 "abstract-text-document.vala"
	g_return_if_fail (ui_manager != NULL);
#line 128 "uimanager.vala"
	_tmp1_ = NULL;
#line 711 "abstract-text-document.vala"
	gtk_action_set_sensitive (_tmp1_ = valide_ui_manager_get_action (ui_manager, "edit-undo"), (g_object_get ((GtkSourceBuffer*) valide_abstract_text_document_get_buffer (self), "can-undo", &_tmp0_, NULL), _tmp0_));
#line 1438 "abstract-text-document.c"
	(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
#line 128 "uimanager.vala"
	_tmp3_ = NULL;
#line 712 "abstract-text-document.vala"
	gtk_action_set_sensitive (_tmp3_ = valide_ui_manager_get_action (ui_manager, "edit-redo"), (g_object_get ((GtkSourceBuffer*) valide_abstract_text_document_get_buffer (self), "can-redo", &_tmp2_, NULL), _tmp2_));
#line 1444 "abstract-text-document.c"
	(_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL));
}


/**
   * Select a range
   *
   * @param pos The position of the beginning and end of the selection
   */
#line 720 "abstract-text-document.vala"
void valide_abstract_text_document_select_range (ValideAbstractTextDocument* self, const ValidePosition* pos) {
#line 1456 "abstract-text-document.c"
	GtkTextIter start = {0};
	GtkTextIter end = {0};
#line 720 "abstract-text-document.vala"
	g_return_if_fail (self != NULL);
#line 725 "abstract-text-document.vala"
	if ((*pos).start_line >= 0) {
#line 1463 "abstract-text-document.c"
		gboolean _tmp0_;
		gboolean _tmp1_;
#line 727 "abstract-text-document.vala"
		gtk_text_buffer_get_start_iter ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (self), &start);
#line 728 "abstract-text-document.vala"
		gtk_text_iter_set_line (&start, (*pos).start_line);
#line 1470 "abstract-text-document.c"
		_tmp0_ = FALSE;
#line 729 "abstract-text-document.vala"
		if ((*pos).start_row >= 0) {
#line 729 "abstract-text-document.vala"
			_tmp0_ = (*pos).start_row < gtk_text_iter_get_chars_in_line (&start);
#line 1476 "abstract-text-document.c"
		} else {
#line 729 "abstract-text-document.vala"
			_tmp0_ = FALSE;
#line 1480 "abstract-text-document.c"
		}
#line 729 "abstract-text-document.vala"
		if (_tmp0_) {
#line 731 "abstract-text-document.vala"
			gtk_text_iter_set_line_offset (&start, (*pos).start_row);
#line 1486 "abstract-text-document.c"
		}
#line 733 "abstract-text-document.vala"
		end = start;
#line 1490 "abstract-text-document.c"
		_tmp1_ = FALSE;
#line 734 "abstract-text-document.vala"
		if ((*pos).end_line >= 0) {
#line 734 "abstract-text-document.vala"
			_tmp1_ = (*pos).end_line < gtk_text_buffer_get_line_count ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (self));
#line 1496 "abstract-text-document.c"
		} else {
#line 734 "abstract-text-document.vala"
			_tmp1_ = FALSE;
#line 1500 "abstract-text-document.c"
		}
#line 734 "abstract-text-document.vala"
		if (_tmp1_) {
#line 1504 "abstract-text-document.c"
			gboolean _tmp2_;
#line 736 "abstract-text-document.vala"
			gtk_text_iter_set_line (&end, (*pos).end_line);
#line 1508 "abstract-text-document.c"
			_tmp2_ = FALSE;
#line 737 "abstract-text-document.vala"
			if ((*pos).end_row >= 0) {
#line 737 "abstract-text-document.vala"
				_tmp2_ = (*pos).end_row < gtk_text_iter_get_chars_in_line (&end);
#line 1514 "abstract-text-document.c"
			} else {
#line 737 "abstract-text-document.vala"
				_tmp2_ = FALSE;
#line 1518 "abstract-text-document.c"
			}
#line 737 "abstract-text-document.vala"
			if (_tmp2_) {
#line 739 "abstract-text-document.vala"
				gtk_text_iter_set_line_offset (&end, (*pos).end_row);
#line 1524 "abstract-text-document.c"
			}
		}
#line 743 "abstract-text-document.vala"
		gtk_text_buffer_select_range ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (self), &start, &end);
#line 744 "abstract-text-document.vala"
		valide_abstract_text_document_scroll_to_iter (self, &start);
#line 1531 "abstract-text-document.c"
	} else {
#line 748 "abstract-text-document.vala"
		g_debug (_ ("Invalid range"));
#line 1535 "abstract-text-document.c"
	}
}


ValideSourceBuffer* valide_abstract_text_document_get_buffer (ValideAbstractTextDocument* self) {
	ValideSourceBuffer* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = valide_source_view_get_buffer (valide_abstract_text_document_get_text_view (self));
	return result;
}


ValideSourceView* valide_abstract_text_document_get_text_view (ValideAbstractTextDocument* self) {
	ValideSourceView* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = valide_split_source_view_get_active_view (self->split_view);
	return result;
}


static gboolean valide_abstract_text_document_real_get_is_save (ValideiDocument* base) {
	gboolean result;
	ValideAbstractTextDocument* self;
	self = (ValideAbstractTextDocument*) base;
	result = self->priv->_save;
	return result;
}


static void valide_abstract_text_document_real_set_is_save (ValideiDocument* base, gboolean value) {
	ValideAbstractTextDocument* self;
	self = (ValideAbstractTextDocument*) base;
#line 81 "abstract-text-document.vala"
	self->priv->_save = value;
#line 82 "abstract-text-document.vala"
	if (value) {
#line 84 "abstract-text-document.vala"
		g_signal_emit_by_name ((ValideiDocument*) self, "saved");
#line 1574 "abstract-text-document.c"
	}
#line 86 "abstract-text-document.vala"
	g_signal_emit_by_name ((ValideiDocument*) self, "title-change");
#line 1578 "abstract-text-document.c"
	g_object_notify ((GObject *) self, "is-save");
}


static ValideDocumentState valide_abstract_text_document_real_get_state (ValideiDocument* base) {
	ValideDocumentState result;
	ValideAbstractTextDocument* self;
	self = (ValideAbstractTextDocument*) base;
	result = self->priv->_state;
	return result;
}


static void valide_abstract_text_document_real_set_state (ValideiDocument* base, ValideDocumentState value) {
	ValideAbstractTextDocument* self;
	self = (ValideAbstractTextDocument*) base;
#line 101 "abstract-text-document.vala"
	self->priv->_state = value;
#line 102 "abstract-text-document.vala"
	g_signal_emit_by_name ((ValideiDocument*) self, "title-change");
#line 103 "abstract-text-document.vala"
	switch (valide_idocument_get_state ((ValideiDocument*) self)) {
#line 1601 "abstract-text-document.c"
		case VALIDE_DOCUMENT_STATE_NORMAL:
		{
#line 106 "abstract-text-document.vala"
			valide_abstract_text_document_set_editable (self, TRUE);
#line 107 "abstract-text-document.vala"
			break;
#line 1608 "abstract-text-document.c"
		}
		case VALIDE_DOCUMENT_STATE_EXTERNALLY_MODIFIED:
		case VALIDE_DOCUMENT_STATE_BAD_ENCODING:
		{
#line 110 "abstract-text-document.vala"
			valide_abstract_text_document_set_editable (self, FALSE);
#line 111 "abstract-text-document.vala"
			break;
#line 1617 "abstract-text-document.c"
		}
	}
#line 113 "abstract-text-document.vala"
	g_signal_emit_by_name ((ValideiDocument*) self, "state-change", valide_idocument_get_state ((ValideiDocument*) self));
#line 1622 "abstract-text-document.c"
	g_object_notify ((GObject *) self, "state");
}


gboolean valide_abstract_text_document_get_editable (ValideAbstractTextDocument* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = gtk_text_view_get_editable ((GtkTextView*) valide_abstract_text_document_get_text_view (self));
	return result;
}


void valide_abstract_text_document_set_editable (ValideAbstractTextDocument* self, gboolean value) {
	g_return_if_fail (self != NULL);
#line 128 "abstract-text-document.vala"
	gtk_text_view_set_editable ((GtkTextView*) valide_abstract_text_document_get_text_view (self), value);
#line 1639 "abstract-text-document.c"
	g_object_notify ((GObject *) self, "editable");
}


const char* valide_abstract_text_document_get_selected_text (ValideAbstractTextDocument* self) {
	const char* result;
	GtkTextIter start = {0};
	GtkTextIter end = {0};
	g_return_val_if_fail (self != NULL, NULL);
#line 143 "abstract-text-document.vala"
	if (gtk_text_buffer_get_selection_bounds ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (self), &start, &end)) {
#line 1651 "abstract-text-document.c"
		char* _tmp1_;
		const char* _tmp0_;
		_tmp1_ = NULL;
#line 145 "abstract-text-document.vala"
		_tmp0_ = NULL;
#line 145 "abstract-text-document.vala"
		self->priv->_selected_text = (_tmp1_ = (_tmp0_ = gtk_text_buffer_get_text ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (self), &start, &end, FALSE), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_selected_text = (g_free (self->priv->_selected_text), NULL), _tmp1_);
#line 1659 "abstract-text-document.c"
	}
	result = self->priv->_selected_text;
	return result;
}


static const char* valide_abstract_text_document_real_get_path (ValideiDocument* base) {
	const char* result;
	ValideAbstractTextDocument* self;
	self = (ValideAbstractTextDocument*) base;
	result = valide_abstract_text_document_get_buffer (self)->path;
	return result;
}


static void valide_abstract_text_document_real_set_path (ValideiDocument* base, const char* value) {
	ValideAbstractTextDocument* self;
	char* old_path;
	char* _tmp1_;
	const char* _tmp0_;
	char* _tmp4_;
	char* *_tmp3_;
	const char* _tmp2_;
	self = (ValideAbstractTextDocument*) base;
	old_path = NULL;
	_tmp1_ = NULL;
#line 169 "abstract-text-document.vala"
	_tmp0_ = NULL;
#line 169 "abstract-text-document.vala"
	old_path = (_tmp1_ = (_tmp0_ = valide_abstract_text_document_get_buffer (self)->path, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), old_path = (g_free (old_path), NULL), _tmp1_);
#line 170 "abstract-text-document.vala"
	g_object_set_data ((GObject*) self, "old_path", old_path);
#line 1692 "abstract-text-document.c"
	_tmp4_ = NULL;
#line 171 "abstract-text-document.vala"
	_tmp2_ = NULL;
#line 171 "abstract-text-document.vala"
	_tmp3_ = &valide_abstract_text_document_get_buffer (self)->path;
#line 171 "abstract-text-document.vala"
	(*_tmp3_) = (_tmp4_ = (_tmp2_ = value, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)), (*_tmp3_) = (g_free ((*_tmp3_)), NULL), _tmp4_);
#line 172 "abstract-text-document.vala"
	g_signal_emit_by_name ((ValideiDocument*) self, "title-change");
#line 173 "abstract-text-document.vala"
	g_object_set_data ((GObject*) self, "old_path", NULL);
#line 1704 "abstract-text-document.c"
	old_path = (g_free (old_path), NULL);
	g_object_notify ((GObject *) self, "path");
}


static const char* valide_abstract_text_document_real_get_filename (ValideiDocument* base) {
	const char* result;
	ValideAbstractTextDocument* self;
	char* _tmp0_;
	self = (ValideAbstractTextDocument*) base;
	_tmp0_ = NULL;
#line 187 "abstract-text-document.vala"
	self->priv->_filename = (_tmp0_ = g_path_get_basename (valide_idocument_get_path ((ValideiDocument*) self)), self->priv->_filename = (g_free (self->priv->_filename), NULL), _tmp0_);
#line 1718 "abstract-text-document.c"
	result = self->priv->_filename;
	return result;
}


static const char* valide_abstract_text_document_real_get_title (ValideiDocument* base) {
	const char* result;
	ValideAbstractTextDocument* self;
	char* _tmp0_;
	char* _tmp2_;
	self = (ValideAbstractTextDocument*) base;
	_tmp0_ = NULL;
#line 200 "abstract-text-document.vala"
	self->priv->_title = (_tmp0_ = g_strdup (""), self->priv->_title = (g_free (self->priv->_title), NULL), _tmp0_);
#line 201 "abstract-text-document.vala"
	if (gtk_text_buffer_get_modified ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (self))) {
#line 1735 "abstract-text-document.c"
		char* _tmp1_;
		_tmp1_ = NULL;
#line 203 "abstract-text-document.vala"
		self->priv->_title = (_tmp1_ = g_strdup ("* "), self->priv->_title = (g_free (self->priv->_title), NULL), _tmp1_);
#line 1740 "abstract-text-document.c"
	}
	_tmp2_ = NULL;
#line 205 "abstract-text-document.vala"
	self->priv->_title = (_tmp2_ = g_strconcat (self->priv->_title, valide_idocument_get_filename ((ValideiDocument*) self), NULL), self->priv->_title = (g_free (self->priv->_title), NULL), _tmp2_);
#line 1745 "abstract-text-document.c"
	result = self->priv->_title;
	return result;
}


static void _lambda76_ (ValideAbstractTextDocument* self) {
#line 317 "abstract-text-document.vala"
	valide_abstract_text_document_cursor_move (self);
#line 1754 "abstract-text-document.c"
}


static void __lambda76__gtk_text_buffer_changed (GtkTextBuffer* _sender, gpointer self) {
	_lambda76_ (self);
}


static void _lambda77_ (ValideAbstractTextDocument* self) {
#line 320 "abstract-text-document.vala"
	valide_idocument_set_is_save ((ValideiDocument*) self, !gtk_text_buffer_get_modified ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (self)));
#line 321 "abstract-text-document.vala"
	g_signal_emit_by_name ((ValideiDocument*) self, "title-change");
#line 1768 "abstract-text-document.c"
}


static void __lambda77__gtk_text_buffer_modified_changed (GtkTextBuffer* _sender, gpointer self) {
	_lambda77_ (self);
}


static void _lambda78_ (ValideAbstractTextDocument* self) {
#line 324 "abstract-text-document.vala"
	valide_abstract_text_document_cursor_move (self);
#line 1780 "abstract-text-document.c"
}


static void __lambda78__gtk_text_buffer_mark_set (GtkTextBuffer* _sender, const GtkTextIter* location, GtkTextMark* mark, gpointer self) {
	_lambda78_ (self);
}


static gboolean _lambda79_ (ValideAbstractTextDocument* self) {
	gboolean result;
#line 335 "abstract-text-document.vala"
	valide_abstract_text_document_check_externally_modified (self);
#line 336 "abstract-text-document.vala"
	valide_abstract_text_document_cursor_move (self);
#line 1795 "abstract-text-document.c"
	result = FALSE;
	return result;
}


static gboolean __lambda79__gtk_widget_focus_in_event (GtkWidget* _sender, const GdkEventFocus* event, gpointer self) {
	return _lambda79_ (self);
}


#line 272 "abstract-text-document.vala"
static void _valide_abstract_text_document_on_action_search_valide_split_source_view_search (ValideSplitSourceView* _sender, gpointer self) {
#line 1808 "abstract-text-document.c"
	valide_abstract_text_document_on_action_search (self);
}


#line 277 "abstract-text-document.vala"
static void _valide_abstract_text_document_on_action_find_next_valide_split_source_view_find_next (ValideSplitSourceView* _sender, gpointer self) {
#line 1815 "abstract-text-document.c"
	valide_abstract_text_document_on_action_find_next (self);
}


#line 282 "abstract-text-document.vala"
static void _valide_abstract_text_document_on_action_find_prev_valide_split_source_view_find_prev (ValideSplitSourceView* _sender, gpointer self) {
#line 1822 "abstract-text-document.c"
	valide_abstract_text_document_on_action_find_prev (self);
}


#line 287 "abstract-text-document.vala"
static void _valide_abstract_text_document_on_action_replace_valide_split_source_view_replace (ValideSplitSourceView* _sender, gpointer self) {
#line 1829 "abstract-text-document.c"
	valide_abstract_text_document_on_action_replace (self);
}


static GObject * valide_abstract_text_document_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValideAbstractTextDocumentClass * klass;
	GObjectClass * parent_class;
	ValideAbstractTextDocument * self;
	klass = VALIDE_ABSTRACT_TEXT_DOCUMENT_CLASS (g_type_class_peek (VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_ABSTRACT_TEXT_DOCUMENT (obj);
	{
		ValideSplitSourceView* _tmp0_;
		ValideSearching* _tmp1_;
		GtkBindingSet* binding_set;
		GTypeClass* _tmp2_;
#line 312 "abstract-text-document.vala"
		valide_idocument_set_is_save ((ValideiDocument*) self, TRUE);
#line 1850 "abstract-text-document.c"
		_tmp0_ = NULL;
#line 313 "abstract-text-document.vala"
		self->split_view = (_tmp0_ = valide_abstract_text_document_create_source_view (self), (self->split_view == NULL) ? NULL : (self->split_view = (g_object_unref (self->split_view), NULL)), _tmp0_);
#line 314 "abstract-text-document.vala"
		valide_idocument_set_path ((ValideiDocument*) self, _ ("New file"));
#line 315 "abstract-text-document.vala"
		gtk_box_pack_start ((GtkBox*) self, (GtkWidget*) self->split_view, TRUE, TRUE, (guint) 0);
#line 316 "abstract-text-document.vala"
		g_signal_connect ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (self), "changed", (GCallback) __lambda76__gtk_text_buffer_changed, self);
#line 319 "abstract-text-document.vala"
		g_signal_connect ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (self), "modified-changed", (GCallback) __lambda77__gtk_text_buffer_modified_changed, self);
#line 323 "abstract-text-document.vala"
		g_signal_connect ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (self), "mark-set", (GCallback) __lambda78__gtk_text_buffer_mark_set, self);
#line 1864 "abstract-text-document.c"
		_tmp1_ = NULL;
#line 327 "abstract-text-document.vala"
		self->priv->searching = (_tmp1_ = g_object_ref_sink (valide_searching_new (self)), (self->priv->searching == NULL) ? NULL : (self->priv->searching = (g_object_unref (self->priv->searching), NULL)), _tmp1_);
#line 328 "abstract-text-document.vala"
		gtk_box_pack_start ((GtkBox*) self, (GtkWidget*) self->priv->searching, FALSE, TRUE, (guint) 0);
#line 329 "abstract-text-document.vala"
		gtk_widget_hide ((GtkWidget*) self->priv->searching);
#line 331 "abstract-text-document.vala"
		valide_abstract_text_document_cursor_move (self);
#line 333 "abstract-text-document.vala"
		valide_idocument_set_state ((ValideiDocument*) self, VALIDE_DOCUMENT_STATE_NORMAL);
#line 334 "abstract-text-document.vala"
		g_signal_connect ((GtkWidget*) valide_abstract_text_document_get_text_view (self), "focus-in-event", (GCallback) __lambda79__gtk_widget_focus_in_event, self);
#line 1878 "abstract-text-document.c"
		/* Bindings */
		binding_set = NULL;
#line 66 "gobject-2.0.vapi"
		_tmp2_ = NULL;
#line 343 "abstract-text-document.vala"
		binding_set = gtk_binding_set_by_class (_tmp2_ = g_type_class_ref (G_TYPE_FROM_INSTANCE ((GObject*) self)));
#line 1885 "abstract-text-document.c"
		(_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_type_class_unref (_tmp2_), NULL));
#line 345 "abstract-text-document.vala"
		g_signal_connect_object (self->split_view, "search", (GCallback) _valide_abstract_text_document_on_action_search_valide_split_source_view_search, self, 0);
#line 346 "abstract-text-document.vala"
		g_signal_connect_object (self->split_view, "find-next", (GCallback) _valide_abstract_text_document_on_action_find_next_valide_split_source_view_find_next, self, 0);
#line 347 "abstract-text-document.vala"
		g_signal_connect_object (self->split_view, "find-prev", (GCallback) _valide_abstract_text_document_on_action_find_prev_valide_split_source_view_find_prev, self, 0);
#line 348 "abstract-text-document.vala"
		g_signal_connect_object (self->split_view, "replace", (GCallback) _valide_abstract_text_document_on_action_replace_valide_split_source_view_replace, self, 0);
#line 1895 "abstract-text-document.c"
	}
	return obj;
}


static void valide_abstract_text_document_class_init (ValideAbstractTextDocumentClass * klass) {
	valide_abstract_text_document_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideAbstractTextDocumentPrivate));
	VALIDE_ABSTRACT_TEXT_DOCUMENT_CLASS (klass)->create_source_view = valide_abstract_text_document_real_create_source_view;
	VALIDE_ABSTRACT_TEXT_DOCUMENT_CLASS (klass)->get_contents = valide_abstract_text_document_real_get_contents;
	VALIDE_ABSTRACT_TEXT_DOCUMENT_CLASS (klass)->set_contents = valide_abstract_text_document_real_set_contents;
	GTK_WIDGET_CLASS (klass)->show_all = valide_abstract_text_document_real_show_all;
	VALIDE_ABSTRACT_TEXT_DOCUMENT_CLASS (klass)->get_icon = valide_abstract_text_document_real_get_icon;
	VALIDE_ABSTRACT_TEXT_DOCUMENT_CLASS (klass)->get_tooltips = valide_abstract_text_document_real_get_tooltips;
	G_OBJECT_CLASS (klass)->get_property = valide_abstract_text_document_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_abstract_text_document_set_property;
	G_OBJECT_CLASS (klass)->constructor = valide_abstract_text_document_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_abstract_text_document_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_ABSTRACT_TEXT_DOCUMENT_BUFFER, g_param_spec_object ("buffer", "buffer", "buffer", VALIDE_TYPE_SOURCE_BUFFER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_ABSTRACT_TEXT_DOCUMENT_TEXT_VIEW, g_param_spec_object ("text-view", "text-view", "text-view", VALIDE_TYPE_SOURCE_VIEW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_override_property (G_OBJECT_CLASS (klass), VALIDE_ABSTRACT_TEXT_DOCUMENT_IS_SAVE, "is-save");
	g_object_class_override_property (G_OBJECT_CLASS (klass), VALIDE_ABSTRACT_TEXT_DOCUMENT_STATE, "state");
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_ABSTRACT_TEXT_DOCUMENT_EDITABLE, g_param_spec_boolean ("editable", "editable", "editable", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_ABSTRACT_TEXT_DOCUMENT_SELECTED_TEXT, g_param_spec_string ("selected-text", "selected-text", "selected-text", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_override_property (G_OBJECT_CLASS (klass), VALIDE_ABSTRACT_TEXT_DOCUMENT_PATH, "path");
	g_object_class_override_property (G_OBJECT_CLASS (klass), VALIDE_ABSTRACT_TEXT_DOCUMENT_FILENAME, "filename");
	g_object_class_override_property (G_OBJECT_CLASS (klass), VALIDE_ABSTRACT_TEXT_DOCUMENT_TITLE, "title");
	g_signal_new ("cursor_moved", VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_VOID__INT_INT, G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_INT);
}


static void valide_abstract_text_document_valide_idocument_interface_init (ValideiDocumentIface * iface) {
	valide_abstract_text_document_valide_idocument_parent_iface = g_type_interface_peek_parent (iface);
	iface->open = valide_abstract_text_document_real_open;
	iface->save = valide_abstract_text_document_real_save;
	iface->save_as = valide_abstract_text_document_real_save_as;
	iface->rename = valide_abstract_text_document_real_rename;
	iface->close = valide_abstract_text_document_real_close;
	iface->get_icon = (GdkPixbuf* (*)(ValideiDocument*)) valide_abstract_text_document_get_icon;
	iface->get_tooltips = (char* (*)(ValideiDocument*)) valide_abstract_text_document_get_tooltips;
	iface->setup_ui = valide_abstract_text_document_real_setup_ui;
	iface->get_is_save = valide_abstract_text_document_real_get_is_save;
	iface->set_is_save = valide_abstract_text_document_real_set_is_save;
	iface->get_state = valide_abstract_text_document_real_get_state;
	iface->set_state = valide_abstract_text_document_real_set_state;
	iface->get_path = valide_abstract_text_document_real_get_path;
	iface->set_path = valide_abstract_text_document_real_set_path;
	iface->get_filename = valide_abstract_text_document_real_get_filename;
	iface->get_title = valide_abstract_text_document_real_get_title;
}


static void valide_abstract_text_document_instance_init (ValideAbstractTextDocument * self) {
	self->priv = VALIDE_ABSTRACT_TEXT_DOCUMENT_GET_PRIVATE (self);
	self->priv->_selected_text = g_strdup ("");
}


static void valide_abstract_text_document_finalize (GObject* obj) {
	ValideAbstractTextDocument * self;
	self = VALIDE_ABSTRACT_TEXT_DOCUMENT (obj);
	(self->split_view == NULL) ? NULL : (self->split_view = (g_object_unref (self->split_view), NULL));
	(self->priv->searching == NULL) ? NULL : (self->priv->searching = (g_object_unref (self->priv->searching), NULL));
	self->priv->_selected_text = (g_free (self->priv->_selected_text), NULL);
	self->priv->_filename = (g_free (self->priv->_filename), NULL);
	self->priv->_title = (g_free (self->priv->_title), NULL);
	G_OBJECT_CLASS (valide_abstract_text_document_parent_class)->finalize (obj);
}


GType valide_abstract_text_document_get_type (void) {
	static GType valide_abstract_text_document_type_id = 0;
	if (valide_abstract_text_document_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideAbstractTextDocumentClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_abstract_text_document_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideAbstractTextDocument), 0, (GInstanceInitFunc) valide_abstract_text_document_instance_init, NULL };
		static const GInterfaceInfo valide_idocument_info = { (GInterfaceInitFunc) valide_abstract_text_document_valide_idocument_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		valide_abstract_text_document_type_id = g_type_register_static (GTK_TYPE_VBOX, "ValideAbstractTextDocument", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
		g_type_add_interface_static (valide_abstract_text_document_type_id, VALIDE_TYPE_IDOCUMENT, &valide_idocument_info);
	}
	return valide_abstract_text_document_type_id;
}


static void valide_abstract_text_document_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideAbstractTextDocument * self;
	gpointer boxed;
	self = VALIDE_ABSTRACT_TEXT_DOCUMENT (object);
	switch (property_id) {
		case VALIDE_ABSTRACT_TEXT_DOCUMENT_BUFFER:
		g_value_set_object (value, valide_abstract_text_document_get_buffer (self));
		break;
		case VALIDE_ABSTRACT_TEXT_DOCUMENT_TEXT_VIEW:
		g_value_set_object (value, valide_abstract_text_document_get_text_view (self));
		break;
		case VALIDE_ABSTRACT_TEXT_DOCUMENT_IS_SAVE:
		g_value_set_boolean (value, valide_idocument_get_is_save ((ValideiDocument*) self));
		break;
		case VALIDE_ABSTRACT_TEXT_DOCUMENT_STATE:
		g_value_set_enum (value, valide_idocument_get_state ((ValideiDocument*) self));
		break;
		case VALIDE_ABSTRACT_TEXT_DOCUMENT_EDITABLE:
		g_value_set_boolean (value, valide_abstract_text_document_get_editable (self));
		break;
		case VALIDE_ABSTRACT_TEXT_DOCUMENT_SELECTED_TEXT:
		g_value_set_string (value, valide_abstract_text_document_get_selected_text (self));
		break;
		case VALIDE_ABSTRACT_TEXT_DOCUMENT_PATH:
		g_value_set_string (value, valide_idocument_get_path ((ValideiDocument*) self));
		break;
		case VALIDE_ABSTRACT_TEXT_DOCUMENT_FILENAME:
		g_value_set_string (value, valide_idocument_get_filename ((ValideiDocument*) self));
		break;
		case VALIDE_ABSTRACT_TEXT_DOCUMENT_TITLE:
		g_value_set_string (value, valide_idocument_get_title ((ValideiDocument*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_abstract_text_document_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideAbstractTextDocument * self;
	self = VALIDE_ABSTRACT_TEXT_DOCUMENT (object);
	switch (property_id) {
		case VALIDE_ABSTRACT_TEXT_DOCUMENT_IS_SAVE:
		valide_idocument_set_is_save ((ValideiDocument*) self, g_value_get_boolean (value));
		break;
		case VALIDE_ABSTRACT_TEXT_DOCUMENT_STATE:
		valide_idocument_set_state ((ValideiDocument*) self, g_value_get_enum (value));
		break;
		case VALIDE_ABSTRACT_TEXT_DOCUMENT_EDITABLE:
		valide_abstract_text_document_set_editable (self, g_value_get_boolean (value));
		break;
		case VALIDE_ABSTRACT_TEXT_DOCUMENT_PATH:
		valide_idocument_set_path ((ValideiDocument*) self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}



static void g_cclosure_user_marshal_VOID__INT_INT (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data) {
	typedef void (*GMarshalFunc_VOID__INT_INT) (gpointer data1, gint arg_1, gint arg_2, gpointer data2);
	register GMarshalFunc_VOID__INT_INT callback;
	register GCClosure * cc;
	register gpointer data1, data2;
	cc = (GCClosure *) closure;
	g_return_if_fail (n_param_values == 3);
	if (G_CCLOSURE_SWAP_DATA (closure)) {
		data1 = closure->data;
		data2 = param_values->data[0].v_pointer;
	} else {
		data1 = param_values->data[0].v_pointer;
		data2 = closure->data;
	}
	callback = (GMarshalFunc_VOID__INT_INT) (marshal_data ? marshal_data : cc->callback);
	callback (data1, g_value_get_int (param_values + 1), g_value_get_int (param_values + 2), data2);
}



