/* builder-manager.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <glib/gi18n-lib.h>


#define VALIDE_TYPE_BUILDER_MANAGER (valide_builder_manager_get_type ())
#define VALIDE_BUILDER_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_BUILDER_MANAGER, ValideBuilderManager))
#define VALIDE_BUILDER_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_BUILDER_MANAGER, ValideBuilderManagerClass))
#define VALIDE_IS_BUILDER_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_BUILDER_MANAGER))
#define VALIDE_IS_BUILDER_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_BUILDER_MANAGER))
#define VALIDE_BUILDER_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_BUILDER_MANAGER, ValideBuilderManagerClass))

typedef struct _ValideBuilderManager ValideBuilderManager;
typedef struct _ValideBuilderManagerClass ValideBuilderManagerClass;
typedef struct _ValideBuilderManagerPrivate ValideBuilderManagerPrivate;

#define VALIDE_TYPE_EXECUTABLE_MANAGER (valide_executable_manager_get_type ())
#define VALIDE_EXECUTABLE_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_EXECUTABLE_MANAGER, ValideExecutableManager))
#define VALIDE_EXECUTABLE_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_EXECUTABLE_MANAGER, ValideExecutableManagerClass))
#define VALIDE_IS_EXECUTABLE_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_EXECUTABLE_MANAGER))
#define VALIDE_IS_EXECUTABLE_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_EXECUTABLE_MANAGER))
#define VALIDE_EXECUTABLE_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_EXECUTABLE_MANAGER, ValideExecutableManagerClass))

typedef struct _ValideExecutableManager ValideExecutableManager;
typedef struct _ValideExecutableManagerClass ValideExecutableManagerClass;

#define VALIDE_TYPE_BUILDER (valide_builder_get_type ())
#define VALIDE_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_BUILDER, ValideBuilder))
#define VALIDE_IS_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_BUILDER))
#define VALIDE_BUILDER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), VALIDE_TYPE_BUILDER, ValideBuilderIface))

typedef struct _ValideBuilder ValideBuilder;
typedef struct _ValideBuilderIface ValideBuilderIface;

#define VALIDE_TYPE_PROJECT (valide_project_get_type ())
#define VALIDE_PROJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PROJECT, ValideProject))
#define VALIDE_PROJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PROJECT, ValideProjectClass))
#define VALIDE_IS_PROJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PROJECT))
#define VALIDE_IS_PROJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PROJECT))
#define VALIDE_PROJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PROJECT, ValideProjectClass))

typedef struct _ValideProject ValideProject;
typedef struct _ValideProjectClass ValideProjectClass;

#define VALIDE_TYPE_NATIVE_BUILDER (valide_native_builder_get_type ())
#define VALIDE_NATIVE_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_NATIVE_BUILDER, ValideNativeBuilder))
#define VALIDE_NATIVE_BUILDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_NATIVE_BUILDER, ValideNativeBuilderClass))
#define VALIDE_IS_NATIVE_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_NATIVE_BUILDER))
#define VALIDE_IS_NATIVE_BUILDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_NATIVE_BUILDER))
#define VALIDE_NATIVE_BUILDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_NATIVE_BUILDER, ValideNativeBuilderClass))

typedef struct _ValideNativeBuilder ValideNativeBuilder;
typedef struct _ValideNativeBuilderClass ValideNativeBuilderClass;

#define VALIDE_TYPE_NATIVE_BUILDER_OPTIONS (valide_native_builder_options_get_type ())
#define VALIDE_NATIVE_BUILDER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_NATIVE_BUILDER_OPTIONS, ValideNativeBuilderOptions))
#define VALIDE_NATIVE_BUILDER_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_NATIVE_BUILDER_OPTIONS, ValideNativeBuilderOptionsClass))
#define VALIDE_IS_NATIVE_BUILDER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_NATIVE_BUILDER_OPTIONS))
#define VALIDE_IS_NATIVE_BUILDER_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_NATIVE_BUILDER_OPTIONS))
#define VALIDE_NATIVE_BUILDER_OPTIONS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_NATIVE_BUILDER_OPTIONS, ValideNativeBuilderOptionsClass))

typedef struct _ValideNativeBuilderOptions ValideNativeBuilderOptions;
typedef struct _ValideNativeBuilderOptionsClass ValideNativeBuilderOptionsClass;

typedef enum  {
	VALIDE_BUILDER_ERROR_NOT_FOUND
} ValideBuilderError;
#define VALIDE_BUILDER_ERROR valide_builder_error_quark ()
/**
   * Builder not found
   
*
 * The builder manager
 */
struct _ValideBuilderManager {
	GObject parent_instance;
	ValideBuilderManagerPrivate * priv;
};

struct _ValideBuilderManagerClass {
	GObjectClass parent_class;
};

struct _ValideBuilderManagerPrivate {
	GList* builders;
	ValideExecutableManager* _executables;
};

/**
 * This instance use to create a new builder
 *
 * [[warning:
 *   You should register builder with BuilderManager.register_builder function.
 * ]]
 */
struct _ValideBuilderIface {
	GTypeInterface parent_iface;
	void (*configure) (ValideBuilder* self);
	void (*build) (ValideBuilder* self);
	void (*install) (ValideBuilder* self);
	void (*dist) (ValideBuilder* self);
	void (*clean) (ValideBuilder* self);
	void (*distclean) (ValideBuilder* self);
	void (*uninstall) (ValideBuilder* self);
	GtkWidget* (*preferences) (ValideBuilder* self);
	ValideProject* (*get_project) (ValideBuilder* self);
	void (*set_project) (ValideBuilder* self, ValideProject* value);
	ValideExecutableManager* (*get_executables) (ValideBuilder* self);
	void (*set_executables) (ValideBuilder* self, ValideExecutableManager* value);
};


static ValideBuilderManager* valide_builder_manager_instance;
static ValideBuilderManager* valide_builder_manager_instance = NULL;
static gpointer valide_builder_manager_parent_class = NULL;

GQuark valide_builder_error_quark (void);
GType valide_builder_manager_get_type (void);
GType valide_executable_manager_get_type (void);
#define VALIDE_BUILDER_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_BUILDER_MANAGER, ValideBuilderManagerPrivate))
enum  {
	VALIDE_BUILDER_MANAGER_DUMMY_PROPERTY,
	VALIDE_BUILDER_MANAGER_EXECUTABLES
};
static void _g_list_free_g_free (GList* self);
static ValideBuilderManager* valide_builder_manager_new (void);
static ValideBuilderManager* valide_builder_manager_construct (GType object_type);
ValideBuilderManager* valide_builder_manager_get_instance (void);
void valide_builder_manager_register_builder (ValideBuilderManager* self, GType builder, GType builder_options);
GType valide_project_get_type (void);
GType valide_builder_get_type (void);
const char* valide_project_get_builder (ValideProject* self);
void valide_project_set_builder (ValideProject* self, const char* value);
ValideExecutableManager* valide_builder_manager_get_executables (ValideBuilderManager* self);
ValideBuilder* valide_builder_manager_create_builder (ValideBuilderManager* self, ValideProject* project, GError** error);
GList* valide_builder_manager_get_builders (ValideBuilderManager* self);
void valide_builder_manager_set_executables (ValideBuilderManager* self, ValideExecutableManager* value);
GType valide_native_builder_get_type (void);
GType valide_native_builder_options_get_type (void);
static GObject * valide_builder_manager_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void valide_builder_manager_finalize (GObject* obj);
static void valide_builder_manager_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_builder_manager_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);



GQuark valide_builder_error_quark (void) {
	return g_quark_from_static_string ("valide_builder_error-quark");
}


static void _g_list_free_g_free (GList* self) {
	g_list_foreach (self, (GFunc) g_free, NULL);
	g_list_free (self);
}


#line 47 "builder-manager.vala"
static ValideBuilderManager* valide_builder_manager_construct (GType object_type) {
#line 185 "builder-manager.c"
	ValideBuilderManager * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


#line 47 "builder-manager.vala"
static ValideBuilderManager* valide_builder_manager_new (void) {
#line 47 "builder-manager.vala"
	return valide_builder_manager_construct (VALIDE_TYPE_BUILDER_MANAGER);
#line 196 "builder-manager.c"
}


/**
   * Get the uniq instance of the builder manager
   *
   * @return The BuilderManager
   */
#line 62 "builder-manager.vala"
ValideBuilderManager* valide_builder_manager_get_instance (void) {
#line 207 "builder-manager.c"
	ValideBuilderManager* result;
	ValideBuilderManager* _tmp1_;
#line 64 "builder-manager.vala"
	if (valide_builder_manager_instance == NULL) {
#line 212 "builder-manager.c"
		ValideBuilderManager* _tmp0_;
		_tmp0_ = NULL;
#line 66 "builder-manager.vala"
		valide_builder_manager_instance = (_tmp0_ = valide_builder_manager_new (), (valide_builder_manager_instance == NULL) ? NULL : (valide_builder_manager_instance = (g_object_unref (valide_builder_manager_instance), NULL)), _tmp0_);
#line 217 "builder-manager.c"
	}
#line 68 "builder-manager.vala"
	_tmp1_ = NULL;
#line 221 "builder-manager.c"
	result = (_tmp1_ = valide_builder_manager_instance, (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_));
	return result;
}


#line 922 "glib-2.0.vapi"
static char* string_substring (const char* self, glong offset, glong len) {
#line 229 "builder-manager.c"
	char* result;
	glong string_length;
	const char* start;
#line 922 "glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 235 "builder-manager.c"
	string_length = g_utf8_strlen (self, -1);
#line 924 "glib-2.0.vapi"
	if (offset < 0) {
#line 925 "glib-2.0.vapi"
		offset = string_length + offset;
#line 926 "glib-2.0.vapi"
		g_return_val_if_fail (offset >= 0, NULL);
#line 243 "builder-manager.c"
	} else {
#line 928 "glib-2.0.vapi"
		g_return_val_if_fail (offset <= string_length, NULL);
#line 247 "builder-manager.c"
	}
#line 930 "glib-2.0.vapi"
	if (len < 0) {
#line 931 "glib-2.0.vapi"
		len = string_length - offset;
#line 253 "builder-manager.c"
	}
#line 933 "glib-2.0.vapi"
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
#line 257 "builder-manager.c"
	start = g_utf8_offset_to_pointer (self, offset);
	result = g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
	return result;
}


/**
   * Register a new builder
   *
   * @param The Type of the new builder
   * @param The type ofthe builder options
   */
#line 77 "builder-manager.vala"
void valide_builder_manager_register_builder (ValideBuilderManager* self, GType builder, GType builder_options) {
#line 272 "builder-manager.c"
	char* builder_name;
	char* _tmp1_;
	const char* _tmp0_;
#line 77 "builder-manager.vala"
	g_return_if_fail (self != NULL);
#line 278 "builder-manager.c"
	builder_name = NULL;
	_tmp1_ = NULL;
#line 81 "builder-manager.vala"
	_tmp0_ = NULL;
#line 81 "builder-manager.vala"
	builder_name = (_tmp1_ = (_tmp0_ = g_type_name (builder), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), builder_name = (g_free (builder_name), NULL), _tmp1_);
#line 82 "builder-manager.vala"
	if (g_str_has_prefix (builder_name, "Valide")) {
#line 287 "builder-manager.c"
		char* _tmp2_;
		_tmp2_ = NULL;
#line 84 "builder-manager.vala"
		builder_name = (_tmp2_ = string_substring (builder_name, (glong) 6, -1), builder_name = (g_free (builder_name), NULL), _tmp2_);
#line 292 "builder-manager.c"
	}
#line 86 "builder-manager.vala"
	if (g_str_has_suffix (builder_name, "Builder")) {
#line 296 "builder-manager.c"
		char* _tmp3_;
		_tmp3_ = NULL;
#line 88 "builder-manager.vala"
		builder_name = (_tmp3_ = string_substring (builder_name, (glong) 0, g_utf8_strlen (builder_name, -1) - 7), builder_name = (g_free (builder_name), NULL), _tmp3_);
#line 301 "builder-manager.c"
	}
#line 90 "builder-manager.vala"
	if (g_list_find_custom (self->priv->builders, builder_name, strcmp) == NULL) {
#line 305 "builder-manager.c"
		const char* _tmp4_;
#line 92 "builder-manager.vala"
		_tmp4_ = NULL;
#line 92 "builder-manager.vala"
		self->priv->builders = g_list_append (self->priv->builders, (_tmp4_ = builder_name, (_tmp4_ == NULL) ? NULL : g_strdup (_tmp4_)));
#line 311 "builder-manager.c"
	}
	builder_name = (g_free (builder_name), NULL);
}


/**
   * Create a new builder from its name
   *
   * @param project The project
   *
   * @return A new builder
   */
#line 103 "builder-manager.vala"
ValideBuilder* valide_builder_manager_create_builder (ValideBuilderManager* self, ValideProject* project, GError** error) {
#line 326 "builder-manager.c"
	ValideBuilder* result;
	GError * _inner_error_;
	GType type;
	char* type_name;
	ValideBuilder* builder;
	char* _tmp3_;
	char* _tmp2_;
	char* _tmp1_;
	char* _tmp0_;
#line 103 "builder-manager.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 103 "builder-manager.vala"
	g_return_val_if_fail (project != NULL, NULL);
#line 340 "builder-manager.c"
	_inner_error_ = NULL;
	type = 0UL;
	type_name = NULL;
	builder = NULL;
#line 109 "builder-manager.vala"
	if (valide_project_get_builder (project) == NULL) {
#line 111 "builder-manager.vala"
		valide_project_set_builder (project, "native");
#line 349 "builder-manager.c"
	}
	/* Check the plugin builder*/
	_tmp3_ = NULL;
	_tmp2_ = NULL;
#line 871 "glib-2.0.vapi"
	_tmp1_ = NULL;
#line 869 "glib-2.0.vapi"
	_tmp0_ = NULL;
#line 114 "builder-manager.vala"
	type_name = (_tmp3_ = g_strconcat (_tmp2_ = g_strconcat (_tmp0_ = g_utf8_strup (valide_project_get_builder (project), (gssize) 1), _tmp1_ = g_utf8_strdown (g_utf8_offset_to_pointer (valide_project_get_builder (project), (glong) 1), -1), NULL), "Builder", NULL), type_name = (g_free (type_name), NULL), _tmp3_);
#line 360 "builder-manager.c"
	_tmp2_ = (g_free (_tmp2_), NULL);
	_tmp1_ = (g_free (_tmp1_), NULL);
	_tmp0_ = (g_free (_tmp0_), NULL);
#line 115 "builder-manager.vala"
	type = g_type_from_name (type_name);
#line 116 "builder-manager.vala"
	if (type == 0) {
#line 368 "builder-manager.c"
		char* _tmp4_;
		/* Check the internal builder*/
		_tmp4_ = NULL;
#line 119 "builder-manager.vala"
		type_name = (_tmp4_ = g_strconcat ("Valide", type_name, NULL), type_name = (g_free (type_name), NULL), _tmp4_);
#line 120 "builder-manager.vala"
		type = g_type_from_name (type_name);
#line 376 "builder-manager.c"
	}
#line 122 "builder-manager.vala"
	if (type != 0) {
#line 380 "builder-manager.c"
		ValideBuilder* _tmp6_;
		GObject* _tmp5_;
		_tmp6_ = NULL;
#line 124 "builder-manager.vala"
		_tmp5_ = NULL;
#line 124 "builder-manager.vala"
		builder = (_tmp6_ = (_tmp5_ = g_object_new (type, "executables", self->priv->_executables, "project", project, NULL), VALIDE_IS_BUILDER (_tmp5_) ? ((ValideBuilder*) _tmp5_) : NULL), (builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL)), _tmp6_);
#line 388 "builder-manager.c"
	} else {
		_inner_error_ = g_error_new (VALIDE_BUILDER_ERROR, VALIDE_BUILDER_ERROR_NOT_FOUND, _ ("The builder '%s' doesn't exist!"), valide_project_get_builder (project));
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == VALIDE_BUILDER_ERROR) {
				g_propagate_error (error, _inner_error_);
				type_name = (g_free (type_name), NULL);
				(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
				return NULL;
			} else {
				type_name = (g_free (type_name), NULL);
				(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return NULL;
			}
		}
	}
	result = builder;
	type_name = (g_free (type_name), NULL);
	return result;
}


/**
   * Get the list of the available builders
   *
   * @return The list of the available builders
   */
#line 141 "builder-manager.vala"
GList* valide_builder_manager_get_builders (ValideBuilderManager* self) {
#line 419 "builder-manager.c"
	GList* result;
#line 141 "builder-manager.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 423 "builder-manager.c"
	result = self->priv->builders;
	return result;
}


ValideExecutableManager* valide_builder_manager_get_executables (ValideBuilderManager* self) {
	ValideExecutableManager* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_executables;
	return result;
}


void valide_builder_manager_set_executables (ValideBuilderManager* self, ValideExecutableManager* value) {
	ValideExecutableManager* _tmp1_;
	ValideExecutableManager* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 44 "builder-manager.vala"
	_tmp0_ = NULL;
#line 444 "builder-manager.c"
	self->priv->_executables = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_executables == NULL) ? NULL : (self->priv->_executables = (g_object_unref (self->priv->_executables), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "executables");
}


static GObject * valide_builder_manager_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValideBuilderManagerClass * klass;
	GObjectClass * parent_class;
	ValideBuilderManager * self;
	klass = VALIDE_BUILDER_MANAGER_CLASS (g_type_class_peek (VALIDE_TYPE_BUILDER_MANAGER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_BUILDER_MANAGER (obj);
	{
		GList* _tmp0_;
		_tmp0_ = NULL;
#line 53 "builder-manager.vala"
		self->priv->builders = (_tmp0_ = NULL, (self->priv->builders == NULL) ? NULL : (self->priv->builders = (_g_list_free_g_free (self->priv->builders), NULL)), _tmp0_);
#line 54 "builder-manager.vala"
		valide_builder_manager_register_builder (self, VALIDE_TYPE_NATIVE_BUILDER, VALIDE_TYPE_NATIVE_BUILDER_OPTIONS);
#line 466 "builder-manager.c"
	}
	return obj;
}


static void valide_builder_manager_class_init (ValideBuilderManagerClass * klass) {
	valide_builder_manager_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideBuilderManagerPrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_builder_manager_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_builder_manager_set_property;
	G_OBJECT_CLASS (klass)->constructor = valide_builder_manager_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_builder_manager_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_BUILDER_MANAGER_EXECUTABLES, g_param_spec_object ("executables", "executables", "executables", VALIDE_TYPE_EXECUTABLE_MANAGER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void valide_builder_manager_instance_init (ValideBuilderManager * self) {
	self->priv = VALIDE_BUILDER_MANAGER_GET_PRIVATE (self);
}


static void valide_builder_manager_finalize (GObject* obj) {
	ValideBuilderManager * self;
	self = VALIDE_BUILDER_MANAGER (obj);
	(self->priv->builders == NULL) ? NULL : (self->priv->builders = (_g_list_free_g_free (self->priv->builders), NULL));
	(self->priv->_executables == NULL) ? NULL : (self->priv->_executables = (g_object_unref (self->priv->_executables), NULL));
	G_OBJECT_CLASS (valide_builder_manager_parent_class)->finalize (obj);
}


GType valide_builder_manager_get_type (void) {
	static GType valide_builder_manager_type_id = 0;
	if (valide_builder_manager_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideBuilderManagerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_builder_manager_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideBuilderManager), 0, (GInstanceInitFunc) valide_builder_manager_instance_init, NULL };
		valide_builder_manager_type_id = g_type_register_static (G_TYPE_OBJECT, "ValideBuilderManager", &g_define_type_info, 0);
	}
	return valide_builder_manager_type_id;
}


static void valide_builder_manager_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideBuilderManager * self;
	gpointer boxed;
	self = VALIDE_BUILDER_MANAGER (object);
	switch (property_id) {
		case VALIDE_BUILDER_MANAGER_EXECUTABLES:
		g_value_set_object (value, valide_builder_manager_get_executables (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_builder_manager_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideBuilderManager * self;
	self = VALIDE_BUILDER_MANAGER (object);
	switch (property_id) {
		case VALIDE_BUILDER_MANAGER_EXECUTABLES:
		valide_builder_manager_set_executables (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}




