/* compiler.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixdata.h>


#define VALIDE_TYPE_EXECUTABLE (valide_executable_get_type ())
#define VALIDE_EXECUTABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_EXECUTABLE, ValideExecutable))
#define VALIDE_EXECUTABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_EXECUTABLE, ValideExecutableClass))
#define VALIDE_IS_EXECUTABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_EXECUTABLE))
#define VALIDE_IS_EXECUTABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_EXECUTABLE))
#define VALIDE_EXECUTABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_EXECUTABLE, ValideExecutableClass))

typedef struct _ValideExecutable ValideExecutable;
typedef struct _ValideExecutableClass ValideExecutableClass;
typedef struct _ValideExecutablePrivate ValideExecutablePrivate;

#define VALIDE_TYPE_COMPILER (valide_compiler_get_type ())
#define VALIDE_COMPILER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_COMPILER, ValideCompiler))
#define VALIDE_COMPILER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_COMPILER, ValideCompilerClass))
#define VALIDE_IS_COMPILER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_COMPILER))
#define VALIDE_IS_COMPILER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_COMPILER))
#define VALIDE_COMPILER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_COMPILER, ValideCompilerClass))

typedef struct _ValideCompiler ValideCompiler;
typedef struct _ValideCompilerClass ValideCompilerClass;
typedef struct _ValideCompilerPrivate ValideCompilerPrivate;

#define VALIDE_TYPE_POSITION (valide_position_get_type ())
typedef struct _ValidePosition ValidePosition;

#define VALIDE_TYPE_PROJECT (valide_project_get_type ())
#define VALIDE_PROJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PROJECT, ValideProject))
#define VALIDE_PROJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PROJECT, ValideProjectClass))
#define VALIDE_IS_PROJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PROJECT))
#define VALIDE_IS_PROJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PROJECT))
#define VALIDE_PROJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PROJECT, ValideProjectClass))

typedef struct _ValideProject ValideProject;
typedef struct _ValideProjectClass ValideProjectClass;

#define VALIDE_TYPE_IDOCUMENT (valide_idocument_get_type ())
#define VALIDE_IDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_IDOCUMENT, ValideiDocument))
#define VALIDE_IS_IDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_IDOCUMENT))
#define VALIDE_IDOCUMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), VALIDE_TYPE_IDOCUMENT, ValideiDocumentIface))

typedef struct _ValideiDocument ValideiDocument;
typedef struct _ValideiDocumentIface ValideiDocumentIface;

#define VALIDE_TYPE_UI_MANAGER (valide_ui_manager_get_type ())
#define VALIDE_UI_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_UI_MANAGER, ValideUIManager))
#define VALIDE_UI_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_UI_MANAGER, ValideUIManagerClass))
#define VALIDE_IS_UI_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_UI_MANAGER))
#define VALIDE_IS_UI_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_UI_MANAGER))
#define VALIDE_UI_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_UI_MANAGER, ValideUIManagerClass))

typedef struct _ValideUIManager ValideUIManager;
typedef struct _ValideUIManagerClass ValideUIManagerClass;

#define VALIDE_TYPE_DOCUMENT_STATE (valide_document_state_get_type ())

#define VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT (valide_abstract_text_document_get_type ())
#define VALIDE_ABSTRACT_TEXT_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocument))
#define VALIDE_ABSTRACT_TEXT_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocumentClass))
#define VALIDE_IS_ABSTRACT_TEXT_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT))
#define VALIDE_IS_ABSTRACT_TEXT_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT))
#define VALIDE_ABSTRACT_TEXT_DOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocumentClass))

typedef struct _ValideAbstractTextDocument ValideAbstractTextDocument;
typedef struct _ValideAbstractTextDocumentClass ValideAbstractTextDocumentClass;

typedef enum  {
	VALIDE_ERROR_PARSING_FILE,
	VALIDE_ERROR_PARSING_LINE
} ValideErrorParsing;
#define VALIDE_ERROR_PARSING valide_error_parsing_quark ()
/**
 * A class representing an program can launch by #ExecutableManager.
 */
struct _ValideExecutable {
	GObject parent_instance;
	ValideExecutablePrivate * priv;
};

struct _ValideExecutableClass {
	GObjectClass parent_class;
	char* (*get_start_msg) (ValideExecutable* self);
	char* (*get_end_msg) (ValideExecutable* self);
	GdkColor (*get_color_line) (ValideExecutable* self, char** line);
	void (*line_activated) (ValideExecutable* self, const char* line);
	char* (*get_executable) (ValideExecutable* self);
	void (*set_executable) (ValideExecutable* self, char* value);
};

/**
 * A class for represente a rectangular position
 */
struct _ValidePosition {
	gint start_line;
	gint end_line;
	gint start_row;
	gint end_row;
};

/**
 * The vala compiler
 */
struct _ValideCompiler {
	ValideExecutable parent_instance;
	ValideCompilerPrivate * priv;
};

struct _ValideCompilerClass {
	ValideExecutableClass parent_class;
	char* (*get_file) (ValideCompiler* self, const char* line, GError** error);
	ValidePosition (*get_line) (ValideCompiler* self, const char* line, GError** error);
};

struct _ValideCompilerPrivate {
	ValideProject* _project;
};

typedef enum  {
	VALIDE_DOCUMENT_STATE_NORMAL,
	VALIDE_DOCUMENT_STATE_EXTERNALLY_MODIFIED,
	VALIDE_DOCUMENT_STATE_BAD_ENCODING,
	VALIDE_DOCUMENT_STATE_INFO
} ValideDocumentState;

/**
 * An interface for document
 */
struct _ValideiDocumentIface {
	GTypeInterface parent_iface;
	void (*open) (ValideiDocument* self, const char* path, GError** error);
	void (*save) (ValideiDocument* self, const char* current_folder);
	void (*save_as) (ValideiDocument* self, const char* current_folder);
	void (*rename) (ValideiDocument* self);
	gboolean (*close) (ValideiDocument* self);
	GdkPixbuf* (*get_icon) (ValideiDocument* self);
	char* (*get_tooltips) (ValideiDocument* self);
	void (*setup_ui) (ValideiDocument* self, ValideUIManager* ui_manager);
	ValideDocumentState (*get_state) (ValideiDocument* self);
	void (*set_state) (ValideiDocument* self, ValideDocumentState value);
	gboolean (*get_is_save) (ValideiDocument* self);
	void (*set_is_save) (ValideiDocument* self, gboolean value);
	const char* (*get_path) (ValideiDocument* self);
	void (*set_path) (ValideiDocument* self, const char* value);
	const char* (*get_title) (ValideiDocument* self);
	const char* (*get_filename) (ValideiDocument* self);
};

typedef enum  {
	VALIDE_DOCUMENT_ERROR_UNKNOW_DOCUMENT,
	VALIDE_DOCUMENT_ERROR_BAD_URI,
	VALIDE_DOCUMENT_ERROR_NO_DOCUMENT,
	VALIDE_DOCUMENT_ERROR_MIME
} ValideDocumentError;
#define VALIDE_DOCUMENT_ERROR valide_document_error_quark ()

static gpointer valide_compiler_parent_class = NULL;

GQuark valide_error_parsing_quark (void);
GType valide_executable_get_type (void);
GType valide_compiler_get_type (void);
GType valide_position_get_type (void);
ValidePosition* valide_position_dup (const ValidePosition* self);
void valide_position_free (ValidePosition* self);
GType valide_project_get_type (void);
#define VALIDE_COMPILER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_COMPILER, ValideCompilerPrivate))
enum  {
	VALIDE_COMPILER_DUMMY_PROPERTY,
	VALIDE_COMPILER_PROJECT,
	VALIDE_COMPILER_EXECUTABLE
};
#define VALIDE_COMPILER_EXECUTABLE_PATH "valac -q"
char* valide_compiler_get_file (ValideCompiler* self, const char* line, GError** error);
static char* valide_compiler_real_get_file (ValideCompiler* self, const char* line, GError** error);
ValidePosition valide_compiler_get_line (ValideCompiler* self, const char* line, GError** error);
static ValidePosition valide_compiler_real_get_line (ValideCompiler* self, const char* line, GError** error);
#define VALIDE_UTILS_COLOR_ERROR "#FF0000"
#define VALIDE_UTILS_COLOR_SUCCESS "#19A630"
#define VALIDE_UTILS_COLOR_WARNING "#ED7F10"
GdkColor valide_executable_get_color_line (ValideExecutable* self, char** line);
static GdkColor valide_compiler_real_get_color_line (ValideExecutable* base, char** line);
char* valide_executable_get_start_msg (ValideExecutable* self);
char* valide_executable_get_end_msg (ValideExecutable* self);
ValideProject* valide_compiler_get_project (ValideCompiler* self);
GType valide_ui_manager_get_type (void);
GType valide_document_state_get_type (void);
GType valide_idocument_get_type (void);
GQuark valide_document_error_quark (void);
ValideiDocument* valide_project_open_file (ValideProject* self, const char* filename, GError** error);
GType valide_abstract_text_document_get_type (void);
void valide_abstract_text_document_select_range (ValideAbstractTextDocument* self, const ValidePosition* pos);
void valide_abstract_text_document_grab_focus (ValideAbstractTextDocument* self);
static void valide_compiler_real_line_activated (ValideExecutable* base, const char* line);
static void valide_compiler_set_project (ValideCompiler* self, ValideProject* value);
ValideCompiler* valide_compiler_new (ValideProject* project);
ValideCompiler* valide_compiler_construct (GType object_type, ValideProject* project);
static void valide_compiler_finalize (GObject* obj);
static void valide_compiler_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_compiler_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);



GQuark valide_error_parsing_quark (void) {
	return g_quark_from_static_string ("valide_error_parsing-quark");
}


/**
   * Get the filename from command line
   *
   * @param line A comand line
   *
   * @return The name of the file
   */
#line 59 "compiler.vala"
static char* valide_compiler_real_get_file (ValideCompiler* self, const char* line, GError** error) {
#line 250 "compiler.c"
	char* result;
	GError * _inner_error_;
	gint fields_size;
	gint fields_length1;
	char** fields;
	char** _tmp1_;
	char** _tmp0_;
	const char* _tmp2_;
#line 59 "compiler.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 59 "compiler.vala"
	g_return_val_if_fail (line != NULL, NULL);
#line 263 "compiler.c"
	_inner_error_ = NULL;
	fields = (fields_length1 = 0, NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
#line 63 "compiler.vala"
	fields = (_tmp1_ = _tmp0_ = g_strsplit (line, ":", -1), fields = (_vala_array_free (fields, fields_length1, (GDestroyNotify) g_free), NULL), fields_length1 = _vala_array_length (_tmp0_), fields_size = fields_length1, _tmp1_);
#line 64 "compiler.vala"
	if (g_strv_length (fields) < 2) {
#line 272 "compiler.c"
		_inner_error_ = g_error_new_literal (VALIDE_ERROR_PARSING, VALIDE_ERROR_PARSING_FILE, line);
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			fields = (_vala_array_free (fields, fields_length1, (GDestroyNotify) g_free), NULL);
			return NULL;
		}
	}
#line 68 "compiler.vala"
	_tmp2_ = NULL;
#line 282 "compiler.c"
	result = (_tmp2_ = fields[0], (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_));
	fields = (_vala_array_free (fields, fields_length1, (GDestroyNotify) g_free), NULL);
	return result;
	fields = (_vala_array_free (fields, fields_length1, (GDestroyNotify) g_free), NULL);
}


#line 59 "compiler.vala"
char* valide_compiler_get_file (ValideCompiler* self, const char* line, GError** error) {
#line 59 "compiler.vala"
	return VALIDE_COMPILER_GET_CLASS (self)->get_file (self, line, error);
#line 294 "compiler.c"
}


/**
   * Get the line from command line
   *
   * @param line A comand line
   *
   * @return The line position
   */
#line 78 "compiler.vala"
static ValidePosition valide_compiler_real_get_line (ValideCompiler* self, const char* line, GError** error) {
#line 307 "compiler.c"
	ValidePosition result;
	GError * _inner_error_;
	GRegex* regex;
	ValidePosition pos = {0};
	GMatchInfo* match_info;
	ValidePosition _tmp0_ = {0};
	GRegex* _tmp1_;
	GRegex* _tmp2_;
	GMatchInfo* _tmp5_;
	gboolean _tmp4_;
	GMatchInfo* _tmp3_;
	gint i;
	_inner_error_ = NULL;
	regex = NULL;
	match_info = NULL;
#line 84 "compiler.vala"
	pos = (memset (&_tmp0_, 0, sizeof (ValidePosition)), _tmp0_);
#line 325 "compiler.c"
	_tmp1_ = g_regex_new ("([\\d]+)", G_REGEX_OPTIMIZE, G_REGEX_MATCH_NOTEMPTY, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
		(match_info == NULL) ? NULL : (match_info = (g_match_info_free (match_info), NULL));
		return;
	}
	_tmp2_ = NULL;
#line 85 "compiler.vala"
	regex = (_tmp2_ = _tmp1_, (regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL)), _tmp2_);
#line 336 "compiler.c"
	/*"([\\d]+)\\.([\\d]+)-([\\d]+)\\.([\\d]+)",*/
	_tmp5_ = NULL;
	_tmp3_ = NULL;
#line 89 "compiler.vala"
	_tmp4_ = g_regex_match (regex, line, 0, &_tmp3_);
#line 89 "compiler.vala"
	match_info = (_tmp5_ = _tmp3_, (match_info == NULL) ? NULL : (match_info = (g_match_info_free (match_info), NULL)), _tmp5_);
#line 89 "compiler.vala"
	_tmp4_;
#line 346 "compiler.c"
	i = 0;
#line 92 "compiler.vala"
	while (TRUE) {
#line 92 "compiler.vala"
		if (!g_match_info_matches (match_info)) {
#line 92 "compiler.vala"
			break;
#line 354 "compiler.c"
		}
#line 94 "compiler.vala"
		switch (i) {
#line 358 "compiler.c"
			case 0:
			{
				char* _tmp6_;
#line 2698 "glib-2.0.vapi"
				_tmp6_ = NULL;
#line 97 "compiler.vala"
				pos.start_line = atoi (_tmp6_ = g_match_info_fetch (match_info, 0)) - 1;
#line 366 "compiler.c"
				_tmp6_ = (g_free (_tmp6_), NULL);
#line 98 "compiler.vala"
				break;
#line 370 "compiler.c"
			}
			case 1:
			{
				char* _tmp7_;
#line 2698 "glib-2.0.vapi"
				_tmp7_ = NULL;
#line 100 "compiler.vala"
				pos.start_row = atoi (_tmp7_ = g_match_info_fetch (match_info, 0)) - 1;
#line 379 "compiler.c"
				_tmp7_ = (g_free (_tmp7_), NULL);
#line 101 "compiler.vala"
				break;
#line 383 "compiler.c"
			}
			case 2:
			{
				char* _tmp8_;
#line 2698 "glib-2.0.vapi"
				_tmp8_ = NULL;
#line 103 "compiler.vala"
				pos.end_line = atoi (_tmp8_ = g_match_info_fetch (match_info, 0)) - 1;
#line 392 "compiler.c"
				_tmp8_ = (g_free (_tmp8_), NULL);
#line 104 "compiler.vala"
				break;
#line 396 "compiler.c"
			}
			case 3:
			{
				char* _tmp9_;
#line 2698 "glib-2.0.vapi"
				_tmp9_ = NULL;
#line 106 "compiler.vala"
				pos.end_row = atoi (_tmp9_ = g_match_info_fetch (match_info, 0));
#line 405 "compiler.c"
				_tmp9_ = (g_free (_tmp9_), NULL);
#line 107 "compiler.vala"
				break;
#line 409 "compiler.c"
			}
		}
#line 109 "compiler.vala"
		i++;
#line 110 "compiler.vala"
		g_match_info_next (match_info, &_inner_error_);
#line 416 "compiler.c"
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
			(match_info == NULL) ? NULL : (match_info = (g_match_info_free (match_info), NULL));
			return;
		}
	}
#line 112 "compiler.vala"
	if (i < 4) {
#line 114 "compiler.vala"
		pos.end_line = pos.start_line;
#line 115 "compiler.vala"
		pos.start_row = pos.end_row = 0;
#line 430 "compiler.c"
	}
	result = pos;
	(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
	(match_info == NULL) ? NULL : (match_info = (g_match_info_free (match_info), NULL));
	return result;
}


#line 78 "compiler.vala"
ValidePosition valide_compiler_get_line (ValideCompiler* self, const char* line, GError** error) {
#line 78 "compiler.vala"
	return VALIDE_COMPILER_GET_CLASS (self)->get_line (self, line, error);
#line 443 "compiler.c"
}


/**
   * @see Valide.Executable.get_color_line
   */
#line 123 "compiler.vala"
static GdkColor valide_compiler_real_get_color_line (ValideExecutable* base, char** line) {
#line 452 "compiler.c"
	ValideCompiler * self;
	GdkColor result;
	GdkColor color = {0};
	self = (ValideCompiler*) base;
#line 127 "compiler.vala"
	if (g_str_has_prefix ((*line), "Compilation failed")) {
#line 129 "compiler.vala"
		gdk_color_parse (VALIDE_UTILS_COLOR_ERROR, &color);
#line 461 "compiler.c"
	} else {
#line 131 "compiler.vala"
		if (strstr ((*line), "error:") != NULL) {
#line 133 "compiler.vala"
			gdk_color_parse (VALIDE_UTILS_COLOR_ERROR, &color);
#line 467 "compiler.c"
		} else {
#line 135 "compiler.vala"
			if (g_str_has_prefix ((*line), "Compilation succeeded")) {
#line 137 "compiler.vala"
				gdk_color_parse (VALIDE_UTILS_COLOR_SUCCESS, &color);
#line 473 "compiler.c"
			} else {
#line 139 "compiler.vala"
				if (strstr ((*line), "warning:") != NULL) {
#line 141 "compiler.vala"
					gdk_color_parse (VALIDE_UTILS_COLOR_WARNING, &color);
#line 479 "compiler.c"
				} else {
#line 145 "compiler.vala"
					color = VALIDE_EXECUTABLE_CLASS (valide_compiler_parent_class)->get_color_line (VALIDE_EXECUTABLE (self), &(*line));
#line 483 "compiler.c"
				}
			}
		}
	}
	result = color;
	return result;
}


/**
   * @see Valide.Executable.line_activated
   */
#line 153 "compiler.vala"
static void valide_compiler_real_line_activated (ValideExecutable* base, const char* line) {
#line 498 "compiler.c"
	ValideCompiler * self;
	GError * _inner_error_;
	gboolean _tmp0_;
	char* _tmp1_;
	gboolean _tmp2_;
	self = (ValideCompiler*) base;
#line 153 "compiler.vala"
	g_return_if_fail (line != NULL);
#line 507 "compiler.c"
	_inner_error_ = NULL;
	_tmp0_ = FALSE;
#line 360 "executable.vala"
	_tmp1_ = NULL;
#line 155 "compiler.vala"
	if ((_tmp2_ = _vala_strcmp0 (line, _tmp1_ = valide_executable_get_start_msg ((ValideExecutable*) self)) != 0, _tmp1_ = (g_free (_tmp1_), NULL), _tmp2_)) {
#line 514 "compiler.c"
		char* _tmp3_;
#line 372 "executable.vala"
		_tmp3_ = NULL;
#line 155 "compiler.vala"
		_tmp0_ = _vala_strcmp0 (line, _tmp3_ = valide_executable_get_end_msg ((ValideExecutable*) self)) != 0;
#line 520 "compiler.c"
		_tmp3_ = (g_free (_tmp3_), NULL);
	} else {
#line 155 "compiler.vala"
		_tmp0_ = FALSE;
#line 525 "compiler.c"
	}
#line 155 "compiler.vala"
	if (_tmp0_) {
#line 529 "compiler.c"
		{
#line 159 "compiler.vala"
			if (self->priv->_project != NULL) {
#line 533 "compiler.c"
				char* file;
				ValideiDocument* document;
				char* _tmp4_;
				char* _tmp5_;
				ValideiDocument* _tmp6_;
				ValideiDocument* _tmp7_;
				file = NULL;
				document = NULL;
				_tmp4_ = valide_compiler_get_file (self, line, &_inner_error_);
				if (_inner_error_ != NULL) {
					file = (g_free (file), NULL);
					(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
					goto __catch3_g_error;
					goto __finally3;
				}
				_tmp5_ = NULL;
#line 164 "compiler.vala"
				file = (_tmp5_ = _tmp4_, file = (g_free (file), NULL), _tmp5_);
#line 552 "compiler.c"
				_tmp6_ = valide_project_open_file (self->priv->_project, file, &_inner_error_);
				if (_inner_error_ != NULL) {
					file = (g_free (file), NULL);
					(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
					goto __catch3_g_error;
					goto __finally3;
				}
				_tmp7_ = NULL;
#line 165 "compiler.vala"
				document = (_tmp7_ = _tmp6_, (document == NULL) ? NULL : (document = (g_object_unref (document), NULL)), _tmp7_);
#line 563 "compiler.c"
				{
					ValidePosition pos = {0};
					ValidePosition _tmp8_;
					_tmp8_ = valide_compiler_get_line (self, line, &_inner_error_);
					if (_inner_error_ != NULL) {
						goto __catch4_g_error;
						goto __finally4;
					}
#line 170 "compiler.vala"
					pos = _tmp8_;
#line 574 "compiler.c"
					/* Only text documents have lines */
#line 172 "compiler.vala"
					if (VALIDE_IS_ABSTRACT_TEXT_DOCUMENT (document)) {
#line 578 "compiler.c"
						ValideiDocument* _tmp9_;
						ValideiDocument* _tmp10_;
#line 174 "compiler.vala"
						_tmp9_ = NULL;
#line 174 "compiler.vala"
						valide_abstract_text_document_select_range ((_tmp9_ = document, VALIDE_IS_ABSTRACT_TEXT_DOCUMENT (_tmp9_) ? ((ValideAbstractTextDocument*) _tmp9_) : NULL), &pos);
#line 175 "compiler.vala"
						_tmp10_ = NULL;
#line 175 "compiler.vala"
						valide_abstract_text_document_grab_focus ((_tmp10_ = document, VALIDE_IS_ABSTRACT_TEXT_DOCUMENT (_tmp10_) ? ((ValideAbstractTextDocument*) _tmp10_) : NULL));
#line 589 "compiler.c"
					}
				}
				goto __finally4;
				__catch4_g_error:
				{
					GError * e;
					e = _inner_error_;
					_inner_error_ = NULL;
					{
#line 180 "compiler.vala"
						g_debug (e->message);
#line 601 "compiler.c"
						(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
					}
				}
				__finally4:
				if (_inner_error_ != NULL) {
					file = (g_free (file), NULL);
					(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
					goto __catch3_g_error;
					goto __finally3;
				}
				file = (g_free (file), NULL);
				(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
			}
		}
		goto __finally3;
		__catch3_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
#line 186 "compiler.vala"
				g_warning (e->message);
#line 625 "compiler.c"
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally3:
		if (_inner_error_ != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
}


/**
   * Create a new Valide.Compiler
   *
   * @param project A project
   */
#line 196 "compiler.vala"
ValideCompiler* valide_compiler_construct (GType object_type, ValideProject* project) {
#line 646 "compiler.c"
	GParameter * __params;
	GParameter * __params_it;
	ValideCompiler * self;
#line 196 "compiler.vala"
	g_return_val_if_fail (project != NULL, NULL);
#line 652 "compiler.c"
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
#line 198 "compiler.vala"
	__params_it->name = "project";
#line 198 "compiler.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_PROJECT);
#line 198 "compiler.vala"
	g_value_set_object (&__params_it->value, project);
#line 198 "compiler.vala"
	__params_it++;
#line 663 "compiler.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 196 "compiler.vala"
ValideCompiler* valide_compiler_new (ValideProject* project) {
#line 196 "compiler.vala"
	return valide_compiler_construct (VALIDE_TYPE_COMPILER, project);
#line 678 "compiler.c"
}


ValideProject* valide_compiler_get_project (ValideCompiler* self) {
	ValideProject* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_project;
	return result;
}


static void valide_compiler_set_project (ValideCompiler* self, ValideProject* value) {
	ValideProject* _tmp1_;
	ValideProject* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 39 "compiler.vala"
	_tmp0_ = NULL;
#line 697 "compiler.c"
	self->priv->_project = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_project == NULL) ? NULL : (self->priv->_project = (g_object_unref (self->priv->_project), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "project");
}


static const char* valide_compiler_real_get_executable (ValideExecutable* base) {
	const char* result;
	ValideCompiler* self;
	self = (ValideCompiler*) base;
	result = VALIDE_COMPILER_EXECUTABLE_PATH;
	return result;
}


static void valide_compiler_real_set_executable (ValideExecutable* base, const char* value) {
	ValideCompiler* self;
	self = (ValideCompiler*) base;
	g_object_notify ((GObject *) self, "executable");
}


static void valide_compiler_class_init (ValideCompilerClass * klass) {
	valide_compiler_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideCompilerPrivate));
	VALIDE_COMPILER_CLASS (klass)->get_file = valide_compiler_real_get_file;
	VALIDE_COMPILER_CLASS (klass)->get_line = valide_compiler_real_get_line;
	VALIDE_EXECUTABLE_CLASS (klass)->get_color_line = valide_compiler_real_get_color_line;
	VALIDE_EXECUTABLE_CLASS (klass)->line_activated = valide_compiler_real_line_activated;
	VALIDE_EXECUTABLE_CLASS (klass)->get_executable = valide_compiler_real_get_executable;
	VALIDE_EXECUTABLE_CLASS (klass)->set_executable = valide_compiler_real_set_executable;
	G_OBJECT_CLASS (klass)->get_property = valide_compiler_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_compiler_set_property;
	G_OBJECT_CLASS (klass)->finalize = valide_compiler_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_COMPILER_PROJECT, g_param_spec_object ("project", "project", "project", VALIDE_TYPE_PROJECT, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_override_property (G_OBJECT_CLASS (klass), VALIDE_COMPILER_EXECUTABLE, "executable");
}


static void valide_compiler_instance_init (ValideCompiler * self) {
	self->priv = VALIDE_COMPILER_GET_PRIVATE (self);
}


static void valide_compiler_finalize (GObject* obj) {
	ValideCompiler * self;
	self = VALIDE_COMPILER (obj);
	(self->priv->_project == NULL) ? NULL : (self->priv->_project = (g_object_unref (self->priv->_project), NULL));
	G_OBJECT_CLASS (valide_compiler_parent_class)->finalize (obj);
}


GType valide_compiler_get_type (void) {
	static GType valide_compiler_type_id = 0;
	if (valide_compiler_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideCompilerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_compiler_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideCompiler), 0, (GInstanceInitFunc) valide_compiler_instance_init, NULL };
		valide_compiler_type_id = g_type_register_static (VALIDE_TYPE_EXECUTABLE, "ValideCompiler", &g_define_type_info, 0);
	}
	return valide_compiler_type_id;
}


static void valide_compiler_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideCompiler * self;
	gpointer boxed;
	self = VALIDE_COMPILER (object);
	switch (property_id) {
		case VALIDE_COMPILER_PROJECT:
		g_value_set_object (value, valide_compiler_get_project (self));
		break;
		case VALIDE_COMPILER_EXECUTABLE:
		g_value_set_string (value, valide_executable_get_executable ((ValideExecutable*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_compiler_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideCompiler * self;
	self = VALIDE_COMPILER (object);
	switch (property_id) {
		case VALIDE_COMPILER_PROJECT:
		valide_compiler_set_project (self, g_value_get_object (value));
		break;
		case VALIDE_COMPILER_EXECUTABLE:
		valide_executable_set_executable ((ValideExecutable*) self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




