/* config-manager.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>


#define VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER (valide_abstract_config_manager_get_type ())
#define VALIDE_ABSTRACT_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManager))
#define VALIDE_ABSTRACT_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManagerClass))
#define VALIDE_IS_ABSTRACT_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER))
#define VALIDE_IS_ABSTRACT_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER))
#define VALIDE_ABSTRACT_CONFIG_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManagerClass))

typedef struct _ValideAbstractConfigManager ValideAbstractConfigManager;
typedef struct _ValideAbstractConfigManagerClass ValideAbstractConfigManagerClass;
typedef struct _ValideAbstractConfigManagerPrivate ValideAbstractConfigManagerPrivate;

#define VALIDE_TYPE_WINDOW (valide_window_get_type ())
#define VALIDE_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_WINDOW, ValideWindow))
#define VALIDE_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_WINDOW, ValideWindowClass))
#define VALIDE_IS_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_WINDOW))
#define VALIDE_IS_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_WINDOW))
#define VALIDE_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_WINDOW, ValideWindowClass))

typedef struct _ValideWindow ValideWindow;
typedef struct _ValideWindowClass ValideWindowClass;

#define VALIDE_TYPE_CONFIG_MANAGER (valide_config_manager_get_type ())
#define VALIDE_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_CONFIG_MANAGER, ValideConfigManager))
#define VALIDE_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_CONFIG_MANAGER, ValideConfigManagerClass))
#define VALIDE_IS_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_CONFIG_MANAGER))
#define VALIDE_IS_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_CONFIG_MANAGER))
#define VALIDE_CONFIG_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_CONFIG_MANAGER, ValideConfigManagerClass))

typedef struct _ValideConfigManager ValideConfigManager;
typedef struct _ValideConfigManagerClass ValideConfigManagerClass;
typedef struct _ValideConfigManagerPrivate ValideConfigManagerPrivate;

#define VALIDE_TYPE_ABSTRACT_PREFERENCES_DIALOG (valide_abstract_preferences_dialog_get_type ())
#define VALIDE_ABSTRACT_PREFERENCES_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_PREFERENCES_DIALOG, ValideAbstractPreferencesDialog))
#define VALIDE_ABSTRACT_PREFERENCES_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_PREFERENCES_DIALOG, ValideAbstractPreferencesDialogClass))
#define VALIDE_IS_ABSTRACT_PREFERENCES_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_PREFERENCES_DIALOG))
#define VALIDE_IS_ABSTRACT_PREFERENCES_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_PREFERENCES_DIALOG))
#define VALIDE_ABSTRACT_PREFERENCES_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_PREFERENCES_DIALOG, ValideAbstractPreferencesDialogClass))

typedef struct _ValideAbstractPreferencesDialog ValideAbstractPreferencesDialog;
typedef struct _ValideAbstractPreferencesDialogClass ValideAbstractPreferencesDialogClass;

#define VALIDE_TYPE_PREFERENCES_DIALOG (valide_preferences_dialog_get_type ())
#define VALIDE_PREFERENCES_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PREFERENCES_DIALOG, ValidePreferencesDialog))
#define VALIDE_PREFERENCES_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PREFERENCES_DIALOG, ValidePreferencesDialogClass))
#define VALIDE_IS_PREFERENCES_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PREFERENCES_DIALOG))
#define VALIDE_IS_PREFERENCES_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PREFERENCES_DIALOG))
#define VALIDE_PREFERENCES_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PREFERENCES_DIALOG, ValidePreferencesDialogClass))

typedef struct _ValidePreferencesDialog ValidePreferencesDialog;
typedef struct _ValidePreferencesDialogClass ValidePreferencesDialogClass;

/**
 * Abstract class for manage configation files.
 */
struct _ValideAbstractConfigManager {
	GObject parent_instance;
	ValideAbstractConfigManagerPrivate * priv;
};

struct _ValideAbstractConfigManagerClass {
	GObjectClass parent_class;
	void (*preferences) (ValideAbstractConfigManager* self, ValideWindow* window);
	void (*first_run) (ValideAbstractConfigManager* self);
	char* (*get_config_file) (ValideAbstractConfigManager* self);
};

/**
 * Class for manage the configuration file
 */
struct _ValideConfigManager {
	ValideAbstractConfigManager parent_instance;
	ValideConfigManagerPrivate * priv;
};

struct _ValideConfigManagerClass {
	ValideAbstractConfigManagerClass parent_class;
};


static ValideConfigManager* valide_config_manager_instance;
static ValideConfigManager* valide_config_manager_instance = NULL;
static gpointer valide_config_manager_parent_class = NULL;

GType valide_abstract_config_manager_get_type (void);
GType valide_window_get_type (void);
GType valide_config_manager_get_type (void);
enum  {
	VALIDE_CONFIG_MANAGER_DUMMY_PROPERTY
};
#define VALIDE_CONFIG_MANAGER_CONFIG_FILE_ "config.ini"
char* valide_abstract_config_manager_create_preferences_file (ValideAbstractConfigManager* self, const char* filename, const char* contents);
static char* valide_config_manager_real_get_config_file (ValideAbstractConfigManager* base);
static ValideConfigManager* valide_config_manager_new (void);
static ValideConfigManager* valide_config_manager_construct (GType object_type);
char* valide_abstract_config_manager_get_home_dir (ValideAbstractConfigManager* self);
static void valide_config_manager_real_first_run (ValideAbstractConfigManager* base);
ValideConfigManager* valide_config_manager_get_instance (void);
GType valide_abstract_preferences_dialog_get_type (void);
GType valide_preferences_dialog_get_type (void);
ValidePreferencesDialog* valide_preferences_dialog_new (ValideWindow* window);
ValidePreferencesDialog* valide_preferences_dialog_construct (GType object_type, ValideWindow* window);
gint valide_preferences_dialog_run (ValidePreferencesDialog* self);
void valide_preferences_dialog_destroy (ValidePreferencesDialog* self);
static void valide_config_manager_real_preferences (ValideAbstractConfigManager* base, ValideWindow* window);
static void valide_config_manager_finalize (GObject* obj);



/**
   * @see Valide.AbstractConfigManager.config_file
   */
#line 33 "config-manager.vala"
static char* valide_config_manager_real_get_config_file (ValideAbstractConfigManager* base) {
#line 144 "config-manager.c"
	ValideConfigManager * self;
	char* result;
	self = (ValideConfigManager*) base;
	result = valide_abstract_config_manager_create_preferences_file ((ValideAbstractConfigManager*) self, VALIDE_CONFIG_MANAGER_CONFIG_FILE_, "[General]\nfull-screen=true\nwindow-width=1024\nwindow-height=768\n\n[GtkSourceView]\nauto-indent=true\nindent-on-tab=true\ntab-width=2\nhighlight-current-line=true\nshow-right-margin=true\nright-margin-position=80\nshow-line-numbers=false\ninsert-spaces-instead-of-tabs=true\nauto-indent=true\nhighlight-brackets=true\nstyle-scheme=classic\nfont=Monospace 12\n\n[Plugins]\nTodo=true\nSymbol=true\nCompletion=true\nBrowser=true\nFile browser=true\nOpened Documents=true\n\n[SymbolBrowser]\nshow-language=false\nshow-source-file=false\nshow-line-number=false\nhierarchic-view=true\n");
	return result;
}


#line 70 "config-manager.vala"
static ValideConfigManager* valide_config_manager_construct (GType object_type) {
#line 155 "config-manager.c"
	ValideConfigManager * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


#line 70 "config-manager.vala"
static ValideConfigManager* valide_config_manager_new (void) {
#line 70 "config-manager.vala"
	return valide_config_manager_construct (VALIDE_TYPE_CONFIG_MANAGER);
#line 166 "config-manager.c"
}


/**
   * @see Valide.AbstractConfigManager.first_run
   */
#line 77 "config-manager.vala"
static void valide_config_manager_real_first_run (ValideAbstractConfigManager* base) {
#line 175 "config-manager.c"
	ValideConfigManager * self;
	char* _tmp1_;
	char* _tmp0_;
	char* _tmp3_;
	char* _tmp2_;
	self = (ValideConfigManager*) base;
#line 2034 "glib-2.0.vapi"
	_tmp1_ = NULL;
#line 49 "abstract-config-manager.vala"
	_tmp0_ = NULL;
#line 79 "config-manager.vala"
	g_mkdir (_tmp1_ = g_build_filename (_tmp0_ = valide_abstract_config_manager_get_home_dir ((ValideAbstractConfigManager*) self), "plugins", NULL), 0700);
#line 188 "config-manager.c"
	_tmp1_ = (g_free (_tmp1_), NULL);
	_tmp0_ = (g_free (_tmp0_), NULL);
#line 2034 "glib-2.0.vapi"
	_tmp3_ = NULL;
#line 49 "abstract-config-manager.vala"
	_tmp2_ = NULL;
#line 80 "config-manager.vala"
	g_mkdir (_tmp3_ = g_build_filename (_tmp2_ = valide_abstract_config_manager_get_home_dir ((ValideAbstractConfigManager*) self), "template", NULL), 0700);
#line 197 "config-manager.c"
	_tmp3_ = (g_free (_tmp3_), NULL);
	_tmp2_ = (g_free (_tmp2_), NULL);
}


/**
   * Get the uniq instance of Valide.ConfigManager
   *
   * @return The Valide.ConfigManager instance
   */
#line 88 "config-manager.vala"
ValideConfigManager* valide_config_manager_get_instance (void) {
#line 210 "config-manager.c"
	ValideConfigManager* result;
	ValideConfigManager* _tmp1_;
#line 90 "config-manager.vala"
	if (valide_config_manager_instance == NULL) {
#line 215 "config-manager.c"
		ValideConfigManager* _tmp0_;
		_tmp0_ = NULL;
#line 92 "config-manager.vala"
		valide_config_manager_instance = (_tmp0_ = valide_config_manager_new (), (valide_config_manager_instance == NULL) ? NULL : (valide_config_manager_instance = (g_object_unref (valide_config_manager_instance), NULL)), _tmp0_);
#line 220 "config-manager.c"
	}
#line 94 "config-manager.vala"
	_tmp1_ = NULL;
#line 224 "config-manager.c"
	result = (_tmp1_ = valide_config_manager_instance, (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_));
	return result;
}


/**
   * @see Valide.AbstractConfigManager.preferences
   */
#line 100 "config-manager.vala"
static void valide_config_manager_real_preferences (ValideAbstractConfigManager* base, ValideWindow* window) {
#line 235 "config-manager.c"
	ValideConfigManager * self;
	ValidePreferencesDialog* dialog;
	ValidePreferencesDialog* _tmp0_;
	self = (ValideConfigManager*) base;
#line 100 "config-manager.vala"
	g_return_if_fail (window != NULL);
#line 242 "config-manager.c"
	dialog = NULL;
	_tmp0_ = NULL;
#line 104 "config-manager.vala"
	dialog = (_tmp0_ = g_object_ref_sink (valide_preferences_dialog_new (window)), (dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL)), _tmp0_);
#line 105 "config-manager.vala"
	valide_preferences_dialog_run (dialog);
#line 106 "config-manager.vala"
	valide_preferences_dialog_destroy (dialog);
#line 251 "config-manager.c"
	(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
}


static void valide_config_manager_class_init (ValideConfigManagerClass * klass) {
	valide_config_manager_parent_class = g_type_class_peek_parent (klass);
	VALIDE_ABSTRACT_CONFIG_MANAGER_CLASS (klass)->get_config_file = valide_config_manager_real_get_config_file;
	VALIDE_ABSTRACT_CONFIG_MANAGER_CLASS (klass)->first_run = valide_config_manager_real_first_run;
	VALIDE_ABSTRACT_CONFIG_MANAGER_CLASS (klass)->preferences = valide_config_manager_real_preferences;
	G_OBJECT_CLASS (klass)->finalize = valide_config_manager_finalize;
}


static void valide_config_manager_instance_init (ValideConfigManager * self) {
}


static void valide_config_manager_finalize (GObject* obj) {
	ValideConfigManager * self;
	self = VALIDE_CONFIG_MANAGER (obj);
	G_OBJECT_CLASS (valide_config_manager_parent_class)->finalize (obj);
}


GType valide_config_manager_get_type (void) {
	static GType valide_config_manager_type_id = 0;
	if (valide_config_manager_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideConfigManagerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_config_manager_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideConfigManager), 0, (GInstanceInitFunc) valide_config_manager_instance_init, NULL };
		valide_config_manager_type_id = g_type_register_static (VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, "ValideConfigManager", &g_define_type_info, 0);
	}
	return valide_config_manager_type_id;
}




