/* document-manager.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <glib/gi18n-lib.h>
#include <config.h>
#include <gio/gio.h>
#include <gdk/gdk.h>
#include <gtksourceview/gtksourceview.h>


#define VALIDE_TYPE_DOCUMENT_MANAGER (valide_document_manager_get_type ())
#define VALIDE_DOCUMENT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_DOCUMENT_MANAGER, ValideDocumentManager))
#define VALIDE_DOCUMENT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_DOCUMENT_MANAGER, ValideDocumentManagerClass))
#define VALIDE_IS_DOCUMENT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_DOCUMENT_MANAGER))
#define VALIDE_IS_DOCUMENT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_DOCUMENT_MANAGER))
#define VALIDE_DOCUMENT_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_DOCUMENT_MANAGER, ValideDocumentManagerClass))

typedef struct _ValideDocumentManager ValideDocumentManager;
typedef struct _ValideDocumentManagerClass ValideDocumentManagerClass;
typedef struct _ValideDocumentManagerPrivate ValideDocumentManagerPrivate;

#define VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT (valide_abstract_text_document_get_type ())
#define VALIDE_ABSTRACT_TEXT_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocument))
#define VALIDE_ABSTRACT_TEXT_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocumentClass))
#define VALIDE_IS_ABSTRACT_TEXT_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT))
#define VALIDE_IS_ABSTRACT_TEXT_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT))
#define VALIDE_ABSTRACT_TEXT_DOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocumentClass))

typedef struct _ValideAbstractTextDocument ValideAbstractTextDocument;
typedef struct _ValideAbstractTextDocumentClass ValideAbstractTextDocumentClass;

#define VALIDE_TYPE_UI_MANAGER (valide_ui_manager_get_type ())
#define VALIDE_UI_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_UI_MANAGER, ValideUIManager))
#define VALIDE_UI_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_UI_MANAGER, ValideUIManagerClass))
#define VALIDE_IS_UI_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_UI_MANAGER))
#define VALIDE_IS_UI_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_UI_MANAGER))
#define VALIDE_UI_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_UI_MANAGER, ValideUIManagerClass))

typedef struct _ValideUIManager ValideUIManager;
typedef struct _ValideUIManagerClass ValideUIManagerClass;

#define VALIDE_TYPE_IDOCUMENT (valide_idocument_get_type ())
#define VALIDE_IDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_IDOCUMENT, ValideiDocument))
#define VALIDE_IS_IDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_IDOCUMENT))
#define VALIDE_IDOCUMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), VALIDE_TYPE_IDOCUMENT, ValideiDocumentIface))

typedef struct _ValideiDocument ValideiDocument;
typedef struct _ValideiDocumentIface ValideiDocumentIface;

#define VALIDE_TYPE_DOCUMENT_STATE (valide_document_state_get_type ())

#define VALIDE_TYPE_TXT_DOCUMENT (valide_txt_document_get_type ())
#define VALIDE_TXT_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_TXT_DOCUMENT, ValideTxtDocument))
#define VALIDE_TXT_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_TXT_DOCUMENT, ValideTxtDocumentClass))
#define VALIDE_IS_TXT_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_TXT_DOCUMENT))
#define VALIDE_IS_TXT_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_TXT_DOCUMENT))
#define VALIDE_TXT_DOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_TXT_DOCUMENT, ValideTxtDocumentClass))

typedef struct _ValideTxtDocument ValideTxtDocument;
typedef struct _ValideTxtDocumentClass ValideTxtDocumentClass;

#define VALIDE_TYPE_GS_DOCUMENT (valide_gs_document_get_type ())
#define VALIDE_GS_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_GS_DOCUMENT, ValideGsDocument))
#define VALIDE_GS_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_GS_DOCUMENT, ValideGsDocumentClass))
#define VALIDE_IS_GS_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_GS_DOCUMENT))
#define VALIDE_IS_GS_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_GS_DOCUMENT))
#define VALIDE_GS_DOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_GS_DOCUMENT, ValideGsDocumentClass))

typedef struct _ValideGsDocument ValideGsDocument;
typedef struct _ValideGsDocumentClass ValideGsDocumentClass;

#define VALIDE_TYPE_VALA_DOCUMENT (valide_vala_document_get_type ())
#define VALIDE_VALA_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_VALA_DOCUMENT, ValideValaDocument))
#define VALIDE_VALA_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_VALA_DOCUMENT, ValideValaDocumentClass))
#define VALIDE_IS_VALA_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_VALA_DOCUMENT))
#define VALIDE_IS_VALA_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_VALA_DOCUMENT))
#define VALIDE_VALA_DOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_VALA_DOCUMENT, ValideValaDocumentClass))

typedef struct _ValideValaDocument ValideValaDocument;
typedef struct _ValideValaDocumentClass ValideValaDocumentClass;

#define VALIDE_TYPE_VAPI_DOCUMENT (valide_vapi_document_get_type ())
#define VALIDE_VAPI_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_VAPI_DOCUMENT, ValideVapiDocument))
#define VALIDE_VAPI_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_VAPI_DOCUMENT, ValideVapiDocumentClass))
#define VALIDE_IS_VAPI_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_VAPI_DOCUMENT))
#define VALIDE_IS_VAPI_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_VAPI_DOCUMENT))
#define VALIDE_VAPI_DOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_VAPI_DOCUMENT, ValideVapiDocumentClass))

typedef struct _ValideVapiDocument ValideVapiDocument;
typedef struct _ValideVapiDocumentClass ValideVapiDocumentClass;

#define VALIDE_TYPE_XML_DOCUMENT (valide_xml_document_get_type ())
#define VALIDE_XML_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_XML_DOCUMENT, ValideXmlDocument))
#define VALIDE_XML_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_XML_DOCUMENT, ValideXmlDocumentClass))
#define VALIDE_IS_XML_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_XML_DOCUMENT))
#define VALIDE_IS_XML_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_XML_DOCUMENT))
#define VALIDE_XML_DOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_XML_DOCUMENT, ValideXmlDocumentClass))

typedef struct _ValideXmlDocument ValideXmlDocument;
typedef struct _ValideXmlDocumentClass ValideXmlDocumentClass;

#define VALIDE_TYPE_GLADE_DOCUMENT (valide_glade_document_get_type ())
#define VALIDE_GLADE_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_GLADE_DOCUMENT, ValideGladeDocument))
#define VALIDE_GLADE_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_GLADE_DOCUMENT, ValideGladeDocumentClass))
#define VALIDE_IS_GLADE_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_GLADE_DOCUMENT))
#define VALIDE_IS_GLADE_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_GLADE_DOCUMENT))
#define VALIDE_GLADE_DOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_GLADE_DOCUMENT, ValideGladeDocumentClass))

typedef struct _ValideGladeDocument ValideGladeDocument;
typedef struct _ValideGladeDocumentClass ValideGladeDocumentClass;

#define VALIDE_TYPE_UI_DOCUMENT (valide_ui_document_get_type ())
#define VALIDE_UI_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_UI_DOCUMENT, ValideUiDocument))
#define VALIDE_UI_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_UI_DOCUMENT, ValideUiDocumentClass))
#define VALIDE_IS_UI_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_UI_DOCUMENT))
#define VALIDE_IS_UI_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_UI_DOCUMENT))
#define VALIDE_UI_DOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_UI_DOCUMENT, ValideUiDocumentClass))

typedef struct _ValideUiDocument ValideUiDocument;
typedef struct _ValideUiDocumentClass ValideUiDocumentClass;

#define VALIDE_TYPE_SOURCE_BUFFER (valide_source_buffer_get_type ())
#define VALIDE_SOURCE_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_SOURCE_BUFFER, ValideSourceBuffer))
#define VALIDE_SOURCE_BUFFER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_SOURCE_BUFFER, ValideSourceBufferClass))
#define VALIDE_IS_SOURCE_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_SOURCE_BUFFER))
#define VALIDE_IS_SOURCE_BUFFER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_SOURCE_BUFFER))
#define VALIDE_SOURCE_BUFFER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_SOURCE_BUFFER, ValideSourceBufferClass))

typedef struct _ValideSourceBuffer ValideSourceBuffer;
typedef struct _ValideSourceBufferClass ValideSourceBufferClass;

#define VALIDE_TYPE_ABSTRACT_CLOSE_DIALOG (valide_abstract_close_dialog_get_type ())
#define VALIDE_ABSTRACT_CLOSE_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_CLOSE_DIALOG, ValideAbstractCloseDialog))
#define VALIDE_ABSTRACT_CLOSE_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_CLOSE_DIALOG, ValideAbstractCloseDialogClass))
#define VALIDE_IS_ABSTRACT_CLOSE_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_CLOSE_DIALOG))
#define VALIDE_IS_ABSTRACT_CLOSE_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_CLOSE_DIALOG))
#define VALIDE_ABSTRACT_CLOSE_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_CLOSE_DIALOG, ValideAbstractCloseDialogClass))

typedef struct _ValideAbstractCloseDialog ValideAbstractCloseDialog;
typedef struct _ValideAbstractCloseDialogClass ValideAbstractCloseDialogClass;

#define VALIDE_TYPE_DOCUMENT_CLOSE_DIALOG (valide_document_close_dialog_get_type ())
#define VALIDE_DOCUMENT_CLOSE_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_DOCUMENT_CLOSE_DIALOG, ValideDocumentCloseDialog))
#define VALIDE_DOCUMENT_CLOSE_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_DOCUMENT_CLOSE_DIALOG, ValideDocumentCloseDialogClass))
#define VALIDE_IS_DOCUMENT_CLOSE_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_DOCUMENT_CLOSE_DIALOG))
#define VALIDE_IS_DOCUMENT_CLOSE_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_DOCUMENT_CLOSE_DIALOG))
#define VALIDE_DOCUMENT_CLOSE_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_DOCUMENT_CLOSE_DIALOG, ValideDocumentCloseDialogClass))

typedef struct _ValideDocumentCloseDialog ValideDocumentCloseDialog;
typedef struct _ValideDocumentCloseDialogClass ValideDocumentCloseDialogClass;

typedef enum  {
	VALIDE_DOCUMENT_ERROR_UNKNOW_DOCUMENT,
	VALIDE_DOCUMENT_ERROR_BAD_URI,
	VALIDE_DOCUMENT_ERROR_NO_DOCUMENT,
	VALIDE_DOCUMENT_ERROR_MIME
} ValideDocumentError;
#define VALIDE_DOCUMENT_ERROR valide_document_error_quark ()
/**
 * The manager for opened documents
 */
struct _ValideDocumentManager {
	GtkNotebook parent_instance;
	ValideDocumentManagerPrivate * priv;
};

struct _ValideDocumentManagerClass {
	GtkNotebookClass parent_class;
	void (*setup_ui) (ValideDocumentManager* self);
	ValideAbstractTextDocument* (*create_document) (ValideDocumentManager* self, const char* filename, GError** error);
};

typedef enum  {
	VALIDE_DOCUMENT_STATE_NORMAL,
	VALIDE_DOCUMENT_STATE_EXTERNALLY_MODIFIED,
	VALIDE_DOCUMENT_STATE_BAD_ENCODING,
	VALIDE_DOCUMENT_STATE_INFO
} ValideDocumentState;

/**
 * An interface for document
 */
struct _ValideiDocumentIface {
	GTypeInterface parent_iface;
	void (*open) (ValideiDocument* self, const char* path, GError** error);
	void (*save) (ValideiDocument* self, const char* current_folder);
	void (*save_as) (ValideiDocument* self, const char* current_folder);
	void (*rename) (ValideiDocument* self);
	gboolean (*close) (ValideiDocument* self);
	GdkPixbuf* (*get_icon) (ValideiDocument* self);
	char* (*get_tooltips) (ValideiDocument* self);
	void (*setup_ui) (ValideiDocument* self, ValideUIManager* ui_manager);
	ValideDocumentState (*get_state) (ValideiDocument* self);
	void (*set_state) (ValideiDocument* self, ValideDocumentState value);
	gboolean (*get_is_save) (ValideiDocument* self);
	void (*set_is_save) (ValideiDocument* self, gboolean value);
	const char* (*get_path) (ValideiDocument* self);
	void (*set_path) (ValideiDocument* self, const char* value);
	const char* (*get_title) (ValideiDocument* self);
	const char* (*get_filename) (ValideiDocument* self);
};

struct _ValideDocumentManagerPrivate {
	ValideUIManager* _ui_manager;
	GList* _documents;
};

typedef enum  {
	VALIDE_PROJECT_ERROR_NO_PROJECT,
	VALIDE_PROJECT_ERROR_MAX_OPEN
} ValideProjectError;
#define VALIDE_PROJECT_ERROR valide_project_error_quark ()

static gpointer valide_document_manager_parent_class = NULL;

GQuark valide_document_error_quark (void);
GType valide_document_manager_get_type (void);
GType valide_abstract_text_document_get_type (void);
GType valide_ui_manager_get_type (void);
GType valide_document_state_get_type (void);
GType valide_idocument_get_type (void);
#define VALIDE_DOCUMENT_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_DOCUMENT_MANAGER, ValideDocumentManagerPrivate))
enum  {
	VALIDE_DOCUMENT_MANAGER_DUMMY_PROPERTY,
	VALIDE_DOCUMENT_MANAGER_UI_MANAGER,
	VALIDE_DOCUMENT_MANAGER_CURRENT,
	VALIDE_DOCUMENT_MANAGER_DOCUMENTS
};
char* valide_idocument_get_tooltips (ValideiDocument* self);
GdkPixbuf* valide_idocument_get_icon (ValideiDocument* self);
const char* valide_idocument_get_title (ValideiDocument* self);
GtkButton* valide_utils_create_small_button (GtkImage* image);
gboolean valide_idocument_close (ValideiDocument* self);
static void _lambda9_ (GtkButton* s, ValideDocumentManager* self);
static void __lambda9__gtk_button_clicked (GtkButton* _sender, gpointer self);
static void valide_document_manager_title_change (ValideDocumentManager* self, ValideiDocument* sender);
ValideiDocument* valide_document_manager_get_current (ValideDocumentManager* self);
void valide_idocument_setup_ui (ValideiDocument* self, ValideUIManager* ui_manager);
ValideUIManager* valide_document_manager_get_ui_manager (ValideDocumentManager* self);
static void valide_document_manager_switch_page_cb (GtkNotebook* sender, void* page, gint page_num, ValideDocumentManager* _self_);
static void valide_document_manager_launch_external (ValideDocumentManager* self, const char* filename, GError** error);
GtkAction* valide_ui_manager_get_action (ValideUIManager* self, const char* name);
void valide_document_manager_setup_ui (ValideDocumentManager* self);
static void valide_document_manager_real_setup_ui (ValideDocumentManager* self);
char* valide_utils_get_extension (const char* filename);
char* valide_utils_get_mime_type (const char* filename, GError** error);
GType valide_txt_document_get_type (void);
ValideAbstractTextDocument* valide_document_manager_create_document (ValideDocumentManager* self, const char* filename, GError** error);
static ValideAbstractTextDocument* valide_document_manager_real_create_document (ValideDocumentManager* self, const char* filename, GError** error);
static void _g_slist_free_g_free (GSList* self);
ValideiDocument* valide_document_manager_create (ValideDocumentManager* self, const char* filename, GError** error);
static void valide_document_manager_drag_data_received_cb (ValideDocumentManager* self, GtkWidget* sender, GdkDragContext* drag_context, gint x, gint y, GtkSelectionData* data, guint info, guint time);
static gint valide_document_manager_find_tab_num_at_pos (ValideDocumentManager* self, gint abs_x, gint abs_y);
static gboolean valide_document_manager_on_button_press (ValideDocumentManager* self, GtkWidget* sender, const GdkEventButton* event);
static void valide_document_manager_set_ui_manager (ValideDocumentManager* self, ValideUIManager* value);
GType valide_gs_document_get_type (void);
GType valide_vala_document_get_type (void);
GType valide_vapi_document_get_type (void);
GType valide_xml_document_get_type (void);
GType valide_glade_document_get_type (void);
GType valide_ui_document_get_type (void);
ValideDocumentManager* valide_document_manager_new (ValideUIManager* ui_manager);
ValideDocumentManager* valide_document_manager_construct (GType object_type, ValideUIManager* ui_manager);
gboolean valide_document_manager_is_open (ValideDocumentManager* self, const char* filename, gint* pos);
void valide_idocument_open (ValideiDocument* self, const char* path, GError** error);
static void _valide_document_manager_title_change_valide_idocument_title_change (ValideiDocument* _sender, gpointer self);
gint valide_document_manager_append_page (ValideDocumentManager* self, ValideiDocument* child);
void valide_abstract_text_document_show (ValideAbstractTextDocument* self);
void valide_document_manager_set_current (ValideDocumentManager* self, ValideiDocument* value);
GType valide_source_buffer_get_type (void);
ValideSourceBuffer* valide_abstract_text_document_get_buffer (ValideAbstractTextDocument* self);
static void _lambda25_ (ValideDocumentManager* self);
static void __lambda25__g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self);
static void _lambda26_ (ValideDocumentManager* self);
static void __lambda26__g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self);
static void _lambda27_ (ValideDocumentManager* self);
static void __lambda27__gtk_text_buffer_changed (GtkTextBuffer* _sender, gpointer self);
static void _lambda28_ (ValideAbstractTextDocument* s, gint r, gint c, ValideDocumentManager* self);
static void __lambda28__valide_abstract_text_document_cursor_moved (ValideAbstractTextDocument* _sender, gint row, gint col, gpointer self);
void valide_document_manager_remove_page (ValideDocumentManager* self, ValideiDocument* child);
static void _lambda29_ (ValideiDocument* s, ValideDocumentManager* self);
static void __lambda29__valide_idocument_closed (ValideiDocument* _sender, gpointer self);
static void _lambda30_ (ValideiDocument* s, ValideDocumentManager* self);
static void __lambda30__valide_idocument_saved (ValideiDocument* _sender, gpointer self);
static void _lambda31_ (ValideiDocument* sender, ValideDocumentState state, ValideDocumentManager* self);
static void __lambda31__valide_idocument_state_change (ValideiDocument* _sender, ValideDocumentState state, gpointer self);
void valide_utils_process_gtk_events (void);
void valide_idocument_set_is_save (ValideiDocument* self, gboolean value);
GQuark valide_project_error_quark (void);
void valide_document_manager_open (ValideDocumentManager* self, GError** error);
GList* valide_document_manager_get_documents (ValideDocumentManager* self);
void valide_idocument_save (ValideiDocument* self, const char* current_folder);
gboolean valide_idocument_get_is_save (ValideiDocument* self);
gboolean valide_document_manager_save_all (ValideDocumentManager* self);
GType valide_abstract_close_dialog_get_type (void);
GType valide_document_close_dialog_get_type (void);
ValideDocumentCloseDialog* valide_document_close_dialog_new (GList* documents);
ValideDocumentCloseDialog* valide_document_close_dialog_construct (GType object_type, GList* documents);
void valide_document_close_dialog_set_transient_for (ValideDocumentCloseDialog* self, GtkWindow* parent);
void valide_document_close_dialog_set_modal (ValideDocumentCloseDialog* self, gboolean modal);
gint valide_document_close_dialog_run (ValideDocumentCloseDialog* self);
void valide_document_close_dialog_destroy (ValideDocumentCloseDialog* self);
GList* valide_document_close_dialog_get_selected (ValideDocumentCloseDialog* self);
static void _g_list_free_g_object_unref (GList* self);
gboolean valide_document_manager_close_all (ValideDocumentManager* self);
ValideiDocument* valide_document_manager_get_nth (ValideDocumentManager* self, gint page_num);
const char* valide_idocument_get_path (ValideiDocument* self);
static void valide_document_manager_set_documents (ValideDocumentManager* self, GList* value);
static gboolean _valide_document_manager_on_button_press_gtk_widget_button_press_event (GtkWidget* _sender, const GdkEventButton* event, gpointer self);
static void _valide_document_manager_drag_data_received_cb_gtk_widget_drag_data_received (GtkWidget* _sender, GdkDragContext* context, gint x, gint y, GtkSelectionData* selection_data, guint info, guint time_, gpointer self);
static GObject * valide_document_manager_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void valide_document_manager_finalize (GObject* obj);
static void valide_document_manager_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_document_manager_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);


static void g_cclosure_user_marshal_VOID__OBJECT_INT_INT (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data);

GQuark valide_document_error_quark (void) {
	return g_quark_from_static_string ("valide_document_error-quark");
}


static void _lambda9_ (GtkButton* s, ValideDocumentManager* self) {
	ValideiDocument* document;
	ValideiDocument* _tmp2_;
	ValideiDocument* _tmp1_;
	void* _tmp0_;
	g_return_if_fail (s != NULL);
	document = NULL;
	_tmp2_ = NULL;
#line 121 "document-manager.vala"
	_tmp1_ = NULL;
#line 121 "document-manager.vala"
	_tmp0_ = NULL;
#line 121 "document-manager.vala"
	document = (_tmp2_ = (_tmp1_ = (_tmp0_ = g_object_get_data ((GObject*) s, "document"), VALIDE_IS_IDOCUMENT (_tmp0_) ? ((ValideiDocument*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (document == NULL) ? NULL : (document = (g_object_unref (document), NULL)), _tmp2_);
#line 122 "document-manager.vala"
	valide_idocument_close (document);
#line 369 "document-manager.c"
	(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
}


static void __lambda9__gtk_button_clicked (GtkButton* _sender, gpointer self) {
	_lambda9_ (_sender, self);
}


#line 98 "document-manager.vala"
static void valide_document_manager_title_change (ValideDocumentManager* self, ValideiDocument* sender) {
#line 381 "document-manager.c"
	GtkHBox* title;
	GtkImage* image;
	GtkLabel* label;
	GtkButton* button;
	GtkHBox* _tmp0_;
	char* _tmp1_;
	GtkImage* _tmp3_;
	GdkPixbuf* _tmp2_;
	GtkLabel* _tmp4_;
	GtkImage* _tmp5_;
	GtkButton* _tmp6_;
#line 98 "document-manager.vala"
	g_return_if_fail (self != NULL);
#line 98 "document-manager.vala"
	g_return_if_fail (sender != NULL);
#line 397 "document-manager.c"
	title = NULL;
	image = NULL;
	label = NULL;
	button = NULL;
	_tmp0_ = NULL;
#line 105 "document-manager.vala"
	title = (_tmp0_ = g_object_ref_sink ((GtkHBox*) gtk_hbox_new (FALSE, 4)), (title == NULL) ? NULL : (title = (g_object_unref (title), NULL)), _tmp0_);
#line 169 "idocument.vala"
	_tmp1_ = NULL;
#line 106 "document-manager.vala"
	gtk_widget_set_tooltip_markup ((GtkWidget*) title, _tmp1_ = valide_idocument_get_tooltips (sender));
#line 409 "document-manager.c"
	_tmp1_ = (g_free (_tmp1_), NULL);
	_tmp3_ = NULL;
#line 157 "idocument.vala"
	_tmp2_ = NULL;
#line 108 "document-manager.vala"
	image = (_tmp3_ = g_object_ref_sink ((GtkImage*) gtk_image_new_from_pixbuf (_tmp2_ = valide_idocument_get_icon (sender))), (image == NULL) ? NULL : (image = (g_object_unref (image), NULL)), _tmp3_);
#line 416 "document-manager.c"
	(_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL));
#line 109 "document-manager.vala"
	gtk_box_pack_start ((GtkBox*) title, (GtkWidget*) image, FALSE, TRUE, (guint) 0);
#line 420 "document-manager.c"
	_tmp4_ = NULL;
#line 111 "document-manager.vala"
	label = (_tmp4_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (valide_idocument_get_title (sender))), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp4_);
#line 112 "document-manager.vala"
	gtk_box_pack_start ((GtkBox*) title, (GtkWidget*) label, FALSE, FALSE, (guint) 0);
#line 426 "document-manager.c"
	/* Close button */
	_tmp5_ = NULL;
#line 115 "document-manager.vala"
	image = (_tmp5_ = g_object_ref_sink ((GtkImage*) gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU)), (image == NULL) ? NULL : (image = (g_object_unref (image), NULL)), _tmp5_);
#line 431 "document-manager.c"
	_tmp6_ = NULL;
#line 116 "document-manager.vala"
	button = (_tmp6_ = valide_utils_create_small_button (image), (button == NULL) ? NULL : (button = (g_object_unref (button), NULL)), _tmp6_);
#line 117 "document-manager.vala"
	g_object_set_data ((GObject*) button, "document", sender);
#line 118 "document-manager.vala"
	g_signal_connect (button, "clicked", (GCallback) __lambda9__gtk_button_clicked, self);
#line 124 "document-manager.vala"
	gtk_widget_set_tooltip_text ((GtkWidget*) button, _ ("Close document"));
#line 125 "document-manager.vala"
	gtk_box_pack_start ((GtkBox*) title, (GtkWidget*) button, FALSE, FALSE, (guint) 0);
#line 127 "document-manager.vala"
	gtk_widget_show_all ((GtkWidget*) title);
#line 128 "document-manager.vala"
	gtk_notebook_set_tab_label ((GtkNotebook*) self, (GtkWidget*) sender, (GtkWidget*) title);
#line 129 "document-manager.vala"
	g_signal_emit_by_name (self, "tab-state-changed", sender);
#line 449 "document-manager.c"
	(title == NULL) ? NULL : (title = (g_object_unref (title), NULL));
	(image == NULL) ? NULL : (image = (g_object_unref (image), NULL));
	(label == NULL) ? NULL : (label = (g_object_unref (label), NULL));
	(button == NULL) ? NULL : (button = (g_object_unref (button), NULL));
}


#line 132 "document-manager.vala"
static void valide_document_manager_switch_page_cb (GtkNotebook* sender, void* page, gint page_num, ValideDocumentManager* _self_) {
#line 132 "document-manager.vala"
	g_return_if_fail (sender != NULL);
#line 132 "document-manager.vala"
	g_return_if_fail (_self_ != NULL);
#line 135 "document-manager.vala"
	valide_idocument_setup_ui (valide_document_manager_get_current (_self_), _self_->priv->_ui_manager);
#line 136 "document-manager.vala"
	g_signal_emit_by_name (_self_, "tab-changed", valide_document_manager_get_current (_self_));
#line 467 "document-manager.c"
}


#line 139 "document-manager.vala"
static void valide_document_manager_launch_external (ValideDocumentManager* self, const char* filename, GError** error) {
#line 473 "document-manager.c"
	GError * _inner_error_;
#line 139 "document-manager.vala"
	g_return_if_fail (self != NULL);
#line 477 "document-manager.c"
	_inner_error_ = NULL;
#line 141 "document-manager.vala"
	if (filename != NULL) {
#line 481 "document-manager.c"
		char* msg;
		GtkWindow* toplevel;
		GtkMessageDialog* dialog;
		char* _tmp0_;
		GtkWindow* _tmp3_;
		GtkWindow* _tmp2_;
		GtkWidget* _tmp1_;
		GtkMessageDialog* _tmp4_;
		msg = NULL;
		toplevel = NULL;
		dialog = NULL;
		_tmp0_ = NULL;
#line 147 "document-manager.vala"
		msg = (_tmp0_ = g_strdup_printf (_ ("%s doesn't support this type of document, would you like open this with the default program?"), APPNAME), msg = (g_free (msg), NULL), _tmp0_);
#line 496 "document-manager.c"
		_tmp3_ = NULL;
#line 149 "document-manager.vala"
		_tmp2_ = NULL;
#line 149 "document-manager.vala"
		_tmp1_ = NULL;
#line 149 "document-manager.vala"
		toplevel = (_tmp3_ = (_tmp2_ = (_tmp1_ = gtk_widget_get_toplevel ((GtkWidget*) self), GTK_IS_WINDOW (_tmp1_) ? ((GtkWindow*) _tmp1_) : NULL), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (toplevel == NULL) ? NULL : (toplevel = (g_object_unref (toplevel), NULL)), _tmp3_);
#line 504 "document-manager.c"
		_tmp4_ = NULL;
#line 150 "document-manager.vala"
		dialog = (_tmp4_ = g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new (toplevel, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, msg, NULL)), (dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL)), _tmp4_);
#line 154 "document-manager.vala"
		if (gtk_dialog_run ((GtkDialog*) dialog) == GTK_RESPONSE_YES) {
#line 510 "document-manager.c"
			char* _tmp5_;
			_tmp5_ = NULL;
#line 156 "document-manager.vala"
			g_app_info_launch_default_for_uri (_tmp5_ = g_strconcat ("file://", filename, NULL), NULL, &_inner_error_);
#line 515 "document-manager.c"
			if (_inner_error_ != NULL) {
				g_propagate_error (error, _inner_error_);
				msg = (g_free (msg), NULL);
				(toplevel == NULL) ? NULL : (toplevel = (g_object_unref (toplevel), NULL));
				(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
				return;
			}
			_tmp5_ = (g_free (_tmp5_), NULL);
		}
#line 158 "document-manager.vala"
		gtk_object_destroy ((GtkObject*) dialog);
#line 527 "document-manager.c"
		msg = (g_free (msg), NULL);
		(toplevel == NULL) ? NULL : (toplevel = (g_object_unref (toplevel), NULL));
		(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
	}
}


/**
   * Update the user interface
   */
#line 165 "document-manager.vala"
static void valide_document_manager_real_setup_ui (ValideDocumentManager* self) {
#line 540 "document-manager.c"
	gboolean active;
	GtkAction* _tmp0_;
	GtkAction* _tmp1_;
	GtkAction* _tmp2_;
	GtkAction* _tmp3_;
	GtkAction* _tmp4_;
	GtkAction* _tmp5_;
	GtkAction* _tmp6_;
	GtkAction* _tmp7_;
	GtkAction* _tmp8_;
	GtkAction* _tmp9_;
	GtkAction* _tmp10_;
	GtkAction* _tmp11_;
	GtkAction* _tmp12_;
	GtkAction* _tmp13_;
	GtkAction* _tmp14_;
	GtkAction* _tmp15_;
	GtkAction* _tmp16_;
#line 165 "document-manager.vala"
	g_return_if_fail (self != NULL);
#line 561 "document-manager.c"
	active = FALSE;
#line 169 "document-manager.vala"
	if (valide_document_manager_get_current (self) != NULL) {
#line 171 "document-manager.vala"
		active = TRUE;
#line 567 "document-manager.c"
	}
#line 128 "uimanager.vala"
	_tmp0_ = NULL;
#line 173 "document-manager.vala"
	gtk_action_set_sensitive (_tmp0_ = valide_ui_manager_get_action (self->priv->_ui_manager, "file-save"), active);
#line 573 "document-manager.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
#line 128 "uimanager.vala"
	_tmp1_ = NULL;
#line 174 "document-manager.vala"
	gtk_action_set_sensitive (_tmp1_ = valide_ui_manager_get_action (self->priv->_ui_manager, "file-save-as"), active);
#line 579 "document-manager.c"
	(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
#line 128 "uimanager.vala"
	_tmp2_ = NULL;
#line 175 "document-manager.vala"
	gtk_action_set_sensitive (_tmp2_ = valide_ui_manager_get_action (self->priv->_ui_manager, "file-rename"), active);
#line 585 "document-manager.c"
	(_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL));
#line 128 "uimanager.vala"
	_tmp3_ = NULL;
#line 176 "document-manager.vala"
	gtk_action_set_sensitive (_tmp3_ = valide_ui_manager_get_action (self->priv->_ui_manager, "file-close"), active);
#line 591 "document-manager.c"
	(_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL));
#line 128 "uimanager.vala"
	_tmp4_ = NULL;
#line 177 "document-manager.vala"
	gtk_action_set_sensitive (_tmp4_ = valide_ui_manager_get_action (self->priv->_ui_manager, "file-close-all"), active);
#line 597 "document-manager.c"
	(_tmp4_ == NULL) ? NULL : (_tmp4_ = (g_object_unref (_tmp4_), NULL));
#line 128 "uimanager.vala"
	_tmp5_ = NULL;
#line 178 "document-manager.vala"
	gtk_action_set_sensitive (_tmp5_ = valide_ui_manager_get_action (self->priv->_ui_manager, "edit-undo"), active);
#line 603 "document-manager.c"
	(_tmp5_ == NULL) ? NULL : (_tmp5_ = (g_object_unref (_tmp5_), NULL));
#line 128 "uimanager.vala"
	_tmp6_ = NULL;
#line 179 "document-manager.vala"
	gtk_action_set_sensitive (_tmp6_ = valide_ui_manager_get_action (self->priv->_ui_manager, "edit-redo"), active);
#line 609 "document-manager.c"
	(_tmp6_ == NULL) ? NULL : (_tmp6_ = (g_object_unref (_tmp6_), NULL));
#line 128 "uimanager.vala"
	_tmp7_ = NULL;
#line 180 "document-manager.vala"
	gtk_action_set_sensitive (_tmp7_ = valide_ui_manager_get_action (self->priv->_ui_manager, "edit-copy"), active);
#line 615 "document-manager.c"
	(_tmp7_ == NULL) ? NULL : (_tmp7_ = (g_object_unref (_tmp7_), NULL));
#line 128 "uimanager.vala"
	_tmp8_ = NULL;
#line 181 "document-manager.vala"
	gtk_action_set_sensitive (_tmp8_ = valide_ui_manager_get_action (self->priv->_ui_manager, "edit-cut"), active);
#line 621 "document-manager.c"
	(_tmp8_ == NULL) ? NULL : (_tmp8_ = (g_object_unref (_tmp8_), NULL));
#line 128 "uimanager.vala"
	_tmp9_ = NULL;
#line 182 "document-manager.vala"
	gtk_action_set_sensitive (_tmp9_ = valide_ui_manager_get_action (self->priv->_ui_manager, "edit-paste"), active);
#line 627 "document-manager.c"
	(_tmp9_ == NULL) ? NULL : (_tmp9_ = (g_object_unref (_tmp9_), NULL));
#line 128 "uimanager.vala"
	_tmp10_ = NULL;
#line 183 "document-manager.vala"
	gtk_action_set_sensitive (_tmp10_ = valide_ui_manager_get_action (self->priv->_ui_manager, "edit-lower"), active);
#line 633 "document-manager.c"
	(_tmp10_ == NULL) ? NULL : (_tmp10_ = (g_object_unref (_tmp10_), NULL));
#line 128 "uimanager.vala"
	_tmp11_ = NULL;
#line 184 "document-manager.vala"
	gtk_action_set_sensitive (_tmp11_ = valide_ui_manager_get_action (self->priv->_ui_manager, "edit-upper"), active);
#line 639 "document-manager.c"
	(_tmp11_ == NULL) ? NULL : (_tmp11_ = (g_object_unref (_tmp11_), NULL));
#line 128 "uimanager.vala"
	_tmp12_ = NULL;
#line 185 "document-manager.vala"
	gtk_action_set_sensitive (_tmp12_ = valide_ui_manager_get_action (self->priv->_ui_manager, "search-find"), active);
#line 645 "document-manager.c"
	(_tmp12_ == NULL) ? NULL : (_tmp12_ = (g_object_unref (_tmp12_), NULL));
#line 128 "uimanager.vala"
	_tmp13_ = NULL;
#line 186 "document-manager.vala"
	gtk_action_set_sensitive (_tmp13_ = valide_ui_manager_get_action (self->priv->_ui_manager, "search-find-next"), active);
#line 651 "document-manager.c"
	(_tmp13_ == NULL) ? NULL : (_tmp13_ = (g_object_unref (_tmp13_), NULL));
#line 128 "uimanager.vala"
	_tmp14_ = NULL;
#line 187 "document-manager.vala"
	gtk_action_set_sensitive (_tmp14_ = valide_ui_manager_get_action (self->priv->_ui_manager, "search-find-prev"), active);
#line 657 "document-manager.c"
	(_tmp14_ == NULL) ? NULL : (_tmp14_ = (g_object_unref (_tmp14_), NULL));
#line 128 "uimanager.vala"
	_tmp15_ = NULL;
#line 188 "document-manager.vala"
	gtk_action_set_sensitive (_tmp15_ = valide_ui_manager_get_action (self->priv->_ui_manager, "search-replace"), active);
#line 663 "document-manager.c"
	(_tmp15_ == NULL) ? NULL : (_tmp15_ = (g_object_unref (_tmp15_), NULL));
#line 128 "uimanager.vala"
	_tmp16_ = NULL;
#line 189 "document-manager.vala"
	gtk_action_set_sensitive (_tmp16_ = valide_ui_manager_get_action (self->priv->_ui_manager, "search-goto-line"), active);
#line 669 "document-manager.c"
	(_tmp16_ == NULL) ? NULL : (_tmp16_ = (g_object_unref (_tmp16_), NULL));
#line 190 "document-manager.vala"
	if (valide_document_manager_get_current (self) != NULL) {
#line 192 "document-manager.vala"
		valide_idocument_setup_ui (valide_document_manager_get_current (self), self->priv->_ui_manager);
#line 675 "document-manager.c"
	}
}


#line 165 "document-manager.vala"
void valide_document_manager_setup_ui (ValideDocumentManager* self) {
#line 165 "document-manager.vala"
	VALIDE_DOCUMENT_MANAGER_GET_CLASS (self)->setup_ui (self);
#line 684 "document-manager.c"
}


/**
   * Create a new Valide.AbstractTextDocument for filename
   *
   * @param filename A name of file
   *
   * @throws DocumentError
   */
#line 203 "document-manager.vala"
static ValideAbstractTextDocument* valide_document_manager_real_create_document (ValideDocumentManager* self, const char* filename, GError** error) {
#line 697 "document-manager.c"
	ValideAbstractTextDocument* result;
	GError * _inner_error_;
	GType type;
	char* ext;
	char* _tmp5_;
	char* _tmp4_;
	GObject* _tmp8_;
#line 203 "document-manager.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 707 "document-manager.c"
	_inner_error_ = NULL;
	/* @todo Use plugins */
	type = (GType) 0;
	ext = g_strdup ("Unknow");
#line 209 "document-manager.vala"
	if (filename != NULL) {
#line 714 "document-manager.c"
		char* tmp;
		char* _tmp0_;
		tmp = NULL;
		_tmp0_ = NULL;
#line 213 "document-manager.vala"
		tmp = (_tmp0_ = valide_utils_get_extension (filename), tmp = (g_free (tmp), NULL), _tmp0_);
#line 214 "document-manager.vala"
		if (tmp != NULL) {
#line 723 "document-manager.c"
			char* _tmp2_;
			char* _tmp1_;
			_tmp2_ = NULL;
#line 869 "glib-2.0.vapi"
			_tmp1_ = NULL;
#line 216 "document-manager.vala"
			ext = (_tmp2_ = g_strconcat (_tmp1_ = g_utf8_strup (tmp, (gssize) 1), g_utf8_offset_to_pointer (tmp, (glong) 1), NULL), ext = (g_free (ext), NULL), _tmp2_);
#line 731 "document-manager.c"
			_tmp1_ = (g_free (_tmp1_), NULL);
		}
		tmp = (g_free (tmp), NULL);
	} else {
		char* _tmp3_;
		_tmp3_ = NULL;
#line 221 "document-manager.vala"
		ext = (_tmp3_ = g_strdup ("Vala"), ext = (g_free (ext), NULL), _tmp3_);
#line 740 "document-manager.c"
	}
	/* Search standard behavior */
	_tmp5_ = NULL;
	_tmp4_ = NULL;
#line 225 "document-manager.vala"
	type = g_type_from_name (_tmp5_ = g_strconcat (_tmp4_ = g_strconcat ("Valide", ext, NULL), "Document", NULL));
#line 747 "document-manager.c"
	_tmp5_ = (g_free (_tmp5_), NULL);
	_tmp4_ = (g_free (_tmp4_), NULL);
	/* Or, open as text */
#line 228 "document-manager.vala"
	if (type == 0) {
#line 753 "document-manager.c"
		char* mime;
		mime = NULL;
		{
			char* _tmp6_;
			char* _tmp7_;
			_tmp6_ = valide_utils_get_mime_type (filename, &_inner_error_);
			if (_inner_error_ != NULL) {
				goto __catch9_g_error;
				goto __finally9;
			}
			_tmp7_ = NULL;
#line 234 "document-manager.vala"
			mime = (_tmp7_ = _tmp6_, mime = (g_free (mime), NULL), _tmp7_);
#line 767 "document-manager.c"
		}
		goto __finally9;
		__catch9_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
				_inner_error_ = g_error_new_literal (VALIDE_DOCUMENT_ERROR, VALIDE_DOCUMENT_ERROR_MIME, _ ("Unable to retrieve mime type"));
				if (_inner_error_ != NULL) {
					(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
					mime = (g_free (mime), NULL);
					ext = (g_free (ext), NULL);
					goto __finally9;
				}
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally9:
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == VALIDE_DOCUMENT_ERROR) {
				g_propagate_error (error, _inner_error_);
				mime = (g_free (mime), NULL);
				ext = (g_free (ext), NULL);
				return NULL;
			} else {
				mime = (g_free (mime), NULL);
				ext = (g_free (ext), NULL);
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return NULL;
			}
		}
#line 241 "document-manager.vala"
		if (mime == NULL) {
#line 803 "document-manager.c"
			_inner_error_ = g_error_new_literal (VALIDE_DOCUMENT_ERROR, VALIDE_DOCUMENT_ERROR_MIME, _ ("Unable to retrieve mime type"));
			if (_inner_error_ != NULL) {
				if (_inner_error_->domain == VALIDE_DOCUMENT_ERROR) {
					g_propagate_error (error, _inner_error_);
					mime = (g_free (mime), NULL);
					ext = (g_free (ext), NULL);
					return NULL;
				} else {
					mime = (g_free (mime), NULL);
					ext = (g_free (ext), NULL);
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return NULL;
				}
			}
		}
#line 246 "document-manager.vala"
		if (g_str_has_prefix (mime, "text/")) {
#line 248 "document-manager.vala"
			type = VALIDE_TYPE_TXT_DOCUMENT;
#line 824 "document-manager.c"
		}
		/* Default system behavior */
#line 252 "document-manager.vala"
		if (type == 0) {
#line 829 "document-manager.c"
			_inner_error_ = g_error_new_literal (VALIDE_DOCUMENT_ERROR, VALIDE_DOCUMENT_ERROR_UNKNOW_DOCUMENT, _ ("Couldn't open this document"));
			if (_inner_error_ != NULL) {
				if (_inner_error_->domain == VALIDE_DOCUMENT_ERROR) {
					g_propagate_error (error, _inner_error_);
					mime = (g_free (mime), NULL);
					ext = (g_free (ext), NULL);
					return NULL;
				} else {
					mime = (g_free (mime), NULL);
					ext = (g_free (ext), NULL);
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return NULL;
				}
			}
		}
		mime = (g_free (mime), NULL);
	}
#line 257 "document-manager.vala"
	_tmp8_ = NULL;
#line 850 "document-manager.c"
	result = (_tmp8_ = g_object_new (type, NULL), VALIDE_IS_ABSTRACT_TEXT_DOCUMENT (_tmp8_) ? ((ValideAbstractTextDocument*) _tmp8_) : NULL);
	ext = (g_free (ext), NULL);
	return result;
	ext = (g_free (ext), NULL);
}


#line 203 "document-manager.vala"
ValideAbstractTextDocument* valide_document_manager_create_document (ValideDocumentManager* self, const char* filename, GError** error) {
#line 203 "document-manager.vala"
	return VALIDE_DOCUMENT_MANAGER_GET_CLASS (self)->create_document (self, filename, error);
#line 862 "document-manager.c"
}


static void _g_slist_free_g_free (GSList* self) {
	g_slist_foreach (self, (GFunc) g_free, NULL);
	g_slist_free (self);
}


#line 260 "document-manager.vala"
static void valide_document_manager_drag_data_received_cb (ValideDocumentManager* self, GtkWidget* sender, GdkDragContext* drag_context, gint x, gint y, GtkSelectionData* data, guint info, guint time) {
#line 874 "document-manager.c"
	GError * _inner_error_;
	GSList* files;
	GSList* _tmp0_;
#line 260 "document-manager.vala"
	g_return_if_fail (self != NULL);
#line 260 "document-manager.vala"
	g_return_if_fail (sender != NULL);
#line 260 "document-manager.vala"
	g_return_if_fail (drag_context != NULL);
#line 260 "document-manager.vala"
	g_return_if_fail (data != NULL);
#line 886 "document-manager.c"
	_inner_error_ = NULL;
	files = NULL;
	_tmp0_ = NULL;
#line 268 "document-manager.vala"
	files = (_tmp0_ = NULL, (files == NULL) ? NULL : (files = (_g_slist_free_g_free (files), NULL)), _tmp0_);
#line 892 "document-manager.c"
	{
		char** _tmp1_;
		char** uri_collection;
		int uri_collection_length1;
		int uri_it;
		_tmp1_ = NULL;
#line 269 "document-manager.vala"
		uri_collection = _tmp1_ = g_strsplit ((const char*) data->data, "\r\n", 0);
#line 901 "document-manager.c"
		uri_collection_length1 = _vala_array_length (_tmp1_);
		for (uri_it = 0; uri_it < _vala_array_length (_tmp1_); uri_it = uri_it + 1) {
			const char* _tmp5_;
			char* uri;
#line 831 "glib-2.0.vapi"
			_tmp5_ = NULL;
#line 908 "document-manager.c"
			uri = (_tmp5_ = uri_collection[uri_it], (_tmp5_ == NULL) ? NULL : g_strdup (_tmp5_));
			{
				{
#line 273 "document-manager.vala"
					if (_vala_strcmp0 (uri, "") != 0) {
#line 914 "document-manager.c"
						GFile* file;
						char* filename;
						GFile* _tmp2_;
						char* _tmp3_;
						ValideiDocument* _tmp4_;
						file = NULL;
						filename = NULL;
						_tmp2_ = NULL;
#line 278 "document-manager.vala"
						file = (_tmp2_ = g_file_new_for_uri (uri), (file == NULL) ? NULL : (file = (g_object_unref (file), NULL)), _tmp2_);
#line 925 "document-manager.c"
						_tmp3_ = NULL;
#line 279 "document-manager.vala"
						filename = (_tmp3_ = g_file_get_path (file), filename = (g_free (filename), NULL), _tmp3_);
#line 414 "document-manager.vala"
						_tmp4_ = NULL;
#line 280 "document-manager.vala"
						_tmp4_ = valide_document_manager_create (self, filename, &_inner_error_);
#line 933 "document-manager.c"
						if (_inner_error_ != NULL) {
							(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
							filename = (g_free (filename), NULL);
							goto __catch10_g_error;
							goto __finally10;
						}
						(_tmp4_ == NULL) ? NULL : (_tmp4_ = (g_object_unref (_tmp4_), NULL));
						(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
						filename = (g_free (filename), NULL);
					}
				}
				goto __finally10;
				__catch10_g_error:
				{
					GError * e;
					e = _inner_error_;
					_inner_error_ = NULL;
					{
#line 285 "document-manager.vala"
						g_debug (e->message);
#line 954 "document-manager.c"
						(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
					}
				}
				__finally10:
				if (_inner_error_ != NULL) {
					uri = (g_free (uri), NULL);
					uri_collection = (_vala_array_free (uri_collection, uri_collection_length1, (GDestroyNotify) g_free), NULL);
					(files == NULL) ? NULL : (files = (_g_slist_free_g_free (files), NULL));
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return;
				}
				uri = (g_free (uri), NULL);
			}
		}
#line 269 "document-manager.vala"
		uri_collection = (_vala_array_free (uri_collection, uri_collection_length1, (GDestroyNotify) g_free), NULL);
#line 972 "document-manager.c"
	}
	(files == NULL) ? NULL : (files = (_g_slist_free_g_free (files), NULL));
}


#line 290 "document-manager.vala"
static gint valide_document_manager_find_tab_num_at_pos (ValideDocumentManager* self, gint abs_x, gint abs_y) {
#line 980 "document-manager.c"
	gint result;
	GtkWidget* page;
	gint page_num;
	GtkPositionType tab_pos;
#line 290 "document-manager.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 987 "document-manager.c"
	page = NULL;
	page_num = 0;
	tab_pos = 0;
#line 296 "document-manager.vala"
	tab_pos = gtk_notebook_get_tab_pos ((GtkNotebook*) self);
#line 298 "document-manager.vala"
	if (((GtkNotebook*) self)->first_tab == NULL) {
#line 995 "document-manager.c"
		result = -1;
		(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
		return result;
	}
#line 303 "document-manager.vala"
	while (TRUE) {
#line 1002 "document-manager.c"
		GtkWidget* _tmp1_;
		GtkWidget* _tmp0_;
		GtkWidget* tab;
		gint max_x;
		gint max_y;
		gint x_root;
		gint y_root;
		GtkWidget* _tmp3_;
		GtkWidget* _tmp2_;
		gboolean _tmp4_;
		gboolean _tmp5_;
		_tmp1_ = NULL;
#line 303 "document-manager.vala"
		_tmp0_ = NULL;
#line 303 "document-manager.vala"
		if (!((page = (_tmp1_ = (_tmp0_ = gtk_notebook_get_nth_page ((GtkNotebook*) self, page_num), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (page == NULL) ? NULL : (page = (g_object_unref (page), NULL)), _tmp1_)) != NULL)) {
#line 303 "document-manager.vala"
			break;
#line 1021 "document-manager.c"
		}
		tab = NULL;
		max_x = 0;
		max_y = 0;
		x_root = 0;
		y_root = 0;
		_tmp3_ = NULL;
#line 309 "document-manager.vala"
		_tmp2_ = NULL;
#line 309 "document-manager.vala"
		tab = (_tmp3_ = (_tmp2_ = gtk_notebook_get_tab_label ((GtkNotebook*) self, page), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (tab == NULL) ? NULL : (tab = (g_object_unref (tab), NULL)), _tmp3_);
#line 310 "document-manager.vala"
		g_return_val_if_fail (tab != NULL, -1);
#line 312 "document-manager.vala"
		if (!GTK_WIDGET_MAPPED (tab)) {
#line 314 "document-manager.vala"
			page_num++;
#line 1039 "document-manager.c"
			(tab == NULL) ? NULL : (tab = (g_object_unref (tab), NULL));
#line 315 "document-manager.vala"
			continue;
#line 1043 "document-manager.c"
		}
#line 318 "document-manager.vala"
		gdk_window_get_origin (tab->window, &x_root, &y_root);
#line 320 "document-manager.vala"
		max_x = (x_root + tab->allocation.x) + tab->allocation.width;
#line 321 "document-manager.vala"
		max_y = (y_root + tab->allocation.y) + tab->allocation.height;
#line 1051 "document-manager.c"
		_tmp4_ = FALSE;
		_tmp5_ = FALSE;
#line 323 "document-manager.vala"
		if (tab_pos == GTK_POS_TOP) {
#line 323 "document-manager.vala"
			_tmp5_ = TRUE;
#line 1058 "document-manager.c"
		} else {
#line 323 "document-manager.vala"
			_tmp5_ = tab_pos == GTK_POS_BOTTOM;
#line 1062 "document-manager.c"
		}
#line 323 "document-manager.vala"
		if (_tmp5_) {
#line 324 "document-manager.vala"
			_tmp4_ = abs_x <= max_x;
#line 1068 "document-manager.c"
		} else {
#line 323 "document-manager.vala"
			_tmp4_ = FALSE;
#line 1072 "document-manager.c"
		}
#line 323 "document-manager.vala"
		if (_tmp4_) {
#line 1076 "document-manager.c"
			result = page_num;
			(tab == NULL) ? NULL : (tab = (g_object_unref (tab), NULL));
			(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
			return result;
		} else {
			gboolean _tmp6_;
			gboolean _tmp7_;
			_tmp6_ = FALSE;
			_tmp7_ = FALSE;
#line 328 "document-manager.vala"
			if (tab_pos == GTK_POS_LEFT) {
#line 328 "document-manager.vala"
				_tmp7_ = TRUE;
#line 1090 "document-manager.c"
			} else {
#line 328 "document-manager.vala"
				_tmp7_ = tab_pos == GTK_POS_RIGHT;
#line 1094 "document-manager.c"
			}
#line 328 "document-manager.vala"
			if (_tmp7_) {
#line 329 "document-manager.vala"
				_tmp6_ = abs_y <= max_y;
#line 1100 "document-manager.c"
			} else {
#line 328 "document-manager.vala"
				_tmp6_ = FALSE;
#line 1104 "document-manager.c"
			}
#line 328 "document-manager.vala"
			if (_tmp6_) {
#line 1108 "document-manager.c"
				result = page_num;
				(tab == NULL) ? NULL : (tab = (g_object_unref (tab), NULL));
				(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
				return result;
			}
		}
#line 334 "document-manager.vala"
		page_num++;
#line 1117 "document-manager.c"
		(tab == NULL) ? NULL : (tab = (g_object_unref (tab), NULL));
	}
	result = -1;
	(page == NULL) ? NULL : (page = (g_object_unref (page), NULL));
	return result;
}


#line 339 "document-manager.vala"
static gboolean valide_document_manager_on_button_press (ValideDocumentManager* self, GtkWidget* sender, const GdkEventButton* event) {
#line 1128 "document-manager.c"
	gboolean result;
	gboolean catched;
#line 339 "document-manager.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 339 "document-manager.vala"
	g_return_val_if_fail (sender != NULL, FALSE);
#line 1135 "document-manager.c"
	catched = FALSE;
#line 343 "document-manager.vala"
	if ((*event).button == 3) {
#line 1139 "document-manager.c"
		gint tab_clicked;
		tab_clicked = 0;
#line 347 "document-manager.vala"
		tab_clicked = valide_document_manager_find_tab_num_at_pos (self, (gint) (*event).x_root, (gint) (*event).y_root);
#line 348 "document-manager.vala"
		if (tab_clicked == (-1)) {
#line 1146 "document-manager.c"
			/* CHECK: do we really need it?
			 consume event, so that we don't pop up the context menu when
			         * the mouse if not over a tab label
			         */
#line 355 "document-manager.vala"
			catched = TRUE;
#line 1153 "document-manager.c"
		} else {
			GtkMenu* menu;
			GtkMenu* _tmp2_;
			GtkMenu* _tmp1_;
			GtkWidget* _tmp0_;
			menu = NULL;
			/* Switch to the page the mouse is over, but don't consume the event */
#line 362 "document-manager.vala"
			gtk_notebook_set_current_page ((GtkNotebook*) self, tab_clicked);
#line 1163 "document-manager.c"
			_tmp2_ = NULL;
#line 364 "document-manager.vala"
			_tmp1_ = NULL;
#line 364 "document-manager.vala"
			_tmp0_ = NULL;
#line 364 "document-manager.vala"
			menu = (_tmp2_ = (_tmp1_ = (_tmp0_ = gtk_ui_manager_get_widget ((GtkUIManager*) self->priv->_ui_manager, "/document-popup"), GTK_IS_MENU (_tmp0_) ? ((GtkMenu*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (menu == NULL) ? NULL : (menu = (g_object_unref (menu), NULL)), _tmp2_);
#line 365 "document-manager.vala"
			gtk_menu_popup (menu, NULL, NULL, NULL, NULL, (*event).button, (*event).time);
#line 1173 "document-manager.c"
			(menu == NULL) ? NULL : (menu = (g_object_unref (menu), NULL));
		}
	}
	result = catched;
	return result;
}


/**
   * Create a new Valide.DocumentManager
   */
#line 374 "document-manager.vala"
ValideDocumentManager* valide_document_manager_construct (GType object_type, ValideUIManager* ui_manager) {
#line 1187 "document-manager.c"
	GParameter * __params;
	GParameter * __params_it;
	ValideDocumentManager * self;
#line 374 "document-manager.vala"
	g_return_val_if_fail (ui_manager != NULL, NULL);
#line 1193 "document-manager.c"
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
#line 376 "document-manager.vala"
	__params_it->name = "ui-manager";
#line 376 "document-manager.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_UI_MANAGER);
#line 376 "document-manager.vala"
	g_value_set_object (&__params_it->value, ui_manager);
#line 376 "document-manager.vala"
	__params_it++;
#line 1204 "document-manager.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
#line 378 "document-manager.vala"
	VALIDE_TYPE_TXT_DOCUMENT;
#line 379 "document-manager.vala"
	VALIDE_TYPE_GS_DOCUMENT;
#line 380 "document-manager.vala"
	VALIDE_TYPE_VALA_DOCUMENT;
#line 381 "document-manager.vala"
	VALIDE_TYPE_VAPI_DOCUMENT;
#line 382 "document-manager.vala"
	VALIDE_TYPE_XML_DOCUMENT;
#line 383 "document-manager.vala"
	VALIDE_TYPE_GLADE_DOCUMENT;
#line 384 "document-manager.vala"
	VALIDE_TYPE_UI_DOCUMENT;
#line 1220 "document-manager.c"
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 374 "document-manager.vala"
ValideDocumentManager* valide_document_manager_new (ValideUIManager* ui_manager) {
#line 374 "document-manager.vala"
	return valide_document_manager_construct (VALIDE_TYPE_DOCUMENT_MANAGER, ui_manager);
#line 1234 "document-manager.c"
}


#line 98 "document-manager.vala"
static void _valide_document_manager_title_change_valide_idocument_title_change (ValideiDocument* _sender, gpointer self) {
#line 1240 "document-manager.c"
	valide_document_manager_title_change (self, _sender);
}


static void _lambda25_ (ValideDocumentManager* self) {
#line 448 "document-manager.vala"
	valide_document_manager_setup_ui (self);
#line 1248 "document-manager.c"
}


static void __lambda25__g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self) {
	_lambda25_ (self);
}


static void _lambda26_ (ValideDocumentManager* self) {
#line 451 "document-manager.vala"
	valide_document_manager_setup_ui (self);
#line 1260 "document-manager.c"
}


static void __lambda26__g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self) {
	_lambda26_ (self);
}


static void _lambda27_ (ValideDocumentManager* self) {
#line 454 "document-manager.vala"
	valide_document_manager_setup_ui (self);
#line 1272 "document-manager.c"
}


static void __lambda27__gtk_text_buffer_changed (GtkTextBuffer* _sender, gpointer self) {
	_lambda27_ (self);
}


static void _lambda28_ (ValideAbstractTextDocument* s, gint r, gint c, ValideDocumentManager* self) {
	g_return_if_fail (s != NULL);
#line 457 "document-manager.vala"
	g_signal_emit_by_name (self, "tab-cursor-moved", (ValideiDocument*) s, r, c);
#line 1285 "document-manager.c"
}


static void __lambda28__valide_abstract_text_document_cursor_moved (ValideAbstractTextDocument* _sender, gint row, gint col, gpointer self) {
	_lambda28_ (_sender, row, col, self);
}


static void _lambda29_ (ValideiDocument* s, ValideDocumentManager* self) {
	g_return_if_fail (s != NULL);
#line 461 "document-manager.vala"
	valide_document_manager_remove_page (self, s);
#line 1298 "document-manager.c"
}


static void __lambda29__valide_idocument_closed (ValideiDocument* _sender, gpointer self) {
	_lambda29_ (_sender, self);
}


static void _lambda30_ (ValideiDocument* s, ValideDocumentManager* self) {
	g_return_if_fail (s != NULL);
#line 464 "document-manager.vala"
	g_signal_emit_by_name (self, "tab-saved", s);
#line 1311 "document-manager.c"
}


static void __lambda30__valide_idocument_saved (ValideiDocument* _sender, gpointer self) {
	_lambda30_ (_sender, self);
}


static void _lambda31_ (ValideiDocument* sender, ValideDocumentState state, ValideDocumentManager* self) {
	g_return_if_fail (sender != NULL);
#line 467 "document-manager.vala"
	g_signal_emit_by_name (self, "tab-state-changed", sender);
#line 1324 "document-manager.c"
}


static void __lambda31__valide_idocument_state_change (ValideiDocument* _sender, ValideDocumentState state, gpointer self) {
	_lambda31_ (_sender, state, self);
}


/**
   * Create a new document.
   *
   * @param filename a name of a file or null for create a new document.
   *
   * @return The new document.
   */
#line 414 "document-manager.vala"
ValideiDocument* valide_document_manager_create (ValideDocumentManager* self, const char* filename, GError** error) {
#line 1342 "document-manager.c"
	ValideiDocument* result;
	GError * _inner_error_;
	gint pos;
	ValideiDocument* _tmp5_;
#line 414 "document-manager.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 1349 "document-manager.c"
	_inner_error_ = NULL;
	pos = 0;
#line 418 "document-manager.vala"
	if (!valide_document_manager_is_open (self, filename, &pos)) {
#line 1354 "document-manager.c"
		ValideAbstractTextDocument* document;
		document = NULL;
		{
			ValideAbstractTextDocument* _tmp0_;
			ValideAbstractTextDocument* _tmp1_;
			_tmp0_ = valide_document_manager_create_document (self, filename, &_inner_error_);
			if (_inner_error_ != NULL) {
				if (_inner_error_->domain == VALIDE_DOCUMENT_ERROR) {
					goto __catch11_valide_document_error;
				}
				goto __finally11;
			}
			_tmp1_ = NULL;
#line 424 "document-manager.vala"
			document = (_tmp1_ = _tmp0_, (document == NULL) ? NULL : (document = (g_object_unref (document), NULL)), _tmp1_);
#line 426 "document-manager.vala"
			if (filename != NULL) {
#line 1372 "document-manager.c"
				{
#line 430 "document-manager.vala"
					valide_idocument_open ((ValideiDocument*) document, filename, &_inner_error_);
#line 1376 "document-manager.c"
					if (_inner_error_ != NULL) {
						goto __catch12_g_error;
						goto __finally12;
					}
				}
				goto __finally12;
				__catch12_g_error:
				{
					GError * e;
					e = _inner_error_;
					_inner_error_ = NULL;
					{
#line 434 "document-manager.vala"
						g_warning (e->message);
#line 1391 "document-manager.c"
						(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
					}
				}
				__finally12:
				if (_inner_error_ != NULL) {
					if (_inner_error_->domain == VALIDE_DOCUMENT_ERROR) {
						goto __catch11_valide_document_error;
					}
					goto __finally11;
				}
			}
#line 437 "document-manager.vala"
			g_signal_connect_object ((ValideiDocument*) document, "title-change", (GCallback) _valide_document_manager_title_change_valide_idocument_title_change, self, 0);
#line 438 "document-manager.vala"
			valide_document_manager_append_page (self, (ValideiDocument*) document);
#line 439 "document-manager.vala"
			valide_abstract_text_document_show (document);
#line 440 "document-manager.vala"
			valide_document_manager_set_current (self, (ValideiDocument*) document);
#line 441 "document-manager.vala"
			gtk_widget_grab_focus ((GtkWidget*) valide_document_manager_get_current (self));
#line 442 "document-manager.vala"
			if (VALIDE_IS_ABSTRACT_TEXT_DOCUMENT (valide_document_manager_get_current (self))) {
#line 1415 "document-manager.c"
				ValideAbstractTextDocument* atd;
				ValideAbstractTextDocument* _tmp4_;
				ValideAbstractTextDocument* _tmp3_;
				ValideiDocument* _tmp2_;
				atd = NULL;
				_tmp4_ = NULL;
#line 446 "document-manager.vala"
				_tmp3_ = NULL;
#line 446 "document-manager.vala"
				_tmp2_ = NULL;
#line 446 "document-manager.vala"
				atd = (_tmp4_ = (_tmp3_ = (_tmp2_ = valide_document_manager_get_current (self), VALIDE_IS_ABSTRACT_TEXT_DOCUMENT (_tmp2_) ? ((ValideAbstractTextDocument*) _tmp2_) : NULL), (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), (atd == NULL) ? NULL : (atd = (g_object_unref (atd), NULL)), _tmp4_);
#line 447 "document-manager.vala"
				g_signal_connect ((GObject*) valide_abstract_text_document_get_buffer (atd), "notify::can-undo", (GCallback) __lambda25__g_object_notify, self);
#line 450 "document-manager.vala"
				g_signal_connect ((GObject*) valide_abstract_text_document_get_buffer (atd), "notify::can-redo", (GCallback) __lambda26__g_object_notify, self);
#line 453 "document-manager.vala"
				g_signal_connect ((GtkTextBuffer*) valide_abstract_text_document_get_buffer (atd), "changed", (GCallback) __lambda27__gtk_text_buffer_changed, self);
#line 456 "document-manager.vala"
				g_signal_connect (atd, "cursor-moved", (GCallback) __lambda28__valide_abstract_text_document_cursor_moved, self);
#line 1436 "document-manager.c"
				(atd == NULL) ? NULL : (atd = (g_object_unref (atd), NULL));
			}
#line 460 "document-manager.vala"
			g_signal_connect (valide_document_manager_get_current (self), "closed", (GCallback) __lambda29__valide_idocument_closed, self);
#line 463 "document-manager.vala"
			g_signal_connect (valide_document_manager_get_current (self), "saved", (GCallback) __lambda30__valide_idocument_saved, self);
#line 466 "document-manager.vala"
			g_signal_connect (valide_document_manager_get_current (self), "state-change", (GCallback) __lambda31__valide_idocument_state_change, self);
#line 1445 "document-manager.c"
		}
		goto __finally11;
		__catch11_valide_document_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
#line 472 "document-manager.vala"
				if (g_error_matches (e, VALIDE_DOCUMENT_ERROR, VALIDE_DOCUMENT_ERROR_UNKNOW_DOCUMENT)) {
#line 474 "document-manager.vala"
					valide_document_manager_launch_external (self, filename, &_inner_error_);
#line 1458 "document-manager.c"
					if (_inner_error_ != NULL) {
						(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
						(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
						goto __finally11;
					}
				} else {
#line 478 "document-manager.vala"
					g_warning (e->message);
#line 1467 "document-manager.c"
				}
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally11:
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
			return NULL;
		}
		(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
	} else {
#line 484 "document-manager.vala"
		gtk_notebook_set_current_page ((GtkNotebook*) self, pos);
#line 1482 "document-manager.c"
	}
#line 487 "document-manager.vala"
	valide_utils_process_gtk_events ();
#line 488 "document-manager.vala"
	_tmp5_ = NULL;
#line 1488 "document-manager.c"
	result = (_tmp5_ = valide_document_manager_get_current (self), (_tmp5_ == NULL) ? NULL : g_object_ref (_tmp5_));
	return result;
}


/**
   * Open document with #Gtk.FileChosserDialog.
   */
#line 494 "document-manager.vala"
void valide_document_manager_open (ValideDocumentManager* self, GError** error) {
#line 1499 "document-manager.c"
	GError * _inner_error_;
	GSList* files;
	GtkFileChooserDialog* dialog;
	GSList* _tmp0_;
	GtkFileChooserDialog* _tmp1_;
#line 494 "document-manager.vala"
	g_return_if_fail (self != NULL);
#line 1507 "document-manager.c"
	_inner_error_ = NULL;
	files = NULL;
	dialog = NULL;
	_tmp0_ = NULL;
#line 499 "document-manager.vala"
	files = (_tmp0_ = NULL, (files == NULL) ? NULL : (files = (_g_slist_free_g_free (files), NULL)), _tmp0_);
#line 1514 "document-manager.c"
	_tmp1_ = NULL;
#line 500 "document-manager.vala"
	dialog = (_tmp1_ = g_object_ref_sink ((GtkFileChooserDialog*) gtk_file_chooser_dialog_new (_ ("Open file"), NULL, GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL, NULL)), (dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL)), _tmp1_);
#line 506 "document-manager.vala"
	gtk_file_chooser_set_select_multiple ((GtkFileChooser*) dialog, TRUE);
#line 507 "document-manager.vala"
	if (gtk_dialog_run ((GtkDialog*) dialog) == GTK_RESPONSE_ACCEPT) {
#line 1522 "document-manager.c"
		GSList* _tmp2_;
		_tmp2_ = NULL;
#line 509 "document-manager.vala"
		files = (_tmp2_ = gtk_file_chooser_get_filenames ((GtkFileChooser*) dialog), (files == NULL) ? NULL : (files = (_g_slist_free_g_free (files), NULL)), _tmp2_);
#line 1527 "document-manager.c"
	}
#line 511 "document-manager.vala"
	gtk_object_destroy ((GtkObject*) dialog);
#line 1531 "document-manager.c"
	{
		GSList* file_collection;
		GSList* file_it;
#line 513 "document-manager.vala"
		file_collection = files;
#line 1537 "document-manager.c"
		for (file_it = file_collection; file_it != NULL; file_it = file_it->next) {
			const char* _tmp5_;
			char* file;
#line 496 "document-manager.vala"
			_tmp5_ = NULL;
#line 513 "document-manager.vala"
			file = (_tmp5_ = (const char*) file_it->data, (_tmp5_ == NULL) ? NULL : g_strdup (_tmp5_));
#line 1545 "document-manager.c"
			{
				ValideiDocument* document;
				ValideiDocument* _tmp3_;
				ValideiDocument* _tmp4_;
				document = NULL;
				_tmp3_ = valide_document_manager_create (self, file, &_inner_error_);
				if (_inner_error_ != NULL) {
					if ((_inner_error_->domain == VALIDE_DOCUMENT_ERROR) || (_inner_error_->domain == VALIDE_PROJECT_ERROR)) {
						g_propagate_error (error, _inner_error_);
						file = (g_free (file), NULL);
						(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
						(files == NULL) ? NULL : (files = (_g_slist_free_g_free (files), NULL));
						(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
						return;
					} else {
						file = (g_free (file), NULL);
						(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
						(files == NULL) ? NULL : (files = (_g_slist_free_g_free (files), NULL));
						(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
						g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
						g_clear_error (&_inner_error_);
						return;
					}
				}
				_tmp4_ = NULL;
#line 517 "document-manager.vala"
				document = (_tmp4_ = _tmp3_, (document == NULL) ? NULL : (document = (g_object_unref (document), NULL)), _tmp4_);
#line 518 "document-manager.vala"
				if (document != NULL) {
#line 520 "document-manager.vala"
					valide_idocument_set_is_save (valide_document_manager_get_current (self), TRUE);
#line 1577 "document-manager.c"
				}
				file = (g_free (file), NULL);
				(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
			}
		}
	}
	(files == NULL) ? NULL : (files = (_g_slist_free_g_free (files), NULL));
	(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
}


/**
   * Save all opening documents.
   *
   * @return true if all documents are really saved.
   */
#line 530 "document-manager.vala"
gboolean valide_document_manager_save_all (ValideDocumentManager* self) {
#line 1596 "document-manager.c"
	gboolean result;
	gboolean saved;
#line 530 "document-manager.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1601 "document-manager.c"
	saved = TRUE;
	{
		GList* document_collection;
		GList* document_it;
#line 534 "document-manager.vala"
		document_collection = self->priv->_documents;
#line 1608 "document-manager.c"
		for (document_it = document_collection; document_it != NULL; document_it = document_it->next) {
			ValideiDocument* _tmp0_;
			ValideiDocument* document;
#line 92 "document-manager.vala"
			_tmp0_ = NULL;
#line 534 "document-manager.vala"
			document = (_tmp0_ = (ValideiDocument*) document_it->data, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
#line 1616 "document-manager.c"
			{
#line 536 "document-manager.vala"
				valide_idocument_save (document, NULL);
#line 537 "document-manager.vala"
				if (!valide_idocument_get_is_save (document)) {
#line 539 "document-manager.vala"
					saved = FALSE;
#line 1624 "document-manager.c"
					(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
#line 540 "document-manager.vala"
					break;
#line 1628 "document-manager.c"
				}
				(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
			}
		}
	}
	result = saved;
	return result;
}


static void _g_list_free_g_object_unref (GList* self) {
	g_list_foreach (self, (GFunc) g_object_unref, NULL);
	g_list_free (self);
}


/**
   * Close all opening documents.
   *
   * @return true if all documents are really closed.
   */
#line 551 "document-manager.vala"
gboolean valide_document_manager_close_all (ValideDocumentManager* self) {
#line 1652 "document-manager.c"
	gboolean result;
	gint unsave;
	gboolean close;
#line 551 "document-manager.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1658 "document-manager.c"
	unsave = 0;
	close = TRUE;
	{
		GList* document_collection;
		GList* document_it;
#line 556 "document-manager.vala"
		document_collection = self->priv->_documents;
#line 1666 "document-manager.c"
		for (document_it = document_collection; document_it != NULL; document_it = document_it->next) {
			ValideiDocument* _tmp0_;
			ValideiDocument* document;
#line 92 "document-manager.vala"
			_tmp0_ = NULL;
#line 556 "document-manager.vala"
			document = (_tmp0_ = (ValideiDocument*) document_it->data, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
#line 1674 "document-manager.c"
			{
#line 558 "document-manager.vala"
				if (!valide_idocument_get_is_save (document)) {
#line 560 "document-manager.vala"
					unsave++;
#line 1680 "document-manager.c"
				}
				(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
			}
		}
	}
#line 564 "document-manager.vala"
	if (unsave > 0) {
#line 1688 "document-manager.c"
		ValideDocumentCloseDialog* dialog;
		GtkWindow* parent;
		gint response;
		ValideDocumentCloseDialog* _tmp1_;
		GtkWindow* _tmp4_;
		GtkWindow* _tmp3_;
		GtkWidget* _tmp2_;
		dialog = NULL;
		parent = NULL;
		response = 0;
		_tmp1_ = NULL;
#line 570 "document-manager.vala"
		dialog = (_tmp1_ = g_object_ref_sink (valide_document_close_dialog_new (self->priv->_documents)), (dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL)), _tmp1_);
#line 1702 "document-manager.c"
		_tmp4_ = NULL;
#line 571 "document-manager.vala"
		_tmp3_ = NULL;
#line 571 "document-manager.vala"
		_tmp2_ = NULL;
#line 571 "document-manager.vala"
		parent = (_tmp4_ = (_tmp3_ = (_tmp2_ = gtk_widget_get_ancestor ((GtkWidget*) self, GTK_TYPE_WINDOW), GTK_IS_WINDOW (_tmp2_) ? ((GtkWindow*) _tmp2_) : NULL), (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), (parent == NULL) ? NULL : (parent = (g_object_unref (parent), NULL)), _tmp4_);
#line 572 "document-manager.vala"
		valide_document_close_dialog_set_transient_for (dialog, parent);
#line 573 "document-manager.vala"
		valide_document_close_dialog_set_modal (dialog, TRUE);
#line 574 "document-manager.vala"
		response = valide_document_close_dialog_run (dialog);
#line 575 "document-manager.vala"
		valide_document_close_dialog_destroy (dialog);
#line 577 "document-manager.vala"
		switch (response) {
#line 1720 "document-manager.c"
			case GTK_RESPONSE_OK:
			{
				{
					GList* document_collection;
					GList* document_it;
#line 580 "document-manager.vala"
					document_collection = valide_document_close_dialog_get_selected (dialog);
#line 1728 "document-manager.c"
					for (document_it = document_collection; document_it != NULL; document_it = document_it->next) {
						ValideiDocument* _tmp5_;
						ValideiDocument* document;
#line 43 "document-close-dialog.vala"
						_tmp5_ = NULL;
#line 580 "document-manager.vala"
						document = (_tmp5_ = (ValideiDocument*) document_it->data, (_tmp5_ == NULL) ? NULL : g_object_ref (_tmp5_));
#line 1736 "document-manager.c"
						{
#line 582 "document-manager.vala"
							valide_idocument_save (document, NULL);
#line 1740 "document-manager.c"
							(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
						}
					}
				}
#line 584 "document-manager.vala"
				close = TRUE;
#line 585 "document-manager.vala"
				break;
#line 1749 "document-manager.c"
			}
			case GTK_RESPONSE_CLOSE:
			{
#line 587 "document-manager.vala"
				close = TRUE;
#line 588 "document-manager.vala"
				break;
#line 1757 "document-manager.c"
			}
			case GTK_RESPONSE_CANCEL:
			{
#line 590 "document-manager.vala"
				close = FALSE;
#line 591 "document-manager.vala"
				break;
#line 1765 "document-manager.c"
			}
		}
		(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
		(parent == NULL) ? NULL : (parent = (g_object_unref (parent), NULL));
	}
#line 595 "document-manager.vala"
	if (close) {
#line 1773 "document-manager.c"
		GList* documents;
		GList* _tmp6_;
		documents = NULL;
		_tmp6_ = NULL;
#line 599 "document-manager.vala"
		documents = (_tmp6_ = g_list_copy (self->priv->_documents), (documents == NULL) ? NULL : (documents = (_g_list_free_g_object_unref (documents), NULL)), _tmp6_);
#line 1780 "document-manager.c"
		{
			GList* document_collection;
			GList* document_it;
#line 600 "document-manager.vala"
			document_collection = documents;
#line 1786 "document-manager.c"
			for (document_it = document_collection; document_it != NULL; document_it = document_it->next) {
				ValideiDocument* _tmp7_;
				ValideiDocument* document;
#line 597 "document-manager.vala"
				_tmp7_ = NULL;
#line 600 "document-manager.vala"
				document = (_tmp7_ = (ValideiDocument*) document_it->data, (_tmp7_ == NULL) ? NULL : g_object_ref (_tmp7_));
#line 1794 "document-manager.c"
				{
#line 602 "document-manager.vala"
					valide_idocument_set_is_save (document, TRUE);
#line 603 "document-manager.vala"
					valide_idocument_close (document);
#line 1800 "document-manager.c"
					(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
				}
			}
		}
		(documents == NULL) ? NULL : (documents = (_g_list_free_g_object_unref (documents), NULL));
	}
	result = gtk_notebook_get_n_pages ((GtkNotebook*) self) == 0;
	return result;
}


/**
   * Append new page in notebook.
   *
   * @param child the document added.
   */
#line 614 "document-manager.vala"
gint valide_document_manager_append_page (ValideDocumentManager* self, ValideiDocument* child) {
#line 1819 "document-manager.c"
	gint result;
	gint page_num;
	ValideiDocument* _tmp0_;
#line 614 "document-manager.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 614 "document-manager.vala"
	g_return_val_if_fail (child != NULL, 0);
#line 1827 "document-manager.c"
	page_num = 0;
#line 618 "document-manager.vala"
	_tmp0_ = NULL;
#line 618 "document-manager.vala"
	self->priv->_documents = g_list_append (self->priv->_documents, (_tmp0_ = child, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)));
#line 619 "document-manager.vala"
	page_num = gtk_notebook_append_page (GTK_NOTEBOOK (self), (GtkWidget*) child, NULL);
#line 620 "document-manager.vala"
	gtk_notebook_set_tab_reorderable ((GtkNotebook*) self, (GtkWidget*) child, TRUE);
#line 621 "document-manager.vala"
	valide_document_manager_title_change (self, child);
#line 622 "document-manager.vala"
	valide_document_manager_setup_ui (self);
#line 623 "document-manager.vala"
	g_signal_emit_by_name (self, "tab-added", child);
#line 1843 "document-manager.c"
	result = page_num;
	return result;
}


/**
   * Remove page from notebook.
   *
   * @param child a document.
   */
#line 632 "document-manager.vala"
void valide_document_manager_remove_page (ValideDocumentManager* self, ValideiDocument* child) {
#line 1856 "document-manager.c"
	gint page_num;
#line 632 "document-manager.vala"
	g_return_if_fail (self != NULL);
#line 632 "document-manager.vala"
	g_return_if_fail (child != NULL);
#line 1862 "document-manager.c"
	page_num = 0;
#line 636 "document-manager.vala"
	page_num = gtk_notebook_page_num ((GtkNotebook*) self, (GtkWidget*) child);
#line 637 "document-manager.vala"
	if (page_num >= 0) {
#line 639 "document-manager.vala"
		gtk_notebook_remove_page (GTK_NOTEBOOK (self), page_num);
#line 640 "document-manager.vala"
		self->priv->_documents = g_list_remove (self->priv->_documents, child);
#line 641 "document-manager.vala"
		valide_document_manager_setup_ui (self);
#line 642 "document-manager.vala"
		g_signal_emit_by_name (self, "tab-removed", child);
#line 1876 "document-manager.c"
	}
}


/**
   * Returns the document contained in page number page_num.
   *
   * @param page_num the index of a page in the notebook, or -1 to get the last page. 
   *
   * @return The document, or NULL if page_num is out of bounds.
   */
#line 653 "document-manager.vala"
ValideiDocument* valide_document_manager_get_nth (ValideDocumentManager* self, gint page_num) {
#line 1890 "document-manager.c"
	ValideiDocument* result;
	ValideiDocument* _tmp1_;
	GtkWidget* _tmp0_;
#line 653 "document-manager.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 655 "document-manager.vala"
	_tmp1_ = NULL;
#line 655 "document-manager.vala"
	_tmp0_ = NULL;
#line 1900 "document-manager.c"
	result = (_tmp1_ = (_tmp0_ = gtk_notebook_get_nth_page ((GtkNotebook*) self, page_num), VALIDE_IS_IDOCUMENT (_tmp0_) ? ((ValideiDocument*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_));
	return result;
}


/**
   * Check if file is already open.
   *
   * @param filename the path of file.
   * @param pos return tab position.
   *
   * @return true if file is already open.
   */
#line 666 "document-manager.vala"
gboolean valide_document_manager_is_open (ValideDocumentManager* self, const char* filename, gint* pos) {
#line 1916 "document-manager.c"
	gboolean result;
	gboolean opened;
#line 666 "document-manager.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1921 "document-manager.c"
	opened = FALSE;
#line 670 "document-manager.vala"
	if (filename != NULL) {
#line 672 "document-manager.vala"
		(*pos) = 0;
#line 1927 "document-manager.c"
		{
			GList* d_collection;
			GList* d_it;
#line 673 "document-manager.vala"
			d_collection = self->priv->_documents;
#line 1933 "document-manager.c"
			for (d_it = d_collection; d_it != NULL; d_it = d_it->next) {
				ValideiDocument* _tmp0_;
				ValideiDocument* d;
#line 92 "document-manager.vala"
				_tmp0_ = NULL;
#line 673 "document-manager.vala"
				d = (_tmp0_ = (ValideiDocument*) d_it->data, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
#line 1941 "document-manager.c"
				{
#line 675 "document-manager.vala"
					if (_vala_strcmp0 (valide_idocument_get_path (d), filename) == 0) {
#line 677 "document-manager.vala"
						opened = TRUE;
#line 1947 "document-manager.c"
						(d == NULL) ? NULL : (d = (g_object_unref (d), NULL));
#line 678 "document-manager.vala"
						break;
#line 1951 "document-manager.c"
					}
#line 680 "document-manager.vala"
					(*pos)++;
#line 1955 "document-manager.c"
					(d == NULL) ? NULL : (d = (g_object_unref (d), NULL));
				}
			}
		}
	}
	result = opened;
	return result;
}


ValideUIManager* valide_document_manager_get_ui_manager (ValideDocumentManager* self) {
	ValideUIManager* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_ui_manager;
	return result;
}


static void valide_document_manager_set_ui_manager (ValideDocumentManager* self, ValideUIManager* value) {
	ValideUIManager* _tmp1_;
	ValideUIManager* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 41 "document-manager.vala"
	_tmp0_ = NULL;
#line 1981 "document-manager.c"
	self->priv->_ui_manager = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_ui_manager == NULL) ? NULL : (self->priv->_ui_manager = (g_object_unref (self->priv->_ui_manager), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "ui-manager");
}


ValideiDocument* valide_document_manager_get_current (ValideDocumentManager* self) {
	ValideiDocument* result;
	gint _tmp0_;
	GtkWidget* _tmp1_;
	g_return_val_if_fail (self != NULL, NULL);
#line 81 "document-manager.vala"
	_tmp1_ = NULL;
#line 1994 "document-manager.c"
	result = (_tmp1_ = gtk_notebook_get_nth_page ((GtkNotebook*) self, (g_object_get ((GtkNotebook*) self, "page", &_tmp0_, NULL), _tmp0_)), VALIDE_IS_IDOCUMENT (_tmp1_) ? ((ValideiDocument*) _tmp1_) : NULL);
	return result;
}


void valide_document_manager_set_current (ValideDocumentManager* self, ValideiDocument* value) {
	g_return_if_fail (self != NULL);
#line 85 "document-manager.vala"
	gtk_notebook_set_current_page ((GtkNotebook*) self, gtk_notebook_page_num ((GtkNotebook*) self, (GtkWidget*) value));
#line 2004 "document-manager.c"
	g_object_notify ((GObject *) self, "current");
}


GList* valide_document_manager_get_documents (ValideDocumentManager* self) {
	GList* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_documents;
	return result;
}


static void valide_document_manager_set_documents (ValideDocumentManager* self, GList* value) {
	g_return_if_fail (self != NULL);
	self->priv->_documents = value;
	g_object_notify ((GObject *) self, "documents");
}


#line 339 "document-manager.vala"
static gboolean _valide_document_manager_on_button_press_gtk_widget_button_press_event (GtkWidget* _sender, const GdkEventButton* event, gpointer self) {
#line 2026 "document-manager.c"
	return valide_document_manager_on_button_press (self, _sender, event);
}


#line 260 "document-manager.vala"
static void _valide_document_manager_drag_data_received_cb_gtk_widget_drag_data_received (GtkWidget* _sender, GdkDragContext* context, gint x, gint y, GtkSelectionData* selection_data, guint info, guint time_, gpointer self) {
#line 2033 "document-manager.c"
	valide_document_manager_drag_data_received_cb (self, _sender, context, x, y, selection_data, info, time_);
}


static GObject * valide_document_manager_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValideDocumentManagerClass * klass;
	GObjectClass * parent_class;
	ValideDocumentManager * self;
	klass = VALIDE_DOCUMENT_MANAGER_CLASS (g_type_class_peek (VALIDE_TYPE_DOCUMENT_MANAGER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_DOCUMENT_MANAGER (obj);
	{
		gint targets_size;
		gint targets_length1;
		GtkTargetEntry* targets;
		GtkTargetEntry* _tmp0_;
		targets = (targets_length1 = 0, NULL);
#line 391 "document-manager.vala"
		gtk_notebook_set_scrollable ((GtkNotebook*) self, TRUE);
#line 392 "document-manager.vala"
		valide_document_manager_setup_ui (self);
#line 393 "document-manager.vala"
		g_signal_connect_after (self, "switch-page", (GCallback) valide_document_manager_switch_page_cb, self);
#line 394 "document-manager.vala"
		g_signal_connect_object ((GtkWidget*) self, "button-press-event", (GCallback) _valide_document_manager_on_button_press_gtk_widget_button_press_event, self, 0);
#line 2061 "document-manager.c"
		/* Seting up the drag & drop */
		_tmp0_ = NULL;
#line 397 "document-manager.vala"
		targets = (_tmp0_ = g_new0 (GtkTargetEntry, 1), targets = (g_free (targets), NULL), targets_length1 = 1, targets_size = targets_length1, _tmp0_);
#line 398 "document-manager.vala"
		targets[0].target = "text/uri-list";
#line 399 "document-manager.vala"
		targets[0].flags = (guint) 0;
#line 400 "document-manager.vala"
		targets[0].info = (guint) 0;
#line 402 "document-manager.vala"
		gtk_drag_dest_set ((GtkWidget*) self, GTK_DEST_DEFAULT_ALL, targets, targets_length1, GDK_ACTION_COPY);
#line 404 "document-manager.vala"
		g_signal_connect_object ((GtkWidget*) self, "drag-data-received", (GCallback) _valide_document_manager_drag_data_received_cb_gtk_widget_drag_data_received, self, 0);
#line 2076 "document-manager.c"
		targets = (g_free (targets), NULL);
	}
	return obj;
}


static void valide_document_manager_class_init (ValideDocumentManagerClass * klass) {
	valide_document_manager_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideDocumentManagerPrivate));
	VALIDE_DOCUMENT_MANAGER_CLASS (klass)->setup_ui = valide_document_manager_real_setup_ui;
	VALIDE_DOCUMENT_MANAGER_CLASS (klass)->create_document = valide_document_manager_real_create_document;
	G_OBJECT_CLASS (klass)->get_property = valide_document_manager_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_document_manager_set_property;
	G_OBJECT_CLASS (klass)->constructor = valide_document_manager_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_document_manager_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_DOCUMENT_MANAGER_UI_MANAGER, g_param_spec_object ("ui-manager", "ui-manager", "ui-manager", VALIDE_TYPE_UI_MANAGER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_DOCUMENT_MANAGER_CURRENT, g_param_spec_object ("current", "current", "current", VALIDE_TYPE_IDOCUMENT, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_DOCUMENT_MANAGER_DOCUMENTS, g_param_spec_pointer ("documents", "documents", "documents", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_signal_new ("tab_added", VALIDE_TYPE_DOCUMENT_MANAGER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, VALIDE_TYPE_IDOCUMENT);
	g_signal_new ("tab_saved", VALIDE_TYPE_DOCUMENT_MANAGER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, VALIDE_TYPE_IDOCUMENT);
	g_signal_new ("tab_removed", VALIDE_TYPE_DOCUMENT_MANAGER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, VALIDE_TYPE_IDOCUMENT);
	g_signal_new ("tab_changed", VALIDE_TYPE_DOCUMENT_MANAGER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, VALIDE_TYPE_IDOCUMENT);
	g_signal_new ("tab_state_changed", VALIDE_TYPE_DOCUMENT_MANAGER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, VALIDE_TYPE_IDOCUMENT);
	g_signal_new ("tab_cursor_moved", VALIDE_TYPE_DOCUMENT_MANAGER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_VOID__OBJECT_INT_INT, G_TYPE_NONE, 3, VALIDE_TYPE_IDOCUMENT, G_TYPE_INT, G_TYPE_INT);
}


static void valide_document_manager_instance_init (ValideDocumentManager * self) {
	self->priv = VALIDE_DOCUMENT_MANAGER_GET_PRIVATE (self);
}


static void valide_document_manager_finalize (GObject* obj) {
	ValideDocumentManager * self;
	self = VALIDE_DOCUMENT_MANAGER (obj);
	(self->priv->_ui_manager == NULL) ? NULL : (self->priv->_ui_manager = (g_object_unref (self->priv->_ui_manager), NULL));
	G_OBJECT_CLASS (valide_document_manager_parent_class)->finalize (obj);
}


GType valide_document_manager_get_type (void) {
	static GType valide_document_manager_type_id = 0;
	if (valide_document_manager_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideDocumentManagerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_document_manager_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideDocumentManager), 0, (GInstanceInitFunc) valide_document_manager_instance_init, NULL };
		valide_document_manager_type_id = g_type_register_static (GTK_TYPE_NOTEBOOK, "ValideDocumentManager", &g_define_type_info, 0);
	}
	return valide_document_manager_type_id;
}


static void valide_document_manager_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideDocumentManager * self;
	gpointer boxed;
	self = VALIDE_DOCUMENT_MANAGER (object);
	switch (property_id) {
		case VALIDE_DOCUMENT_MANAGER_UI_MANAGER:
		g_value_set_object (value, valide_document_manager_get_ui_manager (self));
		break;
		case VALIDE_DOCUMENT_MANAGER_CURRENT:
		g_value_set_object (value, valide_document_manager_get_current (self));
		break;
		case VALIDE_DOCUMENT_MANAGER_DOCUMENTS:
		g_value_set_pointer (value, valide_document_manager_get_documents (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_document_manager_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideDocumentManager * self;
	self = VALIDE_DOCUMENT_MANAGER (object);
	switch (property_id) {
		case VALIDE_DOCUMENT_MANAGER_UI_MANAGER:
		valide_document_manager_set_ui_manager (self, g_value_get_object (value));
		break;
		case VALIDE_DOCUMENT_MANAGER_CURRENT:
		valide_document_manager_set_current (self, g_value_get_object (value));
		break;
		case VALIDE_DOCUMENT_MANAGER_DOCUMENTS:
		valide_document_manager_set_documents (self, g_value_get_pointer (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}



static void g_cclosure_user_marshal_VOID__OBJECT_INT_INT (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data) {
	typedef void (*GMarshalFunc_VOID__OBJECT_INT_INT) (gpointer data1, gpointer arg_1, gint arg_2, gint arg_3, gpointer data2);
	register GMarshalFunc_VOID__OBJECT_INT_INT callback;
	register GCClosure * cc;
	register gpointer data1, data2;
	cc = (GCClosure *) closure;
	g_return_if_fail (n_param_values == 4);
	if (G_CCLOSURE_SWAP_DATA (closure)) {
		data1 = closure->data;
		data2 = param_values->data[0].v_pointer;
	} else {
		data1 = param_values->data[0].v_pointer;
		data2 = closure->data;
	}
	callback = (GMarshalFunc_VOID__OBJECT_INT_INT) (marshal_data ? marshal_data : cc->callback);
	callback (data1, g_value_get_object (param_values + 1), g_value_get_int (param_values + 2), g_value_get_int (param_values + 3), data2);
}



