/* document-message.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <math.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <glib/gi18n-lib.h>


#define VALIDE_TYPE_DOCUMENT_MESSAGE (valide_document_message_get_type ())
#define VALIDE_DOCUMENT_MESSAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_DOCUMENT_MESSAGE, ValideDocumentMessage))
#define VALIDE_DOCUMENT_MESSAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_DOCUMENT_MESSAGE, ValideDocumentMessageClass))
#define VALIDE_IS_DOCUMENT_MESSAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_DOCUMENT_MESSAGE))
#define VALIDE_IS_DOCUMENT_MESSAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_DOCUMENT_MESSAGE))
#define VALIDE_DOCUMENT_MESSAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_DOCUMENT_MESSAGE, ValideDocumentMessageClass))

typedef struct _ValideDocumentMessage ValideDocumentMessage;
typedef struct _ValideDocumentMessageClass ValideDocumentMessageClass;
typedef struct _ValideDocumentMessagePrivate ValideDocumentMessagePrivate;

#define VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT (valide_abstract_text_document_get_type ())
#define VALIDE_ABSTRACT_TEXT_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocument))
#define VALIDE_ABSTRACT_TEXT_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocumentClass))
#define VALIDE_IS_ABSTRACT_TEXT_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT))
#define VALIDE_IS_ABSTRACT_TEXT_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT))
#define VALIDE_ABSTRACT_TEXT_DOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocumentClass))

typedef struct _ValideAbstractTextDocument ValideAbstractTextDocument;
typedef struct _ValideAbstractTextDocumentClass ValideAbstractTextDocumentClass;

#define VALIDE_TYPE_DOCUMENT_STATE (valide_document_state_get_type ())

#define VALIDE_TYPE_IDOCUMENT (valide_idocument_get_type ())
#define VALIDE_IDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_IDOCUMENT, ValideiDocument))
#define VALIDE_IS_IDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_IDOCUMENT))
#define VALIDE_IDOCUMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), VALIDE_TYPE_IDOCUMENT, ValideiDocumentIface))

typedef struct _ValideiDocument ValideiDocument;
typedef struct _ValideiDocumentIface ValideiDocumentIface;

#define VALIDE_TYPE_UI_MANAGER (valide_ui_manager_get_type ())
#define VALIDE_UI_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_UI_MANAGER, ValideUIManager))
#define VALIDE_UI_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_UI_MANAGER, ValideUIManagerClass))
#define VALIDE_IS_UI_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_UI_MANAGER))
#define VALIDE_IS_UI_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_UI_MANAGER))
#define VALIDE_UI_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_UI_MANAGER, ValideUIManagerClass))

typedef struct _ValideUIManager ValideUIManager;
typedef struct _ValideUIManagerClass ValideUIManagerClass;

/**
 * A message widget for Valide.AbstractTextDocument
 */
struct _ValideDocumentMessage {
	GtkHBox parent_instance;
	ValideDocumentMessagePrivate * priv;
};

struct _ValideDocumentMessageClass {
	GtkHBoxClass parent_class;
};

struct _ValideDocumentMessagePrivate {
	GtkLabel* primary_label;
	GtkLabel* secondary_label;
	ValideAbstractTextDocument* _document;
};

typedef enum  {
	VALIDE_DOCUMENT_STATE_NORMAL,
	VALIDE_DOCUMENT_STATE_EXTERNALLY_MODIFIED,
	VALIDE_DOCUMENT_STATE_BAD_ENCODING,
	VALIDE_DOCUMENT_STATE_INFO
} ValideDocumentState;

/**
 * An interface for document
 */
struct _ValideiDocumentIface {
	GTypeInterface parent_iface;
	void (*open) (ValideiDocument* self, const char* path, GError** error);
	void (*save) (ValideiDocument* self, const char* current_folder);
	void (*save_as) (ValideiDocument* self, const char* current_folder);
	void (*rename) (ValideiDocument* self);
	gboolean (*close) (ValideiDocument* self);
	GdkPixbuf* (*get_icon) (ValideiDocument* self);
	char* (*get_tooltips) (ValideiDocument* self);
	void (*setup_ui) (ValideiDocument* self, ValideUIManager* ui_manager);
	ValideDocumentState (*get_state) (ValideiDocument* self);
	void (*set_state) (ValideiDocument* self, ValideDocumentState value);
	gboolean (*get_is_save) (ValideiDocument* self);
	void (*set_is_save) (ValideiDocument* self, gboolean value);
	const char* (*get_path) (ValideiDocument* self);
	void (*set_path) (ValideiDocument* self, const char* value);
	const char* (*get_title) (ValideiDocument* self);
	const char* (*get_filename) (ValideiDocument* self);
};


static gpointer valide_document_message_parent_class = NULL;

GType valide_document_message_get_type (void);
GType valide_abstract_text_document_get_type (void);
#define VALIDE_DOCUMENT_MESSAGE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_DOCUMENT_MESSAGE, ValideDocumentMessagePrivate))
enum  {
	VALIDE_DOCUMENT_MESSAGE_DUMMY_PROPERTY,
	VALIDE_DOCUMENT_MESSAGE_DOCUMENT
};
static void valide_document_message_set_document (ValideDocumentMessage* self, ValideAbstractTextDocument* value);
ValideDocumentMessage* valide_document_message_new (ValideAbstractTextDocument* document);
ValideDocumentMessage* valide_document_message_construct (GType object_type, ValideAbstractTextDocument* document);
ValideAbstractTextDocument* valide_document_message_get_document (ValideDocumentMessage* self);
void valide_abstract_text_document_reload (ValideAbstractTextDocument* self);
GType valide_document_state_get_type (void);
GType valide_ui_manager_get_type (void);
GType valide_idocument_get_type (void);
void valide_idocument_set_state (ValideiDocument* self, ValideDocumentState value);
void valide_abstract_text_document_grab_focus (ValideAbstractTextDocument* self);
static void _lambda13_ (ValideDocumentMessage* self);
static void __lambda13__gtk_button_clicked (GtkButton* _sender, gpointer self);
void valide_idocument_set_is_save (ValideiDocument* self, gboolean value);
static void _lambda14_ (ValideDocumentMessage* self);
static void __lambda14__gtk_button_clicked (GtkButton* _sender, gpointer self);
ValideDocumentState valide_idocument_get_state (ValideiDocument* self);
const char* valide_idocument_get_path (ValideiDocument* self);
gboolean valide_idocument_get_is_save (ValideiDocument* self);
static GObject * valide_document_message_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void valide_document_message_finalize (GObject* obj);
static void valide_document_message_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_document_message_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);



/**
   * Create a new DocumentMessage
   *
   * @param document The attached document
   */
#line 44 "document-message.vala"
ValideDocumentMessage* valide_document_message_construct (GType object_type, ValideAbstractTextDocument* document) {
#line 165 "document-message.c"
	GParameter * __params;
	GParameter * __params_it;
	ValideDocumentMessage * self;
#line 44 "document-message.vala"
	g_return_val_if_fail (document != NULL, NULL);
#line 171 "document-message.c"
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
#line 46 "document-message.vala"
	__params_it->name = "document";
#line 46 "document-message.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT);
#line 46 "document-message.vala"
	g_value_set_object (&__params_it->value, document);
#line 46 "document-message.vala"
	__params_it++;
#line 182 "document-message.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 44 "document-message.vala"
ValideDocumentMessage* valide_document_message_new (ValideAbstractTextDocument* document) {
#line 44 "document-message.vala"
	return valide_document_message_construct (VALIDE_TYPE_DOCUMENT_MESSAGE, document);
#line 197 "document-message.c"
}


ValideAbstractTextDocument* valide_document_message_get_document (ValideDocumentMessage* self) {
	ValideAbstractTextDocument* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_document;
	return result;
}


static void valide_document_message_set_document (ValideDocumentMessage* self, ValideAbstractTextDocument* value) {
	ValideAbstractTextDocument* _tmp1_;
	ValideAbstractTextDocument* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 35 "document-message.vala"
	_tmp0_ = NULL;
#line 216 "document-message.c"
	self->priv->_document = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_document == NULL) ? NULL : (self->priv->_document = (g_object_unref (self->priv->_document), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "document");
}


static void _lambda13_ (ValideDocumentMessage* self) {
#line 84 "document-message.vala"
	valide_abstract_text_document_reload (self->priv->_document);
#line 85 "document-message.vala"
	valide_idocument_set_state ((ValideiDocument*) self->priv->_document, VALIDE_DOCUMENT_STATE_NORMAL);
#line 86 "document-message.vala"
	valide_abstract_text_document_grab_focus (self->priv->_document);
#line 87 "document-message.vala"
	gtk_object_destroy ((GtkObject*) self);
#line 231 "document-message.c"
}


static void __lambda13__gtk_button_clicked (GtkButton* _sender, gpointer self) {
	_lambda13_ (self);
}


static void _lambda14_ (ValideDocumentMessage* self) {
#line 93 "document-message.vala"
	valide_idocument_set_is_save ((ValideiDocument*) self->priv->_document, FALSE);
#line 94 "document-message.vala"
	valide_idocument_set_state ((ValideiDocument*) self->priv->_document, VALIDE_DOCUMENT_STATE_NORMAL);
#line 95 "document-message.vala"
	valide_abstract_text_document_grab_focus (self->priv->_document);
#line 96 "document-message.vala"
	gtk_object_destroy ((GtkObject*) self);
#line 249 "document-message.c"
}


static void __lambda14__gtk_button_clicked (GtkButton* _sender, gpointer self) {
	_lambda14_ (self);
}


static GObject * valide_document_message_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValideDocumentMessageClass * klass;
	GObjectClass * parent_class;
	ValideDocumentMessage * self;
	klass = VALIDE_DOCUMENT_MESSAGE_CLASS (g_type_class_peek (VALIDE_TYPE_DOCUMENT_MESSAGE));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_DOCUMENT_MESSAGE (obj);
	{
		GtkImage* image;
		GtkVBox* vbox;
		GtkVButtonBox* box;
		GtkButton* button;
		char* text;
		GtkImage* _tmp0_;
		GtkVBox* _tmp1_;
		GtkLabel* _tmp2_;
		GtkLabel* _tmp3_;
		GtkVButtonBox* _tmp4_;
		GtkButton* _tmp5_;
		GtkButton* _tmp6_;
		image = NULL;
		vbox = NULL;
		box = NULL;
		button = NULL;
		text = NULL;
#line 57 "document-message.vala"
		gtk_box_set_spacing ((GtkBox*) self, 16);
#line 58 "document-message.vala"
		gtk_container_set_border_width ((GtkContainer*) self, (guint) 8);
#line 289 "document-message.c"
		_tmp0_ = NULL;
#line 60 "document-message.vala"
		image = (_tmp0_ = g_object_ref_sink ((GtkImage*) gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING, GTK_ICON_SIZE_DIALOG)), (image == NULL) ? NULL : (image = (g_object_unref (image), NULL)), _tmp0_);
#line 62 "document-message.vala"
		gtk_box_pack_start ((GtkBox*) self, (GtkWidget*) image, FALSE, TRUE, (guint) 0);
#line 295 "document-message.c"
		_tmp1_ = NULL;
#line 65 "document-message.vala"
		vbox = (_tmp1_ = g_object_ref_sink ((GtkVBox*) gtk_vbox_new (FALSE, 0)), (vbox == NULL) ? NULL : (vbox = (g_object_unref (vbox), NULL)), _tmp1_);
#line 66 "document-message.vala"
		gtk_box_pack_start ((GtkBox*) self, (GtkWidget*) vbox, TRUE, TRUE, (guint) 0);
#line 301 "document-message.c"
		_tmp2_ = NULL;
#line 68 "document-message.vala"
		self->priv->primary_label = (_tmp2_ = g_object_ref_sink ((GtkLabel*) gtk_label_new ("")), (self->priv->primary_label == NULL) ? NULL : (self->priv->primary_label = (g_object_unref (self->priv->primary_label), NULL)), _tmp2_);
#line 69 "document-message.vala"
		gtk_misc_set_alignment ((GtkMisc*) self->priv->primary_label, (float) 0, 0.5f);
#line 70 "document-message.vala"
		gtk_label_set_justify (self->priv->primary_label, GTK_JUSTIFY_LEFT);
#line 71 "document-message.vala"
		gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) self->priv->primary_label, TRUE, TRUE, (guint) 0);
#line 311 "document-message.c"
		_tmp3_ = NULL;
#line 73 "document-message.vala"
		self->priv->secondary_label = (_tmp3_ = g_object_ref_sink ((GtkLabel*) gtk_label_new ("")), (self->priv->secondary_label == NULL) ? NULL : (self->priv->secondary_label = (g_object_unref (self->priv->secondary_label), NULL)), _tmp3_);
#line 74 "document-message.vala"
		gtk_misc_set_alignment ((GtkMisc*) self->priv->secondary_label, (float) 0, 0.5f);
#line 75 "document-message.vala"
		gtk_label_set_justify (self->priv->secondary_label, GTK_JUSTIFY_LEFT);
#line 76 "document-message.vala"
		gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) self->priv->secondary_label, FALSE, TRUE, (guint) 0);
#line 321 "document-message.c"
		_tmp4_ = NULL;
#line 78 "document-message.vala"
		box = (_tmp4_ = g_object_ref_sink ((GtkVButtonBox*) gtk_vbutton_box_new ()), (box == NULL) ? NULL : (box = (g_object_unref (box), NULL)), _tmp4_);
#line 79 "document-message.vala"
		gtk_box_pack_start ((GtkBox*) self, (GtkWidget*) box, FALSE, TRUE, (guint) 0);
#line 327 "document-message.c"
		_tmp5_ = NULL;
#line 81 "document-message.vala"
		button = (_tmp5_ = g_object_ref_sink ((GtkButton*) gtk_button_new_from_stock (GTK_STOCK_REFRESH)), (button == NULL) ? NULL : (button = (g_object_unref (button), NULL)), _tmp5_);
#line 82 "document-message.vala"
		gtk_box_pack_start ((GtkBox*) box, (GtkWidget*) button, TRUE, TRUE, (guint) 0);
#line 83 "document-message.vala"
		g_signal_connect (button, "clicked", (GCallback) __lambda13__gtk_button_clicked, self);
#line 335 "document-message.c"
		_tmp6_ = NULL;
#line 90 "document-message.vala"
		button = (_tmp6_ = g_object_ref_sink ((GtkButton*) gtk_button_new_from_stock (GTK_STOCK_CANCEL)), (button == NULL) ? NULL : (button = (g_object_unref (button), NULL)), _tmp6_);
#line 91 "document-message.vala"
		gtk_box_pack_start ((GtkBox*) box, (GtkWidget*) button, TRUE, TRUE, (guint) 0);
#line 92 "document-message.vala"
		g_signal_connect (button, "clicked", (GCallback) __lambda14__gtk_button_clicked, self);
#line 99 "document-message.vala"
		switch (valide_idocument_get_state ((ValideiDocument*) self->priv->_document)) {
#line 345 "document-message.c"
			case VALIDE_DOCUMENT_STATE_EXTERNALLY_MODIFIED:
			{
				char* _tmp7_;
				_tmp7_ = NULL;
#line 102 "document-message.vala"
				text = (_tmp7_ = g_strdup_printf (_ ("The file %s changed on disk."), valide_idocument_get_path ((ValideiDocument*) self->priv->_document)), text = (g_free (text), NULL), _tmp7_);
#line 103 "document-message.vala"
				gtk_label_set_text (self->priv->primary_label, text);
#line 105 "document-message.vala"
				if (!valide_idocument_get_is_save ((ValideiDocument*) self->priv->_document)) {
#line 356 "document-message.c"
					char* _tmp9_;
					const char* _tmp8_;
					_tmp9_ = NULL;
#line 107 "document-message.vala"
					_tmp8_ = NULL;
#line 107 "document-message.vala"
					text = (_tmp9_ = (_tmp8_ = _ ("Do you want to drop your changes and reload the file?"), (_tmp8_ == NULL) ? NULL : g_strdup (_tmp8_)), text = (g_free (text), NULL), _tmp9_);
#line 364 "document-message.c"
				} else {
					char* _tmp11_;
					const char* _tmp10_;
					_tmp11_ = NULL;
#line 111 "document-message.vala"
					_tmp10_ = NULL;
#line 111 "document-message.vala"
					text = (_tmp11_ = (_tmp10_ = _ ("Do you want to reload the file?"), (_tmp10_ == NULL) ? NULL : g_strdup (_tmp10_)), text = (g_free (text), NULL), _tmp11_);
#line 373 "document-message.c"
				}
#line 113 "document-message.vala"
				gtk_label_set_text (self->priv->secondary_label, text);
#line 114 "document-message.vala"
				break;
#line 379 "document-message.c"
			}
			case VALIDE_DOCUMENT_STATE_BAD_ENCODING:
			{
				char* _tmp12_;
				char* _tmp13_;
				_tmp12_ = NULL;
#line 116 "document-message.vala"
				text = (_tmp12_ = g_strdup_printf (_ ("Couldn't open the file %s."), valide_idocument_get_path ((ValideiDocument*) self->priv->_document)), text = (g_free (text), NULL), _tmp12_);
#line 117 "document-message.vala"
				gtk_label_set_text (self->priv->primary_label, text);
#line 390 "document-message.c"
				_tmp13_ = NULL;
#line 119 "document-message.vala"
				text = (_tmp13_ = g_strdup ("Val(a)IDE has not been able to detect the character coding.\nPlease check that you are not trying to open a binary file."), text = (g_free (text), NULL), _tmp13_);
#line 121 "document-message.vala"
				gtk_label_set_text (self->priv->secondary_label, text);
#line 122 "document-message.vala"
				break;
#line 398 "document-message.c"
			}
		}
		(image == NULL) ? NULL : (image = (g_object_unref (image), NULL));
		(vbox == NULL) ? NULL : (vbox = (g_object_unref (vbox), NULL));
		(box == NULL) ? NULL : (box = (g_object_unref (box), NULL));
		(button == NULL) ? NULL : (button = (g_object_unref (button), NULL));
		text = (g_free (text), NULL);
	}
	return obj;
}


static void valide_document_message_class_init (ValideDocumentMessageClass * klass) {
	valide_document_message_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideDocumentMessagePrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_document_message_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_document_message_set_property;
	G_OBJECT_CLASS (klass)->constructor = valide_document_message_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_document_message_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_DOCUMENT_MESSAGE_DOCUMENT, g_param_spec_object ("document", "document", "document", VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void valide_document_message_instance_init (ValideDocumentMessage * self) {
	self->priv = VALIDE_DOCUMENT_MESSAGE_GET_PRIVATE (self);
}


static void valide_document_message_finalize (GObject* obj) {
	ValideDocumentMessage * self;
	self = VALIDE_DOCUMENT_MESSAGE (obj);
	(self->priv->primary_label == NULL) ? NULL : (self->priv->primary_label = (g_object_unref (self->priv->primary_label), NULL));
	(self->priv->secondary_label == NULL) ? NULL : (self->priv->secondary_label = (g_object_unref (self->priv->secondary_label), NULL));
	(self->priv->_document == NULL) ? NULL : (self->priv->_document = (g_object_unref (self->priv->_document), NULL));
	G_OBJECT_CLASS (valide_document_message_parent_class)->finalize (obj);
}


GType valide_document_message_get_type (void) {
	static GType valide_document_message_type_id = 0;
	if (valide_document_message_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideDocumentMessageClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_document_message_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideDocumentMessage), 0, (GInstanceInitFunc) valide_document_message_instance_init, NULL };
		valide_document_message_type_id = g_type_register_static (GTK_TYPE_HBOX, "ValideDocumentMessage", &g_define_type_info, 0);
	}
	return valide_document_message_type_id;
}


static void valide_document_message_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideDocumentMessage * self;
	gpointer boxed;
	self = VALIDE_DOCUMENT_MESSAGE (object);
	switch (property_id) {
		case VALIDE_DOCUMENT_MESSAGE_DOCUMENT:
		g_value_set_object (value, valide_document_message_get_document (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_document_message_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideDocumentMessage * self;
	self = VALIDE_DOCUMENT_MESSAGE (object);
	switch (property_id) {
		case VALIDE_DOCUMENT_MESSAGE_DOCUMENT:
		valide_document_message_set_document (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}




