/* encoding.vala
 *
 * Copyright (C) 2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <glib/gstdio.h>
#include <gtk/gtk.h>


#define VALIDE_TYPE_ENCODING (valide_encoding_get_type ())
#define VALIDE_ENCODING(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ENCODING, ValideEncoding))
#define VALIDE_ENCODING_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ENCODING, ValideEncodingClass))
#define VALIDE_IS_ENCODING(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ENCODING))
#define VALIDE_IS_ENCODING_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ENCODING))
#define VALIDE_ENCODING_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ENCODING, ValideEncodingClass))

typedef struct _ValideEncoding ValideEncoding;
typedef struct _ValideEncodingClass ValideEncodingClass;
typedef struct _ValideEncodingPrivate ValideEncodingPrivate;

typedef enum  {
	VALIDE_ENCODING_ERROR_UNKNOW_ENCODING
} ValideEncodingError;
#define VALIDE_ENCODING_ERROR valide_encoding_error_quark ()
/**
 * A class for manager the file encoding
 */
struct _ValideEncoding {
	GObject parent_instance;
	ValideEncodingPrivate * priv;
};

struct _ValideEncodingClass {
	GObjectClass parent_class;
};


static gpointer valide_encoding_parent_class = NULL;

GQuark valide_encoding_error_quark (void);
GType valide_encoding_get_type (void);
enum  {
	VALIDE_ENCODING_DUMMY_PROPERTY
};
static char* valide_encoding_convert (const char* text, const char* to_codeset, const char* from_codeset, GError** error);
static gboolean valide_encoding_test (const char* text, const char* charset);
static char* valide_encoding_get_charset (const char* text);
char* valide_encoding_to_utf8 (const char* path, char** encoding, GError** error);
char* valide_encoding_from_utf8 (GtkTextBuffer* text_buffer, char** encoding, GError** error);
ValideEncoding* valide_encoding_new (void);
ValideEncoding* valide_encoding_construct (GType object_type);

static const char* VALIDE_ENCODING_charsets[] = {"UTF-8", "ISO-8859-15"};


GQuark valide_encoding_error_quark (void) {
	return g_quark_from_static_string ("valide_encoding_error-quark");
}


#line 34 "encoding.vala"
static char* valide_encoding_convert (const char* text, const char* to_codeset, const char* from_codeset, GError** error) {
#line 84 "encoding.c"
	char* result;
	GError * _inner_error_;
	char* _tmp0_;
#line 34 "encoding.vala"
	g_return_val_if_fail (text != NULL, NULL);
#line 34 "encoding.vala"
	g_return_val_if_fail (to_codeset != NULL, NULL);
#line 34 "encoding.vala"
	g_return_val_if_fail (from_codeset != NULL, NULL);
#line 94 "encoding.c"
	_inner_error_ = NULL;
	_tmp0_ = g_convert (text, (glong) (-1), to_codeset, from_codeset, NULL, NULL, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return NULL;
	}
	result = _tmp0_;
	return result;
}


#line 41 "encoding.vala"
static gboolean valide_encoding_test (const char* text, const char* charset) {
#line 108 "encoding.c"
	gboolean result;
	GError * _inner_error_;
	gboolean valid;
#line 41 "encoding.vala"
	g_return_val_if_fail (text != NULL, FALSE);
#line 41 "encoding.vala"
	g_return_val_if_fail (charset != NULL, FALSE);
#line 116 "encoding.c"
	_inner_error_ = NULL;
	valid = FALSE;
	{
		char* convert;
		char* _tmp0_;
		char* _tmp1_;
		convert = NULL;
		_tmp0_ = g_convert (text, (glong) (-1), "UTF-8", charset, NULL, NULL, &_inner_error_);
		if (_inner_error_ != NULL) {
			convert = (g_free (convert), NULL);
			goto __catch2_g_error;
			goto __finally2;
		}
		_tmp1_ = NULL;
#line 49 "encoding.vala"
		convert = (_tmp1_ = _tmp0_, convert = (g_free (convert), NULL), _tmp1_);
#line 50 "encoding.vala"
		valid = TRUE;
#line 135 "encoding.c"
		convert = (g_free (convert), NULL);
	}
	goto __finally2;
	__catch2_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 54 "encoding.vala"
			g_debug (e->message);
#line 147 "encoding.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally2:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return FALSE;
	}
	result = valid;
	return result;
}


#line 59 "encoding.vala"
static char* valide_encoding_get_charset (const char* text) {
#line 164 "encoding.c"
	char* result;
	char* charset;
#line 59 "encoding.vala"
	g_return_val_if_fail (text != NULL, NULL);
#line 169 "encoding.c"
	charset = NULL;
	{
		const char** c_collection;
		int c_collection_length1;
		int c_it;
#line 63 "encoding.vala"
		c_collection = VALIDE_ENCODING_charsets;
#line 177 "encoding.c"
		c_collection_length1 = G_N_ELEMENTS (VALIDE_ENCODING_charsets);
		for (c_it = 0; c_it < G_N_ELEMENTS (VALIDE_ENCODING_charsets); c_it = c_it + 1) {
			const char* _tmp2_;
			char* c;
#line 32 "encoding.vala"
			_tmp2_ = NULL;
#line 184 "encoding.c"
			c = (_tmp2_ = c_collection[c_it], (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_));
			{
#line 65 "encoding.vala"
				if (valide_encoding_test (text, c)) {
#line 189 "encoding.c"
					char* _tmp1_;
					const char* _tmp0_;
					_tmp1_ = NULL;
#line 67 "encoding.vala"
					_tmp0_ = NULL;
#line 67 "encoding.vala"
					charset = (_tmp1_ = (_tmp0_ = c, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), charset = (g_free (charset), NULL), _tmp1_);
#line 197 "encoding.c"
					c = (g_free (c), NULL);
#line 68 "encoding.vala"
					break;
#line 201 "encoding.c"
				}
				c = (g_free (c), NULL);
			}
		}
	}
	result = charset;
	return result;
}


/**
   * Retrun the content of a file encoding in UTF8
   *
   * @param path The name of file
   * @param encoding The encoding of the file
   *
   * @return The content of the file in UTF8
   */
#line 82 "encoding.vala"
char* valide_encoding_to_utf8 (const char* path, char** encoding, GError** error) {
#line 222 "encoding.c"
	char* result;
	GError * _inner_error_;
	char* text;
	char* contents;
	char* _tmp2_;
	gboolean _tmp1_;
	char* _tmp0_;
	char* _tmp3_;
#line 82 "encoding.vala"
	g_return_val_if_fail (path != NULL, NULL);
#line 233 "encoding.c"
	if (encoding != NULL) {
		*encoding = NULL;
	}
	_inner_error_ = NULL;
	text = NULL;
	contents = g_strdup ("");
	_tmp2_ = NULL;
	_tmp0_ = NULL;
#line 87 "encoding.vala"
	_tmp1_ = g_file_get_contents (path, &_tmp0_, NULL, &_inner_error_);
#line 87 "encoding.vala"
	text = (_tmp2_ = _tmp0_, text = (g_free (text), NULL), _tmp2_);
#line 87 "encoding.vala"
	_tmp1_;
#line 248 "encoding.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		text = (g_free (text), NULL);
		contents = (g_free (contents), NULL);
		return NULL;
	}
	_tmp3_ = NULL;
#line 88 "encoding.vala"
	(*encoding) = (_tmp3_ = valide_encoding_get_charset (text), (*encoding) = (g_free ((*encoding)), NULL), _tmp3_);
#line 89 "encoding.vala"
	if ((*encoding) != NULL) {
#line 260 "encoding.c"
		char* _tmp4_;
		char* _tmp5_;
		_tmp4_ = valide_encoding_convert (text, "UTF-8", (*encoding), &_inner_error_);
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			text = (g_free (text), NULL);
			contents = (g_free (contents), NULL);
			return NULL;
		}
		_tmp5_ = NULL;
#line 91 "encoding.vala"
		contents = (_tmp5_ = _tmp4_, contents = (g_free (contents), NULL), _tmp5_);
#line 273 "encoding.c"
	} else {
		_inner_error_ = g_error_new_literal (VALIDE_ENCODING_ERROR, VALIDE_ENCODING_ERROR_UNKNOW_ENCODING, "unknow encoding");
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			text = (g_free (text), NULL);
			contents = (g_free (contents), NULL);
			return NULL;
		}
	}
	result = contents;
	text = (g_free (text), NULL);
	return result;
}


/* @bug ?! 
*
   * Retrun the content of a Gtk.TextView encoding in UTF8
   *
   * @param text_buffer A Gtk.TextBuffer
   * @param encoding The encoding of the file
   *
   * @return The content of the Gtk.TextView in UTF8
   */
#line 109 "encoding.vala"
char* valide_encoding_from_utf8 (GtkTextBuffer* text_buffer, char** encoding, GError** error) {
#line 300 "encoding.c"
	char* result;
	GError * _inner_error_;
	char* text;
	GtkTextIter end = {0};
	GtkTextIter start = {0};
	char* convert;
	char* _tmp1_;
	const char* _tmp0_;
#line 109 "encoding.vala"
	g_return_val_if_fail (text_buffer != NULL, NULL);
#line 311 "encoding.c"
	if (encoding != NULL) {
		*encoding = NULL;
	}
	_inner_error_ = NULL;
	text = NULL;
	convert = NULL;
#line 116 "encoding.vala"
	gtk_text_buffer_get_bounds (text_buffer, &start, &end);
#line 320 "encoding.c"
	_tmp1_ = NULL;
#line 117 "encoding.vala"
	_tmp0_ = NULL;
#line 117 "encoding.vala"
	text = (_tmp1_ = (_tmp0_ = gtk_text_iter_get_text (&start, &end), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), text = (g_free (text), NULL), _tmp1_);
#line 118 "encoding.vala"
	if (text != NULL) {
#line 328 "encoding.c"
		char* _tmp2_;
		_tmp2_ = NULL;
#line 120 "encoding.vala"
		(*encoding) = (_tmp2_ = valide_encoding_get_charset (text), (*encoding) = (g_free ((*encoding)), NULL), _tmp2_);
#line 121 "encoding.vala"
		if ((*encoding) != NULL) {
#line 335 "encoding.c"
			char* _tmp3_;
			char* _tmp4_;
			_tmp3_ = valide_encoding_convert (text, (*encoding), "UTF-8", &_inner_error_);
			if (_inner_error_ != NULL) {
				g_propagate_error (error, _inner_error_);
				text = (g_free (text), NULL);
				convert = (g_free (convert), NULL);
				return NULL;
			}
			_tmp4_ = NULL;
#line 123 "encoding.vala"
			convert = (_tmp4_ = _tmp3_, convert = (g_free (convert), NULL), _tmp4_);
#line 348 "encoding.c"
		}
	}
	result = convert;
	text = (g_free (text), NULL);
	return result;
}


/**
 * A class for manager the file encoding
 */
#line 30 "encoding.vala"
ValideEncoding* valide_encoding_construct (GType object_type) {
#line 362 "encoding.c"
	ValideEncoding * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


#line 30 "encoding.vala"
ValideEncoding* valide_encoding_new (void) {
#line 30 "encoding.vala"
	return valide_encoding_construct (VALIDE_TYPE_ENCODING);
#line 373 "encoding.c"
}


static void valide_encoding_class_init (ValideEncodingClass * klass) {
	valide_encoding_parent_class = g_type_class_peek_parent (klass);
}


static void valide_encoding_instance_init (ValideEncoding * self) {
}


GType valide_encoding_get_type (void) {
	static GType valide_encoding_type_id = 0;
	if (valide_encoding_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideEncodingClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_encoding_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideEncoding), 0, (GInstanceInitFunc) valide_encoding_instance_init, NULL };
		valide_encoding_type_id = g_type_register_static (G_TYPE_OBJECT, "ValideEncoding", &g_define_type_info, 0);
	}
	return valide_encoding_type_id;
}




