/* executable-manager.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <gdk/gdk.h>
#include <glib/gi18n-lib.h>
#include <float.h>
#include <math.h>


#define VALIDE_TYPE_EXECUTABLE_MANAGER (valide_executable_manager_get_type ())
#define VALIDE_EXECUTABLE_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_EXECUTABLE_MANAGER, ValideExecutableManager))
#define VALIDE_EXECUTABLE_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_EXECUTABLE_MANAGER, ValideExecutableManagerClass))
#define VALIDE_IS_EXECUTABLE_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_EXECUTABLE_MANAGER))
#define VALIDE_IS_EXECUTABLE_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_EXECUTABLE_MANAGER))
#define VALIDE_EXECUTABLE_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_EXECUTABLE_MANAGER, ValideExecutableManagerClass))

typedef struct _ValideExecutableManager ValideExecutableManager;
typedef struct _ValideExecutableManagerClass ValideExecutableManagerClass;
typedef struct _ValideExecutableManagerPrivate ValideExecutableManagerPrivate;

#define VALIDE_TYPE_UI_MANAGER (valide_ui_manager_get_type ())
#define VALIDE_UI_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_UI_MANAGER, ValideUIManager))
#define VALIDE_UI_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_UI_MANAGER, ValideUIManagerClass))
#define VALIDE_IS_UI_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_UI_MANAGER))
#define VALIDE_IS_UI_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_UI_MANAGER))
#define VALIDE_UI_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_UI_MANAGER, ValideUIManagerClass))

typedef struct _ValideUIManager ValideUIManager;
typedef struct _ValideUIManagerClass ValideUIManagerClass;

#define VALIDE_TYPE_EXECUTABLE (valide_executable_get_type ())
#define VALIDE_EXECUTABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_EXECUTABLE, ValideExecutable))
#define VALIDE_EXECUTABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_EXECUTABLE, ValideExecutableClass))
#define VALIDE_IS_EXECUTABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_EXECUTABLE))
#define VALIDE_IS_EXECUTABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_EXECUTABLE))
#define VALIDE_EXECUTABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_EXECUTABLE, ValideExecutableClass))

typedef struct _ValideExecutable ValideExecutable;
typedef struct _ValideExecutableClass ValideExecutableClass;

#define VALIDE_EXECUTABLE_MANAGER_TYPE_COL (valide_executable_manager_col_get_type ())

#define VALIDE_TYPE_EXECUTABLE_OPTIONS (valide_executable_options_get_type ())
#define VALIDE_EXECUTABLE_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_EXECUTABLE_OPTIONS, ValideExecutableOptions))
#define VALIDE_EXECUTABLE_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_EXECUTABLE_OPTIONS, ValideExecutableOptionsClass))
#define VALIDE_IS_EXECUTABLE_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_EXECUTABLE_OPTIONS))
#define VALIDE_IS_EXECUTABLE_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_EXECUTABLE_OPTIONS))
#define VALIDE_EXECUTABLE_OPTIONS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_EXECUTABLE_OPTIONS, ValideExecutableOptionsClass))

typedef struct _ValideExecutableOptions ValideExecutableOptions;
typedef struct _ValideExecutableOptionsClass ValideExecutableOptionsClass;

typedef enum  {
	VALIDE_EXECUTABLE_ERROR_NO_EXECUTABLE
} ValideExecutableError;
#define VALIDE_EXECUTABLE_ERROR valide_executable_error_quark ()
/**
 * Manager for launch external programs
 */
struct _ValideExecutableManager {
	GtkHBox parent_instance;
	ValideExecutableManagerPrivate * priv;
};

struct _ValideExecutableManagerClass {
	GtkHBoxClass parent_class;
};

struct _ValideExecutableManagerPrivate {
	GPid pid;
	GTimer* timer;
	GtkTreeView* tree_view;
	GtkListStore* list_store;
	ValideUIManager* _ui_manager;
	ValideExecutable* _current;
	gboolean _running;
	char* _text;
};

typedef enum  {
	VALIDE_EXECUTABLE_MANAGER_COL_TEXT,
	VALIDE_EXECUTABLE_MANAGER_COL_COLOR,
	VALIDE_EXECUTABLE_MANAGER_COL_NB
} ValideExecutableManagerCol;


static gpointer valide_executable_manager_parent_class = NULL;

GQuark valide_executable_error_quark (void);
GType valide_executable_manager_get_type (void);
GType valide_ui_manager_get_type (void);
GType valide_executable_get_type (void);
#define VALIDE_EXECUTABLE_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_EXECUTABLE_MANAGER, ValideExecutableManagerPrivate))
enum  {
	VALIDE_EXECUTABLE_MANAGER_DUMMY_PROPERTY,
	VALIDE_EXECUTABLE_MANAGER_UI_MANAGER
};
static GType valide_executable_manager_col_get_type (void);
static gboolean valide_executable_manager_get_running (ValideExecutableManager* self);
ValideUIManager* valide_executable_manager_get_ui_manager (ValideExecutableManager* self);
GtkAction* valide_ui_manager_get_action (ValideUIManager* self, const char* name);
static void valide_executable_manager_setup_ui (ValideExecutableManager* self);
static void valide_executable_manager_clear (ValideExecutableManager* self);
static const char* valide_executable_manager_get_text (ValideExecutableManager* self);
static void valide_executable_manager_copy (ValideExecutableManager* self);
static void valide_executable_manager_previous_line (ValideExecutableManager* self);
static void valide_executable_manager_next_line (ValideExecutableManager* self);
static ValideExecutable* valide_executable_manager_get_current (ValideExecutableManager* self);
void valide_executable_line_activated (ValideExecutable* self, const char* line);
static void valide_executable_manager_row_activated_cb (ValideExecutableManager* self, GtkTreeView* sender, const GtkTreePath* path, GtkTreeViewColumn* column);
GdkColor valide_executable_get_color_line (ValideExecutable* self, char** line);
static void valide_executable_manager_add_line_markup (ValideExecutableManager* self, const char* line);
static void valide_executable_manager_add_line (ValideExecutableManager* self, const char* line);
static gboolean valide_executable_manager_build_iofunc (ValideExecutableManager* self, GIOChannel* ioc, GIOCondition cond);
static void valide_executable_manager_set_running (ValideExecutableManager* self, gboolean value);
char* valide_executable_cmd_end (ValideExecutable* self, gint status, double time);
static void valide_executable_manager_cmd_watch (ValideExecutableManager* self, GPid pid, gint status);
static void valide_executable_manager_set_ui_manager (ValideExecutableManager* self, ValideUIManager* value);
ValideExecutableManager* valide_executable_manager_new (ValideUIManager* ui_manager);
ValideExecutableManager* valide_executable_manager_construct (GType object_type, ValideUIManager* ui_manager);
GType valide_executable_options_get_type (void);
ValideExecutableOptions* valide_executable_options_new (void);
ValideExecutableOptions* valide_executable_options_construct (GType object_type);
static void valide_executable_manager_set_current (ValideExecutableManager* self, ValideExecutable* value);
const char* valide_executable_get_executable (ValideExecutable* self);
const char* valide_executable_options_get_arguments (ValideExecutableOptions* self);
char* valide_executable_cmd_start (ValideExecutable* self, const char* cmd);
const char* valide_executable_options_get_working_dir (ValideExecutableOptions* self);
char** valide_executable_options_get_environment_variables (ValideExecutableOptions* self, int* result_length1);
void valide_utils_set_up_io_channel (gint fd, GIOCondition cond, gboolean nblock, GIOFunc func, void* func_target, GError** error);
static gboolean _valide_executable_manager_build_iofunc_gio_func (GIOChannel* source, GIOCondition condition, gpointer self);
static void _valide_executable_manager_cmd_watch_gchild_watch_func (GPid pid, gint status, gpointer self);
void valide_executable_manager_run (ValideExecutableManager* self, ValideExecutable* executable, ValideExecutableOptions* options);
void valide_executable_manager_stop (ValideExecutableManager* self, GError** error);
static void _valide_executable_manager_clear_gtk_button_clicked (GtkButton* _sender, gpointer self);
static void _valide_executable_manager_copy_gtk_button_clicked (GtkButton* _sender, gpointer self);
static void _valide_executable_manager_previous_line_gtk_button_clicked (GtkButton* _sender, gpointer self);
static void _valide_executable_manager_next_line_gtk_button_clicked (GtkButton* _sender, gpointer self);
static void _valide_executable_manager_row_activated_cb_gtk_tree_view_row_activated (GtkTreeView* _sender, const GtkTreePath* path, GtkTreeViewColumn* column, gpointer self);
static void _valide_executable_manager_clear_g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self);
static GObject * valide_executable_manager_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void valide_executable_manager_finalize (GObject* obj);
static void valide_executable_manager_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_executable_manager_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);



GQuark valide_executable_error_quark (void) {
	return g_quark_from_static_string ("valide_executable_error-quark");
}



static GType valide_executable_manager_col_get_type (void) {
	static GType valide_executable_manager_col_type_id = 0;
	if (G_UNLIKELY (valide_executable_manager_col_type_id == 0)) {
		static const GEnumValue values[] = {{VALIDE_EXECUTABLE_MANAGER_COL_TEXT, "VALIDE_EXECUTABLE_MANAGER_COL_TEXT", "text"}, {VALIDE_EXECUTABLE_MANAGER_COL_COLOR, "VALIDE_EXECUTABLE_MANAGER_COL_COLOR", "color"}, {VALIDE_EXECUTABLE_MANAGER_COL_NB, "VALIDE_EXECUTABLE_MANAGER_COL_NB", "nb"}, {0, NULL, NULL}};
		valide_executable_manager_col_type_id = g_enum_register_static ("ValideExecutableManagerCol", values);
	}
	return valide_executable_manager_col_type_id;
}


#line 97 "executable-manager.vala"
static void valide_executable_manager_setup_ui (ValideExecutableManager* self) {
#line 194 "executable-manager.c"
	gboolean active;
	GtkAction* _tmp0_;
#line 97 "executable-manager.vala"
	g_return_if_fail (self != NULL);
#line 199 "executable-manager.c"
	active = FALSE;
#line 101 "executable-manager.vala"
	active = self->priv->_running;
#line 128 "uimanager.vala"
	_tmp0_ = NULL;
#line 102 "executable-manager.vala"
	gtk_action_set_sensitive (_tmp0_ = valide_ui_manager_get_action (self->priv->_ui_manager, "project-abort"), active);
#line 207 "executable-manager.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
}


#line 105 "executable-manager.vala"
static void valide_executable_manager_clear (ValideExecutableManager* self) {
#line 105 "executable-manager.vala"
	g_return_if_fail (self != NULL);
#line 107 "executable-manager.vala"
	gtk_list_store_clear (self->priv->list_store);
#line 218 "executable-manager.c"
}


#line 110 "executable-manager.vala"
static void valide_executable_manager_copy (ValideExecutableManager* self) {
#line 224 "executable-manager.c"
	char* str;
	GtkClipboard* clipboard;
	char* _tmp1_;
	const char* _tmp0_;
#line 110 "executable-manager.vala"
	g_return_if_fail (self != NULL);
#line 231 "executable-manager.c"
	str = NULL;
	clipboard = NULL;
#line 115 "executable-manager.vala"
	clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
#line 236 "executable-manager.c"
	_tmp1_ = NULL;
#line 116 "executable-manager.vala"
	_tmp0_ = NULL;
#line 116 "executable-manager.vala"
	str = (_tmp1_ = (_tmp0_ = valide_executable_manager_get_text (self), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), str = (g_free (str), NULL), _tmp1_);
#line 118 "executable-manager.vala"
	if (_vala_strcmp0 (str, "") != 0) {
#line 120 "executable-manager.vala"
		gtk_clipboard_set_text (clipboard, str, -1);
#line 246 "executable-manager.c"
	} else {
#line 124 "executable-manager.vala"
		g_warning (_ ("Nothing to copy!"));
#line 250 "executable-manager.c"
	}
	str = (g_free (str), NULL);
}


#line 128 "executable-manager.vala"
static void valide_executable_manager_previous_line (ValideExecutableManager* self) {
#line 258 "executable-manager.c"
	GtkTreeIter iter = {0};
	GtkTreePath* path;
	GtkTreeSelection* selection;
	GtkTreeSelection* _tmp1_;
	GtkTreeSelection* _tmp0_;
	gboolean _tmp2_;
	GtkTreePath* _tmp3_;
	GtkTreePath* _tmp4_;
#line 128 "executable-manager.vala"
	g_return_if_fail (self != NULL);
#line 269 "executable-manager.c"
	path = NULL;
	selection = NULL;
	_tmp1_ = NULL;
#line 134 "executable-manager.vala"
	_tmp0_ = NULL;
#line 134 "executable-manager.vala"
	selection = (_tmp1_ = (_tmp0_ = gtk_tree_view_get_selection (self->priv->tree_view), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL)), _tmp1_);
#line 277 "executable-manager.c"
	_tmp2_ = FALSE;
#line 135 "executable-manager.vala"
	if (!gtk_tree_selection_get_selected (selection, NULL, &iter)) {
#line 136 "executable-manager.vala"
		_tmp2_ = !gtk_tree_model_get_iter_first ((GtkTreeModel*) self->priv->list_store, &iter);
#line 283 "executable-manager.c"
	} else {
#line 135 "executable-manager.vala"
		_tmp2_ = FALSE;
#line 287 "executable-manager.c"
	}
#line 135 "executable-manager.vala"
	if (_tmp2_) {
#line 137 "executable-manager.vala"
		g_debug ("executable-manager.vala:137: the output line list is empty.");
#line 293 "executable-manager.c"
		(path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL));
		(selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL));
		return;
	}
	_tmp3_ = NULL;
#line 141 "executable-manager.vala"
	path = (_tmp3_ = gtk_tree_model_get_path ((GtkTreeModel*) self->priv->list_store, &iter), (path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL)), _tmp3_);
#line 142 "executable-manager.vala"
	if (gtk_tree_path_prev (path)) {
#line 144 "executable-manager.vala"
		gtk_tree_model_get_iter ((GtkTreeModel*) self->priv->list_store, &iter, path);
#line 305 "executable-manager.c"
	} else {
		gint children;
		children = 0;
#line 150 "executable-manager.vala"
		children = gtk_tree_model_iter_n_children ((GtkTreeModel*) self->priv->list_store, NULL);
#line 151 "executable-manager.vala"
		if (children > 0) {
#line 153 "executable-manager.vala"
			gtk_tree_model_iter_nth_child ((GtkTreeModel*) self->priv->list_store, &iter, NULL, children - 1);
#line 315 "executable-manager.c"
		}
	}
	_tmp4_ = NULL;
#line 157 "executable-manager.vala"
	path = (_tmp4_ = gtk_tree_model_get_path ((GtkTreeModel*) self->priv->list_store, &iter), (path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL)), _tmp4_);
#line 158 "executable-manager.vala"
	if (gtk_list_store_iter_is_valid (self->priv->list_store, &iter)) {
#line 323 "executable-manager.c"
		GtkTreeViewColumn* column;
		GtkTreeViewColumn* _tmp6_;
		GtkTreeViewColumn* _tmp5_;
		column = NULL;
#line 162 "executable-manager.vala"
		gtk_tree_selection_select_iter (selection, &iter);
#line 330 "executable-manager.c"
		_tmp6_ = NULL;
#line 163 "executable-manager.vala"
		_tmp5_ = NULL;
#line 163 "executable-manager.vala"
		column = (_tmp6_ = (_tmp5_ = gtk_tree_view_get_column (self->priv->tree_view, (gint) VALIDE_EXECUTABLE_MANAGER_COL_TEXT), (_tmp5_ == NULL) ? NULL : g_object_ref (_tmp5_)), (column == NULL) ? NULL : (column = (g_object_unref (column), NULL)), _tmp6_);
#line 164 "executable-manager.vala"
		gtk_tree_view_scroll_to_cell (self->priv->tree_view, path, NULL, FALSE, (float) 0, (float) 0);
#line 165 "executable-manager.vala"
		gtk_tree_view_row_activated (self->priv->tree_view, path, column);
#line 340 "executable-manager.c"
		(column == NULL) ? NULL : (column = (g_object_unref (column), NULL));
	}
	(path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL));
	(selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL));
}


#line 169 "executable-manager.vala"
static void valide_executable_manager_next_line (ValideExecutableManager* self) {
#line 350 "executable-manager.c"
	GtkTreeIter iter = {0};
	GtkTreePath* path;
	GtkTreeViewColumn* column;
	GtkTreeSelection* selection;
	GtkTreeSelection* _tmp1_;
	GtkTreeSelection* _tmp0_;
#line 169 "executable-manager.vala"
	g_return_if_fail (self != NULL);
#line 359 "executable-manager.c"
	path = NULL;
	column = NULL;
	selection = NULL;
	_tmp1_ = NULL;
#line 176 "executable-manager.vala"
	_tmp0_ = NULL;
#line 176 "executable-manager.vala"
	selection = (_tmp1_ = (_tmp0_ = gtk_tree_view_get_selection (self->priv->tree_view), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL)), _tmp1_);
#line 177 "executable-manager.vala"
	if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
#line 179 "executable-manager.vala"
		if (!gtk_tree_model_iter_next ((GtkTreeModel*) self->priv->list_store, &iter)) {
#line 181 "executable-manager.vala"
			gtk_tree_model_get_iter_first ((GtkTreeModel*) self->priv->list_store, &iter);
#line 374 "executable-manager.c"
		}
	} else {
#line 186 "executable-manager.vala"
		gtk_tree_model_get_iter_first ((GtkTreeModel*) self->priv->list_store, &iter);
#line 379 "executable-manager.c"
	}
#line 189 "executable-manager.vala"
	if (gtk_list_store_iter_is_valid (self->priv->list_store, &iter)) {
#line 383 "executable-manager.c"
		GtkTreePath* _tmp2_;
		GtkTreeViewColumn* _tmp4_;
		GtkTreeViewColumn* _tmp3_;
#line 191 "executable-manager.vala"
		gtk_tree_selection_select_iter (selection, &iter);
#line 389 "executable-manager.c"
		_tmp2_ = NULL;
#line 192 "executable-manager.vala"
		path = (_tmp2_ = gtk_tree_model_get_path ((GtkTreeModel*) self->priv->list_store, &iter), (path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL)), _tmp2_);
#line 393 "executable-manager.c"
		_tmp4_ = NULL;
#line 193 "executable-manager.vala"
		_tmp3_ = NULL;
#line 193 "executable-manager.vala"
		column = (_tmp4_ = (_tmp3_ = gtk_tree_view_get_column (self->priv->tree_view, (gint) VALIDE_EXECUTABLE_MANAGER_COL_TEXT), (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), (column == NULL) ? NULL : (column = (g_object_unref (column), NULL)), _tmp4_);
#line 194 "executable-manager.vala"
		gtk_tree_view_scroll_to_cell (self->priv->tree_view, path, NULL, FALSE, (float) 0, (float) 0);
#line 195 "executable-manager.vala"
		gtk_tree_view_row_activated (self->priv->tree_view, path, column);
#line 403 "executable-manager.c"
	}
	(path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL));
	(column == NULL) ? NULL : (column = (g_object_unref (column), NULL));
	(selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL));
}


#line 199 "executable-manager.vala"
static void valide_executable_manager_row_activated_cb (ValideExecutableManager* self, GtkTreeView* sender, const GtkTreePath* path, GtkTreeViewColumn* column) {
#line 413 "executable-manager.c"
	char* line;
	GtkTreeIter iter = {0};
#line 199 "executable-manager.vala"
	g_return_if_fail (self != NULL);
#line 199 "executable-manager.vala"
	g_return_if_fail (sender != NULL);
#line 199 "executable-manager.vala"
	g_return_if_fail (path != NULL);
#line 199 "executable-manager.vala"
	g_return_if_fail (column != NULL);
#line 424 "executable-manager.c"
	line = NULL;
#line 205 "executable-manager.vala"
	gtk_tree_model_get_iter (gtk_tree_view_get_model (sender), &iter, path);
#line 206 "executable-manager.vala"
	gtk_tree_model_get (gtk_tree_view_get_model (sender), &iter, VALIDE_EXECUTABLE_MANAGER_COL_TEXT, &line, -1);
#line 207 "executable-manager.vala"
	valide_executable_line_activated (self->priv->_current, line);
#line 432 "executable-manager.c"
	line = (g_free (line), NULL);
}


#line 210 "executable-manager.vala"
static void valide_executable_manager_add_line_markup (ValideExecutableManager* self, const char* line) {
#line 439 "executable-manager.c"
	GError * _inner_error_;
#line 210 "executable-manager.vala"
	g_return_if_fail (self != NULL);
#line 210 "executable-manager.vala"
	g_return_if_fail (line != NULL);
#line 445 "executable-manager.c"
	_inner_error_ = NULL;
	{
		GRegex* regex;
		char* line2;
		GRegex* _tmp0_;
		GRegex* _tmp1_;
		char* _tmp2_;
		char* _tmp3_;
		regex = NULL;
		line2 = NULL;
		_tmp0_ = g_regex_new ("[\n\r]*$", G_REGEX_OPTIMIZE, G_REGEX_MATCH_NOTEMPTY, &_inner_error_);
		if (_inner_error_ != NULL) {
			(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
			line2 = (g_free (line2), NULL);
			goto __catch61_g_error;
			goto __finally61;
		}
		_tmp1_ = NULL;
#line 217 "executable-manager.vala"
		regex = (_tmp1_ = _tmp0_, (regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL)), _tmp1_);
#line 466 "executable-manager.c"
		_tmp2_ = g_regex_replace (regex, line, (glong) (-1), 0, "", G_REGEX_MATCH_NOTEMPTY, &_inner_error_);
		if (_inner_error_ != NULL) {
			(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
			line2 = (g_free (line2), NULL);
			goto __catch61_g_error;
			goto __finally61;
		}
		_tmp3_ = NULL;
#line 221 "executable-manager.vala"
		line2 = (_tmp3_ = _tmp2_, line2 = (g_free (line2), NULL), _tmp3_);
#line 223 "executable-manager.vala"
		if (g_utf8_strlen (line2, -1) > 0) {
#line 479 "executable-manager.c"
			gint lines_size;
			gint lines_length1;
			char** lines;
			char** _tmp5_;
			char** _tmp4_;
			lines = (lines_length1 = 0, NULL);
			_tmp5_ = NULL;
			_tmp4_ = NULL;
#line 227 "executable-manager.vala"
			lines = (_tmp5_ = _tmp4_ = g_strsplit (line2, "\n", 0), lines = (_vala_array_free (lines, lines_length1, (GDestroyNotify) g_free), NULL), lines_length1 = _vala_array_length (_tmp4_), lines_size = lines_length1, _tmp5_);
#line 490 "executable-manager.c"
			{
				char** l_collection;
				int l_collection_length1;
				int l_it;
#line 228 "executable-manager.vala"
				l_collection = lines;
#line 497 "executable-manager.c"
				l_collection_length1 = lines_length1;
				for (l_it = 0; l_it < lines_length1; l_it = l_it + 1) {
					const char* _tmp7_;
					char* l;
#line 225 "executable-manager.vala"
					_tmp7_ = NULL;
#line 504 "executable-manager.c"
					l = (_tmp7_ = l_collection[l_it], (_tmp7_ == NULL) ? NULL : g_strdup (_tmp7_));
					{
						GdkColor color = {0};
						GtkTreeIter iter = {0};
						GtkTreePath* path;
						GtkTreePath* _tmp6_;
						path = NULL;
#line 234 "executable-manager.vala"
						color = valide_executable_get_color_line (self->priv->_current, &l);
#line 236 "executable-manager.vala"
						gtk_list_store_append (self->priv->list_store, &iter);
#line 237 "executable-manager.vala"
						gtk_list_store_set (self->priv->list_store, &iter, VALIDE_EXECUTABLE_MANAGER_COL_TEXT, l, VALIDE_EXECUTABLE_MANAGER_COL_COLOR, &color, -1);
#line 518 "executable-manager.c"
						_tmp6_ = NULL;
#line 239 "executable-manager.vala"
						path = (_tmp6_ = gtk_tree_model_get_path ((GtkTreeModel*) self->priv->list_store, &iter), (path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL)), _tmp6_);
#line 240 "executable-manager.vala"
						gtk_tree_view_scroll_to_cell (self->priv->tree_view, path, NULL, FALSE, (float) 0, (float) 0);
#line 524 "executable-manager.c"
						l = (g_free (l), NULL);
						(path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL));
					}
				}
			}
			lines = (_vala_array_free (lines, lines_length1, (GDestroyNotify) g_free), NULL);
		}
		(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
		line2 = (g_free (line2), NULL);
	}
	goto __finally61;
	__catch61_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 246 "executable-manager.vala"
			g_warning (e->message);
#line 544 "executable-manager.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally61:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
}


#line 942 "glib-2.0.vapi"
static char* string_replace (const char* self, const char* old, const char* replacement) {
#line 559 "executable-manager.c"
	char* result;
	GError * _inner_error_;
#line 942 "glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 942 "glib-2.0.vapi"
	g_return_val_if_fail (old != NULL, NULL);
#line 942 "glib-2.0.vapi"
	g_return_val_if_fail (replacement != NULL, NULL);
#line 568 "executable-manager.c"
	_inner_error_ = NULL;
	{
		char* _tmp0_;
		GRegex* _tmp1_;
		GRegex* regex;
		char* _tmp2_;
#line 2667 "glib-2.0.vapi"
		_tmp0_ = NULL;
#line 944 "glib-2.0.vapi"
		_tmp1_ = NULL;
#line 579 "executable-manager.c"
		regex = (_tmp1_ = g_regex_new (_tmp0_ = g_regex_escape_string (old, -1), 0, 0, &_inner_error_), _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_REGEX_ERROR) {
				goto __catch62_g_regex_error;
			}
			goto __finally62;
		}
		_tmp2_ = g_regex_replace_literal (regex, self, (glong) (-1), 0, replacement, 0, &_inner_error_);
		if (_inner_error_ != NULL) {
			(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
			if (_inner_error_->domain == G_REGEX_ERROR) {
				goto __catch62_g_regex_error;
			}
			goto __finally62;
		}
		result = _tmp2_;
		(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
		return result;
	}
	goto __finally62;
	__catch62_g_regex_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 947 "glib-2.0.vapi"
			g_assert_not_reached ();
#line 608 "executable-manager.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally62:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
}


#line 250 "executable-manager.vala"
static void valide_executable_manager_add_line (ValideExecutableManager* self, const char* line) {
#line 623 "executable-manager.c"
	char* line2;
	char* _tmp0_;
	char* _tmp1_;
#line 250 "executable-manager.vala"
	g_return_if_fail (self != NULL);
#line 250 "executable-manager.vala"
	g_return_if_fail (line != NULL);
#line 631 "executable-manager.c"
	line2 = NULL;
	_tmp0_ = NULL;
#line 254 "executable-manager.vala"
	line2 = (_tmp0_ = string_replace (line, "<", "&lt;"), line2 = (g_free (line2), NULL), _tmp0_);
#line 636 "executable-manager.c"
	_tmp1_ = NULL;
#line 255 "executable-manager.vala"
	line2 = (_tmp1_ = string_replace (line2, ">", "&gt;"), line2 = (g_free (line2), NULL), _tmp1_);
#line 256 "executable-manager.vala"
	valide_executable_manager_add_line_markup (self, line2);
#line 642 "executable-manager.c"
	line2 = (g_free (line2), NULL);
}


#line 259 "executable-manager.vala"
static gboolean valide_executable_manager_build_iofunc (ValideExecutableManager* self, GIOChannel* ioc, GIOCondition cond) {
#line 649 "executable-manager.c"
	gboolean result;
	GError * _inner_error_;
	gboolean ret;
#line 259 "executable-manager.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 259 "executable-manager.vala"
	g_return_val_if_fail (ioc != NULL, FALSE);
#line 657 "executable-manager.c"
	_inner_error_ = NULL;
	ret = FALSE;
#line 263 "executable-manager.vala"
	if ((cond & (G_IO_IN | G_IO_PRI)) != 0) {
#line 662 "executable-manager.c"
		char* msg;
		GIOStatus rv;
		msg = NULL;
		rv = 0;
		{
			char* _tmp2_;
			GIOStatus _tmp1_;
			char* _tmp0_;
			GIOStatus _tmp3_;
			gboolean _tmp4_;
			_tmp2_ = NULL;
			_tmp0_ = NULL;
			_tmp3_ = (_tmp1_ = g_io_channel_read_line (ioc, &_tmp0_, NULL, NULL, &_inner_error_), msg = (_tmp2_ = _tmp0_, msg = (g_free (msg), NULL), _tmp2_), _tmp1_);
			if (_inner_error_ != NULL) {
				goto __catch63_g_error;
				goto __finally63;
			}
#line 270 "executable-manager.vala"
			rv = _tmp3_;
#line 271 "executable-manager.vala"
			if (msg != NULL) {
#line 273 "executable-manager.vala"
				valide_executable_manager_add_line (self, msg);
#line 686 "executable-manager.c"
			}
			_tmp4_ = FALSE;
#line 275 "executable-manager.vala"
			if (rv == G_IO_STATUS_NORMAL) {
#line 275 "executable-manager.vala"
				_tmp4_ = TRUE;
#line 693 "executable-manager.c"
			} else {
#line 275 "executable-manager.vala"
				_tmp4_ = rv == G_IO_STATUS_AGAIN;
#line 697 "executable-manager.c"
			}
#line 275 "executable-manager.vala"
			if (_tmp4_) {
#line 277 "executable-manager.vala"
				ret = TRUE;
#line 703 "executable-manager.c"
			}
		}
		goto __finally63;
		__catch63_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
#line 282 "executable-manager.vala"
				g_warning (e->message);
#line 715 "executable-manager.c"
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally63:
		if (_inner_error_ != NULL) {
			msg = (g_free (msg), NULL);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return FALSE;
		}
		msg = (g_free (msg), NULL);
	}
	result = ret;
	return result;
}


#line 288 "executable-manager.vala"
static void valide_executable_manager_cmd_watch (ValideExecutableManager* self, GPid pid, gint status) {
#line 735 "executable-manager.c"
	char* _tmp0_;
#line 288 "executable-manager.vala"
	g_return_if_fail (self != NULL);
#line 290 "executable-manager.vala"
	valide_executable_manager_set_running (self, FALSE);
#line 291 "executable-manager.vala"
	g_timer_stop (self->priv->timer);
#line 424 "executable.vala"
	_tmp0_ = NULL;
#line 292 "executable-manager.vala"
	valide_executable_manager_add_line_markup (self, _tmp0_ = valide_executable_cmd_end (self->priv->_current, status, g_timer_elapsed (self->priv->timer, NULL)));
#line 747 "executable-manager.c"
	_tmp0_ = (g_free (_tmp0_), NULL);
#line 293 "executable-manager.vala"
	valide_executable_manager_setup_ui (self);
#line 751 "executable-manager.c"
}


/**
   * Create a new Valide.ExecutableManager
   *
   * @param ui_manager The ui manager
   */
#line 301 "executable-manager.vala"
ValideExecutableManager* valide_executable_manager_construct (GType object_type, ValideUIManager* ui_manager) {
#line 762 "executable-manager.c"
	GParameter * __params;
	GParameter * __params_it;
	ValideExecutableManager * self;
#line 301 "executable-manager.vala"
	g_return_val_if_fail (ui_manager != NULL, NULL);
#line 768 "executable-manager.c"
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
#line 303 "executable-manager.vala"
	__params_it->name = "ui-manager";
#line 303 "executable-manager.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_UI_MANAGER);
#line 303 "executable-manager.vala"
	g_value_set_object (&__params_it->value, ui_manager);
#line 303 "executable-manager.vala"
	__params_it++;
#line 779 "executable-manager.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 301 "executable-manager.vala"
ValideExecutableManager* valide_executable_manager_new (ValideUIManager* ui_manager) {
#line 301 "executable-manager.vala"
	return valide_executable_manager_construct (VALIDE_TYPE_EXECUTABLE_MANAGER, ui_manager);
#line 794 "executable-manager.c"
}


#line 259 "executable-manager.vala"
static gboolean _valide_executable_manager_build_iofunc_gio_func (GIOChannel* source, GIOCondition condition, gpointer self) {
#line 800 "executable-manager.c"
	return valide_executable_manager_build_iofunc (self, source, condition);
}


#line 288 "executable-manager.vala"
static void _valide_executable_manager_cmd_watch_gchild_watch_func (GPid pid, gint status, gpointer self) {
#line 807 "executable-manager.c"
	valide_executable_manager_cmd_watch (self, pid, status);
}


/**
   * Launch an executable
   *
   * @param executable An executable
   * @param options The executable optons
   */
#line 382 "executable-manager.vala"
void valide_executable_manager_run (ValideExecutableManager* self, ValideExecutable* executable, ValideExecutableOptions* options) {
#line 820 "executable-manager.c"
	GError * _inner_error_;
#line 382 "executable-manager.vala"
	g_return_if_fail (self != NULL);
#line 382 "executable-manager.vala"
	g_return_if_fail (executable != NULL);
#line 826 "executable-manager.c"
	_inner_error_ = NULL;
#line 384 "executable-manager.vala"
	if (!self->priv->_running) {
#line 830 "executable-manager.c"
		{
			char* cmd;
			gint std_out;
			gint std_err;
			gint args_size;
			gint args_length1;
			char** args;
			GIOCondition cond;
			ValideExecutableOptions* exec_options;
			char* _tmp3_;
			char* _tmp4_;
			gint _tmp6__length1;
			char** _tmp6_;
			gint _tmp5_;
			GtkAction* _tmp7_;
			cmd = NULL;
			std_out = 0;
			std_err = 0;
			args = (args_length1 = 0, NULL);
			cond = 0;
			exec_options = NULL;
#line 395 "executable-manager.vala"
			if (options == NULL) {
#line 854 "executable-manager.c"
				ValideExecutableOptions* _tmp0_;
				_tmp0_ = NULL;
#line 397 "executable-manager.vala"
				exec_options = (_tmp0_ = valide_executable_options_new (), (exec_options == NULL) ? NULL : (exec_options = (g_object_unref (exec_options), NULL)), _tmp0_);
#line 859 "executable-manager.c"
			} else {
				ValideExecutableOptions* _tmp2_;
				ValideExecutableOptions* _tmp1_;
				_tmp2_ = NULL;
#line 401 "executable-manager.vala"
				_tmp1_ = NULL;
#line 401 "executable-manager.vala"
				exec_options = (_tmp2_ = (_tmp1_ = options, (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (exec_options == NULL) ? NULL : (exec_options = (g_object_unref (exec_options), NULL)), _tmp2_);
#line 868 "executable-manager.c"
			}
#line 403 "executable-manager.vala"
			valide_executable_manager_set_current (self, executable);
#line 404 "executable-manager.vala"
			valide_executable_manager_set_running (self, TRUE);
#line 405 "executable-manager.vala"
			valide_executable_manager_clear (self);
#line 876 "executable-manager.c"
			_tmp3_ = NULL;
#line 407 "executable-manager.vala"
			cmd = (_tmp3_ = g_strdup_printf ("%s %s", valide_executable_get_executable (executable), valide_executable_options_get_arguments (exec_options)), cmd = (g_free (cmd), NULL), _tmp3_);
#line 408 "executable-manager.vala"
			g_shell_parse_argv (cmd, &args_length1, &args, &_inner_error_);
#line 882 "executable-manager.c"
			if (_inner_error_ != NULL) {
				cmd = (g_free (cmd), NULL);
				args = (_vala_array_free (args, args_length1, (GDestroyNotify) g_free), NULL);
				(exec_options == NULL) ? NULL : (exec_options = (g_object_unref (exec_options), NULL));
				goto __catch64_g_error;
				goto __finally64;
			}
#line 409 "executable-manager.vala"
			valide_executable_manager_setup_ui (self);
#line 407 "executable.vala"
			_tmp4_ = NULL;
#line 410 "executable-manager.vala"
			valide_executable_manager_add_line_markup (self, _tmp4_ = valide_executable_cmd_start (executable, cmd));
#line 896 "executable-manager.c"
			_tmp4_ = (g_free (_tmp4_), NULL);
#line 412 "executable-manager.vala"
			g_timer_start (self->priv->timer);
#line 103 "executable.vala"
			_tmp6_ = NULL;
#line 413 "executable-manager.vala"
			g_spawn_async_with_pipes (valide_executable_options_get_working_dir (exec_options), args, (_tmp6_ = valide_executable_options_get_environment_variables (exec_options, &_tmp5_), _tmp6__length1 = _tmp5_, _tmp6_), G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &self->priv->pid, NULL, &std_out, &std_err, &_inner_error_);
#line 904 "executable-manager.c"
			if (_inner_error_ != NULL) {
				cmd = (g_free (cmd), NULL);
				args = (_vala_array_free (args, args_length1, (GDestroyNotify) g_free), NULL);
				(exec_options == NULL) ? NULL : (exec_options = (g_object_unref (exec_options), NULL));
				goto __catch64_g_error;
				goto __finally64;
			}
			_tmp6_ = (_vala_array_free (_tmp6_, _tmp6__length1, (GDestroyNotify) g_free), NULL);
#line 128 "uimanager.vala"
			_tmp7_ = NULL;
#line 418 "executable-manager.vala"
			gtk_action_set_sensitive (_tmp7_ = valide_ui_manager_get_action (self->priv->_ui_manager, "project-abort"), TRUE);
#line 917 "executable-manager.c"
			(_tmp7_ == NULL) ? NULL : (_tmp7_ = (g_object_unref (_tmp7_), NULL));
#line 419 "executable-manager.vala"
			cond = (((G_IO_IN | G_IO_PRI) | G_IO_ERR) | G_IO_HUP) | G_IO_NVAL;
#line 421 "executable-manager.vala"
			valide_utils_set_up_io_channel (std_out, cond, TRUE, _valide_executable_manager_build_iofunc_gio_func, self, &_inner_error_);
#line 923 "executable-manager.c"
			if (_inner_error_ != NULL) {
				cmd = (g_free (cmd), NULL);
				args = (_vala_array_free (args, args_length1, (GDestroyNotify) g_free), NULL);
				(exec_options == NULL) ? NULL : (exec_options = (g_object_unref (exec_options), NULL));
				goto __catch64_g_error;
				goto __finally64;
			}
#line 422 "executable-manager.vala"
			valide_utils_set_up_io_channel (std_err, cond, TRUE, _valide_executable_manager_build_iofunc_gio_func, self, &_inner_error_);
#line 933 "executable-manager.c"
			if (_inner_error_ != NULL) {
				cmd = (g_free (cmd), NULL);
				args = (_vala_array_free (args, args_length1, (GDestroyNotify) g_free), NULL);
				(exec_options == NULL) ? NULL : (exec_options = (g_object_unref (exec_options), NULL));
				goto __catch64_g_error;
				goto __finally64;
			}
#line 423 "executable-manager.vala"
			g_child_watch_add (self->priv->pid, _valide_executable_manager_cmd_watch_gchild_watch_func, self);
#line 943 "executable-manager.c"
			cmd = (g_free (cmd), NULL);
			args = (_vala_array_free (args, args_length1, (GDestroyNotify) g_free), NULL);
			(exec_options == NULL) ? NULL : (exec_options = (g_object_unref (exec_options), NULL));
		}
		goto __finally64;
		__catch64_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
				char* _tmp8_;
#line 427 "executable-manager.vala"
				g_warning (e->message);
#line 428 "executable-manager.vala"
				valide_executable_manager_set_running (self, FALSE);
#line 424 "executable.vala"
				_tmp8_ = NULL;
#line 429 "executable-manager.vala"
				_tmp8_ = valide_executable_cmd_end (executable, -1, 0.0);
#line 964 "executable-manager.c"
				_tmp8_ = (g_free (_tmp8_), NULL);
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally64:
		if (_inner_error_ != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	} else {
#line 434 "executable-manager.vala"
		g_warning (_ ("Already running!"));
#line 978 "executable-manager.c"
	}
}


/**
   * Stop the current executable
   *
   * @throws ExecutableError If no executable running
   */
#line 443 "executable-manager.vala"
void valide_executable_manager_stop (ValideExecutableManager* self, GError** error) {
#line 990 "executable-manager.c"
	GError * _inner_error_;
#line 443 "executable-manager.vala"
	g_return_if_fail (self != NULL);
#line 994 "executable-manager.c"
	_inner_error_ = NULL;
#line 445 "executable-manager.vala"
	if (self->priv->_running) {
#line 447 "executable-manager.vala"
		pid_kill (self->priv->pid);
#line 1000 "executable-manager.c"
	} else {
		_inner_error_ = g_error_new_literal (VALIDE_EXECUTABLE_ERROR, VALIDE_EXECUTABLE_ERROR_NO_EXECUTABLE, _ ("No executable launched!"));
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == VALIDE_EXECUTABLE_ERROR) {
				g_propagate_error (error, _inner_error_);
				return;
			} else {
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return;
			}
		}
	}
}


ValideUIManager* valide_executable_manager_get_ui_manager (ValideExecutableManager* self) {
	ValideUIManager* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_ui_manager;
	return result;
}


static void valide_executable_manager_set_ui_manager (ValideExecutableManager* self, ValideUIManager* value) {
	ValideUIManager* _tmp1_;
	ValideUIManager* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 55 "executable-manager.vala"
	_tmp0_ = NULL;
#line 1032 "executable-manager.c"
	self->priv->_ui_manager = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_ui_manager == NULL) ? NULL : (self->priv->_ui_manager = (g_object_unref (self->priv->_ui_manager), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "ui-manager");
}


static ValideExecutable* valide_executable_manager_get_current (ValideExecutableManager* self) {
	ValideExecutable* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_current;
	return result;
}


static void valide_executable_manager_set_current (ValideExecutableManager* self, ValideExecutable* value) {
	ValideExecutable* _tmp1_;
	ValideExecutable* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 61 "executable-manager.vala"
	_tmp0_ = NULL;
#line 1053 "executable-manager.c"
	self->priv->_current = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_current == NULL) ? NULL : (self->priv->_current = (g_object_unref (self->priv->_current), NULL)), _tmp1_);
}


static gboolean valide_executable_manager_get_running (ValideExecutableManager* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = self->priv->_running;
	return result;
}


static void valide_executable_manager_set_running (ValideExecutableManager* self, gboolean value) {
	g_return_if_fail (self != NULL);
	self->priv->_running = value;
}


static const char* valide_executable_manager_get_text (ValideExecutableManager* self) {
	const char* result;
	char* _tmp0_;
	GString* sb;
	GtkTreeIter iter = {0};
	char* _tmp3_;
	const char* _tmp2_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = NULL;
#line 76 "executable-manager.vala"
	self->priv->_text = (_tmp0_ = NULL, self->priv->_text = (g_free (self->priv->_text), NULL), _tmp0_);
#line 1083 "executable-manager.c"
	sb = g_string_new ("");
#line 80 "executable-manager.vala"
	if (gtk_tree_model_get_iter_first ((GtkTreeModel*) self->priv->list_store, &iter) == TRUE) {
#line 1087 "executable-manager.c"
		char* val;
		val = NULL;
		{
			gboolean _tmp1_;
			_tmp1_ = TRUE;
#line 84 "executable-manager.vala"
			while (TRUE) {
#line 84 "executable-manager.vala"
				if (!_tmp1_) {
#line 89 "executable-manager.vala"
					if (!gtk_tree_model_iter_next ((GtkTreeModel*) self->priv->list_store, &iter)) {
#line 89 "executable-manager.vala"
						break;
#line 1101 "executable-manager.c"
					}
				}
#line 84 "executable-manager.vala"
				_tmp1_ = FALSE;
#line 86 "executable-manager.vala"
				gtk_tree_model_get ((GtkTreeModel*) self->priv->list_store, &iter, VALIDE_EXECUTABLE_MANAGER_COL_TEXT, &val, -1);
#line 87 "executable-manager.vala"
				g_string_append (sb, val);
#line 88 "executable-manager.vala"
				g_string_append_unichar (sb, (gunichar) '\n');
#line 1112 "executable-manager.c"
			}
		}
		val = (g_free (val), NULL);
	}
	_tmp3_ = NULL;
#line 92 "executable-manager.vala"
	_tmp2_ = NULL;
#line 92 "executable-manager.vala"
	self->priv->_text = (_tmp3_ = (_tmp2_ = sb->str, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)), self->priv->_text = (g_free (self->priv->_text), NULL), _tmp3_);
#line 1122 "executable-manager.c"
	result = self->priv->_text;
	(sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL));
	return result;
}


#line 105 "executable-manager.vala"
static void _valide_executable_manager_clear_gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 1131 "executable-manager.c"
	valide_executable_manager_clear (self);
}


#line 110 "executable-manager.vala"
static void _valide_executable_manager_copy_gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 1138 "executable-manager.c"
	valide_executable_manager_copy (self);
}


#line 128 "executable-manager.vala"
static void _valide_executable_manager_previous_line_gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 1145 "executable-manager.c"
	valide_executable_manager_previous_line (self);
}


#line 169 "executable-manager.vala"
static void _valide_executable_manager_next_line_gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 1152 "executable-manager.c"
	valide_executable_manager_next_line (self);
}


#line 199 "executable-manager.vala"
static void _valide_executable_manager_row_activated_cb_gtk_tree_view_row_activated (GtkTreeView* _sender, const GtkTreePath* path, GtkTreeViewColumn* column, gpointer self) {
#line 1159 "executable-manager.c"
	valide_executable_manager_row_activated_cb (self, _sender, path, column);
}


#line 105 "executable-manager.vala"
static void _valide_executable_manager_clear_g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self) {
#line 1166 "executable-manager.c"
	valide_executable_manager_clear (self);
}


static GObject * valide_executable_manager_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValideExecutableManagerClass * klass;
	GObjectClass * parent_class;
	ValideExecutableManager * self;
	klass = VALIDE_EXECUTABLE_MANAGER_CLASS (g_type_class_peek (VALIDE_TYPE_EXECUTABLE_MANAGER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_EXECUTABLE_MANAGER (obj);
	{
		GtkVBox* vbox;
		GtkButton* button;
		GtkHSeparator* separator;
		GtkTreeViewColumn* column;
		GtkCellRendererText* render;
		GtkScrolledWindow* scrolled_window;
		GTimer* _tmp0_;
		GtkVBox* _tmp1_;
		GtkButton* _tmp2_;
		GtkImage* _tmp3_;
		GtkButton* _tmp4_;
		GtkImage* _tmp5_;
		GtkHSeparator* _tmp6_;
		GtkButton* _tmp7_;
		GtkImage* _tmp8_;
		GtkButton* _tmp9_;
		GtkImage* _tmp10_;
		GtkScrolledWindow* _tmp11_;
		GtkListStore* _tmp12_;
		GtkTreeView* _tmp13_;
		GtkCellRendererText* _tmp14_;
		GtkTreeViewColumn* _tmp15_;
		vbox = NULL;
		button = NULL;
		separator = NULL;
		column = NULL;
		render = NULL;
		scrolled_window = NULL;
		_tmp0_ = NULL;
#line 315 "executable-manager.vala"
		self->priv->timer = (_tmp0_ = g_timer_new (), (self->priv->timer == NULL) ? NULL : (self->priv->timer = (g_timer_destroy (self->priv->timer), NULL)), _tmp0_);
#line 1212 "executable-manager.c"
		_tmp1_ = NULL;
#line 317 "executable-manager.vala"
		vbox = (_tmp1_ = g_object_ref_sink ((GtkVBox*) gtk_vbox_new (FALSE, 0)), (vbox == NULL) ? NULL : (vbox = (g_object_unref (vbox), NULL)), _tmp1_);
#line 318 "executable-manager.vala"
		gtk_box_pack_start ((GtkBox*) self, (GtkWidget*) vbox, FALSE, FALSE, (guint) 0);
#line 1218 "executable-manager.c"
		_tmp2_ = NULL;
#line 320 "executable-manager.vala"
		button = (_tmp2_ = g_object_ref_sink ((GtkButton*) gtk_button_new ()), (button == NULL) ? NULL : (button = (g_object_unref (button), NULL)), _tmp2_);
#line 321 "executable-manager.vala"
		gtk_button_set_relief (button, GTK_RELIEF_NONE);
#line 1224 "executable-manager.c"
		_tmp3_ = NULL;
#line 322 "executable-manager.vala"
		gtk_button_set_image (button, (GtkWidget*) (_tmp3_ = g_object_ref_sink ((GtkImage*) gtk_image_new_from_stock (GTK_STOCK_CLEAR, GTK_ICON_SIZE_SMALL_TOOLBAR))));
#line 1228 "executable-manager.c"
		(_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL));
#line 324 "executable-manager.vala"
		gtk_widget_set_tooltip_text ((GtkWidget*) button, _ ("Clear"));
#line 325 "executable-manager.vala"
		g_signal_connect_object (button, "clicked", (GCallback) _valide_executable_manager_clear_gtk_button_clicked, self, 0);
#line 326 "executable-manager.vala"
		gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) button, FALSE, FALSE, (guint) 0);
#line 1236 "executable-manager.c"
		_tmp4_ = NULL;
#line 328 "executable-manager.vala"
		button = (_tmp4_ = g_object_ref_sink ((GtkButton*) gtk_button_new ()), (button == NULL) ? NULL : (button = (g_object_unref (button), NULL)), _tmp4_);
#line 329 "executable-manager.vala"
		gtk_button_set_relief (button, GTK_RELIEF_NONE);
#line 1242 "executable-manager.c"
		_tmp5_ = NULL;
#line 330 "executable-manager.vala"
		gtk_button_set_image (button, (GtkWidget*) (_tmp5_ = g_object_ref_sink ((GtkImage*) gtk_image_new_from_stock (GTK_STOCK_COPY, GTK_ICON_SIZE_SMALL_TOOLBAR))));
#line 1246 "executable-manager.c"
		(_tmp5_ == NULL) ? NULL : (_tmp5_ = (g_object_unref (_tmp5_), NULL));
#line 332 "executable-manager.vala"
		gtk_widget_set_tooltip_text ((GtkWidget*) button, _ ("Copy"));
#line 333 "executable-manager.vala"
		g_signal_connect_object (button, "clicked", (GCallback) _valide_executable_manager_copy_gtk_button_clicked, self, 0);
#line 334 "executable-manager.vala"
		gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) button, FALSE, FALSE, (guint) 0);
#line 1254 "executable-manager.c"
		_tmp6_ = NULL;
#line 336 "executable-manager.vala"
		separator = (_tmp6_ = g_object_ref_sink ((GtkHSeparator*) gtk_hseparator_new ()), (separator == NULL) ? NULL : (separator = (g_object_unref (separator), NULL)), _tmp6_);
#line 337 "executable-manager.vala"
		gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) separator, FALSE, FALSE, (guint) 0);
#line 1260 "executable-manager.c"
		_tmp7_ = NULL;
#line 339 "executable-manager.vala"
		button = (_tmp7_ = g_object_ref_sink ((GtkButton*) gtk_button_new ()), (button == NULL) ? NULL : (button = (g_object_unref (button), NULL)), _tmp7_);
#line 340 "executable-manager.vala"
		gtk_button_set_relief (button, GTK_RELIEF_NONE);
#line 1266 "executable-manager.c"
		_tmp8_ = NULL;
#line 341 "executable-manager.vala"
		gtk_button_set_image (button, (GtkWidget*) (_tmp8_ = g_object_ref_sink ((GtkImage*) gtk_image_new_from_stock (GTK_STOCK_GO_UP, GTK_ICON_SIZE_SMALL_TOOLBAR))));
#line 1270 "executable-manager.c"
		(_tmp8_ == NULL) ? NULL : (_tmp8_ = (g_object_unref (_tmp8_), NULL));
#line 343 "executable-manager.vala"
		gtk_widget_set_tooltip_text ((GtkWidget*) button, _ ("Previous line"));
#line 344 "executable-manager.vala"
		g_signal_connect_object (button, "clicked", (GCallback) _valide_executable_manager_previous_line_gtk_button_clicked, self, 0);
#line 345 "executable-manager.vala"
		gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) button, FALSE, FALSE, (guint) 0);
#line 1278 "executable-manager.c"
		_tmp9_ = NULL;
#line 347 "executable-manager.vala"
		button = (_tmp9_ = g_object_ref_sink ((GtkButton*) gtk_button_new ()), (button == NULL) ? NULL : (button = (g_object_unref (button), NULL)), _tmp9_);
#line 348 "executable-manager.vala"
		gtk_button_set_relief (button, GTK_RELIEF_NONE);
#line 1284 "executable-manager.c"
		_tmp10_ = NULL;
#line 349 "executable-manager.vala"
		gtk_button_set_image (button, (GtkWidget*) (_tmp10_ = g_object_ref_sink ((GtkImage*) gtk_image_new_from_stock (GTK_STOCK_GO_DOWN, GTK_ICON_SIZE_SMALL_TOOLBAR))));
#line 1288 "executable-manager.c"
		(_tmp10_ == NULL) ? NULL : (_tmp10_ = (g_object_unref (_tmp10_), NULL));
#line 351 "executable-manager.vala"
		gtk_widget_set_tooltip_text ((GtkWidget*) button, _ ("Next line"));
#line 352 "executable-manager.vala"
		g_signal_connect_object (button, "clicked", (GCallback) _valide_executable_manager_next_line_gtk_button_clicked, self, 0);
#line 353 "executable-manager.vala"
		gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) button, FALSE, FALSE, (guint) 0);
#line 1296 "executable-manager.c"
		_tmp11_ = NULL;
#line 355 "executable-manager.vala"
		scrolled_window = (_tmp11_ = g_object_ref_sink ((GtkScrolledWindow*) gtk_scrolled_window_new (NULL, NULL)), (scrolled_window == NULL) ? NULL : (scrolled_window = (g_object_unref (scrolled_window), NULL)), _tmp11_);
#line 356 "executable-manager.vala"
		gtk_scrolled_window_set_policy (scrolled_window, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
#line 357 "executable-manager.vala"
		gtk_box_pack_start ((GtkBox*) self, (GtkWidget*) scrolled_window, TRUE, TRUE, (guint) 0);
#line 1304 "executable-manager.c"
		_tmp12_ = NULL;
#line 359 "executable-manager.vala"
		self->priv->list_store = (_tmp12_ = gtk_list_store_new ((gint) VALIDE_EXECUTABLE_MANAGER_COL_NB, G_TYPE_STRING, GDK_TYPE_COLOR, NULL), (self->priv->list_store == NULL) ? NULL : (self->priv->list_store = (g_object_unref (self->priv->list_store), NULL)), _tmp12_);
#line 1308 "executable-manager.c"
		_tmp13_ = NULL;
#line 360 "executable-manager.vala"
		self->priv->tree_view = (_tmp13_ = g_object_ref_sink ((GtkTreeView*) gtk_tree_view_new_with_model ((GtkTreeModel*) self->priv->list_store)), (self->priv->tree_view == NULL) ? NULL : (self->priv->tree_view = (g_object_unref (self->priv->tree_view), NULL)), _tmp13_);
#line 361 "executable-manager.vala"
		g_signal_connect_object (self->priv->tree_view, "row-activated", (GCallback) _valide_executable_manager_row_activated_cb_gtk_tree_view_row_activated, self, 0);
#line 362 "executable-manager.vala"
		gtk_container_add ((GtkContainer*) scrolled_window, (GtkWidget*) self->priv->tree_view);
#line 1316 "executable-manager.c"
		_tmp14_ = NULL;
#line 364 "executable-manager.vala"
		render = (_tmp14_ = g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (render == NULL) ? NULL : (render = (g_object_unref (render), NULL)), _tmp14_);
#line 1320 "executable-manager.c"
		_tmp15_ = NULL;
#line 365 "executable-manager.vala"
		column = (_tmp15_ = g_object_ref_sink (gtk_tree_view_column_new_with_attributes (_ ("Message"), (GtkCellRenderer*) render, "markup", VALIDE_EXECUTABLE_MANAGER_COL_TEXT, "foreground-gdk", VALIDE_EXECUTABLE_MANAGER_COL_COLOR, NULL)), (column == NULL) ? NULL : (column = (g_object_unref (column), NULL)), _tmp15_);
#line 368 "executable-manager.vala"
		gtk_tree_view_append_column (self->priv->tree_view, column);
#line 369 "executable-manager.vala"
		gtk_tree_view_set_headers_visible (self->priv->tree_view, FALSE);
#line 371 "executable-manager.vala"
		g_signal_connect_object ((GObject*) self, "notify::sensitive", (GCallback) _valide_executable_manager_clear_g_object_notify, self, 0);
#line 373 "executable-manager.vala"
		valide_executable_manager_setup_ui (self);
#line 1332 "executable-manager.c"
		(vbox == NULL) ? NULL : (vbox = (g_object_unref (vbox), NULL));
		(button == NULL) ? NULL : (button = (g_object_unref (button), NULL));
		(separator == NULL) ? NULL : (separator = (g_object_unref (separator), NULL));
		(column == NULL) ? NULL : (column = (g_object_unref (column), NULL));
		(render == NULL) ? NULL : (render = (g_object_unref (render), NULL));
		(scrolled_window == NULL) ? NULL : (scrolled_window = (g_object_unref (scrolled_window), NULL));
	}
	return obj;
}


static void valide_executable_manager_class_init (ValideExecutableManagerClass * klass) {
	valide_executable_manager_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideExecutableManagerPrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_executable_manager_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_executable_manager_set_property;
	G_OBJECT_CLASS (klass)->constructor = valide_executable_manager_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_executable_manager_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_EXECUTABLE_MANAGER_UI_MANAGER, g_param_spec_object ("ui-manager", "ui-manager", "ui-manager", VALIDE_TYPE_UI_MANAGER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_signal_new ("run_exec", VALIDE_TYPE_EXECUTABLE_MANAGER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, VALIDE_TYPE_EXECUTABLE);
}


static void valide_executable_manager_instance_init (ValideExecutableManager * self) {
	self->priv = VALIDE_EXECUTABLE_MANAGER_GET_PRIVATE (self);
	self->priv->_running = FALSE;
}


static void valide_executable_manager_finalize (GObject* obj) {
	ValideExecutableManager * self;
	self = VALIDE_EXECUTABLE_MANAGER (obj);
	(self->priv->timer == NULL) ? NULL : (self->priv->timer = (g_timer_destroy (self->priv->timer), NULL));
	(self->priv->tree_view == NULL) ? NULL : (self->priv->tree_view = (g_object_unref (self->priv->tree_view), NULL));
	(self->priv->list_store == NULL) ? NULL : (self->priv->list_store = (g_object_unref (self->priv->list_store), NULL));
	(self->priv->_ui_manager == NULL) ? NULL : (self->priv->_ui_manager = (g_object_unref (self->priv->_ui_manager), NULL));
	(self->priv->_current == NULL) ? NULL : (self->priv->_current = (g_object_unref (self->priv->_current), NULL));
	self->priv->_text = (g_free (self->priv->_text), NULL);
	G_OBJECT_CLASS (valide_executable_manager_parent_class)->finalize (obj);
}


GType valide_executable_manager_get_type (void) {
	static GType valide_executable_manager_type_id = 0;
	if (valide_executable_manager_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideExecutableManagerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_executable_manager_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideExecutableManager), 0, (GInstanceInitFunc) valide_executable_manager_instance_init, NULL };
		valide_executable_manager_type_id = g_type_register_static (GTK_TYPE_HBOX, "ValideExecutableManager", &g_define_type_info, 0);
	}
	return valide_executable_manager_type_id;
}


static void valide_executable_manager_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideExecutableManager * self;
	gpointer boxed;
	self = VALIDE_EXECUTABLE_MANAGER (object);
	switch (property_id) {
		case VALIDE_EXECUTABLE_MANAGER_UI_MANAGER:
		g_value_set_object (value, valide_executable_manager_get_ui_manager (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_executable_manager_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideExecutableManager * self;
	self = VALIDE_EXECUTABLE_MANAGER (object);
	switch (property_id) {
		case VALIDE_EXECUTABLE_MANAGER_UI_MANAGER:
		valide_executable_manager_set_ui_manager (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




