/* executable.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <float.h>
#include <math.h>
#include <glib/gi18n-lib.h>
#include <gdk/gdk.h>


#define G_YAML_TYPE_BUILDABLE (g_yaml_buildable_get_type ())
#define G_YAML_BUILDABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_TYPE_BUILDABLE, GYAMLBuildable))
#define G_YAML_IS_BUILDABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_TYPE_BUILDABLE))
#define G_YAML_BUILDABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), G_YAML_TYPE_BUILDABLE, GYAMLBuildableIface))

typedef struct _GYAMLBuildable GYAMLBuildable;
typedef struct _GYAMLBuildableIface GYAMLBuildableIface;

#define G_YAML_TYPE_BUILDER (g_yaml_builder_get_type ())
#define G_YAML_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_TYPE_BUILDER, GYAMLBuilder))
#define G_YAML_BUILDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), G_YAML_TYPE_BUILDER, GYAMLBuilderClass))
#define G_YAML_IS_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_TYPE_BUILDER))
#define G_YAML_IS_BUILDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_YAML_TYPE_BUILDER))
#define G_YAML_BUILDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), G_YAML_TYPE_BUILDER, GYAMLBuilderClass))

typedef struct _GYAMLBuilder GYAMLBuilder;
typedef struct _GYAMLBuilderClass GYAMLBuilderClass;

#define G_YAML_TYPE_NODE (g_yaml_node_get_type ())
#define G_YAML_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_TYPE_NODE, GYAMLNode))
#define G_YAML_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), G_YAML_TYPE_NODE, GYAMLNodeClass))
#define G_YAML_IS_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_TYPE_NODE))
#define G_YAML_IS_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_YAML_TYPE_NODE))
#define G_YAML_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), G_YAML_TYPE_NODE, GYAMLNodeClass))

typedef struct _GYAMLNode GYAMLNode;
typedef struct _GYAMLNodeClass GYAMLNodeClass;

#define VALIDE_TYPE_ENVIRONMENT_VARIABLE (valide_environment_variable_get_type ())
#define VALIDE_ENVIRONMENT_VARIABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ENVIRONMENT_VARIABLE, ValideEnvironmentVariable))
#define VALIDE_ENVIRONMENT_VARIABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ENVIRONMENT_VARIABLE, ValideEnvironmentVariableClass))
#define VALIDE_IS_ENVIRONMENT_VARIABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ENVIRONMENT_VARIABLE))
#define VALIDE_IS_ENVIRONMENT_VARIABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ENVIRONMENT_VARIABLE))
#define VALIDE_ENVIRONMENT_VARIABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ENVIRONMENT_VARIABLE, ValideEnvironmentVariableClass))

typedef struct _ValideEnvironmentVariable ValideEnvironmentVariable;
typedef struct _ValideEnvironmentVariableClass ValideEnvironmentVariableClass;
typedef struct _ValideEnvironmentVariablePrivate ValideEnvironmentVariablePrivate;

#define VALIDE_TYPE_EXECUTABLE_OPTIONS (valide_executable_options_get_type ())
#define VALIDE_EXECUTABLE_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_EXECUTABLE_OPTIONS, ValideExecutableOptions))
#define VALIDE_EXECUTABLE_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_EXECUTABLE_OPTIONS, ValideExecutableOptionsClass))
#define VALIDE_IS_EXECUTABLE_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_EXECUTABLE_OPTIONS))
#define VALIDE_IS_EXECUTABLE_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_EXECUTABLE_OPTIONS))
#define VALIDE_EXECUTABLE_OPTIONS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_EXECUTABLE_OPTIONS, ValideExecutableOptionsClass))

typedef struct _ValideExecutableOptions ValideExecutableOptions;
typedef struct _ValideExecutableOptionsClass ValideExecutableOptionsClass;
typedef struct _ValideExecutableOptionsPrivate ValideExecutableOptionsPrivate;

#define VALIDE_TYPE_ABSTRACT_EXECUTABLE_PREFERENCES (valide_abstract_executable_preferences_get_type ())
#define VALIDE_ABSTRACT_EXECUTABLE_PREFERENCES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_EXECUTABLE_PREFERENCES, ValideAbstractExecutablePreferences))
#define VALIDE_ABSTRACT_EXECUTABLE_PREFERENCES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_EXECUTABLE_PREFERENCES, ValideAbstractExecutablePreferencesClass))
#define VALIDE_IS_ABSTRACT_EXECUTABLE_PREFERENCES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_EXECUTABLE_PREFERENCES))
#define VALIDE_IS_ABSTRACT_EXECUTABLE_PREFERENCES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_EXECUTABLE_PREFERENCES))
#define VALIDE_ABSTRACT_EXECUTABLE_PREFERENCES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_EXECUTABLE_PREFERENCES, ValideAbstractExecutablePreferencesClass))

typedef struct _ValideAbstractExecutablePreferences ValideAbstractExecutablePreferences;
typedef struct _ValideAbstractExecutablePreferencesClass ValideAbstractExecutablePreferencesClass;
typedef struct _ValideAbstractExecutablePreferencesPrivate ValideAbstractExecutablePreferencesPrivate;

#define VALIDE_ABSTRACT_EXECUTABLE_PREFERENCES_TYPE_WINDOW_WIDGETS (valide_abstract_executable_preferences_window_widgets_get_type ())
#define VALIDE_ABSTRACT_EXECUTABLE_PREFERENCES_WINDOW_WIDGETS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_ABSTRACT_EXECUTABLE_PREFERENCES_TYPE_WINDOW_WIDGETS, ValideAbstractExecutablePreferenceswindow_widgets))
#define VALIDE_ABSTRACT_EXECUTABLE_PREFERENCES_WINDOW_WIDGETS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_ABSTRACT_EXECUTABLE_PREFERENCES_TYPE_WINDOW_WIDGETS, ValideAbstractExecutablePreferenceswindow_widgetsClass))
#define VALIDE_ABSTRACT_EXECUTABLE_PREFERENCES_IS_WINDOW_WIDGETS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_ABSTRACT_EXECUTABLE_PREFERENCES_TYPE_WINDOW_WIDGETS))
#define VALIDE_ABSTRACT_EXECUTABLE_PREFERENCES_IS_WINDOW_WIDGETS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_ABSTRACT_EXECUTABLE_PREFERENCES_TYPE_WINDOW_WIDGETS))
#define VALIDE_ABSTRACT_EXECUTABLE_PREFERENCES_WINDOW_WIDGETS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_ABSTRACT_EXECUTABLE_PREFERENCES_TYPE_WINDOW_WIDGETS, ValideAbstractExecutablePreferenceswindow_widgetsClass))

typedef struct _ValideAbstractExecutablePreferenceswindow_widgets ValideAbstractExecutablePreferenceswindow_widgets;
typedef struct _ValideAbstractExecutablePreferenceswindow_widgetsClass ValideAbstractExecutablePreferenceswindow_widgetsClass;

#define VALIDE_TYPE_EXECUTABLE_PREFERENCES (valide_executable_preferences_get_type ())
#define VALIDE_EXECUTABLE_PREFERENCES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_EXECUTABLE_PREFERENCES, ValideExecutablePreferences))
#define VALIDE_EXECUTABLE_PREFERENCES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_EXECUTABLE_PREFERENCES, ValideExecutablePreferencesClass))
#define VALIDE_IS_EXECUTABLE_PREFERENCES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_EXECUTABLE_PREFERENCES))
#define VALIDE_IS_EXECUTABLE_PREFERENCES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_EXECUTABLE_PREFERENCES))
#define VALIDE_EXECUTABLE_PREFERENCES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_EXECUTABLE_PREFERENCES, ValideExecutablePreferencesClass))

typedef struct _ValideExecutablePreferences ValideExecutablePreferences;
typedef struct _ValideExecutablePreferencesClass ValideExecutablePreferencesClass;
typedef struct _ValideExecutablePreferencesPrivate ValideExecutablePreferencesPrivate;

#define VALIDE_TYPE_PROJECT (valide_project_get_type ())
#define VALIDE_PROJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PROJECT, ValideProject))
#define VALIDE_PROJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PROJECT, ValideProjectClass))
#define VALIDE_IS_PROJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PROJECT))
#define VALIDE_IS_PROJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PROJECT))
#define VALIDE_PROJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PROJECT, ValideProjectClass))

typedef struct _ValideProject ValideProject;
typedef struct _ValideProjectClass ValideProjectClass;

#define VALIDE_EXECUTABLE_PREFERENCES_TYPE_COL (valide_executable_preferences_col_get_type ())
typedef struct _ValideAbstractExecutablePreferenceswindow_widgetsPrivate ValideAbstractExecutablePreferenceswindow_widgetsPrivate;

#define VALIDE_TYPE_EXECUTABLE (valide_executable_get_type ())
#define VALIDE_EXECUTABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_EXECUTABLE, ValideExecutable))
#define VALIDE_EXECUTABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_EXECUTABLE, ValideExecutableClass))
#define VALIDE_IS_EXECUTABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_EXECUTABLE))
#define VALIDE_IS_EXECUTABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_EXECUTABLE))
#define VALIDE_EXECUTABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_EXECUTABLE, ValideExecutableClass))

typedef struct _ValideExecutable ValideExecutable;
typedef struct _ValideExecutableClass ValideExecutableClass;
typedef struct _ValideExecutablePrivate ValideExecutablePrivate;

/**
 * Buildable GObjects, from YAML.
 *
 *
 * Objects that implements GLib.YAML.Buildable is buildable by
 * GLib.YAML.Builder.
 *
 * This interface is almost the same as GtkBuildable.
 *
 **/
struct _GYAMLBuildableIface {
	GTypeInterface parent_iface;
	const char* (*get_name) (GYAMLBuildable* self);
	void (*set_name) (GYAMLBuildable* self, const char* name);
	void (*add_child) (GYAMLBuildable* self, GYAMLBuilder* builder, GObject* child, const char* type, GError** error);
	char** (*get_child_tags) (GYAMLBuildable* self, int* result_length1);
	GType* (*get_child_types) (GYAMLBuildable* self, int* result_length1);
	GList* (*get_children) (GYAMLBuildable* self, const char* type);
	GObject* (*get_internal_child) (GYAMLBuildable* self, GYAMLBuilder* builder, const char* child_name);
	void (*custom_node) (GYAMLBuildable* self, GYAMLBuilder* builder, const char* tag, GYAMLNode* node, GError** error);
};

struct _ValideEnvironmentVariable {
	GObject parent_instance;
	ValideEnvironmentVariablePrivate * priv;
};

struct _ValideEnvironmentVariableClass {
	GObjectClass parent_class;
};

struct _ValideEnvironmentVariablePrivate {
	char* _name;
	char* _value;
};

struct _ValideExecutableOptions {
	GObject parent_instance;
	ValideExecutableOptionsPrivate * priv;
	GList* environment;
};

struct _ValideExecutableOptionsClass {
	GObjectClass parent_class;
};

struct _ValideExecutableOptionsPrivate {
	char* _program;
	char* _arguments;
	char* _working_dir;
};

struct _ValideAbstractExecutablePreferences {
	GtkObject parent_instance;
	ValideAbstractExecutablePreferencesPrivate * priv;
	char* name_space;
	GtkBuilder* xml;
	ValideAbstractExecutablePreferenceswindow_widgets* widgets;
};

struct _ValideAbstractExecutablePreferencesClass {
	GtkObjectClass parent_class;
};

struct _ValideExecutablePreferences {
	ValideAbstractExecutablePreferences parent_instance;
	ValideExecutablePreferencesPrivate * priv;
};

struct _ValideExecutablePreferencesClass {
	ValideAbstractExecutablePreferencesClass parent_class;
};

struct _ValideExecutablePreferencesPrivate {
	ValideExecutableOptions* options;
	GtkListStore* list_store;
	ValideProject* _project;
};

typedef enum  {
	VALIDE_EXECUTABLE_PREFERENCES_COL_NAME,
	VALIDE_EXECUTABLE_PREFERENCES_COL_VALUE,
	VALIDE_EXECUTABLE_PREFERENCES_COL_NB
} ValideExecutablePreferencesCol;

struct _ValideAbstractExecutablePreferenceswindow_widgets {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValideAbstractExecutablePreferenceswindow_widgetsPrivate * priv;
	GtkTable* table;
	GtkLabel* label2;
	GtkEntry* executable_arguments;
	GtkLabel* label3;
	GtkEntry* executable_program;
	GtkButton* button2;
	GtkLabel* label4;
	GtkFileChooserButton* executable_working_dir;
	GtkExpander* expander1;
	GtkVBox* vbox1;
	GtkScrolledWindow* scrolledwindow1;
	GtkTreeView* tree_view;
	GtkHButtonBox* hbuttonbox1;
	GtkButton* button1;
	GtkButton* button3;
	GtkButton* button4;
	GtkLabel* label5;
};

struct _ValideAbstractExecutablePreferenceswindow_widgetsClass {
	GTypeClass parent_class;
	void (*finalize) (ValideAbstractExecutablePreferenceswindow_widgets *self);
};

/**
 * A class representing an program can launch by #ExecutableManager.
 */
struct _ValideExecutable {
	GObject parent_instance;
	ValideExecutablePrivate * priv;
};

struct _ValideExecutableClass {
	GObjectClass parent_class;
	char* (*get_start_msg) (ValideExecutable* self);
	char* (*get_end_msg) (ValideExecutable* self);
	GdkColor (*get_color_line) (ValideExecutable* self, char** line);
	void (*line_activated) (ValideExecutable* self, const char* line);
	char* (*get_executable) (ValideExecutable* self);
	void (*set_executable) (ValideExecutable* self, char* value);
};

struct _ValideExecutablePrivate {
	char* _executable;
};


static gpointer valide_environment_variable_parent_class = NULL;
static GYAMLBuildableIface* valide_environment_variable_g_yaml_buildable_parent_iface = NULL;
static GType* valide_executable_options_child_types;
static gint valide_executable_options_child_types_length1;
static GType* valide_executable_options_child_types = NULL;
static gint valide_executable_options_child_types_length1 = 0;
static gint valide_executable_options_child_types_size = 0;
static gpointer valide_executable_options_parent_class = NULL;
static GYAMLBuildableIface* valide_executable_options_g_yaml_buildable_parent_iface = NULL;
static gpointer valide_executable_preferences_parent_class = NULL;
static ValideExecutablePreferences* valide_executable_preferences_widget;
static ValideExecutablePreferences* valide_executable_preferences_widget = NULL;
static gpointer valide_executable_parent_class = NULL;

GType g_yaml_builder_get_type (void);
gpointer g_yaml_node_ref (gpointer instance);
void g_yaml_node_unref (gpointer instance);
GParamSpec* g_yaml_param_spec_node (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void g_yaml_value_set_node (GValue* value, gpointer v_object);
gpointer g_yaml_value_get_node (const GValue* value);
GType g_yaml_node_get_type (void);
GType g_yaml_buildable_get_type (void);
GType valide_environment_variable_get_type (void);
#define VALIDE_ENVIRONMENT_VARIABLE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_ENVIRONMENT_VARIABLE, ValideEnvironmentVariablePrivate))
enum  {
	VALIDE_ENVIRONMENT_VARIABLE_DUMMY_PROPERTY,
	VALIDE_ENVIRONMENT_VARIABLE_NAME,
	VALIDE_ENVIRONMENT_VARIABLE_VALUE
};
void valide_environment_variable_set_name (ValideEnvironmentVariable* self, const char* value);
void valide_environment_variable_set_value (ValideEnvironmentVariable* self, const char* value);
ValideEnvironmentVariable* valide_environment_variable_new (const char* name, const char* value);
ValideEnvironmentVariable* valide_environment_variable_construct (GType object_type, const char* name, const char* value);
const char* valide_environment_variable_get_name (ValideEnvironmentVariable* self);
const char* valide_environment_variable_get_value (ValideEnvironmentVariable* self);
char* valide_environment_variable_to_string (ValideEnvironmentVariable* self);
static void valide_environment_variable_finalize (GObject* obj);
static void valide_environment_variable_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_environment_variable_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType valide_executable_options_get_type (void);
#define VALIDE_EXECUTABLE_OPTIONS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_EXECUTABLE_OPTIONS, ValideExecutableOptionsPrivate))
enum  {
	VALIDE_EXECUTABLE_OPTIONS_DUMMY_PROPERTY,
	VALIDE_EXECUTABLE_OPTIONS_PROGRAM,
	VALIDE_EXECUTABLE_OPTIONS_ARGUMENTS,
	VALIDE_EXECUTABLE_OPTIONS_WORKING_DIR
};
static void _g_list_free_g_object_unref (GList* self);
static void valide_executable_options_real_add_child (GYAMLBuildable* base, GYAMLBuilder* builder, GObject* child, const char* type, GError** error);
static GType* valide_executable_options_real_get_child_types (GYAMLBuildable* base, int* result_length1);
static char** valide_executable_options_real_get_child_tags (GYAMLBuildable* base, int* result_length1);
static GList* valide_executable_options_real_get_children (GYAMLBuildable* base, const char* type);
char** valide_executable_options_get_environment_variables (ValideExecutableOptions* self, int* result_length1);
ValideExecutableOptions* valide_executable_options_new (void);
ValideExecutableOptions* valide_executable_options_construct (GType object_type);
const char* valide_executable_options_get_program (ValideExecutableOptions* self);
void valide_executable_options_set_program (ValideExecutableOptions* self, const char* value);
const char* valide_executable_options_get_arguments (ValideExecutableOptions* self);
void valide_executable_options_set_arguments (ValideExecutableOptions* self, const char* value);
const char* valide_executable_options_get_working_dir (ValideExecutableOptions* self);
void valide_executable_options_set_working_dir (ValideExecutableOptions* self, const char* value);
static GObject * valide_executable_options_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void valide_executable_options_finalize (GObject* obj);
static void valide_executable_options_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_executable_options_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType valide_abstract_executable_preferences_get_type (void);
gpointer valide_abstract_executable_preferences_window_widgets_ref (gpointer instance);
void valide_abstract_executable_preferences_window_widgets_unref (gpointer instance);
GParamSpec* valide_abstract_executable_preferences_param_spec_window_widgets (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void valide_abstract_executable_preferences_value_set_window_widgets (GValue* value, gpointer v_object);
gpointer valide_abstract_executable_preferences_value_get_window_widgets (const GValue* value);
GType valide_abstract_executable_preferences_window_widgets_get_type (void);
GType valide_executable_preferences_get_type (void);
GType valide_project_get_type (void);
#define VALIDE_EXECUTABLE_PREFERENCES_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_EXECUTABLE_PREFERENCES, ValideExecutablePreferencesPrivate))
enum  {
	VALIDE_EXECUTABLE_PREFERENCES_DUMMY_PROPERTY,
	VALIDE_EXECUTABLE_PREFERENCES_PROJECT
};
static GType valide_executable_preferences_col_get_type (void);
ValideProject* valide_executable_preferences_get_project (ValideExecutablePreferences* self);
void valide_project_save (ValideProject* self);
static void valide_executable_preferences_update_environment (ValideExecutablePreferences* self);
static void valide_executable_preferences_on_name_edit (ValideExecutablePreferences* self, GtkCellRenderer* sender, const char* path, const char* new_text);
static void valide_executable_preferences_on_value_edit (ValideExecutablePreferences* self, GtkCellRenderer* sender, const char* path, const char* new_text);
void valide_executable_preferences_add_variable (GtkButton* sender, ValideExecutablePreferences* self);
void valide_executable_preferences_modify_variable (GtkButton* sender, ValideExecutablePreferences* self);
void valide_executable_preferences_delete_variable (GtkButton* sender, ValideExecutablePreferences* self);
void valide_executable_preferences_on_program_changed (GtkButton* sender, ValideExecutablePreferences* self);
static void valide_executable_preferences_set_project (ValideExecutablePreferences* self, ValideProject* value);
ValideExecutablePreferences* valide_executable_preferences_new (ValideProject* project);
ValideExecutablePreferences* valide_executable_preferences_construct (GType object_type, ValideProject* project);
ValideExecutableOptions* valide_project_get_executable_options (ValideProject* self);
static void _lambda82_ (GtkEditable* s, ValideExecutablePreferences* self);
static void __lambda82__gtk_editable_changed (GtkEditable* _sender, gpointer self);
static void _lambda83_ (GtkEditable* s, ValideExecutablePreferences* self);
static void __lambda83__gtk_editable_changed (GtkEditable* _sender, gpointer self);
static void _lambda84_ (GtkFileChooser* s, ValideExecutablePreferences* self);
static void __lambda84__gtk_file_chooser_current_folder_changed (GtkFileChooser* _sender, gpointer self);
static void _valide_executable_preferences_on_name_edit_gtk_cell_renderer_text_edited (GtkCellRendererText* _sender, const char* path, const char* new_text, gpointer self);
static void _valide_executable_preferences_on_value_edit_gtk_cell_renderer_text_edited (GtkCellRendererText* _sender, const char* path, const char* new_text, gpointer self);
void valide_abstract_executable_preferences_connect_signals (ValideAbstractExecutablePreferences* self, const char* name_space);
static GObject * valide_executable_preferences_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void valide_executable_preferences_finalize (GObject* obj);
static void valide_executable_preferences_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_executable_preferences_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType valide_executable_get_type (void);
#define VALIDE_EXECUTABLE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_EXECUTABLE, ValideExecutablePrivate))
enum  {
	VALIDE_EXECUTABLE_DUMMY_PROPERTY,
	VALIDE_EXECUTABLE_EXECUTABLE
};
char* valide_executable_get_start_msg (ValideExecutable* self);
static char* valide_executable_real_get_start_msg (ValideExecutable* self);
char* valide_executable_get_end_msg (ValideExecutable* self);
static char* valide_executable_real_get_end_msg (ValideExecutable* self);
#define VALIDE_UTILS_COLOR_DEFAULT "#000000"
GdkColor valide_executable_get_color_line (ValideExecutable* self, char** line);
static GdkColor valide_executable_real_get_color_line (ValideExecutable* self, char** line);
void valide_executable_line_activated (ValideExecutable* self, const char* line);
static void valide_executable_real_line_activated (ValideExecutable* self, const char* line);
char* valide_executable_cmd_start (ValideExecutable* self, const char* cmd);
char* valide_executable_cmd_end (ValideExecutable* self, gint status, double time);
GtkWidget* valide_executable_preferences (ValideProject* project);
ValideExecutable* valide_executable_new (void);
ValideExecutable* valide_executable_construct (GType object_type);
const char* valide_executable_get_executable (ValideExecutable* self);
void valide_executable_set_executable (ValideExecutable* self, const char* value);
static void valide_executable_finalize (GObject* obj);
static void valide_executable_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_executable_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);

static const char* VALIDE_EXECUTABLE_OPTIONS_child_tags[] = {"environment"};


#line 27 "executable.vala"
ValideEnvironmentVariable* valide_environment_variable_construct (GType object_type, const char* name, const char* value) {
#line 417 "executable.c"
	ValideEnvironmentVariable * self;
#line 27 "executable.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 27 "executable.vala"
	g_return_val_if_fail (value != NULL, NULL);
#line 423 "executable.c"
	self = g_object_newv (object_type, 0, NULL);
#line 29 "executable.vala"
	valide_environment_variable_set_name (self, name);
#line 30 "executable.vala"
	valide_environment_variable_set_value (self, value);
#line 429 "executable.c"
	return self;
}


#line 27 "executable.vala"
ValideEnvironmentVariable* valide_environment_variable_new (const char* name, const char* value) {
#line 27 "executable.vala"
	return valide_environment_variable_construct (VALIDE_TYPE_ENVIRONMENT_VARIABLE, name, value);
#line 438 "executable.c"
}


#line 33 "executable.vala"
char* valide_environment_variable_to_string (ValideEnvironmentVariable* self) {
#line 444 "executable.c"
	char* result;
#line 33 "executable.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 448 "executable.c"
	result = g_strdup_printf ("%s=%s", self->priv->_name, self->priv->_value);
	return result;
}


const char* valide_environment_variable_get_name (ValideEnvironmentVariable* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_name;
	return result;
}


void valide_environment_variable_set_name (ValideEnvironmentVariable* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 24 "executable.vala"
	_tmp0_ = NULL;
#line 469 "executable.c"
	self->priv->_name = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_name = (g_free (self->priv->_name), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "name");
}


const char* valide_environment_variable_get_value (ValideEnvironmentVariable* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_value;
	return result;
}


void valide_environment_variable_set_value (ValideEnvironmentVariable* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 25 "executable.vala"
	_tmp0_ = NULL;
#line 490 "executable.c"
	self->priv->_value = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_value = (g_free (self->priv->_value), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "value");
}


static void valide_environment_variable_class_init (ValideEnvironmentVariableClass * klass) {
	valide_environment_variable_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideEnvironmentVariablePrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_environment_variable_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_environment_variable_set_property;
	G_OBJECT_CLASS (klass)->finalize = valide_environment_variable_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_ENVIRONMENT_VARIABLE_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_ENVIRONMENT_VARIABLE_VALUE, g_param_spec_string ("value", "value", "value", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void valide_environment_variable_g_yaml_buildable_interface_init (GYAMLBuildableIface * iface) {
	valide_environment_variable_g_yaml_buildable_parent_iface = g_type_interface_peek_parent (iface);
}


static void valide_environment_variable_instance_init (ValideEnvironmentVariable * self) {
	self->priv = VALIDE_ENVIRONMENT_VARIABLE_GET_PRIVATE (self);
}


static void valide_environment_variable_finalize (GObject* obj) {
	ValideEnvironmentVariable * self;
	self = VALIDE_ENVIRONMENT_VARIABLE (obj);
	self->priv->_name = (g_free (self->priv->_name), NULL);
	self->priv->_value = (g_free (self->priv->_value), NULL);
	G_OBJECT_CLASS (valide_environment_variable_parent_class)->finalize (obj);
}


GType valide_environment_variable_get_type (void) {
	static GType valide_environment_variable_type_id = 0;
	if (valide_environment_variable_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideEnvironmentVariableClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_environment_variable_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideEnvironmentVariable), 0, (GInstanceInitFunc) valide_environment_variable_instance_init, NULL };
		static const GInterfaceInfo g_yaml_buildable_info = { (GInterfaceInitFunc) valide_environment_variable_g_yaml_buildable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		valide_environment_variable_type_id = g_type_register_static (G_TYPE_OBJECT, "ValideEnvironmentVariable", &g_define_type_info, 0);
		g_type_add_interface_static (valide_environment_variable_type_id, G_YAML_TYPE_BUILDABLE, &g_yaml_buildable_info);
	}
	return valide_environment_variable_type_id;
}


static void valide_environment_variable_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideEnvironmentVariable * self;
	gpointer boxed;
	self = VALIDE_ENVIRONMENT_VARIABLE (object);
	switch (property_id) {
		case VALIDE_ENVIRONMENT_VARIABLE_NAME:
		g_value_set_string (value, valide_environment_variable_get_name (self));
		break;
		case VALIDE_ENVIRONMENT_VARIABLE_VALUE:
		g_value_set_string (value, valide_environment_variable_get_value (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_environment_variable_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideEnvironmentVariable * self;
	self = VALIDE_ENVIRONMENT_VARIABLE (object);
	switch (property_id) {
		case VALIDE_ENVIRONMENT_VARIABLE_NAME:
		valide_environment_variable_set_name (self, g_value_get_string (value));
		break;
		case VALIDE_ENVIRONMENT_VARIABLE_VALUE:
		valide_environment_variable_set_value (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _g_list_free_g_object_unref (GList* self) {
	g_list_foreach (self, (GFunc) g_object_unref, NULL);
	g_list_free (self);
}


/**
   * @see Glib.YAML.Buildable.add_child
   */
#line 66 "executable.vala"
static void valide_executable_options_real_add_child (GYAMLBuildable* base, GYAMLBuilder* builder, GObject* child, const char* type, GError** error) {
#line 584 "executable.c"
	ValideExecutableOptions * self;
	self = (ValideExecutableOptions*) base;
#line 66 "executable.vala"
	g_return_if_fail (builder != NULL);
#line 66 "executable.vala"
	g_return_if_fail (child != NULL);
#line 69 "executable.vala"
	if (_vala_strcmp0 (type, "environment") == 0) {
#line 593 "executable.c"
		ValideEnvironmentVariable* _tmp0_;
#line 71 "executable.vala"
		_tmp0_ = NULL;
#line 71 "executable.vala"
		self->environment = g_list_prepend (self->environment, (_tmp0_ = VALIDE_ENVIRONMENT_VARIABLE (child), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)));
#line 599 "executable.c"
	}
}


/**
   * @see Glib.YAML.Buildable.get_child_types
   */
#line 78 "executable.vala"
static GType* valide_executable_options_real_get_child_types (GYAMLBuildable* base, int* result_length1) {
#line 609 "executable.c"
	ValideExecutableOptions * self;
	GType* result;
	GType* _tmp0_;
	self = (ValideExecutableOptions*) base;
#line 80 "executable.vala"
	_tmp0_ = NULL;
#line 616 "executable.c"
	result = (_tmp0_ = valide_executable_options_child_types, *result_length1 = valide_executable_options_child_types_length1, _tmp0_);
	return result;
}


/**
   * @see Glib.YAML.Buildable.get_child_tags
   */
#line 86 "executable.vala"
static char** valide_executable_options_real_get_child_tags (GYAMLBuildable* base, int* result_length1) {
#line 627 "executable.c"
	ValideExecutableOptions * self;
	char** result;
	const char** _tmp0_;
	self = (ValideExecutableOptions*) base;
#line 88 "executable.vala"
	_tmp0_ = NULL;
#line 634 "executable.c"
	result = (_tmp0_ = VALIDE_EXECUTABLE_OPTIONS_child_tags, *result_length1 = G_N_ELEMENTS (VALIDE_EXECUTABLE_OPTIONS_child_tags), _tmp0_);
	return result;
}


/**
   * @see Glib.YAML.Buildable.get_children
   */
#line 94 "executable.vala"
static GList* valide_executable_options_real_get_children (GYAMLBuildable* base, const char* type) {
#line 645 "executable.c"
	ValideExecutableOptions * self;
	GList* result;
	self = (ValideExecutableOptions*) base;
#line 96 "executable.vala"
	if (_vala_strcmp0 (type, "environment") == 0) {
#line 651 "executable.c"
		result = g_list_copy (self->environment);
		return result;
	}
	result = NULL;
	return result;
}


#line 103 "executable.vala"
char** valide_executable_options_get_environment_variables (ValideExecutableOptions* self, int* result_length1) {
#line 662 "executable.c"
	char** result;
	gint i;
	guint size;
	gint env_size;
	gint env_length1;
	char** env;
	char** _tmp9_;
#line 103 "executable.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 672 "executable.c"
	i = 0;
	size = 0U;
	env = (env_length1 = 0, NULL);
#line 109 "executable.vala"
	size = g_list_length (self->environment);
#line 110 "executable.vala"
	if (size > 0) {
#line 680 "executable.c"
		const char* value;
		gint default_env_size;
		gint default_env_length1;
		char** default_env;
		char** _tmp1_;
		char** _tmp0_;
		char** _tmp2_;
		char* _tmp7_;
		value = NULL;
		default_env = (default_env_length1 = 0, NULL);
		/* Copy the default environment */
		_tmp1_ = NULL;
		_tmp0_ = NULL;
#line 116 "executable.vala"
		default_env = (_tmp1_ = _tmp0_ = g_listenv (), default_env = (_vala_array_free (default_env, default_env_length1, (GDestroyNotify) g_free), NULL), default_env_length1 = _vala_array_length (_tmp0_), default_env_size = default_env_length1, _tmp1_);
#line 117 "executable.vala"
		size = size + ((guint) default_env_length1);
#line 698 "executable.c"
		_tmp2_ = NULL;
#line 118 "executable.vala"
		env = (_tmp2_ = g_new0 (char*, (size + 1) + 1), env = (_vala_array_free (env, env_length1, (GDestroyNotify) g_free), NULL), env_length1 = size + 1, env_size = env_length1, _tmp2_);
#line 702 "executable.c"
		{
			char** name_collection;
			int name_collection_length1;
			int name_it;
#line 119 "executable.vala"
			name_collection = default_env;
#line 709 "executable.c"
			name_collection_length1 = default_env_length1;
			for (name_it = 0; name_it < default_env_length1; name_it = name_it + 1) {
				const char* _tmp4_;
				char* name;
#line 113 "executable.vala"
				_tmp4_ = NULL;
#line 716 "executable.c"
				name = (_tmp4_ = name_collection[name_it], (_tmp4_ == NULL) ? NULL : g_strdup (_tmp4_));
				{
#line 121 "executable.vala"
					value = g_getenv (name);
#line 122 "executable.vala"
					if (value != NULL) {
#line 723 "executable.c"
						char* _tmp3_;
						_tmp3_ = NULL;
#line 124 "executable.vala"
						env[i] = (_tmp3_ = g_strdup_printf ("%s=%s", name, value), env[i] = (g_free (env[i]), NULL), _tmp3_);
#line 125 "executable.vala"
						i++;
#line 730 "executable.c"
					}
					name = (g_free (name), NULL);
				}
			}
		}
		/* Add the user defined environment */
		{
			GList* variable_collection;
			GList* variable_it;
#line 130 "executable.vala"
			variable_collection = self->environment;
#line 742 "executable.c"
			for (variable_it = variable_collection; variable_it != NULL; variable_it = variable_it->next) {
				ValideEnvironmentVariable* _tmp6_;
				ValideEnvironmentVariable* variable;
#line 41 "executable.vala"
				_tmp6_ = NULL;
#line 130 "executable.vala"
				variable = (_tmp6_ = (ValideEnvironmentVariable*) variable_it->data, (_tmp6_ == NULL) ? NULL : g_object_ref (_tmp6_));
#line 750 "executable.c"
				{
					char* _tmp5_;
					_tmp5_ = NULL;
#line 132 "executable.vala"
					env[i] = (_tmp5_ = valide_environment_variable_to_string (variable), env[i] = (g_free (env[i]), NULL), _tmp5_);
#line 133 "executable.vala"
					i++;
#line 758 "executable.c"
					(variable == NULL) ? NULL : (variable = (g_object_unref (variable), NULL));
				}
			}
		}
		_tmp7_ = NULL;
#line 135 "executable.vala"
		env[i] = (_tmp7_ = NULL, env[i] = (g_free (env[i]), NULL), _tmp7_);
#line 766 "executable.c"
		default_env = (_vala_array_free (default_env, default_env_length1, (GDestroyNotify) g_free), NULL);
	} else {
		char** _tmp8_;
		_tmp8_ = NULL;
#line 139 "executable.vala"
		env = (_tmp8_ = NULL, env = (_vala_array_free (env, env_length1, (GDestroyNotify) g_free), NULL), env_length1 = 0, env_size = env_length1, _tmp8_);
#line 773 "executable.c"
	}
#line 141 "executable.vala"
	_tmp9_ = NULL;
#line 777 "executable.c"
	result = (_tmp9_ = env, *result_length1 = env_length1, _tmp9_);
	return result;
	env = (_vala_array_free (env, env_length1, (GDestroyNotify) g_free), NULL);
}


#line 39 "executable.vala"
ValideExecutableOptions* valide_executable_options_construct (GType object_type) {
#line 786 "executable.c"
	ValideExecutableOptions * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


#line 39 "executable.vala"
ValideExecutableOptions* valide_executable_options_new (void) {
#line 39 "executable.vala"
	return valide_executable_options_construct (VALIDE_TYPE_EXECUTABLE_OPTIONS);
#line 797 "executable.c"
}


const char* valide_executable_options_get_program (ValideExecutableOptions* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_program;
	return result;
}


void valide_executable_options_set_program (ValideExecutableOptions* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 51 "executable.vala"
	_tmp0_ = NULL;
#line 816 "executable.c"
	self->priv->_program = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_program = (g_free (self->priv->_program), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "program");
}


const char* valide_executable_options_get_arguments (ValideExecutableOptions* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_arguments;
	return result;
}


void valide_executable_options_set_arguments (ValideExecutableOptions* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 52 "executable.vala"
	_tmp0_ = NULL;
#line 837 "executable.c"
	self->priv->_arguments = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_arguments = (g_free (self->priv->_arguments), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "arguments");
}


const char* valide_executable_options_get_working_dir (ValideExecutableOptions* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_working_dir;
	return result;
}


void valide_executable_options_set_working_dir (ValideExecutableOptions* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 53 "executable.vala"
	_tmp0_ = NULL;
#line 858 "executable.c"
	self->priv->_working_dir = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_working_dir = (g_free (self->priv->_working_dir), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "working-dir");
}


static GObject * valide_executable_options_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValideExecutableOptionsClass * klass;
	GObjectClass * parent_class;
	ValideExecutableOptions * self;
	klass = VALIDE_EXECUTABLE_OPTIONS_CLASS (g_type_class_peek (VALIDE_TYPE_EXECUTABLE_OPTIONS));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_EXECUTABLE_OPTIONS (obj);
	{
		GList* _tmp1_;
#line 57 "executable.vala"
		valide_executable_options_set_program (self, "");
#line 58 "executable.vala"
		valide_executable_options_set_arguments (self, "");
#line 59 "executable.vala"
		valide_executable_options_set_working_dir (self, "");
#line 881 "executable.c"
		_tmp1_ = NULL;
#line 60 "executable.vala"
		self->environment = (_tmp1_ = NULL, (self->environment == NULL) ? NULL : (self->environment = (_g_list_free_g_object_unref (self->environment), NULL)), _tmp1_);
#line 885 "executable.c"
	}
	return obj;
}


static void valide_executable_options_class_init (ValideExecutableOptionsClass * klass) {
	GType* _tmp0_;
	valide_executable_options_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideExecutableOptionsPrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_executable_options_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_executable_options_set_property;
	G_OBJECT_CLASS (klass)->constructor = valide_executable_options_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_executable_options_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_EXECUTABLE_OPTIONS_PROGRAM, g_param_spec_string ("program", "program", "program", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_EXECUTABLE_OPTIONS_ARGUMENTS, g_param_spec_string ("arguments", "arguments", "arguments", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_EXECUTABLE_OPTIONS_WORKING_DIR, g_param_spec_string ("working-dir", "working-dir", "working-dir", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	valide_executable_options_child_types = (_tmp0_ = g_new0 (GType, 1), _tmp0_[0] = VALIDE_TYPE_ENVIRONMENT_VARIABLE, _tmp0_);
	valide_executable_options_child_types_length1 = 1;
#line 43 "executable.vala"
	_tmp0_ = NULL;
#line 906 "executable.c"
}


static void valide_executable_options_g_yaml_buildable_interface_init (GYAMLBuildableIface * iface) {
	valide_executable_options_g_yaml_buildable_parent_iface = g_type_interface_peek_parent (iface);
	iface->add_child = valide_executable_options_real_add_child;
	iface->get_child_types = valide_executable_options_real_get_child_types;
	iface->get_child_tags = valide_executable_options_real_get_child_tags;
	iface->get_children = valide_executable_options_real_get_children;
}


static void valide_executable_options_instance_init (ValideExecutableOptions * self) {
	self->priv = VALIDE_EXECUTABLE_OPTIONS_GET_PRIVATE (self);
}


static void valide_executable_options_finalize (GObject* obj) {
	ValideExecutableOptions * self;
	self = VALIDE_EXECUTABLE_OPTIONS (obj);
	(self->environment == NULL) ? NULL : (self->environment = (_g_list_free_g_object_unref (self->environment), NULL));
	self->priv->_program = (g_free (self->priv->_program), NULL);
	self->priv->_arguments = (g_free (self->priv->_arguments), NULL);
	self->priv->_working_dir = (g_free (self->priv->_working_dir), NULL);
	G_OBJECT_CLASS (valide_executable_options_parent_class)->finalize (obj);
}


GType valide_executable_options_get_type (void) {
	static GType valide_executable_options_type_id = 0;
	if (valide_executable_options_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideExecutableOptionsClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_executable_options_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideExecutableOptions), 0, (GInstanceInitFunc) valide_executable_options_instance_init, NULL };
		static const GInterfaceInfo g_yaml_buildable_info = { (GInterfaceInitFunc) valide_executable_options_g_yaml_buildable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		valide_executable_options_type_id = g_type_register_static (G_TYPE_OBJECT, "ValideExecutableOptions", &g_define_type_info, 0);
		g_type_add_interface_static (valide_executable_options_type_id, G_YAML_TYPE_BUILDABLE, &g_yaml_buildable_info);
	}
	return valide_executable_options_type_id;
}


static void valide_executable_options_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideExecutableOptions * self;
	gpointer boxed;
	self = VALIDE_EXECUTABLE_OPTIONS (object);
	switch (property_id) {
		case VALIDE_EXECUTABLE_OPTIONS_PROGRAM:
		g_value_set_string (value, valide_executable_options_get_program (self));
		break;
		case VALIDE_EXECUTABLE_OPTIONS_ARGUMENTS:
		g_value_set_string (value, valide_executable_options_get_arguments (self));
		break;
		case VALIDE_EXECUTABLE_OPTIONS_WORKING_DIR:
		g_value_set_string (value, valide_executable_options_get_working_dir (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_executable_options_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideExecutableOptions * self;
	self = VALIDE_EXECUTABLE_OPTIONS (object);
	switch (property_id) {
		case VALIDE_EXECUTABLE_OPTIONS_PROGRAM:
		valide_executable_options_set_program (self, g_value_get_string (value));
		break;
		case VALIDE_EXECUTABLE_OPTIONS_ARGUMENTS:
		valide_executable_options_set_arguments (self, g_value_get_string (value));
		break;
		case VALIDE_EXECUTABLE_OPTIONS_WORKING_DIR:
		valide_executable_options_set_working_dir (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}



static GType valide_executable_preferences_col_get_type (void) {
	static GType valide_executable_preferences_col_type_id = 0;
	if (G_UNLIKELY (valide_executable_preferences_col_type_id == 0)) {
		static const GEnumValue values[] = {{VALIDE_EXECUTABLE_PREFERENCES_COL_NAME, "VALIDE_EXECUTABLE_PREFERENCES_COL_NAME", "name"}, {VALIDE_EXECUTABLE_PREFERENCES_COL_VALUE, "VALIDE_EXECUTABLE_PREFERENCES_COL_VALUE", "value"}, {VALIDE_EXECUTABLE_PREFERENCES_COL_NB, "VALIDE_EXECUTABLE_PREFERENCES_COL_NB", "nb"}, {0, NULL, NULL}};
		valide_executable_preferences_col_type_id = g_enum_register_static ("ValideExecutablePreferencesCol", values);
	}
	return valide_executable_preferences_col_type_id;
}


#line 163 "executable.vala"
static void valide_executable_preferences_update_environment (ValideExecutablePreferences* self) {
#line 1001 "executable.c"
	GtkTreeIter iter = {0};
	GList* _tmp0_;
#line 163 "executable.vala"
	g_return_if_fail (self != NULL);
#line 1006 "executable.c"
	_tmp0_ = NULL;
#line 167 "executable.vala"
	self->priv->options->environment = (_tmp0_ = NULL, (self->priv->options->environment == NULL) ? NULL : (self->priv->options->environment = (_g_list_free_g_object_unref (self->priv->options->environment), NULL)), _tmp0_);
#line 168 "executable.vala"
	if (gtk_tree_model_get_iter_first ((GtkTreeModel*) self->priv->list_store, &iter)) {
#line 1012 "executable.c"
		char* name;
		char* value;
		name = NULL;
		value = NULL;
		{
			gboolean _tmp1_;
			_tmp1_ = TRUE;
#line 173 "executable.vala"
			while (TRUE) {
#line 173 "executable.vala"
				if (!_tmp1_) {
#line 177 "executable.vala"
					if (!gtk_tree_model_iter_next ((GtkTreeModel*) self->priv->list_store, &iter)) {
#line 177 "executable.vala"
						break;
#line 1028 "executable.c"
					}
				}
#line 173 "executable.vala"
				_tmp1_ = FALSE;
#line 175 "executable.vala"
				gtk_tree_model_get ((GtkTreeModel*) self->priv->list_store, &iter, VALIDE_EXECUTABLE_PREFERENCES_COL_NAME, &name, VALIDE_EXECUTABLE_PREFERENCES_COL_VALUE, &value, -1);
#line 176 "executable.vala"
				self->priv->options->environment = g_list_append (self->priv->options->environment, valide_environment_variable_new (name, value));
#line 1037 "executable.c"
			}
		}
		name = (g_free (name), NULL);
		value = (g_free (value), NULL);
	}
#line 179 "executable.vala"
	valide_project_save (self->priv->_project);
#line 1045 "executable.c"
}


#line 182 "executable.vala"
static void valide_executable_preferences_on_name_edit (ValideExecutablePreferences* self, GtkCellRenderer* sender, const char* path, const char* new_text) {
#line 1051 "executable.c"
	GtkTreeIter iter = {0};
#line 182 "executable.vala"
	g_return_if_fail (self != NULL);
#line 182 "executable.vala"
	g_return_if_fail (sender != NULL);
#line 182 "executable.vala"
	g_return_if_fail (path != NULL);
#line 182 "executable.vala"
	g_return_if_fail (new_text != NULL);
#line 186 "executable.vala"
	gtk_tree_model_get_iter_from_string ((GtkTreeModel*) self->priv->list_store, &iter, path);
#line 187 "executable.vala"
	gtk_list_store_set (self->priv->list_store, &iter, VALIDE_EXECUTABLE_PREFERENCES_COL_NAME, new_text, -1);
#line 188 "executable.vala"
	valide_executable_preferences_update_environment (self);
#line 1067 "executable.c"
}


#line 191 "executable.vala"
static void valide_executable_preferences_on_value_edit (ValideExecutablePreferences* self, GtkCellRenderer* sender, const char* path, const char* new_text) {
#line 1073 "executable.c"
	GtkTreeIter iter = {0};
#line 191 "executable.vala"
	g_return_if_fail (self != NULL);
#line 191 "executable.vala"
	g_return_if_fail (sender != NULL);
#line 191 "executable.vala"
	g_return_if_fail (path != NULL);
#line 191 "executable.vala"
	g_return_if_fail (new_text != NULL);
#line 195 "executable.vala"
	gtk_tree_model_get_iter_from_string ((GtkTreeModel*) self->priv->list_store, &iter, path);
#line 196 "executable.vala"
	gtk_list_store_set (self->priv->list_store, &iter, VALIDE_EXECUTABLE_PREFERENCES_COL_VALUE, new_text, -1);
#line 197 "executable.vala"
	valide_executable_preferences_update_environment (self);
#line 1089 "executable.c"
}


#line 201 "executable.vala"
void valide_executable_preferences_add_variable (GtkButton* sender, ValideExecutablePreferences* self) {
#line 1095 "executable.c"
	GtkTreeIter iter = {0};
	GtkTreePath* path;
	GtkTreeViewColumn* column;
	GtkTreePath* _tmp0_;
	GtkTreeViewColumn* _tmp2_;
	GtkTreeViewColumn* _tmp1_;
#line 201 "executable.vala"
	g_return_if_fail (self != NULL);
#line 201 "executable.vala"
	g_return_if_fail (sender != NULL);
#line 1106 "executable.c"
	path = NULL;
	column = NULL;
#line 207 "executable.vala"
	gtk_list_store_append (self->priv->list_store, &iter);
#line 208 "executable.vala"
	gtk_list_store_set (self->priv->list_store, &iter, VALIDE_EXECUTABLE_PREFERENCES_COL_NAME, "", VALIDE_EXECUTABLE_PREFERENCES_COL_VALUE, "", -1);
#line 1113 "executable.c"
	_tmp0_ = NULL;
#line 210 "executable.vala"
	path = (_tmp0_ = gtk_tree_model_get_path ((GtkTreeModel*) self->priv->list_store, &iter), (path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL)), _tmp0_);
#line 1117 "executable.c"
	_tmp2_ = NULL;
#line 211 "executable.vala"
	_tmp1_ = NULL;
#line 211 "executable.vala"
	column = (_tmp2_ = (_tmp1_ = gtk_tree_view_get_column (((ValideAbstractExecutablePreferences*) self)->widgets->tree_view, (gint) VALIDE_EXECUTABLE_PREFERENCES_COL_NAME), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (column == NULL) ? NULL : (column = (g_object_unref (column), NULL)), _tmp2_);
#line 212 "executable.vala"
	gtk_tree_view_scroll_to_cell (((ValideAbstractExecutablePreferences*) self)->widgets->tree_view, path, column, FALSE, (float) 0, (float) 0);
#line 213 "executable.vala"
	gtk_tree_view_set_cursor (((ValideAbstractExecutablePreferences*) self)->widgets->tree_view, path, column, TRUE);
#line 1127 "executable.c"
	(path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL));
	(column == NULL) ? NULL : (column = (g_object_unref (column), NULL));
}


#line 217 "executable.vala"
void valide_executable_preferences_modify_variable (GtkButton* sender, ValideExecutablePreferences* self) {
#line 1135 "executable.c"
	GtkTreeIter iter = {0};
	GtkTreeSelection* selection;
	GtkTreeSelection* _tmp1_;
	GtkTreeSelection* _tmp0_;
#line 217 "executable.vala"
	g_return_if_fail (self != NULL);
#line 217 "executable.vala"
	g_return_if_fail (sender != NULL);
#line 1144 "executable.c"
	selection = NULL;
	_tmp1_ = NULL;
#line 222 "executable.vala"
	_tmp0_ = NULL;
#line 222 "executable.vala"
	selection = (_tmp1_ = (_tmp0_ = gtk_tree_view_get_selection (((ValideAbstractExecutablePreferences*) self)->widgets->tree_view), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL)), _tmp1_);
#line 223 "executable.vala"
	if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
#line 1153 "executable.c"
		GtkTreePath* path;
		GtkTreeViewColumn* column;
		GtkTreePath* _tmp2_;
		GtkTreeViewColumn* _tmp4_;
		GtkTreeViewColumn* _tmp3_;
		path = NULL;
		column = NULL;
		_tmp2_ = NULL;
#line 228 "executable.vala"
		path = (_tmp2_ = gtk_tree_model_get_path ((GtkTreeModel*) self->priv->list_store, &iter), (path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL)), _tmp2_);
#line 1164 "executable.c"
		_tmp4_ = NULL;
#line 229 "executable.vala"
		_tmp3_ = NULL;
#line 229 "executable.vala"
		column = (_tmp4_ = (_tmp3_ = gtk_tree_view_get_column (((ValideAbstractExecutablePreferences*) self)->widgets->tree_view, (gint) VALIDE_EXECUTABLE_PREFERENCES_COL_VALUE), (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), (column == NULL) ? NULL : (column = (g_object_unref (column), NULL)), _tmp4_);
#line 230 "executable.vala"
		gtk_tree_view_scroll_to_cell (((ValideAbstractExecutablePreferences*) self)->widgets->tree_view, path, column, FALSE, (float) 0, (float) 0);
#line 231 "executable.vala"
		gtk_tree_view_set_cursor (((ValideAbstractExecutablePreferences*) self)->widgets->tree_view, path, column, TRUE);
#line 1174 "executable.c"
		(path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL));
		(column == NULL) ? NULL : (column = (g_object_unref (column), NULL));
	}
	(selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL));
}


#line 236 "executable.vala"
void valide_executable_preferences_delete_variable (GtkButton* sender, ValideExecutablePreferences* self) {
#line 1184 "executable.c"
	GtkTreeIter iter = {0};
	GtkTreeSelection* selection;
	GtkTreeSelection* _tmp1_;
	GtkTreeSelection* _tmp0_;
#line 236 "executable.vala"
	g_return_if_fail (self != NULL);
#line 236 "executable.vala"
	g_return_if_fail (sender != NULL);
#line 1193 "executable.c"
	selection = NULL;
	_tmp1_ = NULL;
#line 241 "executable.vala"
	_tmp0_ = NULL;
#line 241 "executable.vala"
	selection = (_tmp1_ = (_tmp0_ = gtk_tree_view_get_selection (((ValideAbstractExecutablePreferences*) self)->widgets->tree_view), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL)), _tmp1_);
#line 242 "executable.vala"
	if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
#line 244 "executable.vala"
		gtk_list_store_remove (self->priv->list_store, &iter);
#line 245 "executable.vala"
		valide_executable_preferences_update_environment (self);
#line 1206 "executable.c"
	}
	(selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL));
}


#line 250 "executable.vala"
void valide_executable_preferences_on_program_changed (GtkButton* sender, ValideExecutablePreferences* self) {
#line 1214 "executable.c"
	GtkFileChooserDialog* dialog;
	GtkFileChooserDialog* _tmp0_;
#line 250 "executable.vala"
	g_return_if_fail (self != NULL);
#line 250 "executable.vala"
	g_return_if_fail (sender != NULL);
#line 1221 "executable.c"
	dialog = NULL;
	_tmp0_ = NULL;
#line 254 "executable.vala"
	dialog = (_tmp0_ = g_object_ref_sink ((GtkFileChooserDialog*) gtk_file_chooser_dialog_new (_ ("Choose the executable"), NULL, GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OK, GTK_RESPONSE_OK, NULL)), (dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL)), _tmp0_);
#line 258 "executable.vala"
	gtk_file_chooser_set_filename ((GtkFileChooser*) dialog, gtk_entry_get_text (((ValideAbstractExecutablePreferences*) self)->widgets->executable_program));
#line 259 "executable.vala"
	if (gtk_dialog_run ((GtkDialog*) dialog) == GTK_RESPONSE_OK) {
#line 1230 "executable.c"
		char* _tmp1_;
#line 5545 "gtk+-2.0.vapi"
		_tmp1_ = NULL;
#line 261 "executable.vala"
		gtk_entry_set_text (((ValideAbstractExecutablePreferences*) self)->widgets->executable_program, _tmp1_ = gtk_file_chooser_get_filename ((GtkFileChooser*) dialog));
#line 1236 "executable.c"
		_tmp1_ = (g_free (_tmp1_), NULL);
	}
#line 263 "executable.vala"
	gtk_object_destroy ((GtkObject*) dialog);
#line 1241 "executable.c"
	(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
}


#line 266 "executable.vala"
ValideExecutablePreferences* valide_executable_preferences_construct (GType object_type, ValideProject* project) {
#line 1248 "executable.c"
	GParameter * __params;
	GParameter * __params_it;
	ValideExecutablePreferences * self;
#line 266 "executable.vala"
	g_return_val_if_fail (project != NULL, NULL);
#line 1254 "executable.c"
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
#line 268 "executable.vala"
	__params_it->name = "project";
#line 268 "executable.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_PROJECT);
#line 268 "executable.vala"
	g_value_set_object (&__params_it->value, project);
#line 268 "executable.vala"
	__params_it++;
#line 1265 "executable.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 266 "executable.vala"
ValideExecutablePreferences* valide_executable_preferences_new (ValideProject* project) {
#line 266 "executable.vala"
	return valide_executable_preferences_construct (VALIDE_TYPE_EXECUTABLE_PREFERENCES, project);
#line 1280 "executable.c"
}


ValideProject* valide_executable_preferences_get_project (ValideExecutablePreferences* self) {
	ValideProject* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_project;
	return result;
}


static void valide_executable_preferences_set_project (ValideExecutablePreferences* self, ValideProject* value) {
	ValideProject* _tmp1_;
	ValideProject* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 160 "executable.vala"
	_tmp0_ = NULL;
#line 1299 "executable.c"
	self->priv->_project = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_project == NULL) ? NULL : (self->priv->_project = (g_object_unref (self->priv->_project), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "project");
}


static void _lambda82_ (GtkEditable* s, ValideExecutablePreferences* self) {
	GtkEditable* _tmp0_;
	g_return_if_fail (s != NULL);
#line 284 "executable.vala"
	_tmp0_ = NULL;
#line 284 "executable.vala"
	valide_executable_options_set_program (self->priv->options, gtk_entry_get_text ((_tmp0_ = s, GTK_IS_ENTRY (_tmp0_) ? ((GtkEntry*) _tmp0_) : NULL)));
#line 285 "executable.vala"
	valide_project_save (self->priv->_project);
#line 1314 "executable.c"
}


static void __lambda82__gtk_editable_changed (GtkEditable* _sender, gpointer self) {
	_lambda82_ (_sender, self);
}


static void _lambda83_ (GtkEditable* s, ValideExecutablePreferences* self) {
	GtkEditable* _tmp0_;
	g_return_if_fail (s != NULL);
#line 293 "executable.vala"
	_tmp0_ = NULL;
#line 293 "executable.vala"
	valide_executable_options_set_arguments (self->priv->options, gtk_entry_get_text ((_tmp0_ = s, GTK_IS_ENTRY (_tmp0_) ? ((GtkEntry*) _tmp0_) : NULL)));
#line 294 "executable.vala"
	valide_project_save (self->priv->_project);
#line 1332 "executable.c"
}


static void __lambda83__gtk_editable_changed (GtkEditable* _sender, gpointer self) {
	_lambda83_ (_sender, self);
}


static void _lambda84_ (GtkFileChooser* s, ValideExecutablePreferences* self) {
	char* _tmp0_;
	g_return_if_fail (s != NULL);
#line 5545 "gtk+-2.0.vapi"
	_tmp0_ = NULL;
#line 302 "executable.vala"
	valide_executable_options_set_working_dir (self->priv->options, _tmp0_ = gtk_file_chooser_get_filename (s));
#line 1348 "executable.c"
	_tmp0_ = (g_free (_tmp0_), NULL);
#line 303 "executable.vala"
	valide_project_save (self->priv->_project);
#line 1352 "executable.c"
}


static void __lambda84__gtk_file_chooser_current_folder_changed (GtkFileChooser* _sender, gpointer self) {
	_lambda84_ (_sender, self);
}


#line 182 "executable.vala"
static void _valide_executable_preferences_on_name_edit_gtk_cell_renderer_text_edited (GtkCellRendererText* _sender, const char* path, const char* new_text, gpointer self) {
#line 1363 "executable.c"
	valide_executable_preferences_on_name_edit (self, _sender, path, new_text);
}


#line 191 "executable.vala"
static void _valide_executable_preferences_on_value_edit_gtk_cell_renderer_text_edited (GtkCellRendererText* _sender, const char* path, const char* new_text, gpointer self) {
#line 1370 "executable.c"
	valide_executable_preferences_on_value_edit (self, _sender, path, new_text);
}


static GObject * valide_executable_preferences_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValideExecutablePreferencesClass * klass;
	GObjectClass * parent_class;
	ValideExecutablePreferences * self;
	klass = VALIDE_EXECUTABLE_PREFERENCES_CLASS (g_type_class_peek (VALIDE_TYPE_EXECUTABLE_PREFERENCES));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_EXECUTABLE_PREFERENCES (obj);
	{
		GtkTreeIter iter = {0};
		GtkTreeViewColumn* column;
		GtkCellRendererText* renderer;
		ValideExecutableOptions* _tmp3_;
		ValideExecutableOptions* _tmp2_;
		GtkListStore* _tmp4_;
		GtkTreeViewColumn* _tmp5_;
		GtkCellRendererText* _tmp6_;
		GtkTreeViewColumn* _tmp7_;
		GtkCellRendererText* _tmp8_;
		column = NULL;
		renderer = NULL;
		_tmp3_ = NULL;
#line 277 "executable.vala"
		_tmp2_ = NULL;
#line 277 "executable.vala"
		self->priv->options = (_tmp3_ = (_tmp2_ = valide_project_get_executable_options (self->priv->_project), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (self->priv->options == NULL) ? NULL : (self->priv->options = (g_object_unref (self->priv->options), NULL)), _tmp3_);
#line 279 "executable.vala"
		if (valide_executable_options_get_program (self->priv->options) != NULL) {
#line 281 "executable.vala"
			gtk_entry_set_text (((ValideAbstractExecutablePreferences*) self)->widgets->executable_program, valide_executable_options_get_program (self->priv->options));
#line 1406 "executable.c"
		}
#line 283 "executable.vala"
		g_signal_connect ((GtkEditable*) ((ValideAbstractExecutablePreferences*) self)->widgets->executable_program, "changed", (GCallback) __lambda82__gtk_editable_changed, self);
#line 288 "executable.vala"
		if (valide_executable_options_get_arguments (self->priv->options) != NULL) {
#line 290 "executable.vala"
			gtk_entry_set_text (((ValideAbstractExecutablePreferences*) self)->widgets->executable_arguments, valide_executable_options_get_arguments (self->priv->options));
#line 1414 "executable.c"
		}
#line 292 "executable.vala"
		g_signal_connect ((GtkEditable*) ((ValideAbstractExecutablePreferences*) self)->widgets->executable_arguments, "changed", (GCallback) __lambda83__gtk_editable_changed, self);
#line 297 "executable.vala"
		if (valide_executable_options_get_working_dir (self->priv->options) != NULL) {
#line 299 "executable.vala"
			gtk_file_chooser_set_current_folder ((GtkFileChooser*) ((ValideAbstractExecutablePreferences*) self)->widgets->executable_working_dir, valide_executable_options_get_working_dir (self->priv->options));
#line 1422 "executable.c"
		}
#line 301 "executable.vala"
		g_signal_connect ((GtkFileChooser*) ((ValideAbstractExecutablePreferences*) self)->widgets->executable_working_dir, "current-folder-changed", (GCallback) __lambda84__gtk_file_chooser_current_folder_changed, self);
#line 1426 "executable.c"
		_tmp4_ = NULL;
#line 306 "executable.vala"
		self->priv->list_store = (_tmp4_ = gtk_list_store_new ((gint) VALIDE_EXECUTABLE_PREFERENCES_COL_NB, G_TYPE_STRING, G_TYPE_STRING, NULL), (self->priv->list_store == NULL) ? NULL : (self->priv->list_store = (g_object_unref (self->priv->list_store), NULL)), _tmp4_);
#line 308 "executable.vala"
		gtk_tree_view_set_model (((ValideAbstractExecutablePreferences*) self)->widgets->tree_view, (GtkTreeModel*) self->priv->list_store);
#line 1432 "executable.c"
		_tmp5_ = NULL;
#line 310 "executable.vala"
		column = (_tmp5_ = g_object_ref_sink (gtk_tree_view_column_new ()), (column == NULL) ? NULL : (column = (g_object_unref (column), NULL)), _tmp5_);
#line 311 "executable.vala"
		gtk_tree_view_column_set_title (column, _ ("Name"));
#line 1438 "executable.c"
		_tmp6_ = NULL;
#line 312 "executable.vala"
		renderer = (_tmp6_ = g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL)), _tmp6_);
#line 313 "executable.vala"
		g_object_set (renderer, "editable", TRUE, NULL);
#line 314 "executable.vala"
		g_signal_connect_object (renderer, "edited", (GCallback) _valide_executable_preferences_on_name_edit_gtk_cell_renderer_text_edited, self, 0);
#line 315 "executable.vala"
		gtk_cell_layout_pack_start ((GtkCellLayout*) column, (GtkCellRenderer*) renderer, TRUE);
#line 316 "executable.vala"
		gtk_tree_view_column_set_attributes (column, (GtkCellRenderer*) renderer, "text", VALIDE_EXECUTABLE_PREFERENCES_COL_NAME, NULL);
#line 317 "executable.vala"
		gtk_tree_view_append_column (((ValideAbstractExecutablePreferences*) self)->widgets->tree_view, column);
#line 1452 "executable.c"
		_tmp7_ = NULL;
#line 319 "executable.vala"
		column = (_tmp7_ = g_object_ref_sink (gtk_tree_view_column_new ()), (column == NULL) ? NULL : (column = (g_object_unref (column), NULL)), _tmp7_);
#line 320 "executable.vala"
		gtk_tree_view_column_set_title (column, _ ("Value"));
#line 1458 "executable.c"
		_tmp8_ = NULL;
#line 321 "executable.vala"
		renderer = (_tmp8_ = g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL)), _tmp8_);
#line 322 "executable.vala"
		g_object_set (renderer, "editable", TRUE, NULL);
#line 323 "executable.vala"
		g_signal_connect_object (renderer, "edited", (GCallback) _valide_executable_preferences_on_value_edit_gtk_cell_renderer_text_edited, self, 0);
#line 324 "executable.vala"
		gtk_cell_layout_pack_start ((GtkCellLayout*) column, (GtkCellRenderer*) renderer, TRUE);
#line 325 "executable.vala"
		gtk_tree_view_column_set_attributes (column, (GtkCellRenderer*) renderer, "text", VALIDE_EXECUTABLE_PREFERENCES_COL_VALUE, NULL);
#line 326 "executable.vala"
		gtk_tree_view_append_column (((ValideAbstractExecutablePreferences*) self)->widgets->tree_view, column);
#line 1472 "executable.c"
		{
			GList* variable_collection;
			GList* variable_it;
#line 328 "executable.vala"
			variable_collection = self->priv->options->environment;
#line 1478 "executable.c"
			for (variable_it = variable_collection; variable_it != NULL; variable_it = variable_it->next) {
				ValideEnvironmentVariable* _tmp9_;
				ValideEnvironmentVariable* variable;
#line 41 "executable.vala"
				_tmp9_ = NULL;
#line 328 "executable.vala"
				variable = (_tmp9_ = (ValideEnvironmentVariable*) variable_it->data, (_tmp9_ == NULL) ? NULL : g_object_ref (_tmp9_));
#line 1486 "executable.c"
				{
#line 330 "executable.vala"
					gtk_list_store_append (self->priv->list_store, &iter);
#line 331 "executable.vala"
					gtk_list_store_set (self->priv->list_store, &iter, VALIDE_EXECUTABLE_PREFERENCES_COL_NAME, valide_environment_variable_get_name (variable), VALIDE_EXECUTABLE_PREFERENCES_COL_VALUE, valide_environment_variable_get_value (variable), -1);
#line 1492 "executable.c"
					(variable == NULL) ? NULL : (variable = (g_object_unref (variable), NULL));
				}
			}
		}
#line 335 "executable.vala"
		valide_abstract_executable_preferences_connect_signals ((ValideAbstractExecutablePreferences*) self, "valide_executable_preferences_");
#line 1499 "executable.c"
		(column == NULL) ? NULL : (column = (g_object_unref (column), NULL));
		(renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL));
	}
	return obj;
}


static void valide_executable_preferences_class_init (ValideExecutablePreferencesClass * klass) {
	valide_executable_preferences_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideExecutablePreferencesPrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_executable_preferences_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_executable_preferences_set_property;
	G_OBJECT_CLASS (klass)->constructor = valide_executable_preferences_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_executable_preferences_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_EXECUTABLE_PREFERENCES_PROJECT, g_param_spec_object ("project", "project", "project", VALIDE_TYPE_PROJECT, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void valide_executable_preferences_instance_init (ValideExecutablePreferences * self) {
	self->priv = VALIDE_EXECUTABLE_PREFERENCES_GET_PRIVATE (self);
}


static void valide_executable_preferences_finalize (GObject* obj) {
	ValideExecutablePreferences * self;
	self = VALIDE_EXECUTABLE_PREFERENCES (obj);
	(self->priv->options == NULL) ? NULL : (self->priv->options = (g_object_unref (self->priv->options), NULL));
	(self->priv->list_store == NULL) ? NULL : (self->priv->list_store = (g_object_unref (self->priv->list_store), NULL));
	(self->priv->_project == NULL) ? NULL : (self->priv->_project = (g_object_unref (self->priv->_project), NULL));
	G_OBJECT_CLASS (valide_executable_preferences_parent_class)->finalize (obj);
}


GType valide_executable_preferences_get_type (void) {
	static GType valide_executable_preferences_type_id = 0;
	if (valide_executable_preferences_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideExecutablePreferencesClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_executable_preferences_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideExecutablePreferences), 0, (GInstanceInitFunc) valide_executable_preferences_instance_init, NULL };
		valide_executable_preferences_type_id = g_type_register_static (VALIDE_TYPE_ABSTRACT_EXECUTABLE_PREFERENCES, "ValideExecutablePreferences", &g_define_type_info, 0);
	}
	return valide_executable_preferences_type_id;
}


static void valide_executable_preferences_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideExecutablePreferences * self;
	gpointer boxed;
	self = VALIDE_EXECUTABLE_PREFERENCES (object);
	switch (property_id) {
		case VALIDE_EXECUTABLE_PREFERENCES_PROJECT:
		g_value_set_object (value, valide_executable_preferences_get_project (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_executable_preferences_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideExecutablePreferences * self;
	self = VALIDE_EXECUTABLE_PREFERENCES (object);
	switch (property_id) {
		case VALIDE_EXECUTABLE_PREFERENCES_PROJECT:
		valide_executable_preferences_set_project (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


/**
   * A message show when executable was launched.
   *
   * You can use $cmd for show the command.
   */
#line 360 "executable.vala"
static char* valide_executable_real_get_start_msg (ValideExecutable* self) {
#line 1579 "executable.c"
	char* result;
#line 360 "executable.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 1583 "executable.c"
	result = g_strdup ("");
	return result;
}


#line 360 "executable.vala"
char* valide_executable_get_start_msg (ValideExecutable* self) {
#line 360 "executable.vala"
	return VALIDE_EXECUTABLE_GET_CLASS (self)->get_start_msg (self);
#line 1593 "executable.c"
}


/**
   * A message show when executable stop.
   *
   * You can use :
   *   - $status the exit status of the command.
   *   - $time the execution time 
   */
#line 372 "executable.vala"
static char* valide_executable_real_get_end_msg (ValideExecutable* self) {
#line 1606 "executable.c"
	char* result;
	const char* _tmp0_;
#line 372 "executable.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 374 "executable.vala"
	_tmp0_ = NULL;
#line 1613 "executable.c"
	result = (_tmp0_ = _ ("<b>Process return {$status}  execution time: {$time} s</b>"), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
	return result;
}


#line 372 "executable.vala"
char* valide_executable_get_end_msg (ValideExecutable* self) {
#line 372 "executable.vala"
	return VALIDE_EXECUTABLE_GET_CLASS (self)->get_end_msg (self);
#line 1623 "executable.c"
}


/**
   * This function was called before adding line in ExecutableManager tree view.
   * You can modify it.
   *
   * @param line the text.
   */
#line 383 "executable.vala"
static GdkColor valide_executable_real_get_color_line (ValideExecutable* self, char** line) {
#line 1635 "executable.c"
	GdkColor result;
	GdkColor color = {0};
#line 387 "executable.vala"
	gdk_color_parse (VALIDE_UTILS_COLOR_DEFAULT, &color);
#line 1640 "executable.c"
	result = color;
	return result;
}


#line 383 "executable.vala"
GdkColor valide_executable_get_color_line (ValideExecutable* self, char** line) {
#line 383 "executable.vala"
	return VALIDE_EXECUTABLE_GET_CLASS (self)->get_color_line (self, line);
#line 1650 "executable.c"
}


/**
   * This function was called when line in ExecutableManager was activated.
   *
   * @param line the line was actived.
   */
#line 396 "executable.vala"
static void valide_executable_real_line_activated (ValideExecutable* self, const char* line) {
#line 396 "executable.vala"
	g_return_if_fail (self != NULL);
#line 396 "executable.vala"
	g_return_if_fail (line != NULL);
#line 1665 "executable.c"
}


#line 396 "executable.vala"
void valide_executable_line_activated (ValideExecutable* self, const char* line) {
#line 396 "executable.vala"
	VALIDE_EXECUTABLE_GET_CLASS (self)->line_activated (self, line);
#line 1673 "executable.c"
}


#line 942 "glib-2.0.vapi"
static char* string_replace (const char* self, const char* old, const char* replacement) {
#line 1679 "executable.c"
	char* result;
	GError * _inner_error_;
#line 942 "glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 942 "glib-2.0.vapi"
	g_return_val_if_fail (old != NULL, NULL);
#line 942 "glib-2.0.vapi"
	g_return_val_if_fail (replacement != NULL, NULL);
#line 1688 "executable.c"
	_inner_error_ = NULL;
	{
		char* _tmp0_;
		GRegex* _tmp1_;
		GRegex* regex;
		char* _tmp2_;
#line 2667 "glib-2.0.vapi"
		_tmp0_ = NULL;
#line 944 "glib-2.0.vapi"
		_tmp1_ = NULL;
#line 1699 "executable.c"
		regex = (_tmp1_ = g_regex_new (_tmp0_ = g_regex_escape_string (old, -1), 0, 0, &_inner_error_), _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_REGEX_ERROR) {
				goto __catch73_g_regex_error;
			}
			goto __finally73;
		}
		_tmp2_ = g_regex_replace_literal (regex, self, (glong) (-1), 0, replacement, 0, &_inner_error_);
		if (_inner_error_ != NULL) {
			(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
			if (_inner_error_->domain == G_REGEX_ERROR) {
				goto __catch73_g_regex_error;
			}
			goto __finally73;
		}
		result = _tmp2_;
		(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
		return result;
	}
	goto __finally73;
	__catch73_g_regex_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 947 "glib-2.0.vapi"
			g_assert_not_reached ();
#line 1728 "executable.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally73:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
}


/**
   * This function is called when executable is launched
   *
   * @param cmd The command line
   *
   * @return A text
   */
#line 407 "executable.vala"
char* valide_executable_cmd_start (ValideExecutable* self, const char* cmd) {
#line 1750 "executable.c"
	char* result;
	char* msg;
	char* _tmp0_;
	char* _tmp1_;
#line 407 "executable.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 407 "executable.vala"
	g_return_val_if_fail (cmd != NULL, NULL);
#line 1759 "executable.c"
	msg = NULL;
	_tmp0_ = NULL;
#line 411 "executable.vala"
	msg = (_tmp0_ = valide_executable_get_start_msg (self), msg = (g_free (msg), NULL), _tmp0_);
#line 1764 "executable.c"
	_tmp1_ = NULL;
#line 412 "executable.vala"
	msg = (_tmp1_ = string_replace (msg, "{$cmd}", cmd), msg = (g_free (msg), NULL), _tmp1_);
#line 1768 "executable.c"
	result = msg;
	return result;
}


/**
   * This function is called when executable quit
   *
   * @param status The exit status
   * @param time The enlapsed time
   *
   * @return A text
   */
#line 424 "executable.vala"
char* valide_executable_cmd_end (ValideExecutable* self, gint status, double time) {
#line 1784 "executable.c"
	char* result;
	char* msg;
	char* _tmp0_;
	char* _tmp2_;
	char* _tmp1_;
	char* _tmp4_;
	char* _tmp3_;
#line 424 "executable.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 1794 "executable.c"
	msg = NULL;
	_tmp0_ = NULL;
#line 428 "executable.vala"
	msg = (_tmp0_ = valide_executable_get_end_msg (self), msg = (g_free (msg), NULL), _tmp0_);
#line 1799 "executable.c"
	_tmp2_ = NULL;
#line 99 "glib-2.0.vapi"
	_tmp1_ = NULL;
#line 429 "executable.vala"
	msg = (_tmp2_ = string_replace (msg, "{$status}", _tmp1_ = g_strdup_printf ("%i", status)), msg = (g_free (msg), NULL), _tmp2_);
#line 1805 "executable.c"
	_tmp1_ = (g_free (_tmp1_), NULL);
	_tmp4_ = NULL;
#line 820 "glib-2.0.vapi"
	_tmp3_ = NULL;
#line 430 "executable.vala"
	msg = (_tmp4_ = string_replace (msg, "{$time}", _tmp3_ = g_strdup_printf ("%.2f", time)), msg = (g_free (msg), NULL), _tmp4_);
#line 1812 "executable.c"
	_tmp3_ = (g_free (_tmp3_), NULL);
	result = msg;
	return result;
}


#line 434 "executable.vala"
GtkWidget* valide_executable_preferences (ValideProject* project) {
#line 1821 "executable.c"
	GtkWidget* result;
	ValideExecutablePreferences* _tmp0_;
	GtkWidget* _tmp1_;
#line 434 "executable.vala"
	g_return_val_if_fail (project != NULL, NULL);
#line 1827 "executable.c"
	_tmp0_ = NULL;
#line 436 "executable.vala"
	valide_executable_preferences_widget = (_tmp0_ = g_object_ref_sink (valide_executable_preferences_new (project)), (valide_executable_preferences_widget == NULL) ? NULL : (valide_executable_preferences_widget = (g_object_unref (valide_executable_preferences_widget), NULL)), _tmp0_);
#line 437 "executable.vala"
	_tmp1_ = NULL;
#line 1833 "executable.c"
	result = (_tmp1_ = (GtkWidget*) ((ValideAbstractExecutablePreferences*) valide_executable_preferences_widget)->widgets->table, (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_));
	return result;
}


/**
 * A class representing an program can launch by #ExecutableManager.
 */
#line 342 "executable.vala"
ValideExecutable* valide_executable_construct (GType object_type) {
#line 1844 "executable.c"
	ValideExecutable * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


#line 342 "executable.vala"
ValideExecutable* valide_executable_new (void) {
#line 342 "executable.vala"
	return valide_executable_construct (VALIDE_TYPE_EXECUTABLE);
#line 1855 "executable.c"
}


const char* valide_executable_get_executable (ValideExecutable* self) {
	return VALIDE_EXECUTABLE_GET_CLASS (self)->get_executable (self);
}


static const char* valide_executable_real_get_executable (ValideExecutable* base) {
	const char* result;
	ValideExecutable* self;
	self = base;
	result = self->priv->_executable;
	return result;
}


void valide_executable_set_executable (ValideExecutable* self, const char* value) {
	VALIDE_EXECUTABLE_GET_CLASS (self)->set_executable (self, value);
}


static void valide_executable_real_set_executable (ValideExecutable* base, const char* value) {
	ValideExecutable* self;
	char* _tmp1_;
	const char* _tmp0_;
	self = base;
	_tmp1_ = NULL;
#line 352 "executable.vala"
	_tmp0_ = NULL;
#line 1886 "executable.c"
	self->priv->_executable = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_executable = (g_free (self->priv->_executable), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "executable");
}


static void valide_executable_class_init (ValideExecutableClass * klass) {
	valide_executable_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideExecutablePrivate));
	VALIDE_EXECUTABLE_CLASS (klass)->get_start_msg = valide_executable_real_get_start_msg;
	VALIDE_EXECUTABLE_CLASS (klass)->get_end_msg = valide_executable_real_get_end_msg;
	VALIDE_EXECUTABLE_CLASS (klass)->get_color_line = valide_executable_real_get_color_line;
	VALIDE_EXECUTABLE_CLASS (klass)->line_activated = valide_executable_real_line_activated;
	VALIDE_EXECUTABLE_CLASS (klass)->get_executable = valide_executable_real_get_executable;
	VALIDE_EXECUTABLE_CLASS (klass)->set_executable = valide_executable_real_set_executable;
	G_OBJECT_CLASS (klass)->get_property = valide_executable_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_executable_set_property;
	G_OBJECT_CLASS (klass)->finalize = valide_executable_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_EXECUTABLE_EXECUTABLE, g_param_spec_string ("executable", "executable", "executable", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void valide_executable_instance_init (ValideExecutable * self) {
	self->priv = VALIDE_EXECUTABLE_GET_PRIVATE (self);
}


static void valide_executable_finalize (GObject* obj) {
	ValideExecutable * self;
	self = VALIDE_EXECUTABLE (obj);
	self->priv->_executable = (g_free (self->priv->_executable), NULL);
	G_OBJECT_CLASS (valide_executable_parent_class)->finalize (obj);
}


GType valide_executable_get_type (void) {
	static GType valide_executable_type_id = 0;
	if (valide_executable_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideExecutableClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_executable_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideExecutable), 0, (GInstanceInitFunc) valide_executable_instance_init, NULL };
		valide_executable_type_id = g_type_register_static (G_TYPE_OBJECT, "ValideExecutable", &g_define_type_info, 0);
	}
	return valide_executable_type_id;
}


static void valide_executable_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideExecutable * self;
	gpointer boxed;
	self = VALIDE_EXECUTABLE (object);
	switch (property_id) {
		case VALIDE_EXECUTABLE_EXECUTABLE:
		g_value_set_string (value, valide_executable_get_executable (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_executable_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideExecutable * self;
	self = VALIDE_EXECUTABLE (object);
	switch (property_id) {
		case VALIDE_EXECUTABLE_EXECUTABLE:
		valide_executable_set_executable (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




