/* goto-dialog.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <gtksourceview/gtksourceview.h>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>
#include <stdlib.h>
#include <string.h>


#define VALIDE_TYPE_GOTO_DIALOG (valide_goto_dialog_get_type ())
#define VALIDE_GOTO_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_GOTO_DIALOG, ValideGotoDialog))
#define VALIDE_GOTO_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_GOTO_DIALOG, ValideGotoDialogClass))
#define VALIDE_IS_GOTO_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_GOTO_DIALOG))
#define VALIDE_IS_GOTO_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_GOTO_DIALOG))
#define VALIDE_GOTO_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_GOTO_DIALOG, ValideGotoDialogClass))

typedef struct _ValideGotoDialog ValideGotoDialog;
typedef struct _ValideGotoDialogClass ValideGotoDialogClass;
typedef struct _ValideGotoDialogPrivate ValideGotoDialogPrivate;

#define VALIDE_TYPE_SOURCE_VIEW (valide_source_view_get_type ())
#define VALIDE_SOURCE_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_SOURCE_VIEW, ValideSourceView))
#define VALIDE_SOURCE_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_SOURCE_VIEW, ValideSourceViewClass))
#define VALIDE_IS_SOURCE_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_SOURCE_VIEW))
#define VALIDE_IS_SOURCE_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_SOURCE_VIEW))
#define VALIDE_SOURCE_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_SOURCE_VIEW, ValideSourceViewClass))

typedef struct _ValideSourceView ValideSourceView;
typedef struct _ValideSourceViewClass ValideSourceViewClass;

/**
 * A goto dialog widget for Valide.SourceView
 */
struct _ValideGotoDialog {
	GtkWindow parent_instance;
	ValideGotoDialogPrivate * priv;
};

struct _ValideGotoDialogClass {
	GtkWindowClass parent_class;
};

struct _ValideGotoDialogPrivate {
	GtkEntry* entry;
	ValideSourceView* _source_view;
};


static gpointer valide_goto_dialog_parent_class = NULL;

GType valide_goto_dialog_get_type (void);
GType valide_source_view_get_type (void);
#define VALIDE_GOTO_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_GOTO_DIALOG, ValideGotoDialogPrivate))
enum  {
	VALIDE_GOTO_DIALOG_DUMMY_PROPERTY,
	VALIDE_GOTO_DIALOG_SOURCE_VIEW
};
ValideSourceView* valide_goto_dialog_get_source_view (ValideGotoDialog* self);
static void valide_goto_dialog_hide_search (ValideGotoDialog* self, gboolean cancel);
static void valide_goto_dialog_update_position (ValideGotoDialog* self);
static gboolean valide_goto_dialog_key_press_event_cb (ValideGotoDialog* self, GtkWidget* sender, const GdkEventKey* event);
gboolean valide_source_view_goto_line (ValideSourceView* self, gint line);
static void valide_goto_dialog_changed_cb (ValideGotoDialog* self);
static void valide_goto_dialog_insert_text_cb (ValideGotoDialog* self, GtkEditable* sender, const char* new_text, gint new_text_length, void* position);
static void valide_goto_dialog_set_source_view (ValideGotoDialog* self, ValideSourceView* value);
ValideGotoDialog* valide_goto_dialog_new (ValideSourceView* source_view);
ValideGotoDialog* valide_goto_dialog_construct (GType object_type, ValideSourceView* source_view);
void valide_goto_dialog_show (ValideGotoDialog* self);
static void _lambda10_ (ValideGotoDialog* self);
static void __lambda10__gtk_entry_activate (GtkEntry* _sender, gpointer self);
static gboolean _lambda11_ (ValideGotoDialog* self);
static gboolean __lambda11__gtk_widget_delete_event (GtkWidget* _sender, GdkEvent* event, gpointer self);
static gboolean _lambda12_ (ValideGotoDialog* self);
static gboolean __lambda12__gtk_widget_focus_out_event (GtkWidget* _sender, const GdkEventFocus* event, gpointer self);
static void _valide_goto_dialog_changed_cb_gtk_editable_changed (GtkEditable* _sender, gpointer self);
static void _valide_goto_dialog_insert_text_cb_gtk_editable_insert_text (GtkEditable* _sender, const char* text, gint length, void* position, gpointer self);
static gboolean _valide_goto_dialog_key_press_event_cb_gtk_widget_key_press_event (GtkWidget* _sender, const GdkEventKey* event, gpointer self);
static GObject * valide_goto_dialog_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void valide_goto_dialog_finalize (GObject* obj);
static void valide_goto_dialog_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_goto_dialog_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static int _vala_strcmp0 (const char * str1, const char * str2);



#line 38 "goto-dialog.vala"
static void valide_goto_dialog_hide_search (ValideGotoDialog* self, gboolean cancel) {
#line 38 "goto-dialog.vala"
	g_return_if_fail (self != NULL);
#line 40 "goto-dialog.vala"
	gtk_widget_hide ((GtkWidget*) self);
#line 42 "goto-dialog.vala"
	if (cancel) {
#line 117 "goto-dialog.c"
	}
#line 45 "goto-dialog.vala"
	gtk_widget_grab_focus ((GtkWidget*) self->priv->_source_view);
#line 121 "goto-dialog.c"
}


#line 48 "goto-dialog.vala"
static void valide_goto_dialog_update_position (ValideGotoDialog* self) {
#line 127 "goto-dialog.c"
	gint view_x;
	gint view_y;
	GdkWindow* view_window;
	GdkWindow* _tmp1_;
	GdkWindow* _tmp0_;
#line 48 "goto-dialog.vala"
	g_return_if_fail (self != NULL);
#line 135 "goto-dialog.c"
	view_x = 0;
	view_y = 0;
	view_window = NULL;
	_tmp1_ = NULL;
#line 53 "goto-dialog.vala"
	_tmp0_ = NULL;
#line 53 "goto-dialog.vala"
	view_window = (_tmp1_ = (_tmp0_ = ((GtkWidget*) self->priv->_source_view)->window, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (view_window == NULL) ? NULL : (view_window = (g_object_unref (view_window), NULL)), _tmp1_);
#line 54 "goto-dialog.vala"
	gtk_widget_realize ((GtkWidget*) self);
#line 56 "goto-dialog.vala"
	gdk_window_get_origin (view_window, &view_x, &view_y);
#line 58 "goto-dialog.vala"
	gtk_window_move ((GtkWindow*) self, view_x, view_y);
#line 150 "goto-dialog.c"
	(view_window == NULL) ? NULL : (view_window = (g_object_unref (view_window), NULL));
}


#line 61 "goto-dialog.vala"
static gboolean valide_goto_dialog_key_press_event_cb (ValideGotoDialog* self, GtkWidget* sender, const GdkEventKey* event) {
#line 157 "goto-dialog.c"
	gboolean result;
	gboolean catched;
#line 61 "goto-dialog.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 61 "goto-dialog.vala"
	g_return_val_if_fail (sender != NULL, FALSE);
#line 164 "goto-dialog.c"
	catched = FALSE;
#line 65 "goto-dialog.vala"
	switch ((*event).keyval) {
#line 168 "goto-dialog.c"
		case GDK_Return:
		case GDK_Tab:
		{
#line 69 "goto-dialog.vala"
			valide_goto_dialog_hide_search (self, FALSE);
#line 70 "goto-dialog.vala"
			catched = TRUE;
#line 71 "goto-dialog.vala"
			break;
#line 178 "goto-dialog.c"
		}
		case GDK_Escape:
		{
#line 74 "goto-dialog.vala"
			valide_goto_dialog_hide_search (self, TRUE);
#line 75 "goto-dialog.vala"
			catched = TRUE;
#line 76 "goto-dialog.vala"
			break;
#line 188 "goto-dialog.c"
		}
	}
	result = catched;
	return result;
}


#line 81 "goto-dialog.vala"
static void valide_goto_dialog_changed_cb (ValideGotoDialog* self) {
#line 81 "goto-dialog.vala"
	g_return_if_fail (self != NULL);
#line 83 "goto-dialog.vala"
	if (_vala_strcmp0 (gtk_entry_get_text (self->priv->entry), "") != 0) {
#line 202 "goto-dialog.c"
		gint line;
		gboolean moved;
		GdkColor color = {0};
		line = 0;
		moved = FALSE;
#line 89 "goto-dialog.vala"
		line = atoi (gtk_entry_get_text (self->priv->entry));
#line 90 "goto-dialog.vala"
		if (line > 0) {
#line 92 "goto-dialog.vala"
			line--;
#line 214 "goto-dialog.c"
		}
#line 94 "goto-dialog.vala"
		moved = valide_source_view_goto_line (self->priv->_source_view, line);
#line 96 "goto-dialog.vala"
		if (!moved) {
#line 98 "goto-dialog.vala"
			gdk_color_parse ("#FFBFBF", &color);
#line 222 "goto-dialog.c"
		} else {
#line 102 "goto-dialog.vala"
			gdk_color_parse ("white", &color);
#line 226 "goto-dialog.c"
		}
#line 104 "goto-dialog.vala"
		gtk_widget_modify_base ((GtkWidget*) self->priv->entry, GTK_STATE_NORMAL, &color);
#line 230 "goto-dialog.c"
	}
}


#line 108 "goto-dialog.vala"
static void valide_goto_dialog_insert_text_cb (ValideGotoDialog* self, GtkEditable* sender, const char* new_text, gint new_text_length, void* position) {
#line 237 "goto-dialog.c"
	gint i;
	gunichar c;
#line 108 "goto-dialog.vala"
	g_return_if_fail (self != NULL);
#line 108 "goto-dialog.vala"
	g_return_if_fail (sender != NULL);
#line 108 "goto-dialog.vala"
	g_return_if_fail (new_text != NULL);
#line 246 "goto-dialog.c"
	i = 0;
	c = 0U;
#line 114 "goto-dialog.vala"
	while (TRUE) {
#line 114 "goto-dialog.vala"
		if (!(i < new_text_length)) {
#line 114 "goto-dialog.vala"
			break;
#line 255 "goto-dialog.c"
		}
#line 116 "goto-dialog.vala"
		c = g_utf8_get_char (g_utf8_offset_to_pointer (new_text, i));
#line 118 "goto-dialog.vala"
		if (!g_unichar_isdigit (c)) {
#line 120 "goto-dialog.vala"
			g_signal_stop_emission_by_name (sender, "insert_text");
#line 121 "goto-dialog.vala"
			break;
#line 265 "goto-dialog.c"
		}
#line 123 "goto-dialog.vala"
		i++;
#line 269 "goto-dialog.c"
	}
}


/**
   * Create a new Valide.GotoDialog
   *
   * @param source_view The attached source view
   */
#line 132 "goto-dialog.vala"
ValideGotoDialog* valide_goto_dialog_construct (GType object_type, ValideSourceView* source_view) {
#line 281 "goto-dialog.c"
	GParameter * __params;
	GParameter * __params_it;
	ValideGotoDialog * self;
#line 132 "goto-dialog.vala"
	g_return_val_if_fail (source_view != NULL, NULL);
#line 287 "goto-dialog.c"
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
#line 134 "goto-dialog.vala"
	__params_it->name = "source-view";
#line 134 "goto-dialog.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_SOURCE_VIEW);
#line 134 "goto-dialog.vala"
	g_value_set_object (&__params_it->value, source_view);
#line 134 "goto-dialog.vala"
	__params_it++;
#line 298 "goto-dialog.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 132 "goto-dialog.vala"
ValideGotoDialog* valide_goto_dialog_new (ValideSourceView* source_view) {
#line 132 "goto-dialog.vala"
	return valide_goto_dialog_construct (VALIDE_TYPE_GOTO_DIALOG, source_view);
#line 313 "goto-dialog.c"
}


/**
   * @see Gtk.Widget.Show
   */
#line 187 "goto-dialog.vala"
void valide_goto_dialog_show (ValideGotoDialog* self) {
#line 187 "goto-dialog.vala"
	g_return_if_fail (self != NULL);
#line 189 "goto-dialog.vala"
	GTK_WIDGET_CLASS (valide_goto_dialog_parent_class)->show ((GtkWidget*) GTK_WINDOW (self));
#line 190 "goto-dialog.vala"
	valide_goto_dialog_update_position (self);
#line 191 "goto-dialog.vala"
	gtk_widget_grab_focus ((GtkWidget*) self->priv->entry);
#line 330 "goto-dialog.c"
}


ValideSourceView* valide_goto_dialog_get_source_view (ValideGotoDialog* self) {
	ValideSourceView* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_source_view;
	return result;
}


static void valide_goto_dialog_set_source_view (ValideGotoDialog* self, ValideSourceView* value) {
	ValideSourceView* _tmp1_;
	ValideSourceView* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 35 "goto-dialog.vala"
	_tmp0_ = NULL;
#line 349 "goto-dialog.c"
	self->priv->_source_view = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_source_view == NULL) ? NULL : (self->priv->_source_view = (g_object_unref (self->priv->_source_view), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "source-view");
}


static void _lambda10_ (ValideGotoDialog* self) {
#line 166 "goto-dialog.vala"
	valide_goto_dialog_hide_search (self, FALSE);
#line 358 "goto-dialog.c"
}


static void __lambda10__gtk_entry_activate (GtkEntry* _sender, gpointer self) {
	_lambda10_ (self);
}


static gboolean _lambda11_ (ValideGotoDialog* self) {
	gboolean result;
#line 170 "goto-dialog.vala"
	valide_goto_dialog_hide_search (self, FALSE);
#line 371 "goto-dialog.c"
	result = FALSE;
	return result;
}


static gboolean __lambda11__gtk_widget_delete_event (GtkWidget* _sender, GdkEvent* event, gpointer self) {
	return _lambda11_ (self);
}


static gboolean _lambda12_ (ValideGotoDialog* self) {
	gboolean result;
#line 175 "goto-dialog.vala"
	valide_goto_dialog_hide_search (self, TRUE);
#line 386 "goto-dialog.c"
	result = FALSE;
	return result;
}


static gboolean __lambda12__gtk_widget_focus_out_event (GtkWidget* _sender, const GdkEventFocus* event, gpointer self) {
	return _lambda12_ (self);
}


#line 81 "goto-dialog.vala"
static void _valide_goto_dialog_changed_cb_gtk_editable_changed (GtkEditable* _sender, gpointer self) {
#line 399 "goto-dialog.c"
	valide_goto_dialog_changed_cb (self);
}


#line 108 "goto-dialog.vala"
static void _valide_goto_dialog_insert_text_cb_gtk_editable_insert_text (GtkEditable* _sender, const char* text, gint length, void* position, gpointer self) {
#line 406 "goto-dialog.c"
	valide_goto_dialog_insert_text_cb (self, _sender, text, length, position);
}


#line 61 "goto-dialog.vala"
static gboolean _valide_goto_dialog_key_press_event_cb_gtk_widget_key_press_event (GtkWidget* _sender, const GdkEventKey* event, gpointer self) {
#line 413 "goto-dialog.c"
	return valide_goto_dialog_key_press_event_cb (self, _sender, event);
}


static GObject * valide_goto_dialog_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValideGotoDialogClass * klass;
	GObjectClass * parent_class;
	ValideGotoDialog * self;
	klass = VALIDE_GOTO_DIALOG_CLASS (g_type_class_peek (VALIDE_TYPE_GOTO_DIALOG));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_GOTO_DIALOG (obj);
	{
		GtkFrame* frame;
		GtkVBox* vbox;
		GtkWindow* toplevel;
		GtkWindow* _tmp2_;
		GtkWindow* _tmp1_;
		GtkWidget* _tmp0_;
		GtkFrame* _tmp3_;
		GtkVBox* _tmp4_;
		GtkEntry* _tmp5_;
		frame = NULL;
		vbox = NULL;
		toplevel = NULL;
#line 143 "goto-dialog.vala"
		gtk_window_set_decorated ((GtkWindow*) self, FALSE);
#line 144 "goto-dialog.vala"
		gtk_window_set_skip_taskbar_hint ((GtkWindow*) self, TRUE);
#line 145 "goto-dialog.vala"
		gtk_window_set_skip_pager_hint ((GtkWindow*) self, TRUE);
#line 446 "goto-dialog.c"
		_tmp2_ = NULL;
#line 146 "goto-dialog.vala"
		_tmp1_ = NULL;
#line 146 "goto-dialog.vala"
		_tmp0_ = NULL;
#line 146 "goto-dialog.vala"
		toplevel = (_tmp2_ = (_tmp1_ = (_tmp0_ = gtk_widget_get_toplevel ((GtkWidget*) self->priv->_source_view), GTK_IS_WINDOW (_tmp0_) ? ((GtkWindow*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (toplevel == NULL) ? NULL : (toplevel = (g_object_unref (toplevel), NULL)), _tmp2_);
#line 147 "goto-dialog.vala"
		gtk_window_group_add_window (toplevel->group, (GtkWindow*) self);
#line 148 "goto-dialog.vala"
		gtk_window_set_modal ((GtkWindow*) self, TRUE);
#line 458 "goto-dialog.c"
		_tmp3_ = NULL;
#line 150 "goto-dialog.vala"
		frame = (_tmp3_ = g_object_ref_sink ((GtkFrame*) gtk_frame_new (NULL)), (frame == NULL) ? NULL : (frame = (g_object_unref (frame), NULL)), _tmp3_);
#line 151 "goto-dialog.vala"
		gtk_frame_set_shadow_type (frame, GTK_SHADOW_ETCHED_IN);
#line 152 "goto-dialog.vala"
		gtk_container_add ((GtkContainer*) self, (GtkWidget*) frame);
#line 153 "goto-dialog.vala"
		gtk_widget_show ((GtkWidget*) frame);
#line 468 "goto-dialog.c"
		_tmp4_ = NULL;
#line 155 "goto-dialog.vala"
		vbox = (_tmp4_ = g_object_ref_sink ((GtkVBox*) gtk_vbox_new (FALSE, 0)), (vbox == NULL) ? NULL : (vbox = (g_object_unref (vbox), NULL)), _tmp4_);
#line 156 "goto-dialog.vala"
		gtk_container_set_border_width ((GtkContainer*) vbox, (guint) 3);
#line 157 "goto-dialog.vala"
		gtk_container_add ((GtkContainer*) frame, (GtkWidget*) vbox);
#line 158 "goto-dialog.vala"
		gtk_widget_show ((GtkWidget*) vbox);
#line 478 "goto-dialog.c"
		_tmp5_ = NULL;
#line 160 "goto-dialog.vala"
		self->priv->entry = (_tmp5_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (self->priv->entry == NULL) ? NULL : (self->priv->entry = (g_object_unref (self->priv->entry), NULL)), _tmp5_);
#line 161 "goto-dialog.vala"
		gtk_container_add ((GtkContainer*) vbox, (GtkWidget*) self->priv->entry);
#line 162 "goto-dialog.vala"
		gtk_widget_show ((GtkWidget*) self->priv->entry);
#line 163 "goto-dialog.vala"
		gtk_widget_realize ((GtkWidget*) self->priv->entry);
#line 165 "goto-dialog.vala"
		g_signal_connect (self->priv->entry, "activate", (GCallback) __lambda10__gtk_entry_activate, self);
#line 169 "goto-dialog.vala"
		g_signal_connect ((GtkWidget*) self, "delete-event", (GCallback) __lambda11__gtk_widget_delete_event, self);
#line 174 "goto-dialog.vala"
		g_signal_connect ((GtkWidget*) self, "focus-out-event", (GCallback) __lambda12__gtk_widget_focus_out_event, self);
#line 179 "goto-dialog.vala"
		g_signal_connect_object ((GtkEditable*) self->priv->entry, "changed", (GCallback) _valide_goto_dialog_changed_cb_gtk_editable_changed, self, 0);
#line 180 "goto-dialog.vala"
		g_signal_connect_object ((GtkEditable*) self->priv->entry, "insert-text", (GCallback) _valide_goto_dialog_insert_text_cb_gtk_editable_insert_text, self, 0);
#line 181 "goto-dialog.vala"
		g_signal_connect_object ((GtkWidget*) self->priv->entry, "key-press-event", (GCallback) _valide_goto_dialog_key_press_event_cb_gtk_widget_key_press_event, self, 0);
#line 500 "goto-dialog.c"
		(frame == NULL) ? NULL : (frame = (g_object_unref (frame), NULL));
		(vbox == NULL) ? NULL : (vbox = (g_object_unref (vbox), NULL));
		(toplevel == NULL) ? NULL : (toplevel = (g_object_unref (toplevel), NULL));
	}
	return obj;
}


static void valide_goto_dialog_class_init (ValideGotoDialogClass * klass) {
	valide_goto_dialog_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideGotoDialogPrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_goto_dialog_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_goto_dialog_set_property;
	G_OBJECT_CLASS (klass)->constructor = valide_goto_dialog_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_goto_dialog_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_GOTO_DIALOG_SOURCE_VIEW, g_param_spec_object ("source-view", "source-view", "source-view", VALIDE_TYPE_SOURCE_VIEW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void valide_goto_dialog_instance_init (ValideGotoDialog * self) {
	self->priv = VALIDE_GOTO_DIALOG_GET_PRIVATE (self);
}


static void valide_goto_dialog_finalize (GObject* obj) {
	ValideGotoDialog * self;
	self = VALIDE_GOTO_DIALOG (obj);
	(self->priv->entry == NULL) ? NULL : (self->priv->entry = (g_object_unref (self->priv->entry), NULL));
	(self->priv->_source_view == NULL) ? NULL : (self->priv->_source_view = (g_object_unref (self->priv->_source_view), NULL));
	G_OBJECT_CLASS (valide_goto_dialog_parent_class)->finalize (obj);
}


GType valide_goto_dialog_get_type (void) {
	static GType valide_goto_dialog_type_id = 0;
	if (valide_goto_dialog_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideGotoDialogClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_goto_dialog_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideGotoDialog), 0, (GInstanceInitFunc) valide_goto_dialog_instance_init, NULL };
		valide_goto_dialog_type_id = g_type_register_static (GTK_TYPE_WINDOW, "ValideGotoDialog", &g_define_type_info, 0);
	}
	return valide_goto_dialog_type_id;
}


static void valide_goto_dialog_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideGotoDialog * self;
	gpointer boxed;
	self = VALIDE_GOTO_DIALOG (object);
	switch (property_id) {
		case VALIDE_GOTO_DIALOG_SOURCE_VIEW:
		g_value_set_object (value, valide_goto_dialog_get_source_view (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_goto_dialog_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideGotoDialog * self;
	self = VALIDE_GOTO_DIALOG (object);
	switch (property_id) {
		case VALIDE_GOTO_DIALOG_SOURCE_VIEW:
		valide_goto_dialog_set_source_view (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




