/* ************
 *
 * Copyright (C) 2009  Yu Feng
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to 
 *
 * the Free Software Foundation, Inc., 
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Yu Feng <rainwoodman@gmail.com>
 *
 * This work is sponsed by C.Y Liu at Indiana University Cyclotron Facility.
 ***/

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <yaml.h>
#include <float.h>
#include <math.h>
#include <gmodule.h>
#include <gobject/gvaluecollector.h>


#define G_YAML_TYPE_BUILDER (g_yaml_builder_get_type ())
#define G_YAML_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_TYPE_BUILDER, GYAMLBuilder))
#define G_YAML_BUILDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), G_YAML_TYPE_BUILDER, GYAMLBuilderClass))
#define G_YAML_IS_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_TYPE_BUILDER))
#define G_YAML_IS_BUILDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_YAML_TYPE_BUILDER))
#define G_YAML_BUILDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), G_YAML_TYPE_BUILDER, GYAMLBuilderClass))

typedef struct _GYAMLBuilder GYAMLBuilder;
typedef struct _GYAMLBuilderClass GYAMLBuilderClass;
typedef struct _GYAMLBuilderPrivate GYAMLBuilderPrivate;

#define G_YAML_TYPE_DOCUMENT (g_yaml_document_get_type ())
#define G_YAML_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_TYPE_DOCUMENT, GYAMLDocument))
#define G_YAML_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), G_YAML_TYPE_DOCUMENT, GYAMLDocumentClass))
#define G_YAML_IS_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_TYPE_DOCUMENT))
#define G_YAML_IS_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_YAML_TYPE_DOCUMENT))
#define G_YAML_DOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), G_YAML_TYPE_DOCUMENT, GYAMLDocumentClass))

typedef struct _GYAMLDocument GYAMLDocument;
typedef struct _GYAMLDocumentClass GYAMLDocumentClass;

#define G_YAML_BUILDER_TYPE___ENUM (g_yaml_builder___enum_get_type ())

#define G_YAML_TYPE_NODE (g_yaml_node_get_type ())
#define G_YAML_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_TYPE_NODE, GYAMLNode))
#define G_YAML_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), G_YAML_TYPE_NODE, GYAMLNodeClass))
#define G_YAML_IS_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_TYPE_NODE))
#define G_YAML_IS_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_YAML_TYPE_NODE))
#define G_YAML_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), G_YAML_TYPE_NODE, GYAMLNodeClass))

typedef struct _GYAMLNode GYAMLNode;
typedef struct _GYAMLNodeClass GYAMLNodeClass;
typedef struct _GYAMLDocumentPrivate GYAMLDocumentPrivate;

#define G_YAML_NODE_TYPE_MAPPING (g_yaml_node_mapping_get_type ())
#define G_YAML_NODE_MAPPING(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_NODE_TYPE_MAPPING, GYAMLNodeMapping))
#define G_YAML_NODE_MAPPING_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), G_YAML_NODE_TYPE_MAPPING, GYAMLNodeMappingClass))
#define G_YAML_NODE_IS_MAPPING(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_NODE_TYPE_MAPPING))
#define G_YAML_NODE_IS_MAPPING_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_YAML_NODE_TYPE_MAPPING))
#define G_YAML_NODE_MAPPING_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), G_YAML_NODE_TYPE_MAPPING, GYAMLNodeMappingClass))

typedef struct _GYAMLNodeMapping GYAMLNodeMapping;
typedef struct _GYAMLNodeMappingClass GYAMLNodeMappingClass;
typedef struct _GYAMLNodePrivate GYAMLNodePrivate;

#define G_YAML_TYPE_BUILDABLE (g_yaml_buildable_get_type ())
#define G_YAML_BUILDABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_TYPE_BUILDABLE, GYAMLBuildable))
#define G_YAML_IS_BUILDABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_TYPE_BUILDABLE))
#define G_YAML_BUILDABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), G_YAML_TYPE_BUILDABLE, GYAMLBuildableIface))

typedef struct _GYAMLBuildable GYAMLBuildable;
typedef struct _GYAMLBuildableIface GYAMLBuildableIface;
typedef struct _GYAMLNodeMappingPrivate GYAMLNodeMappingPrivate;

#define G_YAML_NODE_TYPE_SCALAR (g_yaml_node_scalar_get_type ())
#define G_YAML_NODE_SCALAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_NODE_TYPE_SCALAR, GYAMLNodeScalar))
#define G_YAML_NODE_SCALAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), G_YAML_NODE_TYPE_SCALAR, GYAMLNodeScalarClass))
#define G_YAML_NODE_IS_SCALAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_NODE_TYPE_SCALAR))
#define G_YAML_NODE_IS_SCALAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_YAML_NODE_TYPE_SCALAR))
#define G_YAML_NODE_SCALAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), G_YAML_NODE_TYPE_SCALAR, GYAMLNodeScalarClass))

typedef struct _GYAMLNodeScalar GYAMLNodeScalar;
typedef struct _GYAMLNodeScalarClass GYAMLNodeScalarClass;
typedef struct _GYAMLNodeScalarPrivate GYAMLNodeScalarPrivate;

#define G_YAML_NODE_TYPE_SEQUENCE (g_yaml_node_sequence_get_type ())
#define G_YAML_NODE_SEQUENCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_NODE_TYPE_SEQUENCE, GYAMLNodeSequence))
#define G_YAML_NODE_SEQUENCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), G_YAML_NODE_TYPE_SEQUENCE, GYAMLNodeSequenceClass))
#define G_YAML_NODE_IS_SEQUENCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_NODE_TYPE_SEQUENCE))
#define G_YAML_NODE_IS_SEQUENCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_YAML_NODE_TYPE_SEQUENCE))
#define G_YAML_NODE_SEQUENCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), G_YAML_NODE_TYPE_SEQUENCE, GYAMLNodeSequenceClass))

typedef struct _GYAMLNodeSequence GYAMLNodeSequence;
typedef struct _GYAMLNodeSequenceClass GYAMLNodeSequenceClass;
typedef struct _GYAMLNodeSequencePrivate GYAMLNodeSequencePrivate;

#define G_YAML_TYPE_DEMANGLER (g_yaml_demangler_get_type ())
#define G_YAML_DEMANGLER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_TYPE_DEMANGLER, GYAMLDemangler))
#define G_YAML_DEMANGLER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), G_YAML_TYPE_DEMANGLER, GYAMLDemanglerClass))
#define G_YAML_IS_DEMANGLER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_TYPE_DEMANGLER))
#define G_YAML_IS_DEMANGLER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_YAML_TYPE_DEMANGLER))
#define G_YAML_DEMANGLER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), G_YAML_TYPE_DEMANGLER, GYAMLDemanglerClass))

typedef struct _GYAMLDemangler GYAMLDemangler;
typedef struct _GYAMLDemanglerClass GYAMLDemanglerClass;
typedef struct _GYAMLDemanglerPrivate GYAMLDemanglerPrivate;
typedef struct _GYAMLParamSpecDemangler GYAMLParamSpecDemangler;

/**
 * Building GObjects from a YAML stream.
 *
 * Refer to GLib.YAML.Buildable
 *
 * The root of the document is a mapping, which is built to an object. 
 * Other objects are inserted into children of the root object, 
 * supplied as items in sequences.
 *
 * ++ Two Stages of Building ++
 * # An object is created for each mapping with a YAML tag that 
 *   corresponds to a valid GObject type.
 * # Every mapping corresponding to an object is examined. 
 *   Each mapping key in the pairs is scanned to match the 
 *   rules discussed in the following text.
 *   # `internals'
 *     * Internal children are referred by their internal names.
 *       Buildables should implement 
 *       Buildable.get_internel_child which returns the child object.
 *   # `objects', and other custom children key
 *     * Every child is supplied as a sequence item in the 
 *       mapping value. Buildables should implement 
 *       Buildable.get_child_type which returns the GType of 
 *       the custom child for the given key scalar.
 *   # properties
 *     * The key of the mapping matched against a property name 
 *       of the object. If found, the value is parsed to fill 
 *       in the value.
 *     * If the property refers to an Object and the value node is 
 *       a mapping, a new object is created, and the object is 
 *       initialized by the supplied mapping.
 *     * If the property refers to an Object and the value node is 
 *       an alias, the object referred by the alias is 
 *       assigned to the property.
 *     * If the property refers to a Boxed structure, 
 *       a temperory buffer of 65530 bytes is allocated, 
 *       and a parse function, conforming the gdk_color_parse 
 *       calling convention ([CCode (instance_pos = -1)] 
 *       is called to initialize the buffer with the value scalar.
 *   # signals
 *     * Signal connection is not currently supported, but planned.
 *   # custom nodes
 *     * If the key doesn't match to any of the priorly mentioned 
 *       catagories, Buildable.custom_node is invoked. 
 *     * If the buildable doesn't understand the node, 
 *       it should report a PROPERTY_NOT_FOUND error.
 *
 * [warning:
 *   Be aware that if the Boxed struct is larger than the temporary buffer, 
 *   there will be a memory corruption. Currently no good fix is found, 
 *   because GLib doesn't store the size of a Boxed type.
 * ]
 *
 * Here is an example of the YAML file, 
 * which is semantically equivalent to the GtkBuilder example given in 
 * DevHelp, GtkBuildable.
 *
 * {{{
 * --- !GtkDialog
 * internals:
 *   vbox : { border-width : 10 }
 *   action-area : 
 *     border-width : 20
 *     objects:
 *       - !GtkHButtonBox &hbuttonbox1
 *         objects:
 *         - !GtkButton &ok_button
 *           label: gtk-ok
 *           use-stock: true
 * }}}
 *
 * [warning:
 *   Although this example is a valid YAML understood by the Builder, 
 *   it cannot be used to build the widgets, because GtkWidget 
 *   doesn't implement the required GYAMLBuildable interface.
 * ]
 *
 *
 * */
struct _GYAMLBuilder {
	GObject parent_instance;
	GYAMLBuilderPrivate * priv;
};

struct _GYAMLBuilderClass {
	GObjectClass parent_class;
};

struct _GYAMLBuilderPrivate {
	char* prefix;
	GHashTable* anchors;
	GList* objects;
	GYAMLDocument* document;
};

typedef enum  {
	G_YAML_BUILDER___ENUM_FOO
} GYAMLBuilder__enum;

typedef enum  {
	G_YAML_ERROR_SYMBOL_NOT_FOUND,
	G_YAML_ERROR_TYPE_NOT_FOUND,
	G_YAML_ERROR_NOT_A_BUILDABLE,
	G_YAML_ERROR_PROPERTY_NOT_FOUND,
	G_YAML_ERROR_OBJECT_NOT_FOUND,
	G_YAML_ERROR_UNKNOWN_PROPERTY_TYPE,
	G_YAML_ERROR_CUSTOM_NODE_ERROR,
	G_YAML_ERROR_UNEXPECTED_NODE,
	G_YAML_ERROR_NOT_IMPLEMENTED,
	G_YAML_ERROR_FILE_NOT_FOUND,
	G_YAML_ERROR_PARSER_ERROR,
	G_YAML_ERROR_UNRESOLVED_ALIAS
} GYAMLError;
#define G_YAML_ERROR g_yaml_error_quark ()
/**
 * A YAML Document
 *
 * Refer to the YAML 1.1 spec for the definitions.
 *
 * The document model based on GType classes replaces the original libyaml
 * document model.
 *
 * [warning:
 *  This is not a full implementation of a YAML document.
 *  The document tag directive is missing.
 *  Alias is not immediately resolved and replaced with the referred node.
 * ]
 */
struct _GYAMLDocument {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GYAMLDocumentPrivate * priv;
	GList* nodes;
	yaml_mark_t start_mark;
	yaml_mark_t end_mark;
	GHashTable* anchors;
	GYAMLNode* root;
};

struct _GYAMLDocumentClass {
	GTypeClass parent_class;
	void (*finalize) (GYAMLDocument *self);
};

/**
 * A YAML Node.
 *
 * YAML supports three types of nodes. They are converted to
 * GTypes
 *
 *   [| ++YAML++ || ++GType++ |]
 *   [| Scalar || Node.Scalar |]
 *   [| Sequence || Node.Sequence |]
 *   [| Mapping || Node.Mapping |]
 *   [| Alias || Node.Alias |]
 *
 * Each node type utilizes the fundamental GLib types to store the
 * YAML data.
 *
 * A pointer can be binded to the node with get_pointer and set_pointer.
 * This pointer is used by GLib.YAML.Builder to hold the built object.
 *
 * */
struct _GYAMLNode {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GYAMLNodePrivate * priv;
	yaml_node_type_t type;
	char* tag;
	yaml_mark_t start_mark;
	yaml_mark_t end_mark;
	char* anchor;
};

struct _GYAMLNodeClass {
	GTypeClass parent_class;
	void (*finalize) (GYAMLNode *self);
};

/**
 * Buildable GObjects, from YAML.
 *
 *
 * Objects that implements GLib.YAML.Buildable is buildable by
 * GLib.YAML.Builder.
 *
 * This interface is almost the same as GtkBuildable.
 *
 **/
struct _GYAMLBuildableIface {
	GTypeInterface parent_iface;
	const char* (*get_name) (GYAMLBuildable* self);
	void (*set_name) (GYAMLBuildable* self, const char* name);
	void (*add_child) (GYAMLBuildable* self, GYAMLBuilder* builder, GObject* child, const char* type, GError** error);
	char** (*get_child_tags) (GYAMLBuildable* self, int* result_length1);
	GType* (*get_child_types) (GYAMLBuildable* self, int* result_length1);
	GList* (*get_children) (GYAMLBuildable* self, const char* type);
	GObject* (*get_internal_child) (GYAMLBuildable* self, GYAMLBuilder* builder, const char* child_name);
	void (*custom_node) (GYAMLBuildable* self, GYAMLBuilder* builder, const char* tag, GYAMLNode* node, GError** error);
};

/**
 * A Mapping Node
 * 
 * Refer to the YAML 1.1 spec for the definitions.
 *
 * The mapping is internally stored as a GHashTable.
 *
 * An extra list of keys is stored to ease iterating overall
 * elements. GHashTable.get_keys is not available in GLib 2.12.
 */
struct _GYAMLNodeMapping {
	GYAMLNode parent_instance;
	GYAMLNodeMappingPrivate * priv;
	GHashTable* pairs;
	GList* keys;
	yaml_mapping_style_t style;
};

struct _GYAMLNodeMappingClass {
	GYAMLNodeClass parent_class;
};

typedef gboolean (*GYAMLBuilderParseFunc) (const char* foo, void* location);
/**
 * A Scalar Node
 *
 * Refer to the YAML 1.1 spec for the definitions.
 *
 * The scalar value is internally stored as a string,
 * or `gchar*'.
 */
struct _GYAMLNodeScalar {
	GYAMLNode parent_instance;
	GYAMLNodeScalarPrivate * priv;
	char* value;
	yaml_scalar_style_t style;
};

struct _GYAMLNodeScalarClass {
	GYAMLNodeClass parent_class;
};

/**
 * A Sequence Node
 * 
 * Refer to the YAML 1.1 spec for the definitions.
 *
 * The sequence is internally stored as a GList.
 */
struct _GYAMLNodeSequence {
	GYAMLNode parent_instance;
	GYAMLNodeSequencePrivate * priv;
	GList* items;
	yaml_sequence_style_t style;
};

struct _GYAMLNodeSequenceClass {
	GYAMLNodeClass parent_class;
};

/**
 * Demangle vala names to c names in the standard way.
 *
 * [warning:
 *   GI information is not used, therefore 
 *   if there is any special tricks by the CCode annotations, 
 *   you have to be careful and understand what you are doing. 
 * ]
 **/
struct _GYAMLDemangler {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GYAMLDemanglerPrivate * priv;
};

struct _GYAMLDemanglerClass {
	GTypeClass parent_class;
	void (*finalize) (GYAMLDemangler *self);
};

typedef GType (*GYAMLDemanglerTypeFunc) ();
struct _GYAMLParamSpecDemangler {
	GParamSpec parent_instance;
};


static GType g_yaml_builder_enum_type;
static GType g_yaml_builder_enum_type = 0UL;
static gpointer g_yaml_builder_parent_class = NULL;
static gpointer g_yaml_demangler_parent_class = NULL;

GType g_yaml_builder_get_type (void);
gpointer g_yaml_document_ref (gpointer instance);
void g_yaml_document_unref (gpointer instance);
GParamSpec* g_yaml_param_spec_document (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void g_yaml_value_set_document (GValue* value, gpointer v_object);
gpointer g_yaml_value_get_document (const GValue* value);
GType g_yaml_document_get_type (void);
#define G_YAML_BUILDER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), G_YAML_TYPE_BUILDER, GYAMLBuilderPrivate))
enum  {
	G_YAML_BUILDER_DUMMY_PROPERTY
};
static GType g_yaml_builder___enum_get_type (void);
static void _g_list_free_g_object_unref (GList* self);
GYAMLBuilder* g_yaml_builder_new (const char* prefix);
GYAMLBuilder* g_yaml_builder_construct (GType object_type, const char* prefix);
GQuark g_yaml_error_quark (void);
GYAMLDocument* g_yaml_document_new_from_string (const char* str, GError** error);
GYAMLDocument* g_yaml_document_construct_from_string (GType object_type, const char* str, GError** error);
static void g_yaml_builder_bootstrap_objects (GYAMLBuilder* self, GYAMLDocument* document, GError** error);
static void g_yaml_builder_process_value_nodes (GYAMLBuilder* self, GError** error);
void g_yaml_builder_add_from_string (GYAMLBuilder* self, const char* str, GError** error);
GYAMLDocument* g_yaml_document_new_from_file (FILE* file, GError** error);
GYAMLDocument* g_yaml_document_construct_from_file (GType object_type, FILE* file, GError** error);
void g_yaml_builder_add_from_file (GYAMLBuilder* self, FILE* file, GError** error);
GObject* g_yaml_builder_get_root_object (GYAMLBuilder* self);
GObject* g_yaml_builder_build_from_string (GYAMLBuilder* self, const char* str, GError** error);
GObject* g_yaml_builder_build_from_file (GYAMLBuilder* self, FILE* file, GError** error);
gpointer g_yaml_node_ref (gpointer instance);
void g_yaml_node_unref (gpointer instance);
GParamSpec* g_yaml_param_spec_node (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void g_yaml_value_set_node (GValue* value, gpointer v_object);
gpointer g_yaml_value_get_node (const GValue* value);
GType g_yaml_node_get_type (void);
void* g_yaml_node_get_pointer (GYAMLNode* self);
static GObject* g_yaml_builder_bootstrap_object (GYAMLBuilder* self, GYAMLNode* node, GType type, GError** error);
static void g_yaml_builder_process_object_value_node (GYAMLBuilder* self, GObject* obj, GYAMLNode* node, GError** error);
GObject* g_yaml_builder_build_object (GYAMLBuilder* self, GYAMLNode* node, GType type, GError** error);
static char* g_yaml_builder_get_full_class_name (GYAMLBuilder* self, const char* class_name);
GType g_yaml_node_mapping_get_type (void);
GType g_yaml_demangler_resolve_type (const char* class_name, GError** error);
GType g_yaml_buildable_get_type (void);
void g_yaml_buildable_set_name (GYAMLBuildable* self, const char* name);
void g_yaml_node_set_pointer (GYAMLNode* self, void* pointer, GDestroyNotify notify);
GType g_yaml_buildable_get_child_type (GYAMLBuildable* self, GYAMLBuilder* builder, const char* tag);
static GType g_yaml_builder_get_child_type (GYAMLBuilder* self, GObject* obj, const char* tag);
static const char* g_yaml_builder_cast_to_scalar (GYAMLBuilder* self, GYAMLNode* node, GError** error);
GYAMLNode* g_yaml_node_get_resolved (GYAMLNode* self);
static void g_yaml_builder_process_children (GYAMLBuilder* self, GObject* obj, const char* type, GYAMLNode* node, GError** error);
static void g_yaml_builder_process_property (GYAMLBuilder* self, GObject* obj, GParamSpec* pspec, GYAMLNode* node, GError** error);
void g_yaml_buildable_custom_node (GYAMLBuildable* self, GYAMLBuilder* builder, const char* tag, GYAMLNode* node, GError** error);
GType g_yaml_node_scalar_get_type (void);
GObject* g_yaml_builder_get_object (GYAMLBuilder* self, const char* anchor);
void* g_yaml_demangler_resolve_function (const char* class_name, const char* member_name, GError** error);
GObject* g_yaml_buildable_get_internal_child (GYAMLBuildable* self, GYAMLBuilder* builder, const char* child_name);
static void g_yaml_builder_process_internal_children (GYAMLBuilder* self, GObject* obj, GYAMLNode* node, GError** error);
GType g_yaml_node_sequence_get_type (void);
void g_yaml_buildable_add_child (GYAMLBuildable* self, GYAMLBuilder* builder, GObject* child, const char* type, GError** error);
GList* g_yaml_builder_get_objects (GYAMLBuilder* self);
static void g_yaml_builder_finalize (GObject* obj);
gpointer g_yaml_demangler_ref (gpointer instance);
void g_yaml_demangler_unref (gpointer instance);
GParamSpec* g_yaml_param_spec_demangler (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void g_yaml_value_set_demangler (GValue* value, gpointer v_object);
gpointer g_yaml_value_get_demangler (const GValue* value);
GType g_yaml_demangler_get_type (void);
enum  {
	G_YAML_DEMANGLER_DUMMY_PROPERTY
};
char* g_yaml_demangler_demangle (const char* vala_name);
GYAMLDemangler* g_yaml_demangler_new (void);
GYAMLDemangler* g_yaml_demangler_construct (GType object_type);
static void g_yaml_demangler_finalize (GYAMLDemangler* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);




static GType g_yaml_builder___enum_get_type (void) {
	static GType g_yaml_builder___enum_type_id = 0;
	if (G_UNLIKELY (g_yaml_builder___enum_type_id == 0)) {
		static const GEnumValue values[] = {{G_YAML_BUILDER___ENUM_FOO, "G_YAML_BUILDER___ENUM_FOO", "foo"}, {0, NULL, NULL}};
		g_yaml_builder___enum_type_id = g_enum_register_static ("GYAMLBuilder__enum", values);
	}
	return g_yaml_builder___enum_type_id;
}


static void _g_list_free_g_object_unref (GList* self) {
	g_list_foreach (self, (GFunc) g_object_unref, NULL);
	g_list_free (self);
}


/**
 * Create a builder with the given prefix.
 **/
#line 126 "builder.vala"
GYAMLBuilder* g_yaml_builder_construct (GType object_type, const char* prefix) {
#line 518 "builder.c"
	GYAMLBuilder * self;
	char* _tmp1_;
	const char* _tmp0_;
	self = g_object_newv (object_type, 0, NULL);
	_tmp1_ = NULL;
#line 127 "builder.vala"
	_tmp0_ = NULL;
#line 127 "builder.vala"
	self->priv->prefix = (_tmp1_ = (_tmp0_ = prefix, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->prefix = (g_free (self->priv->prefix), NULL), _tmp1_);
#line 528 "builder.c"
	return self;
}


#line 126 "builder.vala"
GYAMLBuilder* g_yaml_builder_new (const char* prefix) {
#line 126 "builder.vala"
	return g_yaml_builder_construct (G_YAML_TYPE_BUILDER, prefix);
#line 537 "builder.c"
}


/**
 * Add objects from a string
 **/
#line 132 "builder.vala"
void g_yaml_builder_add_from_string (GYAMLBuilder* self, const char* str, GError** error) {
#line 546 "builder.c"
	GError * _inner_error_;
	GYAMLDocument* _tmp0_;
	GYAMLDocument* _tmp1_;
#line 132 "builder.vala"
	g_return_if_fail (self != NULL);
#line 132 "builder.vala"
	g_return_if_fail (str != NULL);
#line 554 "builder.c"
	_inner_error_ = NULL;
#line 133 "builder.vala"
	g_warning ("builder.vala:133: this function is deprecated.");
#line 134 "builder.vala"
	g_assert (self->priv->document == NULL);
#line 560 "builder.c"
	_tmp0_ = g_yaml_document_new_from_string (str, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return;
	}
	_tmp1_ = NULL;
#line 135 "builder.vala"
	self->priv->document = (_tmp1_ = _tmp0_, (self->priv->document == NULL) ? NULL : (self->priv->document = (g_yaml_document_unref (self->priv->document), NULL)), _tmp1_);
#line 136 "builder.vala"
	g_yaml_builder_bootstrap_objects (self, self->priv->document, &_inner_error_);
#line 571 "builder.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return;
	}
#line 137 "builder.vala"
	g_yaml_builder_process_value_nodes (self, &_inner_error_);
#line 578 "builder.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return;
	}
}


/**
 * Add objects from a file stream
 **/
#line 142 "builder.vala"
void g_yaml_builder_add_from_file (GYAMLBuilder* self, FILE* file, GError** error) {
#line 591 "builder.c"
	GError * _inner_error_;
	GYAMLDocument* _tmp0_;
	GYAMLDocument* _tmp1_;
#line 142 "builder.vala"
	g_return_if_fail (self != NULL);
#line 142 "builder.vala"
	g_return_if_fail (file != NULL);
#line 599 "builder.c"
	_inner_error_ = NULL;
#line 143 "builder.vala"
	g_warning ("builder.vala:143: this function is deprecated.");
#line 144 "builder.vala"
	g_assert (self->priv->document == NULL);
#line 605 "builder.c"
	_tmp0_ = g_yaml_document_new_from_file (file, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return;
	}
	_tmp1_ = NULL;
#line 145 "builder.vala"
	self->priv->document = (_tmp1_ = _tmp0_, (self->priv->document == NULL) ? NULL : (self->priv->document = (g_yaml_document_unref (self->priv->document), NULL)), _tmp1_);
#line 146 "builder.vala"
	g_yaml_builder_bootstrap_objects (self, self->priv->document, &_inner_error_);
#line 616 "builder.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return;
	}
#line 147 "builder.vala"
	g_yaml_builder_process_value_nodes (self, &_inner_error_);
#line 623 "builder.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return;
	}
}


/**
 * Build an object from a given string
 **/
#line 153 "builder.vala"
GObject* g_yaml_builder_build_from_string (GYAMLBuilder* self, const char* str, GError** error) {
#line 636 "builder.c"
	GObject* result;
	GError * _inner_error_;
	GYAMLDocument* _tmp0_;
	GYAMLDocument* _tmp1_;
	GYAMLDocument* _tmp2_;
#line 153 "builder.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 153 "builder.vala"
	g_return_val_if_fail (str != NULL, NULL);
#line 646 "builder.c"
	_inner_error_ = NULL;
	_tmp0_ = NULL;
#line 154 "builder.vala"
	self->priv->document = (_tmp0_ = NULL, (self->priv->document == NULL) ? NULL : (self->priv->document = (g_yaml_document_unref (self->priv->document), NULL)), _tmp0_);
#line 651 "builder.c"
	_tmp1_ = g_yaml_document_new_from_string (str, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return NULL;
	}
	_tmp2_ = NULL;
#line 155 "builder.vala"
	self->priv->document = (_tmp2_ = _tmp1_, (self->priv->document == NULL) ? NULL : (self->priv->document = (g_yaml_document_unref (self->priv->document), NULL)), _tmp2_);
#line 156 "builder.vala"
	g_yaml_builder_bootstrap_objects (self, self->priv->document, &_inner_error_);
#line 662 "builder.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return NULL;
	}
#line 157 "builder.vala"
	g_yaml_builder_process_value_nodes (self, &_inner_error_);
#line 669 "builder.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return NULL;
	}
	result = g_yaml_builder_get_root_object (self);
	return result;
}


/**
 * Build an object from a given filestream
 **/
#line 164 "builder.vala"
GObject* g_yaml_builder_build_from_file (GYAMLBuilder* self, FILE* file, GError** error) {
#line 684 "builder.c"
	GObject* result;
	GError * _inner_error_;
	GYAMLDocument* _tmp0_;
	GYAMLDocument* _tmp1_;
	GYAMLDocument* _tmp2_;
#line 164 "builder.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 164 "builder.vala"
	g_return_val_if_fail (file != NULL, NULL);
#line 694 "builder.c"
	_inner_error_ = NULL;
	_tmp0_ = NULL;
#line 165 "builder.vala"
	self->priv->document = (_tmp0_ = NULL, (self->priv->document == NULL) ? NULL : (self->priv->document = (g_yaml_document_unref (self->priv->document), NULL)), _tmp0_);
#line 699 "builder.c"
	_tmp1_ = g_yaml_document_new_from_file (file, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return NULL;
	}
	_tmp2_ = NULL;
#line 166 "builder.vala"
	self->priv->document = (_tmp2_ = _tmp1_, (self->priv->document == NULL) ? NULL : (self->priv->document = (g_yaml_document_unref (self->priv->document), NULL)), _tmp2_);
#line 167 "builder.vala"
	g_yaml_builder_bootstrap_objects (self, self->priv->document, &_inner_error_);
#line 710 "builder.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return NULL;
	}
#line 168 "builder.vala"
	g_yaml_builder_process_value_nodes (self, &_inner_error_);
#line 717 "builder.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return NULL;
	}
	result = g_yaml_builder_get_root_object (self);
	return result;
}


/**
 * Build a object according to a given type. This function is 
 * intended to be used in implementing Buildable.custom_node .
 *
 * If the an object is already built for the node, it will return 
 * the built object.
 *
 * [warning:
 *   Do not call this function in any places other than 
 *   the implementation of the Builable interface. It depends 
 *   on the internal data structures that are available only in 
 *   the object building process. Once the objects are built, 
 *   these data structures are destroyed.
 * ]
 *
 * @param node
 *   the node to build the object
 * @param type
 *   the expected type of the object
 *
 * @return the built object
 */
#line 194 "builder.vala"
GObject* g_yaml_builder_build_object (GYAMLBuilder* self, GYAMLNode* node, GType type, GError** error) {
#line 751 "builder.c"
	GObject* result;
	GError * _inner_error_;
	GObject* obj;
#line 194 "builder.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 194 "builder.vala"
	g_return_val_if_fail (node != NULL, NULL);
#line 759 "builder.c"
	_inner_error_ = NULL;
#line 195 "builder.vala"
	if (g_yaml_node_get_pointer (node) != NULL) {
#line 763 "builder.c"
		GObject* _tmp0_;
#line 196 "builder.vala"
		_tmp0_ = NULL;
#line 767 "builder.c"
		result = (_tmp0_ = G_OBJECT (g_yaml_node_get_pointer (node)), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
		return result;
	}
	obj = g_yaml_builder_bootstrap_object (self, node, type, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		return NULL;
	}
#line 199 "builder.vala"
	g_yaml_builder_process_object_value_node (self, obj, node, &_inner_error_);
#line 778 "builder.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		(obj == NULL) ? NULL : (obj = (g_object_unref (obj), NULL));
		return NULL;
	}
	result = obj;
	return result;
}


#line 203 "builder.vala"
static char* g_yaml_builder_get_full_class_name (GYAMLBuilder* self, const char* class_name) {
#line 791 "builder.c"
	char* result;
#line 203 "builder.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 203 "builder.vala"
	g_return_val_if_fail (class_name != NULL, NULL);
#line 204 "builder.vala"
	if (self->priv->prefix != NULL) {
#line 799 "builder.c"
		char* _tmp0_;
		char* _tmp1_;
		_tmp0_ = NULL;
#line 205 "builder.vala"
		_tmp1_ = NULL;
#line 805 "builder.c"
		result = (_tmp1_ = g_strconcat (_tmp0_ = g_strconcat (self->priv->prefix, ".", NULL), class_name, NULL), _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_);
		return result;
	} else {
		const char* _tmp2_;
#line 207 "builder.vala"
		_tmp2_ = NULL;
#line 812 "builder.c"
		result = (_tmp2_ = class_name, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_));
		return result;
	}
}


#line 210 "builder.vala"
static void g_yaml_builder_bootstrap_objects (GYAMLBuilder* self, GYAMLDocument* document, GError** error) {
#line 821 "builder.c"
	GError * _inner_error_;
#line 210 "builder.vala"
	g_return_if_fail (self != NULL);
#line 210 "builder.vala"
	g_return_if_fail (document != NULL);
#line 827 "builder.c"
	_inner_error_ = NULL;
	{
		GList* node_collection;
		GList* node_it;
#line 212 "builder.vala"
		node_collection = document->nodes;
#line 834 "builder.c"
		for (node_it = node_collection; node_it != NULL; node_it = node_it->next) {
			GYAMLNode* _tmp1_;
			GYAMLNode* node;
#line 190 "libyaml-glib.vala"
			_tmp1_ = NULL;
#line 212 "builder.vala"
			node = (_tmp1_ = (GYAMLNode*) node_it->data, (_tmp1_ == NULL) ? NULL : g_yaml_node_ref (_tmp1_));
#line 842 "builder.c"
			{
				GObject* _tmp0_;
				/* skip non objects */
#line 214 "builder.vala"
				if (!G_YAML_NODE_IS_MAPPING (node)) {
#line 848 "builder.c"
					(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
#line 214 "builder.vala"
					continue;
#line 852 "builder.c"
				}
#line 215 "builder.vala"
				if (g_utf8_get_char (node->tag) != '!') {
#line 856 "builder.c"
					(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
#line 215 "builder.vala"
					continue;
#line 860 "builder.c"
				}
				/* bootstrap all objects with sufficient information */
#line 233 "builder.vala"
				_tmp0_ = NULL;
#line 217 "builder.vala"
				_tmp0_ = g_yaml_builder_bootstrap_object (self, node, G_TYPE_INVALID, &_inner_error_);
#line 867 "builder.c"
				if (_inner_error_ != NULL) {
					g_propagate_error (error, _inner_error_);
					(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
					return;
				}
				(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
				(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
			}
		}
	}
}


#line 221 "builder.vala"
static void g_yaml_builder_process_value_nodes (GYAMLBuilder* self, GError** error) {
#line 883 "builder.c"
	GError * _inner_error_;
#line 221 "builder.vala"
	g_return_if_fail (self != NULL);
#line 887 "builder.c"
	_inner_error_ = NULL;
	{
		GList* obj_collection;
		GList* obj_it;
#line 222 "builder.vala"
		obj_collection = self->priv->objects;
#line 894 "builder.c"
		for (obj_it = obj_collection; obj_it != NULL; obj_it = obj_it->next) {
			GObject* _tmp1_;
			GObject* obj;
#line 114 "builder.vala"
			_tmp1_ = NULL;
#line 222 "builder.vala"
			obj = (_tmp1_ = (GObject*) obj_it->data, (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_));
#line 902 "builder.c"
			{
				GYAMLNode* _tmp0_;
				GYAMLNode* node;
#line 223 "builder.vala"
				_tmp0_ = NULL;
#line 908 "builder.c"
				node = (_tmp0_ = G_YAML_NODE (g_object_get_data (obj, "node")), (_tmp0_ == NULL) ? NULL : g_yaml_node_ref (_tmp0_));
#line 224 "builder.vala"
				g_yaml_builder_process_object_value_node (self, obj, node, &_inner_error_);
#line 912 "builder.c"
				if (_inner_error_ != NULL) {
					g_propagate_error (error, _inner_error_);
					(obj == NULL) ? NULL : (obj = (g_object_unref (obj), NULL));
					(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
					return;
				}
				(obj == NULL) ? NULL : (obj = (g_object_unref (obj), NULL));
				(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
			}
		}
	}
}


#line 115 "libyaml-1.0.vapi"
static char* yaml_mark_to_string (yaml_mark_t *self) {
#line 929 "builder.c"
	char* result;
	result = g_strdup_printf ("index:%u line:%u column:%u", (guint) (*self).index, (guint) (*self).line, (guint) (*self).column);
	return result;
}


/* 
 * Build a object without setting its properties and children, 
 * if type == Type.INVALID, the type is deducted from the node.tag.
 */
#line 233 "builder.vala"
static GObject* g_yaml_builder_bootstrap_object (GYAMLBuilder* self, GYAMLNode* node, GType type, GError** error) {
#line 942 "builder.c"
	GObject* result;
	GError * _inner_error_;
	char* real_name;
#line 233 "builder.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 233 "builder.vala"
	g_return_val_if_fail (node != NULL, NULL);
#line 950 "builder.c"
	_inner_error_ = NULL;
	real_name = g_yaml_builder_get_full_class_name (self, g_utf8_next_char (node->tag));
#line 235 "builder.vala"
	if (g_yaml_node_get_pointer (node) != NULL) {
#line 955 "builder.c"
		GObject* _tmp0_;
#line 236 "builder.vala"
		_tmp0_ = NULL;
#line 959 "builder.c"
		result = (_tmp0_ = G_OBJECT (g_yaml_node_get_pointer (node)), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
		real_name = (g_free (real_name), NULL);
		return result;
	}
	{
		GObject* obj;
		GObject* _tmp3_;
#line 239 "builder.vala"
		if (type == G_TYPE_INVALID) {
#line 969 "builder.c"
			GType _tmp1_;
			_tmp1_ = g_yaml_demangler_resolve_type (real_name, &_inner_error_);
			if (_inner_error_ != NULL) {
				if (g_error_matches (_inner_error_, G_YAML_ERROR, G_YAML_ERROR_SYMBOL_NOT_FOUND)) {
					goto __catch16_g_yaml_error_symbol_not_found;
				}
				goto __finally16;
			}
#line 239 "builder.vala"
			type = _tmp1_;
#line 980 "builder.c"
		}
#line 241 "builder.vala"
		g_debug ("builder.vala:241: creating object of type `%s'", g_type_name (type));
#line 984 "builder.c"
		obj = g_object_new (type, NULL);
#line 243 "builder.vala"
		g_yaml_buildable_set_name ((GYAMLBuildable*) obj, node->anchor);
#line 244 "builder.vala"
		if (node->anchor != NULL) {
#line 990 "builder.c"
			const char* _tmp2_;
#line 245 "builder.vala"
			_tmp2_ = NULL;
#line 245 "builder.vala"
			g_hash_table_insert (self->priv->anchors, (_tmp2_ = node->anchor, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)), obj);
#line 996 "builder.c"
		}
#line 247 "builder.vala"
		g_yaml_node_set_pointer (node, g_object_ref (obj), g_object_unref);
#line 248 "builder.vala"
		g_object_set_data (obj, "node", node);
#line 249 "builder.vala"
		_tmp3_ = NULL;
#line 249 "builder.vala"
		self->priv->objects = g_list_prepend (self->priv->objects, (_tmp3_ = obj, (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)));
#line 1006 "builder.c"
		result = obj;
		real_name = (g_free (real_name), NULL);
		return result;
	}
	goto __finally16;
	__catch16_g_yaml_error_symbol_not_found:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			char* _tmp4_;
			GError* _tmp5_;
#line 115 "libyaml-1.0.vapi"
			_tmp4_ = NULL;
#line 252 "builder.vala"
			_tmp5_ = NULL;
#line 1024 "builder.c"
			_inner_error_ = (_tmp5_ = g_error_new (G_YAML_ERROR, G_YAML_ERROR_TYPE_NOT_FOUND, "Type %s(%s) is not found", real_name, _tmp4_ = yaml_mark_to_string (&node->start_mark)), _tmp4_ = (g_free (_tmp4_), NULL), _tmp5_);
			if (_inner_error_ != NULL) {
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
				real_name = (g_free (real_name), NULL);
				goto __finally16;
			}
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally16:
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		real_name = (g_free (real_name), NULL);
		return NULL;
	}
	real_name = (g_free (real_name), NULL);
}


#line 258 "builder.vala"
static GType g_yaml_builder_get_child_type (GYAMLBuilder* self, GObject* obj, const char* tag) {
#line 1046 "builder.c"
	GType result;
	GType type;
#line 258 "builder.vala"
	g_return_val_if_fail (self != NULL, 0UL);
#line 258 "builder.vala"
	g_return_val_if_fail (obj != NULL, 0UL);
#line 258 "builder.vala"
	g_return_val_if_fail (tag != NULL, 0UL);
#line 1055 "builder.c"
	type = g_yaml_buildable_get_child_type ((GYAMLBuildable*) obj, self, tag);
#line 260 "builder.vala"
	if (type == G_TYPE_INVALID) {
#line 261 "builder.vala"
		if (_vala_strcmp0 (tag, "objects") == 0) {
#line 1061 "builder.c"
			result = G_TYPE_OBJECT;
			return result;
		}
#line 262 "builder.vala"
		if (_vala_strcmp0 (tag, "internals") == 0) {
#line 1067 "builder.c"
			result = G_TYPE_OBJECT;
			return result;
		}
	}
	result = type;
	return result;
}


#line 267 "builder.vala"
static void g_yaml_builder_process_object_value_node (GYAMLBuilder* self, GObject* obj, GYAMLNode* node, GError** error) {
#line 1079 "builder.c"
	GError * _inner_error_;
	GYAMLNodeMapping* _tmp1_;
	GYAMLNode* _tmp0_;
	GYAMLNodeMapping* mapping;
#line 267 "builder.vala"
	g_return_if_fail (self != NULL);
#line 267 "builder.vala"
	g_return_if_fail (obj != NULL);
#line 267 "builder.vala"
	g_return_if_fail (node != NULL);
#line 1090 "builder.c"
	_inner_error_ = NULL;
#line 268 "builder.vala"
	_tmp1_ = NULL;
#line 268 "builder.vala"
	_tmp0_ = NULL;
#line 1096 "builder.c"
	mapping = (_tmp1_ = (_tmp0_ = node, G_YAML_NODE_IS_MAPPING (_tmp0_) ? ((GYAMLNodeMapping*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_yaml_node_ref (_tmp1_));
	{
		GList* key_node_collection;
		GList* key_node_it;
#line 269 "builder.vala"
		key_node_collection = mapping->keys;
#line 1103 "builder.c"
		for (key_node_it = key_node_collection; key_node_it != NULL; key_node_it = key_node_it->next) {
			GYAMLNode* _tmp3_;
			GYAMLNode* key_node;
#line 170 "libyaml-glib.vala"
			_tmp3_ = NULL;
#line 269 "builder.vala"
			key_node = (_tmp3_ = (GYAMLNode*) key_node_it->data, (_tmp3_ == NULL) ? NULL : g_yaml_node_ref (_tmp3_));
#line 1111 "builder.c"
			{
				const char* key;
				GYAMLNode* value_node;
				GParamSpec* _tmp2_;
				GParamSpec* pspec;
				key = g_yaml_builder_cast_to_scalar (self, key_node, &_inner_error_);
				if (_inner_error_ != NULL) {
					g_propagate_error (error, _inner_error_);
					(key_node == NULL) ? NULL : (key_node = (g_yaml_node_unref (key_node), NULL));
					(mapping == NULL) ? NULL : (mapping = (g_yaml_node_unref (mapping), NULL));
					return;
				}
				value_node = g_yaml_node_get_resolved ((GYAMLNode*) g_hash_table_lookup (mapping->pairs, key_node));
#line 272 "builder.vala"
				if (g_yaml_builder_get_child_type (self, obj, key) != G_TYPE_INVALID) {
#line 273 "builder.vala"
					g_yaml_builder_process_children (self, obj, key, value_node, &_inner_error_);
#line 1129 "builder.c"
					if (_inner_error_ != NULL) {
						g_propagate_error (error, _inner_error_);
						(key_node == NULL) ? NULL : (key_node = (g_yaml_node_unref (key_node), NULL));
						(value_node == NULL) ? NULL : (value_node = (g_yaml_node_unref (value_node), NULL));
						(mapping == NULL) ? NULL : (mapping = (g_yaml_node_unref (mapping), NULL));
						return;
					}
					(key_node == NULL) ? NULL : (key_node = (g_yaml_node_unref (key_node), NULL));
					(value_node == NULL) ? NULL : (value_node = (g_yaml_node_unref (value_node), NULL));
#line 274 "builder.vala"
					continue;
#line 1141 "builder.c"
				}
#line 276 "builder.vala"
				_tmp2_ = NULL;
#line 1145 "builder.c"
				pspec = (_tmp2_ = g_object_class_find_property ((GObjectClass*) g_type_class_peek (G_TYPE_FROM_INSTANCE (obj)), key), (_tmp2_ == NULL) ? NULL : g_param_spec_ref (_tmp2_));
#line 277 "builder.vala"
				if (pspec != NULL) {
#line 278 "builder.vala"
					g_yaml_builder_process_property (self, obj, pspec, value_node, &_inner_error_);
#line 1151 "builder.c"
					if (_inner_error_ != NULL) {
						g_propagate_error (error, _inner_error_);
						(key_node == NULL) ? NULL : (key_node = (g_yaml_node_unref (key_node), NULL));
						(value_node == NULL) ? NULL : (value_node = (g_yaml_node_unref (value_node), NULL));
						(pspec == NULL) ? NULL : (pspec = (g_param_spec_unref (pspec), NULL));
						(mapping == NULL) ? NULL : (mapping = (g_yaml_node_unref (mapping), NULL));
						return;
					}
				} else {
#line 280 "builder.vala"
					g_yaml_buildable_custom_node ((GYAMLBuildable*) obj, self, key, value_node, &_inner_error_);
#line 1163 "builder.c"
					if (_inner_error_ != NULL) {
						g_propagate_error (error, _inner_error_);
						(key_node == NULL) ? NULL : (key_node = (g_yaml_node_unref (key_node), NULL));
						(value_node == NULL) ? NULL : (value_node = (g_yaml_node_unref (value_node), NULL));
						(pspec == NULL) ? NULL : (pspec = (g_param_spec_unref (pspec), NULL));
						(mapping == NULL) ? NULL : (mapping = (g_yaml_node_unref (mapping), NULL));
						return;
					}
				}
				(key_node == NULL) ? NULL : (key_node = (g_yaml_node_unref (key_node), NULL));
				(value_node == NULL) ? NULL : (value_node = (g_yaml_node_unref (value_node), NULL));
				(pspec == NULL) ? NULL : (pspec = (g_param_spec_unref (pspec), NULL));
			}
		}
	}
	(mapping == NULL) ? NULL : (mapping = (g_yaml_node_unref (mapping), NULL));
}


#line 900 "glib-2.0.vapi"
static gboolean string_to_bool (const char* self) {
#line 1185 "builder.c"
	gboolean result;
#line 900 "glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, FALSE);
#line 901 "glib-2.0.vapi"
	if (_vala_strcmp0 (self, "true") == 0) {
#line 1191 "builder.c"
		result = TRUE;
		return result;
	} else {
		result = FALSE;
		return result;
	}
}


#line 285 "builder.vala"
static void g_yaml_builder_process_property (GYAMLBuilder* self, GObject* obj, GParamSpec* pspec, GYAMLNode* node, GError** error) {
#line 1203 "builder.c"
	GError * _inner_error_;
	GValue _tmp0_ = {0};
	GValue gvalue;
#line 285 "builder.vala"
	g_return_if_fail (self != NULL);
#line 285 "builder.vala"
	g_return_if_fail (obj != NULL);
#line 285 "builder.vala"
	g_return_if_fail (pspec != NULL);
#line 285 "builder.vala"
	g_return_if_fail (node != NULL);
#line 1215 "builder.c"
	_inner_error_ = NULL;
	gvalue = (g_value_init (&_tmp0_, pspec->value_type), _tmp0_);
#line 288 "builder.vala"
	if (pspec->value_type == G_TYPE_INT) {
#line 1220 "builder.c"
		const char* _tmp1_;
		_tmp1_ = g_yaml_builder_cast_to_scalar (self, node, &_inner_error_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_YAML_ERROR) {
				g_propagate_error (error, _inner_error_);
				G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
				return;
			} else {
				G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return;
			}
		}
#line 289 "builder.vala"
		g_value_set_int (&gvalue, (gint) atol (_tmp1_));
#line 1237 "builder.c"
	} else {
#line 291 "builder.vala"
		if (pspec->value_type == G_TYPE_UINT) {
#line 1241 "builder.c"
			const char* _tmp2_;
			_tmp2_ = g_yaml_builder_cast_to_scalar (self, node, &_inner_error_);
			if (_inner_error_ != NULL) {
				if (_inner_error_->domain == G_YAML_ERROR) {
					g_propagate_error (error, _inner_error_);
					G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
					return;
				} else {
					G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return;
				}
			}
#line 292 "builder.vala"
			g_value_set_uint (&gvalue, (guint) atol (_tmp2_));
#line 1258 "builder.c"
		} else {
#line 294 "builder.vala"
			if (pspec->value_type == G_TYPE_LONG) {
#line 1262 "builder.c"
				const char* _tmp3_;
				_tmp3_ = g_yaml_builder_cast_to_scalar (self, node, &_inner_error_);
				if (_inner_error_ != NULL) {
					if (_inner_error_->domain == G_YAML_ERROR) {
						g_propagate_error (error, _inner_error_);
						G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
						return;
					} else {
						G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
						g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
						g_clear_error (&_inner_error_);
						return;
					}
				}
#line 295 "builder.vala"
				g_value_set_long (&gvalue, atol (_tmp3_));
#line 1279 "builder.c"
			} else {
#line 297 "builder.vala"
				if (pspec->value_type == G_TYPE_ULONG) {
#line 1283 "builder.c"
					const char* _tmp4_;
					_tmp4_ = g_yaml_builder_cast_to_scalar (self, node, &_inner_error_);
					if (_inner_error_ != NULL) {
						if (_inner_error_->domain == G_YAML_ERROR) {
							g_propagate_error (error, _inner_error_);
							G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
							return;
						} else {
							G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
							g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
							g_clear_error (&_inner_error_);
							return;
						}
					}
#line 298 "builder.vala"
					g_value_set_ulong (&gvalue, strtoul (_tmp4_, NULL, 0));
#line 1300 "builder.c"
				} else {
#line 300 "builder.vala"
					if (pspec->value_type == G_TYPE_STRING) {
#line 1304 "builder.c"
						const char* _tmp5_;
						_tmp5_ = g_yaml_builder_cast_to_scalar (self, node, &_inner_error_);
						if (_inner_error_ != NULL) {
							if (_inner_error_->domain == G_YAML_ERROR) {
								g_propagate_error (error, _inner_error_);
								G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
								return;
							} else {
								G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
								g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
								g_clear_error (&_inner_error_);
								return;
							}
						}
#line 301 "builder.vala"
						g_value_set_string (&gvalue, _tmp5_);
#line 1321 "builder.c"
					} else {
#line 303 "builder.vala"
						if (pspec->value_type == G_TYPE_FLOAT) {
#line 1325 "builder.c"
							const char* _tmp6_;
							_tmp6_ = g_yaml_builder_cast_to_scalar (self, node, &_inner_error_);
							if (_inner_error_ != NULL) {
								if (_inner_error_->domain == G_YAML_ERROR) {
									g_propagate_error (error, _inner_error_);
									G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
									return;
								} else {
									G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
									g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
									g_clear_error (&_inner_error_);
									return;
								}
							}
#line 304 "builder.vala"
							g_value_set_float (&gvalue, (float) g_ascii_strtod (_tmp6_, NULL));
#line 1342 "builder.c"
						} else {
#line 306 "builder.vala"
							if (pspec->value_type == G_TYPE_DOUBLE) {
#line 1346 "builder.c"
								const char* _tmp7_;
								_tmp7_ = g_yaml_builder_cast_to_scalar (self, node, &_inner_error_);
								if (_inner_error_ != NULL) {
									if (_inner_error_->domain == G_YAML_ERROR) {
										g_propagate_error (error, _inner_error_);
										G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
										return;
									} else {
										G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
										g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
										g_clear_error (&_inner_error_);
										return;
									}
								}
#line 307 "builder.vala"
								g_value_set_double (&gvalue, g_ascii_strtod (_tmp7_, NULL));
#line 1363 "builder.c"
							} else {
#line 309 "builder.vala"
								if (pspec->value_type == G_TYPE_BOOLEAN) {
#line 1367 "builder.c"
									const char* _tmp8_;
									_tmp8_ = g_yaml_builder_cast_to_scalar (self, node, &_inner_error_);
									if (_inner_error_ != NULL) {
										if (_inner_error_->domain == G_YAML_ERROR) {
											g_propagate_error (error, _inner_error_);
											G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
											return;
										} else {
											G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
											g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
											g_clear_error (&_inner_error_);
											return;
										}
									}
#line 310 "builder.vala"
									g_value_set_boolean (&gvalue, string_to_bool (_tmp8_));
#line 1384 "builder.c"
								} else {
#line 312 "builder.vala"
									if (pspec->value_type == G_TYPE_GTYPE) {
#line 1388 "builder.c"
										const char* _tmp9_;
										char* _tmp10_;
										GType _tmp11_;
										GType _tmp12_;
										_tmp9_ = g_yaml_builder_cast_to_scalar (self, node, &_inner_error_);
										if (_inner_error_ != NULL) {
											if (_inner_error_->domain == G_YAML_ERROR) {
												g_propagate_error (error, _inner_error_);
												G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
												return;
											} else {
												G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
												g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
												g_clear_error (&_inner_error_);
												return;
											}
										}
#line 203 "builder.vala"
										_tmp10_ = NULL;
#line 1408 "builder.c"
										_tmp12_ = (_tmp11_ = g_yaml_demangler_resolve_type (_tmp10_ = g_yaml_builder_get_full_class_name (self, _tmp9_), &_inner_error_), _tmp10_ = (g_free (_tmp10_), NULL), _tmp11_);
										if (_inner_error_ != NULL) {
											if (_inner_error_->domain == G_YAML_ERROR) {
												g_propagate_error (error, _inner_error_);
												G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
												return;
											} else {
												G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
												g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
												g_clear_error (&_inner_error_);
												return;
											}
										}
#line 313 "builder.vala"
										g_value_set_gtype (&gvalue, _tmp12_);
#line 1424 "builder.c"
									} else {
#line 315 "builder.vala"
										if (g_type_is_a (pspec->value_type, G_TYPE_OBJECT)) {
#line 1428 "builder.c"
											GObject* ref_obj;
											ref_obj = NULL;
#line 317 "builder.vala"
											if (G_YAML_NODE_IS_SCALAR (node)) {
#line 1433 "builder.c"
												const char* _tmp13_;
												GObject* _tmp14_;
												_tmp13_ = g_yaml_builder_cast_to_scalar (self, node, &_inner_error_);
												if (_inner_error_ != NULL) {
													if (_inner_error_->domain == G_YAML_ERROR) {
														g_propagate_error (error, _inner_error_);
														(ref_obj == NULL) ? NULL : (ref_obj = (g_object_unref (ref_obj), NULL));
														G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
														return;
													} else {
														(ref_obj == NULL) ? NULL : (ref_obj = (g_object_unref (ref_obj), NULL));
														G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
														g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
														g_clear_error (&_inner_error_);
														return;
													}
												}
												_tmp14_ = NULL;
#line 318 "builder.vala"
												ref_obj = (_tmp14_ = g_yaml_builder_get_object (self, _tmp13_), (ref_obj == NULL) ? NULL : (ref_obj = (g_object_unref (ref_obj), NULL)), _tmp14_);
#line 319 "builder.vala"
												if (ref_obj == NULL) {
#line 1456 "builder.c"
													const char* _tmp15_;
													_tmp15_ = g_yaml_builder_cast_to_scalar (self, node, &_inner_error_);
													if (_inner_error_ != NULL) {
														if (_inner_error_->domain == G_YAML_ERROR) {
															g_propagate_error (error, _inner_error_);
															(ref_obj == NULL) ? NULL : (ref_obj = (g_object_unref (ref_obj), NULL));
															G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
															return;
														} else {
															(ref_obj == NULL) ? NULL : (ref_obj = (g_object_unref (ref_obj), NULL));
															G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
															g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
															g_clear_error (&_inner_error_);
															return;
														}
													}
													_inner_error_ = g_error_new (G_YAML_ERROR, G_YAML_ERROR_OBJECT_NOT_FOUND, "Object '%s' not found", _tmp15_);
													if (_inner_error_ != NULL) {
														if (_inner_error_->domain == G_YAML_ERROR) {
															g_propagate_error (error, _inner_error_);
															(ref_obj == NULL) ? NULL : (ref_obj = (g_object_unref (ref_obj), NULL));
															G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
															return;
														} else {
															(ref_obj == NULL) ? NULL : (ref_obj = (g_object_unref (ref_obj), NULL));
															G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
															g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
															g_clear_error (&_inner_error_);
															return;
														}
													}
												}
											}
#line 325 "builder.vala"
											if (G_YAML_NODE_IS_MAPPING (node)) {
#line 1492 "builder.c"
												GObject* _tmp16_;
												GObject* _tmp17_;
												_tmp16_ = g_yaml_builder_build_object (self, node, pspec->value_type, &_inner_error_);
												if (_inner_error_ != NULL) {
													if (_inner_error_->domain == G_YAML_ERROR) {
														g_propagate_error (error, _inner_error_);
														(ref_obj == NULL) ? NULL : (ref_obj = (g_object_unref (ref_obj), NULL));
														G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
														return;
													} else {
														(ref_obj == NULL) ? NULL : (ref_obj = (g_object_unref (ref_obj), NULL));
														G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
														g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
														g_clear_error (&_inner_error_);
														return;
													}
												}
												_tmp17_ = NULL;
#line 326 "builder.vala"
												ref_obj = (_tmp17_ = _tmp16_, (ref_obj == NULL) ? NULL : (ref_obj = (g_object_unref (ref_obj), NULL)), _tmp17_);
#line 1513 "builder.c"
											} else {
												char* _tmp18_;
												GError* _tmp19_;
#line 115 "libyaml-1.0.vapi"
												_tmp18_ = NULL;
#line 328 "builder.vala"
												_tmp19_ = NULL;
#line 1521 "builder.c"
												_inner_error_ = (_tmp19_ = g_error_new (G_YAML_ERROR, G_YAML_ERROR_OBJECT_NOT_FOUND, "Donot know how to build the object for `%s' (%s)", pspec->name, _tmp18_ = yaml_mark_to_string (&node->start_mark)), _tmp18_ = (g_free (_tmp18_), NULL), _tmp19_);
												if (_inner_error_ != NULL) {
													if (_inner_error_->domain == G_YAML_ERROR) {
														g_propagate_error (error, _inner_error_);
														(ref_obj == NULL) ? NULL : (ref_obj = (g_object_unref (ref_obj), NULL));
														G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
														return;
													} else {
														(ref_obj == NULL) ? NULL : (ref_obj = (g_object_unref (ref_obj), NULL));
														G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
														g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
														g_clear_error (&_inner_error_);
														return;
													}
												}
											}
#line 333 "builder.vala"
											g_value_set_object (&gvalue, ref_obj);
#line 1540 "builder.c"
											(ref_obj == NULL) ? NULL : (ref_obj = (g_object_unref (ref_obj), NULL));
										} else {
#line 335 "builder.vala"
											if (g_type_is_a (pspec->value_type, G_TYPE_BOXED)) {
#line 1545 "builder.c"
												const char* _tmp20_;
												char* strval;
												void* symbol;
												gchar* _tmp21_;
												gint memory_size;
												gint memory_length1;
												gchar* memory;
												GYAMLBuilderParseFunc func;
#line 336 "builder.vala"
												_tmp20_ = NULL;
#line 1556 "builder.c"
												strval = (_tmp20_ = g_yaml_builder_cast_to_scalar (self, node, &_inner_error_), (_tmp20_ == NULL) ? NULL : g_strdup (_tmp20_));
												if (_inner_error_ != NULL) {
													if (_inner_error_->domain == G_YAML_ERROR) {
														g_propagate_error (error, _inner_error_);
														G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
														return;
													} else {
														G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
														g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
														g_clear_error (&_inner_error_);
														return;
													}
												}
#line 337 "builder.vala"
												g_debug ("builder.vala:337: working on a boxed type %s <- %s", g_type_name (pspec->value_type), strval);
#line 1572 "builder.c"
												symbol = g_yaml_demangler_resolve_function (g_type_name (pspec->value_type), "parse", &_inner_error_);
												if (_inner_error_ != NULL) {
													if (_inner_error_->domain == G_YAML_ERROR) {
														g_propagate_error (error, _inner_error_);
														strval = (g_free (strval), NULL);
														G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
														return;
													} else {
														strval = (g_free (strval), NULL);
														G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
														g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
														g_clear_error (&_inner_error_);
														return;
													}
												}
#line 339 "builder.vala"
												_tmp21_ = NULL;
#line 1590 "builder.c"
												memory = (_tmp21_ = g_new0 (gchar, 65500), memory_length1 = 65500, memory_size = memory_length1, _tmp21_);
												func = (GYAMLBuilderParseFunc) symbol;
#line 341 "builder.vala"
												if (!func (strval, (void*) memory)) {
#line 1595 "builder.c"
													char* _tmp22_;
													GError* _tmp23_;
#line 115 "libyaml-1.0.vapi"
													_tmp22_ = NULL;
#line 342 "builder.vala"
													_tmp23_ = NULL;
#line 1602 "builder.c"
													_inner_error_ = (_tmp23_ = g_error_new (G_YAML_ERROR, G_YAML_ERROR_UNEXPECTED_NODE, "Failed to parse the boxed type %s at (%s)", g_type_name (pspec->value_type), _tmp22_ = yaml_mark_to_string (&node->start_mark)), _tmp22_ = (g_free (_tmp22_), NULL), _tmp23_);
													if (_inner_error_ != NULL) {
														if (_inner_error_->domain == G_YAML_ERROR) {
															g_propagate_error (error, _inner_error_);
															strval = (g_free (strval), NULL);
															memory = (g_free (memory), NULL);
															G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
															return;
														} else {
															strval = (g_free (strval), NULL);
															memory = (g_free (memory), NULL);
															G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
															g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
															g_clear_error (&_inner_error_);
															return;
														}
													}
												}
#line 347 "builder.vala"
												g_value_set_boxed (&gvalue, memory);
#line 1623 "builder.c"
												strval = (g_free (strval), NULL);
												memory = (g_free (memory), NULL);
											} else {
#line 349 "builder.vala"
												if (g_type_is_a (pspec->value_type, g_yaml_builder_enum_type)) {
#line 1629 "builder.c"
													const char* _tmp24_;
													_tmp24_ = g_yaml_builder_cast_to_scalar (self, node, &_inner_error_);
													if (_inner_error_ != NULL) {
														if (_inner_error_->domain == G_YAML_ERROR) {
															g_propagate_error (error, _inner_error_);
															G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
															return;
														} else {
															G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
															g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
															g_clear_error (&_inner_error_);
															return;
														}
													}
#line 350 "builder.vala"
													g_value_set_enum (&gvalue, (gint) atol (_tmp24_));
#line 1646 "builder.c"
												} else {
													_inner_error_ = g_error_new (G_YAML_ERROR, G_YAML_ERROR_UNKNOWN_PROPERTY_TYPE, "Unhandled property type %s", g_type_name (pspec->value_type));
													if (_inner_error_ != NULL) {
														if (_inner_error_->domain == G_YAML_ERROR) {
															g_propagate_error (error, _inner_error_);
															G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
															return;
														} else {
															G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
															g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
															g_clear_error (&_inner_error_);
															return;
														}
													}
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
#line 357 "builder.vala"
	g_object_set_property (obj, pspec->name, &gvalue);
#line 1675 "builder.c"
	G_IS_VALUE (&gvalue) ? (g_value_unset (&gvalue), NULL) : NULL;
}


#line 360 "builder.vala"
static const char* g_yaml_builder_cast_to_scalar (GYAMLBuilder* self, GYAMLNode* node, GError** error) {
#line 1682 "builder.c"
	const char* result;
	GError * _inner_error_;
	GYAMLNodeScalar* _tmp1_;
	GYAMLNode* _tmp0_;
	GYAMLNodeScalar* value_scalar;
#line 360 "builder.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 360 "builder.vala"
	g_return_val_if_fail (node != NULL, NULL);
#line 1692 "builder.c"
	_inner_error_ = NULL;
#line 361 "builder.vala"
	_tmp1_ = NULL;
#line 361 "builder.vala"
	_tmp0_ = NULL;
#line 1698 "builder.c"
	value_scalar = (_tmp1_ = (_tmp0_ = node, G_YAML_NODE_IS_SCALAR (_tmp0_) ? ((GYAMLNodeScalar*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_yaml_node_ref (_tmp1_));
#line 362 "builder.vala"
	if (value_scalar == NULL) {
#line 1702 "builder.c"
		char* _tmp2_;
		GError* _tmp3_;
#line 115 "libyaml-1.0.vapi"
		_tmp2_ = NULL;
#line 363 "builder.vala"
		_tmp3_ = NULL;
#line 1709 "builder.c"
		_inner_error_ = (_tmp3_ = g_error_new (G_YAML_ERROR, G_YAML_ERROR_UNEXPECTED_NODE, "Expecting a Scalar (%s)", _tmp2_ = yaml_mark_to_string (&node->start_mark)), _tmp2_ = (g_free (_tmp2_), NULL), _tmp3_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_YAML_ERROR) {
				g_propagate_error (error, _inner_error_);
				(value_scalar == NULL) ? NULL : (value_scalar = (g_yaml_node_unref (value_scalar), NULL));
				return NULL;
			} else {
				(value_scalar == NULL) ? NULL : (value_scalar = (g_yaml_node_unref (value_scalar), NULL));
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return NULL;
			}
		}
	}
	result = value_scalar->value;
	(value_scalar == NULL) ? NULL : (value_scalar = (g_yaml_node_unref (value_scalar), NULL));
	return result;
}


#line 370 "builder.vala"
static void g_yaml_builder_process_internal_children (GYAMLBuilder* self, GObject* obj, GYAMLNode* node, GError** error) {
#line 1732 "builder.c"
	GError * _inner_error_;
	GYAMLNodeMapping* _tmp1_;
	GYAMLNode* _tmp0_;
	GYAMLNodeMapping* children;
#line 370 "builder.vala"
	g_return_if_fail (self != NULL);
#line 370 "builder.vala"
	g_return_if_fail (obj != NULL);
#line 370 "builder.vala"
	g_return_if_fail (node != NULL);
#line 1743 "builder.c"
	_inner_error_ = NULL;
#line 371 "builder.vala"
	_tmp1_ = NULL;
#line 371 "builder.vala"
	_tmp0_ = NULL;
#line 1749 "builder.c"
	children = (_tmp1_ = (_tmp0_ = node, G_YAML_NODE_IS_MAPPING (_tmp0_) ? ((GYAMLNodeMapping*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_yaml_node_ref (_tmp1_));
	{
		GList* key_node_collection;
		GList* key_node_it;
#line 372 "builder.vala"
		key_node_collection = children->keys;
#line 1756 "builder.c"
		for (key_node_it = key_node_collection; key_node_it != NULL; key_node_it = key_node_it->next) {
			GYAMLNode* _tmp5_;
			GYAMLNode* key_node;
#line 170 "libyaml-glib.vala"
			_tmp5_ = NULL;
#line 372 "builder.vala"
			key_node = (_tmp5_ = (GYAMLNode*) key_node_it->data, (_tmp5_ == NULL) ? NULL : g_yaml_node_ref (_tmp5_));
#line 1764 "builder.c"
			{
				const char* _tmp2_;
				char* key;
				GYAMLNode* value_node;
				GObject* child;
#line 373 "builder.vala"
				_tmp2_ = NULL;
#line 1772 "builder.c"
				key = (_tmp2_ = g_yaml_builder_cast_to_scalar (self, key_node, &_inner_error_), (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_));
				if (_inner_error_ != NULL) {
					g_propagate_error (error, _inner_error_);
					(key_node == NULL) ? NULL : (key_node = (g_yaml_node_unref (key_node), NULL));
					(children == NULL) ? NULL : (children = (g_yaml_node_unref (children), NULL));
					return;
				}
				value_node = g_yaml_node_get_resolved ((GYAMLNode*) g_hash_table_lookup (children->pairs, key_node));
				child = g_yaml_buildable_get_internal_child ((GYAMLBuildable*) obj, self, key);
#line 376 "builder.vala"
				if (child == NULL) {
#line 1784 "builder.c"
					char* _tmp3_;
					GError* _tmp4_;
#line 115 "libyaml-1.0.vapi"
					_tmp3_ = NULL;
#line 377 "builder.vala"
					_tmp4_ = NULL;
#line 1791 "builder.c"
					_inner_error_ = (_tmp4_ = g_error_new (G_YAML_ERROR, G_YAML_ERROR_OBJECT_NOT_FOUND, "Expecting an internal child `%s', found nothing (%s)", key, _tmp3_ = yaml_mark_to_string (&node->start_mark)), _tmp3_ = (g_free (_tmp3_), NULL), _tmp4_);
					if (_inner_error_ != NULL) {
						g_propagate_error (error, _inner_error_);
						(key_node == NULL) ? NULL : (key_node = (g_yaml_node_unref (key_node), NULL));
						key = (g_free (key), NULL);
						(value_node == NULL) ? NULL : (value_node = (g_yaml_node_unref (value_node), NULL));
						(child == NULL) ? NULL : (child = (g_object_unref (child), NULL));
						(children == NULL) ? NULL : (children = (g_yaml_node_unref (children), NULL));
						return;
					}
				}
#line 381 "builder.vala"
				g_yaml_builder_process_object_value_node (self, child, value_node, &_inner_error_);
#line 1805 "builder.c"
				if (_inner_error_ != NULL) {
					g_propagate_error (error, _inner_error_);
					(key_node == NULL) ? NULL : (key_node = (g_yaml_node_unref (key_node), NULL));
					key = (g_free (key), NULL);
					(value_node == NULL) ? NULL : (value_node = (g_yaml_node_unref (value_node), NULL));
					(child == NULL) ? NULL : (child = (g_object_unref (child), NULL));
					(children == NULL) ? NULL : (children = (g_yaml_node_unref (children), NULL));
					return;
				}
				(key_node == NULL) ? NULL : (key_node = (g_yaml_node_unref (key_node), NULL));
				key = (g_free (key), NULL);
				(value_node == NULL) ? NULL : (value_node = (g_yaml_node_unref (value_node), NULL));
				(child == NULL) ? NULL : (child = (g_object_unref (child), NULL));
			}
		}
	}
	(children == NULL) ? NULL : (children = (g_yaml_node_unref (children), NULL));
}


#line 385 "builder.vala"
static void g_yaml_builder_process_children (GYAMLBuilder* self, GObject* obj, const char* type, GYAMLNode* node, GError** error) {
#line 1828 "builder.c"
	GError * _inner_error_;
	GYAMLNodeSequence* _tmp1_;
	GYAMLNode* _tmp0_;
	GYAMLNodeSequence* children;
#line 385 "builder.vala"
	g_return_if_fail (self != NULL);
#line 385 "builder.vala"
	g_return_if_fail (obj != NULL);
#line 385 "builder.vala"
	g_return_if_fail (type != NULL);
#line 385 "builder.vala"
	g_return_if_fail (node != NULL);
#line 1841 "builder.c"
	_inner_error_ = NULL;
#line 386 "builder.vala"
	if (_vala_strcmp0 (type, "internals") == 0) {
#line 387 "builder.vala"
		g_yaml_builder_process_internal_children (self, obj, node, &_inner_error_);
#line 1847 "builder.c"
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			return;
		}
		return;
	}
#line 390 "builder.vala"
	_tmp1_ = NULL;
#line 390 "builder.vala"
	_tmp0_ = NULL;
#line 1858 "builder.c"
	children = (_tmp1_ = (_tmp0_ = node, G_YAML_NODE_IS_SEQUENCE (_tmp0_) ? ((GYAMLNodeSequence*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_yaml_node_ref (_tmp1_));
	{
		GList* item_collection;
		GList* item_it;
#line 391 "builder.vala"
		item_collection = children->items;
#line 1865 "builder.c"
		for (item_it = item_collection; item_it != NULL; item_it = item_it->next) {
			GYAMLNode* _tmp6_;
			GYAMLNode* item;
#line 154 "libyaml-glib.vala"
			_tmp6_ = NULL;
#line 391 "builder.vala"
			item = (_tmp6_ = (GYAMLNode*) item_it->data, (_tmp6_ == NULL) ? NULL : g_yaml_node_ref (_tmp6_));
#line 1873 "builder.c"
			{
				GYAMLNode* _tmp2_;
				GObject* _tmp3_;
				GObject* child;
#line 114 "libyaml-glib.vala"
				_tmp2_ = NULL;
#line 392 "builder.vala"
				_tmp3_ = NULL;
#line 1882 "builder.c"
				child = (_tmp3_ = g_yaml_builder_build_object (self, _tmp2_ = g_yaml_node_get_resolved (item), g_yaml_buildable_get_child_type ((GYAMLBuildable*) obj, self, type), &_inner_error_), (_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_yaml_node_unref (_tmp2_), NULL)), _tmp3_);
				if (_inner_error_ != NULL) {
					g_propagate_error (error, _inner_error_);
					(item == NULL) ? NULL : (item = (g_yaml_node_unref (item), NULL));
					(children == NULL) ? NULL : (children = (g_yaml_node_unref (children), NULL));
					return;
				}
#line 393 "builder.vala"
				if (child == NULL) {
#line 1892 "builder.c"
					char* _tmp4_;
					GError* _tmp5_;
#line 115 "libyaml-1.0.vapi"
					_tmp4_ = NULL;
#line 394 "builder.vala"
					_tmp5_ = NULL;
#line 1899 "builder.c"
					_inner_error_ = (_tmp5_ = g_error_new (G_YAML_ERROR, G_YAML_ERROR_OBJECT_NOT_FOUND, "Expecting an object, found nothing (%s)", _tmp4_ = yaml_mark_to_string (&node->start_mark)), _tmp4_ = (g_free (_tmp4_), NULL), _tmp5_);
					if (_inner_error_ != NULL) {
						g_propagate_error (error, _inner_error_);
						(item == NULL) ? NULL : (item = (g_yaml_node_unref (item), NULL));
						(child == NULL) ? NULL : (child = (g_object_unref (child), NULL));
						(children == NULL) ? NULL : (children = (g_yaml_node_unref (children), NULL));
						return;
					}
				}
#line 398 "builder.vala"
				g_yaml_buildable_add_child ((GYAMLBuildable*) obj, self, child, type, &_inner_error_);
#line 1911 "builder.c"
				if (_inner_error_ != NULL) {
					g_propagate_error (error, _inner_error_);
					(item == NULL) ? NULL : (item = (g_yaml_node_unref (item), NULL));
					(child == NULL) ? NULL : (child = (g_object_unref (child), NULL));
					(children == NULL) ? NULL : (children = (g_yaml_node_unref (children), NULL));
					return;
				}
				(item == NULL) ? NULL : (item = (g_yaml_node_unref (item), NULL));
				(child == NULL) ? NULL : (child = (g_object_unref (child), NULL));
			}
		}
	}
	(children == NULL) ? NULL : (children = (g_yaml_node_unref (children), NULL));
}


/** 
 * Obtain a built object referred by the anchor, or the root object 
 * of the latest added document if anchor == null.
 *
 * @return the object referred by the anchor or the root object.
 *
  */
#line 408 "builder.vala"
GObject* g_yaml_builder_get_object (GYAMLBuilder* self, const char* anchor) {
#line 1937 "builder.c"
	GObject* result;
#line 408 "builder.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 409 "builder.vala"
	if (anchor != NULL) {
#line 1943 "builder.c"
		GObject* _tmp0_;
#line 410 "builder.vala"
		_tmp0_ = NULL;
#line 1947 "builder.c"
		result = (_tmp0_ = (GObject*) g_hash_table_lookup (self->priv->anchors, anchor), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
		return result;
	} else {
		result = g_yaml_builder_get_root_object (self);
		return result;
	}
}


/**
 * Obtain the root object created for the document
 *
 * @return the root object
 */
#line 420 "builder.vala"
GObject* g_yaml_builder_get_root_object (GYAMLBuilder* self) {
#line 1964 "builder.c"
	GObject* result;
	GObject* _tmp1_;
	void* _tmp0_;
#line 420 "builder.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 421 "builder.vala"
	_tmp1_ = NULL;
#line 421 "builder.vala"
	_tmp0_ = NULL;
#line 1974 "builder.c"
	result = (_tmp1_ = (_tmp0_ = g_yaml_node_get_pointer (self->priv->document->root), G_IS_OBJECT (_tmp0_) ? ((GObject*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_));
	return result;
}


/**
 * Obtain a list of all objects created by the builder. The 
 * returned list should not be modified.
 * 
 * @return a read-only weak reference to the list of all objects 
 * created by the builder.
 */
#line 430 "builder.vala"
GList* g_yaml_builder_get_objects (GYAMLBuilder* self) {
#line 1989 "builder.c"
	GList* result;
#line 430 "builder.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 1993 "builder.c"
	result = self->priv->objects;
	return result;
}


static void g_yaml_builder_class_init (GYAMLBuilderClass * klass) {
	g_yaml_builder_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GYAMLBuilderPrivate));
	G_OBJECT_CLASS (klass)->finalize = g_yaml_builder_finalize;
	{
#line 121 "builder.vala"
		g_yaml_builder_enum_type = g_type_parent (G_YAML_BUILDER_TYPE___ENUM);
#line 2006 "builder.c"
	}
}


static void g_yaml_builder_instance_init (GYAMLBuilder * self) {
	self->priv = G_YAML_BUILDER_GET_PRIVATE (self);
	self->priv->prefix = NULL;
	self->priv->anchors = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
}


static void g_yaml_builder_finalize (GObject* obj) {
	GYAMLBuilder * self;
	self = G_YAML_BUILDER (obj);
	self->priv->prefix = (g_free (self->priv->prefix), NULL);
	(self->priv->anchors == NULL) ? NULL : (self->priv->anchors = (g_hash_table_unref (self->priv->anchors), NULL));
	(self->priv->objects == NULL) ? NULL : (self->priv->objects = (_g_list_free_g_object_unref (self->priv->objects), NULL));
	(self->priv->document == NULL) ? NULL : (self->priv->document = (g_yaml_document_unref (self->priv->document), NULL));
	G_OBJECT_CLASS (g_yaml_builder_parent_class)->finalize (obj);
}


GType g_yaml_builder_get_type (void) {
	static GType g_yaml_builder_type_id = 0;
	if (g_yaml_builder_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GYAMLBuilderClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) g_yaml_builder_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GYAMLBuilder), 0, (GInstanceInitFunc) g_yaml_builder_instance_init, NULL };
		g_yaml_builder_type_id = g_type_register_static (G_TYPE_OBJECT, "GYAMLBuilder", &g_define_type_info, 0);
	}
	return g_yaml_builder_type_id;
}


/**
 * A yet powerful Vala type name to c name demangler.
 *
 * @param vala_name
 *   the class name. eg, UCN.ColdNeutron
 *
 * */
#line 451 "builder.vala"
char* g_yaml_demangler_demangle (const char* vala_name) {
#line 2048 "builder.c"
	char* result;
	GString* sb;
	gboolean already_underscoped;
	const char* p0;
	const char* p1;
	gunichar c0;
	gunichar c1;
	const char* _tmp3_;
#line 451 "builder.vala"
	g_return_val_if_fail (vala_name != NULL, NULL);
#line 2059 "builder.c"
	sb = g_string_new ("");
	already_underscoped = TRUE;
	p0 = NULL;
	p1 = vala_name;
	c0 = (gunichar) 0;
	c1 = g_utf8_get_char (p1);
	{
		gboolean _tmp0_;
		_tmp0_ = TRUE;
#line 459 "builder.vala"
		while (TRUE) {
#line 2071 "builder.c"
			gboolean _tmp1_;
			gboolean _tmp2_;
#line 459 "builder.vala"
			if (!_tmp0_) {
#line 461 "builder.vala"
				p0 = p1;
#line 461 "builder.vala"
				c0 = c1;
#line 461 "builder.vala"
				p1 = g_utf8_next_char (p1);
#line 461 "builder.vala"
				c1 = g_utf8_get_char (p1);
#line 2084 "builder.c"
			}
#line 459 "builder.vala"
			_tmp0_ = FALSE;
#line 460 "builder.vala"
			if (!(c1 != 0)) {
#line 460 "builder.vala"
				break;
#line 2092 "builder.c"
			}
			/* Do not take any real action before we have two chars. */
#line 464 "builder.vala"
			if (c0 == 0) {
#line 464 "builder.vala"
				continue;
#line 2099 "builder.c"
			}
			_tmp1_ = FALSE;
#line 466 "builder.vala"
			if (g_unichar_islower (c0)) {
#line 466 "builder.vala"
				_tmp1_ = g_unichar_isupper (c1);
#line 2106 "builder.c"
			} else {
#line 466 "builder.vala"
				_tmp1_ = FALSE;
#line 2110 "builder.c"
			}
#line 466 "builder.vala"
			if (_tmp1_) {
#line 467 "builder.vala"
				g_string_append_unichar (sb, g_unichar_tolower (c0));
#line 468 "builder.vala"
				g_string_append_unichar (sb, (gunichar) '_');
#line 469 "builder.vala"
				already_underscoped = TRUE;
#line 470 "builder.vala"
				continue;
#line 2122 "builder.c"
			}
			_tmp2_ = FALSE;
#line 472 "builder.vala"
			if (g_unichar_isupper (c0)) {
#line 472 "builder.vala"
				_tmp2_ = g_unichar_islower (c1);
#line 2129 "builder.c"
			} else {
#line 472 "builder.vala"
				_tmp2_ = FALSE;
#line 2133 "builder.c"
			}
#line 472 "builder.vala"
			if (_tmp2_) {
#line 473 "builder.vala"
				if (!already_underscoped) {
#line 474 "builder.vala"
					g_string_append_unichar (sb, (gunichar) '_');
#line 2141 "builder.c"
				}
#line 476 "builder.vala"
				g_string_append_unichar (sb, g_unichar_tolower (c0));
#line 477 "builder.vala"
				already_underscoped = FALSE;
#line 478 "builder.vala"
				continue;
#line 2149 "builder.c"
			}
#line 481 "builder.vala"
			if (c0 == '.') {
#line 482 "builder.vala"
				g_string_append_unichar (sb, (gunichar) '_');
#line 483 "builder.vala"
				already_underscoped = TRUE;
#line 484 "builder.vala"
				continue;
#line 2159 "builder.c"
			} else {
#line 486 "builder.vala"
				g_string_append_unichar (sb, g_unichar_tolower (c0));
#line 487 "builder.vala"
				already_underscoped = FALSE;
#line 488 "builder.vala"
				continue;
#line 2167 "builder.c"
			}
		}
	}
#line 491 "builder.vala"
	g_string_append_unichar (sb, g_unichar_tolower (c0));
#line 492 "builder.vala"
	_tmp3_ = NULL;
#line 2175 "builder.c"
	result = (_tmp3_ = sb->str, (_tmp3_ == NULL) ? NULL : g_strdup (_tmp3_));
	(sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL));
	return result;
	(sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL));
}


/**
 * Resolve a member function to a function pointer.
 *
 * @param class_name
 *   the name of the class, usually from GType.name(), 
 * @param member_name
 *   the name of the member.
 *
 * @return the function pointer.
 */
#line 504 "builder.vala"
void* g_yaml_demangler_resolve_function (const char* class_name, const char* member_name, GError** error) {
#line 2195 "builder.c"
	void* result;
	GError * _inner_error_;
	void* symbol;
	GString* sb;
	char* _tmp0_;
	char* _tmp1_;
	const char* _tmp2_;
	char* func_name;
	GModule* _self_;
#line 504 "builder.vala"
	g_return_val_if_fail (class_name != NULL, NULL);
#line 504 "builder.vala"
	g_return_val_if_fail (member_name != NULL, NULL);
#line 2209 "builder.c"
	_inner_error_ = NULL;
	symbol = NULL;
	sb = g_string_new ("");
#line 451 "builder.vala"
	_tmp0_ = NULL;
#line 508 "builder.vala"
	g_string_append (sb, _tmp0_ = g_yaml_demangler_demangle (class_name));
#line 2217 "builder.c"
	_tmp0_ = (g_free (_tmp0_), NULL);
#line 509 "builder.vala"
	g_string_append_unichar (sb, (gunichar) '_');
#line 451 "builder.vala"
	_tmp1_ = NULL;
#line 510 "builder.vala"
	g_string_append (sb, _tmp1_ = g_yaml_demangler_demangle (member_name));
#line 2225 "builder.c"
	_tmp1_ = (g_free (_tmp1_), NULL);
#line 511 "builder.vala"
	_tmp2_ = NULL;
#line 2229 "builder.c"
	func_name = (_tmp2_ = sb->str, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_));
	_self_ = g_module_open (NULL, 0);
#line 513 "builder.vala"
	if (!g_module_symbol (_self_, func_name, &symbol)) {
#line 2234 "builder.c"
		_inner_error_ = g_error_new (G_YAML_ERROR, G_YAML_ERROR_SYMBOL_NOT_FOUND, "Symbol %s.%s (%s) not found", class_name, member_name, func_name);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_YAML_ERROR) {
				g_propagate_error (error, _inner_error_);
				(sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL));
				func_name = (g_free (func_name), NULL);
				(_self_ == NULL) ? NULL : (_self_ = (g_module_close (_self_), NULL));
				return NULL;
			} else {
				(sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL));
				func_name = (g_free (func_name), NULL);
				(_self_ == NULL) ? NULL : (_self_ = (g_module_close (_self_), NULL));
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return NULL;
			}
		}
	}
	result = symbol;
	(sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL));
	func_name = (g_free (func_name), NULL);
	(_self_ == NULL) ? NULL : (_self_ = (g_module_close (_self_), NULL));
	return result;
}


/**
 * Resolve a GType from the class_name.
 *
 * Notice that GType.from_name is not used, instead, the get_type 
 * static member is called to obtain the name. 
 * This is necessary because the type we are looking for might 
 * have not yet been registered to the type system, whereas the 
 * get_type functions are mandatory for any GType classes. 
 *
 * @return the GType
 */
#line 532 "builder.vala"
GType g_yaml_demangler_resolve_type (const char* class_name, GError** error) {
#line 2274 "builder.c"
	GType result;
	GError * _inner_error_;
	void* symbol;
	GYAMLDemanglerTypeFunc type_func;
#line 532 "builder.vala"
	g_return_val_if_fail (class_name != NULL, 0UL);
#line 2281 "builder.c"
	_inner_error_ = NULL;
	symbol = g_yaml_demangler_resolve_function (class_name, "get_type", &_inner_error_);
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_YAML_ERROR) {
			g_propagate_error (error, _inner_error_);
			return 0UL;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return 0UL;
		}
	}
	type_func = (GYAMLDemanglerTypeFunc) symbol;
	result = type_func ();
	return result;
}


/**
 * Demangle vala names to c names in the standard way.
 *
 * [warning:
 *   GI information is not used, therefore 
 *   if there is any special tricks by the CCode annotations, 
 *   you have to be careful and understand what you are doing. 
 * ]
 **/
#line 443 "builder.vala"
GYAMLDemangler* g_yaml_demangler_construct (GType object_type) {
#line 2311 "builder.c"
	GYAMLDemangler* self;
	self = (GYAMLDemangler*) g_type_create_instance (object_type);
	return self;
}


#line 443 "builder.vala"
GYAMLDemangler* g_yaml_demangler_new (void) {
#line 443 "builder.vala"
	return g_yaml_demangler_construct (G_YAML_TYPE_DEMANGLER);
#line 2322 "builder.c"
}


static void g_yaml_value_demangler_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void g_yaml_value_demangler_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		g_yaml_demangler_unref (value->data[0].v_pointer);
	}
}


static void g_yaml_value_demangler_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = g_yaml_demangler_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer g_yaml_value_demangler_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* g_yaml_value_demangler_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		GYAMLDemangler* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = g_yaml_demangler_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* g_yaml_value_demangler_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	GYAMLDemangler** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = g_yaml_demangler_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* g_yaml_param_spec_demangler (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	GYAMLParamSpecDemangler* spec;
	g_return_val_if_fail (g_type_is_a (object_type, G_YAML_TYPE_DEMANGLER), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer g_yaml_value_get_demangler (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, G_YAML_TYPE_DEMANGLER), NULL);
	return value->data[0].v_pointer;
}


void g_yaml_value_set_demangler (GValue* value, gpointer v_object) {
	GYAMLDemangler* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, G_YAML_TYPE_DEMANGLER));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, G_YAML_TYPE_DEMANGLER));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		g_yaml_demangler_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		g_yaml_demangler_unref (old);
	}
}


static void g_yaml_demangler_class_init (GYAMLDemanglerClass * klass) {
	g_yaml_demangler_parent_class = g_type_class_peek_parent (klass);
	G_YAML_DEMANGLER_CLASS (klass)->finalize = g_yaml_demangler_finalize;
}


static void g_yaml_demangler_instance_init (GYAMLDemangler * self) {
	self->ref_count = 1;
}


static void g_yaml_demangler_finalize (GYAMLDemangler* obj) {
	GYAMLDemangler * self;
	self = G_YAML_DEMANGLER (obj);
}


GType g_yaml_demangler_get_type (void) {
	static GType g_yaml_demangler_type_id = 0;
	if (g_yaml_demangler_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { g_yaml_value_demangler_init, g_yaml_value_demangler_free_value, g_yaml_value_demangler_copy_value, g_yaml_value_demangler_peek_pointer, "p", g_yaml_value_demangler_collect_value, "p", g_yaml_value_demangler_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (GYAMLDemanglerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) g_yaml_demangler_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GYAMLDemangler), 0, (GInstanceInitFunc) g_yaml_demangler_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		g_yaml_demangler_type_id = g_type_register_fundamental (g_type_fundamental_next (), "GYAMLDemangler", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return g_yaml_demangler_type_id;
}


gpointer g_yaml_demangler_ref (gpointer instance) {
	GYAMLDemangler* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void g_yaml_demangler_unref (gpointer instance) {
	GYAMLDemangler* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		G_YAML_DEMANGLER_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




