/* ************
 *
 * Copyright (C) 2009  Yu Feng
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to 
 *
 * the Free Software Foundation, Inc., 
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Yu Feng <rainwoodman@gmail.com>
 ***/

#include <glib.h>
#include <glib-object.h>
#include <yaml.h>
#include <stdlib.h>
#include <string.h>
#include <gobject/gvaluecollector.h>


#define G_YAML_TYPE_LOADER (g_yaml_loader_get_type ())
#define G_YAML_LOADER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_TYPE_LOADER, GYAMLLoader))
#define G_YAML_LOADER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), G_YAML_TYPE_LOADER, GYAMLLoaderClass))
#define G_YAML_IS_LOADER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_TYPE_LOADER))
#define G_YAML_IS_LOADER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_YAML_TYPE_LOADER))
#define G_YAML_LOADER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), G_YAML_TYPE_LOADER, GYAMLLoaderClass))

typedef struct _GYAMLLoader GYAMLLoader;
typedef struct _GYAMLLoaderClass GYAMLLoaderClass;
typedef struct _GYAMLLoaderPrivate GYAMLLoaderPrivate;

#define G_YAML_TYPE_DOCUMENT (g_yaml_document_get_type ())
#define G_YAML_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_TYPE_DOCUMENT, GYAMLDocument))
#define G_YAML_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), G_YAML_TYPE_DOCUMENT, GYAMLDocumentClass))
#define G_YAML_IS_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_TYPE_DOCUMENT))
#define G_YAML_IS_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_YAML_TYPE_DOCUMENT))
#define G_YAML_DOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), G_YAML_TYPE_DOCUMENT, GYAMLDocumentClass))

typedef struct _GYAMLDocument GYAMLDocument;
typedef struct _GYAMLDocumentClass GYAMLDocumentClass;
typedef struct _GYAMLDocumentPrivate GYAMLDocumentPrivate;

#define G_YAML_TYPE_NODE (g_yaml_node_get_type ())
#define G_YAML_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_TYPE_NODE, GYAMLNode))
#define G_YAML_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), G_YAML_TYPE_NODE, GYAMLNodeClass))
#define G_YAML_IS_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_TYPE_NODE))
#define G_YAML_IS_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_YAML_TYPE_NODE))
#define G_YAML_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), G_YAML_TYPE_NODE, GYAMLNodeClass))

typedef struct _GYAMLNode GYAMLNode;
typedef struct _GYAMLNodeClass GYAMLNodeClass;

#define G_YAML_NODE_TYPE_ALIAS (g_yaml_node_alias_get_type ())
#define G_YAML_NODE_ALIAS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_NODE_TYPE_ALIAS, GYAMLNodeAlias))
#define G_YAML_NODE_ALIAS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), G_YAML_NODE_TYPE_ALIAS, GYAMLNodeAliasClass))
#define G_YAML_NODE_IS_ALIAS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_NODE_TYPE_ALIAS))
#define G_YAML_NODE_IS_ALIAS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_YAML_NODE_TYPE_ALIAS))
#define G_YAML_NODE_ALIAS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), G_YAML_NODE_TYPE_ALIAS, GYAMLNodeAliasClass))

typedef struct _GYAMLNodeAlias GYAMLNodeAlias;
typedef struct _GYAMLNodeAliasClass GYAMLNodeAliasClass;
typedef struct _GYAMLNodePrivate GYAMLNodePrivate;
typedef struct _GYAMLNodeAliasPrivate GYAMLNodeAliasPrivate;

#define G_YAML_NODE_TYPE_SCALAR (g_yaml_node_scalar_get_type ())
#define G_YAML_NODE_SCALAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_NODE_TYPE_SCALAR, GYAMLNodeScalar))
#define G_YAML_NODE_SCALAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), G_YAML_NODE_TYPE_SCALAR, GYAMLNodeScalarClass))
#define G_YAML_NODE_IS_SCALAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_NODE_TYPE_SCALAR))
#define G_YAML_NODE_IS_SCALAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_YAML_NODE_TYPE_SCALAR))
#define G_YAML_NODE_SCALAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), G_YAML_NODE_TYPE_SCALAR, GYAMLNodeScalarClass))

typedef struct _GYAMLNodeScalar GYAMLNodeScalar;
typedef struct _GYAMLNodeScalarClass GYAMLNodeScalarClass;
typedef struct _GYAMLNodeScalarPrivate GYAMLNodeScalarPrivate;

#define G_YAML_NODE_TYPE_SEQUENCE (g_yaml_node_sequence_get_type ())
#define G_YAML_NODE_SEQUENCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_NODE_TYPE_SEQUENCE, GYAMLNodeSequence))
#define G_YAML_NODE_SEQUENCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), G_YAML_NODE_TYPE_SEQUENCE, GYAMLNodeSequenceClass))
#define G_YAML_NODE_IS_SEQUENCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_NODE_TYPE_SEQUENCE))
#define G_YAML_NODE_IS_SEQUENCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_YAML_NODE_TYPE_SEQUENCE))
#define G_YAML_NODE_SEQUENCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), G_YAML_NODE_TYPE_SEQUENCE, GYAMLNodeSequenceClass))

typedef struct _GYAMLNodeSequence GYAMLNodeSequence;
typedef struct _GYAMLNodeSequenceClass GYAMLNodeSequenceClass;
typedef struct _GYAMLNodeSequencePrivate GYAMLNodeSequencePrivate;

#define G_YAML_NODE_TYPE_MAPPING (g_yaml_node_mapping_get_type ())
#define G_YAML_NODE_MAPPING(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_NODE_TYPE_MAPPING, GYAMLNodeMapping))
#define G_YAML_NODE_MAPPING_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), G_YAML_NODE_TYPE_MAPPING, GYAMLNodeMappingClass))
#define G_YAML_NODE_IS_MAPPING(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_NODE_TYPE_MAPPING))
#define G_YAML_NODE_IS_MAPPING_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_YAML_NODE_TYPE_MAPPING))
#define G_YAML_NODE_MAPPING_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), G_YAML_NODE_TYPE_MAPPING, GYAMLNodeMappingClass))

typedef struct _GYAMLNodeMapping GYAMLNodeMapping;
typedef struct _GYAMLNodeMappingClass GYAMLNodeMappingClass;
typedef struct _GYAMLNodeMappingPrivate GYAMLNodeMappingPrivate;
typedef struct _GYAMLParamSpecLoader GYAMLParamSpecLoader;

/**
 * Internal class used to load the document
 */
struct _GYAMLLoader {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GYAMLLoaderPrivate * priv;
};

struct _GYAMLLoaderClass {
	GTypeClass parent_class;
	void (*finalize) (GYAMLLoader *self);
};

struct _GYAMLLoaderPrivate {
	GYAMLDocument* document;
};

typedef enum  {
	G_YAML_ERROR_SYMBOL_NOT_FOUND,
	G_YAML_ERROR_TYPE_NOT_FOUND,
	G_YAML_ERROR_NOT_A_BUILDABLE,
	G_YAML_ERROR_PROPERTY_NOT_FOUND,
	G_YAML_ERROR_OBJECT_NOT_FOUND,
	G_YAML_ERROR_UNKNOWN_PROPERTY_TYPE,
	G_YAML_ERROR_CUSTOM_NODE_ERROR,
	G_YAML_ERROR_UNEXPECTED_NODE,
	G_YAML_ERROR_NOT_IMPLEMENTED,
	G_YAML_ERROR_FILE_NOT_FOUND,
	G_YAML_ERROR_PARSER_ERROR,
	G_YAML_ERROR_UNRESOLVED_ALIAS
} GYAMLError;
#define G_YAML_ERROR g_yaml_error_quark ()
/**
 * A YAML Document
 *
 * Refer to the YAML 1.1 spec for the definitions.
 *
 * The document model based on GType classes replaces the original libyaml
 * document model.
 *
 * [warning:
 *  This is not a full implementation of a YAML document.
 *  The document tag directive is missing.
 *  Alias is not immediately resolved and replaced with the referred node.
 * ]
 */
struct _GYAMLDocument {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GYAMLDocumentPrivate * priv;
	GList* nodes;
	yaml_mark_t start_mark;
	yaml_mark_t end_mark;
	GHashTable* anchors;
	GYAMLNode* root;
};

struct _GYAMLDocumentClass {
	GTypeClass parent_class;
	void (*finalize) (GYAMLDocument *self);
};

/**
 * A YAML Node.
 *
 * YAML supports three types of nodes. They are converted to
 * GTypes
 *
 *   [| ++YAML++ || ++GType++ |]
 *   [| Scalar || Node.Scalar |]
 *   [| Sequence || Node.Sequence |]
 *   [| Mapping || Node.Mapping |]
 *   [| Alias || Node.Alias |]
 *
 * Each node type utilizes the fundamental GLib types to store the
 * YAML data.
 *
 * A pointer can be binded to the node with get_pointer and set_pointer.
 * This pointer is used by GLib.YAML.Builder to hold the built object.
 *
 * */
struct _GYAMLNode {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GYAMLNodePrivate * priv;
	yaml_node_type_t type;
	char* tag;
	yaml_mark_t start_mark;
	yaml_mark_t end_mark;
	char* anchor;
};

struct _GYAMLNodeClass {
	GTypeClass parent_class;
	void (*finalize) (GYAMLNode *self);
};

/**
 * An Alias Node
 *
 * Refer to the YAML 1.1 spec for the definitions.
 *
 * Note that the explanation of alias
 * is different from the explanation of alias in standard YAML.
 * The resolution of aliases are deferred, allowing forward-
 * referring aliases; whereas in standard YAML, forward-referring
 * aliases is undefined.
 * */
struct _GYAMLNodeAlias {
	GYAMLNode parent_instance;
	GYAMLNodeAliasPrivate * priv;
	GYAMLNode* node;
};

struct _GYAMLNodeAliasClass {
	GYAMLNodeClass parent_class;
};

/**
 * A Scalar Node
 *
 * Refer to the YAML 1.1 spec for the definitions.
 *
 * The scalar value is internally stored as a string,
 * or `gchar*'.
 */
struct _GYAMLNodeScalar {
	GYAMLNode parent_instance;
	GYAMLNodeScalarPrivate * priv;
	char* value;
	yaml_scalar_style_t style;
};

struct _GYAMLNodeScalarClass {
	GYAMLNodeClass parent_class;
};

/**
 * A Sequence Node
 * 
 * Refer to the YAML 1.1 spec for the definitions.
 *
 * The sequence is internally stored as a GList.
 */
struct _GYAMLNodeSequence {
	GYAMLNode parent_instance;
	GYAMLNodeSequencePrivate * priv;
	GList* items;
	yaml_sequence_style_t style;
};

struct _GYAMLNodeSequenceClass {
	GYAMLNodeClass parent_class;
};

/**
 * A Mapping Node
 * 
 * Refer to the YAML 1.1 spec for the definitions.
 *
 * The mapping is internally stored as a GHashTable.
 *
 * An extra list of keys is stored to ease iterating overall
 * elements. GHashTable.get_keys is not available in GLib 2.12.
 */
struct _GYAMLNodeMapping {
	GYAMLNode parent_instance;
	GYAMLNodeMappingPrivate * priv;
	GHashTable* pairs;
	GList* keys;
	yaml_mapping_style_t style;
};

struct _GYAMLNodeMappingClass {
	GYAMLNodeClass parent_class;
};

struct _GYAMLParamSpecLoader {
	GParamSpec parent_instance;
};


static gpointer g_yaml_loader_parent_class = NULL;

gpointer g_yaml_loader_ref (gpointer instance);
void g_yaml_loader_unref (gpointer instance);
GParamSpec* g_yaml_param_spec_loader (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void g_yaml_value_set_loader (GValue* value, gpointer v_object);
gpointer g_yaml_value_get_loader (const GValue* value);
GType g_yaml_loader_get_type (void);
gpointer g_yaml_document_ref (gpointer instance);
void g_yaml_document_unref (gpointer instance);
GParamSpec* g_yaml_param_spec_document (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void g_yaml_value_set_document (GValue* value, gpointer v_object);
gpointer g_yaml_value_get_document (const GValue* value);
GType g_yaml_document_get_type (void);
#define G_YAML_LOADER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), G_YAML_TYPE_LOADER, GYAMLLoaderPrivate))
enum  {
	G_YAML_LOADER_DUMMY_PROPERTY
};
GYAMLLoader* g_yaml_loader_new (void);
GYAMLLoader* g_yaml_loader_construct (GType object_type);
GQuark g_yaml_error_quark (void);
static void g_yaml_loader_parse_with_throw (GYAMLLoader* self, yaml_parser_t* parser, yaml_event_t* event, GError** error);
gpointer g_yaml_node_ref (gpointer instance);
void g_yaml_node_unref (gpointer instance);
GParamSpec* g_yaml_param_spec_node (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void g_yaml_value_set_node (GValue* value, gpointer v_object);
gpointer g_yaml_value_get_node (const GValue* value);
GType g_yaml_node_get_type (void);
GYAMLNode* g_yaml_loader_load_node (GYAMLLoader* self, yaml_parser_t* parser, yaml_event_t* last_event, GError** error);
GType g_yaml_node_alias_get_type (void);
gboolean g_yaml_loader_load (GYAMLLoader* self, yaml_parser_t* parser, GYAMLDocument* document, GError** error);
GYAMLNode* g_yaml_loader_load_alias (GYAMLLoader* self, yaml_parser_t* parser, yaml_event_t* event, GError** error);
GYAMLNode* g_yaml_loader_load_scalar (GYAMLLoader* self, yaml_parser_t* parser, yaml_event_t* event, GError** error);
GYAMLNode* g_yaml_loader_load_sequence (GYAMLLoader* self, yaml_parser_t* parser, yaml_event_t* event, GError** error);
GYAMLNode* g_yaml_loader_load_mapping (GYAMLLoader* self, yaml_parser_t* parser, yaml_event_t* event, GError** error);
GYAMLNodeAlias* g_yaml_node_alias_new (void);
GYAMLNodeAlias* g_yaml_node_alias_construct (GType object_type);
static char* g_yaml_loader_normalize_tag (const char* tag, const char* _default_, GError** error);
GYAMLNodeScalar* g_yaml_node_scalar_new (void);
GYAMLNodeScalar* g_yaml_node_scalar_construct (GType object_type);
GType g_yaml_node_scalar_get_type (void);
GYAMLNodeSequence* g_yaml_node_sequence_new (void);
GYAMLNodeSequence* g_yaml_node_sequence_construct (GType object_type);
GType g_yaml_node_sequence_get_type (void);
GYAMLNodeMapping* g_yaml_node_mapping_new (void);
GYAMLNodeMapping* g_yaml_node_mapping_construct (GType object_type);
GType g_yaml_node_mapping_get_type (void);
static void g_yaml_loader_finalize (GYAMLLoader* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);



#line 37 "loader.vala"
GYAMLLoader* g_yaml_loader_construct (GType object_type) {
#line 349 "loader.c"
	GYAMLLoader* self;
	self = (GYAMLLoader*) g_type_create_instance (object_type);
	return self;
}


#line 37 "loader.vala"
GYAMLLoader* g_yaml_loader_new (void) {
#line 37 "loader.vala"
	return g_yaml_loader_construct (G_YAML_TYPE_LOADER);
#line 360 "loader.c"
}


#line 115 "libyaml-1.0.vapi"
static char* yaml_mark_to_string (yaml_mark_t *self) {
#line 366 "loader.c"
	char* result;
	result = g_strdup_printf ("index:%u line:%u column:%u", (guint) (*self).index, (guint) (*self).line, (guint) (*self).column);
	return result;
}


#line 38 "loader.vala"
static void g_yaml_loader_parse_with_throw (GYAMLLoader* self, yaml_parser_t* parser, yaml_event_t* event, GError** error) {
#line 375 "loader.c"
	GError * _inner_error_;
	yaml_event_t _tmp2_ = {0};
	gboolean _tmp1_;
	yaml_event_t _tmp0_ = {0};
	char* _tmp3_;
	GError* _tmp4_;
#line 38 "loader.vala"
	g_return_if_fail (self != NULL);
#line 384 "loader.c"
	_inner_error_ = NULL;
#line 40 "loader.vala"
	if ((_tmp1_ = yaml_parser_parse (&(*parser), &_tmp0_), (*event) = (_tmp2_ = _tmp0_, yaml_event_delete (&(*event)), _tmp2_), _tmp1_)) {
#line 388 "loader.c"
		return;
	}
#line 115 "libyaml-1.0.vapi"
	_tmp3_ = NULL;
#line 43 "loader.vala"
	_tmp4_ = NULL;
#line 395 "loader.c"
	_inner_error_ = (_tmp4_ = g_error_new (G_YAML_ERROR, G_YAML_ERROR_PARSER_ERROR, "Parser encounters an error: %s at %u(%s)\n" "Error Context: '%s'", (*parser).problem, (*parser).problem_offset, _tmp3_ = yaml_mark_to_string (&(*parser).problem_mark), (*parser).context), _tmp3_ = (g_free (_tmp3_), NULL), _tmp4_);
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_YAML_ERROR) {
			g_propagate_error (error, _inner_error_);
			return;
		} else {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
}


/**
 * Load a YAML stream from a Parser to a Document.
 *
 * Alias are looked up at the very end of the stage.
 */
#line 59 "loader.vala"
gboolean g_yaml_loader_load (GYAMLLoader* self, yaml_parser_t* parser, GYAMLDocument* document, GError** error) {
#line 417 "loader.c"
	gboolean result;
	GError * _inner_error_;
	GYAMLDocument* _tmp1_;
	GYAMLDocument* _tmp0_;
	yaml_event_t event = {0};
	yaml_event_t _tmp5_ = {0};
	yaml_event_t _tmp4_ = {0};
	yaml_event_t _tmp7_ = {0};
	yaml_event_t _tmp6_ = {0};
	GYAMLNode* _tmp8_;
	GYAMLNode* _tmp9_;
	yaml_event_t _tmp11_ = {0};
	yaml_event_t _tmp10_ = {0};
#line 59 "loader.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 59 "loader.vala"
	g_return_val_if_fail (document != NULL, FALSE);
#line 435 "loader.c"
	_inner_error_ = NULL;
	_tmp1_ = NULL;
#line 61 "loader.vala"
	_tmp0_ = NULL;
#line 61 "loader.vala"
	self->priv->document = (_tmp1_ = (_tmp0_ = document, (_tmp0_ == NULL) ? NULL : g_yaml_document_ref (_tmp0_)), (self->priv->document == NULL) ? NULL : (self->priv->document = (g_yaml_document_unref (self->priv->document), NULL)), _tmp1_);
#line 442 "loader.c"
	/* Look for a StreamStart */
#line 64 "loader.vala"
	if (!(*parser).stream_start_produced) {
#line 446 "loader.c"
		yaml_event_t _tmp3_ = {0};
		yaml_event_t _tmp2_ = {0};
#line 65 "loader.vala"
		g_yaml_loader_parse_with_throw (self, &(*parser), &_tmp2_, &_inner_error_);
#line 65 "loader.vala"
		event = (_tmp3_ = _tmp2_, yaml_event_delete (&event), _tmp3_);
#line 453 "loader.c"
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_YAML_ERROR) {
				g_propagate_error (error, _inner_error_);
				yaml_event_delete (&event);
				return FALSE;
			} else {
				yaml_event_delete (&event);
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return FALSE;
			}
		}
#line 66 "loader.vala"
		g_assert (event.type == YAML_STREAM_START_EVENT);
#line 468 "loader.c"
	}
#line 68 "loader.vala"
	g_return_val_if_fail (!(*parser).stream_end_produced, TRUE);
#line 70 "loader.vala"
	g_yaml_loader_parse_with_throw (self, &(*parser), &_tmp4_, &_inner_error_);
#line 70 "loader.vala"
	event = (_tmp5_ = _tmp4_, yaml_event_delete (&event), _tmp5_);
#line 476 "loader.c"
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_YAML_ERROR) {
			g_propagate_error (error, _inner_error_);
			yaml_event_delete (&event);
			return FALSE;
		} else {
			yaml_event_delete (&event);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return FALSE;
		}
	}
	/* if a StreamEnd seen, return OK */
#line 72 "loader.vala"
	g_return_val_if_fail (event.type != YAML_STREAM_END_EVENT, TRUE);
#line 492 "loader.c"
	/* expecting a DocumentStart otherwise */
#line 75 "loader.vala"
	g_assert (event.type == YAML_DOCUMENT_START_EVENT);
#line 76 "loader.vala"
	document->start_mark = event.start_mark;
#line 78 "loader.vala"
	g_yaml_loader_parse_with_throw (self, &(*parser), &_tmp6_, &_inner_error_);
#line 78 "loader.vala"
	event = (_tmp7_ = _tmp6_, yaml_event_delete (&event), _tmp7_);
#line 502 "loader.c"
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_YAML_ERROR) {
			g_propagate_error (error, _inner_error_);
			yaml_event_delete (&event);
			return FALSE;
		} else {
			yaml_event_delete (&event);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return FALSE;
		}
	}
	_tmp8_ = g_yaml_loader_load_node (self, &(*parser), &event, &_inner_error_);
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_YAML_ERROR) {
			g_propagate_error (error, _inner_error_);
			yaml_event_delete (&event);
			return FALSE;
		} else {
			yaml_event_delete (&event);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return FALSE;
		}
	}
	/* Load the first node. 
	 * load_node with recursively load other nodes */
	_tmp9_ = NULL;
#line 81 "loader.vala"
	document->root = (_tmp9_ = _tmp8_, (document->root == NULL) ? NULL : (document->root = (g_yaml_node_unref (document->root), NULL)), _tmp9_);
#line 533 "loader.c"
	/* expecting for a DocumentEnd */
#line 84 "loader.vala"
	g_yaml_loader_parse_with_throw (self, &(*parser), &_tmp10_, &_inner_error_);
#line 84 "loader.vala"
	event = (_tmp11_ = _tmp10_, yaml_event_delete (&event), _tmp11_);
#line 539 "loader.c"
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_YAML_ERROR) {
			g_propagate_error (error, _inner_error_);
			yaml_event_delete (&event);
			return FALSE;
		} else {
			yaml_event_delete (&event);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return FALSE;
		}
	}
#line 85 "loader.vala"
	g_assert (event.type == YAML_DOCUMENT_END_EVENT);
#line 86 "loader.vala"
	document->end_mark = event.end_mark;
#line 556 "loader.c"
	/* preserve the document order */
#line 89 "loader.vala"
	document->nodes = g_list_reverse (document->nodes);
#line 560 "loader.c"
	/* resolve the aliases */
	{
		GList* node_collection;
		GList* node_it;
#line 92 "loader.vala"
		node_collection = document->nodes;
#line 567 "loader.c"
		for (node_it = node_collection; node_it != NULL; node_it = node_it->next) {
			GYAMLNode* _tmp16_;
			GYAMLNode* node;
#line 190 "libyaml-glib.vala"
			_tmp16_ = NULL;
#line 92 "loader.vala"
			node = (_tmp16_ = (GYAMLNode*) node_it->data, (_tmp16_ == NULL) ? NULL : g_yaml_node_ref (_tmp16_));
#line 575 "loader.c"
			{
				GYAMLNodeAlias* _tmp13_;
				GYAMLNode* _tmp12_;
				GYAMLNodeAlias* alias_node;
				GYAMLNode* _tmp15_;
				GYAMLNode* _tmp14_;
#line 93 "loader.vala"
				if (!G_YAML_NODE_IS_ALIAS (node)) {
#line 584 "loader.c"
					(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
#line 93 "loader.vala"
					continue;
#line 588 "loader.c"
				}
#line 94 "loader.vala"
				_tmp13_ = NULL;
#line 94 "loader.vala"
				_tmp12_ = NULL;
#line 594 "loader.c"
				alias_node = (_tmp13_ = (_tmp12_ = node, G_YAML_NODE_IS_ALIAS (_tmp12_) ? ((GYAMLNodeAlias*) _tmp12_) : NULL), (_tmp13_ == NULL) ? NULL : g_yaml_node_ref (_tmp13_));
				_tmp15_ = NULL;
#line 95 "loader.vala"
				_tmp14_ = NULL;
#line 95 "loader.vala"
				alias_node->node = (_tmp15_ = (_tmp14_ = (GYAMLNode*) g_hash_table_lookup (document->anchors, ((GYAMLNode*) alias_node)->anchor), (_tmp14_ == NULL) ? NULL : g_yaml_node_ref (_tmp14_)), (alias_node->node == NULL) ? NULL : (alias_node->node = (g_yaml_node_unref (alias_node->node), NULL)), _tmp15_);
#line 96 "loader.vala"
				if (alias_node != NULL) {
#line 603 "loader.c"
					(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
					(alias_node == NULL) ? NULL : (alias_node = (g_yaml_node_unref (alias_node), NULL));
#line 96 "loader.vala"
					continue;
#line 608 "loader.c"
				}
				_inner_error_ = g_error_new (G_YAML_ERROR, G_YAML_ERROR_UNRESOLVED_ALIAS, "Alias '%s' cannot be resolved.", ((GYAMLNode*) alias_node)->anchor);
				if (_inner_error_ != NULL) {
					if (_inner_error_->domain == G_YAML_ERROR) {
						g_propagate_error (error, _inner_error_);
						(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
						(alias_node == NULL) ? NULL : (alias_node = (g_yaml_node_unref (alias_node), NULL));
						yaml_event_delete (&event);
						return FALSE;
					} else {
						(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
						(alias_node == NULL) ? NULL : (alias_node = (g_yaml_node_unref (alias_node), NULL));
						yaml_event_delete (&event);
						g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
						g_clear_error (&_inner_error_);
						return FALSE;
					}
				}
				(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
				(alias_node == NULL) ? NULL : (alias_node = (g_yaml_node_unref (alias_node), NULL));
			}
		}
	}
	result = TRUE;
	yaml_event_delete (&event);
	return result;
}


/**
 * Load a node from a YAML Event.
 * 
 * @return the loaded node.
 */
#line 108 "loader.vala"
GYAMLNode* g_yaml_loader_load_node (GYAMLLoader* self, yaml_parser_t* parser, yaml_event_t* last_event, GError** error) {
#line 645 "loader.c"
	GYAMLNode* result;
	GError * _inner_error_;
#line 108 "loader.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 650 "loader.c"
	_inner_error_ = NULL;
#line 110 "loader.vala"
	switch ((*last_event).type) {
#line 654 "loader.c"
		case YAML_ALIAS_EVENT:
		{
			GYAMLNode* _tmp0_;
			_tmp0_ = g_yaml_loader_load_alias (self, &(*parser), &(*last_event), &_inner_error_);
			if (_inner_error_ != NULL) {
				if (_inner_error_->domain == G_YAML_ERROR) {
					g_propagate_error (error, _inner_error_);
					return NULL;
				} else {
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return NULL;
				}
			}
			result = _tmp0_;
			return result;
		}
		case YAML_SCALAR_EVENT:
		{
			GYAMLNode* _tmp1_;
			_tmp1_ = g_yaml_loader_load_scalar (self, &(*parser), &(*last_event), &_inner_error_);
			if (_inner_error_ != NULL) {
				if (_inner_error_->domain == G_YAML_ERROR) {
					g_propagate_error (error, _inner_error_);
					return NULL;
				} else {
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return NULL;
				}
			}
			result = _tmp1_;
			return result;
		}
		case YAML_SEQUENCE_START_EVENT:
		{
			GYAMLNode* _tmp2_;
			_tmp2_ = g_yaml_loader_load_sequence (self, &(*parser), &(*last_event), &_inner_error_);
			if (_inner_error_ != NULL) {
				if (_inner_error_->domain == G_YAML_ERROR) {
					g_propagate_error (error, _inner_error_);
					return NULL;
				} else {
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return NULL;
				}
			}
			result = _tmp2_;
			return result;
		}
		case YAML_MAPPING_START_EVENT:
		{
			GYAMLNode* _tmp3_;
			_tmp3_ = g_yaml_loader_load_mapping (self, &(*parser), &(*last_event), &_inner_error_);
			if (_inner_error_ != NULL) {
				if (_inner_error_->domain == G_YAML_ERROR) {
					g_propagate_error (error, _inner_error_);
					return NULL;
				} else {
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return NULL;
				}
			}
			result = _tmp3_;
			return result;
		}
		default:
		{
#line 120 "loader.vala"
			g_assert_not_reached ();
#line 727 "loader.c"
		}
	}
}


#line 123 "loader.vala"
GYAMLNode* g_yaml_loader_load_alias (GYAMLLoader* self, yaml_parser_t* parser, yaml_event_t* event, GError** error) {
#line 735 "loader.c"
	GYAMLNode* result;
	GYAMLNodeAlias* node;
	char* _tmp1_;
	const char* _tmp0_;
	GYAMLNode* _tmp2_;
#line 123 "loader.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 743 "loader.c"
	node = g_yaml_node_alias_new ();
	_tmp1_ = NULL;
#line 126 "loader.vala"
	_tmp0_ = NULL;
#line 126 "loader.vala"
	((GYAMLNode*) node)->anchor = (_tmp1_ = (_tmp0_ = (*event).data.alias.anchor, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), ((GYAMLNode*) node)->anchor = (g_free (((GYAMLNode*) node)->anchor), NULL), _tmp1_);
#line 750 "loader.c"
	/* Push the node to the document stack
	 * Do not register the anchor because it is an alias */
#line 130 "loader.vala"
	_tmp2_ = NULL;
#line 130 "loader.vala"
	self->priv->document->nodes = g_list_prepend (self->priv->document->nodes, (_tmp2_ = (GYAMLNode*) node, (_tmp2_ == NULL) ? NULL : g_yaml_node_ref (_tmp2_)));
#line 757 "loader.c"
	result = (GYAMLNode*) node;
	return result;
}


#line 134 "loader.vala"
static char* g_yaml_loader_normalize_tag (const char* tag, const char* _default_, GError** error) {
#line 765 "loader.c"
	char* result;
	gboolean _tmp0_;
	const char* _tmp2_;
#line 134 "loader.vala"
	g_return_val_if_fail (_default_ != NULL, NULL);
#line 771 "loader.c"
	_tmp0_ = FALSE;
#line 136 "loader.vala"
	if (tag == NULL) {
#line 136 "loader.vala"
		_tmp0_ = TRUE;
#line 777 "loader.c"
	} else {
#line 136 "loader.vala"
		_tmp0_ = _vala_strcmp0 (tag, "!") == 0;
#line 781 "loader.c"
	}
#line 136 "loader.vala"
	if (_tmp0_) {
#line 785 "loader.c"
		const char* _tmp1_;
#line 137 "loader.vala"
		_tmp1_ = NULL;
#line 789 "loader.c"
		result = (_tmp1_ = _default_, (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_));
		return result;
	}
#line 139 "loader.vala"
	_tmp2_ = NULL;
#line 795 "loader.c"
	result = (_tmp2_ = tag, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_));
	return result;
}


#line 141 "loader.vala"
GYAMLNode* g_yaml_loader_load_scalar (GYAMLLoader* self, yaml_parser_t* parser, yaml_event_t* event, GError** error) {
#line 803 "loader.c"
	GYAMLNode* result;
	GError * _inner_error_;
	GYAMLNodeScalar* node;
	char* _tmp1_;
	const char* _tmp0_;
	char* _tmp2_;
	char* _tmp3_;
	char* _tmp5_;
	const char* _tmp4_;
	GYAMLNode* _tmp6_;
#line 141 "loader.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 816 "loader.c"
	_inner_error_ = NULL;
	node = g_yaml_node_scalar_new ();
	_tmp1_ = NULL;
#line 144 "loader.vala"
	_tmp0_ = NULL;
#line 144 "loader.vala"
	((GYAMLNode*) node)->anchor = (_tmp1_ = (_tmp0_ = (*event).data.scalar.anchor, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), ((GYAMLNode*) node)->anchor = (g_free (((GYAMLNode*) node)->anchor), NULL), _tmp1_);
#line 824 "loader.c"
	_tmp2_ = g_yaml_loader_normalize_tag ((*event).data.scalar.tag, YAML_DEFAULT_SCALAR_TAG, &_inner_error_);
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_YAML_ERROR) {
			g_propagate_error (error, _inner_error_);
			(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
			return NULL;
		} else {
			(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return NULL;
		}
	}
	_tmp3_ = NULL;
#line 145 "loader.vala"
	((GYAMLNode*) node)->tag = (_tmp3_ = _tmp2_, ((GYAMLNode*) node)->tag = (g_free (((GYAMLNode*) node)->tag), NULL), _tmp3_);
#line 841 "loader.c"
	_tmp5_ = NULL;
#line 147 "loader.vala"
	_tmp4_ = NULL;
#line 147 "loader.vala"
	node->value = (_tmp5_ = (_tmp4_ = (*event).data.scalar.value, (_tmp4_ == NULL) ? NULL : g_strdup (_tmp4_)), node->value = (g_free (node->value), NULL), _tmp5_);
#line 148 "loader.vala"
	node->style = (*event).data.scalar.style;
#line 149 "loader.vala"
	((GYAMLNode*) node)->start_mark = (*event).start_mark;
#line 150 "loader.vala"
	((GYAMLNode*) node)->end_mark = (*event).end_mark;
#line 853 "loader.c"
	/* Push the node to the document stack
	 * and register the anchor */
#line 154 "loader.vala"
	_tmp6_ = NULL;
#line 154 "loader.vala"
	self->priv->document->nodes = g_list_prepend (self->priv->document->nodes, (_tmp6_ = (GYAMLNode*) node, (_tmp6_ == NULL) ? NULL : g_yaml_node_ref (_tmp6_)));
#line 155 "loader.vala"
	if (((GYAMLNode*) node)->anchor != NULL) {
#line 862 "loader.c"
		GYAMLNode* _tmp8_;
		const char* _tmp7_;
#line 156 "loader.vala"
		_tmp8_ = NULL;
#line 156 "loader.vala"
		_tmp7_ = NULL;
#line 156 "loader.vala"
		g_hash_table_insert (self->priv->document->anchors, (_tmp7_ = ((GYAMLNode*) node)->anchor, (_tmp7_ == NULL) ? NULL : g_strdup (_tmp7_)), (_tmp8_ = (GYAMLNode*) node, (_tmp8_ == NULL) ? NULL : g_yaml_node_ref (_tmp8_)));
#line 871 "loader.c"
	}
	result = (GYAMLNode*) node;
	return result;
}


#line 159 "loader.vala"
GYAMLNode* g_yaml_loader_load_sequence (GYAMLLoader* self, yaml_parser_t* parser, yaml_event_t* event, GError** error) {
#line 880 "loader.c"
	GYAMLNode* result;
	GError * _inner_error_;
	GYAMLNodeSequence* node;
	char* _tmp1_;
	const char* _tmp0_;
	char* _tmp2_;
	char* _tmp3_;
	GYAMLNode* _tmp4_;
	yaml_event_t _tmp8_ = {0};
	yaml_event_t _tmp7_ = {0};
#line 159 "loader.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 893 "loader.c"
	_inner_error_ = NULL;
	node = g_yaml_node_sequence_new ();
	_tmp1_ = NULL;
#line 162 "loader.vala"
	_tmp0_ = NULL;
#line 162 "loader.vala"
	((GYAMLNode*) node)->anchor = (_tmp1_ = (_tmp0_ = (*event).data.sequence_start.anchor, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), ((GYAMLNode*) node)->anchor = (g_free (((GYAMLNode*) node)->anchor), NULL), _tmp1_);
#line 901 "loader.c"
	_tmp2_ = g_yaml_loader_normalize_tag ((*event).data.sequence_start.tag, YAML_DEFAULT_SEQUENCE_TAG, &_inner_error_);
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_YAML_ERROR) {
			g_propagate_error (error, _inner_error_);
			(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
			return NULL;
		} else {
			(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return NULL;
		}
	}
	_tmp3_ = NULL;
#line 163 "loader.vala"
	((GYAMLNode*) node)->tag = (_tmp3_ = _tmp2_, ((GYAMLNode*) node)->tag = (g_free (((GYAMLNode*) node)->tag), NULL), _tmp3_);
#line 165 "loader.vala"
	node->style = (*event).data.sequence_start.style;
#line 166 "loader.vala"
	((GYAMLNode*) node)->start_mark = (*event).start_mark;
#line 167 "loader.vala"
	((GYAMLNode*) node)->end_mark = (*event).end_mark;
#line 924 "loader.c"
	/* Push the node to the document stack
	 * and register the anchor */
#line 171 "loader.vala"
	_tmp4_ = NULL;
#line 171 "loader.vala"
	self->priv->document->nodes = g_list_prepend (self->priv->document->nodes, (_tmp4_ = (GYAMLNode*) node, (_tmp4_ == NULL) ? NULL : g_yaml_node_ref (_tmp4_)));
#line 172 "loader.vala"
	if (((GYAMLNode*) node)->anchor != NULL) {
#line 933 "loader.c"
		GYAMLNode* _tmp6_;
		const char* _tmp5_;
#line 173 "loader.vala"
		_tmp6_ = NULL;
#line 173 "loader.vala"
		_tmp5_ = NULL;
#line 173 "loader.vala"
		g_hash_table_insert (self->priv->document->anchors, (_tmp5_ = ((GYAMLNode*) node)->anchor, (_tmp5_ == NULL) ? NULL : g_strdup (_tmp5_)), (_tmp6_ = (GYAMLNode*) node, (_tmp6_ == NULL) ? NULL : g_yaml_node_ref (_tmp6_)));
#line 942 "loader.c"
	}
	/* Load the items in the sequence */
#line 176 "loader.vala"
	g_yaml_loader_parse_with_throw (self, &(*parser), &_tmp7_, &_inner_error_);
#line 176 "loader.vala"
	(*event) = (_tmp8_ = _tmp7_, yaml_event_delete (&(*event)), _tmp8_);
#line 949 "loader.c"
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_YAML_ERROR) {
			g_propagate_error (error, _inner_error_);
			(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
			return NULL;
		} else {
			(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return NULL;
		}
	}
#line 177 "loader.vala"
	while (TRUE) {
#line 964 "loader.c"
		GYAMLNode* item;
		GYAMLNode* _tmp9_;
		yaml_event_t _tmp11_ = {0};
		yaml_event_t _tmp10_ = {0};
#line 177 "loader.vala"
		if (!((*event).type != YAML_SEQUENCE_END_EVENT)) {
#line 177 "loader.vala"
			break;
#line 973 "loader.c"
		}
		item = g_yaml_loader_load_node (self, &(*parser), &(*event), &_inner_error_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_YAML_ERROR) {
				g_propagate_error (error, _inner_error_);
				(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
				return NULL;
			} else {
				(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return NULL;
			}
		}
		/* prepend is faster than append */
#line 180 "loader.vala"
		_tmp9_ = NULL;
#line 180 "loader.vala"
		node->items = g_list_prepend (node->items, (_tmp9_ = item, (_tmp9_ == NULL) ? NULL : g_yaml_node_ref (_tmp9_)));
#line 181 "loader.vala"
		g_yaml_loader_parse_with_throw (self, &(*parser), &_tmp10_, &_inner_error_);
#line 181 "loader.vala"
		(*event) = (_tmp11_ = _tmp10_, yaml_event_delete (&(*event)), _tmp11_);
#line 997 "loader.c"
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_YAML_ERROR) {
				g_propagate_error (error, _inner_error_);
				(item == NULL) ? NULL : (item = (g_yaml_node_unref (item), NULL));
				(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
				return NULL;
			} else {
				(item == NULL) ? NULL : (item = (g_yaml_node_unref (item), NULL));
				(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return NULL;
			}
		}
		(item == NULL) ? NULL : (item = (g_yaml_node_unref (item), NULL));
	}
	/* Preserve the document order */
#line 184 "loader.vala"
	node->items = g_list_reverse (node->items);
#line 1017 "loader.c"
	/* move the end mark of the mapping
	 * to the END_SEQUENCE_EVENT */
#line 188 "loader.vala"
	((GYAMLNode*) node)->end_mark = (*event).end_mark;
#line 1022 "loader.c"
	result = (GYAMLNode*) node;
	return result;
}


#line 191 "loader.vala"
GYAMLNode* g_yaml_loader_load_mapping (GYAMLLoader* self, yaml_parser_t* parser, yaml_event_t* event, GError** error) {
#line 1030 "loader.c"
	GYAMLNode* result;
	GError * _inner_error_;
	GYAMLNodeMapping* node;
	char* _tmp0_;
	char* _tmp1_;
	char* _tmp3_;
	const char* _tmp2_;
	GYAMLNode* _tmp4_;
	yaml_event_t _tmp8_ = {0};
	yaml_event_t _tmp7_ = {0};
#line 191 "loader.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 1043 "loader.c"
	_inner_error_ = NULL;
	node = g_yaml_node_mapping_new ();
	_tmp0_ = g_yaml_loader_normalize_tag ((*event).data.mapping_start.tag, YAML_DEFAULT_MAPPING_TAG, &_inner_error_);
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_YAML_ERROR) {
			g_propagate_error (error, _inner_error_);
			(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
			return NULL;
		} else {
			(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return NULL;
		}
	}
	_tmp1_ = NULL;
#line 194 "loader.vala"
	((GYAMLNode*) node)->tag = (_tmp1_ = _tmp0_, ((GYAMLNode*) node)->tag = (g_free (((GYAMLNode*) node)->tag), NULL), _tmp1_);
#line 1062 "loader.c"
	_tmp3_ = NULL;
#line 196 "loader.vala"
	_tmp2_ = NULL;
#line 196 "loader.vala"
	((GYAMLNode*) node)->anchor = (_tmp3_ = (_tmp2_ = (*event).data.mapping_start.anchor, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)), ((GYAMLNode*) node)->anchor = (g_free (((GYAMLNode*) node)->anchor), NULL), _tmp3_);
#line 197 "loader.vala"
	node->style = (*event).data.mapping_start.style;
#line 198 "loader.vala"
	((GYAMLNode*) node)->start_mark = (*event).start_mark;
#line 199 "loader.vala"
	((GYAMLNode*) node)->end_mark = (*event).end_mark;
#line 1074 "loader.c"
	/* Push the node to the document stack
	 * and register the anchor */
#line 203 "loader.vala"
	_tmp4_ = NULL;
#line 203 "loader.vala"
	self->priv->document->nodes = g_list_prepend (self->priv->document->nodes, (_tmp4_ = (GYAMLNode*) node, (_tmp4_ == NULL) ? NULL : g_yaml_node_ref (_tmp4_)));
#line 204 "loader.vala"
	if (((GYAMLNode*) node)->anchor != NULL) {
#line 1083 "loader.c"
		GYAMLNode* _tmp6_;
		const char* _tmp5_;
#line 205 "loader.vala"
		_tmp6_ = NULL;
#line 205 "loader.vala"
		_tmp5_ = NULL;
#line 205 "loader.vala"
		g_hash_table_insert (self->priv->document->anchors, (_tmp5_ = ((GYAMLNode*) node)->anchor, (_tmp5_ == NULL) ? NULL : g_strdup (_tmp5_)), (_tmp6_ = (GYAMLNode*) node, (_tmp6_ == NULL) ? NULL : g_yaml_node_ref (_tmp6_)));
#line 1092 "loader.c"
	}
	/* Load the items in the mapping */
#line 208 "loader.vala"
	g_yaml_loader_parse_with_throw (self, &(*parser), &_tmp7_, &_inner_error_);
#line 208 "loader.vala"
	(*event) = (_tmp8_ = _tmp7_, yaml_event_delete (&(*event)), _tmp8_);
#line 1099 "loader.c"
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_YAML_ERROR) {
			g_propagate_error (error, _inner_error_);
			(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
			return NULL;
		} else {
			(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return NULL;
		}
	}
#line 209 "loader.vala"
	while (TRUE) {
#line 1114 "loader.c"
		GYAMLNode* key;
		yaml_event_t _tmp10_ = {0};
		yaml_event_t _tmp9_ = {0};
		GYAMLNode* value;
		GYAMLNode* _tmp12_;
		GYAMLNode* _tmp11_;
		GYAMLNode* _tmp13_;
		yaml_event_t _tmp15_ = {0};
		yaml_event_t _tmp14_ = {0};
#line 209 "loader.vala"
		if (!((*event).type != YAML_MAPPING_END_EVENT)) {
#line 209 "loader.vala"
			break;
#line 1128 "loader.c"
		}
		key = g_yaml_loader_load_node (self, &(*parser), &(*event), &_inner_error_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_YAML_ERROR) {
				g_propagate_error (error, _inner_error_);
				(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
				return NULL;
			} else {
				(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return NULL;
			}
		}
#line 211 "loader.vala"
		g_yaml_loader_parse_with_throw (self, &(*parser), &_tmp9_, &_inner_error_);
#line 211 "loader.vala"
		(*event) = (_tmp10_ = _tmp9_, yaml_event_delete (&(*event)), _tmp10_);
#line 1147 "loader.c"
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_YAML_ERROR) {
				g_propagate_error (error, _inner_error_);
				(key == NULL) ? NULL : (key = (g_yaml_node_unref (key), NULL));
				(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
				return NULL;
			} else {
				(key == NULL) ? NULL : (key = (g_yaml_node_unref (key), NULL));
				(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return NULL;
			}
		}
		value = g_yaml_loader_load_node (self, &(*parser), &(*event), &_inner_error_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_YAML_ERROR) {
				g_propagate_error (error, _inner_error_);
				(key == NULL) ? NULL : (key = (g_yaml_node_unref (key), NULL));
				(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
				return NULL;
			} else {
				(key == NULL) ? NULL : (key = (g_yaml_node_unref (key), NULL));
				(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return NULL;
			}
		}
#line 213 "loader.vala"
		_tmp12_ = NULL;
#line 213 "loader.vala"
		_tmp11_ = NULL;
#line 213 "loader.vala"
		g_hash_table_insert (node->pairs, (_tmp11_ = key, (_tmp11_ == NULL) ? NULL : g_yaml_node_ref (_tmp11_)), (_tmp12_ = value, (_tmp12_ == NULL) ? NULL : g_yaml_node_ref (_tmp12_)));
#line 214 "loader.vala"
		_tmp13_ = NULL;
#line 214 "loader.vala"
		node->keys = g_list_prepend (node->keys, (_tmp13_ = key, (_tmp13_ == NULL) ? NULL : g_yaml_node_ref (_tmp13_)));
#line 215 "loader.vala"
		g_yaml_loader_parse_with_throw (self, &(*parser), &_tmp14_, &_inner_error_);
#line 215 "loader.vala"
		(*event) = (_tmp15_ = _tmp14_, yaml_event_delete (&(*event)), _tmp15_);
#line 1191 "loader.c"
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_YAML_ERROR) {
				g_propagate_error (error, _inner_error_);
				(key == NULL) ? NULL : (key = (g_yaml_node_unref (key), NULL));
				(value == NULL) ? NULL : (value = (g_yaml_node_unref (value), NULL));
				(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
				return NULL;
			} else {
				(key == NULL) ? NULL : (key = (g_yaml_node_unref (key), NULL));
				(value == NULL) ? NULL : (value = (g_yaml_node_unref (value), NULL));
				(node == NULL) ? NULL : (node = (g_yaml_node_unref (node), NULL));
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return NULL;
			}
		}
		(key == NULL) ? NULL : (key = (g_yaml_node_unref (key), NULL));
		(value == NULL) ? NULL : (value = (g_yaml_node_unref (value), NULL));
	}
	/* Preserve the document order */
#line 218 "loader.vala"
	node->keys = g_list_reverse (node->keys);
#line 1214 "loader.c"
	/* move the end mark of the mapping
	 * to the END_MAPPING_EVENT */
#line 222 "loader.vala"
	((GYAMLNode*) node)->end_mark = (*event).end_mark;
#line 1219 "loader.c"
	result = (GYAMLNode*) node;
	return result;
}


static void g_yaml_value_loader_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void g_yaml_value_loader_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		g_yaml_loader_unref (value->data[0].v_pointer);
	}
}


static void g_yaml_value_loader_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = g_yaml_loader_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer g_yaml_value_loader_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* g_yaml_value_loader_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		GYAMLLoader* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = g_yaml_loader_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* g_yaml_value_loader_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	GYAMLLoader** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = g_yaml_loader_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* g_yaml_param_spec_loader (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	GYAMLParamSpecLoader* spec;
	g_return_val_if_fail (g_type_is_a (object_type, G_YAML_TYPE_LOADER), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer g_yaml_value_get_loader (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, G_YAML_TYPE_LOADER), NULL);
	return value->data[0].v_pointer;
}


void g_yaml_value_set_loader (GValue* value, gpointer v_object) {
	GYAMLLoader* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, G_YAML_TYPE_LOADER));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, G_YAML_TYPE_LOADER));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		g_yaml_loader_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		g_yaml_loader_unref (old);
	}
}


static void g_yaml_loader_class_init (GYAMLLoaderClass * klass) {
	g_yaml_loader_parent_class = g_type_class_peek_parent (klass);
	G_YAML_LOADER_CLASS (klass)->finalize = g_yaml_loader_finalize;
	g_type_class_add_private (klass, sizeof (GYAMLLoaderPrivate));
}


static void g_yaml_loader_instance_init (GYAMLLoader * self) {
	self->priv = G_YAML_LOADER_GET_PRIVATE (self);
	self->ref_count = 1;
}


static void g_yaml_loader_finalize (GYAMLLoader* obj) {
	GYAMLLoader * self;
	self = G_YAML_LOADER (obj);
	(self->priv->document == NULL) ? NULL : (self->priv->document = (g_yaml_document_unref (self->priv->document), NULL));
}


GType g_yaml_loader_get_type (void) {
	static GType g_yaml_loader_type_id = 0;
	if (g_yaml_loader_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { g_yaml_value_loader_init, g_yaml_value_loader_free_value, g_yaml_value_loader_copy_value, g_yaml_value_loader_peek_pointer, "p", g_yaml_value_loader_collect_value, "p", g_yaml_value_loader_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (GYAMLLoaderClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) g_yaml_loader_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GYAMLLoader), 0, (GInstanceInitFunc) g_yaml_loader_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		g_yaml_loader_type_id = g_type_register_fundamental (g_type_fundamental_next (), "GYAMLLoader", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return g_yaml_loader_type_id;
}


gpointer g_yaml_loader_ref (gpointer instance) {
	GYAMLLoader* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void g_yaml_loader_unref (gpointer instance) {
	GYAMLLoader* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		G_YAML_LOADER_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




