/* ************
 *
 * Copyright (C) 2009  Yu Feng
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to 
 *
 * the Free Software Foundation, Inc., 
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Yu Feng <rainwoodman@gmail.com>
 ***/

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <yaml.h>
#include <float.h>
#include <math.h>
#include <gobject/gvaluecollector.h>


#define G_YAML_TYPE_WRITER (g_yaml_writer_get_type ())
#define G_YAML_WRITER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_TYPE_WRITER, GYAMLWriter))
#define G_YAML_WRITER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), G_YAML_TYPE_WRITER, GYAMLWriterClass))
#define G_YAML_IS_WRITER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_TYPE_WRITER))
#define G_YAML_IS_WRITER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_YAML_TYPE_WRITER))
#define G_YAML_WRITER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), G_YAML_TYPE_WRITER, GYAMLWriterClass))

typedef struct _GYAMLWriter GYAMLWriter;
typedef struct _GYAMLWriterClass GYAMLWriterClass;
typedef struct _GYAMLWriterPrivate GYAMLWriterPrivate;

#define G_YAML_WRITER_TYPE___ENUM (g_yaml_writer___enum_get_type ())

#define G_YAML_TYPE_BUILDABLE (g_yaml_buildable_get_type ())
#define G_YAML_BUILDABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_TYPE_BUILDABLE, GYAMLBuildable))
#define G_YAML_IS_BUILDABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_TYPE_BUILDABLE))
#define G_YAML_BUILDABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), G_YAML_TYPE_BUILDABLE, GYAMLBuildableIface))

typedef struct _GYAMLBuildable GYAMLBuildable;
typedef struct _GYAMLBuildableIface GYAMLBuildableIface;

#define G_YAML_TYPE_BUILDER (g_yaml_builder_get_type ())
#define G_YAML_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_TYPE_BUILDER, GYAMLBuilder))
#define G_YAML_BUILDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), G_YAML_TYPE_BUILDER, GYAMLBuilderClass))
#define G_YAML_IS_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_TYPE_BUILDER))
#define G_YAML_IS_BUILDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_YAML_TYPE_BUILDER))
#define G_YAML_BUILDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), G_YAML_TYPE_BUILDER, GYAMLBuilderClass))

typedef struct _GYAMLBuilder GYAMLBuilder;
typedef struct _GYAMLBuilderClass GYAMLBuilderClass;

#define G_YAML_TYPE_NODE (g_yaml_node_get_type ())
#define G_YAML_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_TYPE_NODE, GYAMLNode))
#define G_YAML_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), G_YAML_TYPE_NODE, GYAMLNodeClass))
#define G_YAML_IS_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_TYPE_NODE))
#define G_YAML_IS_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_YAML_TYPE_NODE))
#define G_YAML_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), G_YAML_TYPE_NODE, GYAMLNodeClass))

typedef struct _GYAMLNode GYAMLNode;
typedef struct _GYAMLNodeClass GYAMLNodeClass;
typedef struct _GYAMLParamSpecWriter GYAMLParamSpecWriter;

/**
 * Write an Object to a YAML string
 */
struct _GYAMLWriter {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GYAMLWriterPrivate * priv;
};

struct _GYAMLWriterClass {
	GTypeClass parent_class;
	void (*finalize) (GYAMLWriter *self);
};

struct _GYAMLWriterPrivate {
	char* prefix;
	GString* sb;
	yaml_emitter_t emitter;
};

typedef enum  {
	G_YAML_WRITER___ENUM_FOO
} GYAMLWriter__enum;

typedef enum  {
	G_YAML_ERROR_SYMBOL_NOT_FOUND,
	G_YAML_ERROR_TYPE_NOT_FOUND,
	G_YAML_ERROR_NOT_A_BUILDABLE,
	G_YAML_ERROR_PROPERTY_NOT_FOUND,
	G_YAML_ERROR_OBJECT_NOT_FOUND,
	G_YAML_ERROR_UNKNOWN_PROPERTY_TYPE,
	G_YAML_ERROR_CUSTOM_NODE_ERROR,
	G_YAML_ERROR_UNEXPECTED_NODE,
	G_YAML_ERROR_NOT_IMPLEMENTED,
	G_YAML_ERROR_FILE_NOT_FOUND,
	G_YAML_ERROR_PARSER_ERROR,
	G_YAML_ERROR_UNRESOLVED_ALIAS
} GYAMLError;
#define G_YAML_ERROR g_yaml_error_quark ()
/**
 * Buildable GObjects, from YAML.
 *
 *
 * Objects that implements GLib.YAML.Buildable is buildable by
 * GLib.YAML.Builder.
 *
 * This interface is almost the same as GtkBuildable.
 *
 **/
struct _GYAMLBuildableIface {
	GTypeInterface parent_iface;
	const char* (*get_name) (GYAMLBuildable* self);
	void (*set_name) (GYAMLBuildable* self, const char* name);
	void (*add_child) (GYAMLBuildable* self, GYAMLBuilder* builder, GObject* child, const char* type, GError** error);
	char** (*get_child_tags) (GYAMLBuildable* self, int* result_length1);
	GType* (*get_child_types) (GYAMLBuildable* self, int* result_length1);
	GList* (*get_children) (GYAMLBuildable* self, const char* type);
	GObject* (*get_internal_child) (GYAMLBuildable* self, GYAMLBuilder* builder, const char* child_name);
	void (*custom_node) (GYAMLBuildable* self, GYAMLBuilder* builder, const char* tag, GYAMLNode* node, GError** error);
};

struct _GYAMLParamSpecWriter {
	GParamSpec parent_instance;
};


static GType g_yaml_writer_enum_type;
static GType g_yaml_writer_enum_type = 0UL;
static gpointer g_yaml_writer_parent_class = NULL;

gpointer g_yaml_writer_ref (gpointer instance);
void g_yaml_writer_unref (gpointer instance);
GParamSpec* g_yaml_param_spec_writer (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void g_yaml_value_set_writer (GValue* value, gpointer v_object);
gpointer g_yaml_value_get_writer (const GValue* value);
GType g_yaml_writer_get_type (void);
#define G_YAML_WRITER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), G_YAML_TYPE_WRITER, GYAMLWriterPrivate))
enum  {
	G_YAML_WRITER_DUMMY_PROPERTY
};
static GType g_yaml_writer___enum_get_type (void);
GYAMLWriter* g_yaml_writer_new (const char* prefix);
GYAMLWriter* g_yaml_writer_construct (GType object_type, const char* prefix);
gint g_yaml_writer_handler (GYAMLWriter* self, gchar* buffer, int buffer_length1);
static gint _g_yaml_writer_handler_yaml_write_handler_t (gpointer self, gchar* buffer, int buffer_length1);
GQuark g_yaml_error_quark (void);
static void g_yaml_writer_write_object (GYAMLWriter* self, GObject* object, gboolean write_type_tag, GError** error);
void g_yaml_writer_stream_object (GYAMLWriter* self, GObject* object, GString* sb, GError** error);
static GParamSpec** _vala_array_dup1 (GParamSpec** self, int length);
static void g_yaml_writer_write_property (GYAMLWriter* self, GObject* object, GParamSpec* pspec, GError** error);
GType g_yaml_builder_get_type (void);
gpointer g_yaml_node_ref (gpointer instance);
void g_yaml_node_unref (gpointer instance);
GParamSpec* g_yaml_param_spec_node (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void g_yaml_value_set_node (GValue* value, gpointer v_object);
gpointer g_yaml_value_get_node (const GValue* value);
GType g_yaml_node_get_type (void);
GType g_yaml_buildable_get_type (void);
char** g_yaml_buildable_get_child_tags (GYAMLBuildable* self, int* result_length1);
GType* g_yaml_buildable_get_child_types (GYAMLBuildable* self, int* result_length1);
static void g_yaml_writer_write_children (GYAMLWriter* self, GYAMLBuildable* buildable, const char* tag, GType type, GError** error);
GList* g_yaml_buildable_get_children (GYAMLBuildable* self, const char* type);
static void g_yaml_writer_finalize (GYAMLWriter* obj);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);




static GType g_yaml_writer___enum_get_type (void) {
	static GType g_yaml_writer___enum_type_id = 0;
	if (G_UNLIKELY (g_yaml_writer___enum_type_id == 0)) {
		static const GEnumValue values[] = {{G_YAML_WRITER___ENUM_FOO, "G_YAML_WRITER___ENUM_FOO", "foo"}, {0, NULL, NULL}};
		g_yaml_writer___enum_type_id = g_enum_register_static ("GYAMLWriter__enum", values);
	}
	return g_yaml_writer___enum_type_id;
}


#line 37 "writer.vala"
GYAMLWriter* g_yaml_writer_construct (GType object_type, const char* prefix) {
#line 199 "writer.c"
	GYAMLWriter* self;
	char* _tmp1_;
	const char* _tmp0_;
	self = (GYAMLWriter*) g_type_create_instance (object_type);
	_tmp1_ = NULL;
#line 38 "writer.vala"
	_tmp0_ = NULL;
#line 38 "writer.vala"
	self->priv->prefix = (_tmp1_ = (_tmp0_ = prefix, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->prefix = (g_free (self->priv->prefix), NULL), _tmp1_);
#line 209 "writer.c"
	return self;
}


#line 37 "writer.vala"
GYAMLWriter* g_yaml_writer_new (const char* prefix) {
#line 37 "writer.vala"
	return g_yaml_writer_construct (G_YAML_TYPE_WRITER, prefix);
#line 218 "writer.c"
}


#line 226 "writer.vala"
static gint _g_yaml_writer_handler_yaml_write_handler_t (gpointer self, gchar* buffer, int buffer_length1) {
#line 224 "writer.c"
	return g_yaml_writer_handler (self, buffer, buffer_length1);
}


#line 226 "libyaml-1.0.vapi"
static void yaml_event_clean (yaml_event_t* event) {
#line 227 "libyaml-1.0.vapi"
	(*event).type = YAML_NO_EVENT;
#line 233 "writer.c"
}


#line 53 "writer.vala"
void g_yaml_writer_stream_object (GYAMLWriter* self, GObject* object, GString* sb, GError** error) {
#line 239 "writer.c"
	GError * _inner_error_;
	yaml_event_t event = {0};
	yaml_emitter_t _tmp1_ = {0};
	yaml_emitter_t _tmp0_ = {0};
#line 53 "writer.vala"
	g_return_if_fail (self != NULL);
#line 53 "writer.vala"
	g_return_if_fail (object != NULL);
#line 53 "writer.vala"
	g_return_if_fail (sb != NULL);
#line 250 "writer.c"
	_inner_error_ = NULL;
#line 55 "writer.vala"
	self->priv->sb = sb;
#line 56 "writer.vala"
	g_string_truncate (sb, (gsize) 0);
#line 57 "writer.vala"
	self->priv->emitter = (_tmp1_ = (yaml_emitter_initialize (&_tmp0_), _tmp0_), yaml_emitter_delete (&self->priv->emitter), _tmp1_);
#line 58 "writer.vala"
	yaml_emitter_set_output (&self->priv->emitter, _g_yaml_writer_handler_yaml_write_handler_t, self);
#line 60 "writer.vala"
	yaml_stream_start_event_initialize (&event, YAML_ANY_ENCODING);
#line 61 "writer.vala"
	yaml_emitter_emit (&self->priv->emitter, &event);
#line 62 "writer.vala"
	yaml_document_start_event_initialize (&event, NULL, NULL, NULL, FALSE);
#line 63 "writer.vala"
	yaml_emitter_emit (&self->priv->emitter, &event);
#line 64 "writer.vala"
	yaml_event_clean (&event);
#line 270 "writer.c"
	{
#line 67 "writer.vala"
		g_yaml_writer_write_object (self, object, TRUE, &_inner_error_);
#line 274 "writer.c"
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_YAML_ERROR) {
				goto __catch15_g_yaml_error;
			}
			goto __finally15;
		}
	}
	goto __finally15;
	__catch15_g_yaml_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			GError* _tmp2_;
#line 69 "writer.vala"
			_tmp2_ = NULL;
#line 292 "writer.c"
			_inner_error_ = (_tmp2_ = e, (_tmp2_ == NULL) ? ((gpointer) _tmp2_) : g_error_copy (_tmp2_));
			if (_inner_error_ != NULL) {
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
				yaml_event_delete (&event);
				goto __finally15;
			}
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally15:
	{
#line 71 "writer.vala"
		yaml_document_end_event_initialize (&event, TRUE);
#line 72 "writer.vala"
		yaml_emitter_emit (&self->priv->emitter, &event);
#line 73 "writer.vala"
		yaml_stream_end_event_initialize (&event);
#line 74 "writer.vala"
		yaml_emitter_emit (&self->priv->emitter, &event);
#line 75 "writer.vala"
		yaml_event_clean (&event);
#line 76 "writer.vala"
		yaml_emitter_flush (&self->priv->emitter);
#line 77 "writer.vala"
		self->priv->sb = NULL;
#line 318 "writer.c"
	}
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_YAML_ERROR) {
			g_propagate_error (error, _inner_error_);
			yaml_event_delete (&event);
			return;
		} else {
			yaml_event_delete (&event);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
	yaml_event_delete (&event);
	return;
}


static glong string_get_length (const char* self) {
	glong result;
	g_return_val_if_fail (self != NULL, 0L);
	result = g_utf8_strlen (self, -1);
	return result;
}


static GParamSpec** _vala_array_dup1 (GParamSpec** self, int length) {
	GParamSpec** result;
	int i;
	GParamSpec* _tmp4_;
	gint _tmp3_;
	result = g_new0 (GParamSpec*, length);
	for (i = 0; i < length; i++) {
		result[i] = (_tmp4_ = self[i], (_tmp4_ == NULL) ? NULL : g_param_spec_ref (_tmp4_));
	}
	return result;
}


#line 81 "writer.vala"
static void g_yaml_writer_write_object (GYAMLWriter* self, GObject* object, gboolean write_type_tag, GError** error) {
#line 360 "writer.c"
	GError * _inner_error_;
	yaml_event_t event = {0};
	GObjectClass* klass;
	GParamSpec** _tmp6_;
	gint specs_size;
	gint specs_length1;
	GParamSpec** _tmp5_;
	GParamSpec* _tmp4_;
	gint _tmp3_;
	GParamSpec** specs;
#line 81 "writer.vala"
	g_return_if_fail (self != NULL);
#line 81 "writer.vala"
	g_return_if_fail (object != NULL);
#line 375 "writer.c"
	_inner_error_ = NULL;
#line 83 "writer.vala"
	if (write_type_tag) {
#line 379 "writer.c"
		const char* _tmp0_;
		char* type_name;
#line 84 "writer.vala"
		_tmp0_ = NULL;
#line 384 "writer.c"
		type_name = (_tmp0_ = g_type_name (G_TYPE_FROM_INSTANCE (object)), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
#line 85 "writer.vala"
		if (self->priv->prefix != NULL) {
#line 86 "writer.vala"
			if (g_str_has_prefix (type_name, self->priv->prefix)) {
#line 390 "writer.c"
				char* _tmp1_;
				_tmp1_ = NULL;
#line 87 "writer.vala"
				yaml_mapping_start_event_initialize (&event, NULL, _tmp1_ = g_strconcat ("!", g_utf8_offset_to_pointer (type_name, string_get_length (self->priv->prefix)), NULL), FALSE, YAML_ANY_MAPPING_STYLE);
#line 395 "writer.c"
				_tmp1_ = (g_free (_tmp1_), NULL);
			} else {
#line 89 "writer.vala"
				g_error ("writer.vala:89: Internal program error, trying to serialize a object that is not in current namespace(%s)", self->priv->prefix);
#line 400 "writer.c"
			}
		} else {
			char* _tmp2_;
			_tmp2_ = NULL;
#line 91 "writer.vala"
			yaml_mapping_start_event_initialize (&event, NULL, _tmp2_ = g_strconcat ("!", type_name, NULL), FALSE, YAML_ANY_MAPPING_STYLE);
#line 407 "writer.c"
			_tmp2_ = (g_free (_tmp2_), NULL);
		}
		type_name = (g_free (type_name), NULL);
	} else {
#line 94 "writer.vala"
		yaml_mapping_start_event_initialize (&event, NULL, NULL, TRUE, YAML_ANY_MAPPING_STYLE);
#line 414 "writer.c"
	}
#line 96 "writer.vala"
	yaml_emitter_emit (&self->priv->emitter, &event);
#line 418 "writer.c"
	klass = (GObjectClass*) g_type_class_ref (G_TYPE_FROM_INSTANCE (object));
#line 99 "writer.vala"
	_tmp6_ = NULL;
#line 99 "writer.vala"
	_tmp5_ = NULL;
#line 99 "writer.vala"
	_tmp4_ = NULL;
#line 426 "writer.c"
	specs = (_tmp6_ = (_tmp5_ = g_object_class_list_properties (klass, &_tmp3_), (_tmp5_ == NULL) ? ((gpointer) _tmp5_) : _vala_array_dup1 (_tmp5_, _tmp3_)), specs_length1 = _tmp3_, specs_size = specs_length1, _tmp6_);
	{
		GParamSpec** spec_collection;
		int spec_collection_length1;
		int spec_it;
#line 101 "writer.vala"
		spec_collection = specs;
#line 434 "writer.c"
		spec_collection_length1 = specs_length1;
		for (spec_it = 0; spec_it < specs_length1; spec_it = spec_it + 1) {
			GParamSpec* spec;
			spec = spec_collection[spec_it];
			{
#line 102 "writer.vala"
				g_yaml_writer_write_property (self, object, spec, &_inner_error_);
#line 442 "writer.c"
				if (_inner_error_ != NULL) {
					if (_inner_error_->domain == G_YAML_ERROR) {
						g_propagate_error (error, _inner_error_);
						yaml_event_delete (&event);
						(klass == NULL) ? NULL : (klass = (g_type_class_unref (klass), NULL));
						specs = (_vala_array_free (specs, specs_length1, (GDestroyNotify) g_param_spec_unref), NULL);
						return;
					} else {
						yaml_event_delete (&event);
						(klass == NULL) ? NULL : (klass = (g_type_class_unref (klass), NULL));
						specs = (_vala_array_free (specs, specs_length1, (GDestroyNotify) g_param_spec_unref), NULL);
						g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
						g_clear_error (&_inner_error_);
						return;
					}
				}
			}
		}
	}
#line 104 "writer.vala"
	if (G_YAML_IS_BUILDABLE (object)) {
#line 464 "writer.c"
		GYAMLBuildable* _tmp8_;
		GObject* _tmp7_;
		GYAMLBuildable* buildable;
		char** _tmp10_;
		gint tags_size;
		gint tags_length1;
		gint _tmp9_;
		char** tags;
		GType* _tmp12_;
		gint types_size;
		gint types_length1;
		gint _tmp11_;
		GType* types;
#line 105 "writer.vala"
		_tmp8_ = NULL;
#line 105 "writer.vala"
		_tmp7_ = NULL;
#line 482 "writer.c"
		buildable = (_tmp8_ = (_tmp7_ = object, G_YAML_IS_BUILDABLE (_tmp7_) ? ((GYAMLBuildable*) _tmp7_) : NULL), (_tmp8_ == NULL) ? NULL : g_object_ref (_tmp8_));
#line 106 "writer.vala"
		_tmp10_ = NULL;
#line 486 "writer.c"
		tags = (_tmp10_ = g_yaml_buildable_get_child_tags (buildable, &_tmp9_), tags_length1 = _tmp9_, tags_size = tags_length1, _tmp10_);
#line 107 "writer.vala"
		_tmp12_ = NULL;
#line 490 "writer.c"
		types = (_tmp12_ = g_yaml_buildable_get_child_types (buildable, &_tmp11_), types_length1 = _tmp11_, types_size = types_length1, _tmp12_);
#line 108 "writer.vala"
		if (tags != NULL) {
#line 494 "writer.c"
			{
				gint i;
				i = 0;
				{
					gboolean _tmp13_;
					_tmp13_ = TRUE;
#line 109 "writer.vala"
					while (TRUE) {
#line 109 "writer.vala"
						if (!_tmp13_) {
#line 109 "writer.vala"
							i++;
#line 507 "writer.c"
						}
#line 109 "writer.vala"
						_tmp13_ = FALSE;
#line 109 "writer.vala"
						if (!(i < tags_length1)) {
#line 109 "writer.vala"
							break;
#line 515 "writer.c"
						}
#line 110 "writer.vala"
						g_yaml_writer_write_children (self, buildable, tags[i], types[i], &_inner_error_);
#line 519 "writer.c"
						if (_inner_error_ != NULL) {
							if (_inner_error_->domain == G_YAML_ERROR) {
								g_propagate_error (error, _inner_error_);
								(buildable == NULL) ? NULL : (buildable = (g_object_unref (buildable), NULL));
								yaml_event_delete (&event);
								(klass == NULL) ? NULL : (klass = (g_type_class_unref (klass), NULL));
								specs = (_vala_array_free (specs, specs_length1, (GDestroyNotify) g_param_spec_unref), NULL);
								return;
							} else {
								(buildable == NULL) ? NULL : (buildable = (g_object_unref (buildable), NULL));
								yaml_event_delete (&event);
								(klass == NULL) ? NULL : (klass = (g_type_class_unref (klass), NULL));
								specs = (_vala_array_free (specs, specs_length1, (GDestroyNotify) g_param_spec_unref), NULL);
								g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
								g_clear_error (&_inner_error_);
								return;
							}
						}
					}
				}
			}
		}
		(buildable == NULL) ? NULL : (buildable = (g_object_unref (buildable), NULL));
	}
#line 114 "writer.vala"
	yaml_mapping_end_event_initialize (&event);
#line 115 "writer.vala"
	yaml_emitter_emit (&self->priv->emitter, &event);
#line 116 "writer.vala"
	yaml_event_clean (&event);
#line 550 "writer.c"
	yaml_event_delete (&event);
	(klass == NULL) ? NULL : (klass = (g_type_class_unref (klass), NULL));
	specs = (_vala_array_free (specs, specs_length1, (GDestroyNotify) g_param_spec_unref), NULL);
}


#line 119 "writer.vala"
static void g_yaml_writer_write_children (GYAMLWriter* self, GYAMLBuildable* buildable, const char* tag, GType type, GError** error) {
#line 559 "writer.c"
	GError * _inner_error_;
	yaml_event_t event = {0};
	GList* children;
#line 119 "writer.vala"
	g_return_if_fail (self != NULL);
#line 119 "writer.vala"
	g_return_if_fail (buildable != NULL);
#line 119 "writer.vala"
	g_return_if_fail (tag != NULL);
#line 569 "writer.c"
	_inner_error_ = NULL;
#line 121 "writer.vala"
	yaml_scalar_event_initialize (&event, NULL, NULL, tag, (gint) strlen (tag), TRUE, TRUE, YAML_ANY_SCALAR_STYLE);
#line 122 "writer.vala"
	yaml_emitter_emit (&self->priv->emitter, &event);
#line 575 "writer.c"
	children = g_yaml_buildable_get_children (buildable, tag);
#line 126 "writer.vala"
	yaml_sequence_start_event_initialize (&event, NULL, NULL, TRUE, YAML_ANY_SEQUENCE_STYLE);
#line 127 "writer.vala"
	yaml_emitter_emit (&self->priv->emitter, &event);
#line 581 "writer.c"
	{
		GList* child_collection;
		GList* child_it;
#line 128 "writer.vala"
		child_collection = children;
#line 587 "writer.c"
		for (child_it = child_collection; child_it != NULL; child_it = child_it->next) {
			GObject* child;
#line 128 "writer.vala"
			child = (GObject*) child_it->data;
#line 592 "writer.c"
			{
#line 129 "writer.vala"
				if (G_TYPE_FROM_INSTANCE (child) != type) {
#line 130 "writer.vala"
					g_yaml_writer_write_object (self, child, TRUE, &_inner_error_);
#line 598 "writer.c"
					if (_inner_error_ != NULL) {
						if (_inner_error_->domain == G_YAML_ERROR) {
							g_propagate_error (error, _inner_error_);
							yaml_event_delete (&event);
							(children == NULL) ? NULL : (children = (g_list_free (children), NULL));
							return;
						} else {
							yaml_event_delete (&event);
							(children == NULL) ? NULL : (children = (g_list_free (children), NULL));
							g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
							g_clear_error (&_inner_error_);
							return;
						}
					}
				} else {
#line 132 "writer.vala"
					g_yaml_writer_write_object (self, child, FALSE, &_inner_error_);
#line 616 "writer.c"
					if (_inner_error_ != NULL) {
						if (_inner_error_->domain == G_YAML_ERROR) {
							g_propagate_error (error, _inner_error_);
							yaml_event_delete (&event);
							(children == NULL) ? NULL : (children = (g_list_free (children), NULL));
							return;
						} else {
							yaml_event_delete (&event);
							(children == NULL) ? NULL : (children = (g_list_free (children), NULL));
							g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
							g_clear_error (&_inner_error_);
							return;
						}
					}
				}
			}
		}
	}
#line 134 "writer.vala"
	yaml_sequence_end_event_initialize (&event);
#line 135 "writer.vala"
	yaml_emitter_emit (&self->priv->emitter, &event);
#line 136 "writer.vala"
	yaml_event_clean (&event);
#line 641 "writer.c"
	yaml_event_delete (&event);
	(children == NULL) ? NULL : (children = (g_list_free (children), NULL));
}


#line 670 "glib-2.0.vapi"
static char* double_to_string (double self) {
#line 649 "writer.c"
	char* result;
	const char* _tmp1_;
	gint _tmp0__length1;
	gchar* _tmp0_;
	char* _tmp2_;
#line 671 "glib-2.0.vapi"
	_tmp1_ = NULL;
#line 671 "glib-2.0.vapi"
	_tmp0_ = NULL;
#line 671 "glib-2.0.vapi"
	_tmp2_ = NULL;
#line 661 "writer.c"
	result = (_tmp2_ = (_tmp1_ = g_ascii_dtostr ((_tmp0_ = g_new0 (gchar, G_ASCII_DTOSTR_BUF_SIZE), _tmp0__length1 = G_ASCII_DTOSTR_BUF_SIZE, _tmp0_), G_ASCII_DTOSTR_BUF_SIZE, self), (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_)), _tmp0_ = (g_free (_tmp0_), NULL), _tmp2_);
	return result;
}


#line 35 "glib-2.0.vapi"
static char* bool_to_string (gboolean self) {
#line 669 "writer.c"
	char* result;
#line 36 "glib-2.0.vapi"
	if (self) {
#line 673 "writer.c"
		result = g_strdup ("true");
		return result;
	} else {
		result = g_strdup ("false");
		return result;
	}
}


#line 139 "writer.vala"
static void g_yaml_writer_write_property (GYAMLWriter* self, GObject* object, GParamSpec* pspec, GError** error) {
#line 685 "writer.c"
	GError * _inner_error_;
	yaml_event_t event = {0};
	GValue _tmp0_ = {0};
	GValue value;
	char* str;
#line 139 "writer.vala"
	g_return_if_fail (self != NULL);
#line 139 "writer.vala"
	g_return_if_fail (object != NULL);
#line 139 "writer.vala"
	g_return_if_fail (pspec != NULL);
#line 697 "writer.c"
	_inner_error_ = NULL;
#line 141 "writer.vala"
	yaml_scalar_event_initialize (&event, NULL, NULL, pspec->name, (gint) strlen (pspec->name), TRUE, TRUE, YAML_ANY_SCALAR_STYLE);
#line 142 "writer.vala"
	yaml_emitter_emit (&self->priv->emitter, &event);
#line 703 "writer.c"
	value = (g_value_init (&_tmp0_, pspec->value_type), _tmp0_);
	str = g_strdup ("(*unsupported)");
#line 145 "writer.vala"
	g_object_get_property (object, pspec->name, &value);
#line 147 "writer.vala"
	if (pspec->value_type == G_TYPE_INT) {
#line 710 "writer.c"
		char* _tmp1_;
		_tmp1_ = NULL;
#line 148 "writer.vala"
		str = (_tmp1_ = g_strdup_printf ("%i", g_value_get_int (&value)), str = (g_free (str), NULL), _tmp1_);
#line 715 "writer.c"
	} else {
#line 150 "writer.vala"
		if (pspec->value_type == G_TYPE_UINT) {
#line 719 "writer.c"
			char* _tmp2_;
			_tmp2_ = NULL;
#line 151 "writer.vala"
			str = (_tmp2_ = g_strdup_printf ("%u", g_value_get_uint (&value)), str = (g_free (str), NULL), _tmp2_);
#line 724 "writer.c"
		} else {
#line 153 "writer.vala"
			if (pspec->value_type == G_TYPE_LONG) {
#line 728 "writer.c"
				char* _tmp3_;
				_tmp3_ = NULL;
#line 154 "writer.vala"
				str = (_tmp3_ = g_strdup_printf ("%li", g_value_get_long (&value)), str = (g_free (str), NULL), _tmp3_);
#line 733 "writer.c"
			} else {
#line 156 "writer.vala"
				if (pspec->value_type == G_TYPE_ULONG) {
#line 737 "writer.c"
					char* _tmp4_;
					_tmp4_ = NULL;
#line 157 "writer.vala"
					str = (_tmp4_ = g_strdup_printf ("%lu", g_value_get_ulong (&value)), str = (g_free (str), NULL), _tmp4_);
#line 742 "writer.c"
				} else {
#line 159 "writer.vala"
					if (pspec->value_type == G_TYPE_STRING) {
#line 746 "writer.c"
						char* _tmp5_;
						_tmp5_ = NULL;
#line 160 "writer.vala"
						str = (_tmp5_ = g_value_dup_string (&value), str = (g_free (str), NULL), _tmp5_);
#line 161 "writer.vala"
						if (str == NULL) {
#line 753 "writer.c"
							char* _tmp6_;
							_tmp6_ = NULL;
#line 161 "writer.vala"
							str = (_tmp6_ = g_strdup ("~"), str = (g_free (str), NULL), _tmp6_);
#line 758 "writer.c"
						}
					} else {
#line 163 "writer.vala"
						if (pspec->value_type == G_TYPE_FLOAT) {
#line 763 "writer.c"
							char* _tmp7_;
							_tmp7_ = NULL;
#line 164 "writer.vala"
							str = (_tmp7_ = g_strdup_printf ("%g", g_value_get_float (&value)), str = (g_free (str), NULL), _tmp7_);
#line 768 "writer.c"
						} else {
#line 166 "writer.vala"
							if (pspec->value_type == G_TYPE_DOUBLE) {
#line 772 "writer.c"
								char* _tmp8_;
								_tmp8_ = NULL;
#line 167 "writer.vala"
								str = (_tmp8_ = double_to_string (g_value_get_double (&value)), str = (g_free (str), NULL), _tmp8_);
#line 777 "writer.c"
							} else {
#line 169 "writer.vala"
								if (pspec->value_type == G_TYPE_BOOLEAN) {
#line 781 "writer.c"
									char* _tmp9_;
									_tmp9_ = NULL;
#line 170 "writer.vala"
									str = (_tmp9_ = bool_to_string (g_value_get_boolean (&value)), str = (g_free (str), NULL), _tmp9_);
#line 786 "writer.c"
								} else {
#line 172 "writer.vala"
									if (pspec->value_type == G_TYPE_GTYPE) {
#line 790 "writer.c"
										char* _tmp11_;
										const char* _tmp10_;
										_tmp11_ = NULL;
#line 173 "writer.vala"
										_tmp10_ = NULL;
#line 173 "writer.vala"
										str = (_tmp11_ = (_tmp10_ = g_type_name (g_value_get_gtype (&value)), (_tmp10_ == NULL) ? NULL : g_strdup (_tmp10_)), str = (g_free (str), NULL), _tmp11_);
#line 798 "writer.c"
									} else {
#line 175 "writer.vala"
										if (g_type_is_a (pspec->value_type, G_TYPE_OBJECT)) {
#line 802 "writer.c"
											GObject* child;
											child = g_value_get_object (&value);
#line 177 "writer.vala"
											if (child != NULL) {
#line 807 "writer.c"
												char* _tmp12_;
#line 178 "writer.vala"
												if (G_TYPE_FROM_INSTANCE (child) != pspec->value_type) {
#line 811 "writer.c"
													/* if the object type differs from the property
													 * schema, assume we have a child class instance
													 * therefore the type tag of this object is important
													 * */
#line 183 "writer.vala"
													g_yaml_writer_write_object (self, child, TRUE, &_inner_error_);
#line 818 "writer.c"
													if (_inner_error_ != NULL) {
														if (_inner_error_->domain == G_YAML_ERROR) {
															g_propagate_error (error, _inner_error_);
															yaml_event_delete (&event);
															G_IS_VALUE (&value) ? (g_value_unset (&value), NULL) : NULL;
															str = (g_free (str), NULL);
															return;
														} else {
															yaml_event_delete (&event);
															G_IS_VALUE (&value) ? (g_value_unset (&value), NULL) : NULL;
															str = (g_free (str), NULL);
															g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
															g_clear_error (&_inner_error_);
															return;
														}
													}
												} else {
#line 185 "writer.vala"
													g_yaml_writer_write_object (self, child, FALSE, &_inner_error_);
#line 838 "writer.c"
													if (_inner_error_ != NULL) {
														if (_inner_error_->domain == G_YAML_ERROR) {
															g_propagate_error (error, _inner_error_);
															yaml_event_delete (&event);
															G_IS_VALUE (&value) ? (g_value_unset (&value), NULL) : NULL;
															str = (g_free (str), NULL);
															return;
														} else {
															yaml_event_delete (&event);
															G_IS_VALUE (&value) ? (g_value_unset (&value), NULL) : NULL;
															str = (g_free (str), NULL);
															g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
															g_clear_error (&_inner_error_);
															return;
														}
													}
												}
												_tmp12_ = NULL;
#line 187 "writer.vala"
												str = (_tmp12_ = NULL, str = (g_free (str), NULL), _tmp12_);
#line 859 "writer.c"
											} else {
												char* _tmp13_;
												_tmp13_ = NULL;
#line 189 "writer.vala"
												str = (_tmp13_ = g_strdup ("~"), str = (g_free (str), NULL), _tmp13_);
#line 865 "writer.c"
											}
										} else {
#line 192 "writer.vala"
											if (g_type_is_a (pspec->value_type, G_TYPE_BOXED)) {
#line 870 "writer.c"
												_inner_error_ = g_error_new (G_YAML_ERROR, G_YAML_ERROR_UNKNOWN_PROPERTY_TYPE, "Unhandled property type %s", g_type_name (pspec->value_type));
												if (_inner_error_ != NULL) {
													if (_inner_error_->domain == G_YAML_ERROR) {
														g_propagate_error (error, _inner_error_);
														yaml_event_delete (&event);
														G_IS_VALUE (&value) ? (g_value_unset (&value), NULL) : NULL;
														str = (g_free (str), NULL);
														return;
													} else {
														yaml_event_delete (&event);
														G_IS_VALUE (&value) ? (g_value_unset (&value), NULL) : NULL;
														str = (g_free (str), NULL);
														g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
														g_clear_error (&_inner_error_);
														return;
													}
												}
											} else {
#line 198 "writer.vala"
												if (g_type_is_a (pspec->value_type, g_yaml_writer_enum_type)) {
#line 891 "writer.c"
													GEnumClass* eclass;
													gint e;
													GEnumValue* evalue;
													eclass = (GEnumClass*) g_type_class_ref (pspec->value_type);
													e = g_value_get_enum (&value);
													evalue = g_enum_get_value (eclass, e);
#line 202 "writer.vala"
													if (evalue != NULL) {
#line 900 "writer.c"
														char* _tmp15_;
														const char* _tmp14_;
														_tmp15_ = NULL;
#line 203 "writer.vala"
														_tmp14_ = NULL;
#line 203 "writer.vala"
														str = (_tmp15_ = (_tmp14_ = evalue->value_name, (_tmp14_ == NULL) ? NULL : g_strdup (_tmp14_)), str = (g_free (str), NULL), _tmp15_);
#line 908 "writer.c"
													} else {
														char* _tmp16_;
														_tmp16_ = NULL;
#line 205 "writer.vala"
														str = (_tmp16_ = g_strdup_printf ("%i", e), str = (g_free (str), NULL), _tmp16_);
#line 914 "writer.c"
													}
													(eclass == NULL) ? NULL : (eclass = (g_type_class_unref (eclass), NULL));
												} else {
													_inner_error_ = g_error_new (G_YAML_ERROR, G_YAML_ERROR_UNKNOWN_PROPERTY_TYPE, "Unhandled property type %s", g_type_name (pspec->value_type));
													if (_inner_error_ != NULL) {
														if (_inner_error_->domain == G_YAML_ERROR) {
															g_propagate_error (error, _inner_error_);
															yaml_event_delete (&event);
															G_IS_VALUE (&value) ? (g_value_unset (&value), NULL) : NULL;
															str = (g_free (str), NULL);
															return;
														} else {
															yaml_event_delete (&event);
															G_IS_VALUE (&value) ? (g_value_unset (&value), NULL) : NULL;
															str = (g_free (str), NULL);
															g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
															g_clear_error (&_inner_error_);
															return;
														}
													}
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
#line 213 "writer.vala"
	if (str != NULL) {
#line 949 "writer.c"
		/* FIXME: str != null is not a good indicator,
		 * use a boolean like need_scalar or something !*/
#line 216 "writer.vala"
		if (NULL != g_utf8_strchr (str, (gssize) (-1), (gunichar) '\n')) {
#line 217 "writer.vala"
			yaml_scalar_event_initialize (&event, NULL, NULL, str, (gint) strlen (str), TRUE, TRUE, YAML_LITERAL_SCALAR_STYLE);
#line 956 "writer.c"
		} else {
#line 220 "writer.vala"
			yaml_scalar_event_initialize (&event, NULL, NULL, str, (gint) strlen (str), TRUE, TRUE, YAML_ANY_SCALAR_STYLE);
#line 960 "writer.c"
		}
#line 222 "writer.vala"
		yaml_emitter_emit (&self->priv->emitter, &event);
#line 964 "writer.c"
	}
#line 224 "writer.vala"
	yaml_event_clean (&event);
#line 968 "writer.c"
	yaml_event_delete (&event);
	G_IS_VALUE (&value) ? (g_value_unset (&value), NULL) : NULL;
	str = (g_free (str), NULL);
}


#line 226 "writer.vala"
gint g_yaml_writer_handler (GYAMLWriter* self, gchar* buffer, int buffer_length1) {
#line 977 "writer.c"
	gint result;
#line 226 "writer.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 227 "writer.vala"
	g_string_append_len (self->priv->sb, (const char*) buffer, (glong) buffer_length1);
#line 983 "writer.c"
	result = 1;
	return result;
}


static void g_yaml_value_writer_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void g_yaml_value_writer_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		g_yaml_writer_unref (value->data[0].v_pointer);
	}
}


static void g_yaml_value_writer_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = g_yaml_writer_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer g_yaml_value_writer_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* g_yaml_value_writer_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		GYAMLWriter* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = g_yaml_writer_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* g_yaml_value_writer_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	GYAMLWriter** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = g_yaml_writer_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* g_yaml_param_spec_writer (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	GYAMLParamSpecWriter* spec;
	g_return_val_if_fail (g_type_is_a (object_type, G_YAML_TYPE_WRITER), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer g_yaml_value_get_writer (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, G_YAML_TYPE_WRITER), NULL);
	return value->data[0].v_pointer;
}


void g_yaml_value_set_writer (GValue* value, gpointer v_object) {
	GYAMLWriter* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, G_YAML_TYPE_WRITER));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, G_YAML_TYPE_WRITER));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		g_yaml_writer_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		g_yaml_writer_unref (old);
	}
}


static void g_yaml_writer_class_init (GYAMLWriterClass * klass) {
	g_yaml_writer_parent_class = g_type_class_peek_parent (klass);
	G_YAML_WRITER_CLASS (klass)->finalize = g_yaml_writer_finalize;
	g_type_class_add_private (klass, sizeof (GYAMLWriterPrivate));
	{
#line 47 "writer.vala"
		g_yaml_writer_enum_type = g_type_parent (G_YAML_WRITER_TYPE___ENUM);
#line 1089 "writer.c"
	}
}


static void g_yaml_writer_instance_init (GYAMLWriter * self) {
	self->priv = G_YAML_WRITER_GET_PRIVATE (self);
	self->priv->prefix = NULL;
	self->priv->sb = NULL;
	self->ref_count = 1;
}


static void g_yaml_writer_finalize (GYAMLWriter* obj) {
	GYAMLWriter * self;
	self = G_YAML_WRITER (obj);
	self->priv->prefix = (g_free (self->priv->prefix), NULL);
	yaml_emitter_delete (&self->priv->emitter);
}


GType g_yaml_writer_get_type (void) {
	static GType g_yaml_writer_type_id = 0;
	if (g_yaml_writer_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { g_yaml_value_writer_init, g_yaml_value_writer_free_value, g_yaml_value_writer_copy_value, g_yaml_value_writer_peek_pointer, "p", g_yaml_value_writer_collect_value, "p", g_yaml_value_writer_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (GYAMLWriterClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) g_yaml_writer_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GYAMLWriter), 0, (GInstanceInitFunc) g_yaml_writer_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		g_yaml_writer_type_id = g_type_register_fundamental (g_type_fundamental_next (), "GYAMLWriter", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return g_yaml_writer_type_id;
}


gpointer g_yaml_writer_ref (gpointer instance) {
	GYAMLWriter* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void g_yaml_writer_unref (gpointer instance) {
	GYAMLWriter* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		G_YAML_WRITER_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}




