/* panel.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <float.h>
#include <math.h>
#include <pango/pango.h>
#include <gobject/gvaluecollector.h>


#define VALIDE_TYPE_PANEL_ITEM (valide_panel_item_get_type ())
#define VALIDE_PANEL_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PANEL_ITEM, ValidePanelItem))
#define VALIDE_PANEL_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PANEL_ITEM, ValidePanelItemClass))
#define VALIDE_IS_PANEL_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PANEL_ITEM))
#define VALIDE_IS_PANEL_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PANEL_ITEM))
#define VALIDE_PANEL_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PANEL_ITEM, ValidePanelItemClass))

typedef struct _ValidePanelItem ValidePanelItem;
typedef struct _ValidePanelItemClass ValidePanelItemClass;
typedef struct _ValidePanelItemPrivate ValidePanelItemPrivate;
typedef struct _ValideParamSpecPanelItem ValideParamSpecPanelItem;

#define VALIDE_TYPE_PANEL (valide_panel_get_type ())
#define VALIDE_PANEL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PANEL, ValidePanel))
#define VALIDE_PANEL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PANEL, ValidePanelClass))
#define VALIDE_IS_PANEL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PANEL))
#define VALIDE_IS_PANEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PANEL))
#define VALIDE_PANEL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PANEL, ValidePanelClass))

typedef struct _ValidePanel ValidePanel;
typedef struct _ValidePanelClass ValidePanelClass;
typedef struct _ValidePanelPrivate ValidePanelPrivate;

/*
 * This code is re-writing in the vala language of GeditPanel
 *
 * Copyright (C) 2005 - Paolo Maggi 
 */
struct _ValidePanelItem {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValidePanelItemPrivate * priv;
	char* name;
	GtkImage* icon;
};

struct _ValidePanelItemClass {
	GTypeClass parent_class;
	void (*finalize) (ValidePanelItem *self);
};

struct _ValideParamSpecPanelItem {
	GParamSpec parent_instance;
};

/**
 * A side or bottom panel widget
 */
struct _ValidePanel {
	GtkVBox parent_instance;
	ValidePanelPrivate * priv;
};

struct _ValidePanelClass {
	GtkVBoxClass parent_class;
	void (*close) (ValidePanel* self);
};

struct _ValidePanelPrivate {
	GtkNotebook* notebook;
	GtkImage* title_image;
	GtkLabel* title_label;
	GList* items;
	GtkOrientation _orientation;
};


static gpointer valide_panel_item_parent_class = NULL;
static gpointer valide_panel_parent_class = NULL;

gpointer valide_panel_item_ref (gpointer instance);
void valide_panel_item_unref (gpointer instance);
GParamSpec* valide_param_spec_panel_item (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void valide_value_set_panel_item (GValue* value, gpointer v_object);
gpointer valide_value_get_panel_item (const GValue* value);
GType valide_panel_item_get_type (void);
enum  {
	VALIDE_PANEL_ITEM_DUMMY_PROPERTY
};
#define VALIDE_PANEL_ITEM_KEY "ValidePanelItemKey"
ValidePanelItem* valide_panel_item_new (void);
ValidePanelItem* valide_panel_item_construct (GType object_type);
static void valide_panel_item_finalize (ValidePanelItem* obj);
GType valide_panel_get_type (void);
#define VALIDE_PANEL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_PANEL, ValidePanelPrivate))
enum  {
	VALIDE_PANEL_DUMMY_PROPERTY,
	VALIDE_PANEL_ORIENTATION
};
static void _g_list_free_valide_panel_item_unref (GList* self);
static void valide_panel_notebook_page_changed (ValidePanel* self, GtkNotebook* sender, void* page, guint page_num);
static void valide_panel_panel_show (ValidePanel* self);
static void valide_panel_sync_title (ValidePanel* self, ValidePanelItem* item);
void valide_panel_close (ValidePanel* self);
static void valide_panel_close_button_clicked_cb (ValidePanel* self);
static void valide_panel_set_gtk_image_from_gtk_image (ValidePanel* self, GtkImage* image, GtkImage* source);
GtkOrientation valide_panel_get_orientation (ValidePanel* self);
static void _valide_panel_notebook_page_changed_gtk_notebook_switch_page (GtkNotebook* _sender, void* page, guint page_num, gpointer self);
static void valide_panel_build_notebook (ValidePanel* self);
GtkButton* valide_utils_create_small_button (GtkImage* image);
static void _valide_panel_close_button_clicked_cb_gtk_button_clicked (GtkButton* _sender, gpointer self);
static void valide_panel_build_horizontal_panel (ValidePanel* self);
static void valide_panel_build_vertical_panel (ValidePanel* self);
static GtkWidget* valide_panel_build_tab_label (ValidePanel* self, GtkWidget* item, const char* name, GtkImage* icon);
static void valide_panel_set_orientation (ValidePanel* self, GtkOrientation value);
ValidePanel* valide_panel_new (GtkOrientation orientation);
ValidePanel* valide_panel_construct (GType object_type, GtkOrientation orientation);
void valide_panel_add_item (ValidePanel* self, GtkWidget* item, const char* name, GtkImage* image);
void valide_panel_add_item_with_stock_icon (ValidePanel* self, GtkWidget* item, const char* name, const char* stock_id);
static void valide_panel_real_close (ValidePanel* self);
static void valide_panel_real_grab_focus (GtkWidget* base);
gboolean valide_panel_remove_item (ValidePanel* self, GtkWidget* item);
void valide_panel_set_current_page (ValidePanel* self, GtkWidget* widget);
static void _valide_panel_panel_show_gtk_widget_show (GtkWidget* _sender, gpointer self);
static GObject * valide_panel_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void valide_panel_finalize (GObject* obj);
static void valide_panel_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_panel_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);



/*
 * This code is re-writing in the vala language of GeditPanel
 *
 * Copyright (C) 2005 - Paolo Maggi 
 */
#line 28 "panel.vala"
ValidePanelItem* valide_panel_item_construct (GType object_type) {
#line 164 "panel.c"
	ValidePanelItem* self;
	self = (ValidePanelItem*) g_type_create_instance (object_type);
	return self;
}


#line 28 "panel.vala"
ValidePanelItem* valide_panel_item_new (void) {
#line 28 "panel.vala"
	return valide_panel_item_construct (VALIDE_TYPE_PANEL_ITEM);
#line 175 "panel.c"
}


static void valide_value_panel_item_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void valide_value_panel_item_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		valide_panel_item_unref (value->data[0].v_pointer);
	}
}


static void valide_value_panel_item_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = valide_panel_item_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer valide_value_panel_item_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* valide_value_panel_item_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		ValidePanelItem* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = valide_panel_item_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* valide_value_panel_item_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	ValidePanelItem** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = valide_panel_item_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* valide_param_spec_panel_item (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ValideParamSpecPanelItem* spec;
	g_return_val_if_fail (g_type_is_a (object_type, VALIDE_TYPE_PANEL_ITEM), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer valide_value_get_panel_item (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALIDE_TYPE_PANEL_ITEM), NULL);
	return value->data[0].v_pointer;
}


void valide_value_set_panel_item (GValue* value, gpointer v_object) {
	ValidePanelItem* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALIDE_TYPE_PANEL_ITEM));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, VALIDE_TYPE_PANEL_ITEM));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		valide_panel_item_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		valide_panel_item_unref (old);
	}
}


static void valide_panel_item_class_init (ValidePanelItemClass * klass) {
	valide_panel_item_parent_class = g_type_class_peek_parent (klass);
	VALIDE_PANEL_ITEM_CLASS (klass)->finalize = valide_panel_item_finalize;
}


static void valide_panel_item_instance_init (ValidePanelItem * self) {
	self->ref_count = 1;
}


static void valide_panel_item_finalize (ValidePanelItem* obj) {
	ValidePanelItem * self;
	self = VALIDE_PANEL_ITEM (obj);
	self->name = (g_free (self->name), NULL);
	(self->icon == NULL) ? NULL : (self->icon = (g_object_unref (self->icon), NULL));
}


GType valide_panel_item_get_type (void) {
	static GType valide_panel_item_type_id = 0;
	if (valide_panel_item_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { valide_value_panel_item_init, valide_value_panel_item_free_value, valide_value_panel_item_copy_value, valide_value_panel_item_peek_pointer, "p", valide_value_panel_item_collect_value, "p", valide_value_panel_item_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (ValidePanelItemClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_panel_item_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValidePanelItem), 0, (GInstanceInitFunc) valide_panel_item_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		valide_panel_item_type_id = g_type_register_fundamental (g_type_fundamental_next (), "ValidePanelItem", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return valide_panel_item_type_id;
}


gpointer valide_panel_item_ref (gpointer instance) {
	ValidePanelItem* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void valide_panel_item_unref (gpointer instance) {
	ValidePanelItem* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		VALIDE_PANEL_ITEM_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


static void _g_list_free_valide_panel_item_unref (GList* self) {
	g_list_foreach (self, (GFunc) valide_panel_item_unref, NULL);
	g_list_free (self);
}


#line 69 "panel.vala"
static void valide_panel_panel_show (ValidePanel* self) {
#line 329 "panel.c"
	gint page;
#line 69 "panel.vala"
	g_return_if_fail (self != NULL);
#line 333 "panel.c"
	page = gtk_notebook_get_current_page (self->priv->notebook);
#line 73 "panel.vala"
	if (page != (-1)) {
#line 75 "panel.vala"
		valide_panel_notebook_page_changed (self, self->priv->notebook, NULL, (guint) page);
#line 339 "panel.c"
	}
}


#line 79 "panel.vala"
static void valide_panel_notebook_page_changed (ValidePanel* self, GtkNotebook* sender, void* page, guint page_num) {
#line 346 "panel.c"
	GtkWidget* _tmp0_;
	GtkWidget* item;
	ValidePanelItem* data;
#line 79 "panel.vala"
	g_return_if_fail (self != NULL);
#line 79 "panel.vala"
	g_return_if_fail (sender != NULL);
#line 83 "panel.vala"
	_tmp0_ = NULL;
#line 356 "panel.c"
	item = (_tmp0_ = gtk_notebook_get_nth_page (sender, (gint) page_num), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
#line 84 "panel.vala"
	g_return_if_fail (item != NULL);
#line 360 "panel.c"
	data = VALIDE_PANEL_ITEM (g_object_get_data ((GObject*) item, VALIDE_PANEL_ITEM_KEY));
#line 87 "panel.vala"
	g_return_if_fail (data != NULL);
#line 89 "panel.vala"
	valide_panel_sync_title (self, data);
#line 366 "panel.c"
	(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
}


#line 92 "panel.vala"
static void valide_panel_close_button_clicked_cb (ValidePanel* self) {
#line 92 "panel.vala"
	g_return_if_fail (self != NULL);
#line 94 "panel.vala"
	valide_panel_close (self);
#line 377 "panel.c"
}


/* This is ugly, since it supports only known
   * storage types of GtkImage, otherwise fall back
   * to the empty icon.
   * See http://bugzilla.gnome.org/show_bug.cgi?id=317520.
   */
#line 102 "panel.vala"
static void valide_panel_set_gtk_image_from_gtk_image (ValidePanel* self, GtkImage* image, GtkImage* source) {
#line 102 "panel.vala"
	g_return_if_fail (self != NULL);
#line 102 "panel.vala"
	g_return_if_fail (image != NULL);
#line 102 "panel.vala"
	g_return_if_fail (source != NULL);
#line 104 "panel.vala"
	switch (gtk_image_get_storage_type (source)) {
#line 396 "panel.c"
		case GTK_IMAGE_EMPTY:
		{
#line 107 "panel.vala"
			gtk_image_clear (image);
#line 108 "panel.vala"
			break;
#line 403 "panel.c"
		}
		case GTK_IMAGE_PIXMAP:
		{
			{
				GdkPixmap* pm;
				GdkBitmap* bm;
				pm = NULL;
				bm = NULL;
#line 114 "panel.vala"
				gtk_image_get_pixmap (source, &pm, &bm);
#line 115 "panel.vala"
				gtk_image_set_from_pixmap (image, pm, bm);
#line 416 "panel.c"
			}
#line 117 "panel.vala"
			break;
#line 420 "panel.c"
		}
		case GTK_IMAGE_IMAGE:
		{
			{
				GdkImage* i;
				GdkBitmap* bm;
				i = NULL;
				bm = NULL;
#line 123 "panel.vala"
				gtk_image_get_image (source, &i, &bm);
#line 124 "panel.vala"
				gtk_image_set_from_image (image, i, bm);
#line 433 "panel.c"
			}
#line 126 "panel.vala"
			break;
#line 437 "panel.c"
		}
		case GTK_IMAGE_PIXBUF:
		{
			{
				GdkPixbuf* pb;
				GdkPixbuf* _tmp1_;
				GdkPixbuf* _tmp0_;
				pb = NULL;
				_tmp1_ = NULL;
#line 131 "panel.vala"
				_tmp0_ = NULL;
#line 131 "panel.vala"
				pb = (_tmp1_ = (_tmp0_ = gtk_image_get_pixbuf (source), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (pb == NULL) ? NULL : (pb = (g_object_unref (pb), NULL)), _tmp1_);
#line 132 "panel.vala"
				gtk_image_set_from_pixbuf (image, pb);
#line 453 "panel.c"
				(pb == NULL) ? NULL : (pb = (g_object_unref (pb), NULL));
			}
#line 134 "panel.vala"
			break;
#line 458 "panel.c"
		}
		case GTK_IMAGE_STOCK:
		{
			{
				char* s_id;
				GtkIconSize s;
				char* _tmp3_;
				char* _tmp2_;
				s_id = NULL;
				s = 0;
				_tmp3_ = NULL;
				_tmp2_ = NULL;
#line 140 "panel.vala"
				gtk_image_get_stock (source, &_tmp2_, &s);
#line 140 "panel.vala"
				s_id = (_tmp3_ = _tmp2_, s_id = (g_free (s_id), NULL), _tmp3_);
#line 141 "panel.vala"
				gtk_image_set_from_stock (image, s_id, s);
#line 477 "panel.c"
				s_id = (g_free (s_id), NULL);
			}
#line 143 "panel.vala"
			break;
#line 482 "panel.c"
		}
		case GTK_IMAGE_ICON_SET:
		{
			{
				GtkIconSet* i;
				GtkIconSize s;
				GtkIconSet* _tmp6_;
				GtkIconSet* _tmp5_;
				GtkIconSet* _tmp4_;
				i = NULL;
				s = 0;
#line 149 "panel.vala"
				_tmp6_ = NULL;
#line 496 "panel.c"
				_tmp5_ = NULL;
				_tmp4_ = NULL;
#line 149 "panel.vala"
				gtk_image_get_icon_set (source, &_tmp4_, &s);
#line 149 "panel.vala"
				i = (_tmp5_ = (_tmp6_ = _tmp4_, (_tmp6_ == NULL) ? NULL : gtk_icon_set_ref (_tmp6_)), (i == NULL) ? NULL : (i = (gtk_icon_set_unref (i), NULL)), _tmp5_);
#line 150 "panel.vala"
				gtk_image_set_from_icon_set (image, i, s);
#line 505 "panel.c"
				(i == NULL) ? NULL : (i = (gtk_icon_set_unref (i), NULL));
			}
#line 152 "panel.vala"
			break;
#line 510 "panel.c"
		}
		case GTK_IMAGE_ANIMATION:
		{
			{
				GdkPixbufAnimation* a;
				GdkPixbufAnimation* _tmp8_;
				GdkPixbufAnimation* _tmp7_;
				a = NULL;
				_tmp8_ = NULL;
#line 157 "panel.vala"
				_tmp7_ = NULL;
#line 157 "panel.vala"
				a = (_tmp8_ = (_tmp7_ = gtk_image_get_animation (source), (_tmp7_ == NULL) ? NULL : g_object_ref (_tmp7_)), (a == NULL) ? NULL : (a = (g_object_unref (a), NULL)), _tmp8_);
#line 158 "panel.vala"
				gtk_image_set_from_animation (image, a);
#line 526 "panel.c"
				(a == NULL) ? NULL : (a = (g_object_unref (a), NULL));
			}
#line 160 "panel.vala"
			break;
#line 531 "panel.c"
		}
		case GTK_IMAGE_ICON_NAME:
		{
			{
				char* n;
				GtkIconSize s;
				const char* _tmp11_;
				char* _tmp10_;
				const char* _tmp9_;
				n = NULL;
				s = 0;
#line 166 "panel.vala"
				_tmp11_ = NULL;
#line 545 "panel.c"
				_tmp10_ = NULL;
				_tmp9_ = NULL;
#line 166 "panel.vala"
				gtk_image_get_icon_name (source, &_tmp9_, &s);
#line 166 "panel.vala"
				n = (_tmp10_ = (_tmp11_ = _tmp9_, (_tmp11_ == NULL) ? NULL : g_strdup (_tmp11_)), n = (g_free (n), NULL), _tmp10_);
#line 167 "panel.vala"
				gtk_image_set_from_icon_name (image, n, s);
#line 554 "panel.c"
				n = (g_free (n), NULL);
			}
#line 169 "panel.vala"
			break;
#line 559 "panel.c"
		}
		default:
		{
#line 171 "panel.vala"
			gtk_image_set_from_stock (image, GTK_STOCK_FILE, GTK_ICON_SIZE_MENU);
#line 172 "panel.vala"
			break;
#line 567 "panel.c"
		}
	}
}


#line 176 "panel.vala"
static void valide_panel_sync_title (ValidePanel* self, ValidePanelItem* item) {
#line 575 "panel.c"
	gboolean _tmp0_;
#line 176 "panel.vala"
	g_return_if_fail (self != NULL);
#line 579 "panel.c"
	_tmp0_ = FALSE;
#line 178 "panel.vala"
	if (self->priv->_orientation == GTK_ORIENTATION_VERTICAL) {
#line 179 "panel.vala"
		_tmp0_ = self->priv->title_label != NULL;
#line 585 "panel.c"
	} else {
#line 178 "panel.vala"
		_tmp0_ = FALSE;
#line 589 "panel.c"
	}
#line 178 "panel.vala"
	if (_tmp0_) {
#line 181 "panel.vala"
		if (item != NULL) {
#line 183 "panel.vala"
			gtk_label_set_text (self->priv->title_label, item->name);
#line 184 "panel.vala"
			valide_panel_set_gtk_image_from_gtk_image (self, self->priv->title_image, item->icon);
#line 599 "panel.c"
		} else {
#line 188 "panel.vala"
			gtk_label_set_text (self->priv->title_label, "Empty");
#line 189 "panel.vala"
			gtk_image_set_from_stock (self->priv->title_image, GTK_STOCK_FILE, GTK_ICON_SIZE_MENU);
#line 605 "panel.c"
		}
	}
}


#line 79 "panel.vala"
static void _valide_panel_notebook_page_changed_gtk_notebook_switch_page (GtkNotebook* _sender, void* page, guint page_num, gpointer self) {
#line 613 "panel.c"
	valide_panel_notebook_page_changed (self, _sender, page, page_num);
}


#line 194 "panel.vala"
static void valide_panel_build_notebook (ValidePanel* self) {
#line 620 "panel.c"
	GtkNotebook* _tmp0_;
#line 194 "panel.vala"
	g_return_if_fail (self != NULL);
#line 624 "panel.c"
	/* Create the panel notebook */
	_tmp0_ = NULL;
#line 197 "panel.vala"
	self->priv->notebook = (_tmp0_ = g_object_ref_sink ((GtkNotebook*) gtk_notebook_new ()), (self->priv->notebook == NULL) ? NULL : (self->priv->notebook = (g_object_unref (self->priv->notebook), NULL)), _tmp0_);
#line 199 "panel.vala"
	gtk_notebook_set_tab_pos (self->priv->notebook, GTK_POS_BOTTOM);
#line 200 "panel.vala"
	gtk_notebook_set_scrollable (self->priv->notebook, TRUE);
#line 201 "panel.vala"
	gtk_notebook_popup_enable (self->priv->notebook);
#line 202 "panel.vala"
	gtk_widget_show ((GtkWidget*) self->priv->notebook);
#line 203 "panel.vala"
	g_signal_connect_object (self->priv->notebook, "switch-page", (GCallback) _valide_panel_notebook_page_changed_gtk_notebook_switch_page, self, 0);
#line 639 "panel.c"
}


#line 92 "panel.vala"
static void _valide_panel_close_button_clicked_cb_gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 645 "panel.c"
	valide_panel_close_button_clicked_cb (self);
}


#line 206 "panel.vala"
static void valide_panel_build_horizontal_panel (ValidePanel* self) {
#line 652 "panel.c"
	GtkLabel* _tmp0_;
	GtkImage* _tmp1_;
	GtkHBox* box;
	GtkVBox* sidebar;
	GtkImage* image;
	GtkButton* close_button;
#line 206 "panel.vala"
	g_return_if_fail (self != NULL);
#line 661 "panel.c"
	/* Unused widget */
	_tmp0_ = NULL;
#line 209 "panel.vala"
	self->priv->title_label = (_tmp0_ = g_object_ref_sink ((GtkLabel*) gtk_label_new ("Empty")), (self->priv->title_label == NULL) ? NULL : (self->priv->title_label = (g_object_unref (self->priv->title_label), NULL)), _tmp0_);
#line 666 "panel.c"
	_tmp1_ = NULL;
#line 210 "panel.vala"
	self->priv->title_image = (_tmp1_ = g_object_ref_sink ((GtkImage*) gtk_image_new ()), (self->priv->title_image == NULL) ? NULL : (self->priv->title_image = (g_object_unref (self->priv->title_image), NULL)), _tmp1_);
#line 670 "panel.c"
	box = g_object_ref_sink ((GtkHBox*) gtk_hbox_new (FALSE, 0));
#line 213 "panel.vala"
	gtk_box_pack_start ((GtkBox*) box, (GtkWidget*) self->priv->notebook, TRUE, TRUE, (guint) 0);
#line 674 "panel.c"
	/* Toolbar, close button and first separator */
	sidebar = g_object_ref_sink ((GtkVBox*) gtk_vbox_new (FALSE, 6));
#line 217 "panel.vala"
	gtk_container_set_border_width ((GtkContainer*) sidebar, (guint) 4);
#line 219 "panel.vala"
	gtk_box_pack_start ((GtkBox*) box, (GtkWidget*) sidebar, FALSE, FALSE, (guint) 0);
#line 681 "panel.c"
	image = g_object_ref_sink ((GtkImage*) gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU));
	close_button = valide_utils_create_small_button (image);
#line 223 "panel.vala"
	gtk_box_pack_start ((GtkBox*) sidebar, (GtkWidget*) close_button, FALSE, FALSE, (guint) 0);
#line 224 "panel.vala"
	gtk_widget_set_tooltip_text ((GtkWidget*) close_button, "Hide panel");
#line 225 "panel.vala"
	g_signal_connect_object (close_button, "clicked", (GCallback) _valide_panel_close_button_clicked_cb_gtk_button_clicked, self, 0);
#line 227 "panel.vala"
	gtk_widget_show_all ((GtkWidget*) box);
#line 229 "panel.vala"
	gtk_box_pack_start ((GtkBox*) self, (GtkWidget*) box, TRUE, TRUE, (guint) 0);
#line 694 "panel.c"
	(box == NULL) ? NULL : (box = (g_object_unref (box), NULL));
	(sidebar == NULL) ? NULL : (sidebar = (g_object_unref (sidebar), NULL));
	(image == NULL) ? NULL : (image = (g_object_unref (image), NULL));
	(close_button == NULL) ? NULL : (close_button = (g_object_unref (close_button), NULL));
}


#line 232 "panel.vala"
static void valide_panel_build_vertical_panel (ValidePanel* self) {
#line 704 "panel.c"
	GtkHBox* title_hbox;
	GtkHBox* icon_name_hbox;
	GtkImage* _tmp0_;
	GtkLabel* dummy_label;
	GtkLabel* _tmp1_;
	GtkImage* image;
	GtkButton* close_button;
#line 232 "panel.vala"
	g_return_if_fail (self != NULL);
#line 714 "panel.c"
	/* Create title hbox */
	title_hbox = g_object_ref_sink ((GtkHBox*) gtk_hbox_new (FALSE, 6));
#line 236 "panel.vala"
	gtk_container_set_border_width ((GtkContainer*) title_hbox, (guint) 5);
#line 237 "panel.vala"
	gtk_box_pack_start ((GtkBox*) self, (GtkWidget*) title_hbox, FALSE, FALSE, (guint) 0);
#line 721 "panel.c"
	icon_name_hbox = g_object_ref_sink ((GtkHBox*) gtk_hbox_new (FALSE, 0));
#line 240 "panel.vala"
	gtk_box_pack_start ((GtkBox*) title_hbox, (GtkWidget*) icon_name_hbox, TRUE, TRUE, (guint) 0);
#line 725 "panel.c"
	_tmp0_ = NULL;
#line 242 "panel.vala"
	self->priv->title_image = (_tmp0_ = g_object_ref_sink ((GtkImage*) gtk_image_new_from_stock (GTK_STOCK_FILE, GTK_ICON_SIZE_MENU)), (self->priv->title_image == NULL) ? NULL : (self->priv->title_image = (g_object_unref (self->priv->title_image), NULL)), _tmp0_);
#line 243 "panel.vala"
	gtk_box_pack_start ((GtkBox*) icon_name_hbox, (GtkWidget*) self->priv->title_image, FALSE, TRUE, (guint) 0);
#line 731 "panel.c"
	dummy_label = g_object_ref_sink ((GtkLabel*) gtk_label_new (" "));
#line 246 "panel.vala"
	gtk_box_pack_start ((GtkBox*) icon_name_hbox, (GtkWidget*) dummy_label, FALSE, FALSE, (guint) 0);
#line 735 "panel.c"
	_tmp1_ = NULL;
#line 248 "panel.vala"
	self->priv->title_label = (_tmp1_ = g_object_ref_sink ((GtkLabel*) gtk_label_new ("Empty")), (self->priv->title_label == NULL) ? NULL : (self->priv->title_label = (g_object_unref (self->priv->title_label), NULL)), _tmp1_);
#line 249 "panel.vala"
	gtk_misc_set_alignment ((GtkMisc*) self->priv->title_label, (float) 0, (float) 0.5);
#line 250 "panel.vala"
	gtk_label_set_ellipsize (self->priv->title_label, PANGO_ELLIPSIZE_END);
#line 251 "panel.vala"
	gtk_box_pack_start ((GtkBox*) icon_name_hbox, (GtkWidget*) self->priv->title_label, TRUE, TRUE, (guint) 0);
#line 745 "panel.c"
	image = g_object_ref_sink ((GtkImage*) gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU));
	close_button = valide_utils_create_small_button (image);
#line 255 "panel.vala"
	gtk_widget_set_tooltip_text ((GtkWidget*) close_button, "Hide panel");
#line 256 "panel.vala"
	g_signal_connect_object (close_button, "clicked", (GCallback) _valide_panel_close_button_clicked_cb_gtk_button_clicked, self, 0);
#line 257 "panel.vala"
	gtk_box_pack_start ((GtkBox*) title_hbox, (GtkWidget*) close_button, FALSE, FALSE, (guint) 0);
#line 259 "panel.vala"
	gtk_widget_show_all ((GtkWidget*) title_hbox);
#line 261 "panel.vala"
	gtk_box_pack_start ((GtkBox*) self, (GtkWidget*) self->priv->notebook, TRUE, TRUE, (guint) 0);
#line 758 "panel.c"
	(title_hbox == NULL) ? NULL : (title_hbox = (g_object_unref (title_hbox), NULL));
	(icon_name_hbox == NULL) ? NULL : (icon_name_hbox = (g_object_unref (icon_name_hbox), NULL));
	(dummy_label == NULL) ? NULL : (dummy_label = (g_object_unref (dummy_label), NULL));
	(image == NULL) ? NULL : (image = (g_object_unref (image), NULL));
	(close_button == NULL) ? NULL : (close_button = (g_object_unref (close_button), NULL));
}


#line 264 "panel.vala"
static GtkWidget* valide_panel_build_tab_label (ValidePanel* self, GtkWidget* item, const char* name, GtkImage* icon) {
#line 769 "panel.c"
	GtkWidget* result;
	GtkHBox* hbox;
	GtkEventBox* label_ebox;
	GtkHBox* label_hbox;
	GtkLabel* label;
#line 264 "panel.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 264 "panel.vala"
	g_return_val_if_fail (item != NULL, NULL);
#line 264 "panel.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 264 "panel.vala"
	g_return_val_if_fail (icon != NULL, NULL);
#line 783 "panel.c"
	/* set hbox spacing and label padding (see below) so that there's an
	     * equal amount of space around the label */
	hbox = g_object_ref_sink ((GtkHBox*) gtk_hbox_new (FALSE, 4));
	label_ebox = g_object_ref_sink ((GtkEventBox*) gtk_event_box_new ());
#line 272 "panel.vala"
	gtk_event_box_set_visible_window (label_ebox, FALSE);
#line 273 "panel.vala"
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) label_ebox, TRUE, TRUE, (guint) 0);
#line 792 "panel.c"
	label_hbox = g_object_ref_sink ((GtkHBox*) gtk_hbox_new (FALSE, 4));
#line 276 "panel.vala"
	gtk_container_add ((GtkContainer*) label_ebox, (GtkWidget*) label_hbox);
#line 796 "panel.c"
	/* setup icon */
#line 279 "panel.vala"
	gtk_box_pack_start ((GtkBox*) label_hbox, (GtkWidget*) icon, FALSE, FALSE, (guint) 0);
#line 800 "panel.c"
	/* setup label */
	label = g_object_ref_sink ((GtkLabel*) gtk_label_new (name));
#line 283 "panel.vala"
	gtk_misc_set_alignment ((GtkMisc*) label, (float) 0.0, (float) 0.5);
#line 284 "panel.vala"
	gtk_misc_set_padding ((GtkMisc*) label, 0, 0);
#line 285 "panel.vala"
	gtk_box_pack_start ((GtkBox*) label_hbox, (GtkWidget*) label, TRUE, TRUE, (guint) 0);
#line 287 "panel.vala"
	gtk_widget_set_tooltip_text ((GtkWidget*) label_ebox, name);
#line 289 "panel.vala"
	gtk_widget_show_all ((GtkWidget*) hbox);
#line 291 "panel.vala"
	if (self->priv->_orientation == GTK_ORIENTATION_VERTICAL) {
#line 293 "panel.vala"
		gtk_widget_hide ((GtkWidget*) label);
#line 817 "panel.c"
	}
#line 295 "panel.vala"
	g_object_set_data ((GObject*) item, "label", label);
#line 296 "panel.vala"
	g_object_set_data ((GObject*) item, "hbox", hbox);
#line 297 "panel.vala"
	g_object_set_data ((GObject*) item, "label-ebox", label_ebox);
#line 825 "panel.c"
	result = (GtkWidget*) hbox;
	(label_ebox == NULL) ? NULL : (label_ebox = (g_object_unref (label_ebox), NULL));
	(label_hbox == NULL) ? NULL : (label_hbox = (g_object_unref (label_hbox), NULL));
	(label == NULL) ? NULL : (label = (g_object_unref (label), NULL));
	return result;
}


/**
   * Create a new panel
   *
   * @param orientation The orientation of the panel
   */
#line 306 "panel.vala"
ValidePanel* valide_panel_construct (GType object_type, GtkOrientation orientation) {
#line 841 "panel.c"
	GParameter * __params;
	GParameter * __params_it;
	ValidePanel * self;
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
#line 308 "panel.vala"
	__params_it->name = "orientation";
#line 308 "panel.vala"
	g_value_init (&__params_it->value, GTK_TYPE_ORIENTATION);
#line 308 "panel.vala"
	g_value_set_enum (&__params_it->value, orientation);
#line 308 "panel.vala"
	__params_it++;
#line 855 "panel.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 306 "panel.vala"
ValidePanel* valide_panel_new (GtkOrientation orientation) {
#line 306 "panel.vala"
	return valide_panel_construct (VALIDE_TYPE_PANEL, orientation);
#line 870 "panel.c"
}


/**
   * Add a item to the panel
   *
   * @param item The item to ad
   * @param name Thename of the item
   * @param image An optionnal image representing the item
   */
#line 332 "panel.vala"
void valide_panel_add_item (ValidePanel* self, GtkWidget* item, const char* name, GtkImage* image) {
#line 883 "panel.c"
	gint w;
	gint h;
	ValidePanelItem* data;
	GtkLabel* menu_label;
	GtkWidget* tab_label;
	ValidePanelItem* _tmp0_;
	char* _tmp2_;
	const char* _tmp1_;
	GtkWidget* _tmp6_;
	GtkLabel* _tmp7_;
	gboolean _tmp8_;
	ValidePanelItem* _tmp9_;
#line 332 "panel.vala"
	g_return_if_fail (self != NULL);
#line 332 "panel.vala"
	g_return_if_fail (item != NULL);
#line 332 "panel.vala"
	g_return_if_fail (name != NULL);
#line 902 "panel.c"
	w = 0;
	h = 0;
	data = NULL;
	menu_label = NULL;
	tab_label = NULL;
	_tmp0_ = NULL;
#line 339 "panel.vala"
	data = (_tmp0_ = valide_panel_item_new (), (data == NULL) ? NULL : (data = (valide_panel_item_unref (data), NULL)), _tmp0_);
#line 911 "panel.c"
	_tmp2_ = NULL;
#line 340 "panel.vala"
	_tmp1_ = NULL;
#line 340 "panel.vala"
	data->name = (_tmp2_ = (_tmp1_ = name, (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_)), data->name = (g_free (data->name), NULL), _tmp2_);
#line 342 "panel.vala"
	if (image == NULL) {
#line 919 "panel.c"
		GtkImage* _tmp3_;
		/* default to empty */
		_tmp3_ = NULL;
#line 345 "panel.vala"
		data->icon = (_tmp3_ = g_object_ref_sink ((GtkImage*) gtk_image_new_from_stock (GTK_STOCK_FILE, GTK_ICON_SIZE_MENU)), (data->icon == NULL) ? NULL : (data->icon = (g_object_unref (data->icon), NULL)), _tmp3_);
#line 925 "panel.c"
	} else {
		GtkImage* _tmp5_;
		GtkImage* _tmp4_;
		_tmp5_ = NULL;
#line 349 "panel.vala"
		_tmp4_ = NULL;
#line 349 "panel.vala"
		data->icon = (_tmp5_ = (_tmp4_ = image, (_tmp4_ == NULL) ? NULL : g_object_ref (_tmp4_)), (data->icon == NULL) ? NULL : (data->icon = (g_object_unref (data->icon), NULL)), _tmp5_);
#line 934 "panel.c"
	}
#line 352 "panel.vala"
	gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &w, &h);
#line 353 "panel.vala"
	gtk_widget_set_size_request ((GtkWidget*) data->icon, w, h);
#line 355 "panel.vala"
	g_object_set_data ((GObject*) item, VALIDE_PANEL_ITEM_KEY, data);
#line 942 "panel.c"
	_tmp6_ = NULL;
#line 357 "panel.vala"
	tab_label = (_tmp6_ = valide_panel_build_tab_label (self, item, data->name, data->icon), (tab_label == NULL) ? NULL : (tab_label = (g_object_unref (tab_label), NULL)), _tmp6_);
#line 946 "panel.c"
	_tmp7_ = NULL;
#line 359 "panel.vala"
	menu_label = (_tmp7_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (name)), (menu_label == NULL) ? NULL : (menu_label = (g_object_unref (menu_label), NULL)), _tmp7_);
#line 360 "panel.vala"
	gtk_misc_set_alignment ((GtkMisc*) menu_label, (float) 0.0, (float) 0.5);
#line 362 "panel.vala"
	if (!(g_object_get (item, "visible", &_tmp8_, NULL), _tmp8_)) {
#line 364 "panel.vala"
		gtk_widget_show (item);
#line 956 "panel.c"
	}
#line 367 "panel.vala"
	gtk_notebook_append_page_menu (self->priv->notebook, item, tab_label, (GtkWidget*) menu_label);
#line 368 "panel.vala"
	g_signal_emit_by_name (self, "item-added", item);
#line 369 "panel.vala"
	_tmp9_ = NULL;
#line 369 "panel.vala"
	self->priv->items = g_list_append (self->priv->items, (_tmp9_ = data, (_tmp9_ == NULL) ? NULL : valide_panel_item_ref (_tmp9_)));
#line 966 "panel.c"
	(data == NULL) ? NULL : (data = (valide_panel_item_unref (data), NULL));
	(menu_label == NULL) ? NULL : (menu_label = (g_object_unref (menu_label), NULL));
	(tab_label == NULL) ? NULL : (tab_label = (g_object_unref (tab_label), NULL));
}


/**
   * Add a item to the panel with stock icon
   *
   * @param item The item to ad
   * @param name Thename of the item
   * @param stock_id The stock id for image representing the item
   */
#line 379 "panel.vala"
void valide_panel_add_item_with_stock_icon (ValidePanel* self, GtkWidget* item, const char* name, const char* stock_id) {
#line 982 "panel.c"
	GtkImage* icon;
#line 379 "panel.vala"
	g_return_if_fail (self != NULL);
#line 379 "panel.vala"
	g_return_if_fail (item != NULL);
#line 379 "panel.vala"
	g_return_if_fail (name != NULL);
#line 990 "panel.c"
	icon = NULL;
#line 384 "panel.vala"
	if (stock_id != NULL) {
#line 994 "panel.c"
		GtkImage* _tmp0_;
		_tmp0_ = NULL;
#line 386 "panel.vala"
		icon = (_tmp0_ = g_object_ref_sink ((GtkImage*) gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_MENU)), (icon == NULL) ? NULL : (icon = (g_object_unref (icon), NULL)), _tmp0_);
#line 999 "panel.c"
	}
#line 388 "panel.vala"
	valide_panel_add_item (self, item, name, icon);
#line 1003 "panel.c"
	(icon == NULL) ? NULL : (icon = (g_object_unref (icon), NULL));
}


/**
   * Close the pannel
   */
#line 394 "panel.vala"
static void valide_panel_real_close (ValidePanel* self) {
#line 394 "panel.vala"
	g_return_if_fail (self != NULL);
#line 396 "panel.vala"
	gtk_widget_hide ((GtkWidget*) self);
#line 1017 "panel.c"
}


#line 394 "panel.vala"
void valide_panel_close (ValidePanel* self) {
#line 394 "panel.vala"
	VALIDE_PANEL_GET_CLASS (self)->close (self);
#line 1025 "panel.c"
}


/**
   * @see Gtk.widget.grab_focus
   */
#line 402 "panel.vala"
static void valide_panel_real_grab_focus (GtkWidget* base) {
#line 1034 "panel.c"
	ValidePanel * self;
	gint n;
	GtkWidget* tab;
	self = (ValidePanel*) base;
	n = 0;
	tab = NULL;
#line 407 "panel.vala"
	n = gtk_notebook_get_current_page (self->priv->notebook);
#line 408 "panel.vala"
	if (n != (-1)) {
#line 1045 "panel.c"
		GtkWidget* _tmp1_;
		GtkWidget* _tmp0_;
		_tmp1_ = NULL;
#line 410 "panel.vala"
		_tmp0_ = NULL;
#line 410 "panel.vala"
		tab = (_tmp1_ = (_tmp0_ = gtk_notebook_get_nth_page (self->priv->notebook, n), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (tab == NULL) ? NULL : (tab = (g_object_unref (tab), NULL)), _tmp1_);
#line 411 "panel.vala"
		g_return_if_fail (tab != NULL);
#line 412 "panel.vala"
		gtk_widget_grab_focus (tab);
#line 1057 "panel.c"
	}
	(tab == NULL) ? NULL : (tab = (g_object_unref (tab), NULL));
}


/**
   * Remove an item form the panel
   *
   * @param item The item to remve
   */
#line 421 "panel.vala"
gboolean valide_panel_remove_item (ValidePanel* self, GtkWidget* item) {
#line 1070 "panel.c"
	gboolean result;
	gboolean removed;
	ValidePanelItem* data;
	gint page_num;
	GtkWidget* ebox;
#line 421 "panel.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 421 "panel.vala"
	g_return_val_if_fail (item != NULL, FALSE);
#line 1080 "panel.c"
	removed = FALSE;
	data = NULL;
	page_num = 0;
	ebox = NULL;
#line 428 "panel.vala"
	page_num = gtk_notebook_page_num (self->priv->notebook, item);
#line 430 "panel.vala"
	if (page_num > 0) {
#line 1089 "panel.c"
		ValidePanelItem* _tmp1_;
		ValidePanelItem* _tmp0_;
		_tmp1_ = NULL;
#line 432 "panel.vala"
		_tmp0_ = NULL;
#line 432 "panel.vala"
		data = (_tmp1_ = (_tmp0_ = VALIDE_PANEL_ITEM (g_object_get_data ((GObject*) item, VALIDE_PANEL_ITEM_KEY)), (_tmp0_ == NULL) ? NULL : valide_panel_item_ref (_tmp0_)), (data == NULL) ? NULL : (data = (valide_panel_item_unref (data), NULL)), _tmp1_);
#line 433 "panel.vala"
		if (data != NULL) {
#line 1099 "panel.c"
			GtkWidget* _tmp3_;
			GtkWidget* _tmp2_;
#line 435 "panel.vala"
			g_object_set_data ((GObject*) item, VALIDE_PANEL_ITEM_KEY, NULL);
#line 1104 "panel.c"
			_tmp3_ = NULL;
#line 437 "panel.vala"
			_tmp2_ = NULL;
#line 437 "panel.vala"
			ebox = (_tmp3_ = (_tmp2_ = GTK_WIDGET (g_object_get_data ((GObject*) item, "label-ebox")), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (ebox == NULL) ? NULL : (ebox = (g_object_unref (ebox), NULL)), _tmp3_);
#line 439 "panel.vala"
			gtk_notebook_remove_page (self->priv->notebook, page_num);
#line 1112 "panel.c"
			/* if we removed all the pages, reset the title */
#line 442 "panel.vala"
			if (gtk_notebook_get_n_pages (self->priv->notebook) == 0) {
#line 444 "panel.vala"
				valide_panel_sync_title (self, NULL);
#line 1118 "panel.c"
			}
#line 447 "panel.vala"
			g_signal_emit_by_name (self, "item-removed", item);
#line 448 "panel.vala"
			removed = TRUE;
#line 1124 "panel.c"
		}
	}
	result = removed;
	(data == NULL) ? NULL : (data = (valide_panel_item_unref (data), NULL));
	(ebox == NULL) ? NULL : (ebox = (g_object_unref (ebox), NULL));
	return result;
}


/**
   * Switch to the page contenning the widget
   *
   * @param widget The widget
   */
#line 459 "panel.vala"
void valide_panel_set_current_page (ValidePanel* self, GtkWidget* widget) {
#line 1141 "panel.c"
	gint page_num;
#line 459 "panel.vala"
	g_return_if_fail (self != NULL);
#line 459 "panel.vala"
	g_return_if_fail (widget != NULL);
#line 1147 "panel.c"
	page_num = 0;
#line 463 "panel.vala"
	page_num = gtk_notebook_page_num (self->priv->notebook, widget);
#line 464 "panel.vala"
	gtk_notebook_set_current_page (self->priv->notebook, page_num);
#line 1153 "panel.c"
}


GtkOrientation valide_panel_get_orientation (ValidePanel* self) {
	GtkOrientation result;
	g_return_val_if_fail (self != NULL, 0);
	result = self->priv->_orientation;
	return result;
}


static void valide_panel_set_orientation (ValidePanel* self, GtkOrientation value) {
	g_return_if_fail (self != NULL);
	self->priv->_orientation = value;
	g_object_notify ((GObject *) self, "orientation");
}


#line 69 "panel.vala"
static void _valide_panel_panel_show_gtk_widget_show (GtkWidget* _sender, gpointer self) {
#line 1174 "panel.c"
	valide_panel_panel_show (self);
}


static GObject * valide_panel_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValidePanelClass * klass;
	GObjectClass * parent_class;
	ValidePanel * self;
	klass = VALIDE_PANEL_CLASS (g_type_class_peek (VALIDE_TYPE_PANEL));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_PANEL (obj);
	{
#line 313 "panel.vala"
		valide_panel_build_notebook (self);
#line 314 "panel.vala"
		if (self->priv->_orientation == GTK_ORIENTATION_HORIZONTAL) {
#line 316 "panel.vala"
			valide_panel_build_horizontal_panel (self);
#line 1195 "panel.c"
		} else {
#line 320 "panel.vala"
			valide_panel_build_vertical_panel (self);
#line 1199 "panel.c"
		}
#line 322 "panel.vala"
		g_signal_connect_object ((GtkWidget*) self, "show", (GCallback) _valide_panel_panel_show_gtk_widget_show, self, 0);
#line 1203 "panel.c"
	}
	return obj;
}


static void valide_panel_class_init (ValidePanelClass * klass) {
	valide_panel_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValidePanelPrivate));
	VALIDE_PANEL_CLASS (klass)->close = valide_panel_real_close;
	GTK_WIDGET_CLASS (klass)->grab_focus = valide_panel_real_grab_focus;
	G_OBJECT_CLASS (klass)->get_property = valide_panel_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_panel_set_property;
	G_OBJECT_CLASS (klass)->constructor = valide_panel_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_panel_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PANEL_ORIENTATION, g_param_spec_enum ("orientation", "orientation", "orientation", GTK_TYPE_ORIENTATION, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_signal_new ("item_added", VALIDE_TYPE_PANEL, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, GTK_TYPE_WIDGET);
	g_signal_new ("item_removed", VALIDE_TYPE_PANEL, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, GTK_TYPE_WIDGET);
	g_signal_new ("closed", VALIDE_TYPE_PANEL, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}


static void valide_panel_instance_init (ValidePanel * self) {
	self->priv = VALIDE_PANEL_GET_PRIVATE (self);
}


static void valide_panel_finalize (GObject* obj) {
	ValidePanel * self;
	self = VALIDE_PANEL (obj);
	(self->priv->notebook == NULL) ? NULL : (self->priv->notebook = (g_object_unref (self->priv->notebook), NULL));
	(self->priv->title_image == NULL) ? NULL : (self->priv->title_image = (g_object_unref (self->priv->title_image), NULL));
	(self->priv->title_label == NULL) ? NULL : (self->priv->title_label = (g_object_unref (self->priv->title_label), NULL));
	(self->priv->items == NULL) ? NULL : (self->priv->items = (_g_list_free_valide_panel_item_unref (self->priv->items), NULL));
	G_OBJECT_CLASS (valide_panel_parent_class)->finalize (obj);
}


GType valide_panel_get_type (void) {
	static GType valide_panel_type_id = 0;
	if (valide_panel_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValidePanelClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_panel_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValidePanel), 0, (GInstanceInitFunc) valide_panel_instance_init, NULL };
		valide_panel_type_id = g_type_register_static (GTK_TYPE_VBOX, "ValidePanel", &g_define_type_info, 0);
	}
	return valide_panel_type_id;
}


static void valide_panel_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValidePanel * self;
	gpointer boxed;
	self = VALIDE_PANEL (object);
	switch (property_id) {
		case VALIDE_PANEL_ORIENTATION:
		g_value_set_enum (value, valide_panel_get_orientation (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_panel_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValidePanel * self;
	self = VALIDE_PANEL (object);
	switch (property_id) {
		case VALIDE_PANEL_ORIENTATION:
		valide_panel_set_orientation (self, g_value_get_enum (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}




