/* plugin-manager.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <config.h>
#include <glib/gstdio.h>


#define VALIDE_TYPE_PLUGIN_MANAGER (valide_plugin_manager_get_type ())
#define VALIDE_PLUGIN_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PLUGIN_MANAGER, ValidePluginManager))
#define VALIDE_PLUGIN_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PLUGIN_MANAGER, ValidePluginManagerClass))
#define VALIDE_IS_PLUGIN_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PLUGIN_MANAGER))
#define VALIDE_IS_PLUGIN_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PLUGIN_MANAGER))
#define VALIDE_PLUGIN_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PLUGIN_MANAGER, ValidePluginManagerClass))

typedef struct _ValidePluginManager ValidePluginManager;
typedef struct _ValidePluginManagerClass ValidePluginManagerClass;
typedef struct _ValidePluginManagerPrivate ValidePluginManagerPrivate;

#define VALIDE_TYPE_PLUGIN_REGISTRAR (valide_plugin_registrar_get_type ())
#define VALIDE_PLUGIN_REGISTRAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PLUGIN_REGISTRAR, ValidePluginRegistrar))
#define VALIDE_PLUGIN_REGISTRAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PLUGIN_REGISTRAR, ValidePluginRegistrarClass))
#define VALIDE_IS_PLUGIN_REGISTRAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PLUGIN_REGISTRAR))
#define VALIDE_IS_PLUGIN_REGISTRAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PLUGIN_REGISTRAR))
#define VALIDE_PLUGIN_REGISTRAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PLUGIN_REGISTRAR, ValidePluginRegistrarClass))

typedef struct _ValidePluginRegistrar ValidePluginRegistrar;
typedef struct _ValidePluginRegistrarClass ValidePluginRegistrarClass;

#define VALIDE_TYPE_WINDOW (valide_window_get_type ())
#define VALIDE_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_WINDOW, ValideWindow))
#define VALIDE_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_WINDOW, ValideWindowClass))
#define VALIDE_IS_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_WINDOW))
#define VALIDE_IS_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_WINDOW))
#define VALIDE_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_WINDOW, ValideWindowClass))

typedef struct _ValideWindow ValideWindow;
typedef struct _ValideWindowClass ValideWindowClass;

#define VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER (valide_abstract_config_manager_get_type ())
#define VALIDE_ABSTRACT_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManager))
#define VALIDE_ABSTRACT_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManagerClass))
#define VALIDE_IS_ABSTRACT_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER))
#define VALIDE_IS_ABSTRACT_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER))
#define VALIDE_ABSTRACT_CONFIG_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManagerClass))

typedef struct _ValideAbstractConfigManager ValideAbstractConfigManager;
typedef struct _ValideAbstractConfigManagerClass ValideAbstractConfigManagerClass;

#define VALIDE_TYPE_CONFIG_MANAGER (valide_config_manager_get_type ())
#define VALIDE_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_CONFIG_MANAGER, ValideConfigManager))
#define VALIDE_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_CONFIG_MANAGER, ValideConfigManagerClass))
#define VALIDE_IS_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_CONFIG_MANAGER))
#define VALIDE_IS_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_CONFIG_MANAGER))
#define VALIDE_CONFIG_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_CONFIG_MANAGER, ValideConfigManagerClass))

typedef struct _ValideConfigManager ValideConfigManager;
typedef struct _ValideConfigManagerClass ValideConfigManagerClass;

#define VALIDE_TYPE_PLUGIN_PREFERENCES (valide_plugin_preferences_get_type ())
#define VALIDE_PLUGIN_PREFERENCES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PLUGIN_PREFERENCES, ValidePluginPreferences))
#define VALIDE_PLUGIN_PREFERENCES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PLUGIN_PREFERENCES, ValidePluginPreferencesClass))
#define VALIDE_IS_PLUGIN_PREFERENCES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PLUGIN_PREFERENCES))
#define VALIDE_IS_PLUGIN_PREFERENCES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PLUGIN_PREFERENCES))
#define VALIDE_PLUGIN_PREFERENCES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PLUGIN_PREFERENCES, ValidePluginPreferencesClass))

typedef struct _ValidePluginPreferences ValidePluginPreferences;
typedef struct _ValidePluginPreferencesClass ValidePluginPreferencesClass;

/**
 * The plugin manager
 */
struct _ValidePluginManager {
	GObject parent_instance;
	ValidePluginManagerPrivate * priv;
	GList* plugins;
};

struct _ValidePluginManagerClass {
	GObjectClass parent_class;
};

struct _ValidePluginManagerPrivate {
	ValideWindow* _window;
};


static gpointer valide_plugin_manager_parent_class = NULL;

GType valide_plugin_manager_get_type (void);
GType valide_plugin_registrar_get_type (void);
GType valide_window_get_type (void);
#define VALIDE_PLUGIN_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_PLUGIN_MANAGER, ValidePluginManagerPrivate))
enum  {
	VALIDE_PLUGIN_MANAGER_DUMMY_PROPERTY,
	VALIDE_PLUGIN_MANAGER_WINDOW
};
static void _g_list_free_g_object_unref (GList* self);
ValideWindow* valide_plugin_manager_get_window (ValidePluginManager* self);
ValidePluginRegistrar* valide_plugin_registrar_new (const char* filename, ValideWindow* window);
ValidePluginRegistrar* valide_plugin_registrar_construct (GType object_type, const char* filename, ValideWindow* window);
GType valide_abstract_config_manager_get_type (void);
GType valide_config_manager_get_type (void);
ValideConfigManager* valide_config_manager_get_instance (void);
gboolean valide_abstract_config_manager_get_boolean (ValideAbstractConfigManager* self, const char* group, const char* key);
const char* valide_plugin_registrar_get_name (ValidePluginRegistrar* self);
void valide_plugin_registrar_load (ValidePluginRegistrar* self, GError** error);
static void valide_plugin_manager_register_plugin (ValidePluginManager* self, const char* filename, GError** error);
static void valide_plugin_manager_set_window (ValidePluginManager* self, ValideWindow* value);
ValidePluginManager* valide_plugin_manager_new (ValideWindow* window);
ValidePluginManager* valide_plugin_manager_construct (GType object_type, ValideWindow* window);
ValidePluginPreferences* valide_plugin_preferences_new (ValidePluginManager* plugin_manager);
ValidePluginPreferences* valide_plugin_preferences_construct (GType object_type, ValidePluginManager* plugin_manager);
GType valide_plugin_preferences_get_type (void);
GtkWidget* valide_plugin_manager_preferences (ValidePluginManager* self);
char* valide_abstract_config_manager_get_home_dir (ValideAbstractConfigManager* self);
static GObject * valide_plugin_manager_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
void valide_plugin_registrar_unload (ValidePluginRegistrar* self, GError** error);
static void valide_plugin_manager_finalize (GObject* obj);
static void valide_plugin_manager_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_plugin_manager_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);

static const char* VALIDE_PLUGIN_REGISTRAR_FILE_EXT[] = {"valide-plugin"};


static void _g_list_free_g_object_unref (GList* self) {
	g_list_foreach (self, (GFunc) g_object_unref, NULL);
	g_list_free (self);
}


#line 41 "plugin-manager.vala"
static void valide_plugin_manager_register_plugin (ValidePluginManager* self, const char* filename, GError** error) {
#line 159 "plugin-manager.c"
	GError * _inner_error_;
	ValidePluginRegistrar* registrar;
	ValidePluginRegistrar* _tmp0_;
	ValideConfigManager* _tmp1_;
	gboolean _tmp2_;
	ValidePluginRegistrar* _tmp3_;
#line 41 "plugin-manager.vala"
	g_return_if_fail (self != NULL);
#line 41 "plugin-manager.vala"
	g_return_if_fail (filename != NULL);
#line 170 "plugin-manager.c"
	_inner_error_ = NULL;
	registrar = NULL;
	_tmp0_ = NULL;
#line 45 "plugin-manager.vala"
	registrar = (_tmp0_ = valide_plugin_registrar_new (filename, self->priv->_window), (registrar == NULL) ? NULL : (registrar = (g_object_unref (registrar), NULL)), _tmp0_);
#line 88 "config-manager.vala"
	_tmp1_ = NULL;
#line 46 "plugin-manager.vala"
	if ((_tmp2_ = valide_abstract_config_manager_get_boolean ((ValideAbstractConfigManager*) (_tmp1_ = valide_config_manager_get_instance ()), "Plugins", valide_plugin_registrar_get_name (registrar)), (_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL)), _tmp2_)) {
#line 48 "plugin-manager.vala"
		valide_plugin_registrar_load (registrar, &_inner_error_);
#line 182 "plugin-manager.c"
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(registrar == NULL) ? NULL : (registrar = (g_object_unref (registrar), NULL));
			return;
		}
	}
#line 50 "plugin-manager.vala"
	_tmp3_ = NULL;
#line 50 "plugin-manager.vala"
	self->plugins = g_list_append (self->plugins, (_tmp3_ = registrar, (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)));
#line 193 "plugin-manager.c"
	(registrar == NULL) ? NULL : (registrar = (g_object_unref (registrar), NULL));
}


/**
   * Create a new plugin manager
   *
   * @param window The main window of the application
   */
#line 58 "plugin-manager.vala"
ValidePluginManager* valide_plugin_manager_construct (GType object_type, ValideWindow* window) {
#line 205 "plugin-manager.c"
	GParameter * __params;
	GParameter * __params_it;
	ValidePluginManager * self;
#line 58 "plugin-manager.vala"
	g_return_val_if_fail (window != NULL, NULL);
#line 211 "plugin-manager.c"
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
#line 60 "plugin-manager.vala"
	__params_it->name = "window";
#line 60 "plugin-manager.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_WINDOW);
#line 60 "plugin-manager.vala"
	g_value_set_object (&__params_it->value, window);
#line 60 "plugin-manager.vala"
	__params_it++;
#line 222 "plugin-manager.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 58 "plugin-manager.vala"
ValidePluginManager* valide_plugin_manager_new (ValideWindow* window) {
#line 58 "plugin-manager.vala"
	return valide_plugin_manager_construct (VALIDE_TYPE_PLUGIN_MANAGER, window);
#line 237 "plugin-manager.c"
}


/**
   * Get the preference widget
   *
   * @return The preference widget
   */
#line 129 "plugin-manager.vala"
GtkWidget* valide_plugin_manager_preferences (ValidePluginManager* self) {
#line 248 "plugin-manager.c"
	GtkWidget* result;
#line 129 "plugin-manager.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 252 "plugin-manager.c"
	result = (GtkWidget*) g_object_ref_sink (valide_plugin_preferences_new (self));
	return result;
}


ValideWindow* valide_plugin_manager_get_window (ValidePluginManager* self) {
	ValideWindow* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_window;
	return result;
}


static void valide_plugin_manager_set_window (ValidePluginManager* self, ValideWindow* value) {
	ValideWindow* _tmp1_;
	ValideWindow* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 33 "plugin-manager.vala"
	_tmp0_ = NULL;
#line 273 "plugin-manager.c"
	self->priv->_window = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "window");
}


static GObject * valide_plugin_manager_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValidePluginManagerClass * klass;
	GObjectClass * parent_class;
	ValidePluginManager * self;
	GError * _inner_error_;
	klass = VALIDE_PLUGIN_MANAGER_CLASS (g_type_class_peek (VALIDE_TYPE_PLUGIN_MANAGER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_PLUGIN_MANAGER (obj);
	_inner_error_ = NULL;
	{
		char** _tmp4_;
		gint plugin_dir_size;
		gint plugin_dir_length1;
		char** _tmp2_;
		char* _tmp1_;
		ValideConfigManager* _tmp0_;
		char** _tmp3_;
		char** plugin_dir;
#line 65 "plugin-manager.vala"
		_tmp4_ = NULL;
#line 65 "plugin-manager.vala"
		_tmp2_ = NULL;
#line 49 "abstract-config-manager.vala"
		_tmp1_ = NULL;
#line 88 "config-manager.vala"
		_tmp0_ = NULL;
#line 65 "plugin-manager.vala"
		_tmp3_ = NULL;
#line 309 "plugin-manager.c"
		plugin_dir = (_tmp4_ = (_tmp3_ = (_tmp2_ = g_new0 (char*, 2 + 1), _tmp2_[0] = g_strdup (PLUGINS_DIR), _tmp2_[1] = g_build_filename (_tmp1_ = valide_abstract_config_manager_get_home_dir ((ValideAbstractConfigManager*) (_tmp0_ = valide_config_manager_get_instance ())), "plugins", NULL), _tmp2_), _tmp1_ = (g_free (_tmp1_), NULL), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp3_), plugin_dir_length1 = 2, plugin_dir_size = plugin_dir_length1, _tmp4_);
		{
			char** d_collection;
			int d_collection_length1;
			int d_it;
#line 70 "plugin-manager.vala"
			d_collection = plugin_dir;
#line 317 "plugin-manager.c"
			d_collection_length1 = plugin_dir_length1;
			for (d_it = 0; d_it < plugin_dir_length1; d_it = d_it + 1) {
				const char* _tmp13_;
				char* d;
#line 65 "plugin-manager.vala"
				_tmp13_ = NULL;
#line 324 "plugin-manager.c"
				d = (_tmp13_ = d_collection[d_it], (_tmp13_ == NULL) ? NULL : g_strdup (_tmp13_));
				{
					{
						GDir* dir;
						dir = g_dir_open (d, 0, &_inner_error_);
						if (_inner_error_ != NULL) {
							goto __catch27_g_error;
							goto __finally27;
						}
#line 76 "plugin-manager.vala"
						while (TRUE) {
#line 336 "plugin-manager.c"
							char* filename;
							char* _tmp6_;
							const char* _tmp5_;
							filename = NULL;
							_tmp6_ = NULL;
#line 80 "plugin-manager.vala"
							_tmp5_ = NULL;
#line 80 "plugin-manager.vala"
							filename = (_tmp6_ = (_tmp5_ = g_dir_read_name (dir), (_tmp5_ == NULL) ? NULL : g_strdup (_tmp5_)), filename = (g_free (filename), NULL), _tmp6_);
#line 81 "plugin-manager.vala"
							if (filename != NULL) {
#line 348 "plugin-manager.c"
								char* file;
								char* _tmp7_;
								char* _tmp11_;
								gboolean _tmp12_;
								file = NULL;
								_tmp7_ = NULL;
#line 85 "plugin-manager.vala"
								file = (_tmp7_ = g_build_filename (d, filename, NULL), file = (g_free (file), NULL), _tmp7_);
#line 86 "plugin-manager.vala"
								if (g_file_test (file, G_FILE_TEST_IS_DIR)) {
#line 359 "plugin-manager.c"
									char* _tmp10_;
									char* _tmp9_;
									char* _tmp8_;
									_tmp10_ = NULL;
									_tmp9_ = NULL;
									_tmp8_ = NULL;
#line 88 "plugin-manager.vala"
									file = (_tmp10_ = g_build_filename (file, _tmp9_ = g_strconcat (_tmp8_ = g_strconcat (filename, ".", NULL), VALIDE_PLUGIN_REGISTRAR_FILE_EXT[0], NULL), NULL), file = (g_free (file), NULL), _tmp10_);
#line 368 "plugin-manager.c"
									_tmp9_ = (g_free (_tmp9_), NULL);
									_tmp8_ = (g_free (_tmp8_), NULL);
								}
								_tmp11_ = NULL;
#line 91 "plugin-manager.vala"
								if ((_tmp12_ = g_str_has_suffix (file, _tmp11_ = g_strconcat (".", VALIDE_PLUGIN_REGISTRAR_FILE_EXT[0], NULL)), _tmp11_ = (g_free (_tmp11_), NULL), _tmp12_)) {
#line 375 "plugin-manager.c"
									{
#line 95 "plugin-manager.vala"
										valide_plugin_manager_register_plugin (self, file, &_inner_error_);
#line 379 "plugin-manager.c"
										if (_inner_error_ != NULL) {
											goto __catch28_g_error;
											goto __finally28;
										}
									}
									goto __finally28;
									__catch28_g_error:
									{
										GError * e;
										e = _inner_error_;
										_inner_error_ = NULL;
										{
#line 99 "plugin-manager.vala"
											g_warning (e->message);
#line 394 "plugin-manager.c"
											(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
										}
									}
									__finally28:
									if (_inner_error_ != NULL) {
										file = (g_free (file), NULL);
										filename = (g_free (filename), NULL);
										(dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL));
										goto __catch27_g_error;
										goto __finally27;
									}
								}
								file = (g_free (file), NULL);
							} else {
								filename = (g_free (filename), NULL);
#line 105 "plugin-manager.vala"
								break;
#line 412 "plugin-manager.c"
							}
							filename = (g_free (filename), NULL);
						}
						(dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL));
					}
					goto __finally27;
					__catch27_g_error:
					{
						GError * e;
						e = _inner_error_;
						_inner_error_ = NULL;
						{
#line 111 "plugin-manager.vala"
							g_debug (e->message);
#line 427 "plugin-manager.c"
							(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
						}
					}
					__finally27:
					if (_inner_error_ != NULL) {
						d = (g_free (d), NULL);
						plugin_dir = (_vala_array_free (plugin_dir, plugin_dir_length1, (GDestroyNotify) g_free), NULL);
						g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
						g_clear_error (&_inner_error_);
					}
					d = (g_free (d), NULL);
				}
			}
		}
		plugin_dir = (_vala_array_free (plugin_dir, plugin_dir_length1, (GDestroyNotify) g_free), NULL);
	}
	return obj;
}


static void valide_plugin_manager_class_init (ValidePluginManagerClass * klass) {
	valide_plugin_manager_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValidePluginManagerPrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_plugin_manager_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_plugin_manager_set_property;
	G_OBJECT_CLASS (klass)->constructor = valide_plugin_manager_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_plugin_manager_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PLUGIN_MANAGER_WINDOW, g_param_spec_object ("window", "window", "window", VALIDE_TYPE_WINDOW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void valide_plugin_manager_instance_init (ValidePluginManager * self) {
	self->priv = VALIDE_PLUGIN_MANAGER_GET_PRIVATE (self);
}


static void valide_plugin_manager_finalize (GObject* obj) {
	ValidePluginManager * self;
	GError * _inner_error_;
	self = VALIDE_PLUGIN_MANAGER (obj);
	_inner_error_ = NULL;
	{
		{
			GList* pr_collection;
			GList* pr_it;
#line 118 "plugin-manager.vala"
			pr_collection = self->plugins;
#line 475 "plugin-manager.c"
			for (pr_it = pr_collection; pr_it != NULL; pr_it = pr_it->next) {
				ValidePluginRegistrar* _tmp14_;
				ValidePluginRegistrar* pr;
#line 39 "plugin-manager.vala"
				_tmp14_ = NULL;
#line 118 "plugin-manager.vala"
				pr = (_tmp14_ = (ValidePluginRegistrar*) pr_it->data, (_tmp14_ == NULL) ? NULL : g_object_ref (_tmp14_));
#line 483 "plugin-manager.c"
				{
#line 120 "plugin-manager.vala"
					valide_plugin_registrar_unload (pr, &_inner_error_);
#line 487 "plugin-manager.c"
					if (_inner_error_ != NULL) {
						(pr == NULL) ? NULL : (pr = (g_object_unref (pr), NULL));
						g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
						g_clear_error (&_inner_error_);
					}
					(pr == NULL) ? NULL : (pr = (g_object_unref (pr), NULL));
				}
			}
		}
	}
	(self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL));
	(self->plugins == NULL) ? NULL : (self->plugins = (_g_list_free_g_object_unref (self->plugins), NULL));
	G_OBJECT_CLASS (valide_plugin_manager_parent_class)->finalize (obj);
}


GType valide_plugin_manager_get_type (void) {
	static GType valide_plugin_manager_type_id = 0;
	if (valide_plugin_manager_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValidePluginManagerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_plugin_manager_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValidePluginManager), 0, (GInstanceInitFunc) valide_plugin_manager_instance_init, NULL };
		valide_plugin_manager_type_id = g_type_register_static (G_TYPE_OBJECT, "ValidePluginManager", &g_define_type_info, 0);
	}
	return valide_plugin_manager_type_id;
}


static void valide_plugin_manager_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValidePluginManager * self;
	gpointer boxed;
	self = VALIDE_PLUGIN_MANAGER (object);
	switch (property_id) {
		case VALIDE_PLUGIN_MANAGER_WINDOW:
		g_value_set_object (value, valide_plugin_manager_get_window (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_plugin_manager_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValidePluginManager * self;
	self = VALIDE_PLUGIN_MANAGER (object);
	switch (property_id) {
		case VALIDE_PLUGIN_MANAGER_WINDOW:
		valide_plugin_manager_set_window (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}




