/* plugin-registrar.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gmodule.h>
#include <gtk/gtk.h>
#include <config.h>
#include <glib/gstdio.h>
#include <glib/gi18n-lib.h>


#define VALIDE_TYPE_PLUGIN_REGISTRAR (valide_plugin_registrar_get_type ())
#define VALIDE_PLUGIN_REGISTRAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PLUGIN_REGISTRAR, ValidePluginRegistrar))
#define VALIDE_PLUGIN_REGISTRAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PLUGIN_REGISTRAR, ValidePluginRegistrarClass))
#define VALIDE_IS_PLUGIN_REGISTRAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PLUGIN_REGISTRAR))
#define VALIDE_IS_PLUGIN_REGISTRAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PLUGIN_REGISTRAR))
#define VALIDE_PLUGIN_REGISTRAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PLUGIN_REGISTRAR, ValidePluginRegistrarClass))

typedef struct _ValidePluginRegistrar ValidePluginRegistrar;
typedef struct _ValidePluginRegistrarClass ValidePluginRegistrarClass;
typedef struct _ValidePluginRegistrarPrivate ValidePluginRegistrarPrivate;

#define VALIDE_TYPE_PLUGIN (valide_plugin_get_type ())
#define VALIDE_PLUGIN(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PLUGIN, ValidePlugin))
#define VALIDE_IS_PLUGIN(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PLUGIN))
#define VALIDE_PLUGIN_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), VALIDE_TYPE_PLUGIN, ValidePluginIface))

typedef struct _ValidePlugin ValidePlugin;
typedef struct _ValidePluginIface ValidePluginIface;

#define VALIDE_TYPE_WINDOW (valide_window_get_type ())
#define VALIDE_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_WINDOW, ValideWindow))
#define VALIDE_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_WINDOW, ValideWindowClass))
#define VALIDE_IS_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_WINDOW))
#define VALIDE_IS_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_WINDOW))
#define VALIDE_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_WINDOW, ValideWindowClass))

typedef struct _ValideWindow ValideWindow;
typedef struct _ValideWindowClass ValideWindowClass;

typedef enum  {
	VALIDE_PLUGIN_ERROR_MODULE_LOADING,
	VALIDE_PLUGIN_ERROR_MODULE_VERSION
} ValidePluginError;
#define VALIDE_PLUGIN_ERROR valide_plugin_error_quark ()
/**
 * This class represents a registered plugin
 */
struct _ValidePluginRegistrar {
	GObject parent_instance;
	ValidePluginRegistrarPrivate * priv;
	char** authors;
	gint authors_length1;
};

struct _ValidePluginRegistrarClass {
	GObjectClass parent_class;
};

/**
 * Interface for create a plugin
 */
struct _ValidePluginIface {
	GTypeInterface parent_iface;
	const char* (*get_path) (ValidePlugin* self);
	void (*set_path) (ValidePlugin* self, const char* value);
	ValideWindow* (*get_window) (ValidePlugin* self);
	void (*set_window) (ValidePlugin* self, ValideWindow* value);
};

typedef GtkWidget* (*ValidePluginRegistrarPreferencePluginFunction) (ValidePlugin* _self_, void* user_data);
struct _ValidePluginRegistrarPrivate {
	GType type;
	GModule* module;
	char* module_path;
	char* _path;
	ValidePluginRegistrarPreferencePluginFunction preferences_plugin;
	gpointer preferences_plugin_target;
	ValideWindow* _window;
	char* _module_name;
	gint _vapi_version;
	gboolean _is_available;
	char* _name;
	char* _description;
	char* _icon_name;
	char* _website;
	char* _copyright;
	gboolean _is_configurable;
	ValidePlugin* _plugin;
};

typedef GType (*ValidePluginRegistrarRegisterPluginFunction) (void* user_data);

static gpointer valide_plugin_registrar_parent_class = NULL;

GQuark valide_plugin_error_quark (void);
GType valide_plugin_registrar_get_type (void);
GType valide_window_get_type (void);
GType valide_plugin_get_type (void);
#define VALIDE_PLUGIN_REGISTRAR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_PLUGIN_REGISTRAR, ValidePluginRegistrarPrivate))
enum  {
	VALIDE_PLUGIN_REGISTRAR_DUMMY_PROPERTY,
	VALIDE_PLUGIN_REGISTRAR_WINDOW,
	VALIDE_PLUGIN_REGISTRAR_PATH,
	VALIDE_PLUGIN_REGISTRAR_IS_ACTIVE,
	VALIDE_PLUGIN_REGISTRAR_IS_AVAILABLE,
	VALIDE_PLUGIN_REGISTRAR_NAME,
	VALIDE_PLUGIN_REGISTRAR_DESCRIPTION,
	VALIDE_PLUGIN_REGISTRAR_ICON_NAME,
	VALIDE_PLUGIN_REGISTRAR_WEBSITE,
	VALIDE_PLUGIN_REGISTRAR_COPYRIGHT,
	VALIDE_PLUGIN_REGISTRAR_IS_CONFIGURABLE,
	VALIDE_PLUGIN_REGISTRAR_PLUGIN
};
const char* valide_plugin_registrar_get_path (ValidePluginRegistrar* self);
static const char* valide_plugin_registrar_get_module_name (ValidePluginRegistrar* self);
static gint valide_plugin_registrar_get_vapi_version (ValidePluginRegistrar* self);
static void valide_plugin_registrar_set_is_configurable (ValidePluginRegistrar* self, gboolean value);
static void valide_plugin_registrar_set_is_available (ValidePluginRegistrar* self, gboolean value);
static void valide_plugin_registrar_register_plugin (ValidePluginRegistrar* self, GError** error);
static void valide_plugin_registrar_set_path (ValidePluginRegistrar* self, const char* value);
static void valide_plugin_registrar_set_window (ValidePluginRegistrar* self, ValideWindow* value);
ValidePluginRegistrar* valide_plugin_registrar_new (const char* filename, ValideWindow* window);
ValidePluginRegistrar* valide_plugin_registrar_construct (GType object_type, const char* filename, ValideWindow* window);
gboolean valide_plugin_registrar_get_is_available (ValidePluginRegistrar* self);
ValideWindow* valide_plugin_registrar_get_window (ValidePluginRegistrar* self);
static void valide_plugin_registrar_set_plugin (ValidePluginRegistrar* self, ValidePlugin* value);
void valide_plugin_registrar_load (ValidePluginRegistrar* self, GError** error);
void valide_plugin_registrar_unload (ValidePluginRegistrar* self, GError** error);
gboolean valide_plugin_registrar_get_is_active (ValidePluginRegistrar* self);
gboolean valide_plugin_registrar_get_is_configurable (ValidePluginRegistrar* self);
ValidePlugin* valide_plugin_registrar_get_plugin (ValidePluginRegistrar* self);
void valide_plugin_registrar_preferences (ValidePluginRegistrar* self);
static void valide_plugin_registrar_set_module_name (ValidePluginRegistrar* self, const char* value);
static void valide_plugin_registrar_set_vapi_version (ValidePluginRegistrar* self, gint value);
static void valide_plugin_registrar_set_name (ValidePluginRegistrar* self, const char* value);
static void valide_plugin_registrar_set_description (ValidePluginRegistrar* self, const char* value);
static void valide_plugin_registrar_set_icon_name (ValidePluginRegistrar* self, const char* value);
static void valide_plugin_registrar_set_website (ValidePluginRegistrar* self, const char* value);
static void valide_plugin_registrar_set_copyright (ValidePluginRegistrar* self, const char* value);
const char* valide_plugin_registrar_get_name (ValidePluginRegistrar* self);
const char* valide_plugin_registrar_get_description (ValidePluginRegistrar* self);
const char* valide_plugin_registrar_get_icon_name (ValidePluginRegistrar* self);
const char* valide_plugin_registrar_get_website (ValidePluginRegistrar* self);
const char* valide_plugin_registrar_get_copyright (ValidePluginRegistrar* self);
static GObject * valide_plugin_registrar_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void valide_plugin_registrar_finalize (GObject* obj);
static void valide_plugin_registrar_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_plugin_registrar_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);

static const char* VALIDE_PLUGIN_REGISTRAR_FILE_EXT[] = {"valide-plugin"};


GQuark valide_plugin_error_quark (void) {
	return g_quark_from_static_string ("valide_plugin_error-quark");
}


#line 197 "plugin-registrar.vala"
static void valide_plugin_registrar_register_plugin (ValidePluginRegistrar* self, GError** error) {
#line 185 "plugin-registrar.c"
	GError * _inner_error_;
	char* _tmp1_;
	char* _tmp0_;
#line 197 "plugin-registrar.vala"
	g_return_if_fail (self != NULL);
#line 191 "plugin-registrar.c"
	_inner_error_ = NULL;
	_tmp1_ = NULL;
#line 2032 "glib-2.0.vapi"
	_tmp0_ = NULL;
#line 199 "plugin-registrar.vala"
	self->priv->module_path = (_tmp1_ = g_module_build_path (_tmp0_ = g_path_get_dirname (valide_plugin_registrar_get_path (self)), self->priv->_module_name), self->priv->module_path = (g_free (self->priv->module_path), NULL), _tmp1_);
#line 198 "plugin-registrar.c"
	_tmp0_ = (g_free (_tmp0_), NULL);
#line 202 "plugin-registrar.vala"
	if (self->priv->_vapi_version <= atoi (VAPI_VERSION)) {
#line 204 "plugin-registrar.vala"
		if (g_file_test (self->priv->module_path, G_FILE_TEST_EXISTS)) {
#line 204 "plugin-registrar.c"
			GModule* _tmp2_;
			_tmp2_ = NULL;
#line 206 "plugin-registrar.vala"
			self->priv->module = (_tmp2_ = g_module_open (self->priv->module_path, G_MODULE_BIND_LAZY), (self->priv->module == NULL) ? NULL : (self->priv->module = (g_module_close (self->priv->module), NULL)), _tmp2_);
#line 207 "plugin-registrar.vala"
			if (self->priv->module != NULL) {
#line 211 "plugin-registrar.c"
				void* function;
				void* register_plugin_target;
				ValidePluginRegistrarRegisterPluginFunction register_plugin;
				ValidePluginRegistrarRegisterPluginFunction _tmp3_;
				ValidePluginRegistrarPreferencePluginFunction _tmp4_;
				function = NULL;
				register_plugin_target = NULL;
				register_plugin = NULL;
#line 212 "plugin-registrar.vala"
				g_module_symbol (self->priv->module, "register_plugin", &function);
#line 213 "plugin-registrar.vala"
				g_return_if_fail (function != NULL);
#line 214 "plugin-registrar.vala"
				register_plugin = (_tmp3_ = (ValidePluginRegistrarRegisterPluginFunction) function, register_plugin_target = NULL, _tmp3_);
#line 216 "plugin-registrar.vala"
				self->priv->type = register_plugin (register_plugin_target);
#line 218 "plugin-registrar.vala"
				g_module_symbol (self->priv->module, "create_configure_dialog", &function);
#line 219 "plugin-registrar.vala"
				self->priv->preferences_plugin = (_tmp4_ = (ValidePluginRegistrarPreferencePluginFunction) function, self->priv->preferences_plugin_target = NULL, _tmp4_);
#line 220 "plugin-registrar.vala"
				valide_plugin_registrar_set_is_configurable (self, function != NULL);
#line 234 "plugin-registrar.c"
			} else {
#line 224 "plugin-registrar.vala"
				valide_plugin_registrar_set_is_available (self, FALSE);
#line 238 "plugin-registrar.c"
				_inner_error_ = g_error_new (VALIDE_PLUGIN_ERROR, VALIDE_PLUGIN_ERROR_MODULE_LOADING, "%s", g_module_error);
				if (_inner_error_ != NULL) {
					g_propagate_error (error, _inner_error_);
					return;
				}
			}
		} else {
#line 230 "plugin-registrar.vala"
			valide_plugin_registrar_set_is_available (self, FALSE);
#line 248 "plugin-registrar.c"
			_inner_error_ = g_error_new_literal (VALIDE_PLUGIN_ERROR, VALIDE_PLUGIN_ERROR_MODULE_LOADING, _ ("the module doesn't exist"));
			if (_inner_error_ != NULL) {
				g_propagate_error (error, _inner_error_);
				return;
			}
		}
	} else {
#line 236 "plugin-registrar.vala"
		valide_plugin_registrar_set_is_available (self, FALSE);
#line 258 "plugin-registrar.c"
		_inner_error_ = g_error_new_literal (VALIDE_PLUGIN_ERROR, VALIDE_PLUGIN_ERROR_MODULE_VERSION, _ ("Incompatible vapi version"));
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			return;
		}
	}
}


/**
   * Register a new plugin
   *
   * @param filename The path of plugin
   */
#line 251 "plugin-registrar.vala"
ValidePluginRegistrar* valide_plugin_registrar_construct (GType object_type, const char* filename, ValideWindow* window) {
#line 275 "plugin-registrar.c"
	GParameter * __params;
	GParameter * __params_it;
	ValidePluginRegistrar * self;
#line 251 "plugin-registrar.vala"
	g_return_val_if_fail (filename != NULL, NULL);
#line 251 "plugin-registrar.vala"
	g_return_val_if_fail (window != NULL, NULL);
#line 283 "plugin-registrar.c"
	__params = g_new0 (GParameter, 2);
	__params_it = __params;
#line 253 "plugin-registrar.vala"
	__params_it->name = "path";
#line 253 "plugin-registrar.vala"
	g_value_init (&__params_it->value, G_TYPE_STRING);
#line 253 "plugin-registrar.vala"
	g_value_set_string (&__params_it->value, filename);
#line 253 "plugin-registrar.vala"
	__params_it++;
#line 254 "plugin-registrar.vala"
	__params_it->name = "window";
#line 254 "plugin-registrar.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_WINDOW);
#line 254 "plugin-registrar.vala"
	g_value_set_object (&__params_it->value, window);
#line 254 "plugin-registrar.vala"
	__params_it++;
#line 302 "plugin-registrar.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 251 "plugin-registrar.vala"
ValidePluginRegistrar* valide_plugin_registrar_new (const char* filename, ValideWindow* window) {
#line 251 "plugin-registrar.vala"
	return valide_plugin_registrar_construct (VALIDE_TYPE_PLUGIN_REGISTRAR, filename, window);
#line 317 "plugin-registrar.c"
}


/**
   * Load the plugin
   */
#line 273 "plugin-registrar.vala"
void valide_plugin_registrar_load (ValidePluginRegistrar* self, GError** error) {
#line 326 "plugin-registrar.c"
	GError * _inner_error_;
#line 273 "plugin-registrar.vala"
	g_return_if_fail (self != NULL);
#line 330 "plugin-registrar.c"
	_inner_error_ = NULL;
#line 275 "plugin-registrar.vala"
	if (self->priv->_is_available) {
#line 334 "plugin-registrar.c"
		ValidePlugin* _tmp2_;
		char* _tmp0_;
		GObject* _tmp1_;
#line 279 "plugin-registrar.vala"
		_tmp2_ = NULL;
#line 2032 "glib-2.0.vapi"
		_tmp0_ = NULL;
#line 277 "plugin-registrar.vala"
		_tmp1_ = NULL;
#line 277 "plugin-registrar.vala"
		valide_plugin_registrar_set_plugin (self, _tmp2_ = (_tmp1_ = g_object_new (self->priv->type, "path", _tmp0_ = g_path_get_dirname (self->priv->module_path), "window", self->priv->_window, NULL), VALIDE_IS_PLUGIN (_tmp1_) ? ((ValidePlugin*) _tmp1_) : NULL));
#line 346 "plugin-registrar.c"
		(_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL));
		_tmp0_ = (g_free (_tmp0_), NULL);
	} else {
		_inner_error_ = g_error_new (VALIDE_PLUGIN_ERROR, VALIDE_PLUGIN_ERROR_MODULE_LOADING, _ ("Can't load module (%s): module is unavailable"), valide_plugin_registrar_get_path (self));
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			return;
		}
	}
}


/**
   * Unload the plugin
   */
#line 291 "plugin-registrar.vala"
void valide_plugin_registrar_unload (ValidePluginRegistrar* self, GError** error) {
#line 291 "plugin-registrar.vala"
	g_return_if_fail (self != NULL);
#line 293 "plugin-registrar.vala"
	valide_plugin_registrar_set_plugin (self, NULL);
#line 368 "plugin-registrar.c"
}


/**
   * Show the plugin preferences
   */
#line 299 "plugin-registrar.vala"
void valide_plugin_registrar_preferences (ValidePluginRegistrar* self) {
#line 377 "plugin-registrar.c"
	gboolean _tmp0_;
	gboolean _tmp1_;
#line 299 "plugin-registrar.vala"
	g_return_if_fail (self != NULL);
#line 382 "plugin-registrar.c"
	_tmp0_ = FALSE;
	_tmp1_ = FALSE;
#line 301 "plugin-registrar.vala"
	if (self->priv->_is_available) {
#line 301 "plugin-registrar.vala"
		_tmp1_ = valide_plugin_registrar_get_is_active (self);
#line 389 "plugin-registrar.c"
	} else {
#line 301 "plugin-registrar.vala"
		_tmp1_ = FALSE;
#line 393 "plugin-registrar.c"
	}
#line 301 "plugin-registrar.vala"
	if (_tmp1_) {
#line 301 "plugin-registrar.vala"
		_tmp0_ = self->priv->_is_configurable;
#line 399 "plugin-registrar.c"
	} else {
#line 301 "plugin-registrar.vala"
		_tmp0_ = FALSE;
#line 403 "plugin-registrar.c"
	}
#line 301 "plugin-registrar.vala"
	if (_tmp0_) {
#line 407 "plugin-registrar.c"
		GtkDialog* dialog;
		GtkDialog* _tmp2_;
		GtkWidget* _tmp3_;
		dialog = NULL;
		_tmp2_ = NULL;
#line 305 "plugin-registrar.vala"
		dialog = (_tmp2_ = g_object_ref_sink ((GtkDialog*) gtk_dialog_new_with_buttons ("Plugin preferences", NULL, GTK_DIALOG_MODAL, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL)), (dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL)), _tmp2_);
#line 51 "plugin-registrar.vala"
		_tmp3_ = NULL;
#line 310 "plugin-registrar.vala"
		gtk_box_pack_start ((GtkBox*) dialog->vbox, _tmp3_ = self->priv->preferences_plugin (self->priv->_plugin, self->priv->preferences_plugin_target), TRUE, TRUE, (guint) 0);
#line 419 "plugin-registrar.c"
		(_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL));
#line 312 "plugin-registrar.vala"
		gtk_widget_show_all ((GtkWidget*) dialog);
#line 313 "plugin-registrar.vala"
		gtk_dialog_run (dialog);
#line 314 "plugin-registrar.vala"
		gtk_object_destroy ((GtkObject*) dialog);
#line 427 "plugin-registrar.c"
		(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
	}
}


ValideWindow* valide_plugin_registrar_get_window (ValidePluginRegistrar* self) {
	ValideWindow* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_window;
	return result;
}


static void valide_plugin_registrar_set_window (ValidePluginRegistrar* self, ValideWindow* value) {
	ValideWindow* _tmp1_;
	ValideWindow* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 59 "plugin-registrar.vala"
	_tmp0_ = NULL;
#line 448 "plugin-registrar.c"
	self->priv->_window = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "window");
}


const char* valide_plugin_registrar_get_path (ValidePluginRegistrar* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_path;
	return result;
}


static void valide_plugin_registrar_set_path (ValidePluginRegistrar* self, const char* value) {
	GError * _inner_error_;
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	_tmp1_ = NULL;
#line 69 "plugin-registrar.vala"
	_tmp0_ = NULL;
#line 69 "plugin-registrar.vala"
	self->priv->_path = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_path = (g_free (self->priv->_path), NULL), _tmp1_);
#line 473 "plugin-registrar.c"
	{
		GKeyFile* key_file;
		GKeyFile* _tmp2_;
		char* _tmp3_;
		char* _tmp4_;
		gint _tmp5_;
		char* _tmp6_;
		char* _tmp7_;
		char* _tmp8_;
		char* _tmp9_;
		char* _tmp10_;
		char* _tmp11_;
		char** _tmp14_;
		gint _tmp13__size;
		gint _tmp13__length1;
		gint _tmp12_;
		char** _tmp13_;
		char** _tmp15_;
		char* _tmp16_;
		char* _tmp17_;
		char* _tmp18_;
		char* _tmp19_;
		key_file = NULL;
		_tmp2_ = NULL;
#line 74 "plugin-registrar.vala"
		key_file = (_tmp2_ = g_key_file_new (), (key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL)), _tmp2_);
#line 75 "plugin-registrar.vala"
		g_key_file_load_from_file (key_file, value, G_KEY_FILE_NONE, &_inner_error_);
#line 502 "plugin-registrar.c"
		if (_inner_error_ != NULL) {
			(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
			goto __catch25_g_error;
			goto __finally25;
		}
		_tmp3_ = g_key_file_get_string (key_file, "Valide Plugin", "Module", &_inner_error_);
		if (_inner_error_ != NULL) {
			(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
			goto __catch25_g_error;
			goto __finally25;
		}
#line 2802 "glib-2.0.vapi"
		_tmp4_ = NULL;
#line 76 "plugin-registrar.vala"
		valide_plugin_registrar_set_module_name (self, _tmp4_ = _tmp3_);
#line 518 "plugin-registrar.c"
		_tmp4_ = (g_free (_tmp4_), NULL);
		_tmp5_ = g_key_file_get_integer (key_file, "Valide Plugin", "Vapi", &_inner_error_);
		if (_inner_error_ != NULL) {
			(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
			goto __catch25_g_error;
			goto __finally25;
		}
#line 77 "plugin-registrar.vala"
		valide_plugin_registrar_set_vapi_version (self, _tmp5_);
#line 528 "plugin-registrar.c"
		_tmp6_ = g_key_file_get_string (key_file, "Valide Plugin", "Name", &_inner_error_);
		if (_inner_error_ != NULL) {
			(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
			goto __catch25_g_error;
			goto __finally25;
		}
#line 2802 "glib-2.0.vapi"
		_tmp7_ = NULL;
#line 78 "plugin-registrar.vala"
		valide_plugin_registrar_set_name (self, _tmp7_ = _tmp6_);
#line 539 "plugin-registrar.c"
		_tmp7_ = (g_free (_tmp7_), NULL);
		_tmp8_ = g_key_file_get_string (key_file, "Valide Plugin", "Description", &_inner_error_);
		if (_inner_error_ != NULL) {
			(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
			goto __catch25_g_error;
			goto __finally25;
		}
#line 2802 "glib-2.0.vapi"
		_tmp9_ = NULL;
#line 79 "plugin-registrar.vala"
		valide_plugin_registrar_set_description (self, _tmp9_ = _tmp8_);
#line 551 "plugin-registrar.c"
		_tmp9_ = (g_free (_tmp9_), NULL);
		_tmp10_ = g_key_file_get_string (key_file, "Valide Plugin", "Icon", &_inner_error_);
		if (_inner_error_ != NULL) {
			(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
			goto __catch25_g_error;
			goto __finally25;
		}
#line 2802 "glib-2.0.vapi"
		_tmp11_ = NULL;
#line 80 "plugin-registrar.vala"
		valide_plugin_registrar_set_icon_name (self, _tmp11_ = _tmp10_);
#line 563 "plugin-registrar.c"
		_tmp11_ = (g_free (_tmp11_), NULL);
#line 81 "plugin-registrar.vala"
		_tmp14_ = NULL;
#line 567 "plugin-registrar.c"
		_tmp13_ = (_tmp14_ = g_key_file_get_string_list (key_file, "Valide Plugin", "Authors", &_tmp12_, &_inner_error_), _tmp13__length1 = _tmp12_, _tmp13__size = _tmp13__length1, _tmp14_);
		if (_inner_error_ != NULL) {
			(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
			goto __catch25_g_error;
			goto __finally25;
		}
		_tmp15_ = NULL;
#line 81 "plugin-registrar.vala"
		self->authors = (_tmp15_ = _tmp13_, self->authors = (_vala_array_free (self->authors, self->authors_length1, (GDestroyNotify) g_free), NULL), self->authors_length1 = _tmp13__length1, _tmp15_);
#line 577 "plugin-registrar.c"
		_tmp16_ = g_key_file_get_string (key_file, "Valide Plugin", "Website", &_inner_error_);
		if (_inner_error_ != NULL) {
			(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
			goto __catch25_g_error;
			goto __finally25;
		}
#line 2802 "glib-2.0.vapi"
		_tmp17_ = NULL;
#line 82 "plugin-registrar.vala"
		valide_plugin_registrar_set_website (self, _tmp17_ = _tmp16_);
#line 588 "plugin-registrar.c"
		_tmp17_ = (g_free (_tmp17_), NULL);
		_tmp18_ = g_key_file_get_string (key_file, "Valide Plugin", "Copyright", &_inner_error_);
		if (_inner_error_ != NULL) {
			(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
			goto __catch25_g_error;
			goto __finally25;
		}
#line 2802 "glib-2.0.vapi"
		_tmp19_ = NULL;
#line 83 "plugin-registrar.vala"
		valide_plugin_registrar_set_copyright (self, _tmp19_ = _tmp18_);
#line 600 "plugin-registrar.c"
		_tmp19_ = (g_free (_tmp19_), NULL);
#line 84 "plugin-registrar.vala"
		valide_plugin_registrar_register_plugin (self, &_inner_error_);
#line 604 "plugin-registrar.c"
		if (_inner_error_ != NULL) {
			(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
			goto __catch25_g_error;
			goto __finally25;
		}
#line 85 "plugin-registrar.vala"
		valide_plugin_registrar_set_is_available (self, TRUE);
#line 612 "plugin-registrar.c"
		(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
	}
	goto __finally25;
	__catch25_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			char* _tmp21_;
			char* _tmp20_;
#line 89 "plugin-registrar.vala"
			g_debug (_ ("Can't load the plugin '%s': %s"), value, e->message);
#line 90 "plugin-registrar.vala"
			valide_plugin_registrar_set_description (self, e->message);
#line 2031 "glib-2.0.vapi"
			_tmp21_ = NULL;
#line 2032 "glib-2.0.vapi"
			_tmp20_ = NULL;
#line 91 "plugin-registrar.vala"
			valide_plugin_registrar_set_name (self, _tmp21_ = g_path_get_basename (_tmp20_ = g_path_get_dirname (value)));
#line 634 "plugin-registrar.c"
			_tmp21_ = (g_free (_tmp21_), NULL);
			_tmp20_ = (g_free (_tmp20_), NULL);
#line 92 "plugin-registrar.vala"
			valide_plugin_registrar_set_is_available (self, FALSE);
#line 639 "plugin-registrar.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally25:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	g_object_notify ((GObject *) self, "path");
}


static const char* valide_plugin_registrar_get_module_name (ValidePluginRegistrar* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_module_name;
	return result;
}


static void valide_plugin_registrar_set_module_name (ValidePluginRegistrar* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 105 "plugin-registrar.vala"
	_tmp0_ = NULL;
#line 668 "plugin-registrar.c"
	self->priv->_module_name = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_module_name = (g_free (self->priv->_module_name), NULL), _tmp1_);
}


static gint valide_plugin_registrar_get_vapi_version (ValidePluginRegistrar* self) {
	gint result;
	g_return_val_if_fail (self != NULL, 0);
	result = self->priv->_vapi_version;
	return result;
}


static void valide_plugin_registrar_set_vapi_version (ValidePluginRegistrar* self, gint value) {
	g_return_if_fail (self != NULL);
	self->priv->_vapi_version = value;
}


gboolean valide_plugin_registrar_get_is_active (ValidePluginRegistrar* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = self->priv->_plugin != NULL;
	return result;
}


gboolean valide_plugin_registrar_get_is_available (ValidePluginRegistrar* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = self->priv->_is_available;
	return result;
}


static void valide_plugin_registrar_set_is_available (ValidePluginRegistrar* self, gboolean value) {
	g_return_if_fail (self != NULL);
	self->priv->_is_available = value;
	g_object_notify ((GObject *) self, "is-available");
}


const char* valide_plugin_registrar_get_name (ValidePluginRegistrar* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_name;
	return result;
}


static void valide_plugin_registrar_set_name (ValidePluginRegistrar* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 140 "plugin-registrar.vala"
	_tmp0_ = NULL;
#line 725 "plugin-registrar.c"
	self->priv->_name = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_name = (g_free (self->priv->_name), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "name");
}


const char* valide_plugin_registrar_get_description (ValidePluginRegistrar* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_description;
	return result;
}


static void valide_plugin_registrar_set_description (ValidePluginRegistrar* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 149 "plugin-registrar.vala"
	_tmp0_ = NULL;
#line 746 "plugin-registrar.c"
	self->priv->_description = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_description = (g_free (self->priv->_description), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "description");
}


const char* valide_plugin_registrar_get_icon_name (ValidePluginRegistrar* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_icon_name;
	return result;
}


static void valide_plugin_registrar_set_icon_name (ValidePluginRegistrar* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 158 "plugin-registrar.vala"
	_tmp0_ = NULL;
#line 767 "plugin-registrar.c"
	self->priv->_icon_name = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_icon_name = (g_free (self->priv->_icon_name), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "icon-name");
}


const char* valide_plugin_registrar_get_website (ValidePluginRegistrar* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_website;
	return result;
}


static void valide_plugin_registrar_set_website (ValidePluginRegistrar* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 167 "plugin-registrar.vala"
	_tmp0_ = NULL;
#line 788 "plugin-registrar.c"
	self->priv->_website = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_website = (g_free (self->priv->_website), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "website");
}


const char* valide_plugin_registrar_get_copyright (ValidePluginRegistrar* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_copyright;
	return result;
}


static void valide_plugin_registrar_set_copyright (ValidePluginRegistrar* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 176 "plugin-registrar.vala"
	_tmp0_ = NULL;
#line 809 "plugin-registrar.c"
	self->priv->_copyright = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_copyright = (g_free (self->priv->_copyright), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "copyright");
}


gboolean valide_plugin_registrar_get_is_configurable (ValidePluginRegistrar* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = self->priv->_is_configurable;
	return result;
}


static void valide_plugin_registrar_set_is_configurable (ValidePluginRegistrar* self, gboolean value) {
	g_return_if_fail (self != NULL);
	self->priv->_is_configurable = value;
	g_object_notify ((GObject *) self, "is-configurable");
}


ValidePlugin* valide_plugin_registrar_get_plugin (ValidePluginRegistrar* self) {
	ValidePlugin* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_plugin;
	return result;
}


static void valide_plugin_registrar_set_plugin (ValidePluginRegistrar* self, ValidePlugin* value) {
	ValidePlugin* _tmp1_;
	ValidePlugin* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 194 "plugin-registrar.vala"
	_tmp0_ = NULL;
#line 845 "plugin-registrar.c"
	self->priv->_plugin = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_plugin == NULL) ? NULL : (self->priv->_plugin = (g_object_unref (self->priv->_plugin), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "plugin");
}


static GObject * valide_plugin_registrar_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValidePluginRegistrarClass * klass;
	GObjectClass * parent_class;
	ValidePluginRegistrar * self;
	klass = VALIDE_PLUGIN_REGISTRAR_CLASS (g_type_class_peek (VALIDE_TYPE_PLUGIN_REGISTRAR));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_PLUGIN_REGISTRAR (obj);
	{
#line 243 "plugin-registrar.vala"
		g_assert (g_module_supported ());
#line 863 "plugin-registrar.c"
	}
	return obj;
}


static void valide_plugin_registrar_class_init (ValidePluginRegistrarClass * klass) {
	valide_plugin_registrar_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValidePluginRegistrarPrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_plugin_registrar_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_plugin_registrar_set_property;
	G_OBJECT_CLASS (klass)->constructor = valide_plugin_registrar_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_plugin_registrar_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PLUGIN_REGISTRAR_WINDOW, g_param_spec_object ("window", "window", "window", VALIDE_TYPE_WINDOW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PLUGIN_REGISTRAR_PATH, g_param_spec_string ("path", "path", "path", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PLUGIN_REGISTRAR_IS_ACTIVE, g_param_spec_boolean ("is-active", "is-active", "is-active", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PLUGIN_REGISTRAR_IS_AVAILABLE, g_param_spec_boolean ("is-available", "is-available", "is-available", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PLUGIN_REGISTRAR_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PLUGIN_REGISTRAR_DESCRIPTION, g_param_spec_string ("description", "description", "description", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PLUGIN_REGISTRAR_ICON_NAME, g_param_spec_string ("icon-name", "icon-name", "icon-name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PLUGIN_REGISTRAR_WEBSITE, g_param_spec_string ("website", "website", "website", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PLUGIN_REGISTRAR_COPYRIGHT, g_param_spec_string ("copyright", "copyright", "copyright", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PLUGIN_REGISTRAR_IS_CONFIGURABLE, g_param_spec_boolean ("is-configurable", "is-configurable", "is-configurable", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PLUGIN_REGISTRAR_PLUGIN, g_param_spec_object ("plugin", "plugin", "plugin", VALIDE_TYPE_PLUGIN, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void valide_plugin_registrar_instance_init (ValidePluginRegistrar * self) {
	self->priv = VALIDE_PLUGIN_REGISTRAR_GET_PRIVATE (self);
}


static void valide_plugin_registrar_finalize (GObject* obj) {
	ValidePluginRegistrar * self;
	GError * _inner_error_;
	self = VALIDE_PLUGIN_REGISTRAR (obj);
	_inner_error_ = NULL;
	{
		{
			GModule* _tmp0_;
#line 261 "plugin-registrar.vala"
			valide_plugin_registrar_unload (self, &_inner_error_);
#line 905 "plugin-registrar.c"
			if (_inner_error_ != NULL) {
				goto __catch26_g_error;
				goto __finally26;
			}
			_tmp0_ = NULL;
#line 262 "plugin-registrar.vala"
			self->priv->module = (_tmp0_ = NULL, (self->priv->module == NULL) ? NULL : (self->priv->module = (g_module_close (self->priv->module), NULL)), _tmp0_);
#line 913 "plugin-registrar.c"
		}
		goto __finally26;
		__catch26_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
#line 266 "plugin-registrar.vala"
				g_debug (e->message);
#line 924 "plugin-registrar.c"
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally26:
		if (_inner_error_ != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
		}
	}
	(self->priv->module == NULL) ? NULL : (self->priv->module = (g_module_close (self->priv->module), NULL));
	self->priv->module_path = (g_free (self->priv->module_path), NULL);
	self->priv->_path = (g_free (self->priv->_path), NULL);
	self->authors = (_vala_array_free (self->authors, self->authors_length1, (GDestroyNotify) g_free), NULL);
	(self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL));
	self->priv->_module_name = (g_free (self->priv->_module_name), NULL);
	self->priv->_name = (g_free (self->priv->_name), NULL);
	self->priv->_description = (g_free (self->priv->_description), NULL);
	self->priv->_icon_name = (g_free (self->priv->_icon_name), NULL);
	self->priv->_website = (g_free (self->priv->_website), NULL);
	self->priv->_copyright = (g_free (self->priv->_copyright), NULL);
	(self->priv->_plugin == NULL) ? NULL : (self->priv->_plugin = (g_object_unref (self->priv->_plugin), NULL));
	G_OBJECT_CLASS (valide_plugin_registrar_parent_class)->finalize (obj);
}


GType valide_plugin_registrar_get_type (void) {
	static GType valide_plugin_registrar_type_id = 0;
	if (valide_plugin_registrar_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValidePluginRegistrarClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_plugin_registrar_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValidePluginRegistrar), 0, (GInstanceInitFunc) valide_plugin_registrar_instance_init, NULL };
		valide_plugin_registrar_type_id = g_type_register_static (G_TYPE_OBJECT, "ValidePluginRegistrar", &g_define_type_info, 0);
	}
	return valide_plugin_registrar_type_id;
}


static void valide_plugin_registrar_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValidePluginRegistrar * self;
	gpointer boxed;
	self = VALIDE_PLUGIN_REGISTRAR (object);
	switch (property_id) {
		case VALIDE_PLUGIN_REGISTRAR_WINDOW:
		g_value_set_object (value, valide_plugin_registrar_get_window (self));
		break;
		case VALIDE_PLUGIN_REGISTRAR_PATH:
		g_value_set_string (value, valide_plugin_registrar_get_path (self));
		break;
		case VALIDE_PLUGIN_REGISTRAR_IS_ACTIVE:
		g_value_set_boolean (value, valide_plugin_registrar_get_is_active (self));
		break;
		case VALIDE_PLUGIN_REGISTRAR_IS_AVAILABLE:
		g_value_set_boolean (value, valide_plugin_registrar_get_is_available (self));
		break;
		case VALIDE_PLUGIN_REGISTRAR_NAME:
		g_value_set_string (value, valide_plugin_registrar_get_name (self));
		break;
		case VALIDE_PLUGIN_REGISTRAR_DESCRIPTION:
		g_value_set_string (value, valide_plugin_registrar_get_description (self));
		break;
		case VALIDE_PLUGIN_REGISTRAR_ICON_NAME:
		g_value_set_string (value, valide_plugin_registrar_get_icon_name (self));
		break;
		case VALIDE_PLUGIN_REGISTRAR_WEBSITE:
		g_value_set_string (value, valide_plugin_registrar_get_website (self));
		break;
		case VALIDE_PLUGIN_REGISTRAR_COPYRIGHT:
		g_value_set_string (value, valide_plugin_registrar_get_copyright (self));
		break;
		case VALIDE_PLUGIN_REGISTRAR_IS_CONFIGURABLE:
		g_value_set_boolean (value, valide_plugin_registrar_get_is_configurable (self));
		break;
		case VALIDE_PLUGIN_REGISTRAR_PLUGIN:
		g_value_set_object (value, valide_plugin_registrar_get_plugin (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_plugin_registrar_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValidePluginRegistrar * self;
	self = VALIDE_PLUGIN_REGISTRAR (object);
	switch (property_id) {
		case VALIDE_PLUGIN_REGISTRAR_WINDOW:
		valide_plugin_registrar_set_window (self, g_value_get_object (value));
		break;
		case VALIDE_PLUGIN_REGISTRAR_PATH:
		valide_plugin_registrar_set_path (self, g_value_get_string (value));
		break;
		case VALIDE_PLUGIN_REGISTRAR_IS_AVAILABLE:
		valide_plugin_registrar_set_is_available (self, g_value_get_boolean (value));
		break;
		case VALIDE_PLUGIN_REGISTRAR_NAME:
		valide_plugin_registrar_set_name (self, g_value_get_string (value));
		break;
		case VALIDE_PLUGIN_REGISTRAR_DESCRIPTION:
		valide_plugin_registrar_set_description (self, g_value_get_string (value));
		break;
		case VALIDE_PLUGIN_REGISTRAR_ICON_NAME:
		valide_plugin_registrar_set_icon_name (self, g_value_get_string (value));
		break;
		case VALIDE_PLUGIN_REGISTRAR_WEBSITE:
		valide_plugin_registrar_set_website (self, g_value_get_string (value));
		break;
		case VALIDE_PLUGIN_REGISTRAR_COPYRIGHT:
		valide_plugin_registrar_set_copyright (self, g_value_get_string (value));
		break;
		case VALIDE_PLUGIN_REGISTRAR_IS_CONFIGURABLE:
		valide_plugin_registrar_set_is_configurable (self, g_value_get_boolean (value));
		break;
		case VALIDE_PLUGIN_REGISTRAR_PLUGIN:
		valide_plugin_registrar_set_plugin (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}




