/* preferences-dialog.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <gtksourceview/gtksourceview.h>
#include <gtksourceview/gtksourcestyleschememanager.h>
#include <glib/gi18n-lib.h>


#define VALIDE_TYPE_ABSTRACT_PREFERENCES_DIALOG (valide_abstract_preferences_dialog_get_type ())
#define VALIDE_ABSTRACT_PREFERENCES_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_PREFERENCES_DIALOG, ValideAbstractPreferencesDialog))
#define VALIDE_ABSTRACT_PREFERENCES_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_PREFERENCES_DIALOG, ValideAbstractPreferencesDialogClass))
#define VALIDE_IS_ABSTRACT_PREFERENCES_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_PREFERENCES_DIALOG))
#define VALIDE_IS_ABSTRACT_PREFERENCES_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_PREFERENCES_DIALOG))
#define VALIDE_ABSTRACT_PREFERENCES_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_PREFERENCES_DIALOG, ValideAbstractPreferencesDialogClass))

typedef struct _ValideAbstractPreferencesDialog ValideAbstractPreferencesDialog;
typedef struct _ValideAbstractPreferencesDialogClass ValideAbstractPreferencesDialogClass;
typedef struct _ValideAbstractPreferencesDialogPrivate ValideAbstractPreferencesDialogPrivate;

#define VALIDE_ABSTRACT_PREFERENCES_DIALOG_TYPE_WINDOW_WIDGETS (valide_abstract_preferences_dialog_window_widgets_get_type ())
#define VALIDE_ABSTRACT_PREFERENCES_DIALOG_WINDOW_WIDGETS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_ABSTRACT_PREFERENCES_DIALOG_TYPE_WINDOW_WIDGETS, ValideAbstractPreferencesDialogwindow_widgets))
#define VALIDE_ABSTRACT_PREFERENCES_DIALOG_WINDOW_WIDGETS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_ABSTRACT_PREFERENCES_DIALOG_TYPE_WINDOW_WIDGETS, ValideAbstractPreferencesDialogwindow_widgetsClass))
#define VALIDE_ABSTRACT_PREFERENCES_DIALOG_IS_WINDOW_WIDGETS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_ABSTRACT_PREFERENCES_DIALOG_TYPE_WINDOW_WIDGETS))
#define VALIDE_ABSTRACT_PREFERENCES_DIALOG_IS_WINDOW_WIDGETS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_ABSTRACT_PREFERENCES_DIALOG_TYPE_WINDOW_WIDGETS))
#define VALIDE_ABSTRACT_PREFERENCES_DIALOG_WINDOW_WIDGETS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_ABSTRACT_PREFERENCES_DIALOG_TYPE_WINDOW_WIDGETS, ValideAbstractPreferencesDialogwindow_widgetsClass))

typedef struct _ValideAbstractPreferencesDialogwindow_widgets ValideAbstractPreferencesDialogwindow_widgets;
typedef struct _ValideAbstractPreferencesDialogwindow_widgetsClass ValideAbstractPreferencesDialogwindow_widgetsClass;

#define VALIDE_TYPE_PREFERENCES_DIALOG (valide_preferences_dialog_get_type ())
#define VALIDE_PREFERENCES_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PREFERENCES_DIALOG, ValidePreferencesDialog))
#define VALIDE_PREFERENCES_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PREFERENCES_DIALOG, ValidePreferencesDialogClass))
#define VALIDE_IS_PREFERENCES_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PREFERENCES_DIALOG))
#define VALIDE_IS_PREFERENCES_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PREFERENCES_DIALOG))
#define VALIDE_PREFERENCES_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PREFERENCES_DIALOG, ValidePreferencesDialogClass))

typedef struct _ValidePreferencesDialog ValidePreferencesDialog;
typedef struct _ValidePreferencesDialogClass ValidePreferencesDialogClass;
typedef struct _ValidePreferencesDialogPrivate ValidePreferencesDialogPrivate;

#define VALIDE_TYPE_WINDOW (valide_window_get_type ())
#define VALIDE_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_WINDOW, ValideWindow))
#define VALIDE_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_WINDOW, ValideWindowClass))
#define VALIDE_IS_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_WINDOW))
#define VALIDE_IS_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_WINDOW))
#define VALIDE_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_WINDOW, ValideWindowClass))

typedef struct _ValideWindow ValideWindow;
typedef struct _ValideWindowClass ValideWindowClass;

#define VALIDE_PREFERENCES_DIALOG_TYPE_COL (valide_preferences_dialog_col_get_type ())

#define VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER (valide_abstract_config_manager_get_type ())
#define VALIDE_ABSTRACT_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManager))
#define VALIDE_ABSTRACT_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManagerClass))
#define VALIDE_IS_ABSTRACT_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER))
#define VALIDE_IS_ABSTRACT_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER))
#define VALIDE_ABSTRACT_CONFIG_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManagerClass))

typedef struct _ValideAbstractConfigManager ValideAbstractConfigManager;
typedef struct _ValideAbstractConfigManagerClass ValideAbstractConfigManagerClass;

#define VALIDE_TYPE_CONFIG_MANAGER (valide_config_manager_get_type ())
#define VALIDE_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_CONFIG_MANAGER, ValideConfigManager))
#define VALIDE_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_CONFIG_MANAGER, ValideConfigManagerClass))
#define VALIDE_IS_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_CONFIG_MANAGER))
#define VALIDE_IS_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_CONFIG_MANAGER))
#define VALIDE_CONFIG_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_CONFIG_MANAGER, ValideConfigManagerClass))

typedef struct _ValideConfigManager ValideConfigManager;
typedef struct _ValideConfigManagerClass ValideConfigManagerClass;

#define VALIDE_TYPE_DOCUMENT_MANAGER (valide_document_manager_get_type ())
#define VALIDE_DOCUMENT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_DOCUMENT_MANAGER, ValideDocumentManager))
#define VALIDE_DOCUMENT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_DOCUMENT_MANAGER, ValideDocumentManagerClass))
#define VALIDE_IS_DOCUMENT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_DOCUMENT_MANAGER))
#define VALIDE_IS_DOCUMENT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_DOCUMENT_MANAGER))
#define VALIDE_DOCUMENT_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_DOCUMENT_MANAGER, ValideDocumentManagerClass))

typedef struct _ValideDocumentManager ValideDocumentManager;
typedef struct _ValideDocumentManagerClass ValideDocumentManagerClass;

#define VALIDE_TYPE_IDOCUMENT (valide_idocument_get_type ())
#define VALIDE_IDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_IDOCUMENT, ValideiDocument))
#define VALIDE_IS_IDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_IDOCUMENT))
#define VALIDE_IDOCUMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), VALIDE_TYPE_IDOCUMENT, ValideiDocumentIface))

typedef struct _ValideiDocument ValideiDocument;
typedef struct _ValideiDocumentIface ValideiDocumentIface;

#define VALIDE_TYPE_UI_MANAGER (valide_ui_manager_get_type ())
#define VALIDE_UI_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_UI_MANAGER, ValideUIManager))
#define VALIDE_UI_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_UI_MANAGER, ValideUIManagerClass))
#define VALIDE_IS_UI_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_UI_MANAGER))
#define VALIDE_IS_UI_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_UI_MANAGER))
#define VALIDE_UI_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_UI_MANAGER, ValideUIManagerClass))

typedef struct _ValideUIManager ValideUIManager;
typedef struct _ValideUIManagerClass ValideUIManagerClass;

#define VALIDE_TYPE_DOCUMENT_STATE (valide_document_state_get_type ())

#define VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT (valide_abstract_text_document_get_type ())
#define VALIDE_ABSTRACT_TEXT_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocument))
#define VALIDE_ABSTRACT_TEXT_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocumentClass))
#define VALIDE_IS_ABSTRACT_TEXT_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT))
#define VALIDE_IS_ABSTRACT_TEXT_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT))
#define VALIDE_ABSTRACT_TEXT_DOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocumentClass))

typedef struct _ValideAbstractTextDocument ValideAbstractTextDocument;
typedef struct _ValideAbstractTextDocumentClass ValideAbstractTextDocumentClass;

#define VALIDE_TYPE_SOURCE_VIEW (valide_source_view_get_type ())
#define VALIDE_SOURCE_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_SOURCE_VIEW, ValideSourceView))
#define VALIDE_SOURCE_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_SOURCE_VIEW, ValideSourceViewClass))
#define VALIDE_IS_SOURCE_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_SOURCE_VIEW))
#define VALIDE_IS_SOURCE_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_SOURCE_VIEW))
#define VALIDE_SOURCE_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_SOURCE_VIEW, ValideSourceViewClass))

typedef struct _ValideSourceView ValideSourceView;
typedef struct _ValideSourceViewClass ValideSourceViewClass;
typedef struct _ValideAbstractPreferencesDialogwindow_widgetsPrivate ValideAbstractPreferencesDialogwindow_widgetsPrivate;

#define VALIDE_TYPE_PLUGIN_MANAGER (valide_plugin_manager_get_type ())
#define VALIDE_PLUGIN_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PLUGIN_MANAGER, ValidePluginManager))
#define VALIDE_PLUGIN_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PLUGIN_MANAGER, ValidePluginManagerClass))
#define VALIDE_IS_PLUGIN_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PLUGIN_MANAGER))
#define VALIDE_IS_PLUGIN_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PLUGIN_MANAGER))
#define VALIDE_PLUGIN_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PLUGIN_MANAGER, ValidePluginManagerClass))

typedef struct _ValidePluginManager ValidePluginManager;
typedef struct _ValidePluginManagerClass ValidePluginManagerClass;

struct _ValideAbstractPreferencesDialog {
	GtkObject parent_instance;
	ValideAbstractPreferencesDialogPrivate * priv;
	char* name_space;
	GtkBuilder* xml;
	ValideAbstractPreferencesDialogwindow_widgets* widgets;
};

struct _ValideAbstractPreferencesDialogClass {
	GtkObjectClass parent_class;
};

/* @todo set_smart_home_end 
*
 * The preferences dialog box
 */
struct _ValidePreferencesDialog {
	ValideAbstractPreferencesDialog parent_instance;
	ValidePreferencesDialogPrivate * priv;
};

struct _ValidePreferencesDialogClass {
	ValideAbstractPreferencesDialogClass parent_class;
};

struct _ValidePreferencesDialogPrivate {
	ValideWindow* _window;
};

typedef enum  {
	VALIDE_PREFERENCES_DIALOG_COL_ID,
	VALIDE_PREFERENCES_DIALOG_COL_TEXT,
	VALIDE_PREFERENCES_DIALOG_COL_NB
} ValidePreferencesDialogCol;

typedef enum  {
	VALIDE_DOCUMENT_STATE_NORMAL,
	VALIDE_DOCUMENT_STATE_EXTERNALLY_MODIFIED,
	VALIDE_DOCUMENT_STATE_BAD_ENCODING,
	VALIDE_DOCUMENT_STATE_INFO
} ValideDocumentState;

/**
 * An interface for document
 */
struct _ValideiDocumentIface {
	GTypeInterface parent_iface;
	void (*open) (ValideiDocument* self, const char* path, GError** error);
	void (*save) (ValideiDocument* self, const char* current_folder);
	void (*save_as) (ValideiDocument* self, const char* current_folder);
	void (*rename) (ValideiDocument* self);
	gboolean (*close) (ValideiDocument* self);
	GdkPixbuf* (*get_icon) (ValideiDocument* self);
	char* (*get_tooltips) (ValideiDocument* self);
	void (*setup_ui) (ValideiDocument* self, ValideUIManager* ui_manager);
	ValideDocumentState (*get_state) (ValideiDocument* self);
	void (*set_state) (ValideiDocument* self, ValideDocumentState value);
	gboolean (*get_is_save) (ValideiDocument* self);
	void (*set_is_save) (ValideiDocument* self, gboolean value);
	const char* (*get_path) (ValideiDocument* self);
	void (*set_path) (ValideiDocument* self, const char* value);
	const char* (*get_title) (ValideiDocument* self);
	const char* (*get_filename) (ValideiDocument* self);
};

struct _ValideAbstractPreferencesDialogwindow_widgets {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValideAbstractPreferencesDialogwindow_widgetsPrivate * priv;
	GtkDialog* dialog;
	GtkVBox* vbox;
	GtkNotebook* notebook;
	GtkVBox* frame_editor;
	GtkFrame* frame1;
	GtkAlignment* alignment1;
	GtkVBox* vbox1;
	GtkCheckButton* auto_indent;
	GtkCheckButton* insert_spaces_instead_of_tabs;
	GtkCheckButton* indent_on_tab;
	GtkHBox* hbox2;
	GtkLabel* label5;
	GtkSpinButton* tab_width;
	GtkLabel* label3;
	GtkFrame* frame2;
	GtkAlignment* alignment2;
	GtkVBox* vbox2;
	GtkCheckButton* show_right_margin;
	GtkHBox* hbox3;
	GtkLabel* label7;
	GtkSpinButton* right_margin_position;
	GtkLabel* label6;
	GtkFrame* frame4;
	GtkAlignment* alignment4;
	GtkCheckButton* show_line_numbers;
	GtkLabel* label11;
	GtkFrame* frame5;
	GtkAlignment* alignment5;
	GtkCheckButton* highlight_current_line;
	GtkLabel* label12;
	GtkLabel* label1;
	GtkVBox* vbox4;
	GtkFrame* frame3;
	GtkAlignment* alignment3;
	GtkHBox* hbox4;
	GtkLabel* label8;
	GtkFontButton* font;
	GtkLabel* label2;
	GtkFrame* frame6;
	GtkAlignment* alignment6;
	GtkScrolledWindow* scrolledwindow1;
	GtkTreeView* style_scheme;
	GtkLabel* label14;
	GtkLabel* label13;
	GtkVBox* vbox3;
	GtkHBox* hbox5;
	GtkLabel* label10;
	GtkFileChooserButton* default_directory;
	GtkLabel* label9;
	GtkHButtonBox* dialog_action_area1;
	GtkButton* button;
};

struct _ValideAbstractPreferencesDialogwindow_widgetsClass {
	GTypeClass parent_class;
	void (*finalize) (ValideAbstractPreferencesDialogwindow_widgets *self);
};


static gpointer valide_preferences_dialog_parent_class = NULL;

GType valide_abstract_preferences_dialog_get_type (void);
gpointer valide_abstract_preferences_dialog_window_widgets_ref (gpointer instance);
void valide_abstract_preferences_dialog_window_widgets_unref (gpointer instance);
GParamSpec* valide_abstract_preferences_dialog_param_spec_window_widgets (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void valide_abstract_preferences_dialog_value_set_window_widgets (GValue* value, gpointer v_object);
gpointer valide_abstract_preferences_dialog_value_get_window_widgets (const GValue* value);
GType valide_abstract_preferences_dialog_window_widgets_get_type (void);
GType valide_preferences_dialog_get_type (void);
GType valide_window_get_type (void);
#define VALIDE_PREFERENCES_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_PREFERENCES_DIALOG, ValidePreferencesDialogPrivate))
enum  {
	VALIDE_PREFERENCES_DIALOG_DUMMY_PROPERTY,
	VALIDE_PREFERENCES_DIALOG_WINDOW
};
static GType valide_preferences_dialog_col_get_type (void);
GType valide_abstract_config_manager_get_type (void);
GType valide_config_manager_get_type (void);
ValideConfigManager* valide_config_manager_get_instance (void);
void valide_abstract_config_manager_set_integer (ValideAbstractConfigManager* self, const char* group, const char* key, gint val);
void valide_abstract_config_manager_set_string (ValideAbstractConfigManager* self, const char* group, const char* key, const char* val);
void valide_abstract_config_manager_set_boolean (ValideAbstractConfigManager* self, const char* group, const char* key, gboolean val);
ValideWindow* valide_preferences_dialog_get_window (ValidePreferencesDialog* self);
GType valide_document_manager_get_type (void);
ValideDocumentManager* valide_window_get_documents (ValideWindow* self);
GType valide_ui_manager_get_type (void);
GType valide_document_state_get_type (void);
GType valide_idocument_get_type (void);
GList* valide_document_manager_get_documents (ValideDocumentManager* self);
GType valide_abstract_text_document_get_type (void);
GType valide_source_view_get_type (void);
ValideSourceView* valide_abstract_text_document_get_text_view (ValideAbstractTextDocument* self);
static void valide_preferences_dialog_update_param (ValidePreferencesDialog* self, GtkWidget* sender, const char* group);
void valide_preferences_dialog_update_editor_param (GtkWidget* sender, ValidePreferencesDialog* self);
void valide_preferences_dialog_update_font_param (GtkWidget* sender, ValidePreferencesDialog* self);
void valide_preferences_dialog_update_projects_param (GtkWidget* sender, ValidePreferencesDialog* self);
char* valide_abstract_config_manager_get_config_file (ValideAbstractConfigManager* self);
gint valide_abstract_config_manager_get_integer (ValideAbstractConfigManager* self, const char* group, const char* key);
char* valide_abstract_config_manager_get_string (ValideAbstractConfigManager* self, const char* group, const char* key);
gboolean valide_abstract_config_manager_get_boolean (ValideAbstractConfigManager* self, const char* group, const char* key);
static void valide_preferences_dialog_set_param (ValidePreferencesDialog* self, GError** error);
static void valide_preferences_dialog_populate_style_scheme (ValidePreferencesDialog* self);
static void valide_preferences_dialog_set_window (ValidePreferencesDialog* self, ValideWindow* value);
ValidePreferencesDialog* valide_preferences_dialog_new (ValideWindow* window);
ValidePreferencesDialog* valide_preferences_dialog_construct (GType object_type, ValideWindow* window);
gint valide_preferences_dialog_run (ValidePreferencesDialog* self);
void valide_preferences_dialog_destroy (ValidePreferencesDialog* self);
static void _lambda48_ (GtkToggleButton* s, ValidePreferencesDialog* self);
static void __lambda48__gtk_toggle_button_toggled (GtkToggleButton* _sender, gpointer self);
void valide_abstract_preferences_dialog_connect_signals (ValideAbstractPreferencesDialog* self, const char* name_space);
GType valide_plugin_manager_get_type (void);
ValidePluginManager* valide_window_get_plugins (ValideWindow* self);
GtkWidget* valide_plugin_manager_preferences (ValidePluginManager* self);
static GObject * valide_preferences_dialog_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void valide_preferences_dialog_finalize (GObject* obj);
static void valide_preferences_dialog_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_preferences_dialog_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);




static GType valide_preferences_dialog_col_get_type (void) {
	static GType valide_preferences_dialog_col_type_id = 0;
	if (G_UNLIKELY (valide_preferences_dialog_col_type_id == 0)) {
		static const GEnumValue values[] = {{VALIDE_PREFERENCES_DIALOG_COL_ID, "VALIDE_PREFERENCES_DIALOG_COL_ID", "id"}, {VALIDE_PREFERENCES_DIALOG_COL_TEXT, "VALIDE_PREFERENCES_DIALOG_COL_TEXT", "text"}, {VALIDE_PREFERENCES_DIALOG_COL_NB, "VALIDE_PREFERENCES_DIALOG_COL_NB", "nb"}, {0, NULL, NULL}};
		valide_preferences_dialog_col_type_id = g_enum_register_static ("ValidePreferencesDialogCol", values);
	}
	return valide_preferences_dialog_col_type_id;
}


#line 942 "glib-2.0.vapi"
static char* string_replace (const char* self, const char* old, const char* replacement) {
#line 365 "preferences-dialog.c"
	char* result;
	GError * _inner_error_;
#line 942 "glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 942 "glib-2.0.vapi"
	g_return_val_if_fail (old != NULL, NULL);
#line 942 "glib-2.0.vapi"
	g_return_val_if_fail (replacement != NULL, NULL);
#line 374 "preferences-dialog.c"
	_inner_error_ = NULL;
	{
		char* _tmp0_;
		GRegex* _tmp1_;
		GRegex* regex;
		char* _tmp2_;
#line 2667 "glib-2.0.vapi"
		_tmp0_ = NULL;
#line 944 "glib-2.0.vapi"
		_tmp1_ = NULL;
#line 385 "preferences-dialog.c"
		regex = (_tmp1_ = g_regex_new (_tmp0_ = g_regex_escape_string (old, -1), 0, 0, &_inner_error_), _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_REGEX_ERROR) {
				goto __catch18_g_regex_error;
			}
			goto __finally18;
		}
		_tmp2_ = g_regex_replace_literal (regex, self, (glong) (-1), 0, replacement, 0, &_inner_error_);
		if (_inner_error_ != NULL) {
			(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
			if (_inner_error_->domain == G_REGEX_ERROR) {
				goto __catch18_g_regex_error;
			}
			goto __finally18;
		}
		result = _tmp2_;
		(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
		return result;
	}
	goto __finally18;
	__catch18_g_regex_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 947 "glib-2.0.vapi"
			g_assert_not_reached ();
#line 414 "preferences-dialog.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally18:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
}


#line 44 "preferences-dialog.vala"
static void valide_preferences_dialog_update_param (ValidePreferencesDialog* self, GtkWidget* sender, const char* group) {
#line 429 "preferences-dialog.c"
	GError * _inner_error_;
#line 44 "preferences-dialog.vala"
	g_return_if_fail (self != NULL);
#line 44 "preferences-dialog.vala"
	g_return_if_fail (sender != NULL);
#line 44 "preferences-dialog.vala"
	g_return_if_fail (group != NULL);
#line 437 "preferences-dialog.c"
	_inner_error_ = NULL;
	{
		GValue val = {0};
		char* key;
		ValideConfigManager* config;
		ValideConfigManager* _tmp0_;
		char* _tmp1_;
		GQuark _tmp35_;
		const char* _tmp34_;
		static GQuark _tmp35__label0 = 0;
		static GQuark _tmp35__label1 = 0;
		static GQuark _tmp35__label2 = 0;
		static GQuark _tmp35__label3 = 0;
		static GQuark _tmp35__label4 = 0;
		key = NULL;
		config = NULL;
		_tmp0_ = NULL;
#line 52 "preferences-dialog.vala"
		config = (_tmp0_ = valide_config_manager_get_instance (), (config == NULL) ? NULL : (config = (g_object_unref (config), NULL)), _tmp0_);
#line 457 "preferences-dialog.c"
		_tmp1_ = NULL;
#line 53 "preferences-dialog.vala"
		key = (_tmp1_ = string_replace (gtk_buildable_get_name ((GtkBuildable*) sender), "_", "-"), key = (g_free (key), NULL), _tmp1_);
#line 54 "preferences-dialog.vala"
		_tmp34_ = NULL;
#line 463 "preferences-dialog.c"
		_tmp34_ = g_type_name (G_TYPE_FROM_INSTANCE ((GObject*) sender));
		_tmp35_ = (NULL == _tmp34_) ? 0 : g_quark_from_string (_tmp34_);
		if (_tmp35_ == ((0 != _tmp35__label0) ? _tmp35__label0 : (_tmp35__label0 = g_quark_from_static_string ("GtkSpinButton"))))
		do {
			GtkSpinButton* btn;
			GtkSpinButton* _tmp4_;
			GtkSpinButton* _tmp3_;
			GtkWidget* _tmp2_;
			GValue _tmp6_ = {0};
			GValue _tmp5_ = {0};
			btn = NULL;
			_tmp4_ = NULL;
#line 59 "preferences-dialog.vala"
			_tmp3_ = NULL;
#line 59 "preferences-dialog.vala"
			_tmp2_ = NULL;
#line 59 "preferences-dialog.vala"
			btn = (_tmp4_ = (_tmp3_ = (_tmp2_ = sender, GTK_IS_SPIN_BUTTON (_tmp2_) ? ((GtkSpinButton*) _tmp2_) : NULL), (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), (btn == NULL) ? NULL : (btn = (g_object_unref (btn), NULL)), _tmp4_);
#line 60 "preferences-dialog.vala"
			val = (_tmp6_ = (g_value_init (&_tmp5_, G_TYPE_INT), _tmp5_), G_IS_VALUE (&val) ? (g_value_unset (&val), NULL) : NULL, _tmp6_);
#line 61 "preferences-dialog.vala"
			g_value_set_int (&val, gtk_spin_button_get_value_as_int (btn));
#line 62 "preferences-dialog.vala"
			valide_abstract_config_manager_set_integer ((ValideAbstractConfigManager*) config, group, key, g_value_get_int (&val));
#line 488 "preferences-dialog.c"
			(btn == NULL) ? NULL : (btn = (g_object_unref (btn), NULL));
#line 63 "preferences-dialog.vala"
			break;
#line 492 "preferences-dialog.c"
		} while (0); else if (_tmp35_ == ((0 != _tmp35__label1) ? _tmp35__label1 : (_tmp35__label1 = g_quark_from_static_string ("GtkFontButton"))))
		do {
			GtkFontButton* btn;
			GtkFontButton* _tmp9_;
			GtkFontButton* _tmp8_;
			GtkWidget* _tmp7_;
			GValue _tmp11_ = {0};
			GValue _tmp10_ = {0};
			btn = NULL;
			_tmp9_ = NULL;
#line 67 "preferences-dialog.vala"
			_tmp8_ = NULL;
#line 67 "preferences-dialog.vala"
			_tmp7_ = NULL;
#line 67 "preferences-dialog.vala"
			btn = (_tmp9_ = (_tmp8_ = (_tmp7_ = sender, GTK_IS_FONT_BUTTON (_tmp7_) ? ((GtkFontButton*) _tmp7_) : NULL), (_tmp8_ == NULL) ? NULL : g_object_ref (_tmp8_)), (btn == NULL) ? NULL : (btn = (g_object_unref (btn), NULL)), _tmp9_);
#line 68 "preferences-dialog.vala"
			val = (_tmp11_ = (g_value_init (&_tmp10_, G_TYPE_STRING), _tmp10_), G_IS_VALUE (&val) ? (g_value_unset (&val), NULL) : NULL, _tmp11_);
#line 69 "preferences-dialog.vala"
			g_value_set_string (&val, gtk_font_button_get_font_name (btn));
#line 70 "preferences-dialog.vala"
			valide_abstract_config_manager_set_string ((ValideAbstractConfigManager*) config, group, key, g_value_get_string (&val));
#line 515 "preferences-dialog.c"
			(btn == NULL) ? NULL : (btn = (g_object_unref (btn), NULL));
#line 71 "preferences-dialog.vala"
			break;
#line 519 "preferences-dialog.c"
		} while (0); else if (_tmp35_ == ((0 != _tmp35__label2) ? _tmp35__label2 : (_tmp35__label2 = g_quark_from_static_string ("GtkCheckButton"))))
		do {
			GtkCheckButton* btn;
			GtkCheckButton* _tmp14_;
			GtkCheckButton* _tmp13_;
			GtkWidget* _tmp12_;
			GValue _tmp16_ = {0};
			GValue _tmp15_ = {0};
			btn = NULL;
			_tmp14_ = NULL;
#line 75 "preferences-dialog.vala"
			_tmp13_ = NULL;
#line 75 "preferences-dialog.vala"
			_tmp12_ = NULL;
#line 75 "preferences-dialog.vala"
			btn = (_tmp14_ = (_tmp13_ = (_tmp12_ = sender, GTK_IS_CHECK_BUTTON (_tmp12_) ? ((GtkCheckButton*) _tmp12_) : NULL), (_tmp13_ == NULL) ? NULL : g_object_ref (_tmp13_)), (btn == NULL) ? NULL : (btn = (g_object_unref (btn), NULL)), _tmp14_);
#line 76 "preferences-dialog.vala"
			val = (_tmp16_ = (g_value_init (&_tmp15_, G_TYPE_BOOLEAN), _tmp15_), G_IS_VALUE (&val) ? (g_value_unset (&val), NULL) : NULL, _tmp16_);
#line 77 "preferences-dialog.vala"
			g_value_set_boolean (&val, gtk_toggle_button_get_active ((GtkToggleButton*) btn));
#line 78 "preferences-dialog.vala"
			valide_abstract_config_manager_set_boolean ((ValideAbstractConfigManager*) config, group, key, g_value_get_boolean (&val));
#line 542 "preferences-dialog.c"
			(btn == NULL) ? NULL : (btn = (g_object_unref (btn), NULL));
#line 79 "preferences-dialog.vala"
			break;
#line 546 "preferences-dialog.c"
		} while (0); else if (_tmp35_ == ((0 != _tmp35__label3) ? _tmp35__label3 : (_tmp35__label3 = g_quark_from_static_string ("GtkFileChooserButton"))))
		do {
			GtkFileChooserButton* btn;
			GtkFileChooserButton* _tmp19_;
			GtkFileChooserButton* _tmp18_;
			GtkWidget* _tmp17_;
			GValue _tmp21_ = {0};
			GValue _tmp20_ = {0};
			char* _tmp22_;
			btn = NULL;
			_tmp19_ = NULL;
#line 83 "preferences-dialog.vala"
			_tmp18_ = NULL;
#line 83 "preferences-dialog.vala"
			_tmp17_ = NULL;
#line 83 "preferences-dialog.vala"
			btn = (_tmp19_ = (_tmp18_ = (_tmp17_ = sender, GTK_IS_FILE_CHOOSER_BUTTON (_tmp17_) ? ((GtkFileChooserButton*) _tmp17_) : NULL), (_tmp18_ == NULL) ? NULL : g_object_ref (_tmp18_)), (btn == NULL) ? NULL : (btn = (g_object_unref (btn), NULL)), _tmp19_);
#line 84 "preferences-dialog.vala"
			val = (_tmp21_ = (g_value_init (&_tmp20_, G_TYPE_STRING), _tmp20_), G_IS_VALUE (&val) ? (g_value_unset (&val), NULL) : NULL, _tmp21_);
#line 5545 "gtk+-2.0.vapi"
			_tmp22_ = NULL;
#line 85 "preferences-dialog.vala"
			g_value_set_string (&val, _tmp22_ = gtk_file_chooser_get_filename ((GtkFileChooser*) btn));
#line 570 "preferences-dialog.c"
			_tmp22_ = (g_free (_tmp22_), NULL);
#line 86 "preferences-dialog.vala"
			valide_abstract_config_manager_set_string ((ValideAbstractConfigManager*) config, group, key, g_value_get_string (&val));
#line 574 "preferences-dialog.c"
			(btn == NULL) ? NULL : (btn = (g_object_unref (btn), NULL));
#line 87 "preferences-dialog.vala"
			break;
#line 578 "preferences-dialog.c"
		} while (0); else if (_tmp35_ == ((0 != _tmp35__label4) ? _tmp35__label4 : (_tmp35__label4 = g_quark_from_static_string ("GtkTreeView"))))
		do {
			char* id;
			GtkTreeView* tree_view;
			GtkTreeModel* tree_model;
			GtkTreeIter iter = {0};
			GtkTreeView* _tmp25_;
			GtkTreeView* _tmp24_;
			GtkWidget* _tmp23_;
			GtkTreeModel* _tmp29_;
			GtkTreeModel* _tmp28_;
			gboolean _tmp27_;
			GtkTreeModel* _tmp26_;
			GValue _tmp31_ = {0};
			GValue _tmp30_ = {0};
			id = NULL;
			tree_view = NULL;
			tree_model = NULL;
			_tmp25_ = NULL;
#line 94 "preferences-dialog.vala"
			_tmp24_ = NULL;
#line 94 "preferences-dialog.vala"
			_tmp23_ = NULL;
#line 94 "preferences-dialog.vala"
			tree_view = (_tmp25_ = (_tmp24_ = (_tmp23_ = sender, GTK_IS_TREE_VIEW (_tmp23_) ? ((GtkTreeView*) _tmp23_) : NULL), (_tmp24_ == NULL) ? NULL : g_object_ref (_tmp24_)), (tree_view == NULL) ? NULL : (tree_view = (g_object_unref (tree_view), NULL)), _tmp25_);
#line 95 "preferences-dialog.vala"
			_tmp29_ = NULL;
#line 606 "preferences-dialog.c"
			_tmp28_ = NULL;
			_tmp26_ = NULL;
#line 95 "preferences-dialog.vala"
			_tmp27_ = gtk_tree_selection_get_selected (gtk_tree_view_get_selection (tree_view), &_tmp26_, &iter);
#line 95 "preferences-dialog.vala"
			tree_model = (_tmp28_ = (_tmp29_ = _tmp26_, (_tmp29_ == NULL) ? NULL : g_object_ref (_tmp29_)), (tree_model == NULL) ? NULL : (tree_model = (g_object_unref (tree_model), NULL)), _tmp28_);
#line 95 "preferences-dialog.vala"
			_tmp27_;
#line 96 "preferences-dialog.vala"
			gtk_tree_model_get (tree_model, &iter, VALIDE_PREFERENCES_DIALOG_COL_ID, &id, -1);
#line 97 "preferences-dialog.vala"
			val = (_tmp31_ = (g_value_init (&_tmp30_, G_TYPE_STRING), _tmp30_), G_IS_VALUE (&val) ? (g_value_unset (&val), NULL) : NULL, _tmp31_);
#line 98 "preferences-dialog.vala"
			g_value_set_string (&val, id);
#line 99 "preferences-dialog.vala"
			valide_abstract_config_manager_set_string ((ValideAbstractConfigManager*) config, group, key, id);
#line 623 "preferences-dialog.c"
			id = (g_free (id), NULL);
			(tree_view == NULL) ? NULL : (tree_view = (g_object_unref (tree_view), NULL));
			(tree_model == NULL) ? NULL : (tree_model = (g_object_unref (tree_model), NULL));
#line 100 "preferences-dialog.vala"
			break;
#line 629 "preferences-dialog.c"
		} while (0); else
		do {
			GValue _tmp33_ = {0};
			GValue _tmp32_ = {0};
#line 102 "preferences-dialog.vala"
			val = (_tmp33_ = (g_value_init (&_tmp32_, G_TYPE_STRING), _tmp32_), G_IS_VALUE (&val) ? (g_value_unset (&val), NULL) : NULL, _tmp33_);
#line 103 "preferences-dialog.vala"
			g_value_set_string (&val, "");
#line 104 "preferences-dialog.vala"
			g_debug ("preferences-dialog.vala:104: Unknow type: %s", g_type_name (G_TYPE_FROM_INSTANCE ((GObject*) sender)));
#line 105 "preferences-dialog.vala"
			break;
#line 642 "preferences-dialog.c"
		} while (0);
		/* Update GtkSourceView display */
#line 109 "preferences-dialog.vala"
		if (_vala_strcmp0 (group, "GtkSourceView") == 0) {
#line 647 "preferences-dialog.c"
			{
				GList* document_collection;
				GList* document_it;
#line 111 "preferences-dialog.vala"
				document_collection = valide_document_manager_get_documents (valide_window_get_documents (self->priv->_window));
#line 653 "preferences-dialog.c"
				for (document_it = document_collection; document_it != NULL; document_it = document_it->next) {
					ValideiDocument* _tmp39_;
					ValideiDocument* document;
#line 92 "document-manager.vala"
					_tmp39_ = NULL;
#line 111 "preferences-dialog.vala"
					document = (_tmp39_ = (ValideiDocument*) document_it->data, (_tmp39_ == NULL) ? NULL : g_object_ref (_tmp39_));
#line 661 "preferences-dialog.c"
					{
#line 113 "preferences-dialog.vala"
						if (VALIDE_IS_ABSTRACT_TEXT_DOCUMENT (document)) {
#line 665 "preferences-dialog.c"
							ValideAbstractTextDocument* atd;
							ValideAbstractTextDocument* _tmp38_;
							ValideAbstractTextDocument* _tmp37_;
							ValideiDocument* _tmp36_;
							atd = NULL;
							_tmp38_ = NULL;
#line 117 "preferences-dialog.vala"
							_tmp37_ = NULL;
#line 117 "preferences-dialog.vala"
							_tmp36_ = NULL;
#line 117 "preferences-dialog.vala"
							atd = (_tmp38_ = (_tmp37_ = (_tmp36_ = document, VALIDE_IS_ABSTRACT_TEXT_DOCUMENT (_tmp36_) ? ((ValideAbstractTextDocument*) _tmp36_) : NULL), (_tmp37_ == NULL) ? NULL : g_object_ref (_tmp37_)), (atd == NULL) ? NULL : (atd = (g_object_unref (atd), NULL)), _tmp38_);
#line 118 "preferences-dialog.vala"
							g_object_set_property ((GObject*) valide_abstract_text_document_get_text_view (atd), key, &val);
#line 680 "preferences-dialog.c"
							(atd == NULL) ? NULL : (atd = (g_object_unref (atd), NULL));
						}
						(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
					}
				}
			}
		}
		G_IS_VALUE (&val) ? (g_value_unset (&val), NULL) : NULL;
		key = (g_free (key), NULL);
		(config == NULL) ? NULL : (config = (g_object_unref (config), NULL));
	}
	goto __finally17;
	__catch17_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 125 "preferences-dialog.vala"
			g_debug (e->message);
#line 701 "preferences-dialog.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally17:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
}


/**
   * Callback for update the editor preferences
   */
#line 133 "preferences-dialog.vala"
void valide_preferences_dialog_update_editor_param (GtkWidget* sender, ValidePreferencesDialog* self) {
#line 133 "preferences-dialog.vala"
	g_return_if_fail (self != NULL);
#line 133 "preferences-dialog.vala"
	g_return_if_fail (sender != NULL);
#line 135 "preferences-dialog.vala"
	valide_preferences_dialog_update_param (self, sender, "GtkSourceView");
#line 725 "preferences-dialog.c"
}


/**
   * Callback for update the font preferences
   */
#line 142 "preferences-dialog.vala"
void valide_preferences_dialog_update_font_param (GtkWidget* sender, ValidePreferencesDialog* self) {
#line 142 "preferences-dialog.vala"
	g_return_if_fail (self != NULL);
#line 142 "preferences-dialog.vala"
	g_return_if_fail (sender != NULL);
#line 144 "preferences-dialog.vala"
	valide_preferences_dialog_update_param (self, sender, "GtkSourceView");
#line 740 "preferences-dialog.c"
}


/**
   * Callback for update the project preferences
   */
#line 151 "preferences-dialog.vala"
void valide_preferences_dialog_update_projects_param (GtkWidget* sender, ValidePreferencesDialog* self) {
#line 151 "preferences-dialog.vala"
	g_return_if_fail (self != NULL);
#line 151 "preferences-dialog.vala"
	g_return_if_fail (sender != NULL);
#line 153 "preferences-dialog.vala"
	valide_preferences_dialog_update_param (self, sender, "Projects");
#line 755 "preferences-dialog.c"
}


#line 156 "preferences-dialog.vala"
static void valide_preferences_dialog_set_param (ValidePreferencesDialog* self, GError** error) {
#line 761 "preferences-dialog.c"
	GError * _inner_error_;
	GKeyFile* key_file;
	gint groups_size;
	gint groups_length1;
	char** groups;
	ValideConfigManager* config;
	ValideConfigManager* _tmp0_;
	GKeyFile* _tmp1_;
	char* _tmp2_;
	char** _tmp4_;
	gint _tmp3_;
#line 156 "preferences-dialog.vala"
	g_return_if_fail (self != NULL);
#line 775 "preferences-dialog.c"
	_inner_error_ = NULL;
	key_file = NULL;
	groups = (groups_length1 = 0, NULL);
	config = NULL;
	_tmp0_ = NULL;
#line 162 "preferences-dialog.vala"
	config = (_tmp0_ = valide_config_manager_get_instance (), (config == NULL) ? NULL : (config = (g_object_unref (config), NULL)), _tmp0_);
#line 783 "preferences-dialog.c"
	_tmp1_ = NULL;
#line 164 "preferences-dialog.vala"
	key_file = (_tmp1_ = g_key_file_new (), (key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL)), _tmp1_);
#line 44 "abstract-config-manager.vala"
	_tmp2_ = NULL;
#line 165 "preferences-dialog.vala"
	g_key_file_load_from_file (key_file, _tmp2_ = valide_abstract_config_manager_get_config_file ((ValideAbstractConfigManager*) config), G_KEY_FILE_NONE, &_inner_error_);
#line 791 "preferences-dialog.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
		groups = (_vala_array_free (groups, groups_length1, (GDestroyNotify) g_free), NULL);
		(config == NULL) ? NULL : (config = (g_object_unref (config), NULL));
		return;
	}
	_tmp2_ = (g_free (_tmp2_), NULL);
	_tmp4_ = NULL;
#line 167 "preferences-dialog.vala"
	groups = (_tmp4_ = g_key_file_get_groups (key_file, &_tmp3_), groups = (_vala_array_free (groups, groups_length1, (GDestroyNotify) g_free), NULL), groups_length1 = _tmp3_, groups_size = groups_length1, _tmp4_);
#line 803 "preferences-dialog.c"
	{
		char** g_collection;
		int g_collection_length1;
		int g_it;
#line 168 "preferences-dialog.vala"
		g_collection = groups;
#line 810 "preferences-dialog.c"
		g_collection_length1 = groups_length1;
		for (g_it = 0; g_it < groups_length1; g_it = g_it + 1) {
			const char* _tmp36_;
			char* g;
#line 159 "preferences-dialog.vala"
			_tmp36_ = NULL;
#line 817 "preferences-dialog.c"
			g = (_tmp36_ = g_collection[g_it], (_tmp36_ == NULL) ? NULL : g_strdup (_tmp36_));
			{
				{
					gint keys_size;
					gint keys_length1;
					char** keys;
					char** _tmp7_;
					gint _tmp6__size;
					gint _tmp6__length1;
					gint _tmp5_;
					char** _tmp6_;
					char** _tmp8_;
					keys = (keys_length1 = 0, NULL);
#line 174 "preferences-dialog.vala"
					_tmp7_ = NULL;
#line 833 "preferences-dialog.c"
					_tmp6_ = (_tmp7_ = g_key_file_get_keys (key_file, g, &_tmp5_, &_inner_error_), _tmp6__length1 = _tmp5_, _tmp6__size = _tmp6__length1, _tmp7_);
					if (_inner_error_ != NULL) {
						keys = (_vala_array_free (keys, keys_length1, (GDestroyNotify) g_free), NULL);
						goto __catch19_g_error;
						goto __finally19;
					}
					_tmp8_ = NULL;
#line 174 "preferences-dialog.vala"
					keys = (_tmp8_ = _tmp6_, keys = (_vala_array_free (keys, keys_length1, (GDestroyNotify) g_free), NULL), keys_length1 = _tmp6__length1, keys_size = keys_length1, _tmp8_);
#line 843 "preferences-dialog.c"
					{
						char** k_collection;
						int k_collection_length1;
						int k_it;
#line 175 "preferences-dialog.vala"
						k_collection = keys;
#line 850 "preferences-dialog.c"
						k_collection_length1 = keys_length1;
						for (k_it = 0; k_it < keys_length1; k_it = k_it + 1) {
							const char* _tmp35_;
							char* k;
#line 172 "preferences-dialog.vala"
							_tmp35_ = NULL;
#line 857 "preferences-dialog.c"
							k = (_tmp35_ = k_collection[k_it], (_tmp35_ == NULL) ? NULL : g_strdup (_tmp35_));
							{
								char* name;
								GObject* object;
								char* _tmp9_;
								GObject* _tmp11_;
								GObject* _tmp10_;
								name = NULL;
								object = NULL;
								_tmp9_ = NULL;
#line 180 "preferences-dialog.vala"
								name = (_tmp9_ = string_replace (k, "-", "_"), name = (g_free (name), NULL), _tmp9_);
#line 870 "preferences-dialog.c"
								_tmp11_ = NULL;
#line 181 "preferences-dialog.vala"
								_tmp10_ = NULL;
#line 181 "preferences-dialog.vala"
								object = (_tmp11_ = (_tmp10_ = gtk_builder_get_object (((ValideAbstractPreferencesDialog*) self)->xml, name), (_tmp10_ == NULL) ? NULL : g_object_ref (_tmp10_)), (object == NULL) ? NULL : (object = (g_object_unref (object), NULL)), _tmp11_);
#line 183 "preferences-dialog.vala"
								if (object != NULL) {
#line 878 "preferences-dialog.c"
									GQuark _tmp34_;
									const char* _tmp33_;
									static GQuark _tmp34__label0 = 0;
									static GQuark _tmp34__label1 = 0;
									static GQuark _tmp34__label2 = 0;
									static GQuark _tmp34__label3 = 0;
									static GQuark _tmp34__label4 = 0;
#line 185 "preferences-dialog.vala"
									_tmp33_ = NULL;
#line 888 "preferences-dialog.c"
									_tmp33_ = g_type_name (G_TYPE_FROM_INSTANCE (object));
									_tmp34_ = (NULL == _tmp33_) ? 0 : g_quark_from_string (_tmp33_);
									if (_tmp34_ == ((0 != _tmp34__label0) ? _tmp34__label0 : (_tmp34__label0 = g_quark_from_static_string ("GtkSpinButton"))))
									do {
										GtkSpinButton* btn;
										GtkSpinButton* _tmp14_;
										GtkSpinButton* _tmp13_;
										GObject* _tmp12_;
										btn = NULL;
										_tmp14_ = NULL;
#line 190 "preferences-dialog.vala"
										_tmp13_ = NULL;
#line 190 "preferences-dialog.vala"
										_tmp12_ = NULL;
#line 190 "preferences-dialog.vala"
										btn = (_tmp14_ = (_tmp13_ = (_tmp12_ = object, GTK_IS_SPIN_BUTTON (_tmp12_) ? ((GtkSpinButton*) _tmp12_) : NULL), (_tmp13_ == NULL) ? NULL : g_object_ref (_tmp13_)), (btn == NULL) ? NULL : (btn = (g_object_unref (btn), NULL)), _tmp14_);
#line 191 "preferences-dialog.vala"
										gtk_spin_button_set_value (btn, (double) valide_abstract_config_manager_get_integer ((ValideAbstractConfigManager*) config, g, k));
#line 907 "preferences-dialog.c"
										(btn == NULL) ? NULL : (btn = (g_object_unref (btn), NULL));
#line 192 "preferences-dialog.vala"
										break;
#line 911 "preferences-dialog.c"
									} while (0); else if (_tmp34_ == ((0 != _tmp34__label1) ? _tmp34__label1 : (_tmp34__label1 = g_quark_from_static_string ("GtkFontButton"))))
									do {
										GtkFontButton* btn;
										GtkFontButton* _tmp17_;
										GtkFontButton* _tmp16_;
										GObject* _tmp15_;
										char* _tmp18_;
										btn = NULL;
										_tmp17_ = NULL;
#line 196 "preferences-dialog.vala"
										_tmp16_ = NULL;
#line 196 "preferences-dialog.vala"
										_tmp15_ = NULL;
#line 196 "preferences-dialog.vala"
										btn = (_tmp17_ = (_tmp16_ = (_tmp15_ = object, GTK_IS_FONT_BUTTON (_tmp15_) ? ((GtkFontButton*) _tmp15_) : NULL), (_tmp16_ == NULL) ? NULL : g_object_ref (_tmp16_)), (btn == NULL) ? NULL : (btn = (g_object_unref (btn), NULL)), _tmp17_);
#line 169 "abstract-config-manager.vala"
										_tmp18_ = NULL;
#line 197 "preferences-dialog.vala"
										gtk_font_button_set_font_name (btn, _tmp18_ = valide_abstract_config_manager_get_string ((ValideAbstractConfigManager*) config, g, k));
#line 931 "preferences-dialog.c"
										_tmp18_ = (g_free (_tmp18_), NULL);
										(btn == NULL) ? NULL : (btn = (g_object_unref (btn), NULL));
#line 198 "preferences-dialog.vala"
										break;
#line 936 "preferences-dialog.c"
									} while (0); else if (_tmp34_ == ((0 != _tmp34__label2) ? _tmp34__label2 : (_tmp34__label2 = g_quark_from_static_string ("GtkCheckButton"))))
									do {
										GtkCheckButton* btn;
										GtkCheckButton* _tmp21_;
										GtkCheckButton* _tmp20_;
										GObject* _tmp19_;
										btn = NULL;
										_tmp21_ = NULL;
#line 202 "preferences-dialog.vala"
										_tmp20_ = NULL;
#line 202 "preferences-dialog.vala"
										_tmp19_ = NULL;
#line 202 "preferences-dialog.vala"
										btn = (_tmp21_ = (_tmp20_ = (_tmp19_ = object, GTK_IS_CHECK_BUTTON (_tmp19_) ? ((GtkCheckButton*) _tmp19_) : NULL), (_tmp20_ == NULL) ? NULL : g_object_ref (_tmp20_)), (btn == NULL) ? NULL : (btn = (g_object_unref (btn), NULL)), _tmp21_);
#line 203 "preferences-dialog.vala"
										gtk_toggle_button_set_active ((GtkToggleButton*) btn, valide_abstract_config_manager_get_boolean ((ValideAbstractConfigManager*) config, g, k));
#line 953 "preferences-dialog.c"
										(btn == NULL) ? NULL : (btn = (g_object_unref (btn), NULL));
#line 204 "preferences-dialog.vala"
										break;
#line 957 "preferences-dialog.c"
									} while (0); else if (_tmp34_ == ((0 != _tmp34__label3) ? _tmp34__label3 : (_tmp34__label3 = g_quark_from_static_string ("GtkFileChooserButton"))))
									do {
										GtkFileChooserButton* btn;
										GtkFileChooserButton* _tmp24_;
										GtkFileChooserButton* _tmp23_;
										GObject* _tmp22_;
										char* _tmp25_;
										btn = NULL;
										_tmp24_ = NULL;
#line 208 "preferences-dialog.vala"
										_tmp23_ = NULL;
#line 208 "preferences-dialog.vala"
										_tmp22_ = NULL;
#line 208 "preferences-dialog.vala"
										btn = (_tmp24_ = (_tmp23_ = (_tmp22_ = object, GTK_IS_FILE_CHOOSER_BUTTON (_tmp22_) ? ((GtkFileChooserButton*) _tmp22_) : NULL), (_tmp23_ == NULL) ? NULL : g_object_ref (_tmp23_)), (btn == NULL) ? NULL : (btn = (g_object_unref (btn), NULL)), _tmp24_);
#line 169 "abstract-config-manager.vala"
										_tmp25_ = NULL;
#line 209 "preferences-dialog.vala"
										gtk_file_chooser_select_filename ((GtkFileChooser*) btn, _tmp25_ = valide_abstract_config_manager_get_string ((ValideAbstractConfigManager*) config, g, k));
#line 977 "preferences-dialog.c"
										_tmp25_ = (g_free (_tmp25_), NULL);
										(btn == NULL) ? NULL : (btn = (g_object_unref (btn), NULL));
#line 210 "preferences-dialog.vala"
										break;
#line 982 "preferences-dialog.c"
									} while (0); else if (_tmp34_ == ((0 != _tmp34__label4) ? _tmp34__label4 : (_tmp34__label4 = g_quark_from_static_string ("GtkTreeView"))))
									do {
										char* id;
										char* select;
										GtkTreeView* tree_view;
										GtkTreeModel* tree_model;
										GtkTreeIter iter = {0};
										char* _tmp26_;
										GtkTreeView* _tmp29_;
										GtkTreeView* _tmp28_;
										GObject* _tmp27_;
										GtkTreeModel* _tmp31_;
										GtkTreeModel* _tmp30_;
										id = NULL;
										select = NULL;
										tree_view = NULL;
										tree_model = NULL;
										_tmp26_ = NULL;
#line 218 "preferences-dialog.vala"
										select = (_tmp26_ = valide_abstract_config_manager_get_string ((ValideAbstractConfigManager*) config, g, k), select = (g_free (select), NULL), _tmp26_);
#line 1003 "preferences-dialog.c"
										_tmp29_ = NULL;
#line 219 "preferences-dialog.vala"
										_tmp28_ = NULL;
#line 219 "preferences-dialog.vala"
										_tmp27_ = NULL;
#line 219 "preferences-dialog.vala"
										tree_view = (_tmp29_ = (_tmp28_ = (_tmp27_ = object, GTK_IS_TREE_VIEW (_tmp27_) ? ((GtkTreeView*) _tmp27_) : NULL), (_tmp28_ == NULL) ? NULL : g_object_ref (_tmp28_)), (tree_view == NULL) ? NULL : (tree_view = (g_object_unref (tree_view), NULL)), _tmp29_);
#line 1011 "preferences-dialog.c"
										_tmp31_ = NULL;
#line 220 "preferences-dialog.vala"
										_tmp30_ = NULL;
#line 220 "preferences-dialog.vala"
										tree_model = (_tmp31_ = (_tmp30_ = gtk_tree_view_get_model (tree_view), (_tmp30_ == NULL) ? NULL : g_object_ref (_tmp30_)), (tree_model == NULL) ? NULL : (tree_model = (g_object_unref (tree_model), NULL)), _tmp31_);
#line 221 "preferences-dialog.vala"
										if (gtk_tree_model_get_iter_first (tree_model, &iter)) {
#line 1019 "preferences-dialog.c"
											{
												gboolean _tmp32_;
												_tmp32_ = TRUE;
#line 223 "preferences-dialog.vala"
												while (TRUE) {
#line 223 "preferences-dialog.vala"
													if (!_tmp32_) {
#line 231 "preferences-dialog.vala"
														if (!gtk_tree_model_iter_next (tree_model, &iter)) {
#line 231 "preferences-dialog.vala"
															break;
#line 1031 "preferences-dialog.c"
														}
													}
#line 223 "preferences-dialog.vala"
													_tmp32_ = FALSE;
#line 225 "preferences-dialog.vala"
													gtk_tree_model_get (tree_model, &iter, VALIDE_PREFERENCES_DIALOG_COL_ID, &id, -1);
#line 226 "preferences-dialog.vala"
													if (_vala_strcmp0 (id, select) == 0) {
#line 228 "preferences-dialog.vala"
														gtk_tree_selection_select_iter (gtk_tree_view_get_selection (tree_view), &iter);
#line 229 "preferences-dialog.vala"
														break;
#line 1044 "preferences-dialog.c"
													}
												}
											}
										}
										id = (g_free (id), NULL);
										select = (g_free (select), NULL);
										(tree_view == NULL) ? NULL : (tree_view = (g_object_unref (tree_view), NULL));
										(tree_model == NULL) ? NULL : (tree_model = (g_object_unref (tree_model), NULL));
#line 233 "preferences-dialog.vala"
										break;
#line 1055 "preferences-dialog.c"
									} while (0); else
									do {
#line 235 "preferences-dialog.vala"
										g_debug ("preferences-dialog.vala:235: Unknow type: %s", g_type_name (G_TYPE_FROM_INSTANCE (object)));
#line 236 "preferences-dialog.vala"
										break;
#line 1062 "preferences-dialog.c"
									} while (0);
								}
								k = (g_free (k), NULL);
								name = (g_free (name), NULL);
								(object == NULL) ? NULL : (object = (g_object_unref (object), NULL));
							}
						}
					}
					keys = (_vala_array_free (keys, keys_length1, (GDestroyNotify) g_free), NULL);
				}
				goto __finally19;
				__catch19_g_error:
				{
					GError * e;
					e = _inner_error_;
					_inner_error_ = NULL;
					{
#line 243 "preferences-dialog.vala"
						g_debug (e->message);
#line 1082 "preferences-dialog.c"
						(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
					}
				}
				__finally19:
				if (_inner_error_ != NULL) {
					g_propagate_error (error, _inner_error_);
					g = (g_free (g), NULL);
					(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
					groups = (_vala_array_free (groups, groups_length1, (GDestroyNotify) g_free), NULL);
					(config == NULL) ? NULL : (config = (g_object_unref (config), NULL));
					return;
				}
				g = (g_free (g), NULL);
			}
		}
	}
	(key_file == NULL) ? NULL : (key_file = (g_key_file_free (key_file), NULL));
	groups = (_vala_array_free (groups, groups_length1, (GDestroyNotify) g_free), NULL);
	(config == NULL) ? NULL : (config = (g_object_unref (config), NULL));
}


#line 248 "preferences-dialog.vala"
static void valide_preferences_dialog_populate_style_scheme (ValidePreferencesDialog* self) {
#line 1107 "preferences-dialog.c"
	char* text;
	GtkTreeIter iter = {0};
	GtkTreeView* tree_view;
	GtkListStore* list_store;
	GtkTreeViewColumn* col;
	GtkCellRenderer* renderer;
	GtkSourceStyleScheme* scheme;
	GtkSourceStyleSchemeManager* manager;
	GtkListStore* _tmp0_;
	GtkTreeView* _tmp2_;
	GtkTreeView* _tmp1_;
	GtkTreeViewColumn* _tmp3_;
	GtkCellRenderer* _tmp4_;
	GtkSourceStyleSchemeManager* _tmp5_;
#line 248 "preferences-dialog.vala"
	g_return_if_fail (self != NULL);
#line 1124 "preferences-dialog.c"
	text = NULL;
	tree_view = NULL;
	list_store = NULL;
	col = NULL;
	renderer = NULL;
	scheme = NULL;
	manager = NULL;
	_tmp0_ = NULL;
#line 259 "preferences-dialog.vala"
	list_store = (_tmp0_ = gtk_list_store_new ((gint) VALIDE_PREFERENCES_DIALOG_COL_NB, G_TYPE_STRING, G_TYPE_STRING, NULL), (list_store == NULL) ? NULL : (list_store = (g_object_unref (list_store), NULL)), _tmp0_);
#line 260 "preferences-dialog.vala"
	gtk_tree_sortable_set_sort_column_id ((GtkTreeSortable*) list_store, (gint) VALIDE_PREFERENCES_DIALOG_COL_ID, GTK_SORT_ASCENDING);
#line 1137 "preferences-dialog.c"
	_tmp2_ = NULL;
#line 262 "preferences-dialog.vala"
	_tmp1_ = NULL;
#line 262 "preferences-dialog.vala"
	tree_view = (_tmp2_ = (_tmp1_ = ((ValideAbstractPreferencesDialog*) self)->widgets->style_scheme, (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (tree_view == NULL) ? NULL : (tree_view = (g_object_unref (tree_view), NULL)), _tmp2_);
#line 263 "preferences-dialog.vala"
	gtk_tree_view_set_model (tree_view, (GtkTreeModel*) list_store);
#line 264 "preferences-dialog.vala"
	gtk_tree_view_set_headers_visible (tree_view, FALSE);
#line 1147 "preferences-dialog.c"
	_tmp3_ = NULL;
#line 266 "preferences-dialog.vala"
	col = (_tmp3_ = g_object_ref_sink (gtk_tree_view_column_new ()), (col == NULL) ? NULL : (col = (g_object_unref (col), NULL)), _tmp3_);
#line 267 "preferences-dialog.vala"
	gtk_tree_view_column_set_title (col, _ ("Style"));
#line 1153 "preferences-dialog.c"
	_tmp4_ = NULL;
#line 269 "preferences-dialog.vala"
	renderer = (_tmp4_ = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL)), _tmp4_);
#line 270 "preferences-dialog.vala"
	gtk_cell_layout_pack_start ((GtkCellLayout*) col, renderer, FALSE);
#line 271 "preferences-dialog.vala"
	gtk_tree_view_column_set_attributes (col, renderer, "markup", VALIDE_PREFERENCES_DIALOG_COL_TEXT, NULL);
#line 273 "preferences-dialog.vala"
	gtk_tree_view_append_column (tree_view, col);
#line 1163 "preferences-dialog.c"
	_tmp5_ = NULL;
#line 275 "preferences-dialog.vala"
	manager = (_tmp5_ = gtk_source_style_scheme_manager_new (), (manager == NULL) ? NULL : (manager = (g_object_unref (manager), NULL)), _tmp5_);
#line 1167 "preferences-dialog.c"
	{
		char** _tmp6_;
		char** id_collection;
		int id_collection_length1;
		int id_it;
		_tmp6_ = NULL;
#line 276 "preferences-dialog.vala"
		id_collection = _tmp6_ = gtk_source_style_scheme_manager_get_scheme_ids (manager);
#line 1176 "preferences-dialog.c"
		id_collection_length1 = _vala_array_length (_tmp6_);
		for (id_it = 0; id_it < _vala_array_length (_tmp6_); id_it = id_it + 1) {
			const char* _tmp10_;
			char* id;
#line 191 "gtksourceview-2.0.vapi"
			_tmp10_ = NULL;
#line 1183 "preferences-dialog.c"
			id = (_tmp10_ = id_collection[id_it], (_tmp10_ == NULL) ? NULL : g_strdup (_tmp10_));
			{
				GtkSourceStyleScheme* _tmp8_;
				GtkSourceStyleScheme* _tmp7_;
				char* _tmp9_;
				_tmp8_ = NULL;
#line 278 "preferences-dialog.vala"
				_tmp7_ = NULL;
#line 278 "preferences-dialog.vala"
				scheme = (_tmp8_ = (_tmp7_ = gtk_source_style_scheme_manager_get_scheme (manager, id), (_tmp7_ == NULL) ? NULL : g_object_ref (_tmp7_)), (scheme == NULL) ? NULL : (scheme = (g_object_unref (scheme), NULL)), _tmp8_);
#line 1194 "preferences-dialog.c"
				_tmp9_ = NULL;
#line 279 "preferences-dialog.vala"
				text = (_tmp9_ = g_strdup_printf ("<b>%s</b> - %s", gtk_source_style_scheme_get_name (scheme), gtk_source_style_scheme_get_description (scheme)), text = (g_free (text), NULL), _tmp9_);
#line 280 "preferences-dialog.vala"
				gtk_list_store_append (list_store, &iter);
#line 281 "preferences-dialog.vala"
				gtk_list_store_set (list_store, &iter, VALIDE_PREFERENCES_DIALOG_COL_ID, id, VALIDE_PREFERENCES_DIALOG_COL_TEXT, text, -1);
#line 1202 "preferences-dialog.c"
				id = (g_free (id), NULL);
			}
		}
	}
	text = (g_free (text), NULL);
	(tree_view == NULL) ? NULL : (tree_view = (g_object_unref (tree_view), NULL));
	(list_store == NULL) ? NULL : (list_store = (g_object_unref (list_store), NULL));
	(col == NULL) ? NULL : (col = (g_object_unref (col), NULL));
	(renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL));
	(scheme == NULL) ? NULL : (scheme = (g_object_unref (scheme), NULL));
	(manager == NULL) ? NULL : (manager = (g_object_unref (manager), NULL));
}


/**
   * Create a new Valide.PreferenceDialog
   *
   * @param window The main window of the application
   */
#line 290 "preferences-dialog.vala"
ValidePreferencesDialog* valide_preferences_dialog_construct (GType object_type, ValideWindow* window) {
#line 1224 "preferences-dialog.c"
	GParameter * __params;
	GParameter * __params_it;
	ValidePreferencesDialog * self;
#line 290 "preferences-dialog.vala"
	g_return_val_if_fail (window != NULL, NULL);
#line 1230 "preferences-dialog.c"
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
#line 292 "preferences-dialog.vala"
	__params_it->name = "window";
#line 292 "preferences-dialog.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_WINDOW);
#line 292 "preferences-dialog.vala"
	g_value_set_object (&__params_it->value, window);
#line 292 "preferences-dialog.vala"
	__params_it++;
#line 1241 "preferences-dialog.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 290 "preferences-dialog.vala"
ValidePreferencesDialog* valide_preferences_dialog_new (ValideWindow* window) {
#line 290 "preferences-dialog.vala"
	return valide_preferences_dialog_construct (VALIDE_TYPE_PREFERENCES_DIALOG, window);
#line 1256 "preferences-dialog.c"
}


/**
   * @see Gtk.Dialog.run
   */
#line 336 "preferences-dialog.vala"
gint valide_preferences_dialog_run (ValidePreferencesDialog* self) {
#line 1265 "preferences-dialog.c"
	gint result;
#line 336 "preferences-dialog.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 338 "preferences-dialog.vala"
	gtk_widget_show_all ((GtkWidget*) ((ValideAbstractPreferencesDialog*) self)->widgets->dialog);
#line 1271 "preferences-dialog.c"
	result = gtk_dialog_run (((ValideAbstractPreferencesDialog*) self)->widgets->dialog);
	return result;
}


/**
   * @see Gtk.Object.destroy
   */
#line 345 "preferences-dialog.vala"
void valide_preferences_dialog_destroy (ValidePreferencesDialog* self) {
#line 345 "preferences-dialog.vala"
	g_return_if_fail (self != NULL);
#line 347 "preferences-dialog.vala"
	gtk_object_destroy ((GtkObject*) ((ValideAbstractPreferencesDialog*) self)->widgets->dialog);
#line 1286 "preferences-dialog.c"
}


ValideWindow* valide_preferences_dialog_get_window (ValidePreferencesDialog* self) {
	ValideWindow* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_window;
	return result;
}


static void valide_preferences_dialog_set_window (ValidePreferencesDialog* self, ValideWindow* value) {
	ValideWindow* _tmp1_;
	ValideWindow* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 41 "preferences-dialog.vala"
	_tmp0_ = NULL;
#line 1305 "preferences-dialog.c"
	self->priv->_window = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "window");
}


static void _lambda48_ (GtkToggleButton* s, ValidePreferencesDialog* self) {
	gboolean active;
	g_return_if_fail (s != NULL);
	active = FALSE;
#line 310 "preferences-dialog.vala"
	active = gtk_toggle_button_get_active (s);
#line 311 "preferences-dialog.vala"
	g_object_set ((GtkWidget*) ((ValideAbstractPreferencesDialog*) self)->widgets->label7, "sensitive", active, NULL);
#line 312 "preferences-dialog.vala"
	g_object_set ((GtkWidget*) ((ValideAbstractPreferencesDialog*) self)->widgets->right_margin_position, "sensitive", active, NULL);
#line 1321 "preferences-dialog.c"
}


static void __lambda48__gtk_toggle_button_toggled (GtkToggleButton* _sender, gpointer self) {
	_lambda48_ (_sender, self);
}


static GObject * valide_preferences_dialog_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValidePreferencesDialogClass * klass;
	GObjectClass * parent_class;
	ValidePreferencesDialog * self;
	GError * _inner_error_;
	klass = VALIDE_PREFERENCES_DIALOG_CLASS (g_type_class_peek (VALIDE_TYPE_PREFERENCES_DIALOG));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_PREFERENCES_DIALOG (obj);
	_inner_error_ = NULL;
	{
		GtkAdjustment* adj;
		GtkAdjustment* _tmp0_;
		GtkAdjustment* _tmp1_;
		GtkLabel* _tmp3_;
		GtkWidget* _tmp2_;
		adj = NULL;
#line 299 "preferences-dialog.vala"
		gtk_window_set_transient_for ((GtkWindow*) ((ValideAbstractPreferencesDialog*) self)->widgets->dialog, (GtkWindow*) self->priv->_window);
#line 1350 "preferences-dialog.c"
		_tmp0_ = NULL;
#line 301 "preferences-dialog.vala"
		adj = (_tmp0_ = g_object_ref_sink ((GtkAdjustment*) gtk_adjustment_new (0.0, 0.0, 100.0, 1.0, 10.0, 0.0)), (adj == NULL) ? NULL : (adj = (g_object_unref (adj), NULL)), _tmp0_);
#line 302 "preferences-dialog.vala"
		gtk_spin_button_set_adjustment (((ValideAbstractPreferencesDialog*) self)->widgets->tab_width, adj);
#line 1356 "preferences-dialog.c"
		_tmp1_ = NULL;
#line 304 "preferences-dialog.vala"
		adj = (_tmp1_ = g_object_ref_sink ((GtkAdjustment*) gtk_adjustment_new (0.0, 0.0, 100.0, 1.0, 10.0, 0.0)), (adj == NULL) ? NULL : (adj = (g_object_unref (adj), NULL)), _tmp1_);
#line 305 "preferences-dialog.vala"
		gtk_spin_button_set_adjustment (((ValideAbstractPreferencesDialog*) self)->widgets->right_margin_position, adj);
#line 307 "preferences-dialog.vala"
		g_signal_connect ((GtkToggleButton*) ((ValideAbstractPreferencesDialog*) self)->widgets->show_right_margin, "toggled", (GCallback) __lambda48__gtk_toggle_button_toggled, self);
#line 315 "preferences-dialog.vala"
		valide_preferences_dialog_populate_style_scheme (self);
#line 1366 "preferences-dialog.c"
		{
#line 319 "preferences-dialog.vala"
			valide_preferences_dialog_set_param (self, &_inner_error_);
#line 1370 "preferences-dialog.c"
			if (_inner_error_ != NULL) {
				goto __catch20_g_error;
				goto __finally20;
			}
#line 320 "preferences-dialog.vala"
			gtk_toggle_button_toggled ((GtkToggleButton*) ((ValideAbstractPreferencesDialog*) self)->widgets->show_right_margin);
#line 1377 "preferences-dialog.c"
		}
		goto __finally20;
		__catch20_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
#line 324 "preferences-dialog.vala"
				g_debug (e->message);
#line 1388 "preferences-dialog.c"
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally20:
		if (_inner_error_ != NULL) {
			(adj == NULL) ? NULL : (adj = (g_object_unref (adj), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
		}
#line 327 "preferences-dialog.vala"
		valide_abstract_preferences_dialog_connect_signals ((ValideAbstractPreferencesDialog*) self, "valide_preferences_dialog_");
#line 1400 "preferences-dialog.c"
		_tmp3_ = NULL;
#line 129 "plugin-manager.vala"
		_tmp2_ = NULL;
#line 329 "preferences-dialog.vala"
		gtk_notebook_append_page (((ValideAbstractPreferencesDialog*) self)->widgets->notebook, _tmp2_ = valide_plugin_manager_preferences (valide_window_get_plugins (self->priv->_window)), (GtkWidget*) (_tmp3_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("Plugins")))));
#line 1406 "preferences-dialog.c"
		(_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL));
		(_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL));
		(adj == NULL) ? NULL : (adj = (g_object_unref (adj), NULL));
	}
	return obj;
}


static void valide_preferences_dialog_class_init (ValidePreferencesDialogClass * klass) {
	valide_preferences_dialog_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValidePreferencesDialogPrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_preferences_dialog_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_preferences_dialog_set_property;
	G_OBJECT_CLASS (klass)->constructor = valide_preferences_dialog_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_preferences_dialog_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PREFERENCES_DIALOG_WINDOW, g_param_spec_object ("window", "window", "window", VALIDE_TYPE_WINDOW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void valide_preferences_dialog_instance_init (ValidePreferencesDialog * self) {
	self->priv = VALIDE_PREFERENCES_DIALOG_GET_PRIVATE (self);
}


static void valide_preferences_dialog_finalize (GObject* obj) {
	ValidePreferencesDialog * self;
	self = VALIDE_PREFERENCES_DIALOG (obj);
	(self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL));
	G_OBJECT_CLASS (valide_preferences_dialog_parent_class)->finalize (obj);
}


GType valide_preferences_dialog_get_type (void) {
	static GType valide_preferences_dialog_type_id = 0;
	if (valide_preferences_dialog_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValidePreferencesDialogClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_preferences_dialog_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValidePreferencesDialog), 0, (GInstanceInitFunc) valide_preferences_dialog_instance_init, NULL };
		valide_preferences_dialog_type_id = g_type_register_static (VALIDE_TYPE_ABSTRACT_PREFERENCES_DIALOG, "ValidePreferencesDialog", &g_define_type_info, 0);
	}
	return valide_preferences_dialog_type_id;
}


static void valide_preferences_dialog_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValidePreferencesDialog * self;
	gpointer boxed;
	self = VALIDE_PREFERENCES_DIALOG (object);
	switch (property_id) {
		case VALIDE_PREFERENCES_DIALOG_WINDOW:
		g_value_set_object (value, valide_preferences_dialog_get_window (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_preferences_dialog_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValidePreferencesDialog * self;
	self = VALIDE_PREFERENCES_DIALOG (object);
	switch (property_id) {
		case VALIDE_PREFERENCES_DIALOG_WINDOW:
		valide_preferences_dialog_set_window (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




