/* project-dialog.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <gio/gio.h>
#include <config.h>
#include <gdk/gdk.h>


#define VALIDE_TYPE_COLUMN_TYPE (valide_column_type_get_type ())

#define VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG (valide_abstract_project_dialog_get_type ())
#define VALIDE_ABSTRACT_PROJECT_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG, ValideAbstractProjectDialog))
#define VALIDE_ABSTRACT_PROJECT_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG, ValideAbstractProjectDialogClass))
#define VALIDE_IS_ABSTRACT_PROJECT_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG))
#define VALIDE_IS_ABSTRACT_PROJECT_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG))
#define VALIDE_ABSTRACT_PROJECT_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG, ValideAbstractProjectDialogClass))

typedef struct _ValideAbstractProjectDialog ValideAbstractProjectDialog;
typedef struct _ValideAbstractProjectDialogClass ValideAbstractProjectDialogClass;
typedef struct _ValideAbstractProjectDialogPrivate ValideAbstractProjectDialogPrivate;

#define VALIDE_ABSTRACT_PROJECT_DIALOG_TYPE_WINDOW_WIDGETS (valide_abstract_project_dialog_window_widgets_get_type ())
#define VALIDE_ABSTRACT_PROJECT_DIALOG_WINDOW_WIDGETS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_ABSTRACT_PROJECT_DIALOG_TYPE_WINDOW_WIDGETS, ValideAbstractProjectDialogwindow_widgets))
#define VALIDE_ABSTRACT_PROJECT_DIALOG_WINDOW_WIDGETS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_ABSTRACT_PROJECT_DIALOG_TYPE_WINDOW_WIDGETS, ValideAbstractProjectDialogwindow_widgetsClass))
#define VALIDE_ABSTRACT_PROJECT_DIALOG_IS_WINDOW_WIDGETS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_ABSTRACT_PROJECT_DIALOG_TYPE_WINDOW_WIDGETS))
#define VALIDE_ABSTRACT_PROJECT_DIALOG_IS_WINDOW_WIDGETS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_ABSTRACT_PROJECT_DIALOG_TYPE_WINDOW_WIDGETS))
#define VALIDE_ABSTRACT_PROJECT_DIALOG_WINDOW_WIDGETS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_ABSTRACT_PROJECT_DIALOG_TYPE_WINDOW_WIDGETS, ValideAbstractProjectDialogwindow_widgetsClass))

typedef struct _ValideAbstractProjectDialogwindow_widgets ValideAbstractProjectDialogwindow_widgets;
typedef struct _ValideAbstractProjectDialogwindow_widgetsClass ValideAbstractProjectDialogwindow_widgetsClass;

#define VALIDE_TYPE_PROJECT_DIALOG (valide_project_dialog_get_type ())
#define VALIDE_PROJECT_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PROJECT_DIALOG, ValideProjectDialog))
#define VALIDE_PROJECT_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PROJECT_DIALOG, ValideProjectDialogClass))
#define VALIDE_IS_PROJECT_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PROJECT_DIALOG))
#define VALIDE_IS_PROJECT_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PROJECT_DIALOG))
#define VALIDE_PROJECT_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PROJECT_DIALOG, ValideProjectDialogClass))

typedef struct _ValideProjectDialog ValideProjectDialog;
typedef struct _ValideProjectDialogClass ValideProjectDialogClass;
typedef struct _ValideProjectDialogPrivate ValideProjectDialogPrivate;

#define VALIDE_TYPE_TEMPLATE_MANAGER (valide_template_manager_get_type ())
#define VALIDE_TEMPLATE_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_TEMPLATE_MANAGER, ValideTemplateManager))
#define VALIDE_TEMPLATE_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_TEMPLATE_MANAGER, ValideTemplateManagerClass))
#define VALIDE_IS_TEMPLATE_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_TEMPLATE_MANAGER))
#define VALIDE_IS_TEMPLATE_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_TEMPLATE_MANAGER))
#define VALIDE_TEMPLATE_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_TEMPLATE_MANAGER, ValideTemplateManagerClass))

typedef struct _ValideTemplateManager ValideTemplateManager;
typedef struct _ValideTemplateManagerClass ValideTemplateManagerClass;
typedef struct _ValideAbstractProjectDialogwindow_widgetsPrivate ValideAbstractProjectDialogwindow_widgetsPrivate;

#define VALIDE_TYPE_TEMPLATE (valide_template_get_type ())
#define VALIDE_TEMPLATE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_TEMPLATE, ValideTemplate))
#define VALIDE_TEMPLATE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_TEMPLATE, ValideTemplateClass))
#define VALIDE_IS_TEMPLATE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_TEMPLATE))
#define VALIDE_IS_TEMPLATE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_TEMPLATE))
#define VALIDE_TEMPLATE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_TEMPLATE, ValideTemplateClass))

typedef struct _ValideTemplate ValideTemplate;
typedef struct _ValideTemplateClass ValideTemplateClass;

#define VALIDE_TYPE_BUILDER_MANAGER (valide_builder_manager_get_type ())
#define VALIDE_BUILDER_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_BUILDER_MANAGER, ValideBuilderManager))
#define VALIDE_BUILDER_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_BUILDER_MANAGER, ValideBuilderManagerClass))
#define VALIDE_IS_BUILDER_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_BUILDER_MANAGER))
#define VALIDE_IS_BUILDER_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_BUILDER_MANAGER))
#define VALIDE_BUILDER_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_BUILDER_MANAGER, ValideBuilderManagerClass))

typedef struct _ValideBuilderManager ValideBuilderManager;
typedef struct _ValideBuilderManagerClass ValideBuilderManagerClass;

#define VALIDE_PROJECT_DIALOG_TYPE_RUN_INFO (valide_project_dialog_run_info_get_type ())
typedef struct _ValideProjectDialogRunInfo ValideProjectDialogRunInfo;

#define VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER (valide_abstract_config_manager_get_type ())
#define VALIDE_ABSTRACT_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManager))
#define VALIDE_ABSTRACT_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManagerClass))
#define VALIDE_IS_ABSTRACT_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER))
#define VALIDE_IS_ABSTRACT_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER))
#define VALIDE_ABSTRACT_CONFIG_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManagerClass))

typedef struct _ValideAbstractConfigManager ValideAbstractConfigManager;
typedef struct _ValideAbstractConfigManagerClass ValideAbstractConfigManagerClass;

#define VALIDE_TYPE_CONFIG_MANAGER (valide_config_manager_get_type ())
#define VALIDE_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_CONFIG_MANAGER, ValideConfigManager))
#define VALIDE_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_CONFIG_MANAGER, ValideConfigManagerClass))
#define VALIDE_IS_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_CONFIG_MANAGER))
#define VALIDE_IS_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_CONFIG_MANAGER))
#define VALIDE_CONFIG_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_CONFIG_MANAGER, ValideConfigManagerClass))

typedef struct _ValideConfigManager ValideConfigManager;
typedef struct _ValideConfigManagerClass ValideConfigManagerClass;

typedef enum  {
	VALIDE_COLUMN_TYPE_ICON,
	VALIDE_COLUMN_TYPE_NAME,
	VALIDE_COLUMN_TYPE_DESCRIPTION,
	VALIDE_COLUMN_TYPE_PATH,
	VALIDE_COLUMN_TYPE_NB_COLUMNS
} ValideColumnType;

struct _ValideAbstractProjectDialog {
	GtkObject parent_instance;
	ValideAbstractProjectDialogPrivate * priv;
	char* name_space;
	GtkBuilder* xml;
	ValideAbstractProjectDialogwindow_widgets* widgets;
};

struct _ValideAbstractProjectDialogClass {
	GtkObjectClass parent_class;
};

/**
 * A dialog box widget for create a new project
 */
struct _ValideProjectDialog {
	ValideAbstractProjectDialog parent_instance;
	ValideProjectDialogPrivate * priv;
};

struct _ValideProjectDialogClass {
	ValideAbstractProjectDialogClass parent_class;
};

struct _ValideProjectDialogPrivate {
	GtkListStore* list_store;
	GtkListStore* licenses_list;
	char* _license;
	char* _project_dir;
	char* _project_path;
	char* _lang;
	char* _builder;
	char* _template;
};

struct _ValideAbstractProjectDialogwindow_widgets {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValideAbstractProjectDialogwindow_widgetsPrivate * priv;
	GtkAssistant* assistant;
	GtkLabel* page1;
	GtkVBox* page2;
	GtkScrolledWindow* scrolledwindow1;
	GtkIconView* icon_view;
	GtkHBox* hbox1;
	GtkLabel* label1;
	GtkComboBox* lang_combo;
	GtkCellRendererText* renderer1;
	GtkFrame* frame1;
	GtkAlignment* alignment1;
	GtkLabel* desc_label;
	GtkLabel* label4;
	GtkScrolledWindow* page3;
	GtkViewport* viewport1;
	GtkTable* table1;
	GtkLabel* label2;
	GtkLabel* label3;
	GtkLabel* label6;
	GtkEntry* name_entry;
	GtkEntry* path_entry;
	GtkLabel* label7;
	GtkLabel* label8;
	GtkLabel* label9;
	GtkEntry* author_entry;
	GtkEntry* version_entry;
	GtkComboBox* licenses_combo;
	GtkFileChooserButton* dir_chooser;
	GtkLabel* label5;
	GtkComboBox* builder;
	GtkListStore* model1;
	GtkListStore* liststore1;
};

struct _ValideAbstractProjectDialogwindow_widgetsClass {
	GTypeClass parent_class;
	void (*finalize) (ValideAbstractProjectDialogwindow_widgets *self);
};

/* Rewriting of the Gtk.Dialog.run function */
struct _ValideProjectDialogRunInfo {
	GMainLoop* loop;
	GtkResponseType response_id;
	gboolean destroyed;
};


static gpointer valide_project_dialog_parent_class = NULL;

GType valide_column_type_get_type (void);
GType valide_abstract_project_dialog_get_type (void);
gpointer valide_abstract_project_dialog_window_widgets_ref (gpointer instance);
void valide_abstract_project_dialog_window_widgets_unref (gpointer instance);
GParamSpec* valide_abstract_project_dialog_param_spec_window_widgets (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void valide_abstract_project_dialog_value_set_window_widgets (GValue* value, gpointer v_object);
gpointer valide_abstract_project_dialog_value_get_window_widgets (const GValue* value);
GType valide_abstract_project_dialog_window_widgets_get_type (void);
GType valide_project_dialog_get_type (void);
#define VALIDE_PROJECT_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_PROJECT_DIALOG, ValideProjectDialogPrivate))
enum  {
	VALIDE_PROJECT_DIALOG_DUMMY_PROPERTY,
	VALIDE_PROJECT_DIALOG_PROJECT_NAME,
	VALIDE_PROJECT_DIALOG_PROJECT_AUTHOR,
	VALIDE_PROJECT_DIALOG_PROJECT_VERSION,
	VALIDE_PROJECT_DIALOG_PROJECT_LICENSE,
	VALIDE_PROJECT_DIALOG_PROJECT_DIR,
	VALIDE_PROJECT_DIALOG_PROJECT_PATH,
	VALIDE_PROJECT_DIALOG_LANG,
	VALIDE_PROJECT_DIALOG_BUILDER,
	VALIDE_PROJECT_DIALOG_TEMPLATE,
	VALIDE_PROJECT_DIALOG_PROJECT_DESC
};
GType valide_template_manager_get_type (void);
ValideTemplateManager* valide_template_manager_get_instance (void);
GList* valide_template_manager_get_list (ValideTemplateManager* self);
static void _g_list_free_g_free (GList* self);
GType valide_template_get_type (void);
ValideTemplate* _valide_template_new (const char* filename, GError** error);
const char* valide_template_get_path (ValideTemplate* self);
const char* valide_template_get_icon (ValideTemplate* self);
const char* valide_template_get_name (ValideTemplate* self);
const char* valide_template_get_description (ValideTemplate* self);
static void valide_project_dialog_populate_icon_view (ValideProjectDialog* self);
static void valide_project_dialog_populate_license (ValideProjectDialog* self);
GType valide_builder_manager_get_type (void);
ValideBuilderManager* valide_builder_manager_get_instance (void);
GList* valide_builder_manager_get_builders (ValideBuilderManager* self);
static void valide_project_dialog_populate_builder (ValideProjectDialog* self);
static void valide_project_dialog_set_template (ValideProjectDialog* self, const char* value);
void valide_project_dialog_set_project_desc (ValideProjectDialog* self, const char* value);
void valide_project_dialog_selection_changed (GtkIconView* icon_view, ValideProjectDialog* self);
const char* valide_project_dialog_get_project_dir (ValideProjectDialog* self);
const char* valide_project_dialog_get_project_name (ValideProjectDialog* self);
void valide_project_dialog_project_path_change (GtkWidget* sender, ValideProjectDialog* self);
void valide_project_dialog_show (ValideProjectDialog* self);
static GType valide_project_dialog_run_info_get_type (void);
static ValideProjectDialogRunInfo* valide_project_dialog_run_info_dup (const ValideProjectDialogRunInfo* self);
static void valide_project_dialog_run_info_free (ValideProjectDialogRunInfo* self);
static void valide_project_dialog_run_info_copy (const ValideProjectDialogRunInfo* self, ValideProjectDialogRunInfo* dest);
static void valide_project_dialog_run_info_destroy (ValideProjectDialogRunInfo* self);
static void valide_project_dialog_shutdown_loop (const ValideProjectDialogRunInfo* ri);
static void valide_project_dialog_apply_cb (GtkWidget* sender, ValideProjectDialogRunInfo* ri);
static void valide_project_dialog_cancel_cb (GtkWidget* sender, ValideProjectDialogRunInfo* ri);
static void valide_project_dialog_close_cb (GtkWidget* sender, ValideProjectDialogRunInfo* ri);
static void valide_project_dialog_unmap_cb (GtkWidget* sender, ValideProjectDialogRunInfo* ri);
static gboolean valide_project_dialog_delete_cb (GtkWidget* sender, ValideProjectDialogRunInfo* ri);
static void valide_project_dialog_destroy_cb (GtkWidget* sender, ValideProjectDialogRunInfo* ri);
guint valide_project_dialog_run (ValideProjectDialog* self);
ValideProjectDialog* valide_project_dialog_new (void);
ValideProjectDialog* valide_project_dialog_construct (GType object_type);
const char* valide_project_dialog_get_project_author (ValideProjectDialog* self);
const char* valide_project_dialog_get_project_version (ValideProjectDialog* self);
const char* valide_project_dialog_get_project_license (ValideProjectDialog* self);
void valide_project_dialog_set_project_dir (ValideProjectDialog* self, const char* value);
const char* valide_project_dialog_get_project_path (ValideProjectDialog* self);
const char* valide_project_dialog_get_lang (ValideProjectDialog* self);
const char* valide_project_dialog_get_builder (ValideProjectDialog* self);
const char* valide_project_dialog_get_template (ValideProjectDialog* self);
void valide_abstract_project_dialog_connect_signals (ValideAbstractProjectDialog* self, const char* name_space);
static void _lambda32_ (ValideProjectDialog* self);
static void __lambda32__gtk_assistant_apply (GtkAssistant* _sender, gpointer self);
static void _lambda33_ (ValideProjectDialog* self);
static void __lambda33__gtk_assistant_cancel (GtkAssistant* _sender, gpointer self);
static void _lambda34_ (ValideProjectDialog* self);
static void __lambda34__gtk_assistant_close (GtkAssistant* _sender, gpointer self);
GType valide_abstract_config_manager_get_type (void);
GType valide_config_manager_get_type (void);
ValideConfigManager* valide_config_manager_get_instance (void);
char* valide_abstract_config_manager_get_string (ValideAbstractConfigManager* self, const char* group, const char* key);
static GObject * valide_project_dialog_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void valide_project_dialog_finalize (GObject* obj);
static void valide_project_dialog_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_project_dialog_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static int _vala_strcmp0 (const char * str1, const char * str2);




GType valide_column_type_get_type (void) {
	static GType valide_column_type_type_id = 0;
	if (G_UNLIKELY (valide_column_type_type_id == 0)) {
		static const GEnumValue values[] = {{VALIDE_COLUMN_TYPE_ICON, "VALIDE_COLUMN_TYPE_ICON", "icon"}, {VALIDE_COLUMN_TYPE_NAME, "VALIDE_COLUMN_TYPE_NAME", "name"}, {VALIDE_COLUMN_TYPE_DESCRIPTION, "VALIDE_COLUMN_TYPE_DESCRIPTION", "description"}, {VALIDE_COLUMN_TYPE_PATH, "VALIDE_COLUMN_TYPE_PATH", "path"}, {VALIDE_COLUMN_TYPE_NB_COLUMNS, "VALIDE_COLUMN_TYPE_NB_COLUMNS", "nb-columns"}, {0, NULL, NULL}};
		valide_column_type_type_id = g_enum_register_static ("ValideColumnType", values);
	}
	return valide_column_type_type_id;
}


static void _g_list_free_g_free (GList* self) {
	g_list_foreach (self, (GFunc) g_free, NULL);
	g_list_free (self);
}


#line 191 "project-dialog.vala"
static void valide_project_dialog_populate_icon_view (ValideProjectDialog* self) {
#line 325 "project-dialog.c"
	GError * _inner_error_;
	GList* files;
	GList* _tmp1_;
	ValideTemplateManager* _tmp0_;
	GtkListStore* _tmp2_;
#line 191 "project-dialog.vala"
	g_return_if_fail (self != NULL);
#line 333 "project-dialog.c"
	_inner_error_ = NULL;
	files = NULL;
	_tmp1_ = NULL;
#line 38 "template-manager.vala"
	_tmp0_ = NULL;
#line 195 "project-dialog.vala"
	files = (_tmp1_ = valide_template_manager_get_list (_tmp0_ = valide_template_manager_get_instance ()), (files == NULL) ? NULL : (files = (_g_list_free_g_free (files), NULL)), _tmp1_);
#line 341 "project-dialog.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
	_tmp2_ = NULL;
#line 196 "project-dialog.vala"
	self->priv->list_store = (_tmp2_ = gtk_list_store_new ((gint) VALIDE_COLUMN_TYPE_NB_COLUMNS, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, NULL), (self->priv->list_store == NULL) ? NULL : (self->priv->list_store = (g_object_unref (self->priv->list_store), NULL)), _tmp2_);
#line 201 "project-dialog.vala"
	gtk_icon_view_set_model (((ValideAbstractProjectDialog*) self)->widgets->icon_view, (GtkTreeModel*) self->priv->list_store);
#line 202 "project-dialog.vala"
	gtk_icon_view_set_pixbuf_column (((ValideAbstractProjectDialog*) self)->widgets->icon_view, (gint) VALIDE_COLUMN_TYPE_ICON);
#line 203 "project-dialog.vala"
	gtk_icon_view_set_text_column (((ValideAbstractProjectDialog*) self)->widgets->icon_view, (gint) VALIDE_COLUMN_TYPE_NAME);
#line 352 "project-dialog.c"
	{
		GList* filename_collection;
		GList* filename_it;
#line 205 "project-dialog.vala"
		filename_collection = files;
#line 358 "project-dialog.c"
		for (filename_it = filename_collection; filename_it != NULL; filename_it = filename_it->next) {
			const char* _tmp9_;
			char* filename;
#line 193 "project-dialog.vala"
			_tmp9_ = NULL;
#line 205 "project-dialog.vala"
			filename = (_tmp9_ = (const char*) filename_it->data, (_tmp9_ == NULL) ? NULL : g_strdup (_tmp9_));
#line 366 "project-dialog.c"
			{
				GtkTreeIter iter = {0};
				ValideTemplate* template;
				GdkPixbuf* pixbuf;
				ValideTemplate* _tmp3_;
				ValideTemplate* _tmp4_;
				char* _tmp5_;
				GdkPixbuf* _tmp6_;
				GdkPixbuf* _tmp7_;
				GdkPixbuf* _tmp8_;
				template = NULL;
				pixbuf = NULL;
				_tmp3_ = _valide_template_new (filename, &_inner_error_);
				if (_inner_error_ != NULL) {
					filename = (g_free (filename), NULL);
					(template == NULL) ? NULL : (template = (g_object_unref (template), NULL));
					(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
					(files == NULL) ? NULL : (files = (_g_list_free_g_free (files), NULL));
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return;
				}
				_tmp4_ = NULL;
#line 211 "project-dialog.vala"
				template = (_tmp4_ = _tmp3_, (template == NULL) ? NULL : (template = (g_object_unref (template), NULL)), _tmp4_);
#line 2034 "glib-2.0.vapi"
				_tmp5_ = NULL;
#line 212 "project-dialog.vala"
				_tmp6_ = NULL;
#line 396 "project-dialog.c"
				_tmp7_ = (_tmp6_ = gdk_pixbuf_new_from_file (_tmp5_ = g_build_filename (valide_template_get_path (template), valide_template_get_icon (template), NULL), &_inner_error_), _tmp5_ = (g_free (_tmp5_), NULL), _tmp6_);
				if (_inner_error_ != NULL) {
					filename = (g_free (filename), NULL);
					(template == NULL) ? NULL : (template = (g_object_unref (template), NULL));
					(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
					(files == NULL) ? NULL : (files = (_g_list_free_g_free (files), NULL));
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return;
				}
				_tmp8_ = NULL;
#line 212 "project-dialog.vala"
				pixbuf = (_tmp8_ = _tmp7_, (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp8_);
#line 213 "project-dialog.vala"
				gtk_list_store_append (self->priv->list_store, &iter);
#line 214 "project-dialog.vala"
				gtk_list_store_set (self->priv->list_store, &iter, VALIDE_COLUMN_TYPE_ICON, pixbuf, VALIDE_COLUMN_TYPE_NAME, valide_template_get_name (template), VALIDE_COLUMN_TYPE_DESCRIPTION, valide_template_get_description (template), VALIDE_COLUMN_TYPE_PATH, valide_template_get_path (template), -1);
#line 414 "project-dialog.c"
				filename = (g_free (filename), NULL);
				(template == NULL) ? NULL : (template = (g_object_unref (template), NULL));
				(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
			}
		}
	}
	(files == NULL) ? NULL : (files = (_g_list_free_g_free (files), NULL));
}


#line 221 "project-dialog.vala"
static void valide_project_dialog_populate_license (ValideProjectDialog* self) {
#line 427 "project-dialog.c"
	GError * _inner_error_;
	GtkCellRendererText* cell;
	GtkListStore* _tmp0_;
	GtkCellRendererText* _tmp1_;
#line 221 "project-dialog.vala"
	g_return_if_fail (self != NULL);
#line 434 "project-dialog.c"
	_inner_error_ = NULL;
	cell = NULL;
	_tmp0_ = NULL;
#line 225 "project-dialog.vala"
	self->priv->licenses_list = (_tmp0_ = gtk_list_store_new (1, G_TYPE_STRING, NULL), (self->priv->licenses_list == NULL) ? NULL : (self->priv->licenses_list = (g_object_unref (self->priv->licenses_list), NULL)), _tmp0_);
#line 226 "project-dialog.vala"
	gtk_combo_box_set_model (((ValideAbstractProjectDialog*) self)->widgets->licenses_combo, (GtkTreeModel*) self->priv->licenses_list);
#line 442 "project-dialog.c"
	_tmp1_ = NULL;
#line 228 "project-dialog.vala"
	cell = (_tmp1_ = g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (cell == NULL) ? NULL : (cell = (g_object_unref (cell), NULL)), _tmp1_);
#line 229 "project-dialog.vala"
	gtk_cell_layout_pack_start ((GtkCellLayout*) ((ValideAbstractProjectDialog*) self)->widgets->licenses_combo, (GtkCellRenderer*) cell, TRUE);
#line 230 "project-dialog.vala"
	gtk_cell_layout_set_attributes ((GtkCellLayout*) ((ValideAbstractProjectDialog*) self)->widgets->licenses_combo, (GtkCellRenderer*) cell, "text", 0, NULL);
#line 450 "project-dialog.c"
	{
		char* filename;
		GList* files;
		GDir* dir;
		GList* _tmp2_;
		GDir* _tmp3_;
		GDir* _tmp4_;
		filename = NULL;
		files = NULL;
		dir = NULL;
		_tmp2_ = NULL;
#line 238 "project-dialog.vala"
		files = (_tmp2_ = NULL, (files == NULL) ? NULL : (files = (_g_list_free_g_free (files), NULL)), _tmp2_);
#line 464 "project-dialog.c"
		_tmp3_ = g_dir_open (LICENSES_DIR, 0, &_inner_error_);
		if (_inner_error_ != NULL) {
			filename = (g_free (filename), NULL);
			(files == NULL) ? NULL : (files = (_g_list_free_g_free (files), NULL));
			(dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL));
			goto __catch13_g_error;
			goto __finally13;
		}
		_tmp4_ = NULL;
#line 239 "project-dialog.vala"
		dir = (_tmp4_ = _tmp3_, (dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL)), _tmp4_);
#line 240 "project-dialog.vala"
		while (TRUE) {
#line 478 "project-dialog.c"
			char* _tmp6_;
			const char* _tmp5_;
			const char* _tmp7_;
			_tmp6_ = NULL;
#line 240 "project-dialog.vala"
			_tmp5_ = NULL;
#line 240 "project-dialog.vala"
			if (!((filename = (_tmp6_ = (_tmp5_ = g_dir_read_name (dir), (_tmp5_ == NULL) ? NULL : g_strdup (_tmp5_)), filename = (g_free (filename), NULL), _tmp6_)) != NULL)) {
#line 240 "project-dialog.vala"
				break;
#line 489 "project-dialog.c"
			}
#line 242 "project-dialog.vala"
			_tmp7_ = NULL;
#line 242 "project-dialog.vala"
			files = g_list_append (files, (_tmp7_ = filename, (_tmp7_ == NULL) ? NULL : g_strdup (_tmp7_)));
#line 495 "project-dialog.c"
		}
#line 244 "project-dialog.vala"
		files = g_list_sort (files, strcmp);
#line 499 "project-dialog.c"
		{
			GList* file_collection;
			GList* file_it;
#line 246 "project-dialog.vala"
			file_collection = files;
#line 505 "project-dialog.c"
			for (file_it = file_collection; file_it != NULL; file_it = file_it->next) {
				const char* _tmp8_;
				char* file;
#line 235 "project-dialog.vala"
				_tmp8_ = NULL;
#line 246 "project-dialog.vala"
				file = (_tmp8_ = (const char*) file_it->data, (_tmp8_ == NULL) ? NULL : g_strdup (_tmp8_));
#line 513 "project-dialog.c"
				{
					GtkTreeIter iter = {0};
#line 250 "project-dialog.vala"
					gtk_list_store_append (self->priv->licenses_list, &iter);
#line 251 "project-dialog.vala"
					gtk_list_store_set (self->priv->licenses_list, &iter, 0, file, -1);
#line 520 "project-dialog.c"
					file = (g_free (file), NULL);
				}
			}
		}
		filename = (g_free (filename), NULL);
		(files == NULL) ? NULL : (files = (_g_list_free_g_free (files), NULL));
		(dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL));
	}
	goto __finally13;
	__catch13_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 256 "project-dialog.vala"
			g_debug (e->message);
#line 538 "project-dialog.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally13:
	if (_inner_error_ != NULL) {
		(cell == NULL) ? NULL : (cell = (g_object_unref (cell), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	(cell == NULL) ? NULL : (cell = (g_object_unref (cell), NULL));
}


#line 260 "project-dialog.vala"
static void valide_project_dialog_populate_builder (ValideProjectDialog* self) {
#line 555 "project-dialog.c"
	GtkTreeIter iter = {0};
	GtkListStore* list_store;
	GtkCellRendererText* cell;
	GtkListStore* _tmp0_;
	GtkListStore* _tmp3_;
	GtkListStore* _tmp2_;
	GtkTreeModel* _tmp1_;
	GtkCellRendererText* _tmp4_;
#line 260 "project-dialog.vala"
	g_return_if_fail (self != NULL);
#line 566 "project-dialog.c"
	list_store = NULL;
	cell = NULL;
	_tmp0_ = NULL;
#line 266 "project-dialog.vala"
	list_store = (_tmp0_ = gtk_list_store_new (1, G_TYPE_STRING, NULL), (list_store == NULL) ? NULL : (list_store = (g_object_unref (list_store), NULL)), _tmp0_);
#line 572 "project-dialog.c"
	_tmp3_ = NULL;
#line 267 "project-dialog.vala"
	_tmp2_ = NULL;
#line 267 "project-dialog.vala"
	_tmp1_ = NULL;
#line 267 "project-dialog.vala"
	list_store = (_tmp3_ = (_tmp2_ = (_tmp1_ = gtk_combo_box_get_model (((ValideAbstractProjectDialog*) self)->widgets->builder), GTK_IS_LIST_STORE (_tmp1_) ? ((GtkListStore*) _tmp1_) : NULL), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (list_store == NULL) ? NULL : (list_store = (g_object_unref (list_store), NULL)), _tmp3_);
#line 580 "project-dialog.c"
	_tmp4_ = NULL;
#line 268 "project-dialog.vala"
	cell = (_tmp4_ = g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (cell == NULL) ? NULL : (cell = (g_object_unref (cell), NULL)), _tmp4_);
#line 269 "project-dialog.vala"
	gtk_cell_layout_pack_start ((GtkCellLayout*) ((ValideAbstractProjectDialog*) self)->widgets->builder, (GtkCellRenderer*) cell, TRUE);
#line 270 "project-dialog.vala"
	gtk_cell_layout_set_attributes ((GtkCellLayout*) ((ValideAbstractProjectDialog*) self)->widgets->builder, (GtkCellRenderer*) cell, "text", 0, NULL);
#line 588 "project-dialog.c"
	{
		ValideBuilderManager* _tmp5_;
		GList* _tmp6_;
		GList* builder_collection;
		GList* builder_it;
#line 62 "builder-manager.vala"
		_tmp5_ = NULL;
#line 271 "project-dialog.vala"
		_tmp6_ = NULL;
#line 271 "project-dialog.vala"
		builder_collection = (_tmp6_ = valide_builder_manager_get_builders (_tmp5_ = valide_builder_manager_get_instance ()), (_tmp5_ == NULL) ? NULL : (_tmp5_ = (g_object_unref (_tmp5_), NULL)), _tmp6_);
#line 600 "project-dialog.c"
		for (builder_it = builder_collection; builder_it != NULL; builder_it = builder_it->next) {
			const char* _tmp7_;
			char* builder;
#line 141 "builder-manager.vala"
			_tmp7_ = NULL;
#line 271 "project-dialog.vala"
			builder = (_tmp7_ = (const char*) builder_it->data, (_tmp7_ == NULL) ? NULL : g_strdup (_tmp7_));
#line 608 "project-dialog.c"
			{
#line 273 "project-dialog.vala"
				gtk_list_store_append (list_store, &iter);
#line 274 "project-dialog.vala"
				gtk_list_store_set (list_store, &iter, 0, builder, -1);
#line 614 "project-dialog.c"
				builder = (g_free (builder), NULL);
			}
		}
	}
	(list_store == NULL) ? NULL : (list_store = (g_object_unref (list_store), NULL));
	(cell == NULL) ? NULL : (cell = (g_object_unref (cell), NULL));
}


/**
   * This callback is called when the selected template changed
   */
#line 282 "project-dialog.vala"
void valide_project_dialog_selection_changed (GtkIconView* icon_view, ValideProjectDialog* self) {
#line 629 "project-dialog.c"
	GtkTreeIter iter = {0};
	GList* item;
#line 282 "project-dialog.vala"
	g_return_if_fail (self != NULL);
#line 282 "project-dialog.vala"
	g_return_if_fail (icon_view != NULL);
#line 636 "project-dialog.c"
	item = NULL;
#line 287 "project-dialog.vala"
	item = gtk_icon_view_get_selected_items (icon_view);
#line 288 "project-dialog.vala"
	if (item != NULL) {
#line 642 "project-dialog.c"
		char* tmp;
		const GtkTreePath* path;
		GtkTreePath* path2;
		GtkTreePath* _tmp0_;
		char* _tmp1_;
		tmp = NULL;
		path = NULL;
		path2 = NULL;
#line 294 "project-dialog.vala"
		path = item->data;
#line 653 "project-dialog.c"
		_tmp0_ = NULL;
#line 295 "project-dialog.vala"
		path2 = (_tmp0_ = gtk_tree_path_new_from_string (gtk_tree_path_to_string (path)), (path2 == NULL) ? NULL : (path2 = (gtk_tree_path_free (path2), NULL)), _tmp0_);
#line 296 "project-dialog.vala"
		gtk_tree_model_get_iter ((GtkTreeModel*) self->priv->list_store, &iter, path2);
#line 298 "project-dialog.vala"
		gtk_tree_model_get ((GtkTreeModel*) self->priv->list_store, &iter, VALIDE_COLUMN_TYPE_PATH, &tmp, -1);
#line 2034 "glib-2.0.vapi"
		_tmp1_ = NULL;
#line 299 "project-dialog.vala"
		valide_project_dialog_set_template (self, _tmp1_ = g_build_filename (tmp, "template.yml", NULL));
#line 665 "project-dialog.c"
		_tmp1_ = (g_free (_tmp1_), NULL);
#line 301 "project-dialog.vala"
		gtk_tree_model_get ((GtkTreeModel*) self->priv->list_store, &iter, VALIDE_COLUMN_TYPE_DESCRIPTION, &tmp, -1);
#line 302 "project-dialog.vala"
		valide_project_dialog_set_project_desc (self, tmp);
#line 303 "project-dialog.vala"
		gtk_assistant_set_page_complete (((ValideAbstractProjectDialog*) self)->widgets->assistant, (GtkWidget*) ((ValideAbstractProjectDialog*) self)->widgets->page2, TRUE);
#line 673 "project-dialog.c"
		tmp = (g_free (tmp), NULL);
		(path2 == NULL) ? NULL : (path2 = (gtk_tree_path_free (path2), NULL));
	} else {
#line 307 "project-dialog.vala"
		gtk_assistant_set_page_complete (((ValideAbstractProjectDialog*) self)->widgets->assistant, (GtkWidget*) ((ValideAbstractProjectDialog*) self)->widgets->page2, FALSE);
#line 679 "project-dialog.c"
	}
}


/**
   * The callback is called when the path of the project changed
   */
#line 315 "project-dialog.vala"
void valide_project_dialog_project_path_change (GtkWidget* sender, ValideProjectDialog* self) {
#line 689 "project-dialog.c"
	char* _tmp1_;
	char* _tmp0_;
#line 315 "project-dialog.vala"
	g_return_if_fail (self != NULL);
#line 315 "project-dialog.vala"
	g_return_if_fail (sender != NULL);
#line 696 "project-dialog.c"
	_tmp1_ = NULL;
#line 2034 "glib-2.0.vapi"
	_tmp0_ = NULL;
#line 317 "project-dialog.vala"
	gtk_entry_set_text (((ValideAbstractProjectDialog*) self)->widgets->path_entry, _tmp1_ = g_strconcat (_tmp0_ = g_build_filename (valide_project_dialog_get_project_dir (self), valide_project_dialog_get_project_name (self), NULL), G_DIR_SEPARATOR_S, NULL));
#line 702 "project-dialog.c"
	_tmp1_ = (g_free (_tmp1_), NULL);
	_tmp0_ = (g_free (_tmp0_), NULL);
#line 319 "project-dialog.vala"
	if (_vala_strcmp0 (valide_project_dialog_get_project_name (self), "") != 0) {
#line 321 "project-dialog.vala"
		gtk_assistant_set_page_complete (((ValideAbstractProjectDialog*) self)->widgets->assistant, (GtkWidget*) ((ValideAbstractProjectDialog*) self)->widgets->page3, TRUE);
#line 709 "project-dialog.c"
	} else {
#line 325 "project-dialog.vala"
		gtk_assistant_set_page_complete (((ValideAbstractProjectDialog*) self)->widgets->assistant, (GtkWidget*) ((ValideAbstractProjectDialog*) self)->widgets->page3, FALSE);
#line 713 "project-dialog.c"
	}
}


/**
   * @see Gtk.Widget.show
   */
#line 368 "project-dialog.vala"
void valide_project_dialog_show (ValideProjectDialog* self) {
#line 368 "project-dialog.vala"
	g_return_if_fail (self != NULL);
#line 370 "project-dialog.vala"
	gtk_widget_show_all ((GtkWidget*) ((ValideAbstractProjectDialog*) self)->widgets->assistant);
#line 727 "project-dialog.c"
}


#line 381 "project-dialog.vala"
static void valide_project_dialog_shutdown_loop (const ValideProjectDialogRunInfo* ri) {
#line 383 "project-dialog.vala"
	if (g_main_loop_is_running ((*ri).loop)) {
#line 385 "project-dialog.vala"
		g_main_loop_quit ((*ri).loop);
#line 737 "project-dialog.c"
	}
}


#line 389 "project-dialog.vala"
static void valide_project_dialog_apply_cb (GtkWidget* sender, ValideProjectDialogRunInfo* ri) {
#line 389 "project-dialog.vala"
	g_return_if_fail (sender != NULL);
#line 391 "project-dialog.vala"
	(*ri).response_id = GTK_RESPONSE_APPLY;
#line 392 "project-dialog.vala"
	valide_project_dialog_shutdown_loop (&(*ri));
#line 750 "project-dialog.c"
}


#line 395 "project-dialog.vala"
static void valide_project_dialog_cancel_cb (GtkWidget* sender, ValideProjectDialogRunInfo* ri) {
#line 395 "project-dialog.vala"
	g_return_if_fail (sender != NULL);
#line 397 "project-dialog.vala"
	(*ri).response_id = GTK_RESPONSE_CANCEL;
#line 398 "project-dialog.vala"
	valide_project_dialog_shutdown_loop (&(*ri));
#line 762 "project-dialog.c"
}


#line 401 "project-dialog.vala"
static void valide_project_dialog_close_cb (GtkWidget* sender, ValideProjectDialogRunInfo* ri) {
#line 401 "project-dialog.vala"
	g_return_if_fail (sender != NULL);
#line 403 "project-dialog.vala"
	(*ri).response_id = GTK_RESPONSE_CLOSE;
#line 404 "project-dialog.vala"
	valide_project_dialog_shutdown_loop (&(*ri));
#line 774 "project-dialog.c"
}


#line 408 "project-dialog.vala"
static void valide_project_dialog_unmap_cb (GtkWidget* sender, ValideProjectDialogRunInfo* ri) {
#line 408 "project-dialog.vala"
	g_return_if_fail (sender != NULL);
#line 410 "project-dialog.vala"
	valide_project_dialog_shutdown_loop (&(*ri));
#line 784 "project-dialog.c"
}


#line 413 "project-dialog.vala"
static gboolean valide_project_dialog_delete_cb (GtkWidget* sender, ValideProjectDialogRunInfo* ri) {
#line 790 "project-dialog.c"
	gboolean result;
#line 413 "project-dialog.vala"
	g_return_val_if_fail (sender != NULL, FALSE);
#line 415 "project-dialog.vala"
	valide_project_dialog_shutdown_loop (&(*ri));
#line 796 "project-dialog.c"
	result = TRUE;
	return result;
}


#line 419 "project-dialog.vala"
static void valide_project_dialog_destroy_cb (GtkWidget* sender, ValideProjectDialogRunInfo* ri) {
#line 419 "project-dialog.vala"
	g_return_if_fail (sender != NULL);
#line 806 "project-dialog.c"
	/* shutdown_loop will be called by run_unmap_handler */
#line 422 "project-dialog.vala"
	(*ri).destroyed = TRUE;
#line 810 "project-dialog.c"
}


/**
   * @see Gtk.Dialog.run
   */
#line 428 "project-dialog.vala"
guint valide_project_dialog_run (ValideProjectDialog* self) {
#line 819 "project-dialog.c"
	guint result;
	ValideProjectDialogRunInfo ri = {NULL, GTK_RESPONSE_NONE, FALSE};
	GMainLoop* _tmp0_;
	GMainLoop* _tmp1_;
#line 428 "project-dialog.vala"
	g_return_val_if_fail (self != NULL, 0U);
#line 432 "project-dialog.vala"
	valide_project_dialog_show (self);
#line 433 "project-dialog.vala"
	g_signal_connect (((ValideAbstractProjectDialog*) self)->widgets->assistant, "apply", (GCallback) valide_project_dialog_apply_cb, (void*) (&ri));
#line 434 "project-dialog.vala"
	g_signal_connect (((ValideAbstractProjectDialog*) self)->widgets->assistant, "cancel", (GCallback) valide_project_dialog_cancel_cb, (void*) (&ri));
#line 435 "project-dialog.vala"
	g_signal_connect (((ValideAbstractProjectDialog*) self)->widgets->assistant, "close", (GCallback) valide_project_dialog_close_cb, (void*) (&ri));
#line 436 "project-dialog.vala"
	g_signal_connect (((ValideAbstractProjectDialog*) self)->widgets->assistant, "delete_event", (GCallback) valide_project_dialog_delete_cb, (void*) (&ri));
#line 437 "project-dialog.vala"
	g_signal_connect (((ValideAbstractProjectDialog*) self)->widgets->assistant, "unmap", (GCallback) valide_project_dialog_unmap_cb, (void*) (&ri));
#line 438 "project-dialog.vala"
	g_signal_connect (((ValideAbstractProjectDialog*) self)->widgets->assistant, "destroy", (GCallback) valide_project_dialog_destroy_cb, (void*) (&ri));
#line 840 "project-dialog.c"
	_tmp0_ = NULL;
#line 440 "project-dialog.vala"
	ri.loop = (_tmp0_ = g_main_loop_new (NULL, FALSE), (ri.loop == NULL) ? NULL : (ri.loop = (g_main_loop_unref (ri.loop), NULL)), _tmp0_);
#line 441 "project-dialog.vala"
	gdk_threads_leave ();
#line 442 "project-dialog.vala"
	g_main_loop_run (ri.loop);
#line 443 "project-dialog.vala"
	gdk_threads_enter ();
#line 850 "project-dialog.c"
	_tmp1_ = NULL;
#line 444 "project-dialog.vala"
	ri.loop = (_tmp1_ = NULL, (ri.loop == NULL) ? NULL : (ri.loop = (g_main_loop_unref (ri.loop), NULL)), _tmp1_);
#line 445 "project-dialog.vala"
	if (!ri.destroyed) {
#line 447 "project-dialog.vala"
		g_signal_handlers_disconnect_by_func (((ValideAbstractProjectDialog*) self)->widgets->assistant, (void*) valide_project_dialog_apply_cb, (void*) (&ri));
#line 448 "project-dialog.vala"
		g_signal_handlers_disconnect_by_func (((ValideAbstractProjectDialog*) self)->widgets->assistant, (void*) valide_project_dialog_cancel_cb, (void*) (&ri));
#line 449 "project-dialog.vala"
		g_signal_handlers_disconnect_by_func (((ValideAbstractProjectDialog*) self)->widgets->assistant, (void*) valide_project_dialog_close_cb, (void*) (&ri));
#line 450 "project-dialog.vala"
		g_signal_handlers_disconnect_by_func (((ValideAbstractProjectDialog*) self)->widgets->assistant, (void*) valide_project_dialog_delete_cb, (void*) (&ri));
#line 451 "project-dialog.vala"
		g_signal_handlers_disconnect_by_func (((ValideAbstractProjectDialog*) self)->widgets->assistant, (void*) valide_project_dialog_unmap_cb, (void*) (&ri));
#line 452 "project-dialog.vala"
		g_signal_handlers_disconnect_by_func (((ValideAbstractProjectDialog*) self)->widgets->assistant, (void*) valide_project_dialog_destroy_cb, (void*) (&ri));
#line 868 "project-dialog.c"
	}
	result = (guint) ri.response_id;
	valide_project_dialog_run_info_destroy (&ri);
	return result;
}


/**
 * A dialog box widget for create a new project
 */
#line 34 "project-dialog.vala"
ValideProjectDialog* valide_project_dialog_construct (GType object_type) {
#line 881 "project-dialog.c"
	ValideProjectDialog * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


#line 34 "project-dialog.vala"
ValideProjectDialog* valide_project_dialog_new (void) {
#line 34 "project-dialog.vala"
	return valide_project_dialog_construct (VALIDE_TYPE_PROJECT_DIALOG);
#line 892 "project-dialog.c"
}


const char* valide_project_dialog_get_project_name (ValideProjectDialog* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = gtk_entry_get_text (((ValideAbstractProjectDialog*) self)->widgets->name_entry);
	return result;
}


const char* valide_project_dialog_get_project_author (ValideProjectDialog* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = gtk_entry_get_text (((ValideAbstractProjectDialog*) self)->widgets->author_entry);
	return result;
}


const char* valide_project_dialog_get_project_version (ValideProjectDialog* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = gtk_entry_get_text (((ValideAbstractProjectDialog*) self)->widgets->version_entry);
	return result;
}


const char* valide_project_dialog_get_project_license (ValideProjectDialog* self) {
	const char* result;
	GtkTreeIter iter = {0};
	char* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = NULL;
#line 87 "project-dialog.vala"
	self->priv->_license = (_tmp0_ = g_strdup ("None"), self->priv->_license = (g_free (self->priv->_license), NULL), _tmp0_);
#line 88 "project-dialog.vala"
	if (gtk_combo_box_get_active_iter (((ValideAbstractProjectDialog*) self)->widgets->licenses_combo, &iter)) {
#line 90 "project-dialog.vala"
		gtk_tree_model_get ((GtkTreeModel*) self->priv->licenses_list, &iter, 0, &self->priv->_license, -1);
#line 932 "project-dialog.c"
	}
	result = self->priv->_license;
	return result;
}


const char* valide_project_dialog_get_project_dir (ValideProjectDialog* self) {
	const char* result;
	char* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = NULL;
#line 104 "project-dialog.vala"
	self->priv->_project_dir = (_tmp0_ = gtk_file_chooser_get_filename ((GtkFileChooser*) ((ValideAbstractProjectDialog*) self)->widgets->dir_chooser), self->priv->_project_dir = (g_free (self->priv->_project_dir), NULL), _tmp0_);
#line 946 "project-dialog.c"
	result = self->priv->_project_dir;
	return result;
}


void valide_project_dialog_set_project_dir (ValideProjectDialog* self, const char* value) {
	char* _tmp2_;
	const char* _tmp1_;
	g_return_if_fail (self != NULL);
#line 109 "project-dialog.vala"
	gtk_file_chooser_set_filename ((GtkFileChooser*) ((ValideAbstractProjectDialog*) self)->widgets->dir_chooser, value);
#line 958 "project-dialog.c"
	_tmp2_ = NULL;
#line 110 "project-dialog.vala"
	_tmp1_ = NULL;
#line 110 "project-dialog.vala"
	self->priv->_project_dir = (_tmp2_ = (_tmp1_ = value, (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_)), self->priv->_project_dir = (g_free (self->priv->_project_dir), NULL), _tmp2_);
#line 964 "project-dialog.c"
	g_object_notify ((GObject *) self, "project-dir");
}


const char* valide_project_dialog_get_project_path (ValideProjectDialog* self) {
	const char* result;
	char* _tmp1_;
	const char* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp1_ = NULL;
#line 122 "project-dialog.vala"
	_tmp0_ = NULL;
#line 122 "project-dialog.vala"
	self->priv->_project_path = (_tmp1_ = (_tmp0_ = gtk_entry_get_text (((ValideAbstractProjectDialog*) self)->widgets->path_entry), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_project_path = (g_free (self->priv->_project_path), NULL), _tmp1_);
#line 979 "project-dialog.c"
	result = self->priv->_project_path;
	return result;
}


const char* valide_project_dialog_get_lang (ValideProjectDialog* self) {
	const char* result;
	GtkTreeIter iter = {0};
	GtkTreeModel* model;
	char* _tmp0_;
	char* _tmp3_;
	g_return_val_if_fail (self != NULL, NULL);
	model = NULL;
	_tmp0_ = NULL;
#line 138 "project-dialog.vala"
	self->priv->_lang = (_tmp0_ = g_strdup (""), self->priv->_lang = (g_free (self->priv->_lang), NULL), _tmp0_);
#line 139 "project-dialog.vala"
	if (gtk_combo_box_get_active_iter (((ValideAbstractProjectDialog*) self)->widgets->lang_combo, &iter)) {
#line 998 "project-dialog.c"
		GtkTreeModel* _tmp2_;
		GtkTreeModel* _tmp1_;
		_tmp2_ = NULL;
#line 141 "project-dialog.vala"
		_tmp1_ = NULL;
#line 141 "project-dialog.vala"
		model = (_tmp2_ = (_tmp1_ = gtk_combo_box_get_model (((ValideAbstractProjectDialog*) self)->widgets->lang_combo), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (model == NULL) ? NULL : (model = (g_object_unref (model), NULL)), _tmp2_);
#line 142 "project-dialog.vala"
		gtk_tree_model_get (model, &iter, 0, &self->priv->_lang, -1);
#line 1008 "project-dialog.c"
	}
	_tmp3_ = NULL;
#line 144 "project-dialog.vala"
	self->priv->_lang = (_tmp3_ = g_utf8_strdown (self->priv->_lang, -1), self->priv->_lang = (g_free (self->priv->_lang), NULL), _tmp3_);
#line 1013 "project-dialog.c"
	result = self->priv->_lang;
	(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
	return result;
}


const char* valide_project_dialog_get_builder (ValideProjectDialog* self) {
	const char* result;
	GtkTreeIter iter = {0};
	GtkTreeModel* model;
	char* _tmp0_;
	char* _tmp3_;
	g_return_val_if_fail (self != NULL, NULL);
	model = NULL;
	_tmp0_ = NULL;
#line 160 "project-dialog.vala"
	self->priv->_builder = (_tmp0_ = g_strdup (""), self->priv->_builder = (g_free (self->priv->_builder), NULL), _tmp0_);
#line 161 "project-dialog.vala"
	if (gtk_combo_box_get_active_iter (((ValideAbstractProjectDialog*) self)->widgets->builder, &iter)) {
#line 1033 "project-dialog.c"
		GtkTreeModel* _tmp2_;
		GtkTreeModel* _tmp1_;
		_tmp2_ = NULL;
#line 163 "project-dialog.vala"
		_tmp1_ = NULL;
#line 163 "project-dialog.vala"
		model = (_tmp2_ = (_tmp1_ = gtk_combo_box_get_model (((ValideAbstractProjectDialog*) self)->widgets->builder), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (model == NULL) ? NULL : (model = (g_object_unref (model), NULL)), _tmp2_);
#line 164 "project-dialog.vala"
		gtk_tree_model_get (model, &iter, 0, &self->priv->_builder, -1);
#line 1043 "project-dialog.c"
	}
	_tmp3_ = NULL;
#line 166 "project-dialog.vala"
	self->priv->_builder = (_tmp3_ = g_utf8_strdown (self->priv->_builder, -1), self->priv->_builder = (g_free (self->priv->_builder), NULL), _tmp3_);
#line 1048 "project-dialog.c"
	result = self->priv->_builder;
	(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
	return result;
}


const char* valide_project_dialog_get_template (ValideProjectDialog* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_template;
	return result;
}


static void valide_project_dialog_set_template (ValideProjectDialog* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 177 "project-dialog.vala"
	_tmp0_ = NULL;
#line 1070 "project-dialog.c"
	self->priv->_template = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_template = (g_free (self->priv->_template), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "template");
}


void valide_project_dialog_set_project_desc (ValideProjectDialog* self, const char* value) {
	g_return_if_fail (self != NULL);
#line 187 "project-dialog.vala"
	gtk_label_set_text (((ValideAbstractProjectDialog*) self)->widgets->desc_label, value);
#line 1080 "project-dialog.c"
	g_object_notify ((GObject *) self, "project-desc");
}


static void _lambda32_ (ValideProjectDialog* self) {
#line 351 "project-dialog.vala"
	g_signal_emit_by_name (self, "apply");
#line 1088 "project-dialog.c"
}


static void __lambda32__gtk_assistant_apply (GtkAssistant* _sender, gpointer self) {
	_lambda32_ (self);
}


static void _lambda33_ (ValideProjectDialog* self) {
#line 354 "project-dialog.vala"
	gtk_object_destroy ((GtkObject*) ((ValideAbstractProjectDialog*) self)->widgets->assistant);
#line 1100 "project-dialog.c"
}


static void __lambda33__gtk_assistant_cancel (GtkAssistant* _sender, gpointer self) {
	_lambda33_ (self);
}


static void _lambda34_ (ValideProjectDialog* self) {
#line 357 "project-dialog.vala"
	gtk_object_destroy ((GtkObject*) ((ValideAbstractProjectDialog*) self)->widgets->assistant);
#line 1112 "project-dialog.c"
}


static void __lambda34__gtk_assistant_close (GtkAssistant* _sender, gpointer self) {
	_lambda34_ (self);
}


static GObject * valide_project_dialog_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValideProjectDialogClass * klass;
	GObjectClass * parent_class;
	ValideProjectDialog * self;
	GError * _inner_error_;
	klass = VALIDE_PROJECT_DIALOG_CLASS (g_type_class_peek (VALIDE_TYPE_PROJECT_DIALOG));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_PROJECT_DIALOG (obj);
	_inner_error_ = NULL;
	{
		char* _tmp5_;
		ValideConfigManager* _tmp4_;
		{
			GdkPixbuf* icon;
			char* _tmp0_;
			GdkPixbuf* _tmp1_;
			GdkPixbuf* _tmp2_;
			GdkPixbuf* _tmp3_;
			icon = NULL;
#line 2034 "glib-2.0.vapi"
			_tmp0_ = NULL;
#line 335 "project-dialog.vala"
			_tmp1_ = NULL;
#line 1146 "project-dialog.c"
			_tmp2_ = (_tmp1_ = gdk_pixbuf_new_from_file (_tmp0_ = g_build_filename (PIXMAPS_DIR, "icone-32.xpm", NULL), &_inner_error_), _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_);
			if (_inner_error_ != NULL) {
				(icon == NULL) ? NULL : (icon = (g_object_unref (icon), NULL));
				goto __catch14_g_error;
				goto __finally14;
			}
			_tmp3_ = NULL;
#line 335 "project-dialog.vala"
			icon = (_tmp3_ = _tmp2_, (icon == NULL) ? NULL : (icon = (g_object_unref (icon), NULL)), _tmp3_);
#line 336 "project-dialog.vala"
			gtk_assistant_set_page_header_image (((ValideAbstractProjectDialog*) self)->widgets->assistant, (GtkWidget*) ((ValideAbstractProjectDialog*) self)->widgets->page1, icon);
#line 337 "project-dialog.vala"
			gtk_assistant_set_page_header_image (((ValideAbstractProjectDialog*) self)->widgets->assistant, (GtkWidget*) ((ValideAbstractProjectDialog*) self)->widgets->page2, icon);
#line 338 "project-dialog.vala"
			gtk_assistant_set_page_header_image (((ValideAbstractProjectDialog*) self)->widgets->assistant, (GtkWidget*) ((ValideAbstractProjectDialog*) self)->widgets->page3, icon);
#line 1162 "project-dialog.c"
			(icon == NULL) ? NULL : (icon = (g_object_unref (icon), NULL));
		}
		goto __finally14;
		__catch14_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
#line 342 "project-dialog.vala"
				g_debug (e->message);
#line 1174 "project-dialog.c"
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally14:
		if (_inner_error_ != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
		}
#line 345 "project-dialog.vala"
		valide_project_dialog_populate_icon_view (self);
#line 346 "project-dialog.vala"
		valide_project_dialog_populate_license (self);
#line 347 "project-dialog.vala"
		valide_project_dialog_populate_builder (self);
#line 348 "project-dialog.vala"
		valide_abstract_project_dialog_connect_signals ((ValideAbstractProjectDialog*) self, "valide_project_dialog_");
#line 349 "project-dialog.vala"
		gtk_assistant_set_page_complete (((ValideAbstractProjectDialog*) self)->widgets->assistant, (GtkWidget*) ((ValideAbstractProjectDialog*) self)->widgets->page1, TRUE);
#line 350 "project-dialog.vala"
		g_signal_connect (((ValideAbstractProjectDialog*) self)->widgets->assistant, "apply", (GCallback) __lambda32__gtk_assistant_apply, self);
#line 353 "project-dialog.vala"
		g_signal_connect (((ValideAbstractProjectDialog*) self)->widgets->assistant, "cancel", (GCallback) __lambda33__gtk_assistant_cancel, self);
#line 356 "project-dialog.vala"
		g_signal_connect (((ValideAbstractProjectDialog*) self)->widgets->assistant, "close", (GCallback) __lambda34__gtk_assistant_close, self);
#line 360 "project-dialog.vala"
		gtk_combo_box_set_active (((ValideAbstractProjectDialog*) self)->widgets->lang_combo, 0);
#line 361 "project-dialog.vala"
		gtk_combo_box_set_active (((ValideAbstractProjectDialog*) self)->widgets->builder, 0);
#line 169 "abstract-config-manager.vala"
		_tmp5_ = NULL;
#line 88 "config-manager.vala"
		_tmp4_ = NULL;
#line 362 "project-dialog.vala"
		valide_project_dialog_set_project_dir (self, _tmp5_ = valide_abstract_config_manager_get_string ((ValideAbstractConfigManager*) (_tmp4_ = valide_config_manager_get_instance ()), "Projects", "default-directory"));
#line 1209 "project-dialog.c"
		_tmp5_ = (g_free (_tmp5_), NULL);
		(_tmp4_ == NULL) ? NULL : (_tmp4_ = (g_object_unref (_tmp4_), NULL));
	}
	return obj;
}


static void valide_project_dialog_run_info_copy (const ValideProjectDialogRunInfo* self, ValideProjectDialogRunInfo* dest) {
	GMainLoop* _tmp6_;
#line 376 "project-dialog.vala"
	_tmp6_ = NULL;
#line 1221 "project-dialog.c"
	dest->loop = (_tmp6_ = self->loop, (_tmp6_ == NULL) ? NULL : g_main_loop_ref (_tmp6_));
	dest->response_id = self->response_id;
	dest->destroyed = self->destroyed;
}


static void valide_project_dialog_run_info_destroy (ValideProjectDialogRunInfo* self) {
	(self->loop == NULL) ? NULL : (self->loop = (g_main_loop_unref (self->loop), NULL));
}


static ValideProjectDialogRunInfo* valide_project_dialog_run_info_dup (const ValideProjectDialogRunInfo* self) {
	ValideProjectDialogRunInfo* dup;
	dup = g_new0 (ValideProjectDialogRunInfo, 1);
	valide_project_dialog_run_info_copy (self, dup);
	return dup;
}


static void valide_project_dialog_run_info_free (ValideProjectDialogRunInfo* self) {
	valide_project_dialog_run_info_destroy (self);
	g_free (self);
}


static GType valide_project_dialog_run_info_get_type (void) {
	static GType valide_project_dialog_run_info_type_id = 0;
	if (valide_project_dialog_run_info_type_id == 0) {
		valide_project_dialog_run_info_type_id = g_boxed_type_register_static ("ValideProjectDialogRunInfo", (GBoxedCopyFunc) valide_project_dialog_run_info_dup, (GBoxedFreeFunc) valide_project_dialog_run_info_free);
	}
	return valide_project_dialog_run_info_type_id;
}


static void valide_project_dialog_class_init (ValideProjectDialogClass * klass) {
	valide_project_dialog_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideProjectDialogPrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_project_dialog_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_project_dialog_set_property;
	G_OBJECT_CLASS (klass)->constructor = valide_project_dialog_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_project_dialog_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PROJECT_DIALOG_PROJECT_NAME, g_param_spec_string ("project-name", "project-name", "project-name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PROJECT_DIALOG_PROJECT_AUTHOR, g_param_spec_string ("project-author", "project-author", "project-author", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PROJECT_DIALOG_PROJECT_VERSION, g_param_spec_string ("project-version", "project-version", "project-version", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PROJECT_DIALOG_PROJECT_LICENSE, g_param_spec_string ("project-license", "project-license", "project-license", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PROJECT_DIALOG_PROJECT_DIR, g_param_spec_string ("project-dir", "project-dir", "project-dir", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PROJECT_DIALOG_PROJECT_PATH, g_param_spec_string ("project-path", "project-path", "project-path", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PROJECT_DIALOG_LANG, g_param_spec_string ("lang", "lang", "lang", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PROJECT_DIALOG_BUILDER, g_param_spec_string ("builder", "builder", "builder", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PROJECT_DIALOG_TEMPLATE, g_param_spec_string ("template", "template", "template", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PROJECT_DIALOG_PROJECT_DESC, g_param_spec_string ("project-desc", "project-desc", "project-desc", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE));
	g_signal_new ("apply", VALIDE_TYPE_PROJECT_DIALOG, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}


static void valide_project_dialog_instance_init (ValideProjectDialog * self) {
	self->priv = VALIDE_PROJECT_DIALOG_GET_PRIVATE (self);
}


static void valide_project_dialog_finalize (GObject* obj) {
	ValideProjectDialog * self;
	self = VALIDE_PROJECT_DIALOG (obj);
	(self->priv->list_store == NULL) ? NULL : (self->priv->list_store = (g_object_unref (self->priv->list_store), NULL));
	(self->priv->licenses_list == NULL) ? NULL : (self->priv->licenses_list = (g_object_unref (self->priv->licenses_list), NULL));
	self->priv->_license = (g_free (self->priv->_license), NULL);
	self->priv->_project_dir = (g_free (self->priv->_project_dir), NULL);
	self->priv->_project_path = (g_free (self->priv->_project_path), NULL);
	self->priv->_lang = (g_free (self->priv->_lang), NULL);
	self->priv->_builder = (g_free (self->priv->_builder), NULL);
	self->priv->_template = (g_free (self->priv->_template), NULL);
	G_OBJECT_CLASS (valide_project_dialog_parent_class)->finalize (obj);
}


GType valide_project_dialog_get_type (void) {
	static GType valide_project_dialog_type_id = 0;
	if (valide_project_dialog_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideProjectDialogClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_project_dialog_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideProjectDialog), 0, (GInstanceInitFunc) valide_project_dialog_instance_init, NULL };
		valide_project_dialog_type_id = g_type_register_static (VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG, "ValideProjectDialog", &g_define_type_info, 0);
	}
	return valide_project_dialog_type_id;
}


static void valide_project_dialog_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideProjectDialog * self;
	gpointer boxed;
	self = VALIDE_PROJECT_DIALOG (object);
	switch (property_id) {
		case VALIDE_PROJECT_DIALOG_PROJECT_NAME:
		g_value_set_string (value, valide_project_dialog_get_project_name (self));
		break;
		case VALIDE_PROJECT_DIALOG_PROJECT_AUTHOR:
		g_value_set_string (value, valide_project_dialog_get_project_author (self));
		break;
		case VALIDE_PROJECT_DIALOG_PROJECT_VERSION:
		g_value_set_string (value, valide_project_dialog_get_project_version (self));
		break;
		case VALIDE_PROJECT_DIALOG_PROJECT_LICENSE:
		g_value_set_string (value, valide_project_dialog_get_project_license (self));
		break;
		case VALIDE_PROJECT_DIALOG_PROJECT_DIR:
		g_value_set_string (value, valide_project_dialog_get_project_dir (self));
		break;
		case VALIDE_PROJECT_DIALOG_PROJECT_PATH:
		g_value_set_string (value, valide_project_dialog_get_project_path (self));
		break;
		case VALIDE_PROJECT_DIALOG_LANG:
		g_value_set_string (value, valide_project_dialog_get_lang (self));
		break;
		case VALIDE_PROJECT_DIALOG_BUILDER:
		g_value_set_string (value, valide_project_dialog_get_builder (self));
		break;
		case VALIDE_PROJECT_DIALOG_TEMPLATE:
		g_value_set_string (value, valide_project_dialog_get_template (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_project_dialog_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideProjectDialog * self;
	self = VALIDE_PROJECT_DIALOG (object);
	switch (property_id) {
		case VALIDE_PROJECT_DIALOG_PROJECT_DIR:
		valide_project_dialog_set_project_dir (self, g_value_get_string (value));
		break;
		case VALIDE_PROJECT_DIALOG_TEMPLATE:
		valide_project_dialog_set_template (self, g_value_get_string (value));
		break;
		case VALIDE_PROJECT_DIALOG_PROJECT_DESC:
		valide_project_dialog_set_project_desc (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




