/* project-manager.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <gio/gio.h>
#include <glib/gstdio.h>
#include <config.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <gdk/gdk.h>
#include <glib/gi18n-lib.h>


#define VALIDE_TYPE_PROJECT_MANAGER (valide_project_manager_get_type ())
#define VALIDE_PROJECT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PROJECT_MANAGER, ValideProjectManager))
#define VALIDE_PROJECT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PROJECT_MANAGER, ValideProjectManagerClass))
#define VALIDE_IS_PROJECT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PROJECT_MANAGER))
#define VALIDE_IS_PROJECT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PROJECT_MANAGER))
#define VALIDE_PROJECT_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PROJECT_MANAGER, ValideProjectManagerClass))

typedef struct _ValideProjectManager ValideProjectManager;
typedef struct _ValideProjectManagerClass ValideProjectManagerClass;
typedef struct _ValideProjectManagerPrivate ValideProjectManagerPrivate;

#define VALIDE_TYPE_PROJECT (valide_project_get_type ())
#define VALIDE_PROJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PROJECT, ValideProject))
#define VALIDE_PROJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PROJECT, ValideProjectClass))
#define VALIDE_IS_PROJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PROJECT))
#define VALIDE_IS_PROJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PROJECT))
#define VALIDE_PROJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PROJECT, ValideProjectClass))

typedef struct _ValideProject ValideProject;
typedef struct _ValideProjectClass ValideProjectClass;

#define VALIDE_TYPE_UI_MANAGER (valide_ui_manager_get_type ())
#define VALIDE_UI_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_UI_MANAGER, ValideUIManager))
#define VALIDE_UI_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_UI_MANAGER, ValideUIManagerClass))
#define VALIDE_IS_UI_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_UI_MANAGER))
#define VALIDE_IS_UI_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_UI_MANAGER))
#define VALIDE_UI_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_UI_MANAGER, ValideUIManagerClass))

typedef struct _ValideUIManager ValideUIManager;
typedef struct _ValideUIManagerClass ValideUIManagerClass;

#define VALIDE_TYPE_DOCUMENT_MANAGER (valide_document_manager_get_type ())
#define VALIDE_DOCUMENT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_DOCUMENT_MANAGER, ValideDocumentManager))
#define VALIDE_DOCUMENT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_DOCUMENT_MANAGER, ValideDocumentManagerClass))
#define VALIDE_IS_DOCUMENT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_DOCUMENT_MANAGER))
#define VALIDE_IS_DOCUMENT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_DOCUMENT_MANAGER))
#define VALIDE_DOCUMENT_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_DOCUMENT_MANAGER, ValideDocumentManagerClass))

typedef struct _ValideDocumentManager ValideDocumentManager;
typedef struct _ValideDocumentManagerClass ValideDocumentManagerClass;

#define VALIDE_TYPE_BUILDER_MANAGER (valide_builder_manager_get_type ())
#define VALIDE_BUILDER_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_BUILDER_MANAGER, ValideBuilderManager))
#define VALIDE_BUILDER_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_BUILDER_MANAGER, ValideBuilderManagerClass))
#define VALIDE_IS_BUILDER_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_BUILDER_MANAGER))
#define VALIDE_IS_BUILDER_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_BUILDER_MANAGER))
#define VALIDE_BUILDER_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_BUILDER_MANAGER, ValideBuilderManagerClass))

typedef struct _ValideBuilderManager ValideBuilderManager;
typedef struct _ValideBuilderManagerClass ValideBuilderManagerClass;

#define VALIDE_PROJECT_MANAGER_TYPE_COL (valide_project_manager_col_get_type ())

#define G_YAML_TYPE_BUILDABLE (g_yaml_buildable_get_type ())
#define G_YAML_BUILDABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_TYPE_BUILDABLE, GYAMLBuildable))
#define G_YAML_IS_BUILDABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_TYPE_BUILDABLE))
#define G_YAML_BUILDABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), G_YAML_TYPE_BUILDABLE, GYAMLBuildableIface))

typedef struct _GYAMLBuildable GYAMLBuildable;
typedef struct _GYAMLBuildableIface GYAMLBuildableIface;

#define G_YAML_TYPE_BUILDER (g_yaml_builder_get_type ())
#define G_YAML_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_TYPE_BUILDER, GYAMLBuilder))
#define G_YAML_BUILDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), G_YAML_TYPE_BUILDER, GYAMLBuilderClass))
#define G_YAML_IS_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_TYPE_BUILDER))
#define G_YAML_IS_BUILDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_YAML_TYPE_BUILDER))
#define G_YAML_BUILDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), G_YAML_TYPE_BUILDER, GYAMLBuilderClass))

typedef struct _GYAMLBuilder GYAMLBuilder;
typedef struct _GYAMLBuilderClass GYAMLBuilderClass;

#define G_YAML_TYPE_NODE (g_yaml_node_get_type ())
#define G_YAML_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_TYPE_NODE, GYAMLNode))
#define G_YAML_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), G_YAML_TYPE_NODE, GYAMLNodeClass))
#define G_YAML_IS_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_TYPE_NODE))
#define G_YAML_IS_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_YAML_TYPE_NODE))
#define G_YAML_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), G_YAML_TYPE_NODE, GYAMLNodeClass))

typedef struct _GYAMLNode GYAMLNode;
typedef struct _GYAMLNodeClass GYAMLNodeClass;
typedef struct _ValideProjectPrivate ValideProjectPrivate;

#define VALIDE_TYPE_SOURCE (valide_source_get_type ())
#define VALIDE_SOURCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_SOURCE, ValideSource))
#define VALIDE_SOURCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_SOURCE, ValideSourceClass))
#define VALIDE_IS_SOURCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_SOURCE))
#define VALIDE_IS_SOURCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_SOURCE))
#define VALIDE_SOURCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_SOURCE, ValideSourceClass))

typedef struct _ValideSource ValideSource;
typedef struct _ValideSourceClass ValideSourceClass;

#define VALIDE_TYPE_PACKAGE (valide_package_get_type ())
#define VALIDE_PACKAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PACKAGE, ValidePackage))
#define VALIDE_PACKAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PACKAGE, ValidePackageClass))
#define VALIDE_IS_PACKAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PACKAGE))
#define VALIDE_IS_PACKAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PACKAGE))
#define VALIDE_PACKAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PACKAGE, ValidePackageClass))

typedef struct _ValidePackage ValidePackage;
typedef struct _ValidePackageClass ValidePackageClass;

#define VALIDE_TYPE_VAPI_DIR (valide_vapi_dir_get_type ())
#define VALIDE_VAPI_DIR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_VAPI_DIR, ValideVapiDir))
#define VALIDE_VAPI_DIR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_VAPI_DIR, ValideVapiDirClass))
#define VALIDE_IS_VAPI_DIR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_VAPI_DIR))
#define VALIDE_IS_VAPI_DIR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_VAPI_DIR))
#define VALIDE_VAPI_DIR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_VAPI_DIR, ValideVapiDirClass))

typedef struct _ValideVapiDir ValideVapiDir;
typedef struct _ValideVapiDirClass ValideVapiDirClass;

#define VALIDE_TYPE_BUILDER_OPTIONS (valide_builder_options_get_type ())
#define VALIDE_BUILDER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_BUILDER_OPTIONS, ValideBuilderOptions))
#define VALIDE_IS_BUILDER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_BUILDER_OPTIONS))
#define VALIDE_BUILDER_OPTIONS_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), VALIDE_TYPE_BUILDER_OPTIONS, ValideBuilderOptionsIface))

typedef struct _ValideBuilderOptions ValideBuilderOptions;
typedef struct _ValideBuilderOptionsIface ValideBuilderOptionsIface;

#define VALIDE_TYPE_TEMPLATE (valide_template_get_type ())
#define VALIDE_TEMPLATE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_TEMPLATE, ValideTemplate))
#define VALIDE_TEMPLATE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_TEMPLATE, ValideTemplateClass))
#define VALIDE_IS_TEMPLATE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_TEMPLATE))
#define VALIDE_IS_TEMPLATE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_TEMPLATE))
#define VALIDE_TEMPLATE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_TEMPLATE, ValideTemplateClass))

typedef struct _ValideTemplate ValideTemplate;
typedef struct _ValideTemplateClass ValideTemplateClass;
typedef struct _ValideTemplatePrivate ValideTemplatePrivate;

#define VALIDE_TYPE_EXECUTABLE_OPTIONS (valide_executable_options_get_type ())
#define VALIDE_EXECUTABLE_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_EXECUTABLE_OPTIONS, ValideExecutableOptions))
#define VALIDE_EXECUTABLE_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_EXECUTABLE_OPTIONS, ValideExecutableOptionsClass))
#define VALIDE_IS_EXECUTABLE_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_EXECUTABLE_OPTIONS))
#define VALIDE_IS_EXECUTABLE_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_EXECUTABLE_OPTIONS))
#define VALIDE_EXECUTABLE_OPTIONS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_EXECUTABLE_OPTIONS, ValideExecutableOptionsClass))

typedef struct _ValideExecutableOptions ValideExecutableOptions;
typedef struct _ValideExecutableOptionsClass ValideExecutableOptionsClass;

#define VALIDE_TYPE_IDOCUMENT (valide_idocument_get_type ())
#define VALIDE_IDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_IDOCUMENT, ValideiDocument))
#define VALIDE_IS_IDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_IDOCUMENT))
#define VALIDE_IDOCUMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), VALIDE_TYPE_IDOCUMENT, ValideiDocumentIface))

typedef struct _ValideiDocument ValideiDocument;
typedef struct _ValideiDocumentIface ValideiDocumentIface;

#define VALIDE_TYPE_DOCUMENT_STATE (valide_document_state_get_type ())

#define VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG (valide_abstract_project_dialog_get_type ())
#define VALIDE_ABSTRACT_PROJECT_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG, ValideAbstractProjectDialog))
#define VALIDE_ABSTRACT_PROJECT_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG, ValideAbstractProjectDialogClass))
#define VALIDE_IS_ABSTRACT_PROJECT_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG))
#define VALIDE_IS_ABSTRACT_PROJECT_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG))
#define VALIDE_ABSTRACT_PROJECT_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG, ValideAbstractProjectDialogClass))

typedef struct _ValideAbstractProjectDialog ValideAbstractProjectDialog;
typedef struct _ValideAbstractProjectDialogClass ValideAbstractProjectDialogClass;

#define VALIDE_TYPE_PROJECT_DIALOG (valide_project_dialog_get_type ())
#define VALIDE_PROJECT_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PROJECT_DIALOG, ValideProjectDialog))
#define VALIDE_PROJECT_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PROJECT_DIALOG, ValideProjectDialogClass))
#define VALIDE_IS_PROJECT_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PROJECT_DIALOG))
#define VALIDE_IS_PROJECT_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PROJECT_DIALOG))
#define VALIDE_PROJECT_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PROJECT_DIALOG, ValideProjectDialogClass))

typedef struct _ValideProjectDialog ValideProjectDialog;
typedef struct _ValideProjectDialogClass ValideProjectDialogClass;

#define VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER (valide_abstract_config_manager_get_type ())
#define VALIDE_ABSTRACT_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManager))
#define VALIDE_ABSTRACT_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManagerClass))
#define VALIDE_IS_ABSTRACT_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER))
#define VALIDE_IS_ABSTRACT_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER))
#define VALIDE_ABSTRACT_CONFIG_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManagerClass))

typedef struct _ValideAbstractConfigManager ValideAbstractConfigManager;
typedef struct _ValideAbstractConfigManagerClass ValideAbstractConfigManagerClass;

#define VALIDE_TYPE_CONFIG_MANAGER (valide_config_manager_get_type ())
#define VALIDE_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_CONFIG_MANAGER, ValideConfigManager))
#define VALIDE_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_CONFIG_MANAGER, ValideConfigManagerClass))
#define VALIDE_IS_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_CONFIG_MANAGER))
#define VALIDE_IS_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_CONFIG_MANAGER))
#define VALIDE_CONFIG_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_CONFIG_MANAGER, ValideConfigManagerClass))

typedef struct _ValideConfigManager ValideConfigManager;
typedef struct _ValideConfigManagerClass ValideConfigManagerClass;

typedef enum  {
	VALIDE_PROJECT_ERROR_NO_PROJECT,
	VALIDE_PROJECT_ERROR_MAX_OPEN
} ValideProjectError;
#define VALIDE_PROJECT_ERROR valide_project_error_quark ()
/**
 * The project manager
 */
struct _ValideProjectManager {
	GtkScrolledWindow parent_instance;
	ValideProjectManagerPrivate * priv;
};

struct _ValideProjectManagerClass {
	GtkScrolledWindowClass parent_class;
};

struct _ValideProjectManagerPrivate {
	GtkMenu* popup;
	ValideProject* _current;
	GtkTreeView* tree_view;
	GList* projects;
	GtkTreeStore* tree_store;
	ValideUIManager* _ui_manager;
	ValideDocumentManager* _documents;
	ValideBuilderManager* _builders;
};

typedef enum  {
	VALIDE_PROJECT_MANAGER_COL_ICON,
	VALIDE_PROJECT_MANAGER_COL_NAME,
	VALIDE_PROJECT_MANAGER_COL_PATH,
	VALIDE_PROJECT_MANAGER_COL_REAL_PATH,
	VALIDE_PROJECT_MANAGER_COL_NB
} ValideProjectManagerCol;

/**
 * Buildable GObjects, from YAML.
 *
 *
 * Objects that implements GLib.YAML.Buildable is buildable by
 * GLib.YAML.Builder.
 *
 * This interface is almost the same as GtkBuildable.
 *
 **/
struct _GYAMLBuildableIface {
	GTypeInterface parent_iface;
	const char* (*get_name) (GYAMLBuildable* self);
	void (*set_name) (GYAMLBuildable* self, const char* name);
	void (*add_child) (GYAMLBuildable* self, GYAMLBuilder* builder, GObject* child, const char* type, GError** error);
	char** (*get_child_tags) (GYAMLBuildable* self, int* result_length1);
	GType* (*get_child_types) (GYAMLBuildable* self, int* result_length1);
	GList* (*get_children) (GYAMLBuildable* self, const char* type);
	GObject* (*get_internal_child) (GYAMLBuildable* self, GYAMLBuilder* builder, const char* child_name);
	void (*custom_node) (GYAMLBuildable* self, GYAMLBuilder* builder, const char* tag, GYAMLNode* node, GError** error);
};

/**
 * A class for project
 */
struct _ValideProject {
	GObject parent_instance;
	ValideProjectPrivate * priv;
	char* filename;
	ValideBuilderManager* builders;
	GList* files;
	GList* packages;
	GList* vapi_dir;
};

struct _ValideProjectClass {
	GObjectClass parent_class;
};

struct _ValideBuilderOptionsIface {
	GTypeInterface parent_iface;
};

/**
 * A class representing a template for project
 */
struct _ValideTemplate {
	GObject parent_instance;
	ValideTemplatePrivate * priv;
	char* filename;
	GList* files;
	GList* files_vala;
	GList* files_genie;
	GList* packages;
	GList* vapi_dir;
};

struct _ValideTemplateClass {
	GObjectClass parent_class;
};

typedef enum  {
	VALIDE_DOCUMENT_STATE_NORMAL,
	VALIDE_DOCUMENT_STATE_EXTERNALLY_MODIFIED,
	VALIDE_DOCUMENT_STATE_BAD_ENCODING,
	VALIDE_DOCUMENT_STATE_INFO
} ValideDocumentState;

/**
 * An interface for document
 */
struct _ValideiDocumentIface {
	GTypeInterface parent_iface;
	void (*open) (ValideiDocument* self, const char* path, GError** error);
	void (*save) (ValideiDocument* self, const char* current_folder);
	void (*save_as) (ValideiDocument* self, const char* current_folder);
	void (*rename) (ValideiDocument* self);
	gboolean (*close) (ValideiDocument* self);
	GdkPixbuf* (*get_icon) (ValideiDocument* self);
	char* (*get_tooltips) (ValideiDocument* self);
	void (*setup_ui) (ValideiDocument* self, ValideUIManager* ui_manager);
	ValideDocumentState (*get_state) (ValideiDocument* self);
	void (*set_state) (ValideiDocument* self, ValideDocumentState value);
	gboolean (*get_is_save) (ValideiDocument* self);
	void (*set_is_save) (ValideiDocument* self, gboolean value);
	const char* (*get_path) (ValideiDocument* self);
	void (*set_path) (ValideiDocument* self, const char* value);
	const char* (*get_title) (ValideiDocument* self);
	const char* (*get_filename) (ValideiDocument* self);
};

typedef enum  {
	VALIDE_DOCUMENT_ERROR_UNKNOW_DOCUMENT,
	VALIDE_DOCUMENT_ERROR_BAD_URI,
	VALIDE_DOCUMENT_ERROR_NO_DOCUMENT,
	VALIDE_DOCUMENT_ERROR_MIME
} ValideDocumentError;
#define VALIDE_DOCUMENT_ERROR valide_document_error_quark ()

static gpointer valide_project_manager_parent_class = NULL;

GQuark valide_project_error_quark (void);
GType valide_project_manager_get_type (void);
GType valide_project_get_type (void);
GType valide_ui_manager_get_type (void);
GType valide_document_manager_get_type (void);
GType valide_builder_manager_get_type (void);
#define VALIDE_PROJECT_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_PROJECT_MANAGER, ValideProjectManagerPrivate))
enum  {
	VALIDE_PROJECT_MANAGER_DUMMY_PROPERTY,
	VALIDE_PROJECT_MANAGER_UI_MANAGER,
	VALIDE_PROJECT_MANAGER_DOCUMENTS,
	VALIDE_PROJECT_MANAGER_BUILDERS,
	VALIDE_PROJECT_MANAGER_CURRENT
};
static GType valide_project_manager_col_get_type (void);
static void _g_list_free_g_object_unref (GList* self);
ValideProject* valide_project_new (void);
ValideProject* valide_project_construct (GType object_type);
void valide_project_set_name (ValideProject* self, const char* value);
void valide_project_set_author (ValideProject* self, const char* value);
void valide_project_set_version (ValideProject* self, const char* value);
void valide_project_set_license (ValideProject* self, const char* value);
void valide_project_set_builder (ValideProject* self, const char* value);
GType g_yaml_builder_get_type (void);
gpointer g_yaml_node_ref (gpointer instance);
void g_yaml_node_unref (gpointer instance);
GParamSpec* g_yaml_param_spec_node (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void g_yaml_value_set_node (GValue* value, gpointer v_object);
gpointer g_yaml_value_get_node (const GValue* value);
GType g_yaml_node_get_type (void);
GType g_yaml_buildable_get_type (void);
GType valide_source_get_type (void);
GType valide_package_get_type (void);
GType valide_vapi_dir_get_type (void);
GType valide_builder_options_get_type (void);
GType valide_template_get_type (void);
ValideBuilderOptions* valide_template_get_builder_options (ValideTemplate* self);
void valide_project_set_builder_options (ValideProject* self, ValideBuilderOptions* value);
GType valide_executable_options_get_type (void);
void valide_executable_options_set_working_dir (ValideExecutableOptions* self, const char* value);
ValideExecutableOptions* valide_project_get_executable_options (ValideProject* self);
const char* valide_template_get_path (ValideTemplate* self);
const char* valide_source_get_path (ValideSource* self);
ValideSource* valide_source_new (const char* path);
ValideSource* valide_source_construct (GType object_type, const char* path);
void valide_project_save (ValideProject* self);
static char* valide_project_manager_copy_template (ValideProjectManager* self, ValideTemplate* template, const char* path, const char* name, const char* author, const char* version, const char* license, const char* language, const char* builder, GError** error);
ValideProject* valide_project_manager_get_current (ValideProjectManager* self);
GType valide_document_state_get_type (void);
GType valide_idocument_get_type (void);
GQuark valide_document_error_quark (void);
ValideiDocument* valide_project_open_file (ValideProject* self, const char* filename, GError** error);
static void valide_project_manager_file_select (ValideProjectManager* self, GtkTreeView* sender, const GtkTreePath* path, GtkTreeViewColumn* column);
static gboolean valide_project_manager_button_press (ValideProjectManager* self, GtkWidget* sender, const GdkEventButton* event);
ValideUIManager* valide_project_manager_get_ui_manager (ValideProjectManager* self);
GtkAction* valide_ui_manager_get_action (ValideUIManager* self, const char* name);
static void valide_project_manager_setup_ui (ValideProjectManager* self);
const char* valide_project_get_name (ValideProject* self);
char* valide_project_get_real_filename (ValideProject* self, const char* filename);
const char* valide_project_get_path (ValideProject* self);
char* valide_utils_replace_home_dir_with_tilde (const char* uri);
static GtkTreeIter* _gtk_tree_iter_dup (GtkTreeIter* self);
GdkPixbuf* valide_utils_get_pixbuf_for_file (const char* filename, GtkIconSize size, GError** error);
GdkPixbuf* valide_utils_get_pixbuf_for_stock (const char* stock_id, GtkIconSize size);
static void valide_project_manager_refresh (ValideProjectManager* self);
static gboolean valide_project_manager_drag_motion_cb (ValideProjectManager* self, GtkWidget* sender, GdkDragContext* drag_context, gint x, gint y, guint time);
static ValideProject* valide_project_manager_get_project_by_name (ValideProjectManager* self, const char* name);
static void _g_slist_free_g_free (GSList* self);
void valide_project_add_file (ValideProject* self, GSList* files);
static void valide_project_manager_drag_data_received_cb (ValideProjectManager* self, GtkWidget* sender, GdkDragContext* drag_context, gint x, gint y, GtkSelectionData* data, guint info, guint time);
gint valide_utils_cmp_filename (const char* path_a, const char* path_b);
static gint valide_project_manager_sort_func (ValideProjectManager* self, GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b);
static void valide_project_manager_set_ui_manager (ValideProjectManager* self, ValideUIManager* value);
static void valide_project_manager_set_documents (ValideProjectManager* self, ValideDocumentManager* value);
static void valide_project_manager_set_builders (ValideProjectManager* self, ValideBuilderManager* value);
ValideProjectManager* valide_project_manager_new (ValideUIManager* ui_manager, ValideDocumentManager* documents, ValideBuilderManager* builders);
ValideProjectManager* valide_project_manager_construct (GType object_type, ValideUIManager* ui_manager, ValideDocumentManager* documents, ValideBuilderManager* builders);
GType valide_abstract_project_dialog_get_type (void);
GType valide_project_dialog_get_type (void);
ValideProjectDialog* valide_project_dialog_new (void);
ValideProjectDialog* valide_project_dialog_construct (GType object_type);
guint valide_project_dialog_run (ValideProjectDialog* self);
const char* valide_project_dialog_get_project_dir (ValideProjectDialog* self);
const char* valide_project_dialog_get_project_path (ValideProjectDialog* self);
ValideTemplate* _valide_template_new (const char* filename, GError** error);
const char* valide_project_dialog_get_template (ValideProjectDialog* self);
const char* valide_project_dialog_get_project_name (ValideProjectDialog* self);
const char* valide_project_dialog_get_project_author (ValideProjectDialog* self);
const char* valide_project_dialog_get_project_version (ValideProjectDialog* self);
const char* valide_project_dialog_get_project_license (ValideProjectDialog* self);
const char* valide_project_dialog_get_lang (ValideProjectDialog* self);
const char* valide_project_dialog_get_builder (ValideProjectDialog* self);
gboolean valide_project_manager_open (ValideProjectManager* self, const char* filename, GError** error);
gboolean valide_project_manager_create (ValideProjectManager* self);
GType valide_abstract_config_manager_get_type (void);
GType valide_config_manager_get_type (void);
ValideConfigManager* valide_config_manager_get_instance (void);
char* valide_abstract_config_manager_get_string (ValideAbstractConfigManager* self, const char* group, const char* key);
ValideProject* _valide_project_new_from_filename (const char* filename, ValideDocumentManager* documents, ValideBuilderManager* builders, GError** error);
ValideDocumentManager* valide_project_manager_get_documents (ValideProjectManager* self);
ValideBuilderManager* valide_project_manager_get_builders (ValideProjectManager* self);
void valide_project_manager_close (ValideProjectManager* self, ValideProject* project);
static void _valide_project_manager_close_valide_project_closed (ValideProject* _sender, gpointer self);
static void _valide_project_manager_refresh_valide_project_added_file (ValideProject* _sender, gpointer self);
static void _valide_project_manager_refresh_valide_project_removed_file (ValideProject* _sender, gpointer self);
static void _lambda38_ (ValideProject* s, ValideProjectManager* self);
static void __lambda38__valide_project_options_changed (ValideProject* _sender, gpointer self);
void valide_project_manager_set_current (ValideProjectManager* self, ValideProject* value);
void valide_project_manager_open_selected_file (ValideProjectManager* self);
static void _g_list_free_g_free (GList* self);
void valide_project_remove_file (ValideProject* self, GList* files);
void valide_project_manager_remove_selected_file (ValideProjectManager* self);
char* valide_utils_get_extension (const char* filename);
gboolean valide_project_manager_accept_file (ValideProjectManager* self, const char* filename);
static gint _valide_project_manager_sort_func_gtk_tree_iter_compare_func (GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b, gpointer self);
static void _valide_project_manager_file_select_gtk_tree_view_row_activated (GtkTreeView* _sender, const GtkTreePath* path, GtkTreeViewColumn* column, gpointer self);
static gboolean _valide_project_manager_button_press_gtk_widget_button_press_event (GtkWidget* _sender, const GdkEventButton* event, gpointer self);
static gboolean _valide_project_manager_drag_motion_cb_gtk_widget_drag_motion (GtkWidget* _sender, GdkDragContext* context, gint x, gint y, guint time_, gpointer self);
static void _valide_project_manager_drag_data_received_cb_gtk_widget_drag_data_received (GtkWidget* _sender, GdkDragContext* context, gint x, gint y, GtkSelectionData* selection_data, guint info, guint time_, gpointer self);
static GObject * valide_project_manager_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void valide_project_manager_finalize (GObject* obj);
static void valide_project_manager_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_project_manager_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);

static const char* VALIDE_PROJECT_FILE_EXT[] = {"vide"};


GQuark valide_project_error_quark (void) {
	return g_quark_from_static_string ("valide_project_error-quark");
}



static GType valide_project_manager_col_get_type (void) {
	static GType valide_project_manager_col_type_id = 0;
	if (G_UNLIKELY (valide_project_manager_col_type_id == 0)) {
		static const GEnumValue values[] = {{VALIDE_PROJECT_MANAGER_COL_ICON, "VALIDE_PROJECT_MANAGER_COL_ICON", "icon"}, {VALIDE_PROJECT_MANAGER_COL_NAME, "VALIDE_PROJECT_MANAGER_COL_NAME", "name"}, {VALIDE_PROJECT_MANAGER_COL_PATH, "VALIDE_PROJECT_MANAGER_COL_PATH", "path"}, {VALIDE_PROJECT_MANAGER_COL_REAL_PATH, "VALIDE_PROJECT_MANAGER_COL_REAL_PATH", "real-path"}, {VALIDE_PROJECT_MANAGER_COL_NB, "VALIDE_PROJECT_MANAGER_COL_NB", "nb"}, {0, NULL, NULL}};
		valide_project_manager_col_type_id = g_enum_register_static ("ValideProjectManagerCol", values);
	}
	return valide_project_manager_col_type_id;
}


static void _g_list_free_g_object_unref (GList* self) {
	g_list_foreach (self, (GFunc) g_object_unref, NULL);
	g_list_free (self);
}


#line 94 "project-manager.vala"
static char* valide_project_manager_copy_template (ValideProjectManager* self, ValideTemplate* template, const char* path, const char* name, const char* author, const char* version, const char* license, const char* language, const char* builder, GError** error) {
#line 519 "project-manager.c"
	char* result;
	GError * _inner_error_;
	char* contents;
	ValideProject* project;
	ValideProject* _tmp0_;
	char* _tmp3_;
	char* _tmp2_;
	char* _tmp1_;
	GList* _tmp4_;
	GList* _tmp5_;
	GList* _tmp6_;
	const char* _tmp18_;
#line 94 "project-manager.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 94 "project-manager.vala"
	g_return_val_if_fail (template != NULL, NULL);
#line 94 "project-manager.vala"
	g_return_val_if_fail (path != NULL, NULL);
#line 94 "project-manager.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 94 "project-manager.vala"
	g_return_val_if_fail (author != NULL, NULL);
#line 94 "project-manager.vala"
	g_return_val_if_fail (version != NULL, NULL);
#line 94 "project-manager.vala"
	g_return_val_if_fail (license != NULL, NULL);
#line 94 "project-manager.vala"
	g_return_val_if_fail (language != NULL, NULL);
#line 94 "project-manager.vala"
	g_return_val_if_fail (builder != NULL, NULL);
#line 550 "project-manager.c"
	_inner_error_ = NULL;
	contents = NULL;
	project = NULL;
	_tmp0_ = NULL;
#line 102 "project-manager.vala"
	project = (_tmp0_ = valide_project_new (), (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), _tmp0_);
#line 103 "project-manager.vala"
	valide_project_set_name (project, name);
#line 104 "project-manager.vala"
	valide_project_set_author (project, author);
#line 105 "project-manager.vala"
	valide_project_set_version (project, version);
#line 106 "project-manager.vala"
	valide_project_set_license (project, license);
#line 107 "project-manager.vala"
	valide_project_set_builder (project, builder);
#line 567 "project-manager.c"
	_tmp3_ = NULL;
	_tmp2_ = NULL;
	_tmp1_ = NULL;
#line 108 "project-manager.vala"
	project->filename = (_tmp3_ = g_build_filename (path, _tmp2_ = g_strconcat (_tmp1_ = g_strconcat (name, ".", NULL), VALIDE_PROJECT_FILE_EXT[0], NULL), NULL), project->filename = (g_free (project->filename), NULL), _tmp3_);
#line 573 "project-manager.c"
	_tmp2_ = (g_free (_tmp2_), NULL);
	_tmp1_ = (g_free (_tmp1_), NULL);
#line 109 "project-manager.vala"
	valide_project_set_builder_options (project, valide_template_get_builder_options (template));
#line 578 "project-manager.c"
	_tmp4_ = NULL;
#line 110 "project-manager.vala"
	project->packages = (_tmp4_ = g_list_copy (template->packages), (project->packages == NULL) ? NULL : (project->packages = (_g_list_free_g_object_unref (project->packages), NULL)), _tmp4_);
#line 582 "project-manager.c"
	_tmp5_ = NULL;
#line 111 "project-manager.vala"
	project->vapi_dir = (_tmp5_ = g_list_copy (template->vapi_dir), (project->vapi_dir == NULL) ? NULL : (project->vapi_dir = (_g_list_free_g_object_unref (project->vapi_dir), NULL)), _tmp5_);
#line 586 "project-manager.c"
	_tmp6_ = NULL;
#line 112 "project-manager.vala"
	project->files = (_tmp6_ = g_list_copy (template->files), (project->files == NULL) ? NULL : (project->files = (_g_list_free_g_object_unref (project->files), NULL)), _tmp6_);
#line 113 "project-manager.vala"
	valide_executable_options_set_working_dir (valide_project_get_executable_options (project), path);
#line 114 "project-manager.vala"
	if (_vala_strcmp0 (language, "vala") == 0) {
#line 116 "project-manager.vala"
		project->files = g_list_concat (project->files, g_list_copy (template->files_vala));
#line 596 "project-manager.c"
	} else {
#line 118 "project-manager.vala"
		if (_vala_strcmp0 (language, "genie") == 0) {
#line 120 "project-manager.vala"
			project->files = g_list_concat (project->files, g_list_copy (template->files_genie));
#line 602 "project-manager.c"
		}
	}
	/* Copy source files*/
	{
		GList* source_collection;
		GList* source_it;
#line 124 "project-manager.vala"
		source_collection = project->files;
#line 611 "project-manager.c"
		for (source_it = source_collection; source_it != NULL; source_it = source_it->next) {
			ValideSource* _tmp12_;
			ValideSource* source;
#line 72 "project.vala"
			_tmp12_ = NULL;
#line 124 "project-manager.vala"
			source = (_tmp12_ = (ValideSource*) source_it->data, (_tmp12_ == NULL) ? NULL : g_object_ref (_tmp12_));
#line 619 "project-manager.c"
			{
				GFile* src;
				GFile* dst;
				GFile* _tmp8_;
				char* _tmp7_;
				GFile* _tmp11_;
				char* _tmp10_;
				char* _tmp9_;
				src = NULL;
				dst = NULL;
				_tmp8_ = NULL;
#line 2034 "glib-2.0.vapi"
				_tmp7_ = NULL;
#line 129 "project-manager.vala"
				src = (_tmp8_ = g_file_new_for_path (_tmp7_ = g_build_filename (valide_template_get_path (template), valide_source_get_path (source), NULL)), (src == NULL) ? NULL : (src = (g_object_unref (src), NULL)), _tmp8_);
#line 635 "project-manager.c"
				_tmp7_ = (g_free (_tmp7_), NULL);
				_tmp11_ = NULL;
#line 2034 "glib-2.0.vapi"
				_tmp10_ = NULL;
#line 2031 "glib-2.0.vapi"
				_tmp9_ = NULL;
#line 130 "project-manager.vala"
				dst = (_tmp11_ = g_file_new_for_path (_tmp10_ = g_build_filename (path, _tmp9_ = g_path_get_basename (valide_source_get_path (source)), NULL)), (dst == NULL) ? NULL : (dst = (g_object_unref (dst), NULL)), _tmp11_);
#line 644 "project-manager.c"
				_tmp10_ = (g_free (_tmp10_), NULL);
				_tmp9_ = (g_free (_tmp9_), NULL);
#line 131 "project-manager.vala"
				g_file_copy (src, dst, G_FILE_COPY_OVERWRITE, NULL, NULL, NULL, &_inner_error_);
#line 649 "project-manager.c"
				if (_inner_error_ != NULL) {
					g_propagate_error (error, _inner_error_);
					(source == NULL) ? NULL : (source = (g_object_unref (source), NULL));
					(src == NULL) ? NULL : (src = (g_object_unref (src), NULL));
					(dst == NULL) ? NULL : (dst = (g_object_unref (dst), NULL));
					contents = (g_free (contents), NULL);
					(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
					return NULL;
				}
				(source == NULL) ? NULL : (source = (g_object_unref (source), NULL));
				(src == NULL) ? NULL : (src = (g_object_unref (src), NULL));
				(dst == NULL) ? NULL : (dst = (g_object_unref (dst), NULL));
			}
		}
	}
	/* Copy license*/
#line 135 "project-manager.vala"
	if (_vala_strcmp0 (license, "None") != 0) {
#line 668 "project-manager.c"
		char* _tmp16_;
		gboolean _tmp15_;
		char* _tmp14_;
		char* _tmp13_;
		char* _tmp17_;
		_tmp16_ = NULL;
		_tmp14_ = NULL;
#line 2034 "glib-2.0.vapi"
		_tmp13_ = NULL;
#line 137 "project-manager.vala"
		_tmp15_ = g_file_get_contents (_tmp13_ = g_build_filename (LICENSES_DIR, license, NULL), &_tmp14_, NULL, &_inner_error_);
#line 137 "project-manager.vala"
		contents = (_tmp16_ = _tmp14_, contents = (g_free (contents), NULL), _tmp16_);
#line 137 "project-manager.vala"
		_tmp15_;
#line 684 "project-manager.c"
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			contents = (g_free (contents), NULL);
			(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
			return NULL;
		}
		_tmp13_ = (g_free (_tmp13_), NULL);
#line 2034 "glib-2.0.vapi"
		_tmp17_ = NULL;
#line 139 "project-manager.vala"
		g_file_set_contents (_tmp17_ = g_build_filename (path, "COPYING", NULL), contents, (glong) (-1), &_inner_error_);
#line 696 "project-manager.c"
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			contents = (g_free (contents), NULL);
			(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
			return NULL;
		}
		_tmp17_ = (g_free (_tmp17_), NULL);
#line 142 "project-manager.vala"
		project->files = g_list_append (project->files, valide_source_new ("COPYING"));
#line 706 "project-manager.c"
	}
#line 144 "project-manager.vala"
	valide_project_save (project);
#line 145 "project-manager.vala"
	_tmp18_ = NULL;
#line 712 "project-manager.c"
	result = (_tmp18_ = project->filename, (_tmp18_ == NULL) ? NULL : g_strdup (_tmp18_));
	contents = (g_free (contents), NULL);
	(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
	return result;
	contents = (g_free (contents), NULL);
	(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
}


#line 164 "project-manager.vala"
static void valide_project_manager_file_select (ValideProjectManager* self, GtkTreeView* sender, const GtkTreePath* path, GtkTreeViewColumn* column) {
#line 724 "project-manager.c"
	GError * _inner_error_;
	GtkTreeIter iter = {0};
	char* filename;
#line 164 "project-manager.vala"
	g_return_if_fail (self != NULL);
#line 164 "project-manager.vala"
	g_return_if_fail (sender != NULL);
#line 164 "project-manager.vala"
	g_return_if_fail (path != NULL);
#line 164 "project-manager.vala"
	g_return_if_fail (column != NULL);
#line 736 "project-manager.c"
	_inner_error_ = NULL;
	filename = NULL;
#line 170 "project-manager.vala"
	gtk_tree_model_get_iter ((GtkTreeModel*) self->priv->tree_store, &iter, path);
#line 171 "project-manager.vala"
	gtk_tree_model_get ((GtkTreeModel*) self->priv->tree_store, &iter, VALIDE_PROJECT_MANAGER_COL_REAL_PATH, &filename, -1);
#line 172 "project-manager.vala"
	if (!gtk_tree_model_iter_has_child ((GtkTreeModel*) self->priv->tree_store, &iter)) {
#line 745 "project-manager.c"
		{
			ValideiDocument* _tmp0_;
#line 534 "project.vala"
			_tmp0_ = NULL;
#line 176 "project-manager.vala"
			_tmp0_ = valide_project_open_file (valide_project_manager_get_current (self), filename, &_inner_error_);
#line 752 "project-manager.c"
			if (_inner_error_ != NULL) {
				goto __catch49_g_error;
				goto __finally49;
			}
			(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
		}
		goto __finally49;
		__catch49_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
#line 180 "project-manager.vala"
				g_warning (e->message);
#line 768 "project-manager.c"
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally49:
		if (_inner_error_ != NULL) {
			filename = (g_free (filename), NULL);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	} else {
		/* @todo Active project
		 @todo Unexpand other projects*/
#line 188 "project-manager.vala"
		if (gtk_tree_view_row_expanded (self->priv->tree_view, path)) {
#line 190 "project-manager.vala"
			gtk_tree_view_collapse_row (self->priv->tree_view, path);
#line 786 "project-manager.c"
		} else {
#line 194 "project-manager.vala"
			gtk_tree_view_expand_row (self->priv->tree_view, path, FALSE);
#line 790 "project-manager.c"
		}
	}
	filename = (g_free (filename), NULL);
}


#line 199 "project-manager.vala"
static gboolean valide_project_manager_button_press (ValideProjectManager* self, GtkWidget* sender, const GdkEventButton* event) {
#line 799 "project-manager.c"
	gboolean result;
	gboolean _tmp0_;
#line 199 "project-manager.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 199 "project-manager.vala"
	g_return_val_if_fail (sender != NULL, FALSE);
#line 806 "project-manager.c"
	_tmp0_ = FALSE;
#line 201 "project-manager.vala"
	if ((*event).type == GDK_BUTTON_PRESS) {
#line 201 "project-manager.vala"
		_tmp0_ = (*event).button == 3;
#line 812 "project-manager.c"
	} else {
#line 201 "project-manager.vala"
		_tmp0_ = FALSE;
#line 816 "project-manager.c"
	}
#line 201 "project-manager.vala"
	if (_tmp0_) {
#line 203 "project-manager.vala"
		gtk_menu_popup (self->priv->popup, NULL, NULL, NULL, NULL, (*event).button, (*event).time);
#line 822 "project-manager.c"
		result = TRUE;
		return result;
	}
	result = FALSE;
	return result;
}


#line 209 "project-manager.vala"
static void valide_project_manager_setup_ui (ValideProjectManager* self) {
#line 833 "project-manager.c"
	gboolean active;
	GtkAction* _tmp0_;
	GtkAction* _tmp1_;
	GtkAction* _tmp2_;
	GtkAction* _tmp3_;
	GtkAction* _tmp4_;
	GtkAction* _tmp5_;
	GtkAction* _tmp6_;
	GtkAction* _tmp7_;
	GtkAction* _tmp8_;
	GtkAction* _tmp9_;
	GtkAction* _tmp10_;
	GtkAction* _tmp11_;
#line 209 "project-manager.vala"
	g_return_if_fail (self != NULL);
#line 849 "project-manager.c"
	active = FALSE;
#line 213 "project-manager.vala"
	if (valide_project_manager_get_current (self) != NULL) {
#line 215 "project-manager.vala"
		active = TRUE;
#line 855 "project-manager.c"
	}
#line 128 "uimanager.vala"
	_tmp0_ = NULL;
#line 217 "project-manager.vala"
	gtk_action_set_sensitive (_tmp0_ = valide_ui_manager_get_action (self->priv->_ui_manager, "project-add"), active);
#line 861 "project-manager.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
#line 128 "uimanager.vala"
	_tmp1_ = NULL;
#line 218 "project-manager.vala"
	gtk_action_set_sensitive (_tmp1_ = valide_ui_manager_get_action (self->priv->_ui_manager, "project-remove"), active);
#line 867 "project-manager.c"
	(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
#line 128 "uimanager.vala"
	_tmp2_ = NULL;
#line 219 "project-manager.vala"
	gtk_action_set_sensitive (_tmp2_ = valide_ui_manager_get_action (self->priv->_ui_manager, "project-options"), active);
#line 873 "project-manager.c"
	(_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL));
#line 128 "uimanager.vala"
	_tmp3_ = NULL;
#line 220 "project-manager.vala"
	gtk_action_set_sensitive (_tmp3_ = valide_ui_manager_get_action (self->priv->_ui_manager, "project-configure"), active);
#line 879 "project-manager.c"
	(_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL));
#line 128 "uimanager.vala"
	_tmp4_ = NULL;
#line 221 "project-manager.vala"
	gtk_action_set_sensitive (_tmp4_ = valide_ui_manager_get_action (self->priv->_ui_manager, "project-build"), active);
#line 885 "project-manager.c"
	(_tmp4_ == NULL) ? NULL : (_tmp4_ = (g_object_unref (_tmp4_), NULL));
#line 128 "uimanager.vala"
	_tmp5_ = NULL;
#line 222 "project-manager.vala"
	gtk_action_set_sensitive (_tmp5_ = valide_ui_manager_get_action (self->priv->_ui_manager, "project-install"), active);
#line 891 "project-manager.c"
	(_tmp5_ == NULL) ? NULL : (_tmp5_ = (g_object_unref (_tmp5_), NULL));
#line 128 "uimanager.vala"
	_tmp6_ = NULL;
#line 223 "project-manager.vala"
	gtk_action_set_sensitive (_tmp6_ = valide_ui_manager_get_action (self->priv->_ui_manager, "project-dist"), active);
#line 897 "project-manager.c"
	(_tmp6_ == NULL) ? NULL : (_tmp6_ = (g_object_unref (_tmp6_), NULL));
#line 128 "uimanager.vala"
	_tmp7_ = NULL;
#line 224 "project-manager.vala"
	gtk_action_set_sensitive (_tmp7_ = valide_ui_manager_get_action (self->priv->_ui_manager, "project-clean"), active);
#line 903 "project-manager.c"
	(_tmp7_ == NULL) ? NULL : (_tmp7_ = (g_object_unref (_tmp7_), NULL));
#line 128 "uimanager.vala"
	_tmp8_ = NULL;
#line 225 "project-manager.vala"
	gtk_action_set_sensitive (_tmp8_ = valide_ui_manager_get_action (self->priv->_ui_manager, "project-distclean"), active);
#line 909 "project-manager.c"
	(_tmp8_ == NULL) ? NULL : (_tmp8_ = (g_object_unref (_tmp8_), NULL));
#line 128 "uimanager.vala"
	_tmp9_ = NULL;
#line 226 "project-manager.vala"
	gtk_action_set_sensitive (_tmp9_ = valide_ui_manager_get_action (self->priv->_ui_manager, "project-uninstall"), active);
#line 915 "project-manager.c"
	(_tmp9_ == NULL) ? NULL : (_tmp9_ = (g_object_unref (_tmp9_), NULL));
#line 128 "uimanager.vala"
	_tmp10_ = NULL;
#line 227 "project-manager.vala"
	gtk_action_set_sensitive (_tmp10_ = valide_ui_manager_get_action (self->priv->_ui_manager, "project-execute"), active);
#line 921 "project-manager.c"
	(_tmp10_ == NULL) ? NULL : (_tmp10_ = (g_object_unref (_tmp10_), NULL));
#line 128 "uimanager.vala"
	_tmp11_ = NULL;
#line 228 "project-manager.vala"
	gtk_action_set_sensitive (_tmp11_ = valide_ui_manager_get_action (self->priv->_ui_manager, "project-close"), active);
#line 927 "project-manager.c"
	(_tmp11_ == NULL) ? NULL : (_tmp11_ = (g_object_unref (_tmp11_), NULL));
#line 229 "project-manager.vala"
	g_object_set ((GtkWidget*) self, "sensitive", active, NULL);
#line 931 "project-manager.c"
}


#line 922 "glib-2.0.vapi"
static char* string_substring (const char* self, glong offset, glong len) {
#line 937 "project-manager.c"
	char* result;
	glong string_length;
	const char* start;
#line 922 "glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 943 "project-manager.c"
	string_length = g_utf8_strlen (self, -1);
#line 924 "glib-2.0.vapi"
	if (offset < 0) {
#line 925 "glib-2.0.vapi"
		offset = string_length + offset;
#line 926 "glib-2.0.vapi"
		g_return_val_if_fail (offset >= 0, NULL);
#line 951 "project-manager.c"
	} else {
#line 928 "glib-2.0.vapi"
		g_return_val_if_fail (offset <= string_length, NULL);
#line 955 "project-manager.c"
	}
#line 930 "glib-2.0.vapi"
	if (len < 0) {
#line 931 "glib-2.0.vapi"
		len = string_length - offset;
#line 961 "project-manager.c"
	}
#line 933 "glib-2.0.vapi"
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
#line 965 "project-manager.c"
	start = g_utf8_offset_to_pointer (self, offset);
	result = g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
	return result;
}


static glong string_get_length (const char* self) {
	glong result;
	g_return_val_if_fail (self != NULL, 0L);
	result = g_utf8_strlen (self, -1);
	return result;
}


static GtkTreeIter* _gtk_tree_iter_dup (GtkTreeIter* self) {
	GtkTreeIter* dup;
	dup = g_new0 (GtkTreeIter, 1);
	memcpy (dup, self, sizeof (GtkTreeIter));
	return dup;
}


/* @TODO Save all expanded rows (childrens of root) */
#line 233 "project-manager.vala"
static void valide_project_manager_refresh (ValideProjectManager* self) {
#line 991 "project-manager.c"
	GError * _inner_error_;
	gboolean expanded;
	gboolean was_empty;
	GtkTreePath* tree_path;
	GtkTreeIter iter = {0};
	GHashTable* dir_iters;
	GtkTreePath* _tmp0_;
	gboolean _tmp28_;
	gboolean _tmp29_;
#line 233 "project-manager.vala"
	g_return_if_fail (self != NULL);
#line 1003 "project-manager.c"
	_inner_error_ = NULL;
	expanded = FALSE;
	was_empty = TRUE;
	tree_path = NULL;
	dir_iters = NULL;
	_tmp0_ = NULL;
#line 241 "project-manager.vala"
	tree_path = (_tmp0_ = gtk_tree_path_new (), (tree_path == NULL) ? NULL : (tree_path = (gtk_tree_path_free (tree_path), NULL)), _tmp0_);
#line 1012 "project-manager.c"
	/* Save expanded rows */
#line 243 "project-manager.vala"
	if (gtk_tree_model_get_iter_first ((GtkTreeModel*) self->priv->tree_store, &iter)) {
#line 1016 "project-manager.c"
		GtkTreePath* _tmp1_;
		_tmp1_ = NULL;
#line 245 "project-manager.vala"
		tree_path = (_tmp1_ = gtk_tree_model_get_path ((GtkTreeModel*) self->priv->tree_store, &iter), (tree_path == NULL) ? NULL : (tree_path = (gtk_tree_path_free (tree_path), NULL)), _tmp1_);
#line 246 "project-manager.vala"
		expanded = gtk_tree_view_row_expanded (self->priv->tree_view, tree_path);
#line 247 "project-manager.vala"
		was_empty = FALSE;
#line 1025 "project-manager.c"
	} else {
#line 251 "project-manager.vala"
		expanded = FALSE;
#line 252 "project-manager.vala"
		was_empty = TRUE;
#line 1031 "project-manager.c"
	}
#line 255 "project-manager.vala"
	gtk_tree_store_clear (self->priv->tree_store);
#line 1035 "project-manager.c"
	{
		GList* p_collection;
		GList* p_it;
#line 256 "project-manager.vala"
		p_collection = self->priv->projects;
#line 1041 "project-manager.c"
		for (p_it = p_collection; p_it != NULL; p_it = p_it->next) {
			ValideProject* _tmp27_;
			ValideProject* p;
#line 45 "project-manager.vala"
			_tmp27_ = NULL;
#line 256 "project-manager.vala"
			p = (_tmp27_ = (ValideProject*) p_it->data, (_tmp27_ == NULL) ? NULL : g_object_ref (_tmp27_));
#line 1049 "project-manager.c"
			{
				GtkTreeIter root = {0};
				GHashTable* _tmp4_;
#line 260 "project-manager.vala"
				gtk_tree_store_append (self->priv->tree_store, &root, NULL);
#line 262 "project-manager.vala"
				if (p == valide_project_manager_get_current (self)) {
#line 1057 "project-manager.c"
					char* _tmp3_;
					char* _tmp2_;
					_tmp3_ = NULL;
					_tmp2_ = NULL;
#line 264 "project-manager.vala"
					gtk_tree_store_set (self->priv->tree_store, &root, VALIDE_PROJECT_MANAGER_COL_NAME, _tmp3_ = g_strconcat (_tmp2_ = g_strconcat ("<b>", valide_project_get_name (p), NULL), "</b>", NULL), -1, -1);
#line 1064 "project-manager.c"
					_tmp3_ = (g_free (_tmp3_), NULL);
					_tmp2_ = (g_free (_tmp2_), NULL);
				} else {
#line 268 "project-manager.vala"
					gtk_tree_store_set (self->priv->tree_store, &root, VALIDE_PROJECT_MANAGER_COL_NAME, valide_project_get_name (p), -1, -1);
#line 1070 "project-manager.c"
				}
				_tmp4_ = NULL;
#line 271 "project-manager.vala"
				dir_iters = (_tmp4_ = g_hash_table_new (g_str_hash, g_str_equal), (dir_iters == NULL) ? NULL : (dir_iters = (g_hash_table_unref (dir_iters), NULL)), _tmp4_);
#line 1075 "project-manager.c"
				{
					GList* file_collection;
					GList* file_it;
#line 272 "project-manager.vala"
					file_collection = p->files;
#line 1081 "project-manager.c"
					for (file_it = file_collection; file_it != NULL; file_it = file_it->next) {
						ValideSource* _tmp26_;
						ValideSource* file;
#line 72 "project.vala"
						_tmp26_ = NULL;
#line 272 "project-manager.vala"
						file = (_tmp26_ = (ValideSource*) file_it->data, (_tmp26_ == NULL) ? NULL : g_object_ref (_tmp26_));
#line 1089 "project-manager.c"
						{
							char* dir;
							char* path;
							char* dirname;
							char* filename;
							GdkPixbuf* pixbuf;
							GtkTreeIter* dir_iter;
							char* _tmp5_;
							char* _tmp6_;
							char* _tmp7_;
							gboolean _tmp8_;
							gboolean _tmp11_;
							dir = NULL;
							path = NULL;
							dirname = NULL;
							filename = NULL;
							pixbuf = NULL;
							dir_iter = NULL;
							_tmp5_ = NULL;
#line 281 "project-manager.vala"
							path = (_tmp5_ = valide_project_get_real_filename (p, valide_source_get_path (file)), path = (g_free (path), NULL), _tmp5_);
#line 1111 "project-manager.c"
							_tmp6_ = NULL;
#line 282 "project-manager.vala"
							dir = (_tmp6_ = g_path_get_dirname (path), dir = (g_free (dir), NULL), _tmp6_);
#line 1115 "project-manager.c"
							_tmp7_ = NULL;
#line 283 "project-manager.vala"
							filename = (_tmp7_ = g_path_get_basename (valide_source_get_path (file)), filename = (g_free (filename), NULL), _tmp7_);
#line 1119 "project-manager.c"
							_tmp8_ = FALSE;
#line 284 "project-manager.vala"
							if (g_str_has_prefix (dir, valide_project_get_path (p))) {
#line 284 "project-manager.vala"
								_tmp8_ = _vala_strcmp0 (dir, valide_project_get_path (p)) != 0;
#line 1125 "project-manager.c"
							} else {
#line 284 "project-manager.vala"
								_tmp8_ = FALSE;
#line 1129 "project-manager.c"
							}
#line 284 "project-manager.vala"
							if (_tmp8_) {
#line 1133 "project-manager.c"
								char* _tmp9_;
								_tmp9_ = NULL;
#line 286 "project-manager.vala"
								dirname = (_tmp9_ = string_substring (dir, string_get_length (valide_project_get_path (p)) + 1, -1), dirname = (g_free (dirname), NULL), _tmp9_);
#line 1138 "project-manager.c"
							} else {
								char* _tmp10_;
								_tmp10_ = NULL;
#line 290 "project-manager.vala"
								dirname = (_tmp10_ = valide_utils_replace_home_dir_with_tilde (dir), dirname = (g_free (dirname), NULL), _tmp10_);
#line 1144 "project-manager.c"
							}
							_tmp11_ = FALSE;
#line 293 "project-manager.vala"
							if (_vala_strcmp0 (dir, ".") == 0) {
#line 293 "project-manager.vala"
								_tmp11_ = TRUE;
#line 1151 "project-manager.c"
							} else {
#line 293 "project-manager.vala"
								_tmp11_ = _vala_strcmp0 (dir, valide_project_get_path (p)) == 0;
#line 1155 "project-manager.c"
							}
#line 293 "project-manager.vala"
							if (_tmp11_) {
#line 1159 "project-manager.c"
								GtkTreeIter* _tmp13_;
								GtkTreeIter* _tmp12_;
								_tmp13_ = NULL;
#line 295 "project-manager.vala"
								_tmp12_ = NULL;
#line 295 "project-manager.vala"
								dir_iter = (_tmp13_ = (_tmp12_ = &root, (_tmp12_ == NULL) ? NULL : _gtk_tree_iter_dup (_tmp12_)), (dir_iter == NULL) ? NULL : (dir_iter = (g_free (dir_iter), NULL)), _tmp13_);
#line 1167 "project-manager.c"
							} else {
								GtkTreeIter* _tmp15_;
								GtkTreeIter* _tmp14_;
								_tmp15_ = NULL;
#line 299 "project-manager.vala"
								_tmp14_ = NULL;
#line 299 "project-manager.vala"
								dir_iter = (_tmp15_ = (_tmp14_ = (GtkTreeIter*) g_hash_table_lookup (dir_iters, dir), (_tmp14_ == NULL) ? NULL : _gtk_tree_iter_dup (_tmp14_)), (dir_iter == NULL) ? NULL : (dir_iter = (g_free (dir_iter), NULL)), _tmp15_);
#line 301 "project-manager.vala"
								if (dir_iter == NULL) {
#line 1178 "project-manager.c"
									GtkTreeIter tmp = {0};
									GtkTreeIter* _tmp17_;
									GtkTreeIter* _tmp16_;
									GtkTreeIter* _tmp22_;
									const char* _tmp21_;
#line 305 "project-manager.vala"
									gtk_tree_store_append (self->priv->tree_store, &tmp, &root);
#line 1186 "project-manager.c"
									_tmp17_ = NULL;
#line 306 "project-manager.vala"
									_tmp16_ = NULL;
#line 306 "project-manager.vala"
									dir_iter = (_tmp17_ = (_tmp16_ = &tmp, (_tmp16_ == NULL) ? NULL : _gtk_tree_iter_dup (_tmp16_)), (dir_iter == NULL) ? NULL : (dir_iter = (g_free (dir_iter), NULL)), _tmp17_);
#line 1192 "project-manager.c"
									{
										GdkPixbuf* _tmp18_;
										GdkPixbuf* _tmp19_;
										_tmp18_ = valide_utils_get_pixbuf_for_file (dir, GTK_ICON_SIZE_MENU, &_inner_error_);
										if (_inner_error_ != NULL) {
											goto __catch50_g_error;
											goto __finally50;
										}
										_tmp19_ = NULL;
#line 309 "project-manager.vala"
										pixbuf = (_tmp19_ = _tmp18_, (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp19_);
#line 1204 "project-manager.c"
									}
									goto __finally50;
									__catch50_g_error:
									{
										GError * e;
										e = _inner_error_;
										_inner_error_ = NULL;
										{
											GdkPixbuf* _tmp20_;
											_tmp20_ = NULL;
#line 313 "project-manager.vala"
											pixbuf = (_tmp20_ = valide_utils_get_pixbuf_for_stock (GTK_STOCK_CANCEL, GTK_ICON_SIZE_MENU), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp20_);
#line 1217 "project-manager.c"
											(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
										}
									}
									__finally50:
									if (_inner_error_ != NULL) {
										(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
										dir = (g_free (dir), NULL);
										path = (g_free (path), NULL);
										dirname = (g_free (dirname), NULL);
										filename = (g_free (filename), NULL);
										(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
										(dir_iter == NULL) ? NULL : (dir_iter = (g_free (dir_iter), NULL));
										(p == NULL) ? NULL : (p = (g_object_unref (p), NULL));
										(tree_path == NULL) ? NULL : (tree_path = (gtk_tree_path_free (tree_path), NULL));
										(dir_iters == NULL) ? NULL : (dir_iters = (g_hash_table_unref (dir_iters), NULL));
										g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
										g_clear_error (&_inner_error_);
										return;
									}
#line 315 "project-manager.vala"
									gtk_tree_store_set (self->priv->tree_store, dir_iter, VALIDE_PROJECT_MANAGER_COL_ICON, pixbuf, VALIDE_PROJECT_MANAGER_COL_NAME, dirname, VALIDE_PROJECT_MANAGER_COL_PATH, dir, VALIDE_PROJECT_MANAGER_COL_REAL_PATH, dir, -1);
#line 317 "project-manager.vala"
									_tmp22_ = NULL;
#line 317 "project-manager.vala"
									_tmp21_ = NULL;
#line 317 "project-manager.vala"
									g_hash_table_insert (dir_iters, (_tmp21_ = dir, (_tmp21_ == NULL) ? NULL : g_strdup (_tmp21_)), (_tmp22_ = dir_iter, (_tmp22_ == NULL) ? NULL : _gtk_tree_iter_dup (_tmp22_)));
#line 1245 "project-manager.c"
								}
							}
#line 320 "project-manager.vala"
							gtk_tree_store_append (self->priv->tree_store, &iter, dir_iter);
#line 1250 "project-manager.c"
							{
								GdkPixbuf* _tmp23_;
								GdkPixbuf* _tmp24_;
								_tmp23_ = valide_utils_get_pixbuf_for_file (path, GTK_ICON_SIZE_MENU, &_inner_error_);
								if (_inner_error_ != NULL) {
									goto __catch51_g_error;
									goto __finally51;
								}
								_tmp24_ = NULL;
#line 323 "project-manager.vala"
								pixbuf = (_tmp24_ = _tmp23_, (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp24_);
#line 1262 "project-manager.c"
							}
							goto __finally51;
							__catch51_g_error:
							{
								GError * e;
								e = _inner_error_;
								_inner_error_ = NULL;
								{
									GdkPixbuf* _tmp25_;
									_tmp25_ = NULL;
#line 327 "project-manager.vala"
									pixbuf = (_tmp25_ = valide_utils_get_pixbuf_for_stock (GTK_STOCK_CANCEL, GTK_ICON_SIZE_MENU), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp25_);
#line 1275 "project-manager.c"
									(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
								}
							}
							__finally51:
							if (_inner_error_ != NULL) {
								(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
								dir = (g_free (dir), NULL);
								path = (g_free (path), NULL);
								dirname = (g_free (dirname), NULL);
								filename = (g_free (filename), NULL);
								(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
								(dir_iter == NULL) ? NULL : (dir_iter = (g_free (dir_iter), NULL));
								(p == NULL) ? NULL : (p = (g_object_unref (p), NULL));
								(tree_path == NULL) ? NULL : (tree_path = (gtk_tree_path_free (tree_path), NULL));
								(dir_iters == NULL) ? NULL : (dir_iters = (g_hash_table_unref (dir_iters), NULL));
								g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
								g_clear_error (&_inner_error_);
								return;
							}
#line 329 "project-manager.vala"
							gtk_tree_store_set (self->priv->tree_store, &iter, VALIDE_PROJECT_MANAGER_COL_NAME, filename, VALIDE_PROJECT_MANAGER_COL_ICON, pixbuf, VALIDE_PROJECT_MANAGER_COL_PATH, valide_source_get_path (file), VALIDE_PROJECT_MANAGER_COL_REAL_PATH, path, -1);
#line 1297 "project-manager.c"
							(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
							dir = (g_free (dir), NULL);
							path = (g_free (path), NULL);
							dirname = (g_free (dirname), NULL);
							filename = (g_free (filename), NULL);
							(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
							(dir_iter == NULL) ? NULL : (dir_iter = (g_free (dir_iter), NULL));
						}
					}
				}
				(p == NULL) ? NULL : (p = (g_object_unref (p), NULL));
			}
		}
	}
#line 333 "project-manager.vala"
	valide_project_manager_setup_ui (self);
#line 1314 "project-manager.c"
	_tmp28_ = FALSE;
#line 336 "project-manager.vala"
	if ((g_object_get ((GtkWidget*) self, "sensitive", &_tmp29_, NULL), _tmp29_)) {
#line 336 "project-manager.vala"
		_tmp28_ = expanded;
#line 1320 "project-manager.c"
	} else {
#line 336 "project-manager.vala"
		_tmp28_ = FALSE;
#line 1324 "project-manager.c"
	}
	/* Restore expanded rows */
#line 336 "project-manager.vala"
	if (_tmp28_) {
#line 338 "project-manager.vala"
		gtk_tree_view_expand_row (self->priv->tree_view, tree_path, TRUE);
#line 1331 "project-manager.c"
	}
#line 340 "project-manager.vala"
	if (was_empty) {
#line 342 "project-manager.vala"
		if (gtk_tree_model_get_iter_first ((GtkTreeModel*) self->priv->tree_store, &iter)) {
#line 1337 "project-manager.c"
			GtkTreePath* _tmp30_;
			_tmp30_ = NULL;
#line 344 "project-manager.vala"
			tree_path = (_tmp30_ = gtk_tree_model_get_path ((GtkTreeModel*) self->priv->tree_store, &iter), (tree_path == NULL) ? NULL : (tree_path = (gtk_tree_path_free (tree_path), NULL)), _tmp30_);
#line 345 "project-manager.vala"
			gtk_tree_view_expand_row (self->priv->tree_view, tree_path, TRUE);
#line 1344 "project-manager.c"
		}
	}
	(tree_path == NULL) ? NULL : (tree_path = (gtk_tree_path_free (tree_path), NULL));
	(dir_iters == NULL) ? NULL : (dir_iters = (g_hash_table_unref (dir_iters), NULL));
}


#line 350 "project-manager.vala"
static gboolean valide_project_manager_drag_motion_cb (ValideProjectManager* self, GtkWidget* sender, GdkDragContext* drag_context, gint x, gint y, guint time) {
#line 1354 "project-manager.c"
	gboolean result;
	GtkTreePath* path;
	GtkTreeSelection* selection;
	GtkTreeSelection* _tmp1_;
	GtkTreeSelection* _tmp0_;
	GtkTreePath* _tmp4_;
	gboolean _tmp3_;
	GtkTreePath* _tmp2_;
#line 350 "project-manager.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 350 "project-manager.vala"
	g_return_val_if_fail (sender != NULL, FALSE);
#line 350 "project-manager.vala"
	g_return_val_if_fail (drag_context != NULL, FALSE);
#line 1369 "project-manager.c"
	path = NULL;
	selection = NULL;
	_tmp1_ = NULL;
#line 358 "project-manager.vala"
	_tmp0_ = NULL;
#line 358 "project-manager.vala"
	selection = (_tmp1_ = (_tmp0_ = gtk_tree_view_get_selection (self->priv->tree_view), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL)), _tmp1_);
#line 1377 "project-manager.c"
	/* Retrive the drag location */
	_tmp4_ = NULL;
	_tmp2_ = NULL;
#line 360 "project-manager.vala"
	_tmp3_ = gtk_tree_view_get_path_at_pos (self->priv->tree_view, x, y, &_tmp2_, NULL, NULL, NULL);
#line 360 "project-manager.vala"
	path = (_tmp4_ = _tmp2_, (path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL)), _tmp4_);
#line 360 "project-manager.vala"
	_tmp3_;
#line 361 "project-manager.vala"
	if (path != NULL) {
#line 363 "project-manager.vala"
		gtk_tree_selection_select_path (selection, path);
#line 1391 "project-manager.c"
	} else {
#line 367 "project-manager.vala"
		gtk_tree_selection_unselect_all (selection);
#line 1395 "project-manager.c"
	}
	result = TRUE;
	(path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL));
	(selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL));
	return result;
}


static void _g_slist_free_g_free (GSList* self) {
	g_slist_foreach (self, (GFunc) g_free, NULL);
	g_slist_free (self);
}


#line 372 "project-manager.vala"
static void valide_project_manager_drag_data_received_cb (ValideProjectManager* self, GtkWidget* sender, GdkDragContext* drag_context, gint x, gint y, GtkSelectionData* data, guint info, guint time) {
#line 1412 "project-manager.c"
	GError * _inner_error_;
	gboolean del;
	ValideProject* project;
	GSList* files;
	GtkTreePath* path;
	GtkTreeIter iter = {0};
	char* filename;
	char* name;
	char* project_name;
	char* destination;
	GtkTreePath* _tmp2_;
	gboolean _tmp1_;
	GtkTreePath* _tmp0_;
	GSList* _tmp12_;
#line 372 "project-manager.vala"
	g_return_if_fail (self != NULL);
#line 372 "project-manager.vala"
	g_return_if_fail (sender != NULL);
#line 372 "project-manager.vala"
	g_return_if_fail (drag_context != NULL);
#line 372 "project-manager.vala"
	g_return_if_fail (data != NULL);
#line 1435 "project-manager.c"
	_inner_error_ = NULL;
	del = FALSE;
	project = NULL;
	files = NULL;
	path = NULL;
	filename = NULL;
	name = NULL;
	project_name = NULL;
	destination = NULL;
	/* Retrive the drag location */
	_tmp2_ = NULL;
	_tmp0_ = NULL;
#line 389 "project-manager.vala"
	_tmp1_ = gtk_tree_view_get_path_at_pos (self->priv->tree_view, x, y, &_tmp0_, NULL, NULL, NULL);
#line 389 "project-manager.vala"
	path = (_tmp2_ = _tmp0_, (path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL)), _tmp2_);
#line 389 "project-manager.vala"
	_tmp1_;
#line 390 "project-manager.vala"
	if (path == NULL) {
#line 1456 "project-manager.c"
		ValideProject* _tmp4_;
		ValideProject* _tmp3_;
		char* _tmp6_;
		const char* _tmp5_;
		_tmp4_ = NULL;
#line 392 "project-manager.vala"
		_tmp3_ = NULL;
#line 392 "project-manager.vala"
		project = (_tmp4_ = (_tmp3_ = valide_project_manager_get_current (self), (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), _tmp4_);
#line 1466 "project-manager.c"
		_tmp6_ = NULL;
#line 393 "project-manager.vala"
		_tmp5_ = NULL;
#line 393 "project-manager.vala"
		destination = (_tmp6_ = (_tmp5_ = valide_project_get_path (project), (_tmp5_ == NULL) ? NULL : g_strdup (_tmp5_)), destination = (g_free (destination), NULL), _tmp6_);
#line 1472 "project-manager.c"
	} else {
#line 397 "project-manager.vala"
		gtk_tree_model_get_iter ((GtkTreeModel*) self->priv->tree_store, &iter, path);
#line 398 "project-manager.vala"
		gtk_tree_model_get ((GtkTreeModel*) self->priv->tree_store, &iter, VALIDE_PROJECT_MANAGER_COL_NAME, &project_name, -1);
#line 399 "project-manager.vala"
		gtk_tree_model_get ((GtkTreeModel*) self->priv->tree_store, &iter, VALIDE_PROJECT_MANAGER_COL_REAL_PATH, &filename, -1);
#line 400 "project-manager.vala"
		if (filename == NULL) {
#line 1482 "project-manager.c"
			ValideProject* _tmp7_;
			char* _tmp9_;
			const char* _tmp8_;
			_tmp7_ = NULL;
#line 402 "project-manager.vala"
			project = (_tmp7_ = valide_project_manager_get_project_by_name (self, project_name), (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), _tmp7_);
#line 1489 "project-manager.c"
			_tmp9_ = NULL;
#line 403 "project-manager.vala"
			_tmp8_ = NULL;
#line 403 "project-manager.vala"
			destination = (_tmp9_ = (_tmp8_ = valide_project_get_path (project), (_tmp8_ == NULL) ? NULL : g_strdup (_tmp8_)), destination = (g_free (destination), NULL), _tmp9_);
#line 1495 "project-manager.c"
		} else {
			GtkTreeIter parent = {0};
			ValideProject* _tmp10_;
			char* _tmp11_;
#line 409 "project-manager.vala"
			if (!gtk_tree_model_iter_has_child ((GtkTreeModel*) self->priv->tree_store, &iter)) {
#line 411 "project-manager.vala"
				gtk_tree_model_iter_parent ((GtkTreeModel*) self->priv->tree_store, &parent, &iter);
#line 412 "project-manager.vala"
				iter = parent;
#line 1506 "project-manager.c"
			}
#line 414 "project-manager.vala"
			gtk_tree_model_get ((GtkTreeModel*) self->priv->tree_store, &iter, VALIDE_PROJECT_MANAGER_COL_NAME, &name, -1);
#line 416 "project-manager.vala"
			gtk_tree_model_iter_parent ((GtkTreeModel*) self->priv->tree_store, &parent, &iter);
#line 417 "project-manager.vala"
			gtk_tree_model_get ((GtkTreeModel*) self->priv->tree_store, &iter, VALIDE_PROJECT_MANAGER_COL_NAME, &project_name, -1);
#line 1514 "project-manager.c"
			_tmp10_ = NULL;
#line 418 "project-manager.vala"
			project = (_tmp10_ = valide_project_manager_get_project_by_name (self, name), (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), _tmp10_);
#line 1518 "project-manager.c"
			_tmp11_ = NULL;
#line 420 "project-manager.vala"
			destination = (_tmp11_ = g_build_filename (valide_project_get_path (project), name, NULL), destination = (g_free (destination), NULL), _tmp11_);
#line 1522 "project-manager.c"
		}
	}
	_tmp12_ = NULL;
#line 424 "project-manager.vala"
	files = (_tmp12_ = NULL, (files == NULL) ? NULL : (files = (_g_slist_free_g_free (files), NULL)), _tmp12_);
#line 1528 "project-manager.c"
	{
		char** _tmp13_;
		char** uri_collection;
		int uri_collection_length1;
		int uri_it;
		_tmp13_ = NULL;
#line 425 "project-manager.vala"
		uri_collection = _tmp13_ = g_strsplit ((const char*) data->data, "\r\n", 0);
#line 1537 "project-manager.c"
		uri_collection_length1 = _vala_array_length (_tmp13_);
		for (uri_it = 0; uri_it < _vala_array_length (_tmp13_); uri_it = uri_it + 1) {
			const char* _tmp21_;
			char* uri;
#line 831 "glib-2.0.vapi"
			_tmp21_ = NULL;
#line 1544 "project-manager.c"
			uri = (_tmp21_ = uri_collection[uri_it], (_tmp21_ == NULL) ? NULL : g_strdup (_tmp21_));
			{
#line 427 "project-manager.vala"
				if (_vala_strcmp0 (uri, "") != 0) {
#line 1549 "project-manager.c"
					GFile* src;
					char* filepath;
					char* basename;
					GFile* _tmp14_;
					char* _tmp15_;
					char* _tmp16_;
					src = NULL;
					filepath = NULL;
					basename = NULL;
					_tmp14_ = NULL;
#line 433 "project-manager.vala"
					src = (_tmp14_ = g_file_new_for_uri (uri), (src == NULL) ? NULL : (src = (g_object_unref (src), NULL)), _tmp14_);
#line 1562 "project-manager.c"
					_tmp15_ = NULL;
#line 434 "project-manager.vala"
					filepath = (_tmp15_ = g_file_get_path (src), filepath = (g_free (filepath), NULL), _tmp15_);
#line 1566 "project-manager.c"
					_tmp16_ = NULL;
#line 435 "project-manager.vala"
					basename = (_tmp16_ = g_path_get_basename (filepath), basename = (g_free (basename), NULL), _tmp16_);
#line 437 "project-manager.vala"
					if (drag_context->suggested_action == GDK_ACTION_LINK) {
#line 1572 "project-manager.c"
						const char* _tmp17_;
#line 439 "project-manager.vala"
						_tmp17_ = NULL;
#line 439 "project-manager.vala"
						files = g_slist_append (files, (_tmp17_ = filepath, (_tmp17_ == NULL) ? NULL : g_strdup (_tmp17_)));
#line 1578 "project-manager.c"
					} else {
						{
							GFile* dest;
							char* destpath;
							char* _tmp19_;
							char* _tmp18_;
							GFile* _tmp20_;
							dest = NULL;
							destpath = NULL;
							_tmp19_ = NULL;
#line 947 "gio-2.0.vapi"
							_tmp18_ = NULL;
#line 448 "project-manager.vala"
							destpath = (_tmp19_ = g_build_filename (destination, _tmp18_ = g_file_get_basename (src), NULL), destpath = (g_free (destpath), NULL), _tmp19_);
#line 1593 "project-manager.c"
							_tmp18_ = (g_free (_tmp18_), NULL);
							_tmp20_ = NULL;
#line 449 "project-manager.vala"
							dest = (_tmp20_ = g_file_new_for_path (destpath), (dest == NULL) ? NULL : (dest = (g_object_unref (dest), NULL)), _tmp20_);
#line 1598 "project-manager.c"
							/* @TODO Ask for overwriting */
#line 451 "project-manager.vala"
							g_file_copy (src, dest, G_FILE_COPY_OVERWRITE, NULL, NULL, NULL, &_inner_error_);
#line 1602 "project-manager.c"
							if (_inner_error_ != NULL) {
								(dest == NULL) ? NULL : (dest = (g_object_unref (dest), NULL));
								destpath = (g_free (destpath), NULL);
								goto __catch52_g_error;
								goto __finally52;
							}
#line 452 "project-manager.vala"
							files = g_slist_append (files, g_file_get_path (dest));
#line 453 "project-manager.vala"
							if (drag_context->suggested_action == GDK_ACTION_MOVE) {
#line 455 "project-manager.vala"
								g_file_delete (src, NULL, &_inner_error_);
#line 1615 "project-manager.c"
								if (_inner_error_ != NULL) {
									(dest == NULL) ? NULL : (dest = (g_object_unref (dest), NULL));
									destpath = (g_free (destpath), NULL);
									goto __catch52_g_error;
									goto __finally52;
								}
#line 456 "project-manager.vala"
								del = TRUE;
#line 1624 "project-manager.c"
							}
							(dest == NULL) ? NULL : (dest = (g_object_unref (dest), NULL));
							destpath = (g_free (destpath), NULL);
						}
						goto __finally52;
						__catch52_g_error:
						{
							GError * e;
							e = _inner_error_;
							_inner_error_ = NULL;
							{
#line 461 "project-manager.vala"
								g_warning (e->message);
#line 1638 "project-manager.c"
								(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
							}
						}
						__finally52:
						if (_inner_error_ != NULL) {
							(src == NULL) ? NULL : (src = (g_object_unref (src), NULL));
							filepath = (g_free (filepath), NULL);
							basename = (g_free (basename), NULL);
							uri = (g_free (uri), NULL);
							uri_collection = (_vala_array_free (uri_collection, uri_collection_length1, (GDestroyNotify) g_free), NULL);
							(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
							(files == NULL) ? NULL : (files = (_g_slist_free_g_free (files), NULL));
							(path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL));
							filename = (g_free (filename), NULL);
							name = (g_free (name), NULL);
							project_name = (g_free (project_name), NULL);
							destination = (g_free (destination), NULL);
							g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
							g_clear_error (&_inner_error_);
							return;
						}
#line 463 "project-manager.vala"
						drag_context->suggested_action = drag_context->action;
#line 1662 "project-manager.c"
					}
					(src == NULL) ? NULL : (src = (g_object_unref (src), NULL));
					filepath = (g_free (filepath), NULL);
					basename = (g_free (basename), NULL);
				}
				uri = (g_free (uri), NULL);
			}
		}
#line 425 "project-manager.vala"
		uri_collection = (_vala_array_free (uri_collection, uri_collection_length1, (GDestroyNotify) g_free), NULL);
#line 1673 "project-manager.c"
	}
#line 467 "project-manager.vala"
	if (files != NULL) {
#line 469 "project-manager.vala"
		valide_project_add_file (project, files);
#line 1679 "project-manager.c"
	}
#line 471 "project-manager.vala"
	gtk_drag_finish (drag_context, TRUE, del, (guint32) time);
#line 1683 "project-manager.c"
	(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
	(files == NULL) ? NULL : (files = (_g_slist_free_g_free (files), NULL));
	(path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL));
	filename = (g_free (filename), NULL);
	name = (g_free (name), NULL);
	project_name = (g_free (project_name), NULL);
	destination = (g_free (destination), NULL);
}


#line 474 "project-manager.vala"
static gint valide_project_manager_sort_func (ValideProjectManager* self, GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b) {
#line 1696 "project-manager.c"
	gint result;
	char* path_a;
	char* path_b;
#line 474 "project-manager.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 474 "project-manager.vala"
	g_return_val_if_fail (model != NULL, 0);
#line 1704 "project-manager.c"
	path_a = NULL;
	path_b = NULL;
#line 478 "project-manager.vala"
	gtk_tree_model_get (model, &(*a), VALIDE_PROJECT_MANAGER_COL_REAL_PATH, &path_a, -1);
#line 479 "project-manager.vala"
	gtk_tree_model_get (model, &(*b), VALIDE_PROJECT_MANAGER_COL_REAL_PATH, &path_b, -1);
#line 1711 "project-manager.c"
	result = valide_utils_cmp_filename (path_a, path_b);
	path_a = (g_free (path_a), NULL);
	path_b = (g_free (path_b), NULL);
	return result;
}


#line 484 "project-manager.vala"
static ValideProject* valide_project_manager_get_project_by_name (ValideProjectManager* self, const char* name) {
#line 1721 "project-manager.c"
	ValideProject* result;
	ValideProject* _tmp0_;
#line 484 "project-manager.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 484 "project-manager.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 1728 "project-manager.c"
	/* @TODO */
#line 487 "project-manager.vala"
	_tmp0_ = NULL;
#line 1732 "project-manager.c"
	result = (_tmp0_ = valide_project_manager_get_current (self), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


/**
   * Crea te a new project manager
   *
   * @param ui_manager The ui manager
   * @param documents The document manager
   */
#line 496 "project-manager.vala"
ValideProjectManager* valide_project_manager_construct (GType object_type, ValideUIManager* ui_manager, ValideDocumentManager* documents, ValideBuilderManager* builders) {
#line 1746 "project-manager.c"
	GParameter * __params;
	GParameter * __params_it;
	ValideProjectManager * self;
#line 496 "project-manager.vala"
	g_return_val_if_fail (ui_manager != NULL, NULL);
#line 496 "project-manager.vala"
	g_return_val_if_fail (documents != NULL, NULL);
#line 496 "project-manager.vala"
	g_return_val_if_fail (builders != NULL, NULL);
#line 1756 "project-manager.c"
	__params = g_new0 (GParameter, 3);
	__params_it = __params;
#line 499 "project-manager.vala"
	__params_it->name = "ui-manager";
#line 499 "project-manager.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_UI_MANAGER);
#line 499 "project-manager.vala"
	g_value_set_object (&__params_it->value, ui_manager);
#line 499 "project-manager.vala"
	__params_it++;
#line 500 "project-manager.vala"
	__params_it->name = "documents";
#line 500 "project-manager.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_DOCUMENT_MANAGER);
#line 500 "project-manager.vala"
	g_value_set_object (&__params_it->value, documents);
#line 500 "project-manager.vala"
	__params_it++;
#line 501 "project-manager.vala"
	__params_it->name = "builders";
#line 501 "project-manager.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_BUILDER_MANAGER);
#line 501 "project-manager.vala"
	g_value_set_object (&__params_it->value, builders);
#line 501 "project-manager.vala"
	__params_it++;
#line 1783 "project-manager.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
#line 502 "project-manager.vala"
	g_object_set ((GtkWidget*) self, "width-request", 200, NULL);
#line 1787 "project-manager.c"
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 496 "project-manager.vala"
ValideProjectManager* valide_project_manager_new (ValideUIManager* ui_manager, ValideDocumentManager* documents, ValideBuilderManager* builders) {
#line 496 "project-manager.vala"
	return valide_project_manager_construct (VALIDE_TYPE_PROJECT_MANAGER, ui_manager, documents, builders);
#line 1801 "project-manager.c"
}


/**
   * Create a new project
   *
   * @return true if the project is created
   */
#line 565 "project-manager.vala"
gboolean valide_project_manager_create (ValideProjectManager* self) {
#line 1812 "project-manager.c"
	gboolean result;
	GError * _inner_error_;
	gboolean created;
	ValideProjectDialog* dialog;
	ValideProjectDialog* _tmp0_;
#line 565 "project-manager.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1820 "project-manager.c"
	_inner_error_ = NULL;
	created = FALSE;
	dialog = NULL;
	_tmp0_ = NULL;
#line 570 "project-manager.vala"
	dialog = (_tmp0_ = g_object_ref_sink (valide_project_dialog_new ()), (dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL)), _tmp0_);
#line 571 "project-manager.vala"
	if (valide_project_dialog_run (dialog) == GTK_RESPONSE_APPLY) {
#line 1829 "project-manager.c"
		char* path;
		char* _tmp2_;
		const char* _tmp1_;
		path = NULL;
		_tmp2_ = NULL;
#line 575 "project-manager.vala"
		_tmp1_ = NULL;
#line 575 "project-manager.vala"
		path = (_tmp2_ = (_tmp1_ = valide_project_dialog_get_project_dir (dialog), (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_)), path = (g_free (path), NULL), _tmp2_);
#line 576 "project-manager.vala"
		if (!g_file_test (path, G_FILE_TEST_EXISTS)) {
#line 1841 "project-manager.c"
			GtkDialog* msg;
			GtkDialog* _tmp3_;
			GtkLabel* _tmp4_;
			msg = NULL;
			_tmp3_ = NULL;
#line 580 "project-manager.vala"
			msg = (_tmp3_ = g_object_ref_sink ((GtkDialog*) gtk_dialog_new_with_buttons (_ ("New project"), NULL, GTK_DIALOG_MODAL, GTK_STOCK_NO, GTK_RESPONSE_NO, GTK_STOCK_YES, GTK_RESPONSE_YES, NULL, NULL)), (msg == NULL) ? NULL : (msg = (g_object_unref (msg), NULL)), _tmp3_);
#line 1849 "project-manager.c"
			_tmp4_ = NULL;
#line 586 "project-manager.vala"
			gtk_box_pack_start ((GtkBox*) msg->vbox, (GtkWidget*) (_tmp4_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("Project directory doesn't exist. Would you like to create it?")))), TRUE, TRUE, (guint) 10);
#line 1853 "project-manager.c"
			(_tmp4_ == NULL) ? NULL : (_tmp4_ = (g_object_unref (_tmp4_), NULL));
#line 588 "project-manager.vala"
			gtk_widget_show_all ((GtkWidget*) msg);
#line 589 "project-manager.vala"
			if (gtk_dialog_run (msg) == GTK_RESPONSE_YES) {
#line 591 "project-manager.vala"
				g_mkdir_with_parents (path, 0755);
#line 592 "project-manager.vala"
				created = TRUE;
#line 1863 "project-manager.c"
			}
#line 594 "project-manager.vala"
			gtk_object_destroy ((GtkObject*) msg);
#line 1867 "project-manager.c"
			(msg == NULL) ? NULL : (msg = (g_object_unref (msg), NULL));
		} else {
#line 598 "project-manager.vala"
			created = TRUE;
#line 1872 "project-manager.c"
		}
#line 601 "project-manager.vala"
		if (created == TRUE) {
#line 1876 "project-manager.c"
			char* _tmp6_;
			const char* _tmp5_;
			_tmp6_ = NULL;
#line 603 "project-manager.vala"
			_tmp5_ = NULL;
#line 603 "project-manager.vala"
			path = (_tmp6_ = (_tmp5_ = valide_project_dialog_get_project_path (dialog), (_tmp5_ == NULL) ? NULL : g_strdup (_tmp5_)), path = (g_free (path), NULL), _tmp6_);
#line 604 "project-manager.vala"
			if (g_file_test (path, G_FILE_TEST_EXISTS)) {
#line 1886 "project-manager.c"
				GtkDialog* msg;
				GtkDialog* _tmp7_;
				GtkLabel* _tmp8_;
				msg = NULL;
				_tmp7_ = NULL;
#line 608 "project-manager.vala"
				msg = (_tmp7_ = g_object_ref_sink ((GtkDialog*) gtk_dialog_new_with_buttons (_ ("New project"), NULL, GTK_DIALOG_MODAL, GTK_STOCK_NO, GTK_RESPONSE_NO, GTK_STOCK_YES, GTK_RESPONSE_YES, NULL, NULL)), (msg == NULL) ? NULL : (msg = (g_object_unref (msg), NULL)), _tmp7_);
#line 1894 "project-manager.c"
				_tmp8_ = NULL;
#line 614 "project-manager.vala"
				gtk_box_pack_start ((GtkBox*) msg->vbox, (GtkWidget*) (_tmp8_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("Directory already exists. Would you like to overwrite it?")))), TRUE, TRUE, (guint) 10);
#line 1898 "project-manager.c"
				(_tmp8_ == NULL) ? NULL : (_tmp8_ = (g_object_unref (_tmp8_), NULL));
#line 616 "project-manager.vala"
				gtk_widget_show_all ((GtkWidget*) msg);
#line 617 "project-manager.vala"
				if (gtk_dialog_run (msg) == GTK_RESPONSE_NO) {
#line 619 "project-manager.vala"
					created = FALSE;
#line 1906 "project-manager.c"
				}
#line 621 "project-manager.vala"
				gtk_object_destroy ((GtkObject*) msg);
#line 1910 "project-manager.c"
				(msg == NULL) ? NULL : (msg = (g_object_unref (msg), NULL));
			} else {
#line 625 "project-manager.vala"
				g_mkdir (path, 0755);
#line 1915 "project-manager.c"
			}
		}
#line 629 "project-manager.vala"
		if (created == TRUE) {
#line 1920 "project-manager.c"
			{
				char* prj;
				ValideTemplate* _tmp9_;
				ValideTemplate* _tmp10_;
				char* _tmp11_;
				char* _tmp12_;
				char* _tmp13_;
				prj = NULL;
				_tmp9_ = _valide_template_new (valide_project_dialog_get_template (dialog), &_inner_error_);
				if (_inner_error_ != NULL) {
					prj = (g_free (prj), NULL);
					goto __catch53_g_error;
					goto __finally53;
				}
#line 152 "template.vala"
				_tmp10_ = NULL;
#line 635 "project-manager.vala"
				_tmp11_ = NULL;
#line 1939 "project-manager.c"
				_tmp12_ = (_tmp11_ = valide_project_manager_copy_template (self, _tmp10_ = _tmp9_, path, valide_project_dialog_get_project_name (dialog), valide_project_dialog_get_project_author (dialog), valide_project_dialog_get_project_version (dialog), valide_project_dialog_get_project_license (dialog), valide_project_dialog_get_lang (dialog), valide_project_dialog_get_builder (dialog), &_inner_error_), (_tmp10_ == NULL) ? NULL : (_tmp10_ = (g_object_unref (_tmp10_), NULL)), _tmp11_);
				if (_inner_error_ != NULL) {
					prj = (g_free (prj), NULL);
					goto __catch53_g_error;
					goto __finally53;
				}
				_tmp13_ = NULL;
#line 635 "project-manager.vala"
				prj = (_tmp13_ = _tmp12_, prj = (g_free (prj), NULL), _tmp13_);
#line 642 "project-manager.vala"
				valide_project_manager_open (self, prj, &_inner_error_);
#line 1951 "project-manager.c"
				if (_inner_error_ != NULL) {
					prj = (g_free (prj), NULL);
					goto __catch53_g_error;
					goto __finally53;
				}
				prj = (g_free (prj), NULL);
			}
			goto __finally53;
			__catch53_g_error:
			{
				GError * e;
				e = _inner_error_;
				_inner_error_ = NULL;
				{
#line 646 "project-manager.vala"
					g_warning (e->message);
#line 647 "project-manager.vala"
					created = FALSE;
#line 1970 "project-manager.c"
					(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
				}
			}
			__finally53:
			if (_inner_error_ != NULL) {
				path = (g_free (path), NULL);
				(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return FALSE;
			}
		}
		path = (g_free (path), NULL);
	}
#line 651 "project-manager.vala"
	gtk_object_destroy ((GtkObject*) dialog);
#line 1987 "project-manager.c"
	result = created;
	(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
	return result;
}


#line 745 "project-manager.vala"
static void _valide_project_manager_close_valide_project_closed (ValideProject* _sender, gpointer self) {
#line 1996 "project-manager.c"
	valide_project_manager_close (self, _sender);
}


#line 233 "project-manager.vala"
static void _valide_project_manager_refresh_valide_project_added_file (ValideProject* _sender, gpointer self) {
#line 2003 "project-manager.c"
	valide_project_manager_refresh (self);
}


#line 233 "project-manager.vala"
static void _valide_project_manager_refresh_valide_project_removed_file (ValideProject* _sender, gpointer self) {
#line 2010 "project-manager.c"
	valide_project_manager_refresh (self);
}


static void _lambda38_ (ValideProject* s, ValideProjectManager* self) {
	g_return_if_fail (s != NULL);
#line 717 "project-manager.vala"
	valide_project_manager_refresh (self);
#line 718 "project-manager.vala"
	g_signal_emit_by_name (self, "project-options-changed", s);
#line 2021 "project-manager.c"
}


static void __lambda38__valide_project_options_changed (ValideProject* _sender, gpointer self) {
	_lambda38_ (_sender, self);
}


/**
   * Open a project
   *
   * @param filename The name of project file
   *
   * @return ture if the project is opened
   *
   * @throws ProjectError
   */
#line 664 "project-manager.vala"
gboolean valide_project_manager_open (ValideProjectManager* self, const char* filename, GError** error) {
#line 2041 "project-manager.c"
	gboolean result;
	GError * _inner_error_;
	gboolean opened;
	GtkFileFilter* filter;
	const char* _tmp0_;
	char* file_name;
#line 664 "project-manager.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 2050 "project-manager.c"
	_inner_error_ = NULL;
	opened = FALSE;
	filter = NULL;
#line 668 "project-manager.vala"
	_tmp0_ = NULL;
#line 2056 "project-manager.c"
	file_name = (_tmp0_ = filename, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
#line 670 "project-manager.vala"
	if (valide_project_manager_get_current (self) == NULL) {
#line 672 "project-manager.vala"
		if (file_name == NULL) {
#line 2062 "project-manager.c"
			GtkFileChooserDialog* dialog;
			GtkFileChooserDialog* _tmp1_;
			char* _tmp3_;
			ValideConfigManager* _tmp2_;
			GtkFileFilter* _tmp4_;
			dialog = NULL;
			_tmp1_ = NULL;
#line 676 "project-manager.vala"
			dialog = (_tmp1_ = g_object_ref_sink ((GtkFileChooserDialog*) gtk_file_chooser_dialog_new (_ ("Open project"), NULL, GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL, NULL)), (dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL)), _tmp1_);
#line 169 "abstract-config-manager.vala"
			_tmp3_ = NULL;
#line 88 "config-manager.vala"
			_tmp2_ = NULL;
#line 681 "project-manager.vala"
			gtk_file_chooser_set_current_folder ((GtkFileChooser*) dialog, _tmp3_ = valide_abstract_config_manager_get_string ((ValideAbstractConfigManager*) (_tmp2_ = valide_config_manager_get_instance ()), "Projects", "default-directory"));
#line 2078 "project-manager.c"
			_tmp3_ = (g_free (_tmp3_), NULL);
			(_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL));
			/* Add filter */
			_tmp4_ = NULL;
#line 684 "project-manager.vala"
			filter = (_tmp4_ = g_object_ref_sink (gtk_file_filter_new ()), (filter == NULL) ? NULL : (filter = (g_object_unref (filter), NULL)), _tmp4_);
#line 685 "project-manager.vala"
			gtk_file_filter_set_name (filter, _ ("Val(a)IDE project file"));
#line 2087 "project-manager.c"
			{
				const char** ext_collection;
				int ext_collection_length1;
				int ext_it;
#line 686 "project-manager.vala"
				ext_collection = VALIDE_PROJECT_FILE_EXT;
#line 2094 "project-manager.c"
				ext_collection_length1 = G_N_ELEMENTS (VALIDE_PROJECT_FILE_EXT);
				for (ext_it = 0; ext_it < G_N_ELEMENTS (VALIDE_PROJECT_FILE_EXT); ext_it = ext_it + 1) {
					const char* _tmp6_;
					char* ext;
#line 60 "project.vala"
					_tmp6_ = NULL;
#line 2101 "project-manager.c"
					ext = (_tmp6_ = ext_collection[ext_it], (_tmp6_ == NULL) ? NULL : g_strdup (_tmp6_));
					{
						char* _tmp5_;
						_tmp5_ = NULL;
#line 688 "project-manager.vala"
						gtk_file_filter_add_pattern (filter, _tmp5_ = g_strconcat ("*.", ext, NULL));
#line 2108 "project-manager.c"
						_tmp5_ = (g_free (_tmp5_), NULL);
						ext = (g_free (ext), NULL);
					}
				}
			}
#line 690 "project-manager.vala"
			gtk_file_chooser_add_filter ((GtkFileChooser*) dialog, filter);
#line 692 "project-manager.vala"
			if (gtk_dialog_run ((GtkDialog*) dialog) == GTK_RESPONSE_ACCEPT) {
#line 2118 "project-manager.c"
				{
					char* _tmp7_;
					_tmp7_ = NULL;
#line 696 "project-manager.vala"
					file_name = (_tmp7_ = gtk_file_chooser_get_filename ((GtkFileChooser*) dialog), file_name = (g_free (file_name), NULL), _tmp7_);
#line 2124 "project-manager.c"
				}
				goto __finally54;
				__catch54_g_error:
				{
					GError * e;
					e = _inner_error_;
					_inner_error_ = NULL;
					{
#line 700 "project-manager.vala"
						g_warning (e->message);
#line 2135 "project-manager.c"
						(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
					}
				}
				__finally54:
				if (_inner_error_ != NULL) {
					if (_inner_error_->domain == VALIDE_PROJECT_ERROR) {
						g_propagate_error (error, _inner_error_);
						(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
						(filter == NULL) ? NULL : (filter = (g_object_unref (filter), NULL));
						file_name = (g_free (file_name), NULL);
						return FALSE;
					} else {
						(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
						(filter == NULL) ? NULL : (filter = (g_object_unref (filter), NULL));
						file_name = (g_free (file_name), NULL);
						g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
						g_clear_error (&_inner_error_);
						return FALSE;
					}
				}
			}
#line 703 "project-manager.vala"
			gtk_object_destroy ((GtkObject*) dialog);
#line 2159 "project-manager.c"
			(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
		}
#line 705 "project-manager.vala"
		if (file_name != NULL) {
#line 707 "project-manager.vala"
			if (g_file_test (file_name, G_FILE_TEST_EXISTS)) {
#line 2166 "project-manager.c"
				ValideProject* p;
				ValideProject* _tmp8_;
				ValideProject* _tmp9_;
				ValideProject* _tmp10_;
				p = NULL;
				_tmp8_ = _valide_project_new_from_filename (file_name, self->priv->_documents, self->priv->_builders, &_inner_error_);
				if (_inner_error_ != NULL) {
					if (_inner_error_->domain == VALIDE_PROJECT_ERROR) {
						g_propagate_error (error, _inner_error_);
						(p == NULL) ? NULL : (p = (g_object_unref (p), NULL));
						(filter == NULL) ? NULL : (filter = (g_object_unref (filter), NULL));
						file_name = (g_free (file_name), NULL);
						return FALSE;
					} else {
						(p == NULL) ? NULL : (p = (g_object_unref (p), NULL));
						(filter == NULL) ? NULL : (filter = (g_object_unref (filter), NULL));
						file_name = (g_free (file_name), NULL);
						g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
						g_clear_error (&_inner_error_);
						return FALSE;
					}
				}
				_tmp9_ = NULL;
#line 711 "project-manager.vala"
				p = (_tmp9_ = _tmp8_, (p == NULL) ? NULL : (p = (g_object_unref (p), NULL)), _tmp9_);
#line 713 "project-manager.vala"
				g_signal_connect_object (p, "closed", (GCallback) _valide_project_manager_close_valide_project_closed, self, 0);
#line 714 "project-manager.vala"
				g_signal_connect_object (p, "added-file", (GCallback) _valide_project_manager_refresh_valide_project_added_file, self, 0);
#line 715 "project-manager.vala"
				g_signal_connect_object (p, "removed-file", (GCallback) _valide_project_manager_refresh_valide_project_removed_file, self, 0);
#line 716 "project-manager.vala"
				g_signal_connect (p, "options-changed", (GCallback) __lambda38__valide_project_options_changed, self);
#line 720 "project-manager.vala"
				_tmp10_ = NULL;
#line 720 "project-manager.vala"
				self->priv->projects = g_list_append (self->priv->projects, (_tmp10_ = p, (_tmp10_ == NULL) ? NULL : g_object_ref (_tmp10_)));
#line 721 "project-manager.vala"
				valide_project_manager_set_current (self, p);
#line 722 "project-manager.vala"
				valide_project_manager_refresh (self);
#line 724 "project-manager.vala"
				g_signal_emit_by_name (self, "project-opened", p);
#line 725 "project-manager.vala"
				opened = TRUE;
#line 2212 "project-manager.c"
				(p == NULL) ? NULL : (p = (g_object_unref (p), NULL));
			} else {
#line 729 "project-manager.vala"
				g_warning (_ ("The project file doesn't exist!"));
#line 2217 "project-manager.c"
			}
		}
	} else {
		_inner_error_ = g_error_new_literal (VALIDE_PROJECT_ERROR, VALIDE_PROJECT_ERROR_MAX_OPEN, _ ("Maximum projects open!"));
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == VALIDE_PROJECT_ERROR) {
				g_propagate_error (error, _inner_error_);
				(filter == NULL) ? NULL : (filter = (g_object_unref (filter), NULL));
				file_name = (g_free (file_name), NULL);
				return FALSE;
			} else {
				(filter == NULL) ? NULL : (filter = (g_object_unref (filter), NULL));
				file_name = (g_free (file_name), NULL);
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return FALSE;
			}
		}
	}
	result = opened;
	(filter == NULL) ? NULL : (filter = (g_object_unref (filter), NULL));
	file_name = (g_free (file_name), NULL);
	return result;
}


/**
   * Close the project
   *
   * @param project The project
   */
#line 745 "project-manager.vala"
void valide_project_manager_close (ValideProjectManager* self, ValideProject* project) {
#line 745 "project-manager.vala"
	g_return_if_fail (self != NULL);
#line 745 "project-manager.vala"
	g_return_if_fail (project != NULL);
#line 747 "project-manager.vala"
	self->priv->projects = g_list_remove (self->priv->projects, project);
#line 748 "project-manager.vala"
	valide_project_manager_set_current (self, NULL);
#line 749 "project-manager.vala"
	g_signal_emit_by_name (self, "project-closed");
#line 2261 "project-manager.c"
}


/**
   * Open the selected file
   */
#line 755 "project-manager.vala"
void valide_project_manager_open_selected_file (ValideProjectManager* self) {
#line 2270 "project-manager.c"
	GtkTreeIter iter = {0};
	GtkTreeSelection* selection;
	GtkTreeSelection* _tmp1_;
	GtkTreeSelection* _tmp0_;
#line 755 "project-manager.vala"
	g_return_if_fail (self != NULL);
#line 2277 "project-manager.c"
	selection = NULL;
	_tmp1_ = NULL;
#line 760 "project-manager.vala"
	_tmp0_ = NULL;
#line 760 "project-manager.vala"
	selection = (_tmp1_ = (_tmp0_ = gtk_tree_view_get_selection (self->priv->tree_view), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL)), _tmp1_);
#line 761 "project-manager.vala"
	if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
#line 2286 "project-manager.c"
		GtkTreePath* path;
		GtkTreeViewColumn* column;
		GtkTreePath* _tmp2_;
		GtkTreeViewColumn* _tmp4_;
		GtkTreeViewColumn* _tmp3_;
		path = NULL;
		column = NULL;
		_tmp2_ = NULL;
#line 766 "project-manager.vala"
		path = (_tmp2_ = gtk_tree_model_get_path ((GtkTreeModel*) self->priv->tree_store, &iter), (path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL)), _tmp2_);
#line 2297 "project-manager.c"
		_tmp4_ = NULL;
#line 767 "project-manager.vala"
		_tmp3_ = NULL;
#line 767 "project-manager.vala"
		column = (_tmp4_ = (_tmp3_ = gtk_tree_view_get_column (self->priv->tree_view, 0), (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), (column == NULL) ? NULL : (column = (g_object_unref (column), NULL)), _tmp4_);
#line 768 "project-manager.vala"
		valide_project_manager_file_select (self, self->priv->tree_view, path, column);
#line 2305 "project-manager.c"
		(path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL));
		(column == NULL) ? NULL : (column = (g_object_unref (column), NULL));
	}
	(selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL));
}


static void _g_list_free_g_free (GList* self) {
	g_list_foreach (self, (GFunc) g_free, NULL);
	g_list_free (self);
}


/**
   * Remove the selected file
   */
#line 775 "project-manager.vala"
void valide_project_manager_remove_selected_file (ValideProjectManager* self) {
#line 2324 "project-manager.c"
	GtkTreeIter iter = {0};
	GtkTreeSelection* selection;
	GtkTreeSelection* _tmp1_;
	GtkTreeSelection* _tmp0_;
#line 775 "project-manager.vala"
	g_return_if_fail (self != NULL);
#line 2331 "project-manager.c"
	selection = NULL;
	_tmp1_ = NULL;
#line 781 "project-manager.vala"
	_tmp0_ = NULL;
#line 781 "project-manager.vala"
	selection = (_tmp1_ = (_tmp0_ = gtk_tree_view_get_selection (self->priv->tree_view), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL)), _tmp1_);
#line 782 "project-manager.vala"
	if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
#line 2340 "project-manager.c"
		GtkTreePath* path;
		GtkTreePath* _tmp2_;
		path = NULL;
		_tmp2_ = NULL;
#line 786 "project-manager.vala"
		path = (_tmp2_ = gtk_tree_model_get_path ((GtkTreeModel*) self->priv->tree_store, &iter), (path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL)), _tmp2_);
#line 787 "project-manager.vala"
		if (gtk_tree_path_get_depth (path) > 1) {
#line 2349 "project-manager.c"
			char* filename;
			GList* files;
			GList* _tmp3_;
			const char* _tmp4_;
			filename = NULL;
			files = NULL;
			_tmp3_ = NULL;
#line 792 "project-manager.vala"
			files = (_tmp3_ = NULL, (files == NULL) ? NULL : (files = (_g_list_free_g_free (files), NULL)), _tmp3_);
#line 793 "project-manager.vala"
			gtk_tree_model_get ((GtkTreeModel*) self->priv->tree_store, &iter, VALIDE_PROJECT_MANAGER_COL_PATH, &filename, -1);
#line 794 "project-manager.vala"
			_tmp4_ = NULL;
#line 794 "project-manager.vala"
			files = g_list_append (files, (_tmp4_ = filename, (_tmp4_ == NULL) ? NULL : g_strdup (_tmp4_)));
#line 795 "project-manager.vala"
			valide_project_remove_file (valide_project_manager_get_current (self), files);
#line 2367 "project-manager.c"
			filename = (g_free (filename), NULL);
			(files == NULL) ? NULL : (files = (_g_list_free_g_free (files), NULL));
		}
		(path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL));
	}
	(selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL));
}


/**
   * Test if a filename is a project file.
   *
   * @param filename a filename.
   *
   * @return true if a filename is a project file.
   */
#line 807 "project-manager.vala"
gboolean valide_project_manager_accept_file (ValideProjectManager* self, const char* filename) {
#line 2386 "project-manager.c"
	gboolean result;
	gboolean accept;
	char* ext;
#line 807 "project-manager.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 807 "project-manager.vala"
	g_return_val_if_fail (filename != NULL, FALSE);
#line 2394 "project-manager.c"
	accept = FALSE;
	ext = valide_utils_get_extension (filename);
	{
		const char** e_collection;
		int e_collection_length1;
		int e_it;
#line 812 "project-manager.vala"
		e_collection = VALIDE_PROJECT_FILE_EXT;
#line 2403 "project-manager.c"
		e_collection_length1 = G_N_ELEMENTS (VALIDE_PROJECT_FILE_EXT);
		for (e_it = 0; e_it < G_N_ELEMENTS (VALIDE_PROJECT_FILE_EXT); e_it = e_it + 1) {
			const char* _tmp0_;
			char* e;
#line 60 "project.vala"
			_tmp0_ = NULL;
#line 2410 "project-manager.c"
			e = (_tmp0_ = e_collection[e_it], (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
			{
#line 814 "project-manager.vala"
				if (_vala_strcmp0 (e, ext) == 0) {
#line 816 "project-manager.vala"
					accept = TRUE;
#line 2417 "project-manager.c"
					e = (g_free (e), NULL);
#line 817 "project-manager.vala"
					break;
#line 2421 "project-manager.c"
				}
				e = (g_free (e), NULL);
			}
		}
	}
	result = accept;
	ext = (g_free (ext), NULL);
	return result;
}


ValideUIManager* valide_project_manager_get_ui_manager (ValideProjectManager* self) {
	ValideUIManager* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_ui_manager;
	return result;
}


static void valide_project_manager_set_ui_manager (ValideProjectManager* self, ValideUIManager* value) {
	ValideUIManager* _tmp1_;
	ValideUIManager* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 73 "project-manager.vala"
	_tmp0_ = NULL;
#line 2448 "project-manager.c"
	self->priv->_ui_manager = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_ui_manager == NULL) ? NULL : (self->priv->_ui_manager = (g_object_unref (self->priv->_ui_manager), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "ui-manager");
}


ValideDocumentManager* valide_project_manager_get_documents (ValideProjectManager* self) {
	ValideDocumentManager* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_documents;
	return result;
}


static void valide_project_manager_set_documents (ValideProjectManager* self, ValideDocumentManager* value) {
	ValideDocumentManager* _tmp1_;
	ValideDocumentManager* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 82 "project-manager.vala"
	_tmp0_ = NULL;
#line 2469 "project-manager.c"
	self->priv->_documents = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_documents == NULL) ? NULL : (self->priv->_documents = (g_object_unref (self->priv->_documents), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "documents");
}


ValideBuilderManager* valide_project_manager_get_builders (ValideProjectManager* self) {
	ValideBuilderManager* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_builders;
	return result;
}


static void valide_project_manager_set_builders (ValideProjectManager* self, ValideBuilderManager* value) {
	ValideBuilderManager* _tmp1_;
	ValideBuilderManager* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 91 "project-manager.vala"
	_tmp0_ = NULL;
#line 2490 "project-manager.c"
	self->priv->_builders = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_builders == NULL) ? NULL : (self->priv->_builders = (g_object_unref (self->priv->_builders), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "builders");
}


ValideProject* valide_project_manager_get_current (ValideProjectManager* self) {
	ValideProject* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_current;
	return result;
}


void valide_project_manager_set_current (ValideProjectManager* self, ValideProject* value) {
	g_return_if_fail (self != NULL);
#line 159 "project-manager.vala"
	self->priv->_current = value;
#line 160 "project-manager.vala"
	valide_project_manager_refresh (self);
#line 2510 "project-manager.c"
	g_object_notify ((GObject *) self, "current");
}


#line 474 "project-manager.vala"
static gint _valide_project_manager_sort_func_gtk_tree_iter_compare_func (GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b, gpointer self) {
#line 2517 "project-manager.c"
	return valide_project_manager_sort_func (self, model, a, b);
}


#line 164 "project-manager.vala"
static void _valide_project_manager_file_select_gtk_tree_view_row_activated (GtkTreeView* _sender, const GtkTreePath* path, GtkTreeViewColumn* column, gpointer self) {
#line 2524 "project-manager.c"
	valide_project_manager_file_select (self, _sender, path, column);
}


#line 199 "project-manager.vala"
static gboolean _valide_project_manager_button_press_gtk_widget_button_press_event (GtkWidget* _sender, const GdkEventButton* event, gpointer self) {
#line 2531 "project-manager.c"
	return valide_project_manager_button_press (self, _sender, event);
}


#line 350 "project-manager.vala"
static gboolean _valide_project_manager_drag_motion_cb_gtk_widget_drag_motion (GtkWidget* _sender, GdkDragContext* context, gint x, gint y, guint time_, gpointer self) {
#line 2538 "project-manager.c"
	return valide_project_manager_drag_motion_cb (self, _sender, context, x, y, time_);
}


#line 372 "project-manager.vala"
static void _valide_project_manager_drag_data_received_cb_gtk_widget_drag_data_received (GtkWidget* _sender, GdkDragContext* context, gint x, gint y, GtkSelectionData* selection_data, guint info, guint time_, gpointer self) {
#line 2545 "project-manager.c"
	valide_project_manager_drag_data_received_cb (self, _sender, context, x, y, selection_data, info, time_);
}


static GObject * valide_project_manager_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValideProjectManagerClass * klass;
	GObjectClass * parent_class;
	ValideProjectManager * self;
	klass = VALIDE_PROJECT_MANAGER_CLASS (g_type_class_peek (VALIDE_TYPE_PROJECT_MANAGER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_PROJECT_MANAGER (obj);
	{
		GtkTreeViewColumn* col;
		GtkCellRenderer* renderer;
		gint targets_size;
		gint targets_length1;
		GtkTargetEntry* targets;
		GtkTreeView* _tmp0_;
		GtkTreeStore* _tmp1_;
		GtkTreeViewColumn* _tmp2_;
		GtkCellRenderer* _tmp3_;
		GtkCellRenderer* _tmp4_;
		GtkMenu* _tmp7_;
		GtkMenu* _tmp6_;
		GtkWidget* _tmp5_;
		GtkTargetEntry* _tmp8_;
		col = NULL;
		renderer = NULL;
		targets = (targets_length1 = 0, NULL);
#line 511 "project-manager.vala"
		gtk_scrolled_window_set_policy ((GtkScrolledWindow*) self, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
#line 2579 "project-manager.c"
		/* create some nice padding */
#line 513 "project-manager.vala"
		gtk_scrolled_window_set_shadow_type ((GtkScrolledWindow*) self, GTK_SHADOW_IN);
#line 514 "project-manager.vala"
		gtk_container_set_border_width ((GtkContainer*) self, (guint) 2);
#line 2585 "project-manager.c"
		_tmp0_ = NULL;
#line 516 "project-manager.vala"
		self->priv->tree_view = (_tmp0_ = g_object_ref_sink ((GtkTreeView*) gtk_tree_view_new ()), (self->priv->tree_view == NULL) ? NULL : (self->priv->tree_view = (g_object_unref (self->priv->tree_view), NULL)), _tmp0_);
#line 517 "project-manager.vala"
		gtk_tree_view_set_headers_visible (self->priv->tree_view, FALSE);
#line 518 "project-manager.vala"
		gtk_container_add ((GtkContainer*) self, (GtkWidget*) self->priv->tree_view);
#line 2593 "project-manager.c"
		_tmp1_ = NULL;
#line 520 "project-manager.vala"
		self->priv->tree_store = (_tmp1_ = gtk_tree_store_new ((gint) VALIDE_PROJECT_MANAGER_COL_NB, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, NULL), (self->priv->tree_store == NULL) ? NULL : (self->priv->tree_store = (g_object_unref (self->priv->tree_store), NULL)), _tmp1_);
#line 2597 "project-manager.c"
		/* @todo Use sorting function for show directories before */
#line 524 "project-manager.vala"
		gtk_tree_sortable_set_sort_column_id ((GtkTreeSortable*) self->priv->tree_store, (gint) VALIDE_PROJECT_MANAGER_COL_NAME, GTK_SORT_ASCENDING);
#line 525 "project-manager.vala"
		gtk_tree_sortable_set_sort_func ((GtkTreeSortable*) self->priv->tree_store, (gint) VALIDE_PROJECT_MANAGER_COL_NAME, _valide_project_manager_sort_func_gtk_tree_iter_compare_func, g_object_ref (self), g_object_unref);
#line 526 "project-manager.vala"
		gtk_tree_view_set_model (self->priv->tree_view, (GtkTreeModel*) self->priv->tree_store);
#line 2605 "project-manager.c"
		_tmp2_ = NULL;
#line 528 "project-manager.vala"
		col = (_tmp2_ = g_object_ref_sink (gtk_tree_view_column_new ()), (col == NULL) ? NULL : (col = (g_object_unref (col), NULL)), _tmp2_);
#line 529 "project-manager.vala"
		gtk_tree_view_column_set_title (col, _ ("Projects"));
#line 2611 "project-manager.c"
		_tmp3_ = NULL;
#line 531 "project-manager.vala"
		renderer = (_tmp3_ = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererPixbuf*) gtk_cell_renderer_pixbuf_new ()), (renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL)), _tmp3_);
#line 532 "project-manager.vala"
		gtk_cell_layout_pack_start ((GtkCellLayout*) col, renderer, FALSE);
#line 533 "project-manager.vala"
		gtk_tree_view_column_set_attributes (col, renderer, "pixbuf", VALIDE_PROJECT_MANAGER_COL_ICON, NULL);
#line 2619 "project-manager.c"
		_tmp4_ = NULL;
#line 535 "project-manager.vala"
		renderer = (_tmp4_ = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL)), _tmp4_);
#line 536 "project-manager.vala"
		gtk_cell_layout_pack_start ((GtkCellLayout*) col, renderer, FALSE);
#line 537 "project-manager.vala"
		gtk_tree_view_column_set_attributes (col, renderer, "markup", VALIDE_PROJECT_MANAGER_COL_NAME, NULL);
#line 539 "project-manager.vala"
		gtk_tree_view_append_column (self->priv->tree_view, col);
#line 541 "project-manager.vala"
		g_signal_connect_object (self->priv->tree_view, "row-activated", (GCallback) _valide_project_manager_file_select_gtk_tree_view_row_activated, self, 0);
#line 2631 "project-manager.c"
		/* Add popup */
		_tmp7_ = NULL;
#line 543 "project-manager.vala"
		_tmp6_ = NULL;
#line 543 "project-manager.vala"
		_tmp5_ = NULL;
#line 543 "project-manager.vala"
		self->priv->popup = (_tmp7_ = (_tmp6_ = (_tmp5_ = gtk_ui_manager_get_widget ((GtkUIManager*) self->priv->_ui_manager, "/project-popup"), GTK_IS_MENU (_tmp5_) ? ((GtkMenu*) _tmp5_) : NULL), (_tmp6_ == NULL) ? NULL : g_object_ref (_tmp6_)), (self->priv->popup == NULL) ? NULL : (self->priv->popup = (g_object_unref (self->priv->popup), NULL)), _tmp7_);
#line 544 "project-manager.vala"
		g_signal_connect_object ((GtkWidget*) self->priv->tree_view, "button-press-event", (GCallback) _valide_project_manager_button_press_gtk_widget_button_press_event, self, 0);
#line 546 "project-manager.vala"
		valide_project_manager_setup_ui (self);
#line 2644 "project-manager.c"
		/* Seting up the drag & drop */
		_tmp8_ = NULL;
#line 549 "project-manager.vala"
		targets = (_tmp8_ = g_new0 (GtkTargetEntry, 1), targets = (g_free (targets), NULL), targets_length1 = 1, targets_size = targets_length1, _tmp8_);
#line 550 "project-manager.vala"
		targets[0].target = "text/uri-list";
#line 551 "project-manager.vala"
		targets[0].flags = (guint) 0;
#line 552 "project-manager.vala"
		targets[0].info = (guint) 0;
#line 554 "project-manager.vala"
		gtk_drag_dest_set ((GtkWidget*) self->priv->tree_view, GTK_DEST_DEFAULT_ALL, targets, targets_length1, (GDK_ACTION_MOVE | GDK_ACTION_COPY) | GDK_ACTION_LINK);
#line 556 "project-manager.vala"
		g_signal_connect_object ((GtkWidget*) self->priv->tree_view, "drag-motion", (GCallback) _valide_project_manager_drag_motion_cb_gtk_widget_drag_motion, self, 0);
#line 557 "project-manager.vala"
		g_signal_connect_object ((GtkWidget*) self->priv->tree_view, "drag-data-received", (GCallback) _valide_project_manager_drag_data_received_cb_gtk_widget_drag_data_received, self, 0);
#line 2661 "project-manager.c"
		(col == NULL) ? NULL : (col = (g_object_unref (col), NULL));
		(renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL));
		targets = (g_free (targets), NULL);
	}
	return obj;
}


static void valide_project_manager_class_init (ValideProjectManagerClass * klass) {
	valide_project_manager_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideProjectManagerPrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_project_manager_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_project_manager_set_property;
	G_OBJECT_CLASS (klass)->constructor = valide_project_manager_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_project_manager_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PROJECT_MANAGER_UI_MANAGER, g_param_spec_object ("ui-manager", "ui-manager", "ui-manager", VALIDE_TYPE_UI_MANAGER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PROJECT_MANAGER_DOCUMENTS, g_param_spec_object ("documents", "documents", "documents", VALIDE_TYPE_DOCUMENT_MANAGER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PROJECT_MANAGER_BUILDERS, g_param_spec_object ("builders", "builders", "builders", VALIDE_TYPE_BUILDER_MANAGER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PROJECT_MANAGER_CURRENT, g_param_spec_object ("current", "current", "current", VALIDE_TYPE_PROJECT, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_signal_new ("project_opened", VALIDE_TYPE_PROJECT_MANAGER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, VALIDE_TYPE_PROJECT);
	g_signal_new ("project_closed", VALIDE_TYPE_PROJECT_MANAGER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("project_options_changed", VALIDE_TYPE_PROJECT_MANAGER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, VALIDE_TYPE_PROJECT);
}


static void valide_project_manager_instance_init (ValideProjectManager * self) {
	self->priv = VALIDE_PROJECT_MANAGER_GET_PRIVATE (self);
}


static void valide_project_manager_finalize (GObject* obj) {
	ValideProjectManager * self;
	self = VALIDE_PROJECT_MANAGER (obj);
	(self->priv->popup == NULL) ? NULL : (self->priv->popup = (g_object_unref (self->priv->popup), NULL));
	(self->priv->tree_view == NULL) ? NULL : (self->priv->tree_view = (g_object_unref (self->priv->tree_view), NULL));
	(self->priv->projects == NULL) ? NULL : (self->priv->projects = (_g_list_free_g_object_unref (self->priv->projects), NULL));
	(self->priv->tree_store == NULL) ? NULL : (self->priv->tree_store = (g_object_unref (self->priv->tree_store), NULL));
	(self->priv->_ui_manager == NULL) ? NULL : (self->priv->_ui_manager = (g_object_unref (self->priv->_ui_manager), NULL));
	(self->priv->_documents == NULL) ? NULL : (self->priv->_documents = (g_object_unref (self->priv->_documents), NULL));
	(self->priv->_builders == NULL) ? NULL : (self->priv->_builders = (g_object_unref (self->priv->_builders), NULL));
	G_OBJECT_CLASS (valide_project_manager_parent_class)->finalize (obj);
}


GType valide_project_manager_get_type (void) {
	static GType valide_project_manager_type_id = 0;
	if (valide_project_manager_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideProjectManagerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_project_manager_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideProjectManager), 0, (GInstanceInitFunc) valide_project_manager_instance_init, NULL };
		valide_project_manager_type_id = g_type_register_static (GTK_TYPE_SCROLLED_WINDOW, "ValideProjectManager", &g_define_type_info, 0);
	}
	return valide_project_manager_type_id;
}


static void valide_project_manager_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideProjectManager * self;
	gpointer boxed;
	self = VALIDE_PROJECT_MANAGER (object);
	switch (property_id) {
		case VALIDE_PROJECT_MANAGER_UI_MANAGER:
		g_value_set_object (value, valide_project_manager_get_ui_manager (self));
		break;
		case VALIDE_PROJECT_MANAGER_DOCUMENTS:
		g_value_set_object (value, valide_project_manager_get_documents (self));
		break;
		case VALIDE_PROJECT_MANAGER_BUILDERS:
		g_value_set_object (value, valide_project_manager_get_builders (self));
		break;
		case VALIDE_PROJECT_MANAGER_CURRENT:
		g_value_set_object (value, valide_project_manager_get_current (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_project_manager_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideProjectManager * self;
	self = VALIDE_PROJECT_MANAGER (object);
	switch (property_id) {
		case VALIDE_PROJECT_MANAGER_UI_MANAGER:
		valide_project_manager_set_ui_manager (self, g_value_get_object (value));
		break;
		case VALIDE_PROJECT_MANAGER_DOCUMENTS:
		valide_project_manager_set_documents (self, g_value_get_object (value));
		break;
		case VALIDE_PROJECT_MANAGER_BUILDERS:
		valide_project_manager_set_builders (self, g_value_get_object (value));
		break;
		case VALIDE_PROJECT_MANAGER_CURRENT:
		valide_project_manager_set_current (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




