/* project.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <glib/gstdio.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <stdio.h>
#include <gio/gio.h>
#include <glib/gi18n-lib.h>
#include <gtksourceview/gtksourceview.h>
#include <config.h>


#define G_YAML_TYPE_BUILDABLE (g_yaml_buildable_get_type ())
#define G_YAML_BUILDABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_TYPE_BUILDABLE, GYAMLBuildable))
#define G_YAML_IS_BUILDABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_TYPE_BUILDABLE))
#define G_YAML_BUILDABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), G_YAML_TYPE_BUILDABLE, GYAMLBuildableIface))

typedef struct _GYAMLBuildable GYAMLBuildable;
typedef struct _GYAMLBuildableIface GYAMLBuildableIface;

#define G_YAML_TYPE_BUILDER (g_yaml_builder_get_type ())
#define G_YAML_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_TYPE_BUILDER, GYAMLBuilder))
#define G_YAML_BUILDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), G_YAML_TYPE_BUILDER, GYAMLBuilderClass))
#define G_YAML_IS_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_TYPE_BUILDER))
#define G_YAML_IS_BUILDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_YAML_TYPE_BUILDER))
#define G_YAML_BUILDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), G_YAML_TYPE_BUILDER, GYAMLBuilderClass))

typedef struct _GYAMLBuilder GYAMLBuilder;
typedef struct _GYAMLBuilderClass GYAMLBuilderClass;

#define G_YAML_TYPE_NODE (g_yaml_node_get_type ())
#define G_YAML_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_TYPE_NODE, GYAMLNode))
#define G_YAML_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), G_YAML_TYPE_NODE, GYAMLNodeClass))
#define G_YAML_IS_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_TYPE_NODE))
#define G_YAML_IS_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_YAML_TYPE_NODE))
#define G_YAML_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), G_YAML_TYPE_NODE, GYAMLNodeClass))

typedef struct _GYAMLNode GYAMLNode;
typedef struct _GYAMLNodeClass GYAMLNodeClass;

#define VALIDE_TYPE_SOURCE (valide_source_get_type ())
#define VALIDE_SOURCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_SOURCE, ValideSource))
#define VALIDE_SOURCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_SOURCE, ValideSourceClass))
#define VALIDE_IS_SOURCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_SOURCE))
#define VALIDE_IS_SOURCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_SOURCE))
#define VALIDE_SOURCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_SOURCE, ValideSourceClass))

typedef struct _ValideSource ValideSource;
typedef struct _ValideSourceClass ValideSourceClass;
typedef struct _ValideSourcePrivate ValideSourcePrivate;

#define VALIDE_TYPE_VAPI_DIR (valide_vapi_dir_get_type ())
#define VALIDE_VAPI_DIR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_VAPI_DIR, ValideVapiDir))
#define VALIDE_VAPI_DIR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_VAPI_DIR, ValideVapiDirClass))
#define VALIDE_IS_VAPI_DIR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_VAPI_DIR))
#define VALIDE_IS_VAPI_DIR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_VAPI_DIR))
#define VALIDE_VAPI_DIR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_VAPI_DIR, ValideVapiDirClass))

typedef struct _ValideVapiDir ValideVapiDir;
typedef struct _ValideVapiDirClass ValideVapiDirClass;
typedef struct _ValideVapiDirPrivate ValideVapiDirPrivate;

#define VALIDE_TYPE_PACKAGE (valide_package_get_type ())
#define VALIDE_PACKAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PACKAGE, ValidePackage))
#define VALIDE_PACKAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PACKAGE, ValidePackageClass))
#define VALIDE_IS_PACKAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PACKAGE))
#define VALIDE_IS_PACKAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PACKAGE))
#define VALIDE_PACKAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PACKAGE, ValidePackageClass))

typedef struct _ValidePackage ValidePackage;
typedef struct _ValidePackageClass ValidePackageClass;
typedef struct _ValidePackagePrivate ValidePackagePrivate;

#define VALIDE_TYPE_PROJECT (valide_project_get_type ())
#define VALIDE_PROJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PROJECT, ValideProject))
#define VALIDE_PROJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PROJECT, ValideProjectClass))
#define VALIDE_IS_PROJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PROJECT))
#define VALIDE_IS_PROJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PROJECT))
#define VALIDE_PROJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PROJECT, ValideProjectClass))

typedef struct _ValideProject ValideProject;
typedef struct _ValideProjectClass ValideProjectClass;
typedef struct _ValideProjectPrivate ValideProjectPrivate;

#define VALIDE_TYPE_BUILDER_MANAGER (valide_builder_manager_get_type ())
#define VALIDE_BUILDER_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_BUILDER_MANAGER, ValideBuilderManager))
#define VALIDE_BUILDER_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_BUILDER_MANAGER, ValideBuilderManagerClass))
#define VALIDE_IS_BUILDER_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_BUILDER_MANAGER))
#define VALIDE_IS_BUILDER_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_BUILDER_MANAGER))
#define VALIDE_BUILDER_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_BUILDER_MANAGER, ValideBuilderManagerClass))

typedef struct _ValideBuilderManager ValideBuilderManager;
typedef struct _ValideBuilderManagerClass ValideBuilderManagerClass;

#define VALIDE_TYPE_DOCUMENT_MANAGER (valide_document_manager_get_type ())
#define VALIDE_DOCUMENT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_DOCUMENT_MANAGER, ValideDocumentManager))
#define VALIDE_DOCUMENT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_DOCUMENT_MANAGER, ValideDocumentManagerClass))
#define VALIDE_IS_DOCUMENT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_DOCUMENT_MANAGER))
#define VALIDE_IS_DOCUMENT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_DOCUMENT_MANAGER))
#define VALIDE_DOCUMENT_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_DOCUMENT_MANAGER, ValideDocumentManagerClass))

typedef struct _ValideDocumentManager ValideDocumentManager;
typedef struct _ValideDocumentManagerClass ValideDocumentManagerClass;

#define VALIDE_TYPE_BUILDER_OPTIONS (valide_builder_options_get_type ())
#define VALIDE_BUILDER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_BUILDER_OPTIONS, ValideBuilderOptions))
#define VALIDE_IS_BUILDER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_BUILDER_OPTIONS))
#define VALIDE_BUILDER_OPTIONS_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), VALIDE_TYPE_BUILDER_OPTIONS, ValideBuilderOptionsIface))

typedef struct _ValideBuilderOptions ValideBuilderOptions;
typedef struct _ValideBuilderOptionsIface ValideBuilderOptionsIface;

#define VALIDE_TYPE_EXECUTABLE_OPTIONS (valide_executable_options_get_type ())
#define VALIDE_EXECUTABLE_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_EXECUTABLE_OPTIONS, ValideExecutableOptions))
#define VALIDE_EXECUTABLE_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_EXECUTABLE_OPTIONS, ValideExecutableOptionsClass))
#define VALIDE_IS_EXECUTABLE_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_EXECUTABLE_OPTIONS))
#define VALIDE_IS_EXECUTABLE_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_EXECUTABLE_OPTIONS))
#define VALIDE_EXECUTABLE_OPTIONS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_EXECUTABLE_OPTIONS, ValideExecutableOptionsClass))

typedef struct _ValideExecutableOptions ValideExecutableOptions;
typedef struct _ValideExecutableOptionsClass ValideExecutableOptionsClass;

#define VALIDE_TYPE_NATIVE_BUILDER_OPTIONS (valide_native_builder_options_get_type ())
#define VALIDE_NATIVE_BUILDER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_NATIVE_BUILDER_OPTIONS, ValideNativeBuilderOptions))
#define VALIDE_NATIVE_BUILDER_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_NATIVE_BUILDER_OPTIONS, ValideNativeBuilderOptionsClass))
#define VALIDE_IS_NATIVE_BUILDER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_NATIVE_BUILDER_OPTIONS))
#define VALIDE_IS_NATIVE_BUILDER_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_NATIVE_BUILDER_OPTIONS))
#define VALIDE_NATIVE_BUILDER_OPTIONS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_NATIVE_BUILDER_OPTIONS, ValideNativeBuilderOptionsClass))

typedef struct _ValideNativeBuilderOptions ValideNativeBuilderOptions;
typedef struct _ValideNativeBuilderOptionsClass ValideNativeBuilderOptionsClass;

#define VALIDE_TYPE_IDOCUMENT (valide_idocument_get_type ())
#define VALIDE_IDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_IDOCUMENT, ValideiDocument))
#define VALIDE_IS_IDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_IDOCUMENT))
#define VALIDE_IDOCUMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), VALIDE_TYPE_IDOCUMENT, ValideiDocumentIface))

typedef struct _ValideiDocument ValideiDocument;
typedef struct _ValideiDocumentIface ValideiDocumentIface;

#define VALIDE_TYPE_UI_MANAGER (valide_ui_manager_get_type ())
#define VALIDE_UI_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_UI_MANAGER, ValideUIManager))
#define VALIDE_UI_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_UI_MANAGER, ValideUIManagerClass))
#define VALIDE_IS_UI_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_UI_MANAGER))
#define VALIDE_IS_UI_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_UI_MANAGER))
#define VALIDE_UI_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_UI_MANAGER, ValideUIManagerClass))

typedef struct _ValideUIManager ValideUIManager;
typedef struct _ValideUIManagerClass ValideUIManagerClass;

#define VALIDE_TYPE_DOCUMENT_STATE (valide_document_state_get_type ())

#define G_YAML_TYPE_WRITER (g_yaml_writer_get_type ())
#define G_YAML_WRITER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_YAML_TYPE_WRITER, GYAMLWriter))
#define G_YAML_WRITER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), G_YAML_TYPE_WRITER, GYAMLWriterClass))
#define G_YAML_IS_WRITER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), G_YAML_TYPE_WRITER))
#define G_YAML_IS_WRITER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), G_YAML_TYPE_WRITER))
#define G_YAML_WRITER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), G_YAML_TYPE_WRITER, GYAMLWriterClass))

typedef struct _GYAMLWriter GYAMLWriter;
typedef struct _GYAMLWriterClass GYAMLWriterClass;

#define VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT (valide_abstract_text_document_get_type ())
#define VALIDE_ABSTRACT_TEXT_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocument))
#define VALIDE_ABSTRACT_TEXT_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocumentClass))
#define VALIDE_IS_ABSTRACT_TEXT_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT))
#define VALIDE_IS_ABSTRACT_TEXT_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT))
#define VALIDE_ABSTRACT_TEXT_DOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocumentClass))

typedef struct _ValideAbstractTextDocument ValideAbstractTextDocument;
typedef struct _ValideAbstractTextDocumentClass ValideAbstractTextDocumentClass;

#define VALIDE_TYPE_SOURCE_BUFFER (valide_source_buffer_get_type ())
#define VALIDE_SOURCE_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_SOURCE_BUFFER, ValideSourceBuffer))
#define VALIDE_SOURCE_BUFFER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_SOURCE_BUFFER, ValideSourceBufferClass))
#define VALIDE_IS_SOURCE_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_SOURCE_BUFFER))
#define VALIDE_IS_SOURCE_BUFFER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_SOURCE_BUFFER))
#define VALIDE_SOURCE_BUFFER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_SOURCE_BUFFER, ValideSourceBufferClass))

typedef struct _ValideSourceBuffer ValideSourceBuffer;
typedef struct _ValideSourceBufferClass ValideSourceBufferClass;

#define VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG_REMOVE (valide_abstract_project_dialog_remove_get_type ())
#define VALIDE_ABSTRACT_PROJECT_DIALOG_REMOVE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG_REMOVE, ValideAbstractProjectDialogRemove))
#define VALIDE_ABSTRACT_PROJECT_DIALOG_REMOVE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG_REMOVE, ValideAbstractProjectDialogRemoveClass))
#define VALIDE_IS_ABSTRACT_PROJECT_DIALOG_REMOVE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG_REMOVE))
#define VALIDE_IS_ABSTRACT_PROJECT_DIALOG_REMOVE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG_REMOVE))
#define VALIDE_ABSTRACT_PROJECT_DIALOG_REMOVE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG_REMOVE, ValideAbstractProjectDialogRemoveClass))

typedef struct _ValideAbstractProjectDialogRemove ValideAbstractProjectDialogRemove;
typedef struct _ValideAbstractProjectDialogRemoveClass ValideAbstractProjectDialogRemoveClass;

#define VALIDE_TYPE_PROJECT_DIALOG_REMOVE (valide_project_dialog_remove_get_type ())
#define VALIDE_PROJECT_DIALOG_REMOVE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PROJECT_DIALOG_REMOVE, ValideProjectDialogRemove))
#define VALIDE_PROJECT_DIALOG_REMOVE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PROJECT_DIALOG_REMOVE, ValideProjectDialogRemoveClass))
#define VALIDE_IS_PROJECT_DIALOG_REMOVE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PROJECT_DIALOG_REMOVE))
#define VALIDE_IS_PROJECT_DIALOG_REMOVE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PROJECT_DIALOG_REMOVE))
#define VALIDE_PROJECT_DIALOG_REMOVE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PROJECT_DIALOG_REMOVE, ValideProjectDialogRemoveClass))

typedef struct _ValideProjectDialogRemove ValideProjectDialogRemove;
typedef struct _ValideProjectDialogRemoveClass ValideProjectDialogRemoveClass;

#define VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG_OPTIONS (valide_abstract_project_dialog_options_get_type ())
#define VALIDE_ABSTRACT_PROJECT_DIALOG_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG_OPTIONS, ValideAbstractProjectDialogOptions))
#define VALIDE_ABSTRACT_PROJECT_DIALOG_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG_OPTIONS, ValideAbstractProjectDialogOptionsClass))
#define VALIDE_IS_ABSTRACT_PROJECT_DIALOG_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG_OPTIONS))
#define VALIDE_IS_ABSTRACT_PROJECT_DIALOG_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG_OPTIONS))
#define VALIDE_ABSTRACT_PROJECT_DIALOG_OPTIONS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_PROJECT_DIALOG_OPTIONS, ValideAbstractProjectDialogOptionsClass))

typedef struct _ValideAbstractProjectDialogOptions ValideAbstractProjectDialogOptions;
typedef struct _ValideAbstractProjectDialogOptionsClass ValideAbstractProjectDialogOptionsClass;

#define VALIDE_TYPE_PROJECT_DIALOG_OPTIONS (valide_project_dialog_options_get_type ())
#define VALIDE_PROJECT_DIALOG_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PROJECT_DIALOG_OPTIONS, ValideProjectDialogOptions))
#define VALIDE_PROJECT_DIALOG_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PROJECT_DIALOG_OPTIONS, ValideProjectDialogOptionsClass))
#define VALIDE_IS_PROJECT_DIALOG_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PROJECT_DIALOG_OPTIONS))
#define VALIDE_IS_PROJECT_DIALOG_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PROJECT_DIALOG_OPTIONS))
#define VALIDE_PROJECT_DIALOG_OPTIONS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PROJECT_DIALOG_OPTIONS, ValideProjectDialogOptionsClass))

typedef struct _ValideProjectDialogOptions ValideProjectDialogOptions;
typedef struct _ValideProjectDialogOptionsClass ValideProjectDialogOptionsClass;

#define VALIDE_TYPE_BUILDER (valide_builder_get_type ())
#define VALIDE_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_BUILDER, ValideBuilder))
#define VALIDE_IS_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_BUILDER))
#define VALIDE_BUILDER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), VALIDE_TYPE_BUILDER, ValideBuilderIface))

typedef struct _ValideBuilder ValideBuilder;
typedef struct _ValideBuilderIface ValideBuilderIface;

#define VALIDE_TYPE_EXECUTABLE_MANAGER (valide_executable_manager_get_type ())
#define VALIDE_EXECUTABLE_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_EXECUTABLE_MANAGER, ValideExecutableManager))
#define VALIDE_EXECUTABLE_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_EXECUTABLE_MANAGER, ValideExecutableManagerClass))
#define VALIDE_IS_EXECUTABLE_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_EXECUTABLE_MANAGER))
#define VALIDE_IS_EXECUTABLE_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_EXECUTABLE_MANAGER))
#define VALIDE_EXECUTABLE_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_EXECUTABLE_MANAGER, ValideExecutableManagerClass))

typedef struct _ValideExecutableManager ValideExecutableManager;
typedef struct _ValideExecutableManagerClass ValideExecutableManagerClass;

#define VALIDE_TYPE_EXECUTABLE (valide_executable_get_type ())
#define VALIDE_EXECUTABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_EXECUTABLE, ValideExecutable))
#define VALIDE_EXECUTABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_EXECUTABLE, ValideExecutableClass))
#define VALIDE_IS_EXECUTABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_EXECUTABLE))
#define VALIDE_IS_EXECUTABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_EXECUTABLE))
#define VALIDE_EXECUTABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_EXECUTABLE, ValideExecutableClass))

typedef struct _ValideExecutable ValideExecutable;
typedef struct _ValideExecutableClass ValideExecutableClass;

/**
 * Buildable GObjects, from YAML.
 *
 *
 * Objects that implements GLib.YAML.Buildable is buildable by
 * GLib.YAML.Builder.
 *
 * This interface is almost the same as GtkBuildable.
 *
 **/
struct _GYAMLBuildableIface {
	GTypeInterface parent_iface;
	const char* (*get_name) (GYAMLBuildable* self);
	void (*set_name) (GYAMLBuildable* self, const char* name);
	void (*add_child) (GYAMLBuildable* self, GYAMLBuilder* builder, GObject* child, const char* type, GError** error);
	char** (*get_child_tags) (GYAMLBuildable* self, int* result_length1);
	GType* (*get_child_types) (GYAMLBuildable* self, int* result_length1);
	GList* (*get_children) (GYAMLBuildable* self, const char* type);
	GObject* (*get_internal_child) (GYAMLBuildable* self, GYAMLBuilder* builder, const char* child_name);
	void (*custom_node) (GYAMLBuildable* self, GYAMLBuilder* builder, const char* tag, GYAMLNode* node, GError** error);
};

struct _ValideSource {
	GObject parent_instance;
	ValideSourcePrivate * priv;
};

struct _ValideSourceClass {
	GObjectClass parent_class;
};

struct _ValideSourcePrivate {
	char* _path;
};

struct _ValideVapiDir {
	GObject parent_instance;
	ValideVapiDirPrivate * priv;
};

struct _ValideVapiDirClass {
	GObjectClass parent_class;
};

struct _ValideVapiDirPrivate {
	char* _path;
};

struct _ValidePackage {
	GObject parent_instance;
	ValidePackagePrivate * priv;
};

struct _ValidePackageClass {
	GObjectClass parent_class;
};

struct _ValidePackagePrivate {
	char* _name;
};

/**
 * A class for project
 */
struct _ValideProject {
	GObject parent_instance;
	ValideProjectPrivate * priv;
	char* filename;
	ValideBuilderManager* builders;
	GList* files;
	GList* packages;
	GList* vapi_dir;
};

struct _ValideProjectClass {
	GObjectClass parent_class;
};

struct _ValideBuilderOptionsIface {
	GTypeInterface parent_iface;
};

struct _ValideProjectPrivate {
	ValideDocumentManager* _documents;
	char* _path;
	char* _name;
	char* _author;
	char* _version;
	char* _license;
	char* _builder;
	ValideBuilderOptions* _builder_options;
	ValideExecutableOptions* _executable_options;
};

typedef enum  {
	VALIDE_DOCUMENT_STATE_NORMAL,
	VALIDE_DOCUMENT_STATE_EXTERNALLY_MODIFIED,
	VALIDE_DOCUMENT_STATE_BAD_ENCODING,
	VALIDE_DOCUMENT_STATE_INFO
} ValideDocumentState;

/**
 * An interface for document
 */
struct _ValideiDocumentIface {
	GTypeInterface parent_iface;
	void (*open) (ValideiDocument* self, const char* path, GError** error);
	void (*save) (ValideiDocument* self, const char* current_folder);
	void (*save_as) (ValideiDocument* self, const char* current_folder);
	void (*rename) (ValideiDocument* self);
	gboolean (*close) (ValideiDocument* self);
	GdkPixbuf* (*get_icon) (ValideiDocument* self);
	char* (*get_tooltips) (ValideiDocument* self);
	void (*setup_ui) (ValideiDocument* self, ValideUIManager* ui_manager);
	ValideDocumentState (*get_state) (ValideiDocument* self);
	void (*set_state) (ValideiDocument* self, ValideDocumentState value);
	gboolean (*get_is_save) (ValideiDocument* self);
	void (*set_is_save) (ValideiDocument* self, gboolean value);
	const char* (*get_path) (ValideiDocument* self);
	void (*set_path) (ValideiDocument* self, const char* value);
	const char* (*get_title) (ValideiDocument* self);
	const char* (*get_filename) (ValideiDocument* self);
};

typedef enum  {
	G_YAML_ERROR_SYMBOL_NOT_FOUND,
	G_YAML_ERROR_TYPE_NOT_FOUND,
	G_YAML_ERROR_NOT_A_BUILDABLE,
	G_YAML_ERROR_PROPERTY_NOT_FOUND,
	G_YAML_ERROR_OBJECT_NOT_FOUND,
	G_YAML_ERROR_UNKNOWN_PROPERTY_TYPE,
	G_YAML_ERROR_CUSTOM_NODE_ERROR,
	G_YAML_ERROR_UNEXPECTED_NODE,
	G_YAML_ERROR_NOT_IMPLEMENTED,
	G_YAML_ERROR_FILE_NOT_FOUND,
	G_YAML_ERROR_PARSER_ERROR,
	G_YAML_ERROR_UNRESOLVED_ALIAS
} GYAMLError;
#define G_YAML_ERROR g_yaml_error_quark ()
typedef enum  {
	VALIDE_DOCUMENT_ERROR_UNKNOW_DOCUMENT,
	VALIDE_DOCUMENT_ERROR_BAD_URI,
	VALIDE_DOCUMENT_ERROR_NO_DOCUMENT,
	VALIDE_DOCUMENT_ERROR_MIME
} ValideDocumentError;
#define VALIDE_DOCUMENT_ERROR valide_document_error_quark ()
/**
 * This instance use to create a new builder
 *
 * [[warning:
 *   You should register builder with BuilderManager.register_builder function.
 * ]]
 */
struct _ValideBuilderIface {
	GTypeInterface parent_iface;
	void (*configure) (ValideBuilder* self);
	void (*build) (ValideBuilder* self);
	void (*install) (ValideBuilder* self);
	void (*dist) (ValideBuilder* self);
	void (*clean) (ValideBuilder* self);
	void (*distclean) (ValideBuilder* self);
	void (*uninstall) (ValideBuilder* self);
	GtkWidget* (*preferences) (ValideBuilder* self);
	ValideProject* (*get_project) (ValideBuilder* self);
	void (*set_project) (ValideBuilder* self, ValideProject* value);
	ValideExecutableManager* (*get_executables) (ValideBuilder* self);
	void (*set_executables) (ValideBuilder* self, ValideExecutableManager* value);
};

typedef enum  {
	VALIDE_BUILDER_ERROR_NOT_FOUND
} ValideBuilderError;
#define VALIDE_BUILDER_ERROR valide_builder_error_quark ()

static gpointer valide_source_parent_class = NULL;
static GYAMLBuildableIface* valide_source_g_yaml_buildable_parent_iface = NULL;
static gpointer valide_vapi_dir_parent_class = NULL;
static GYAMLBuildableIface* valide_vapi_dir_g_yaml_buildable_parent_iface = NULL;
static gpointer valide_package_parent_class = NULL;
static GYAMLBuildableIface* valide_package_g_yaml_buildable_parent_iface = NULL;
static GType* valide_project_child_types;
static gint valide_project_child_types_length1;
static GType* valide_project_child_types = NULL;
static gint valide_project_child_types_length1 = 0;
static gint valide_project_child_types_size = 0;
static gpointer valide_project_parent_class = NULL;
static GYAMLBuildableIface* valide_project_g_yaml_buildable_parent_iface = NULL;

GType g_yaml_builder_get_type (void);
gpointer g_yaml_node_ref (gpointer instance);
void g_yaml_node_unref (gpointer instance);
GParamSpec* g_yaml_param_spec_node (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void g_yaml_value_set_node (GValue* value, gpointer v_object);
gpointer g_yaml_value_get_node (const GValue* value);
GType g_yaml_node_get_type (void);
GType g_yaml_buildable_get_type (void);
GType valide_source_get_type (void);
#define VALIDE_SOURCE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_SOURCE, ValideSourcePrivate))
enum  {
	VALIDE_SOURCE_DUMMY_PROPERTY,
	VALIDE_SOURCE_PATH
};
void valide_source_set_path (ValideSource* self, const char* value);
ValideSource* valide_source_new (const char* path);
ValideSource* valide_source_construct (GType object_type, const char* path);
const char* valide_source_get_path (ValideSource* self);
static void valide_source_finalize (GObject* obj);
static void valide_source_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_source_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType valide_vapi_dir_get_type (void);
#define VALIDE_VAPI_DIR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_VAPI_DIR, ValideVapiDirPrivate))
enum  {
	VALIDE_VAPI_DIR_DUMMY_PROPERTY,
	VALIDE_VAPI_DIR_PATH
};
void valide_vapi_dir_set_path (ValideVapiDir* self, const char* value);
ValideVapiDir* valide_vapi_dir_new (const char* path);
ValideVapiDir* valide_vapi_dir_construct (GType object_type, const char* path);
const char* valide_vapi_dir_get_path (ValideVapiDir* self);
static void valide_vapi_dir_finalize (GObject* obj);
static void valide_vapi_dir_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_vapi_dir_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType valide_package_get_type (void);
#define VALIDE_PACKAGE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_PACKAGE, ValidePackagePrivate))
enum  {
	VALIDE_PACKAGE_DUMMY_PROPERTY,
	VALIDE_PACKAGE_NAME
};
void valide_package_set_name (ValidePackage* self, const char* value);
ValidePackage* valide_package_new (const char* name);
ValidePackage* valide_package_construct (GType object_type, const char* name);
const char* valide_package_get_name (ValidePackage* self);
static void valide_package_finalize (GObject* obj);
static void valide_package_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_package_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType valide_project_get_type (void);
GType valide_builder_manager_get_type (void);
GType valide_document_manager_get_type (void);
GType valide_builder_options_get_type (void);
GType valide_executable_options_get_type (void);
#define VALIDE_PROJECT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_PROJECT, ValideProjectPrivate))
enum  {
	VALIDE_PROJECT_DUMMY_PROPERTY,
	VALIDE_PROJECT_PATH,
	VALIDE_PROJECT_NAME,
	VALIDE_PROJECT_AUTHOR,
	VALIDE_PROJECT_VERSION,
	VALIDE_PROJECT_LICENSE,
	VALIDE_PROJECT_BUILDER,
	VALIDE_PROJECT_BUILDER_OPTIONS,
	VALIDE_PROJECT_EXECUTABLE_OPTIONS
};
static void _g_list_free_g_object_unref (GList* self);
static void _lambda35_ (GMarkupParseContext* context, const char* element_name, char** attribute_names, int attribute_names_length1, char** attribute_values, int attribute_values_length1, ValideProject* self, GError** error);
static void __lambda35__gmarkup_parser_start_element_func (GMarkupParseContext* context, const char* element_name, char** attribute_names, char** attribute_values, gpointer self, GError** error);
static void _lambda36_ (GMarkupParseContext* context, const char* element_name, ValideProject* self, GError** error);
static void __lambda36__gmarkup_parser_end_element_func (GMarkupParseContext* context, const char* element_name, gpointer self, GError** error);
GType valide_native_builder_options_get_type (void);
ValideBuilderOptions* valide_project_get_builder_options (ValideProject* self);
void valide_project_set_name (ValideProject* self, const char* value);
void valide_project_set_author (ValideProject* self, const char* value);
void valide_project_set_version (ValideProject* self, const char* value);
void valide_project_set_license (ValideProject* self, const char* value);
void valide_native_builder_options_set_other (ValideNativeBuilderOptions* self, const char* value);
void valide_native_builder_options_set_debug (ValideNativeBuilderOptions* self, gboolean value);
void valide_native_builder_options_set_disable_assert (ValideNativeBuilderOptions* self, gboolean value);
void valide_native_builder_options_set_disable_checking (ValideNativeBuilderOptions* self, gboolean value);
void valide_native_builder_options_set_disable_non_null (ValideNativeBuilderOptions* self, gboolean value);
void valide_native_builder_options_set_quiet (ValideNativeBuilderOptions* self, gboolean value);
void valide_native_builder_options_set_save_temps (ValideNativeBuilderOptions* self, gboolean value);
void valide_native_builder_options_set_thread (ValideNativeBuilderOptions* self, gboolean value);
static void _lambda37_ (GMarkupParseContext* context, const char* text, gulong text_len, ValideProject* self, GError** error);
static void __lambda37__gmarkup_parser_text_func (GMarkupParseContext* context, const char* text, gulong text_len, gpointer self, GError** error);
ValideNativeBuilderOptions* valide_native_builder_options_new (void);
ValideNativeBuilderOptions* valide_native_builder_options_construct (GType object_type);
void valide_project_set_builder_options (ValideProject* self, ValideBuilderOptions* value);
void valide_project_save (ValideProject* self);
static void valide_project_xml_to_yml (ValideProject* self, const char* filename, GError** error);
static gboolean valide_project_exist (ValideProject* self, const char* filename);
const char* valide_project_get_path (ValideProject* self);
char* valide_utils_get_relative_path (const char* filename, const char* path);
GType valide_ui_manager_get_type (void);
GType valide_document_state_get_type (void);
GType valide_idocument_get_type (void);
const char* valide_idocument_get_path (ValideiDocument* self);
static void valide_project_document_changed (ValideProject* self, ValideDocumentManager* sender, ValideiDocument* document);
gpointer g_yaml_writer_ref (gpointer instance);
void g_yaml_writer_unref (gpointer instance);
GParamSpec* g_yaml_param_spec_writer (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void g_yaml_value_set_writer (GValue* value, gpointer v_object);
gpointer g_yaml_value_get_writer (const GValue* value);
GType g_yaml_writer_get_type (void);
GYAMLWriter* g_yaml_writer_new (const char* prefix);
GYAMLWriter* g_yaml_writer_construct (GType object_type, const char* prefix);
GQuark g_yaml_error_quark (void);
void g_yaml_writer_stream_object (GYAMLWriter* self, GObject* object, GString* sb, GError** error);
static char* valide_project_to_string (ValideProject* self, GError** error);
static void valide_project_real_add_child (GYAMLBuildable* base, GYAMLBuilder* builder, GObject* child, const char* type, GError** error);
static GType* valide_project_real_get_child_types (GYAMLBuildable* base, int* result_length1);
static char** valide_project_real_get_child_tags (GYAMLBuildable* base, int* result_length1);
static GList* valide_project_real_get_children (GYAMLBuildable* base, const char* type);
ValideProject* valide_project_new (void);
ValideProject* valide_project_construct (GType object_type);
GYAMLBuilder* g_yaml_builder_new (const char* prefix);
GYAMLBuilder* g_yaml_builder_construct (GType object_type, const char* prefix);
GObject* g_yaml_builder_build_from_file (GYAMLBuilder* self, FILE* file, GError** error);
static void valide_project_set_documents (ValideProject* self, ValideDocumentManager* value);
ValideProject* _valide_project_new_from_filename (const char* filename, ValideDocumentManager* documents, ValideBuilderManager* builders, GError** error);
char* valide_project_get_real_filename (ValideProject* self, const char* filename);
static ValideDocumentManager* valide_project_get_documents (ValideProject* self);
gboolean valide_document_manager_is_open (ValideDocumentManager* self, const char* filename, gint* pos);
void valide_document_manager_remove_page (ValideDocumentManager* self, ValideiDocument* child);
void valide_project_close (ValideProject* self);
ValideiDocument* valide_document_manager_create (ValideDocumentManager* self, const char* filename, GError** error);
GType valide_abstract_text_document_get_type (void);
GType valide_source_buffer_get_type (void);
ValideSourceBuffer* valide_abstract_text_document_get_buffer (ValideAbstractTextDocument* self);
void valide_idocument_save (ValideiDocument* self, const char* current_folder);
gboolean valide_idocument_get_is_save (ValideiDocument* self);
static void _g_slist_free_g_free (GSList* self);
void valide_project_add_file (ValideProject* self, GSList* files);
GQuark valide_document_error_quark (void);
void valide_project_create_file (ValideProject* self, GError** error);
ValideiDocument* valide_project_open_file (ValideProject* self, const char* filename, GError** error);
GType valide_abstract_project_dialog_remove_get_type (void);
GType valide_project_dialog_remove_get_type (void);
ValideProjectDialogRemove* valide_project_dialog_remove_new (ValideProject* project);
ValideProjectDialogRemove* valide_project_dialog_remove_construct (GType object_type, ValideProject* project);
gint valide_project_dialog_remove_run (ValideProjectDialogRemove* self);
GList* valide_project_dialog_remove_get_selected_files (ValideProjectDialogRemove* self);
void valide_project_dialog_remove_hide (ValideProjectDialogRemove* self);
ValideiDocument* valide_document_manager_get_nth (ValideDocumentManager* self, gint page_num);
gboolean valide_idocument_close (ValideiDocument* self);
void valide_project_dialog_remove_destroy (ValideProjectDialogRemove* self);
void valide_project_remove_file (ValideProject* self, GList* files);
char* valide_project_get_executable_name (ValideProject* self);
gboolean valide_project_is_compiled (ValideProject* self);
guint64 valide_utils_get_mtime (const char* filename, GError** error);
gboolean valide_project_is_uptodate (ValideProject* self);
GType valide_abstract_project_dialog_options_get_type (void);
GType valide_project_dialog_options_get_type (void);
ValideProjectDialogOptions* valide_project_dialog_options_new (ValideProject* project, GtkWindow* parent);
ValideProjectDialogOptions* valide_project_dialog_options_construct (GType object_type, ValideProject* project, GtkWindow* parent);
gint valide_project_dialog_options_run (ValideProjectDialogOptions* self);
void valide_project_dialog_options_destroy (ValideProjectDialogOptions* self);
void valide_project_option_dialog (ValideProject* self, GtkWindow* parent);
GType valide_executable_manager_get_type (void);
GType valide_builder_get_type (void);
GQuark valide_builder_error_quark (void);
ValideBuilder* valide_builder_manager_create_builder (ValideBuilderManager* self, ValideProject* project, GError** error);
void valide_builder_configure (ValideBuilder* self);
void valide_project_configure (ValideProject* self);
void valide_builder_build (ValideBuilder* self);
void valide_project_build (ValideProject* self);
void valide_builder_install (ValideBuilder* self);
void valide_project_install (ValideProject* self);
void valide_builder_uninstall (ValideBuilder* self);
void valide_project_uninstall (ValideProject* self);
void valide_builder_dist (ValideBuilder* self);
void valide_project_dist (ValideProject* self);
void valide_builder_clean (ValideBuilder* self);
void valide_project_clean (ValideProject* self);
void valide_builder_distclean (ValideBuilder* self);
void valide_project_distclean (ValideProject* self);
GType valide_executable_get_type (void);
ValideExecutable* valide_executable_new (void);
ValideExecutable* valide_executable_construct (GType object_type);
void valide_executable_set_executable (ValideExecutable* self, const char* value);
ValideExecutableManager* valide_builder_manager_get_executables (ValideBuilderManager* self);
void valide_executable_manager_run (ValideExecutableManager* self, ValideExecutable* executable, ValideExecutableOptions* options);
ValideExecutableOptions* valide_project_get_executable_options (ValideProject* self);
void valide_project_execute (ValideProject* self);
char* valide_utils_get_extension (const char* filename);
gboolean valide_project_accept_file (const char* filename);
const char* valide_executable_options_get_program (ValideExecutableOptions* self);
const char* valide_project_get_name (ValideProject* self);
const char* valide_project_get_author (ValideProject* self);
const char* valide_project_get_version (ValideProject* self);
const char* valide_project_get_license (ValideProject* self);
const char* valide_project_get_builder (ValideProject* self);
void valide_project_set_builder (ValideProject* self, const char* value);
void valide_project_set_executable_options (ValideProject* self, ValideExecutableOptions* value);
static void _valide_project_document_changed_valide_document_manager_tab_state_changed (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self);
ValideExecutableOptions* valide_executable_options_new (void);
ValideExecutableOptions* valide_executable_options_construct (GType object_type);
static void _lambda80_ (ValideProject* s, ValideProject* self);
static void __lambda80__valide_project_added_file (ValideProject* _sender, gpointer self);
static void _lambda81_ (ValideProject* s, ValideProject* self);
static void __lambda81__valide_project_removed_file (ValideProject* _sender, gpointer self);
static GObject * valide_project_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void valide_project_finalize (GObject* obj);
static void valide_project_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_project_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);

static const char* VALIDE_PROJECT_FILE_EXT[] = {"vide"};
static const char* VALIDE_PROJECT_child_tags[] = {"files", "packages", "vapi_dir"};


#line 26 "project.vala"
ValideSource* valide_source_construct (GType object_type, const char* path) {
#line 677 "project.c"
	ValideSource * self;
#line 26 "project.vala"
	g_return_val_if_fail (path != NULL, NULL);
#line 681 "project.c"
	self = g_object_newv (object_type, 0, NULL);
#line 28 "project.vala"
	valide_source_set_path (self, path);
#line 685 "project.c"
	return self;
}


#line 26 "project.vala"
ValideSource* valide_source_new (const char* path) {
#line 26 "project.vala"
	return valide_source_construct (VALIDE_TYPE_SOURCE, path);
#line 694 "project.c"
}


const char* valide_source_get_path (ValideSource* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_path;
	return result;
}


void valide_source_set_path (ValideSource* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 24 "project.vala"
	_tmp0_ = NULL;
#line 713 "project.c"
	self->priv->_path = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_path = (g_free (self->priv->_path), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "path");
}


static void valide_source_class_init (ValideSourceClass * klass) {
	valide_source_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideSourcePrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_source_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_source_set_property;
	G_OBJECT_CLASS (klass)->finalize = valide_source_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_SOURCE_PATH, g_param_spec_string ("path", "path", "path", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void valide_source_g_yaml_buildable_interface_init (GYAMLBuildableIface * iface) {
	valide_source_g_yaml_buildable_parent_iface = g_type_interface_peek_parent (iface);
}


static void valide_source_instance_init (ValideSource * self) {
	self->priv = VALIDE_SOURCE_GET_PRIVATE (self);
}


static void valide_source_finalize (GObject* obj) {
	ValideSource * self;
	self = VALIDE_SOURCE (obj);
	self->priv->_path = (g_free (self->priv->_path), NULL);
	G_OBJECT_CLASS (valide_source_parent_class)->finalize (obj);
}


GType valide_source_get_type (void) {
	static GType valide_source_type_id = 0;
	if (valide_source_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideSourceClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_source_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideSource), 0, (GInstanceInitFunc) valide_source_instance_init, NULL };
		static const GInterfaceInfo g_yaml_buildable_info = { (GInterfaceInitFunc) valide_source_g_yaml_buildable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		valide_source_type_id = g_type_register_static (G_TYPE_OBJECT, "ValideSource", &g_define_type_info, 0);
		g_type_add_interface_static (valide_source_type_id, G_YAML_TYPE_BUILDABLE, &g_yaml_buildable_info);
	}
	return valide_source_type_id;
}


static void valide_source_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideSource * self;
	gpointer boxed;
	self = VALIDE_SOURCE (object);
	switch (property_id) {
		case VALIDE_SOURCE_PATH:
		g_value_set_string (value, valide_source_get_path (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_source_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideSource * self;
	self = VALIDE_SOURCE (object);
	switch (property_id) {
		case VALIDE_SOURCE_PATH:
		valide_source_set_path (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


#line 36 "project.vala"
ValideVapiDir* valide_vapi_dir_construct (GType object_type, const char* path) {
#line 790 "project.c"
	GParameter * __params;
	GParameter * __params_it;
	ValideVapiDir * self;
#line 36 "project.vala"
	g_return_val_if_fail (path != NULL, NULL);
#line 796 "project.c"
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
#line 38 "project.vala"
	__params_it->name = "path";
#line 38 "project.vala"
	g_value_init (&__params_it->value, G_TYPE_STRING);
#line 38 "project.vala"
	g_value_set_string (&__params_it->value, path);
#line 38 "project.vala"
	__params_it++;
#line 807 "project.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 36 "project.vala"
ValideVapiDir* valide_vapi_dir_new (const char* path) {
#line 36 "project.vala"
	return valide_vapi_dir_construct (VALIDE_TYPE_VAPI_DIR, path);
#line 822 "project.c"
}


const char* valide_vapi_dir_get_path (ValideVapiDir* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_path;
	return result;
}


void valide_vapi_dir_set_path (ValideVapiDir* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 34 "project.vala"
	_tmp0_ = NULL;
#line 841 "project.c"
	self->priv->_path = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_path = (g_free (self->priv->_path), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "path");
}


static void valide_vapi_dir_class_init (ValideVapiDirClass * klass) {
	valide_vapi_dir_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideVapiDirPrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_vapi_dir_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_vapi_dir_set_property;
	G_OBJECT_CLASS (klass)->finalize = valide_vapi_dir_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_VAPI_DIR_PATH, g_param_spec_string ("path", "path", "path", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
}


static void valide_vapi_dir_g_yaml_buildable_interface_init (GYAMLBuildableIface * iface) {
	valide_vapi_dir_g_yaml_buildable_parent_iface = g_type_interface_peek_parent (iface);
}


static void valide_vapi_dir_instance_init (ValideVapiDir * self) {
	self->priv = VALIDE_VAPI_DIR_GET_PRIVATE (self);
}


static void valide_vapi_dir_finalize (GObject* obj) {
	ValideVapiDir * self;
	self = VALIDE_VAPI_DIR (obj);
	self->priv->_path = (g_free (self->priv->_path), NULL);
	G_OBJECT_CLASS (valide_vapi_dir_parent_class)->finalize (obj);
}


GType valide_vapi_dir_get_type (void) {
	static GType valide_vapi_dir_type_id = 0;
	if (valide_vapi_dir_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideVapiDirClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_vapi_dir_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideVapiDir), 0, (GInstanceInitFunc) valide_vapi_dir_instance_init, NULL };
		static const GInterfaceInfo g_yaml_buildable_info = { (GInterfaceInitFunc) valide_vapi_dir_g_yaml_buildable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		valide_vapi_dir_type_id = g_type_register_static (G_TYPE_OBJECT, "ValideVapiDir", &g_define_type_info, 0);
		g_type_add_interface_static (valide_vapi_dir_type_id, G_YAML_TYPE_BUILDABLE, &g_yaml_buildable_info);
	}
	return valide_vapi_dir_type_id;
}


static void valide_vapi_dir_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideVapiDir * self;
	gpointer boxed;
	self = VALIDE_VAPI_DIR (object);
	switch (property_id) {
		case VALIDE_VAPI_DIR_PATH:
		g_value_set_string (value, valide_vapi_dir_get_path (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_vapi_dir_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideVapiDir * self;
	self = VALIDE_VAPI_DIR (object);
	switch (property_id) {
		case VALIDE_VAPI_DIR_PATH:
		valide_vapi_dir_set_path (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


#line 46 "project.vala"
ValidePackage* valide_package_construct (GType object_type, const char* name) {
#line 918 "project.c"
	GParameter * __params;
	GParameter * __params_it;
	ValidePackage * self;
#line 46 "project.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 924 "project.c"
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
#line 48 "project.vala"
	__params_it->name = "name";
#line 48 "project.vala"
	g_value_init (&__params_it->value, G_TYPE_STRING);
#line 48 "project.vala"
	g_value_set_string (&__params_it->value, name);
#line 48 "project.vala"
	__params_it++;
#line 935 "project.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 46 "project.vala"
ValidePackage* valide_package_new (const char* name) {
#line 46 "project.vala"
	return valide_package_construct (VALIDE_TYPE_PACKAGE, name);
#line 950 "project.c"
}


const char* valide_package_get_name (ValidePackage* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_name;
	return result;
}


void valide_package_set_name (ValidePackage* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 44 "project.vala"
	_tmp0_ = NULL;
#line 969 "project.c"
	self->priv->_name = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_name = (g_free (self->priv->_name), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "name");
}


static void valide_package_class_init (ValidePackageClass * klass) {
	valide_package_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValidePackagePrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_package_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_package_set_property;
	G_OBJECT_CLASS (klass)->finalize = valide_package_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PACKAGE_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
}


static void valide_package_g_yaml_buildable_interface_init (GYAMLBuildableIface * iface) {
	valide_package_g_yaml_buildable_parent_iface = g_type_interface_peek_parent (iface);
}


static void valide_package_instance_init (ValidePackage * self) {
	self->priv = VALIDE_PACKAGE_GET_PRIVATE (self);
}


static void valide_package_finalize (GObject* obj) {
	ValidePackage * self;
	self = VALIDE_PACKAGE (obj);
	self->priv->_name = (g_free (self->priv->_name), NULL);
	G_OBJECT_CLASS (valide_package_parent_class)->finalize (obj);
}


GType valide_package_get_type (void) {
	static GType valide_package_type_id = 0;
	if (valide_package_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValidePackageClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_package_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValidePackage), 0, (GInstanceInitFunc) valide_package_instance_init, NULL };
		static const GInterfaceInfo g_yaml_buildable_info = { (GInterfaceInitFunc) valide_package_g_yaml_buildable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		valide_package_type_id = g_type_register_static (G_TYPE_OBJECT, "ValidePackage", &g_define_type_info, 0);
		g_type_add_interface_static (valide_package_type_id, G_YAML_TYPE_BUILDABLE, &g_yaml_buildable_info);
	}
	return valide_package_type_id;
}


static void valide_package_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValidePackage * self;
	gpointer boxed;
	self = VALIDE_PACKAGE (object);
	switch (property_id) {
		case VALIDE_PACKAGE_NAME:
		g_value_set_string (value, valide_package_get_name (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_package_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValidePackage * self;
	self = VALIDE_PACKAGE (object);
	switch (property_id) {
		case VALIDE_PACKAGE_NAME:
		valide_package_set_name (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _g_list_free_g_object_unref (GList* self) {
	g_list_foreach (self, (GFunc) g_object_unref, NULL);
	g_list_free (self);
}


static void _lambda35_ (GMarkupParseContext* context, const char* element_name, char** attribute_names, int attribute_names_length1, char** attribute_values, int attribute_values_length1, ValideProject* self, GError** error) {
	g_return_if_fail (context != NULL);
	g_return_if_fail (element_name != NULL);
	/* start_element */
#line 199 "project.vala"
	g_object_set_data ((GObject*) self, "current_element", element_name);
#line 1056 "project.c"
}


static void __lambda35__gmarkup_parser_start_element_func (GMarkupParseContext* context, const char* element_name, char** attribute_names, char** attribute_values, gpointer self, GError** error) {
	_lambda35_ (context, element_name, attribute_names, _vala_array_length (attribute_names), attribute_values, _vala_array_length (attribute_values), self, error);
}


static void _lambda36_ (GMarkupParseContext* context, const char* element_name, ValideProject* self, GError** error) {
	g_return_if_fail (context != NULL);
	g_return_if_fail (element_name != NULL);
	/* end_element */
#line 203 "project.vala"
	g_object_set_data ((GObject*) self, "current_element", NULL);
#line 1071 "project.c"
}


static void __lambda36__gmarkup_parser_end_element_func (GMarkupParseContext* context, const char* element_name, gpointer self, GError** error) {
	_lambda36_ (context, element_name, self, error);
}


static void _lambda37_ (GMarkupParseContext* context, const char* text, gulong text_len, ValideProject* self, GError** error) {
	char* property;
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (context != NULL);
	g_return_if_fail (text != NULL);
	/* text */
	property = NULL;
	_tmp1_ = NULL;
#line 209 "project.vala"
	_tmp0_ = NULL;
#line 209 "project.vala"
	property = (_tmp1_ = (_tmp0_ = (const char*) g_object_get_data ((GObject*) self, "current_element"), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), property = (g_free (property), NULL), _tmp1_);
#line 210 "project.vala"
	if (property != NULL) {
#line 1095 "project.c"
		ValideNativeBuilderOptions* options;
		ValideNativeBuilderOptions* _tmp4_;
		ValideNativeBuilderOptions* _tmp3_;
		ValideBuilderOptions* _tmp2_;
		GQuark _tmp6_;
		const char* _tmp5_;
		static GQuark _tmp6__label0 = 0;
		static GQuark _tmp6__label1 = 0;
		static GQuark _tmp6__label2 = 0;
		static GQuark _tmp6__label3 = 0;
		static GQuark _tmp6__label4 = 0;
		static GQuark _tmp6__label5 = 0;
		static GQuark _tmp6__label6 = 0;
		static GQuark _tmp6__label7 = 0;
		static GQuark _tmp6__label8 = 0;
		static GQuark _tmp6__label9 = 0;
		static GQuark _tmp6__label10 = 0;
		static GQuark _tmp6__label11 = 0;
		static GQuark _tmp6__label12 = 0;
		static GQuark _tmp6__label13 = 0;
		static GQuark _tmp6__label14 = 0;
		options = NULL;
		_tmp4_ = NULL;
#line 214 "project.vala"
		_tmp3_ = NULL;
#line 214 "project.vala"
		_tmp2_ = NULL;
#line 214 "project.vala"
		options = (_tmp4_ = (_tmp3_ = (_tmp2_ = self->priv->_builder_options, VALIDE_IS_NATIVE_BUILDER_OPTIONS (_tmp2_) ? ((ValideNativeBuilderOptions*) _tmp2_) : NULL), (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), (options == NULL) ? NULL : (options = (g_object_unref (options), NULL)), _tmp4_);
#line 215 "project.vala"
		_tmp5_ = NULL;
#line 1127 "project.c"
		_tmp5_ = property;
		_tmp6_ = (NULL == _tmp5_) ? 0 : g_quark_from_string (_tmp5_);
		if (_tmp6_ == ((0 != _tmp6__label0) ? _tmp6__label0 : (_tmp6__label0 = g_quark_from_static_string ("file"))))
		do {
#line 218 "project.vala"
			self->files = g_list_append (self->files, valide_source_new (text));
#line 219 "project.vala"
			break;
#line 1136 "project.c"
		} while (0); else if (_tmp6_ == ((0 != _tmp6__label1) ? _tmp6__label1 : (_tmp6__label1 = g_quark_from_static_string ("vapi"))))
		do {
#line 221 "project.vala"
			self->vapi_dir = g_list_append (self->vapi_dir, valide_vapi_dir_new (text));
#line 222 "project.vala"
			break;
#line 1143 "project.c"
		} while (0); else if (_tmp6_ == ((0 != _tmp6__label2) ? _tmp6__label2 : (_tmp6__label2 = g_quark_from_static_string ("pkg"))))
		do {
#line 224 "project.vala"
			self->packages = g_list_append (self->packages, valide_package_new (text));
#line 225 "project.vala"
			break;
#line 1150 "project.c"
		} while (0); else if (_tmp6_ == ((0 != _tmp6__label3) ? _tmp6__label3 : (_tmp6__label3 = g_quark_from_static_string ("name"))))
		do {
#line 227 "project.vala"
			valide_project_set_name (self, text);
#line 228 "project.vala"
			break;
#line 1157 "project.c"
		} while (0); else if (_tmp6_ == ((0 != _tmp6__label4) ? _tmp6__label4 : (_tmp6__label4 = g_quark_from_static_string ("author"))))
		do {
#line 230 "project.vala"
			valide_project_set_author (self, text);
#line 231 "project.vala"
			break;
#line 1164 "project.c"
		} while (0); else if (_tmp6_ == ((0 != _tmp6__label5) ? _tmp6__label5 : (_tmp6__label5 = g_quark_from_static_string ("version"))))
		do {
#line 233 "project.vala"
			valide_project_set_version (self, text);
#line 234 "project.vala"
			break;
#line 1171 "project.c"
		} while (0); else if (_tmp6_ == ((0 != _tmp6__label6) ? _tmp6__label6 : (_tmp6__label6 = g_quark_from_static_string ("license"))))
		do {
#line 236 "project.vala"
			valide_project_set_license (self, text);
#line 237 "project.vala"
			break;
#line 1178 "project.c"
		} while (0); else if (_tmp6_ == ((0 != _tmp6__label7) ? _tmp6__label7 : (_tmp6__label7 = g_quark_from_static_string ("other"))))
		do {
#line 239 "project.vala"
			valide_native_builder_options_set_other (options, text);
#line 240 "project.vala"
			break;
#line 1185 "project.c"
		} while (0); else if (_tmp6_ == ((0 != _tmp6__label8) ? _tmp6__label8 : (_tmp6__label8 = g_quark_from_static_string ("debug"))))
		do {
#line 242 "project.vala"
			valide_native_builder_options_set_debug (options, TRUE);
#line 243 "project.vala"
			break;
#line 1192 "project.c"
		} while (0); else if (_tmp6_ == ((0 != _tmp6__label9) ? _tmp6__label9 : (_tmp6__label9 = g_quark_from_static_string ("disable_assert"))))
		do {
#line 245 "project.vala"
			valide_native_builder_options_set_disable_assert (options, TRUE);
#line 246 "project.vala"
			break;
#line 1199 "project.c"
		} while (0); else if (_tmp6_ == ((0 != _tmp6__label10) ? _tmp6__label10 : (_tmp6__label10 = g_quark_from_static_string ("disable_checking"))))
		do {
#line 248 "project.vala"
			valide_native_builder_options_set_disable_checking (options, TRUE);
#line 249 "project.vala"
			break;
#line 1206 "project.c"
		} while (0); else if (_tmp6_ == ((0 != _tmp6__label11) ? _tmp6__label11 : (_tmp6__label11 = g_quark_from_static_string ("disable_non_null"))))
		do {
#line 251 "project.vala"
			valide_native_builder_options_set_disable_non_null (options, TRUE);
#line 252 "project.vala"
			break;
#line 1213 "project.c"
		} while (0); else if (_tmp6_ == ((0 != _tmp6__label12) ? _tmp6__label12 : (_tmp6__label12 = g_quark_from_static_string ("quiet"))))
		do {
#line 254 "project.vala"
			valide_native_builder_options_set_quiet (options, TRUE);
#line 255 "project.vala"
			break;
#line 1220 "project.c"
		} while (0); else if (_tmp6_ == ((0 != _tmp6__label13) ? _tmp6__label13 : (_tmp6__label13 = g_quark_from_static_string ("save_temps"))))
		do {
#line 257 "project.vala"
			valide_native_builder_options_set_save_temps (options, TRUE);
#line 258 "project.vala"
			break;
#line 1227 "project.c"
		} while (0); else if (_tmp6_ == ((0 != _tmp6__label14) ? _tmp6__label14 : (_tmp6__label14 = g_quark_from_static_string ("thread"))))
		do {
#line 260 "project.vala"
			valide_native_builder_options_set_thread (options, TRUE);
#line 261 "project.vala"
			break;
#line 1234 "project.c"
		} while (0);
		(options == NULL) ? NULL : (options = (g_object_unref (options), NULL));
	}
	property = (g_free (property), NULL);
}


static void __lambda37__gmarkup_parser_text_func (GMarkupParseContext* context, const char* text, gulong text_len, gpointer self, GError** error) {
	_lambda37_ (context, text, text_len, self, error);
}


#line 187 "project.vala"
static void valide_project_xml_to_yml (ValideProject* self, const char* filename, GError** error) {
#line 1249 "project.c"
	GError * _inner_error_;
	char* contents;
	char* _tmp2_;
	gboolean _tmp1_;
	char* _tmp0_;
#line 187 "project.vala"
	g_return_if_fail (self != NULL);
#line 187 "project.vala"
	g_return_if_fail (filename != NULL);
#line 1259 "project.c"
	_inner_error_ = NULL;
	contents = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
#line 191 "project.vala"
	_tmp1_ = g_file_get_contents (filename, &_tmp0_, NULL, &_inner_error_);
#line 191 "project.vala"
	contents = (_tmp2_ = _tmp0_, contents = (g_free (contents), NULL), _tmp2_);
#line 191 "project.vala"
	_tmp1_;
#line 1270 "project.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		contents = (g_free (contents), NULL);
		return;
	}
#line 192 "project.vala"
	if (g_str_has_prefix (contents, "<?xml")) {
#line 1278 "project.c"
		GMarkupParseContext* context;
		GMarkupParser parser = {__lambda35__gmarkup_parser_start_element_func, __lambda36__gmarkup_parser_end_element_func, __lambda37__gmarkup_parser_text_func, NULL, NULL};
		char* _tmp4_;
		const char* _tmp3_;
		ValideNativeBuilderOptions* _tmp5_;
		GMarkupParseContext* _tmp6_;
		context = NULL;
		/* passthrough 
		 error */
		_tmp4_ = NULL;
#line 270 "project.vala"
		_tmp3_ = NULL;
#line 270 "project.vala"
		self->filename = (_tmp4_ = (_tmp3_ = filename, (_tmp3_ == NULL) ? NULL : g_strdup (_tmp3_)), self->filename = (g_free (self->filename), NULL), _tmp4_);
#line 1293 "project.c"
		_tmp5_ = NULL;
#line 271 "project.vala"
		valide_project_set_builder_options (self, (ValideBuilderOptions*) (_tmp5_ = valide_native_builder_options_new ()));
#line 1297 "project.c"
		(_tmp5_ == NULL) ? NULL : (_tmp5_ = (g_object_unref (_tmp5_), NULL));
		_tmp6_ = NULL;
#line 272 "project.vala"
		context = (_tmp6_ = g_markup_parse_context_new (&parser, G_MARKUP_TREAT_CDATA_AS_TEXT, (void*) self, NULL), (context == NULL) ? NULL : (context = (g_markup_parse_context_free (context), NULL)), _tmp6_);
#line 275 "project.vala"
		g_markup_parse_context_parse (context, contents, g_utf8_strlen (contents, -1), &_inner_error_);
#line 1304 "project.c"
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(context == NULL) ? NULL : (context = (g_markup_parse_context_free (context), NULL));
			contents = (g_free (contents), NULL);
			return;
		}
#line 276 "project.vala"
		g_markup_parse_context_end_parse (context, &_inner_error_);
#line 1313 "project.c"
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(context == NULL) ? NULL : (context = (g_markup_parse_context_free (context), NULL));
			contents = (g_free (contents), NULL);
			return;
		}
#line 277 "project.vala"
		valide_project_save (self);
#line 1322 "project.c"
		(context == NULL) ? NULL : (context = (g_markup_parse_context_free (context), NULL));
	}
	contents = (g_free (contents), NULL);
}


#line 281 "project.vala"
static gboolean valide_project_exist (ValideProject* self, const char* filename) {
#line 1331 "project.c"
	gboolean result;
	gboolean ret;
#line 281 "project.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 281 "project.vala"
	g_return_val_if_fail (filename != NULL, FALSE);
#line 1338 "project.c"
	ret = FALSE;
	{
		GList* file_collection;
		GList* file_it;
#line 285 "project.vala"
		file_collection = self->files;
#line 1345 "project.c"
		for (file_it = file_collection; file_it != NULL; file_it = file_it->next) {
			ValideSource* _tmp0_;
			ValideSource* file;
#line 72 "project.vala"
			_tmp0_ = NULL;
#line 285 "project.vala"
			file = (_tmp0_ = (ValideSource*) file_it->data, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
#line 1353 "project.c"
			{
#line 287 "project.vala"
				if (_vala_strcmp0 (valide_source_get_path (file), filename) == 0) {
#line 289 "project.vala"
					ret = TRUE;
#line 1359 "project.c"
					(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
#line 290 "project.vala"
					break;
#line 1363 "project.c"
				}
				(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
			}
		}
	}
	result = ret;
	return result;
}


#line 296 "project.vala"
static void valide_project_document_changed (ValideProject* self, ValideDocumentManager* sender, ValideiDocument* document) {
#line 1376 "project.c"
	char* filename;
	char* old_filename;
	char* _tmp1_;
	const char* _tmp0_;
#line 296 "project.vala"
	g_return_if_fail (self != NULL);
#line 296 "project.vala"
	g_return_if_fail (sender != NULL);
#line 296 "project.vala"
	g_return_if_fail (document != NULL);
#line 1387 "project.c"
	filename = NULL;
	old_filename = NULL;
	_tmp1_ = NULL;
#line 301 "project.vala"
	_tmp0_ = NULL;
#line 301 "project.vala"
	old_filename = (_tmp1_ = (_tmp0_ = (const char*) g_object_get_data ((GObject*) document, "old_path"), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), old_filename = (g_free (old_filename), NULL), _tmp1_);
#line 1395 "project.c"
	{
		GList* file_collection;
		GList* file_it;
#line 302 "project.vala"
		file_collection = self->files;
#line 1401 "project.c"
		for (file_it = file_collection; file_it != NULL; file_it = file_it->next) {
			ValideSource* _tmp6_;
			ValideSource* file;
#line 72 "project.vala"
			_tmp6_ = NULL;
#line 302 "project.vala"
			file = (_tmp6_ = (ValideSource*) file_it->data, (_tmp6_ == NULL) ? NULL : g_object_ref (_tmp6_));
#line 1409 "project.c"
			{
#line 304 "project.vala"
				if (!g_path_is_absolute (valide_source_get_path (file))) {
#line 1413 "project.c"
					char* _tmp2_;
					_tmp2_ = NULL;
#line 306 "project.vala"
					filename = (_tmp2_ = g_build_filename (valide_project_get_path (self), valide_source_get_path (file), NULL), filename = (g_free (filename), NULL), _tmp2_);
#line 1418 "project.c"
				} else {
					char* _tmp4_;
					const char* _tmp3_;
					_tmp4_ = NULL;
#line 310 "project.vala"
					_tmp3_ = NULL;
#line 310 "project.vala"
					filename = (_tmp4_ = (_tmp3_ = valide_source_get_path (file), (_tmp3_ == NULL) ? NULL : g_strdup (_tmp3_)), filename = (g_free (filename), NULL), _tmp4_);
#line 1427 "project.c"
				}
#line 312 "project.vala"
				if (_vala_strcmp0 (filename, old_filename) == 0) {
#line 1431 "project.c"
					char* _tmp5_;
#line 57 "utils.vala"
					_tmp5_ = NULL;
#line 314 "project.vala"
					valide_source_set_path (file, _tmp5_ = valide_utils_get_relative_path (valide_idocument_get_path (document), valide_project_get_path (self)));
#line 1437 "project.c"
					_tmp5_ = (g_free (_tmp5_), NULL);
#line 315 "project.vala"
					g_signal_emit_by_name (self, "added-file");
#line 1441 "project.c"
					(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
#line 316 "project.vala"
					break;
#line 1445 "project.c"
				}
				(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
			}
		}
	}
	filename = (g_free (filename), NULL);
	old_filename = (g_free (old_filename), NULL);
}


/**
   * @todo Remove the path property
   */
#line 324 "project.vala"
static char* valide_project_to_string (ValideProject* self, GError** error) {
#line 1461 "project.c"
	char* result;
	GError * _inner_error_;
	GString* sb;
	GYAMLWriter* writer;
	GString* _tmp0_;
	GYAMLWriter* _tmp1_;
	const char* _tmp2_;
#line 324 "project.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 1471 "project.c"
	_inner_error_ = NULL;
	sb = NULL;
	writer = NULL;
	_tmp0_ = NULL;
#line 329 "project.vala"
	sb = (_tmp0_ = g_string_new (""), (sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL)), _tmp0_);
#line 1478 "project.c"
	_tmp1_ = NULL;
#line 330 "project.vala"
	writer = (_tmp1_ = g_yaml_writer_new ("Valide"), (writer == NULL) ? NULL : (writer = (g_yaml_writer_unref (writer), NULL)), _tmp1_);
#line 331 "project.vala"
	g_yaml_writer_stream_object (writer, (GObject*) self, sb, &_inner_error_);
#line 1484 "project.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		(sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL));
		(writer == NULL) ? NULL : (writer = (g_yaml_writer_unref (writer), NULL));
		return NULL;
	}
#line 332 "project.vala"
	_tmp2_ = NULL;
#line 1493 "project.c"
	result = (_tmp2_ = sb->str, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_));
	(sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL));
	(writer == NULL) ? NULL : (writer = (g_yaml_writer_unref (writer), NULL));
	return result;
	(sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL));
	(writer == NULL) ? NULL : (writer = (g_yaml_writer_unref (writer), NULL));
}


/**
   * @see Glib.YAML.Buildable.add_child
   */
#line 356 "project.vala"
static void valide_project_real_add_child (GYAMLBuildable* base, GYAMLBuilder* builder, GObject* child, const char* type, GError** error) {
#line 1508 "project.c"
	ValideProject * self;
	self = (ValideProject*) base;
#line 356 "project.vala"
	g_return_if_fail (builder != NULL);
#line 356 "project.vala"
	g_return_if_fail (child != NULL);
#line 359 "project.vala"
	if (_vala_strcmp0 (type, "files") == 0) {
#line 1517 "project.c"
		ValideSource* _tmp0_;
#line 361 "project.vala"
		_tmp0_ = NULL;
#line 361 "project.vala"
		self->files = g_list_prepend (self->files, (_tmp0_ = VALIDE_SOURCE (child), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)));
#line 1523 "project.c"
	} else {
#line 363 "project.vala"
		if (_vala_strcmp0 (type, "packages") == 0) {
#line 1527 "project.c"
			ValidePackage* _tmp1_;
#line 365 "project.vala"
			_tmp1_ = NULL;
#line 365 "project.vala"
			self->packages = g_list_prepend (self->packages, (_tmp1_ = VALIDE_PACKAGE (child), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)));
#line 1533 "project.c"
		} else {
#line 367 "project.vala"
			if (_vala_strcmp0 (type, "vapi_dir") == 0) {
#line 1537 "project.c"
				ValideVapiDir* _tmp2_;
#line 369 "project.vala"
				_tmp2_ = NULL;
#line 369 "project.vala"
				self->vapi_dir = g_list_prepend (self->vapi_dir, (_tmp2_ = VALIDE_VAPI_DIR (child), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)));
#line 1543 "project.c"
			}
		}
	}
}


/**
   * @see Glib.YAML.Buildable.get_child_types
   */
#line 376 "project.vala"
static GType* valide_project_real_get_child_types (GYAMLBuildable* base, int* result_length1) {
#line 1555 "project.c"
	ValideProject * self;
	GType* result;
	GType* _tmp0_;
	self = (ValideProject*) base;
#line 378 "project.vala"
	_tmp0_ = NULL;
#line 1562 "project.c"
	result = (_tmp0_ = valide_project_child_types, *result_length1 = valide_project_child_types_length1, _tmp0_);
	return result;
}


/**
   * @see Glib.YAML.Buildable.get_child_tags
   */
#line 384 "project.vala"
static char** valide_project_real_get_child_tags (GYAMLBuildable* base, int* result_length1) {
#line 1573 "project.c"
	ValideProject * self;
	char** result;
	const char** _tmp0_;
	self = (ValideProject*) base;
#line 386 "project.vala"
	_tmp0_ = NULL;
#line 1580 "project.c"
	result = (_tmp0_ = VALIDE_PROJECT_child_tags, *result_length1 = G_N_ELEMENTS (VALIDE_PROJECT_child_tags), _tmp0_);
	return result;
}


/**
   * @see Glib.YAML.Buildable.get_children
   */
#line 392 "project.vala"
static GList* valide_project_real_get_children (GYAMLBuildable* base, const char* type) {
#line 1591 "project.c"
	ValideProject * self;
	GList* result;
	self = (ValideProject*) base;
#line 394 "project.vala"
	if (_vala_strcmp0 (type, "files") == 0) {
#line 1597 "project.c"
		result = g_list_copy (self->files);
		return result;
	} else {
#line 398 "project.vala"
		if (_vala_strcmp0 (type, "packages") == 0) {
#line 1603 "project.c"
			result = g_list_copy (self->packages);
			return result;
		} else {
#line 402 "project.vala"
			if (_vala_strcmp0 (type, "vapi_dir") == 0) {
#line 1609 "project.c"
				result = g_list_copy (self->vapi_dir);
				return result;
			}
		}
	}
	result = NULL;
	return result;
}


/**
   * Create a new project
   *
   * @param filename The project file name
   * @param documents The document manager
   * @param builders The builder manager
   */
#line 416 "project.vala"
ValideProject* _valide_project_new_from_filename (const char* filename, ValideDocumentManager* documents, ValideBuilderManager* builders, GError** error) {
#line 1629 "project.c"
	ValideProject* result;
	GError * _inner_error_;
	ValideProject* project;
	FILE* file;
	ValideProject* _tmp0_;
	ValideProject* _tmp1_;
	FILE* _tmp2_;
#line 416 "project.vala"
	g_return_val_if_fail (filename != NULL, NULL);
#line 416 "project.vala"
	g_return_val_if_fail (documents != NULL, NULL);
#line 416 "project.vala"
	g_return_val_if_fail (builders != NULL, NULL);
#line 1643 "project.c"
	_inner_error_ = NULL;
	project = NULL;
	file = NULL;
	_tmp0_ = NULL;
#line 423 "project.vala"
	project = (_tmp0_ = valide_project_new (), (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), _tmp0_);
#line 424 "project.vala"
	valide_project_xml_to_yml (project, filename, &_inner_error_);
#line 1652 "project.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
		(file == NULL) ? NULL : (file = (fclose (file), NULL));
		return NULL;
	}
	_tmp1_ = NULL;
#line 425 "project.vala"
	project = (_tmp1_ = NULL, (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), _tmp1_);
#line 1662 "project.c"
	_tmp2_ = NULL;
#line 426 "project.vala"
	file = (_tmp2_ = fopen (filename, "r"), (file == NULL) ? NULL : (file = (fclose (file), NULL)), _tmp2_);
#line 427 "project.vala"
	if (file != NULL) {
#line 1668 "project.c"
		GYAMLBuilder* builder;
		GYAMLBuilder* _tmp3_;
		GObject* _tmp4_;
		ValideProject* _tmp6_;
		GObject* _tmp5_;
		ValideBuilderManager* _tmp8_;
		ValideBuilderManager* _tmp7_;
		char* _tmp10_;
		const char* _tmp9_;
		builder = NULL;
		_tmp3_ = NULL;
#line 431 "project.vala"
		builder = (_tmp3_ = g_yaml_builder_new ("Valide"), (builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL)), _tmp3_);
#line 1682 "project.c"
		_tmp4_ = g_yaml_builder_build_from_file (builder, file, &_inner_error_);
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
			(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
			(file == NULL) ? NULL : (file = (fclose (file), NULL));
			return NULL;
		}
		_tmp6_ = NULL;
#line 432 "project.vala"
		_tmp5_ = NULL;
#line 432 "project.vala"
		project = (_tmp6_ = (_tmp5_ = _tmp4_, VALIDE_IS_PROJECT (_tmp5_) ? ((ValideProject*) _tmp5_) : NULL), (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), _tmp6_);
#line 433 "project.vala"
		valide_project_set_documents (project, documents);
#line 1698 "project.c"
		_tmp8_ = NULL;
#line 434 "project.vala"
		_tmp7_ = NULL;
#line 434 "project.vala"
		project->builders = (_tmp8_ = (_tmp7_ = builders, (_tmp7_ == NULL) ? NULL : g_object_ref (_tmp7_)), (project->builders == NULL) ? NULL : (project->builders = (g_object_unref (project->builders), NULL)), _tmp8_);
#line 1704 "project.c"
		_tmp10_ = NULL;
#line 435 "project.vala"
		_tmp9_ = NULL;
#line 435 "project.vala"
		project->filename = (_tmp10_ = (_tmp9_ = filename, (_tmp9_ == NULL) ? NULL : g_strdup (_tmp9_)), project->filename = (g_free (project->filename), NULL), _tmp10_);
#line 1710 "project.c"
		(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
	} else {
		_inner_error_ = g_error_new (G_IO_ERROR, G_IO_ERROR_NOT_FOUND, "The file '%s' doesn't exist", filename);
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
			(file == NULL) ? NULL : (file = (fclose (file), NULL));
			return NULL;
		}
	}
	result = project;
	(file == NULL) ? NULL : (file = (fclose (file), NULL));
	return result;
}


/**
   * Close the project
   */
#line 447 "project.vala"
void valide_project_close (ValideProject* self) {
#line 447 "project.vala"
	g_return_if_fail (self != NULL);
#line 1734 "project.c"
	{
		GList* file_collection;
		GList* file_it;
#line 449 "project.vala"
		file_collection = self->files;
#line 1740 "project.c"
		for (file_it = file_collection; file_it != NULL; file_it = file_it->next) {
			ValideSource* _tmp4_;
			ValideSource* file;
#line 72 "project.vala"
			_tmp4_ = NULL;
#line 449 "project.vala"
			file = (_tmp4_ = (ValideSource*) file_it->data, (_tmp4_ == NULL) ? NULL : g_object_ref (_tmp4_));
#line 1748 "project.c"
			{
				gint pos;
				char* path;
				char* _tmp0_;
				pos = 0;
				path = NULL;
				_tmp0_ = NULL;
#line 454 "project.vala"
				path = (_tmp0_ = valide_project_get_real_filename (self, valide_source_get_path (file)), path = (g_free (path), NULL), _tmp0_);
#line 456 "project.vala"
				if (valide_document_manager_is_open (valide_project_get_documents (self), path, &pos)) {
#line 1760 "project.c"
					ValideiDocument* document;
					ValideiDocument* _tmp3_;
					ValideiDocument* _tmp2_;
					GtkWidget* _tmp1_;
					document = NULL;
					_tmp3_ = NULL;
#line 460 "project.vala"
					_tmp2_ = NULL;
#line 460 "project.vala"
					_tmp1_ = NULL;
#line 460 "project.vala"
					document = (_tmp3_ = (_tmp2_ = (_tmp1_ = gtk_notebook_get_nth_page ((GtkNotebook*) valide_project_get_documents (self), pos), VALIDE_IS_IDOCUMENT (_tmp1_) ? ((ValideiDocument*) _tmp1_) : NULL), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (document == NULL) ? NULL : (document = (g_object_unref (document), NULL)), _tmp3_);
#line 461 "project.vala"
					valide_document_manager_remove_page (valide_project_get_documents (self), document);
#line 1775 "project.c"
					(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
				}
				(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
				path = (g_free (path), NULL);
			}
		}
	}
#line 464 "project.vala"
	g_signal_emit_by_name (self, "closed");
#line 1785 "project.c"
}


/**
   * Save the project file
   *
   * [[warning:
   *   Normally, you don't have call this function, the project to save
   *   itself when its properties (options, files, ...) changed.
   * ]]
   */
#line 475 "project.vala"
void valide_project_save (ValideProject* self) {
#line 1799 "project.c"
	GError * _inner_error_;
#line 475 "project.vala"
	g_return_if_fail (self != NULL);
#line 1803 "project.c"
	_inner_error_ = NULL;
	{
		char* _tmp0_;
		char* _tmp1_;
		_tmp0_ = valide_project_to_string (self, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch6_g_error;
			goto __finally6;
		}
#line 324 "project.vala"
		_tmp1_ = NULL;
#line 479 "project.vala"
		g_file_set_contents (self->filename, _tmp1_ = _tmp0_, -1, &_inner_error_);
#line 1817 "project.c"
		if (_inner_error_ != NULL) {
			goto __catch6_g_error;
			goto __finally6;
		}
		_tmp1_ = (g_free (_tmp1_), NULL);
	}
	goto __finally6;
	__catch6_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 483 "project.vala"
			g_warning (_ ("Couldn't save the project: %s"), e->message);
#line 1833 "project.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally6:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
}


static void _g_slist_free_g_free (GSList* self) {
	g_slist_foreach (self, (GFunc) g_free, NULL);
	g_slist_free (self);
}


/**
   * Create a new file
   */
#line 490 "project.vala"
void valide_project_create_file (ValideProject* self, GError** error) {
#line 1857 "project.c"
	GError * _inner_error_;
	ValideiDocument* doc;
	GtkDialog* dialog;
	gboolean adding_file;
	GtkDialog* _tmp0_;
	GtkLabel* _tmp1_;
	ValideiDocument* _tmp2_;
	ValideiDocument* _tmp3_;
#line 490 "project.vala"
	g_return_if_fail (self != NULL);
#line 1868 "project.c"
	_inner_error_ = NULL;
	doc = NULL;
	dialog = NULL;
	adding_file = FALSE;
	_tmp0_ = NULL;
#line 496 "project.vala"
	dialog = (_tmp0_ = g_object_ref_sink ((GtkDialog*) gtk_dialog_new ()), (dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL)), _tmp0_);
#line 497 "project.vala"
	gtk_window_set_title ((GtkWindow*) dialog, _ ("Create file"));
#line 1878 "project.c"
	_tmp1_ = NULL;
#line 498 "project.vala"
	gtk_box_pack_start ((GtkBox*) dialog->vbox, (GtkWidget*) (_tmp1_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("Would you like to add the file to this project?")))), TRUE, TRUE, (guint) 0);
#line 1882 "project.c"
	(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
#line 499 "project.vala"
	gtk_dialog_add_button (dialog, GTK_STOCK_YES, (gint) GTK_RESPONSE_YES);
#line 500 "project.vala"
	gtk_dialog_add_button (dialog, GTK_STOCK_NO, (gint) GTK_RESPONSE_NO);
#line 501 "project.vala"
	gtk_widget_show_all ((GtkWidget*) dialog);
#line 502 "project.vala"
	if (gtk_dialog_run (dialog) == GTK_RESPONSE_YES) {
#line 504 "project.vala"
		adding_file = TRUE;
#line 1894 "project.c"
	}
#line 506 "project.vala"
	gtk_object_destroy ((GtkObject*) dialog);
#line 1898 "project.c"
	_tmp2_ = valide_document_manager_create (valide_project_get_documents (self), NULL, &_inner_error_);
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == VALIDE_DOCUMENT_ERROR) {
			g_propagate_error (error, _inner_error_);
			(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
			(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
			return;
		} else {
			(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
			(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
	_tmp3_ = NULL;
#line 508 "project.vala"
	doc = (_tmp3_ = _tmp2_, (doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL)), _tmp3_);
#line 509 "project.vala"
	if (adding_file) {
#line 511 "project.vala"
		if (VALIDE_IS_ABSTRACT_TEXT_DOCUMENT (doc)) {
#line 1921 "project.c"
			ValideiDocument* _tmp4_;
#line 513 "project.vala"
			_tmp4_ = NULL;
#line 513 "project.vala"
			gtk_text_buffer_set_modified ((GtkTextBuffer*) valide_abstract_text_document_get_buffer ((_tmp4_ = doc, VALIDE_IS_ABSTRACT_TEXT_DOCUMENT (_tmp4_) ? ((ValideAbstractTextDocument*) _tmp4_) : NULL)), TRUE);
#line 1927 "project.c"
		}
#line 515 "project.vala"
		valide_idocument_save (doc, valide_project_get_path (self));
#line 516 "project.vala"
		if (valide_idocument_get_is_save (doc)) {
#line 1933 "project.c"
			GSList* files;
			GSList* _tmp5_;
			const char* _tmp6_;
			files = NULL;
			_tmp5_ = NULL;
#line 520 "project.vala"
			files = (_tmp5_ = NULL, (files == NULL) ? NULL : (files = (_g_slist_free_g_free (files), NULL)), _tmp5_);
#line 521 "project.vala"
			_tmp6_ = NULL;
#line 521 "project.vala"
			files = g_slist_append (files, (_tmp6_ = valide_idocument_get_path (doc), (_tmp6_ == NULL) ? NULL : g_strdup (_tmp6_)));
#line 522 "project.vala"
			valide_project_add_file (self, files);
#line 1947 "project.c"
			(files == NULL) ? NULL : (files = (_g_slist_free_g_free (files), NULL));
		}
	}
	(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
	(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
}


/**
   * Open a file
   *
   * @param filename The name of the file
   *
   * @return The new document
   */
#line 534 "project.vala"
ValideiDocument* valide_project_open_file (ValideProject* self, const char* filename, GError** error) {
#line 1965 "project.c"
	ValideiDocument* result;
	GError * _inner_error_;
	char* path;
	ValideiDocument* document;
	char* _tmp0_;
#line 534 "project.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 534 "project.vala"
	g_return_val_if_fail (filename != NULL, NULL);
#line 1975 "project.c"
	_inner_error_ = NULL;
	path = NULL;
	document = NULL;
	_tmp0_ = NULL;
#line 539 "project.vala"
	path = (_tmp0_ = valide_project_get_real_filename (self, filename), path = (g_free (path), NULL), _tmp0_);
#line 540 "project.vala"
	if (g_file_test (path, G_FILE_TEST_EXISTS)) {
#line 1984 "project.c"
		ValideiDocument* _tmp1_;
		ValideiDocument* _tmp2_;
		_tmp1_ = valide_document_manager_create (valide_project_get_documents (self), path, &_inner_error_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == VALIDE_DOCUMENT_ERROR) {
				g_propagate_error (error, _inner_error_);
				path = (g_free (path), NULL);
				(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
				return NULL;
			} else {
				path = (g_free (path), NULL);
				(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return NULL;
			}
		}
		_tmp2_ = NULL;
#line 542 "project.vala"
		document = (_tmp2_ = _tmp1_, (document == NULL) ? NULL : (document = (g_object_unref (document), NULL)), _tmp2_);
#line 2005 "project.c"
	} else {
		_inner_error_ = g_error_new (VALIDE_DOCUMENT_ERROR, VALIDE_DOCUMENT_ERROR_BAD_URI, _ ("The file %s doesn't exist!"), filename);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == VALIDE_DOCUMENT_ERROR) {
				g_propagate_error (error, _inner_error_);
				path = (g_free (path), NULL);
				(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
				return NULL;
			} else {
				path = (g_free (path), NULL);
				(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return NULL;
			}
		}
	}
	result = document;
	path = (g_free (path), NULL);
	return result;
}


/**
   * Add a list of file in project
   *
   * @param files The list of files
   */
#line 556 "project.vala"
void valide_project_add_file (ValideProject* self, GSList* files) {
#line 2036 "project.c"
	GError * _inner_error_;
	GSList* f;
#line 556 "project.vala"
	g_return_if_fail (self != NULL);
#line 2041 "project.c"
	_inner_error_ = NULL;
	f = NULL;
#line 560 "project.vala"
	if (files == NULL) {
#line 2046 "project.c"
		GtkFileChooserDialog* dialog;
		GtkFileChooserDialog* _tmp0_;
		dialog = NULL;
		_tmp0_ = NULL;
#line 564 "project.vala"
		dialog = (_tmp0_ = g_object_ref_sink ((GtkFileChooserDialog*) gtk_file_chooser_dialog_new (_ ("Add file"), NULL, GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_ADD, GTK_RESPONSE_ACCEPT, NULL)), (dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL)), _tmp0_);
#line 569 "project.vala"
		gtk_file_chooser_set_current_folder ((GtkFileChooser*) dialog, valide_project_get_path (self));
#line 570 "project.vala"
		gtk_file_chooser_set_local_only ((GtkFileChooser*) dialog, TRUE);
#line 571 "project.vala"
		gtk_file_chooser_set_select_multiple ((GtkFileChooser*) dialog, TRUE);
#line 573 "project.vala"
		if (gtk_dialog_run ((GtkDialog*) dialog) == GTK_RESPONSE_ACCEPT) {
#line 2061 "project.c"
			GSList* _tmp1_;
			_tmp1_ = NULL;
#line 575 "project.vala"
			f = (_tmp1_ = gtk_file_chooser_get_filenames ((GtkFileChooser*) dialog), (f == NULL) ? NULL : (f = (_g_slist_free_g_free (f), NULL)), _tmp1_);
#line 576 "project.vala"
			files = f;
#line 2068 "project.c"
		}
#line 578 "project.vala"
		gtk_object_destroy ((GtkObject*) dialog);
#line 2072 "project.c"
		(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
	}
#line 580 "project.vala"
	if (files != NULL) {
#line 2077 "project.c"
		{
			GSList* file_collection;
			GSList* file_it;
#line 582 "project.vala"
			file_collection = files;
#line 2083 "project.c"
			for (file_it = file_collection; file_it != NULL; file_it = file_it->next) {
				const char* _tmp3_;
				char* file;
#line 556 "project.vala"
				_tmp3_ = NULL;
#line 582 "project.vala"
				file = (_tmp3_ = (const char*) file_it->data, (_tmp3_ == NULL) ? NULL : g_strdup (_tmp3_));
#line 2091 "project.c"
				{
					char* filename;
					filename = NULL;
					{
						char* _tmp2_;
#line 588 "project.vala"
						if (!g_file_test (file, G_FILE_TEST_EXISTS)) {
#line 590 "project.vala"
							g_file_set_contents (file, "", (glong) (-1), &_inner_error_);
#line 2101 "project.c"
							if (_inner_error_ != NULL) {
								goto __catch7_g_error;
								goto __finally7;
							}
						}
						_tmp2_ = NULL;
#line 593 "project.vala"
						filename = (_tmp2_ = valide_utils_get_relative_path (file, valide_project_get_path (self)), filename = (g_free (filename), NULL), _tmp2_);
#line 594 "project.vala"
						if (!valide_project_exist (self, filename)) {
#line 596 "project.vala"
							self->files = g_list_append (self->files, valide_source_new (filename));
#line 597 "project.vala"
							g_signal_emit_by_name (self, "added-file");
#line 2116 "project.c"
						} else {
#line 601 "project.vala"
							g_message (_ ("File '%s' already added!"), filename);
#line 2120 "project.c"
						}
					}
					goto __finally7;
					__catch7_g_error:
					{
						GError * e;
						e = _inner_error_;
						_inner_error_ = NULL;
						{
#line 606 "project.vala"
							g_warning (_ ("Couldn't create the file: %s"), file);
#line 2132 "project.c"
							(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
						}
					}
					__finally7:
					if (_inner_error_ != NULL) {
						file = (g_free (file), NULL);
						filename = (g_free (filename), NULL);
						(f == NULL) ? NULL : (f = (_g_slist_free_g_free (f), NULL));
						g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
						g_clear_error (&_inner_error_);
						return;
					}
					file = (g_free (file), NULL);
					filename = (g_free (filename), NULL);
				}
			}
		}
	}
	(f == NULL) ? NULL : (f = (_g_slist_free_g_free (f), NULL));
}


/**
   * Remove a list of file in project
   *
   * @param files The list of files
   */
#line 617 "project.vala"
void valide_project_remove_file (ValideProject* self, GList* files) {
#line 2162 "project.c"
	ValideProjectDialogRemove* dialog;
	GList* files_to_remove;
#line 617 "project.vala"
	g_return_if_fail (self != NULL);
#line 2167 "project.c"
	dialog = NULL;
	files_to_remove = NULL;
#line 622 "project.vala"
	if (files == NULL) {
#line 2172 "project.c"
		ValideProjectDialogRemove* _tmp0_;
		_tmp0_ = NULL;
#line 625 "project.vala"
		dialog = (_tmp0_ = g_object_ref_sink (valide_project_dialog_remove_new (self)), (dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL)), _tmp0_);
#line 626 "project.vala"
		if (valide_project_dialog_remove_run (dialog) == GTK_RESPONSE_APPLY) {
#line 628 "project.vala"
			files_to_remove = valide_project_dialog_remove_get_selected_files (dialog);
#line 2181 "project.c"
		}
#line 630 "project.vala"
		valide_project_dialog_remove_hide (dialog);
#line 2185 "project.c"
	} else {
#line 634 "project.vala"
		files_to_remove = files;
#line 2189 "project.c"
	}
#line 636 "project.vala"
	if (files_to_remove != NULL) {
#line 2193 "project.c"
		char* msg;
		gboolean remove;
		gint response;
		GtkMessageDialog* msg_dialog;
		GtkMessageDialog* _tmp5_;
		msg = NULL;
		remove = FALSE;
		response = 0;
		msg_dialog = NULL;
#line 643 "project.vala"
		if (g_list_length (files_to_remove) < 2) {
#line 2205 "project.c"
			char* _tmp2_;
			const char* _tmp1_;
			_tmp2_ = NULL;
#line 645 "project.vala"
			_tmp1_ = NULL;
#line 645 "project.vala"
			msg = (_tmp2_ = (_tmp1_ = _ ("This will remove the file from the project. Also delete the file?"), (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_)), msg = (g_free (msg), NULL), _tmp2_);
#line 2213 "project.c"
		} else {
			char* _tmp4_;
			const char* _tmp3_;
			_tmp4_ = NULL;
#line 649 "project.vala"
			_tmp3_ = NULL;
#line 649 "project.vala"
			msg = (_tmp4_ = (_tmp3_ = _ ("This will remove the files from the project. Also delete the files?"), (_tmp3_ == NULL) ? NULL : g_strdup (_tmp3_)), msg = (g_free (msg), NULL), _tmp4_);
#line 2222 "project.c"
		}
		_tmp5_ = NULL;
#line 651 "project.vala"
		msg_dialog = (_tmp5_ = g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, msg, NULL)), (msg_dialog == NULL) ? NULL : (msg_dialog = (g_object_unref (msg_dialog), NULL)), _tmp5_);
#line 656 "project.vala"
		gtk_dialog_add_button ((GtkDialog*) msg_dialog, GTK_STOCK_CANCEL, (gint) GTK_RESPONSE_CANCEL);
#line 657 "project.vala"
		gtk_widget_show_all ((GtkWidget*) msg_dialog);
#line 658 "project.vala"
		response = gtk_dialog_run ((GtkDialog*) msg_dialog);
#line 659 "project.vala"
		gtk_object_destroy ((GtkObject*) msg_dialog);
#line 660 "project.vala"
		if (response == GTK_RESPONSE_CANCEL) {
#line 2237 "project.c"
			msg = (g_free (msg), NULL);
			(msg_dialog == NULL) ? NULL : (msg_dialog = (g_object_unref (msg_dialog), NULL));
			(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
			return;
		}
#line 664 "project.vala"
		remove = response == GTK_RESPONSE_YES;
#line 2245 "project.c"
		{
			GList* file_collection;
			GList* file_it;
#line 665 "project.vala"
			file_collection = files_to_remove;
#line 2251 "project.c"
			for (file_it = file_collection; file_it != NULL; file_it = file_it->next) {
				const char* _tmp8_;
				char* file;
#line 620 "project.vala"
				_tmp8_ = NULL;
#line 665 "project.vala"
				file = (_tmp8_ = (const char*) file_it->data, (_tmp8_ == NULL) ? NULL : g_strdup (_tmp8_));
#line 2259 "project.c"
				{
					{
						GList* source_collection;
						GList* source_it;
#line 667 "project.vala"
						source_collection = self->files;
#line 2266 "project.c"
						for (source_it = source_collection; source_it != NULL; source_it = source_it->next) {
							ValideSource* source;
#line 667 "project.vala"
							source = (ValideSource*) source_it->data;
#line 2271 "project.c"
							{
#line 669 "project.vala"
								if (_vala_strcmp0 (valide_source_get_path (source), file) == 0) {
#line 671 "project.vala"
									self->files = g_list_remove (self->files, source);
#line 672 "project.vala"
									if (remove) {
#line 2279 "project.c"
										gint pos;
										char* filename;
										char* _tmp6_;
										pos = 0;
										filename = NULL;
										_tmp6_ = NULL;
#line 677 "project.vala"
										filename = (_tmp6_ = valide_project_get_real_filename (self, file), filename = (g_free (filename), NULL), _tmp6_);
#line 678 "project.vala"
										if (valide_document_manager_is_open (valide_project_get_documents (self), filename, &pos)) {
#line 2290 "project.c"
											ValideiDocument* _tmp7_;
#line 653 "document-manager.vala"
											_tmp7_ = NULL;
#line 680 "project.vala"
											valide_idocument_close (_tmp7_ = valide_document_manager_get_nth (valide_project_get_documents (self), pos));
#line 2296 "project.c"
											(_tmp7_ == NULL) ? NULL : (_tmp7_ = (g_object_unref (_tmp7_), NULL));
										}
#line 682 "project.vala"
										if (g_unlink (filename) < 0) {
#line 684 "project.vala"
											g_warning (_ ("Couldn't remove the file '%s' from disk"), filename);
#line 2303 "project.c"
										}
										filename = (g_free (filename), NULL);
									}
								}
							}
						}
					}
					file = (g_free (file), NULL);
				}
			}
		}
#line 690 "project.vala"
		if (dialog != NULL) {
#line 692 "project.vala"
			valide_project_dialog_remove_destroy (dialog);
#line 2319 "project.c"
		}
#line 694 "project.vala"
		g_signal_emit_by_name (self, "removed-file");
#line 2323 "project.c"
		msg = (g_free (msg), NULL);
		(msg_dialog == NULL) ? NULL : (msg_dialog = (g_object_unref (msg_dialog), NULL));
	}
	(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
}


/**
   * The project is compiled?
   */
#line 701 "project.vala"
gboolean valide_project_is_compiled (ValideProject* self) {
#line 2336 "project.c"
	gboolean result;
	char* _tmp0_;
	gboolean _tmp1_;
#line 701 "project.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 950 "project.vala"
	_tmp0_ = NULL;
#line 2344 "project.c"
	result = (_tmp1_ = g_file_test (_tmp0_ = valide_project_get_executable_name (self), G_FILE_TEST_EXISTS), _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_);
	return result;
}


/**
   * The executable is up to date?
   */
#line 709 "project.vala"
gboolean valide_project_is_uptodate (ValideProject* self) {
#line 2355 "project.c"
	gboolean result;
	GError * _inner_error_;
	char* filename;
	guint64 exec_mtime;
	guint64 file_mtime;
	gboolean uptodate;
	char* _tmp0_;
	guint64 _tmp1_;
	guint64 _tmp2_;
#line 709 "project.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 2367 "project.c"
	_inner_error_ = NULL;
	filename = NULL;
	exec_mtime = 0ULL;
	file_mtime = 0ULL;
	uptodate = TRUE;
#line 950 "project.vala"
	_tmp0_ = NULL;
#line 2375 "project.c"
	_tmp2_ = (_tmp1_ = valide_utils_get_mtime (_tmp0_ = valide_project_get_executable_name (self), &_inner_error_), _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_);
	if (_inner_error_ != NULL) {
		filename = (g_free (filename), NULL);
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return FALSE;
	}
#line 716 "project.vala"
	exec_mtime = _tmp2_;
#line 2385 "project.c"
	{
		GList* file_collection;
		GList* file_it;
#line 717 "project.vala"
		file_collection = self->files;
#line 2391 "project.c"
		for (file_it = file_collection; file_it != NULL; file_it = file_it->next) {
			ValideSource* _tmp5_;
			ValideSource* file;
#line 72 "project.vala"
			_tmp5_ = NULL;
#line 717 "project.vala"
			file = (_tmp5_ = (ValideSource*) file_it->data, (_tmp5_ == NULL) ? NULL : g_object_ref (_tmp5_));
#line 2399 "project.c"
			{
				char* _tmp3_;
				guint64 _tmp4_;
				_tmp3_ = NULL;
#line 719 "project.vala"
				filename = (_tmp3_ = valide_project_get_real_filename (self, valide_source_get_path (file)), filename = (g_free (filename), NULL), _tmp3_);
#line 2406 "project.c"
				_tmp4_ = valide_utils_get_mtime (filename, &_inner_error_);
				if (_inner_error_ != NULL) {
					(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
					filename = (g_free (filename), NULL);
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return FALSE;
				}
#line 720 "project.vala"
				file_mtime = _tmp4_;
#line 721 "project.vala"
				if (file_mtime > exec_mtime) {
#line 723 "project.vala"
					uptodate = FALSE;
#line 2421 "project.c"
					(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
#line 724 "project.vala"
					break;
#line 2425 "project.c"
				}
				(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
			}
		}
	}
	result = uptodate;
	filename = (g_free (filename), NULL);
	return result;
}


/**
   * Show the project option dialog
   */
#line 733 "project.vala"
void valide_project_option_dialog (ValideProject* self, GtkWindow* parent) {
#line 2442 "project.c"
	ValideProjectDialogOptions* dialog;
	ValideProjectDialogOptions* _tmp0_;
#line 733 "project.vala"
	g_return_if_fail (self != NULL);
#line 733 "project.vala"
	g_return_if_fail (parent != NULL);
#line 2449 "project.c"
	dialog = NULL;
	_tmp0_ = NULL;
#line 737 "project.vala"
	dialog = (_tmp0_ = g_object_ref_sink (valide_project_dialog_options_new (self, parent)), (dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL)), _tmp0_);
#line 738 "project.vala"
	valide_project_dialog_options_run (dialog);
#line 739 "project.vala"
	valide_project_dialog_options_destroy (dialog);
#line 2458 "project.c"
	(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
}


/**
   * Configure the project
   */
#line 745 "project.vala"
void valide_project_configure (ValideProject* self) {
#line 2468 "project.c"
	GError * _inner_error_;
	ValideBuilder* builder;
	ValideBuilder* _tmp0_;
	ValideBuilder* _tmp1_;
#line 745 "project.vala"
	g_return_if_fail (self != NULL);
#line 2475 "project.c"
	_inner_error_ = NULL;
	builder = NULL;
	_tmp0_ = valide_builder_manager_create_builder (self->builders, self, &_inner_error_);
	if (_inner_error_ != NULL) {
		(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	_tmp1_ = NULL;
#line 749 "project.vala"
	builder = (_tmp1_ = _tmp0_, (builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL)), _tmp1_);
#line 750 "project.vala"
	valide_builder_configure (builder);
#line 2490 "project.c"
	(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
}


/**
   * Compile the project
   */
#line 756 "project.vala"
void valide_project_build (ValideProject* self) {
#line 2500 "project.c"
	GError * _inner_error_;
	gint pos;
	gboolean saved;
#line 756 "project.vala"
	g_return_if_fail (self != NULL);
#line 2506 "project.c"
	_inner_error_ = NULL;
	pos = 0;
	saved = TRUE;
	{
		GList* file_collection;
		GList* file_it;
#line 761 "project.vala"
		file_collection = self->files;
#line 2515 "project.c"
		for (file_it = file_collection; file_it != NULL; file_it = file_it->next) {
			ValideSource* _tmp2_;
			ValideSource* file;
#line 72 "project.vala"
			_tmp2_ = NULL;
#line 761 "project.vala"
			file = (_tmp2_ = (ValideSource*) file_it->data, (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_));
#line 2523 "project.c"
			{
				char* path;
				char* _tmp0_;
				path = NULL;
				_tmp0_ = NULL;
#line 765 "project.vala"
				path = (_tmp0_ = valide_project_get_real_filename (self, valide_source_get_path (file)), path = (g_free (path), NULL), _tmp0_);
#line 766 "project.vala"
				if (valide_document_manager_is_open (valide_project_get_documents (self), path, &pos)) {
#line 2533 "project.c"
					ValideiDocument* document;
					ValideiDocument* _tmp1_;
					document = NULL;
					_tmp1_ = NULL;
#line 770 "project.vala"
					document = (_tmp1_ = valide_document_manager_get_nth (valide_project_get_documents (self), pos), (document == NULL) ? NULL : (document = (g_object_unref (document), NULL)), _tmp1_);
#line 771 "project.vala"
					valide_idocument_save (document, valide_project_get_path (self));
#line 772 "project.vala"
					if (!valide_idocument_get_is_save (document)) {
#line 774 "project.vala"
						saved = FALSE;
#line 2546 "project.c"
						(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
						(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
						path = (g_free (path), NULL);
#line 775 "project.vala"
						break;
#line 2552 "project.c"
					}
					(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
				}
				(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
				path = (g_free (path), NULL);
			}
		}
	}
#line 780 "project.vala"
	if (saved) {
#line 2563 "project.c"
		ValideBuilder* builder;
		ValideBuilder* _tmp3_;
		ValideBuilder* _tmp4_;
		builder = NULL;
		_tmp3_ = valide_builder_manager_create_builder (self->builders, self, &_inner_error_);
		if (_inner_error_ != NULL) {
			(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
		_tmp4_ = NULL;
#line 784 "project.vala"
		builder = (_tmp4_ = _tmp3_, (builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL)), _tmp4_);
#line 785 "project.vala"
		valide_builder_build (builder);
#line 2580 "project.c"
		(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
	}
}


/**
   * Install the project
   */
#line 792 "project.vala"
void valide_project_install (ValideProject* self) {
#line 2591 "project.c"
	GError * _inner_error_;
	ValideBuilder* builder;
	ValideBuilder* _tmp0_;
	ValideBuilder* _tmp1_;
#line 792 "project.vala"
	g_return_if_fail (self != NULL);
#line 2598 "project.c"
	_inner_error_ = NULL;
	builder = NULL;
	_tmp0_ = valide_builder_manager_create_builder (self->builders, self, &_inner_error_);
	if (_inner_error_ != NULL) {
		(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	_tmp1_ = NULL;
#line 796 "project.vala"
	builder = (_tmp1_ = _tmp0_, (builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL)), _tmp1_);
#line 797 "project.vala"
	valide_builder_install (builder);
#line 2613 "project.c"
	(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
}


/**
   * Uninstall the project
   */
#line 803 "project.vala"
void valide_project_uninstall (ValideProject* self) {
#line 2623 "project.c"
	GError * _inner_error_;
	ValideBuilder* builder;
	ValideBuilder* _tmp0_;
	ValideBuilder* _tmp1_;
#line 803 "project.vala"
	g_return_if_fail (self != NULL);
#line 2630 "project.c"
	_inner_error_ = NULL;
	builder = NULL;
	_tmp0_ = valide_builder_manager_create_builder (self->builders, self, &_inner_error_);
	if (_inner_error_ != NULL) {
		(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	_tmp1_ = NULL;
#line 807 "project.vala"
	builder = (_tmp1_ = _tmp0_, (builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL)), _tmp1_);
#line 808 "project.vala"
	valide_builder_uninstall (builder);
#line 2645 "project.c"
	(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
}


/**
   * Distribute the project
   */
#line 814 "project.vala"
void valide_project_dist (ValideProject* self) {
#line 2655 "project.c"
	GError * _inner_error_;
	ValideBuilder* builder;
	ValideBuilder* _tmp0_;
	ValideBuilder* _tmp1_;
#line 814 "project.vala"
	g_return_if_fail (self != NULL);
#line 2662 "project.c"
	_inner_error_ = NULL;
	builder = NULL;
	_tmp0_ = valide_builder_manager_create_builder (self->builders, self, &_inner_error_);
	if (_inner_error_ != NULL) {
		(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	_tmp1_ = NULL;
#line 818 "project.vala"
	builder = (_tmp1_ = _tmp0_, (builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL)), _tmp1_);
#line 819 "project.vala"
	valide_builder_dist (builder);
#line 2677 "project.c"
	(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
}


/**
   * Clean the project
   */
#line 825 "project.vala"
void valide_project_clean (ValideProject* self) {
#line 2687 "project.c"
	GError * _inner_error_;
	ValideBuilder* builder;
	ValideBuilder* _tmp0_;
	ValideBuilder* _tmp1_;
#line 825 "project.vala"
	g_return_if_fail (self != NULL);
#line 2694 "project.c"
	_inner_error_ = NULL;
	builder = NULL;
	_tmp0_ = valide_builder_manager_create_builder (self->builders, self, &_inner_error_);
	if (_inner_error_ != NULL) {
		(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	_tmp1_ = NULL;
#line 829 "project.vala"
	builder = (_tmp1_ = _tmp0_, (builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL)), _tmp1_);
#line 830 "project.vala"
	valide_builder_clean (builder);
#line 2709 "project.c"
	(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
}


/**
   * Clean the project for distribution
   */
#line 836 "project.vala"
void valide_project_distclean (ValideProject* self) {
#line 2719 "project.c"
	GError * _inner_error_;
	ValideBuilder* builder;
	ValideBuilder* _tmp0_;
	ValideBuilder* _tmp1_;
#line 836 "project.vala"
	g_return_if_fail (self != NULL);
#line 2726 "project.c"
	_inner_error_ = NULL;
	builder = NULL;
	_tmp0_ = valide_builder_manager_create_builder (self->builders, self, &_inner_error_);
	if (_inner_error_ != NULL) {
		(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	_tmp1_ = NULL;
#line 840 "project.vala"
	builder = (_tmp1_ = _tmp0_, (builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL)), _tmp1_);
#line 841 "project.vala"
	valide_builder_distclean (builder);
#line 2741 "project.c"
	(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
}


/**
   * Execute the project
   */
#line 847 "project.vala"
void valide_project_execute (ValideProject* self) {
#line 847 "project.vala"
	g_return_if_fail (self != NULL);
#line 849 "project.vala"
	if (valide_project_is_compiled (self)) {
#line 2755 "project.c"
		gboolean run;
		run = TRUE;
#line 853 "project.vala"
		if (!valide_project_is_uptodate (self)) {
#line 2760 "project.c"
			GtkMessageDialog* dialog;
			GtkMessageDialog* _tmp0_;
			dialog = NULL;
			_tmp0_ = NULL;
#line 857 "project.vala"
			dialog = (_tmp0_ = g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, _ ("The executable is too old, would you like really execute it?"), NULL)), (dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL)), _tmp0_);
#line 861 "project.vala"
			if (gtk_dialog_run ((GtkDialog*) dialog) == GTK_RESPONSE_NO) {
#line 863 "project.vala"
				run = FALSE;
#line 2771 "project.c"
			}
#line 865 "project.vala"
			gtk_object_destroy ((GtkObject*) dialog);
#line 2775 "project.c"
			(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
		}
#line 867 "project.vala"
		if (run) {
#line 2780 "project.c"
			ValideExecutable* executable;
			ValideExecutable* _tmp1_;
			char* _tmp3_;
			char* _tmp2_;
			executable = NULL;
			_tmp1_ = NULL;
#line 871 "project.vala"
			executable = (_tmp1_ = valide_executable_new (), (executable == NULL) ? NULL : (executable = (g_object_unref (executable), NULL)), _tmp1_);
#line 820 "glib-2.0.vapi"
			_tmp3_ = NULL;
#line 950 "project.vala"
			_tmp2_ = NULL;
#line 872 "project.vala"
			valide_executable_set_executable (executable, _tmp3_ = g_strdup_printf ("'%s'", _tmp2_ = valide_project_get_executable_name (self)));
#line 2795 "project.c"
			_tmp3_ = (g_free (_tmp3_), NULL);
			_tmp2_ = (g_free (_tmp2_), NULL);
#line 873 "project.vala"
			valide_executable_manager_run (valide_builder_manager_get_executables (self->builders), executable, self->priv->_executable_options);
#line 2800 "project.c"
			(executable == NULL) ? NULL : (executable = (g_object_unref (executable), NULL));
		}
	} else {
#line 879 "project.vala"
		g_warning (_ ("You should compile the project before execute it!"));
#line 2806 "project.c"
	}
}


/**
   * Test if a filename is a project file.
   *
   * @param filename a filename.
   *
   * @return true if a filename is a project file.
   */
#line 890 "project.vala"
gboolean valide_project_accept_file (const char* filename) {
#line 2820 "project.c"
	gboolean result;
	gboolean accept;
	char* ext;
	char* _tmp0_;
#line 890 "project.vala"
	g_return_val_if_fail (filename != NULL, FALSE);
#line 2827 "project.c"
	accept = FALSE;
	ext = NULL;
	_tmp0_ = NULL;
#line 895 "project.vala"
	ext = (_tmp0_ = valide_utils_get_extension (filename), ext = (g_free (ext), NULL), _tmp0_);
#line 2833 "project.c"
	{
		const char** e_collection;
		int e_collection_length1;
		int e_it;
#line 896 "project.vala"
		e_collection = VALIDE_PROJECT_FILE_EXT;
#line 2840 "project.c"
		e_collection_length1 = G_N_ELEMENTS (VALIDE_PROJECT_FILE_EXT);
		for (e_it = 0; e_it < G_N_ELEMENTS (VALIDE_PROJECT_FILE_EXT); e_it = e_it + 1) {
			const char* _tmp1_;
			char* e;
#line 60 "project.vala"
			_tmp1_ = NULL;
#line 2847 "project.c"
			e = (_tmp1_ = e_collection[e_it], (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_));
			{
#line 898 "project.vala"
				if (_vala_strcmp0 (e, ext) == 0) {
#line 900 "project.vala"
					accept = TRUE;
#line 2854 "project.c"
					e = (g_free (e), NULL);
#line 901 "project.vala"
					break;
#line 2858 "project.c"
				}
				e = (g_free (e), NULL);
			}
		}
	}
	result = accept;
	ext = (g_free (ext), NULL);
	return result;
}


/**
   * Get the real file name
   *
   * @param filename A file include in this project
   *
   * @return The real name of the file
   */
#line 914 "project.vala"
char* valide_project_get_real_filename (ValideProject* self, const char* filename) {
#line 2879 "project.c"
	char* result;
	char* path;
#line 914 "project.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 914 "project.vala"
	g_return_val_if_fail (filename != NULL, NULL);
#line 2886 "project.c"
	path = NULL;
#line 918 "project.vala"
	if (!g_path_is_absolute (filename)) {
#line 2890 "project.c"
		char* _tmp0_;
		_tmp0_ = NULL;
#line 920 "project.vala"
		path = (_tmp0_ = g_build_filename (valide_project_get_path (self), filename, NULL), path = (g_free (path), NULL), _tmp0_);
#line 921 "project.vala"
		if (!g_file_test (path, G_FILE_TEST_EXISTS)) {
#line 2897 "project.c"
			char* _tmp1_;
			_tmp1_ = NULL;
#line 923 "project.vala"
			path = (_tmp1_ = NULL, path = (g_free (path), NULL), _tmp1_);
#line 2902 "project.c"
			{
				GList* file_collection;
				GList* file_it;
#line 924 "project.vala"
				file_collection = self->files;
#line 2908 "project.c"
				for (file_it = file_collection; file_it != NULL; file_it = file_it->next) {
					ValideSource* _tmp4_;
					ValideSource* file;
#line 72 "project.vala"
					_tmp4_ = NULL;
#line 924 "project.vala"
					file = (_tmp4_ = (ValideSource*) file_it->data, (_tmp4_ == NULL) ? NULL : g_object_ref (_tmp4_));
#line 2916 "project.c"
					{
#line 926 "project.vala"
						if (g_str_has_suffix (valide_source_get_path (file), filename)) {
#line 2920 "project.c"
							char* _tmp3_;
							const char* _tmp2_;
							_tmp3_ = NULL;
#line 928 "project.vala"
							_tmp2_ = NULL;
#line 928 "project.vala"
							path = (_tmp3_ = (_tmp2_ = valide_source_get_path (file), (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)), path = (g_free (path), NULL), _tmp3_);
#line 2928 "project.c"
							(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
#line 929 "project.vala"
							break;
#line 2932 "project.c"
						}
						(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
					}
				}
			}
#line 932 "project.vala"
			if (path == NULL) {
#line 2940 "project.c"
				char* _tmp6_;
				const char* _tmp5_;
				_tmp6_ = NULL;
#line 934 "project.vala"
				_tmp5_ = NULL;
#line 934 "project.vala"
				path = (_tmp6_ = (_tmp5_ = filename, (_tmp5_ == NULL) ? NULL : g_strdup (_tmp5_)), path = (g_free (path), NULL), _tmp6_);
#line 2948 "project.c"
			}
		}
	} else {
		char* _tmp8_;
		const char* _tmp7_;
		_tmp8_ = NULL;
#line 940 "project.vala"
		_tmp7_ = NULL;
#line 940 "project.vala"
		path = (_tmp8_ = (_tmp7_ = filename, (_tmp7_ == NULL) ? NULL : g_strdup (_tmp7_)), path = (g_free (path), NULL), _tmp8_);
#line 2959 "project.c"
	}
	result = path;
	return result;
}


/**
   * Return the complete path of the executable
   *
   * @return The name of the executable
   */
#line 950 "project.vala"
char* valide_project_get_executable_name (ValideProject* self) {
#line 2973 "project.c"
	char* result;
	char* exe_name;
#line 950 "project.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 2978 "project.c"
	exe_name = NULL;
#line 954 "project.vala"
	if (_vala_strcmp0 (valide_executable_options_get_program (self->priv->_executable_options), "") == 0) {
#line 2982 "project.c"
		char* _tmp0_;
		_tmp0_ = NULL;
#line 956 "project.vala"
		exe_name = (_tmp0_ = g_build_filename (valide_project_get_path (self), self->priv->_name, NULL), exe_name = (g_free (exe_name), NULL), _tmp0_);
#line 957 "project.vala"
		if (_vala_strcmp0 (OS, "win32") == 0) {
#line 2989 "project.c"
			char* _tmp1_;
			_tmp1_ = NULL;
#line 959 "project.vala"
			exe_name = (_tmp1_ = g_strconcat (exe_name, ".exe", NULL), exe_name = (g_free (exe_name), NULL), _tmp1_);
#line 2994 "project.c"
		}
	} else {
		char* _tmp3_;
		const char* _tmp2_;
		_tmp3_ = NULL;
#line 964 "project.vala"
		_tmp2_ = NULL;
#line 964 "project.vala"
		exe_name = (_tmp3_ = (_tmp2_ = valide_executable_options_get_program (self->priv->_executable_options), (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)), exe_name = (g_free (exe_name), NULL), _tmp3_);
#line 3004 "project.c"
	}
	result = exe_name;
	return result;
}


/**
 * A class for project
 */
#line 55 "project.vala"
ValideProject* valide_project_construct (GType object_type) {
#line 3016 "project.c"
	ValideProject * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


#line 55 "project.vala"
ValideProject* valide_project_new (void) {
#line 55 "project.vala"
	return valide_project_construct (VALIDE_TYPE_PROJECT);
#line 3027 "project.c"
}


const char* valide_project_get_path (ValideProject* self) {
	const char* result;
	char* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = NULL;
#line 126 "project.vala"
	self->priv->_path = (_tmp0_ = g_path_get_dirname (self->filename), self->priv->_path = (g_free (self->priv->_path), NULL), _tmp0_);
#line 3038 "project.c"
	result = self->priv->_path;
	return result;
}


const char* valide_project_get_name (ValideProject* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_name;
	return result;
}


void valide_project_set_name (ValideProject* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 134 "project.vala"
	_tmp0_ = NULL;
#line 3059 "project.c"
	self->priv->_name = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_name = (g_free (self->priv->_name), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "name");
}


const char* valide_project_get_author (ValideProject* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_author;
	return result;
}


void valide_project_set_author (ValideProject* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 139 "project.vala"
	_tmp0_ = NULL;
#line 3080 "project.c"
	self->priv->_author = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_author = (g_free (self->priv->_author), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "author");
}


const char* valide_project_get_version (ValideProject* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_version;
	return result;
}


void valide_project_set_version (ValideProject* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 144 "project.vala"
	_tmp0_ = NULL;
#line 3101 "project.c"
	self->priv->_version = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_version = (g_free (self->priv->_version), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "version");
}


const char* valide_project_get_license (ValideProject* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_license;
	return result;
}


void valide_project_set_license (ValideProject* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 149 "project.vala"
	_tmp0_ = NULL;
#line 3122 "project.c"
	self->priv->_license = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_license = (g_free (self->priv->_license), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "license");
}


const char* valide_project_get_builder (ValideProject* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_builder;
	return result;
}


void valide_project_set_builder (ValideProject* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 154 "project.vala"
	_tmp0_ = NULL;
#line 3143 "project.c"
	self->priv->_builder = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_builder = (g_free (self->priv->_builder), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "builder");
}


ValideBuilderOptions* valide_project_get_builder_options (ValideProject* self) {
	ValideBuilderOptions* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_builder_options;
	return result;
}


void valide_project_set_builder_options (ValideProject* self, ValideBuilderOptions* value) {
	ValideBuilderOptions* _tmp1_;
	ValideBuilderOptions* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 159 "project.vala"
	_tmp0_ = NULL;
#line 3164 "project.c"
	self->priv->_builder_options = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_builder_options == NULL) ? NULL : (self->priv->_builder_options = (g_object_unref (self->priv->_builder_options), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "builder-options");
}


ValideExecutableOptions* valide_project_get_executable_options (ValideProject* self) {
	ValideExecutableOptions* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_executable_options;
	return result;
}


void valide_project_set_executable_options (ValideProject* self, ValideExecutableOptions* value) {
	ValideExecutableOptions* _tmp1_;
	ValideExecutableOptions* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 164 "project.vala"
	_tmp0_ = NULL;
#line 3185 "project.c"
	self->priv->_executable_options = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_executable_options == NULL) ? NULL : (self->priv->_executable_options = (g_object_unref (self->priv->_executable_options), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "executable-options");
}


static ValideDocumentManager* valide_project_get_documents (ValideProject* self) {
	ValideDocumentManager* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_documents;
	return result;
}


#line 296 "project.vala"
static void _valide_project_document_changed_valide_document_manager_tab_state_changed (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self) {
#line 3201 "project.c"
	valide_project_document_changed (self, _sender, document);
}


static void valide_project_set_documents (ValideProject* self, ValideDocumentManager* value) {
	ValideDocumentManager* _tmp2_;
	ValideDocumentManager* _tmp1_;
	g_return_if_fail (self != NULL);
#line 178 "project.vala"
	if (valide_project_get_documents (self) != NULL) {
#line 3212 "project.c"
		guint _tmp0_;
#line 180 "project.vala"
		g_signal_handlers_disconnect_matched (valide_project_get_documents (self), G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("tab-state-changed", VALIDE_TYPE_DOCUMENT_MANAGER, &_tmp0_, NULL, FALSE), _tmp0_), 0, NULL, (GCallback) _valide_project_document_changed_valide_document_manager_tab_state_changed, self);
#line 3216 "project.c"
	}
	_tmp2_ = NULL;
#line 182 "project.vala"
	_tmp1_ = NULL;
#line 182 "project.vala"
	self->priv->_documents = (_tmp2_ = (_tmp1_ = value, (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (self->priv->_documents == NULL) ? NULL : (self->priv->_documents = (g_object_unref (self->priv->_documents), NULL)), _tmp2_);
#line 183 "project.vala"
	g_signal_connect_object (valide_project_get_documents (self), "tab-state-changed", (GCallback) _valide_project_document_changed_valide_document_manager_tab_state_changed, self, 0);
#line 3225 "project.c"
}


static void _lambda80_ (ValideProject* s, ValideProject* self) {
	g_return_if_fail (s != NULL);
#line 346 "project.vala"
	valide_project_save (self);
#line 3233 "project.c"
}


static void __lambda80__valide_project_added_file (ValideProject* _sender, gpointer self) {
	_lambda80_ (_sender, self);
}


static void _lambda81_ (ValideProject* s, ValideProject* self) {
	g_return_if_fail (s != NULL);
#line 349 "project.vala"
	valide_project_save (self);
#line 3246 "project.c"
}


static void __lambda81__valide_project_removed_file (ValideProject* _sender, gpointer self) {
	_lambda81_ (_sender, self);
}


static GObject * valide_project_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValideProjectClass * klass;
	GObjectClass * parent_class;
	ValideProject * self;
	klass = VALIDE_PROJECT_CLASS (g_type_class_peek (VALIDE_TYPE_PROJECT));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_PROJECT (obj);
	{
		GList* _tmp1_;
		GList* _tmp2_;
		GList* _tmp3_;
		ValideExecutableOptions* _tmp4_;
#line 337 "project.vala"
		valide_project_set_name (self, "");
#line 338 "project.vala"
		valide_project_set_author (self, "");
#line 339 "project.vala"
		valide_project_set_license (self, "");
#line 340 "project.vala"
		valide_project_set_version (self, "");
#line 3277 "project.c"
		_tmp1_ = NULL;
#line 341 "project.vala"
		self->files = (_tmp1_ = NULL, (self->files == NULL) ? NULL : (self->files = (_g_list_free_g_object_unref (self->files), NULL)), _tmp1_);
#line 3281 "project.c"
		_tmp2_ = NULL;
#line 342 "project.vala"
		self->vapi_dir = (_tmp2_ = NULL, (self->vapi_dir == NULL) ? NULL : (self->vapi_dir = (_g_list_free_g_object_unref (self->vapi_dir), NULL)), _tmp2_);
#line 3285 "project.c"
		_tmp3_ = NULL;
#line 343 "project.vala"
		self->packages = (_tmp3_ = NULL, (self->packages == NULL) ? NULL : (self->packages = (_g_list_free_g_object_unref (self->packages), NULL)), _tmp3_);
#line 3289 "project.c"
		_tmp4_ = NULL;
#line 344 "project.vala"
		valide_project_set_executable_options (self, _tmp4_ = valide_executable_options_new ());
#line 3293 "project.c"
		(_tmp4_ == NULL) ? NULL : (_tmp4_ = (g_object_unref (_tmp4_), NULL));
#line 345 "project.vala"
		g_signal_connect (self, "added-file", (GCallback) __lambda80__valide_project_added_file, self);
#line 348 "project.vala"
		g_signal_connect (self, "removed-file", (GCallback) __lambda81__valide_project_removed_file, self);
#line 3299 "project.c"
	}
	return obj;
}


static void valide_project_class_init (ValideProjectClass * klass) {
	GType* _tmp0_;
	valide_project_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideProjectPrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_project_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_project_set_property;
	G_OBJECT_CLASS (klass)->constructor = valide_project_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_project_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PROJECT_PATH, g_param_spec_string ("path", "path", "path", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PROJECT_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PROJECT_AUTHOR, g_param_spec_string ("author", "author", "author", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PROJECT_VERSION, g_param_spec_string ("version", "version", "version", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PROJECT_LICENSE, g_param_spec_string ("license", "license", "license", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PROJECT_BUILDER, g_param_spec_string ("builder", "builder", "builder", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PROJECT_BUILDER_OPTIONS, g_param_spec_object ("builder-options", "builder-options", "builder-options", VALIDE_TYPE_BUILDER_OPTIONS, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_PROJECT_EXECUTABLE_OPTIONS, g_param_spec_object ("executable-options", "executable-options", "executable-options", VALIDE_TYPE_EXECUTABLE_OPTIONS, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_signal_new ("closed", VALIDE_TYPE_PROJECT, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("removed_file", VALIDE_TYPE_PROJECT, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("added_file", VALIDE_TYPE_PROJECT, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("options_changed", VALIDE_TYPE_PROJECT, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	valide_project_child_types = (_tmp0_ = g_new0 (GType, 3), _tmp0_[0] = VALIDE_TYPE_SOURCE, _tmp0_[1] = VALIDE_TYPE_PACKAGE, _tmp0_[2] = VALIDE_TYPE_VAPI_DIR, _tmp0_);
	valide_project_child_types_length1 = 3;
#line 86 "project.vala"
	_tmp0_ = NULL;
#line 3329 "project.c"
}


static void valide_project_g_yaml_buildable_interface_init (GYAMLBuildableIface * iface) {
	valide_project_g_yaml_buildable_parent_iface = g_type_interface_peek_parent (iface);
	iface->add_child = valide_project_real_add_child;
	iface->get_child_types = valide_project_real_get_child_types;
	iface->get_child_tags = valide_project_real_get_child_tags;
	iface->get_children = valide_project_real_get_children;
}


static void valide_project_instance_init (ValideProject * self) {
	self->priv = VALIDE_PROJECT_GET_PRIVATE (self);
}


static void valide_project_finalize (GObject* obj) {
	ValideProject * self;
	self = VALIDE_PROJECT (obj);
	self->filename = (g_free (self->filename), NULL);
	(self->builders == NULL) ? NULL : (self->builders = (g_object_unref (self->builders), NULL));
	(self->files == NULL) ? NULL : (self->files = (_g_list_free_g_object_unref (self->files), NULL));
	(self->packages == NULL) ? NULL : (self->packages = (_g_list_free_g_object_unref (self->packages), NULL));
	(self->vapi_dir == NULL) ? NULL : (self->vapi_dir = (_g_list_free_g_object_unref (self->vapi_dir), NULL));
	(self->priv->_documents == NULL) ? NULL : (self->priv->_documents = (g_object_unref (self->priv->_documents), NULL));
	self->priv->_path = (g_free (self->priv->_path), NULL);
	self->priv->_name = (g_free (self->priv->_name), NULL);
	self->priv->_author = (g_free (self->priv->_author), NULL);
	self->priv->_version = (g_free (self->priv->_version), NULL);
	self->priv->_license = (g_free (self->priv->_license), NULL);
	self->priv->_builder = (g_free (self->priv->_builder), NULL);
	(self->priv->_builder_options == NULL) ? NULL : (self->priv->_builder_options = (g_object_unref (self->priv->_builder_options), NULL));
	(self->priv->_executable_options == NULL) ? NULL : (self->priv->_executable_options = (g_object_unref (self->priv->_executable_options), NULL));
	G_OBJECT_CLASS (valide_project_parent_class)->finalize (obj);
}


GType valide_project_get_type (void) {
	static GType valide_project_type_id = 0;
	if (valide_project_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideProjectClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_project_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideProject), 0, (GInstanceInitFunc) valide_project_instance_init, NULL };
		static const GInterfaceInfo g_yaml_buildable_info = { (GInterfaceInitFunc) valide_project_g_yaml_buildable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		valide_project_type_id = g_type_register_static (G_TYPE_OBJECT, "ValideProject", &g_define_type_info, 0);
		g_type_add_interface_static (valide_project_type_id, G_YAML_TYPE_BUILDABLE, &g_yaml_buildable_info);
	}
	return valide_project_type_id;
}


static void valide_project_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideProject * self;
	gpointer boxed;
	self = VALIDE_PROJECT (object);
	switch (property_id) {
		case VALIDE_PROJECT_PATH:
		g_value_set_string (value, valide_project_get_path (self));
		break;
		case VALIDE_PROJECT_NAME:
		g_value_set_string (value, valide_project_get_name (self));
		break;
		case VALIDE_PROJECT_AUTHOR:
		g_value_set_string (value, valide_project_get_author (self));
		break;
		case VALIDE_PROJECT_VERSION:
		g_value_set_string (value, valide_project_get_version (self));
		break;
		case VALIDE_PROJECT_LICENSE:
		g_value_set_string (value, valide_project_get_license (self));
		break;
		case VALIDE_PROJECT_BUILDER:
		g_value_set_string (value, valide_project_get_builder (self));
		break;
		case VALIDE_PROJECT_BUILDER_OPTIONS:
		g_value_set_object (value, valide_project_get_builder_options (self));
		break;
		case VALIDE_PROJECT_EXECUTABLE_OPTIONS:
		g_value_set_object (value, valide_project_get_executable_options (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_project_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideProject * self;
	self = VALIDE_PROJECT (object);
	switch (property_id) {
		case VALIDE_PROJECT_NAME:
		valide_project_set_name (self, g_value_get_string (value));
		break;
		case VALIDE_PROJECT_AUTHOR:
		valide_project_set_author (self, g_value_get_string (value));
		break;
		case VALIDE_PROJECT_VERSION:
		valide_project_set_version (self, g_value_get_string (value));
		break;
		case VALIDE_PROJECT_LICENSE:
		valide_project_set_license (self, g_value_get_string (value));
		break;
		case VALIDE_PROJECT_BUILDER:
		valide_project_set_builder (self, g_value_get_string (value));
		break;
		case VALIDE_PROJECT_BUILDER_OPTIONS:
		valide_project_set_builder_options (self, g_value_get_object (value));
		break;
		case VALIDE_PROJECT_EXECUTABLE_OPTIONS:
		valide_project_set_executable_options (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




