/* searching.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <glib/gi18n-lib.h>
#include <gtksourceview/gtksourceview.h>
#include <gtksourceview/gtksourceiter.h>
#include <string.h>
#include <stdlib.h>
#include <gdk/gdkkeysyms.h>


#define VALIDE_TYPE_OPTIONS_WINDOW (valide_options_window_get_type ())
#define VALIDE_OPTIONS_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_OPTIONS_WINDOW, ValideOptionsWindow))
#define VALIDE_OPTIONS_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_OPTIONS_WINDOW, ValideOptionsWindowClass))
#define VALIDE_IS_OPTIONS_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_OPTIONS_WINDOW))
#define VALIDE_IS_OPTIONS_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_OPTIONS_WINDOW))
#define VALIDE_OPTIONS_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_OPTIONS_WINDOW, ValideOptionsWindowClass))

typedef struct _ValideOptionsWindow ValideOptionsWindow;
typedef struct _ValideOptionsWindowClass ValideOptionsWindowClass;
typedef struct _ValideOptionsWindowPrivate ValideOptionsWindowPrivate;

#define VALIDE_TYPE_SEARCHING (valide_searching_get_type ())
#define VALIDE_SEARCHING(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_SEARCHING, ValideSearching))
#define VALIDE_SEARCHING_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_SEARCHING, ValideSearchingClass))
#define VALIDE_IS_SEARCHING(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_SEARCHING))
#define VALIDE_IS_SEARCHING_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_SEARCHING))
#define VALIDE_SEARCHING_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_SEARCHING, ValideSearchingClass))

typedef struct _ValideSearching ValideSearching;
typedef struct _ValideSearchingClass ValideSearchingClass;
typedef struct _ValideSearchingPrivate ValideSearchingPrivate;

#define VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT (valide_abstract_text_document_get_type ())
#define VALIDE_ABSTRACT_TEXT_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocument))
#define VALIDE_ABSTRACT_TEXT_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocumentClass))
#define VALIDE_IS_ABSTRACT_TEXT_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT))
#define VALIDE_IS_ABSTRACT_TEXT_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT))
#define VALIDE_ABSTRACT_TEXT_DOCUMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, ValideAbstractTextDocumentClass))

typedef struct _ValideAbstractTextDocument ValideAbstractTextDocument;
typedef struct _ValideAbstractTextDocumentClass ValideAbstractTextDocumentClass;

#define VALIDE_TYPE_SOURCE_BUFFER (valide_source_buffer_get_type ())
#define VALIDE_SOURCE_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_SOURCE_BUFFER, ValideSourceBuffer))
#define VALIDE_SOURCE_BUFFER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_SOURCE_BUFFER, ValideSourceBufferClass))
#define VALIDE_IS_SOURCE_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_SOURCE_BUFFER))
#define VALIDE_IS_SOURCE_BUFFER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_SOURCE_BUFFER))
#define VALIDE_SOURCE_BUFFER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_SOURCE_BUFFER, ValideSourceBufferClass))

typedef struct _ValideSourceBuffer ValideSourceBuffer;
typedef struct _ValideSourceBufferClass ValideSourceBufferClass;

#define VALIDE_TYPE_SOURCE_VIEW (valide_source_view_get_type ())
#define VALIDE_SOURCE_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_SOURCE_VIEW, ValideSourceView))
#define VALIDE_SOURCE_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_SOURCE_VIEW, ValideSourceViewClass))
#define VALIDE_IS_SOURCE_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_SOURCE_VIEW))
#define VALIDE_IS_SOURCE_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_SOURCE_VIEW))
#define VALIDE_SOURCE_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_SOURCE_VIEW, ValideSourceViewClass))

typedef struct _ValideSourceView ValideSourceView;
typedef struct _ValideSourceViewClass ValideSourceViewClass;

struct _ValideOptionsWindow {
	GtkWindow parent_instance;
	ValideOptionsWindowPrivate * priv;
	GtkCheckButton* check_casse;
	GtkCheckButton* check_word;
};

struct _ValideOptionsWindowClass {
	GtkWindowClass parent_class;
};

/**
 * A widget for search/replace in a Valide.AbstractTextDocument
 */
struct _ValideSearching {
	GtkVBox parent_instance;
	ValideSearchingPrivate * priv;
};

struct _ValideSearchingClass {
	GtkVBoxClass parent_class;
};

struct _ValideSearchingPrivate {
	gboolean reverse;
	GtkHBox* hbox_search;
	GtkHBox* hbox_replace;
	GtkEntry* entry_search;
	GtkEntry* entry_replace;
	GtkLabel* message_label;
	ValideOptionsWindow* options_window;
	GtkToggleButton* toggle_button;
	ValideAbstractTextDocument* _document;
};


static gpointer valide_options_window_parent_class = NULL;
static gpointer valide_searching_parent_class = NULL;

GType valide_options_window_get_type (void);
enum  {
	VALIDE_OPTIONS_WINDOW_DUMMY_PROPERTY
};
static gboolean valide_options_window_delete_event_cb (ValideOptionsWindow* self, GtkWidget* widget, GdkEvent* event);
static gboolean valide_options_window_focus_out_event_cb (ValideOptionsWindow* self, GtkWidget* sender, const GdkEventFocus* event);
ValideOptionsWindow* valide_options_window_new (void);
ValideOptionsWindow* valide_options_window_construct (GType object_type);
static gboolean _valide_options_window_delete_event_cb_gtk_widget_delete_event (GtkWidget* _sender, GdkEvent* event, gpointer self);
static gboolean _valide_options_window_focus_out_event_cb_gtk_widget_focus_out_event (GtkWidget* _sender, const GdkEventFocus* event, gpointer self);
static GObject * valide_options_window_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void valide_options_window_finalize (GObject* obj);
GType valide_searching_get_type (void);
GType valide_abstract_text_document_get_type (void);
#define VALIDE_SEARCHING_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_SEARCHING, ValideSearchingPrivate))
enum  {
	VALIDE_SEARCHING_DUMMY_PROPERTY,
	VALIDE_SEARCHING_DOCUMENT,
	VALIDE_SEARCHING_SEARCH_TEXT,
	VALIDE_SEARCHING_REPLACE_TEXT,
	VALIDE_SEARCHING_CASSE,
	VALIDE_SEARCHING_WORD
};
ValideAbstractTextDocument* valide_searching_get_document (ValideSearching* self);
GType valide_source_buffer_get_type (void);
ValideSourceBuffer* valide_abstract_text_document_get_buffer (ValideAbstractTextDocument* self);
gboolean valide_searching_get_casse (ValideSearching* self);
const char* valide_searching_get_search_text (ValideSearching* self);
gboolean valide_searching_get_word (ValideSearching* self);
GType valide_source_view_get_type (void);
ValideSourceView* valide_abstract_text_document_get_text_view (ValideAbstractTextDocument* self);
static gboolean valide_searching_search (ValideSearching* self);
void valide_abstract_text_document_grab_focus (ValideAbstractTextDocument* self);
gboolean valide_searching_find_prev (ValideSearching* self);
gboolean valide_searching_replace_prev (ValideSearching* self);
gboolean valide_searching_find_next (ValideSearching* self);
gboolean valide_searching_replace_next (ValideSearching* self);
static gboolean valide_searching_key_press_event_cb (ValideSearching* self, GtkWidget* sender, const GdkEventKey* event);
static void valide_searching_show_options (ValideSearching* self);
static GtkButton* valide_searching_create_button (ValideSearching* self, const char* stock_id, const char* tooltip);
static gboolean _lambda15_ (ValideSearching* self);
static gboolean __lambda15__gsource_func (gpointer self);
static void valide_searching_send_message (ValideSearching* self, const char* message);
static void valide_searching_set_document (ValideSearching* self, ValideAbstractTextDocument* value);
ValideSearching* valide_searching_new (ValideAbstractTextDocument* document);
ValideSearching* valide_searching_construct (GType object_type, ValideAbstractTextDocument* document);
const char* valide_abstract_text_document_get_selected_text (ValideAbstractTextDocument* self);
void valide_searching_set_search_text (ValideSearching* self, const char* value);
void valide_searching_show_search (ValideSearching* self);
void valide_searching_show_replace (ValideSearching* self);
gboolean valide_searching_find (ValideSearching* self, gboolean show_replace, gboolean show_dialog);
void valide_searching_replace (ValideSearching* self, gboolean show_dialog);
const char* valide_searching_get_replace_text (ValideSearching* self);
void valide_searching_replace_all (ValideSearching* self);
void valide_searching_set_replace_text (ValideSearching* self, const char* value);
void valide_searching_set_casse (ValideSearching* self, gboolean value);
void valide_searching_set_word (ValideSearching* self, gboolean value);
static void _lambda16_ (ValideSearching* self);
static void __lambda16__gtk_button_clicked (GtkButton* _sender, gpointer self);
static void _lambda17_ (ValideSearching* self);
static void __lambda17__gtk_editable_changed (GtkEditable* _sender, gpointer self);
static gboolean _valide_searching_key_press_event_cb_gtk_widget_key_press_event (GtkWidget* _sender, const GdkEventKey* event, gpointer self);
static void _lambda18_ (ValideSearching* self);
static void __lambda18__gtk_widget_hide (GtkWidget* _sender, gpointer self);
static void _lambda19_ (GtkToggleButton* s, ValideSearching* self);
static void __lambda19__gtk_toggle_button_toggled (GtkToggleButton* _sender, gpointer self);
static void _lambda20_ (ValideSearching* self);
static void __lambda20__gtk_button_clicked (GtkButton* _sender, gpointer self);
static void _lambda21_ (ValideSearching* self);
static void __lambda21__gtk_button_clicked (GtkButton* _sender, gpointer self);
static void _lambda22_ (ValideSearching* self);
static void __lambda22__gtk_button_clicked (GtkButton* _sender, gpointer self);
static void _lambda23_ (ValideSearching* self);
static void __lambda23__gtk_button_clicked (GtkButton* _sender, gpointer self);
static void _lambda24_ (ValideSearching* self);
static void __lambda24__gtk_button_clicked (GtkButton* _sender, gpointer self);
static GObject * valide_searching_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void valide_searching_finalize (GObject* obj);
static void valide_searching_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_searching_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static int _vala_strcmp0 (const char * str1, const char * str2);



#line 27 "searching.vala"
static gboolean valide_options_window_delete_event_cb (ValideOptionsWindow* self, GtkWidget* widget, GdkEvent* event) {
#line 212 "searching.c"
	gboolean result;
#line 27 "searching.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 27 "searching.vala"
	g_return_val_if_fail (widget != NULL, FALSE);
#line 27 "searching.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 220 "searching.c"
	/* Prevent the alt+F4 keys*/
	result = TRUE;
	return result;
}


#line 33 "searching.vala"
static gboolean valide_options_window_focus_out_event_cb (ValideOptionsWindow* self, GtkWidget* sender, const GdkEventFocus* event) {
#line 229 "searching.c"
	gboolean result;
#line 33 "searching.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 33 "searching.vala"
	g_return_val_if_fail (sender != NULL, FALSE);
#line 35 "searching.vala"
	gtk_widget_hide ((GtkWidget*) self);
#line 237 "searching.c"
	result = FALSE;
	return result;
}


#line 22 "searching.vala"
ValideOptionsWindow* valide_options_window_construct (GType object_type) {
#line 245 "searching.c"
	ValideOptionsWindow * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


#line 22 "searching.vala"
ValideOptionsWindow* valide_options_window_new (void) {
#line 22 "searching.vala"
	return valide_options_window_construct (VALIDE_TYPE_OPTIONS_WINDOW);
#line 256 "searching.c"
}


#line 27 "searching.vala"
static gboolean _valide_options_window_delete_event_cb_gtk_widget_delete_event (GtkWidget* _sender, GdkEvent* event, gpointer self) {
#line 262 "searching.c"
	return valide_options_window_delete_event_cb (self, _sender, event);
}


#line 33 "searching.vala"
static gboolean _valide_options_window_focus_out_event_cb_gtk_widget_focus_out_event (GtkWidget* _sender, const GdkEventFocus* event, gpointer self) {
#line 269 "searching.c"
	return valide_options_window_focus_out_event_cb (self, _sender, event);
}


static GObject * valide_options_window_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValideOptionsWindowClass * klass;
	GObjectClass * parent_class;
	ValideOptionsWindow * self;
	klass = VALIDE_OPTIONS_WINDOW_CLASS (g_type_class_peek (VALIDE_TYPE_OPTIONS_WINDOW));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_OPTIONS_WINDOW (obj);
	{
		GtkVBox* vbox;
		GtkVBox* _tmp0_;
		GtkCheckButton* _tmp1_;
		GtkCheckButton* _tmp2_;
		vbox = NULL;
#line 43 "searching.vala"
		gtk_container_set_border_width ((GtkContainer*) self, (guint) 5);
#line 44 "searching.vala"
		gtk_window_set_decorated ((GtkWindow*) self, FALSE);
#line 45 "searching.vala"
		gtk_window_set_gravity ((GtkWindow*) self, GDK_GRAVITY_SOUTH_WEST);
#line 46 "searching.vala"
		gtk_window_set_position ((GtkWindow*) self, GTK_WIN_POS_MOUSE);
#line 47 "searching.vala"
		gtk_window_set_skip_pager_hint ((GtkWindow*) self, TRUE);
#line 48 "searching.vala"
		gtk_window_set_skip_taskbar_hint ((GtkWindow*) self, TRUE);
#line 301 "searching.c"
		_tmp0_ = NULL;
#line 50 "searching.vala"
		vbox = (_tmp0_ = g_object_ref_sink ((GtkVBox*) gtk_vbox_new (FALSE, 0)), (vbox == NULL) ? NULL : (vbox = (g_object_unref (vbox), NULL)), _tmp0_);
#line 51 "searching.vala"
		gtk_container_add ((GtkContainer*) self, (GtkWidget*) vbox);
#line 307 "searching.c"
		_tmp1_ = NULL;
#line 53 "searching.vala"
		self->check_casse = (_tmp1_ = g_object_ref_sink ((GtkCheckButton*) gtk_check_button_new_with_label (_ ("Match case"))), (self->check_casse == NULL) ? NULL : (self->check_casse = (g_object_unref (self->check_casse), NULL)), _tmp1_);
#line 54 "searching.vala"
		gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) self->check_casse, FALSE, FALSE, (guint) 0);
#line 313 "searching.c"
		_tmp2_ = NULL;
#line 56 "searching.vala"
		self->check_word = (_tmp2_ = g_object_ref_sink ((GtkCheckButton*) gtk_check_button_new_with_label (_ ("Match entire word only"))), (self->check_word == NULL) ? NULL : (self->check_word = (g_object_unref (self->check_word), NULL)), _tmp2_);
#line 57 "searching.vala"
		gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) self->check_word, FALSE, FALSE, (guint) 0);
#line 319 "searching.c"
		/* Connect signals*/
#line 60 "searching.vala"
		g_signal_connect_object ((GtkWidget*) self, "delete-event", (GCallback) _valide_options_window_delete_event_cb_gtk_widget_delete_event, self, 0);
#line 61 "searching.vala"
		g_signal_connect_object ((GtkWidget*) self, "focus-out-event", (GCallback) _valide_options_window_focus_out_event_cb_gtk_widget_focus_out_event, self, 0);
#line 325 "searching.c"
		(vbox == NULL) ? NULL : (vbox = (g_object_unref (vbox), NULL));
	}
	return obj;
}


static void valide_options_window_class_init (ValideOptionsWindowClass * klass) {
	valide_options_window_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->constructor = valide_options_window_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_options_window_finalize;
}


static void valide_options_window_instance_init (ValideOptionsWindow * self) {
}


static void valide_options_window_finalize (GObject* obj) {
	ValideOptionsWindow * self;
	self = VALIDE_OPTIONS_WINDOW (obj);
	(self->check_casse == NULL) ? NULL : (self->check_casse = (g_object_unref (self->check_casse), NULL));
	(self->check_word == NULL) ? NULL : (self->check_word = (g_object_unref (self->check_word), NULL));
	G_OBJECT_CLASS (valide_options_window_parent_class)->finalize (obj);
}


GType valide_options_window_get_type (void) {
	static GType valide_options_window_type_id = 0;
	if (valide_options_window_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideOptionsWindowClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_options_window_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideOptionsWindow), 0, (GInstanceInitFunc) valide_options_window_instance_init, NULL };
		valide_options_window_type_id = g_type_register_static (GTK_TYPE_WINDOW, "ValideOptionsWindow", &g_define_type_info, 0);
	}
	return valide_options_window_type_id;
}


#line 124 "searching.vala"
static gboolean valide_searching_search (ValideSearching* self) {
#line 364 "searching.c"
	gboolean result;
	gint count;
	GdkColor color = {0};
	GtkTextIter end = {0};
	GtkTextIter iter = {0};
	gboolean found;
	GtkTextIter start = {0};
	GtkSourceBuffer* buffer;
	GtkSourceSearchFlags flags;
	GtkSourceBuffer* _tmp1_;
	GtkSourceBuffer* _tmp0_;
	GtkTextIter _tmp2_ = {0};
	GtkTextIter _tmp3_ = {0};
#line 124 "searching.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 380 "searching.c"
	count = 0;
	found = FALSE;
	buffer = NULL;
	flags = 0;
#line 135 "searching.vala"
	flags = GTK_SOURCE_SEARCH_TEXT_ONLY;
#line 387 "searching.c"
	_tmp1_ = NULL;
#line 136 "searching.vala"
	_tmp0_ = NULL;
#line 136 "searching.vala"
	buffer = (_tmp1_ = (_tmp0_ = (GtkSourceBuffer*) valide_abstract_text_document_get_buffer (self->priv->_document), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (buffer == NULL) ? NULL : (buffer = (g_object_unref (buffer), NULL)), _tmp1_);
#line 137 "searching.vala"
	gtk_text_buffer_get_selection_bounds ((GtkTextBuffer*) buffer, NULL, &iter);
#line 139 "searching.vala"
	if (!valide_searching_get_casse (self)) {
#line 141 "searching.vala"
		flags = flags | GTK_SOURCE_SEARCH_CASE_INSENSITIVE;
#line 399 "searching.c"
	}
#line 144 "searching.vala"
	start = (memset (&_tmp2_, 0, sizeof (GtkTextIter)), _tmp2_);
#line 145 "searching.vala"
	end = (memset (&_tmp3_, 0, sizeof (GtkTextIter)), _tmp3_);
#line 147 "searching.vala"
	while (TRUE) {
#line 407 "searching.c"
		gboolean _tmp4_;
#line 147 "searching.vala"
		if (!(count < 2)) {
#line 147 "searching.vala"
			break;
#line 413 "searching.c"
		}
#line 149 "searching.vala"
		if (!self->priv->reverse) {
#line 151 "searching.vala"
			found = gtk_source_iter_forward_search (&iter, valide_searching_get_search_text (self), flags, &start, &end, NULL);
#line 419 "searching.c"
		} else {
#line 156 "searching.vala"
			if (count == 0) {
#line 158 "searching.vala"
				gtk_text_iter_backward_word_start (&iter);
#line 425 "searching.c"
			}
#line 160 "searching.vala"
			found = gtk_source_iter_backward_search (&iter, valide_searching_get_search_text (self), flags, &start, &end, NULL);
#line 429 "searching.c"
		}
		_tmp4_ = FALSE;
#line 163 "searching.vala"
		if (found) {
#line 163 "searching.vala"
			_tmp4_ = valide_searching_get_word (self);
#line 436 "searching.c"
		} else {
#line 163 "searching.vala"
			_tmp4_ = FALSE;
#line 440 "searching.c"
		}
#line 163 "searching.vala"
		if (_tmp4_) {
#line 444 "searching.c"
			gboolean _tmp5_;
			_tmp5_ = FALSE;
#line 165 "searching.vala"
			if (gtk_text_iter_starts_word (&start)) {
#line 165 "searching.vala"
				_tmp5_ = gtk_text_iter_ends_word (&end);
#line 451 "searching.c"
			} else {
#line 165 "searching.vala"
				_tmp5_ = FALSE;
#line 455 "searching.c"
			}
#line 165 "searching.vala"
			found = _tmp5_;
#line 459 "searching.c"
		}
#line 167 "searching.vala"
		if (!found) {
#line 169 "searching.vala"
			if (!self->priv->reverse) {
#line 171 "searching.vala"
				gtk_text_buffer_get_start_iter ((GtkTextBuffer*) buffer, &iter);
#line 467 "searching.c"
			} else {
#line 175 "searching.vala"
				gtk_text_buffer_get_end_iter ((GtkTextBuffer*) buffer, &iter);
#line 471 "searching.c"
			}
#line 177 "searching.vala"
			count++;
#line 475 "searching.c"
		} else {
#line 181 "searching.vala"
			break;
#line 479 "searching.c"
		}
	}
#line 185 "searching.vala"
	if (found) {
#line 187 "searching.vala"
		gtk_text_buffer_place_cursor ((GtkTextBuffer*) buffer, &start);
#line 188 "searching.vala"
		gtk_text_buffer_move_mark_by_name ((GtkTextBuffer*) buffer, "selection_bound", &end);
#line 189 "searching.vala"
		gtk_text_view_scroll_to_mark ((GtkTextView*) valide_abstract_text_document_get_text_view (self->priv->_document), gtk_text_buffer_get_insert ((GtkTextBuffer*) buffer), 0.25, FALSE, 0.0, 0.0);
#line 191 "searching.vala"
		gdk_color_parse ("white", &color);
#line 492 "searching.c"
	} else {
#line 195 "searching.vala"
		gtk_text_buffer_place_cursor ((GtkTextBuffer*) buffer, &iter);
#line 196 "searching.vala"
		gdk_color_parse ("#FFBFBF", &color);
#line 498 "searching.c"
	}
#line 198 "searching.vala"
	gtk_widget_modify_base ((GtkWidget*) self->priv->entry_search, GTK_STATE_NORMAL, &color);
#line 502 "searching.c"
	result = found;
	(buffer == NULL) ? NULL : (buffer = (g_object_unref (buffer), NULL));
	return result;
}


#line 202 "searching.vala"
static gboolean valide_searching_key_press_event_cb (ValideSearching* self, GtkWidget* sender, const GdkEventKey* event) {
#line 511 "searching.c"
	gboolean result;
	gboolean catched;
#line 202 "searching.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 202 "searching.vala"
	g_return_val_if_fail (sender != NULL, FALSE);
#line 518 "searching.c"
	catched = FALSE;
#line 206 "searching.vala"
	if ((*event).keyval == GDK_Escape) {
#line 208 "searching.vala"
		gtk_widget_hide ((GtkWidget*) self);
#line 209 "searching.vala"
		valide_abstract_text_document_grab_focus (self->priv->_document);
#line 210 "searching.vala"
		catched = TRUE;
#line 528 "searching.c"
	} else {
#line 212 "searching.vala"
		if ((*event).keyval == GDK_Return) {
#line 214 "searching.vala"
			if (((*event).state & GDK_SHIFT_MASK) == GDK_SHIFT_MASK) {
#line 216 "searching.vala"
				if (sender == GTK_WIDGET (self->priv->entry_search)) {
#line 218 "searching.vala"
					valide_searching_find_prev (self);
#line 538 "searching.c"
				} else {
#line 222 "searching.vala"
					valide_searching_replace_prev (self);
#line 542 "searching.c"
				}
			} else {
#line 227 "searching.vala"
				if (sender == GTK_WIDGET (self->priv->entry_search)) {
#line 229 "searching.vala"
					valide_searching_find_next (self);
#line 549 "searching.c"
				} else {
#line 233 "searching.vala"
					valide_searching_replace_next (self);
#line 553 "searching.c"
				}
			}
		}
	}
	result = catched;
	return result;
}


#line 240 "searching.vala"
static void valide_searching_show_options (ValideSearching* self) {
#line 565 "searching.c"
	gint x;
	gint y;
	gint wx;
	gint wy;
	GtkWindow* parent;
	GdkDisplay* display;
	GtkWindow* _tmp2_;
	GtkWindow* _tmp1_;
	GtkWidget* _tmp0_;
	GdkDisplay* _tmp4_;
	GdkDisplay* _tmp3_;
#line 240 "searching.vala"
	g_return_if_fail (self != NULL);
#line 579 "searching.c"
	x = 0;
	y = 0;
	wx = 0;
	wy = 0;
	parent = NULL;
	display = NULL;
	_tmp2_ = NULL;
#line 247 "searching.vala"
	_tmp1_ = NULL;
#line 247 "searching.vala"
	_tmp0_ = NULL;
#line 247 "searching.vala"
	parent = (_tmp2_ = (_tmp1_ = (_tmp0_ = gtk_widget_get_toplevel ((GtkWidget*) self), GTK_IS_WINDOW (_tmp0_) ? ((GtkWindow*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (parent == NULL) ? NULL : (parent = (g_object_unref (parent), NULL)), _tmp2_);
#line 248 "searching.vala"
	gtk_window_set_transient_for ((GtkWindow*) self->priv->options_window, parent);
#line 595 "searching.c"
	_tmp4_ = NULL;
#line 250 "searching.vala"
	_tmp3_ = NULL;
#line 250 "searching.vala"
	display = (_tmp4_ = (_tmp3_ = gdk_display_get_default (), (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), (display == NULL) ? NULL : (display = (g_object_unref (display), NULL)), _tmp4_);
#line 251 "searching.vala"
	gdk_display_get_pointer (display, NULL, &x, &y, NULL);
#line 252 "searching.vala"
	gtk_widget_show_all ((GtkWidget*) self->priv->options_window);
#line 253 "searching.vala"
	gtk_window_get_size ((GtkWindow*) self->priv->options_window, &wx, &wy);
#line 254 "searching.vala"
	y = y - wy;
#line 255 "searching.vala"
	gtk_window_move ((GtkWindow*) self->priv->options_window, x, y);
#line 611 "searching.c"
	(parent == NULL) ? NULL : (parent = (g_object_unref (parent), NULL));
	(display == NULL) ? NULL : (display = (g_object_unref (display), NULL));
}


#line 258 "searching.vala"
static GtkButton* valide_searching_create_button (ValideSearching* self, const char* stock_id, const char* tooltip) {
#line 619 "searching.c"
	GtkButton* result;
	GtkButton* button;
	GtkImage* image;
	GtkImage* _tmp0_;
	GtkButton* _tmp1_;
#line 258 "searching.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 258 "searching.vala"
	g_return_val_if_fail (stock_id != NULL, NULL);
#line 258 "searching.vala"
	g_return_val_if_fail (tooltip != NULL, NULL);
#line 631 "searching.c"
	button = NULL;
	image = NULL;
	_tmp0_ = NULL;
#line 263 "searching.vala"
	image = (_tmp0_ = g_object_ref_sink ((GtkImage*) gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_BUTTON)), (image == NULL) ? NULL : (image = (g_object_unref (image), NULL)), _tmp0_);
#line 637 "searching.c"
	_tmp1_ = NULL;
#line 264 "searching.vala"
	button = (_tmp1_ = g_object_ref_sink ((GtkButton*) gtk_button_new ()), (button == NULL) ? NULL : (button = (g_object_unref (button), NULL)), _tmp1_);
#line 265 "searching.vala"
	gtk_container_add ((GtkContainer*) button, (GtkWidget*) image);
#line 266 "searching.vala"
	gtk_button_set_relief (button, GTK_RELIEF_NONE);
#line 267 "searching.vala"
	gtk_widget_set_tooltip_text ((GtkWidget*) button, tooltip);
#line 268 "searching.vala"
	g_object_set ((GtkWidget*) button, "can-focus", FALSE, NULL);
#line 649 "searching.c"
	result = button;
	(image == NULL) ? NULL : (image = (g_object_unref (image), NULL));
	return result;
}


static gboolean _lambda15_ (ValideSearching* self) {
	gboolean result;
#line 277 "searching.vala"
	gtk_widget_hide ((GtkWidget*) self->priv->message_label);
#line 660 "searching.c"
	result = FALSE;
	return result;
}


static gboolean __lambda15__gsource_func (gpointer self) {
	return _lambda15_ (self);
}


#line 272 "searching.vala"
static void valide_searching_send_message (ValideSearching* self, const char* message) {
#line 272 "searching.vala"
	g_return_if_fail (self != NULL);
#line 272 "searching.vala"
	g_return_if_fail (message != NULL);
#line 274 "searching.vala"
	gtk_label_set_text (self->priv->message_label, message);
#line 275 "searching.vala"
	gtk_widget_show ((GtkWidget*) self->priv->message_label);
#line 276 "searching.vala"
	g_timeout_add ((guint) (1000 * 5), __lambda15__gsource_func, self);
#line 683 "searching.c"
}


/**
   * Create a new Valide.Searching
   *
   * @param document The attached document
   */
#line 287 "searching.vala"
ValideSearching* valide_searching_construct (GType object_type, ValideAbstractTextDocument* document) {
#line 694 "searching.c"
	GParameter * __params;
	GParameter * __params_it;
	ValideSearching * self;
#line 287 "searching.vala"
	g_return_val_if_fail (document != NULL, NULL);
#line 700 "searching.c"
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
#line 289 "searching.vala"
	__params_it->name = "document";
#line 289 "searching.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT);
#line 289 "searching.vala"
	g_value_set_object (&__params_it->value, document);
#line 289 "searching.vala"
	__params_it++;
#line 711 "searching.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 287 "searching.vala"
ValideSearching* valide_searching_new (ValideAbstractTextDocument* document) {
#line 287 "searching.vala"
	return valide_searching_construct (VALIDE_TYPE_SEARCHING, document);
#line 726 "searching.c"
}


/**
   * Find the next world
   *
   * @param show_replace Show the replace dialog
   * @param show_dialog Show the dialog
   *
   * @return true if the world is found
   */
#line 400 "searching.vala"
gboolean valide_searching_find (ValideSearching* self, gboolean show_replace, gboolean show_dialog) {
#line 740 "searching.c"
	gboolean result;
	char* t;
	gboolean found;
	char* _tmp1_;
	const char* _tmp0_;
	gboolean _tmp2_;
#line 400 "searching.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 749 "searching.c"
	t = NULL;
	found = FALSE;
	_tmp1_ = NULL;
#line 405 "searching.vala"
	_tmp0_ = NULL;
#line 405 "searching.vala"
	t = (_tmp1_ = (_tmp0_ = valide_abstract_text_document_get_selected_text (self->priv->_document), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), t = (g_free (t), NULL), _tmp1_);
#line 757 "searching.c"
	_tmp2_ = FALSE;
#line 406 "searching.vala"
	if (show_dialog) {
#line 406 "searching.vala"
		_tmp2_ = TRUE;
#line 763 "searching.c"
	} else {
		gboolean _tmp3_;
		_tmp3_ = FALSE;
#line 406 "searching.vala"
		if (_vala_strcmp0 (t, "") == 0) {
#line 406 "searching.vala"
			_tmp3_ = _vala_strcmp0 (valide_searching_get_search_text (self), "") == 0;
#line 771 "searching.c"
		} else {
#line 406 "searching.vala"
			_tmp3_ = FALSE;
#line 775 "searching.c"
		}
#line 406 "searching.vala"
		_tmp2_ = _tmp3_;
#line 779 "searching.c"
	}
#line 406 "searching.vala"
	if (_tmp2_) {
#line 408 "searching.vala"
		if (_vala_strcmp0 (t, "") != 0) {
#line 410 "searching.vala"
			valide_searching_set_search_text (self, t);
#line 787 "searching.c"
		}
#line 413 "searching.vala"
		valide_searching_show_search (self);
#line 414 "searching.vala"
		if (show_replace) {
#line 416 "searching.vala"
			valide_searching_show_replace (self);
#line 795 "searching.c"
		}
	} else {
#line 421 "searching.vala"
		found = valide_searching_search (self);
#line 800 "searching.c"
	}
	result = found;
	t = (g_free (t), NULL);
	return result;
}


/**
   * Find the next world
   *
   * @return true if the world is found
   */
#line 431 "searching.vala"
gboolean valide_searching_find_next (ValideSearching* self) {
#line 815 "searching.c"
	gboolean result;
#line 431 "searching.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 433 "searching.vala"
	self->priv->reverse = FALSE;
#line 821 "searching.c"
	result = valide_searching_find (self, FALSE, FALSE);
	return result;
}


/**
   * Find the previous world
   *
   * @return true if the world is found
   */
#line 442 "searching.vala"
gboolean valide_searching_find_prev (ValideSearching* self) {
#line 834 "searching.c"
	gboolean result;
#line 442 "searching.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 444 "searching.vala"
	self->priv->reverse = TRUE;
#line 840 "searching.c"
	result = valide_searching_find (self, FALSE, FALSE);
	return result;
}


/**
   * Replace the next world
   *
   * @param show_dialog Show the search dialog
   */
#line 453 "searching.vala"
void valide_searching_replace (ValideSearching* self, gboolean show_dialog) {
#line 453 "searching.vala"
	g_return_if_fail (self != NULL);
#line 455 "searching.vala"
	valide_searching_find (self, TRUE, show_dialog);
#line 857 "searching.c"
}


/**
   * Replace the next word
   *
   * @return true if the world is found
   */
#line 463 "searching.vala"
gboolean valide_searching_replace_next (ValideSearching* self) {
#line 868 "searching.c"
	gboolean result;
	char* text;
	GtkTextIter start = {0};
	GtkTextIter end = {0};
	GtkTextBuffer* buffer;
	GtkTextBuffer* _tmp1_;
	GtkTextBuffer* _tmp0_;
	gboolean _tmp4_;
#line 463 "searching.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 879 "searching.c"
	text = g_strdup ("");
	buffer = NULL;
	_tmp1_ = NULL;
#line 470 "searching.vala"
	_tmp0_ = NULL;
#line 470 "searching.vala"
	buffer = (_tmp1_ = (_tmp0_ = (GtkTextBuffer*) valide_abstract_text_document_get_buffer (self->priv->_document), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (buffer == NULL) ? NULL : (buffer = (g_object_unref (buffer), NULL)), _tmp1_);
#line 471 "searching.vala"
	if (gtk_text_buffer_get_selection_bounds (buffer, &start, &end)) {
#line 889 "searching.c"
		char* _tmp3_;
		const char* _tmp2_;
		_tmp3_ = NULL;
#line 473 "searching.vala"
		_tmp2_ = NULL;
#line 473 "searching.vala"
		text = (_tmp3_ = (_tmp2_ = gtk_text_buffer_get_text (buffer, &start, &end, FALSE), (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)), text = (g_free (text), NULL), _tmp3_);
#line 897 "searching.c"
	}
	_tmp4_ = FALSE;
#line 476 "searching.vala"
	if (_vala_strcmp0 (text, "") != 0) {
#line 476 "searching.vala"
		_tmp4_ = _vala_strcmp0 (text, valide_searching_get_search_text (self)) == 0;
#line 904 "searching.c"
	} else {
#line 476 "searching.vala"
		_tmp4_ = FALSE;
#line 908 "searching.c"
	}
#line 476 "searching.vala"
	if (_tmp4_) {
#line 478 "searching.vala"
		gtk_text_buffer_delete (buffer, &start, &end);
#line 479 "searching.vala"
		gtk_text_buffer_insert (buffer, &start, valide_searching_get_replace_text (self), -1);
#line 916 "searching.c"
	}
	result = valide_searching_find_next (self);
	text = (g_free (text), NULL);
	(buffer == NULL) ? NULL : (buffer = (g_object_unref (buffer), NULL));
	return result;
}


/**
   * Replace the previous word
   *
   * @return true if the world is found
   */
#line 489 "searching.vala"
gboolean valide_searching_replace_prev (ValideSearching* self) {
#line 932 "searching.c"
	gboolean result;
	char* text;
	GtkTextIter start = {0};
	GtkTextIter end = {0};
	GtkTextBuffer* buffer;
	GtkTextBuffer* _tmp1_;
	GtkTextBuffer* _tmp0_;
	gboolean _tmp4_;
#line 489 "searching.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 943 "searching.c"
	text = g_strdup ("");
	buffer = NULL;
	_tmp1_ = NULL;
#line 496 "searching.vala"
	_tmp0_ = NULL;
#line 496 "searching.vala"
	buffer = (_tmp1_ = (_tmp0_ = (GtkTextBuffer*) valide_abstract_text_document_get_buffer (self->priv->_document), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (buffer == NULL) ? NULL : (buffer = (g_object_unref (buffer), NULL)), _tmp1_);
#line 497 "searching.vala"
	if (gtk_text_buffer_get_selection_bounds (buffer, &start, &end)) {
#line 953 "searching.c"
		char* _tmp3_;
		const char* _tmp2_;
		_tmp3_ = NULL;
#line 499 "searching.vala"
		_tmp2_ = NULL;
#line 499 "searching.vala"
		text = (_tmp3_ = (_tmp2_ = gtk_text_buffer_get_text (buffer, &start, &end, FALSE), (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)), text = (g_free (text), NULL), _tmp3_);
#line 961 "searching.c"
	}
	_tmp4_ = FALSE;
#line 502 "searching.vala"
	if (_vala_strcmp0 (text, "") != 0) {
#line 502 "searching.vala"
		_tmp4_ = _vala_strcmp0 (text, valide_searching_get_search_text (self)) == 0;
#line 968 "searching.c"
	} else {
#line 502 "searching.vala"
		_tmp4_ = FALSE;
#line 972 "searching.c"
	}
#line 502 "searching.vala"
	if (_tmp4_) {
#line 504 "searching.vala"
		gtk_text_buffer_delete (buffer, &start, &end);
#line 505 "searching.vala"
		gtk_text_buffer_insert (buffer, &start, valide_searching_get_replace_text (self), -1);
#line 980 "searching.c"
	}
	result = valide_searching_find_prev (self);
	text = (g_free (text), NULL);
	(buffer == NULL) ? NULL : (buffer = (g_object_unref (buffer), NULL));
	return result;
}


/**
   * Replace the all words
   */
#line 513 "searching.vala"
void valide_searching_replace_all (ValideSearching* self) {
#line 994 "searching.c"
	gboolean _tmp0_;
#line 513 "searching.vala"
	g_return_if_fail (self != NULL);
#line 998 "searching.c"
	_tmp0_ = FALSE;
#line 515 "searching.vala"
	if (_vala_strcmp0 (valide_searching_get_search_text (self), "") != 0) {
#line 515 "searching.vala"
		_tmp0_ = _vala_strcmp0 (valide_searching_get_search_text (self), valide_searching_get_replace_text (self)) != 0;
#line 1004 "searching.c"
	} else {
#line 515 "searching.vala"
		_tmp0_ = FALSE;
#line 1008 "searching.c"
	}
#line 515 "searching.vala"
	if (_tmp0_) {
#line 1012 "searching.c"
		gboolean found;
		guint count;
		GtkTextIter end = {0};
		GtkTextMark* mark;
		GtkTextIter iter = {0};
		GtkTextIter start = {0};
		GtkTextBuffer* buffer;
		GtkSourceSearchFlags flags;
		GtkTextBuffer* _tmp2_;
		GtkTextBuffer* _tmp1_;
		found = FALSE;
		count = (guint) 0;
		mark = NULL;
		buffer = NULL;
		flags = 0;
#line 526 "searching.vala"
		flags = GTK_SOURCE_SEARCH_TEXT_ONLY;
#line 1030 "searching.c"
		_tmp2_ = NULL;
#line 527 "searching.vala"
		_tmp1_ = NULL;
#line 527 "searching.vala"
		buffer = (_tmp2_ = (_tmp1_ = (GtkTextBuffer*) valide_abstract_text_document_get_buffer (self->priv->_document), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (buffer == NULL) ? NULL : (buffer = (g_object_unref (buffer), NULL)), _tmp2_);
#line 529 "searching.vala"
		if (!valide_searching_get_casse (self)) {
#line 531 "searching.vala"
			flags = flags | GTK_SOURCE_SEARCH_CASE_INSENSITIVE;
#line 1040 "searching.c"
		}
#line 534 "searching.vala"
		gtk_text_buffer_get_start_iter (buffer, &iter);
#line 535 "searching.vala"
		while (TRUE) {
#line 1046 "searching.c"
			gboolean _tmp3_;
#line 535 "searching.vala"
			if (!(found = gtk_source_iter_forward_search (&iter, valide_searching_get_search_text (self), flags, &start, &end, NULL))) {
#line 535 "searching.vala"
				break;
#line 1052 "searching.c"
			}
			_tmp3_ = FALSE;
#line 539 "searching.vala"
			if (found) {
#line 539 "searching.vala"
				_tmp3_ = valide_searching_get_word (self);
#line 1059 "searching.c"
			} else {
#line 539 "searching.vala"
				_tmp3_ = FALSE;
#line 1063 "searching.c"
			}
#line 539 "searching.vala"
			if (_tmp3_) {
#line 1067 "searching.c"
				gboolean _tmp4_;
				_tmp4_ = FALSE;
#line 541 "searching.vala"
				if (gtk_text_iter_starts_word (&start)) {
#line 541 "searching.vala"
					_tmp4_ = gtk_text_iter_ends_word (&end);
#line 1074 "searching.c"
				} else {
#line 541 "searching.vala"
					_tmp4_ = FALSE;
#line 1078 "searching.c"
				}
#line 541 "searching.vala"
				found = _tmp4_;
#line 1082 "searching.c"
			}
#line 543 "searching.vala"
			if (found) {
#line 1086 "searching.c"
				GtkTextMark* _tmp6_;
				GtkTextMark* _tmp5_;
				_tmp6_ = NULL;
#line 545 "searching.vala"
				_tmp5_ = NULL;
#line 545 "searching.vala"
				mark = (_tmp6_ = (_tmp5_ = gtk_text_buffer_create_mark (buffer, "search", &end, FALSE), (_tmp5_ == NULL) ? NULL : g_object_ref (_tmp5_)), (mark == NULL) ? NULL : (mark = (g_object_unref (mark), NULL)), _tmp6_);
#line 546 "searching.vala"
				gtk_text_buffer_delete (buffer, &start, &end);
#line 547 "searching.vala"
				gtk_text_buffer_insert (buffer, &start, valide_searching_get_replace_text (self), -1);
#line 548 "searching.vala"
				gtk_text_buffer_get_iter_at_mark (buffer, &end, mark);
#line 549 "searching.vala"
				gtk_text_buffer_delete_mark (buffer, mark);
#line 550 "searching.vala"
				count++;
#line 1104 "searching.c"
			}
#line 552 "searching.vala"
			iter = end;
#line 1108 "searching.c"
		}
#line 554 "searching.vala"
		if (count < 2) {
#line 1112 "searching.c"
			char* _tmp7_;
#line 820 "glib-2.0.vapi"
			_tmp7_ = NULL;
#line 556 "searching.vala"
			valide_searching_send_message (self, _tmp7_ = g_strdup_printf (_ ("Found and replaced %d occurrence"), count));
#line 1118 "searching.c"
			_tmp7_ = (g_free (_tmp7_), NULL);
		} else {
			char* _tmp8_;
#line 820 "glib-2.0.vapi"
			_tmp8_ = NULL;
#line 560 "searching.vala"
			valide_searching_send_message (self, _tmp8_ = g_strdup_printf (_ ("Found and replaced %d occurrences"), count));
#line 1126 "searching.c"
			_tmp8_ = (g_free (_tmp8_), NULL);
		}
		(mark == NULL) ? NULL : (mark = (g_object_unref (mark), NULL));
		(buffer == NULL) ? NULL : (buffer = (g_object_unref (buffer), NULL));
	}
}


/**
   * Show the search dialog
   */
#line 568 "searching.vala"
void valide_searching_show_search (ValideSearching* self) {
#line 568 "searching.vala"
	g_return_if_fail (self != NULL);
#line 570 "searching.vala"
	gtk_widget_show_all ((GtkWidget*) self);
#line 571 "searching.vala"
	gtk_widget_hide ((GtkWidget*) self->priv->message_label);
#line 572 "searching.vala"
	gtk_widget_hide ((GtkWidget*) self->priv->hbox_replace);
#line 573 "searching.vala"
	gtk_widget_grab_focus ((GtkWidget*) self->priv->entry_search);
#line 574 "searching.vala"
	if (_vala_strcmp0 (valide_searching_get_search_text (self), "") != 0) {
#line 576 "searching.vala"
		gtk_editable_select_region ((GtkEditable*) self->priv->entry_search, 0, (gint) strlen (valide_searching_get_search_text (self)));
#line 1154 "searching.c"
	}
}


/**
   * Show the replace dialog
   */
#line 583 "searching.vala"
void valide_searching_show_replace (ValideSearching* self) {
#line 583 "searching.vala"
	g_return_if_fail (self != NULL);
#line 585 "searching.vala"
	gtk_widget_show_all ((GtkWidget*) self);
#line 586 "searching.vala"
	gtk_widget_hide ((GtkWidget*) self->priv->message_label);
#line 587 "searching.vala"
	if (_vala_strcmp0 (valide_searching_get_search_text (self), "") != 0) {
#line 589 "searching.vala"
		gtk_widget_grab_focus ((GtkWidget*) self->priv->entry_replace);
#line 591 "searching.vala"
		if (_vala_strcmp0 (valide_searching_get_replace_text (self), "") != 0) {
#line 593 "searching.vala"
			gtk_editable_select_region ((GtkEditable*) self->priv->entry_replace, 0, (gint) strlen (valide_searching_get_replace_text (self)));
#line 1178 "searching.c"
		}
	} else {
#line 598 "searching.vala"
		gtk_widget_grab_focus ((GtkWidget*) self->priv->entry_search);
#line 1183 "searching.c"
	}
}


ValideAbstractTextDocument* valide_searching_get_document (ValideSearching* self) {
	ValideAbstractTextDocument* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_document;
	return result;
}


static void valide_searching_set_document (ValideSearching* self, ValideAbstractTextDocument* value) {
	g_return_if_fail (self != NULL);
	self->priv->_document = value;
	g_object_notify ((GObject *) self, "document");
}


const char* valide_searching_get_search_text (ValideSearching* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = gtk_entry_get_text (self->priv->entry_search);
	return result;
}


void valide_searching_set_search_text (ValideSearching* self, const char* value) {
	g_return_if_fail (self != NULL);
#line 94 "searching.vala"
	gtk_entry_set_text (self->priv->entry_search, value);
#line 1215 "searching.c"
	g_object_notify ((GObject *) self, "search-text");
}


const char* valide_searching_get_replace_text (ValideSearching* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = gtk_entry_get_text (self->priv->entry_replace);
	return result;
}


void valide_searching_set_replace_text (ValideSearching* self, const char* value) {
	g_return_if_fail (self != NULL);
#line 103 "searching.vala"
	gtk_entry_set_text (self->priv->entry_replace, value);
#line 1232 "searching.c"
	g_object_notify ((GObject *) self, "replace-text");
}


gboolean valide_searching_get_casse (ValideSearching* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = gtk_toggle_button_get_active ((GtkToggleButton*) self->priv->options_window->check_casse);
	return result;
}


void valide_searching_set_casse (ValideSearching* self, gboolean value) {
	g_return_if_fail (self != NULL);
#line 112 "searching.vala"
	gtk_toggle_button_set_active ((GtkToggleButton*) self->priv->options_window->check_casse, value);
#line 1249 "searching.c"
	g_object_notify ((GObject *) self, "casse");
}


gboolean valide_searching_get_word (ValideSearching* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = gtk_toggle_button_get_active ((GtkToggleButton*) self->priv->options_window->check_word);
	return result;
}


void valide_searching_set_word (ValideSearching* self, gboolean value) {
	g_return_if_fail (self != NULL);
#line 121 "searching.vala"
	gtk_toggle_button_set_active ((GtkToggleButton*) self->priv->options_window->check_word, value);
#line 1266 "searching.c"
	g_object_notify ((GObject *) self, "word");
}


static void _lambda16_ (ValideSearching* self) {
#line 315 "searching.vala"
	gtk_widget_hide ((GtkWidget*) self);
#line 1274 "searching.c"
}


static void __lambda16__gtk_button_clicked (GtkButton* _sender, gpointer self) {
	_lambda16_ (self);
}


static void _lambda17_ (ValideSearching* self) {
#line 322 "searching.vala"
	valide_searching_find (self, FALSE, FALSE);
#line 1286 "searching.c"
}


static void __lambda17__gtk_editable_changed (GtkEditable* _sender, gpointer self) {
	_lambda17_ (self);
}


#line 202 "searching.vala"
static gboolean _valide_searching_key_press_event_cb_gtk_widget_key_press_event (GtkWidget* _sender, const GdkEventKey* event, gpointer self) {
#line 1297 "searching.c"
	return valide_searching_key_press_event_cb (self, _sender, event);
}


static void _lambda18_ (ValideSearching* self) {
#line 336 "searching.vala"
	gtk_toggle_button_set_active (self->priv->toggle_button, FALSE);
#line 1305 "searching.c"
}


static void __lambda18__gtk_widget_hide (GtkWidget* _sender, gpointer self) {
	_lambda18_ (self);
}


static void _lambda19_ (GtkToggleButton* s, ValideSearching* self) {
	g_return_if_fail (s != NULL);
#line 339 "searching.vala"
	if (gtk_toggle_button_get_active (s)) {
#line 341 "searching.vala"
		valide_searching_show_options (self);
#line 1320 "searching.c"
	} else {
#line 345 "searching.vala"
		gtk_widget_hide ((GtkWidget*) self->priv->options_window);
#line 1324 "searching.c"
	}
}


static void __lambda19__gtk_toggle_button_toggled (GtkToggleButton* _sender, gpointer self) {
	_lambda19_ (_sender, self);
}


static void _lambda20_ (ValideSearching* self) {
#line 352 "searching.vala"
	valide_searching_find_prev (self);
#line 1337 "searching.c"
}


static void __lambda20__gtk_button_clicked (GtkButton* _sender, gpointer self) {
	_lambda20_ (self);
}


static void _lambda21_ (ValideSearching* self) {
#line 358 "searching.vala"
	valide_searching_find_next (self);
#line 1349 "searching.c"
}


static void __lambda21__gtk_button_clicked (GtkButton* _sender, gpointer self) {
	_lambda21_ (self);
}


static void _lambda22_ (ValideSearching* self) {
#line 373 "searching.vala"
	valide_searching_replace_prev (self);
#line 1361 "searching.c"
}


static void __lambda22__gtk_button_clicked (GtkButton* _sender, gpointer self) {
	_lambda22_ (self);
}


static void _lambda23_ (ValideSearching* self) {
#line 379 "searching.vala"
	valide_searching_replace_next (self);
#line 1373 "searching.c"
}


static void __lambda23__gtk_button_clicked (GtkButton* _sender, gpointer self) {
	_lambda23_ (self);
}


static void _lambda24_ (ValideSearching* self) {
#line 387 "searching.vala"
	valide_searching_replace_all (self);
#line 1385 "searching.c"
}


static void __lambda24__gtk_button_clicked (GtkButton* _sender, gpointer self) {
	_lambda24_ (self);
}


static GObject * valide_searching_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValideSearchingClass * klass;
	GObjectClass * parent_class;
	ValideSearching * self;
	klass = VALIDE_SEARCHING_CLASS (g_type_class_peek (VALIDE_TYPE_SEARCHING));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_SEARCHING (obj);
	{
		GtkHBox* hbox;
		GtkImage* image;
		GtkButton* button;
		GtkHSeparator* _tmp3_;
		GtkLabel* _tmp4_;
		GtkHBox* _tmp5_;
		GtkHBox* _tmp6_;
		GtkButton* _tmp7_;
		GtkLabel* _tmp8_;
		GtkEntry* _tmp9_;
		ValideOptionsWindow* _tmp10_;
		GtkImage* _tmp11_;
		GtkToggleButton* _tmp12_;
		GtkButton* _tmp13_;
		GtkButton* _tmp14_;
		GtkHBox* _tmp15_;
		GtkLabel* _tmp16_;
		GtkEntry* _tmp17_;
		GtkButton* _tmp18_;
		GtkButton* _tmp19_;
		GtkButton* _tmp20_;
		hbox = NULL;
		image = NULL;
		button = NULL;
		_tmp3_ = NULL;
#line 298 "searching.vala"
		gtk_box_pack_start ((GtkBox*) self, (GtkWidget*) (_tmp3_ = g_object_ref_sink ((GtkHSeparator*) gtk_hseparator_new ())), FALSE, TRUE, (guint) 0);
#line 1431 "searching.c"
		(_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL));
		/* Message box */
		_tmp4_ = NULL;
#line 301 "searching.vala"
		self->priv->message_label = (_tmp4_ = g_object_ref_sink ((GtkLabel*) gtk_label_new ("")), (self->priv->message_label == NULL) ? NULL : (self->priv->message_label = (g_object_unref (self->priv->message_label), NULL)), _tmp4_);
#line 302 "searching.vala"
		gtk_misc_set_alignment ((GtkMisc*) self->priv->message_label, 0.0f, 0.5f);
#line 1439 "searching.c"
		/** @todo Find a better placement
		    this.pack_start (this.message_label, false, true, 0);*/
		_tmp5_ = NULL;
#line 306 "searching.vala"
		hbox = (_tmp5_ = g_object_ref_sink ((GtkHBox*) gtk_hbox_new (FALSE, 5)), (hbox == NULL) ? NULL : (hbox = (g_object_unref (hbox), NULL)), _tmp5_);
#line 307 "searching.vala"
		gtk_box_pack_start ((GtkBox*) self, (GtkWidget*) hbox, FALSE, FALSE, (guint) 0);
#line 1447 "searching.c"
		/* Search */
		_tmp6_ = NULL;
#line 310 "searching.vala"
		self->priv->hbox_search = (_tmp6_ = g_object_ref_sink ((GtkHBox*) gtk_hbox_new (FALSE, 0)), (self->priv->hbox_search == NULL) ? NULL : (self->priv->hbox_search = (g_object_unref (self->priv->hbox_search), NULL)), _tmp6_);
#line 311 "searching.vala"
		gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) self->priv->hbox_search, FALSE, FALSE, (guint) 0);
#line 1454 "searching.c"
		_tmp7_ = NULL;
#line 313 "searching.vala"
		button = (_tmp7_ = valide_searching_create_button (self, GTK_STOCK_CLOSE, _ ("Close")), (button == NULL) ? NULL : (button = (g_object_unref (button), NULL)), _tmp7_);
#line 314 "searching.vala"
		g_signal_connect (button, "clicked", (GCallback) __lambda16__gtk_button_clicked, self);
#line 317 "searching.vala"
		gtk_box_pack_start ((GtkBox*) self->priv->hbox_search, (GtkWidget*) button, FALSE, FALSE, (guint) 0);
#line 1462 "searching.c"
		_tmp8_ = NULL;
#line 319 "searching.vala"
		gtk_box_pack_start ((GtkBox*) self->priv->hbox_search, (GtkWidget*) (_tmp8_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("Find: ")))), FALSE, FALSE, (guint) 0);
#line 1466 "searching.c"
		(_tmp8_ == NULL) ? NULL : (_tmp8_ = (g_object_unref (_tmp8_), NULL));
		_tmp9_ = NULL;
#line 320 "searching.vala"
		self->priv->entry_search = (_tmp9_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (self->priv->entry_search == NULL) ? NULL : (self->priv->entry_search = (g_object_unref (self->priv->entry_search), NULL)), _tmp9_);
#line 321 "searching.vala"
		g_signal_connect ((GtkEditable*) self->priv->entry_search, "changed", (GCallback) __lambda17__gtk_editable_changed, self);
#line 324 "searching.vala"
		g_signal_connect_object ((GtkWidget*) self->priv->entry_search, "key-press-event", (GCallback) _valide_searching_key_press_event_cb_gtk_widget_key_press_event, self, 0);
#line 325 "searching.vala"
		gtk_box_pack_start ((GtkBox*) self->priv->hbox_search, (GtkWidget*) self->priv->entry_search, FALSE, TRUE, (guint) 0);
#line 1477 "searching.c"
		_tmp10_ = NULL;
#line 327 "searching.vala"
		self->priv->options_window = (_tmp10_ = g_object_ref_sink (valide_options_window_new ()), (self->priv->options_window == NULL) ? NULL : (self->priv->options_window = (g_object_unref (self->priv->options_window), NULL)), _tmp10_);
#line 1481 "searching.c"
		_tmp11_ = NULL;
#line 329 "searching.vala"
		image = (_tmp11_ = g_object_ref_sink ((GtkImage*) gtk_image_new_from_stock (GTK_STOCK_PREFERENCES, GTK_ICON_SIZE_BUTTON)), (image == NULL) ? NULL : (image = (g_object_unref (image), NULL)), _tmp11_);
#line 1485 "searching.c"
		_tmp12_ = NULL;
#line 330 "searching.vala"
		self->priv->toggle_button = (_tmp12_ = g_object_ref_sink ((GtkToggleButton*) gtk_toggle_button_new ()), (self->priv->toggle_button == NULL) ? NULL : (self->priv->toggle_button = (g_object_unref (self->priv->toggle_button), NULL)), _tmp12_);
#line 331 "searching.vala"
		gtk_container_add ((GtkContainer*) self->priv->toggle_button, (GtkWidget*) image);
#line 332 "searching.vala"
		gtk_button_set_relief ((GtkButton*) self->priv->toggle_button, GTK_RELIEF_NONE);
#line 333 "searching.vala"
		gtk_widget_set_tooltip_text ((GtkWidget*) self->priv->toggle_button, _ ("Options"));
#line 334 "searching.vala"
		g_object_set ((GtkWidget*) self->priv->toggle_button, "can-focus", FALSE, NULL);
#line 335 "searching.vala"
		g_signal_connect ((GtkWidget*) self->priv->options_window, "hide", (GCallback) __lambda18__gtk_widget_hide, self);
#line 338 "searching.vala"
		g_signal_connect (self->priv->toggle_button, "toggled", (GCallback) __lambda19__gtk_toggle_button_toggled, self);
#line 348 "searching.vala"
		gtk_box_pack_start ((GtkBox*) self->priv->hbox_search, (GtkWidget*) self->priv->toggle_button, FALSE, FALSE, (guint) 0);
#line 1503 "searching.c"
		_tmp13_ = NULL;
#line 350 "searching.vala"
		button = (_tmp13_ = valide_searching_create_button (self, GTK_STOCK_GO_BACK, _ ("Find next")), (button == NULL) ? NULL : (button = (g_object_unref (button), NULL)), _tmp13_);
#line 351 "searching.vala"
		g_signal_connect (button, "clicked", (GCallback) __lambda20__gtk_button_clicked, self);
#line 354 "searching.vala"
		gtk_box_pack_start ((GtkBox*) self->priv->hbox_search, (GtkWidget*) button, FALSE, FALSE, (guint) 0);
#line 1511 "searching.c"
		_tmp14_ = NULL;
#line 356 "searching.vala"
		button = (_tmp14_ = valide_searching_create_button (self, GTK_STOCK_GO_FORWARD, _ ("Find previous")), (button == NULL) ? NULL : (button = (g_object_unref (button), NULL)), _tmp14_);
#line 357 "searching.vala"
		g_signal_connect (button, "clicked", (GCallback) __lambda21__gtk_button_clicked, self);
#line 360 "searching.vala"
		gtk_box_pack_start ((GtkBox*) self->priv->hbox_search, (GtkWidget*) button, FALSE, FALSE, (guint) 0);
#line 1519 "searching.c"
		/* Replace */
		_tmp15_ = NULL;
#line 363 "searching.vala"
		self->priv->hbox_replace = (_tmp15_ = g_object_ref_sink ((GtkHBox*) gtk_hbox_new (FALSE, 0)), (self->priv->hbox_replace == NULL) ? NULL : (self->priv->hbox_replace = (g_object_unref (self->priv->hbox_replace), NULL)), _tmp15_);
#line 364 "searching.vala"
		gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) self->priv->hbox_replace, FALSE, FALSE, (guint) 0);
#line 1526 "searching.c"
		_tmp16_ = NULL;
#line 366 "searching.vala"
		gtk_box_pack_start ((GtkBox*) self->priv->hbox_replace, (GtkWidget*) (_tmp16_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("Replace with: ")))), FALSE, FALSE, (guint) 0);
#line 1530 "searching.c"
		(_tmp16_ == NULL) ? NULL : (_tmp16_ = (g_object_unref (_tmp16_), NULL));
		_tmp17_ = NULL;
#line 367 "searching.vala"
		self->priv->entry_replace = (_tmp17_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (self->priv->entry_replace == NULL) ? NULL : (self->priv->entry_replace = (g_object_unref (self->priv->entry_replace), NULL)), _tmp17_);
#line 368 "searching.vala"
		g_signal_connect_object ((GtkWidget*) self->priv->entry_replace, "key-press-event", (GCallback) _valide_searching_key_press_event_cb_gtk_widget_key_press_event, self, 0);
#line 369 "searching.vala"
		gtk_box_pack_start ((GtkBox*) self->priv->hbox_replace, (GtkWidget*) self->priv->entry_replace, FALSE, TRUE, (guint) 0);
#line 1539 "searching.c"
		_tmp18_ = NULL;
#line 371 "searching.vala"
		button = (_tmp18_ = valide_searching_create_button (self, GTK_STOCK_GO_BACK, _ ("Find and replace next")), (button == NULL) ? NULL : (button = (g_object_unref (button), NULL)), _tmp18_);
#line 372 "searching.vala"
		g_signal_connect (button, "clicked", (GCallback) __lambda22__gtk_button_clicked, self);
#line 375 "searching.vala"
		gtk_box_pack_start ((GtkBox*) self->priv->hbox_replace, (GtkWidget*) button, FALSE, FALSE, (guint) 0);
#line 1547 "searching.c"
		_tmp19_ = NULL;
#line 377 "searching.vala"
		button = (_tmp19_ = valide_searching_create_button (self, GTK_STOCK_GO_FORWARD, _ ("Find and replace previous")), (button == NULL) ? NULL : (button = (g_object_unref (button), NULL)), _tmp19_);
#line 378 "searching.vala"
		g_signal_connect (button, "clicked", (GCallback) __lambda23__gtk_button_clicked, self);
#line 381 "searching.vala"
		gtk_box_pack_start ((GtkBox*) self->priv->hbox_replace, (GtkWidget*) button, FALSE, FALSE, (guint) 0);
#line 1555 "searching.c"
		_tmp20_ = NULL;
#line 383 "searching.vala"
		button = (_tmp20_ = g_object_ref_sink ((GtkButton*) gtk_button_new_with_label (_ ("Replace all"))), (button == NULL) ? NULL : (button = (g_object_unref (button), NULL)), _tmp20_);
#line 384 "searching.vala"
		gtk_button_set_relief (button, GTK_RELIEF_HALF);
#line 385 "searching.vala"
		g_object_set ((GtkWidget*) button, "can-focus", FALSE, NULL);
#line 386 "searching.vala"
		g_signal_connect (button, "clicked", (GCallback) __lambda24__gtk_button_clicked, self);
#line 389 "searching.vala"
		gtk_box_pack_start ((GtkBox*) self->priv->hbox_replace, (GtkWidget*) button, FALSE, FALSE, (guint) 0);
#line 1567 "searching.c"
		(hbox == NULL) ? NULL : (hbox = (g_object_unref (hbox), NULL));
		(image == NULL) ? NULL : (image = (g_object_unref (image), NULL));
		(button == NULL) ? NULL : (button = (g_object_unref (button), NULL));
	}
	return obj;
}


static void valide_searching_class_init (ValideSearchingClass * klass) {
	valide_searching_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideSearchingPrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_searching_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_searching_set_property;
	G_OBJECT_CLASS (klass)->constructor = valide_searching_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_searching_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_SEARCHING_DOCUMENT, g_param_spec_object ("document", "document", "document", VALIDE_TYPE_ABSTRACT_TEXT_DOCUMENT, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_SEARCHING_SEARCH_TEXT, g_param_spec_string ("search-text", "search-text", "search-text", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_SEARCHING_REPLACE_TEXT, g_param_spec_string ("replace-text", "replace-text", "replace-text", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_SEARCHING_CASSE, g_param_spec_boolean ("casse", "casse", "casse", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_SEARCHING_WORD, g_param_spec_boolean ("word", "word", "word", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void valide_searching_instance_init (ValideSearching * self) {
	self->priv = VALIDE_SEARCHING_GET_PRIVATE (self);
	self->priv->reverse = FALSE;
}


static void valide_searching_finalize (GObject* obj) {
	ValideSearching * self;
	self = VALIDE_SEARCHING (obj);
	(self->priv->hbox_search == NULL) ? NULL : (self->priv->hbox_search = (g_object_unref (self->priv->hbox_search), NULL));
	(self->priv->hbox_replace == NULL) ? NULL : (self->priv->hbox_replace = (g_object_unref (self->priv->hbox_replace), NULL));
	(self->priv->entry_search == NULL) ? NULL : (self->priv->entry_search = (g_object_unref (self->priv->entry_search), NULL));
	(self->priv->entry_replace == NULL) ? NULL : (self->priv->entry_replace = (g_object_unref (self->priv->entry_replace), NULL));
	(self->priv->message_label == NULL) ? NULL : (self->priv->message_label = (g_object_unref (self->priv->message_label), NULL));
	(self->priv->options_window == NULL) ? NULL : (self->priv->options_window = (g_object_unref (self->priv->options_window), NULL));
	(self->priv->toggle_button == NULL) ? NULL : (self->priv->toggle_button = (g_object_unref (self->priv->toggle_button), NULL));
	G_OBJECT_CLASS (valide_searching_parent_class)->finalize (obj);
}


GType valide_searching_get_type (void) {
	static GType valide_searching_type_id = 0;
	if (valide_searching_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideSearchingClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_searching_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideSearching), 0, (GInstanceInitFunc) valide_searching_instance_init, NULL };
		valide_searching_type_id = g_type_register_static (GTK_TYPE_VBOX, "ValideSearching", &g_define_type_info, 0);
	}
	return valide_searching_type_id;
}


static void valide_searching_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideSearching * self;
	gpointer boxed;
	self = VALIDE_SEARCHING (object);
	switch (property_id) {
		case VALIDE_SEARCHING_DOCUMENT:
		g_value_set_object (value, valide_searching_get_document (self));
		break;
		case VALIDE_SEARCHING_SEARCH_TEXT:
		g_value_set_string (value, valide_searching_get_search_text (self));
		break;
		case VALIDE_SEARCHING_REPLACE_TEXT:
		g_value_set_string (value, valide_searching_get_replace_text (self));
		break;
		case VALIDE_SEARCHING_CASSE:
		g_value_set_boolean (value, valide_searching_get_casse (self));
		break;
		case VALIDE_SEARCHING_WORD:
		g_value_set_boolean (value, valide_searching_get_word (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_searching_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideSearching * self;
	self = VALIDE_SEARCHING (object);
	switch (property_id) {
		case VALIDE_SEARCHING_DOCUMENT:
		valide_searching_set_document (self, g_value_get_object (value));
		break;
		case VALIDE_SEARCHING_SEARCH_TEXT:
		valide_searching_set_search_text (self, g_value_get_string (value));
		break;
		case VALIDE_SEARCHING_REPLACE_TEXT:
		valide_searching_set_replace_text (self, g_value_get_string (value));
		break;
		case VALIDE_SEARCHING_CASSE:
		valide_searching_set_casse (self, g_value_get_boolean (value));
		break;
		case VALIDE_SEARCHING_WORD:
		valide_searching_set_word (self, g_value_get_boolean (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




