/* source-buffer.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtksourceview/gtksourceview.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <glib/gstdio.h>


#define VALIDE_TYPE_SOURCE_BUFFER (valide_source_buffer_get_type ())
#define VALIDE_SOURCE_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_SOURCE_BUFFER, ValideSourceBuffer))
#define VALIDE_SOURCE_BUFFER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_SOURCE_BUFFER, ValideSourceBufferClass))
#define VALIDE_IS_SOURCE_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_SOURCE_BUFFER))
#define VALIDE_IS_SOURCE_BUFFER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_SOURCE_BUFFER))
#define VALIDE_SOURCE_BUFFER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_SOURCE_BUFFER, ValideSourceBufferClass))

typedef struct _ValideSourceBuffer ValideSourceBuffer;
typedef struct _ValideSourceBufferClass ValideSourceBufferClass;
typedef struct _ValideSourceBufferPrivate ValideSourceBufferPrivate;

struct _ValideSourceBuffer {
	GtkSourceBuffer parent_instance;
	ValideSourceBufferPrivate * priv;
	char* path;
};

struct _ValideSourceBufferClass {
	GtkSourceBufferClass parent_class;
};

struct _ValideSourceBufferPrivate {
	char* _encoding;
};


static gpointer valide_source_buffer_parent_class = NULL;

GType valide_source_buffer_get_type (void);
#define VALIDE_SOURCE_BUFFER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_SOURCE_BUFFER, ValideSourceBufferPrivate))
enum  {
	VALIDE_SOURCE_BUFFER_DUMMY_PROPERTY,
	VALIDE_SOURCE_BUFFER_ENCODING
};
char* valide_source_buffer_get_buffer_contents (ValideSourceBuffer* self);
char* valide_encoding_to_utf8 (const char* path, char** encoding, GError** error);
char* valide_source_buffer_get_contents (ValideSourceBuffer* self, GError** error);
char* valide_encoding_from_utf8 (GtkTextBuffer* text_buffer, char** encoding, GError** error);
void valide_source_buffer_set_contents (ValideSourceBuffer* self, const char* new_path, GError** error);
gboolean valide_utils_char_is_separator (gunichar ch);
char* valide_source_buffer_get_last_word_and_iter (ValideSourceBuffer* self, GtkTextIter* start_word, GtkTextIter* end_word);
char* valide_source_buffer_get_last_word (ValideSourceBuffer* self);
void valide_source_buffer_replace_actual_word (ValideSourceBuffer* self, const char* text);
ValideSourceBuffer* valide_source_buffer_new (void);
ValideSourceBuffer* valide_source_buffer_construct (GType object_type);
const char* valide_source_buffer_get_encoding (ValideSourceBuffer* self);
void valide_source_buffer_set_encoding (ValideSourceBuffer* self, const char* value);
static void valide_source_buffer_finalize (GObject* obj);
static void valide_source_buffer_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_source_buffer_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);



/**
   * Return the content of the buffer
   *
   * @return The content of the buffer
   */
#line 43 "source-buffer.vala"
char* valide_source_buffer_get_buffer_contents (ValideSourceBuffer* self) {
#line 92 "source-buffer.c"
	char* result;
	GtkTextIter start = {0};
	GtkTextIter end = {0};
	const char* _tmp0_;
#line 43 "source-buffer.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 47 "source-buffer.vala"
	gtk_text_buffer_get_bounds ((GtkTextBuffer*) self, &start, &end);
#line 48 "source-buffer.vala"
	_tmp0_ = NULL;
#line 103 "source-buffer.c"
	result = (_tmp0_ = gtk_text_buffer_get_text ((GtkTextBuffer*) self, &start, &end, FALSE), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
	return result;
}


/**
   * Return the entire file content in UTF-8 encoding
   *
   * @return The content of the file
   */
#line 56 "source-buffer.vala"
char* valide_source_buffer_get_contents (ValideSourceBuffer* self, GError** error) {
#line 116 "source-buffer.c"
	char* result;
	GError * _inner_error_;
	char* contents;
	char* _tmp2_;
	char* _tmp1_;
	char* _tmp0_;
	char* _tmp3_;
	char* _tmp4_;
#line 56 "source-buffer.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 127 "source-buffer.c"
	_inner_error_ = NULL;
	contents = NULL;
	_tmp2_ = NULL;
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	_tmp3_ = (_tmp1_ = valide_encoding_to_utf8 (self->path, &_tmp0_, &_inner_error_), self->priv->_encoding = (_tmp2_ = _tmp0_, self->priv->_encoding = (g_free (self->priv->_encoding), NULL), _tmp2_), _tmp1_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		contents = (g_free (contents), NULL);
		return NULL;
	}
	_tmp4_ = NULL;
#line 60 "source-buffer.vala"
	contents = (_tmp4_ = _tmp3_, contents = (g_free (contents), NULL), _tmp4_);
#line 142 "source-buffer.c"
	result = contents;
	return result;
}


/**
   * Write the entire buffer content in file
   */
#line 67 "source-buffer.vala"
void valide_source_buffer_set_contents (ValideSourceBuffer* self, const char* new_path, GError** error) {
#line 153 "source-buffer.c"
	GError * _inner_error_;
	char* path;
	char* contents;
	char* _tmp2_;
	char* _tmp1_;
	char* _tmp0_;
	char* _tmp3_;
	char* _tmp4_;
#line 67 "source-buffer.vala"
	g_return_if_fail (self != NULL);
#line 164 "source-buffer.c"
	_inner_error_ = NULL;
	path = NULL;
	contents = NULL;
	_tmp2_ = NULL;
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	_tmp3_ = (_tmp1_ = valide_encoding_from_utf8 ((GtkTextBuffer*) self, &_tmp0_, &_inner_error_), self->priv->_encoding = (_tmp2_ = _tmp0_, self->priv->_encoding = (g_free (self->priv->_encoding), NULL), _tmp2_), _tmp1_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		path = (g_free (path), NULL);
		contents = (g_free (contents), NULL);
		return;
	}
	_tmp4_ = NULL;
#line 72 "source-buffer.vala"
	contents = (_tmp4_ = _tmp3_, contents = (g_free (contents), NULL), _tmp4_);
#line 73 "source-buffer.vala"
	if (new_path == NULL) {
#line 183 "source-buffer.c"
		char* _tmp6_;
		const char* _tmp5_;
		_tmp6_ = NULL;
#line 75 "source-buffer.vala"
		_tmp5_ = NULL;
#line 75 "source-buffer.vala"
		path = (_tmp6_ = (_tmp5_ = self->path, (_tmp5_ == NULL) ? NULL : g_strdup (_tmp5_)), path = (g_free (path), NULL), _tmp6_);
#line 191 "source-buffer.c"
	} else {
		char* _tmp8_;
		const char* _tmp7_;
		_tmp8_ = NULL;
#line 79 "source-buffer.vala"
		_tmp7_ = NULL;
#line 79 "source-buffer.vala"
		path = (_tmp8_ = (_tmp7_ = new_path, (_tmp7_ == NULL) ? NULL : g_strdup (_tmp7_)), path = (g_free (path), NULL), _tmp8_);
#line 200 "source-buffer.c"
	}
#line 81 "source-buffer.vala"
	g_file_set_contents (path, contents, (glong) (-1), &_inner_error_);
#line 204 "source-buffer.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		path = (g_free (path), NULL);
		contents = (g_free (contents), NULL);
		return;
	}
	path = (g_free (path), NULL);
	contents = (g_free (contents), NULL);
}


/**
   * Get the last word before the corsor
   *
   * @param start_word The iter at begining of the word
   * @param start_word The iter at end of the word
   *
   * @return the last word
   */
#line 92 "source-buffer.vala"
char* valide_source_buffer_get_last_word_and_iter (ValideSourceBuffer* self, GtkTextIter* start_word, GtkTextIter* end_word) {
#line 226 "source-buffer.c"
	char* result;
	gboolean found;
	gunichar ch;
	char* text;
	gboolean no_doc_start;
	GtkTextIter actual = {0};
	GtkTextIter start_iter = {0};
	GtkTextMark* insert_mark;
	GtkTextMark* _tmp1_;
	GtkTextMark* _tmp0_;
#line 92 "source-buffer.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 239 "source-buffer.c"
	found = FALSE;
	ch = 0U;
	text = NULL;
	no_doc_start = FALSE;
	insert_mark = NULL;
	_tmp1_ = NULL;
#line 103 "source-buffer.vala"
	_tmp0_ = NULL;
#line 103 "source-buffer.vala"
	insert_mark = (_tmp1_ = (_tmp0_ = gtk_text_buffer_get_insert ((GtkTextBuffer*) self), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (insert_mark == NULL) ? NULL : (insert_mark = (g_object_unref (insert_mark), NULL)), _tmp1_);
#line 104 "source-buffer.vala"
	gtk_text_buffer_get_iter_at_mark ((GtkTextBuffer*) self, &actual, insert_mark);
#line 106 "source-buffer.vala"
	start_iter = actual;
#line 107 "source-buffer.vala"
	found = FALSE;
#line 256 "source-buffer.c"
	{
		gboolean _tmp2_;
		_tmp2_ = TRUE;
#line 109 "source-buffer.vala"
		while (TRUE) {
#line 109 "source-buffer.vala"
			if (!_tmp2_) {
#line 116 "source-buffer.vala"
				if (!gtk_text_iter_forward_char (&actual)) {
#line 116 "source-buffer.vala"
					break;
#line 268 "source-buffer.c"
				}
			}
#line 109 "source-buffer.vala"
			_tmp2_ = FALSE;
#line 111 "source-buffer.vala"
			ch = gtk_text_iter_get_char (&actual);
#line 112 "source-buffer.vala"
			if (valide_utils_char_is_separator (ch)) {
#line 114 "source-buffer.vala"
				break;
#line 279 "source-buffer.c"
			}
		}
	}
#line 117 "source-buffer.vala"
	(*end_word) = actual;
#line 119 "source-buffer.vala"
	while (TRUE) {
#line 119 "source-buffer.vala"
		if (!((no_doc_start = gtk_text_iter_backward_char (&start_iter)) == TRUE)) {
#line 119 "source-buffer.vala"
			break;
#line 291 "source-buffer.c"
		}
#line 121 "source-buffer.vala"
		ch = gtk_text_iter_get_char (&start_iter);
#line 295 "source-buffer.c"
		/*TODO Do better*/
#line 123 "source-buffer.vala"
		if (valide_utils_char_is_separator (ch)) {
#line 125 "source-buffer.vala"
			found = TRUE;
#line 126 "source-buffer.vala"
			break;
#line 303 "source-buffer.c"
		}
	}
#line 130 "source-buffer.vala"
	if (!no_doc_start) {
#line 308 "source-buffer.c"
		char* _tmp4_;
		const char* _tmp3_;
#line 132 "source-buffer.vala"
		gtk_text_buffer_get_start_iter ((GtkTextBuffer*) self, &start_iter);
#line 313 "source-buffer.c"
		_tmp4_ = NULL;
#line 133 "source-buffer.vala"
		_tmp3_ = NULL;
#line 133 "source-buffer.vala"
		text = (_tmp4_ = (_tmp3_ = gtk_text_iter_get_text (&start_iter, &actual), (_tmp3_ == NULL) ? NULL : g_strdup (_tmp3_)), text = (g_free (text), NULL), _tmp4_);
#line 319 "source-buffer.c"
	} else {
#line 137 "source-buffer.vala"
		if (found) {
#line 323 "source-buffer.c"
			char* _tmp6_;
			const char* _tmp5_;
#line 139 "source-buffer.vala"
			gtk_text_iter_forward_char (&start_iter);
#line 328 "source-buffer.c"
			_tmp6_ = NULL;
#line 140 "source-buffer.vala"
			_tmp5_ = NULL;
#line 140 "source-buffer.vala"
			text = (_tmp6_ = (_tmp5_ = gtk_text_iter_get_text (&start_iter, &actual), (_tmp5_ == NULL) ? NULL : g_strdup (_tmp5_)), text = (g_free (text), NULL), _tmp6_);
#line 334 "source-buffer.c"
		} else {
			char* _tmp7_;
#line 144 "source-buffer.vala"
			start_iter = actual;
#line 339 "source-buffer.c"
			_tmp7_ = NULL;
#line 145 "source-buffer.vala"
			text = (_tmp7_ = g_strdup (""), text = (g_free (text), NULL), _tmp7_);
#line 343 "source-buffer.c"
		}
	}
#line 148 "source-buffer.vala"
	(*start_word) = start_iter;
#line 348 "source-buffer.c"
	result = text;
	(insert_mark == NULL) ? NULL : (insert_mark = (g_object_unref (insert_mark), NULL));
	return result;
}


/**
   * Get the last word before the cursor
   *
   * @return the last word
   */
#line 157 "source-buffer.vala"
char* valide_source_buffer_get_last_word (ValideSourceBuffer* self) {
#line 362 "source-buffer.c"
	char* result;
	GtkTextIter end = {0};
	GtkTextIter start = {0};
#line 157 "source-buffer.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 368 "source-buffer.c"
	result = valide_source_buffer_get_last_word_and_iter (self, &start, &end);
	return result;
}


/**
   * Replace the current work
   *
   * @param text The new text
   */
#line 170 "source-buffer.vala"
void valide_source_buffer_replace_actual_word (ValideSourceBuffer* self, const char* text) {
#line 381 "source-buffer.c"
	GtkTextMark* mark;
	GtkTextIter word_end = {0};
	GtkTextIter word_start = {0};
	char* _tmp0_;
	GtkTextMark* _tmp2_;
	GtkTextMark* _tmp1_;
#line 170 "source-buffer.vala"
	g_return_if_fail (self != NULL);
#line 170 "source-buffer.vala"
	g_return_if_fail (text != NULL);
#line 392 "source-buffer.c"
	mark = NULL;
#line 176 "source-buffer.vala"
	gtk_text_buffer_begin_user_action ((GtkTextBuffer*) self);
#line 92 "source-buffer.vala"
	_tmp0_ = NULL;
#line 177 "source-buffer.vala"
	_tmp0_ = valide_source_buffer_get_last_word_and_iter (self, &word_start, &word_end);
#line 400 "source-buffer.c"
	_tmp0_ = (g_free (_tmp0_), NULL);
	_tmp2_ = NULL;
#line 179 "source-buffer.vala"
	_tmp1_ = NULL;
#line 179 "source-buffer.vala"
	mark = (_tmp2_ = (_tmp1_ = gtk_text_buffer_create_mark ((GtkTextBuffer*) self, "temp_replace", &word_start, TRUE), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (mark == NULL) ? NULL : (mark = (g_object_unref (mark), NULL)), _tmp2_);
#line 180 "source-buffer.vala"
	gtk_text_buffer_delete ((GtkTextBuffer*) self, &word_start, &word_end);
#line 181 "source-buffer.vala"
	gtk_text_buffer_get_iter_at_mark ((GtkTextBuffer*) self, &word_start, mark);
#line 182 "source-buffer.vala"
	gtk_text_buffer_insert ((GtkTextBuffer*) self, &word_start, text, -1);
#line 183 "source-buffer.vala"
	gtk_text_buffer_delete_mark ((GtkTextBuffer*) self, mark);
#line 184 "source-buffer.vala"
	gtk_text_buffer_end_user_action ((GtkTextBuffer*) self);
#line 417 "source-buffer.c"
	(mark == NULL) ? NULL : (mark = (g_object_unref (mark), NULL));
}


#line 22 "source-buffer.vala"
ValideSourceBuffer* valide_source_buffer_construct (GType object_type) {
#line 424 "source-buffer.c"
	ValideSourceBuffer * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


#line 22 "source-buffer.vala"
ValideSourceBuffer* valide_source_buffer_new (void) {
#line 22 "source-buffer.vala"
	return valide_source_buffer_construct (VALIDE_TYPE_SOURCE_BUFFER);
#line 435 "source-buffer.c"
}


const char* valide_source_buffer_get_encoding (ValideSourceBuffer* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_encoding;
	return result;
}


void valide_source_buffer_set_encoding (ValideSourceBuffer* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 35 "source-buffer.vala"
	_tmp0_ = NULL;
#line 454 "source-buffer.c"
	self->priv->_encoding = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_encoding = (g_free (self->priv->_encoding), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "encoding");
}


static void valide_source_buffer_class_init (ValideSourceBufferClass * klass) {
	valide_source_buffer_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideSourceBufferPrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_source_buffer_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_source_buffer_set_property;
	G_OBJECT_CLASS (klass)->finalize = valide_source_buffer_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_SOURCE_BUFFER_ENCODING, g_param_spec_string ("encoding", "encoding", "encoding", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void valide_source_buffer_instance_init (ValideSourceBuffer * self) {
	self->priv = VALIDE_SOURCE_BUFFER_GET_PRIVATE (self);
}


static void valide_source_buffer_finalize (GObject* obj) {
	ValideSourceBuffer * self;
	self = VALIDE_SOURCE_BUFFER (obj);
	self->path = (g_free (self->path), NULL);
	self->priv->_encoding = (g_free (self->priv->_encoding), NULL);
	G_OBJECT_CLASS (valide_source_buffer_parent_class)->finalize (obj);
}


GType valide_source_buffer_get_type (void) {
	static GType valide_source_buffer_type_id = 0;
	if (valide_source_buffer_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideSourceBufferClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_source_buffer_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideSourceBuffer), 0, (GInstanceInitFunc) valide_source_buffer_instance_init, NULL };
		valide_source_buffer_type_id = g_type_register_static (GTK_TYPE_SOURCE_BUFFER, "ValideSourceBuffer", &g_define_type_info, 0);
	}
	return valide_source_buffer_type_id;
}


static void valide_source_buffer_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideSourceBuffer * self;
	gpointer boxed;
	self = VALIDE_SOURCE_BUFFER (object);
	switch (property_id) {
		case VALIDE_SOURCE_BUFFER_ENCODING:
		g_value_set_string (value, valide_source_buffer_get_encoding (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_source_buffer_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideSourceBuffer * self;
	self = VALIDE_SOURCE_BUFFER (object);
	switch (property_id) {
		case VALIDE_SOURCE_BUFFER_ENCODING:
		valide_source_buffer_set_encoding (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}




