/* split-source-view.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <gtksourceview/gtksourceview.h>
#include <stdlib.h>
#include <string.h>
#include <gdk/gdk.h>


#define VALIDE_TYPE_SPLIT_SOURCE_VIEW (valide_split_source_view_get_type ())
#define VALIDE_SPLIT_SOURCE_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_SPLIT_SOURCE_VIEW, ValideSplitSourceView))
#define VALIDE_SPLIT_SOURCE_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_SPLIT_SOURCE_VIEW, ValideSplitSourceViewClass))
#define VALIDE_IS_SPLIT_SOURCE_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_SPLIT_SOURCE_VIEW))
#define VALIDE_IS_SPLIT_SOURCE_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_SPLIT_SOURCE_VIEW))
#define VALIDE_SPLIT_SOURCE_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_SPLIT_SOURCE_VIEW, ValideSplitSourceViewClass))

typedef struct _ValideSplitSourceView ValideSplitSourceView;
typedef struct _ValideSplitSourceViewClass ValideSplitSourceViewClass;
typedef struct _ValideSplitSourceViewPrivate ValideSplitSourceViewPrivate;

#define VALIDE_TYPE_SOURCE_VIEW (valide_source_view_get_type ())
#define VALIDE_SOURCE_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_SOURCE_VIEW, ValideSourceView))
#define VALIDE_SOURCE_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_SOURCE_VIEW, ValideSourceViewClass))
#define VALIDE_IS_SOURCE_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_SOURCE_VIEW))
#define VALIDE_IS_SOURCE_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_SOURCE_VIEW))
#define VALIDE_SOURCE_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_SOURCE_VIEW, ValideSourceViewClass))

typedef struct _ValideSourceView ValideSourceView;
typedef struct _ValideSourceViewClass ValideSourceViewClass;

#define VALIDE_TYPE_SOURCE_BUFFER (valide_source_buffer_get_type ())
#define VALIDE_SOURCE_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_SOURCE_BUFFER, ValideSourceBuffer))
#define VALIDE_SOURCE_BUFFER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_SOURCE_BUFFER, ValideSourceBufferClass))
#define VALIDE_IS_SOURCE_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_SOURCE_BUFFER))
#define VALIDE_IS_SOURCE_BUFFER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_SOURCE_BUFFER))
#define VALIDE_SOURCE_BUFFER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_SOURCE_BUFFER, ValideSourceBufferClass))

typedef struct _ValideSourceBuffer ValideSourceBuffer;
typedef struct _ValideSourceBufferClass ValideSourceBufferClass;

/**
 * A widget with two Valide.SourceView with the same buffer
 */
struct _ValideSplitSourceView {
	GtkVPaned parent_instance;
	ValideSplitSourceViewPrivate * priv;
};

struct _ValideSplitSourceViewClass {
	GtkVPanedClass parent_class;
};

struct _ValideSplitSourceViewPrivate {
	ValideSourceView* last_active;
	char* _language;
	ValideSourceView* _view1;
	ValideSourceView* _view2;
};


static gpointer valide_split_source_view_parent_class = NULL;

GType valide_split_source_view_get_type (void);
GType valide_source_view_get_type (void);
#define VALIDE_SPLIT_SOURCE_VIEW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_SPLIT_SOURCE_VIEW, ValideSplitSourceViewPrivate))
enum  {
	VALIDE_SPLIT_SOURCE_VIEW_DUMMY_PROPERTY,
	VALIDE_SPLIT_SOURCE_VIEW_RIGHT_MARGIN_POSITION,
	VALIDE_SPLIT_SOURCE_VIEW_SHOW_RIGHT_MARGIN,
	VALIDE_SPLIT_SOURCE_VIEW_SHOW_LINE_NUMBERS,
	VALIDE_SPLIT_SOURCE_VIEW_AUTO_INDENT,
	VALIDE_SPLIT_SOURCE_VIEW_HIGHLIGHT_CURRENT_LINE,
	VALIDE_SPLIT_SOURCE_VIEW_INDENT_ON_TAB,
	VALIDE_SPLIT_SOURCE_VIEW_INSERT_SPACES_INSTEAD_OF_TABS,
	VALIDE_SPLIT_SOURCE_VIEW_TAB_WIDTH,
	VALIDE_SPLIT_SOURCE_VIEW_LANGUAGE,
	VALIDE_SPLIT_SOURCE_VIEW_ACTIVE_VIEW
};
static gboolean valide_split_source_view_focus_in_event_cb (ValideSplitSourceView* self, GtkWidget* sender, const GdkEventFocus* event);
static void valide_split_source_view_set_language (ValideSplitSourceView* self, const char* value);
ValideSplitSourceView* valide_split_source_view_new (const char* language);
ValideSplitSourceView* valide_split_source_view_construct (GType object_type, const char* language);
static ValideSourceView* valide_split_source_view_get_view1 (ValideSplitSourceView* self);
static ValideSourceView* valide_split_source_view_get_view2 (ValideSplitSourceView* self);
void valide_split_source_view_set_editable (ValideSplitSourceView* self, gboolean setting);
guint valide_split_source_view_get_right_margin_position (ValideSplitSourceView* self);
void valide_split_source_view_set_right_margin_position (ValideSplitSourceView* self, guint value);
gboolean valide_split_source_view_get_show_right_margin (ValideSplitSourceView* self);
void valide_split_source_view_set_show_right_margin (ValideSplitSourceView* self, gboolean value);
gboolean valide_split_source_view_get_show_line_numbers (ValideSplitSourceView* self);
void valide_split_source_view_set_show_line_numbers (ValideSplitSourceView* self, gboolean value);
gboolean valide_split_source_view_get_auto_indent (ValideSplitSourceView* self);
void valide_split_source_view_set_auto_indent (ValideSplitSourceView* self, gboolean value);
gboolean valide_split_source_view_get_highlight_current_line (ValideSplitSourceView* self);
void valide_split_source_view_set_highlight_current_line (ValideSplitSourceView* self, gboolean value);
gboolean valide_split_source_view_get_indent_on_tab (ValideSplitSourceView* self);
void valide_split_source_view_set_indent_on_tab (ValideSplitSourceView* self, gboolean value);
gboolean valide_split_source_view_get_insert_spaces_instead_of_tabs (ValideSplitSourceView* self);
void valide_split_source_view_set_insert_spaces_instead_of_tabs (ValideSplitSourceView* self, gboolean value);
guint valide_split_source_view_get_tab_width (ValideSplitSourceView* self);
void valide_split_source_view_set_tab_width (ValideSplitSourceView* self, guint value);
const char* valide_split_source_view_get_language (ValideSplitSourceView* self);
ValideSourceView* valide_split_source_view_get_active_view (ValideSplitSourceView* self);
static void valide_split_source_view_set_view1 (ValideSplitSourceView* self, ValideSourceView* value);
static void valide_split_source_view_set_view2 (ValideSplitSourceView* self, ValideSourceView* value);
ValideSourceView* valide_source_view_new (const char* language);
ValideSourceView* valide_source_view_construct (GType object_type, const char* language);
static gboolean _valide_split_source_view_focus_in_event_cb_gtk_widget_focus_in_event (GtkWidget* _sender, const GdkEventFocus* event, gpointer self);
GType valide_source_buffer_get_type (void);
ValideSourceBuffer* valide_source_view_get_buffer (ValideSourceView* self);
void valide_source_view_set_buffer (ValideSourceView* self, ValideSourceBuffer* value);
static gboolean _lambda61_ (GtkWidget* s, const GdkEventKey* e, ValideSplitSourceView* self);
static gboolean __lambda61__gtk_widget_key_press_event (GtkWidget* _sender, const GdkEventKey* event, gpointer self);
static gboolean _lambda62_ (GtkWidget* s, const GdkEventKey* e, ValideSplitSourceView* self);
static gboolean __lambda62__gtk_widget_key_press_event (GtkWidget* _sender, const GdkEventKey* event, gpointer self);
static gboolean _lambda63_ (GtkWidget* s, const GdkEventFocus* e, ValideSplitSourceView* self);
static gboolean __lambda63__gtk_widget_focus_out_event (GtkWidget* _sender, const GdkEventFocus* event, gpointer self);
static gboolean _lambda64_ (GtkWidget* s, const GdkEventFocus* e, ValideSplitSourceView* self);
static gboolean __lambda64__gtk_widget_focus_out_event (GtkWidget* _sender, const GdkEventFocus* event, gpointer self);
static gboolean _lambda65_ (GtkWidget* s, const GdkEventButton* e, ValideSplitSourceView* self);
static gboolean __lambda65__gtk_widget_button_press_event (GtkWidget* _sender, const GdkEventButton* event, gpointer self);
static gboolean _lambda66_ (GtkWidget* s, const GdkEventButton* e, ValideSplitSourceView* self);
static gboolean __lambda66__gtk_widget_button_press_event (GtkWidget* _sender, const GdkEventButton* event, gpointer self);
static void _lambda67_ (ValideSourceView* s, ValideSplitSourceView* self);
static void __lambda67__valide_source_view_search (ValideSourceView* _sender, gpointer self);
static void _lambda68_ (ValideSourceView* s, ValideSplitSourceView* self);
static void __lambda68__valide_source_view_search (ValideSourceView* _sender, gpointer self);
static void _lambda69_ (ValideSourceView* s, ValideSplitSourceView* self);
static void __lambda69__valide_source_view_find_next (ValideSourceView* _sender, gpointer self);
static void _lambda70_ (ValideSourceView* s, ValideSplitSourceView* self);
static void __lambda70__valide_source_view_find_next (ValideSourceView* _sender, gpointer self);
static void _lambda71_ (ValideSourceView* s, ValideSplitSourceView* self);
static void __lambda71__valide_source_view_find_prev (ValideSourceView* _sender, gpointer self);
static void _lambda72_ (ValideSourceView* s, ValideSplitSourceView* self);
static void __lambda72__valide_source_view_find_prev (ValideSourceView* _sender, gpointer self);
static void _lambda73_ (ValideSourceView* s, ValideSplitSourceView* self);
static void __lambda73__valide_source_view_replace (ValideSourceView* _sender, gpointer self);
static void _lambda74_ (ValideSourceView* s, ValideSplitSourceView* self);
static void __lambda74__valide_source_view_replace (ValideSourceView* _sender, gpointer self);
static GObject * valide_split_source_view_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void valide_split_source_view_finalize (GObject* obj);
static void valide_split_source_view_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_split_source_view_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);



#line 207 "split-source-view.vala"
static gboolean valide_split_source_view_focus_in_event_cb (ValideSplitSourceView* self, GtkWidget* sender, const GdkEventFocus* event) {
#line 171 "split-source-view.c"
	gboolean result;
	GtkWidget* _tmp0_;
#line 207 "split-source-view.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 207 "split-source-view.vala"
	g_return_val_if_fail (sender != NULL, FALSE);
#line 209 "split-source-view.vala"
	_tmp0_ = NULL;
#line 209 "split-source-view.vala"
	self->priv->last_active = (_tmp0_ = sender, VALIDE_IS_SOURCE_VIEW (_tmp0_) ? ((ValideSourceView*) _tmp0_) : NULL);
#line 182 "split-source-view.c"
	result = FALSE;
	return result;
}


/**
   * Create a new Valide.SplitSourceView
   *
   * @param language The language for coloration
   */
#line 218 "split-source-view.vala"
ValideSplitSourceView* valide_split_source_view_construct (GType object_type, const char* language) {
#line 195 "split-source-view.c"
	GParameter * __params;
	GParameter * __params_it;
	ValideSplitSourceView * self;
#line 218 "split-source-view.vala"
	g_return_val_if_fail (language != NULL, NULL);
#line 201 "split-source-view.c"
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
#line 220 "split-source-view.vala"
	__params_it->name = "language";
#line 220 "split-source-view.vala"
	g_value_init (&__params_it->value, G_TYPE_STRING);
#line 220 "split-source-view.vala"
	g_value_set_string (&__params_it->value, language);
#line 220 "split-source-view.vala"
	__params_it++;
#line 212 "split-source-view.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 218 "split-source-view.vala"
ValideSplitSourceView* valide_split_source_view_new (const char* language) {
#line 218 "split-source-view.vala"
	return valide_split_source_view_construct (VALIDE_TYPE_SPLIT_SOURCE_VIEW, language);
#line 227 "split-source-view.c"
}


/**
   * @see Gtk.TreeView.set_editable
   */
#line 268 "split-source-view.vala"
void valide_split_source_view_set_editable (ValideSplitSourceView* self, gboolean setting) {
#line 268 "split-source-view.vala"
	g_return_if_fail (self != NULL);
#line 270 "split-source-view.vala"
	gtk_text_view_set_editable ((GtkTextView*) self->priv->_view1, setting);
#line 271 "split-source-view.vala"
	gtk_text_view_set_editable ((GtkTextView*) self->priv->_view2, setting);
#line 242 "split-source-view.c"
}


guint valide_split_source_view_get_right_margin_position (ValideSplitSourceView* self) {
	guint result;
	g_return_val_if_fail (self != NULL, 0U);
	result = gtk_source_view_get_right_margin_position ((GtkSourceView*) self->priv->_view1);
	return result;
}


void valide_split_source_view_set_right_margin_position (ValideSplitSourceView* self, guint value) {
	g_return_if_fail (self != NULL);
#line 45 "split-source-view.vala"
	gtk_source_view_set_right_margin_position ((GtkSourceView*) self->priv->_view1, value);
#line 46 "split-source-view.vala"
	gtk_source_view_set_right_margin_position ((GtkSourceView*) self->priv->_view2, value);
#line 260 "split-source-view.c"
	g_object_notify ((GObject *) self, "right-margin-position");
}


gboolean valide_split_source_view_get_show_right_margin (ValideSplitSourceView* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = gtk_source_view_get_show_right_margin ((GtkSourceView*) self->priv->_view1);
	return result;
}


void valide_split_source_view_set_show_right_margin (ValideSplitSourceView* self, gboolean value) {
	g_return_if_fail (self != NULL);
#line 61 "split-source-view.vala"
	gtk_source_view_set_show_right_margin ((GtkSourceView*) self->priv->_view1, value);
#line 62 "split-source-view.vala"
	gtk_source_view_set_show_right_margin ((GtkSourceView*) self->priv->_view2, value);
#line 279 "split-source-view.c"
	g_object_notify ((GObject *) self, "show-right-margin");
}


gboolean valide_split_source_view_get_show_line_numbers (ValideSplitSourceView* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = gtk_source_view_get_show_line_numbers ((GtkSourceView*) self->priv->_view1);
	return result;
}


void valide_split_source_view_set_show_line_numbers (ValideSplitSourceView* self, gboolean value) {
	g_return_if_fail (self != NULL);
#line 77 "split-source-view.vala"
	gtk_source_view_set_show_line_numbers ((GtkSourceView*) self->priv->_view1, value);
#line 78 "split-source-view.vala"
	gtk_source_view_set_show_line_numbers ((GtkSourceView*) self->priv->_view2, value);
#line 298 "split-source-view.c"
	g_object_notify ((GObject *) self, "show-line-numbers");
}


gboolean valide_split_source_view_get_auto_indent (ValideSplitSourceView* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = gtk_source_view_get_auto_indent ((GtkSourceView*) self->priv->_view1);
	return result;
}


void valide_split_source_view_set_auto_indent (ValideSplitSourceView* self, gboolean value) {
	g_return_if_fail (self != NULL);
#line 93 "split-source-view.vala"
	gtk_source_view_set_auto_indent ((GtkSourceView*) self->priv->_view1, value);
#line 94 "split-source-view.vala"
	gtk_source_view_set_auto_indent ((GtkSourceView*) self->priv->_view2, value);
#line 317 "split-source-view.c"
	g_object_notify ((GObject *) self, "auto-indent");
}


gboolean valide_split_source_view_get_highlight_current_line (ValideSplitSourceView* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = gtk_source_view_get_highlight_current_line ((GtkSourceView*) self->priv->_view1);
	return result;
}


void valide_split_source_view_set_highlight_current_line (ValideSplitSourceView* self, gboolean value) {
	g_return_if_fail (self != NULL);
#line 109 "split-source-view.vala"
	gtk_source_view_set_highlight_current_line ((GtkSourceView*) self->priv->_view1, value);
#line 110 "split-source-view.vala"
	gtk_source_view_set_highlight_current_line ((GtkSourceView*) self->priv->_view2, value);
#line 336 "split-source-view.c"
	g_object_notify ((GObject *) self, "highlight-current-line");
}


gboolean valide_split_source_view_get_indent_on_tab (ValideSplitSourceView* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = gtk_source_view_get_indent_on_tab ((GtkSourceView*) self->priv->_view1);
	return result;
}


void valide_split_source_view_set_indent_on_tab (ValideSplitSourceView* self, gboolean value) {
	g_return_if_fail (self != NULL);
#line 125 "split-source-view.vala"
	gtk_source_view_set_indent_on_tab ((GtkSourceView*) self->priv->_view1, value);
#line 126 "split-source-view.vala"
	gtk_source_view_set_indent_on_tab ((GtkSourceView*) self->priv->_view2, value);
#line 355 "split-source-view.c"
	g_object_notify ((GObject *) self, "indent-on-tab");
}


gboolean valide_split_source_view_get_insert_spaces_instead_of_tabs (ValideSplitSourceView* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = gtk_source_view_get_insert_spaces_instead_of_tabs ((GtkSourceView*) self->priv->_view1);
	return result;
}


void valide_split_source_view_set_insert_spaces_instead_of_tabs (ValideSplitSourceView* self, gboolean value) {
	g_return_if_fail (self != NULL);
#line 141 "split-source-view.vala"
	gtk_source_view_set_insert_spaces_instead_of_tabs ((GtkSourceView*) self->priv->_view1, value);
#line 142 "split-source-view.vala"
	gtk_source_view_set_insert_spaces_instead_of_tabs ((GtkSourceView*) self->priv->_view2, value);
#line 374 "split-source-view.c"
	g_object_notify ((GObject *) self, "insert-spaces-instead-of-tabs");
}


guint valide_split_source_view_get_tab_width (ValideSplitSourceView* self) {
	guint result;
	g_return_val_if_fail (self != NULL, 0U);
	result = gtk_source_view_get_tab_width ((GtkSourceView*) self->priv->_view1);
	return result;
}


void valide_split_source_view_set_tab_width (ValideSplitSourceView* self, guint value) {
	g_return_if_fail (self != NULL);
#line 157 "split-source-view.vala"
	gtk_source_view_set_tab_width ((GtkSourceView*) self->priv->_view1, value);
#line 158 "split-source-view.vala"
	gtk_source_view_set_tab_width ((GtkSourceView*) self->priv->_view2, value);
#line 393 "split-source-view.c"
	g_object_notify ((GObject *) self, "tab-width");
}


const char* valide_split_source_view_get_language (ValideSplitSourceView* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_language;
	return result;
}


static void valide_split_source_view_set_language (ValideSplitSourceView* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 168 "split-source-view.vala"
	_tmp0_ = NULL;
#line 413 "split-source-view.c"
	self->priv->_language = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_language = (g_free (self->priv->_language), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "language");
}


ValideSourceView* valide_split_source_view_get_active_view (ValideSplitSourceView* self) {
	ValideSourceView* result;
	g_return_val_if_fail (self != NULL, NULL);
#line 178 "split-source-view.vala"
	if (gtk_paned_get_position ((GtkPaned*) self) == 0) {
#line 424 "split-source-view.c"
		result = self->priv->_view2;
		return result;
	} else {
		result = self->priv->last_active;
		return result;
	}
}


static ValideSourceView* valide_split_source_view_get_view1 (ValideSplitSourceView* self) {
	ValideSourceView* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_view1;
	return result;
}


static void valide_split_source_view_set_view1 (ValideSplitSourceView* self, ValideSourceView* value) {
	ValideSourceView* _tmp1_;
	ValideSourceView* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 195 "split-source-view.vala"
	_tmp0_ = NULL;
#line 449 "split-source-view.c"
	self->priv->_view1 = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_view1 == NULL) ? NULL : (self->priv->_view1 = (g_object_unref (self->priv->_view1), NULL)), _tmp1_);
}


static ValideSourceView* valide_split_source_view_get_view2 (ValideSplitSourceView* self) {
	ValideSourceView* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_view2;
	return result;
}


static void valide_split_source_view_set_view2 (ValideSplitSourceView* self, ValideSourceView* value) {
	ValideSourceView* _tmp1_;
	ValideSourceView* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 204 "split-source-view.vala"
	_tmp0_ = NULL;
#line 469 "split-source-view.c"
	self->priv->_view2 = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_view2 == NULL) ? NULL : (self->priv->_view2 = (g_object_unref (self->priv->_view2), NULL)), _tmp1_);
}


#line 207 "split-source-view.vala"
static gboolean _valide_split_source_view_focus_in_event_cb_gtk_widget_focus_in_event (GtkWidget* _sender, const GdkEventFocus* event, gpointer self) {
#line 476 "split-source-view.c"
	return valide_split_source_view_focus_in_event_cb (self, _sender, event);
}


static gboolean _lambda61_ (GtkWidget* s, const GdkEventKey* e, ValideSplitSourceView* self) {
	gboolean result;
	gboolean _tmp0_;
	g_return_val_if_fail (s != NULL, FALSE);
	result = (g_signal_emit_by_name ((GtkWidget*) self, "key-press-event", &(*e), &_tmp0_), _tmp0_);
	return result;
}


static gboolean __lambda61__gtk_widget_key_press_event (GtkWidget* _sender, const GdkEventKey* event, gpointer self) {
	return _lambda61_ (_sender, event, self);
}


static gboolean _lambda62_ (GtkWidget* s, const GdkEventKey* e, ValideSplitSourceView* self) {
	gboolean result;
	gboolean _tmp0_;
	g_return_val_if_fail (s != NULL, FALSE);
	result = (g_signal_emit_by_name ((GtkWidget*) self, "key-press-event", &(*e), &_tmp0_), _tmp0_);
	return result;
}


static gboolean __lambda62__gtk_widget_key_press_event (GtkWidget* _sender, const GdkEventKey* event, gpointer self) {
	return _lambda62_ (_sender, event, self);
}


static gboolean _lambda63_ (GtkWidget* s, const GdkEventFocus* e, ValideSplitSourceView* self) {
	gboolean result;
	gboolean _tmp0_;
	g_return_val_if_fail (s != NULL, FALSE);
	result = (g_signal_emit_by_name ((GtkWidget*) self, "focus-out-event", &(*e), &_tmp0_), _tmp0_);
	return result;
}


static gboolean __lambda63__gtk_widget_focus_out_event (GtkWidget* _sender, const GdkEventFocus* event, gpointer self) {
	return _lambda63_ (_sender, event, self);
}


static gboolean _lambda64_ (GtkWidget* s, const GdkEventFocus* e, ValideSplitSourceView* self) {
	gboolean result;
	gboolean _tmp0_;
	g_return_val_if_fail (s != NULL, FALSE);
	result = (g_signal_emit_by_name ((GtkWidget*) self, "focus-out-event", &(*e), &_tmp0_), _tmp0_);
	return result;
}


static gboolean __lambda64__gtk_widget_focus_out_event (GtkWidget* _sender, const GdkEventFocus* event, gpointer self) {
	return _lambda64_ (_sender, event, self);
}


static gboolean _lambda65_ (GtkWidget* s, const GdkEventButton* e, ValideSplitSourceView* self) {
	gboolean result;
	gboolean _tmp0_;
	g_return_val_if_fail (s != NULL, FALSE);
	result = (g_signal_emit_by_name ((GtkWidget*) self, "button-press-event", &(*e), &_tmp0_), _tmp0_);
	return result;
}


static gboolean __lambda65__gtk_widget_button_press_event (GtkWidget* _sender, const GdkEventButton* event, gpointer self) {
	return _lambda65_ (_sender, event, self);
}


static gboolean _lambda66_ (GtkWidget* s, const GdkEventButton* e, ValideSplitSourceView* self) {
	gboolean result;
	gboolean _tmp0_;
	g_return_val_if_fail (s != NULL, FALSE);
	result = (g_signal_emit_by_name ((GtkWidget*) self, "button-press-event", &(*e), &_tmp0_), _tmp0_);
	return result;
}


static gboolean __lambda66__gtk_widget_button_press_event (GtkWidget* _sender, const GdkEventButton* event, gpointer self) {
	return _lambda66_ (_sender, event, self);
}


static void _lambda67_ (ValideSourceView* s, ValideSplitSourceView* self) {
	g_return_if_fail (s != NULL);
#line 255 "split-source-view.vala"
	g_signal_emit_by_name (self, "search");
#line 569 "split-source-view.c"
}


static void __lambda67__valide_source_view_search (ValideSourceView* _sender, gpointer self) {
	_lambda67_ (_sender, self);
}


static void _lambda68_ (ValideSourceView* s, ValideSplitSourceView* self) {
	g_return_if_fail (s != NULL);
#line 256 "split-source-view.vala"
	g_signal_emit_by_name (self, "search");
#line 582 "split-source-view.c"
}


static void __lambda68__valide_source_view_search (ValideSourceView* _sender, gpointer self) {
	_lambda68_ (_sender, self);
}


static void _lambda69_ (ValideSourceView* s, ValideSplitSourceView* self) {
	g_return_if_fail (s != NULL);
#line 257 "split-source-view.vala"
	g_signal_emit_by_name (self, "find-next");
#line 595 "split-source-view.c"
}


static void __lambda69__valide_source_view_find_next (ValideSourceView* _sender, gpointer self) {
	_lambda69_ (_sender, self);
}


static void _lambda70_ (ValideSourceView* s, ValideSplitSourceView* self) {
	g_return_if_fail (s != NULL);
#line 258 "split-source-view.vala"
	g_signal_emit_by_name (self, "find-next");
#line 608 "split-source-view.c"
}


static void __lambda70__valide_source_view_find_next (ValideSourceView* _sender, gpointer self) {
	_lambda70_ (_sender, self);
}


static void _lambda71_ (ValideSourceView* s, ValideSplitSourceView* self) {
	g_return_if_fail (s != NULL);
#line 259 "split-source-view.vala"
	g_signal_emit_by_name (self, "find-prev");
#line 621 "split-source-view.c"
}


static void __lambda71__valide_source_view_find_prev (ValideSourceView* _sender, gpointer self) {
	_lambda71_ (_sender, self);
}


static void _lambda72_ (ValideSourceView* s, ValideSplitSourceView* self) {
	g_return_if_fail (s != NULL);
#line 260 "split-source-view.vala"
	g_signal_emit_by_name (self, "find-prev");
#line 634 "split-source-view.c"
}


static void __lambda72__valide_source_view_find_prev (ValideSourceView* _sender, gpointer self) {
	_lambda72_ (_sender, self);
}


static void _lambda73_ (ValideSourceView* s, ValideSplitSourceView* self) {
	g_return_if_fail (s != NULL);
#line 261 "split-source-view.vala"
	g_signal_emit_by_name (self, "replace");
#line 647 "split-source-view.c"
}


static void __lambda73__valide_source_view_replace (ValideSourceView* _sender, gpointer self) {
	_lambda73_ (_sender, self);
}


static void _lambda74_ (ValideSourceView* s, ValideSplitSourceView* self) {
	g_return_if_fail (s != NULL);
#line 262 "split-source-view.vala"
	g_signal_emit_by_name (self, "replace");
#line 660 "split-source-view.c"
}


static void __lambda74__valide_source_view_replace (ValideSourceView* _sender, gpointer self) {
	_lambda74_ (_sender, self);
}


static GObject * valide_split_source_view_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValideSplitSourceViewClass * klass;
	GObjectClass * parent_class;
	ValideSplitSourceView * self;
	klass = VALIDE_SPLIT_SOURCE_VIEW_CLASS (g_type_class_peek (VALIDE_TYPE_SPLIT_SOURCE_VIEW));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_SPLIT_SOURCE_VIEW (obj);
	{
		GtkScrolledWindow* scrolled_window;
		ValideSourceView* _tmp0_;
		ValideSourceView* _tmp1_;
		GtkScrolledWindow* _tmp2_;
		GtkScrolledWindow* _tmp3_;
		scrolled_window = NULL;
		_tmp0_ = NULL;
#line 228 "split-source-view.vala"
		valide_split_source_view_set_view1 (self, _tmp0_ = g_object_ref_sink (valide_source_view_new (self->priv->_language)));
#line 688 "split-source-view.c"
		(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
#line 229 "split-source-view.vala"
		g_signal_connect_object ((GtkWidget*) self->priv->_view1, "focus-in-event", (GCallback) _valide_split_source_view_focus_in_event_cb_gtk_widget_focus_in_event, self, 0);
#line 692 "split-source-view.c"
		_tmp1_ = NULL;
#line 230 "split-source-view.vala"
		valide_split_source_view_set_view2 (self, _tmp1_ = g_object_ref_sink (valide_source_view_new (self->priv->_language)));
#line 696 "split-source-view.c"
		(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
#line 231 "split-source-view.vala"
		g_signal_connect_object ((GtkWidget*) self->priv->_view2, "focus-in-event", (GCallback) _valide_split_source_view_focus_in_event_cb_gtk_widget_focus_in_event, self, 0);
#line 233 "split-source-view.vala"
		valide_source_view_set_buffer (self->priv->_view2, valide_source_view_get_buffer (self->priv->_view1));
#line 702 "split-source-view.c"
		_tmp2_ = NULL;
#line 235 "split-source-view.vala"
		scrolled_window = (_tmp2_ = g_object_ref_sink ((GtkScrolledWindow*) gtk_scrolled_window_new (NULL, NULL)), (scrolled_window == NULL) ? NULL : (scrolled_window = (g_object_unref (scrolled_window), NULL)), _tmp2_);
#line 236 "split-source-view.vala"
		gtk_scrolled_window_set_policy (scrolled_window, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
#line 237 "split-source-view.vala"
		gtk_paned_add1 ((GtkPaned*) self, (GtkWidget*) scrolled_window);
#line 238 "split-source-view.vala"
		gtk_container_add ((GtkContainer*) scrolled_window, (GtkWidget*) self->priv->_view1);
#line 712 "split-source-view.c"
		_tmp3_ = NULL;
#line 240 "split-source-view.vala"
		scrolled_window = (_tmp3_ = g_object_ref_sink ((GtkScrolledWindow*) gtk_scrolled_window_new (NULL, NULL)), (scrolled_window == NULL) ? NULL : (scrolled_window = (g_object_unref (scrolled_window), NULL)), _tmp3_);
#line 241 "split-source-view.vala"
		gtk_scrolled_window_set_policy (scrolled_window, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
#line 242 "split-source-view.vala"
		gtk_paned_add2 ((GtkPaned*) self, (GtkWidget*) scrolled_window);
#line 243 "split-source-view.vala"
		gtk_container_add ((GtkContainer*) scrolled_window, (GtkWidget*) self->priv->_view2);
#line 245 "split-source-view.vala"
		gtk_paned_set_position ((GtkPaned*) self, 0);
#line 246 "split-source-view.vala"
		self->priv->last_active = self->priv->_view2;
#line 249 "split-source-view.vala"
		g_signal_connect ((GtkWidget*) self->priv->_view1, "key-press-event", (GCallback) __lambda61__gtk_widget_key_press_event, self);
#line 250 "split-source-view.vala"
		g_signal_connect ((GtkWidget*) self->priv->_view2, "key-press-event", (GCallback) __lambda62__gtk_widget_key_press_event, self);
#line 251 "split-source-view.vala"
		g_signal_connect ((GtkWidget*) self->priv->_view1, "focus-out-event", (GCallback) __lambda63__gtk_widget_focus_out_event, self);
#line 252 "split-source-view.vala"
		g_signal_connect ((GtkWidget*) self->priv->_view2, "focus-out-event", (GCallback) __lambda64__gtk_widget_focus_out_event, self);
#line 253 "split-source-view.vala"
		g_signal_connect ((GtkWidget*) self->priv->_view1, "button-press-event", (GCallback) __lambda65__gtk_widget_button_press_event, self);
#line 254 "split-source-view.vala"
		g_signal_connect ((GtkWidget*) self->priv->_view2, "button-press-event", (GCallback) __lambda66__gtk_widget_button_press_event, self);
#line 255 "split-source-view.vala"
		g_signal_connect (self->priv->_view1, "search", (GCallback) __lambda67__valide_source_view_search, self);
#line 256 "split-source-view.vala"
		g_signal_connect (self->priv->_view2, "search", (GCallback) __lambda68__valide_source_view_search, self);
#line 257 "split-source-view.vala"
		g_signal_connect (self->priv->_view1, "find-next", (GCallback) __lambda69__valide_source_view_find_next, self);
#line 258 "split-source-view.vala"
		g_signal_connect (self->priv->_view2, "find-next", (GCallback) __lambda70__valide_source_view_find_next, self);
#line 259 "split-source-view.vala"
		g_signal_connect (self->priv->_view1, "find-prev", (GCallback) __lambda71__valide_source_view_find_prev, self);
#line 260 "split-source-view.vala"
		g_signal_connect (self->priv->_view2, "find-prev", (GCallback) __lambda72__valide_source_view_find_prev, self);
#line 261 "split-source-view.vala"
		g_signal_connect (self->priv->_view1, "replace", (GCallback) __lambda73__valide_source_view_replace, self);
#line 262 "split-source-view.vala"
		g_signal_connect (self->priv->_view2, "replace", (GCallback) __lambda74__valide_source_view_replace, self);
#line 754 "split-source-view.c"
		(scrolled_window == NULL) ? NULL : (scrolled_window = (g_object_unref (scrolled_window), NULL));
	}
	return obj;
}


static void valide_split_source_view_class_init (ValideSplitSourceViewClass * klass) {
	valide_split_source_view_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideSplitSourceViewPrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_split_source_view_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_split_source_view_set_property;
	G_OBJECT_CLASS (klass)->constructor = valide_split_source_view_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_split_source_view_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_SPLIT_SOURCE_VIEW_RIGHT_MARGIN_POSITION, g_param_spec_uint ("right-margin-position", "right-margin-position", "right-margin-position", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_SPLIT_SOURCE_VIEW_SHOW_RIGHT_MARGIN, g_param_spec_boolean ("show-right-margin", "show-right-margin", "show-right-margin", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_SPLIT_SOURCE_VIEW_SHOW_LINE_NUMBERS, g_param_spec_boolean ("show-line-numbers", "show-line-numbers", "show-line-numbers", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_SPLIT_SOURCE_VIEW_AUTO_INDENT, g_param_spec_boolean ("auto-indent", "auto-indent", "auto-indent", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_SPLIT_SOURCE_VIEW_HIGHLIGHT_CURRENT_LINE, g_param_spec_boolean ("highlight-current-line", "highlight-current-line", "highlight-current-line", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_SPLIT_SOURCE_VIEW_INDENT_ON_TAB, g_param_spec_boolean ("indent-on-tab", "indent-on-tab", "indent-on-tab", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_SPLIT_SOURCE_VIEW_INSERT_SPACES_INSTEAD_OF_TABS, g_param_spec_boolean ("insert-spaces-instead-of-tabs", "insert-spaces-instead-of-tabs", "insert-spaces-instead-of-tabs", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_SPLIT_SOURCE_VIEW_TAB_WIDTH, g_param_spec_uint ("tab-width", "tab-width", "tab-width", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_SPLIT_SOURCE_VIEW_LANGUAGE, g_param_spec_string ("language", "language", "language", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_SPLIT_SOURCE_VIEW_ACTIVE_VIEW, g_param_spec_object ("active-view", "active-view", "active-view", VALIDE_TYPE_SOURCE_VIEW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_signal_new ("search", VALIDE_TYPE_SPLIT_SOURCE_VIEW, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("find_next", VALIDE_TYPE_SPLIT_SOURCE_VIEW, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("find_prev", VALIDE_TYPE_SPLIT_SOURCE_VIEW, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("replace", VALIDE_TYPE_SPLIT_SOURCE_VIEW, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}


static void valide_split_source_view_instance_init (ValideSplitSourceView * self) {
	self->priv = VALIDE_SPLIT_SOURCE_VIEW_GET_PRIVATE (self);
}


static void valide_split_source_view_finalize (GObject* obj) {
	ValideSplitSourceView * self;
	self = VALIDE_SPLIT_SOURCE_VIEW (obj);
	self->priv->_language = (g_free (self->priv->_language), NULL);
	(self->priv->_view1 == NULL) ? NULL : (self->priv->_view1 = (g_object_unref (self->priv->_view1), NULL));
	(self->priv->_view2 == NULL) ? NULL : (self->priv->_view2 = (g_object_unref (self->priv->_view2), NULL));
	G_OBJECT_CLASS (valide_split_source_view_parent_class)->finalize (obj);
}


GType valide_split_source_view_get_type (void) {
	static GType valide_split_source_view_type_id = 0;
	if (valide_split_source_view_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideSplitSourceViewClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_split_source_view_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideSplitSourceView), 0, (GInstanceInitFunc) valide_split_source_view_instance_init, NULL };
		valide_split_source_view_type_id = g_type_register_static (GTK_TYPE_VPANED, "ValideSplitSourceView", &g_define_type_info, 0);
	}
	return valide_split_source_view_type_id;
}


static void valide_split_source_view_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideSplitSourceView * self;
	gpointer boxed;
	self = VALIDE_SPLIT_SOURCE_VIEW (object);
	switch (property_id) {
		case VALIDE_SPLIT_SOURCE_VIEW_RIGHT_MARGIN_POSITION:
		g_value_set_uint (value, valide_split_source_view_get_right_margin_position (self));
		break;
		case VALIDE_SPLIT_SOURCE_VIEW_SHOW_RIGHT_MARGIN:
		g_value_set_boolean (value, valide_split_source_view_get_show_right_margin (self));
		break;
		case VALIDE_SPLIT_SOURCE_VIEW_SHOW_LINE_NUMBERS:
		g_value_set_boolean (value, valide_split_source_view_get_show_line_numbers (self));
		break;
		case VALIDE_SPLIT_SOURCE_VIEW_AUTO_INDENT:
		g_value_set_boolean (value, valide_split_source_view_get_auto_indent (self));
		break;
		case VALIDE_SPLIT_SOURCE_VIEW_HIGHLIGHT_CURRENT_LINE:
		g_value_set_boolean (value, valide_split_source_view_get_highlight_current_line (self));
		break;
		case VALIDE_SPLIT_SOURCE_VIEW_INDENT_ON_TAB:
		g_value_set_boolean (value, valide_split_source_view_get_indent_on_tab (self));
		break;
		case VALIDE_SPLIT_SOURCE_VIEW_INSERT_SPACES_INSTEAD_OF_TABS:
		g_value_set_boolean (value, valide_split_source_view_get_insert_spaces_instead_of_tabs (self));
		break;
		case VALIDE_SPLIT_SOURCE_VIEW_TAB_WIDTH:
		g_value_set_uint (value, valide_split_source_view_get_tab_width (self));
		break;
		case VALIDE_SPLIT_SOURCE_VIEW_LANGUAGE:
		g_value_set_string (value, valide_split_source_view_get_language (self));
		break;
		case VALIDE_SPLIT_SOURCE_VIEW_ACTIVE_VIEW:
		g_value_set_object (value, valide_split_source_view_get_active_view (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_split_source_view_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideSplitSourceView * self;
	self = VALIDE_SPLIT_SOURCE_VIEW (object);
	switch (property_id) {
		case VALIDE_SPLIT_SOURCE_VIEW_RIGHT_MARGIN_POSITION:
		valide_split_source_view_set_right_margin_position (self, g_value_get_uint (value));
		break;
		case VALIDE_SPLIT_SOURCE_VIEW_SHOW_RIGHT_MARGIN:
		valide_split_source_view_set_show_right_margin (self, g_value_get_boolean (value));
		break;
		case VALIDE_SPLIT_SOURCE_VIEW_SHOW_LINE_NUMBERS:
		valide_split_source_view_set_show_line_numbers (self, g_value_get_boolean (value));
		break;
		case VALIDE_SPLIT_SOURCE_VIEW_AUTO_INDENT:
		valide_split_source_view_set_auto_indent (self, g_value_get_boolean (value));
		break;
		case VALIDE_SPLIT_SOURCE_VIEW_HIGHLIGHT_CURRENT_LINE:
		valide_split_source_view_set_highlight_current_line (self, g_value_get_boolean (value));
		break;
		case VALIDE_SPLIT_SOURCE_VIEW_INDENT_ON_TAB:
		valide_split_source_view_set_indent_on_tab (self, g_value_get_boolean (value));
		break;
		case VALIDE_SPLIT_SOURCE_VIEW_INSERT_SPACES_INSTEAD_OF_TABS:
		valide_split_source_view_set_insert_spaces_instead_of_tabs (self, g_value_get_boolean (value));
		break;
		case VALIDE_SPLIT_SOURCE_VIEW_TAB_WIDTH:
		valide_split_source_view_set_tab_width (self, g_value_get_uint (value));
		break;
		case VALIDE_SPLIT_SOURCE_VIEW_LANGUAGE:
		valide_split_source_view_set_language (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}




