/* start-page.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <float.h>
#include <math.h>
#include <gdk/gdk.h>
#include <gio/gio.h>
#include <glib/gi18n-lib.h>
#include <pango/pango.h>
#include <config.h>


#define VALIDE_TYPE_LINK_LABEL (valide_link_label_get_type ())
#define VALIDE_LINK_LABEL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_LINK_LABEL, ValideLinkLabel))
#define VALIDE_LINK_LABEL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_LINK_LABEL, ValideLinkLabelClass))
#define VALIDE_IS_LINK_LABEL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_LINK_LABEL))
#define VALIDE_IS_LINK_LABEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_LINK_LABEL))
#define VALIDE_LINK_LABEL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_LINK_LABEL, ValideLinkLabelClass))

typedef struct _ValideLinkLabel ValideLinkLabel;
typedef struct _ValideLinkLabelClass ValideLinkLabelClass;
typedef struct _ValideLinkLabelPrivate ValideLinkLabelPrivate;

#define VALIDE_TYPE_ITEM (valide_item_get_type ())
typedef struct _ValideItem ValideItem;

#define VALIDE_TYPE_IDOCUMENT (valide_idocument_get_type ())
#define VALIDE_IDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_IDOCUMENT, ValideiDocument))
#define VALIDE_IS_IDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_IDOCUMENT))
#define VALIDE_IDOCUMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), VALIDE_TYPE_IDOCUMENT, ValideiDocumentIface))

typedef struct _ValideiDocument ValideiDocument;
typedef struct _ValideiDocumentIface ValideiDocumentIface;

#define VALIDE_TYPE_UI_MANAGER (valide_ui_manager_get_type ())
#define VALIDE_UI_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_UI_MANAGER, ValideUIManager))
#define VALIDE_UI_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_UI_MANAGER, ValideUIManagerClass))
#define VALIDE_IS_UI_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_UI_MANAGER))
#define VALIDE_IS_UI_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_UI_MANAGER))
#define VALIDE_UI_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_UI_MANAGER, ValideUIManagerClass))

typedef struct _ValideUIManager ValideUIManager;
typedef struct _ValideUIManagerClass ValideUIManagerClass;

#define VALIDE_TYPE_DOCUMENT_STATE (valide_document_state_get_type ())

#define VALIDE_TYPE_START_PAGE (valide_start_page_get_type ())
#define VALIDE_START_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_START_PAGE, ValideStartPage))
#define VALIDE_START_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_START_PAGE, ValideStartPageClass))
#define VALIDE_IS_START_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_START_PAGE))
#define VALIDE_IS_START_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_START_PAGE))
#define VALIDE_START_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_START_PAGE, ValideStartPageClass))

typedef struct _ValideStartPage ValideStartPage;
typedef struct _ValideStartPageClass ValideStartPageClass;
typedef struct _ValideStartPagePrivate ValideStartPagePrivate;

#define VALIDE_TYPE_DOCUMENT_MANAGER (valide_document_manager_get_type ())
#define VALIDE_DOCUMENT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_DOCUMENT_MANAGER, ValideDocumentManager))
#define VALIDE_DOCUMENT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_DOCUMENT_MANAGER, ValideDocumentManagerClass))
#define VALIDE_IS_DOCUMENT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_DOCUMENT_MANAGER))
#define VALIDE_IS_DOCUMENT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_DOCUMENT_MANAGER))
#define VALIDE_DOCUMENT_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_DOCUMENT_MANAGER, ValideDocumentManagerClass))

typedef struct _ValideDocumentManager ValideDocumentManager;
typedef struct _ValideDocumentManagerClass ValideDocumentManagerClass;

#define VALIDE_TYPE_PROJECT_MANAGER (valide_project_manager_get_type ())
#define VALIDE_PROJECT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PROJECT_MANAGER, ValideProjectManager))
#define VALIDE_PROJECT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PROJECT_MANAGER, ValideProjectManagerClass))
#define VALIDE_IS_PROJECT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PROJECT_MANAGER))
#define VALIDE_IS_PROJECT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PROJECT_MANAGER))
#define VALIDE_PROJECT_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PROJECT_MANAGER, ValideProjectManagerClass))

typedef struct _ValideProjectManager ValideProjectManager;
typedef struct _ValideProjectManagerClass ValideProjectManagerClass;

#define VALIDE_TYPE_RECENT_MANAGER_UI (valide_recent_manager_ui_get_type ())
#define VALIDE_RECENT_MANAGER_UI(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_RECENT_MANAGER_UI, ValideRecentManagerUI))
#define VALIDE_RECENT_MANAGER_UI_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_RECENT_MANAGER_UI, ValideRecentManagerUIClass))
#define VALIDE_IS_RECENT_MANAGER_UI(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_RECENT_MANAGER_UI))
#define VALIDE_IS_RECENT_MANAGER_UI_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_RECENT_MANAGER_UI))
#define VALIDE_RECENT_MANAGER_UI_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_RECENT_MANAGER_UI, ValideRecentManagerUIClass))

typedef struct _ValideRecentManagerUI ValideRecentManagerUI;
typedef struct _ValideRecentManagerUIClass ValideRecentManagerUIClass;

/**
 * Private class
 */
struct _ValideLinkLabel {
	GtkEventBox parent_instance;
	ValideLinkLabelPrivate * priv;
};

struct _ValideLinkLabelClass {
	GtkEventBoxClass parent_class;
};

struct _ValideLinkLabelPrivate {
	char* _uri;
	char* _title;
	GdkPixbuf* _pixbuf;
};

struct _ValideItem {
	GdkPixbuf* icon;
	char* uri;
	char* text;
	GCallback callback;
};

typedef enum  {
	VALIDE_DOCUMENT_STATE_NORMAL,
	VALIDE_DOCUMENT_STATE_EXTERNALLY_MODIFIED,
	VALIDE_DOCUMENT_STATE_BAD_ENCODING,
	VALIDE_DOCUMENT_STATE_INFO
} ValideDocumentState;

/**
 * An interface for document
 */
struct _ValideiDocumentIface {
	GTypeInterface parent_iface;
	void (*open) (ValideiDocument* self, const char* path, GError** error);
	void (*save) (ValideiDocument* self, const char* current_folder);
	void (*save_as) (ValideiDocument* self, const char* current_folder);
	void (*rename) (ValideiDocument* self);
	gboolean (*close) (ValideiDocument* self);
	GdkPixbuf* (*get_icon) (ValideiDocument* self);
	char* (*get_tooltips) (ValideiDocument* self);
	void (*setup_ui) (ValideiDocument* self, ValideUIManager* ui_manager);
	ValideDocumentState (*get_state) (ValideiDocument* self);
	void (*set_state) (ValideiDocument* self, ValideDocumentState value);
	gboolean (*get_is_save) (ValideiDocument* self);
	void (*set_is_save) (ValideiDocument* self, gboolean value);
	const char* (*get_path) (ValideiDocument* self);
	void (*set_path) (ValideiDocument* self, const char* value);
	const char* (*get_title) (ValideiDocument* self);
	const char* (*get_filename) (ValideiDocument* self);
};

/**
 * The start page widget
 */
struct _ValideStartPage {
	GtkScrolledWindow parent_instance;
	ValideStartPagePrivate * priv;
};

struct _ValideStartPageClass {
	GtkScrolledWindowClass parent_class;
};

struct _ValideStartPagePrivate {
	GtkVBox* recent_box;
	GtkFrame* file_frame;
	GtkFrame* project_frame;
	ValideDocumentManager* _documents;
	ValideProjectManager* _projects;
	ValideRecentManagerUI* _recents;
};

typedef enum  {
	VALIDE_PROJECT_ERROR_NO_PROJECT,
	VALIDE_PROJECT_ERROR_MAX_OPEN
} ValideProjectError;
#define VALIDE_PROJECT_ERROR valide_project_error_quark ()
typedef void (*ValideStartPageOpenFunc) (GtkLinkButton* sender, void* user_data);

static gpointer valide_link_label_parent_class = NULL;
static gpointer valide_start_page_parent_class = NULL;
static ValideiDocumentIface* valide_start_page_valide_idocument_parent_iface = NULL;

GType valide_link_label_get_type (void);
#define VALIDE_LINK_LABEL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_LINK_LABEL, ValideLinkLabelPrivate))
enum  {
	VALIDE_LINK_LABEL_DUMMY_PROPERTY,
	VALIDE_LINK_LABEL_URI,
	VALIDE_LINK_LABEL_TITLE,
	VALIDE_LINK_LABEL_PIXBUF
};
static void valide_link_label_set_uri (ValideLinkLabel* self, const char* value);
static void valide_link_label_set_title (ValideLinkLabel* self, const char* value);
static void valide_link_label_set_pixbuf (ValideLinkLabel* self, GdkPixbuf* value);
ValideLinkLabel* valide_link_label_new (const char* uri, const char* title, GdkPixbuf* pixbuf);
ValideLinkLabel* valide_link_label_construct (GType object_type, const char* uri, const char* title, GdkPixbuf* pixbuf);
const char* valide_link_label_get_uri (ValideLinkLabel* self);
const char* valide_link_label_get_title (ValideLinkLabel* self);
GdkPixbuf* valide_link_label_get_pixbuf (ValideLinkLabel* self);
static gboolean _lambda41_ (ValideLinkLabel* self);
static gboolean __lambda41__gtk_widget_button_press_event (GtkWidget* _sender, const GdkEventButton* event, gpointer self);
static gboolean _lambda42_ (ValideLinkLabel* self);
static gboolean __lambda42__gtk_widget_enter_notify_event (GtkWidget* _sender, const GdkEventCrossing* event, gpointer self);
static gboolean _lambda43_ (ValideLinkLabel* self);
static gboolean __lambda43__gtk_widget_leave_notify_event (GtkWidget* _sender, const GdkEventCrossing* event, gpointer self);
static GObject * valide_link_label_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void valide_link_label_finalize (GObject* obj);
static void valide_link_label_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_link_label_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType valide_item_get_type (void);
ValideItem* valide_item_dup (const ValideItem* self);
void valide_item_free (ValideItem* self);
void valide_item_copy (const ValideItem* self, ValideItem* dest);
void valide_item_destroy (ValideItem* self);
GType valide_ui_manager_get_type (void);
GType valide_document_state_get_type (void);
GType valide_idocument_get_type (void);
GType valide_start_page_get_type (void);
GType valide_document_manager_get_type (void);
GType valide_project_manager_get_type (void);
GType valide_recent_manager_ui_get_type (void);
#define VALIDE_START_PAGE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_START_PAGE, ValideStartPagePrivate))
enum  {
	VALIDE_START_PAGE_DUMMY_PROPERTY,
	VALIDE_START_PAGE_DOCUMENTS,
	VALIDE_START_PAGE_PROJECTS,
	VALIDE_START_PAGE_RECENTS,
	VALIDE_START_PAGE_STATE,
	VALIDE_START_PAGE_IS_SAVE,
	VALIDE_START_PAGE_PATH,
	VALIDE_START_PAGE_TITLE,
	VALIDE_START_PAGE_FILENAME
};
ValideProjectManager* valide_start_page_get_projects (ValideStartPage* self);
gboolean valide_project_manager_create (ValideProjectManager* self);
gboolean valide_idocument_close (ValideiDocument* self);
static void valide_start_page_create_project (ValideLinkLabel* sender, const char* uri, ValideStartPage* self);
GQuark valide_project_error_quark (void);
gboolean valide_project_manager_open (ValideProjectManager* self, const char* filename, GError** error);
static void valide_start_page_open_project (ValideLinkLabel* sender, const char* uri, ValideStartPage* self);
ValideDocumentManager* valide_start_page_get_documents (ValideStartPage* self);
ValideiDocument* valide_document_manager_create (ValideDocumentManager* self, const char* filename, GError** error);
static void valide_start_page_open_file (ValideLinkLabel* sender, const char* uri, ValideStartPage* self);
static void valide_start_page_open_link (ValideLinkLabel* sender, const char* uri, ValideStartPage* self);
char* valide_utils_replace_home_dir_with_tilde (const char* uri);
static gboolean valide_start_page_label_enter_notify_event (ValideStartPage* self, GtkWidget* sender, const GdkEventCrossing* event);
static gboolean _valide_start_page_label_enter_notify_event_gtk_widget_enter_notify_event (GtkWidget* _sender, const GdkEventCrossing* event, gpointer self);
static GtkFrame* valide_start_page_create_frame (ValideStartPage* self, const char* title, GList* items);
static void _g_list_free_valide_item_free (GList* self);
static GtkFrame* valide_start_page_populate_frame (ValideStartPage* self, GList* infos, const char* title, GCallback callback);
ValideRecentManagerUI* valide_start_page_get_recents (ValideStartPage* self);
GList* valide_recent_manager_ui_get_recent_files (ValideRecentManagerUI* self);
static void valide_start_page_populate_file_frame (ValideStartPage* self);
GList* valide_recent_manager_ui_get_recent_projects (ValideRecentManagerUI* self);
static void valide_start_page_populate_project_frame (ValideStartPage* self);
static void valide_start_page_set_documents (ValideStartPage* self, ValideDocumentManager* value);
static void valide_start_page_set_projects (ValideStartPage* self, ValideProjectManager* value);
static void valide_start_page_set_recents (ValideStartPage* self, ValideRecentManagerUI* value);
ValideStartPage* valide_start_page_new (ValideDocumentManager* documents, ValideProjectManager* projects, ValideRecentManagerUI* recents);
ValideStartPage* valide_start_page_construct (GType object_type, ValideDocumentManager* documents, ValideProjectManager* projects, ValideRecentManagerUI* recents);
static gboolean valide_start_page_real_close (ValideiDocument* base);
GtkAction* valide_ui_manager_get_action (ValideUIManager* self, const char* name);
static void valide_start_page_real_setup_ui (ValideiDocument* base, ValideUIManager* ui_manager);
GdkPixbuf* valide_utils_get_pixbuf_for_stock (const char* stock_id, GtkIconSize size);
static GdkPixbuf* valide_start_page_real_get_icon (ValideiDocument* base);
static char* valide_start_page_real_get_tooltips (ValideiDocument* base);
const char* valide_idocument_get_filename (ValideiDocument* self);
static void _lambda44_ (ValideStartPage* self);
static void __lambda44__valide_recent_manager_ui_update_recent_projects (ValideRecentManagerUI* _sender, gpointer self);
static void _lambda45_ (ValideStartPage* self);
static void __lambda45__valide_recent_manager_ui_update_recent_files (ValideRecentManagerUI* _sender, gpointer self);
static GObject * valide_start_page_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void valide_start_page_finalize (GObject* obj);
static void valide_start_page_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_start_page_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);



#line 52 "start-page.vala"
ValideLinkLabel* valide_link_label_construct (GType object_type, const char* uri, const char* title, GdkPixbuf* pixbuf) {
#line 297 "start-page.c"
	GParameter * __params;
	GParameter * __params_it;
	ValideLinkLabel * self;
#line 52 "start-page.vala"
	g_return_val_if_fail (title != NULL, NULL);
#line 303 "start-page.c"
	__params = g_new0 (GParameter, 3);
	__params_it = __params;
#line 54 "start-page.vala"
	__params_it->name = "uri";
#line 54 "start-page.vala"
	g_value_init (&__params_it->value, G_TYPE_STRING);
#line 54 "start-page.vala"
	g_value_set_string (&__params_it->value, uri);
#line 54 "start-page.vala"
	__params_it++;
#line 55 "start-page.vala"
	__params_it->name = "title";
#line 55 "start-page.vala"
	g_value_init (&__params_it->value, G_TYPE_STRING);
#line 55 "start-page.vala"
	g_value_set_string (&__params_it->value, title);
#line 55 "start-page.vala"
	__params_it++;
#line 56 "start-page.vala"
	__params_it->name = "pixbuf";
#line 56 "start-page.vala"
	g_value_init (&__params_it->value, GDK_TYPE_PIXBUF);
#line 56 "start-page.vala"
	g_value_set_object (&__params_it->value, pixbuf);
#line 56 "start-page.vala"
	__params_it++;
#line 330 "start-page.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 52 "start-page.vala"
ValideLinkLabel* valide_link_label_new (const char* uri, const char* title, GdkPixbuf* pixbuf) {
#line 52 "start-page.vala"
	return valide_link_label_construct (VALIDE_TYPE_LINK_LABEL, uri, title, pixbuf);
#line 345 "start-page.c"
}


const char* valide_link_label_get_uri (ValideLinkLabel* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_uri;
	return result;
}


static void valide_link_label_set_uri (ValideLinkLabel* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 32 "start-page.vala"
	_tmp0_ = NULL;
#line 364 "start-page.c"
	self->priv->_uri = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_uri = (g_free (self->priv->_uri), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "uri");
}


const char* valide_link_label_get_title (ValideLinkLabel* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_title;
	return result;
}


static void valide_link_label_set_title (ValideLinkLabel* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 38 "start-page.vala"
	_tmp0_ = NULL;
#line 385 "start-page.c"
	self->priv->_title = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_title = (g_free (self->priv->_title), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "title");
}


GdkPixbuf* valide_link_label_get_pixbuf (ValideLinkLabel* self) {
	GdkPixbuf* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_pixbuf;
	return result;
}


static void valide_link_label_set_pixbuf (ValideLinkLabel* self, GdkPixbuf* value) {
	GdkPixbuf* _tmp1_;
	GdkPixbuf* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 44 "start-page.vala"
	_tmp0_ = NULL;
#line 406 "start-page.c"
	self->priv->_pixbuf = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_pixbuf == NULL) ? NULL : (self->priv->_pixbuf = (g_object_unref (self->priv->_pixbuf), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "pixbuf");
}


static gboolean _lambda41_ (ValideLinkLabel* self) {
	gboolean result;
#line 87 "start-page.vala"
	g_signal_emit_by_name (self, "open-uri", self->priv->_uri);
#line 416 "start-page.c"
}


static gboolean __lambda41__gtk_widget_button_press_event (GtkWidget* _sender, const GdkEventButton* event, gpointer self) {
	return _lambda41_ (self);
}


static gboolean _lambda42_ (ValideLinkLabel* self) {
	gboolean result;
	GdkCursor* cursor;
	GdkCursor* _tmp0_;
	cursor = NULL;
	_tmp0_ = NULL;
#line 92 "start-page.vala"
	cursor = (_tmp0_ = gdk_cursor_new (GDK_HAND2), (cursor == NULL) ? NULL : (cursor = (gdk_cursor_unref (cursor), NULL)), _tmp0_);
#line 93 "start-page.vala"
	gdk_window_set_cursor (((GtkWidget*) self)->window, cursor);
#line 435 "start-page.c"
	result = FALSE;
	(cursor == NULL) ? NULL : (cursor = (gdk_cursor_unref (cursor), NULL));
	return result;
}


static gboolean __lambda42__gtk_widget_enter_notify_event (GtkWidget* _sender, const GdkEventCrossing* event, gpointer self) {
	return _lambda42_ (self);
}


static gboolean _lambda43_ (ValideLinkLabel* self) {
	gboolean result;
	GdkCursor* cursor;
	GdkCursor* _tmp0_;
	cursor = NULL;
	_tmp0_ = NULL;
#line 99 "start-page.vala"
	cursor = (_tmp0_ = gdk_cursor_new (GDK_ARROW), (cursor == NULL) ? NULL : (cursor = (gdk_cursor_unref (cursor), NULL)), _tmp0_);
#line 100 "start-page.vala"
	gdk_window_set_cursor (((GtkWidget*) self)->window, cursor);
#line 457 "start-page.c"
	result = FALSE;
	(cursor == NULL) ? NULL : (cursor = (gdk_cursor_unref (cursor), NULL));
	return result;
}


static gboolean __lambda43__gtk_widget_leave_notify_event (GtkWidget* _sender, const GdkEventCrossing* event, gpointer self) {
	return _lambda43_ (self);
}


static GObject * valide_link_label_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValideLinkLabelClass * klass;
	GObjectClass * parent_class;
	ValideLinkLabel * self;
	klass = VALIDE_LINK_LABEL_CLASS (g_type_class_peek (VALIDE_TYPE_LINK_LABEL));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_LINK_LABEL (obj);
	{
		char* text;
		GtkHBox* hbox;
		GtkLabel* label;
		GtkHBox* _tmp0_;
		GtkLabel* _tmp4_;
		char* _tmp6_;
		char* _tmp5_;
		text = NULL;
		hbox = NULL;
		label = NULL;
		_tmp0_ = NULL;
#line 65 "start-page.vala"
		hbox = (_tmp0_ = g_object_ref_sink ((GtkHBox*) gtk_hbox_new (FALSE, 0)), (hbox == NULL) ? NULL : (hbox = (g_object_unref (hbox), NULL)), _tmp0_);
#line 66 "start-page.vala"
		gtk_container_add ((GtkContainer*) self, (GtkWidget*) hbox);
#line 68 "start-page.vala"
		if (self->priv->_pixbuf != NULL) {
#line 496 "start-page.c"
			GtkImage* image;
			GtkImage* _tmp1_;
			char* _tmp2_;
			image = NULL;
			_tmp1_ = NULL;
#line 72 "start-page.vala"
			image = (_tmp1_ = g_object_ref_sink ((GtkImage*) gtk_image_new_from_pixbuf (self->priv->_pixbuf)), (image == NULL) ? NULL : (image = (g_object_unref (image), NULL)), _tmp1_);
#line 73 "start-page.vala"
			gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) image, FALSE, FALSE, (guint) 2);
#line 506 "start-page.c"
			_tmp2_ = NULL;
#line 74 "start-page.vala"
			text = (_tmp2_ = g_strdup (""), text = (g_free (text), NULL), _tmp2_);
#line 510 "start-page.c"
			(image == NULL) ? NULL : (image = (g_object_unref (image), NULL));
		} else {
			char* _tmp3_;
			_tmp3_ = NULL;
#line 78 "start-page.vala"
			text = (_tmp3_ = g_strdup ("• "), text = (g_free (text), NULL), _tmp3_);
#line 517 "start-page.c"
		}
		_tmp4_ = NULL;
#line 81 "start-page.vala"
		label = (_tmp4_ = g_object_ref_sink ((GtkLabel*) gtk_label_new ("")), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp4_);
#line 82 "start-page.vala"
		gtk_misc_set_alignment ((GtkMisc*) label, (float) 0, 0.5f);
#line 524 "start-page.c"
		_tmp6_ = NULL;
#line 820 "glib-2.0.vapi"
		_tmp5_ = NULL;
#line 83 "start-page.vala"
		text = (_tmp6_ = g_strconcat (text, _tmp5_ = g_strdup_printf ("<span color=\"blue\"><u>%s</u></span>", self->priv->_title), NULL), text = (g_free (text), NULL), _tmp6_);
#line 530 "start-page.c"
		_tmp5_ = (g_free (_tmp5_), NULL);
#line 84 "start-page.vala"
		gtk_label_set_markup (label, text);
#line 85 "start-page.vala"
		gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) label, FALSE, TRUE, (guint) 0);
#line 86 "start-page.vala"
		g_signal_connect ((GtkWidget*) self, "button-press-event", (GCallback) __lambda41__gtk_widget_button_press_event, self);
#line 89 "start-page.vala"
		g_signal_connect ((GtkWidget*) self, "enter-notify-event", (GCallback) __lambda42__gtk_widget_enter_notify_event, self);
#line 96 "start-page.vala"
		g_signal_connect ((GtkWidget*) self, "leave-notify-event", (GCallback) __lambda43__gtk_widget_leave_notify_event, self);
#line 542 "start-page.c"
		text = (g_free (text), NULL);
		(hbox == NULL) ? NULL : (hbox = (g_object_unref (hbox), NULL));
		(label == NULL) ? NULL : (label = (g_object_unref (label), NULL));
	}
	return obj;
}


static void valide_link_label_class_init (ValideLinkLabelClass * klass) {
	valide_link_label_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideLinkLabelPrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_link_label_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_link_label_set_property;
	G_OBJECT_CLASS (klass)->constructor = valide_link_label_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_link_label_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_LINK_LABEL_URI, g_param_spec_string ("uri", "uri", "uri", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_LINK_LABEL_TITLE, g_param_spec_string ("title", "title", "title", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_LINK_LABEL_PIXBUF, g_param_spec_object ("pixbuf", "pixbuf", "pixbuf", GDK_TYPE_PIXBUF, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_signal_new ("open_uri", VALIDE_TYPE_LINK_LABEL, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__STRING, G_TYPE_NONE, 1, G_TYPE_STRING);
}


static void valide_link_label_instance_init (ValideLinkLabel * self) {
	self->priv = VALIDE_LINK_LABEL_GET_PRIVATE (self);
}


static void valide_link_label_finalize (GObject* obj) {
	ValideLinkLabel * self;
	self = VALIDE_LINK_LABEL (obj);
	self->priv->_uri = (g_free (self->priv->_uri), NULL);
	self->priv->_title = (g_free (self->priv->_title), NULL);
	(self->priv->_pixbuf == NULL) ? NULL : (self->priv->_pixbuf = (g_object_unref (self->priv->_pixbuf), NULL));
	G_OBJECT_CLASS (valide_link_label_parent_class)->finalize (obj);
}


GType valide_link_label_get_type (void) {
	static GType valide_link_label_type_id = 0;
	if (valide_link_label_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideLinkLabelClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_link_label_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideLinkLabel), 0, (GInstanceInitFunc) valide_link_label_instance_init, NULL };
		valide_link_label_type_id = g_type_register_static (GTK_TYPE_EVENT_BOX, "ValideLinkLabel", &g_define_type_info, 0);
	}
	return valide_link_label_type_id;
}


static void valide_link_label_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideLinkLabel * self;
	gpointer boxed;
	self = VALIDE_LINK_LABEL (object);
	switch (property_id) {
		case VALIDE_LINK_LABEL_URI:
		g_value_set_string (value, valide_link_label_get_uri (self));
		break;
		case VALIDE_LINK_LABEL_TITLE:
		g_value_set_string (value, valide_link_label_get_title (self));
		break;
		case VALIDE_LINK_LABEL_PIXBUF:
		g_value_set_object (value, valide_link_label_get_pixbuf (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_link_label_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideLinkLabel * self;
	self = VALIDE_LINK_LABEL (object);
	switch (property_id) {
		case VALIDE_LINK_LABEL_URI:
		valide_link_label_set_uri (self, g_value_get_string (value));
		break;
		case VALIDE_LINK_LABEL_TITLE:
		valide_link_label_set_title (self, g_value_get_string (value));
		break;
		case VALIDE_LINK_LABEL_PIXBUF:
		valide_link_label_set_pixbuf (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


void valide_item_copy (const ValideItem* self, ValideItem* dest) {
	const char* _tmp9_;
	const char* _tmp8_;
	GdkPixbuf* _tmp7_;
#line 110 "start-page.vala"
	_tmp9_ = NULL;
#line 109 "start-page.vala"
	_tmp8_ = NULL;
#line 108 "start-page.vala"
	_tmp7_ = NULL;
#line 641 "start-page.c"
	dest->icon = (_tmp7_ = self->icon, (_tmp7_ == NULL) ? NULL : g_object_ref (_tmp7_));
	dest->uri = (_tmp8_ = self->uri, (_tmp8_ == NULL) ? NULL : g_strdup (_tmp8_));
	dest->text = (_tmp9_ = self->text, (_tmp9_ == NULL) ? NULL : g_strdup (_tmp9_));
	dest->callback = self->callback;
}


void valide_item_destroy (ValideItem* self) {
	(self->icon == NULL) ? NULL : (self->icon = (g_object_unref (self->icon), NULL));
	self->uri = (g_free (self->uri), NULL);
	self->text = (g_free (self->text), NULL);
}


ValideItem* valide_item_dup (const ValideItem* self) {
	ValideItem* dup;
	dup = g_new0 (ValideItem, 1);
	valide_item_copy (self, dup);
	return dup;
}


void valide_item_free (ValideItem* self) {
	valide_item_destroy (self);
	g_free (self);
}


GType valide_item_get_type (void) {
	static GType valide_item_type_id = 0;
	if (valide_item_type_id == 0) {
		valide_item_type_id = g_boxed_type_register_static ("ValideItem", (GBoxedCopyFunc) valide_item_dup, (GBoxedFreeFunc) valide_item_free);
	}
	return valide_item_type_id;
}


#line 219 "start-page.vala"
static void valide_start_page_create_project (ValideLinkLabel* sender, const char* uri, ValideStartPage* self) {
#line 219 "start-page.vala"
	g_return_if_fail (self != NULL);
#line 219 "start-page.vala"
	g_return_if_fail (sender != NULL);
#line 221 "start-page.vala"
	if (valide_project_manager_create (self->priv->_projects)) {
#line 223 "start-page.vala"
		valide_idocument_close ((ValideiDocument*) self);
#line 689 "start-page.c"
	}
}


#line 228 "start-page.vala"
static void valide_start_page_open_project (ValideLinkLabel* sender, const char* uri, ValideStartPage* self) {
#line 696 "start-page.c"
	GError * _inner_error_;
#line 228 "start-page.vala"
	g_return_if_fail (self != NULL);
#line 228 "start-page.vala"
	g_return_if_fail (sender != NULL);
#line 702 "start-page.c"
	_inner_error_ = NULL;
	{
		gboolean _tmp0_;
		_tmp0_ = valide_project_manager_open (self->priv->_projects, uri, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch21_g_error;
			goto __finally21;
		}
#line 232 "start-page.vala"
		if (_tmp0_) {
#line 234 "start-page.vala"
			valide_idocument_close ((ValideiDocument*) self);
#line 715 "start-page.c"
		}
	}
	goto __finally21;
	__catch21_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 239 "start-page.vala"
			g_warning (e->message);
#line 727 "start-page.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally21:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
}


#line 244 "start-page.vala"
static void valide_start_page_open_file (ValideLinkLabel* sender, const char* uri, ValideStartPage* self) {
#line 742 "start-page.c"
	GError * _inner_error_;
#line 244 "start-page.vala"
	g_return_if_fail (self != NULL);
#line 244 "start-page.vala"
	g_return_if_fail (sender != NULL);
#line 748 "start-page.c"
	_inner_error_ = NULL;
#line 246 "start-page.vala"
	valide_idocument_close ((ValideiDocument*) self);
#line 752 "start-page.c"
	{
		ValideiDocument* _tmp0_;
#line 414 "document-manager.vala"
		_tmp0_ = NULL;
#line 249 "start-page.vala"
		_tmp0_ = valide_document_manager_create (self->priv->_documents, uri, &_inner_error_);
#line 759 "start-page.c"
		if (_inner_error_ != NULL) {
			goto __catch22_g_error;
			goto __finally22;
		}
		(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
	}
	goto __finally22;
	__catch22_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 253 "start-page.vala"
			g_warning (e->message);
#line 775 "start-page.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally22:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
}


#line 258 "start-page.vala"
static void valide_start_page_open_link (ValideLinkLabel* sender, const char* uri, ValideStartPage* self) {
#line 790 "start-page.c"
	GError * _inner_error_;
#line 258 "start-page.vala"
	g_return_if_fail (self != NULL);
#line 258 "start-page.vala"
	g_return_if_fail (sender != NULL);
#line 796 "start-page.c"
	_inner_error_ = NULL;
	{
#line 262 "start-page.vala"
		g_app_info_launch_default_for_uri (uri, NULL, &_inner_error_);
#line 801 "start-page.c"
		if (_inner_error_ != NULL) {
			goto __catch23_g_error;
			goto __finally23;
		}
	}
	goto __finally23;
	__catch23_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 266 "start-page.vala"
			g_warning (e->message);
#line 816 "start-page.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally23:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
}


#line 270 "start-page.vala"
static gboolean valide_start_page_label_enter_notify_event (ValideStartPage* self, GtkWidget* sender, const GdkEventCrossing* event) {
#line 831 "start-page.c"
	gboolean result;
	char* status_message;
	ValideLinkLabel* link_label;
	ValideLinkLabel* _tmp2_;
	ValideLinkLabel* _tmp1_;
	GtkWidget* _tmp0_;
#line 270 "start-page.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 270 "start-page.vala"
	g_return_val_if_fail (sender != NULL, FALSE);
#line 842 "start-page.c"
	status_message = NULL;
	link_label = NULL;
	_tmp2_ = NULL;
#line 276 "start-page.vala"
	_tmp1_ = NULL;
#line 276 "start-page.vala"
	_tmp0_ = NULL;
#line 276 "start-page.vala"
	link_label = (_tmp2_ = (_tmp1_ = (_tmp0_ = sender, VALIDE_IS_LINK_LABEL (_tmp0_) ? ((ValideLinkLabel*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (link_label == NULL) ? NULL : (link_label = (g_object_unref (link_label), NULL)), _tmp2_);
#line 277 "start-page.vala"
	if (valide_link_label_get_uri (link_label) != NULL) {
#line 854 "start-page.c"
		char* uri;
		char* _tmp3_;
		char* _tmp4_;
		uri = NULL;
		_tmp3_ = NULL;
#line 281 "start-page.vala"
		uri = (_tmp3_ = valide_utils_replace_home_dir_with_tilde (valide_link_label_get_uri (link_label)), uri = (g_free (uri), NULL), _tmp3_);
#line 862 "start-page.c"
		_tmp4_ = NULL;
#line 282 "start-page.vala"
		status_message = (_tmp4_ = g_strdup_printf (_ ("Open %s"), uri), status_message = (g_free (status_message), NULL), _tmp4_);
#line 866 "start-page.c"
		uri = (g_free (uri), NULL);
	} else {
		char* _tmp5_;
		_tmp5_ = NULL;
#line 286 "start-page.vala"
		status_message = (_tmp5_ = g_strdup (""), status_message = (g_free (status_message), NULL), _tmp5_);
#line 873 "start-page.c"
	}
#line 288 "start-page.vala"
	g_signal_emit_by_name (self, "send-message", status_message);
#line 877 "start-page.c"
	result = FALSE;
	status_message = (g_free (status_message), NULL);
	(link_label == NULL) ? NULL : (link_label = (g_object_unref (link_label), NULL));
	return result;
}


#line 270 "start-page.vala"
static gboolean _valide_start_page_label_enter_notify_event_gtk_widget_enter_notify_event (GtkWidget* _sender, const GdkEventCrossing* event, gpointer self) {
#line 887 "start-page.c"
	return valide_start_page_label_enter_notify_event (self, _sender, event);
}


#line 292 "start-page.vala"
static GtkFrame* valide_start_page_create_frame (ValideStartPage* self, const char* title, GList* items) {
#line 894 "start-page.c"
	GtkFrame* result;
	GtkVBox* vbox;
	GtkFrame* frame;
	GtkLabel* label;
	GtkAlignment* alignment;
	PangoAttrList* attr_list;
	GtkFrame* _tmp0_;
	PangoAttrList* _tmp1_;
	GtkLabel* _tmp4_;
	GtkLabel* _tmp3_;
	GtkWidget* _tmp2_;
	GtkAlignment* _tmp5_;
	GtkVBox* _tmp6_;
#line 292 "start-page.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 292 "start-page.vala"
	g_return_val_if_fail (title != NULL, NULL);
#line 912 "start-page.c"
	vbox = NULL;
	frame = NULL;
	label = NULL;
	alignment = NULL;
	attr_list = NULL;
	_tmp0_ = NULL;
#line 300 "start-page.vala"
	frame = (_tmp0_ = g_object_ref_sink ((GtkFrame*) gtk_frame_new (title)), (frame == NULL) ? NULL : (frame = (g_object_unref (frame), NULL)), _tmp0_);
#line 921 "start-page.c"
	_tmp1_ = NULL;
#line 301 "start-page.vala"
	attr_list = (_tmp1_ = pango_attr_list_new (), (attr_list == NULL) ? NULL : (attr_list = (pango_attr_list_unref (attr_list), NULL)), _tmp1_);
#line 302 "start-page.vala"
	pango_attr_list_insert (attr_list, pango_attribute_copy (pango_attr_weight_new (PANGO_WEIGHT_BOLD)));
#line 927 "start-page.c"
	_tmp4_ = NULL;
#line 303 "start-page.vala"
	_tmp3_ = NULL;
#line 303 "start-page.vala"
	_tmp2_ = NULL;
#line 303 "start-page.vala"
	label = (_tmp4_ = (_tmp3_ = (_tmp2_ = gtk_frame_get_label_widget (frame), GTK_IS_LABEL (_tmp2_) ? ((GtkLabel*) _tmp2_) : NULL), (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp4_);
#line 304 "start-page.vala"
	gtk_label_set_attributes (label, attr_list);
#line 305 "start-page.vala"
	gtk_frame_set_shadow_type (frame, GTK_SHADOW_NONE);
#line 939 "start-page.c"
	_tmp5_ = NULL;
#line 307 "start-page.vala"
	alignment = (_tmp5_ = g_object_ref_sink ((GtkAlignment*) gtk_alignment_new ((float) 0, (float) 0, (float) 0, (float) 0)), (alignment == NULL) ? NULL : (alignment = (g_object_unref (alignment), NULL)), _tmp5_);
#line 308 "start-page.vala"
	g_object_set (alignment, "left-padding", (guint) 12, NULL);
#line 309 "start-page.vala"
	gtk_container_add ((GtkContainer*) frame, (GtkWidget*) alignment);
#line 947 "start-page.c"
	_tmp6_ = NULL;
#line 311 "start-page.vala"
	vbox = (_tmp6_ = g_object_ref_sink ((GtkVBox*) gtk_vbox_new (FALSE, 5)), (vbox == NULL) ? NULL : (vbox = (g_object_unref (vbox), NULL)), _tmp6_);
#line 312 "start-page.vala"
	gtk_container_set_border_width ((GtkContainer*) vbox, (guint) 5);
#line 313 "start-page.vala"
	gtk_container_add ((GtkContainer*) alignment, (GtkWidget*) vbox);
#line 315 "start-page.vala"
	if (items != NULL) {
#line 957 "start-page.c"
		{
			GList* item_collection;
			GList* item_it;
#line 317 "start-page.vala"
			item_collection = items;
#line 963 "start-page.c"
			for (item_it = item_collection; item_it != NULL; item_it = item_it->next) {
				ValideItem _tmp8_ = {0};
				ValideItem item;
#line 317 "start-page.vala"
				item = (valide_item_copy (&(*((ValideItem*) item_it->data)), &_tmp8_), _tmp8_);
#line 969 "start-page.c"
				{
					ValideLinkLabel* link_label;
					ValideLinkLabel* _tmp7_;
					link_label = NULL;
					_tmp7_ = NULL;
#line 321 "start-page.vala"
					link_label = (_tmp7_ = g_object_ref_sink (valide_link_label_new (item.uri, item.text, item.icon)), (link_label == NULL) ? NULL : (link_label = (g_object_unref (link_label), NULL)), _tmp7_);
#line 322 "start-page.vala"
					g_signal_connect_object ((GtkWidget*) link_label, "enter-notify-event", (GCallback) _valide_start_page_label_enter_notify_event_gtk_widget_enter_notify_event, self, 0);
#line 324 "start-page.vala"
					g_signal_connect (link_label, "open_uri", item.callback, self);
#line 325 "start-page.vala"
					gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) link_label, FALSE, FALSE, (guint) 0);
#line 983 "start-page.c"
					valide_item_destroy (&item);
					(link_label == NULL) ? NULL : (link_label = (g_object_unref (link_label), NULL));
				}
			}
		}
	} else {
		GtkLabel* _tmp9_;
		_tmp9_ = NULL;
#line 330 "start-page.vala"
		label = (_tmp9_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("No item"))), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp9_);
#line 331 "start-page.vala"
		gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) label, FALSE, FALSE, (guint) 0);
#line 996 "start-page.c"
	}
	result = frame;
	(vbox == NULL) ? NULL : (vbox = (g_object_unref (vbox), NULL));
	(label == NULL) ? NULL : (label = (g_object_unref (label), NULL));
	(alignment == NULL) ? NULL : (alignment = (g_object_unref (alignment), NULL));
	(attr_list == NULL) ? NULL : (attr_list = (pango_attr_list_unref (attr_list), NULL));
	return result;
}


static void _g_list_free_valide_item_free (GList* self) {
	g_list_foreach (self, (GFunc) valide_item_free, NULL);
	g_list_free (self);
}


#line 336 "start-page.vala"
static GtkFrame* valide_start_page_populate_frame (ValideStartPage* self, GList* infos, const char* title, GCallback callback) {
#line 1015 "start-page.c"
	GtkFrame* result;
	ValideItem item = {0};
	GList* items;
	GList* _tmp0_;
#line 336 "start-page.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 336 "start-page.vala"
	g_return_val_if_fail (title != NULL, NULL);
#line 1024 "start-page.c"
	items = NULL;
	_tmp0_ = NULL;
#line 341 "start-page.vala"
	items = (_tmp0_ = NULL, (items == NULL) ? NULL : (items = (_g_list_free_valide_item_free (items), NULL)), _tmp0_);
#line 1029 "start-page.c"
	{
		GList* info_collection;
		GList* info_it;
#line 342 "start-page.vala"
		info_collection = infos;
#line 1035 "start-page.c"
		for (info_it = info_collection; info_it != NULL; info_it = info_it->next) {
			GtkRecentInfo* _tmp8_;
			GtkRecentInfo* info;
#line 336 "start-page.vala"
			_tmp8_ = NULL;
#line 342 "start-page.vala"
			info = (_tmp8_ = (GtkRecentInfo*) info_it->data, (_tmp8_ == NULL) ? NULL : gtk_recent_info_ref (_tmp8_));
#line 1043 "start-page.c"
			{
				ValideItem _tmp2_ = {0};
				ValideItem _tmp1_ = {0};
				GdkPixbuf* _tmp3_;
				char* _tmp5_;
				const char* _tmp4_;
				char* _tmp6_;
				ValideItem* _tmp7_;
#line 344 "start-page.vala"
				item = (_tmp2_ = (memset (&_tmp1_, 0, sizeof (ValideItem)), _tmp1_), valide_item_destroy (&item), _tmp2_);
#line 1054 "start-page.c"
				_tmp3_ = NULL;
#line 345 "start-page.vala"
				item.icon = (_tmp3_ = NULL, (item.icon == NULL) ? NULL : (item.icon = (g_object_unref (item.icon), NULL)), _tmp3_);
#line 1058 "start-page.c"
				_tmp5_ = NULL;
#line 346 "start-page.vala"
				_tmp4_ = NULL;
#line 346 "start-page.vala"
				item.uri = (_tmp5_ = (_tmp4_ = gtk_recent_info_get_uri (info), (_tmp4_ == NULL) ? NULL : g_strdup (_tmp4_)), item.uri = (g_free (item.uri), NULL), _tmp5_);
#line 1064 "start-page.c"
				_tmp6_ = NULL;
#line 347 "start-page.vala"
				item.text = (_tmp6_ = g_path_get_basename (item.uri), item.text = (g_free (item.text), NULL), _tmp6_);
#line 348 "start-page.vala"
				item.callback = callback;
#line 350 "start-page.vala"
				_tmp7_ = NULL;
#line 350 "start-page.vala"
				items = g_list_append (items, (_tmp7_ = &item, (_tmp7_ == NULL) ? NULL : valide_item_dup (_tmp7_)));
#line 1074 "start-page.c"
				(info == NULL) ? NULL : (info = (gtk_recent_info_unref (info), NULL));
			}
		}
	}
	result = valide_start_page_create_frame (self, title, items);
	valide_item_destroy (&item);
	(items == NULL) ? NULL : (items = (_g_list_free_valide_item_free (items), NULL));
	return result;
}


#line 355 "start-page.vala"
static void valide_start_page_populate_file_frame (ValideStartPage* self) {
#line 1088 "start-page.c"
	GtkFrame* _tmp0_;
#line 355 "start-page.vala"
	g_return_if_fail (self != NULL);
#line 357 "start-page.vala"
	if (self->priv->file_frame != NULL) {
#line 359 "start-page.vala"
		gtk_object_destroy ((GtkObject*) self->priv->file_frame);
#line 1096 "start-page.c"
	}
	_tmp0_ = NULL;
#line 361 "start-page.vala"
	self->priv->file_frame = (_tmp0_ = valide_start_page_populate_frame (self, valide_recent_manager_ui_get_recent_files (self->priv->_recents), _ ("Recent files"), (GCallback) valide_start_page_open_file), (self->priv->file_frame == NULL) ? NULL : (self->priv->file_frame = (g_object_unref (self->priv->file_frame), NULL)), _tmp0_);
#line 364 "start-page.vala"
	gtk_box_pack_start ((GtkBox*) self->priv->recent_box, (GtkWidget*) self->priv->file_frame, FALSE, FALSE, (guint) 0);
#line 365 "start-page.vala"
	gtk_box_reorder_child ((GtkBox*) self->priv->recent_box, (GtkWidget*) self->priv->file_frame, 2);
#line 366 "start-page.vala"
	gtk_widget_show_all ((GtkWidget*) self->priv->file_frame);
#line 1107 "start-page.c"
}


#line 369 "start-page.vala"
static void valide_start_page_populate_project_frame (ValideStartPage* self) {
#line 1113 "start-page.c"
	GtkFrame* _tmp0_;
#line 369 "start-page.vala"
	g_return_if_fail (self != NULL);
#line 371 "start-page.vala"
	if (self->priv->project_frame != NULL) {
#line 373 "start-page.vala"
		gtk_object_destroy ((GtkObject*) self->priv->project_frame);
#line 1121 "start-page.c"
	}
	_tmp0_ = NULL;
#line 375 "start-page.vala"
	self->priv->project_frame = (_tmp0_ = valide_start_page_populate_frame (self, valide_recent_manager_ui_get_recent_projects (self->priv->_recents), _ ("Recent projects"), (GCallback) valide_start_page_open_project), (self->priv->project_frame == NULL) ? NULL : (self->priv->project_frame = (g_object_unref (self->priv->project_frame), NULL)), _tmp0_);
#line 378 "start-page.vala"
	gtk_box_pack_start ((GtkBox*) self->priv->recent_box, (GtkWidget*) self->priv->project_frame, FALSE, FALSE, (guint) 0);
#line 379 "start-page.vala"
	gtk_box_reorder_child ((GtkBox*) self->priv->recent_box, (GtkWidget*) self->priv->project_frame, 0);
#line 380 "start-page.vala"
	gtk_widget_show_all ((GtkWidget*) self->priv->project_frame);
#line 1132 "start-page.c"
}


/**
   * Create a new Valide.StartPage
   *
   * @param documents The document manager
   * @param projects The project manager
   * @param recents The recents files manager
   */
#line 390 "start-page.vala"
ValideStartPage* valide_start_page_construct (GType object_type, ValideDocumentManager* documents, ValideProjectManager* projects, ValideRecentManagerUI* recents) {
#line 1145 "start-page.c"
	GParameter * __params;
	GParameter * __params_it;
	ValideStartPage * self;
#line 390 "start-page.vala"
	g_return_val_if_fail (documents != NULL, NULL);
#line 390 "start-page.vala"
	g_return_val_if_fail (projects != NULL, NULL);
#line 390 "start-page.vala"
	g_return_val_if_fail (recents != NULL, NULL);
#line 1155 "start-page.c"
	__params = g_new0 (GParameter, 3);
	__params_it = __params;
#line 393 "start-page.vala"
	__params_it->name = "documents";
#line 393 "start-page.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_DOCUMENT_MANAGER);
#line 393 "start-page.vala"
	g_value_set_object (&__params_it->value, documents);
#line 393 "start-page.vala"
	__params_it++;
#line 394 "start-page.vala"
	__params_it->name = "projects";
#line 394 "start-page.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_PROJECT_MANAGER);
#line 394 "start-page.vala"
	g_value_set_object (&__params_it->value, projects);
#line 394 "start-page.vala"
	__params_it++;
#line 395 "start-page.vala"
	__params_it->name = "recents";
#line 395 "start-page.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_RECENT_MANAGER_UI);
#line 395 "start-page.vala"
	g_value_set_object (&__params_it->value, recents);
#line 395 "start-page.vala"
	__params_it++;
#line 1182 "start-page.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 390 "start-page.vala"
ValideStartPage* valide_start_page_new (ValideDocumentManager* documents, ValideProjectManager* projects, ValideRecentManagerUI* recents) {
#line 390 "start-page.vala"
	return valide_start_page_construct (VALIDE_TYPE_START_PAGE, documents, projects, recents);
#line 1197 "start-page.c"
}


/**
   * @see Valide.iDocument.close
   */
#line 522 "start-page.vala"
static gboolean valide_start_page_real_close (ValideiDocument* base) {
#line 1206 "start-page.c"
	ValideStartPage * self;
	gboolean result;
	GdkCursor* cursor;
	GdkCursor* _tmp0_;
	self = (ValideStartPage*) base;
	cursor = NULL;
	_tmp0_ = NULL;
#line 526 "start-page.vala"
	cursor = (_tmp0_ = gdk_cursor_new (GDK_ARROW), (cursor == NULL) ? NULL : (cursor = (gdk_cursor_unref (cursor), NULL)), _tmp0_);
#line 527 "start-page.vala"
	gdk_window_set_cursor (((GtkWidget*) self)->window, cursor);
#line 528 "start-page.vala"
	g_signal_emit_by_name ((ValideiDocument*) self, "closed");
#line 1220 "start-page.c"
	result = TRUE;
	(cursor == NULL) ? NULL : (cursor = (gdk_cursor_unref (cursor), NULL));
	return result;
}


/**
   * @see Valide.iDocument.setup_ui
   */
#line 535 "start-page.vala"
static void valide_start_page_real_setup_ui (ValideiDocument* base, ValideUIManager* ui_manager) {
#line 1232 "start-page.c"
	ValideStartPage * self;
	GtkAction* _tmp0_;
	GtkAction* _tmp1_;
	GtkAction* _tmp2_;
	GtkAction* _tmp3_;
	GtkAction* _tmp4_;
	GtkAction* _tmp5_;
	GtkAction* _tmp6_;
	GtkAction* _tmp7_;
	GtkAction* _tmp8_;
	GtkAction* _tmp9_;
	GtkAction* _tmp10_;
	GtkAction* _tmp11_;
	GtkAction* _tmp12_;
	GtkAction* _tmp13_;
	GtkAction* _tmp14_;
	self = (ValideStartPage*) base;
#line 535 "start-page.vala"
	g_return_if_fail (ui_manager != NULL);
#line 128 "uimanager.vala"
	_tmp0_ = NULL;
#line 537 "start-page.vala"
	gtk_action_set_sensitive (_tmp0_ = valide_ui_manager_get_action (ui_manager, "file-save"), FALSE);
#line 1256 "start-page.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
#line 128 "uimanager.vala"
	_tmp1_ = NULL;
#line 538 "start-page.vala"
	gtk_action_set_sensitive (_tmp1_ = valide_ui_manager_get_action (ui_manager, "file-save-as"), FALSE);
#line 1262 "start-page.c"
	(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
#line 128 "uimanager.vala"
	_tmp2_ = NULL;
#line 539 "start-page.vala"
	gtk_action_set_sensitive (_tmp2_ = valide_ui_manager_get_action (ui_manager, "file-rename"), FALSE);
#line 1268 "start-page.c"
	(_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL));
#line 128 "uimanager.vala"
	_tmp3_ = NULL;
#line 540 "start-page.vala"
	gtk_action_set_sensitive (_tmp3_ = valide_ui_manager_get_action (ui_manager, "edit-undo"), FALSE);
#line 1274 "start-page.c"
	(_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL));
#line 128 "uimanager.vala"
	_tmp4_ = NULL;
#line 541 "start-page.vala"
	gtk_action_set_sensitive (_tmp4_ = valide_ui_manager_get_action (ui_manager, "edit-redo"), FALSE);
#line 1280 "start-page.c"
	(_tmp4_ == NULL) ? NULL : (_tmp4_ = (g_object_unref (_tmp4_), NULL));
#line 128 "uimanager.vala"
	_tmp5_ = NULL;
#line 542 "start-page.vala"
	gtk_action_set_sensitive (_tmp5_ = valide_ui_manager_get_action (ui_manager, "edit-copy"), FALSE);
#line 1286 "start-page.c"
	(_tmp5_ == NULL) ? NULL : (_tmp5_ = (g_object_unref (_tmp5_), NULL));
#line 128 "uimanager.vala"
	_tmp6_ = NULL;
#line 543 "start-page.vala"
	gtk_action_set_sensitive (_tmp6_ = valide_ui_manager_get_action (ui_manager, "edit-cut"), FALSE);
#line 1292 "start-page.c"
	(_tmp6_ == NULL) ? NULL : (_tmp6_ = (g_object_unref (_tmp6_), NULL));
#line 128 "uimanager.vala"
	_tmp7_ = NULL;
#line 544 "start-page.vala"
	gtk_action_set_sensitive (_tmp7_ = valide_ui_manager_get_action (ui_manager, "edit-paste"), FALSE);
#line 1298 "start-page.c"
	(_tmp7_ == NULL) ? NULL : (_tmp7_ = (g_object_unref (_tmp7_), NULL));
#line 128 "uimanager.vala"
	_tmp8_ = NULL;
#line 545 "start-page.vala"
	gtk_action_set_sensitive (_tmp8_ = valide_ui_manager_get_action (ui_manager, "edit-lower"), FALSE);
#line 1304 "start-page.c"
	(_tmp8_ == NULL) ? NULL : (_tmp8_ = (g_object_unref (_tmp8_), NULL));
#line 128 "uimanager.vala"
	_tmp9_ = NULL;
#line 546 "start-page.vala"
	gtk_action_set_sensitive (_tmp9_ = valide_ui_manager_get_action (ui_manager, "edit-upper"), FALSE);
#line 1310 "start-page.c"
	(_tmp9_ == NULL) ? NULL : (_tmp9_ = (g_object_unref (_tmp9_), NULL));
#line 128 "uimanager.vala"
	_tmp10_ = NULL;
#line 547 "start-page.vala"
	gtk_action_set_sensitive (_tmp10_ = valide_ui_manager_get_action (ui_manager, "search-find"), FALSE);
#line 1316 "start-page.c"
	(_tmp10_ == NULL) ? NULL : (_tmp10_ = (g_object_unref (_tmp10_), NULL));
#line 128 "uimanager.vala"
	_tmp11_ = NULL;
#line 548 "start-page.vala"
	gtk_action_set_sensitive (_tmp11_ = valide_ui_manager_get_action (ui_manager, "search-find-next"), FALSE);
#line 1322 "start-page.c"
	(_tmp11_ == NULL) ? NULL : (_tmp11_ = (g_object_unref (_tmp11_), NULL));
#line 128 "uimanager.vala"
	_tmp12_ = NULL;
#line 549 "start-page.vala"
	gtk_action_set_sensitive (_tmp12_ = valide_ui_manager_get_action (ui_manager, "search-find-prev"), FALSE);
#line 1328 "start-page.c"
	(_tmp12_ == NULL) ? NULL : (_tmp12_ = (g_object_unref (_tmp12_), NULL));
#line 128 "uimanager.vala"
	_tmp13_ = NULL;
#line 550 "start-page.vala"
	gtk_action_set_sensitive (_tmp13_ = valide_ui_manager_get_action (ui_manager, "search-replace"), FALSE);
#line 1334 "start-page.c"
	(_tmp13_ == NULL) ? NULL : (_tmp13_ = (g_object_unref (_tmp13_), NULL));
#line 128 "uimanager.vala"
	_tmp14_ = NULL;
#line 551 "start-page.vala"
	gtk_action_set_sensitive (_tmp14_ = valide_ui_manager_get_action (ui_manager, "search-goto-line"), FALSE);
#line 1340 "start-page.c"
	(_tmp14_ == NULL) ? NULL : (_tmp14_ = (g_object_unref (_tmp14_), NULL));
}


/**
   * @see Valide.iDocument.get_icon
   */
#line 557 "start-page.vala"
static GdkPixbuf* valide_start_page_real_get_icon (ValideiDocument* base) {
#line 1350 "start-page.c"
	ValideStartPage * self;
	GdkPixbuf* result;
	self = (ValideStartPage*) base;
	result = valide_utils_get_pixbuf_for_stock (GTK_STOCK_INFO, GTK_ICON_SIZE_MENU);
	return result;
}


/**
   * @see Valide.iDocument.get_tooltips
   */
#line 565 "start-page.vala"
static char* valide_start_page_real_get_tooltips (ValideiDocument* base) {
#line 1364 "start-page.c"
	ValideStartPage * self;
	char* result;
	const char* _tmp0_;
	self = (ValideStartPage*) base;
#line 567 "start-page.vala"
	_tmp0_ = NULL;
#line 1371 "start-page.c"
	result = (_tmp0_ = _ ("Start here"), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
	return result;
}


ValideDocumentManager* valide_start_page_get_documents (ValideStartPage* self) {
	ValideDocumentManager* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_documents;
	return result;
}


static void valide_start_page_set_documents (ValideStartPage* self, ValideDocumentManager* value) {
	ValideDocumentManager* _tmp1_;
	ValideDocumentManager* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 133 "start-page.vala"
	_tmp0_ = NULL;
#line 1392 "start-page.c"
	self->priv->_documents = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_documents == NULL) ? NULL : (self->priv->_documents = (g_object_unref (self->priv->_documents), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "documents");
}


ValideProjectManager* valide_start_page_get_projects (ValideStartPage* self) {
	ValideProjectManager* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_projects;
	return result;
}


static void valide_start_page_set_projects (ValideStartPage* self, ValideProjectManager* value) {
	ValideProjectManager* _tmp1_;
	ValideProjectManager* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 142 "start-page.vala"
	_tmp0_ = NULL;
#line 1413 "start-page.c"
	self->priv->_projects = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_projects == NULL) ? NULL : (self->priv->_projects = (g_object_unref (self->priv->_projects), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "projects");
}


ValideRecentManagerUI* valide_start_page_get_recents (ValideStartPage* self) {
	ValideRecentManagerUI* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_recents;
	return result;
}


static void valide_start_page_set_recents (ValideStartPage* self, ValideRecentManagerUI* value) {
	ValideRecentManagerUI* _tmp1_;
	ValideRecentManagerUI* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 151 "start-page.vala"
	_tmp0_ = NULL;
#line 1434 "start-page.c"
	self->priv->_recents = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_recents == NULL) ? NULL : (self->priv->_recents = (g_object_unref (self->priv->_recents), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "recents");
}


static ValideDocumentState valide_start_page_real_get_state (ValideiDocument* base) {
	ValideDocumentState result;
	ValideStartPage* self;
	self = (ValideStartPage*) base;
	result = VALIDE_DOCUMENT_STATE_INFO;
	return result;
}


static void valide_start_page_real_set_state (ValideiDocument* base, ValideDocumentState value) {
	ValideStartPage* self;
	self = (ValideStartPage*) base;
	g_object_notify ((GObject *) self, "state");
}


static gboolean valide_start_page_real_get_is_save (ValideiDocument* base) {
	gboolean result;
	ValideStartPage* self;
	self = (ValideStartPage*) base;
	result = TRUE;
	return result;
}


static void valide_start_page_real_set_is_save (ValideiDocument* base, gboolean value) {
	ValideStartPage* self;
	self = (ValideStartPage*) base;
	g_object_notify ((GObject *) self, "is-save");
}


static const char* valide_start_page_real_get_path (ValideiDocument* base) {
	const char* result;
	ValideStartPage* self;
	self = (ValideStartPage*) base;
	result = "";
	return result;
}


static void valide_start_page_real_set_path (ValideiDocument* base, const char* value) {
	ValideStartPage* self;
	self = (ValideStartPage*) base;
	g_object_notify ((GObject *) self, "path");
}


static const char* valide_start_page_real_get_title (ValideiDocument* base) {
	const char* result;
	ValideStartPage* self;
	self = (ValideStartPage*) base;
	result = valide_idocument_get_filename ((ValideiDocument*) self);
	return result;
}


static const char* valide_start_page_real_get_filename (ValideiDocument* base) {
	const char* result;
	ValideStartPage* self;
	self = (ValideStartPage*) base;
	result = _ ("Start page");
	return result;
}


static void _lambda44_ (ValideStartPage* self) {
#line 502 "start-page.vala"
	valide_start_page_populate_project_frame (self);
#line 1509 "start-page.c"
}


static void __lambda44__valide_recent_manager_ui_update_recent_projects (ValideRecentManagerUI* _sender, gpointer self) {
	_lambda44_ (self);
}


static void _lambda45_ (ValideStartPage* self) {
#line 506 "start-page.vala"
	valide_start_page_populate_file_frame (self);
#line 1521 "start-page.c"
}


static void __lambda45__valide_recent_manager_ui_update_recent_files (ValideRecentManagerUI* _sender, gpointer self) {
	_lambda45_ (self);
}


static GObject * valide_start_page_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValideStartPageClass * klass;
	GObjectClass * parent_class;
	ValideStartPage * self;
	GError * _inner_error_;
	klass = VALIDE_START_PAGE_CLASS (g_type_class_peek (VALIDE_TYPE_START_PAGE));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_START_PAGE (obj);
	_inner_error_ = NULL;
	{
		GtkVBox* vbox;
		GtkLabel* label;
		GtkTable* table;
		GtkFrame* frame;
		GList* items;
		GtkHSeparator* sep;
		GtkTable* _tmp10_;
		GtkVBox* _tmp18_;
		GList* _tmp19_;
		ValideItem _tmp20_ = {0};
		ValideItem item;
		GdkPixbuf* _tmp21_;
		char* _tmp22_;
		char* _tmp24_;
		const char* _tmp23_;
		ValideItem* _tmp25_;
		ValideItem _tmp27_ = {0};
		ValideItem _tmp26_ = {0};
		GdkPixbuf* _tmp28_;
		char* _tmp29_;
		char* _tmp31_;
		const char* _tmp30_;
		ValideItem* _tmp32_;
		GtkFrame* _tmp33_;
		GtkHSeparator* _tmp34_;
		GList* _tmp35_;
		ValideItem _tmp37_ = {0};
		ValideItem _tmp36_ = {0};
		GdkPixbuf* _tmp38_;
		char* _tmp39_;
		char* _tmp41_;
		const char* _tmp40_;
		ValideItem* _tmp42_;
		ValideItem _tmp44_ = {0};
		ValideItem _tmp43_ = {0};
		GdkPixbuf* _tmp45_;
		char* _tmp46_;
		char* _tmp48_;
		const char* _tmp47_;
		ValideItem* _tmp49_;
		ValideItem _tmp51_ = {0};
		ValideItem _tmp50_ = {0};
		GdkPixbuf* _tmp52_;
		char* _tmp53_;
		char* _tmp55_;
		const char* _tmp54_;
		ValideItem* _tmp56_;
		GtkFrame* _tmp57_;
		GtkVBox* _tmp58_;
		GtkHSeparator* _tmp59_;
		GtkHSeparator* _tmp60_;
		GtkLabel* _tmp61_;
		vbox = NULL;
		label = NULL;
		table = NULL;
		frame = NULL;
		items = NULL;
		sep = NULL;
#line 407 "start-page.vala"
		gtk_scrolled_window_set_policy ((GtkScrolledWindow*) self, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
#line 1602 "start-page.c"
		_tmp10_ = NULL;
#line 409 "start-page.vala"
		table = (_tmp10_ = g_object_ref_sink ((GtkTable*) gtk_table_new ((guint) 7, (guint) 2, FALSE)), (table == NULL) ? NULL : (table = (g_object_unref (table), NULL)), _tmp10_);
#line 410 "start-page.vala"
		gtk_scrolled_window_add_with_viewport ((GtkScrolledWindow*) self, (GtkWidget*) table);
#line 1608 "start-page.c"
		{
			GtkImage* logo;
			GdkPixbuf* pixbuf;
			char* _tmp11_;
			GdkPixbuf* _tmp12_;
			GdkPixbuf* _tmp13_;
			GdkPixbuf* _tmp14_;
			GtkImage* _tmp16_;
			logo = NULL;
			pixbuf = NULL;
#line 2034 "glib-2.0.vapi"
			_tmp11_ = NULL;
#line 417 "start-page.vala"
			_tmp12_ = NULL;
#line 1623 "start-page.c"
			_tmp13_ = (_tmp12_ = gdk_pixbuf_new_from_file (_tmp11_ = g_build_filename (PIXMAPS_DIR, "logo.png", NULL), &_inner_error_), _tmp11_ = (g_free (_tmp11_), NULL), _tmp12_);
			if (_inner_error_ != NULL) {
				(logo == NULL) ? NULL : (logo = (g_object_unref (logo), NULL));
				(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
				goto __catch24_g_error;
				goto __finally24;
			}
			_tmp14_ = NULL;
#line 417 "start-page.vala"
			pixbuf = (_tmp14_ = _tmp13_, (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp14_);
#line 418 "start-page.vala"
			if (DEBUG) {
#line 1636 "start-page.c"
				GdkPixbuf* _tmp15_;
				_tmp15_ = NULL;
#line 420 "start-page.vala"
				pixbuf = (_tmp15_ = gdk_pixbuf_flip (pixbuf, FALSE), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp15_);
#line 1641 "start-page.c"
			}
			_tmp16_ = NULL;
#line 423 "start-page.vala"
			logo = (_tmp16_ = g_object_ref_sink ((GtkImage*) gtk_image_new_from_pixbuf (pixbuf)), (logo == NULL) ? NULL : (logo = (g_object_unref (logo), NULL)), _tmp16_);
#line 424 "start-page.vala"
			gtk_table_attach (table, (GtkWidget*) logo, (guint) 0, (guint) 2, (guint) 0, (guint) 1, GTK_EXPAND, GTK_EXPAND, (guint) 0, (guint) 0);
#line 1648 "start-page.c"
			(logo == NULL) ? NULL : (logo = (g_object_unref (logo), NULL));
			(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
		}
		goto __finally24;
		__catch24_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
				GtkLabel* _tmp17_;
				_tmp17_ = NULL;
#line 429 "start-page.vala"
				label = (_tmp17_ = g_object_ref_sink ((GtkLabel*) gtk_label_new ("Val(a)IDE")), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp17_);
#line 430 "start-page.vala"
				gtk_table_attach (table, (GtkWidget*) label, (guint) 0, (guint) 2, (guint) 0, (guint) 1, GTK_EXPAND, GTK_EXPAND, (guint) 0, (guint) 0);
#line 1665 "start-page.c"
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally24:
		if (_inner_error_ != NULL) {
			(vbox == NULL) ? NULL : (vbox = (g_object_unref (vbox), NULL));
			(label == NULL) ? NULL : (label = (g_object_unref (label), NULL));
			(table == NULL) ? NULL : (table = (g_object_unref (table), NULL));
			(frame == NULL) ? NULL : (frame = (g_object_unref (frame), NULL));
			(items == NULL) ? NULL : (items = (_g_list_free_valide_item_free (items), NULL));
			(sep == NULL) ? NULL : (sep = (g_object_unref (sep), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
		}
		_tmp18_ = NULL;
#line 434 "start-page.vala"
		vbox = (_tmp18_ = g_object_ref_sink ((GtkVBox*) gtk_vbox_new (FALSE, 5)), (vbox == NULL) ? NULL : (vbox = (g_object_unref (vbox), NULL)), _tmp18_);
#line 435 "start-page.vala"
		gtk_container_set_border_width ((GtkContainer*) vbox, (guint) 5);
#line 436 "start-page.vala"
		gtk_table_attach (table, (GtkWidget*) vbox, (guint) 0, (guint) 1, (guint) 1, (guint) 5, GTK_EXPAND, GTK_EXPAND, (guint) 0, (guint) 0);
#line 1687 "start-page.c"
		_tmp19_ = NULL;
#line 439 "start-page.vala"
		items = (_tmp19_ = NULL, (items == NULL) ? NULL : (items = (_g_list_free_valide_item_free (items), NULL)), _tmp19_);
#line 1691 "start-page.c"
		/* New project */
		item = (memset (&_tmp20_, 0, sizeof (ValideItem)), _tmp20_);
		_tmp21_ = NULL;
#line 443 "start-page.vala"
		item.icon = (_tmp21_ = valide_utils_get_pixbuf_for_stock (GTK_STOCK_NEW, GTK_ICON_SIZE_BUTTON), (item.icon == NULL) ? NULL : (item.icon = (g_object_unref (item.icon), NULL)), _tmp21_);
#line 1697 "start-page.c"
		_tmp22_ = NULL;
#line 444 "start-page.vala"
		item.uri = (_tmp22_ = NULL, item.uri = (g_free (item.uri), NULL), _tmp22_);
#line 1701 "start-page.c"
		_tmp24_ = NULL;
#line 445 "start-page.vala"
		_tmp23_ = NULL;
#line 445 "start-page.vala"
		item.text = (_tmp24_ = (_tmp23_ = _ ("New project"), (_tmp23_ == NULL) ? NULL : g_strdup (_tmp23_)), item.text = (g_free (item.text), NULL), _tmp24_);
#line 446 "start-page.vala"
		item.callback = (GCallback) valide_start_page_create_project;
#line 447 "start-page.vala"
		_tmp25_ = NULL;
#line 447 "start-page.vala"
		items = g_list_append (items, (_tmp25_ = &item, (_tmp25_ == NULL) ? NULL : valide_item_dup (_tmp25_)));
#line 1713 "start-page.c"
		/* Open project */
#line 450 "start-page.vala"
		item = (_tmp27_ = (memset (&_tmp26_, 0, sizeof (ValideItem)), _tmp26_), valide_item_destroy (&item), _tmp27_);
#line 1717 "start-page.c"
		_tmp28_ = NULL;
#line 451 "start-page.vala"
		item.icon = (_tmp28_ = valide_utils_get_pixbuf_for_stock (GTK_STOCK_OPEN, GTK_ICON_SIZE_BUTTON), (item.icon == NULL) ? NULL : (item.icon = (g_object_unref (item.icon), NULL)), _tmp28_);
#line 1721 "start-page.c"
		_tmp29_ = NULL;
#line 452 "start-page.vala"
		item.uri = (_tmp29_ = NULL, item.uri = (g_free (item.uri), NULL), _tmp29_);
#line 1725 "start-page.c"
		_tmp31_ = NULL;
#line 453 "start-page.vala"
		_tmp30_ = NULL;
#line 453 "start-page.vala"
		item.text = (_tmp31_ = (_tmp30_ = _ ("Open project"), (_tmp30_ == NULL) ? NULL : g_strdup (_tmp30_)), item.text = (g_free (item.text), NULL), _tmp31_);
#line 454 "start-page.vala"
		item.callback = (GCallback) valide_start_page_open_project;
#line 455 "start-page.vala"
		_tmp32_ = NULL;
#line 455 "start-page.vala"
		items = g_list_append (items, (_tmp32_ = &item, (_tmp32_ == NULL) ? NULL : valide_item_dup (_tmp32_)));
#line 1737 "start-page.c"
		_tmp33_ = NULL;
#line 457 "start-page.vala"
		frame = (_tmp33_ = valide_start_page_create_frame (self, "", items), (frame == NULL) ? NULL : (frame = (g_object_unref (frame), NULL)), _tmp33_);
#line 458 "start-page.vala"
		gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) frame, TRUE, TRUE, (guint) 10);
#line 1743 "start-page.c"
		_tmp34_ = NULL;
#line 460 "start-page.vala"
		gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) (_tmp34_ = g_object_ref_sink ((GtkHSeparator*) gtk_hseparator_new ())), FALSE, FALSE, (guint) 10);
#line 1747 "start-page.c"
		(_tmp34_ == NULL) ? NULL : (_tmp34_ = (g_object_unref (_tmp34_), NULL));
		_tmp35_ = NULL;
#line 462 "start-page.vala"
		items = (_tmp35_ = NULL, (items == NULL) ? NULL : (items = (_g_list_free_valide_item_free (items), NULL)), _tmp35_);
#line 1752 "start-page.c"
		/* Website */
#line 465 "start-page.vala"
		item = (_tmp37_ = (memset (&_tmp36_, 0, sizeof (ValideItem)), _tmp36_), valide_item_destroy (&item), _tmp37_);
#line 1756 "start-page.c"
		_tmp38_ = NULL;
#line 466 "start-page.vala"
		item.icon = (_tmp38_ = valide_utils_get_pixbuf_for_stock (GTK_STOCK_HOME, GTK_ICON_SIZE_BUTTON), (item.icon == NULL) ? NULL : (item.icon = (g_object_unref (item.icon), NULL)), _tmp38_);
#line 1760 "start-page.c"
		_tmp39_ = NULL;
#line 467 "start-page.vala"
		item.uri = (_tmp39_ = g_strdup (WEBSITE), item.uri = (g_free (item.uri), NULL), _tmp39_);
#line 1764 "start-page.c"
		_tmp41_ = NULL;
#line 468 "start-page.vala"
		_tmp40_ = NULL;
#line 468 "start-page.vala"
		item.text = (_tmp41_ = (_tmp40_ = _ ("Website"), (_tmp40_ == NULL) ? NULL : g_strdup (_tmp40_)), item.text = (g_free (item.text), NULL), _tmp41_);
#line 469 "start-page.vala"
		item.callback = (GCallback) valide_start_page_open_link;
#line 470 "start-page.vala"
		_tmp42_ = NULL;
#line 470 "start-page.vala"
		items = g_list_append (items, (_tmp42_ = &item, (_tmp42_ == NULL) ? NULL : valide_item_dup (_tmp42_)));
#line 1776 "start-page.c"
		/* Report a bug */
#line 473 "start-page.vala"
		item = (_tmp44_ = (memset (&_tmp43_, 0, sizeof (ValideItem)), _tmp43_), valide_item_destroy (&item), _tmp44_);
#line 1780 "start-page.c"
		_tmp45_ = NULL;
#line 474 "start-page.vala"
		item.icon = (_tmp45_ = valide_utils_get_pixbuf_for_stock (GTK_STOCK_CANCEL, GTK_ICON_SIZE_BUTTON), (item.icon == NULL) ? NULL : (item.icon = (g_object_unref (item.icon), NULL)), _tmp45_);
#line 1784 "start-page.c"
		_tmp46_ = NULL;
#line 475 "start-page.vala"
		item.uri = (_tmp46_ = g_strdup ("http://code.google.com/p/valide/issues/entry"), item.uri = (g_free (item.uri), NULL), _tmp46_);
#line 1788 "start-page.c"
		_tmp48_ = NULL;
#line 476 "start-page.vala"
		_tmp47_ = NULL;
#line 476 "start-page.vala"
		item.text = (_tmp48_ = (_tmp47_ = _ ("Report a bug"), (_tmp47_ == NULL) ? NULL : g_strdup (_tmp47_)), item.text = (g_free (item.text), NULL), _tmp48_);
#line 477 "start-page.vala"
		item.callback = (GCallback) valide_start_page_open_link;
#line 478 "start-page.vala"
		_tmp49_ = NULL;
#line 478 "start-page.vala"
		items = g_list_append (items, (_tmp49_ = &item, (_tmp49_ == NULL) ? NULL : valide_item_dup (_tmp49_)));
#line 1800 "start-page.c"
		/* Visit the user group */
#line 482 "start-page.vala"
		item = (_tmp51_ = (memset (&_tmp50_, 0, sizeof (ValideItem)), _tmp50_), valide_item_destroy (&item), _tmp51_);
#line 1804 "start-page.c"
		_tmp52_ = NULL;
#line 483 "start-page.vala"
		item.icon = (_tmp52_ = valide_utils_get_pixbuf_for_stock (GTK_STOCK_HELP, GTK_ICON_SIZE_BUTTON), (item.icon == NULL) ? NULL : (item.icon = (g_object_unref (item.icon), NULL)), _tmp52_);
#line 1808 "start-page.c"
		_tmp53_ = NULL;
#line 484 "start-page.vala"
		item.uri = (_tmp53_ = g_strdup ("http://groups.google.com/group/valide-users"), item.uri = (g_free (item.uri), NULL), _tmp53_);
#line 1812 "start-page.c"
		_tmp55_ = NULL;
#line 485 "start-page.vala"
		_tmp54_ = NULL;
#line 485 "start-page.vala"
		item.text = (_tmp55_ = (_tmp54_ = _ ("Visit the user group"), (_tmp54_ == NULL) ? NULL : g_strdup (_tmp54_)), item.text = (g_free (item.text), NULL), _tmp55_);
#line 486 "start-page.vala"
		item.callback = (GCallback) valide_start_page_open_link;
#line 487 "start-page.vala"
		_tmp56_ = NULL;
#line 487 "start-page.vala"
		items = g_list_append (items, (_tmp56_ = &item, (_tmp56_ == NULL) ? NULL : valide_item_dup (_tmp56_)));
#line 1824 "start-page.c"
		_tmp57_ = NULL;
#line 489 "start-page.vala"
		frame = (_tmp57_ = valide_start_page_create_frame (self, "", items), (frame == NULL) ? NULL : (frame = (g_object_unref (frame), NULL)), _tmp57_);
#line 490 "start-page.vala"
		gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) frame, TRUE, TRUE, (guint) 10);
#line 1830 "start-page.c"
		_tmp58_ = NULL;
#line 492 "start-page.vala"
		self->priv->recent_box = (_tmp58_ = g_object_ref_sink ((GtkVBox*) gtk_vbox_new (FALSE, 5)), (self->priv->recent_box == NULL) ? NULL : (self->priv->recent_box = (g_object_unref (self->priv->recent_box), NULL)), _tmp58_);
#line 493 "start-page.vala"
		gtk_container_set_border_width ((GtkContainer*) self->priv->recent_box, (guint) 5);
#line 494 "start-page.vala"
		gtk_table_attach (table, (GtkWidget*) self->priv->recent_box, (guint) 1, (guint) 2, (guint) 1, (guint) 5, GTK_EXPAND, GTK_EXPAND, (guint) 0, (guint) 0);
#line 497 "start-page.vala"
		valide_start_page_populate_project_frame (self);
#line 1840 "start-page.c"
		_tmp59_ = NULL;
#line 498 "start-page.vala"
		gtk_box_pack_start ((GtkBox*) self->priv->recent_box, (GtkWidget*) (_tmp59_ = g_object_ref_sink ((GtkHSeparator*) gtk_hseparator_new ())), FALSE, FALSE, (guint) 10);
#line 1844 "start-page.c"
		(_tmp59_ == NULL) ? NULL : (_tmp59_ = (g_object_unref (_tmp59_), NULL));
#line 499 "start-page.vala"
		valide_start_page_populate_file_frame (self);
#line 501 "start-page.vala"
		g_signal_connect (self->priv->_recents, "update-recent-projects", (GCallback) __lambda44__valide_recent_manager_ui_update_recent_projects, self);
#line 505 "start-page.vala"
		g_signal_connect (self->priv->_recents, "update-recent-files", (GCallback) __lambda45__valide_recent_manager_ui_update_recent_files, self);
#line 1852 "start-page.c"
		_tmp60_ = NULL;
#line 509 "start-page.vala"
		sep = (_tmp60_ = g_object_ref_sink ((GtkHSeparator*) gtk_hseparator_new ()), (sep == NULL) ? NULL : (sep = (g_object_unref (sep), NULL)), _tmp60_);
#line 510 "start-page.vala"
		gtk_table_attach (table, (GtkWidget*) sep, (guint) 0, (guint) 2, (guint) 5, (guint) 6, GTK_FILL, GTK_FILL, (guint) 0, (guint) 5);
#line 1858 "start-page.c"
		_tmp61_ = NULL;
#line 513 "start-page.vala"
		label = (_tmp61_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (COPYRIGHT)), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp61_);
#line 514 "start-page.vala"
		gtk_table_attach (table, (GtkWidget*) label, (guint) 0, (guint) 2, (guint) 6, (guint) 7, GTK_FILL, GTK_FILL, (guint) 0, (guint) 0);
#line 1864 "start-page.c"
		(vbox == NULL) ? NULL : (vbox = (g_object_unref (vbox), NULL));
		(label == NULL) ? NULL : (label = (g_object_unref (label), NULL));
		(table == NULL) ? NULL : (table = (g_object_unref (table), NULL));
		(frame == NULL) ? NULL : (frame = (g_object_unref (frame), NULL));
		(items == NULL) ? NULL : (items = (_g_list_free_valide_item_free (items), NULL));
		(sep == NULL) ? NULL : (sep = (g_object_unref (sep), NULL));
		valide_item_destroy (&item);
	}
	return obj;
}


static void valide_start_page_class_init (ValideStartPageClass * klass) {
	valide_start_page_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideStartPagePrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_start_page_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_start_page_set_property;
	G_OBJECT_CLASS (klass)->constructor = valide_start_page_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_start_page_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_START_PAGE_DOCUMENTS, g_param_spec_object ("documents", "documents", "documents", VALIDE_TYPE_DOCUMENT_MANAGER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_START_PAGE_PROJECTS, g_param_spec_object ("projects", "projects", "projects", VALIDE_TYPE_PROJECT_MANAGER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_START_PAGE_RECENTS, g_param_spec_object ("recents", "recents", "recents", VALIDE_TYPE_RECENT_MANAGER_UI, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_override_property (G_OBJECT_CLASS (klass), VALIDE_START_PAGE_STATE, "state");
	g_object_class_override_property (G_OBJECT_CLASS (klass), VALIDE_START_PAGE_IS_SAVE, "is-save");
	g_object_class_override_property (G_OBJECT_CLASS (klass), VALIDE_START_PAGE_PATH, "path");
	g_object_class_override_property (G_OBJECT_CLASS (klass), VALIDE_START_PAGE_TITLE, "title");
	g_object_class_override_property (G_OBJECT_CLASS (klass), VALIDE_START_PAGE_FILENAME, "filename");
	g_signal_new ("send_message", VALIDE_TYPE_START_PAGE, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__STRING, G_TYPE_NONE, 1, G_TYPE_STRING);
}


static void valide_start_page_valide_idocument_interface_init (ValideiDocumentIface * iface) {
	valide_start_page_valide_idocument_parent_iface = g_type_interface_peek_parent (iface);
	iface->close = valide_start_page_real_close;
	iface->setup_ui = valide_start_page_real_setup_ui;
	iface->get_icon = valide_start_page_real_get_icon;
	iface->get_tooltips = valide_start_page_real_get_tooltips;
	iface->get_state = valide_start_page_real_get_state;
	iface->set_state = valide_start_page_real_set_state;
	iface->get_is_save = valide_start_page_real_get_is_save;
	iface->set_is_save = valide_start_page_real_set_is_save;
	iface->get_path = valide_start_page_real_get_path;
	iface->set_path = valide_start_page_real_set_path;
	iface->get_title = valide_start_page_real_get_title;
	iface->get_filename = valide_start_page_real_get_filename;
}


static void valide_start_page_instance_init (ValideStartPage * self) {
	self->priv = VALIDE_START_PAGE_GET_PRIVATE (self);
}


static void valide_start_page_finalize (GObject* obj) {
	ValideStartPage * self;
	self = VALIDE_START_PAGE (obj);
	(self->priv->recent_box == NULL) ? NULL : (self->priv->recent_box = (g_object_unref (self->priv->recent_box), NULL));
	(self->priv->file_frame == NULL) ? NULL : (self->priv->file_frame = (g_object_unref (self->priv->file_frame), NULL));
	(self->priv->project_frame == NULL) ? NULL : (self->priv->project_frame = (g_object_unref (self->priv->project_frame), NULL));
	(self->priv->_documents == NULL) ? NULL : (self->priv->_documents = (g_object_unref (self->priv->_documents), NULL));
	(self->priv->_projects == NULL) ? NULL : (self->priv->_projects = (g_object_unref (self->priv->_projects), NULL));
	(self->priv->_recents == NULL) ? NULL : (self->priv->_recents = (g_object_unref (self->priv->_recents), NULL));
	G_OBJECT_CLASS (valide_start_page_parent_class)->finalize (obj);
}


GType valide_start_page_get_type (void) {
	static GType valide_start_page_type_id = 0;
	if (valide_start_page_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideStartPageClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_start_page_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideStartPage), 0, (GInstanceInitFunc) valide_start_page_instance_init, NULL };
		static const GInterfaceInfo valide_idocument_info = { (GInterfaceInitFunc) valide_start_page_valide_idocument_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		valide_start_page_type_id = g_type_register_static (GTK_TYPE_SCROLLED_WINDOW, "ValideStartPage", &g_define_type_info, 0);
		g_type_add_interface_static (valide_start_page_type_id, VALIDE_TYPE_IDOCUMENT, &valide_idocument_info);
	}
	return valide_start_page_type_id;
}


static void valide_start_page_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideStartPage * self;
	gpointer boxed;
	self = VALIDE_START_PAGE (object);
	switch (property_id) {
		case VALIDE_START_PAGE_DOCUMENTS:
		g_value_set_object (value, valide_start_page_get_documents (self));
		break;
		case VALIDE_START_PAGE_PROJECTS:
		g_value_set_object (value, valide_start_page_get_projects (self));
		break;
		case VALIDE_START_PAGE_RECENTS:
		g_value_set_object (value, valide_start_page_get_recents (self));
		break;
		case VALIDE_START_PAGE_STATE:
		g_value_set_enum (value, valide_idocument_get_state ((ValideiDocument*) self));
		break;
		case VALIDE_START_PAGE_IS_SAVE:
		g_value_set_boolean (value, valide_idocument_get_is_save ((ValideiDocument*) self));
		break;
		case VALIDE_START_PAGE_PATH:
		g_value_set_string (value, valide_idocument_get_path ((ValideiDocument*) self));
		break;
		case VALIDE_START_PAGE_TITLE:
		g_value_set_string (value, valide_idocument_get_title ((ValideiDocument*) self));
		break;
		case VALIDE_START_PAGE_FILENAME:
		g_value_set_string (value, valide_idocument_get_filename ((ValideiDocument*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_start_page_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideStartPage * self;
	self = VALIDE_START_PAGE (object);
	switch (property_id) {
		case VALIDE_START_PAGE_DOCUMENTS:
		valide_start_page_set_documents (self, g_value_get_object (value));
		break;
		case VALIDE_START_PAGE_PROJECTS:
		valide_start_page_set_projects (self, g_value_get_object (value));
		break;
		case VALIDE_START_PAGE_RECENTS:
		valide_start_page_set_recents (self, g_value_get_object (value));
		break;
		case VALIDE_START_PAGE_STATE:
		valide_idocument_set_state ((ValideiDocument*) self, g_value_get_enum (value));
		break;
		case VALIDE_START_PAGE_IS_SAVE:
		valide_idocument_set_is_save ((ValideiDocument*) self, g_value_get_boolean (value));
		break;
		case VALIDE_START_PAGE_PATH:
		valide_idocument_set_path ((ValideiDocument*) self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}




