/* aboutdialog.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <glib/gi18n-lib.h>


#define VALIDE_TYPE_STATUSBAR (valide_statusbar_get_type ())
#define VALIDE_STATUSBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_STATUSBAR, ValideStatusbar))
#define VALIDE_STATUSBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_STATUSBAR, ValideStatusbarClass))
#define VALIDE_IS_STATUSBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_STATUSBAR))
#define VALIDE_IS_STATUSBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_STATUSBAR))
#define VALIDE_STATUSBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_STATUSBAR, ValideStatusbarClass))

typedef struct _ValideStatusbar ValideStatusbar;
typedef struct _ValideStatusbarClass ValideStatusbarClass;
typedef struct _ValideStatusbarPrivate ValideStatusbarPrivate;

/*
 * This code is re-writing in the vala language of GeditStatusbar
 *
 * Copyright (C) 2006-2008  Jürg Billeter, Raffaele Sandrini
 * Copyright (C) 2007  Mathias Hasselmann
 
*
 * The status bar
 */
struct _ValideStatusbar {
	GtkStatusbar parent_instance;
	ValideStatusbarPrivate * priv;
};

struct _ValideStatusbarClass {
	GtkStatusbarClass parent_class;
};

struct _ValideStatusbarPrivate {
	GtkStatusbar* cursor_position_statusbar;
	guint flash_context_id;
	guint flash_message_id;
	guint flash_timeout;
};


static gpointer valide_statusbar_parent_class = NULL;

GType valide_statusbar_get_type (void);
#define VALIDE_STATUSBAR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_STATUSBAR, ValideStatusbarPrivate))
enum  {
	VALIDE_STATUSBAR_DUMMY_PROPERTY
};
static gboolean valide_statusbar_remove_message_timeout (ValideStatusbar* self);
void valide_statusbar_set_has_resize_grip (ValideStatusbar* self, gboolean show);
void valide_statusbar_set_cursor_position (ValideStatusbar* self, gint line, gint col);
static gboolean _valide_statusbar_remove_message_timeout_gsource_func (gpointer self);
void valide_statusbar_flash_message (ValideStatusbar* self, guint context_id, const char* msg);
ValideStatusbar* valide_statusbar_new (void);
ValideStatusbar* valide_statusbar_construct (GType object_type);
static GObject * valide_statusbar_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void valide_statusbar_finalize (GObject* obj);



#line 39 "statusbar.vala"
static gboolean valide_statusbar_remove_message_timeout (ValideStatusbar* self) {
#line 89 "statusbar.c"
	gboolean result;
#line 39 "statusbar.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 41 "statusbar.vala"
	gtk_statusbar_remove ((GtkStatusbar*) self, self->priv->flash_context_id, self->priv->flash_message_id);
#line 95 "statusbar.c"
	/* remove the timeout */
#line 44 "statusbar.vala"
	self->priv->flash_timeout = (guint) 0;
#line 99 "statusbar.c"
	result = FALSE;
	return result;
}


/**
   * Sets if a resize grip showld be shown.
   *
   * @param show If the resize grip is shown
   */
#line 64 "statusbar.vala"
void valide_statusbar_set_has_resize_grip (ValideStatusbar* self, gboolean show) {
#line 64 "statusbar.vala"
	g_return_if_fail (self != NULL);
#line 66 "statusbar.vala"
	gtk_statusbar_set_has_resize_grip (self->priv->cursor_position_statusbar, show);
#line 116 "statusbar.c"
}


/**
   * Sets the cursor position on the statusbar.
   *
   * @param line Line position
   * @param col Column position
   */
#line 75 "statusbar.vala"
void valide_statusbar_set_cursor_position (ValideStatusbar* self, gint line, gint col) {
#line 128 "statusbar.c"
	char* msg;
	gboolean _tmp0_;
#line 75 "statusbar.vala"
	g_return_if_fail (self != NULL);
#line 133 "statusbar.c"
	msg = NULL;
#line 79 "statusbar.vala"
	gtk_statusbar_pop (self->priv->cursor_position_statusbar, (guint) 0);
#line 137 "statusbar.c"
	_tmp0_ = FALSE;
#line 81 "statusbar.vala"
	if (line != (-1)) {
#line 81 "statusbar.vala"
		_tmp0_ = TRUE;
#line 143 "statusbar.c"
	} else {
#line 81 "statusbar.vala"
		_tmp0_ = col != (-1);
#line 147 "statusbar.c"
	}
#line 81 "statusbar.vala"
	if (_tmp0_) {
#line 151 "statusbar.c"
		char* _tmp1_;
		/* Translators: "Ln" is an abbreviation for "Line", Col is an abbreviation for "Column".
		    Please, use abbreviations if possible to avoid space problems. */
		_tmp1_ = NULL;
#line 85 "statusbar.vala"
		msg = (_tmp1_ = g_strdup_printf (_ ("  Ln %d, Col %d"), line, col), msg = (g_free (msg), NULL), _tmp1_);
#line 87 "statusbar.vala"
		gtk_statusbar_push (self->priv->cursor_position_statusbar, (guint) 0, msg);
#line 160 "statusbar.c"
	}
	msg = (g_free (msg), NULL);
}


#line 39 "statusbar.vala"
static gboolean _valide_statusbar_remove_message_timeout_gsource_func (gpointer self) {
#line 168 "statusbar.c"
	return valide_statusbar_remove_message_timeout (self);
}


/**
   * Flash a temporary message on the statusbar.
   *
   * @param context_id Message context_id
   * @param msg Message to flash on the statusbar
   */
#line 97 "statusbar.vala"
void valide_statusbar_flash_message (ValideStatusbar* self, guint context_id, const char* msg) {
#line 181 "statusbar.c"
	guint32 flash_length;
#line 97 "statusbar.vala"
	g_return_if_fail (self != NULL);
#line 97 "statusbar.vala"
	g_return_if_fail (msg != NULL);
#line 187 "statusbar.c"
	flash_length = (guint32) 3000;
	/* three seconds 
	 remove a currently ongoing flash message */
#line 102 "statusbar.vala"
	if (self->priv->flash_timeout > 0) {
#line 104 "statusbar.vala"
		g_source_remove (self->priv->flash_timeout);
#line 105 "statusbar.vala"
		self->priv->flash_timeout = (guint) 0;
#line 107 "statusbar.vala"
		gtk_statusbar_remove ((GtkStatusbar*) self, self->priv->flash_context_id, self->priv->flash_message_id);
#line 199 "statusbar.c"
	}
#line 110 "statusbar.vala"
	self->priv->flash_context_id = context_id;
#line 111 "statusbar.vala"
	self->priv->flash_message_id = gtk_statusbar_push ((GtkStatusbar*) self, context_id, msg);
#line 113 "statusbar.vala"
	self->priv->flash_timeout = g_timeout_add ((guint) flash_length, _valide_statusbar_remove_message_timeout_gsource_func, self);
#line 207 "statusbar.c"
}


/*
 * This code is re-writing in the vala language of GeditStatusbar
 *
 * Copyright (C) 2006-2008  Jürg Billeter, Raffaele Sandrini
 * Copyright (C) 2007  Mathias Hasselmann
 
*
 * The status bar
 */
#line 32 "statusbar.vala"
ValideStatusbar* valide_statusbar_construct (GType object_type) {
#line 222 "statusbar.c"
	ValideStatusbar * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


#line 32 "statusbar.vala"
ValideStatusbar* valide_statusbar_new (void) {
#line 32 "statusbar.vala"
	return valide_statusbar_construct (VALIDE_TYPE_STATUSBAR);
#line 233 "statusbar.c"
}


static GObject * valide_statusbar_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValideStatusbarClass * klass;
	GObjectClass * parent_class;
	ValideStatusbar * self;
	klass = VALIDE_STATUSBAR_CLASS (g_type_class_peek (VALIDE_TYPE_STATUSBAR));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_STATUSBAR (obj);
	{
		GtkStatusbar* _tmp0_;
#line 50 "statusbar.vala"
		gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (self), FALSE);
#line 250 "statusbar.c"
		_tmp0_ = NULL;
#line 52 "statusbar.vala"
		self->priv->cursor_position_statusbar = (_tmp0_ = g_object_ref_sink ((GtkStatusbar*) gtk_statusbar_new ()), (self->priv->cursor_position_statusbar == NULL) ? NULL : (self->priv->cursor_position_statusbar = (g_object_unref (self->priv->cursor_position_statusbar), NULL)), _tmp0_);
#line 53 "statusbar.vala"
		gtk_widget_show ((GtkWidget*) self->priv->cursor_position_statusbar);
#line 54 "statusbar.vala"
		gtk_widget_set_size_request ((GtkWidget*) self->priv->cursor_position_statusbar, 160, 10);
#line 55 "statusbar.vala"
		gtk_statusbar_set_has_resize_grip (self->priv->cursor_position_statusbar, FALSE);
#line 56 "statusbar.vala"
		gtk_box_pack_end ((GtkBox*) self, (GtkWidget*) self->priv->cursor_position_statusbar, FALSE, TRUE, (guint) 0);
#line 262 "statusbar.c"
	}
	return obj;
}


static void valide_statusbar_class_init (ValideStatusbarClass * klass) {
	valide_statusbar_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideStatusbarPrivate));
	G_OBJECT_CLASS (klass)->constructor = valide_statusbar_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_statusbar_finalize;
}


static void valide_statusbar_instance_init (ValideStatusbar * self) {
	self->priv = VALIDE_STATUSBAR_GET_PRIVATE (self);
}


static void valide_statusbar_finalize (GObject* obj) {
	ValideStatusbar * self;
	self = VALIDE_STATUSBAR (obj);
	(self->priv->cursor_position_statusbar == NULL) ? NULL : (self->priv->cursor_position_statusbar = (g_object_unref (self->priv->cursor_position_statusbar), NULL));
	G_OBJECT_CLASS (valide_statusbar_parent_class)->finalize (obj);
}


GType valide_statusbar_get_type (void) {
	static GType valide_statusbar_type_id = 0;
	if (valide_statusbar_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideStatusbarClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_statusbar_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideStatusbar), 0, (GInstanceInitFunc) valide_statusbar_instance_init, NULL };
		valide_statusbar_type_id = g_type_register_static (GTK_TYPE_STATUSBAR, "ValideStatusbar", &g_define_type_info, 0);
	}
	return valide_statusbar_type_id;
}




