/* template-manager.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <config.h>
#include <glib/gstdio.h>


#define VALIDE_TYPE_TEMPLATE_MANAGER (valide_template_manager_get_type ())
#define VALIDE_TEMPLATE_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_TEMPLATE_MANAGER, ValideTemplateManager))
#define VALIDE_TEMPLATE_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_TEMPLATE_MANAGER, ValideTemplateManagerClass))
#define VALIDE_IS_TEMPLATE_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_TEMPLATE_MANAGER))
#define VALIDE_IS_TEMPLATE_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_TEMPLATE_MANAGER))
#define VALIDE_TEMPLATE_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_TEMPLATE_MANAGER, ValideTemplateManagerClass))

typedef struct _ValideTemplateManager ValideTemplateManager;
typedef struct _ValideTemplateManagerClass ValideTemplateManagerClass;
typedef struct _ValideTemplateManagerPrivate ValideTemplateManagerPrivate;

#define VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER (valide_abstract_config_manager_get_type ())
#define VALIDE_ABSTRACT_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManager))
#define VALIDE_ABSTRACT_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManagerClass))
#define VALIDE_IS_ABSTRACT_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER))
#define VALIDE_IS_ABSTRACT_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER))
#define VALIDE_ABSTRACT_CONFIG_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManagerClass))

typedef struct _ValideAbstractConfigManager ValideAbstractConfigManager;
typedef struct _ValideAbstractConfigManagerClass ValideAbstractConfigManagerClass;

#define VALIDE_TYPE_CONFIG_MANAGER (valide_config_manager_get_type ())
#define VALIDE_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_CONFIG_MANAGER, ValideConfigManager))
#define VALIDE_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_CONFIG_MANAGER, ValideConfigManagerClass))
#define VALIDE_IS_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_CONFIG_MANAGER))
#define VALIDE_IS_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_CONFIG_MANAGER))
#define VALIDE_CONFIG_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_CONFIG_MANAGER, ValideConfigManagerClass))

typedef struct _ValideConfigManager ValideConfigManager;
typedef struct _ValideConfigManagerClass ValideConfigManagerClass;

/**
 * The template manager
 */
struct _ValideTemplateManager {
	GObject parent_instance;
	ValideTemplateManagerPrivate * priv;
};

struct _ValideTemplateManagerClass {
	GObjectClass parent_class;
};


static ValideTemplateManager* valide_template_manager_instance;
static ValideTemplateManager* valide_template_manager_instance = NULL;
static gpointer valide_template_manager_parent_class = NULL;

GType valide_template_manager_get_type (void);
enum  {
	VALIDE_TEMPLATE_MANAGER_DUMMY_PROPERTY
};
static ValideTemplateManager* valide_template_manager_new (void);
static ValideTemplateManager* valide_template_manager_construct (GType object_type);
ValideTemplateManager* valide_template_manager_get_instance (void);
GType valide_abstract_config_manager_get_type (void);
GType valide_config_manager_get_type (void);
ValideConfigManager* valide_config_manager_get_instance (void);
char* valide_abstract_config_manager_get_home_dir (ValideAbstractConfigManager* self);
static void _g_list_free_g_free (GList* self);
GList* valide_template_manager_get_list (ValideTemplateManager* self);
static void valide_template_manager_finalize (GObject* obj);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);



#line 29 "template-manager.vala"
static ValideTemplateManager* valide_template_manager_construct (GType object_type) {
#line 100 "template-manager.c"
	ValideTemplateManager * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


#line 29 "template-manager.vala"
static ValideTemplateManager* valide_template_manager_new (void) {
#line 29 "template-manager.vala"
	return valide_template_manager_construct (VALIDE_TYPE_TEMPLATE_MANAGER);
#line 111 "template-manager.c"
}


/**
   * Get the uniq instance of Valide.TemplateManager
   *
   * @return The Valide.TemplateManager instance
   */
#line 38 "template-manager.vala"
ValideTemplateManager* valide_template_manager_get_instance (void) {
#line 122 "template-manager.c"
	ValideTemplateManager* result;
	ValideTemplateManager* _tmp1_;
#line 40 "template-manager.vala"
	if (valide_template_manager_instance == NULL) {
#line 127 "template-manager.c"
		ValideTemplateManager* _tmp0_;
		_tmp0_ = NULL;
#line 42 "template-manager.vala"
		valide_template_manager_instance = (_tmp0_ = valide_template_manager_new (), (valide_template_manager_instance == NULL) ? NULL : (valide_template_manager_instance = (g_object_unref (valide_template_manager_instance), NULL)), _tmp0_);
#line 132 "template-manager.c"
	}
#line 44 "template-manager.vala"
	_tmp1_ = NULL;
#line 136 "template-manager.c"
	result = (_tmp1_ = valide_template_manager_instance, (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_));
	return result;
}


static void _g_list_free_g_free (GList* self) {
	g_list_foreach (self, (GFunc) g_free, NULL);
	g_list_free (self);
}


/**
   * Get the list of available template
   *
   * @return The template list
   */
#line 52 "template-manager.vala"
GList* valide_template_manager_get_list (ValideTemplateManager* self) {
#line 155 "template-manager.c"
	GList* result;
	GError * _inner_error_;
	GList* list;
	char** _tmp4_;
	gint template_dir_size;
	gint template_dir_length1;
	char** _tmp2_;
	char* _tmp1_;
	ValideConfigManager* _tmp0_;
	char** _tmp3_;
	char** template_dir;
	GList* _tmp5_;
#line 52 "template-manager.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 170 "template-manager.c"
	_inner_error_ = NULL;
	list = NULL;
#line 55 "template-manager.vala"
	_tmp4_ = NULL;
#line 55 "template-manager.vala"
	_tmp2_ = NULL;
#line 49 "abstract-config-manager.vala"
	_tmp1_ = NULL;
#line 88 "config-manager.vala"
	_tmp0_ = NULL;
#line 55 "template-manager.vala"
	_tmp3_ = NULL;
#line 183 "template-manager.c"
	template_dir = (_tmp4_ = (_tmp3_ = (_tmp2_ = g_new0 (char*, 2 + 1), _tmp2_[0] = g_strdup (TEMPLATES_DIR), _tmp2_[1] = g_build_filename (_tmp1_ = valide_abstract_config_manager_get_home_dir ((ValideAbstractConfigManager*) (_tmp0_ = valide_config_manager_get_instance ())), "template", NULL), _tmp2_), _tmp1_ = (g_free (_tmp1_), NULL), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp3_), template_dir_length1 = 2, template_dir_size = template_dir_length1, _tmp4_);
	_tmp5_ = NULL;
#line 61 "template-manager.vala"
	list = (_tmp5_ = NULL, (list == NULL) ? NULL : (list = (_g_list_free_g_free (list), NULL)), _tmp5_);
#line 188 "template-manager.c"
	{
		char** d_collection;
		int d_collection_length1;
		int d_it;
#line 62 "template-manager.vala"
		d_collection = template_dir;
#line 195 "template-manager.c"
		d_collection_length1 = template_dir_length1;
		for (d_it = 0; d_it < template_dir_length1; d_it = d_it + 1) {
			const char* _tmp14_;
			char* d;
#line 55 "template-manager.vala"
			_tmp14_ = NULL;
#line 202 "template-manager.c"
			d = (_tmp14_ = d_collection[d_it], (_tmp14_ == NULL) ? NULL : g_strdup (_tmp14_));
			{
				{
					GDir* dir;
					GDir* _tmp6_;
					GDir* _tmp7_;
					dir = NULL;
					_tmp6_ = g_dir_open (d, 0, &_inner_error_);
					if (_inner_error_ != NULL) {
						(dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL));
						goto __catch37_g_error;
						goto __finally37;
					}
					_tmp7_ = NULL;
#line 68 "template-manager.vala"
					dir = (_tmp7_ = _tmp6_, (dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL)), _tmp7_);
#line 69 "template-manager.vala"
					while (TRUE) {
#line 221 "template-manager.c"
						char* filename;
						char* _tmp9_;
						const char* _tmp8_;
						filename = NULL;
						_tmp9_ = NULL;
#line 73 "template-manager.vala"
						_tmp8_ = NULL;
#line 73 "template-manager.vala"
						filename = (_tmp9_ = (_tmp8_ = g_dir_read_name (dir), (_tmp8_ == NULL) ? NULL : g_strdup (_tmp8_)), filename = (g_free (filename), NULL), _tmp9_);
#line 74 "template-manager.vala"
						if (filename != NULL) {
#line 233 "template-manager.c"
							char* _tmp10_;
							gboolean _tmp11_;
#line 2034 "glib-2.0.vapi"
							_tmp10_ = NULL;
#line 76 "template-manager.vala"
							if ((_tmp11_ = g_file_test (_tmp10_ = g_build_filename (d, filename, NULL), G_FILE_TEST_IS_DIR), _tmp10_ = (g_free (_tmp10_), NULL), _tmp11_)) {
#line 240 "template-manager.c"
								char* file;
								char* _tmp12_;
								file = NULL;
								_tmp12_ = NULL;
#line 80 "template-manager.vala"
								file = (_tmp12_ = g_build_filename (d, filename, "template.yml", NULL), file = (g_free (file), NULL), _tmp12_);
#line 81 "template-manager.vala"
								if (g_file_test (file, G_FILE_TEST_EXISTS)) {
#line 249 "template-manager.c"
									const char* _tmp13_;
#line 83 "template-manager.vala"
									_tmp13_ = NULL;
#line 83 "template-manager.vala"
									list = g_list_append (list, (_tmp13_ = file, (_tmp13_ == NULL) ? NULL : g_strdup (_tmp13_)));
#line 255 "template-manager.c"
								}
								file = (g_free (file), NULL);
							}
						} else {
							filename = (g_free (filename), NULL);
#line 89 "template-manager.vala"
							break;
#line 263 "template-manager.c"
						}
						filename = (g_free (filename), NULL);
					}
					(dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL));
				}
				goto __finally37;
				__catch37_g_error:
				{
					GError * e;
					e = _inner_error_;
					_inner_error_ = NULL;
					{
#line 95 "template-manager.vala"
						g_debug (e->message);
#line 278 "template-manager.c"
						(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
					}
				}
				__finally37:
				if (_inner_error_ != NULL) {
					d = (g_free (d), NULL);
					(list == NULL) ? NULL : (list = (_g_list_free_g_free (list), NULL));
					template_dir = (_vala_array_free (template_dir, template_dir_length1, (GDestroyNotify) g_free), NULL);
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return NULL;
				}
				d = (g_free (d), NULL);
			}
		}
	}
#line 98 "template-manager.vala"
	list = g_list_sort (list, strcmp);
#line 297 "template-manager.c"
	result = list;
	template_dir = (_vala_array_free (template_dir, template_dir_length1, (GDestroyNotify) g_free), NULL);
	return result;
}


static void valide_template_manager_class_init (ValideTemplateManagerClass * klass) {
	valide_template_manager_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = valide_template_manager_finalize;
}


static void valide_template_manager_instance_init (ValideTemplateManager * self) {
}


static void valide_template_manager_finalize (GObject* obj) {
	ValideTemplateManager * self;
	self = VALIDE_TEMPLATE_MANAGER (obj);
	G_OBJECT_CLASS (valide_template_manager_parent_class)->finalize (obj);
}


GType valide_template_manager_get_type (void) {
	static GType valide_template_manager_type_id = 0;
	if (valide_template_manager_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideTemplateManagerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_template_manager_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideTemplateManager), 0, (GInstanceInitFunc) valide_template_manager_instance_init, NULL };
		valide_template_manager_type_id = g_type_register_static (G_TYPE_OBJECT, "ValideTemplateManager", &g_define_type_info, 0);
	}
	return valide_template_manager_type_id;
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}




