/* uimanager.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <glib/gi18n-lib.h>
#include <stdlib.h>
#include <string.h>


#define VALIDE_TYPE_UI_MANAGER (valide_ui_manager_get_type ())
#define VALIDE_UI_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_UI_MANAGER, ValideUIManager))
#define VALIDE_UI_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_UI_MANAGER, ValideUIManagerClass))
#define VALIDE_IS_UI_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_UI_MANAGER))
#define VALIDE_IS_UI_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_UI_MANAGER))
#define VALIDE_UI_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_UI_MANAGER, ValideUIManagerClass))

typedef struct _ValideUIManager ValideUIManager;
typedef struct _ValideUIManagerClass ValideUIManagerClass;
typedef struct _ValideUIManagerPrivate ValideUIManagerPrivate;

/**
 * Class for construct the user interface (menu and toolbar)
 */
struct _ValideUIManager {
	GtkUIManager parent_instance;
	ValideUIManagerPrivate * priv;
};

struct _ValideUIManagerClass {
	GtkUIManagerClass parent_class;
};

struct _ValideUIManagerPrivate {
	GtkActionGroup* action_group;
};


static gpointer valide_ui_manager_parent_class = NULL;

GType valide_ui_manager_get_type (void);
#define VALIDE_UI_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_UI_MANAGER, ValideUIManagerPrivate))
enum  {
	VALIDE_UI_MANAGER_DUMMY_PROPERTY
};
#define VALIDE_UI_MANAGER_xmldef "<ui>\t\t\n  <menubar name=\"menubar\">\n    <menu action=\"file\">\n      <menuitem action=\"file-new\"/>\n      <placeholder name=\"file-ops-1\"/>\n      <menuitem action=\"file-open\"/>\n      <placeholder name=\"file-ops-2\"/>\n      <separator/>\n      <menuitem action=\"file-save\"/>\n      <menuitem action=\"file-save-as\"/>\n      <menuitem action=\"file-save-all\"/>\n      <placeholder name=\"file-ops-3\"/>\n      <separator/>\n      <menuitem action=\"file-rename\"/>\n      <placeholder name=\"file-ops-4\"/>\n      <separator/>\n      <menuitem action=\"file-close\"/>\n      <menuitem action=\"file-close-all\"/>\n      <placeholder name=\"file-recents-placeholder\">\n        <separator/>\n      </placeholder>\n      <separator/>\n      <menuitem action=\"quit\"/>\n    </menu>\n\n    <menu action=\"edit\">\n      <menuitem action=\"edit-undo\"/>\n      <menuitem action=\"edit-redo\"/>\n      <separator/>\n      <menuitem action=\"edit-copy\"/>\n      <menuitem action=\"edit-cut\"/>\n      <menuitem action=\"edit-paste\"/>\n      <placeholder name=\"edit-ops-1\"/>\n      <separator/>\n      <menuitem action=\"edit-lower\"/>\n      <menuitem action=\"edit-upper\"/>\n      <placeholder name=\"edit-ops-2\"/>\n      <separator/>\n      <menuitem action=\"edit-preferences\"/>\n    </menu>\n\n    <menu action=\"search\">\n      <menuitem action=\"search-find\"/>\n      <menuitem action=\"search-find-next\"/>\n      <menuitem action=\"search-find-prev\"/>\n      <placeholder name=\"search-ops-1\"/>\n      <separator/>\n      <menuitem action=\"search-replace\"/>\n      <placeholder name=\"search-ops-2\"/>\n      <separator/>\n      <menuitem action=\"search-goto-line\"/>\n      <placeholder name=\"search-ops-3\"/>\n      <separator/>\n    </menu>\n\n    <menu action=\"view\">\n      <menuitem action=\"view-start-page\"/>\n      <separator/>\n      <menuitem action=\"view-toolbar\"/>\n      <menuitem action=\"view-statusbar\"/>\n      <menuitem action=\"view-side-panel\"/>\n      <menuitem action=\"view-bottom-panel\"/>\n      <separator/>\n      <menuitem action=\"view-fullscreen\"/>\n    </menu>\n\n    <menu action=\"project\">\n      <menuitem action=\"project-new\"/>\n      <menuitem action=\"project-open\"/>\n      <menuitem action=\"project-close\"/>\n      <separator/>\n      <menuitem action=\"project-add\" />\n      <menuitem action=\"project-remove\" />\n      <separator/>\n      <menuitem action=\"project-configure\"/>\n      <menuitem action=\"project-build\"/>\n      <menuitem action=\"project-install\"/>\n      <menuitem action=\"project-dist\"/>\n      <menuitem action=\"project-clean\"/>\n      <menuitem action=\"project-distclean\"/>\n      <menuitem action=\"project-uninstall\"/>\n      <placeholder name=\"project-ops-1\"/>\n      <separator/>\n      <menuitem action=\"project-execute\"/>\n      <menuitem action=\"project-abort\"/>\n      <placeholder name=\"project-ops-2\"/>\n      <separator/>\n      <menuitem action=\"project-options\"/>\n      <placeholder name=\"project-recents-placeholder\">\n        <separator/>\n      </placeholder>\n    </menu>\n\n    <menu action=\"help\">\n      <menuitem action=\"help-about\"/>\n    </menu>\n  </menubar>\n\n  <toolbar name=\"toolbar\">\n    <toolitem action=\"file-new\" />\n    <toolitem action=\"file-open\" />\n    <toolitem action=\"file-save\" />\n    <separator/>\n    <toolitem action=\"edit-undo\" />\n    <toolitem action=\"edit-redo\" />\n    <separator/>\n    <toolitem action=\"search-find\" />\n    <toolitem action=\"search-replace\" />\n    <separator/>\n    <toolitem action=\"project-build\"/>\n    <toolitem action=\"project-execute\"/>\n    <toolitem action=\"project-abort\"/>\n  </toolbar>\n\n  <toolbar name=\"fullscreen-toolbar\">\n    <toolitem action=\"file-new\" />\n    <toolitem action=\"file-open\" />\n    <toolitem action=\"file-save\" />\n    <separator/>\n    <toolitem action=\"edit-undo\" />\n    <toolitem action=\"edit-redo\" />\n    <separator/>\n    <toolitem action=\"search-find\" />\n    <toolitem action=\"search-replace\" />\n    <separator/>\n    <toolitem action=\"project-build\"/>\n    <toolitem action=\"project-execute\"/>\n    <toolitem action=\"project-abort\"/>\n    <separator expand=\"true\"/>\n    <toolitem action=\"leave-fullscreen\"/>\n  </toolbar>\n\n  <popup name=\"project-popup\">\n    <menuitem action=\"project-file-new\" />\n    <menuitem action=\"project-file-open\" />\n    <menuitem action=\"project-file-add\" />\n    <menuitem action=\"project-file-remove\" />\n  </popup>\n\n  <popup name=\"document-popup\">\n    <menuitem action=\"file-rename\"/>\n    <separator/>\n    <menuitem action=\"file-save\"/>\n    <menuitem action=\"file-save-as\"/>\n    <separator/>\n    <menuitem action=\"file-close\"/>\n  </popup>\n</ui>"
void valide_ui_manager_populate (ValideUIManager* self);
GtkAction* valide_ui_manager_get_action (ValideUIManager* self, const char* name);
void valide_ui_manager_action_set_toggled (ValideUIManager* self, const char* name, gboolean active);
ValideUIManager* valide_ui_manager_new (void);
ValideUIManager* valide_ui_manager_construct (GType object_type);
static GObject * valide_ui_manager_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void valide_ui_manager_finalize (GObject* obj);



/**
   * Populate the graphical user interface with menu and toolbar
   */
#line 45 "uimanager.vala"
void valide_ui_manager_populate (ValideUIManager* self) {
#line 82 "uimanager.c"
	GtkAction* _tmp0_;
	GtkAction* _tmp1_;
	GtkAction* _tmp2_;
	GtkAction* _tmp3_;
	GtkAction* _tmp4_;
	GtkAction* _tmp5_;
	GtkAction* _tmp6_;
	GtkAction* _tmp7_;
	GtkAction* _tmp8_;
	GtkAction* _tmp9_;
	GtkAction* _tmp10_;
	GtkAction* _tmp11_;
	GtkAction* _tmp12_;
	GtkAction* _tmp13_;
	GtkAction* _tmp14_;
	GtkAction* _tmp15_;
	GtkAction* _tmp16_;
	GtkAction* _tmp17_;
	GtkAction* _tmp18_;
	GtkAction* _tmp19_;
	GtkAction* _tmp20_;
	GtkAction* _tmp21_;
	GtkAction* _tmp22_;
	GtkAction* _tmp23_;
	GtkAction* _tmp24_;
	GtkAction* _tmp25_;
	GtkToggleAction* _tmp26_;
	GtkToggleAction* _tmp27_;
	GtkToggleAction* _tmp28_;
	GtkToggleAction* _tmp29_;
	GtkToggleAction* _tmp30_;
	GtkToggleAction* _tmp31_;
	GtkAction* _tmp32_;
	GtkAction* _tmp33_;
	GtkAction* _tmp34_;
	GtkAction* _tmp35_;
	GtkAction* _tmp36_;
	GtkAction* _tmp37_;
	GtkAction* _tmp38_;
	GtkAction* _tmp39_;
	GtkAction* _tmp40_;
	GtkAction* _tmp41_;
	GtkAction* _tmp42_;
	GtkAction* _tmp43_;
	GtkAction* _tmp44_;
	GtkAction* _tmp45_;
	GtkAction* _tmp46_;
	GtkAction* _tmp47_;
	GtkAction* _tmp48_;
	GtkAction* _tmp49_;
	GtkAction* _tmp50_;
	GtkAction* _tmp51_;
	GtkAction* _tmp52_;
	GtkAction* _tmp53_;
	GtkAction* _tmp54_;
#line 45 "uimanager.vala"
	g_return_if_fail (self != NULL);
#line 140 "uimanager.c"
	/* File menu */
	_tmp0_ = NULL;
#line 48 "uimanager.vala"
	gtk_action_group_add_action (self->priv->action_group, _tmp0_ = gtk_action_new ("file", _ ("_File"), NULL, NULL));
#line 145 "uimanager.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
	_tmp1_ = NULL;
#line 49 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp1_ = gtk_action_new ("file-new", NULL, _ ("Create new source file"), GTK_STOCK_NEW), NULL);
#line 150 "uimanager.c"
	(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
	_tmp2_ = NULL;
#line 50 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp2_ = gtk_action_new ("file-open", _ ("Open..."), _ ("Open source file"), GTK_STOCK_OPEN), NULL);
#line 155 "uimanager.c"
	(_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL));
	_tmp3_ = NULL;
#line 51 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp3_ = gtk_action_new ("file-save", NULL, _ ("Save source file"), GTK_STOCK_SAVE), NULL);
#line 160 "uimanager.c"
	(_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL));
	_tmp4_ = NULL;
#line 52 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp4_ = gtk_action_new ("file-save-as", "Save As...", _ ("Save as source file"), GTK_STOCK_SAVE_AS), "<ctrl><shift>S");
#line 165 "uimanager.c"
	(_tmp4_ == NULL) ? NULL : (_tmp4_ = (g_object_unref (_tmp4_), NULL));
	_tmp5_ = NULL;
#line 53 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp5_ = gtk_action_new ("file-save-all", "Save All", _ ("Save all unsaved source files"), NULL), "<ctrl><shift>L");
#line 170 "uimanager.c"
	(_tmp5_ == NULL) ? NULL : (_tmp5_ = (g_object_unref (_tmp5_), NULL));
	_tmp6_ = NULL;
#line 54 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp6_ = gtk_action_new ("file-rename", "Rename...", _ ("Rename source file"), NULL), NULL);
#line 175 "uimanager.c"
	(_tmp6_ == NULL) ? NULL : (_tmp6_ = (g_object_unref (_tmp6_), NULL));
	_tmp7_ = NULL;
#line 55 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp7_ = gtk_action_new ("file-close", NULL, _ ("Close source file"), GTK_STOCK_CLOSE), NULL);
#line 180 "uimanager.c"
	(_tmp7_ == NULL) ? NULL : (_tmp7_ = (g_object_unref (_tmp7_), NULL));
	_tmp8_ = NULL;
#line 56 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp8_ = gtk_action_new ("file-close-all", _ ("C_lose All"), _ ("Close all source files"), NULL), "<ctrl><shift>W");
#line 185 "uimanager.c"
	(_tmp8_ == NULL) ? NULL : (_tmp8_ = (g_object_unref (_tmp8_), NULL));
	_tmp9_ = NULL;
#line 57 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp9_ = gtk_action_new ("quit", NULL, _ ("Quit application"), GTK_STOCK_QUIT), NULL);
#line 190 "uimanager.c"
	(_tmp9_ == NULL) ? NULL : (_tmp9_ = (g_object_unref (_tmp9_), NULL));
	/* Edit menu */
	_tmp10_ = NULL;
#line 60 "uimanager.vala"
	gtk_action_group_add_action (self->priv->action_group, _tmp10_ = gtk_action_new ("edit", _ ("_Edit"), NULL, NULL));
#line 196 "uimanager.c"
	(_tmp10_ == NULL) ? NULL : (_tmp10_ = (g_object_unref (_tmp10_), NULL));
	_tmp11_ = NULL;
#line 61 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp11_ = gtk_action_new ("edit-undo", NULL, _ ("Undo last action"), GTK_STOCK_UNDO), "<ctrl>Z");
#line 201 "uimanager.c"
	(_tmp11_ == NULL) ? NULL : (_tmp11_ = (g_object_unref (_tmp11_), NULL));
	_tmp12_ = NULL;
#line 62 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp12_ = gtk_action_new ("edit-redo", NULL, _ ("Redo last undo action"), GTK_STOCK_REDO), "<ctrl><shift>Z");
#line 206 "uimanager.c"
	(_tmp12_ == NULL) ? NULL : (_tmp12_ = (g_object_unref (_tmp12_), NULL));
	_tmp13_ = NULL;
#line 63 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp13_ = gtk_action_new ("edit-copy", NULL, _ ("Copy text"), GTK_STOCK_COPY), NULL);
#line 211 "uimanager.c"
	(_tmp13_ == NULL) ? NULL : (_tmp13_ = (g_object_unref (_tmp13_), NULL));
	_tmp14_ = NULL;
#line 64 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp14_ = gtk_action_new ("edit-cut", NULL, _ ("Cut text"), GTK_STOCK_CUT), NULL);
#line 216 "uimanager.c"
	(_tmp14_ == NULL) ? NULL : (_tmp14_ = (g_object_unref (_tmp14_), NULL));
	_tmp15_ = NULL;
#line 65 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp15_ = gtk_action_new ("edit-paste", NULL, _ ("Paste text"), GTK_STOCK_PASTE), NULL);
#line 221 "uimanager.c"
	(_tmp15_ == NULL) ? NULL : (_tmp15_ = (g_object_unref (_tmp15_), NULL));
	_tmp16_ = NULL;
#line 66 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp16_ = gtk_action_new ("edit-lower", _ ("Make Lowercase"), _ ("Make selection lowercase"), NULL), "<ctrl>L");
#line 226 "uimanager.c"
	(_tmp16_ == NULL) ? NULL : (_tmp16_ = (g_object_unref (_tmp16_), NULL));
	_tmp17_ = NULL;
#line 67 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp17_ = gtk_action_new ("edit-upper", _ ("Make Uppercase"), _ ("Make selection uppercase"), NULL), "<ctrl>U");
#line 231 "uimanager.c"
	(_tmp17_ == NULL) ? NULL : (_tmp17_ = (g_object_unref (_tmp17_), NULL));
	_tmp18_ = NULL;
#line 68 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp18_ = gtk_action_new ("edit-preferences", NULL, _ ("Configure IDE"), GTK_STOCK_PREFERENCES), NULL);
#line 236 "uimanager.c"
	(_tmp18_ == NULL) ? NULL : (_tmp18_ = (g_object_unref (_tmp18_), NULL));
	/* Search menu */
	_tmp19_ = NULL;
#line 71 "uimanager.vala"
	gtk_action_group_add_action (self->priv->action_group, _tmp19_ = gtk_action_new ("search", _ ("_Search"), NULL, NULL));
#line 242 "uimanager.c"
	(_tmp19_ == NULL) ? NULL : (_tmp19_ = (g_object_unref (_tmp19_), NULL));
	_tmp20_ = NULL;
#line 72 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp20_ = gtk_action_new ("search-find", _ ("Find..."), _ ("Find"), GTK_STOCK_FIND), NULL);
#line 247 "uimanager.c"
	(_tmp20_ == NULL) ? NULL : (_tmp20_ = (g_object_unref (_tmp20_), NULL));
	_tmp21_ = NULL;
#line 73 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp21_ = gtk_action_new ("search-find-next", _ ("Find Next"), _ ("Find next"), NULL), "<ctrl>G");
#line 252 "uimanager.c"
	(_tmp21_ == NULL) ? NULL : (_tmp21_ = (g_object_unref (_tmp21_), NULL));
	_tmp22_ = NULL;
#line 74 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp22_ = gtk_action_new ("search-find-prev", _ ("Find Previous"), _ ("Find previous"), NULL), "<ctrl><shift>G");
#line 257 "uimanager.c"
	(_tmp22_ == NULL) ? NULL : (_tmp22_ = (g_object_unref (_tmp22_), NULL));
	_tmp23_ = NULL;
#line 75 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp23_ = gtk_action_new ("search-replace", NULL, _ ("Replace"), GTK_STOCK_FIND_AND_REPLACE), "<ctrl>H");
#line 262 "uimanager.c"
	(_tmp23_ == NULL) ? NULL : (_tmp23_ = (g_object_unref (_tmp23_), NULL));
	_tmp24_ = NULL;
#line 76 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp24_ = gtk_action_new ("search-goto-line", _ ("Go to Line..."), _ ("Goto line"), GTK_STOCK_JUMP_TO), "<ctrl>I");
#line 267 "uimanager.c"
	(_tmp24_ == NULL) ? NULL : (_tmp24_ = (g_object_unref (_tmp24_), NULL));
	/* View menu */
	_tmp25_ = NULL;
#line 79 "uimanager.vala"
	gtk_action_group_add_action (self->priv->action_group, _tmp25_ = gtk_action_new ("view", _ ("_View"), NULL, NULL));
#line 273 "uimanager.c"
	(_tmp25_ == NULL) ? NULL : (_tmp25_ = (g_object_unref (_tmp25_), NULL));
	_tmp26_ = NULL;
#line 80 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, (GtkAction*) (_tmp26_ = gtk_toggle_action_new ("view-start-page", _ ("Start Page"), _ ("View/hide the start page"), NULL)), NULL);
#line 278 "uimanager.c"
	(_tmp26_ == NULL) ? NULL : (_tmp26_ = (g_object_unref (_tmp26_), NULL));
	_tmp27_ = NULL;
#line 81 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, (GtkAction*) (_tmp27_ = gtk_toggle_action_new ("view-toolbar", _ ("Toolbar"), _ ("View/hide the toolbar"), NULL)), NULL);
#line 283 "uimanager.c"
	(_tmp27_ == NULL) ? NULL : (_tmp27_ = (g_object_unref (_tmp27_), NULL));
	_tmp28_ = NULL;
#line 82 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, (GtkAction*) (_tmp28_ = gtk_toggle_action_new ("view-statusbar", _ ("Statusbar"), _ ("View/hide the statusbar"), NULL)), NULL);
#line 288 "uimanager.c"
	(_tmp28_ == NULL) ? NULL : (_tmp28_ = (g_object_unref (_tmp28_), NULL));
	_tmp29_ = NULL;
#line 83 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, (GtkAction*) (_tmp29_ = gtk_toggle_action_new ("view-side-panel", _ ("Side Panel"), _ ("View/hide the side panel"), NULL)), "F9");
#line 293 "uimanager.c"
	(_tmp29_ == NULL) ? NULL : (_tmp29_ = (g_object_unref (_tmp29_), NULL));
	_tmp30_ = NULL;
#line 84 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, (GtkAction*) (_tmp30_ = gtk_toggle_action_new ("view-bottom-panel", _ ("Bottom Panel"), _ ("View/hide the bottom panel"), NULL)), "<ctrl>F9");
#line 298 "uimanager.c"
	(_tmp30_ == NULL) ? NULL : (_tmp30_ = (g_object_unref (_tmp30_), NULL));
	_tmp31_ = NULL;
#line 85 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, (GtkAction*) (_tmp31_ = gtk_toggle_action_new ("view-fullscreen", _ ("Full Screen"), _ ("View/hide the bottom panel"), NULL)), "F11");
#line 303 "uimanager.c"
	(_tmp31_ == NULL) ? NULL : (_tmp31_ = (g_object_unref (_tmp31_), NULL));
	/* Project menu */
	_tmp32_ = NULL;
#line 88 "uimanager.vala"
	gtk_action_group_add_action (self->priv->action_group, _tmp32_ = gtk_action_new ("project", _ ("_Project"), NULL, NULL));
#line 309 "uimanager.c"
	(_tmp32_ == NULL) ? NULL : (_tmp32_ = (g_object_unref (_tmp32_), NULL));
	_tmp33_ = NULL;
#line 89 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp33_ = gtk_action_new ("project-new", _ ("New..."), _ ("Create new project"), GTK_STOCK_NEW), "<ctrl><shift>N");
#line 314 "uimanager.c"
	(_tmp33_ == NULL) ? NULL : (_tmp33_ = (g_object_unref (_tmp33_), NULL));
	_tmp34_ = NULL;
#line 90 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp34_ = gtk_action_new ("project-open", _ ("Open..."), _ ("Open project"), GTK_STOCK_OPEN), "<ctrl><shift>O");
#line 319 "uimanager.c"
	(_tmp34_ == NULL) ? NULL : (_tmp34_ = (g_object_unref (_tmp34_), NULL));
	_tmp35_ = NULL;
#line 91 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp35_ = gtk_action_new ("project-add", _ ("_Add File..."), _ ("Add file to project"), GTK_STOCK_ADD), NULL);
#line 324 "uimanager.c"
	(_tmp35_ == NULL) ? NULL : (_tmp35_ = (g_object_unref (_tmp35_), NULL));
	_tmp36_ = NULL;
#line 92 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp36_ = gtk_action_new ("project-remove", _ ("_Remove File..."), _ ("Remove file to project"), GTK_STOCK_REMOVE), NULL);
#line 329 "uimanager.c"
	(_tmp36_ == NULL) ? NULL : (_tmp36_ = (g_object_unref (_tmp36_), NULL));
	_tmp37_ = NULL;
#line 93 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp37_ = gtk_action_new ("project-options", _ ("O_ptions..."), _ ("Project options"), GTK_STOCK_PROPERTIES), NULL);
#line 334 "uimanager.c"
	(_tmp37_ == NULL) ? NULL : (_tmp37_ = (g_object_unref (_tmp37_), NULL));
	_tmp38_ = NULL;
#line 94 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp38_ = gtk_action_new ("project-configure", _ ("_Configure"), _ ("Configure project"), NULL), "F5");
#line 339 "uimanager.c"
	(_tmp38_ == NULL) ? NULL : (_tmp38_ = (g_object_unref (_tmp38_), NULL));
	_tmp39_ = NULL;
#line 95 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp39_ = gtk_action_new ("project-build", _ ("_Build"), _ ("Build project"), GTK_STOCK_CONVERT), "F6");
#line 344 "uimanager.c"
	(_tmp39_ == NULL) ? NULL : (_tmp39_ = (g_object_unref (_tmp39_), NULL));
	_tmp40_ = NULL;
#line 96 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp40_ = gtk_action_new ("project-install", _ ("_Install"), _ ("Install project"), NULL), "F7");
#line 349 "uimanager.c"
	(_tmp40_ == NULL) ? NULL : (_tmp40_ = (g_object_unref (_tmp40_), NULL));
	_tmp41_ = NULL;
#line 97 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp41_ = gtk_action_new ("project-dist", _ ("_Dist"), _ ("Distribute project"), NULL), NULL);
#line 354 "uimanager.c"
	(_tmp41_ == NULL) ? NULL : (_tmp41_ = (g_object_unref (_tmp41_), NULL));
	_tmp42_ = NULL;
#line 98 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp42_ = gtk_action_new ("project-clean", _ ("_Clean"), _ ("Clean project"), GTK_STOCK_CLEAR), NULL);
#line 359 "uimanager.c"
	(_tmp42_ == NULL) ? NULL : (_tmp42_ = (g_object_unref (_tmp42_), NULL));
	_tmp43_ = NULL;
#line 99 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp43_ = gtk_action_new ("project-distclean", _ ("_distclean"), _ ("Clean project for distribution"), NULL), NULL);
#line 364 "uimanager.c"
	(_tmp43_ == NULL) ? NULL : (_tmp43_ = (g_object_unref (_tmp43_), NULL));
	_tmp44_ = NULL;
#line 100 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp44_ = gtk_action_new ("project-uninstall", _ ("_Uninstall"), _ ("Uninstall project"), NULL), NULL);
#line 369 "uimanager.c"
	(_tmp44_ == NULL) ? NULL : (_tmp44_ = (g_object_unref (_tmp44_), NULL));
	_tmp45_ = NULL;
#line 101 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp45_ = gtk_action_new ("project-execute", NULL, _ ("Execute project"), GTK_STOCK_EXECUTE), "F8");
#line 374 "uimanager.c"
	(_tmp45_ == NULL) ? NULL : (_tmp45_ = (g_object_unref (_tmp45_), NULL));
	_tmp46_ = NULL;
#line 102 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp46_ = gtk_action_new ("project-abort", _ ("Abort"), _ ("Abort executing"), GTK_STOCK_STOP), "<shift>F8");
#line 379 "uimanager.c"
	(_tmp46_ == NULL) ? NULL : (_tmp46_ = (g_object_unref (_tmp46_), NULL));
	_tmp47_ = NULL;
#line 103 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp47_ = gtk_action_new ("project-close", NULL, _ ("Close project"), GTK_STOCK_CLOSE), "");
#line 384 "uimanager.c"
	(_tmp47_ == NULL) ? NULL : (_tmp47_ = (g_object_unref (_tmp47_), NULL));
	/* Help menu */
	_tmp48_ = NULL;
#line 106 "uimanager.vala"
	gtk_action_group_add_action (self->priv->action_group, _tmp48_ = gtk_action_new ("help", _ ("_Help"), NULL, NULL));
#line 390 "uimanager.c"
	(_tmp48_ == NULL) ? NULL : (_tmp48_ = (g_object_unref (_tmp48_), NULL));
	_tmp49_ = NULL;
#line 107 "uimanager.vala"
	gtk_action_group_add_action (self->priv->action_group, _tmp49_ = gtk_action_new ("help-about", NULL, _ ("About this application"), GTK_STOCK_ABOUT));
#line 395 "uimanager.c"
	(_tmp49_ == NULL) ? NULL : (_tmp49_ = (g_object_unref (_tmp49_), NULL));
	/* Fullscreen toolbar */
	_tmp50_ = NULL;
#line 110 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp50_ = gtk_action_new ("leave-fullscreen", _ ("Leave fullscreen"), _ ("Leave the fullscreen mode"), GTK_STOCK_LEAVE_FULLSCREEN), NULL);
#line 401 "uimanager.c"
	(_tmp50_ == NULL) ? NULL : (_tmp50_ = (g_object_unref (_tmp50_), NULL));
	/* Project popup */
	_tmp51_ = NULL;
#line 113 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp51_ = gtk_action_new ("project-file-new", _ ("New file"), _ ("Create new source file"), GTK_STOCK_NEW), NULL);
#line 407 "uimanager.c"
	(_tmp51_ == NULL) ? NULL : (_tmp51_ = (g_object_unref (_tmp51_), NULL));
	_tmp52_ = NULL;
#line 114 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp52_ = gtk_action_new ("project-file-open", _ ("Open file"), _ ("Open source file"), GTK_STOCK_OPEN), NULL);
#line 412 "uimanager.c"
	(_tmp52_ == NULL) ? NULL : (_tmp52_ = (g_object_unref (_tmp52_), NULL));
	_tmp53_ = NULL;
#line 115 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp53_ = gtk_action_new ("project-file-add", _ ("Add file"), _ ("Add source file to project"), GTK_STOCK_ADD), NULL);
#line 417 "uimanager.c"
	(_tmp53_ == NULL) ? NULL : (_tmp53_ = (g_object_unref (_tmp53_), NULL));
	_tmp54_ = NULL;
#line 116 "uimanager.vala"
	gtk_action_group_add_action_with_accel (self->priv->action_group, _tmp54_ = gtk_action_new ("project-file-remove", _ ("Remove file"), _ ("Remove this file from the project"), GTK_STOCK_REMOVE), NULL);
#line 422 "uimanager.c"
	(_tmp54_ == NULL) ? NULL : (_tmp54_ = (g_object_unref (_tmp54_), NULL));
#line 118 "uimanager.vala"
	gtk_ui_manager_insert_action_group ((GtkUIManager*) self, self->priv->action_group, 0);
#line 426 "uimanager.c"
}


/**
   * Get a Gtk.Action from name
   *
   * @param name The action name
   *
   * @return The Gtk.Action
   */
#line 128 "uimanager.vala"
GtkAction* valide_ui_manager_get_action (ValideUIManager* self, const char* name) {
#line 439 "uimanager.c"
	GtkAction* result;
	GtkAction* _tmp0_;
#line 128 "uimanager.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 128 "uimanager.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 130 "uimanager.vala"
	_tmp0_ = NULL;
#line 448 "uimanager.c"
	result = (_tmp0_ = gtk_action_group_get_action (self->priv->action_group, name), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	return result;
}


/**
   * Sets the checked state on the toggle action without signal emit
   *
   * @param name The action name
   * @param active Whether the action should be checked or not 
   */
#line 139 "uimanager.vala"
void valide_ui_manager_action_set_toggled (ValideUIManager* self, const char* name, gboolean active) {
#line 462 "uimanager.c"
	GtkToggleAction* action;
	GtkToggleAction* _tmp1_;
	GtkAction* _tmp0_;
#line 139 "uimanager.vala"
	g_return_if_fail (self != NULL);
#line 139 "uimanager.vala"
	g_return_if_fail (name != NULL);
#line 470 "uimanager.c"
	action = NULL;
	_tmp1_ = NULL;
#line 143 "uimanager.vala"
	_tmp0_ = NULL;
#line 143 "uimanager.vala"
	action = (_tmp1_ = (_tmp0_ = valide_ui_manager_get_action (self, name), GTK_IS_TOGGLE_ACTION (_tmp0_) ? ((GtkToggleAction*) _tmp0_) : NULL), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp1_);
#line 144 "uimanager.vala"
	gtk_action_group_remove_action (self->priv->action_group, (GtkAction*) action);
#line 145 "uimanager.vala"
	gtk_toggle_action_set_active (action, active);
#line 146 "uimanager.vala"
	gtk_action_group_add_action (self->priv->action_group, (GtkAction*) action);
#line 483 "uimanager.c"
	(action == NULL) ? NULL : (action = (g_object_unref (action), NULL));
}


/**
 * Class for construct the user interface (menu and toolbar)
 */
#line 25 "uimanager.vala"
ValideUIManager* valide_ui_manager_construct (GType object_type) {
#line 493 "uimanager.c"
	ValideUIManager * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


#line 25 "uimanager.vala"
ValideUIManager* valide_ui_manager_new (void) {
#line 25 "uimanager.vala"
	return valide_ui_manager_construct (VALIDE_TYPE_UI_MANAGER);
#line 504 "uimanager.c"
}


static GObject * valide_ui_manager_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValideUIManagerClass * klass;
	GObjectClass * parent_class;
	ValideUIManager * self;
	GError * _inner_error_;
	klass = VALIDE_UI_MANAGER_CLASS (g_type_class_peek (VALIDE_TYPE_UI_MANAGER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_UI_MANAGER (obj);
	_inner_error_ = NULL;
	{
		{
			GtkActionGroup* _tmp0_;
#line 33 "uimanager.vala"
			gtk_ui_manager_add_ui_from_string ((GtkUIManager*) self, VALIDE_UI_MANAGER_xmldef, (gssize) (-1), &_inner_error_);
#line 524 "uimanager.c"
			if (_inner_error_ != NULL) {
				goto __catch38_g_error;
				goto __finally38;
			}
			_tmp0_ = NULL;
#line 34 "uimanager.vala"
			self->priv->action_group = (_tmp0_ = gtk_action_group_new ("general"), (self->priv->action_group == NULL) ? NULL : (self->priv->action_group = (g_object_unref (self->priv->action_group), NULL)), _tmp0_);
#line 532 "uimanager.c"
		}
		goto __finally38;
		__catch38_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
#line 38 "uimanager.vala"
				g_critical (e->message);
#line 543 "uimanager.c"
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally38:
		if (_inner_error_ != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
		}
	}
	return obj;
}


static void valide_ui_manager_class_init (ValideUIManagerClass * klass) {
	valide_ui_manager_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideUIManagerPrivate));
	G_OBJECT_CLASS (klass)->constructor = valide_ui_manager_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_ui_manager_finalize;
}


static void valide_ui_manager_instance_init (ValideUIManager * self) {
	self->priv = VALIDE_UI_MANAGER_GET_PRIVATE (self);
}


static void valide_ui_manager_finalize (GObject* obj) {
	ValideUIManager * self;
	self = VALIDE_UI_MANAGER (obj);
	(self->priv->action_group == NULL) ? NULL : (self->priv->action_group = (g_object_unref (self->priv->action_group), NULL));
	G_OBJECT_CLASS (valide_ui_manager_parent_class)->finalize (obj);
}


GType valide_ui_manager_get_type (void) {
	static GType valide_ui_manager_type_id = 0;
	if (valide_ui_manager_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideUIManagerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_ui_manager_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideUIManager), 0, (GInstanceInitFunc) valide_ui_manager_instance_init, NULL };
		valide_ui_manager_type_id = g_type_register_static (GTK_TYPE_UI_MANAGER, "ValideUIManager", &g_define_type_info, 0);
	}
	return valide_ui_manager_type_id;
}




