/* utils.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *  Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <string.h>
#include <stdlib.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <gio/gio.h>
#include <glib/gstdio.h>
#include <gtk/gtk.h>
#include <config.h>
#include <gobject/gvaluecollector.h>


#define VALIDE_TYPE_POSITION (valide_position_get_type ())
typedef struct _ValidePosition ValidePosition;

#define VALIDE_TYPE_UTILS (valide_utils_get_type ())
#define VALIDE_UTILS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_UTILS, ValideUtils))
#define VALIDE_UTILS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_UTILS, ValideUtilsClass))
#define VALIDE_IS_UTILS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_UTILS))
#define VALIDE_IS_UTILS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_UTILS))
#define VALIDE_UTILS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_UTILS, ValideUtilsClass))

typedef struct _ValideUtils ValideUtils;
typedef struct _ValideUtilsClass ValideUtilsClass;
typedef struct _ValideUtilsPrivate ValideUtilsPrivate;

#define VALIDE_UTILS_TYPE_COLOR (valide_utils_color_get_type ())
#define VALIDE_UTILS_COLOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_UTILS_TYPE_COLOR, ValideUtilsColor))
#define VALIDE_UTILS_COLOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_UTILS_TYPE_COLOR, ValideUtilsColorClass))
#define VALIDE_UTILS_IS_COLOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_UTILS_TYPE_COLOR))
#define VALIDE_UTILS_IS_COLOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_UTILS_TYPE_COLOR))
#define VALIDE_UTILS_COLOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_UTILS_TYPE_COLOR, ValideUtilsColorClass))

typedef struct _ValideUtilsColor ValideUtilsColor;
typedef struct _ValideUtilsColorClass ValideUtilsColorClass;
typedef struct _ValideUtilsColorPrivate ValideUtilsColorPrivate;
typedef struct _ValideUtilsParamSpecColor ValideUtilsParamSpecColor;
typedef struct _ValideParamSpecUtils ValideParamSpecUtils;

/**
 * A class for represente a rectangular position
 */
struct _ValidePosition {
	gint start_line;
	gint end_line;
	gint start_row;
	gint end_row;
};

/**
 * Miscellaneous functions
 */
struct _ValideUtils {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValideUtilsPrivate * priv;
};

struct _ValideUtilsClass {
	GTypeClass parent_class;
	void (*finalize) (ValideUtils *self);
};

struct _ValideUtilsColor {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValideUtilsColorPrivate * priv;
};

struct _ValideUtilsColorClass {
	GTypeClass parent_class;
	void (*finalize) (ValideUtilsColor *self);
};

struct _ValideUtilsParamSpecColor {
	GParamSpec parent_instance;
};

struct _ValideParamSpecUtils {
	GParamSpec parent_instance;
};


static GHashTable* valide_utils_icons;
static GHashTable* valide_utils_icons = NULL;
static gpointer valide_utils_color_parent_class = NULL;
static gpointer valide_utils_parent_class = NULL;

GType valide_position_get_type (void);
ValidePosition* valide_position_dup (const ValidePosition* self);
void valide_position_free (ValidePosition* self);
gpointer valide_utils_ref (gpointer instance);
void valide_utils_unref (gpointer instance);
GParamSpec* valide_param_spec_utils (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void valide_value_set_utils (GValue* value, gpointer v_object);
gpointer valide_value_get_utils (const GValue* value);
GType valide_utils_get_type (void);
enum  {
	VALIDE_UTILS_DUMMY_PROPERTY
};
char* valide_utils_get_relative_path (const char* filename, const char* path);
char* valide_utils_get_extension (const char* filename);
char* valide_utils_skip_extension (const char* filename);
gint valide_utils_cmp_filename (const char* path_a, const char* path_b);
guint64 valide_utils_get_mtime (const char* filename, GError** error);
void valide_utils_set_up_io_channel (gint fd, GIOCondition cond, gboolean nblock, GIOFunc func, void* func_target, GError** error);
void valide_utils_chop (char** s);
gboolean valide_utils_str_is_num (const char* s);
gboolean valide_utils_char_is_separator (gunichar ch);
static void _lambda0_ (GtkWidget* s, GtkStyle* style);
static void __lambda0__gtk_widget_style_set (GtkWidget* _sender, GtkStyle* previous_style, gpointer self);
GtkButton* valide_utils_create_small_button (GtkImage* image);
GtkButton* valide_utils_create_small_button_from_stock (const char* stock_id);
GdkPixbuf* valide_utils_get_pixbuf_for_stock (const char* stock_id, GtkIconSize size);
GdkPixbuf* valide_utils_get_pixbuf_for_file (const char* filename, GtkIconSize size, GError** error);
char* valide_utils_get_mime_type (const char* filename, GError** error);
void valide_utils_process_gtk_events (void);
gboolean valide_utils_remove_last_element (char** s, gchar sep);
char* valide_utils_get_last_element (const char* s, gchar sep);
char* valide_utils_replace_home_dir_with_tilde (const char* uri);
GdkPixbuf* valide_utils_get_symbol_pixbuf (const char* type);
ValideUtils* valide_utils_new (void);
ValideUtils* valide_utils_construct (GType object_type);
gpointer valide_utils_color_ref (gpointer instance);
void valide_utils_color_unref (gpointer instance);
GParamSpec* valide_utils_param_spec_color (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void valide_utils_value_set_color (GValue* value, gpointer v_object);
gpointer valide_utils_value_get_color (const GValue* value);
GType valide_utils_color_get_type (void);
enum  {
	VALIDE_UTILS_COLOR_DUMMY_PROPERTY
};
#define VALIDE_UTILS_COLOR_INFO "#0000FF"
#define VALIDE_UTILS_COLOR_SUCCESS "#19A630"
#define VALIDE_UTILS_COLOR_WARNING "#ED7F10"
#define VALIDE_UTILS_COLOR_ERROR "#FF0000"
#define VALIDE_UTILS_COLOR_DEFAULT "#000000"
ValideUtilsColor* valide_utils_color_new (void);
ValideUtilsColor* valide_utils_color_construct (GType object_type);
static void valide_utils_color_finalize (ValideUtilsColor* obj);
static void valide_utils_finalize (ValideUtils* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);



ValidePosition* valide_position_dup (const ValidePosition* self) {
	ValidePosition* dup;
	dup = g_new0 (ValidePosition, 1);
	memcpy (dup, self, sizeof (ValidePosition));
	return dup;
}


void valide_position_free (ValidePosition* self) {
	g_free (self);
}


GType valide_position_get_type (void) {
	static GType valide_position_type_id = 0;
	if (valide_position_type_id == 0) {
		valide_position_type_id = g_boxed_type_register_static ("ValidePosition", (GBoxedCopyFunc) valide_position_dup, (GBoxedFreeFunc) valide_position_free);
	}
	return valide_position_type_id;
}


/**
   * Gets the path for filename relative to path.
   *
   * @param filename
   * @param path
   *
   * @return The relative path from filename to path
   */
#line 57 "utils.vala"
char* valide_utils_get_relative_path (const char* filename, const char* path) {
#line 200 "utils.c"
	char* result;
	GFile* parent;
	GFile* descendant;
	char* relative_path;
#line 57 "utils.vala"
	g_return_val_if_fail (filename != NULL, NULL);
#line 57 "utils.vala"
	g_return_val_if_fail (path != NULL, NULL);
#line 209 "utils.c"
	parent = g_file_new_for_path (path);
	descendant = g_file_new_for_path (filename);
	relative_path = g_file_get_relative_path (parent, descendant);
#line 62 "utils.vala"
	if (relative_path == NULL) {
#line 215 "utils.c"
		char* _tmp1_;
		const char* _tmp0_;
		_tmp1_ = NULL;
#line 64 "utils.vala"
		_tmp0_ = NULL;
#line 64 "utils.vala"
		relative_path = (_tmp1_ = (_tmp0_ = filename, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), relative_path = (g_free (relative_path), NULL), _tmp1_);
#line 223 "utils.c"
	}
	result = relative_path;
	(parent == NULL) ? NULL : (parent = (g_object_unref (parent), NULL));
	(descendant == NULL) ? NULL : (descendant = (g_object_unref (descendant), NULL));
	return result;
}


/**
   * Get the filename extension
   *
   * @param filename The name of a file
   *
   * @return The extension
   */
#line 76 "utils.vala"
char* valide_utils_get_extension (const char* filename) {
#line 241 "utils.c"
	char* result;
	const char* _tmp0_;
	char* ext;
#line 76 "utils.vala"
	g_return_val_if_fail (filename != NULL, NULL);
#line 78 "utils.vala"
	_tmp0_ = NULL;
#line 249 "utils.c"
	ext = (_tmp0_ = g_utf8_strrchr (filename, (gssize) (-1), (gunichar) '.'), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
#line 79 "utils.vala"
	if (ext != NULL) {
#line 253 "utils.c"
		char* _tmp1_;
		_tmp1_ = NULL;
#line 81 "utils.vala"
		ext = (_tmp1_ = g_utf8_strdown (g_utf8_offset_to_pointer (ext, (glong) 1), -1), ext = (g_free (ext), NULL), _tmp1_);
#line 258 "utils.c"
	}
	result = ext;
	return result;
}


/**
   * Remove the extension
   *
   * @param filename The name of a file
   *
   * @return The name of the file without extension
   */
#line 93 "utils.vala"
char* valide_utils_skip_extension (const char* filename) {
#line 274 "utils.c"
	char* result;
	char* basename;
	gboolean _tmp0_;
#line 93 "utils.vala"
	g_return_val_if_fail (filename != NULL, NULL);
#line 280 "utils.c"
	basename = NULL;
	_tmp0_ = FALSE;
#line 97 "utils.vala"
	if (_vala_strcmp0 (filename, ".") != 0) {
#line 97 "utils.vala"
		_tmp0_ = _vala_strcmp0 (filename, "..") != 0;
#line 287 "utils.c"
	} else {
#line 97 "utils.vala"
		_tmp0_ = FALSE;
#line 291 "utils.c"
	}
#line 97 "utils.vala"
	if (_tmp0_) {
#line 295 "utils.c"
		const char* ext;
		ext = NULL;
#line 101 "utils.vala"
		ext = g_utf8_strrchr (filename, (gssize) (-1), (gunichar) '.');
#line 102 "utils.vala"
		if (ext != NULL) {
#line 302 "utils.c"
			glong pos;
			char* _tmp1_;
			pos = 0L;
#line 106 "utils.vala"
			pos = g_utf8_pointer_to_offset (filename, ext);
#line 308 "utils.c"
			_tmp1_ = NULL;
#line 107 "utils.vala"
			basename = (_tmp1_ = g_strndup (filename, (gsize) pos), basename = (g_free (basename), NULL), _tmp1_);
#line 312 "utils.c"
		} else {
			char* _tmp3_;
			const char* _tmp2_;
			_tmp3_ = NULL;
#line 111 "utils.vala"
			_tmp2_ = NULL;
#line 111 "utils.vala"
			basename = (_tmp3_ = (_tmp2_ = filename, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)), basename = (g_free (basename), NULL), _tmp3_);
#line 321 "utils.c"
		}
	} else {
		char* _tmp5_;
		const char* _tmp4_;
		_tmp5_ = NULL;
#line 116 "utils.vala"
		_tmp4_ = NULL;
#line 116 "utils.vala"
		basename = (_tmp5_ = (_tmp4_ = filename, (_tmp4_ == NULL) ? NULL : g_strdup (_tmp4_)), basename = (g_free (basename), NULL), _tmp5_);
#line 331 "utils.c"
	}
	result = basename;
	return result;
}


/**
   * Compare filenames
   */
#line 124 "utils.vala"
gint valide_utils_cmp_filename (const char* path_a, const char* path_b) {
#line 343 "utils.c"
	gint result;
	gint ret;
	char* name_a;
	char* name_b;
	gboolean a_is_dir;
	gboolean b_is_dir;
	char* _tmp0_;
	char* _tmp1_;
	gboolean _tmp2_;
#line 124 "utils.vala"
	g_return_val_if_fail (path_a != NULL, 0);
#line 124 "utils.vala"
	g_return_val_if_fail (path_b != NULL, 0);
#line 357 "utils.c"
	ret = 0;
	name_a = NULL;
	name_b = NULL;
	a_is_dir = FALSE;
	b_is_dir = FALSE;
	_tmp0_ = NULL;
#line 130 "utils.vala"
	name_a = (_tmp0_ = g_path_get_basename (path_a), name_a = (g_free (name_a), NULL), _tmp0_);
#line 366 "utils.c"
	_tmp1_ = NULL;
#line 131 "utils.vala"
	name_b = (_tmp1_ = g_path_get_basename (path_b), name_b = (g_free (name_b), NULL), _tmp1_);
#line 133 "utils.vala"
	a_is_dir = g_file_test (path_a, G_FILE_TEST_IS_DIR);
#line 134 "utils.vala"
	b_is_dir = g_file_test (path_b, G_FILE_TEST_IS_DIR);
#line 374 "utils.c"
	_tmp2_ = FALSE;
#line 136 "utils.vala"
	if (a_is_dir) {
#line 136 "utils.vala"
		_tmp2_ = !b_is_dir;
#line 380 "utils.c"
	} else {
#line 136 "utils.vala"
		_tmp2_ = FALSE;
#line 384 "utils.c"
	}
#line 136 "utils.vala"
	if (_tmp2_) {
#line 138 "utils.vala"
		ret = -1;
#line 390 "utils.c"
	} else {
		gboolean _tmp3_;
		_tmp3_ = FALSE;
#line 140 "utils.vala"
		if (!a_is_dir) {
#line 140 "utils.vala"
			_tmp3_ = b_is_dir;
#line 398 "utils.c"
		} else {
#line 140 "utils.vala"
			_tmp3_ = FALSE;
#line 402 "utils.c"
		}
#line 140 "utils.vala"
		if (_tmp3_) {
#line 142 "utils.vala"
			ret = 1;
#line 408 "utils.c"
		} else {
			char* basename_a;
			char* basename_b;
			char* _tmp4_;
			char* _tmp5_;
			basename_a = NULL;
			basename_b = NULL;
			_tmp4_ = NULL;
#line 148 "utils.vala"
			basename_a = (_tmp4_ = valide_utils_skip_extension (name_a), basename_a = (g_free (basename_a), NULL), _tmp4_);
#line 419 "utils.c"
			_tmp5_ = NULL;
#line 149 "utils.vala"
			basename_b = (_tmp5_ = valide_utils_skip_extension (name_b), basename_b = (g_free (basename_b), NULL), _tmp5_);
#line 150 "utils.vala"
			ret = strcmp (basename_a, basename_b);
#line 151 "utils.vala"
			if (ret == 0) {
#line 153 "utils.vala"
				ret = strcmp (name_a, name_b);
#line 429 "utils.c"
			}
			basename_a = (g_free (basename_a), NULL);
			basename_b = (g_free (basename_b), NULL);
		}
	}
	result = ret;
	name_a = (g_free (name_a), NULL);
	name_b = (g_free (name_b), NULL);
	return result;
}


/**
   * Get the last modification time of a file
   *
   * @param filename The name of a file
   *
   * @return The last modification tome of the file
   */
#line 166 "utils.vala"
guint64 valide_utils_get_mtime (const char* filename, GError** error) {
#line 451 "utils.c"
	guint64 result;
	GError * _inner_error_;
	GFile* file;
	GFileInfo* info;
#line 166 "utils.vala"
	g_return_val_if_fail (filename != NULL, 0ULL);
#line 458 "utils.c"
	_inner_error_ = NULL;
	file = g_file_new_for_path (filename);
	info = g_file_query_info (file, G_FILE_ATTRIBUTE_TIME_MODIFIED, 0, NULL, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
		return 0ULL;
	}
	result = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED);
	(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
	(info == NULL) ? NULL : (info = (g_object_unref (info), NULL));
	return result;
}


/**
   * Setup an io channel
   */
#line 177 "utils.vala"
void valide_utils_set_up_io_channel (gint fd, GIOCondition cond, gboolean nblock, GIOFunc func, void* func_target, GError** error) {
#line 479 "utils.c"
	GError * _inner_error_;
	GIOChannel* ioc;
	GIOChannel* _tmp0_;
	_inner_error_ = NULL;
	ioc = NULL;
	_tmp0_ = NULL;
#line 182 "utils.vala"
	ioc = (_tmp0_ = g_io_channel_unix_new (fd), (ioc == NULL) ? NULL : (ioc = (g_io_channel_unref (ioc), NULL)), _tmp0_);
#line 183 "utils.vala"
	if (nblock) {
#line 490 "utils.c"
		{
#line 187 "utils.vala"
			g_io_channel_set_flags (ioc, G_IO_FLAG_NONBLOCK, &_inner_error_);
#line 494 "utils.c"
			if (_inner_error_ != NULL) {
				goto __catch0_g_error;
				goto __finally0;
			}
		}
		goto __finally0;
		__catch0_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
#line 191 "utils.vala"
				g_debug (e->message);
#line 509 "utils.c"
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally0:
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_IO_CHANNEL_ERROR) {
				g_propagate_error (error, _inner_error_);
				(ioc == NULL) ? NULL : (ioc = (g_io_channel_unref (ioc), NULL));
				return;
			} else {
				(ioc == NULL) ? NULL : (ioc = (g_io_channel_unref (ioc), NULL));
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return;
			}
		}
	}
#line 194 "utils.vala"
	g_io_channel_set_close_on_unref (ioc, TRUE);
#line 195 "utils.vala"
	g_io_add_watch (ioc, cond, func, func_target);
#line 531 "utils.c"
	(ioc == NULL) ? NULL : (ioc = (g_io_channel_unref (ioc), NULL));
}


/**
   * Remove the last \n character of a string
   *
   * @param s A string
   */
#line 203 "utils.vala"
void valide_utils_chop (char** s) {
#line 203 "utils.vala"
	g_return_if_fail (s != NULL);
#line 205 "utils.vala"
	if (g_str_has_suffix ((*s), "\n")) {
#line 547 "utils.c"
		char* _tmp0_;
		_tmp0_ = NULL;
#line 207 "utils.vala"
		(*s) = (_tmp0_ = g_strndup ((*s), (gsize) (g_utf8_strlen ((*s), -1) - 1)), (*s) = (g_free ((*s)), NULL), _tmp0_);
#line 552 "utils.c"
	}
}


static glong string_get_length (const char* self) {
	glong result;
	g_return_val_if_fail (self != NULL, 0L);
	result = g_utf8_strlen (self, -1);
	return result;
}


/**
   * Test if a string is a number
   *
   * @param s A string
   *
   * @return true if the string represente a number
   */
#line 218 "utils.vala"
gboolean valide_utils_str_is_num (const char* s) {
#line 574 "utils.c"
	gboolean result;
	gboolean ret;
#line 218 "utils.vala"
	g_return_val_if_fail (s != NULL, FALSE);
#line 579 "utils.c"
	ret = TRUE;
	{
		gint i;
		i = 0;
		{
			gboolean _tmp0_;
			_tmp0_ = TRUE;
#line 222 "utils.vala"
			while (TRUE) {
#line 222 "utils.vala"
				if (!_tmp0_) {
#line 222 "utils.vala"
					i++;
#line 593 "utils.c"
				}
#line 222 "utils.vala"
				_tmp0_ = FALSE;
#line 222 "utils.vala"
				if (!(i < string_get_length (s))) {
#line 222 "utils.vala"
					break;
#line 601 "utils.c"
				}
#line 224 "utils.vala"
				if (!g_unichar_isdigit (g_utf8_get_char (g_utf8_offset_to_pointer (s, i)))) {
#line 226 "utils.vala"
					ret = FALSE;
#line 227 "utils.vala"
					break;
#line 609 "utils.c"
				}
			}
		}
	}
	result = ret;
	return result;
}


/**
   * Test if a character is a separator
   *
   * @param ch A character
   *
   * @return true if the character is a separator
   */
#line 240 "utils.vala"
gboolean valide_utils_char_is_separator (gunichar ch) {
#line 628 "utils.c"
	gboolean result;
	gboolean _tmp0_;
	_tmp0_ = FALSE;
#line 242 "utils.vala"
	if (g_unichar_isprint (ch)) {
#line 634 "utils.c"
		gboolean _tmp1_;
		_tmp1_ = FALSE;
#line 242 "utils.vala"
		if (g_unichar_isalnum (ch)) {
#line 242 "utils.vala"
			_tmp1_ = TRUE;
#line 641 "utils.c"
		} else {
#line 242 "utils.vala"
			_tmp1_ = ch == '_';
#line 645 "utils.c"
		}
#line 242 "utils.vala"
		_tmp0_ = _tmp1_;
#line 649 "utils.c"
	} else {
#line 242 "utils.vala"
		_tmp0_ = FALSE;
#line 653 "utils.c"
	}
#line 242 "utils.vala"
	if (_tmp0_) {
#line 657 "utils.c"
		result = FALSE;
		return result;
	}
	result = TRUE;
	return result;
}


static void _lambda0_ (GtkWidget* s, GtkStyle* style) {
	gint h;
	gint w;
	g_return_if_fail (s != NULL);
	h = 0;
	w = 0;
#line 277 "utils.vala"
	gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (s), GTK_ICON_SIZE_MENU, &w, &h);
#line 279 "utils.vala"
	gtk_widget_set_size_request (s, w + 2, h + 2);
#line 676 "utils.c"
}


static void __lambda0__gtk_widget_style_set (GtkWidget* _sender, GtkStyle* previous_style, gpointer self) {
	_lambda0_ (_sender, previous_style);
}


/**
   * Create a button with an small image
   *
   * @param image A image
   *
   * @return A new Gtk.Button
   */
#line 256 "utils.vala"
GtkButton* valide_utils_create_small_button (GtkImage* image) {
#line 694 "utils.c"
	GtkButton* result;
	GtkButton* button;
	GtkRcStyle* rcstyle;
#line 256 "utils.vala"
	g_return_val_if_fail (image != NULL, NULL);
#line 700 "utils.c"
	button = g_object_ref_sink ((GtkButton*) gtk_button_new ());
#line 259 "utils.vala"
	gtk_button_set_relief (button, GTK_RELIEF_NONE);
#line 704 "utils.c"
	/* don't allow focus on the close button */
#line 262 "utils.vala"
	gtk_button_set_focus_on_click (button, FALSE);
#line 708 "utils.c"
	/* make it as small as possible */
	rcstyle = gtk_rc_style_new ();
#line 266 "utils.vala"
	rcstyle->xthickness = rcstyle->ythickness = 0;
#line 267 "utils.vala"
	gtk_widget_modify_style ((GtkWidget*) button, rcstyle);
#line 269 "utils.vala"
	gtk_widget_show ((GtkWidget*) image);
#line 271 "utils.vala"
	gtk_container_add ((GtkContainer*) button, (GtkWidget*) image);
#line 274 "utils.vala"
	g_signal_connect ((GtkWidget*) button, "style-set", (GCallback) __lambda0__gtk_widget_style_set, NULL);
#line 721 "utils.c"
	result = button;
	(rcstyle == NULL) ? NULL : (rcstyle = (g_object_unref (rcstyle), NULL));
	return result;
}


/**
   * Create a button with an small image from stock
   *
   * @param stock_id A stock id
   *
   * @return A new Gtk.Button
   */
#line 292 "utils.vala"
GtkButton* valide_utils_create_small_button_from_stock (const char* stock_id) {
#line 737 "utils.c"
	GtkButton* result;
	GtkImage* image;
#line 292 "utils.vala"
	g_return_val_if_fail (stock_id != NULL, NULL);
#line 742 "utils.c"
	image = g_object_ref_sink ((GtkImage*) gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_MENU));
	result = valide_utils_create_small_button (image);
	(image == NULL) ? NULL : (image = (g_object_unref (image), NULL));
	return result;
}


/**
   * Get Gdk.Pixbuf form stock_id
   *
   * @param stock_id A stock id
   * @param size The size of the Gdk.Pixbuf
   *
   * @return A new Gdk.Pixbuf
   */
#line 306 "utils.vala"
GdkPixbuf* valide_utils_get_pixbuf_for_stock (const char* stock_id, GtkIconSize size) {
#line 760 "utils.c"
	GdkPixbuf* result;
	GdkPixbuf* pixbuf;
	GtkInvisible* w;
	GdkPixbuf* _tmp0_;
#line 306 "utils.vala"
	g_return_val_if_fail (stock_id != NULL, NULL);
#line 767 "utils.c"
	pixbuf = NULL;
	w = g_object_ref_sink ((GtkInvisible*) gtk_invisible_new ());
	_tmp0_ = NULL;
#line 311 "utils.vala"
	pixbuf = (_tmp0_ = gtk_widget_render_icon ((GtkWidget*) w, stock_id, size, "vala"), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp0_);
#line 773 "utils.c"
	result = pixbuf;
	(w == NULL) ? NULL : (w = (g_object_unref (w), NULL));
	return result;
}


/**
   * Get Gdk.Pixbuf form file
   *
   * @param filename A image filename
   * @param size The size of the Gdk.Pixbuf
   *
   * @return A new Gdk.Pixbuf
   */
#line 323 "utils.vala"
GdkPixbuf* valide_utils_get_pixbuf_for_file (const char* filename, GtkIconSize size, GError** error) {
#line 790 "utils.c"
	GdkPixbuf* result;
	GError * _inner_error_;
	GFile* file;
	GIcon* icon;
	GFileInfo* file_info;
	GdkPixbuf* ret;
	GFile* _tmp0_;
	GFileInfo* _tmp1_;
	GFileInfo* _tmp2_;
#line 323 "utils.vala"
	g_return_val_if_fail (filename != NULL, NULL);
#line 802 "utils.c"
	_inner_error_ = NULL;
	file = NULL;
	icon = NULL;
	file_info = NULL;
	ret = NULL;
	_tmp0_ = NULL;
#line 330 "utils.vala"
	file = (_tmp0_ = g_file_new_for_path (filename), (file == NULL) ? NULL : (file = (g_object_unref (file), NULL)), _tmp0_);
#line 811 "utils.c"
	_tmp1_ = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_ICON, G_FILE_QUERY_INFO_NONE, NULL, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
		(icon == NULL) ? NULL : (icon = (g_object_unref (icon), NULL));
		(file_info == NULL) ? NULL : (file_info = (g_object_unref (file_info), NULL));
		(ret == NULL) ? NULL : (ret = (g_object_unref (ret), NULL));
		return NULL;
	}
	_tmp2_ = NULL;
#line 331 "utils.vala"
	file_info = (_tmp2_ = _tmp1_, (file_info == NULL) ? NULL : (file_info = (g_object_unref (file_info), NULL)), _tmp2_);
#line 335 "utils.vala"
	if (file_info != NULL) {
#line 826 "utils.c"
		GIcon* _tmp4_;
		GIcon* _tmp3_;
		_tmp4_ = NULL;
#line 337 "utils.vala"
		_tmp3_ = NULL;
#line 337 "utils.vala"
		icon = (_tmp4_ = (_tmp3_ = g_file_info_get_icon (file_info), (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), (icon == NULL) ? NULL : (icon = (g_object_unref (icon), NULL)), _tmp4_);
#line 338 "utils.vala"
		if (icon != NULL) {
#line 836 "utils.c"
			gint width;
			GtkIconTheme* theme;
			GtkIconInfo* icon_info;
			GtkIconTheme* _tmp6_;
			GtkIconTheme* _tmp5_;
			GtkIconInfo* _tmp7_;
			width = 0;
			theme = NULL;
			icon_info = NULL;
			_tmp6_ = NULL;
#line 344 "utils.vala"
			_tmp5_ = NULL;
#line 344 "utils.vala"
			theme = (_tmp6_ = (_tmp5_ = gtk_icon_theme_get_default (), (_tmp5_ == NULL) ? NULL : g_object_ref (_tmp5_)), (theme == NULL) ? NULL : (theme = (g_object_unref (theme), NULL)), _tmp6_);
#line 345 "utils.vala"
			gtk_icon_size_lookup (size, &width, NULL);
#line 853 "utils.c"
			_tmp7_ = NULL;
#line 347 "utils.vala"
			icon_info = (_tmp7_ = gtk_icon_theme_lookup_by_gicon (theme, icon, width, GTK_ICON_LOOKUP_USE_BUILTIN), (icon_info == NULL) ? NULL : (icon_info = (gtk_icon_info_free (icon_info), NULL)), _tmp7_);
#line 349 "utils.vala"
			if (icon_info != NULL) {
#line 859 "utils.c"
				GdkPixbuf* _tmp8_;
				GdkPixbuf* _tmp10_;
				GdkPixbuf* _tmp9_;
				_tmp8_ = gtk_icon_info_load_icon (icon_info, &_inner_error_);
				if (_inner_error_ != NULL) {
					g_propagate_error (error, _inner_error_);
					(theme == NULL) ? NULL : (theme = (g_object_unref (theme), NULL));
					(icon_info == NULL) ? NULL : (icon_info = (gtk_icon_info_free (icon_info), NULL));
					(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
					(icon == NULL) ? NULL : (icon = (g_object_unref (icon), NULL));
					(file_info == NULL) ? NULL : (file_info = (g_object_unref (file_info), NULL));
					(ret == NULL) ? NULL : (ret = (g_object_unref (ret), NULL));
					return NULL;
				}
				_tmp10_ = NULL;
#line 351 "utils.vala"
				_tmp9_ = NULL;
#line 351 "utils.vala"
				ret = (_tmp10_ = (_tmp9_ = _tmp8_, (_tmp9_ == NULL) ? NULL : g_object_ref (_tmp9_)), (ret == NULL) ? NULL : (ret = (g_object_unref (ret), NULL)), _tmp10_);
#line 879 "utils.c"
			}
			(theme == NULL) ? NULL : (theme = (g_object_unref (theme), NULL));
			(icon_info == NULL) ? NULL : (icon_info = (gtk_icon_info_free (icon_info), NULL));
		}
	}
	result = ret;
	(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
	(icon == NULL) ? NULL : (icon = (g_object_unref (icon), NULL));
	(file_info == NULL) ? NULL : (file_info = (g_object_unref (file_info), NULL));
	return result;
}


/**
   * Get the mime type of a file
   *
   * @param filename The name of a file
   *
   * @return The mime type
   */
#line 365 "utils.vala"
char* valide_utils_get_mime_type (const char* filename, GError** error) {
#line 902 "utils.c"
	char* result;
	GError * _inner_error_;
	GFile* file;
	GFileInfo* file_info;
	GFile* _tmp0_;
	GFileInfo* _tmp1_;
	GFileInfo* _tmp2_;
	const char* _tmp3_;
#line 365 "utils.vala"
	g_return_val_if_fail (filename != NULL, NULL);
#line 913 "utils.c"
	_inner_error_ = NULL;
	file = NULL;
	file_info = NULL;
	_tmp0_ = NULL;
#line 370 "utils.vala"
	file = (_tmp0_ = g_file_new_for_path (filename), (file == NULL) ? NULL : (file = (g_object_unref (file), NULL)), _tmp0_);
#line 920 "utils.c"
	_tmp1_ = g_file_query_info (file, "standard::*", G_FILE_QUERY_INFO_NONE, NULL, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
		(file_info == NULL) ? NULL : (file_info = (g_object_unref (file_info), NULL));
		return NULL;
	}
	_tmp2_ = NULL;
#line 371 "utils.vala"
	file_info = (_tmp2_ = _tmp1_, (file_info == NULL) ? NULL : (file_info = (g_object_unref (file_info), NULL)), _tmp2_);
#line 372 "utils.vala"
	_tmp3_ = NULL;
#line 933 "utils.c"
	result = (_tmp3_ = g_content_type_get_mime_type (g_file_info_get_content_type (file_info)), (_tmp3_ == NULL) ? NULL : g_strdup (_tmp3_));
	(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
	(file_info == NULL) ? NULL : (file_info = (g_object_unref (file_info), NULL));
	return result;
	(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
	(file_info == NULL) ? NULL : (file_info = (g_object_unref (file_info), NULL));
}


/**
   * Process the GTK events
   */
#line 378 "utils.vala"
void valide_utils_process_gtk_events (void) {
#line 380 "utils.vala"
	while (TRUE) {
#line 380 "utils.vala"
		if (!gtk_events_pending ()) {
#line 380 "utils.vala"
			break;
#line 954 "utils.c"
		}
#line 382 "utils.vala"
		gtk_main_iteration ();
#line 958 "utils.c"
	}
}


/**
   * Remove the last element of a string
   *
   * @param s A string
   * @param sep The separator
   *
   * @return true if the last element was removed
   */
#line 394 "utils.vala"
gboolean valide_utils_remove_last_element (char** s, gchar sep) {
#line 973 "utils.c"
	gboolean result;
	gint offset;
	gboolean ret;
	const char* pos;
#line 394 "utils.vala"
	g_return_val_if_fail (s != NULL, FALSE);
#line 980 "utils.c"
	offset = 0;
	ret = FALSE;
	pos = NULL;
#line 400 "utils.vala"
	pos = g_utf8_strrchr ((*s), (gssize) (-1), (gunichar) sep);
#line 401 "utils.vala"
	if (pos != NULL) {
#line 988 "utils.c"
		char* _tmp0_;
#line 403 "utils.vala"
		offset = (gint) g_utf8_pointer_to_offset ((*s), pos);
#line 992 "utils.c"
		_tmp0_ = NULL;
#line 404 "utils.vala"
		(*s) = (_tmp0_ = g_strndup ((*s), (gsize) offset), (*s) = (g_free ((*s)), NULL), _tmp0_);
#line 405 "utils.vala"
		ret = TRUE;
#line 998 "utils.c"
	}
	result = ret;
	return result;
}


#line 922 "glib-2.0.vapi"
static char* string_substring (const char* self, glong offset, glong len) {
#line 1007 "utils.c"
	char* result;
	glong string_length;
	const char* start;
#line 922 "glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1013 "utils.c"
	string_length = g_utf8_strlen (self, -1);
#line 924 "glib-2.0.vapi"
	if (offset < 0) {
#line 925 "glib-2.0.vapi"
		offset = string_length + offset;
#line 926 "glib-2.0.vapi"
		g_return_val_if_fail (offset >= 0, NULL);
#line 1021 "utils.c"
	} else {
#line 928 "glib-2.0.vapi"
		g_return_val_if_fail (offset <= string_length, NULL);
#line 1025 "utils.c"
	}
#line 930 "glib-2.0.vapi"
	if (len < 0) {
#line 931 "glib-2.0.vapi"
		len = string_length - offset;
#line 1031 "utils.c"
	}
#line 933 "glib-2.0.vapi"
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
#line 1035 "utils.c"
	start = g_utf8_offset_to_pointer (self, offset);
	result = g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
	return result;
}


/**
   * Get the last element of a string
   *
   * @param s A string
   * @param sep The separator
   *
   * @return The last element
   */
#line 418 "utils.vala"
char* valide_utils_get_last_element (const char* s, gchar sep) {
#line 1052 "utils.c"
	char* result;
	gint offset;
	char* ret;
	const char* pos;
#line 418 "utils.vala"
	g_return_val_if_fail (s != NULL, NULL);
#line 1059 "utils.c"
	offset = 0;
	ret = NULL;
	pos = NULL;
#line 424 "utils.vala"
	pos = g_utf8_strrchr (s, (gssize) (-1), (gunichar) sep);
#line 425 "utils.vala"
	if (pos != NULL) {
#line 1067 "utils.c"
		char* _tmp0_;
#line 427 "utils.vala"
		offset = (gint) g_utf8_pointer_to_offset (s, pos);
#line 428 "utils.vala"
		offset++;
#line 1073 "utils.c"
		_tmp0_ = NULL;
#line 429 "utils.vala"
		ret = (_tmp0_ = string_substring (s, (glong) offset, (glong) (-1)), ret = (g_free (ret), NULL), _tmp0_);
#line 1077 "utils.c"
	} else {
		char* _tmp2_;
		const char* _tmp1_;
		_tmp2_ = NULL;
#line 433 "utils.vala"
		_tmp1_ = NULL;
#line 433 "utils.vala"
		ret = (_tmp2_ = (_tmp1_ = s, (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_)), ret = (g_free (ret), NULL), _tmp2_);
#line 1086 "utils.c"
	}
	result = ret;
	return result;
}


/**
   * Replace home dir with tilde (~)
   *
   * @param uri An URI
   *
   * @return The transformed URI
   */
#line 445 "utils.vala"
char* valide_utils_replace_home_dir_with_tilde (const char* uri) {
#line 1102 "utils.c"
	char* result;
	char* home;
	char* short_uri;
	char* _tmp1_;
	const char* _tmp0_;
#line 445 "utils.vala"
	g_return_val_if_fail (uri != NULL, NULL);
#line 1110 "utils.c"
	home = NULL;
	short_uri = NULL;
	_tmp1_ = NULL;
#line 450 "utils.vala"
	_tmp0_ = NULL;
#line 450 "utils.vala"
	home = (_tmp1_ = (_tmp0_ = g_get_home_dir (), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), home = (g_free (home), NULL), _tmp1_);
#line 451 "utils.vala"
	if (g_str_has_prefix (uri, home)) {
#line 1120 "utils.c"
		char* _tmp3_;
		char* _tmp2_;
		_tmp3_ = NULL;
#line 922 "glib-2.0.vapi"
		_tmp2_ = NULL;
#line 453 "utils.vala"
		short_uri = (_tmp3_ = g_strconcat ("~", _tmp2_ = string_substring (uri, (glong) strlen (home), (glong) (strlen (uri) - strlen (home))), NULL), short_uri = (g_free (short_uri), NULL), _tmp3_);
#line 1128 "utils.c"
		_tmp2_ = (g_free (_tmp2_), NULL);
	} else {
		char* _tmp5_;
		const char* _tmp4_;
		_tmp5_ = NULL;
#line 458 "utils.vala"
		_tmp4_ = NULL;
#line 458 "utils.vala"
		short_uri = (_tmp5_ = (_tmp4_ = uri, (_tmp4_ == NULL) ? NULL : g_strdup (_tmp4_)), short_uri = (g_free (short_uri), NULL), _tmp5_);
#line 1138 "utils.c"
	}
	result = short_uri;
	home = (g_free (home), NULL);
	return result;
}


/**
   * Return the PixBuf that represents the specified type of symbol, or NULL.
   */
#line 466 "utils.vala"
GdkPixbuf* valide_utils_get_symbol_pixbuf (const char* type) {
#line 1151 "utils.c"
	GdkPixbuf* result;
	GError * _inner_error_;
	GdkPixbuf* pixbuf;
	GdkPixbuf* _tmp2_;
	GdkPixbuf* _tmp1_;
#line 466 "utils.vala"
	g_return_val_if_fail (type != NULL, NULL);
#line 1159 "utils.c"
	_inner_error_ = NULL;
	pixbuf = NULL;
#line 470 "utils.vala"
	if (valide_utils_icons == NULL) {
#line 1164 "utils.c"
		GHashTable* _tmp0_;
		_tmp0_ = NULL;
#line 472 "utils.vala"
		valide_utils_icons = (_tmp0_ = g_hash_table_new (g_str_hash, g_str_equal), (valide_utils_icons == NULL) ? NULL : (valide_utils_icons = (g_hash_table_unref (valide_utils_icons), NULL)), _tmp0_);
#line 1169 "utils.c"
	}
	_tmp2_ = NULL;
#line 474 "utils.vala"
	_tmp1_ = NULL;
#line 474 "utils.vala"
	pixbuf = (_tmp2_ = (_tmp1_ = (GdkPixbuf*) g_hash_table_lookup (valide_utils_icons, type), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp2_);
#line 475 "utils.vala"
	if (pixbuf == NULL) {
#line 1178 "utils.c"
		{
			char* path;
			char* filename;
			char* _tmp3_;
			char* _tmp4_;
			GdkPixbuf* _tmp5_;
			GdkPixbuf* _tmp6_;
			GdkPixbuf* _tmp8_;
			const char* _tmp7_;
			path = NULL;
			filename = NULL;
			_tmp3_ = NULL;
#line 482 "utils.vala"
			filename = (_tmp3_ = g_strdup_printf ("%s.png", type), filename = (g_free (filename), NULL), _tmp3_);
#line 1193 "utils.c"
			_tmp4_ = NULL;
#line 483 "utils.vala"
			path = (_tmp4_ = g_build_filename (PIXMAPS_DIR, "symbols", filename, NULL), path = (g_free (path), NULL), _tmp4_);
#line 1197 "utils.c"
			_tmp5_ = gdk_pixbuf_new_from_file (path, &_inner_error_);
			if (_inner_error_ != NULL) {
				path = (g_free (path), NULL);
				filename = (g_free (filename), NULL);
				goto __catch1_g_error;
				goto __finally1;
			}
			_tmp6_ = NULL;
#line 484 "utils.vala"
			pixbuf = (_tmp6_ = _tmp5_, (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp6_);
#line 485 "utils.vala"
			_tmp8_ = NULL;
#line 485 "utils.vala"
			_tmp7_ = NULL;
#line 485 "utils.vala"
			g_hash_table_insert (valide_utils_icons, (_tmp7_ = type, (_tmp7_ == NULL) ? NULL : g_strdup (_tmp7_)), (_tmp8_ = pixbuf, (_tmp8_ == NULL) ? NULL : g_object_ref (_tmp8_)));
#line 1214 "utils.c"
			path = (g_free (path), NULL);
			filename = (g_free (filename), NULL);
		}
		goto __finally1;
		__catch1_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
				GdkPixbuf* _tmp9_;
#line 489 "utils.vala"
				g_debug ("utils.vala:489: Could not load pixbuf: %s\n", e->message);
#line 1228 "utils.c"
				_tmp9_ = NULL;
#line 490 "utils.vala"
				pixbuf = (_tmp9_ = valide_utils_get_pixbuf_for_stock (GTK_STOCK_MISSING_IMAGE, GTK_ICON_SIZE_MENU), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp9_);
#line 1232 "utils.c"
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally1:
		if (_inner_error_ != NULL) {
			(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return NULL;
		}
	}
	result = pixbuf;
	return result;
}


/**
 * Miscellaneous functions
 */
#line 36 "utils.vala"
ValideUtils* valide_utils_construct (GType object_type) {
#line 1254 "utils.c"
	ValideUtils* self;
	self = (ValideUtils*) g_type_create_instance (object_type);
	return self;
}


#line 36 "utils.vala"
ValideUtils* valide_utils_new (void) {
#line 36 "utils.vala"
	return valide_utils_construct (VALIDE_TYPE_UTILS);
#line 1265 "utils.c"
}


#line 40 "utils.vala"
ValideUtilsColor* valide_utils_color_construct (GType object_type) {
#line 1271 "utils.c"
	ValideUtilsColor* self;
	self = (ValideUtilsColor*) g_type_create_instance (object_type);
	return self;
}


#line 40 "utils.vala"
ValideUtilsColor* valide_utils_color_new (void) {
#line 40 "utils.vala"
	return valide_utils_color_construct (VALIDE_UTILS_TYPE_COLOR);
#line 1282 "utils.c"
}


static void valide_utils_value_color_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void valide_utils_value_color_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		valide_utils_color_unref (value->data[0].v_pointer);
	}
}


static void valide_utils_value_color_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = valide_utils_color_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer valide_utils_value_color_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* valide_utils_value_color_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		ValideUtilsColor* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = valide_utils_color_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* valide_utils_value_color_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	ValideUtilsColor** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = valide_utils_color_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* valide_utils_param_spec_color (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ValideUtilsParamSpecColor* spec;
	g_return_val_if_fail (g_type_is_a (object_type, VALIDE_UTILS_TYPE_COLOR), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer valide_utils_value_get_color (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALIDE_UTILS_TYPE_COLOR), NULL);
	return value->data[0].v_pointer;
}


void valide_utils_value_set_color (GValue* value, gpointer v_object) {
	ValideUtilsColor* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALIDE_UTILS_TYPE_COLOR));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, VALIDE_UTILS_TYPE_COLOR));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		valide_utils_color_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		valide_utils_color_unref (old);
	}
}


static void valide_utils_color_class_init (ValideUtilsColorClass * klass) {
	valide_utils_color_parent_class = g_type_class_peek_parent (klass);
	VALIDE_UTILS_COLOR_CLASS (klass)->finalize = valide_utils_color_finalize;
}


static void valide_utils_color_instance_init (ValideUtilsColor * self) {
	self->ref_count = 1;
}


static void valide_utils_color_finalize (ValideUtilsColor* obj) {
	ValideUtilsColor * self;
	self = VALIDE_UTILS_COLOR (obj);
}


GType valide_utils_color_get_type (void) {
	static GType valide_utils_color_type_id = 0;
	if (valide_utils_color_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { valide_utils_value_color_init, valide_utils_value_color_free_value, valide_utils_value_color_copy_value, valide_utils_value_color_peek_pointer, "p", valide_utils_value_color_collect_value, "p", valide_utils_value_color_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (ValideUtilsColorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_utils_color_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideUtilsColor), 0, (GInstanceInitFunc) valide_utils_color_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		valide_utils_color_type_id = g_type_register_fundamental (g_type_fundamental_next (), "ValideUtilsColor", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return valide_utils_color_type_id;
}


gpointer valide_utils_color_ref (gpointer instance) {
	ValideUtilsColor* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void valide_utils_color_unref (gpointer instance) {
	ValideUtilsColor* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		VALIDE_UTILS_COLOR_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


static void valide_value_utils_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void valide_value_utils_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		valide_utils_unref (value->data[0].v_pointer);
	}
}


static void valide_value_utils_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = valide_utils_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer valide_value_utils_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* valide_value_utils_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		ValideUtils* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = valide_utils_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* valide_value_utils_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	ValideUtils** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = valide_utils_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* valide_param_spec_utils (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ValideParamSpecUtils* spec;
	g_return_val_if_fail (g_type_is_a (object_type, VALIDE_TYPE_UTILS), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer valide_value_get_utils (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALIDE_TYPE_UTILS), NULL);
	return value->data[0].v_pointer;
}


void valide_value_set_utils (GValue* value, gpointer v_object) {
	ValideUtils* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALIDE_TYPE_UTILS));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, VALIDE_TYPE_UTILS));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		valide_utils_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		valide_utils_unref (old);
	}
}


static void valide_utils_class_init (ValideUtilsClass * klass) {
	valide_utils_parent_class = g_type_class_peek_parent (klass);
	VALIDE_UTILS_CLASS (klass)->finalize = valide_utils_finalize;
}


static void valide_utils_instance_init (ValideUtils * self) {
	self->ref_count = 1;
}


static void valide_utils_finalize (ValideUtils* obj) {
	ValideUtils * self;
	self = VALIDE_UTILS (obj);
}


GType valide_utils_get_type (void) {
	static GType valide_utils_type_id = 0;
	if (valide_utils_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { valide_value_utils_init, valide_value_utils_free_value, valide_value_utils_copy_value, valide_value_utils_peek_pointer, "p", valide_value_utils_collect_value, "p", valide_value_utils_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (ValideUtilsClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_utils_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideUtils), 0, (GInstanceInitFunc) valide_utils_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		valide_utils_type_id = g_type_register_fundamental (g_type_fundamental_next (), "ValideUtils", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return valide_utils_type_id;
}


gpointer valide_utils_ref (gpointer instance) {
	ValideUtils* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void valide_utils_unref (gpointer instance) {
	ValideUtils* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		VALIDE_UTILS_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




