/* valac.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *  Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <config.h>
#include <glib/gstdio.h>
#include <gobject/gvaluecollector.h>


#define VALIDE_TYPE_VALAC (valide_valac_get_type ())
#define VALIDE_VALAC(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_VALAC, ValideValac))
#define VALIDE_VALAC_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_VALAC, ValideValacClass))
#define VALIDE_IS_VALAC(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_VALAC))
#define VALIDE_IS_VALAC_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_VALAC))
#define VALIDE_VALAC_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_VALAC, ValideValacClass))

typedef struct _ValideValac ValideValac;
typedef struct _ValideValacClass ValideValacClass;
typedef struct _ValideValacPrivate ValideValacPrivate;
typedef struct _ValideParamSpecValac ValideParamSpecValac;

/* @fixme Use static properties 
*
 * Vala compiler informations
 */
struct _ValideValac {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValideValacPrivate * priv;
};

struct _ValideValacClass {
	GTypeClass parent_class;
	void (*finalize) (ValideValac *self);
};

struct _ValideParamSpecValac {
	GParamSpec parent_instance;
};


static char* valide_valac__prefix;
static char* valide_valac__prefix = NULL;
static char* valide_valac__vapi_dir;
static char* valide_valac__vapi_dir = NULL;
static gpointer valide_valac_parent_class = NULL;

gpointer valide_valac_ref (gpointer instance);
void valide_valac_unref (gpointer instance);
GParamSpec* valide_param_spec_valac (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void valide_value_set_valac (GValue* value, gpointer v_object);
gpointer valide_value_get_valac (const GValue* value);
GType valide_valac_get_type (void);
enum  {
	VALIDE_VALAC_DUMMY_PROPERTY
};
#define VALIDE_VALAC_compiler_name "valac"
char* valide_valac_get_prefix (void);
char* valide_valac_get_vapi_dir (void);
static void _g_list_free_g_free (GList* self);
GList* valide_valac_get_package_list (GList* vapi_dir);
ValideValac* valide_valac_new (void);
ValideValac* valide_valac_construct (GType object_type);
static void valide_valac_finalize (ValideValac* obj);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);



#line 37 "valac.vala"
char* valide_valac_get_prefix (void) {
#line 94 "valac.c"
	char* result;
	const char* _tmp1_;
#line 39 "valac.vala"
	if (valide_valac__prefix == NULL) {
#line 99 "valac.c"
		char* vala_path;
		char* bin_path;
		char* _tmp0_;
		vala_path = g_find_program_in_path (VALIDE_VALAC_compiler_name);
		bin_path = g_path_get_dirname (vala_path);
		_tmp0_ = NULL;
#line 43 "valac.vala"
		valide_valac__prefix = (_tmp0_ = g_path_get_dirname (bin_path), valide_valac__prefix = (g_free (valide_valac__prefix), NULL), _tmp0_);
#line 108 "valac.c"
		vala_path = (g_free (vala_path), NULL);
		bin_path = (g_free (bin_path), NULL);
	}
#line 45 "valac.vala"
	_tmp1_ = NULL;
#line 114 "valac.c"
	result = (_tmp1_ = valide_valac__prefix, (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_));
	return result;
}


#line 52 "valac.vala"
char* valide_valac_get_vapi_dir (void) {
#line 122 "valac.c"
	char* result;
	const char* _tmp3_;
#line 54 "valac.vala"
	if (_vala_strcmp0 (OS, "win32") == 0) {
#line 127 "valac.c"
		char* _tmp0_;
		_tmp0_ = NULL;
#line 56 "valac.vala"
		valide_valac__vapi_dir = (_tmp0_ = g_build_filename ("C:", "Documents and Settings", "All Users", "Application Data", "vala", "vapi", NULL), valide_valac__vapi_dir = (g_free (valide_valac__vapi_dir), NULL), _tmp0_);
#line 132 "valac.c"
	} else {
		char* _tmp2_;
		char* _tmp1_;
		_tmp2_ = NULL;
#line 37 "valac.vala"
		_tmp1_ = NULL;
#line 60 "valac.vala"
		valide_valac__vapi_dir = (_tmp2_ = g_build_filename (_tmp1_ = valide_valac_get_prefix (), "share", "vala", "vapi", NULL), valide_valac__vapi_dir = (g_free (valide_valac__vapi_dir), NULL), _tmp2_);
#line 141 "valac.c"
		_tmp1_ = (g_free (_tmp1_), NULL);
	}
#line 62 "valac.vala"
	_tmp3_ = NULL;
#line 146 "valac.c"
	result = (_tmp3_ = valide_valac__vapi_dir, (_tmp3_ == NULL) ? NULL : g_strdup (_tmp3_));
	return result;
}


#line 922 "glib-2.0.vapi"
static char* string_substring (const char* self, glong offset, glong len) {
#line 154 "valac.c"
	char* result;
	glong string_length;
	const char* start;
#line 922 "glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 160 "valac.c"
	string_length = g_utf8_strlen (self, -1);
#line 924 "glib-2.0.vapi"
	if (offset < 0) {
#line 925 "glib-2.0.vapi"
		offset = string_length + offset;
#line 926 "glib-2.0.vapi"
		g_return_val_if_fail (offset >= 0, NULL);
#line 168 "valac.c"
	} else {
#line 928 "glib-2.0.vapi"
		g_return_val_if_fail (offset <= string_length, NULL);
#line 172 "valac.c"
	}
#line 930 "glib-2.0.vapi"
	if (len < 0) {
#line 931 "glib-2.0.vapi"
		len = string_length - offset;
#line 178 "valac.c"
	}
#line 933 "glib-2.0.vapi"
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
#line 182 "valac.c"
	start = g_utf8_offset_to_pointer (self, offset);
	result = g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
	return result;
}


static void _g_list_free_g_free (GList* self) {
	g_list_foreach (self, (GFunc) g_free, NULL);
	g_list_free (self);
}


/**
   * Get the list of the available packages
   *
   * @param vapi_dir The additionnal vapi directory
   *
   * @return The package list
   */
#line 72 "valac.vala"
GList* valide_valac_get_package_list (GList* vapi_dir) {
#line 204 "valac.c"
	GList* result;
	GError * _inner_error_;
	GList* vapi_files;
	GList* vapidir;
	_inner_error_ = NULL;
	vapi_files = NULL;
	vapidir = NULL;
#line 77 "valac.vala"
	if (vapi_dir != NULL) {
#line 214 "valac.c"
		{
			GList* v_collection;
			GList* v_it;
#line 79 "valac.vala"
			v_collection = vapi_dir;
#line 220 "valac.c"
			for (v_it = v_collection; v_it != NULL; v_it = v_it->next) {
				const char* _tmp1_;
				char* v;
#line 72 "valac.vala"
				_tmp1_ = NULL;
#line 79 "valac.vala"
				v = (_tmp1_ = (const char*) v_it->data, (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_));
#line 228 "valac.c"
				{
					const char* _tmp0_;
#line 81 "valac.vala"
					_tmp0_ = NULL;
#line 81 "valac.vala"
					vapidir = g_list_append (vapidir, (_tmp0_ = v, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)));
#line 235 "valac.c"
					v = (g_free (v), NULL);
				}
			}
		}
	}
#line 85 "valac.vala"
	vapidir = g_list_append (vapidir, valide_valac_get_vapi_dir ());
#line 243 "valac.c"
	{
		char** _tmp2_;
		char** d_collection;
		int d_collection_length1;
		int d_it;
		_tmp2_ = NULL;
#line 87 "valac.vala"
		d_collection = _tmp2_ = g_get_system_data_dirs ();
#line 252 "valac.c"
		d_collection_length1 = _vala_array_length (_tmp2_);
		for (d_it = 0; d_it < _vala_array_length (_tmp2_); d_it = d_it + 1) {
			const char* _tmp4_;
			char* d;
#line 1996 "glib-2.0.vapi"
			_tmp4_ = NULL;
#line 259 "valac.c"
			d = (_tmp4_ = d_collection[d_it], (_tmp4_ == NULL) ? NULL : g_strdup (_tmp4_));
			{
#line 89 "valac.vala"
				if (g_file_test (d, G_FILE_TEST_IS_DIR)) {
#line 264 "valac.c"
					const char* _tmp3_;
#line 91 "valac.vala"
					_tmp3_ = NULL;
#line 91 "valac.vala"
					vapidir = g_list_append (vapidir, (_tmp3_ = d, (_tmp3_ == NULL) ? NULL : g_strdup (_tmp3_)));
#line 270 "valac.c"
				}
				d = (g_free (d), NULL);
			}
		}
	}
	{
		GList* d_collection;
		GList* d_it;
#line 95 "valac.vala"
		d_collection = vapidir;
#line 281 "valac.c"
		for (d_it = d_collection; d_it != NULL; d_it = d_it->next) {
			const char* _tmp7_;
			char* d;
#line 75 "valac.vala"
			_tmp7_ = NULL;
#line 95 "valac.vala"
			d = (_tmp7_ = (const char*) d_it->data, (_tmp7_ == NULL) ? NULL : g_strdup (_tmp7_));
#line 289 "valac.c"
			{
				{
					char* filename;
					GDir* dir;
					filename = NULL;
					dir = g_dir_open (d, 0, &_inner_error_);
					if (_inner_error_ != NULL) {
						filename = (g_free (filename), NULL);
						goto __catch5_g_error;
						goto __finally5;
					}
#line 102 "valac.vala"
					while (TRUE) {
#line 303 "valac.c"
						char* _tmp6_;
						const char* _tmp5_;
						_tmp6_ = NULL;
#line 102 "valac.vala"
						_tmp5_ = NULL;
#line 102 "valac.vala"
						if (!((filename = (_tmp6_ = (_tmp5_ = g_dir_read_name (dir), (_tmp5_ == NULL) ? NULL : g_strdup (_tmp5_)), filename = (g_free (filename), NULL), _tmp6_)) != NULL)) {
#line 102 "valac.vala"
							break;
#line 313 "valac.c"
						}
#line 104 "valac.vala"
						if (g_str_has_suffix (filename, ".vapi")) {
#line 106 "valac.vala"
							vapi_files = g_list_append (vapi_files, string_substring (filename, (glong) 0, (glong) (strlen (filename) - 5)));
#line 319 "valac.c"
						}
					}
					filename = (g_free (filename), NULL);
					(dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL));
				}
				goto __finally5;
				__catch5_g_error:
				{
					GError * e;
					e = _inner_error_;
					_inner_error_ = NULL;
					{
#line 112 "valac.vala"
						g_debug (e->message);
#line 334 "valac.c"
						(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
					}
				}
				__finally5:
				if (_inner_error_ != NULL) {
					d = (g_free (d), NULL);
					(vapi_files == NULL) ? NULL : (vapi_files = (_g_list_free_g_free (vapi_files), NULL));
					(vapidir == NULL) ? NULL : (vapidir = (_g_list_free_g_free (vapidir), NULL));
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return NULL;
				}
				d = (g_free (d), NULL);
			}
		}
	}
	result = vapi_files;
	(vapidir == NULL) ? NULL : (vapidir = (_g_list_free_g_free (vapidir), NULL));
	return result;
}


/* @fixme Use static properties 
*
 * Vala compiler informations
 */
#line 26 "valac.vala"
ValideValac* valide_valac_construct (GType object_type) {
#line 363 "valac.c"
	ValideValac* self;
	self = (ValideValac*) g_type_create_instance (object_type);
	return self;
}


#line 26 "valac.vala"
ValideValac* valide_valac_new (void) {
#line 26 "valac.vala"
	return valide_valac_construct (VALIDE_TYPE_VALAC);
#line 374 "valac.c"
}


static void valide_value_valac_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void valide_value_valac_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		valide_valac_unref (value->data[0].v_pointer);
	}
}


static void valide_value_valac_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = valide_valac_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer valide_value_valac_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* valide_value_valac_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		ValideValac* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = valide_valac_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* valide_value_valac_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	ValideValac** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = valide_valac_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* valide_param_spec_valac (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ValideParamSpecValac* spec;
	g_return_val_if_fail (g_type_is_a (object_type, VALIDE_TYPE_VALAC), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer valide_value_get_valac (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALIDE_TYPE_VALAC), NULL);
	return value->data[0].v_pointer;
}


void valide_value_set_valac (GValue* value, gpointer v_object) {
	ValideValac* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALIDE_TYPE_VALAC));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, VALIDE_TYPE_VALAC));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		valide_valac_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		valide_valac_unref (old);
	}
}


static void valide_valac_class_init (ValideValacClass * klass) {
	valide_valac_parent_class = g_type_class_peek_parent (klass);
	VALIDE_VALAC_CLASS (klass)->finalize = valide_valac_finalize;
}


static void valide_valac_instance_init (ValideValac * self) {
	self->ref_count = 1;
}


static void valide_valac_finalize (ValideValac* obj) {
	ValideValac * self;
	self = VALIDE_VALAC (obj);
}


GType valide_valac_get_type (void) {
	static GType valide_valac_type_id = 0;
	if (valide_valac_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { valide_value_valac_init, valide_value_valac_free_value, valide_value_valac_copy_value, valide_value_valac_peek_pointer, "p", valide_value_valac_collect_value, "p", valide_value_valac_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (ValideValacClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_valac_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideValac), 0, (GInstanceInitFunc) valide_valac_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		valide_valac_type_id = g_type_register_fundamental (g_type_fundamental_next (), "ValideValac", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return valide_valac_type_id;
}


gpointer valide_valac_ref (gpointer instance) {
	ValideValac* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void valide_valac_unref (gpointer instance) {
	ValideValac* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		VALIDE_VALAC_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




