/* window.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <stdlib.h>
#include <string.h>
#include <glib/gi18n-lib.h>
#include <config.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <gdk/gdkkeysyms.h>


#define VALIDE_TYPE_FULLSCREEN_TOOLBAR (valide_fullscreen_toolbar_get_type ())
#define VALIDE_FULLSCREEN_TOOLBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_FULLSCREEN_TOOLBAR, ValideFullscreenToolbar))
#define VALIDE_FULLSCREEN_TOOLBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_FULLSCREEN_TOOLBAR, ValideFullscreenToolbarClass))
#define VALIDE_IS_FULLSCREEN_TOOLBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_FULLSCREEN_TOOLBAR))
#define VALIDE_IS_FULLSCREEN_TOOLBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_FULLSCREEN_TOOLBAR))
#define VALIDE_FULLSCREEN_TOOLBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_FULLSCREEN_TOOLBAR, ValideFullscreenToolbarClass))

typedef struct _ValideFullscreenToolbar ValideFullscreenToolbar;
typedef struct _ValideFullscreenToolbarClass ValideFullscreenToolbarClass;
typedef struct _ValideFullscreenToolbarPrivate ValideFullscreenToolbarPrivate;

#define VALIDE_TYPE_WINDOW (valide_window_get_type ())
#define VALIDE_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_WINDOW, ValideWindow))
#define VALIDE_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_WINDOW, ValideWindowClass))
#define VALIDE_IS_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_WINDOW))
#define VALIDE_IS_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_WINDOW))
#define VALIDE_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_WINDOW, ValideWindowClass))

typedef struct _ValideWindow ValideWindow;
typedef struct _ValideWindowClass ValideWindowClass;

#define VALIDE_TYPE_UI_MANAGER (valide_ui_manager_get_type ())
#define VALIDE_UI_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_UI_MANAGER, ValideUIManager))
#define VALIDE_UI_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_UI_MANAGER, ValideUIManagerClass))
#define VALIDE_IS_UI_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_UI_MANAGER))
#define VALIDE_IS_UI_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_UI_MANAGER))
#define VALIDE_UI_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_UI_MANAGER, ValideUIManagerClass))

typedef struct _ValideUIManager ValideUIManager;
typedef struct _ValideUIManagerClass ValideUIManagerClass;
typedef struct _ValideWindowPrivate ValideWindowPrivate;

#define VALIDE_TYPE_START_PAGE (valide_start_page_get_type ())
#define VALIDE_START_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_START_PAGE, ValideStartPage))
#define VALIDE_START_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_START_PAGE, ValideStartPageClass))
#define VALIDE_IS_START_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_START_PAGE))
#define VALIDE_IS_START_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_START_PAGE))
#define VALIDE_START_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_START_PAGE, ValideStartPageClass))

typedef struct _ValideStartPage ValideStartPage;
typedef struct _ValideStartPageClass ValideStartPageClass;

#define VALIDE_TYPE_RECENT_MANAGER_UI (valide_recent_manager_ui_get_type ())
#define VALIDE_RECENT_MANAGER_UI(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_RECENT_MANAGER_UI, ValideRecentManagerUI))
#define VALIDE_RECENT_MANAGER_UI_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_RECENT_MANAGER_UI, ValideRecentManagerUIClass))
#define VALIDE_IS_RECENT_MANAGER_UI(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_RECENT_MANAGER_UI))
#define VALIDE_IS_RECENT_MANAGER_UI_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_RECENT_MANAGER_UI))
#define VALIDE_RECENT_MANAGER_UI_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_RECENT_MANAGER_UI, ValideRecentManagerUIClass))

typedef struct _ValideRecentManagerUI ValideRecentManagerUI;
typedef struct _ValideRecentManagerUIClass ValideRecentManagerUIClass;

#define VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER (valide_abstract_config_manager_get_type ())
#define VALIDE_ABSTRACT_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManager))
#define VALIDE_ABSTRACT_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManagerClass))
#define VALIDE_IS_ABSTRACT_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER))
#define VALIDE_IS_ABSTRACT_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER))
#define VALIDE_ABSTRACT_CONFIG_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABSTRACT_CONFIG_MANAGER, ValideAbstractConfigManagerClass))

typedef struct _ValideAbstractConfigManager ValideAbstractConfigManager;
typedef struct _ValideAbstractConfigManagerClass ValideAbstractConfigManagerClass;

#define VALIDE_TYPE_CONFIG_MANAGER (valide_config_manager_get_type ())
#define VALIDE_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_CONFIG_MANAGER, ValideConfigManager))
#define VALIDE_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_CONFIG_MANAGER, ValideConfigManagerClass))
#define VALIDE_IS_CONFIG_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_CONFIG_MANAGER))
#define VALIDE_IS_CONFIG_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_CONFIG_MANAGER))
#define VALIDE_CONFIG_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_CONFIG_MANAGER, ValideConfigManagerClass))

typedef struct _ValideConfigManager ValideConfigManager;
typedef struct _ValideConfigManagerClass ValideConfigManagerClass;

#define VALIDE_TYPE_DOCUMENT_MANAGER (valide_document_manager_get_type ())
#define VALIDE_DOCUMENT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_DOCUMENT_MANAGER, ValideDocumentManager))
#define VALIDE_DOCUMENT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_DOCUMENT_MANAGER, ValideDocumentManagerClass))
#define VALIDE_IS_DOCUMENT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_DOCUMENT_MANAGER))
#define VALIDE_IS_DOCUMENT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_DOCUMENT_MANAGER))
#define VALIDE_DOCUMENT_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_DOCUMENT_MANAGER, ValideDocumentManagerClass))

typedef struct _ValideDocumentManager ValideDocumentManager;
typedef struct _ValideDocumentManagerClass ValideDocumentManagerClass;

#define VALIDE_TYPE_PROJECT_MANAGER (valide_project_manager_get_type ())
#define VALIDE_PROJECT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PROJECT_MANAGER, ValideProjectManager))
#define VALIDE_PROJECT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PROJECT_MANAGER, ValideProjectManagerClass))
#define VALIDE_IS_PROJECT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PROJECT_MANAGER))
#define VALIDE_IS_PROJECT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PROJECT_MANAGER))
#define VALIDE_PROJECT_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PROJECT_MANAGER, ValideProjectManagerClass))

typedef struct _ValideProjectManager ValideProjectManager;
typedef struct _ValideProjectManagerClass ValideProjectManagerClass;

#define VALIDE_TYPE_BUILDER_MANAGER (valide_builder_manager_get_type ())
#define VALIDE_BUILDER_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_BUILDER_MANAGER, ValideBuilderManager))
#define VALIDE_BUILDER_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_BUILDER_MANAGER, ValideBuilderManagerClass))
#define VALIDE_IS_BUILDER_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_BUILDER_MANAGER))
#define VALIDE_IS_BUILDER_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_BUILDER_MANAGER))
#define VALIDE_BUILDER_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_BUILDER_MANAGER, ValideBuilderManagerClass))

typedef struct _ValideBuilderManager ValideBuilderManager;
typedef struct _ValideBuilderManagerClass ValideBuilderManagerClass;

#define VALIDE_TYPE_EXECUTABLE_MANAGER (valide_executable_manager_get_type ())
#define VALIDE_EXECUTABLE_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_EXECUTABLE_MANAGER, ValideExecutableManager))
#define VALIDE_EXECUTABLE_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_EXECUTABLE_MANAGER, ValideExecutableManagerClass))
#define VALIDE_IS_EXECUTABLE_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_EXECUTABLE_MANAGER))
#define VALIDE_IS_EXECUTABLE_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_EXECUTABLE_MANAGER))
#define VALIDE_EXECUTABLE_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_EXECUTABLE_MANAGER, ValideExecutableManagerClass))

typedef struct _ValideExecutableManager ValideExecutableManager;
typedef struct _ValideExecutableManagerClass ValideExecutableManagerClass;

#define VALIDE_TYPE_PLUGIN_MANAGER (valide_plugin_manager_get_type ())
#define VALIDE_PLUGIN_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PLUGIN_MANAGER, ValidePluginManager))
#define VALIDE_PLUGIN_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PLUGIN_MANAGER, ValidePluginManagerClass))
#define VALIDE_IS_PLUGIN_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PLUGIN_MANAGER))
#define VALIDE_IS_PLUGIN_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PLUGIN_MANAGER))
#define VALIDE_PLUGIN_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PLUGIN_MANAGER, ValidePluginManagerClass))

typedef struct _ValidePluginManager ValidePluginManager;
typedef struct _ValidePluginManagerClass ValidePluginManagerClass;

#define VALIDE_TYPE_PANEL (valide_panel_get_type ())
#define VALIDE_PANEL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PANEL, ValidePanel))
#define VALIDE_PANEL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PANEL, ValidePanelClass))
#define VALIDE_IS_PANEL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PANEL))
#define VALIDE_IS_PANEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PANEL))
#define VALIDE_PANEL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PANEL, ValidePanelClass))

typedef struct _ValidePanel ValidePanel;
typedef struct _ValidePanelClass ValidePanelClass;

#define VALIDE_TYPE_STATUSBAR (valide_statusbar_get_type ())
#define VALIDE_STATUSBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_STATUSBAR, ValideStatusbar))
#define VALIDE_STATUSBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_STATUSBAR, ValideStatusbarClass))
#define VALIDE_IS_STATUSBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_STATUSBAR))
#define VALIDE_IS_STATUSBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_STATUSBAR))
#define VALIDE_STATUSBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_STATUSBAR, ValideStatusbarClass))

typedef struct _ValideStatusbar ValideStatusbar;
typedef struct _ValideStatusbarClass ValideStatusbarClass;

#define VALIDE_TYPE_PROJECT (valide_project_get_type ())
#define VALIDE_PROJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_PROJECT, ValideProject))
#define VALIDE_PROJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_PROJECT, ValideProjectClass))
#define VALIDE_IS_PROJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_PROJECT))
#define VALIDE_IS_PROJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_PROJECT))
#define VALIDE_PROJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_PROJECT, ValideProjectClass))

typedef struct _ValideProject ValideProject;
typedef struct _ValideProjectClass ValideProjectClass;

#define VALIDE_TYPE_IDOCUMENT (valide_idocument_get_type ())
#define VALIDE_IDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_IDOCUMENT, ValideiDocument))
#define VALIDE_IS_IDOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_IDOCUMENT))
#define VALIDE_IDOCUMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), VALIDE_TYPE_IDOCUMENT, ValideiDocumentIface))

typedef struct _ValideiDocument ValideiDocument;
typedef struct _ValideiDocumentIface ValideiDocumentIface;

#define VALIDE_TYPE_DOCUMENT_STATE (valide_document_state_get_type ())

#define VALIDE_TYPE_ABOUT_DIALOG (valide_about_dialog_get_type ())
#define VALIDE_ABOUT_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_ABOUT_DIALOG, ValideAboutDialog))
#define VALIDE_ABOUT_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_ABOUT_DIALOG, ValideAboutDialogClass))
#define VALIDE_IS_ABOUT_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_ABOUT_DIALOG))
#define VALIDE_IS_ABOUT_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_ABOUT_DIALOG))
#define VALIDE_ABOUT_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_ABOUT_DIALOG, ValideAboutDialogClass))

typedef struct _ValideAboutDialog ValideAboutDialog;
typedef struct _ValideAboutDialogClass ValideAboutDialogClass;

#define VALIDE_TYPE_EXECUTABLE (valide_executable_get_type ())
#define VALIDE_EXECUTABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_EXECUTABLE, ValideExecutable))
#define VALIDE_EXECUTABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_EXECUTABLE, ValideExecutableClass))
#define VALIDE_IS_EXECUTABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_EXECUTABLE))
#define VALIDE_IS_EXECUTABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_EXECUTABLE))
#define VALIDE_EXECUTABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_EXECUTABLE, ValideExecutableClass))

typedef struct _ValideExecutable ValideExecutable;
typedef struct _ValideExecutableClass ValideExecutableClass;

struct _ValideFullscreenToolbar {
	GtkWindow parent_instance;
	ValideFullscreenToolbarPrivate * priv;
};

struct _ValideFullscreenToolbarClass {
	GtkWindowClass parent_class;
};

struct _ValideFullscreenToolbarPrivate {
	gboolean animation_enter;
	guint animation_timeout_id;
	ValideWindow* _parent;
};

/**
 * The main window of the application
 */
struct _ValideWindow {
	GtkWindow parent_instance;
	ValideWindowPrivate * priv;
	guint workspace;
};

struct _ValideWindowClass {
	GtkWindowClass parent_class;
};

struct _ValideWindowPrivate {
	guint status_context;
	ValideStartPage* start_page;
	GtkWindowGroup* owned_group;
	GdkWindowState window_state;
	ValideFullscreenToolbar* fullscreen_toolbar;
	ValideUIManager* _ui_manager;
	ValideRecentManagerUI* _recent_manager;
	ValideConfigManager* _config_manager;
	ValideDocumentManager* _documents;
	ValideProjectManager* _projects;
	ValideBuilderManager* _builders;
	ValideExecutableManager* _executables;
	ValidePluginManager* _plugins;
	GtkMenuBar* _menu;
	GtkToolbar* _toolbar;
	ValidePanel* _side_panel;
	ValidePanel* _bottom_panel;
	ValideStatusbar* _statusbar;
};

typedef enum  {
	VALIDE_EXECUTABLE_ERROR_NO_EXECUTABLE
} ValideExecutableError;
#define VALIDE_EXECUTABLE_ERROR valide_executable_error_quark ()
typedef enum  {
	VALIDE_DOCUMENT_ERROR_UNKNOW_DOCUMENT,
	VALIDE_DOCUMENT_ERROR_BAD_URI,
	VALIDE_DOCUMENT_ERROR_NO_DOCUMENT,
	VALIDE_DOCUMENT_ERROR_MIME
} ValideDocumentError;
#define VALIDE_DOCUMENT_ERROR valide_document_error_quark ()
typedef enum  {
	VALIDE_DOCUMENT_STATE_NORMAL,
	VALIDE_DOCUMENT_STATE_EXTERNALLY_MODIFIED,
	VALIDE_DOCUMENT_STATE_BAD_ENCODING,
	VALIDE_DOCUMENT_STATE_INFO
} ValideDocumentState;

/**
 * An interface for document
 */
struct _ValideiDocumentIface {
	GTypeInterface parent_iface;
	void (*open) (ValideiDocument* self, const char* path, GError** error);
	void (*save) (ValideiDocument* self, const char* current_folder);
	void (*save_as) (ValideiDocument* self, const char* current_folder);
	void (*rename) (ValideiDocument* self);
	gboolean (*close) (ValideiDocument* self);
	GdkPixbuf* (*get_icon) (ValideiDocument* self);
	char* (*get_tooltips) (ValideiDocument* self);
	void (*setup_ui) (ValideiDocument* self, ValideUIManager* ui_manager);
	ValideDocumentState (*get_state) (ValideiDocument* self);
	void (*set_state) (ValideiDocument* self, ValideDocumentState value);
	gboolean (*get_is_save) (ValideiDocument* self);
	void (*set_is_save) (ValideiDocument* self, gboolean value);
	const char* (*get_path) (ValideiDocument* self);
	void (*set_path) (ValideiDocument* self, const char* value);
	const char* (*get_title) (ValideiDocument* self);
	const char* (*get_filename) (ValideiDocument* self);
};

typedef enum  {
	VALIDE_PROJECT_ERROR_NO_PROJECT,
	VALIDE_PROJECT_ERROR_MAX_OPEN
} ValideProjectError;
#define VALIDE_PROJECT_ERROR valide_project_error_quark ()

static gpointer valide_fullscreen_toolbar_parent_class = NULL;
static gpointer valide_window_parent_class = NULL;

GType valide_fullscreen_toolbar_get_type (void);
GType valide_window_get_type (void);
#define VALIDE_FULLSCREEN_TOOLBAR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_FULLSCREEN_TOOLBAR, ValideFullscreenToolbarPrivate))
enum  {
	VALIDE_FULLSCREEN_TOOLBAR_DUMMY_PROPERTY,
	VALIDE_FULLSCREEN_TOOLBAR_PARENT
};
static void valide_fullscreen_toolbar_show_hide (ValideFullscreenToolbar* self, gboolean show, gint height);
static gboolean valide_fullscreen_toolbar_on_enter_notify_event (ValideFullscreenToolbar* self, GtkWidget* sender, const GdkEventCrossing* event);
ValideWindow* valide_fullscreen_toolbar_get_parent (ValideFullscreenToolbar* self);
static gboolean valide_fullscreen_toolbar_on_leave_notify_event (ValideFullscreenToolbar* self, GtkWidget* sender, const GdkEventCrossing* event);
static gboolean valide_fullscreen_toolbar_run_animation (ValideFullscreenToolbar* self);
static gboolean _valide_fullscreen_toolbar_run_animation_gsource_func (gpointer self);
static void valide_fullscreen_toolbar_set_parent (ValideFullscreenToolbar* self, ValideWindow* value);
ValideFullscreenToolbar* valide_fullscreen_toolbar_new (ValideWindow* parent);
ValideFullscreenToolbar* valide_fullscreen_toolbar_construct (GType object_type, ValideWindow* parent);
void valide_fullscreen_toolbar_show_all (ValideFullscreenToolbar* self);
void valide_fullscreen_toolbar_show (ValideFullscreenToolbar* self);
GType valide_ui_manager_get_type (void);
ValideUIManager* valide_window_get_ui_manager (ValideWindow* self);
GtkAction* valide_ui_manager_get_action (ValideUIManager* self, const char* name);
static gboolean _valide_fullscreen_toolbar_on_enter_notify_event_gtk_widget_enter_notify_event (GtkWidget* _sender, const GdkEventCrossing* event, gpointer self);
static gboolean _valide_fullscreen_toolbar_on_leave_notify_event_gtk_widget_leave_notify_event (GtkWidget* _sender, const GdkEventCrossing* event, gpointer self);
static GObject * valide_fullscreen_toolbar_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void valide_fullscreen_toolbar_finalize (GObject* obj);
static void valide_fullscreen_toolbar_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_fullscreen_toolbar_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType valide_start_page_get_type (void);
GType valide_recent_manager_ui_get_type (void);
GType valide_abstract_config_manager_get_type (void);
GType valide_config_manager_get_type (void);
GType valide_document_manager_get_type (void);
GType valide_project_manager_get_type (void);
GType valide_builder_manager_get_type (void);
GType valide_executable_manager_get_type (void);
GType valide_plugin_manager_get_type (void);
GType valide_panel_get_type (void);
GType valide_statusbar_get_type (void);
#define VALIDE_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_WINDOW, ValideWindowPrivate))
enum  {
	VALIDE_WINDOW_DUMMY_PROPERTY,
	VALIDE_WINDOW_UI_MANAGER,
	VALIDE_WINDOW_RECENT_MANAGER,
	VALIDE_WINDOW_CONFIG_MANAGER,
	VALIDE_WINDOW_DOCUMENTS,
	VALIDE_WINDOW_PROJECTS,
	VALIDE_WINDOW_BUILDERS,
	VALIDE_WINDOW_EXECUTABLES,
	VALIDE_WINDOW_PLUGINS,
	VALIDE_WINDOW_MENU,
	VALIDE_WINDOW_TOOLBAR,
	VALIDE_WINDOW_SIDE_PANEL,
	VALIDE_WINDOW_BOTTOM_PANEL,
	VALIDE_WINDOW_STATUSBAR
};
static void valide_window_message (ValideWindow* self, const char* log_domain, GLogLevelFlags log_levels, const char* message);
ValideConfigManager* valide_window_get_config_manager (ValideWindow* self);
void valide_abstract_config_manager_set_integer (ValideAbstractConfigManager* self, const char* group, const char* key, gint val);
ValideExecutableManager* valide_window_get_executables (ValideWindow* self);
GQuark valide_executable_error_quark (void);
void valide_executable_manager_stop (ValideExecutableManager* self, GError** error);
ValideDocumentManager* valide_window_get_documents (ValideWindow* self);
gboolean valide_document_manager_close_all (ValideDocumentManager* self);
static gboolean valide_window_on_delete_event (ValideWindow* self);
ValideStatusbar* valide_window_get_statusbar (ValideWindow* self);
static void valide_window_menu_item_select_cb (ValideWindow* self, GtkItem* item);
static void valide_window_menu_item_deselect_cb (ValideWindow* self, GtkItem* item);
static void _valide_window_menu_item_select_cb_gtk_item_select (GtkItem* _sender, gpointer self);
static void _valide_window_menu_item_deselect_cb_gtk_item_deselect (GtkItem* _sender, gpointer self);
static void valide_window_connect_proxy_cb (ValideWindow* self, GtkUIManager* sender, GtkAction* action, GtkWidget* proxy);
static void valide_window_disconnect_proxy_cb (ValideWindow* self, GtkUIManager* sender, GtkAction* action, GtkWidget* proxy);
ValideUIManager* valide_ui_manager_new (void);
ValideUIManager* valide_ui_manager_construct (GType object_type);
static void valide_window_set_ui_manager (ValideWindow* self, ValideUIManager* value);
static void _valide_window_connect_proxy_cb_gtk_ui_manager_connect_proxy (GtkUIManager* _sender, GtkAction* action, GtkWidget* proxy, gpointer self);
static void _valide_window_disconnect_proxy_cb_gtk_ui_manager_disconnect_proxy (GtkUIManager* _sender, GtkAction* action, GtkWidget* proxy, gpointer self);
void valide_ui_manager_populate (ValideUIManager* self);
static void valide_window_set_menu (ValideWindow* self, GtkMenuBar* value);
static void valide_window_set_toolbar (ValideWindow* self, GtkToolbar* value);
void valide_ui_manager_action_set_toggled (ValideUIManager* self, const char* name, gboolean active);
static void valide_window_setup_ui_manager (ValideWindow* self);
ValideRecentManagerUI* valide_window_get_recent_manager (ValideWindow* self);
void valide_recent_manager_ui_update_file (ValideRecentManagerUI* self);
void valide_recent_manager_ui_update_project (ValideRecentManagerUI* self);
GtkMenuBar* valide_window_get_menu (ValideWindow* self);
GtkToolbar* valide_window_get_toolbar (ValideWindow* self);
ValidePanel* valide_panel_new (GtkOrientation orientation);
ValidePanel* valide_panel_construct (GType object_type, GtkOrientation orientation);
static void valide_window_set_bottom_panel (ValideWindow* self, ValidePanel* value);
ValidePanel* valide_window_get_bottom_panel (ValideWindow* self);
void valide_panel_add_item_with_stock_icon (ValidePanel* self, GtkWidget* item, const char* name, const char* stock_id);
static void _lambda46_ (ValideWindow* self);
static void __lambda46__gtk_widget_hide (GtkWidget* _sender, gpointer self);
static void valide_window_set_side_panel (ValideWindow* self, ValidePanel* value);
ValidePanel* valide_window_get_side_panel (ValideWindow* self);
void valide_panel_add_item (ValidePanel* self, GtkWidget* item, const char* name, GtkImage* image);
ValideProjectManager* valide_window_get_projects (ValideWindow* self);
static void _lambda47_ (ValideWindow* self);
static void __lambda47__gtk_widget_hide (GtkWidget* _sender, gpointer self);
static void valide_window_populate_window (ValideWindow* self);
GType valide_project_get_type (void);
ValideProject* valide_project_manager_get_current (ValideProjectManager* self);
GQuark valide_document_error_quark (void);
void valide_project_create_file (ValideProject* self, GError** error);
GType valide_document_state_get_type (void);
GType valide_idocument_get_type (void);
ValideiDocument* valide_document_manager_create (ValideDocumentManager* self, const char* filename, GError** error);
GQuark valide_project_error_quark (void);
void valide_document_manager_open (ValideDocumentManager* self, GError** error);
ValideiDocument* valide_document_manager_get_current (ValideDocumentManager* self);
const char* valide_project_get_path (ValideProject* self);
void valide_idocument_save (ValideiDocument* self, const char* current_folder);
void valide_idocument_save_as (ValideiDocument* self, const char* current_folder);
gboolean valide_document_manager_save_all (ValideDocumentManager* self);
void valide_idocument_rename (ValideiDocument* self);
gboolean valide_idocument_close (ValideiDocument* self);
void valide_abstract_config_manager_preferences (ValideAbstractConfigManager* self, ValideWindow* window);
void valide_window_show_start_page (ValideWindow* self);
void valide_window_hide_start_page (ValideWindow* self);
void valide_window_fullscreen (ValideWindow* self);
void valide_window_unfullscreen (ValideWindow* self);
gboolean valide_project_manager_create (ValideProjectManager* self);
gboolean valide_project_manager_open (ValideProjectManager* self, const char* filename, GError** error);
void valide_project_add_file (ValideProject* self, GSList* files);
void valide_project_remove_file (ValideProject* self, GList* files);
void valide_project_option_dialog (ValideProject* self, GtkWindow* parent);
void valide_project_configure (ValideProject* self);
void valide_project_build (ValideProject* self);
void valide_project_install (ValideProject* self);
void valide_project_uninstall (ValideProject* self);
void valide_project_dist (ValideProject* self);
void valide_project_clean (ValideProject* self);
void valide_project_distclean (ValideProject* self);
void valide_project_execute (ValideProject* self);
void valide_project_close (ValideProject* self);
ValideAboutDialog* valide_about_dialog_new (void);
ValideAboutDialog* valide_about_dialog_construct (GType object_type);
GType valide_about_dialog_get_type (void);
void valide_project_manager_open_selected_file (ValideProjectManager* self);
void valide_project_manager_remove_selected_file (ValideProjectManager* self);
static void valide_window_on_action_activated (ValideWindow* self, GtkUIManager* sender, GtkAction* action);
const char* valide_idocument_get_path (ValideiDocument* self);
char* valide_utils_replace_home_dir_with_tilde (const char* uri);
const char* valide_project_get_name (ValideProject* self);
static void valide_window_update_title (ValideWindow* self);
static gboolean valide_window_is_fullscreen (ValideWindow* self);
gboolean valide_abstract_config_manager_get_boolean (ValideAbstractConfigManager* self, const char* group, const char* key);
gint valide_abstract_config_manager_get_integer (ValideAbstractConfigManager* self, const char* group, const char* key);
static void _valide_window_message_glog_func (const char* log_domain, GLogLevelFlags log_levels, const char* message, gpointer self);
void valide_window_run (ValideWindow* self);
void valide_panel_set_current_page (ValidePanel* self, GtkWidget* widget);
void valide_window_side_panel_show (ValideWindow* self, GtkWidget* widget);
void valide_window_side_panel_hide (ValideWindow* self);
void valide_window_bottom_panel_show (ValideWindow* self, GtkWidget* widget);
void valide_window_bottom_panel_hide (ValideWindow* self);
gint valide_document_manager_append_page (ValideDocumentManager* self, ValideiDocument* child);
void valide_document_manager_set_current (ValideDocumentManager* self, ValideiDocument* value);
void valide_document_manager_remove_page (ValideDocumentManager* self, ValideiDocument* child);
ValideWindow* valide_window_new (void);
ValideWindow* valide_window_construct (GType object_type);
static void valide_window_set_recent_manager (ValideWindow* self, ValideRecentManagerUI* value);
static void valide_window_set_config_manager (ValideWindow* self, ValideConfigManager* value);
static void valide_window_set_documents (ValideWindow* self, ValideDocumentManager* value);
static void valide_window_set_projects (ValideWindow* self, ValideProjectManager* value);
ValideBuilderManager* valide_window_get_builders (ValideWindow* self);
static void valide_window_set_builders (ValideWindow* self, ValideBuilderManager* value);
static void valide_window_set_executables (ValideWindow* self, ValideExecutableManager* value);
ValidePluginManager* valide_window_get_plugins (ValideWindow* self);
static void valide_window_set_plugins (ValideWindow* self, ValidePluginManager* value);
static void valide_window_set_statusbar (ValideWindow* self, ValideStatusbar* value);
static gboolean _valide_window_on_delete_event_gtk_widget_delete_event (GtkWidget* _sender, GdkEvent* event, gpointer self);
void valide_statusbar_set_has_resize_grip (ValideStatusbar* self, gboolean show);
void valide_abstract_config_manager_set_boolean (ValideAbstractConfigManager* self, const char* group, const char* key, gboolean val);
static gboolean _lambda54_ (GtkWidget* s, const GdkEventWindowState* e, ValideWindow* self);
static gboolean __lambda54__gtk_widget_window_state_event (GtkWidget* _sender, const GdkEventWindowState* event, gpointer self);
ValideConfigManager* valide_config_manager_get_instance (void);
ValideDocumentManager* valide_document_manager_new (ValideUIManager* ui_manager);
ValideDocumentManager* valide_document_manager_construct (GType object_type, ValideUIManager* ui_manager);
static void _valide_window_update_title_valide_document_manager_tab_changed (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self);
static void _valide_window_update_title_valide_document_manager_tab_state_changed (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self);
static void _valide_window_update_title_valide_document_manager_tab_added (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self);
static void _valide_window_update_title_valide_document_manager_tab_removed (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self);
void valide_statusbar_set_cursor_position (ValideStatusbar* self, gint line, gint col);
static void _lambda55_ (ValideDocumentManager* s, ValideiDocument* d, gint r, gint c, ValideWindow* self);
static void __lambda55__valide_document_manager_tab_cursor_moved (ValideDocumentManager* _sender, ValideiDocument* document, gint row, gint col, gpointer self);
ValideExecutableManager* valide_executable_manager_new (ValideUIManager* ui_manager);
ValideExecutableManager* valide_executable_manager_construct (GType object_type, ValideUIManager* ui_manager);
static void _lambda56_ (ValideWindow* self);
GType valide_executable_get_type (void);
static void __lambda56__valide_executable_manager_run_exec (ValideExecutableManager* _sender, ValideExecutable* executable, gpointer self);
ValideBuilderManager* valide_builder_manager_get_instance (void);
void valide_builder_manager_set_executables (ValideBuilderManager* self, ValideExecutableManager* value);
ValideProjectManager* valide_project_manager_new (ValideUIManager* ui_manager, ValideDocumentManager* documents, ValideBuilderManager* builders);
ValideProjectManager* valide_project_manager_construct (GType object_type, ValideUIManager* ui_manager, ValideDocumentManager* documents, ValideBuilderManager* builders);
static void _lambda57_ (ValideWindow* self);
static void __lambda57__valide_project_manager_project_opened (ValideProjectManager* _sender, ValideProject* project, gpointer self);
static void _lambda58_ (ValideWindow* self);
static void __lambda58__valide_project_manager_project_options_changed (ValideProjectManager* _sender, ValideProject* project, gpointer self);
static void _valide_window_update_title_valide_project_manager_project_opened (ValideProjectManager* _sender, ValideProject* project, gpointer self);
static void _valide_window_update_title_valide_project_manager_project_closed (ValideProjectManager* _sender, gpointer self);
ValideRecentManagerUI* valide_recent_manager_ui_new (ValideUIManager* ui_manager, ValideDocumentManager* documents, ValideProjectManager* projects);
ValideRecentManagerUI* valide_recent_manager_ui_construct (GType object_type, ValideUIManager* ui_manager, ValideDocumentManager* documents, ValideProjectManager* projects);
ValideStatusbar* valide_statusbar_new (void);
ValideStatusbar* valide_statusbar_construct (GType object_type);
ValideStartPage* valide_start_page_new (ValideDocumentManager* documents, ValideProjectManager* projects, ValideRecentManagerUI* recents);
ValideStartPage* valide_start_page_construct (GType object_type, ValideDocumentManager* documents, ValideProjectManager* projects, ValideRecentManagerUI* recents);
static void _lambda59_ (ValideWindow* self);
static void __lambda59__valide_idocument_closed (ValideiDocument* _sender, gpointer self);
void valide_statusbar_flash_message (ValideStatusbar* self, guint context_id, const char* msg);
static void _lambda60_ (ValideStartPage* s, const char* m, ValideWindow* self);
static void __lambda60__valide_start_page_send_message (ValideStartPage* _sender, const char* message, gpointer self);
static void _valide_window_on_action_activated_gtk_ui_manager_post_activate (GtkUIManager* _sender, GtkAction* action, gpointer self);
static void _g_list_free_g_object_unref (GList* self);
ValidePluginManager* valide_plugin_manager_new (ValideWindow* window);
ValidePluginManager* valide_plugin_manager_construct (GType object_type, ValideWindow* window);
static GObject * valide_window_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void valide_window_finalize (GObject* obj);
static void valide_window_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_window_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static int _vala_strcmp0 (const char * str1, const char * str2);



#line 33 "window.vala"
static gboolean valide_fullscreen_toolbar_on_enter_notify_event (ValideFullscreenToolbar* self, GtkWidget* sender, const GdkEventCrossing* event) {
#line 541 "window.c"
	gboolean result;
#line 33 "window.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 33 "window.vala"
	g_return_val_if_fail (sender != NULL, FALSE);
#line 35 "window.vala"
	valide_fullscreen_toolbar_show_hide (self, TRUE, 0);
#line 549 "window.c"
	result = FALSE;
	return result;
}


#line 39 "window.vala"
static gboolean valide_fullscreen_toolbar_on_leave_notify_event (ValideFullscreenToolbar* self, GtkWidget* sender, const GdkEventCrossing* event) {
#line 557 "window.c"
	gboolean result;
	gint w;
	gint h;
	gint x;
	gint y;
	GdkScreen* screen;
	GdkDisplay* display;
	GdkDisplay* _tmp1_;
	GdkDisplay* _tmp0_;
	GdkScreen* _tmp3_;
	GdkScreen* _tmp2_;
	GdkScreen* _tmp6_;
	GdkScreen* _tmp5_;
	GdkScreen* _tmp4_;
#line 39 "window.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 39 "window.vala"
	g_return_val_if_fail (sender != NULL, FALSE);
#line 576 "window.c"
	w = 0;
	h = 0;
	x = 0;
	y = 0;
	screen = NULL;
	display = NULL;
	_tmp1_ = NULL;
#line 46 "window.vala"
	_tmp0_ = NULL;
#line 46 "window.vala"
	display = (_tmp1_ = (_tmp0_ = gdk_display_get_default (), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (display == NULL) ? NULL : (display = (g_object_unref (display), NULL)), _tmp1_);
#line 588 "window.c"
	_tmp3_ = NULL;
#line 47 "window.vala"
	_tmp2_ = NULL;
#line 47 "window.vala"
	screen = (_tmp3_ = (_tmp2_ = gtk_window_get_screen ((GtkWindow*) self->priv->_parent), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL)), _tmp3_);
#line 49 "window.vala"
	gtk_window_get_size ((GtkWindow*) self, &w, &h);
#line 50 "window.vala"
	_tmp6_ = NULL;
#line 598 "window.c"
	_tmp5_ = NULL;
	_tmp4_ = NULL;
#line 50 "window.vala"
	gdk_display_get_pointer (display, &_tmp4_, &x, &y, NULL);
#line 50 "window.vala"
	screen = (_tmp5_ = (_tmp6_ = _tmp4_, (_tmp6_ == NULL) ? NULL : g_object_ref (_tmp6_)), (screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL)), _tmp5_);
#line 605 "window.c"
	/*
	     * gtk seems to emit leave notify when clicking on tool items,
	     * work around it by checking the coordinates
	     */
#line 56 "window.vala"
	if (y >= h) {
#line 58 "window.vala"
		valide_fullscreen_toolbar_show_hide (self, FALSE, h);
#line 614 "window.c"
	}
	result = FALSE;
	(screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL));
	(display == NULL) ? NULL : (display = (g_object_unref (display), NULL));
	return result;
}


#line 64 "window.vala"
static gboolean valide_fullscreen_toolbar_run_animation (ValideFullscreenToolbar* self) {
#line 625 "window.c"
	gboolean result;
	gint x;
	gint y;
	GdkScreen* screen;
	GdkRectangle fs_rect = {0};
	GdkScreen* _tmp1_;
	GdkScreen* _tmp0_;
#line 64 "window.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 635 "window.c"
	x = 0;
	y = 0;
	screen = NULL;
	_tmp1_ = NULL;
#line 70 "window.vala"
	_tmp0_ = NULL;
#line 70 "window.vala"
	screen = (_tmp1_ = (_tmp0_ = gtk_window_get_screen ((GtkWindow*) self->priv->_parent), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL)), _tmp1_);
#line 71 "window.vala"
	gdk_screen_get_monitor_geometry (screen, gdk_screen_get_monitor_at_window (screen, ((GtkWidget*) self->priv->_parent)->window), &fs_rect);
#line 74 "window.vala"
	gtk_window_get_position ((GtkWindow*) self, &x, &y);
#line 76 "window.vala"
	if (self->priv->animation_enter) {
#line 78 "window.vala"
		if (y == fs_rect.y) {
#line 80 "window.vala"
			self->priv->animation_timeout_id = (guint) 0;
#line 654 "window.c"
			result = FALSE;
			(screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL));
			return result;
		} else {
#line 85 "window.vala"
			gtk_window_move ((GtkWindow*) self, x, y + 1);
#line 661 "window.c"
			result = TRUE;
			(screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL));
			return result;
		}
	} else {
		gint w;
		gint h;
		w = 0;
		h = 0;
#line 93 "window.vala"
		gtk_window_get_size ((GtkWindow*) self, &w, &h);
#line 95 "window.vala"
		if (y == ((fs_rect.y - h) + 1)) {
#line 97 "window.vala"
			self->priv->animation_timeout_id = (guint) 0;
#line 677 "window.c"
			result = FALSE;
			(screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL));
			return result;
		} else {
#line 102 "window.vala"
			gtk_window_move ((GtkWindow*) self, x, y - 1);
#line 684 "window.c"
			result = TRUE;
			(screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL));
			return result;
		}
	}
	(screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL));
}


#line 64 "window.vala"
static gboolean _valide_fullscreen_toolbar_run_animation_gsource_func (gpointer self) {
#line 696 "window.c"
	return valide_fullscreen_toolbar_run_animation (self);
}


#line 108 "window.vala"
static void valide_fullscreen_toolbar_show_hide (ValideFullscreenToolbar* self, gboolean show, gint height) {
#line 703 "window.c"
	GtkSettings* settings;
	gboolean enable_animations;
	GtkSettings* _tmp1_;
	GtkSettings* _tmp0_;
	gboolean _tmp2_;
#line 108 "window.vala"
	g_return_if_fail (self != NULL);
#line 711 "window.c"
	settings = NULL;
	enable_animations = FALSE;
	_tmp1_ = NULL;
#line 113 "window.vala"
	_tmp0_ = NULL;
#line 113 "window.vala"
	settings = (_tmp1_ = (_tmp0_ = gtk_widget_get_settings ((GtkWidget*) self->priv->_parent), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (settings == NULL) ? NULL : (settings = (g_object_unref (settings), NULL)), _tmp1_);
#line 114 "window.vala"
	enable_animations = (g_object_get (settings, "gtk-enable-animations", &_tmp2_, NULL), _tmp2_);
#line 116 "window.vala"
	if (enable_animations) {
#line 118 "window.vala"
		self->priv->animation_enter = show;
#line 120 "window.vala"
		if (self->priv->animation_timeout_id == 0) {
#line 122 "window.vala"
			self->priv->animation_timeout_id = g_timeout_add ((guint) 4, _valide_fullscreen_toolbar_run_animation_gsource_func, self);
#line 729 "window.c"
		}
	} else {
		GdkScreen* screen;
		GdkRectangle fs_rect = {0};
		GdkScreen* _tmp4_;
		GdkScreen* _tmp3_;
		screen = NULL;
		_tmp4_ = NULL;
#line 130 "window.vala"
		_tmp3_ = NULL;
#line 130 "window.vala"
		screen = (_tmp4_ = (_tmp3_ = gtk_window_get_screen ((GtkWindow*) self->priv->_parent), (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), (screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL)), _tmp4_);
#line 131 "window.vala"
		gdk_screen_get_monitor_geometry (screen, gdk_screen_get_monitor_at_window (screen, ((GtkWidget*) self->priv->_parent)->window), &fs_rect);
#line 134 "window.vala"
		if (show) {
#line 136 "window.vala"
			gtk_window_move ((GtkWindow*) self, fs_rect.x, fs_rect.y);
#line 748 "window.c"
		} else {
#line 140 "window.vala"
			gtk_window_move ((GtkWindow*) self, fs_rect.x, (fs_rect.y - height) + 1);
#line 752 "window.c"
		}
		(screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL));
	}
	(settings == NULL) ? NULL : (settings = (g_object_unref (settings), NULL));
}


#line 146 "window.vala"
ValideFullscreenToolbar* valide_fullscreen_toolbar_construct (GType object_type, ValideWindow* parent) {
#line 762 "window.c"
	GParameter * __params;
	GParameter * __params_it;
	ValideFullscreenToolbar * self;
#line 146 "window.vala"
	g_return_val_if_fail (parent != NULL, NULL);
#line 768 "window.c"
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
#line 148 "window.vala"
	__params_it->name = "parent";
#line 148 "window.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_WINDOW);
#line 148 "window.vala"
	g_value_set_object (&__params_it->value, parent);
#line 148 "window.vala"
	__params_it++;
#line 779 "window.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 146 "window.vala"
ValideFullscreenToolbar* valide_fullscreen_toolbar_new (ValideWindow* parent) {
#line 146 "window.vala"
	return valide_fullscreen_toolbar_construct (VALIDE_TYPE_FULLSCREEN_TOOLBAR, parent);
#line 794 "window.c"
}


#line 171 "window.vala"
void valide_fullscreen_toolbar_show (ValideFullscreenToolbar* self) {
#line 171 "window.vala"
	g_return_if_fail (self != NULL);
#line 173 "window.vala"
	valide_fullscreen_toolbar_show_all (self);
#line 804 "window.c"
}


#line 176 "window.vala"
void valide_fullscreen_toolbar_show_all (ValideFullscreenToolbar* self) {
#line 810 "window.c"
	gint w;
	gint h;
	GdkScreen* screen;
	GdkRectangle fs_rect = {0};
	GdkScreen* _tmp1_;
	GdkScreen* _tmp0_;
#line 176 "window.vala"
	g_return_if_fail (self != NULL);
#line 819 "window.c"
	w = 0;
	h = 0;
	screen = NULL;
	_tmp1_ = NULL;
#line 182 "window.vala"
	_tmp0_ = NULL;
#line 182 "window.vala"
	screen = (_tmp1_ = (_tmp0_ = gtk_window_get_screen ((GtkWindow*) self->priv->_parent), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL)), _tmp1_);
#line 183 "window.vala"
	gdk_screen_get_monitor_geometry (screen, gdk_screen_get_monitor_at_window (screen, ((GtkWidget*) self->priv->_parent)->window), &fs_rect);
#line 186 "window.vala"
	gtk_window_get_size ((GtkWindow*) self, &w, &h);
#line 187 "window.vala"
	gtk_window_resize ((GtkWindow*) self, fs_rect.width, h);
#line 188 "window.vala"
	gtk_window_move ((GtkWindow*) self, fs_rect.x, (fs_rect.y - h) + 1);
#line 189 "window.vala"
	GTK_WIDGET_CLASS (valide_fullscreen_toolbar_parent_class)->show_all ((GtkWidget*) GTK_WINDOW (self));
#line 838 "window.c"
	(screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL));
}


ValideWindow* valide_fullscreen_toolbar_get_parent (ValideFullscreenToolbar* self) {
	ValideWindow* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_parent;
	return result;
}


static void valide_fullscreen_toolbar_set_parent (ValideFullscreenToolbar* self, ValideWindow* value) {
	ValideWindow* _tmp1_;
	ValideWindow* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 30 "window.vala"
	_tmp0_ = NULL;
#line 858 "window.c"
	self->priv->_parent = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_parent == NULL) ? NULL : (self->priv->_parent = (g_object_unref (self->priv->_parent), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "parent");
}


#line 33 "window.vala"
static gboolean _valide_fullscreen_toolbar_on_enter_notify_event_gtk_widget_enter_notify_event (GtkWidget* _sender, const GdkEventCrossing* event, gpointer self) {
#line 866 "window.c"
	return valide_fullscreen_toolbar_on_enter_notify_event (self, _sender, event);
}


#line 39 "window.vala"
static gboolean _valide_fullscreen_toolbar_on_leave_notify_event_gtk_widget_leave_notify_event (GtkWidget* _sender, const GdkEventCrossing* event, gpointer self) {
#line 873 "window.c"
	return valide_fullscreen_toolbar_on_leave_notify_event (self, _sender, event);
}


static GObject * valide_fullscreen_toolbar_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValideFullscreenToolbarClass * klass;
	GObjectClass * parent_class;
	ValideFullscreenToolbar * self;
	klass = VALIDE_FULLSCREEN_TOOLBAR_CLASS (g_type_class_peek (VALIDE_TYPE_FULLSCREEN_TOOLBAR));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_FULLSCREEN_TOOLBAR (obj);
	{
		GtkWindow* window;
		GtkAction* action;
		GtkToolbar* toolbar;
		GtkToolbar* _tmp2_;
		GtkToolbar* _tmp1_;
		GtkWidget* _tmp0_;
		GtkAction* _tmp3_;
		window = NULL;
		action = NULL;
		toolbar = NULL;
#line 157 "window.vala"
		g_object_set ((GtkWindow*) self, "type", GTK_WINDOW_POPUP, NULL);
#line 158 "window.vala"
		gtk_window_set_transient_for ((GtkWindow*) self, (GtkWindow*) self->priv->_parent);
#line 902 "window.c"
		_tmp2_ = NULL;
#line 160 "window.vala"
		_tmp1_ = NULL;
#line 160 "window.vala"
		_tmp0_ = NULL;
#line 160 "window.vala"
		toolbar = (_tmp2_ = (_tmp1_ = (_tmp0_ = gtk_ui_manager_get_widget ((GtkUIManager*) valide_window_get_ui_manager (self->priv->_parent), "/fullscreen-toolbar"), GTK_IS_TOOLBAR (_tmp0_) ? ((GtkToolbar*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (toolbar == NULL) ? NULL : (toolbar = (g_object_unref (toolbar), NULL)), _tmp2_);
#line 161 "window.vala"
		gtk_toolbar_set_style (toolbar, GTK_TOOLBAR_BOTH_HORIZ);
#line 162 "window.vala"
		gtk_container_add ((GtkContainer*) self, (GtkWidget*) toolbar);
#line 914 "window.c"
		_tmp3_ = NULL;
#line 164 "window.vala"
		action = (_tmp3_ = valide_ui_manager_get_action (valide_window_get_ui_manager (self->priv->_parent), "leave-fullscreen"), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp3_);
#line 165 "window.vala"
		g_object_set ((GObject*) action, "is_important", TRUE, NULL);
#line 167 "window.vala"
		g_signal_connect_object ((GtkWidget*) self, "enter-notify-event", (GCallback) _valide_fullscreen_toolbar_on_enter_notify_event_gtk_widget_enter_notify_event, self, 0);
#line 168 "window.vala"
		g_signal_connect_object ((GtkWidget*) self, "leave-notify-event", (GCallback) _valide_fullscreen_toolbar_on_leave_notify_event_gtk_widget_leave_notify_event, self, 0);
#line 924 "window.c"
		(window == NULL) ? NULL : (window = (g_object_unref (window), NULL));
		(action == NULL) ? NULL : (action = (g_object_unref (action), NULL));
		(toolbar == NULL) ? NULL : (toolbar = (g_object_unref (toolbar), NULL));
	}
	return obj;
}


static void valide_fullscreen_toolbar_class_init (ValideFullscreenToolbarClass * klass) {
	valide_fullscreen_toolbar_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideFullscreenToolbarPrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_fullscreen_toolbar_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_fullscreen_toolbar_set_property;
	G_OBJECT_CLASS (klass)->constructor = valide_fullscreen_toolbar_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_fullscreen_toolbar_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_FULLSCREEN_TOOLBAR_PARENT, g_param_spec_object ("parent", "parent", "parent", VALIDE_TYPE_WINDOW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void valide_fullscreen_toolbar_instance_init (ValideFullscreenToolbar * self) {
	self->priv = VALIDE_FULLSCREEN_TOOLBAR_GET_PRIVATE (self);
}


static void valide_fullscreen_toolbar_finalize (GObject* obj) {
	ValideFullscreenToolbar * self;
	self = VALIDE_FULLSCREEN_TOOLBAR (obj);
	(self->priv->_parent == NULL) ? NULL : (self->priv->_parent = (g_object_unref (self->priv->_parent), NULL));
	G_OBJECT_CLASS (valide_fullscreen_toolbar_parent_class)->finalize (obj);
}


GType valide_fullscreen_toolbar_get_type (void) {
	static GType valide_fullscreen_toolbar_type_id = 0;
	if (valide_fullscreen_toolbar_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideFullscreenToolbarClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_fullscreen_toolbar_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideFullscreenToolbar), 0, (GInstanceInitFunc) valide_fullscreen_toolbar_instance_init, NULL };
		valide_fullscreen_toolbar_type_id = g_type_register_static (GTK_TYPE_WINDOW, "ValideFullscreenToolbar", &g_define_type_info, 0);
	}
	return valide_fullscreen_toolbar_type_id;
}


static void valide_fullscreen_toolbar_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideFullscreenToolbar * self;
	gpointer boxed;
	self = VALIDE_FULLSCREEN_TOOLBAR (object);
	switch (property_id) {
		case VALIDE_FULLSCREEN_TOOLBAR_PARENT:
		g_value_set_object (value, valide_fullscreen_toolbar_get_parent (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_fullscreen_toolbar_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideFullscreenToolbar * self;
	self = VALIDE_FULLSCREEN_TOOLBAR (object);
	switch (property_id) {
		case VALIDE_FULLSCREEN_TOOLBAR_PARENT:
		valide_fullscreen_toolbar_set_parent (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


#line 323 "window.vala"
static void valide_window_message (ValideWindow* self, const char* log_domain, GLogLevelFlags log_levels, const char* message) {
#line 998 "window.c"
	GtkDialog* dialog;
	GtkMessageType type;
	GtkDialog* _tmp0_;
#line 323 "window.vala"
	g_return_if_fail (self != NULL);
#line 323 "window.vala"
	g_return_if_fail (message != NULL);
#line 1006 "window.c"
	dialog = NULL;
	type = 0;
#line 329 "window.vala"
	switch (log_levels) {
#line 1011 "window.c"
		case G_LOG_LEVEL_MESSAGE:
		{
#line 332 "window.vala"
			type = GTK_MESSAGE_INFO;
#line 333 "window.vala"
			break;
#line 1018 "window.c"
		}
		case G_LOG_LEVEL_WARNING:
		{
#line 335 "window.vala"
			type = GTK_MESSAGE_WARNING;
#line 336 "window.vala"
			break;
#line 1026 "window.c"
		}
		case G_LOG_LEVEL_CRITICAL:
		{
#line 338 "window.vala"
			type = GTK_MESSAGE_ERROR;
#line 339 "window.vala"
			break;
#line 1034 "window.c"
		}
		default:
		{
#line 341 "window.vala"
			type = GTK_MESSAGE_OTHER;
#line 342 "window.vala"
			break;
#line 1042 "window.c"
		}
	}
	_tmp0_ = NULL;
#line 345 "window.vala"
	dialog = (_tmp0_ = (GtkDialog*) g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new ((GtkWindow*) self, GTK_DIALOG_MODAL, type, GTK_BUTTONS_OK, message, NULL)), (dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL)), _tmp0_);
#line 347 "window.vala"
	gtk_dialog_run (dialog);
#line 348 "window.vala"
	gtk_object_destroy ((GtkObject*) dialog);
#line 1052 "window.c"
	(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
}


#line 351 "window.vala"
static gboolean valide_window_on_delete_event (ValideWindow* self) {
#line 1059 "window.c"
	gboolean result;
	GError * _inner_error_;
	gint width;
	gint height;
#line 351 "window.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1066 "window.c"
	_inner_error_ = NULL;
	width = 0;
	height = 0;
#line 356 "window.vala"
	gtk_window_get_size ((GtkWindow*) self, &width, &height);
#line 357 "window.vala"
	valide_abstract_config_manager_set_integer ((ValideAbstractConfigManager*) self->priv->_config_manager, "General", "width", width);
#line 358 "window.vala"
	valide_abstract_config_manager_set_integer ((ValideAbstractConfigManager*) self->priv->_config_manager, "General", "height", height);
#line 1076 "window.c"
	{
#line 361 "window.vala"
		valide_executable_manager_stop (self->priv->_executables, &_inner_error_);
#line 1080 "window.c"
		if (_inner_error_ != NULL) {
			goto __catch58_g_error;
			goto __finally58;
		}
	}
	goto __finally58;
	__catch58_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 365 "window.vala"
			g_debug (e->message);
#line 1095 "window.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally58:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return FALSE;
	}
	result = !valide_document_manager_close_all (self->priv->_documents);
	return result;
}


#line 370 "window.vala"
static void valide_window_menu_item_select_cb (ValideWindow* self, GtkItem* item) {
#line 1112 "window.c"
	char* message;
	GtkAction* action;
	GtkAction* _tmp2_;
	GtkAction* _tmp1_;
	void* _tmp0_;
#line 370 "window.vala"
	g_return_if_fail (self != NULL);
#line 370 "window.vala"
	g_return_if_fail (item != NULL);
#line 1122 "window.c"
	message = NULL;
	action = NULL;
	_tmp2_ = NULL;
#line 375 "window.vala"
	_tmp1_ = NULL;
#line 375 "window.vala"
	_tmp0_ = NULL;
#line 375 "window.vala"
	action = (_tmp2_ = (_tmp1_ = (_tmp0_ = g_object_get_data ((GObject*) item, "gtk-action"), GTK_IS_ACTION (_tmp0_) ? ((GtkAction*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp2_);
#line 376 "window.vala"
	g_return_if_fail (action != NULL);
#line 378 "window.vala"
	g_object_get ((GObject*) action, "tooltip", &message, NULL);
#line 379 "window.vala"
	if (message != NULL) {
#line 381 "window.vala"
		gtk_statusbar_push ((GtkStatusbar*) self->priv->_statusbar, self->priv->status_context, message);
#line 1140 "window.c"
	}
	message = (g_free (message), NULL);
	(action == NULL) ? NULL : (action = (g_object_unref (action), NULL));
}


#line 385 "window.vala"
static void valide_window_menu_item_deselect_cb (ValideWindow* self, GtkItem* item) {
#line 385 "window.vala"
	g_return_if_fail (self != NULL);
#line 385 "window.vala"
	g_return_if_fail (item != NULL);
#line 387 "window.vala"
	gtk_statusbar_pop ((GtkStatusbar*) self->priv->_statusbar, self->priv->status_context);
#line 1155 "window.c"
}


#line 370 "window.vala"
static void _valide_window_menu_item_select_cb_gtk_item_select (GtkItem* _sender, gpointer self) {
#line 1161 "window.c"
	valide_window_menu_item_select_cb (self, _sender);
}


#line 385 "window.vala"
static void _valide_window_menu_item_deselect_cb_gtk_item_deselect (GtkItem* _sender, gpointer self) {
#line 1168 "window.c"
	valide_window_menu_item_deselect_cb (self, _sender);
}


#line 390 "window.vala"
static void valide_window_connect_proxy_cb (ValideWindow* self, GtkUIManager* sender, GtkAction* action, GtkWidget* proxy) {
#line 390 "window.vala"
	g_return_if_fail (self != NULL);
#line 390 "window.vala"
	g_return_if_fail (sender != NULL);
#line 390 "window.vala"
	g_return_if_fail (action != NULL);
#line 390 "window.vala"
	g_return_if_fail (proxy != NULL);
#line 393 "window.vala"
	if (GTK_IS_MENU_ITEM (proxy)) {
#line 1185 "window.c"
		GtkMenuItem* item;
		GtkMenuItem* _tmp2_;
		GtkMenuItem* _tmp1_;
		GtkWidget* _tmp0_;
		item = NULL;
		_tmp2_ = NULL;
#line 397 "window.vala"
		_tmp1_ = NULL;
#line 397 "window.vala"
		_tmp0_ = NULL;
#line 397 "window.vala"
		item = (_tmp2_ = (_tmp1_ = (_tmp0_ = proxy, GTK_IS_MENU_ITEM (_tmp0_) ? ((GtkMenuItem*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (item == NULL) ? NULL : (item = (g_object_unref (item), NULL)), _tmp2_);
#line 398 "window.vala"
		g_signal_connect_object ((GtkItem*) item, "select", (GCallback) _valide_window_menu_item_select_cb_gtk_item_select, self, 0);
#line 399 "window.vala"
		g_signal_connect_object ((GtkItem*) item, "deselect", (GCallback) _valide_window_menu_item_deselect_cb_gtk_item_deselect, self, 0);
#line 1202 "window.c"
		(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
	}
}


#line 403 "window.vala"
static void valide_window_disconnect_proxy_cb (ValideWindow* self, GtkUIManager* sender, GtkAction* action, GtkWidget* proxy) {
#line 403 "window.vala"
	g_return_if_fail (self != NULL);
#line 403 "window.vala"
	g_return_if_fail (sender != NULL);
#line 403 "window.vala"
	g_return_if_fail (action != NULL);
#line 403 "window.vala"
	g_return_if_fail (proxy != NULL);
#line 406 "window.vala"
	if (GTK_IS_MENU_ITEM (proxy)) {
#line 1220 "window.c"
		GtkMenuItem* item;
		GtkMenuItem* _tmp2_;
		GtkMenuItem* _tmp1_;
		GtkWidget* _tmp0_;
		guint _tmp3_;
		guint _tmp4_;
		item = NULL;
		_tmp2_ = NULL;
#line 410 "window.vala"
		_tmp1_ = NULL;
#line 410 "window.vala"
		_tmp0_ = NULL;
#line 410 "window.vala"
		item = (_tmp2_ = (_tmp1_ = (_tmp0_ = proxy, GTK_IS_MENU_ITEM (_tmp0_) ? ((GtkMenuItem*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (item == NULL) ? NULL : (item = (g_object_unref (item), NULL)), _tmp2_);
#line 411 "window.vala"
		g_signal_handlers_disconnect_matched ((GtkItem*) item, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("select", GTK_TYPE_ITEM, &_tmp3_, NULL, FALSE), _tmp3_), 0, NULL, (GCallback) _valide_window_menu_item_select_cb_gtk_item_select, self);
#line 412 "window.vala"
		g_signal_handlers_disconnect_matched ((GtkItem*) item, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("deselect", GTK_TYPE_ITEM, &_tmp4_, NULL, FALSE), _tmp4_), 0, NULL, (GCallback) _valide_window_menu_item_deselect_cb_gtk_item_deselect, self);
#line 1239 "window.c"
		(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
	}
}


#line 390 "window.vala"
static void _valide_window_connect_proxy_cb_gtk_ui_manager_connect_proxy (GtkUIManager* _sender, GtkAction* action, GtkWidget* proxy, gpointer self) {
#line 1247 "window.c"
	valide_window_connect_proxy_cb (self, _sender, action, proxy);
}


#line 403 "window.vala"
static void _valide_window_disconnect_proxy_cb_gtk_ui_manager_disconnect_proxy (GtkUIManager* _sender, GtkAction* action, GtkWidget* proxy, gpointer self) {
#line 1254 "window.c"
	valide_window_disconnect_proxy_cb (self, _sender, action, proxy);
}


#line 416 "window.vala"
static void valide_window_setup_ui_manager (ValideWindow* self) {
#line 1261 "window.c"
	ValideUIManager* _tmp0_;
	GtkWidget* _tmp1_;
	GtkWidget* _tmp2_;
#line 416 "window.vala"
	g_return_if_fail (self != NULL);
#line 1267 "window.c"
	_tmp0_ = NULL;
#line 418 "window.vala"
	valide_window_set_ui_manager (self, _tmp0_ = valide_ui_manager_new ());
#line 1271 "window.c"
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
#line 419 "window.vala"
	g_signal_connect_object ((GtkUIManager*) self->priv->_ui_manager, "connect-proxy", (GCallback) _valide_window_connect_proxy_cb_gtk_ui_manager_connect_proxy, self, 0);
#line 420 "window.vala"
	g_signal_connect_object ((GtkUIManager*) self->priv->_ui_manager, "disconnect-proxy", (GCallback) _valide_window_disconnect_proxy_cb_gtk_ui_manager_disconnect_proxy, self, 0);
#line 421 "window.vala"
	valide_ui_manager_populate (self->priv->_ui_manager);
#line 423 "window.vala"
	_tmp1_ = NULL;
#line 423 "window.vala"
	valide_window_set_menu (self, (_tmp1_ = gtk_ui_manager_get_widget ((GtkUIManager*) self->priv->_ui_manager, "/menubar"), GTK_IS_MENU_BAR (_tmp1_) ? ((GtkMenuBar*) _tmp1_) : NULL));
#line 424 "window.vala"
	_tmp2_ = NULL;
#line 424 "window.vala"
	valide_window_set_toolbar (self, (_tmp2_ = gtk_ui_manager_get_widget ((GtkUIManager*) self->priv->_ui_manager, "/toolbar"), GTK_IS_TOOLBAR (_tmp2_) ? ((GtkToolbar*) _tmp2_) : NULL));
#line 426 "window.vala"
	valide_ui_manager_action_set_toggled (self->priv->_ui_manager, "view-statusbar", TRUE);
#line 427 "window.vala"
	valide_ui_manager_action_set_toggled (self->priv->_ui_manager, "view-toolbar", TRUE);
#line 428 "window.vala"
	valide_ui_manager_action_set_toggled (self->priv->_ui_manager, "view-side-panel", TRUE);
#line 429 "window.vala"
	valide_ui_manager_action_set_toggled (self->priv->_ui_manager, "view-bottom-panel", TRUE);
#line 431 "window.vala"
	gtk_window_add_accel_group ((GtkWindow*) self, gtk_ui_manager_get_accel_group ((GtkUIManager*) self->priv->_ui_manager));
#line 1297 "window.c"
}


static void _lambda46_ (ValideWindow* self) {
#line 458 "window.vala"
	valide_ui_manager_action_set_toggled (self->priv->_ui_manager, "view-bottom-panel", FALSE);
#line 1304 "window.c"
}


static void __lambda46__gtk_widget_hide (GtkWidget* _sender, gpointer self) {
	_lambda46_ (self);
}


static void _lambda47_ (ValideWindow* self) {
#line 470 "window.vala"
	valide_ui_manager_action_set_toggled (self->priv->_ui_manager, "view-side-panel", FALSE);
#line 1316 "window.c"
}


static void __lambda47__gtk_widget_hide (GtkWidget* _sender, gpointer self) {
	_lambda47_ (self);
}


#line 434 "window.vala"
static void valide_window_populate_window (ValideWindow* self) {
#line 1327 "window.c"
	GtkVBox* main_box;
	GtkVPaned* vpaned;
	GtkHPaned* hpaned;
	GtkVBox* _tmp0_;
	ValideFullscreenToolbar* _tmp1_;
	GtkVPaned* _tmp2_;
	ValidePanel* _tmp3_;
	GtkHPaned* _tmp4_;
	ValidePanel* _tmp5_;
	GtkImage* _tmp7_;
	char* _tmp6_;
#line 434 "window.vala"
	g_return_if_fail (self != NULL);
#line 1341 "window.c"
	main_box = NULL;
	vpaned = NULL;
	hpaned = NULL;
	_tmp0_ = NULL;
#line 440 "window.vala"
	main_box = (_tmp0_ = g_object_ref_sink ((GtkVBox*) gtk_vbox_new (FALSE, 0)), (main_box == NULL) ? NULL : (main_box = (g_object_unref (main_box), NULL)), _tmp0_);
#line 441 "window.vala"
	gtk_container_add ((GtkContainer*) self, (GtkWidget*) main_box);
#line 443 "window.vala"
	valide_recent_manager_ui_update_file (self->priv->_recent_manager);
#line 444 "window.vala"
	valide_recent_manager_ui_update_project (self->priv->_recent_manager);
#line 446 "window.vala"
	gtk_box_pack_start ((GtkBox*) main_box, (GtkWidget*) self->priv->_menu, FALSE, TRUE, (guint) 0);
#line 447 "window.vala"
	gtk_box_pack_start ((GtkBox*) main_box, (GtkWidget*) self->priv->_toolbar, FALSE, TRUE, (guint) 0);
#line 1358 "window.c"
	_tmp1_ = NULL;
#line 448 "window.vala"
	self->priv->fullscreen_toolbar = (_tmp1_ = g_object_ref_sink (valide_fullscreen_toolbar_new (self)), (self->priv->fullscreen_toolbar == NULL) ? NULL : (self->priv->fullscreen_toolbar = (g_object_unref (self->priv->fullscreen_toolbar), NULL)), _tmp1_);
#line 1362 "window.c"
	_tmp2_ = NULL;
#line 450 "window.vala"
	vpaned = (_tmp2_ = g_object_ref_sink ((GtkVPaned*) gtk_vpaned_new ()), (vpaned == NULL) ? NULL : (vpaned = (g_object_unref (vpaned), NULL)), _tmp2_);
#line 451 "window.vala"
	gtk_box_pack_start ((GtkBox*) main_box, (GtkWidget*) vpaned, TRUE, TRUE, (guint) 0);
#line 1368 "window.c"
	_tmp3_ = NULL;
#line 453 "window.vala"
	valide_window_set_bottom_panel (self, _tmp3_ = g_object_ref_sink (valide_panel_new (GTK_ORIENTATION_HORIZONTAL)));
#line 1372 "window.c"
	(_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL));
#line 454 "window.vala"
	valide_panel_add_item_with_stock_icon (self->priv->_bottom_panel, (GtkWidget*) self->priv->_executables, _ ("Messages"), GTK_STOCK_EXECUTE);
#line 457 "window.vala"
	g_signal_connect ((GtkWidget*) self->priv->_bottom_panel, "hide", (GCallback) __lambda46__gtk_widget_hide, self);
#line 461 "window.vala"
	gtk_paned_pack2 ((GtkPaned*) vpaned, (GtkWidget*) self->priv->_bottom_panel, FALSE, FALSE);
#line 1380 "window.c"
	_tmp4_ = NULL;
#line 463 "window.vala"
	hpaned = (_tmp4_ = g_object_ref_sink ((GtkHPaned*) gtk_hpaned_new ()), (hpaned == NULL) ? NULL : (hpaned = (g_object_unref (hpaned), NULL)), _tmp4_);
#line 464 "window.vala"
	gtk_paned_pack1 ((GtkPaned*) vpaned, (GtkWidget*) hpaned, TRUE, TRUE);
#line 1386 "window.c"
	_tmp5_ = NULL;
#line 466 "window.vala"
	valide_window_set_side_panel (self, _tmp5_ = g_object_ref_sink (valide_panel_new (GTK_ORIENTATION_VERTICAL)));
#line 1390 "window.c"
	(_tmp5_ == NULL) ? NULL : (_tmp5_ = (g_object_unref (_tmp5_), NULL));
	_tmp7_ = NULL;
#line 2034 "glib-2.0.vapi"
	_tmp6_ = NULL;
#line 467 "window.vala"
	valide_panel_add_item (self->priv->_side_panel, (GtkWidget*) self->priv->_projects, _ ("Project"), _tmp7_ = g_object_ref_sink ((GtkImage*) gtk_image_new_from_file (_tmp6_ = g_build_filename (PIXMAPS_DIR, "icone-16.xpm", NULL))));
#line 1397 "window.c"
	(_tmp7_ == NULL) ? NULL : (_tmp7_ = (g_object_unref (_tmp7_), NULL));
	_tmp6_ = (g_free (_tmp6_), NULL);
#line 469 "window.vala"
	g_signal_connect ((GtkWidget*) self->priv->_side_panel, "hide", (GCallback) __lambda47__gtk_widget_hide, self);
#line 473 "window.vala"
	gtk_paned_add1 ((GtkPaned*) hpaned, (GtkWidget*) self->priv->_side_panel);
#line 474 "window.vala"
	gtk_paned_add2 ((GtkPaned*) hpaned, (GtkWidget*) self->priv->_documents);
#line 476 "window.vala"
	gtk_box_pack_start ((GtkBox*) main_box, (GtkWidget*) self->priv->_statusbar, FALSE, TRUE, (guint) 0);
#line 1408 "window.c"
	(main_box == NULL) ? NULL : (main_box = (g_object_unref (main_box), NULL));
	(vpaned == NULL) ? NULL : (vpaned = (g_object_unref (vpaned), NULL));
	(hpaned == NULL) ? NULL : (hpaned = (g_object_unref (hpaned), NULL));
}


#line 479 "window.vala"
static void valide_window_on_action_activated (ValideWindow* self, GtkUIManager* sender, GtkAction* action) {
#line 1417 "window.c"
	GError * _inner_error_;
#line 479 "window.vala"
	g_return_if_fail (self != NULL);
#line 479 "window.vala"
	g_return_if_fail (sender != NULL);
#line 479 "window.vala"
	g_return_if_fail (action != NULL);
#line 1425 "window.c"
	_inner_error_ = NULL;
	{
		GQuark _tmp24_;
		const char* _tmp23_;
		static GQuark _tmp24__label0 = 0;
		static GQuark _tmp24__label1 = 0;
		static GQuark _tmp24__label2 = 0;
		static GQuark _tmp24__label3 = 0;
		static GQuark _tmp24__label4 = 0;
		static GQuark _tmp24__label5 = 0;
		static GQuark _tmp24__label6 = 0;
		static GQuark _tmp24__label7 = 0;
		static GQuark _tmp24__label8 = 0;
		static GQuark _tmp24__label9 = 0;
		static GQuark _tmp24__label10 = 0;
		static GQuark _tmp24__label11 = 0;
		static GQuark _tmp24__label12 = 0;
		static GQuark _tmp24__label13 = 0;
		static GQuark _tmp24__label14 = 0;
		static GQuark _tmp24__label15 = 0;
		static GQuark _tmp24__label16 = 0;
		static GQuark _tmp24__label17 = 0;
		static GQuark _tmp24__label18 = 0;
		static GQuark _tmp24__label19 = 0;
		static GQuark _tmp24__label20 = 0;
		static GQuark _tmp24__label21 = 0;
		static GQuark _tmp24__label22 = 0;
		static GQuark _tmp24__label23 = 0;
		static GQuark _tmp24__label24 = 0;
		static GQuark _tmp24__label25 = 0;
		static GQuark _tmp24__label26 = 0;
		static GQuark _tmp24__label27 = 0;
		static GQuark _tmp24__label28 = 0;
		static GQuark _tmp24__label29 = 0;
		static GQuark _tmp24__label30 = 0;
		static GQuark _tmp24__label31 = 0;
		static GQuark _tmp24__label32 = 0;
		static GQuark _tmp24__label33 = 0;
		static GQuark _tmp24__label34 = 0;
		static GQuark _tmp24__label35 = 0;
		static GQuark _tmp24__label36 = 0;
		static GQuark _tmp24__label37 = 0;
		static GQuark _tmp24__label38 = 0;
		static GQuark _tmp24__label39 = 0;
		static GQuark _tmp24__label40 = 0;
		static GQuark _tmp24__label41 = 0;
		static GQuark _tmp24__label42 = 0;
		static GQuark _tmp24__label43 = 0;
		static GQuark _tmp24__label44 = 0;
		static GQuark _tmp24__label45 = 0;
		static GQuark _tmp24__label46 = 0;
		static GQuark _tmp24__label47 = 0;
		static GQuark _tmp24__label48 = 0;
#line 483 "window.vala"
		_tmp23_ = NULL;
#line 1481 "window.c"
		_tmp23_ = gtk_action_get_name (action);
		_tmp24_ = (NULL == _tmp23_) ? 0 : g_quark_from_string (_tmp23_);
		if (_tmp24_ == ((0 != _tmp24__label0) ? _tmp24__label0 : (_tmp24__label0 = g_quark_from_static_string ("file-new"))))
		do {
#line 487 "window.vala"
			if (valide_project_manager_get_current (self->priv->_projects) != NULL) {
#line 489 "window.vala"
				valide_project_create_file (valide_project_manager_get_current (self->priv->_projects), &_inner_error_);
#line 1490 "window.c"
				if (_inner_error_ != NULL) {
					goto __catch59_g_error;
					goto __finally59;
				}
			} else {
				ValideiDocument* _tmp0_;
#line 414 "document-manager.vala"
				_tmp0_ = NULL;
#line 493 "window.vala"
				_tmp0_ = valide_document_manager_create (self->priv->_documents, NULL, &_inner_error_);
#line 1501 "window.c"
				if (_inner_error_ != NULL) {
					goto __catch59_g_error;
					goto __finally59;
				}
				(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
			}
#line 495 "window.vala"
			break;
#line 1510 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label1) ? _tmp24__label1 : (_tmp24__label1 = g_quark_from_static_string ("file-open"))))
		do {
#line 497 "window.vala"
			valide_document_manager_open (self->priv->_documents, &_inner_error_);
#line 1515 "window.c"
			if (_inner_error_ != NULL) {
				goto __catch59_g_error;
				goto __finally59;
			}
#line 498 "window.vala"
			break;
#line 1522 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label2) ? _tmp24__label2 : (_tmp24__label2 = g_quark_from_static_string ("file-save"))))
		do {
#line 500 "window.vala"
			if (valide_document_manager_get_current (self->priv->_documents) != NULL) {
#line 1527 "window.c"
				char* current_folder;
				current_folder = NULL;
#line 504 "window.vala"
				if (valide_project_manager_get_current (self->priv->_projects) != NULL) {
#line 1532 "window.c"
					char* _tmp2_;
					const char* _tmp1_;
					_tmp2_ = NULL;
#line 506 "window.vala"
					_tmp1_ = NULL;
#line 506 "window.vala"
					current_folder = (_tmp2_ = (_tmp1_ = valide_project_get_path (valide_project_manager_get_current (self->priv->_projects)), (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_)), current_folder = (g_free (current_folder), NULL), _tmp2_);
#line 1540 "window.c"
				}
#line 508 "window.vala"
				valide_idocument_save (valide_document_manager_get_current (self->priv->_documents), current_folder);
#line 1544 "window.c"
				current_folder = (g_free (current_folder), NULL);
			} else {
				_inner_error_ = g_error_new_literal (VALIDE_DOCUMENT_ERROR, VALIDE_DOCUMENT_ERROR_NO_DOCUMENT, _ ("No document open!"));
				if (_inner_error_ != NULL) {
					goto __catch59_g_error;
					goto __finally59;
				}
			}
#line 514 "window.vala"
			break;
#line 1555 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label3) ? _tmp24__label3 : (_tmp24__label3 = g_quark_from_static_string ("file-save-as"))))
		do {
#line 516 "window.vala"
			if (valide_document_manager_get_current (self->priv->_documents) != NULL) {
#line 1560 "window.c"
				char* current_folder;
				current_folder = NULL;
#line 520 "window.vala"
				if (valide_project_manager_get_current (self->priv->_projects) != NULL) {
#line 1565 "window.c"
					char* _tmp4_;
					const char* _tmp3_;
					_tmp4_ = NULL;
#line 522 "window.vala"
					_tmp3_ = NULL;
#line 522 "window.vala"
					current_folder = (_tmp4_ = (_tmp3_ = valide_project_get_path (valide_project_manager_get_current (self->priv->_projects)), (_tmp3_ == NULL) ? NULL : g_strdup (_tmp3_)), current_folder = (g_free (current_folder), NULL), _tmp4_);
#line 1573 "window.c"
				}
#line 524 "window.vala"
				valide_idocument_save_as (valide_document_manager_get_current (self->priv->_documents), current_folder);
#line 1577 "window.c"
				current_folder = (g_free (current_folder), NULL);
			} else {
				_inner_error_ = g_error_new_literal (VALIDE_DOCUMENT_ERROR, VALIDE_DOCUMENT_ERROR_NO_DOCUMENT, _ ("No document open!"));
				if (_inner_error_ != NULL) {
					goto __catch59_g_error;
					goto __finally59;
				}
			}
#line 530 "window.vala"
			break;
#line 1588 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label4) ? _tmp24__label4 : (_tmp24__label4 = g_quark_from_static_string ("file-save-all"))))
		do {
#line 532 "window.vala"
			if (valide_document_manager_get_current (self->priv->_documents) != NULL) {
#line 534 "window.vala"
				valide_document_manager_save_all (self->priv->_documents);
#line 1595 "window.c"
			} else {
				_inner_error_ = g_error_new_literal (VALIDE_DOCUMENT_ERROR, VALIDE_DOCUMENT_ERROR_NO_DOCUMENT, _ ("No document open!"));
				if (_inner_error_ != NULL) {
					goto __catch59_g_error;
					goto __finally59;
				}
			}
#line 540 "window.vala"
			break;
#line 1605 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label5) ? _tmp24__label5 : (_tmp24__label5 = g_quark_from_static_string ("file-rename"))))
		do {
#line 542 "window.vala"
			if (valide_document_manager_get_current (self->priv->_documents) != NULL) {
#line 544 "window.vala"
				valide_idocument_rename (valide_document_manager_get_current (self->priv->_documents));
#line 1612 "window.c"
			} else {
				_inner_error_ = g_error_new_literal (VALIDE_DOCUMENT_ERROR, VALIDE_DOCUMENT_ERROR_NO_DOCUMENT, _ ("No document open!"));
				if (_inner_error_ != NULL) {
					goto __catch59_g_error;
					goto __finally59;
				}
			}
#line 550 "window.vala"
			break;
#line 1622 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label6) ? _tmp24__label6 : (_tmp24__label6 = g_quark_from_static_string ("file-close"))))
		do {
#line 552 "window.vala"
			if (valide_document_manager_get_current (self->priv->_documents) != NULL) {
#line 554 "window.vala"
				valide_idocument_close (valide_document_manager_get_current (self->priv->_documents));
#line 1629 "window.c"
			} else {
				_inner_error_ = g_error_new_literal (VALIDE_DOCUMENT_ERROR, VALIDE_DOCUMENT_ERROR_NO_DOCUMENT, _ ("No document open!"));
				if (_inner_error_ != NULL) {
					goto __catch59_g_error;
					goto __finally59;
				}
			}
#line 560 "window.vala"
			break;
#line 1639 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label7) ? _tmp24__label7 : (_tmp24__label7 = g_quark_from_static_string ("file-close-all"))))
		do {
#line 562 "window.vala"
			if (valide_document_manager_get_current (self->priv->_documents) != NULL) {
#line 564 "window.vala"
				valide_document_manager_close_all (self->priv->_documents);
#line 1646 "window.c"
			} else {
				_inner_error_ = g_error_new_literal (VALIDE_DOCUMENT_ERROR, VALIDE_DOCUMENT_ERROR_NO_DOCUMENT, _ ("No document open!"));
				if (_inner_error_ != NULL) {
					goto __catch59_g_error;
					goto __finally59;
				}
			}
#line 570 "window.vala"
			break;
#line 1656 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label8) ? _tmp24__label8 : (_tmp24__label8 = g_quark_from_static_string ("quit"))))
		do {
#line 572 "window.vala"
			if (!valide_window_on_delete_event (self)) {
#line 574 "window.vala"
				gtk_object_destroy ((GtkObject*) self);
#line 1663 "window.c"
			}
#line 576 "window.vala"
			break;
#line 1667 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label9) ? _tmp24__label9 : (_tmp24__label9 = g_quark_from_static_string ("edit-undo"))))
		do {
#line 580 "window.vala"
			gtk_bindings_activate ((GtkObject*) gtk_window_get_focus ((GtkWindow*) self), (guint) GDK_z, GDK_CONTROL_MASK);
#line 582 "window.vala"
			break;
#line 1674 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label10) ? _tmp24__label10 : (_tmp24__label10 = g_quark_from_static_string ("edit-redo"))))
		do {
#line 590 "window.vala"
			g_signal_emit_by_name (gtk_window_get_focus ((GtkWindow*) self), "redo", NULL);
#line 591 "window.vala"
			break;
#line 1681 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label11) ? _tmp24__label11 : (_tmp24__label11 = g_quark_from_static_string ("edit-copy"))))
		do {
#line 593 "window.vala"
			gtk_bindings_activate ((GtkObject*) gtk_window_get_focus ((GtkWindow*) self), (guint) GDK_c, GDK_CONTROL_MASK);
#line 595 "window.vala"
			break;
#line 1688 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label12) ? _tmp24__label12 : (_tmp24__label12 = g_quark_from_static_string ("edit-cut"))))
		do {
#line 597 "window.vala"
			gtk_bindings_activate ((GtkObject*) gtk_window_get_focus ((GtkWindow*) self), (guint) GDK_x, GDK_CONTROL_MASK);
#line 599 "window.vala"
			break;
#line 1695 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label13) ? _tmp24__label13 : (_tmp24__label13 = g_quark_from_static_string ("edit-paste"))))
		do {
#line 601 "window.vala"
			gtk_bindings_activate ((GtkObject*) gtk_window_get_focus ((GtkWindow*) self), (guint) GDK_v, GDK_CONTROL_MASK);
#line 603 "window.vala"
			break;
#line 1702 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label14) ? _tmp24__label14 : (_tmp24__label14 = g_quark_from_static_string ("edit-lower"))))
		do {
#line 605 "window.vala"
			gtk_bindings_activate ((GtkObject*) gtk_window_get_focus ((GtkWindow*) self), (guint) GDK_l, GDK_CONTROL_MASK);
#line 607 "window.vala"
			break;
#line 1709 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label15) ? _tmp24__label15 : (_tmp24__label15 = g_quark_from_static_string ("edit-upper"))))
		do {
#line 609 "window.vala"
			gtk_bindings_activate ((GtkObject*) gtk_window_get_focus ((GtkWindow*) self), (guint) GDK_u, GDK_CONTROL_MASK);
#line 611 "window.vala"
			break;
#line 1716 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label16) ? _tmp24__label16 : (_tmp24__label16 = g_quark_from_static_string ("edit-preferences"))))
		do {
#line 613 "window.vala"
			valide_abstract_config_manager_preferences ((ValideAbstractConfigManager*) self->priv->_config_manager, self);
#line 614 "window.vala"
			break;
#line 1723 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label17) ? _tmp24__label17 : (_tmp24__label17 = g_quark_from_static_string ("search-find"))))
		do {
#line 618 "window.vala"
			gtk_bindings_activate ((GtkObject*) gtk_window_get_focus ((GtkWindow*) self), (guint) GDK_f, GDK_CONTROL_MASK);
#line 620 "window.vala"
			break;
#line 1730 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label18) ? _tmp24__label18 : (_tmp24__label18 = g_quark_from_static_string ("search-find-next"))))
		do {
#line 622 "window.vala"
			gtk_bindings_activate ((GtkObject*) gtk_window_get_focus ((GtkWindow*) self), (guint) GDK_g, GDK_CONTROL_MASK);
#line 624 "window.vala"
			break;
#line 1737 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label19) ? _tmp24__label19 : (_tmp24__label19 = g_quark_from_static_string ("search-find-prev"))))
		do {
#line 632 "window.vala"
			g_signal_emit_by_name (gtk_window_get_focus ((GtkWindow*) self), "action_find_prev", NULL);
#line 633 "window.vala"
			break;
#line 1744 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label20) ? _tmp24__label20 : (_tmp24__label20 = g_quark_from_static_string ("search-replace"))))
		do {
#line 635 "window.vala"
			gtk_bindings_activate ((GtkObject*) gtk_window_get_focus ((GtkWindow*) self), (guint) GDK_h, GDK_CONTROL_MASK);
#line 637 "window.vala"
			break;
#line 1751 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label21) ? _tmp24__label21 : (_tmp24__label21 = g_quark_from_static_string ("search-goto-line"))))
		do {
#line 639 "window.vala"
			gtk_bindings_activate ((GtkObject*) gtk_window_get_focus ((GtkWindow*) self), (guint) GDK_i, GDK_CONTROL_MASK);
#line 641 "window.vala"
			break;
#line 1758 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label22) ? _tmp24__label22 : (_tmp24__label22 = g_quark_from_static_string ("view-start-page"))))
		do {
			GtkToggleAction* toggle_action;
			GtkToggleAction* _tmp7_;
			GtkToggleAction* _tmp6_;
			GtkAction* _tmp5_;
			toggle_action = NULL;
			_tmp7_ = NULL;
#line 647 "window.vala"
			_tmp6_ = NULL;
#line 647 "window.vala"
			_tmp5_ = NULL;
#line 647 "window.vala"
			toggle_action = (_tmp7_ = (_tmp6_ = (_tmp5_ = action, GTK_IS_TOGGLE_ACTION (_tmp5_) ? ((GtkToggleAction*) _tmp5_) : NULL), (_tmp6_ == NULL) ? NULL : g_object_ref (_tmp6_)), (toggle_action == NULL) ? NULL : (toggle_action = (g_object_unref (toggle_action), NULL)), _tmp7_);
#line 648 "window.vala"
			if (gtk_toggle_action_get_active (toggle_action)) {
#line 650 "window.vala"
				valide_window_show_start_page (self);
#line 1777 "window.c"
			} else {
#line 654 "window.vala"
				valide_window_hide_start_page (self);
#line 1781 "window.c"
			}
			(toggle_action == NULL) ? NULL : (toggle_action = (g_object_unref (toggle_action), NULL));
#line 656 "window.vala"
			break;
#line 1786 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label23) ? _tmp24__label23 : (_tmp24__label23 = g_quark_from_static_string ("view-toolbar"))))
		do {
			GtkToggleAction* toggle_action;
			GtkToggleAction* _tmp10_;
			GtkToggleAction* _tmp9_;
			GtkAction* _tmp8_;
			toggle_action = NULL;
			_tmp10_ = NULL;
#line 660 "window.vala"
			_tmp9_ = NULL;
#line 660 "window.vala"
			_tmp8_ = NULL;
#line 660 "window.vala"
			toggle_action = (_tmp10_ = (_tmp9_ = (_tmp8_ = action, GTK_IS_TOGGLE_ACTION (_tmp8_) ? ((GtkToggleAction*) _tmp8_) : NULL), (_tmp9_ == NULL) ? NULL : g_object_ref (_tmp9_)), (toggle_action == NULL) ? NULL : (toggle_action = (g_object_unref (toggle_action), NULL)), _tmp10_);
#line 661 "window.vala"
			if (gtk_toggle_action_get_active (toggle_action)) {
#line 663 "window.vala"
				gtk_widget_show ((GtkWidget*) self->priv->_toolbar);
#line 1805 "window.c"
			} else {
#line 667 "window.vala"
				gtk_widget_hide ((GtkWidget*) self->priv->_toolbar);
#line 1809 "window.c"
			}
			(toggle_action == NULL) ? NULL : (toggle_action = (g_object_unref (toggle_action), NULL));
#line 669 "window.vala"
			break;
#line 1814 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label24) ? _tmp24__label24 : (_tmp24__label24 = g_quark_from_static_string ("view-statusbar"))))
		do {
			GtkToggleAction* toggle_action;
			GtkToggleAction* _tmp13_;
			GtkToggleAction* _tmp12_;
			GtkAction* _tmp11_;
			toggle_action = NULL;
			_tmp13_ = NULL;
#line 673 "window.vala"
			_tmp12_ = NULL;
#line 673 "window.vala"
			_tmp11_ = NULL;
#line 673 "window.vala"
			toggle_action = (_tmp13_ = (_tmp12_ = (_tmp11_ = action, GTK_IS_TOGGLE_ACTION (_tmp11_) ? ((GtkToggleAction*) _tmp11_) : NULL), (_tmp12_ == NULL) ? NULL : g_object_ref (_tmp12_)), (toggle_action == NULL) ? NULL : (toggle_action = (g_object_unref (toggle_action), NULL)), _tmp13_);
#line 674 "window.vala"
			if (gtk_toggle_action_get_active (toggle_action)) {
#line 676 "window.vala"
				gtk_widget_show ((GtkWidget*) self->priv->_statusbar);
#line 1833 "window.c"
			} else {
#line 680 "window.vala"
				gtk_widget_hide ((GtkWidget*) self->priv->_statusbar);
#line 1837 "window.c"
			}
			(toggle_action == NULL) ? NULL : (toggle_action = (g_object_unref (toggle_action), NULL));
#line 682 "window.vala"
			break;
#line 1842 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label25) ? _tmp24__label25 : (_tmp24__label25 = g_quark_from_static_string ("view-side-panel"))))
		do {
			GtkToggleAction* toggle_action;
			GtkToggleAction* _tmp16_;
			GtkToggleAction* _tmp15_;
			GtkAction* _tmp14_;
			toggle_action = NULL;
			_tmp16_ = NULL;
#line 686 "window.vala"
			_tmp15_ = NULL;
#line 686 "window.vala"
			_tmp14_ = NULL;
#line 686 "window.vala"
			toggle_action = (_tmp16_ = (_tmp15_ = (_tmp14_ = action, GTK_IS_TOGGLE_ACTION (_tmp14_) ? ((GtkToggleAction*) _tmp14_) : NULL), (_tmp15_ == NULL) ? NULL : g_object_ref (_tmp15_)), (toggle_action == NULL) ? NULL : (toggle_action = (g_object_unref (toggle_action), NULL)), _tmp16_);
#line 687 "window.vala"
			if (gtk_toggle_action_get_active (toggle_action)) {
#line 689 "window.vala"
				gtk_widget_show ((GtkWidget*) self->priv->_side_panel);
#line 1861 "window.c"
			} else {
#line 693 "window.vala"
				gtk_widget_hide ((GtkWidget*) self->priv->_side_panel);
#line 1865 "window.c"
			}
			(toggle_action == NULL) ? NULL : (toggle_action = (g_object_unref (toggle_action), NULL));
#line 695 "window.vala"
			break;
#line 1870 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label26) ? _tmp24__label26 : (_tmp24__label26 = g_quark_from_static_string ("view-bottom-panel"))))
		do {
			GtkToggleAction* toggle_action;
			GtkToggleAction* _tmp19_;
			GtkToggleAction* _tmp18_;
			GtkAction* _tmp17_;
			toggle_action = NULL;
			_tmp19_ = NULL;
#line 699 "window.vala"
			_tmp18_ = NULL;
#line 699 "window.vala"
			_tmp17_ = NULL;
#line 699 "window.vala"
			toggle_action = (_tmp19_ = (_tmp18_ = (_tmp17_ = action, GTK_IS_TOGGLE_ACTION (_tmp17_) ? ((GtkToggleAction*) _tmp17_) : NULL), (_tmp18_ == NULL) ? NULL : g_object_ref (_tmp18_)), (toggle_action == NULL) ? NULL : (toggle_action = (g_object_unref (toggle_action), NULL)), _tmp19_);
#line 700 "window.vala"
			if (gtk_toggle_action_get_active (toggle_action)) {
#line 702 "window.vala"
				gtk_widget_show ((GtkWidget*) self->priv->_bottom_panel);
#line 1889 "window.c"
			} else {
#line 706 "window.vala"
				gtk_widget_hide ((GtkWidget*) self->priv->_bottom_panel);
#line 1893 "window.c"
			}
			(toggle_action == NULL) ? NULL : (toggle_action = (g_object_unref (toggle_action), NULL));
#line 708 "window.vala"
			break;
#line 1898 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label27) ? _tmp24__label27 : (_tmp24__label27 = g_quark_from_static_string ("view-fullscreen"))))
		do {
			GtkToggleAction* toggle_action;
			GtkToggleAction* _tmp22_;
			GtkToggleAction* _tmp21_;
			GtkAction* _tmp20_;
			toggle_action = NULL;
			_tmp22_ = NULL;
#line 712 "window.vala"
			_tmp21_ = NULL;
#line 712 "window.vala"
			_tmp20_ = NULL;
#line 712 "window.vala"
			toggle_action = (_tmp22_ = (_tmp21_ = (_tmp20_ = action, GTK_IS_TOGGLE_ACTION (_tmp20_) ? ((GtkToggleAction*) _tmp20_) : NULL), (_tmp21_ == NULL) ? NULL : g_object_ref (_tmp21_)), (toggle_action == NULL) ? NULL : (toggle_action = (g_object_unref (toggle_action), NULL)), _tmp22_);
#line 713 "window.vala"
			if (gtk_toggle_action_get_active (toggle_action)) {
#line 715 "window.vala"
				valide_window_fullscreen (self);
#line 1917 "window.c"
			} else {
#line 719 "window.vala"
				valide_window_unfullscreen (self);
#line 1921 "window.c"
			}
			(toggle_action == NULL) ? NULL : (toggle_action = (g_object_unref (toggle_action), NULL));
#line 721 "window.vala"
			break;
#line 1926 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label28) ? _tmp24__label28 : (_tmp24__label28 = g_quark_from_static_string ("project-new"))))
		do {
#line 725 "window.vala"
			valide_project_manager_create (self->priv->_projects);
#line 726 "window.vala"
			break;
#line 1933 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label29) ? _tmp24__label29 : (_tmp24__label29 = g_quark_from_static_string ("project-open"))))
		do {
#line 728 "window.vala"
			valide_project_manager_open (self->priv->_projects, NULL, &_inner_error_);
#line 1938 "window.c"
			if (_inner_error_ != NULL) {
				goto __catch59_g_error;
				goto __finally59;
			}
#line 729 "window.vala"
			break;
#line 1945 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label30) ? _tmp24__label30 : (_tmp24__label30 = g_quark_from_static_string ("project-add"))))
		do {
#line 731 "window.vala"
			if (valide_project_manager_get_current (self->priv->_projects) != NULL) {
#line 733 "window.vala"
				valide_project_add_file (valide_project_manager_get_current (self->priv->_projects), NULL);
#line 1952 "window.c"
			} else {
				_inner_error_ = g_error_new_literal (VALIDE_PROJECT_ERROR, VALIDE_PROJECT_ERROR_NO_PROJECT, _ ("No project open!"));
				if (_inner_error_ != NULL) {
					goto __catch59_g_error;
					goto __finally59;
				}
			}
#line 739 "window.vala"
			break;
#line 1962 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label31) ? _tmp24__label31 : (_tmp24__label31 = g_quark_from_static_string ("project-remove"))))
		do {
#line 741 "window.vala"
			if (valide_project_manager_get_current (self->priv->_projects) != NULL) {
#line 743 "window.vala"
				valide_project_remove_file (valide_project_manager_get_current (self->priv->_projects), NULL);
#line 1969 "window.c"
			} else {
				_inner_error_ = g_error_new_literal (VALIDE_PROJECT_ERROR, VALIDE_PROJECT_ERROR_NO_PROJECT, _ ("No project open!"));
				if (_inner_error_ != NULL) {
					goto __catch59_g_error;
					goto __finally59;
				}
			}
#line 749 "window.vala"
			break;
#line 1979 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label32) ? _tmp24__label32 : (_tmp24__label32 = g_quark_from_static_string ("project-options"))))
		do {
#line 751 "window.vala"
			if (valide_project_manager_get_current (self->priv->_projects) != NULL) {
#line 753 "window.vala"
				valide_project_option_dialog (valide_project_manager_get_current (self->priv->_projects), (GtkWindow*) self);
#line 1986 "window.c"
			} else {
				_inner_error_ = g_error_new_literal (VALIDE_PROJECT_ERROR, VALIDE_PROJECT_ERROR_NO_PROJECT, _ ("No project open!"));
				if (_inner_error_ != NULL) {
					goto __catch59_g_error;
					goto __finally59;
				}
			}
#line 759 "window.vala"
			break;
#line 1996 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label33) ? _tmp24__label33 : (_tmp24__label33 = g_quark_from_static_string ("project-configure"))))
		do {
#line 761 "window.vala"
			if (valide_project_manager_get_current (self->priv->_projects) != NULL) {
#line 763 "window.vala"
				valide_project_configure (valide_project_manager_get_current (self->priv->_projects));
#line 2003 "window.c"
			} else {
				_inner_error_ = g_error_new_literal (VALIDE_PROJECT_ERROR, VALIDE_PROJECT_ERROR_NO_PROJECT, _ ("No project open!"));
				if (_inner_error_ != NULL) {
					goto __catch59_g_error;
					goto __finally59;
				}
			}
#line 769 "window.vala"
			break;
#line 2013 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label34) ? _tmp24__label34 : (_tmp24__label34 = g_quark_from_static_string ("project-build"))))
		do {
#line 771 "window.vala"
			if (valide_project_manager_get_current (self->priv->_projects) != NULL) {
#line 773 "window.vala"
				valide_project_build (valide_project_manager_get_current (self->priv->_projects));
#line 2020 "window.c"
			} else {
				_inner_error_ = g_error_new_literal (VALIDE_PROJECT_ERROR, VALIDE_PROJECT_ERROR_NO_PROJECT, _ ("No project open!"));
				if (_inner_error_ != NULL) {
					goto __catch59_g_error;
					goto __finally59;
				}
			}
#line 779 "window.vala"
			break;
#line 2030 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label35) ? _tmp24__label35 : (_tmp24__label35 = g_quark_from_static_string ("project-install"))))
		do {
#line 781 "window.vala"
			if (valide_project_manager_get_current (self->priv->_projects) != NULL) {
#line 783 "window.vala"
				valide_project_install (valide_project_manager_get_current (self->priv->_projects));
#line 2037 "window.c"
			} else {
				_inner_error_ = g_error_new_literal (VALIDE_PROJECT_ERROR, VALIDE_PROJECT_ERROR_NO_PROJECT, _ ("No project open!"));
				if (_inner_error_ != NULL) {
					goto __catch59_g_error;
					goto __finally59;
				}
			}
#line 789 "window.vala"
			break;
#line 2047 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label36) ? _tmp24__label36 : (_tmp24__label36 = g_quark_from_static_string ("project-uninstall"))))
		do {
#line 791 "window.vala"
			if (valide_project_manager_get_current (self->priv->_projects) != NULL) {
#line 793 "window.vala"
				valide_project_uninstall (valide_project_manager_get_current (self->priv->_projects));
#line 2054 "window.c"
			} else {
				_inner_error_ = g_error_new_literal (VALIDE_PROJECT_ERROR, VALIDE_PROJECT_ERROR_NO_PROJECT, _ ("No project open!"));
				if (_inner_error_ != NULL) {
					goto __catch59_g_error;
					goto __finally59;
				}
			}
#line 799 "window.vala"
			break;
#line 2064 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label37) ? _tmp24__label37 : (_tmp24__label37 = g_quark_from_static_string ("project-dist"))))
		do {
#line 801 "window.vala"
			if (valide_project_manager_get_current (self->priv->_projects) != NULL) {
#line 803 "window.vala"
				valide_project_dist (valide_project_manager_get_current (self->priv->_projects));
#line 2071 "window.c"
			} else {
				_inner_error_ = g_error_new_literal (VALIDE_PROJECT_ERROR, VALIDE_PROJECT_ERROR_NO_PROJECT, _ ("No project open!"));
				if (_inner_error_ != NULL) {
					goto __catch59_g_error;
					goto __finally59;
				}
			}
#line 809 "window.vala"
			break;
#line 2081 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label38) ? _tmp24__label38 : (_tmp24__label38 = g_quark_from_static_string ("project-clean"))))
		do {
#line 811 "window.vala"
			if (valide_project_manager_get_current (self->priv->_projects) != NULL) {
#line 813 "window.vala"
				valide_project_clean (valide_project_manager_get_current (self->priv->_projects));
#line 2088 "window.c"
			} else {
				_inner_error_ = g_error_new_literal (VALIDE_PROJECT_ERROR, VALIDE_PROJECT_ERROR_NO_PROJECT, _ ("No project open!"));
				if (_inner_error_ != NULL) {
					goto __catch59_g_error;
					goto __finally59;
				}
			}
#line 819 "window.vala"
			break;
#line 2098 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label39) ? _tmp24__label39 : (_tmp24__label39 = g_quark_from_static_string ("project-distclean"))))
		do {
#line 821 "window.vala"
			if (valide_project_manager_get_current (self->priv->_projects) != NULL) {
#line 823 "window.vala"
				valide_project_distclean (valide_project_manager_get_current (self->priv->_projects));
#line 2105 "window.c"
			} else {
				_inner_error_ = g_error_new_literal (VALIDE_PROJECT_ERROR, VALIDE_PROJECT_ERROR_NO_PROJECT, _ ("No project open!"));
				if (_inner_error_ != NULL) {
					goto __catch59_g_error;
					goto __finally59;
				}
			}
#line 829 "window.vala"
			break;
#line 2115 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label40) ? _tmp24__label40 : (_tmp24__label40 = g_quark_from_static_string ("project-execute"))))
		do {
#line 831 "window.vala"
			if (valide_project_manager_get_current (self->priv->_projects) != NULL) {
#line 833 "window.vala"
				valide_project_execute (valide_project_manager_get_current (self->priv->_projects));
#line 2122 "window.c"
			} else {
				_inner_error_ = g_error_new_literal (VALIDE_PROJECT_ERROR, VALIDE_PROJECT_ERROR_NO_PROJECT, _ ("No project open!"));
				if (_inner_error_ != NULL) {
					goto __catch59_g_error;
					goto __finally59;
				}
			}
#line 839 "window.vala"
			break;
#line 2132 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label41) ? _tmp24__label41 : (_tmp24__label41 = g_quark_from_static_string ("project-abort"))))
		do {
#line 841 "window.vala"
			valide_executable_manager_stop (self->priv->_executables, &_inner_error_);
#line 2137 "window.c"
			if (_inner_error_ != NULL) {
				goto __catch59_g_error;
				goto __finally59;
			}
#line 842 "window.vala"
			break;
#line 2144 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label42) ? _tmp24__label42 : (_tmp24__label42 = g_quark_from_static_string ("project-close"))))
		do {
#line 844 "window.vala"
			if (valide_project_manager_get_current (self->priv->_projects) != NULL) {
#line 846 "window.vala"
				valide_project_close (valide_project_manager_get_current (self->priv->_projects));
#line 2151 "window.c"
			} else {
				_inner_error_ = g_error_new_literal (VALIDE_PROJECT_ERROR, VALIDE_PROJECT_ERROR_NO_PROJECT, _ ("No project open!"));
				if (_inner_error_ != NULL) {
					goto __catch59_g_error;
					goto __finally59;
				}
			}
#line 852 "window.vala"
			break;
#line 2161 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label43) ? _tmp24__label43 : (_tmp24__label43 = g_quark_from_static_string ("help-about"))))
		do {
			ValideAboutDialog* dialog;
			dialog = g_object_ref_sink (valide_about_dialog_new ());
#line 857 "window.vala"
			gtk_dialog_run ((GtkDialog*) dialog);
#line 858 "window.vala"
			gtk_object_destroy ((GtkObject*) dialog);
#line 2170 "window.c"
			(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
#line 859 "window.vala"
			break;
#line 2174 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label44) ? _tmp24__label44 : (_tmp24__label44 = g_quark_from_static_string ("project-file-new"))))
		do {
#line 863 "window.vala"
			if (valide_project_manager_get_current (self->priv->_projects) != NULL) {
#line 865 "window.vala"
				valide_project_create_file (valide_project_manager_get_current (self->priv->_projects), &_inner_error_);
#line 2181 "window.c"
				if (_inner_error_ != NULL) {
					goto __catch59_g_error;
					goto __finally59;
				}
			} else {
				_inner_error_ = g_error_new_literal (VALIDE_PROJECT_ERROR, VALIDE_PROJECT_ERROR_NO_PROJECT, _ ("No project open!"));
				if (_inner_error_ != NULL) {
					goto __catch59_g_error;
					goto __finally59;
				}
			}
#line 871 "window.vala"
			break;
#line 2195 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label45) ? _tmp24__label45 : (_tmp24__label45 = g_quark_from_static_string ("project-file-open"))))
		do {
#line 873 "window.vala"
			valide_project_manager_open_selected_file (self->priv->_projects);
#line 874 "window.vala"
			break;
#line 2202 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label46) ? _tmp24__label46 : (_tmp24__label46 = g_quark_from_static_string ("project-file-add"))))
		do {
#line 876 "window.vala"
			if (valide_project_manager_get_current (self->priv->_projects) != NULL) {
#line 878 "window.vala"
				valide_project_add_file (valide_project_manager_get_current (self->priv->_projects), NULL);
#line 2209 "window.c"
			} else {
				_inner_error_ = g_error_new_literal (VALIDE_PROJECT_ERROR, VALIDE_PROJECT_ERROR_NO_PROJECT, _ ("No project open!"));
				if (_inner_error_ != NULL) {
					goto __catch59_g_error;
					goto __finally59;
				}
			}
#line 884 "window.vala"
			break;
#line 2219 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label47) ? _tmp24__label47 : (_tmp24__label47 = g_quark_from_static_string ("project-file-remove"))))
		do {
#line 886 "window.vala"
			valide_project_manager_remove_selected_file (self->priv->_projects);
#line 887 "window.vala"
			break;
#line 2226 "window.c"
		} while (0); else if (_tmp24_ == ((0 != _tmp24__label48) ? _tmp24__label48 : (_tmp24__label48 = g_quark_from_static_string ("leave-fullscreen"))))
		do {
#line 891 "window.vala"
			valide_window_unfullscreen (self);
#line 892 "window.vala"
			valide_ui_manager_action_set_toggled (self->priv->_ui_manager, "view-fullscreen", FALSE);
#line 893 "window.vala"
			break;
#line 2235 "window.c"
		} while (0);
	}
	goto __finally59;
	__catch59_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 898 "window.vala"
			g_warning (e->message);
#line 2247 "window.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally59:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
}


#line 902 "window.vala"
static void valide_window_update_title (ValideWindow* self) {
#line 2262 "window.c"
	char* title;
	ValideiDocument* document;
	ValideProject* project;
	ValideiDocument* _tmp1_;
	ValideiDocument* _tmp0_;
	ValideProject* _tmp8_;
	ValideProject* _tmp7_;
	char* _tmp11_;
#line 902 "window.vala"
	g_return_if_fail (self != NULL);
#line 2273 "window.c"
	title = g_strdup ("");
	document = NULL;
	project = NULL;
	_tmp1_ = NULL;
#line 908 "window.vala"
	_tmp0_ = NULL;
#line 908 "window.vala"
	document = (_tmp1_ = (_tmp0_ = valide_document_manager_get_current (self->priv->_documents), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (document == NULL) ? NULL : (document = (g_object_unref (document), NULL)), _tmp1_);
#line 909 "window.vala"
	if (document != NULL) {
#line 911 "window.vala"
		if (_vala_strcmp0 (valide_idocument_get_path (document), "") != 0) {
#line 2286 "window.c"
			char* dir;
			char* filename;
			char* _tmp3_;
			char* _tmp2_;
			char* _tmp4_;
			char* _tmp6_;
			char* _tmp5_;
			dir = NULL;
			filename = NULL;
			_tmp3_ = NULL;
#line 2032 "glib-2.0.vapi"
			_tmp2_ = NULL;
#line 916 "window.vala"
			dir = (_tmp3_ = valide_utils_replace_home_dir_with_tilde (_tmp2_ = g_path_get_dirname (valide_idocument_get_path (document))), dir = (g_free (dir), NULL), _tmp3_);
#line 2301 "window.c"
			_tmp2_ = (g_free (_tmp2_), NULL);
			_tmp4_ = NULL;
#line 917 "window.vala"
			filename = (_tmp4_ = g_path_get_basename (valide_idocument_get_path (document)), filename = (g_free (filename), NULL), _tmp4_);
#line 2306 "window.c"
			_tmp6_ = NULL;
#line 820 "glib-2.0.vapi"
			_tmp5_ = NULL;
#line 918 "window.vala"
			title = (_tmp6_ = g_strconcat (title, _tmp5_ = g_strdup_printf ("%s (%s) - ", filename, dir), NULL), title = (g_free (title), NULL), _tmp6_);
#line 2312 "window.c"
			_tmp5_ = (g_free (_tmp5_), NULL);
			dir = (g_free (dir), NULL);
			filename = (g_free (filename), NULL);
		}
	}
	_tmp8_ = NULL;
#line 922 "window.vala"
	_tmp7_ = NULL;
#line 922 "window.vala"
	project = (_tmp8_ = (_tmp7_ = valide_project_manager_get_current (self->priv->_projects), (_tmp7_ == NULL) ? NULL : g_object_ref (_tmp7_)), (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), _tmp8_);
#line 923 "window.vala"
	if (project != NULL) {
#line 925 "window.vala"
		if (_vala_strcmp0 (valide_project_get_name (project), "") != 0) {
#line 2327 "window.c"
			char* _tmp10_;
			char* _tmp9_;
			_tmp10_ = NULL;
			_tmp9_ = NULL;
#line 927 "window.vala"
			title = (_tmp10_ = g_strconcat (title, _tmp9_ = g_strconcat (valide_project_get_name (project), " - ", NULL), NULL), title = (g_free (title), NULL), _tmp10_);
#line 2334 "window.c"
			_tmp9_ = (g_free (_tmp9_), NULL);
		}
	}
	_tmp11_ = NULL;
#line 931 "window.vala"
	title = (_tmp11_ = g_strconcat (title, "Val(a)IDE", NULL), title = (g_free (title), NULL), _tmp11_);
#line 932 "window.vala"
	gtk_window_set_title ((GtkWindow*) self, title);
#line 2343 "window.c"
	title = (g_free (title), NULL);
	(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
	(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
}


#line 935 "window.vala"
static gboolean valide_window_is_fullscreen (ValideWindow* self) {
#line 2352 "window.c"
	gboolean result;
#line 935 "window.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 2356 "window.c"
	result = (self->priv->window_state & GDK_WINDOW_STATE_FULLSCREEN) != 0;
	return result;
}


#line 323 "window.vala"
static void _valide_window_message_glog_func (const char* log_domain, GLogLevelFlags log_levels, const char* message, gpointer self) {
#line 2364 "window.c"
	valide_window_message (self, log_domain, log_levels, message);
}


/**
   * Launch the application
   */
#line 1035 "window.vala"
void valide_window_run (ValideWindow* self) {
#line 2374 "window.c"
	gboolean full_screen;
#line 1035 "window.vala"
	g_return_if_fail (self != NULL);
#line 2378 "window.c"
	full_screen = FALSE;
#line 1039 "window.vala"
	valide_window_show_start_page (self);
#line 1041 "window.vala"
	full_screen = valide_abstract_config_manager_get_boolean ((ValideAbstractConfigManager*) self->priv->_config_manager, "General", "full-screen");
#line 1042 "window.vala"
	if (full_screen) {
#line 1044 "window.vala"
		gtk_window_maximize ((GtkWindow*) self);
#line 2388 "window.c"
	} else {
		gint width;
		gint height;
		gboolean _tmp0_;
		width = 0;
		height = 0;
#line 1051 "window.vala"
		width = valide_abstract_config_manager_get_integer ((ValideAbstractConfigManager*) self->priv->_config_manager, "General", "width");
#line 1052 "window.vala"
		height = valide_abstract_config_manager_get_integer ((ValideAbstractConfigManager*) self->priv->_config_manager, "General", "height");
#line 2399 "window.c"
		_tmp0_ = FALSE;
#line 1053 "window.vala"
		if (width > 1) {
#line 1053 "window.vala"
			_tmp0_ = height > 1;
#line 2405 "window.c"
		} else {
#line 1053 "window.vala"
			_tmp0_ = FALSE;
#line 2409 "window.c"
		}
#line 1053 "window.vala"
		if (_tmp0_) {
#line 1055 "window.vala"
			gtk_window_resize ((GtkWindow*) self, width, height);
#line 2415 "window.c"
		}
	}
#line 1059 "window.vala"
	gtk_widget_hide ((GtkWidget*) self->priv->fullscreen_toolbar);
#line 1060 "window.vala"
	gtk_widget_show_all ((GtkWidget*) self);
#line 1062 "window.vala"
	g_log_set_handler (NULL, (G_LOG_LEVEL_WARNING | G_LOG_LEVEL_MESSAGE) | G_LOG_LEVEL_CRITICAL, _valide_window_message_glog_func, self);
#line 2424 "window.c"
}


/**
   * Show the side panel
   *
   * @param widget The widget to show, or null for the current widget
   */
#line 1072 "window.vala"
void valide_window_side_panel_show (ValideWindow* self, GtkWidget* widget) {
#line 1072 "window.vala"
	g_return_if_fail (self != NULL);
#line 1074 "window.vala"
	if (widget != NULL) {
#line 1076 "window.vala"
		valide_panel_set_current_page (self->priv->_side_panel, widget);
#line 2441 "window.c"
	}
#line 1078 "window.vala"
	gtk_widget_show ((GtkWidget*) self->priv->_side_panel);
#line 1079 "window.vala"
	valide_ui_manager_action_set_toggled (self->priv->_ui_manager, "view-side-panel", TRUE);
#line 2447 "window.c"
}


/**
   * Hide the side panel
   */
#line 1085 "window.vala"
void valide_window_side_panel_hide (ValideWindow* self) {
#line 1085 "window.vala"
	g_return_if_fail (self != NULL);
#line 1087 "window.vala"
	gtk_widget_hide ((GtkWidget*) self->priv->_side_panel);
#line 1088 "window.vala"
	valide_ui_manager_action_set_toggled (self->priv->_ui_manager, "view-side-panel", FALSE);
#line 2462 "window.c"
}


/**
   * Show the bottom panel
   *
   * @param widget The widget to show, or null for the current widget
   */
#line 1096 "window.vala"
void valide_window_bottom_panel_show (ValideWindow* self, GtkWidget* widget) {
#line 1096 "window.vala"
	g_return_if_fail (self != NULL);
#line 1098 "window.vala"
	if (widget != NULL) {
#line 1100 "window.vala"
		valide_panel_set_current_page (self->priv->_bottom_panel, widget);
#line 2479 "window.c"
	}
#line 1102 "window.vala"
	gtk_widget_show ((GtkWidget*) self->priv->_bottom_panel);
#line 1103 "window.vala"
	valide_ui_manager_action_set_toggled (self->priv->_ui_manager, "view-bottom-panel", TRUE);
#line 2485 "window.c"
}


/**
   * Hide the bottom panel
   */
#line 1109 "window.vala"
void valide_window_bottom_panel_hide (ValideWindow* self) {
#line 1109 "window.vala"
	g_return_if_fail (self != NULL);
#line 1111 "window.vala"
	gtk_widget_hide ((GtkWidget*) self->priv->_bottom_panel);
#line 1112 "window.vala"
	valide_ui_manager_action_set_toggled (self->priv->_ui_manager, "view-bottom-panel", FALSE);
#line 2500 "window.c"
}


/**
   * @see Gtk.Window.fullscreen
   */
#line 1118 "window.vala"
void valide_window_fullscreen (ValideWindow* self) {
#line 1118 "window.vala"
	g_return_if_fail (self != NULL);
#line 1120 "window.vala"
	if (!valide_window_is_fullscreen (self)) {
#line 1122 "window.vala"
		gtk_window_fullscreen (GTK_WINDOW (self));
#line 1123 "window.vala"
		gtk_widget_hide ((GtkWidget*) self->priv->_menu);
#line 1124 "window.vala"
		gtk_widget_hide ((GtkWidget*) self->priv->_toolbar);
#line 1125 "window.vala"
		gtk_widget_hide ((GtkWidget*) self->priv->_statusbar);
#line 1126 "window.vala"
		valide_fullscreen_toolbar_show (self->priv->fullscreen_toolbar);
#line 2523 "window.c"
	}
}


/**
   * @see Gtk.Window.unfullscreen
   */
#line 1133 "window.vala"
void valide_window_unfullscreen (ValideWindow* self) {
#line 1133 "window.vala"
	g_return_if_fail (self != NULL);
#line 1135 "window.vala"
	if (valide_window_is_fullscreen (self)) {
#line 1137 "window.vala"
		gtk_window_unfullscreen (GTK_WINDOW (self));
#line 1138 "window.vala"
		gtk_widget_show ((GtkWidget*) self->priv->_menu);
#line 1139 "window.vala"
		gtk_widget_show ((GtkWidget*) self->priv->_toolbar);
#line 1140 "window.vala"
		gtk_widget_show ((GtkWidget*) self->priv->_statusbar);
#line 1141 "window.vala"
		gtk_widget_hide ((GtkWidget*) self->priv->fullscreen_toolbar);
#line 2547 "window.c"
	}
}


/**
   * Show the start page
   */
#line 1148 "window.vala"
void valide_window_show_start_page (ValideWindow* self) {
#line 2557 "window.c"
	gint page_num;
#line 1148 "window.vala"
	g_return_if_fail (self != NULL);
#line 2561 "window.c"
	page_num = 0;
#line 1152 "window.vala"
	page_num = gtk_notebook_page_num ((GtkNotebook*) self->priv->_documents, (GtkWidget*) self->priv->start_page);
#line 1153 "window.vala"
	if (page_num < 0) {
#line 1155 "window.vala"
		valide_document_manager_append_page (self->priv->_documents, (ValideiDocument*) self->priv->start_page);
#line 1156 "window.vala"
		gtk_widget_show_all ((GtkWidget*) self->priv->start_page);
#line 2571 "window.c"
	}
#line 1158 "window.vala"
	valide_document_manager_set_current (self->priv->_documents, (ValideiDocument*) self->priv->start_page);
#line 1159 "window.vala"
	valide_ui_manager_action_set_toggled (self->priv->_ui_manager, "view-start-page", TRUE);
#line 2577 "window.c"
}


/**
   * Hide the start page
   */
#line 1165 "window.vala"
void valide_window_hide_start_page (ValideWindow* self) {
#line 1165 "window.vala"
	g_return_if_fail (self != NULL);
#line 1167 "window.vala"
	valide_document_manager_remove_page (self->priv->_documents, (ValideiDocument*) self->priv->start_page);
#line 1168 "window.vala"
	valide_ui_manager_action_set_toggled (self->priv->_ui_manager, "view-start-page", FALSE);
#line 2592 "window.c"
}


/**
 * The main window of the application
 */
#line 196 "window.vala"
ValideWindow* valide_window_construct (GType object_type) {
#line 2601 "window.c"
	ValideWindow * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


#line 196 "window.vala"
ValideWindow* valide_window_new (void) {
#line 196 "window.vala"
	return valide_window_construct (VALIDE_TYPE_WINDOW);
#line 2612 "window.c"
}


ValideUIManager* valide_window_get_ui_manager (ValideWindow* self) {
	ValideUIManager* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_ui_manager;
	return result;
}


static void valide_window_set_ui_manager (ValideWindow* self, ValideUIManager* value) {
	ValideUIManager* _tmp1_;
	ValideUIManager* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 212 "window.vala"
	_tmp0_ = NULL;
#line 2631 "window.c"
	self->priv->_ui_manager = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_ui_manager == NULL) ? NULL : (self->priv->_ui_manager = (g_object_unref (self->priv->_ui_manager), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "ui-manager");
}


ValideRecentManagerUI* valide_window_get_recent_manager (ValideWindow* self) {
	ValideRecentManagerUI* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_recent_manager;
	return result;
}


static void valide_window_set_recent_manager (ValideWindow* self, ValideRecentManagerUI* value) {
	ValideRecentManagerUI* _tmp1_;
	ValideRecentManagerUI* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 221 "window.vala"
	_tmp0_ = NULL;
#line 2652 "window.c"
	self->priv->_recent_manager = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_recent_manager == NULL) ? NULL : (self->priv->_recent_manager = (g_object_unref (self->priv->_recent_manager), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "recent-manager");
}


ValideConfigManager* valide_window_get_config_manager (ValideWindow* self) {
	ValideConfigManager* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_config_manager;
	return result;
}


static void valide_window_set_config_manager (ValideWindow* self, ValideConfigManager* value) {
	ValideConfigManager* _tmp1_;
	ValideConfigManager* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 230 "window.vala"
	_tmp0_ = NULL;
#line 2673 "window.c"
	self->priv->_config_manager = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_config_manager == NULL) ? NULL : (self->priv->_config_manager = (g_object_unref (self->priv->_config_manager), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "config-manager");
}


ValideDocumentManager* valide_window_get_documents (ValideWindow* self) {
	ValideDocumentManager* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_documents;
	return result;
}


static void valide_window_set_documents (ValideWindow* self, ValideDocumentManager* value) {
	ValideDocumentManager* _tmp1_;
	ValideDocumentManager* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 239 "window.vala"
	_tmp0_ = NULL;
#line 2694 "window.c"
	self->priv->_documents = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_documents == NULL) ? NULL : (self->priv->_documents = (g_object_unref (self->priv->_documents), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "documents");
}


ValideProjectManager* valide_window_get_projects (ValideWindow* self) {
	ValideProjectManager* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_projects;
	return result;
}


static void valide_window_set_projects (ValideWindow* self, ValideProjectManager* value) {
	ValideProjectManager* _tmp1_;
	ValideProjectManager* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 248 "window.vala"
	_tmp0_ = NULL;
#line 2715 "window.c"
	self->priv->_projects = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_projects == NULL) ? NULL : (self->priv->_projects = (g_object_unref (self->priv->_projects), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "projects");
}


ValideBuilderManager* valide_window_get_builders (ValideWindow* self) {
	ValideBuilderManager* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_builders;
	return result;
}


static void valide_window_set_builders (ValideWindow* self, ValideBuilderManager* value) {
	ValideBuilderManager* _tmp1_;
	ValideBuilderManager* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 257 "window.vala"
	_tmp0_ = NULL;
#line 2736 "window.c"
	self->priv->_builders = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_builders == NULL) ? NULL : (self->priv->_builders = (g_object_unref (self->priv->_builders), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "builders");
}


ValideExecutableManager* valide_window_get_executables (ValideWindow* self) {
	ValideExecutableManager* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_executables;
	return result;
}


static void valide_window_set_executables (ValideWindow* self, ValideExecutableManager* value) {
	ValideExecutableManager* _tmp1_;
	ValideExecutableManager* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 266 "window.vala"
	_tmp0_ = NULL;
#line 2757 "window.c"
	self->priv->_executables = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_executables == NULL) ? NULL : (self->priv->_executables = (g_object_unref (self->priv->_executables), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "executables");
}


ValidePluginManager* valide_window_get_plugins (ValideWindow* self) {
	ValidePluginManager* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_plugins;
	return result;
}


static void valide_window_set_plugins (ValideWindow* self, ValidePluginManager* value) {
	ValidePluginManager* _tmp1_;
	ValidePluginManager* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 275 "window.vala"
	_tmp0_ = NULL;
#line 2778 "window.c"
	self->priv->_plugins = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_plugins == NULL) ? NULL : (self->priv->_plugins = (g_object_unref (self->priv->_plugins), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "plugins");
}


GtkMenuBar* valide_window_get_menu (ValideWindow* self) {
	GtkMenuBar* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_menu;
	return result;
}


static void valide_window_set_menu (ValideWindow* self, GtkMenuBar* value) {
	GtkMenuBar* _tmp1_;
	GtkMenuBar* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 284 "window.vala"
	_tmp0_ = NULL;
#line 2799 "window.c"
	self->priv->_menu = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_menu == NULL) ? NULL : (self->priv->_menu = (g_object_unref (self->priv->_menu), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "menu");
}


GtkToolbar* valide_window_get_toolbar (ValideWindow* self) {
	GtkToolbar* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_toolbar;
	return result;
}


static void valide_window_set_toolbar (ValideWindow* self, GtkToolbar* value) {
	GtkToolbar* _tmp1_;
	GtkToolbar* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 293 "window.vala"
	_tmp0_ = NULL;
#line 2820 "window.c"
	self->priv->_toolbar = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_toolbar == NULL) ? NULL : (self->priv->_toolbar = (g_object_unref (self->priv->_toolbar), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "toolbar");
}


ValidePanel* valide_window_get_side_panel (ValideWindow* self) {
	ValidePanel* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_side_panel;
	return result;
}


static void valide_window_set_side_panel (ValideWindow* self, ValidePanel* value) {
	ValidePanel* _tmp1_;
	ValidePanel* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 302 "window.vala"
	_tmp0_ = NULL;
#line 2841 "window.c"
	self->priv->_side_panel = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_side_panel == NULL) ? NULL : (self->priv->_side_panel = (g_object_unref (self->priv->_side_panel), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "side-panel");
}


ValidePanel* valide_window_get_bottom_panel (ValideWindow* self) {
	ValidePanel* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_bottom_panel;
	return result;
}


static void valide_window_set_bottom_panel (ValideWindow* self, ValidePanel* value) {
	ValidePanel* _tmp1_;
	ValidePanel* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 311 "window.vala"
	_tmp0_ = NULL;
#line 2862 "window.c"
	self->priv->_bottom_panel = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_bottom_panel == NULL) ? NULL : (self->priv->_bottom_panel = (g_object_unref (self->priv->_bottom_panel), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "bottom-panel");
}


ValideStatusbar* valide_window_get_statusbar (ValideWindow* self) {
	ValideStatusbar* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_statusbar;
	return result;
}


static void valide_window_set_statusbar (ValideWindow* self, ValideStatusbar* value) {
	ValideStatusbar* _tmp1_;
	ValideStatusbar* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 320 "window.vala"
	_tmp0_ = NULL;
#line 2883 "window.c"
	self->priv->_statusbar = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_statusbar == NULL) ? NULL : (self->priv->_statusbar = (g_object_unref (self->priv->_statusbar), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "statusbar");
}


#line 351 "window.vala"
static gboolean _valide_window_on_delete_event_gtk_widget_delete_event (GtkWidget* _sender, GdkEvent* event, gpointer self) {
#line 2891 "window.c"
	return valide_window_on_delete_event (self);
}


static gboolean _lambda54_ (GtkWidget* s, const GdkEventWindowState* e, ValideWindow* self) {
	gboolean result;
	g_return_val_if_fail (s != NULL, FALSE);
#line 949 "window.vala"
	self->priv->window_state = (*e).new_window_state;
#line 950 "window.vala"
	if (((*e).changed_mask & (GDK_WINDOW_STATE_MAXIMIZED | GDK_WINDOW_STATE_FULLSCREEN)) != 0) {
#line 2903 "window.c"
		gboolean full_screen;
		full_screen = FALSE;
#line 955 "window.vala"
		full_screen = ((*e).new_window_state & (GDK_WINDOW_STATE_MAXIMIZED | GDK_WINDOW_STATE_FULLSCREEN)) != 0;
#line 958 "window.vala"
		valide_statusbar_set_has_resize_grip (self->priv->_statusbar, !full_screen);
#line 959 "window.vala"
		valide_abstract_config_manager_set_boolean ((ValideAbstractConfigManager*) self->priv->_config_manager, "General", "full-screen", full_screen);
#line 2912 "window.c"
	}
}


static gboolean __lambda54__gtk_widget_window_state_event (GtkWidget* _sender, const GdkEventWindowState* event, gpointer self) {
	return _lambda54_ (_sender, event, self);
}


#line 902 "window.vala"
static void _valide_window_update_title_valide_document_manager_tab_changed (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self) {
#line 2924 "window.c"
	valide_window_update_title (self);
}


#line 902 "window.vala"
static void _valide_window_update_title_valide_document_manager_tab_state_changed (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self) {
#line 2931 "window.c"
	valide_window_update_title (self);
}


#line 902 "window.vala"
static void _valide_window_update_title_valide_document_manager_tab_added (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self) {
#line 2938 "window.c"
	valide_window_update_title (self);
}


#line 902 "window.vala"
static void _valide_window_update_title_valide_document_manager_tab_removed (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self) {
#line 2945 "window.c"
	valide_window_update_title (self);
}


static void _lambda55_ (ValideDocumentManager* s, ValideiDocument* d, gint r, gint c, ValideWindow* self) {
	g_return_if_fail (s != NULL);
	g_return_if_fail (d != NULL);
#line 973 "window.vala"
	valide_statusbar_set_cursor_position (self->priv->_statusbar, r, c);
#line 2955 "window.c"
}


static void __lambda55__valide_document_manager_tab_cursor_moved (ValideDocumentManager* _sender, ValideiDocument* document, gint row, gint col, gpointer self) {
	_lambda55_ (_sender, document, row, col, self);
}


static void _lambda56_ (ValideWindow* self) {
#line 978 "window.vala"
	valide_window_bottom_panel_show (self, (GtkWidget*) self->priv->_executables);
#line 2967 "window.c"
}


static void __lambda56__valide_executable_manager_run_exec (ValideExecutableManager* _sender, ValideExecutable* executable, gpointer self) {
	_lambda56_ (self);
}


static void _lambda57_ (ValideWindow* self) {
#line 987 "window.vala"
	valide_window_side_panel_show (self, (GtkWidget*) self->priv->_projects);
#line 2979 "window.c"
}


static void __lambda57__valide_project_manager_project_opened (ValideProjectManager* _sender, ValideProject* project, gpointer self) {
	_lambda57_ (self);
}


static void _lambda58_ (ValideWindow* self) {
#line 990 "window.vala"
	valide_window_update_title (self);
#line 2991 "window.c"
}


static void __lambda58__valide_project_manager_project_options_changed (ValideProjectManager* _sender, ValideProject* project, gpointer self) {
	_lambda58_ (self);
}


#line 902 "window.vala"
static void _valide_window_update_title_valide_project_manager_project_opened (ValideProjectManager* _sender, ValideProject* project, gpointer self) {
#line 3002 "window.c"
	valide_window_update_title (self);
}


#line 902 "window.vala"
static void _valide_window_update_title_valide_project_manager_project_closed (ValideProjectManager* _sender, gpointer self) {
#line 3009 "window.c"
	valide_window_update_title (self);
}


static void _lambda59_ (ValideWindow* self) {
#line 1004 "window.vala"
	valide_window_hide_start_page (self);
#line 3017 "window.c"
}


static void __lambda59__valide_idocument_closed (ValideiDocument* _sender, gpointer self) {
	_lambda59_ (self);
}


static void _lambda60_ (ValideStartPage* s, const char* m, ValideWindow* self) {
	g_return_if_fail (s != NULL);
	g_return_if_fail (m != NULL);
#line 1007 "window.vala"
	valide_statusbar_flash_message (self->priv->_statusbar, self->priv->status_context, m);
#line 3031 "window.c"
}


static void __lambda60__valide_start_page_send_message (ValideStartPage* _sender, const char* message, gpointer self) {
	_lambda60_ (_sender, message, self);
}


#line 479 "window.vala"
static void _valide_window_on_action_activated_gtk_ui_manager_post_activate (GtkUIManager* _sender, GtkAction* action, gpointer self) {
#line 3042 "window.c"
	valide_window_on_action_activated (self, _sender, action);
}


static void _g_list_free_g_object_unref (GList* self) {
	g_list_foreach (self, (GFunc) g_object_unref, NULL);
	g_list_free (self);
}


static GObject * valide_window_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	ValideWindowClass * klass;
	GObjectClass * parent_class;
	ValideWindow * self;
	GError * _inner_error_;
	klass = VALIDE_WINDOW_CLASS (g_type_class_peek (VALIDE_TYPE_WINDOW));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VALIDE_WINDOW (obj);
	_inner_error_ = NULL;
	{
		GtkWindowGroup* _tmp4_;
		ValideConfigManager* _tmp5_;
		ValideDocumentManager* _tmp6_;
		ValideExecutableManager* _tmp7_;
		ValideBuilderManager* _tmp8_;
		ValideProjectManager* _tmp9_;
		ValideRecentManagerUI* _tmp10_;
		ValideStatusbar* _tmp11_;
		ValideStartPage* _tmp12_;
		ValidePluginManager* _tmp25_;
		_tmp4_ = NULL;
#line 942 "window.vala"
		self->priv->owned_group = (_tmp4_ = gtk_window_group_new (), (self->priv->owned_group == NULL) ? NULL : (self->priv->owned_group = (g_object_unref (self->priv->owned_group), NULL)), _tmp4_);
#line 943 "window.vala"
		((GtkWindow*) self)->group = self->priv->owned_group;
#line 944 "window.vala"
		gtk_window_group_add_window (((GtkWindow*) self)->group, (GtkWindow*) self);
#line 946 "window.vala"
		g_signal_connect_object ((GtkWidget*) self, "delete-event", (GCallback) _valide_window_on_delete_event_gtk_widget_delete_event, self, 0);
#line 948 "window.vala"
		g_signal_connect ((GtkWidget*) self, "window-state-event", (GCallback) __lambda54__gtk_widget_window_state_event, self);
#line 963 "window.vala"
		valide_window_setup_ui_manager (self);
#line 88 "config-manager.vala"
		_tmp5_ = NULL;
#line 965 "window.vala"
		valide_window_set_config_manager (self, _tmp5_ = valide_config_manager_get_instance ());
#line 3092 "window.c"
		(_tmp5_ == NULL) ? NULL : (_tmp5_ = (g_object_unref (_tmp5_), NULL));
		_tmp6_ = NULL;
#line 967 "window.vala"
		valide_window_set_documents (self, _tmp6_ = g_object_ref_sink (valide_document_manager_new (self->priv->_ui_manager)));
#line 3097 "window.c"
		(_tmp6_ == NULL) ? NULL : (_tmp6_ = (g_object_unref (_tmp6_), NULL));
#line 968 "window.vala"
		g_signal_connect_object (self->priv->_documents, "tab-changed", (GCallback) _valide_window_update_title_valide_document_manager_tab_changed, self, 0);
#line 969 "window.vala"
		g_signal_connect_object (self->priv->_documents, "tab-state-changed", (GCallback) _valide_window_update_title_valide_document_manager_tab_state_changed, self, 0);
#line 970 "window.vala"
		g_signal_connect_object (self->priv->_documents, "tab-added", (GCallback) _valide_window_update_title_valide_document_manager_tab_added, self, 0);
#line 971 "window.vala"
		g_signal_connect_object (self->priv->_documents, "tab-removed", (GCallback) _valide_window_update_title_valide_document_manager_tab_removed, self, 0);
#line 972 "window.vala"
		g_signal_connect (self->priv->_documents, "tab-cursor-moved", (GCallback) __lambda55__valide_document_manager_tab_cursor_moved, self);
#line 3109 "window.c"
		_tmp7_ = NULL;
#line 976 "window.vala"
		valide_window_set_executables (self, _tmp7_ = g_object_ref_sink (valide_executable_manager_new (self->priv->_ui_manager)));
#line 3113 "window.c"
		(_tmp7_ == NULL) ? NULL : (_tmp7_ = (g_object_unref (_tmp7_), NULL));
#line 977 "window.vala"
		g_signal_connect (self->priv->_executables, "run-exec", (GCallback) __lambda56__valide_executable_manager_run_exec, self);
#line 62 "builder-manager.vala"
		_tmp8_ = NULL;
#line 981 "window.vala"
		valide_window_set_builders (self, _tmp8_ = valide_builder_manager_get_instance ());
#line 3121 "window.c"
		(_tmp8_ == NULL) ? NULL : (_tmp8_ = (g_object_unref (_tmp8_), NULL));
#line 982 "window.vala"
		valide_builder_manager_set_executables (self->priv->_builders, self->priv->_executables);
#line 3125 "window.c"
		_tmp9_ = NULL;
#line 984 "window.vala"
		valide_window_set_projects (self, _tmp9_ = g_object_ref_sink (valide_project_manager_new (self->priv->_ui_manager, self->priv->_documents, self->priv->_builders)));
#line 3129 "window.c"
		(_tmp9_ == NULL) ? NULL : (_tmp9_ = (g_object_unref (_tmp9_), NULL));
#line 986 "window.vala"
		g_signal_connect (self->priv->_projects, "project-opened", (GCallback) __lambda57__valide_project_manager_project_opened, self);
#line 989 "window.vala"
		g_signal_connect (self->priv->_projects, "project-options-changed", (GCallback) __lambda58__valide_project_manager_project_options_changed, self);
#line 992 "window.vala"
		g_signal_connect_object (self->priv->_projects, "project-opened", (GCallback) _valide_window_update_title_valide_project_manager_project_opened, self, 0);
#line 993 "window.vala"
		g_signal_connect_object (self->priv->_projects, "project-closed", (GCallback) _valide_window_update_title_valide_project_manager_project_closed, self, 0);
#line 3139 "window.c"
		_tmp10_ = NULL;
#line 995 "window.vala"
		valide_window_set_recent_manager (self, _tmp10_ = valide_recent_manager_ui_new (self->priv->_ui_manager, self->priv->_documents, self->priv->_projects));
#line 3143 "window.c"
		(_tmp10_ == NULL) ? NULL : (_tmp10_ = (g_object_unref (_tmp10_), NULL));
		_tmp11_ = NULL;
#line 998 "window.vala"
		valide_window_set_statusbar (self, _tmp11_ = g_object_ref_sink (valide_statusbar_new ()));
#line 3148 "window.c"
		(_tmp11_ == NULL) ? NULL : (_tmp11_ = (g_object_unref (_tmp11_), NULL));
#line 999 "window.vala"
		self->priv->status_context = gtk_statusbar_get_context_id ((GtkStatusbar*) self->priv->_statusbar, "valide_window");
#line 3152 "window.c"
		_tmp12_ = NULL;
#line 1001 "window.vala"
		self->priv->start_page = (_tmp12_ = g_object_ref_sink (valide_start_page_new (self->priv->_documents, self->priv->_projects, self->priv->_recent_manager)), (self->priv->start_page == NULL) ? NULL : (self->priv->start_page = (g_object_unref (self->priv->start_page), NULL)), _tmp12_);
#line 1003 "window.vala"
		g_signal_connect ((ValideiDocument*) self->priv->start_page, "closed", (GCallback) __lambda59__valide_idocument_closed, self);
#line 1006 "window.vala"
		g_signal_connect (self->priv->start_page, "send-message", (GCallback) __lambda60__valide_start_page_send_message, self);
#line 1010 "window.vala"
		g_signal_connect_object ((GtkUIManager*) self->priv->_ui_manager, "post-activate", (GCallback) _valide_window_on_action_activated_gtk_ui_manager_post_activate, self, 0);
#line 1011 "window.vala"
		valide_window_populate_window (self);
#line 1012 "window.vala"
		valide_window_update_title (self);
#line 3166 "window.c"
		{
			GList* list;
			char* _tmp13_;
			GdkPixbuf* _tmp14_;
			GdkPixbuf* _tmp15_;
			char* _tmp16_;
			GdkPixbuf* _tmp17_;
			GdkPixbuf* _tmp18_;
			char* _tmp19_;
			GdkPixbuf* _tmp20_;
			GdkPixbuf* _tmp21_;
			char* _tmp22_;
			GdkPixbuf* _tmp23_;
			GdkPixbuf* _tmp24_;
			list = NULL;
#line 2034 "glib-2.0.vapi"
			_tmp13_ = NULL;
#line 1018 "window.vala"
			_tmp14_ = NULL;
#line 3186 "window.c"
			_tmp15_ = (_tmp14_ = gdk_pixbuf_new_from_file (_tmp13_ = g_build_filename (PIXMAPS_DIR, "icone-16.xpm", NULL), &_inner_error_), _tmp13_ = (g_free (_tmp13_), NULL), _tmp14_);
			if (_inner_error_ != NULL) {
				(list == NULL) ? NULL : (list = (_g_list_free_g_object_unref (list), NULL));
				goto __catch60_g_error;
				goto __finally60;
			}
#line 1018 "window.vala"
			list = g_list_append (list, _tmp15_);
#line 2034 "glib-2.0.vapi"
			_tmp16_ = NULL;
#line 1019 "window.vala"
			_tmp17_ = NULL;
#line 3199 "window.c"
			_tmp18_ = (_tmp17_ = gdk_pixbuf_new_from_file (_tmp16_ = g_build_filename (PIXMAPS_DIR, "icone-32.xpm", NULL), &_inner_error_), _tmp16_ = (g_free (_tmp16_), NULL), _tmp17_);
			if (_inner_error_ != NULL) {
				(list == NULL) ? NULL : (list = (_g_list_free_g_object_unref (list), NULL));
				goto __catch60_g_error;
				goto __finally60;
			}
#line 1019 "window.vala"
			list = g_list_append (list, _tmp18_);
#line 2034 "glib-2.0.vapi"
			_tmp19_ = NULL;
#line 1020 "window.vala"
			_tmp20_ = NULL;
#line 3212 "window.c"
			_tmp21_ = (_tmp20_ = gdk_pixbuf_new_from_file (_tmp19_ = g_build_filename (PIXMAPS_DIR, "icone-48.xpm", NULL), &_inner_error_), _tmp19_ = (g_free (_tmp19_), NULL), _tmp20_);
			if (_inner_error_ != NULL) {
				(list == NULL) ? NULL : (list = (_g_list_free_g_object_unref (list), NULL));
				goto __catch60_g_error;
				goto __finally60;
			}
#line 1020 "window.vala"
			list = g_list_append (list, _tmp21_);
#line 2034 "glib-2.0.vapi"
			_tmp22_ = NULL;
#line 1021 "window.vala"
			_tmp23_ = NULL;
#line 3225 "window.c"
			_tmp24_ = (_tmp23_ = gdk_pixbuf_new_from_file (_tmp22_ = g_build_filename (PIXMAPS_DIR, "icone-64.xpm", NULL), &_inner_error_), _tmp22_ = (g_free (_tmp22_), NULL), _tmp23_);
			if (_inner_error_ != NULL) {
				(list == NULL) ? NULL : (list = (_g_list_free_g_object_unref (list), NULL));
				goto __catch60_g_error;
				goto __finally60;
			}
#line 1021 "window.vala"
			list = g_list_append (list, _tmp24_);
#line 1022 "window.vala"
			gtk_window_set_default_icon_list (list);
#line 3236 "window.c"
			(list == NULL) ? NULL : (list = (_g_list_free_g_object_unref (list), NULL));
		}
		goto __finally60;
		__catch60_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
#line 1026 "window.vala"
				g_debug (e->message);
#line 3248 "window.c"
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally60:
		if (_inner_error_ != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
		}
		_tmp25_ = NULL;
#line 1029 "window.vala"
		valide_window_set_plugins (self, _tmp25_ = valide_plugin_manager_new (self));
#line 3260 "window.c"
		(_tmp25_ == NULL) ? NULL : (_tmp25_ = (g_object_unref (_tmp25_), NULL));
	}
	return obj;
}


static void valide_window_class_init (ValideWindowClass * klass) {
	valide_window_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideWindowPrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_window_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_window_set_property;
	G_OBJECT_CLASS (klass)->constructor = valide_window_constructor;
	G_OBJECT_CLASS (klass)->finalize = valide_window_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_WINDOW_UI_MANAGER, g_param_spec_object ("ui-manager", "ui-manager", "ui-manager", VALIDE_TYPE_UI_MANAGER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_WINDOW_RECENT_MANAGER, g_param_spec_object ("recent-manager", "recent-manager", "recent-manager", VALIDE_TYPE_RECENT_MANAGER_UI, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_WINDOW_CONFIG_MANAGER, g_param_spec_object ("config-manager", "config-manager", "config-manager", VALIDE_TYPE_CONFIG_MANAGER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_WINDOW_DOCUMENTS, g_param_spec_object ("documents", "documents", "documents", VALIDE_TYPE_DOCUMENT_MANAGER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_WINDOW_PROJECTS, g_param_spec_object ("projects", "projects", "projects", VALIDE_TYPE_PROJECT_MANAGER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_WINDOW_BUILDERS, g_param_spec_object ("builders", "builders", "builders", VALIDE_TYPE_BUILDER_MANAGER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_WINDOW_EXECUTABLES, g_param_spec_object ("executables", "executables", "executables", VALIDE_TYPE_EXECUTABLE_MANAGER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_WINDOW_PLUGINS, g_param_spec_object ("plugins", "plugins", "plugins", VALIDE_TYPE_PLUGIN_MANAGER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_WINDOW_MENU, g_param_spec_object ("menu", "menu", "menu", GTK_TYPE_MENU_BAR, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_WINDOW_TOOLBAR, g_param_spec_object ("toolbar", "toolbar", "toolbar", GTK_TYPE_TOOLBAR, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_WINDOW_SIDE_PANEL, g_param_spec_object ("side-panel", "side-panel", "side-panel", VALIDE_TYPE_PANEL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_WINDOW_BOTTOM_PANEL, g_param_spec_object ("bottom-panel", "bottom-panel", "bottom-panel", VALIDE_TYPE_PANEL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_WINDOW_STATUSBAR, g_param_spec_object ("statusbar", "statusbar", "statusbar", VALIDE_TYPE_STATUSBAR, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void valide_window_instance_init (ValideWindow * self) {
	self->priv = VALIDE_WINDOW_GET_PRIVATE (self);
}


static void valide_window_finalize (GObject* obj) {
	ValideWindow * self;
	self = VALIDE_WINDOW (obj);
	(self->priv->start_page == NULL) ? NULL : (self->priv->start_page = (g_object_unref (self->priv->start_page), NULL));
	(self->priv->owned_group == NULL) ? NULL : (self->priv->owned_group = (g_object_unref (self->priv->owned_group), NULL));
	(self->priv->fullscreen_toolbar == NULL) ? NULL : (self->priv->fullscreen_toolbar = (g_object_unref (self->priv->fullscreen_toolbar), NULL));
	(self->priv->_ui_manager == NULL) ? NULL : (self->priv->_ui_manager = (g_object_unref (self->priv->_ui_manager), NULL));
	(self->priv->_recent_manager == NULL) ? NULL : (self->priv->_recent_manager = (g_object_unref (self->priv->_recent_manager), NULL));
	(self->priv->_config_manager == NULL) ? NULL : (self->priv->_config_manager = (g_object_unref (self->priv->_config_manager), NULL));
	(self->priv->_documents == NULL) ? NULL : (self->priv->_documents = (g_object_unref (self->priv->_documents), NULL));
	(self->priv->_projects == NULL) ? NULL : (self->priv->_projects = (g_object_unref (self->priv->_projects), NULL));
	(self->priv->_builders == NULL) ? NULL : (self->priv->_builders = (g_object_unref (self->priv->_builders), NULL));
	(self->priv->_executables == NULL) ? NULL : (self->priv->_executables = (g_object_unref (self->priv->_executables), NULL));
	(self->priv->_plugins == NULL) ? NULL : (self->priv->_plugins = (g_object_unref (self->priv->_plugins), NULL));
	(self->priv->_menu == NULL) ? NULL : (self->priv->_menu = (g_object_unref (self->priv->_menu), NULL));
	(self->priv->_toolbar == NULL) ? NULL : (self->priv->_toolbar = (g_object_unref (self->priv->_toolbar), NULL));
	(self->priv->_side_panel == NULL) ? NULL : (self->priv->_side_panel = (g_object_unref (self->priv->_side_panel), NULL));
	(self->priv->_bottom_panel == NULL) ? NULL : (self->priv->_bottom_panel = (g_object_unref (self->priv->_bottom_panel), NULL));
	(self->priv->_statusbar == NULL) ? NULL : (self->priv->_statusbar = (g_object_unref (self->priv->_statusbar), NULL));
	G_OBJECT_CLASS (valide_window_parent_class)->finalize (obj);
}


GType valide_window_get_type (void) {
	static GType valide_window_type_id = 0;
	if (valide_window_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideWindowClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_window_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideWindow), 0, (GInstanceInitFunc) valide_window_instance_init, NULL };
		valide_window_type_id = g_type_register_static (GTK_TYPE_WINDOW, "ValideWindow", &g_define_type_info, 0);
	}
	return valide_window_type_id;
}


static void valide_window_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideWindow * self;
	gpointer boxed;
	self = VALIDE_WINDOW (object);
	switch (property_id) {
		case VALIDE_WINDOW_UI_MANAGER:
		g_value_set_object (value, valide_window_get_ui_manager (self));
		break;
		case VALIDE_WINDOW_RECENT_MANAGER:
		g_value_set_object (value, valide_window_get_recent_manager (self));
		break;
		case VALIDE_WINDOW_CONFIG_MANAGER:
		g_value_set_object (value, valide_window_get_config_manager (self));
		break;
		case VALIDE_WINDOW_DOCUMENTS:
		g_value_set_object (value, valide_window_get_documents (self));
		break;
		case VALIDE_WINDOW_PROJECTS:
		g_value_set_object (value, valide_window_get_projects (self));
		break;
		case VALIDE_WINDOW_BUILDERS:
		g_value_set_object (value, valide_window_get_builders (self));
		break;
		case VALIDE_WINDOW_EXECUTABLES:
		g_value_set_object (value, valide_window_get_executables (self));
		break;
		case VALIDE_WINDOW_PLUGINS:
		g_value_set_object (value, valide_window_get_plugins (self));
		break;
		case VALIDE_WINDOW_MENU:
		g_value_set_object (value, valide_window_get_menu (self));
		break;
		case VALIDE_WINDOW_TOOLBAR:
		g_value_set_object (value, valide_window_get_toolbar (self));
		break;
		case VALIDE_WINDOW_SIDE_PANEL:
		g_value_set_object (value, valide_window_get_side_panel (self));
		break;
		case VALIDE_WINDOW_BOTTOM_PANEL:
		g_value_set_object (value, valide_window_get_bottom_panel (self));
		break;
		case VALIDE_WINDOW_STATUSBAR:
		g_value_set_object (value, valide_window_get_statusbar (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_window_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideWindow * self;
	self = VALIDE_WINDOW (object);
	switch (property_id) {
		case VALIDE_WINDOW_UI_MANAGER:
		valide_window_set_ui_manager (self, g_value_get_object (value));
		break;
		case VALIDE_WINDOW_RECENT_MANAGER:
		valide_window_set_recent_manager (self, g_value_get_object (value));
		break;
		case VALIDE_WINDOW_CONFIG_MANAGER:
		valide_window_set_config_manager (self, g_value_get_object (value));
		break;
		case VALIDE_WINDOW_DOCUMENTS:
		valide_window_set_documents (self, g_value_get_object (value));
		break;
		case VALIDE_WINDOW_PROJECTS:
		valide_window_set_projects (self, g_value_get_object (value));
		break;
		case VALIDE_WINDOW_BUILDERS:
		valide_window_set_builders (self, g_value_get_object (value));
		break;
		case VALIDE_WINDOW_EXECUTABLES:
		valide_window_set_executables (self, g_value_get_object (value));
		break;
		case VALIDE_WINDOW_PLUGINS:
		valide_window_set_plugins (self, g_value_get_object (value));
		break;
		case VALIDE_WINDOW_MENU:
		valide_window_set_menu (self, g_value_get_object (value));
		break;
		case VALIDE_WINDOW_TOOLBAR:
		valide_window_set_toolbar (self, g_value_get_object (value));
		break;
		case VALIDE_WINDOW_SIDE_PANEL:
		valide_window_set_side_panel (self, g_value_get_object (value));
		break;
		case VALIDE_WINDOW_BOTTOM_PANEL:
		valide_window_set_bottom_panel (self, g_value_get_object (value));
		break;
		case VALIDE_WINDOW_STATUSBAR:
		valide_window_set_statusbar (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




