/* calltips.vala
 *
 * Copyright (C) 2008-2009 Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <stdlib.h>
#include <string.h>


#define TYPE_CALLTIPS (calltips_get_type ())
#define CALLTIPS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CALLTIPS, Calltips))
#define CALLTIPS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CALLTIPS, CalltipsClass))
#define IS_CALLTIPS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CALLTIPS))
#define IS_CALLTIPS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CALLTIPS))
#define CALLTIPS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CALLTIPS, CalltipsClass))

typedef struct _Calltips Calltips;
typedef struct _CalltipsClass CalltipsClass;
typedef struct _CalltipsPrivate CalltipsPrivate;

/**
 * The completion calltips
 */
struct _Calltips {
	GtkWindow parent_instance;
	CalltipsPrivate * priv;
};

struct _CalltipsClass {
	GtkWindowClass parent_class;
};

struct _CalltipsPrivate {
	GtkLabel* label;
};


static gpointer calltips_parent_class = NULL;

GType calltips_get_type (void);
#define CALLTIPS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_CALLTIPS, CalltipsPrivate))
enum  {
	CALLTIPS_DUMMY_PROPERTY,
	CALLTIPS_TEXT
};
static gboolean calltips_delete_event_cb (Calltips* self, GtkWidget* widget, GdkEvent* event);
Calltips* calltips_new (void);
Calltips* calltips_construct (GType object_type);
const char* calltips_get_text (Calltips* self);
void calltips_set_text (Calltips* self, const char* value);
static gboolean _calltips_delete_event_cb_gtk_widget_delete_event (GtkWidget* _sender, GdkEvent* event, gpointer self);
static GObject * calltips_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void calltips_finalize (GObject* obj);
static void calltips_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void calltips_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);



#line 29 "calltips.vala"
static gboolean calltips_delete_event_cb (Calltips* self, GtkWidget* widget, GdkEvent* event) {
#line 82 "calltips.c"
	gboolean result;
#line 29 "calltips.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 29 "calltips.vala"
	g_return_val_if_fail (widget != NULL, FALSE);
#line 29 "calltips.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 90 "calltips.c"
	/* Prevent the alt+F4 keys*/
	result = TRUE;
	return result;
}


/**
 * The completion calltips
 */
#line 25 "calltips.vala"
Calltips* calltips_construct (GType object_type) {
#line 102 "calltips.c"
	Calltips * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


#line 25 "calltips.vala"
Calltips* calltips_new (void) {
#line 25 "calltips.vala"
	return calltips_construct (TYPE_CALLTIPS);
#line 113 "calltips.c"
}


const char* calltips_get_text (Calltips* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = gtk_label_get_text (self->priv->label);
	return result;
}


void calltips_set_text (Calltips* self, const char* value) {
	g_return_if_fail (self != NULL);
#line 43 "calltips.vala"
	gtk_label_set_text (self->priv->label, value);
#line 129 "calltips.c"
	g_object_notify ((GObject *) self, "text");
}


#line 29 "calltips.vala"
static gboolean _calltips_delete_event_cb_gtk_widget_delete_event (GtkWidget* _sender, GdkEvent* event, gpointer self) {
#line 136 "calltips.c"
	return calltips_delete_event_cb (self, _sender, event);
}


static GObject * calltips_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	CalltipsClass * klass;
	GObjectClass * parent_class;
	Calltips * self;
	klass = CALLTIPS_CLASS (g_type_class_peek (TYPE_CALLTIPS));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = CALLTIPS (obj);
	{
		GdkColor color = {0};
		GtkLabel* _tmp0_;
#line 51 "calltips.vala"
		g_object_set ((GtkWindow*) self, "type", GTK_WINDOW_POPUP, NULL);
#line 52 "calltips.vala"
		gtk_window_set_type_hint ((GtkWindow*) self, GDK_WINDOW_TYPE_HINT_TOOLTIP);
#line 53 "calltips.vala"
		gtk_window_set_resizable ((GtkWindow*) self, FALSE);
#line 54 "calltips.vala"
		gtk_container_set_border_width ((GtkContainer*) self, (guint) 5);
#line 56 "calltips.vala"
		gdk_color_parse ("#fffbbb", &color);
#line 57 "calltips.vala"
		gtk_widget_modify_bg ((GtkWidget*) self, GTK_STATE_NORMAL, &color);
#line 165 "calltips.c"
		_tmp0_ = NULL;
#line 59 "calltips.vala"
		self->priv->label = (_tmp0_ = g_object_ref_sink ((GtkLabel*) gtk_label_new ("")), (self->priv->label == NULL) ? NULL : (self->priv->label = (g_object_unref (self->priv->label), NULL)), _tmp0_);
#line 60 "calltips.vala"
		g_object_set ((GtkWidget*) self->priv->label, "can-focus", FALSE, NULL);
#line 61 "calltips.vala"
		gtk_widget_show ((GtkWidget*) self->priv->label);
#line 62 "calltips.vala"
		gtk_container_add ((GtkContainer*) self, (GtkWidget*) self->priv->label);
#line 175 "calltips.c"
		/* Connect signals*/
#line 65 "calltips.vala"
		g_signal_connect_object ((GtkWidget*) self, "delete-event", (GCallback) _calltips_delete_event_cb_gtk_widget_delete_event, self, 0);
#line 179 "calltips.c"
	}
	return obj;
}


static void calltips_class_init (CalltipsClass * klass) {
	calltips_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (CalltipsPrivate));
	G_OBJECT_CLASS (klass)->get_property = calltips_get_property;
	G_OBJECT_CLASS (klass)->set_property = calltips_set_property;
	G_OBJECT_CLASS (klass)->constructor = calltips_constructor;
	G_OBJECT_CLASS (klass)->finalize = calltips_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), CALLTIPS_TEXT, g_param_spec_string ("text", "text", "text", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void calltips_instance_init (Calltips * self) {
	self->priv = CALLTIPS_GET_PRIVATE (self);
}


static void calltips_finalize (GObject* obj) {
	Calltips * self;
	self = CALLTIPS (obj);
	(self->priv->label == NULL) ? NULL : (self->priv->label = (g_object_unref (self->priv->label), NULL));
	G_OBJECT_CLASS (calltips_parent_class)->finalize (obj);
}


GType calltips_get_type (void) {
	static GType calltips_type_id = 0;
	if (calltips_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (CalltipsClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) calltips_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Calltips), 0, (GInstanceInitFunc) calltips_instance_init, NULL };
		calltips_type_id = g_type_register_static (GTK_TYPE_WINDOW, "Calltips", &g_define_type_info, 0);
	}
	return calltips_type_id;
}


static void calltips_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	Calltips * self;
	gpointer boxed;
	self = CALLTIPS (object);
	switch (property_id) {
		case CALLTIPS_TEXT:
		g_value_set_string (value, calltips_get_text (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void calltips_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	Calltips * self;
	self = CALLTIPS (object);
	switch (property_id) {
		case CALLTIPS_TEXT:
		calltips_set_text (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}




