/* popup.vala
 *
 * Copyright (C) 2008-2009 Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <stdlib.h>
#include <string.h>


#define TYPE_POPUP (popup_get_type ())
#define POPUP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_POPUP, Popup))
#define POPUP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_POPUP, PopupClass))
#define IS_POPUP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_POPUP))
#define IS_POPUP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_POPUP))
#define POPUP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_POPUP, PopupClass))

typedef struct _Popup Popup;
typedef struct _PopupClass PopupClass;
typedef struct _PopupPrivate PopupPrivate;

#define TYPE_TREE (tree_get_type ())
#define TREE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_TREE, Tree))
#define TREE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_TREE, TreeClass))
#define IS_TREE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_TREE))
#define IS_TREE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_TREE))
#define TREE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_TREE, TreeClass))

typedef struct _Tree Tree;
typedef struct _TreeClass TreeClass;

#define TYPE_PROPOSAL (proposal_get_type ())
#define PROPOSAL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PROPOSAL, Proposal))
#define PROPOSAL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PROPOSAL, ProposalClass))
#define IS_PROPOSAL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PROPOSAL))
#define IS_PROPOSAL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PROPOSAL))
#define PROPOSAL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PROPOSAL, ProposalClass))

typedef struct _Proposal Proposal;
typedef struct _ProposalClass ProposalClass;

/**
 * The completion popup
 */
struct _Popup {
	GtkWindow parent_instance;
	PopupPrivate * priv;
};

struct _PopupClass {
	GtkWindowClass parent_class;
};

struct _PopupPrivate {
	Tree* completion_tree;
};


static gpointer popup_parent_class = NULL;

GType popup_get_type (void);
GType tree_get_type (void);
#define POPUP_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_POPUP, PopupPrivate))
enum  {
	POPUP_DUMMY_PROPERTY,
	POPUP_PROPOSALS
};
#define POPUP_WIDTH 350
#define POPUP_HEIGHT 200
static gboolean popup_delete_event_cb (Popup* self, GtkWidget* widget, GdkEvent* event);
GType proposal_get_type (void);
static void popup_proposal_selected_cb (Popup* self, Tree* sender, Proposal* proposal);
Tree* tree_new (void);
Tree* tree_construct (GType object_type);
static void _popup_proposal_selected_cb_tree_proposal_selected (Tree* _sender, Proposal* proposal, gpointer self);
static gboolean _popup_delete_event_cb_gtk_widget_delete_event (GtkWidget* _sender, GdkEvent* event, gpointer self);
Popup* popup_new (void);
Popup* popup_construct (GType object_type);
gboolean popup_has_proposals (Popup* self);
void popup_show (Popup* self);
void popup_hide (Popup* self);
static void popup_real_realize (GtkWidget* base);
void tree_select_first (Tree* self);
void popup_select_first (Popup* self);
void tree_select_last (Tree* self);
void popup_select_last (Popup* self);
void tree_select_previous (Tree* self, gint rows);
void popup_select_previous (Popup* self, gint rows);
void tree_select_next (Tree* self, gint rows);
void popup_select_next (Popup* self, gint rows);
gboolean tree_get_selected_proposal (Tree* self, Proposal** proposal);
gboolean popup_get_selected_proposal (Popup* self, Proposal** proposal);
gint tree_get_num_proposals (Tree* self);
gint popup_get_num_proposals (Popup* self);
void tree_filter (Tree* self, const char* filter);
void popup_filter (Popup* self, const char* filter);
void tree_clear (Tree* self);
void popup_clear (Popup* self);
void tree_add_data (Tree* self, Proposal* data);
void popup_set_proposals (Popup* self, GList* value);
static void popup_finalize (GObject* obj);
static void popup_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);



#line 51 "popup.vala"
static gboolean popup_delete_event_cb (Popup* self, GtkWidget* widget, GdkEvent* event) {
#line 128 "popup.c"
	gboolean result;
#line 51 "popup.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 51 "popup.vala"
	g_return_val_if_fail (widget != NULL, FALSE);
#line 51 "popup.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 136 "popup.c"
	/* Prevent the alt+F4 keys*/
	result = TRUE;
	return result;
}


#line 57 "popup.vala"
static void popup_proposal_selected_cb (Popup* self, Tree* sender, Proposal* proposal) {
#line 57 "popup.vala"
	g_return_if_fail (self != NULL);
#line 57 "popup.vala"
	g_return_if_fail (sender != NULL);
#line 57 "popup.vala"
	g_return_if_fail (proposal != NULL);
#line 59 "popup.vala"
	g_signal_emit_by_name (self, "proposal-selected", proposal);
#line 153 "popup.c"
}


#line 57 "popup.vala"
static void _popup_proposal_selected_cb_tree_proposal_selected (Tree* _sender, Proposal* proposal, gpointer self) {
#line 159 "popup.c"
	popup_proposal_selected_cb (self, _sender, proposal);
}


#line 51 "popup.vala"
static gboolean _popup_delete_event_cb_gtk_widget_delete_event (GtkWidget* _sender, GdkEvent* event, gpointer self) {
#line 166 "popup.c"
	return popup_delete_event_cb (self, _sender, event);
}


/**
   * Create a new completion popup
   */
#line 65 "popup.vala"
Popup* popup_construct (GType object_type) {
#line 176 "popup.c"
	GParameter * __params;
	GParameter * __params_it;
	Popup * self;
	Tree* _tmp0_;
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
#line 67 "popup.vala"
	__params_it->name = "type";
#line 67 "popup.vala"
	g_value_init (&__params_it->value, GTK_TYPE_WINDOW_TYPE);
#line 67 "popup.vala"
	g_value_set_enum (&__params_it->value, GTK_WINDOW_POPUP);
#line 67 "popup.vala"
	__params_it++;
#line 191 "popup.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	_tmp0_ = NULL;
#line 69 "popup.vala"
	self->priv->completion_tree = (_tmp0_ = g_object_ref_sink (tree_new ()), (self->priv->completion_tree == NULL) ? NULL : (self->priv->completion_tree = (g_object_unref (self->priv->completion_tree), NULL)), _tmp0_);
#line 70 "popup.vala"
	g_object_set ((GtkWidget*) self->priv->completion_tree, "can-focus", FALSE, NULL);
#line 71 "popup.vala"
	g_signal_connect_object (self->priv->completion_tree, "proposal-selected", (GCallback) _popup_proposal_selected_cb_tree_proposal_selected, self, 0);
#line 72 "popup.vala"
	gtk_widget_show ((GtkWidget*) self->priv->completion_tree);
#line 73 "popup.vala"
	gtk_container_add ((GtkContainer*) self, (GtkWidget*) self->priv->completion_tree);
#line 75 "popup.vala"
	g_signal_connect_object ((GtkWidget*) self, "delete-event", (GCallback) _popup_delete_event_cb_gtk_widget_delete_event, self, 0);
#line 206 "popup.c"
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 65 "popup.vala"
Popup* popup_new (void) {
#line 65 "popup.vala"
	return popup_construct (TYPE_POPUP);
#line 220 "popup.c"
}


/**
   * @see Gtk.Widget.show
   */
#line 81 "popup.vala"
void popup_show (Popup* self) {
#line 229 "popup.c"
	gboolean _tmp0_;
	gboolean _tmp1_;
#line 81 "popup.vala"
	g_return_if_fail (self != NULL);
#line 234 "popup.c"
	_tmp0_ = FALSE;
#line 83 "popup.vala"
	if (!(g_object_get ((GtkWidget*) self, "visible", &_tmp1_, NULL), _tmp1_)) {
#line 83 "popup.vala"
		_tmp0_ = popup_has_proposals (self);
#line 240 "popup.c"
	} else {
#line 83 "popup.vala"
		_tmp0_ = FALSE;
#line 244 "popup.c"
	}
#line 83 "popup.vala"
	if (_tmp0_) {
#line 85 "popup.vala"
		GTK_WIDGET_CLASS (popup_parent_class)->show ((GtkWidget*) GTK_WINDOW (self));
#line 250 "popup.c"
	}
}


/**
   * @]see Gtk.Widget.hide
   */
#line 92 "popup.vala"
void popup_hide (Popup* self) {
#line 92 "popup.vala"
	g_return_if_fail (self != NULL);
#line 94 "popup.vala"
	GTK_WIDGET_CLASS (popup_parent_class)->hide ((GtkWidget*) GTK_WINDOW (self));
#line 264 "popup.c"
}


/**
   * @see Gtk.Widget.realize
   */
#line 100 "popup.vala"
static void popup_real_realize (GtkWidget* base) {
#line 273 "popup.c"
	Popup * self;
	self = (Popup*) base;
#line 102 "popup.vala"
	gtk_container_set_border_width ((GtkContainer*) self, (guint) 1);
#line 103 "popup.vala"
	gtk_widget_set_size_request ((GtkWidget*) self, POPUP_WIDTH, POPUP_HEIGHT);
#line 104 "popup.vala"
	gtk_window_set_resizable ((GtkWindow*) self, TRUE);
#line 105 "popup.vala"
	GTK_WIDGET_CLASS (popup_parent_class)->realize ((GtkWidget*) GTK_WINDOW (self));
#line 284 "popup.c"
}


/**
   * @see Valide.Completion.Tree.select_first
   */
#line 111 "popup.vala"
void popup_select_first (Popup* self) {
#line 111 "popup.vala"
	g_return_if_fail (self != NULL);
#line 113 "popup.vala"
	tree_select_first (self->priv->completion_tree);
#line 297 "popup.c"
}


/**
   * @see Valide.Completion.Tree.select_last
   */
#line 119 "popup.vala"
void popup_select_last (Popup* self) {
#line 119 "popup.vala"
	g_return_if_fail (self != NULL);
#line 121 "popup.vala"
	tree_select_last (self->priv->completion_tree);
#line 310 "popup.c"
}


/**
   * @see Valide.Completion.Tree.select_previous
   */
#line 127 "popup.vala"
void popup_select_previous (Popup* self, gint rows) {
#line 127 "popup.vala"
	g_return_if_fail (self != NULL);
#line 129 "popup.vala"
	tree_select_previous (self->priv->completion_tree, rows);
#line 323 "popup.c"
}


/**
   * @see Valide.Completion.Tree.select_next
   */
#line 135 "popup.vala"
void popup_select_next (Popup* self, gint rows) {
#line 135 "popup.vala"
	g_return_if_fail (self != NULL);
#line 137 "popup.vala"
	tree_select_next (self->priv->completion_tree, rows);
#line 336 "popup.c"
}


/**
   * @see Valide.Completion.Tree.get_selected_proposal
   */
#line 143 "popup.vala"
gboolean popup_get_selected_proposal (Popup* self, Proposal** proposal) {
#line 345 "popup.c"
	gboolean result;
	Proposal* _tmp2_;
	gboolean _tmp1_;
	Proposal* _tmp0_;
#line 143 "popup.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 352 "popup.c"
	if (proposal != NULL) {
		*proposal = NULL;
	}
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	result = (_tmp1_ = tree_get_selected_proposal (self->priv->completion_tree, &_tmp0_), (*proposal) = (_tmp2_ = _tmp0_, ((*proposal) == NULL) ? NULL : ((*proposal) = (g_object_unref ((*proposal)), NULL)), _tmp2_), _tmp1_);
	return result;
}


/**
   * @see Valide.Completion.Tree.get_num_proposals
   */
#line 151 "popup.vala"
gint popup_get_num_proposals (Popup* self) {
#line 368 "popup.c"
	gint result;
#line 151 "popup.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 372 "popup.c"
	result = tree_get_num_proposals (self->priv->completion_tree);
	return result;
}


/**
   * Check if popup has proposal
   *
   * @return true if proposals are available
   */
#line 161 "popup.vala"
gboolean popup_has_proposals (Popup* self) {
#line 385 "popup.c"
	gboolean result;
#line 161 "popup.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 389 "popup.c"
	result = popup_get_num_proposals (self) > 0;
	return result;
}


/**
   * @see Valide.Completion.Tree.filter
   */
#line 169 "popup.vala"
void popup_filter (Popup* self, const char* filter) {
#line 169 "popup.vala"
	g_return_if_fail (self != NULL);
#line 169 "popup.vala"
	g_return_if_fail (filter != NULL);
#line 171 "popup.vala"
	tree_filter (self->priv->completion_tree, filter);
#line 406 "popup.c"
}


/**
   * Clear the tree view
   */
#line 177 "popup.vala"
void popup_clear (Popup* self) {
#line 177 "popup.vala"
	g_return_if_fail (self != NULL);
#line 179 "popup.vala"
	tree_clear (self->priv->completion_tree);
#line 419 "popup.c"
}


void popup_set_proposals (Popup* self, GList* value) {
	g_return_if_fail (self != NULL);
#line 43 "popup.vala"
	tree_clear (self->priv->completion_tree);
#line 427 "popup.c"
	{
		GList* proposal_collection;
		GList* proposal_it;
#line 44 "popup.vala"
		proposal_collection = value;
#line 433 "popup.c"
		for (proposal_it = proposal_collection; proposal_it != NULL; proposal_it = proposal_it->next) {
			Proposal* _tmp0_;
			Proposal* proposal;
#line 39 "popup.vala"
			_tmp0_ = NULL;
#line 44 "popup.vala"
			proposal = (_tmp0_ = (Proposal*) proposal_it->data, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
#line 441 "popup.c"
			{
#line 46 "popup.vala"
				tree_add_data (self->priv->completion_tree, proposal);
#line 445 "popup.c"
				(proposal == NULL) ? NULL : (proposal = (g_object_unref (proposal), NULL));
			}
		}
	}
	g_object_notify ((GObject *) self, "proposals");
}


static void popup_class_init (PopupClass * klass) {
	popup_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (PopupPrivate));
	GTK_WIDGET_CLASS (klass)->realize = popup_real_realize;
	G_OBJECT_CLASS (klass)->set_property = popup_set_property;
	G_OBJECT_CLASS (klass)->finalize = popup_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), POPUP_PROPOSALS, g_param_spec_pointer ("proposals", "proposals", "proposals", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE));
	g_signal_new ("proposal_selected", TYPE_POPUP, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, TYPE_PROPOSAL);
}


static void popup_instance_init (Popup * self) {
	self->priv = POPUP_GET_PRIVATE (self);
}


static void popup_finalize (GObject* obj) {
	Popup * self;
	self = POPUP (obj);
	(self->priv->completion_tree == NULL) ? NULL : (self->priv->completion_tree = (g_object_unref (self->priv->completion_tree), NULL));
	G_OBJECT_CLASS (popup_parent_class)->finalize (obj);
}


GType popup_get_type (void) {
	static GType popup_type_id = 0;
	if (popup_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (PopupClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) popup_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Popup), 0, (GInstanceInitFunc) popup_instance_init, NULL };
		popup_type_id = g_type_register_static (GTK_TYPE_WINDOW, "Popup", &g_define_type_info, 0);
	}
	return popup_type_id;
}


static void popup_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	Popup * self;
	self = POPUP (object);
	switch (property_id) {
		case POPUP_PROPOSALS:
		popup_set_proposals (self, g_value_get_pointer (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}




