/* proposal.vala
 *
 * Copyright (C) 2008-2009 Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <stdlib.h>
#include <string.h>
#include <valide-0.0.h>
#include <gtk/gtk.h>


#define TYPE_PROPOSAL (proposal_get_type ())
#define PROPOSAL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PROPOSAL, Proposal))
#define PROPOSAL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PROPOSAL, ProposalClass))
#define IS_PROPOSAL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PROPOSAL))
#define IS_PROPOSAL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PROPOSAL))
#define PROPOSAL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PROPOSAL, ProposalClass))

typedef struct _Proposal Proposal;
typedef struct _ProposalClass ProposalClass;
typedef struct _ProposalPrivate ProposalPrivate;

/**
 * A completion proposal
 */
struct _Proposal {
	GObject parent_instance;
	ProposalPrivate * priv;
};

struct _ProposalClass {
	GObjectClass parent_class;
};

struct _ProposalPrivate {
	GdkPixbuf* _icon;
	char* _label;
};


static gpointer proposal_parent_class = NULL;

GType proposal_get_type (void);
#define PROPOSAL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_PROPOSAL, ProposalPrivate))
enum  {
	PROPOSAL_DUMMY_PROPERTY,
	PROPOSAL_LABEL,
	PROPOSAL_ICON
};
void proposal_set_label (Proposal* self, const char* value);
void proposal_set_icon (Proposal* self, GdkPixbuf* value);
Proposal* proposal_new (const char* label, GdkPixbuf* icon);
Proposal* proposal_construct (GType object_type, const char* label, GdkPixbuf* icon);
const char* proposal_get_label (Proposal* self);
GdkPixbuf* proposal_get_icon (Proposal* self);
static void proposal_finalize (GObject* obj);
static void proposal_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void proposal_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);



/**
   * Create a new proposal
   *
   * @param label A name
   * @param icon A icon
   */
#line 69 "proposal.vala"
Proposal* proposal_construct (GType object_type, const char* label, GdkPixbuf* icon) {
#line 90 "proposal.c"
	GParameter * __params;
	GParameter * __params_it;
	Proposal * self;
#line 69 "proposal.vala"
	g_return_val_if_fail (label != NULL, NULL);
#line 96 "proposal.c"
	__params = g_new0 (GParameter, 2);
	__params_it = __params;
#line 71 "proposal.vala"
	__params_it->name = "label";
#line 71 "proposal.vala"
	g_value_init (&__params_it->value, G_TYPE_STRING);
#line 71 "proposal.vala"
	g_value_set_string (&__params_it->value, label);
#line 71 "proposal.vala"
	__params_it++;
#line 72 "proposal.vala"
	__params_it->name = "icon";
#line 72 "proposal.vala"
	g_value_init (&__params_it->value, GDK_TYPE_PIXBUF);
#line 72 "proposal.vala"
	g_value_set_object (&__params_it->value, icon);
#line 72 "proposal.vala"
	__params_it++;
#line 115 "proposal.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 69 "proposal.vala"
Proposal* proposal_new (const char* label, GdkPixbuf* icon) {
#line 69 "proposal.vala"
	return proposal_construct (TYPE_PROPOSAL, label, icon);
#line 130 "proposal.c"
}


const char* proposal_get_label (Proposal* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_label;
	return result;
}


void proposal_set_label (Proposal* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 37 "proposal.vala"
	_tmp0_ = NULL;
#line 149 "proposal.c"
	self->priv->_label = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_label = (g_free (self->priv->_label), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "label");
}


GdkPixbuf* proposal_get_icon (Proposal* self) {
	GdkPixbuf* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_icon;
	return result;
}


void proposal_set_icon (Proposal* self, GdkPixbuf* value) {
	g_return_if_fail (self != NULL);
#line 52 "proposal.vala"
	if (value == NULL) {
#line 167 "proposal.c"
		GdkPixbuf* _tmp0_;
		_tmp0_ = NULL;
#line 54 "proposal.vala"
		self->priv->_icon = (_tmp0_ = valide_utils_get_pixbuf_for_stock (GTK_STOCK_FILE, GTK_ICON_SIZE_SMALL_TOOLBAR), (self->priv->_icon == NULL) ? NULL : (self->priv->_icon = (g_object_unref (self->priv->_icon), NULL)), _tmp0_);
#line 172 "proposal.c"
	} else {
		GdkPixbuf* _tmp2_;
		GdkPixbuf* _tmp1_;
		_tmp2_ = NULL;
#line 58 "proposal.vala"
		_tmp1_ = NULL;
#line 58 "proposal.vala"
		self->priv->_icon = (_tmp2_ = (_tmp1_ = value, (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (self->priv->_icon == NULL) ? NULL : (self->priv->_icon = (g_object_unref (self->priv->_icon), NULL)), _tmp2_);
#line 181 "proposal.c"
	}
	g_object_notify ((GObject *) self, "icon");
}


static void proposal_class_init (ProposalClass * klass) {
	proposal_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ProposalPrivate));
	G_OBJECT_CLASS (klass)->get_property = proposal_get_property;
	G_OBJECT_CLASS (klass)->set_property = proposal_set_property;
	G_OBJECT_CLASS (klass)->finalize = proposal_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), PROPOSAL_LABEL, g_param_spec_string ("label", "label", "label", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
	g_object_class_install_property (G_OBJECT_CLASS (klass), PROPOSAL_ICON, g_param_spec_object ("icon", "icon", "icon", GDK_TYPE_PIXBUF, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
}


static void proposal_instance_init (Proposal * self) {
	self->priv = PROPOSAL_GET_PRIVATE (self);
}


static void proposal_finalize (GObject* obj) {
	Proposal * self;
	self = PROPOSAL (obj);
	(self->priv->_icon == NULL) ? NULL : (self->priv->_icon = (g_object_unref (self->priv->_icon), NULL));
	self->priv->_label = (g_free (self->priv->_label), NULL);
	G_OBJECT_CLASS (proposal_parent_class)->finalize (obj);
}


GType proposal_get_type (void) {
	static GType proposal_type_id = 0;
	if (proposal_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ProposalClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) proposal_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Proposal), 0, (GInstanceInitFunc) proposal_instance_init, NULL };
		proposal_type_id = g_type_register_static (G_TYPE_OBJECT, "Proposal", &g_define_type_info, 0);
	}
	return proposal_type_id;
}


static void proposal_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	Proposal * self;
	gpointer boxed;
	self = PROPOSAL (object);
	switch (property_id) {
		case PROPOSAL_LABEL:
		g_value_set_string (value, proposal_get_label (self));
		break;
		case PROPOSAL_ICON:
		g_value_set_object (value, proposal_get_icon (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void proposal_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	Proposal * self;
	self = PROPOSAL (object);
	switch (property_id) {
		case PROPOSAL_LABEL:
		proposal_set_label (self, g_value_get_string (value));
		break;
		case PROPOSAL_ICON:
		proposal_set_icon (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}




