/* tree.vala
 *
 * Copyright (C) 2008-2009 Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <float.h>
#include <math.h>


#define TYPE_TREE (tree_get_type ())
#define TREE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_TREE, Tree))
#define TREE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_TREE, TreeClass))
#define IS_TREE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_TREE))
#define IS_TREE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_TREE))
#define TREE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_TREE, TreeClass))

typedef struct _Tree Tree;
typedef struct _TreeClass TreeClass;
typedef struct _TreePrivate TreePrivate;

#define TREE_TYPE_COL (tree_col_get_type ())

#define TYPE_PROPOSAL (proposal_get_type ())
#define PROPOSAL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PROPOSAL, Proposal))
#define PROPOSAL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PROPOSAL, ProposalClass))
#define IS_PROPOSAL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PROPOSAL))
#define IS_PROPOSAL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PROPOSAL))
#define PROPOSAL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PROPOSAL, ProposalClass))

typedef struct _Proposal Proposal;
typedef struct _ProposalClass ProposalClass;

/**
 * The tree view widget for the popup completion
 */
struct _Tree {
	GtkScrolledWindow parent_instance;
	TreePrivate * priv;
};

struct _TreeClass {
	GtkScrolledWindowClass parent_class;
};

struct _TreePrivate {
	gboolean destroy_has_run;
	char* current_filter;
	gboolean active_filter;
	GtkTreeView* tree_view;
	GtkListStore* list_store;
	GtkTreeModelFilter* model_filter;
};

typedef enum  {
	TREE_COL_PIXBUF,
	TREE_COL_NAME,
	TREE_COL_DATA,
	TREE_COL_NB
} TreeCol;


static gpointer tree_parent_class = NULL;

GType tree_get_type (void);
#define TREE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_TREE, TreePrivate))
enum  {
	TREE_DUMMY_PROPERTY
};
static GType tree_col_get_type (void);
GType proposal_get_type (void);
static void tree_tree_row_activated_cb (Tree* self, GtkTreeView* tree_view, const GtkTreePath* path, GtkTreeViewColumn* column);
static gboolean tree_filter_by_name_func (Tree* self, GtkTreeModel* model, GtkTreeIter* iter);
gboolean tree_get_selected_proposal (Tree* self, Proposal** proposal);
GdkPixbuf* proposal_get_icon (Proposal* self);
const char* proposal_get_label (Proposal* self);
void tree_add_data (Tree* self, Proposal* data);
void tree_clear (Tree* self);
void tree_select_first (Tree* self);
void tree_select_last (Tree* self);
void tree_select_previous (Tree* self, gint rows);
void tree_select_next (Tree* self, gint rows);
gint tree_get_num_proposals (Tree* self);
void tree_filter (Tree* self, const char* filter);
Tree* tree_new (void);
Tree* tree_construct (GType object_type);
static gboolean _tree_filter_by_name_func_gtk_tree_model_filter_visible_func (GtkTreeModel* model, GtkTreeIter* iter, gpointer self);
static void _tree_tree_row_activated_cb_gtk_tree_view_row_activated (GtkTreeView* _sender, const GtkTreePath* path, GtkTreeViewColumn* column, gpointer self);
static GObject * tree_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void tree_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);




static GType tree_col_get_type (void) {
	static GType tree_col_type_id = 0;
	if (G_UNLIKELY (tree_col_type_id == 0)) {
		static const GEnumValue values[] = {{TREE_COL_PIXBUF, "TREE_COL_PIXBUF", "pixbuf"}, {TREE_COL_NAME, "TREE_COL_NAME", "name"}, {TREE_COL_DATA, "TREE_COL_DATA", "data"}, {TREE_COL_NB, "TREE_COL_NB", "nb"}, {0, NULL, NULL}};
		tree_col_type_id = g_enum_register_static ("TreeCol", values);
	}
	return tree_col_type_id;
}


#line 47 "tree.vala"
static void tree_tree_row_activated_cb (Tree* self, GtkTreeView* tree_view, const GtkTreePath* path, GtkTreeViewColumn* column) {
#line 130 "tree.c"
	GtkTreeIter iter = {0};
	GtkTreeModel* model;
	Proposal* proposal;
	GtkTreeModel* _tmp1_;
	GtkTreeModel* _tmp0_;
#line 47 "tree.vala"
	g_return_if_fail (self != NULL);
#line 47 "tree.vala"
	g_return_if_fail (tree_view != NULL);
#line 47 "tree.vala"
	g_return_if_fail (path != NULL);
#line 47 "tree.vala"
	g_return_if_fail (column != NULL);
#line 144 "tree.c"
	model = NULL;
	proposal = NULL;
	_tmp1_ = NULL;
#line 55 "tree.vala"
	_tmp0_ = NULL;
#line 55 "tree.vala"
	model = (_tmp1_ = (_tmp0_ = gtk_tree_view_get_model (tree_view), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (model == NULL) ? NULL : (model = (g_object_unref (model), NULL)), _tmp1_);
#line 57 "tree.vala"
	gtk_tree_model_get_iter (model, &iter, path);
#line 58 "tree.vala"
	gtk_tree_model_get (model, &iter, TREE_COL_DATA, &proposal, -1);
#line 60 "tree.vala"
	g_signal_emit_by_name (self, "proposal-selected", proposal);
#line 158 "tree.c"
	(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
	(proposal == NULL) ? NULL : (proposal = (g_object_unref (proposal), NULL));
}


#line 63 "tree.vala"
static gboolean tree_filter_by_name_func (Tree* self, GtkTreeModel* model, GtkTreeIter* iter) {
#line 166 "tree.c"
	gboolean result;
	glong len_cur;
	char* filter;
	char* current_name;
	char* _tmp1_;
	const char* _tmp0_;
	char* _tmp2_;
	gboolean _tmp3_;
#line 63 "tree.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 63 "tree.vala"
	g_return_val_if_fail (model != NULL, FALSE);
#line 179 "tree.c"
	len_cur = 0L;
	filter = NULL;
	current_name = NULL;
#line 69 "tree.vala"
	if (!self->priv->active_filter) {
#line 185 "tree.c"
		result = TRUE;
		filter = (g_free (filter), NULL);
		current_name = (g_free (current_name), NULL);
		return result;
	}
	_tmp1_ = NULL;
#line 74 "tree.vala"
	_tmp0_ = NULL;
#line 74 "tree.vala"
	filter = (_tmp1_ = (_tmp0_ = self->priv->current_filter, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), filter = (g_free (filter), NULL), _tmp1_);
#line 75 "tree.vala"
	if (filter == NULL) {
#line 198 "tree.c"
		result = TRUE;
		filter = (g_free (filter), NULL);
		current_name = (g_free (current_name), NULL);
		return result;
	}
#line 80 "tree.vala"
	gtk_tree_model_get (model, &(*iter), TREE_COL_NAME, &current_name, -1);
#line 81 "tree.vala"
	if (current_name == NULL) {
#line 208 "tree.c"
		result = TRUE;
		filter = (g_free (filter), NULL);
		current_name = (g_free (current_name), NULL);
		return result;
	}
#line 86 "tree.vala"
	len_cur = g_utf8_strlen (filter, -1);
#line 920 "glib-2.0.vapi"
	_tmp2_ = NULL;
#line 87 "tree.vala"
	if ((_tmp3_ = _vala_strcmp0 (filter, _tmp2_ = g_strndup (current_name, (gsize) len_cur)) == 0, _tmp2_ = (g_free (_tmp2_), NULL), _tmp3_)) {
#line 220 "tree.c"
		result = TRUE;
		filter = (g_free (filter), NULL);
		current_name = (g_free (current_name), NULL);
		return result;
	}
	result = FALSE;
	filter = (g_free (filter), NULL);
	current_name = (g_free (current_name), NULL);
	return result;
}


/**
   * Get the selected proposal
   *
   * @param proposal The select proposal
   *
   * @return true if a proposal is selected
   */
#line 153 "tree.vala"
gboolean tree_get_selected_proposal (Tree* self, Proposal** proposal) {
#line 242 "tree.c"
	gboolean result;
	GtkTreeIter iter = {0};
	GtkTreeModel* model;
	GtkTreeSelection* selection;
	GtkTreeSelection* _tmp1_;
	GtkTreeSelection* _tmp0_;
#line 153 "tree.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 251 "tree.c"
	if (proposal != NULL) {
		*proposal = NULL;
	}
	model = NULL;
	selection = NULL;
	_tmp1_ = NULL;
#line 159 "tree.vala"
	_tmp0_ = NULL;
#line 159 "tree.vala"
	selection = (_tmp1_ = (_tmp0_ = gtk_tree_view_get_selection (self->priv->tree_view), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL)), _tmp1_);
#line 160 "tree.vala"
	if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
#line 264 "tree.c"
		GtkTreeModel* _tmp3_;
		GtkTreeModel* _tmp2_;
		_tmp3_ = NULL;
#line 162 "tree.vala"
		_tmp2_ = NULL;
#line 162 "tree.vala"
		model = (_tmp3_ = (_tmp2_ = gtk_tree_view_get_model (self->priv->tree_view), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (model == NULL) ? NULL : (model = (g_object_unref (model), NULL)), _tmp3_);
#line 163 "tree.vala"
		gtk_tree_model_get (model, &iter, TREE_COL_DATA, &(*proposal), -1);
#line 274 "tree.c"
		result = TRUE;
		(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
		(selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL));
		return result;
	}
	result = FALSE;
	(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
	(selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL));
	return result;
}


/**
   * Add a new proposal to the tree
   *
   * @param data The proposal
   */
#line 174 "tree.vala"
void tree_add_data (Tree* self, Proposal* data) {
#line 294 "tree.c"
	GtkTreeIter iter = {0};
	GtkListStore* store;
	GtkListStore* _tmp1_;
	GtkListStore* _tmp0_;
#line 174 "tree.vala"
	g_return_if_fail (self != NULL);
#line 174 "tree.vala"
	g_return_if_fail (data != NULL);
#line 303 "tree.c"
	store = NULL;
	_tmp1_ = NULL;
#line 179 "tree.vala"
	_tmp0_ = NULL;
#line 179 "tree.vala"
	store = (_tmp1_ = (_tmp0_ = self->priv->list_store, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (store == NULL) ? NULL : (store = (g_object_unref (store), NULL)), _tmp1_);
#line 180 "tree.vala"
	gtk_list_store_append (store, &iter);
#line 181 "tree.vala"
	gtk_list_store_set (store, &iter, TREE_COL_PIXBUF, proposal_get_icon (data), TREE_COL_NAME, proposal_get_label (data), TREE_COL_DATA, data, -1);
#line 314 "tree.c"
	(store == NULL) ? NULL : (store = (g_object_unref (store), NULL));
}


/**
   * Clear the tree
   */
#line 189 "tree.vala"
void tree_clear (Tree* self) {
#line 189 "tree.vala"
	g_return_if_fail (self != NULL);
#line 191 "tree.vala"
	gtk_list_store_clear (self->priv->list_store);
#line 328 "tree.c"
}


/**
   * Select the first proposal
   */
#line 197 "tree.vala"
void tree_select_first (Tree* self) {
#line 337 "tree.c"
	gboolean _tmp0_;
#line 197 "tree.vala"
	g_return_if_fail (self != NULL);
#line 199 "tree.vala"
	if ((g_object_get ((GtkWidget*) self, "visible", &_tmp0_, NULL), _tmp0_)) {
#line 343 "tree.c"
		GtkTreeIter iter = {0};
		GtkTreePath* path;
		GtkTreeModel* model;
		GtkTreeSelection* selection;
		GtkTreeSelection* _tmp2_;
		GtkTreeSelection* _tmp1_;
		GtkTreeModel* _tmp4_;
		GtkTreeModel* _tmp3_;
		path = NULL;
		model = NULL;
		selection = NULL;
		_tmp2_ = NULL;
#line 206 "tree.vala"
		_tmp1_ = NULL;
#line 206 "tree.vala"
		selection = (_tmp2_ = (_tmp1_ = gtk_tree_view_get_selection (self->priv->tree_view), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL)), _tmp2_);
#line 360 "tree.c"
		_tmp4_ = NULL;
#line 208 "tree.vala"
		_tmp3_ = NULL;
#line 208 "tree.vala"
		model = (_tmp4_ = (_tmp3_ = gtk_tree_view_get_model (self->priv->tree_view), (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), (model == NULL) ? NULL : (model = (g_object_unref (model), NULL)), _tmp4_);
#line 209 "tree.vala"
		if (gtk_tree_model_get_iter_first (model, &iter)) {
#line 368 "tree.c"
			GtkTreePath* _tmp5_;
#line 211 "tree.vala"
			gtk_tree_selection_select_iter (selection, &iter);
#line 372 "tree.c"
			_tmp5_ = NULL;
#line 212 "tree.vala"
			path = (_tmp5_ = gtk_tree_model_get_path (model, &iter), (path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL)), _tmp5_);
#line 213 "tree.vala"
			gtk_tree_view_scroll_to_cell (self->priv->tree_view, path, NULL, FALSE, (float) 0, (float) 0);
#line 378 "tree.c"
		}
		(path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL));
		(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
		(selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL));
	}
}


/**
   * Select the last proposal
   */
#line 221 "tree.vala"
void tree_select_last (Tree* self) {
#line 392 "tree.c"
	gboolean _tmp0_;
#line 221 "tree.vala"
	g_return_if_fail (self != NULL);
#line 223 "tree.vala"
	if ((g_object_get ((GtkWidget*) self, "visible", &_tmp0_, NULL), _tmp0_)) {
#line 398 "tree.c"
		GtkTreeIter iter = {0};
		GtkTreeModel* model;
		GtkTreeSelection* selection;
		GtkTreePath* path;
		gint children;
		GtkTreeSelection* _tmp2_;
		GtkTreeSelection* _tmp1_;
		GtkTreeModel* _tmp4_;
		GtkTreeModel* _tmp3_;
		model = NULL;
		selection = NULL;
		path = NULL;
		children = 0;
		_tmp2_ = NULL;
#line 231 "tree.vala"
		_tmp1_ = NULL;
#line 231 "tree.vala"
		selection = (_tmp2_ = (_tmp1_ = gtk_tree_view_get_selection (self->priv->tree_view), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL)), _tmp2_);
#line 417 "tree.c"
		_tmp4_ = NULL;
#line 232 "tree.vala"
		_tmp3_ = NULL;
#line 232 "tree.vala"
		model = (_tmp4_ = (_tmp3_ = gtk_tree_view_get_model (self->priv->tree_view), (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), (model == NULL) ? NULL : (model = (g_object_unref (model), NULL)), _tmp4_);
#line 234 "tree.vala"
		children = gtk_tree_model_iter_n_children (model, NULL);
#line 235 "tree.vala"
		if (children > 0) {
#line 427 "tree.c"
			GtkTreePath* _tmp5_;
#line 237 "tree.vala"
			gtk_tree_model_iter_nth_child (model, &iter, NULL, children - 1);
#line 239 "tree.vala"
			gtk_tree_selection_select_iter (selection, &iter);
#line 433 "tree.c"
			_tmp5_ = NULL;
#line 240 "tree.vala"
			path = (_tmp5_ = gtk_tree_model_get_path (model, &iter), (path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL)), _tmp5_);
#line 241 "tree.vala"
			gtk_tree_view_scroll_to_cell (self->priv->tree_view, path, NULL, FALSE, (float) 0, (float) 0);
#line 439 "tree.c"
		}
		(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
		(selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL));
		(path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL));
	}
}


/**
   * Select the nth previous proposal
   *
   * @param rows Number of previous rows
   */
#line 251 "tree.vala"
void tree_select_previous (Tree* self, gint rows) {
#line 455 "tree.c"
	gboolean _tmp0_;
#line 251 "tree.vala"
	g_return_if_fail (self != NULL);
#line 253 "tree.vala"
	if ((g_object_get ((GtkWidget*) self, "visible", &_tmp0_, NULL), _tmp0_)) {
#line 461 "tree.c"
		GtkTreeIter iter = {0};
		GtkTreePath* path;
		GtkTreeModel* model;
		GtkTreeSelection* selection;
		GtkTreeSelection* _tmp2_;
		GtkTreeSelection* _tmp1_;
		GtkTreeModel* _tmp6_;
		GtkTreeModel* _tmp5_;
		gboolean _tmp4_;
		GtkTreeModel* _tmp3_;
		path = NULL;
		model = NULL;
		selection = NULL;
		_tmp2_ = NULL;
#line 260 "tree.vala"
		_tmp1_ = NULL;
#line 260 "tree.vala"
		selection = (_tmp2_ = (_tmp1_ = gtk_tree_view_get_selection (self->priv->tree_view), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL)), _tmp2_);
#line 262 "tree.vala"
		_tmp6_ = NULL;
#line 482 "tree.c"
		_tmp5_ = NULL;
		_tmp3_ = NULL;
#line 262 "tree.vala"
		if ((_tmp4_ = gtk_tree_selection_get_selected (selection, &_tmp3_, &iter), model = (_tmp5_ = (_tmp6_ = _tmp3_, (_tmp6_ == NULL) ? NULL : g_object_ref (_tmp6_)), (model == NULL) ? NULL : (model = (g_object_unref (model), NULL)), _tmp5_), _tmp4_)) {
#line 487 "tree.c"
			GtkTreePath* _tmp7_;
			_tmp7_ = NULL;
#line 264 "tree.vala"
			path = (_tmp7_ = gtk_tree_model_get_path (model, &iter), (path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL)), _tmp7_);
#line 492 "tree.c"
			{
				gint i;
				i = 0;
				{
					gboolean _tmp8_;
					_tmp8_ = TRUE;
#line 265 "tree.vala"
					while (TRUE) {
#line 265 "tree.vala"
						if (!_tmp8_) {
#line 265 "tree.vala"
							i++;
#line 505 "tree.c"
						}
#line 265 "tree.vala"
						_tmp8_ = FALSE;
#line 265 "tree.vala"
						if (!(i < rows)) {
#line 265 "tree.vala"
							break;
#line 513 "tree.c"
						}
#line 267 "tree.vala"
						gtk_tree_path_prev (path);
#line 517 "tree.c"
					}
				}
			}
#line 270 "tree.vala"
			if (gtk_tree_model_get_iter (model, &iter, path)) {
#line 272 "tree.vala"
				gtk_tree_selection_select_iter (selection, &iter);
#line 273 "tree.vala"
				gtk_tree_view_scroll_to_cell (self->priv->tree_view, path, NULL, FALSE, (float) 0, (float) 0);
#line 527 "tree.c"
			}
		} else {
#line 278 "tree.vala"
			tree_select_first (self);
#line 532 "tree.c"
		}
		(path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL));
		(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
		(selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL));
	}
}


/**
   * Select the nth next proposal
   *
   * @param rows Number of next rows
   */
#line 288 "tree.vala"
void tree_select_next (Tree* self, gint rows) {
#line 548 "tree.c"
	gboolean _tmp0_;
#line 288 "tree.vala"
	g_return_if_fail (self != NULL);
#line 290 "tree.vala"
	if ((g_object_get ((GtkWidget*) self, "visible", &_tmp0_, NULL), _tmp0_)) {
#line 554 "tree.c"
		GtkTreeIter iter = {0};
		GtkTreeModel* model;
		GtkTreeSelection* selection;
		GtkTreeSelection* _tmp2_;
		GtkTreeSelection* _tmp1_;
		GtkTreeModel* _tmp6_;
		GtkTreeModel* _tmp5_;
		gboolean _tmp4_;
		GtkTreeModel* _tmp3_;
		model = NULL;
		selection = NULL;
		_tmp2_ = NULL;
#line 296 "tree.vala"
		_tmp1_ = NULL;
#line 296 "tree.vala"
		selection = (_tmp2_ = (_tmp1_ = gtk_tree_view_get_selection (self->priv->tree_view), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL)), _tmp2_);
#line 298 "tree.vala"
		_tmp6_ = NULL;
#line 573 "tree.c"
		_tmp5_ = NULL;
		_tmp3_ = NULL;
#line 298 "tree.vala"
		if ((_tmp4_ = gtk_tree_selection_get_selected (selection, &_tmp3_, &iter), model = (_tmp5_ = (_tmp6_ = _tmp3_, (_tmp6_ == NULL) ? NULL : g_object_ref (_tmp6_)), (model == NULL) ? NULL : (model = (g_object_unref (model), NULL)), _tmp5_), _tmp4_)) {
#line 578 "tree.c"
			GtkTreePath* path;
			GtkTreePath* _tmp7_;
			GtkTreePath* _tmp9_;
			path = NULL;
			_tmp7_ = NULL;
#line 302 "tree.vala"
			path = (_tmp7_ = gtk_tree_model_get_path (model, &iter), (path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL)), _tmp7_);
#line 586 "tree.c"
			{
				gint i;
				i = 0;
				{
					gboolean _tmp8_;
					_tmp8_ = TRUE;
#line 303 "tree.vala"
					while (TRUE) {
#line 303 "tree.vala"
						if (!_tmp8_) {
#line 303 "tree.vala"
							i++;
#line 599 "tree.c"
						}
#line 303 "tree.vala"
						_tmp8_ = FALSE;
#line 303 "tree.vala"
						if (!(i < rows)) {
#line 303 "tree.vala"
							break;
#line 607 "tree.c"
						}
#line 305 "tree.vala"
						if (!gtk_tree_model_iter_next (model, &iter)) {
#line 307 "tree.vala"
							tree_select_last (self);
#line 613 "tree.c"
							(path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL));
							(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
							(selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL));
							return;
						}
					}
				}
			}
#line 312 "tree.vala"
			gtk_tree_selection_select_iter (selection, &iter);
#line 624 "tree.c"
			_tmp9_ = NULL;
#line 313 "tree.vala"
			path = (_tmp9_ = gtk_tree_model_get_path (model, &iter), (path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL)), _tmp9_);
#line 314 "tree.vala"
			gtk_tree_view_scroll_to_cell (self->priv->tree_view, path, NULL, FALSE, (float) 0, (float) 0);
#line 630 "tree.c"
			(path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL));
		} else {
#line 318 "tree.vala"
			tree_select_first (self);
#line 635 "tree.c"
		}
		(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
		(selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL));
	}
}


/**
   * Get the number of proposals
   */
#line 326 "tree.vala"
gint tree_get_num_proposals (Tree* self) {
#line 648 "tree.c"
	gint result;
	GtkTreeModel* model;
	GtkTreeModel* _tmp1_;
	GtkTreeModel* _tmp0_;
#line 326 "tree.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 655 "tree.c"
	model = NULL;
	_tmp1_ = NULL;
#line 330 "tree.vala"
	_tmp0_ = NULL;
#line 330 "tree.vala"
	model = (_tmp1_ = (_tmp0_ = gtk_tree_view_get_model (self->priv->tree_view), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (model == NULL) ? NULL : (model = (g_object_unref (model), NULL)), _tmp1_);
#line 662 "tree.c"
	result = gtk_tree_model_iter_n_children (model, NULL);
	(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
	return result;
}


/**
   * Filter proposals
   *
   * @param filter A filter string
   */
#line 339 "tree.vala"
void tree_filter (Tree* self, const char* filter) {
#line 676 "tree.c"
	char* _tmp1_;
	const char* _tmp0_;
	char* _tmp2_;
#line 339 "tree.vala"
	g_return_if_fail (self != NULL);
#line 339 "tree.vala"
	g_return_if_fail (filter != NULL);
#line 341 "tree.vala"
	self->priv->active_filter = TRUE;
#line 686 "tree.c"
	_tmp1_ = NULL;
#line 342 "tree.vala"
	_tmp0_ = NULL;
#line 342 "tree.vala"
	self->priv->current_filter = (_tmp1_ = (_tmp0_ = filter, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->current_filter = (g_free (self->priv->current_filter), NULL), _tmp1_);
#line 343 "tree.vala"
	gtk_tree_model_filter_refilter (self->priv->model_filter);
#line 344 "tree.vala"
	self->priv->active_filter = FALSE;
#line 696 "tree.c"
	_tmp2_ = NULL;
#line 345 "tree.vala"
	self->priv->current_filter = (_tmp2_ = NULL, self->priv->current_filter = (g_free (self->priv->current_filter), NULL), _tmp2_);
#line 700 "tree.c"
}


/**
 * The tree view widget for the popup completion
 */
#line 25 "tree.vala"
Tree* tree_construct (GType object_type) {
#line 709 "tree.c"
	Tree * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


#line 25 "tree.vala"
Tree* tree_new (void) {
#line 25 "tree.vala"
	return tree_construct (TYPE_TREE);
#line 720 "tree.c"
}


#line 63 "tree.vala"
static gboolean _tree_filter_by_name_func_gtk_tree_model_filter_visible_func (GtkTreeModel* model, GtkTreeIter* iter, gpointer self) {
#line 726 "tree.c"
	return tree_filter_by_name_func (self, model, iter);
}


#line 47 "tree.vala"
static void _tree_tree_row_activated_cb_gtk_tree_view_row_activated (GtkTreeView* _sender, const GtkTreePath* path, GtkTreeViewColumn* column, gpointer self) {
#line 733 "tree.c"
	tree_tree_row_activated_cb (self, _sender, path, column);
}


static GObject * tree_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	TreeClass * klass;
	GObjectClass * parent_class;
	Tree * self;
	klass = TREE_CLASS (g_type_class_peek (TYPE_TREE));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = TREE (obj);
	{
		GtkCellRendererText* renderer;
		GtkTreeViewColumn* column;
		GtkCellRendererPixbuf* renderer_pixbuf;
		GtkTreeViewColumn* column_pixbuf;
		GtkTreeSelection* selection;
		char* _tmp0_;
		GtkTreeView* _tmp1_;
		GtkCellRendererPixbuf* _tmp2_;
		GtkTreeViewColumn* _tmp3_;
		GtkTreeViewColumn* _tmp4_;
		GtkCellRendererText* _tmp5_;
		GtkListStore* _tmp6_;
		GtkTreeModelFilter* _tmp7_;
		GtkTreeSelection* _tmp9_;
		GtkTreeSelection* _tmp8_;
		renderer = NULL;
		column = NULL;
		renderer_pixbuf = NULL;
		column_pixbuf = NULL;
		selection = NULL;
#line 102 "tree.vala"
		self->priv->destroy_has_run = FALSE;
#line 770 "tree.c"
		_tmp0_ = NULL;
#line 103 "tree.vala"
		self->priv->current_filter = (_tmp0_ = NULL, self->priv->current_filter = (g_free (self->priv->current_filter), NULL), _tmp0_);
#line 104 "tree.vala"
		self->priv->active_filter = FALSE;
#line 105 "tree.vala"
		gtk_scrolled_window_set_policy ((GtkScrolledWindow*) self, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
#line 778 "tree.c"
		_tmp1_ = NULL;
#line 107 "tree.vala"
		self->priv->tree_view = (_tmp1_ = g_object_ref_sink ((GtkTreeView*) gtk_tree_view_new ()), (self->priv->tree_view == NULL) ? NULL : (self->priv->tree_view = (g_object_unref (self->priv->tree_view), NULL)), _tmp1_);
#line 108 "tree.vala"
		g_object_set ((GtkWidget*) self->priv->tree_view, "can-focus", FALSE, NULL);
#line 109 "tree.vala"
		gtk_tree_view_set_headers_visible (self->priv->tree_view, FALSE);
#line 110 "tree.vala"
		gtk_widget_show ((GtkWidget*) self->priv->tree_view);
#line 112 "tree.vala"
		gtk_container_add ((GtkContainer*) self, (GtkWidget*) self->priv->tree_view);
#line 790 "tree.c"
		/* Create the Tree*/
		_tmp2_ = NULL;
#line 115 "tree.vala"
		renderer_pixbuf = (_tmp2_ = g_object_ref_sink ((GtkCellRendererPixbuf*) gtk_cell_renderer_pixbuf_new ()), (renderer_pixbuf == NULL) ? NULL : (renderer_pixbuf = (g_object_unref (renderer_pixbuf), NULL)), _tmp2_);
#line 795 "tree.c"
		_tmp3_ = NULL;
#line 116 "tree.vala"
		column_pixbuf = (_tmp3_ = g_object_ref_sink (gtk_tree_view_column_new_with_attributes ("Pixbuf", (GtkCellRenderer*) renderer_pixbuf, "pixbuf", TREE_COL_PIXBUF, NULL)), (column_pixbuf == NULL) ? NULL : (column_pixbuf = (g_object_unref (column_pixbuf), NULL)), _tmp3_);
#line 799 "tree.c"
		_tmp4_ = NULL;
#line 121 "tree.vala"
		column = (_tmp4_ = g_object_ref_sink (gtk_tree_view_column_new ()), (column == NULL) ? NULL : (column = (g_object_unref (column), NULL)), _tmp4_);
#line 803 "tree.c"
		_tmp5_ = NULL;
#line 122 "tree.vala"
		renderer = (_tmp5_ = g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL)), _tmp5_);
#line 123 "tree.vala"
		gtk_cell_layout_pack_start ((GtkCellLayout*) column, (GtkCellRenderer*) renderer, FALSE);
#line 124 "tree.vala"
		gtk_tree_view_column_set_attributes (column, (GtkCellRenderer*) renderer, "text", TREE_COL_NAME, NULL);
#line 126 "tree.vala"
		gtk_tree_view_append_column (self->priv->tree_view, column_pixbuf);
#line 127 "tree.vala"
		gtk_tree_view_append_column (self->priv->tree_view, column);
#line 815 "tree.c"
		/* Create the model*/
		_tmp6_ = NULL;
#line 130 "tree.vala"
		self->priv->list_store = (_tmp6_ = gtk_list_store_new ((gint) TREE_COL_NB, GDK_TYPE_PIXBUF, G_TYPE_STRING, TYPE_PROPOSAL, NULL), (self->priv->list_store == NULL) ? NULL : (self->priv->list_store = (g_object_unref (self->priv->list_store), NULL)), _tmp6_);
#line 133 "tree.vala"
		gtk_tree_sortable_set_sort_column_id ((GtkTreeSortable*) self->priv->list_store, (gint) TREE_COL_NAME, GTK_SORT_ASCENDING);
#line 822 "tree.c"
		_tmp7_ = NULL;
#line 135 "tree.vala"
		self->priv->model_filter = (_tmp7_ = (GtkTreeModelFilter*) gtk_tree_model_filter_new ((GtkTreeModel*) self->priv->list_store, NULL), (self->priv->model_filter == NULL) ? NULL : (self->priv->model_filter = (g_object_unref (self->priv->model_filter), NULL)), _tmp7_);
#line 136 "tree.vala"
		gtk_tree_model_filter_set_visible_func (self->priv->model_filter, _tree_filter_by_name_func_gtk_tree_model_filter_visible_func, g_object_ref (self), g_object_unref);
#line 137 "tree.vala"
		gtk_tree_view_set_model (self->priv->tree_view, (GtkTreeModel*) self->priv->model_filter);
#line 830 "tree.c"
		/* Connect signals*/
#line 141 "tree.vala"
		g_signal_connect_object (self->priv->tree_view, "row-activated", (GCallback) _tree_tree_row_activated_cb_gtk_tree_view_row_activated, self, 0);
#line 834 "tree.c"
		_tmp9_ = NULL;
#line 143 "tree.vala"
		_tmp8_ = NULL;
#line 143 "tree.vala"
		selection = (_tmp9_ = (_tmp8_ = gtk_tree_view_get_selection (self->priv->tree_view), (_tmp8_ == NULL) ? NULL : g_object_ref (_tmp8_)), (selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL)), _tmp9_);
#line 840 "tree.c"
		(renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL));
		(column == NULL) ? NULL : (column = (g_object_unref (column), NULL));
		(renderer_pixbuf == NULL) ? NULL : (renderer_pixbuf = (g_object_unref (renderer_pixbuf), NULL));
		(column_pixbuf == NULL) ? NULL : (column_pixbuf = (g_object_unref (column_pixbuf), NULL));
		(selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL));
	}
	return obj;
}


static void tree_class_init (TreeClass * klass) {
	tree_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (TreePrivate));
	G_OBJECT_CLASS (klass)->constructor = tree_constructor;
	G_OBJECT_CLASS (klass)->finalize = tree_finalize;
	g_signal_new ("proposal_selected", TYPE_TREE, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, TYPE_PROPOSAL);
}


static void tree_instance_init (Tree * self) {
	self->priv = TREE_GET_PRIVATE (self);
}


static void tree_finalize (GObject* obj) {
	Tree * self;
	self = TREE (obj);
	self->priv->current_filter = (g_free (self->priv->current_filter), NULL);
	(self->priv->tree_view == NULL) ? NULL : (self->priv->tree_view = (g_object_unref (self->priv->tree_view), NULL));
	(self->priv->list_store == NULL) ? NULL : (self->priv->list_store = (g_object_unref (self->priv->list_store), NULL));
	(self->priv->model_filter == NULL) ? NULL : (self->priv->model_filter = (g_object_unref (self->priv->model_filter), NULL));
	G_OBJECT_CLASS (tree_parent_class)->finalize (obj);
}


GType tree_get_type (void) {
	static GType tree_type_id = 0;
	if (tree_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (TreeClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) tree_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Tree), 0, (GInstanceInitFunc) tree_instance_init, NULL };
		tree_type_id = g_type_register_static (GTK_TYPE_SCROLLED_WINDOW, "Tree", &g_define_type_info, 0);
	}
	return tree_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




