/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU Lesser General Public License
 * (version 2.1 or later).  See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>


#define VALENCIA_TYPE_TOKEN (valencia_token_get_type ())

#define VALENCIA_TYPE_KEYWORD (valencia_keyword_get_type ())
typedef struct _ValenciaKeyword ValenciaKeyword;

#define VALENCIA_TYPE_SCANNER (valencia_scanner_get_type ())
#define VALENCIA_SCANNER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALENCIA_TYPE_SCANNER, ValenciaScanner))
#define VALENCIA_SCANNER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALENCIA_TYPE_SCANNER, ValenciaScannerClass))
#define VALENCIA_IS_SCANNER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALENCIA_TYPE_SCANNER))
#define VALENCIA_IS_SCANNER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALENCIA_TYPE_SCANNER))
#define VALENCIA_SCANNER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALENCIA_TYPE_SCANNER, ValenciaScannerClass))

typedef struct _ValenciaScanner ValenciaScanner;
typedef struct _ValenciaScannerClass ValenciaScannerClass;
typedef struct _ValenciaScannerPrivate ValenciaScannerPrivate;

typedef enum  {
	VALENCIA_TOKEN_NONE,
	VALENCIA_TOKEN_EOF,
	VALENCIA_TOKEN_CHAR,
	VALENCIA_TOKEN_CHAR_LITERAL,
	VALENCIA_TOKEN_STRING_LITERAL,
	VALENCIA_TOKEN_ID,
	VALENCIA_TOKEN_ASTERISK,
	VALENCIA_TOKEN_LEFT_BRACE,
	VALENCIA_TOKEN_RIGHT_BRACE,
	VALENCIA_TOKEN_LEFT_BRACKET,
	VALENCIA_TOKEN_RIGHT_BRACKET,
	VALENCIA_TOKEN_COLON,
	VALENCIA_TOKEN_COMMA,
	VALENCIA_TOKEN_EQUALS,
	VALENCIA_TOKEN_ELLIPSIS,
	VALENCIA_TOKEN_HASH,
	VALENCIA_TOKEN_LEFT_PAREN,
	VALENCIA_TOKEN_RIGHT_PAREN,
	VALENCIA_TOKEN_PERIOD,
	VALENCIA_TOKEN_QUESTION_MARK,
	VALENCIA_TOKEN_SEMICOLON,
	VALENCIA_TOKEN_LESS_THAN,
	VALENCIA_TOKEN_GREATER_THAN,
	VALENCIA_TOKEN_ABSTRACT,
	VALENCIA_TOKEN_BASE,
	VALENCIA_TOKEN_CLASS,
	VALENCIA_TOKEN_CONST,
	VALENCIA_TOKEN_CONSTRUCT,
	VALENCIA_TOKEN_DELEGATE,
	VALENCIA_TOKEN_ELSE,
	VALENCIA_TOKEN_ENUM,
	VALENCIA_TOKEN_EXTERN,
	VALENCIA_TOKEN_FOR,
	VALENCIA_TOKEN_FOREACH,
	VALENCIA_TOKEN_IF,
	VALENCIA_TOKEN_INLINE,
	VALENCIA_TOKEN_INTERFACE,
	VALENCIA_TOKEN_INTERNAL,
	VALENCIA_TOKEN_NAMESPACE,
	VALENCIA_TOKEN_NEW,
	VALENCIA_TOKEN_OUT,
	VALENCIA_TOKEN_OVERRIDE,
	VALENCIA_TOKEN_OWNED,
	VALENCIA_TOKEN_PRIVATE,
	VALENCIA_TOKEN_PROTECTED,
	VALENCIA_TOKEN_PUBLIC,
	VALENCIA_TOKEN_REF,
	VALENCIA_TOKEN_RETURN,
	VALENCIA_TOKEN_SIGNAL,
	VALENCIA_TOKEN_STATIC,
	VALENCIA_TOKEN_STRUCT,
	VALENCIA_TOKEN_THIS,
	VALENCIA_TOKEN_UNOWNED,
	VALENCIA_TOKEN_USING,
	VALENCIA_TOKEN_VIRTUAL,
	VALENCIA_TOKEN_WEAK,
	VALENCIA_TOKEN_WHILE
} ValenciaToken;

/* an unrecognized punctuation character
 a literal such as 'x'
 punctuation
 keywords*/
struct _ValenciaKeyword {
	char* name;
	ValenciaToken token;
};

struct _ValenciaScanner {
	GObject parent_instance;
	ValenciaScannerPrivate * priv;
	gint start;
	gint end;
};

struct _ValenciaScannerClass {
	GObjectClass parent_class;
};

struct _ValenciaScannerPrivate {
	ValenciaToken token;
	const char* token_start_char;
	const char* input_begin;
	const char* input;
	gint token_start;
	gint input_pos;
	const char* start_char;
	const char* end_char;
};


static gpointer valencia_scanner_parent_class = NULL;

GType valencia_token_get_type (void);
GType valencia_keyword_get_type (void);
ValenciaKeyword* valencia_keyword_dup (const ValenciaKeyword* self);
void valencia_keyword_free (ValenciaKeyword* self);
void valencia_keyword_copy (const ValenciaKeyword* self, ValenciaKeyword* dest);
void valencia_keyword_destroy (ValenciaKeyword* self);
GType valencia_scanner_get_type (void);
#define VALENCIA_SCANNER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALENCIA_TYPE_SCANNER, ValenciaScannerPrivate))
enum  {
	VALENCIA_SCANNER_DUMMY_PROPERTY
};
ValenciaScanner* valencia_scanner_new (const char* input);
ValenciaScanner* valencia_scanner_construct (GType object_type, const char* input);
static void valencia_scanner_advance (ValenciaScanner* self);
static gunichar valencia_scanner_peek_char (ValenciaScanner* self);
static gunichar valencia_scanner_peek_char2 (ValenciaScanner* self);
static gunichar valencia_scanner_next_char (ValenciaScanner* self);
static gboolean valencia_scanner_accept (ValenciaScanner* self, gunichar c);
static gboolean valencia_scanner_match (ValenciaScanner* self, const char* s);
static void valencia_scanner_read_triple_string (ValenciaScanner* self);
static void valencia_scanner_skip_line (ValenciaScanner* self);
static gboolean valencia_scanner_is_first_token_on_line (ValenciaScanner* self);
static ValenciaToken valencia_scanner_read_token (ValenciaScanner* self);
ValenciaToken valencia_scanner_peek_token (ValenciaScanner* self);
ValenciaToken valencia_scanner_next_token (ValenciaScanner* self);
gboolean valencia_scanner_accept_token (ValenciaScanner* self, ValenciaToken t);
gboolean valencia_scanner_eof (ValenciaScanner* self);
char* valencia_scanner_val (ValenciaScanner* self);
const char* valencia_scanner_get_start (ValenciaScanner* self);
const char* valencia_scanner_get_start_after_comments (ValenciaScanner* self);
static void valencia_scanner_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);

static const ValenciaKeyword VALENCIA_keywords[] = {{"abstract", VALENCIA_TOKEN_ABSTRACT}, {"base", VALENCIA_TOKEN_BASE}, {"class", VALENCIA_TOKEN_CLASS}, {"const", VALENCIA_TOKEN_CONST}, {"construct", VALENCIA_TOKEN_CONSTRUCT}, {"delegate", VALENCIA_TOKEN_DELEGATE}, {"else", VALENCIA_TOKEN_ELSE}, {"enum", VALENCIA_TOKEN_ENUM}, {"extern", VALENCIA_TOKEN_EXTERN}, {"for", VALENCIA_TOKEN_FOR}, {"foreach", VALENCIA_TOKEN_FOREACH}, {"if", VALENCIA_TOKEN_IF}, {"inline", VALENCIA_TOKEN_INLINE}, {"interface", VALENCIA_TOKEN_INTERFACE}, {"internal", VALENCIA_TOKEN_INTERNAL}, {"namespace", VALENCIA_TOKEN_NAMESPACE}, {"new", VALENCIA_TOKEN_NEW}, {"out", VALENCIA_TOKEN_OUT}, {"override", VALENCIA_TOKEN_OVERRIDE}, {"owned", VALENCIA_TOKEN_OWNED}, {"private", VALENCIA_TOKEN_PRIVATE}, {"protected", VALENCIA_TOKEN_PROTECTED}, {"public", VALENCIA_TOKEN_PUBLIC}, {"ref", VALENCIA_TOKEN_REF}, {"return", VALENCIA_TOKEN_RETURN}, {"signal", VALENCIA_TOKEN_SIGNAL}, {"static", VALENCIA_TOKEN_STATIC}, {"struct", VALENCIA_TOKEN_STRUCT}, {"this", VALENCIA_TOKEN_THIS}, {"unowned", VALENCIA_TOKEN_UNOWNED}, {"using", VALENCIA_TOKEN_USING}, {"virtual", VALENCIA_TOKEN_VIRTUAL}, {"weak", VALENCIA_TOKEN_WEAK}, {"while", VALENCIA_TOKEN_WHILE}};



GType valencia_token_get_type (void) {
	static GType valencia_token_type_id = 0;
	if (G_UNLIKELY (valencia_token_type_id == 0)) {
		static const GEnumValue values[] = {{VALENCIA_TOKEN_NONE, "VALENCIA_TOKEN_NONE", "none"}, {VALENCIA_TOKEN_EOF, "VALENCIA_TOKEN_EOF", "eof"}, {VALENCIA_TOKEN_CHAR, "VALENCIA_TOKEN_CHAR", "char"}, {VALENCIA_TOKEN_CHAR_LITERAL, "VALENCIA_TOKEN_CHAR_LITERAL", "char-literal"}, {VALENCIA_TOKEN_STRING_LITERAL, "VALENCIA_TOKEN_STRING_LITERAL", "string-literal"}, {VALENCIA_TOKEN_ID, "VALENCIA_TOKEN_ID", "id"}, {VALENCIA_TOKEN_ASTERISK, "VALENCIA_TOKEN_ASTERISK", "asterisk"}, {VALENCIA_TOKEN_LEFT_BRACE, "VALENCIA_TOKEN_LEFT_BRACE", "left-brace"}, {VALENCIA_TOKEN_RIGHT_BRACE, "VALENCIA_TOKEN_RIGHT_BRACE", "right-brace"}, {VALENCIA_TOKEN_LEFT_BRACKET, "VALENCIA_TOKEN_LEFT_BRACKET", "left-bracket"}, {VALENCIA_TOKEN_RIGHT_BRACKET, "VALENCIA_TOKEN_RIGHT_BRACKET", "right-bracket"}, {VALENCIA_TOKEN_COLON, "VALENCIA_TOKEN_COLON", "colon"}, {VALENCIA_TOKEN_COMMA, "VALENCIA_TOKEN_COMMA", "comma"}, {VALENCIA_TOKEN_EQUALS, "VALENCIA_TOKEN_EQUALS", "equals"}, {VALENCIA_TOKEN_ELLIPSIS, "VALENCIA_TOKEN_ELLIPSIS", "ellipsis"}, {VALENCIA_TOKEN_HASH, "VALENCIA_TOKEN_HASH", "hash"}, {VALENCIA_TOKEN_LEFT_PAREN, "VALENCIA_TOKEN_LEFT_PAREN", "left-paren"}, {VALENCIA_TOKEN_RIGHT_PAREN, "VALENCIA_TOKEN_RIGHT_PAREN", "right-paren"}, {VALENCIA_TOKEN_PERIOD, "VALENCIA_TOKEN_PERIOD", "period"}, {VALENCIA_TOKEN_QUESTION_MARK, "VALENCIA_TOKEN_QUESTION_MARK", "question-mark"}, {VALENCIA_TOKEN_SEMICOLON, "VALENCIA_TOKEN_SEMICOLON", "semicolon"}, {VALENCIA_TOKEN_LESS_THAN, "VALENCIA_TOKEN_LESS_THAN", "less-than"}, {VALENCIA_TOKEN_GREATER_THAN, "VALENCIA_TOKEN_GREATER_THAN", "greater-than"}, {VALENCIA_TOKEN_ABSTRACT, "VALENCIA_TOKEN_ABSTRACT", "abstract"}, {VALENCIA_TOKEN_BASE, "VALENCIA_TOKEN_BASE", "base"}, {VALENCIA_TOKEN_CLASS, "VALENCIA_TOKEN_CLASS", "class"}, {VALENCIA_TOKEN_CONST, "VALENCIA_TOKEN_CONST", "const"}, {VALENCIA_TOKEN_CONSTRUCT, "VALENCIA_TOKEN_CONSTRUCT", "construct"}, {VALENCIA_TOKEN_DELEGATE, "VALENCIA_TOKEN_DELEGATE", "delegate"}, {VALENCIA_TOKEN_ELSE, "VALENCIA_TOKEN_ELSE", "else"}, {VALENCIA_TOKEN_ENUM, "VALENCIA_TOKEN_ENUM", "enum"}, {VALENCIA_TOKEN_EXTERN, "VALENCIA_TOKEN_EXTERN", "extern"}, {VALENCIA_TOKEN_FOR, "VALENCIA_TOKEN_FOR", "for"}, {VALENCIA_TOKEN_FOREACH, "VALENCIA_TOKEN_FOREACH", "foreach"}, {VALENCIA_TOKEN_IF, "VALENCIA_TOKEN_IF", "if"}, {VALENCIA_TOKEN_INLINE, "VALENCIA_TOKEN_INLINE", "inline"}, {VALENCIA_TOKEN_INTERFACE, "VALENCIA_TOKEN_INTERFACE", "interface"}, {VALENCIA_TOKEN_INTERNAL, "VALENCIA_TOKEN_INTERNAL", "internal"}, {VALENCIA_TOKEN_NAMESPACE, "VALENCIA_TOKEN_NAMESPACE", "namespace"}, {VALENCIA_TOKEN_NEW, "VALENCIA_TOKEN_NEW", "new"}, {VALENCIA_TOKEN_OUT, "VALENCIA_TOKEN_OUT", "out"}, {VALENCIA_TOKEN_OVERRIDE, "VALENCIA_TOKEN_OVERRIDE", "override"}, {VALENCIA_TOKEN_OWNED, "VALENCIA_TOKEN_OWNED", "owned"}, {VALENCIA_TOKEN_PRIVATE, "VALENCIA_TOKEN_PRIVATE", "private"}, {VALENCIA_TOKEN_PROTECTED, "VALENCIA_TOKEN_PROTECTED", "protected"}, {VALENCIA_TOKEN_PUBLIC, "VALENCIA_TOKEN_PUBLIC", "public"}, {VALENCIA_TOKEN_REF, "VALENCIA_TOKEN_REF", "ref"}, {VALENCIA_TOKEN_RETURN, "VALENCIA_TOKEN_RETURN", "return"}, {VALENCIA_TOKEN_SIGNAL, "VALENCIA_TOKEN_SIGNAL", "signal"}, {VALENCIA_TOKEN_STATIC, "VALENCIA_TOKEN_STATIC", "static"}, {VALENCIA_TOKEN_STRUCT, "VALENCIA_TOKEN_STRUCT", "struct"}, {VALENCIA_TOKEN_THIS, "VALENCIA_TOKEN_THIS", "this"}, {VALENCIA_TOKEN_UNOWNED, "VALENCIA_TOKEN_UNOWNED", "unowned"}, {VALENCIA_TOKEN_USING, "VALENCIA_TOKEN_USING", "using"}, {VALENCIA_TOKEN_VIRTUAL, "VALENCIA_TOKEN_VIRTUAL", "virtual"}, {VALENCIA_TOKEN_WEAK, "VALENCIA_TOKEN_WEAK", "weak"}, {VALENCIA_TOKEN_WHILE, "VALENCIA_TOKEN_WHILE", "while"}, {0, NULL, NULL}};
		valencia_token_type_id = g_enum_register_static ("ValenciaToken", values);
	}
	return valencia_token_type_id;
}


void valencia_keyword_copy (const ValenciaKeyword* self, ValenciaKeyword* dest) {
	const char* _tmp0_;
#line 28 "scanner.vala"
	_tmp0_ = NULL;
#line 175 "scanner.c"
	dest->name = (_tmp0_ = self->name, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
	dest->token = self->token;
}


void valencia_keyword_destroy (ValenciaKeyword* self) {
	self->name = (g_free (self->name), NULL);
}


ValenciaKeyword* valencia_keyword_dup (const ValenciaKeyword* self) {
	ValenciaKeyword* dup;
	dup = g_new0 (ValenciaKeyword, 1);
	valencia_keyword_copy (self, dup);
	return dup;
}


void valencia_keyword_free (ValenciaKeyword* self) {
	valencia_keyword_destroy (self);
	g_free (self);
}


GType valencia_keyword_get_type (void) {
	static GType valencia_keyword_type_id = 0;
	if (valencia_keyword_type_id == 0) {
		valencia_keyword_type_id = g_boxed_type_register_static ("ValenciaKeyword", (GBoxedCopyFunc) valencia_keyword_dup, (GBoxedFreeFunc) valencia_keyword_free);
	}
	return valencia_keyword_type_id;
}


/* ending character position*/
#line 88 "scanner.vala"
ValenciaScanner* valencia_scanner_construct (GType object_type, const char* input) {
#line 212 "scanner.c"
	ValenciaScanner * self;
#line 88 "scanner.vala"
	g_return_val_if_fail (input != NULL, NULL);
#line 216 "scanner.c"
	self = g_object_newv (object_type, 0, NULL);
#line 89 "scanner.vala"
	self->priv->input = input;
#line 90 "scanner.vala"
	self->priv->input_begin = input;
#line 222 "scanner.c"
	return self;
}


#line 88 "scanner.vala"
ValenciaScanner* valencia_scanner_new (const char* input) {
#line 88 "scanner.vala"
	return valencia_scanner_construct (VALENCIA_TYPE_SCANNER, input);
#line 231 "scanner.c"
}


#line 93 "scanner.vala"
static void valencia_scanner_advance (ValenciaScanner* self) {
#line 93 "scanner.vala"
	g_return_if_fail (self != NULL);
#line 94 "scanner.vala"
	self->priv->input = g_utf8_next_char (self->priv->input);
#line 95 "scanner.vala"
	self->priv->input_pos = self->priv->input_pos + 1;
#line 243 "scanner.c"
}


#line 98 "scanner.vala"
static gunichar valencia_scanner_peek_char (ValenciaScanner* self) {
#line 249 "scanner.c"
	gunichar result;
#line 98 "scanner.vala"
	g_return_val_if_fail (self != NULL, 0U);
#line 253 "scanner.c"
	result = g_utf8_get_char (self->priv->input);
	return result;
}


/* Peek two characters ahead.*/
#line 101 "scanner.vala"
static gunichar valencia_scanner_peek_char2 (ValenciaScanner* self) {
#line 262 "scanner.c"
	gunichar result;
	gunichar _tmp0_;
#line 101 "scanner.vala"
	g_return_val_if_fail (self != NULL, 0U);
#line 267 "scanner.c"
	_tmp0_ = 0U;
#line 102 "scanner.vala"
	if (_vala_strcmp0 (self->priv->input, "") == 0) {
#line 102 "scanner.vala"
		_tmp0_ = (gunichar) '\0';
#line 273 "scanner.c"
	} else {
#line 102 "scanner.vala"
		_tmp0_ = g_utf8_get_char (g_utf8_next_char (self->priv->input));
#line 277 "scanner.c"
	}
	result = _tmp0_;
	return result;
}


#line 105 "scanner.vala"
static gunichar valencia_scanner_next_char (ValenciaScanner* self) {
#line 286 "scanner.c"
	gunichar result;
	gunichar c;
#line 105 "scanner.vala"
	g_return_val_if_fail (self != NULL, 0U);
#line 291 "scanner.c"
	c = valencia_scanner_peek_char (self);
#line 107 "scanner.vala"
	valencia_scanner_advance (self);
#line 295 "scanner.c"
	result = c;
	return result;
}


#line 111 "scanner.vala"
static gboolean valencia_scanner_accept (ValenciaScanner* self, gunichar c) {
#line 303 "scanner.c"
	gboolean result;
#line 111 "scanner.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 112 "scanner.vala"
	if (valencia_scanner_peek_char (self) == c) {
#line 113 "scanner.vala"
		valencia_scanner_advance (self);
#line 311 "scanner.c"
		result = TRUE;
		return result;
	}
	result = FALSE;
	return result;
}


/* Return true if the current token equals s.    */
#line 120 "scanner.vala"
static gboolean valencia_scanner_match (ValenciaScanner* self, const char* s) {
#line 323 "scanner.c"
	gboolean result;
	gchar* p;
	gchar* q;
	gboolean _tmp2_;
#line 120 "scanner.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 120 "scanner.vala"
	g_return_val_if_fail (s != NULL, FALSE);
#line 332 "scanner.c"
	p = self->priv->token_start_char;
	q = s;
#line 123 "scanner.vala"
	while (TRUE) {
#line 337 "scanner.c"
		gboolean _tmp0_;
		gboolean _tmp1_;
		_tmp0_ = FALSE;
		_tmp1_ = FALSE;
#line 123 "scanner.vala"
		if ((*p) != 0) {
#line 123 "scanner.vala"
			_tmp1_ = (*q) != 0;
#line 346 "scanner.c"
		} else {
#line 123 "scanner.vala"
			_tmp1_ = FALSE;
#line 350 "scanner.c"
		}
#line 123 "scanner.vala"
		if (_tmp1_) {
#line 123 "scanner.vala"
			_tmp0_ = (*p) == (*q);
#line 356 "scanner.c"
		} else {
#line 123 "scanner.vala"
			_tmp0_ = FALSE;
#line 360 "scanner.c"
		}
#line 123 "scanner.vala"
		if (!_tmp0_) {
#line 123 "scanner.vala"
			break;
#line 366 "scanner.c"
		}
#line 124 "scanner.vala"
		p = p + 1;
#line 125 "scanner.vala"
		q = q + 1;
#line 372 "scanner.c"
	}
	_tmp2_ = FALSE;
#line 127 "scanner.vala"
	if (p == self->priv->input) {
#line 127 "scanner.vala"
		_tmp2_ = (*q) == 0;
#line 379 "scanner.c"
	} else {
#line 127 "scanner.vala"
		_tmp2_ = FALSE;
#line 383 "scanner.c"
	}
	result = _tmp2_;
	return result;
}


/* Read characters until we reach a triple quote (""") string terminator.*/
#line 131 "scanner.vala"
static void valencia_scanner_read_triple_string (ValenciaScanner* self) {
#line 131 "scanner.vala"
	g_return_if_fail (self != NULL);
#line 132 "scanner.vala"
	while (TRUE) {
#line 397 "scanner.c"
		gboolean _tmp0_;
		gboolean _tmp1_;
#line 132 "scanner.vala"
		if (!(_vala_strcmp0 (self->priv->input, "") != 0)) {
#line 132 "scanner.vala"
			break;
#line 404 "scanner.c"
		}
		_tmp0_ = FALSE;
		_tmp1_ = FALSE;
#line 133 "scanner.vala"
		if (valencia_scanner_next_char (self) == '"') {
#line 133 "scanner.vala"
			_tmp1_ = valencia_scanner_accept (self, (gunichar) '"');
#line 412 "scanner.c"
		} else {
#line 133 "scanner.vala"
			_tmp1_ = FALSE;
#line 416 "scanner.c"
		}
#line 133 "scanner.vala"
		if (_tmp1_) {
#line 133 "scanner.vala"
			_tmp0_ = valencia_scanner_accept (self, (gunichar) '"');
#line 422 "scanner.c"
		} else {
#line 133 "scanner.vala"
			_tmp0_ = FALSE;
#line 426 "scanner.c"
		}
#line 133 "scanner.vala"
		if (_tmp0_) {
#line 430 "scanner.c"
			return;
		}
	}
}


#line 137 "scanner.vala"
static void valencia_scanner_skip_line (ValenciaScanner* self) {
#line 137 "scanner.vala"
	g_return_if_fail (self != NULL);
#line 138 "scanner.vala"
	while (TRUE) {
#line 443 "scanner.c"
		gunichar c;
#line 138 "scanner.vala"
		if (!(_vala_strcmp0 (self->priv->input, "") != 0)) {
#line 138 "scanner.vala"
			break;
#line 449 "scanner.c"
		}
		c = valencia_scanner_next_char (self);
#line 140 "scanner.vala"
		if (c == '\n') {
#line 141 "scanner.vala"
			break;
#line 456 "scanner.c"
		}
	}
}


#line 145 "scanner.vala"
static gboolean valencia_scanner_is_first_token_on_line (ValenciaScanner* self) {
#line 464 "scanner.c"
	gboolean result;
	const char* line;
#line 145 "scanner.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 469 "scanner.c"
	line = self->priv->input;
	/* Go back to the '#' character*/
#line 148 "scanner.vala"
	line = g_utf8_prev_char (line);
#line 149 "scanner.vala"
	if (g_direct_equal (line, self->priv->input_begin)) {
#line 476 "scanner.c"
		result = TRUE;
		return result;
	}
#line 152 "scanner.vala"
	while (TRUE) {
#line 482 "scanner.c"
		gunichar c;
		gboolean _tmp0_;
#line 153 "scanner.vala"
		line = g_utf8_prev_char (line);
#line 487 "scanner.c"
		c = g_utf8_get_char (line);
		_tmp0_ = FALSE;
#line 155 "scanner.vala"
		if (g_direct_equal (line, self->priv->input_begin)) {
#line 155 "scanner.vala"
			_tmp0_ = g_unichar_isspace (c);
#line 494 "scanner.c"
		} else {
#line 155 "scanner.vala"
			_tmp0_ = FALSE;
#line 498 "scanner.c"
		}
#line 155 "scanner.vala"
		if (_tmp0_) {
#line 502 "scanner.c"
			result = TRUE;
			return result;
		} else {
#line 157 "scanner.vala"
			if (c == '\n') {
#line 508 "scanner.c"
				result = TRUE;
				return result;
			} else {
#line 159 "scanner.vala"
				if (!g_unichar_isspace (c)) {
#line 514 "scanner.c"
					result = FALSE;
					return result;
				}
			}
		}
	}
}


#line 164 "scanner.vala"
static ValenciaToken valencia_scanner_read_token (ValenciaScanner* self) {
#line 526 "scanner.c"
	ValenciaToken result;
#line 164 "scanner.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 165 "scanner.vala"
	while (TRUE) {
#line 532 "scanner.c"
		gunichar c;
		gboolean accept_all_chars_as_id;
		gboolean _tmp0_;
		gboolean _tmp1_;
#line 165 "scanner.vala"
		if (!(_vala_strcmp0 (self->priv->input, "") != 0)) {
#line 165 "scanner.vala"
			break;
#line 541 "scanner.c"
		}
#line 166 "scanner.vala"
		self->priv->token_start_char = self->priv->input;
#line 167 "scanner.vala"
		self->priv->token_start = self->priv->input_pos;
#line 547 "scanner.c"
		c = valencia_scanner_next_char (self);
#line 170 "scanner.vala"
		if (g_unichar_isspace (c)) {
#line 171 "scanner.vala"
			continue;
#line 553 "scanner.c"
		}
		accept_all_chars_as_id = FALSE;
#line 174 "scanner.vala"
		if (c == '@') {
#line 175 "scanner.vala"
			accept_all_chars_as_id = TRUE;
#line 560 "scanner.c"
			/* Don't include the '@' in ID's*/
#line 177 "scanner.vala"
			self->priv->token_start_char = self->priv->input;
#line 178 "scanner.vala"
			self->priv->token_start = self->priv->input_pos;
#line 179 "scanner.vala"
			c = valencia_scanner_next_char (self);
#line 568 "scanner.c"
		}
		_tmp0_ = FALSE;
		_tmp1_ = FALSE;
#line 183 "scanner.vala"
		if (g_unichar_isalpha (c)) {
#line 183 "scanner.vala"
			_tmp1_ = TRUE;
#line 576 "scanner.c"
		} else {
#line 183 "scanner.vala"
			_tmp1_ = c == '_';
#line 580 "scanner.c"
		}
#line 183 "scanner.vala"
		if (_tmp1_) {
#line 183 "scanner.vala"
			_tmp0_ = TRUE;
#line 586 "scanner.c"
		} else {
			gboolean _tmp2_;
			_tmp2_ = FALSE;
#line 183 "scanner.vala"
			if (accept_all_chars_as_id) {
#line 183 "scanner.vala"
				_tmp2_ = g_unichar_isalnum (c);
#line 594 "scanner.c"
			} else {
#line 183 "scanner.vala"
				_tmp2_ = FALSE;
#line 598 "scanner.c"
			}
#line 183 "scanner.vala"
			_tmp0_ = _tmp2_;
#line 602 "scanner.c"
		}
		/* identifier start*/
#line 183 "scanner.vala"
		if (_tmp0_) {
#line 184 "scanner.vala"
			while (TRUE) {
#line 609 "scanner.c"
				gboolean _tmp3_;
#line 185 "scanner.vala"
				c = valencia_scanner_peek_char (self);
#line 613 "scanner.c"
				_tmp3_ = FALSE;
#line 186 "scanner.vala"
				if (!g_unichar_isalnum (c)) {
#line 186 "scanner.vala"
					_tmp3_ = c != '_';
#line 619 "scanner.c"
				} else {
#line 186 "scanner.vala"
					_tmp3_ = FALSE;
#line 623 "scanner.c"
				}
#line 186 "scanner.vala"
				if (_tmp3_) {
#line 187 "scanner.vala"
					break;
#line 629 "scanner.c"
				}
#line 188 "scanner.vala"
				valencia_scanner_advance (self);
#line 633 "scanner.c"
			}
			/* We don't use the foreach statement to iterate over the keywords array;
			 that would copy the Keyword structure (and the string it contains) on
			 each iteration, which would be slow.*/
#line 193 "scanner.vala"
			if (!accept_all_chars_as_id) {
#line 640 "scanner.c"
				{
					gint i;
					i = 0;
					{
						gboolean _tmp4_;
						_tmp4_ = TRUE;
#line 194 "scanner.vala"
						while (TRUE) {
#line 194 "scanner.vala"
							if (!_tmp4_) {
#line 194 "scanner.vala"
								i = i + 1;
#line 653 "scanner.c"
							}
#line 194 "scanner.vala"
							_tmp4_ = FALSE;
#line 194 "scanner.vala"
							if (!(i < G_N_ELEMENTS (VALENCIA_keywords))) {
#line 194 "scanner.vala"
								break;
#line 661 "scanner.c"
							}
#line 195 "scanner.vala"
							if (valencia_scanner_match (self, VALENCIA_keywords[i].name)) {
#line 665 "scanner.c"
								result = VALENCIA_keywords[i].token;
								return result;
							}
						}
					}
				}
			}
			result = VALENCIA_TOKEN_ID;
			return result;
		}
#line 200 "scanner.vala"
		switch (c) {
#line 678 "scanner.c"
			case '/':
			{
				gunichar d;
				d = valencia_scanner_peek_char (self);
#line 203 "scanner.vala"
				if (d == '/') {
#line 204 "scanner.vala"
					while (TRUE) {
#line 687 "scanner.c"
						gboolean _tmp5_;
						_tmp5_ = FALSE;
#line 204 "scanner.vala"
						if (_vala_strcmp0 (self->priv->input, "") != 0) {
#line 204 "scanner.vala"
							_tmp5_ = valencia_scanner_next_char (self) != '\n';
#line 694 "scanner.c"
						} else {
#line 204 "scanner.vala"
							_tmp5_ = FALSE;
#line 698 "scanner.c"
						}
#line 204 "scanner.vala"
						if (!_tmp5_) {
#line 204 "scanner.vala"
							break;
#line 704 "scanner.c"
						}
						;
					}
#line 206 "scanner.vala"
					self->priv->token_start_char = self->priv->input;
#line 207 "scanner.vala"
					self->priv->token_start = self->priv->input_pos;
#line 208 "scanner.vala"
					continue;
#line 714 "scanner.c"
				}
#line 210 "scanner.vala"
				if (d == '*') {
#line 718 "scanner.c"
					/* multi-line comment*/
#line 211 "scanner.vala"
					valencia_scanner_advance (self);
#line 212 "scanner.vala"
					while (TRUE) {
#line 724 "scanner.c"
						gboolean _tmp6_;
#line 212 "scanner.vala"
						if (!(_vala_strcmp0 (self->priv->input, "") != 0)) {
#line 212 "scanner.vala"
							break;
#line 730 "scanner.c"
						}
						_tmp6_ = FALSE;
#line 213 "scanner.vala"
						if (valencia_scanner_next_char (self) == '*') {
#line 213 "scanner.vala"
							_tmp6_ = valencia_scanner_peek_char (self) == '/';
#line 737 "scanner.c"
						} else {
#line 213 "scanner.vala"
							_tmp6_ = FALSE;
#line 741 "scanner.c"
						}
#line 213 "scanner.vala"
						if (_tmp6_) {
#line 214 "scanner.vala"
							valencia_scanner_advance (self);
#line 747 "scanner.c"
							/* move past '/'*/
#line 215 "scanner.vala"
							break;
#line 751 "scanner.c"
						}
					}
#line 218 "scanner.vala"
					self->priv->token_start_char = self->priv->input;
#line 219 "scanner.vala"
					self->priv->token_start = self->priv->input_pos;
#line 220 "scanner.vala"
					continue;
#line 760 "scanner.c"
				}
				result = VALENCIA_TOKEN_CHAR;
				return result;
			}
			case '"':
			{
#line 224 "scanner.vala"
				if (valencia_scanner_accept (self, (gunichar) '"')) {
#line 769 "scanner.c"
					/* ""*/
#line 225 "scanner.vala"
					if (valencia_scanner_accept (self, (gunichar) '"')) {
#line 226 "scanner.vala"
						valencia_scanner_read_triple_string (self);
#line 775 "scanner.c"
					}
				} else {
#line 228 "scanner.vala"
					while (TRUE) {
#line 780 "scanner.c"
						gunichar d;
						gboolean _tmp7_;
#line 228 "scanner.vala"
						if (!(_vala_strcmp0 (self->priv->input, "") != 0)) {
#line 228 "scanner.vala"
							break;
#line 787 "scanner.c"
						}
						d = valencia_scanner_next_char (self);
						_tmp7_ = FALSE;
#line 230 "scanner.vala"
						if (d == '"') {
#line 230 "scanner.vala"
							_tmp7_ = TRUE;
#line 795 "scanner.c"
						} else {
#line 230 "scanner.vala"
							_tmp7_ = d == '\n';
#line 799 "scanner.c"
						}
#line 230 "scanner.vala"
						if (_tmp7_) {
#line 231 "scanner.vala"
							break;
#line 805 "scanner.c"
						} else {
#line 232 "scanner.vala"
							if (d == '\'') {
#line 233 "scanner.vala"
								valencia_scanner_advance (self);
#line 811 "scanner.c"
							}
						}
					}
				}
				result = VALENCIA_TOKEN_STRING_LITERAL;
				return result;
			}
			case '\'':
			{
#line 238 "scanner.vala"
				valencia_scanner_accept (self, (gunichar) '\\');
#line 239 "scanner.vala"
				valencia_scanner_advance (self);
#line 240 "scanner.vala"
				valencia_scanner_accept (self, (gunichar) '\'');
#line 827 "scanner.c"
				result = VALENCIA_TOKEN_CHAR_LITERAL;
				return result;
			}
			case '*':
			{
				result = VALENCIA_TOKEN_ASTERISK;
				return result;
			}
			case '{':
			{
				result = VALENCIA_TOKEN_LEFT_BRACE;
				return result;
			}
			case '}':
			{
				result = VALENCIA_TOKEN_RIGHT_BRACE;
				return result;
			}
			case '[':
			{
				result = VALENCIA_TOKEN_LEFT_BRACKET;
				return result;
			}
			case ']':
			{
				result = VALENCIA_TOKEN_RIGHT_BRACKET;
				return result;
			}
			case ':':
			{
				result = VALENCIA_TOKEN_COLON;
				return result;
			}
			case ',':
			{
				result = VALENCIA_TOKEN_COMMA;
				return result;
			}
			case '=':
			{
				result = VALENCIA_TOKEN_EQUALS;
				return result;
			}
			case '#':
			{
#line 251 "scanner.vala"
				if (valencia_scanner_is_first_token_on_line (self)) {
#line 252 "scanner.vala"
					valencia_scanner_skip_line (self);
#line 253 "scanner.vala"
					continue;
#line 879 "scanner.c"
				} else {
					result = VALENCIA_TOKEN_HASH;
					return result;
				}
			}
			case '(':
			{
				result = VALENCIA_TOKEN_LEFT_PAREN;
				return result;
			}
			case ')':
			{
				result = VALENCIA_TOKEN_RIGHT_PAREN;
				return result;
			}
			case '.':
			{
				gboolean _tmp8_;
				_tmp8_ = FALSE;
#line 258 "scanner.vala"
				if (valencia_scanner_peek_char (self) == '.') {
#line 258 "scanner.vala"
					_tmp8_ = valencia_scanner_peek_char2 (self) == '.';
#line 903 "scanner.c"
				} else {
#line 258 "scanner.vala"
					_tmp8_ = FALSE;
#line 907 "scanner.c"
				}
#line 258 "scanner.vala"
				if (_tmp8_) {
#line 259 "scanner.vala"
					valencia_scanner_advance (self);
#line 260 "scanner.vala"
					valencia_scanner_advance (self);
#line 915 "scanner.c"
					result = VALENCIA_TOKEN_ELLIPSIS;
					return result;
				}
				result = VALENCIA_TOKEN_PERIOD;
				return result;
			}
			case '?':
			{
				result = VALENCIA_TOKEN_QUESTION_MARK;
				return result;
			}
			case ';':
			{
				result = VALENCIA_TOKEN_SEMICOLON;
				return result;
			}
			case '<':
			{
				result = VALENCIA_TOKEN_LESS_THAN;
				return result;
			}
			case '>':
			{
				result = VALENCIA_TOKEN_GREATER_THAN;
				return result;
			}
			default:
			{
				result = VALENCIA_TOKEN_CHAR;
				return result;
			}
		}
	}
	result = VALENCIA_TOKEN_EOF;
	return result;
}


#line 274 "scanner.vala"
ValenciaToken valencia_scanner_peek_token (ValenciaScanner* self) {
#line 956 "scanner.c"
	ValenciaToken result;
#line 274 "scanner.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 275 "scanner.vala"
	if (self->priv->token == VALENCIA_TOKEN_NONE) {
#line 276 "scanner.vala"
		self->priv->token = valencia_scanner_read_token (self);
#line 964 "scanner.c"
	}
	result = self->priv->token;
	return result;
}


#line 280 "scanner.vala"
ValenciaToken valencia_scanner_next_token (ValenciaScanner* self) {
#line 973 "scanner.c"
	ValenciaToken result;
	ValenciaToken t;
#line 280 "scanner.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 978 "scanner.c"
	t = valencia_scanner_peek_token (self);
#line 282 "scanner.vala"
	self->priv->token = VALENCIA_TOKEN_NONE;
#line 283 "scanner.vala"
	self->priv->start_char = self->priv->token_start_char;
#line 284 "scanner.vala"
	self->priv->end_char = self->priv->input;
#line 285 "scanner.vala"
	self->start = self->priv->token_start;
#line 286 "scanner.vala"
	self->end = self->priv->input_pos;
#line 990 "scanner.c"
	result = t;
	return result;
}


#line 290 "scanner.vala"
gboolean valencia_scanner_accept_token (ValenciaScanner* self, ValenciaToken t) {
#line 998 "scanner.c"
	gboolean result;
#line 290 "scanner.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 291 "scanner.vala"
	if (valencia_scanner_peek_token (self) == t) {
#line 292 "scanner.vala"
		valencia_scanner_next_token (self);
#line 1006 "scanner.c"
		result = TRUE;
		return result;
	}
	result = FALSE;
	return result;
}


#line 298 "scanner.vala"
gboolean valencia_scanner_eof (ValenciaScanner* self) {
#line 1017 "scanner.c"
	gboolean result;
#line 298 "scanner.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1021 "scanner.c"
	result = valencia_scanner_peek_token (self) == VALENCIA_TOKEN_EOF;
	return result;
}


/* Return the source text of the last token retrieved.*/
#line 301 "scanner.vala"
char* valencia_scanner_val (ValenciaScanner* self) {
#line 1030 "scanner.c"
	char* result;
	gsize bytes;
#line 301 "scanner.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 1035 "scanner.c"
	bytes = ((gchar*) self->priv->end_char) - ((gchar*) self->priv->start_char);
	result = g_strndup (self->priv->start_char, bytes);
	return result;
}


#line 306 "scanner.vala"
const char* valencia_scanner_get_start (ValenciaScanner* self) {
#line 1044 "scanner.c"
	const char* result;
#line 306 "scanner.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 1048 "scanner.c"
	result = self->priv->start_char;
	return result;
}


#line 310 "scanner.vala"
const char* valencia_scanner_get_start_after_comments (ValenciaScanner* self) {
#line 1056 "scanner.c"
	const char* result;
#line 310 "scanner.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 1060 "scanner.c"
	/* Skip any comments after the end character and take the first character after them*/
#line 312 "scanner.vala"
	valencia_scanner_peek_token (self);
#line 1064 "scanner.c"
	result = self->priv->token_start_char;
	return result;
}


static void valencia_scanner_class_init (ValenciaScannerClass * klass) {
	valencia_scanner_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValenciaScannerPrivate));
	G_OBJECT_CLASS (klass)->finalize = valencia_scanner_finalize;
}


static void valencia_scanner_instance_init (ValenciaScanner * self) {
	self->priv = VALENCIA_SCANNER_GET_PRIVATE (self);
	self->priv->token = VALENCIA_TOKEN_NONE;
}


static void valencia_scanner_finalize (GObject* obj) {
	ValenciaScanner * self;
	self = VALENCIA_SCANNER (obj);
	G_OBJECT_CLASS (valencia_scanner_parent_class)->finalize (obj);
}


GType valencia_scanner_get_type (void) {
	static GType valencia_scanner_type_id = 0;
	if (valencia_scanner_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValenciaScannerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valencia_scanner_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValenciaScanner), 0, (GInstanceInitFunc) valencia_scanner_instance_init, NULL };
		valencia_scanner_type_id = g_type_register_static (G_TYPE_OBJECT, "ValenciaScanner", &g_define_type_info, 0);
	}
	return valencia_scanner_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




