/* Copyright 2009 Yorba Foundation
 *
 * This software is licensed under the GNU Lesser General Public License
 * (version 2.1 or later).  See the COPYING file in this distribution. 
 */

#include <glib.h>
#include <glib-object.h>
#include <gee.h>
#include <gio/gio.h>
#include <stdlib.h>
#include <string.h>


#define TYPE_PAIR (pair_get_type ())
#define PAIR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PAIR, Pair))
#define PAIR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PAIR, PairClass))
#define IS_PAIR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PAIR))
#define IS_PAIR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PAIR))
#define PAIR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PAIR, PairClass))

typedef struct _Pair Pair;
typedef struct _PairClass PairClass;
typedef struct _PairPrivate PairPrivate;

#define TYPE_STACK (stack_get_type ())
#define STACK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_STACK, Stack))
#define STACK_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_STACK, StackClass))
#define IS_STACK(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_STACK))
#define IS_STACK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_STACK))
#define STACK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_STACK, StackClass))

typedef struct _Stack Stack;
typedef struct _StackClass StackClass;
typedef struct _StackPrivate StackPrivate;

/*// Helper data structures ////*/
struct _Pair {
	GObject parent_instance;
	PairPrivate * priv;
};

struct _PairClass {
	GObjectClass parent_class;
};

struct _PairPrivate {
	GType g1_type;
	GBoxedCopyFunc g1_dup_func;
	GDestroyNotify g1_destroy_func;
	GType g2_type;
	GBoxedCopyFunc g2_dup_func;
	GDestroyNotify g2_destroy_func;
	gpointer _first;
	gpointer _second;
};

struct _Stack {
	GObject parent_instance;
	StackPrivate * priv;
};

struct _StackClass {
	GObjectClass parent_class;
};

struct _StackPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
	GeeArrayList* container;
};


static gpointer pair_parent_class = NULL;
static gpointer stack_parent_class = NULL;

void qsort (void* p, gsize num, gsize size, GCompareFunc func);
GType pair_get_type (void);
#define PAIR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_PAIR, PairPrivate))
enum  {
	PAIR_DUMMY_PROPERTY,
	PAIR_FIRST,
	PAIR_SECOND,
	PAIR_G1_TYPE,
	PAIR_G1_DUP_FUNC,
	PAIR_G1_DESTROY_FUNC,
	PAIR_G2_TYPE,
	PAIR_G2_DUP_FUNC,
	PAIR_G2_DESTROY_FUNC
};
static void pair_set_first (Pair* self, gconstpointer value);
static void pair_set_second (Pair* self, gconstpointer value);
Pair* pair_new (GType g1_type, GBoxedCopyFunc g1_dup_func, GDestroyNotify g1_destroy_func, GType g2_type, GBoxedCopyFunc g2_dup_func, GDestroyNotify g2_destroy_func, gconstpointer object1, gconstpointer object2);
Pair* pair_construct (GType object_type, GType g1_type, GBoxedCopyFunc g1_dup_func, GDestroyNotify g1_destroy_func, GType g2_type, GBoxedCopyFunc g2_dup_func, GDestroyNotify g2_destroy_func, gconstpointer object1, gconstpointer object2);
gconstpointer pair_get_first (Pair* self);
gconstpointer pair_get_second (Pair* self);
static void pair_finalize (GObject* obj);
static void pair_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void pair_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType stack_get_type (void);
#define STACK_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_STACK, StackPrivate))
enum  {
	STACK_DUMMY_PROPERTY,
	STACK_G_TYPE,
	STACK_G_DUP_FUNC,
	STACK_G_DESTROY_FUNC
};
Stack* stack_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
Stack* stack_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
void stack_push (Stack* self, gconstpointer item);
gpointer stack_top (Stack* self);
void stack_pop (Stack* self);
gint stack_size (Stack* self);
static void stack_finalize (GObject* obj);
static void stack_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void stack_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
gboolean dir_has_parent (const char* dir, const char* parent);
gint compare_string (void* a, void* b);
char* filename_to_uri (const char* filename);
void make_pipe (gint fd, GIOFunc func, void* func_target, GError** error);
static int _vala_strcmp0 (const char * str1, const char * str2);



#line 17 "util.vala"
Pair* pair_construct (GType object_type, GType g1_type, GBoxedCopyFunc g1_dup_func, GDestroyNotify g1_destroy_func, GType g2_type, GBoxedCopyFunc g2_dup_func, GDestroyNotify g2_destroy_func, gconstpointer object1, gconstpointer object2) {
#line 129 "util.c"
	GParameter * __params;
	GParameter * __params_it;
	Pair * self;
	__params = g_new0 (GParameter, 8);
	__params_it = __params;
	__params_it->name = "g1-type";
	g_value_init (&__params_it->value, G_TYPE_GTYPE);
	g_value_set_gtype (&__params_it->value, g1_type);
	__params_it++;
	__params_it->name = "g1-dup-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, g1_dup_func);
	__params_it++;
	__params_it->name = "g1-destroy-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, g1_destroy_func);
	__params_it++;
	__params_it->name = "g2-type";
	g_value_init (&__params_it->value, G_TYPE_GTYPE);
	g_value_set_gtype (&__params_it->value, g2_type);
	__params_it++;
	__params_it->name = "g2-dup-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, g2_dup_func);
	__params_it++;
	__params_it->name = "g2-destroy-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, g2_destroy_func);
	__params_it++;
#line 18 "util.vala"
	__params_it->name = "first";
#line 18 "util.vala"
	g_value_init (&__params_it->value, G_TYPE_POINTER);
#line 18 "util.vala"
	g_value_set_pointer (&__params_it->value, object1);
#line 18 "util.vala"
	__params_it++;
#line 19 "util.vala"
	__params_it->name = "second";
#line 19 "util.vala"
	g_value_init (&__params_it->value, G_TYPE_POINTER);
#line 19 "util.vala"
	g_value_set_pointer (&__params_it->value, object2);
#line 19 "util.vala"
	__params_it++;
#line 175 "util.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 17 "util.vala"
Pair* pair_new (GType g1_type, GBoxedCopyFunc g1_dup_func, GDestroyNotify g1_destroy_func, GType g2_type, GBoxedCopyFunc g2_dup_func, GDestroyNotify g2_destroy_func, gconstpointer object1, gconstpointer object2) {
#line 17 "util.vala"
	return pair_construct (TYPE_PAIR, g1_type, g1_dup_func, g1_destroy_func, g2_type, g2_dup_func, g2_destroy_func, object1, object2);
#line 190 "util.c"
}


gconstpointer pair_get_first (Pair* self) {
	gconstpointer result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_first;
	return result;
}


static void pair_set_first (Pair* self, gconstpointer value) {
	gpointer _tmp1_;
	gconstpointer _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 14 "util.vala"
	_tmp0_ = NULL;
#line 209 "util.c"
	self->priv->_first = (_tmp1_ = (_tmp0_ = value, ((_tmp0_ == NULL) || (self->priv->g1_dup_func == NULL)) ? ((gpointer) _tmp0_) : self->priv->g1_dup_func ((gpointer) _tmp0_)), ((self->priv->_first == NULL) || (self->priv->g1_destroy_func == NULL)) ? NULL : (self->priv->_first = (self->priv->g1_destroy_func (self->priv->_first), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "first");
}


gconstpointer pair_get_second (Pair* self) {
	gconstpointer result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_second;
	return result;
}


static void pair_set_second (Pair* self, gconstpointer value) {
	gpointer _tmp1_;
	gconstpointer _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 15 "util.vala"
	_tmp0_ = NULL;
#line 230 "util.c"
	self->priv->_second = (_tmp1_ = (_tmp0_ = value, ((_tmp0_ == NULL) || (self->priv->g2_dup_func == NULL)) ? ((gpointer) _tmp0_) : self->priv->g2_dup_func ((gpointer) _tmp0_)), ((self->priv->_second == NULL) || (self->priv->g2_destroy_func == NULL)) ? NULL : (self->priv->_second = (self->priv->g2_destroy_func (self->priv->_second), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "second");
}


static void pair_class_init (PairClass * klass) {
	pair_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (PairPrivate));
	G_OBJECT_CLASS (klass)->get_property = pair_get_property;
	G_OBJECT_CLASS (klass)->set_property = pair_set_property;
	G_OBJECT_CLASS (klass)->finalize = pair_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), PAIR_G1_TYPE, g_param_spec_gtype ("g1-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), PAIR_G1_DUP_FUNC, g_param_spec_pointer ("g1-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), PAIR_G1_DESTROY_FUNC, g_param_spec_pointer ("g1-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), PAIR_G2_TYPE, g_param_spec_gtype ("g2-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), PAIR_G2_DUP_FUNC, g_param_spec_pointer ("g2-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), PAIR_G2_DESTROY_FUNC, g_param_spec_pointer ("g2-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), PAIR_FIRST, g_param_spec_pointer ("first", "first", "first", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), PAIR_SECOND, g_param_spec_pointer ("second", "second", "second", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void pair_instance_init (Pair * self) {
	self->priv = PAIR_GET_PRIVATE (self);
}


static void pair_finalize (GObject* obj) {
	Pair * self;
	self = PAIR (obj);
	((self->priv->_first == NULL) || (self->priv->g1_destroy_func == NULL)) ? NULL : (self->priv->_first = (self->priv->g1_destroy_func (self->priv->_first), NULL));
	((self->priv->_second == NULL) || (self->priv->g2_destroy_func == NULL)) ? NULL : (self->priv->_second = (self->priv->g2_destroy_func (self->priv->_second), NULL));
	G_OBJECT_CLASS (pair_parent_class)->finalize (obj);
}


GType pair_get_type (void) {
	static GType pair_type_id = 0;
	if (pair_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (PairClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) pair_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Pair), 0, (GInstanceInitFunc) pair_instance_init, NULL };
		pair_type_id = g_type_register_static (G_TYPE_OBJECT, "Pair", &g_define_type_info, 0);
	}
	return pair_type_id;
}


static void pair_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	Pair * self;
	gpointer boxed;
	self = PAIR (object);
	switch (property_id) {
		case PAIR_FIRST:
		g_value_set_pointer (value, pair_get_first (self));
		break;
		case PAIR_SECOND:
		g_value_set_pointer (value, pair_get_second (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void pair_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	Pair * self;
	self = PAIR (object);
	switch (property_id) {
		case PAIR_FIRST:
		pair_set_first (self, g_value_get_pointer (value));
		break;
		case PAIR_SECOND:
		pair_set_second (self, g_value_get_pointer (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
		case PAIR_G1_TYPE:
		self->priv->g1_type = g_value_get_gtype (value);
		break;
		case PAIR_G1_DUP_FUNC:
		self->priv->g1_dup_func = g_value_get_pointer (value);
		break;
		case PAIR_G1_DESTROY_FUNC:
		self->priv->g1_destroy_func = g_value_get_pointer (value);
		break;
		case PAIR_G2_TYPE:
		self->priv->g2_type = g_value_get_gtype (value);
		break;
		case PAIR_G2_DUP_FUNC:
		self->priv->g2_dup_func = g_value_get_pointer (value);
		break;
		case PAIR_G2_DESTROY_FUNC:
		self->priv->g2_destroy_func = g_value_get_pointer (value);
		break;
	}
}


#line 26 "util.vala"
Stack* stack_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func) {
#line 332 "util.c"
	GParameter * __params;
	GParameter * __params_it;
	Stack * self;
	GeeArrayList* _tmp0_;
	__params = g_new0 (GParameter, 3);
	__params_it = __params;
	__params_it->name = "g-type";
	g_value_init (&__params_it->value, G_TYPE_GTYPE);
	g_value_set_gtype (&__params_it->value, g_type);
	__params_it++;
	__params_it->name = "g-dup-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, g_dup_func);
	__params_it++;
	__params_it->name = "g-destroy-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, g_destroy_func);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	_tmp0_ = NULL;
#line 27 "util.vala"
	self->priv->container = (_tmp0_ = gee_array_list_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func, g_direct_equal), (self->priv->container == NULL) ? NULL : (self->priv->container = (gee_collection_object_unref (self->priv->container), NULL)), _tmp0_);
#line 355 "util.c"
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 26 "util.vala"
Stack* stack_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func) {
#line 26 "util.vala"
	return stack_construct (TYPE_STACK, g_type, g_dup_func, g_destroy_func);
#line 369 "util.c"
}


#line 30 "util.vala"
void stack_push (Stack* self, gconstpointer item) {
#line 30 "util.vala"
	g_return_if_fail (self != NULL);
#line 31 "util.vala"
	gee_collection_add ((GeeCollection*) self->priv->container, item);
#line 379 "util.c"
}


#line 34 "util.vala"
gpointer stack_top (Stack* self) {
#line 385 "util.c"
	gpointer result;
#line 34 "util.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 35 "util.vala"
	g_assert (gee_collection_get_size ((GeeCollection*) self->priv->container) > 0);
#line 391 "util.c"
	result = gee_list_get ((GeeList*) self->priv->container, gee_collection_get_size ((GeeCollection*) self->priv->container) - 1);
	return result;
}


#line 39 "util.vala"
void stack_pop (Stack* self) {
#line 39 "util.vala"
	g_return_if_fail (self != NULL);
#line 40 "util.vala"
	g_assert (gee_collection_get_size ((GeeCollection*) self->priv->container) > 0);
#line 41 "util.vala"
	gee_list_remove_at ((GeeList*) self->priv->container, gee_collection_get_size ((GeeCollection*) self->priv->container) - 1);
#line 405 "util.c"
}


#line 44 "util.vala"
gint stack_size (Stack* self) {
#line 411 "util.c"
	gint result;
#line 44 "util.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 415 "util.c"
	result = gee_collection_get_size ((GeeCollection*) self->priv->container);
	return result;
}


static void stack_class_init (StackClass * klass) {
	stack_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (StackPrivate));
	G_OBJECT_CLASS (klass)->get_property = stack_get_property;
	G_OBJECT_CLASS (klass)->set_property = stack_set_property;
	G_OBJECT_CLASS (klass)->finalize = stack_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), STACK_G_TYPE, g_param_spec_gtype ("g-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), STACK_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), STACK_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void stack_instance_init (Stack * self) {
	self->priv = STACK_GET_PRIVATE (self);
}


static void stack_finalize (GObject* obj) {
	Stack * self;
	self = STACK (obj);
	(self->priv->container == NULL) ? NULL : (self->priv->container = (gee_collection_object_unref (self->priv->container), NULL));
	G_OBJECT_CLASS (stack_parent_class)->finalize (obj);
}


GType stack_get_type (void) {
	static GType stack_type_id = 0;
	if (stack_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (StackClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) stack_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Stack), 0, (GInstanceInitFunc) stack_instance_init, NULL };
		stack_type_id = g_type_register_static (G_TYPE_OBJECT, "Stack", &g_define_type_info, 0);
	}
	return stack_type_id;
}


static void stack_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	Stack * self;
	gpointer boxed;
	self = STACK (object);
	switch (property_id) {
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void stack_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	Stack * self;
	self = STACK (object);
	switch (property_id) {
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
		case STACK_G_TYPE:
		self->priv->g_type = g_value_get_gtype (value);
		break;
		case STACK_G_DUP_FUNC:
		self->priv->g_dup_func = g_value_get_pointer (value);
		break;
		case STACK_G_DESTROY_FUNC:
		self->priv->g_destroy_func = g_value_get_pointer (value);
		break;
	}
}


/*// GLib helper functions ////*/
#line 51 "util.vala"
gboolean dir_has_parent (const char* dir, const char* parent) {
#line 491 "util.c"
	gboolean result;
	GFile* new_path;
#line 51 "util.vala"
	g_return_val_if_fail (dir != NULL, FALSE);
#line 51 "util.vala"
	g_return_val_if_fail (parent != NULL, FALSE);
#line 498 "util.c"
	new_path = g_file_new_for_path (dir);
#line 53 "util.vala"
	while (TRUE) {
#line 502 "util.c"
		char* _tmp0_;
		gboolean _tmp1_;
		GFile* _tmp2_;
#line 952 "gio-2.0.vapi"
		_tmp0_ = NULL;
#line 53 "util.vala"
		if ((_tmp1_ = !(_vala_strcmp0 (parent, _tmp0_ = g_file_get_path (new_path)) != 0), _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_)) {
#line 53 "util.vala"
			break;
#line 512 "util.c"
		}
		_tmp2_ = NULL;
#line 54 "util.vala"
		new_path = (_tmp2_ = g_file_get_parent (new_path), (new_path == NULL) ? NULL : (new_path = (g_object_unref (new_path), NULL)), _tmp2_);
#line 56 "util.vala"
		if (new_path == NULL) {
#line 519 "util.c"
			result = FALSE;
			(new_path == NULL) ? NULL : (new_path = (g_object_unref (new_path), NULL));
			return result;
		}
	}
	result = TRUE;
	(new_path == NULL) ? NULL : (new_path = (g_object_unref (new_path), NULL));
	return result;
}


#line 63 "util.vala"
gint compare_string (void* a, void* b) {
#line 533 "util.c"
	gint result;
	gchar** a_string;
	gchar** b_string;
	a_string = a;
	b_string = b;
	result = strcmp (*a_string, *b_string);
	return result;
}


#line 70 "util.vala"
char* filename_to_uri (const char* filename) {
#line 546 "util.c"
	char* result;
	GError * _inner_error_;
#line 70 "util.vala"
	g_return_val_if_fail (filename != NULL, NULL);
#line 551 "util.c"
	_inner_error_ = NULL;
	{
		char* _tmp0_;
		_tmp0_ = g_filename_to_uri (filename, NULL, &_inner_error_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_CONVERT_ERROR) {
				goto __catch12_g_convert_error;
			}
			goto __finally12;
		}
		result = _tmp0_;
		return result;
	}
	goto __finally12;
	__catch12_g_convert_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			result = NULL;
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			return result;
		}
	}
	__finally12:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
}


#line 76 "util.vala"
void make_pipe (gint fd, GIOFunc func, void* func_target, GError** error) {
#line 588 "util.c"
	GError * _inner_error_;
	GIOChannel* pipe;
	_inner_error_ = NULL;
	pipe = g_io_channel_unix_new (fd);
#line 78 "util.vala"
	g_io_channel_set_flags (pipe, G_IO_FLAG_NONBLOCK, &_inner_error_);
#line 595 "util.c"
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_IO_CHANNEL_ERROR) {
			g_propagate_error (error, _inner_error_);
			(pipe == NULL) ? NULL : (pipe = (g_io_channel_unref (pipe), NULL));
			return;
		} else {
			(pipe == NULL) ? NULL : (pipe = (g_io_channel_unref (pipe), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
#line 79 "util.vala"
	g_io_add_watch (pipe, G_IO_IN | G_IO_HUP, func, func_target);
#line 610 "util.c"
	(pipe == NULL) ? NULL : (pipe = (g_io_channel_unref (pipe), NULL));
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




