/* file-browser.vala
 *
 * Copyright (C) 2009 Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <valide-0.0.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <gio/gio.h>
#include <glib/gstdio.h>
#include <glib/gi18n-lib.h>


#define TYPE_FILE_BROWSER (file_browser_get_type ())
#define FILE_BROWSER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_FILE_BROWSER, FileBrowser))
#define FILE_BROWSER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_FILE_BROWSER, FileBrowserClass))
#define IS_FILE_BROWSER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_FILE_BROWSER))
#define IS_FILE_BROWSER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_FILE_BROWSER))
#define FILE_BROWSER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_FILE_BROWSER, FileBrowserClass))

typedef struct _FileBrowser FileBrowser;
typedef struct _FileBrowserClass FileBrowserClass;
typedef struct _FileBrowserPrivate FileBrowserPrivate;

#define FILE_BROWSER_TYPE_COL (file_browser_col_get_type ())

struct _FileBrowser {
	GObject parent_instance;
	FileBrowserPrivate * priv;
};

struct _FileBrowserClass {
	GObjectClass parent_class;
};

struct _FileBrowserPrivate {
	char* _cwd;
	GtkVBox* box;
	gboolean _show_hidden;
	GtkTreeView* tree_view;
	GtkFileChooserButton* file_chooser;
	char* _path;
	ValideWindow* _window;
};

/* enum for tree model columns, not all is currently used */
typedef enum  {
	FILE_BROWSER_COL_ICON,
	FILE_BROWSER_COL_NAME,
	FILE_BROWSER_COL_PATH,
	FILE_BROWSER_COL_NB
} FileBrowserCol;


static gpointer file_browser_parent_class = NULL;
static ValidePluginIface* file_browser_valide_plugin_parent_iface = NULL;

GType file_browser_get_type (void);
#define FILE_BROWSER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_FILE_BROWSER, FileBrowserPrivate))
enum  {
	FILE_BROWSER_DUMMY_PROPERTY,
	FILE_BROWSER_PATH,
	FILE_BROWSER_WINDOW
};
static GType file_browser_col_get_type (void);
static const char* file_browser_get_cwd (FileBrowser* self);
static gboolean file_browser_get_show_hidden (FileBrowser* self);
static void file_browser_populate_asc (FileBrowser* self, GtkTreeIter* root);
static gboolean _lambda0_ (FileBrowser* self);
static gboolean __lambda0__gsource_func (gpointer self);
static void file_browser_populate (FileBrowser* self, GObject* sender);
static void file_browser_set_cwd (FileBrowser* self, const char* value);
static void file_browser_on_row_activated (FileBrowser* self, GtkTreeView* sender, const GtkTreePath* tree_path, GtkTreeViewColumn* column);
static gint file_browser_sort_func (FileBrowser* self, GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b);
FileBrowser* file_browser_new (void);
FileBrowser* file_browser_construct (GType object_type);
static void file_browser_set_show_hidden (FileBrowser* self, gboolean value);
static void _lambda1_ (FileBrowser* self);
static void __lambda1__gtk_button_clicked (GtkButton* _sender, gpointer self);
static void _lambda2_ (FileBrowser* self);
static void __lambda2__gtk_button_clicked (GtkButton* _sender, gpointer self);
static void _lambda3_ (GtkToggleButton* s, FileBrowser* self);
static void __lambda3__gtk_toggle_button_toggled (GtkToggleButton* _sender, gpointer self);
static void _lambda4_ (GtkFileChooser* s, FileBrowser* self);
static void __lambda4__gtk_file_chooser_selection_changed (GtkFileChooser* _sender, gpointer self);
static gint _file_browser_sort_func_gtk_tree_iter_compare_func (GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b, gpointer self);
static void _file_browser_on_row_activated_gtk_tree_view_row_activated (GtkTreeView* _sender, const GtkTreePath* path, GtkTreeViewColumn* column, gpointer self);
static void _lambda5_ (GtkTreeView* s, GtkTreeIter* i, const GtkTreePath* p, FileBrowser* self);
static void __lambda5__gtk_tree_view_row_expanded (GtkTreeView* _sender, GtkTreeIter* iter, const GtkTreePath* path, gpointer self);
static GObject * file_browser_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void file_browser_finalize (GObject* obj);
static void file_browser_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void file_browser_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType register_plugin (void);
static int _vala_strcmp0 (const char * str1, const char * str2);




static GType file_browser_col_get_type (void) {
	static GType file_browser_col_type_id = 0;
	if (G_UNLIKELY (file_browser_col_type_id == 0)) {
		static const GEnumValue values[] = {{FILE_BROWSER_COL_ICON, "FILE_BROWSER_COL_ICON", "icon"}, {FILE_BROWSER_COL_NAME, "FILE_BROWSER_COL_NAME", "name"}, {FILE_BROWSER_COL_PATH, "FILE_BROWSER_COL_PATH", "path"}, {FILE_BROWSER_COL_NB, "FILE_BROWSER_COL_NB", "nb"}, {0, NULL, NULL}};
		file_browser_col_type_id = g_enum_register_static ("FileBrowserCol", values);
	}
	return file_browser_col_type_id;
}


#line 71 "file-browser.vala"
static void file_browser_populate_asc (FileBrowser* self, GtkTreeIter* root) {
#line 133 "file-browser.c"
	GError * _inner_error_;
	GtkTreeStore* model;
	GtkTreeStore* _tmp2_;
	GtkTreeStore* _tmp1_;
	GtkTreeModel* _tmp0_;
#line 71 "file-browser.vala"
	g_return_if_fail (self != NULL);
#line 141 "file-browser.c"
	_inner_error_ = NULL;
	model = NULL;
	_tmp2_ = NULL;
#line 75 "file-browser.vala"
	_tmp1_ = NULL;
#line 75 "file-browser.vala"
	_tmp0_ = NULL;
#line 75 "file-browser.vala"
	model = (_tmp2_ = (_tmp1_ = (_tmp0_ = gtk_tree_view_get_model (self->priv->tree_view), GTK_IS_TREE_STORE (_tmp0_) ? ((GtkTreeStore*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (model == NULL) ? NULL : (model = (g_object_unref (model), NULL)), _tmp2_);
#line 76 "file-browser.vala"
	if (root == NULL) {
#line 78 "file-browser.vala"
		gtk_tree_store_clear (model);
#line 155 "file-browser.c"
	}
	{
		GDir* dir;
		char* path;
		char* dirname;
		GtkTreeIter iter = {0};
		GdkPixbuf* pixbuf;
		GtkTreeIter parent = {0};
		const char* filename;
		GDir* _tmp5_;
		GDir* _tmp6_;
		dir = NULL;
		path = NULL;
		dirname = NULL;
		pixbuf = NULL;
		filename = NULL;
#line 90 "file-browser.vala"
		if (root == NULL) {
#line 174 "file-browser.c"
			char* _tmp4_;
			const char* _tmp3_;
			_tmp4_ = NULL;
#line 92 "file-browser.vala"
			_tmp3_ = NULL;
#line 92 "file-browser.vala"
			dirname = (_tmp4_ = (_tmp3_ = file_browser_get_cwd (self), (_tmp3_ == NULL) ? NULL : g_strdup (_tmp3_)), dirname = (g_free (dirname), NULL), _tmp4_);
#line 182 "file-browser.c"
		} else {
			GtkTreeIter child = {0};
#line 98 "file-browser.vala"
			gtk_tree_model_get ((GtkTreeModel*) model, root, FILE_BROWSER_COL_PATH, &dirname, -1);
#line 99 "file-browser.vala"
			if (gtk_tree_model_iter_children ((GtkTreeModel*) model, &child, root)) {
#line 189 "file-browser.c"
				char* name;
				name = NULL;
#line 103 "file-browser.vala"
				gtk_tree_model_get ((GtkTreeModel*) model, &child, FILE_BROWSER_COL_NAME, &name, -1);
#line 104 "file-browser.vala"
				if (_vala_strcmp0 (name, "...") == 0) {
#line 106 "file-browser.vala"
					gtk_tree_store_remove (model, &child);
#line 198 "file-browser.c"
				} else {
					name = (g_free (name), NULL);
					(dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL));
					path = (g_free (path), NULL);
					dirname = (g_free (dirname), NULL);
					(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
					(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
					return;
				}
				name = (g_free (name), NULL);
			}
		}
		_tmp5_ = g_dir_open (dirname, 0, &_inner_error_);
		if (_inner_error_ != NULL) {
			(dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL));
			path = (g_free (path), NULL);
			dirname = (g_free (dirname), NULL);
			(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
			goto __catch0_g_error;
			goto __finally0;
		}
		_tmp6_ = NULL;
#line 115 "file-browser.vala"
		dir = (_tmp6_ = _tmp5_, (dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL)), _tmp6_);
#line 116 "file-browser.vala"
		while (TRUE) {
#line 116 "file-browser.vala"
			if (!((filename = g_dir_read_name (dir)) != NULL)) {
#line 116 "file-browser.vala"
				break;
#line 229 "file-browser.c"
			}
			{
				GFile* file;
				GFileInfo* file_info;
				char* _tmp7_;
				GFile* _tmp8_;
				GFileInfo* _tmp9_;
				GFileInfo* _tmp10_;
				gboolean _tmp11_;
				file = NULL;
				file_info = NULL;
				_tmp7_ = NULL;
#line 123 "file-browser.vala"
				path = (_tmp7_ = g_build_filename (dirname, filename, NULL), path = (g_free (path), NULL), _tmp7_);
#line 244 "file-browser.c"
				_tmp8_ = NULL;
#line 124 "file-browser.vala"
				file = (_tmp8_ = g_file_new_for_path (path), (file == NULL) ? NULL : (file = (g_object_unref (file), NULL)), _tmp8_);
#line 248 "file-browser.c"
				_tmp9_ = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_IS_HIDDEN, G_FILE_QUERY_INFO_NONE, NULL, &_inner_error_);
				if (_inner_error_ != NULL) {
					(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
					(file_info == NULL) ? NULL : (file_info = (g_object_unref (file_info), NULL));
					goto __catch1_g_error;
					goto __finally1;
				}
				_tmp10_ = NULL;
#line 125 "file-browser.vala"
				file_info = (_tmp10_ = _tmp9_, (file_info == NULL) ? NULL : (file_info = (g_object_unref (file_info), NULL)), _tmp10_);
#line 259 "file-browser.c"
				_tmp11_ = FALSE;
#line 129 "file-browser.vala"
				if (!g_file_info_get_is_hidden (file_info)) {
#line 129 "file-browser.vala"
					_tmp11_ = TRUE;
#line 265 "file-browser.c"
				} else {
#line 129 "file-browser.vala"
					_tmp11_ = file_browser_get_show_hidden (self);
#line 269 "file-browser.c"
				}
#line 129 "file-browser.vala"
				if (_tmp11_) {
#line 273 "file-browser.c"
					GdkPixbuf* _tmp12_;
					GdkPixbuf* _tmp13_;
					_tmp12_ = valide_utils_get_pixbuf_for_file (path, GTK_ICON_SIZE_BUTTON, &_inner_error_);
					if (_inner_error_ != NULL) {
						(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
						(file_info == NULL) ? NULL : (file_info = (g_object_unref (file_info), NULL));
						goto __catch1_g_error;
						goto __finally1;
					}
					_tmp13_ = NULL;
#line 131 "file-browser.vala"
					pixbuf = (_tmp13_ = _tmp12_, (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp13_);
#line 132 "file-browser.vala"
					gtk_tree_store_append (model, &parent, root);
#line 133 "file-browser.vala"
					gtk_tree_store_set (model, &parent, FILE_BROWSER_COL_ICON, pixbuf, FILE_BROWSER_COL_NAME, filename, FILE_BROWSER_COL_PATH, path, -1);
#line 134 "file-browser.vala"
					if (g_file_test (path, G_FILE_TEST_IS_DIR)) {
#line 136 "file-browser.vala"
						gtk_tree_store_append (model, &iter, &parent);
#line 137 "file-browser.vala"
						gtk_tree_store_set (model, &iter, FILE_BROWSER_COL_NAME, "...", -1);
#line 296 "file-browser.c"
					}
				}
				(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
				(file_info == NULL) ? NULL : (file_info = (g_object_unref (file_info), NULL));
			}
			goto __finally1;
			__catch1_g_error:
			{
				GError * e;
				e = _inner_error_;
				_inner_error_ = NULL;
				{
#line 143 "file-browser.vala"
					g_debug (e->message);
#line 311 "file-browser.c"
					(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
				}
			}
			__finally1:
			if (_inner_error_ != NULL) {
				(dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL));
				path = (g_free (path), NULL);
				dirname = (g_free (dirname), NULL);
				(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
				goto __catch0_g_error;
				goto __finally0;
			}
		}
		(dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL));
		path = (g_free (path), NULL);
		dirname = (g_free (dirname), NULL);
		(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
	}
	goto __finally0;
	__catch0_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 149 "file-browser.vala"
			g_warning (e->message);
#line 339 "file-browser.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally0:
	if (_inner_error_ != NULL) {
		(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
}


static gboolean _lambda0_ (FileBrowser* self) {
	gboolean result;
#line 159 "file-browser.vala"
	file_browser_populate_asc (self, NULL);
#line 358 "file-browser.c"
	result = FALSE;
	return result;
}


static gboolean __lambda0__gsource_func (gpointer self) {
	return _lambda0_ (self);
}


/**
   * Call to re-populate tree view.
   */
#line 156 "file-browser.vala"
static void file_browser_populate (FileBrowser* self, GObject* sender) {
#line 156 "file-browser.vala"
	g_return_if_fail (self != NULL);
#line 156 "file-browser.vala"
	g_return_if_fail (sender != NULL);
#line 158 "file-browser.vala"
	g_idle_add (__lambda0__gsource_func, self);
#line 380 "file-browser.c"
}


#line 164 "file-browser.vala"
static void file_browser_on_row_activated (FileBrowser* self, GtkTreeView* sender, const GtkTreePath* tree_path, GtkTreeViewColumn* column) {
#line 386 "file-browser.c"
	GError * _inner_error_;
	char* path;
	GtkTreeIter iter = {0};
	GtkTreeModel* model;
	GtkTreeModel* _tmp1_;
	GtkTreeModel* _tmp0_;
#line 164 "file-browser.vala"
	g_return_if_fail (self != NULL);
#line 164 "file-browser.vala"
	g_return_if_fail (sender != NULL);
#line 164 "file-browser.vala"
	g_return_if_fail (tree_path != NULL);
#line 164 "file-browser.vala"
	g_return_if_fail (column != NULL);
#line 401 "file-browser.c"
	_inner_error_ = NULL;
	path = NULL;
	model = NULL;
	_tmp1_ = NULL;
#line 171 "file-browser.vala"
	_tmp0_ = NULL;
#line 171 "file-browser.vala"
	model = (_tmp1_ = (_tmp0_ = gtk_tree_view_get_model (self->priv->tree_view), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (model == NULL) ? NULL : (model = (g_object_unref (model), NULL)), _tmp1_);
#line 172 "file-browser.vala"
	gtk_tree_model_get_iter (model, &iter, tree_path);
#line 173 "file-browser.vala"
	gtk_tree_model_get (model, &iter, FILE_BROWSER_COL_PATH, &path, -1);
#line 175 "file-browser.vala"
	if (g_file_test (path, G_FILE_TEST_IS_DIR)) {
#line 177 "file-browser.vala"
		file_browser_set_cwd (self, path);
#line 178 "file-browser.vala"
		file_browser_populate (self, (GObject*) self);
#line 420 "file-browser.c"
	} else {
		{
#line 184 "file-browser.vala"
			if (valide_project_manager_accept_file (valide_window_get_projects (valide_plugin_get_window ((ValidePlugin*) self)), path)) {
#line 186 "file-browser.vala"
				valide_project_manager_open (valide_window_get_projects (valide_plugin_get_window ((ValidePlugin*) self)), path, &_inner_error_);
#line 427 "file-browser.c"
				if (_inner_error_ != NULL) {
					goto __catch2_g_error;
					goto __finally2;
				}
			} else {
				ValideiDocument* _tmp2_;
#line 499 "valide-0.0.vapi"
				_tmp2_ = NULL;
#line 190 "file-browser.vala"
				_tmp2_ = valide_document_manager_create (valide_window_get_documents (valide_plugin_get_window ((ValidePlugin*) self)), path, &_inner_error_);
#line 438 "file-browser.c"
				if (_inner_error_ != NULL) {
					goto __catch2_g_error;
					goto __finally2;
				}
				(_tmp2_ == NULL) ? NULL : (_tmp2_ = (g_object_unref (_tmp2_), NULL));
			}
		}
		goto __finally2;
		__catch2_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
#line 195 "file-browser.vala"
				g_warning (e->message);
#line 455 "file-browser.c"
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally2:
		if (_inner_error_ != NULL) {
			path = (g_free (path), NULL);
			(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
	path = (g_free (path), NULL);
	(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
}


#line 200 "file-browser.vala"
static gint file_browser_sort_func (FileBrowser* self, GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b) {
#line 475 "file-browser.c"
	gint result;
	char* path_a;
	char* path_b;
#line 200 "file-browser.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 200 "file-browser.vala"
	g_return_val_if_fail (model != NULL, 0);
#line 483 "file-browser.c"
	path_a = NULL;
	path_b = NULL;
#line 204 "file-browser.vala"
	gtk_tree_model_get (model, &(*a), FILE_BROWSER_COL_PATH, &path_a, -1);
#line 205 "file-browser.vala"
	gtk_tree_model_get (model, &(*b), FILE_BROWSER_COL_PATH, &path_b, -1);
#line 490 "file-browser.c"
	result = valide_utils_cmp_filename (path_a, path_b);
	path_a = (g_free (path_a), NULL);
	path_b = (g_free (path_b), NULL);
	return result;
}


#line 24 "file-browser.vala"
FileBrowser* file_browser_construct (GType object_type) {
#line 500 "file-browser.c"
	FileBrowser * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


#line 24 "file-browser.vala"
FileBrowser* file_browser_new (void) {
#line 24 "file-browser.vala"
	return file_browser_construct (TYPE_FILE_BROWSER);
#line 511 "file-browser.c"
}


static const char* file_browser_real_get_path (ValidePlugin* base) {
	const char* result;
	FileBrowser* self;
	self = (FileBrowser*) base;
	result = self->priv->_path;
	return result;
}


static void file_browser_real_set_path (ValidePlugin* base, const char* value) {
	FileBrowser* self;
	char* _tmp1_;
	const char* _tmp0_;
	self = (FileBrowser*) base;
	_tmp1_ = NULL;
#line 41 "file-browser.vala"
	_tmp0_ = NULL;
#line 532 "file-browser.c"
	self->priv->_path = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_path = (g_free (self->priv->_path), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "path");
}


static ValideWindow* file_browser_real_get_window (ValidePlugin* base) {
	ValideWindow* result;
	FileBrowser* self;
	self = (FileBrowser*) base;
	result = self->priv->_window;
	return result;
}


static void file_browser_real_set_window (ValidePlugin* base, ValideWindow* value) {
	FileBrowser* self;
	ValideWindow* _tmp1_;
	ValideWindow* _tmp0_;
	self = (FileBrowser*) base;
	_tmp1_ = NULL;
#line 42 "file-browser.vala"
	_tmp0_ = NULL;
#line 555 "file-browser.c"
	self->priv->_window = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "window");
}


static const char* file_browser_get_cwd (FileBrowser* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_cwd;
	return result;
}


static void file_browser_set_cwd (FileBrowser* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 52 "file-browser.vala"
	_tmp0_ = NULL;
#line 52 "file-browser.vala"
	self->priv->_cwd = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_cwd = (g_free (self->priv->_cwd), NULL), _tmp1_);
#line 53 "file-browser.vala"
	gtk_file_chooser_set_filename ((GtkFileChooser*) self->priv->file_chooser, value);
#line 54 "file-browser.vala"
	file_browser_populate (self, (GObject*) self);
#line 582 "file-browser.c"
}


static gboolean file_browser_get_show_hidden (FileBrowser* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = self->priv->_show_hidden;
	return result;
}


static void file_browser_set_show_hidden (FileBrowser* self, gboolean value) {
	g_return_if_fail (self != NULL);
#line 66 "file-browser.vala"
	self->priv->_show_hidden = value;
#line 67 "file-browser.vala"
	file_browser_populate (self, (GObject*) self);
#line 600 "file-browser.c"
}


static void _lambda1_ (FileBrowser* self) {
	char* _tmp0_;
#line 2032 "glib-2.0.vapi"
	_tmp0_ = NULL;
#line 236 "file-browser.vala"
	file_browser_set_cwd (self, _tmp0_ = g_path_get_dirname (file_browser_get_cwd (self)));
#line 610 "file-browser.c"
	_tmp0_ = (g_free (_tmp0_), NULL);
}


static void __lambda1__gtk_button_clicked (GtkButton* _sender, gpointer self) {
	_lambda1_ (self);
}


static void _lambda2_ (FileBrowser* self) {
#line 246 "file-browser.vala"
	file_browser_populate (self, (GObject*) self);
#line 623 "file-browser.c"
}


static void __lambda2__gtk_button_clicked (GtkButton* _sender, gpointer self) {
	_lambda2_ (self);
}


static void _lambda3_ (GtkToggleButton* s, FileBrowser* self) {
	g_return_if_fail (s != NULL);
#line 256 "file-browser.vala"
	file_browser_set_show_hidden (self, gtk_toggle_button_get_active (s));
#line 636 "file-browser.c"
}


static void __lambda3__gtk_toggle_button_toggled (GtkToggleButton* _sender, gpointer self) {
	_lambda3_ (_sender, self);
}


static void _lambda4_ (GtkFileChooser* s, FileBrowser* self) {
	char* _tmp0_;
	g_return_if_fail (s != NULL);
	/* Don't use the property for don't launch an infinite loop */
	_tmp0_ = NULL;
#line 264 "file-browser.vala"
	self->priv->_cwd = (_tmp0_ = gtk_file_chooser_get_filename (s), self->priv->_cwd = (g_free (self->priv->_cwd), NULL), _tmp0_);
#line 265 "file-browser.vala"
	file_browser_populate (self, (GObject*) s);
#line 654 "file-browser.c"
}


static void __lambda4__gtk_file_chooser_selection_changed (GtkFileChooser* _sender, gpointer self) {
	_lambda4_ (_sender, self);
}


#line 200 "file-browser.vala"
static gint _file_browser_sort_func_gtk_tree_iter_compare_func (GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b, gpointer self) {
#line 665 "file-browser.c"
	return file_browser_sort_func (self, model, a, b);
}


#line 164 "file-browser.vala"
static void _file_browser_on_row_activated_gtk_tree_view_row_activated (GtkTreeView* _sender, const GtkTreePath* path, GtkTreeViewColumn* column, gpointer self) {
#line 672 "file-browser.c"
	file_browser_on_row_activated (self, _sender, path, column);
}


static void _lambda5_ (GtkTreeView* s, GtkTreeIter* i, const GtkTreePath* p, FileBrowser* self) {
	g_return_if_fail (s != NULL);
	g_return_if_fail (p != NULL);
#line 305 "file-browser.vala"
	file_browser_populate_asc (self, &(*i));
#line 306 "file-browser.vala"
	gtk_tree_view_expand_row (self->priv->tree_view, p, FALSE);
#line 684 "file-browser.c"
}


static void __lambda5__gtk_tree_view_row_expanded (GtkTreeView* _sender, GtkTreeIter* iter, const GtkTreePath* path, gpointer self) {
	_lambda5_ (_sender, iter, path, self);
}


static GObject * file_browser_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	FileBrowserClass * klass;
	GObjectClass * parent_class;
	FileBrowser * self;
	klass = FILE_BROWSER_CLASS (g_type_class_peek (TYPE_FILE_BROWSER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = FILE_BROWSER (obj);
	{
		GtkImage* image;
		GtkButton* button;
		GtkHBox* button_box;
		GtkTreeStore* store;
		GtkTreeViewColumn* col;
		GtkCellRenderer* renderer;
		GtkScrolledWindow* scrolled;
		GtkToggleButton* toggle_button;
		GtkVBox* _tmp0_;
		GtkHBox* _tmp1_;
		GtkButton* _tmp2_;
		GtkImage* _tmp3_;
		GtkButton* _tmp4_;
		GtkImage* _tmp5_;
		GtkToggleButton* _tmp6_;
		GtkImage* _tmp7_;
		GtkFileChooserButton* _tmp8_;
		GtkScrolledWindow* _tmp9_;
		GtkTreeView* _tmp10_;
		GtkTreeStore* _tmp11_;
		GtkTreeViewColumn* _tmp12_;
		GtkCellRenderer* _tmp13_;
		GtkCellRenderer* _tmp14_;
		GtkImage* _tmp16_;
		char* _tmp15_;
		image = NULL;
		button = NULL;
		button_box = NULL;
		store = NULL;
		col = NULL;
		renderer = NULL;
		scrolled = NULL;
		toggle_button = NULL;
		_tmp0_ = NULL;
#line 221 "file-browser.vala"
		self->priv->box = (_tmp0_ = g_object_ref_sink ((GtkVBox*) gtk_vbox_new (FALSE, 0)), (self->priv->box == NULL) ? NULL : (self->priv->box = (g_object_unref (self->priv->box), NULL)), _tmp0_);
#line 739 "file-browser.c"
		/* create some nice padding */
#line 224 "file-browser.vala"
		gtk_box_set_spacing ((GtkBox*) self->priv->box, 2);
#line 225 "file-browser.vala"
		gtk_container_set_border_width ((GtkContainer*) self->priv->box, (guint) 2);
#line 745 "file-browser.c"
		_tmp1_ = NULL;
#line 227 "file-browser.vala"
		button_box = (_tmp1_ = g_object_ref_sink ((GtkHBox*) gtk_hbox_new (FALSE, 2)), (button_box == NULL) ? NULL : (button_box = (g_object_unref (button_box), NULL)), _tmp1_);
#line 228 "file-browser.vala"
		gtk_box_pack_start ((GtkBox*) self->priv->box, (GtkWidget*) button_box, FALSE, FALSE, (guint) 0);
#line 751 "file-browser.c"
		_tmp2_ = NULL;
#line 230 "file-browser.vala"
		button = (_tmp2_ = g_object_ref_sink ((GtkButton*) gtk_button_new ()), (button == NULL) ? NULL : (button = (g_object_unref (button), NULL)), _tmp2_);
#line 231 "file-browser.vala"
		gtk_button_set_relief (button, GTK_RELIEF_NONE);
#line 757 "file-browser.c"
		_tmp3_ = NULL;
#line 232 "file-browser.vala"
		gtk_button_set_image (button, (GtkWidget*) (_tmp3_ = g_object_ref_sink ((GtkImage*) gtk_image_new_from_stock (GTK_STOCK_GO_UP, GTK_ICON_SIZE_SMALL_TOOLBAR))));
#line 761 "file-browser.c"
		(_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL));
#line 234 "file-browser.vala"
		gtk_widget_set_tooltip_text ((GtkWidget*) button, _ ("Goto parent folder"));
#line 235 "file-browser.vala"
		g_signal_connect (button, "clicked", (GCallback) __lambda1__gtk_button_clicked, self);
#line 238 "file-browser.vala"
		gtk_box_pack_start ((GtkBox*) button_box, (GtkWidget*) button, FALSE, FALSE, (guint) 0);
#line 769 "file-browser.c"
		_tmp4_ = NULL;
#line 240 "file-browser.vala"
		button = (_tmp4_ = g_object_ref_sink ((GtkButton*) gtk_button_new ()), (button == NULL) ? NULL : (button = (g_object_unref (button), NULL)), _tmp4_);
#line 241 "file-browser.vala"
		gtk_button_set_relief (button, GTK_RELIEF_NONE);
#line 775 "file-browser.c"
		_tmp5_ = NULL;
#line 242 "file-browser.vala"
		gtk_button_set_image (button, (GtkWidget*) (_tmp5_ = g_object_ref_sink ((GtkImage*) gtk_image_new_from_stock (GTK_STOCK_REFRESH, GTK_ICON_SIZE_SMALL_TOOLBAR))));
#line 779 "file-browser.c"
		(_tmp5_ == NULL) ? NULL : (_tmp5_ = (g_object_unref (_tmp5_), NULL));
#line 244 "file-browser.vala"
		gtk_widget_set_tooltip_text ((GtkWidget*) button, _ ("Refresh"));
#line 245 "file-browser.vala"
		g_signal_connect (button, "clicked", (GCallback) __lambda2__gtk_button_clicked, self);
#line 248 "file-browser.vala"
		gtk_box_pack_start ((GtkBox*) button_box, (GtkWidget*) button, FALSE, FALSE, (guint) 0);
#line 787 "file-browser.c"
		_tmp6_ = NULL;
#line 250 "file-browser.vala"
		toggle_button = (_tmp6_ = g_object_ref_sink ((GtkToggleButton*) gtk_toggle_button_new ()), (toggle_button == NULL) ? NULL : (toggle_button = (g_object_unref (toggle_button), NULL)), _tmp6_);
#line 251 "file-browser.vala"
		gtk_button_set_relief ((GtkButton*) toggle_button, GTK_RELIEF_NONE);
#line 793 "file-browser.c"
		_tmp7_ = NULL;
#line 252 "file-browser.vala"
		gtk_button_set_image ((GtkButton*) toggle_button, (GtkWidget*) (_tmp7_ = g_object_ref_sink ((GtkImage*) gtk_image_new_from_stock (GTK_STOCK_FILE, GTK_ICON_SIZE_SMALL_TOOLBAR))));
#line 797 "file-browser.c"
		(_tmp7_ == NULL) ? NULL : (_tmp7_ = (g_object_unref (_tmp7_), NULL));
#line 254 "file-browser.vala"
		gtk_widget_set_tooltip_text ((GtkWidget*) toggle_button, _ ("Show the hidden files"));
#line 255 "file-browser.vala"
		g_signal_connect (toggle_button, "toggled", (GCallback) __lambda3__gtk_toggle_button_toggled, self);
#line 258 "file-browser.vala"
		gtk_box_pack_start ((GtkBox*) button_box, (GtkWidget*) toggle_button, FALSE, FALSE, (guint) 0);
#line 805 "file-browser.c"
		_tmp8_ = NULL;
#line 260 "file-browser.vala"
		self->priv->file_chooser = (_tmp8_ = g_object_ref_sink ((GtkFileChooserButton*) gtk_file_chooser_button_new ("Select folder", GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER)), (self->priv->file_chooser == NULL) ? NULL : (self->priv->file_chooser = (g_object_unref (self->priv->file_chooser), NULL)), _tmp8_);
#line 262 "file-browser.vala"
		g_signal_connect ((GtkFileChooser*) self->priv->file_chooser, "selection-changed", (GCallback) __lambda4__gtk_file_chooser_selection_changed, self);
#line 267 "file-browser.vala"
		gtk_box_pack_start ((GtkBox*) self->priv->box, (GtkWidget*) self->priv->file_chooser, FALSE, FALSE, (guint) 0);
#line 813 "file-browser.c"
		/* create and pack tree view */
		_tmp9_ = NULL;
#line 270 "file-browser.vala"
		scrolled = (_tmp9_ = g_object_ref_sink ((GtkScrolledWindow*) gtk_scrolled_window_new (NULL, NULL)), (scrolled == NULL) ? NULL : (scrolled = (g_object_unref (scrolled), NULL)), _tmp9_);
#line 271 "file-browser.vala"
		gtk_scrolled_window_set_policy (scrolled, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
#line 272 "file-browser.vala"
		gtk_scrolled_window_set_shadow_type (scrolled, GTK_SHADOW_IN);
#line 273 "file-browser.vala"
		gtk_box_pack_start ((GtkBox*) self->priv->box, (GtkWidget*) scrolled, TRUE, TRUE, (guint) 0);
#line 824 "file-browser.c"
		_tmp10_ = NULL;
#line 275 "file-browser.vala"
		self->priv->tree_view = (_tmp10_ = g_object_ref_sink ((GtkTreeView*) gtk_tree_view_new ()), (self->priv->tree_view == NULL) ? NULL : (self->priv->tree_view = (g_object_unref (self->priv->tree_view), NULL)), _tmp10_);
#line 276 "file-browser.vala"
		gtk_tree_view_set_rules_hint (self->priv->tree_view, FALSE);
#line 277 "file-browser.vala"
		gtk_tree_view_set_headers_visible (self->priv->tree_view, FALSE);
#line 278 "file-browser.vala"
		gtk_container_add ((GtkContainer*) scrolled, (GtkWidget*) self->priv->tree_view);
#line 834 "file-browser.c"
		/* create model */
		_tmp11_ = NULL;
#line 281 "file-browser.vala"
		store = (_tmp11_ = gtk_tree_store_new ((gint) FILE_BROWSER_COL_NB, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, NULL), (store == NULL) ? NULL : (store = (g_object_unref (store), NULL)), _tmp11_);
#line 283 "file-browser.vala"
		gtk_tree_sortable_set_sort_column_id ((GtkTreeSortable*) store, (gint) FILE_BROWSER_COL_NAME, GTK_SORT_ASCENDING);
#line 284 "file-browser.vala"
		gtk_tree_sortable_set_sort_func ((GtkTreeSortable*) store, (gint) FILE_BROWSER_COL_NAME, _file_browser_sort_func_gtk_tree_iter_compare_func, g_object_ref (self), g_object_unref);
#line 843 "file-browser.c"
		/* main column */
		_tmp12_ = NULL;
#line 286 "file-browser.vala"
		col = (_tmp12_ = g_object_ref_sink (gtk_tree_view_column_new ()), (col == NULL) ? NULL : (col = (g_object_unref (col), NULL)), _tmp12_);
#line 287 "file-browser.vala"
		gtk_tree_view_column_set_title (col, _ ("File"));
#line 850 "file-browser.c"
		_tmp13_ = NULL;
#line 289 "file-browser.vala"
		renderer = (_tmp13_ = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererPixbuf*) gtk_cell_renderer_pixbuf_new ()), (renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL)), _tmp13_);
#line 290 "file-browser.vala"
		gtk_cell_layout_pack_start ((GtkCellLayout*) col, renderer, FALSE);
#line 291 "file-browser.vala"
		gtk_tree_view_column_set_attributes (col, renderer, "pixbuf", FILE_BROWSER_COL_ICON, NULL);
#line 858 "file-browser.c"
		_tmp14_ = NULL;
#line 293 "file-browser.vala"
		renderer = (_tmp14_ = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL)), _tmp14_);
#line 294 "file-browser.vala"
		gtk_cell_layout_pack_start ((GtkCellLayout*) col, renderer, FALSE);
#line 295 "file-browser.vala"
		gtk_tree_view_column_set_attributes (col, renderer, "markup", FILE_BROWSER_COL_NAME, NULL);
#line 297 "file-browser.vala"
		gtk_tree_view_append_column (self->priv->tree_view, col);
#line 298 "file-browser.vala"
		gtk_tree_view_set_model (self->priv->tree_view, (GtkTreeModel*) store);
#line 300 "file-browser.vala"
		file_browser_set_cwd (self, g_get_home_dir ());
#line 872 "file-browser.c"
		/* attach signal callbacks for tree view */
#line 303 "file-browser.vala"
		g_signal_connect_object (self->priv->tree_view, "row-activated", (GCallback) _file_browser_on_row_activated_gtk_tree_view_row_activated, self, 0);
#line 304 "file-browser.vala"
		g_signal_connect (self->priv->tree_view, "row-expanded", (GCallback) __lambda5__gtk_tree_view_row_expanded, self);
#line 878 "file-browser.c"
		_tmp16_ = NULL;
#line 2034 "glib-2.0.vapi"
		_tmp15_ = NULL;
#line 309 "file-browser.vala"
		image = (_tmp16_ = g_object_ref_sink ((GtkImage*) gtk_image_new_from_file (_tmp15_ = g_build_filename (valide_plugin_get_path ((ValidePlugin*) self), "file-browser.png", NULL))), (image == NULL) ? NULL : (image = (g_object_unref (image), NULL)), _tmp16_);
#line 884 "file-browser.c"
		_tmp15_ = (g_free (_tmp15_), NULL);
#line 312 "file-browser.vala"
		valide_panel_add_item (valide_window_get_side_panel (valide_plugin_get_window ((ValidePlugin*) self)), (GtkWidget*) self->priv->box, _ ("File Browser"), image);
#line 314 "file-browser.vala"
		gtk_widget_show_all ((GtkWidget*) self->priv->box);
#line 890 "file-browser.c"
		(image == NULL) ? NULL : (image = (g_object_unref (image), NULL));
		(button == NULL) ? NULL : (button = (g_object_unref (button), NULL));
		(button_box == NULL) ? NULL : (button_box = (g_object_unref (button_box), NULL));
		(store == NULL) ? NULL : (store = (g_object_unref (store), NULL));
		(col == NULL) ? NULL : (col = (g_object_unref (col), NULL));
		(renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL));
		(scrolled == NULL) ? NULL : (scrolled = (g_object_unref (scrolled), NULL));
		(toggle_button == NULL) ? NULL : (toggle_button = (g_object_unref (toggle_button), NULL));
	}
	return obj;
}


static void file_browser_class_init (FileBrowserClass * klass) {
	file_browser_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (FileBrowserPrivate));
	G_OBJECT_CLASS (klass)->get_property = file_browser_get_property;
	G_OBJECT_CLASS (klass)->set_property = file_browser_set_property;
	G_OBJECT_CLASS (klass)->constructor = file_browser_constructor;
	G_OBJECT_CLASS (klass)->finalize = file_browser_finalize;
	g_object_class_override_property (G_OBJECT_CLASS (klass), FILE_BROWSER_PATH, "path");
	g_object_class_override_property (G_OBJECT_CLASS (klass), FILE_BROWSER_WINDOW, "window");
}


static void file_browser_valide_plugin_interface_init (ValidePluginIface * iface) {
	file_browser_valide_plugin_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_path = file_browser_real_get_path;
	iface->set_path = file_browser_real_set_path;
	iface->get_window = file_browser_real_get_window;
	iface->set_window = file_browser_real_set_window;
}


static void file_browser_instance_init (FileBrowser * self) {
	self->priv = FILE_BROWSER_GET_PRIVATE (self);
}


static void file_browser_finalize (GObject* obj) {
	FileBrowser * self;
	self = FILE_BROWSER (obj);
	{
#line 319 "file-browser.vala"
		valide_panel_remove_item (valide_window_get_side_panel (valide_plugin_get_window ((ValidePlugin*) self)), (GtkWidget*) self->priv->box);
#line 936 "file-browser.c"
	}
	self->priv->_cwd = (g_free (self->priv->_cwd), NULL);
	(self->priv->box == NULL) ? NULL : (self->priv->box = (g_object_unref (self->priv->box), NULL));
	(self->priv->tree_view == NULL) ? NULL : (self->priv->tree_view = (g_object_unref (self->priv->tree_view), NULL));
	(self->priv->file_chooser == NULL) ? NULL : (self->priv->file_chooser = (g_object_unref (self->priv->file_chooser), NULL));
	self->priv->_path = (g_free (self->priv->_path), NULL);
	(self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL));
	G_OBJECT_CLASS (file_browser_parent_class)->finalize (obj);
}


GType file_browser_get_type (void) {
	static GType file_browser_type_id = 0;
	if (file_browser_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (FileBrowserClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) file_browser_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (FileBrowser), 0, (GInstanceInitFunc) file_browser_instance_init, NULL };
		static const GInterfaceInfo valide_plugin_info = { (GInterfaceInitFunc) file_browser_valide_plugin_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		file_browser_type_id = g_type_register_static (G_TYPE_OBJECT, "FileBrowser", &g_define_type_info, 0);
		g_type_add_interface_static (file_browser_type_id, VALIDE_TYPE_PLUGIN, &valide_plugin_info);
	}
	return file_browser_type_id;
}


static void file_browser_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	FileBrowser * self;
	gpointer boxed;
	self = FILE_BROWSER (object);
	switch (property_id) {
		case FILE_BROWSER_PATH:
		g_value_set_string (value, valide_plugin_get_path ((ValidePlugin*) self));
		break;
		case FILE_BROWSER_WINDOW:
		g_value_set_object (value, valide_plugin_get_window ((ValidePlugin*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void file_browser_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	FileBrowser * self;
	self = FILE_BROWSER (object);
	switch (property_id) {
		case FILE_BROWSER_PATH:
		valide_plugin_set_path ((ValidePlugin*) self, g_value_get_string (value));
		break;
		case FILE_BROWSER_WINDOW:
		valide_plugin_set_window ((ValidePlugin*) self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


#line 324 "file-browser.vala"
GType register_plugin (void) {
#line 997 "file-browser.c"
	GType result;
	result = TYPE_FILE_BROWSER;
	return result;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




