/* make-builder.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <valide-0.0.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>


#define TYPE_MAKE_BUILDER (make_builder_get_type ())
#define MAKE_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MAKE_BUILDER, MakeBuilder))
#define MAKE_BUILDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MAKE_BUILDER, MakeBuilderClass))
#define IS_MAKE_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MAKE_BUILDER))
#define IS_MAKE_BUILDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MAKE_BUILDER))
#define MAKE_BUILDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MAKE_BUILDER, MakeBuilderClass))

typedef struct _MakeBuilder MakeBuilder;
typedef struct _MakeBuilderClass MakeBuilderClass;
typedef struct _MakeBuilderPrivate MakeBuilderPrivate;

#define TYPE_MAKE_EXECUTABLE (make_executable_get_type ())
#define MAKE_EXECUTABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MAKE_EXECUTABLE, MakeExecutable))
#define MAKE_EXECUTABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MAKE_EXECUTABLE, MakeExecutableClass))
#define IS_MAKE_EXECUTABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MAKE_EXECUTABLE))
#define IS_MAKE_EXECUTABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MAKE_EXECUTABLE))
#define MAKE_EXECUTABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MAKE_EXECUTABLE, MakeExecutableClass))

typedef struct _MakeExecutable MakeExecutable;
typedef struct _MakeExecutableClass MakeExecutableClass;

#define VALIDE_TYPE_MAKE_BUILDER_OPTIONS (valide_make_builder_options_get_type ())
#define VALIDE_MAKE_BUILDER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_MAKE_BUILDER_OPTIONS, ValideMakeBuilderOptions))
#define VALIDE_MAKE_BUILDER_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_MAKE_BUILDER_OPTIONS, ValideMakeBuilderOptionsClass))
#define VALIDE_IS_MAKE_BUILDER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_MAKE_BUILDER_OPTIONS))
#define VALIDE_IS_MAKE_BUILDER_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_MAKE_BUILDER_OPTIONS))
#define VALIDE_MAKE_BUILDER_OPTIONS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_MAKE_BUILDER_OPTIONS, ValideMakeBuilderOptionsClass))

typedef struct _ValideMakeBuilderOptions ValideMakeBuilderOptions;
typedef struct _ValideMakeBuilderOptionsClass ValideMakeBuilderOptionsClass;

#define TYPE_CONFIGURE_EXECUTABLE (configure_executable_get_type ())
#define CONFIGURE_EXECUTABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CONFIGURE_EXECUTABLE, ConfigureExecutable))
#define CONFIGURE_EXECUTABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CONFIGURE_EXECUTABLE, ConfigureExecutableClass))
#define IS_CONFIGURE_EXECUTABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CONFIGURE_EXECUTABLE))
#define IS_CONFIGURE_EXECUTABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CONFIGURE_EXECUTABLE))
#define CONFIGURE_EXECUTABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CONFIGURE_EXECUTABLE, ConfigureExecutableClass))

typedef struct _ConfigureExecutable ConfigureExecutable;
typedef struct _ConfigureExecutableClass ConfigureExecutableClass;

#define TYPE_MAKE_BUILDER_PREFERENCES (make_builder_preferences_get_type ())
#define MAKE_BUILDER_PREFERENCES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MAKE_BUILDER_PREFERENCES, MakeBuilderPreferences))
#define MAKE_BUILDER_PREFERENCES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MAKE_BUILDER_PREFERENCES, MakeBuilderPreferencesClass))
#define IS_MAKE_BUILDER_PREFERENCES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MAKE_BUILDER_PREFERENCES))
#define IS_MAKE_BUILDER_PREFERENCES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MAKE_BUILDER_PREFERENCES))
#define MAKE_BUILDER_PREFERENCES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MAKE_BUILDER_PREFERENCES, MakeBuilderPreferencesClass))

typedef struct _MakeBuilderPreferences MakeBuilderPreferences;
typedef struct _MakeBuilderPreferencesClass MakeBuilderPreferencesClass;

/**
 * Builder based on make: http://www.gnu.org/software/make/
 */
struct _MakeBuilder {
	GObject parent_instance;
	MakeBuilderPrivate * priv;
};

struct _MakeBuilderClass {
	GObjectClass parent_class;
};

struct _MakeBuilderPrivate {
	ValideProject* _project;
	ValideExecutableManager* _executables;
	char* _path;
	ValideWindow* _window;
};


static gpointer make_builder_parent_class = NULL;
static ValidePluginIface* make_builder_valide_plugin_parent_iface = NULL;
static ValideBuilderIface* make_builder_valide_builder_parent_iface = NULL;

GType make_builder_get_type (void);
#define MAKE_BUILDER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_MAKE_BUILDER, MakeBuilderPrivate))
enum  {
	MAKE_BUILDER_DUMMY_PROPERTY,
	MAKE_BUILDER_PROJECT,
	MAKE_BUILDER_EXECUTABLES,
	MAKE_BUILDER_PATH,
	MAKE_BUILDER_WINDOW
};
GType make_executable_get_type (void);
GType valide_make_builder_options_get_type (void);
MakeExecutable* make_executable_new (ValideProject* project);
MakeExecutable* make_executable_construct (GType object_type, ValideProject* project);
char* valide_make_builder_options_to_string (ValideMakeBuilderOptions* self, BuildingStep step);
static void make_builder_run_make (MakeBuilder* self, BuildingStep step);
MakeBuilder* make_builder_new (ValideExecutableManager* executables, ValideProject* project);
MakeBuilder* make_builder_construct (GType object_type, ValideExecutableManager* executables, ValideProject* project);
GType configure_executable_get_type (void);
ConfigureExecutable* configure_executable_new (void);
ConfigureExecutable* configure_executable_construct (GType object_type);
static void make_builder_real_configure (ValideBuilder* base);
static void make_builder_real_build (ValideBuilder* base);
static void make_builder_real_install (ValideBuilder* base);
static void make_builder_real_dist (ValideBuilder* base);
static void make_builder_real_clean (ValideBuilder* base);
static void make_builder_real_distclean (ValideBuilder* base);
static void make_builder_real_uninstall (ValideBuilder* base);
MakeBuilderPreferences* make_builder_preferences_new (ValideProject* project);
MakeBuilderPreferences* make_builder_preferences_construct (GType object_type, ValideProject* project);
GType make_builder_preferences_get_type (void);
static GtkWidget* make_builder_real_preferences (ValideBuilder* base);
static GObject * make_builder_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void make_builder_finalize (GObject* obj);
static void make_builder_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void make_builder_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType register_plugin (void);



#line 49 "make-builder.vala"
static void make_builder_run_make (MakeBuilder* self, BuildingStep step) {
#line 147 "make-builder.c"
	MakeExecutable* make;
	ValideMakeBuilderOptions* options;
	ValideExecutableOptions* executable_options;
	MakeExecutable* _tmp0_;
	ValideMakeBuilderOptions* _tmp3_;
	ValideMakeBuilderOptions* _tmp2_;
	ValideBuilderOptions* _tmp1_;
	ValideExecutableOptions* _tmp4_;
	char* _tmp5_;
#line 49 "make-builder.vala"
	g_return_if_fail (self != NULL);
#line 159 "make-builder.c"
	make = NULL;
	options = NULL;
	executable_options = NULL;
	_tmp0_ = NULL;
#line 55 "make-builder.vala"
	make = (_tmp0_ = make_executable_new (valide_builder_get_project ((ValideBuilder*) self)), (make == NULL) ? NULL : (make = (g_object_unref (make), NULL)), _tmp0_);
#line 166 "make-builder.c"
	_tmp3_ = NULL;
#line 56 "make-builder.vala"
	_tmp2_ = NULL;
#line 56 "make-builder.vala"
	_tmp1_ = NULL;
#line 56 "make-builder.vala"
	options = (_tmp3_ = (_tmp2_ = (_tmp1_ = valide_project_get_builder_options (valide_builder_get_project ((ValideBuilder*) self)), VALIDE_IS_MAKE_BUILDER_OPTIONS (_tmp1_) ? ((ValideMakeBuilderOptions*) _tmp1_) : NULL), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (options == NULL) ? NULL : (options = (g_object_unref (options), NULL)), _tmp3_);
#line 174 "make-builder.c"
	_tmp4_ = NULL;
#line 57 "make-builder.vala"
	executable_options = (_tmp4_ = valide_executable_options_new (), (executable_options == NULL) ? NULL : (executable_options = (g_object_unref (executable_options), NULL)), _tmp4_);
#line 58 "make-builder.vala"
	valide_executable_options_set_working_dir (executable_options, valide_project_get_path (valide_builder_get_project ((ValideBuilder*) self)));
#line 38 "make-options.vala"
	_tmp5_ = NULL;
#line 59 "make-builder.vala"
	valide_executable_options_set_arguments (executable_options, _tmp5_ = valide_make_builder_options_to_string (options, step));
#line 184 "make-builder.c"
	_tmp5_ = (g_free (_tmp5_), NULL);
#line 60 "make-builder.vala"
	valide_executable_manager_run (valide_builder_get_executables ((ValideBuilder*) self), (ValideExecutable*) make, executable_options);
#line 188 "make-builder.c"
	(make == NULL) ? NULL : (make = (g_object_unref (make), NULL));
	(options == NULL) ? NULL : (options = (g_object_unref (options), NULL));
	(executable_options == NULL) ? NULL : (executable_options = (g_object_unref (executable_options), NULL));
}


#line 63 "make-builder.vala"
MakeBuilder* make_builder_construct (GType object_type, ValideExecutableManager* executables, ValideProject* project) {
#line 197 "make-builder.c"
	GParameter * __params;
	GParameter * __params_it;
	MakeBuilder * self;
#line 63 "make-builder.vala"
	g_return_val_if_fail (executables != NULL, NULL);
#line 63 "make-builder.vala"
	g_return_val_if_fail (project != NULL, NULL);
#line 205 "make-builder.c"
	__params = g_new0 (GParameter, 2);
	__params_it = __params;
#line 65 "make-builder.vala"
	__params_it->name = "project";
#line 65 "make-builder.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_PROJECT);
#line 65 "make-builder.vala"
	g_value_set_object (&__params_it->value, project);
#line 65 "make-builder.vala"
	__params_it++;
#line 66 "make-builder.vala"
	__params_it->name = "executables";
#line 66 "make-builder.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_EXECUTABLE_MANAGER);
#line 66 "make-builder.vala"
	g_value_set_object (&__params_it->value, executables);
#line 66 "make-builder.vala"
	__params_it++;
#line 224 "make-builder.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 63 "make-builder.vala"
MakeBuilder* make_builder_new (ValideExecutableManager* executables, ValideProject* project) {
#line 63 "make-builder.vala"
	return make_builder_construct (TYPE_MAKE_BUILDER, executables, project);
#line 239 "make-builder.c"
}


/**
   * @see Valide.Builder.configure
   */
#line 78 "make-builder.vala"
static void make_builder_real_configure (ValideBuilder* base) {
#line 248 "make-builder.c"
	MakeBuilder * self;
	ValideMakeBuilderOptions* options;
	ConfigureExecutable* configure;
	ValideExecutableOptions* executable_options;
	ConfigureExecutable* _tmp0_;
	ValideMakeBuilderOptions* _tmp3_;
	ValideMakeBuilderOptions* _tmp2_;
	ValideBuilderOptions* _tmp1_;
	ValideExecutableOptions* _tmp4_;
	char* _tmp5_;
	self = (MakeBuilder*) base;
	options = NULL;
	configure = NULL;
	executable_options = NULL;
	_tmp0_ = NULL;
#line 84 "make-builder.vala"
	configure = (_tmp0_ = configure_executable_new (), (configure == NULL) ? NULL : (configure = (g_object_unref (configure), NULL)), _tmp0_);
#line 266 "make-builder.c"
	_tmp3_ = NULL;
#line 85 "make-builder.vala"
	_tmp2_ = NULL;
#line 85 "make-builder.vala"
	_tmp1_ = NULL;
#line 85 "make-builder.vala"
	options = (_tmp3_ = (_tmp2_ = (_tmp1_ = valide_project_get_builder_options (valide_builder_get_project ((ValideBuilder*) self)), VALIDE_IS_MAKE_BUILDER_OPTIONS (_tmp1_) ? ((ValideMakeBuilderOptions*) _tmp1_) : NULL), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (options == NULL) ? NULL : (options = (g_object_unref (options), NULL)), _tmp3_);
#line 274 "make-builder.c"
	_tmp4_ = NULL;
#line 86 "make-builder.vala"
	executable_options = (_tmp4_ = valide_executable_options_new (), (executable_options == NULL) ? NULL : (executable_options = (g_object_unref (executable_options), NULL)), _tmp4_);
#line 87 "make-builder.vala"
	valide_executable_options_set_working_dir (executable_options, valide_project_get_path (valide_builder_get_project ((ValideBuilder*) self)));
#line 38 "make-options.vala"
	_tmp5_ = NULL;
#line 88 "make-builder.vala"
	valide_executable_options_set_arguments (executable_options, _tmp5_ = valide_make_builder_options_to_string (options, BUILDING_STEP_CONFIGURE));
#line 284 "make-builder.c"
	_tmp5_ = (g_free (_tmp5_), NULL);
#line 89 "make-builder.vala"
	valide_executable_manager_run (valide_builder_get_executables ((ValideBuilder*) self), (ValideExecutable*) configure, executable_options);
#line 288 "make-builder.c"
	(options == NULL) ? NULL : (options = (g_object_unref (options), NULL));
	(configure == NULL) ? NULL : (configure = (g_object_unref (configure), NULL));
	(executable_options == NULL) ? NULL : (executable_options = (g_object_unref (executable_options), NULL));
}


/**
   * @see Valide.Builder.build
   */
#line 95 "make-builder.vala"
static void make_builder_real_build (ValideBuilder* base) {
#line 300 "make-builder.c"
	MakeBuilder * self;
	self = (MakeBuilder*) base;
#line 97 "make-builder.vala"
	make_builder_run_make (self, BUILDING_STEP_BUILD);
#line 305 "make-builder.c"
}


/**
   * @see Valide.Builder.install
   */
#line 103 "make-builder.vala"
static void make_builder_real_install (ValideBuilder* base) {
#line 314 "make-builder.c"
	MakeBuilder * self;
	self = (MakeBuilder*) base;
#line 105 "make-builder.vala"
	make_builder_run_make (self, BUILDING_STEP_INSTALL);
#line 319 "make-builder.c"
}


/**
   * @see Valide.Builder.dist
   */
#line 111 "make-builder.vala"
static void make_builder_real_dist (ValideBuilder* base) {
#line 328 "make-builder.c"
	MakeBuilder * self;
	self = (MakeBuilder*) base;
#line 113 "make-builder.vala"
	make_builder_run_make (self, BUILDING_STEP_DIST);
#line 333 "make-builder.c"
}


/**
   * @see Valide.Builder.clean
   */
#line 119 "make-builder.vala"
static void make_builder_real_clean (ValideBuilder* base) {
#line 342 "make-builder.c"
	MakeBuilder * self;
	self = (MakeBuilder*) base;
#line 121 "make-builder.vala"
	make_builder_run_make (self, BUILDING_STEP_CLEAN);
#line 347 "make-builder.c"
}


/**
   * @see Valide.Builder.distclean
   */
#line 127 "make-builder.vala"
static void make_builder_real_distclean (ValideBuilder* base) {
#line 356 "make-builder.c"
	MakeBuilder * self;
	self = (MakeBuilder*) base;
#line 129 "make-builder.vala"
	make_builder_run_make (self, BUILDING_STEP_DISTCLEAN);
#line 361 "make-builder.c"
}


/**
   * @see Valide.Builder.uninstall
   */
#line 135 "make-builder.vala"
static void make_builder_real_uninstall (ValideBuilder* base) {
#line 370 "make-builder.c"
	MakeBuilder * self;
	self = (MakeBuilder*) base;
#line 137 "make-builder.vala"
	make_builder_run_make (self, BUILDING_STEP_UNINSTALL);
#line 375 "make-builder.c"
}


/**
   * @see Valide.Builder.preferences
   */
#line 143 "make-builder.vala"
static GtkWidget* make_builder_real_preferences (ValideBuilder* base) {
#line 384 "make-builder.c"
	MakeBuilder * self;
	GtkWidget* result;
	self = (MakeBuilder*) base;
	result = (GtkWidget*) g_object_ref_sink (make_builder_preferences_new (valide_builder_get_project ((ValideBuilder*) self)));
	return result;
}


static ValideProject* make_builder_real_get_project (ValideBuilder* base) {
	ValideProject* result;
	MakeBuilder* self;
	self = (MakeBuilder*) base;
	result = self->priv->_project;
	return result;
}


static void make_builder_real_set_project (ValideBuilder* base, ValideProject* value) {
	MakeBuilder* self;
	ValideProject* _tmp1_;
	ValideProject* _tmp0_;
	self = (MakeBuilder*) base;
	_tmp1_ = NULL;
#line 32 "make-builder.vala"
	_tmp0_ = NULL;
#line 410 "make-builder.c"
	self->priv->_project = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_project == NULL) ? NULL : (self->priv->_project = (g_object_unref (self->priv->_project), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "project");
}


static ValideExecutableManager* make_builder_real_get_executables (ValideBuilder* base) {
	ValideExecutableManager* result;
	MakeBuilder* self;
	self = (MakeBuilder*) base;
	result = self->priv->_executables;
	return result;
}


static void make_builder_real_set_executables (ValideBuilder* base, ValideExecutableManager* value) {
	MakeBuilder* self;
	ValideExecutableManager* _tmp1_;
	ValideExecutableManager* _tmp0_;
	self = (MakeBuilder*) base;
	_tmp1_ = NULL;
#line 37 "make-builder.vala"
	_tmp0_ = NULL;
#line 433 "make-builder.c"
	self->priv->_executables = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_executables == NULL) ? NULL : (self->priv->_executables = (g_object_unref (self->priv->_executables), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "executables");
}


static const char* make_builder_real_get_path (ValidePlugin* base) {
	const char* result;
	MakeBuilder* self;
	self = (MakeBuilder*) base;
	result = self->priv->_path;
	return result;
}


static void make_builder_real_set_path (ValidePlugin* base, const char* value) {
	MakeBuilder* self;
	char* _tmp1_;
	const char* _tmp0_;
	self = (MakeBuilder*) base;
	_tmp1_ = NULL;
#line 42 "make-builder.vala"
	_tmp0_ = NULL;
#line 456 "make-builder.c"
	self->priv->_path = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_path = (g_free (self->priv->_path), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "path");
}


static ValideWindow* make_builder_real_get_window (ValidePlugin* base) {
	ValideWindow* result;
	MakeBuilder* self;
	self = (MakeBuilder*) base;
	result = self->priv->_window;
	return result;
}


static void make_builder_real_set_window (ValidePlugin* base, ValideWindow* value) {
	MakeBuilder* self;
	ValideWindow* _tmp1_;
	ValideWindow* _tmp0_;
	self = (MakeBuilder*) base;
	_tmp1_ = NULL;
#line 47 "make-builder.vala"
	_tmp0_ = NULL;
#line 479 "make-builder.c"
	self->priv->_window = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "window");
}


static GObject * make_builder_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	MakeBuilderClass * klass;
	GObjectClass * parent_class;
	MakeBuilder * self;
	klass = MAKE_BUILDER_CLASS (g_type_class_peek (TYPE_MAKE_BUILDER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = MAKE_BUILDER (obj);
	{
		ValideBuilderManager* _tmp0_;
#line 465 "valide-0.0.vapi"
		_tmp0_ = NULL;
#line 71 "make-builder.vala"
		valide_builder_manager_register_builder (_tmp0_ = valide_builder_manager_get_instance (), G_TYPE_FROM_INSTANCE ((GObject*) self), VALIDE_TYPE_MAKE_BUILDER_OPTIONS);
#line 500 "make-builder.c"
		(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
	}
	return obj;
}


static void make_builder_class_init (MakeBuilderClass * klass) {
	make_builder_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (MakeBuilderPrivate));
	G_OBJECT_CLASS (klass)->get_property = make_builder_get_property;
	G_OBJECT_CLASS (klass)->set_property = make_builder_set_property;
	G_OBJECT_CLASS (klass)->constructor = make_builder_constructor;
	G_OBJECT_CLASS (klass)->finalize = make_builder_finalize;
	g_object_class_override_property (G_OBJECT_CLASS (klass), MAKE_BUILDER_PROJECT, "project");
	g_object_class_override_property (G_OBJECT_CLASS (klass), MAKE_BUILDER_EXECUTABLES, "executables");
	g_object_class_override_property (G_OBJECT_CLASS (klass), MAKE_BUILDER_PATH, "path");
	g_object_class_override_property (G_OBJECT_CLASS (klass), MAKE_BUILDER_WINDOW, "window");
}


static void make_builder_valide_plugin_interface_init (ValidePluginIface * iface) {
	make_builder_valide_plugin_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_path = make_builder_real_get_path;
	iface->set_path = make_builder_real_set_path;
	iface->get_window = make_builder_real_get_window;
	iface->set_window = make_builder_real_set_window;
}


static void make_builder_valide_builder_interface_init (ValideBuilderIface * iface) {
	make_builder_valide_builder_parent_iface = g_type_interface_peek_parent (iface);
	iface->configure = make_builder_real_configure;
	iface->build = make_builder_real_build;
	iface->install = make_builder_real_install;
	iface->dist = make_builder_real_dist;
	iface->clean = make_builder_real_clean;
	iface->distclean = make_builder_real_distclean;
	iface->uninstall = make_builder_real_uninstall;
	iface->preferences = make_builder_real_preferences;
	iface->get_project = make_builder_real_get_project;
	iface->set_project = make_builder_real_set_project;
	iface->get_executables = make_builder_real_get_executables;
	iface->set_executables = make_builder_real_set_executables;
}


static void make_builder_instance_init (MakeBuilder * self) {
	self->priv = MAKE_BUILDER_GET_PRIVATE (self);
}


static void make_builder_finalize (GObject* obj) {
	MakeBuilder * self;
	self = MAKE_BUILDER (obj);
	(self->priv->_project == NULL) ? NULL : (self->priv->_project = (g_object_unref (self->priv->_project), NULL));
	(self->priv->_executables == NULL) ? NULL : (self->priv->_executables = (g_object_unref (self->priv->_executables), NULL));
	self->priv->_path = (g_free (self->priv->_path), NULL);
	(self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL));
	G_OBJECT_CLASS (make_builder_parent_class)->finalize (obj);
}


GType make_builder_get_type (void) {
	static GType make_builder_type_id = 0;
	if (make_builder_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (MakeBuilderClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) make_builder_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (MakeBuilder), 0, (GInstanceInitFunc) make_builder_instance_init, NULL };
		static const GInterfaceInfo valide_plugin_info = { (GInterfaceInitFunc) make_builder_valide_plugin_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo valide_builder_info = { (GInterfaceInitFunc) make_builder_valide_builder_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		make_builder_type_id = g_type_register_static (G_TYPE_OBJECT, "MakeBuilder", &g_define_type_info, 0);
		g_type_add_interface_static (make_builder_type_id, VALIDE_TYPE_PLUGIN, &valide_plugin_info);
		g_type_add_interface_static (make_builder_type_id, VALIDE_TYPE_BUILDER, &valide_builder_info);
	}
	return make_builder_type_id;
}


static void make_builder_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	MakeBuilder * self;
	gpointer boxed;
	self = MAKE_BUILDER (object);
	switch (property_id) {
		case MAKE_BUILDER_PROJECT:
		g_value_set_object (value, valide_builder_get_project ((ValideBuilder*) self));
		break;
		case MAKE_BUILDER_EXECUTABLES:
		g_value_set_object (value, valide_builder_get_executables ((ValideBuilder*) self));
		break;
		case MAKE_BUILDER_PATH:
		g_value_set_string (value, valide_plugin_get_path ((ValidePlugin*) self));
		break;
		case MAKE_BUILDER_WINDOW:
		g_value_set_object (value, valide_plugin_get_window ((ValidePlugin*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void make_builder_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	MakeBuilder * self;
	self = MAKE_BUILDER (object);
	switch (property_id) {
		case MAKE_BUILDER_PROJECT:
		valide_builder_set_project ((ValideBuilder*) self, g_value_get_object (value));
		break;
		case MAKE_BUILDER_EXECUTABLES:
		valide_builder_set_executables ((ValideBuilder*) self, g_value_get_object (value));
		break;
		case MAKE_BUILDER_PATH:
		valide_plugin_set_path ((ValidePlugin*) self, g_value_get_string (value));
		break;
		case MAKE_BUILDER_WINDOW:
		valide_plugin_set_window ((ValidePlugin*) self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


#line 150 "make-builder.vala"
GType register_plugin (void) {
#line 626 "make-builder.c"
	GType result;
	result = TYPE_MAKE_BUILDER;
	return result;
}




