/* make-executable.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <valide-0.0.h>
#include <gdk/gdk.h>
#include <stdlib.h>
#include <string.h>


#define TYPE_MAKE_EXECUTABLE (make_executable_get_type ())
#define MAKE_EXECUTABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MAKE_EXECUTABLE, MakeExecutable))
#define MAKE_EXECUTABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MAKE_EXECUTABLE, MakeExecutableClass))
#define IS_MAKE_EXECUTABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MAKE_EXECUTABLE))
#define IS_MAKE_EXECUTABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MAKE_EXECUTABLE))
#define MAKE_EXECUTABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MAKE_EXECUTABLE, MakeExecutableClass))

typedef struct _MakeExecutable MakeExecutable;
typedef struct _MakeExecutableClass MakeExecutableClass;
typedef struct _MakeExecutablePrivate MakeExecutablePrivate;

/**
 * The vala compiler
 */
struct _MakeExecutable {
	ValideCompiler parent_instance;
	MakeExecutablePrivate * priv;
};

struct _MakeExecutableClass {
	ValideCompilerClass parent_class;
};


static gpointer make_executable_parent_class = NULL;

GType make_executable_get_type (void);
enum  {
	MAKE_EXECUTABLE_DUMMY_PROPERTY,
	MAKE_EXECUTABLE_EXECUTABLE
};
#define MAKE_EXECUTABLE_EXECUTABLE_PATH "make"
static GdkColor make_executable_real_get_color_line (ValideExecutable* base, char** line);
MakeExecutable* make_executable_new (ValideProject* project);
MakeExecutable* make_executable_construct (GType object_type, ValideProject* project);
static void make_executable_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void make_executable_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);



/**
   * @see Valide.Executable.get_color_line
   */
#line 42 "make-executable.vala"
static GdkColor make_executable_real_get_color_line (ValideExecutable* base, char** line) {
#line 76 "make-executable.c"
	MakeExecutable * self;
	GdkColor result;
	GdkColor color = {0};
	self = (MakeExecutable*) base;
#line 46 "make-executable.vala"
	color = VALIDE_EXECUTABLE_CLASS (make_executable_parent_class)->get_color_line ((ValideExecutable*) VALIDE_COMPILER (self), &(*line));
#line 47 "make-executable.vala"
	if (strstr ((*line), " error:") != NULL) {
#line 49 "make-executable.vala"
		gdk_color_parse (VALIDE_UTILS_COLOR_ERROR, &color);
#line 87 "make-executable.c"
	} else {
#line 51 "make-executable.vala"
		if (strstr ((*line), " warning:") != NULL) {
#line 53 "make-executable.vala"
			gdk_color_parse (VALIDE_UTILS_COLOR_WARNING, &color);
#line 93 "make-executable.c"
		}
	}
	result = color;
	return result;
}


#line 58 "make-executable.vala"
MakeExecutable* make_executable_construct (GType object_type, ValideProject* project) {
#line 103 "make-executable.c"
	GParameter * __params;
	GParameter * __params_it;
	MakeExecutable * self;
#line 58 "make-executable.vala"
	g_return_val_if_fail (project != NULL, NULL);
#line 109 "make-executable.c"
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
#line 60 "make-executable.vala"
	__params_it->name = "project";
#line 60 "make-executable.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_PROJECT);
#line 60 "make-executable.vala"
	g_value_set_object (&__params_it->value, project);
#line 60 "make-executable.vala"
	__params_it++;
#line 120 "make-executable.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 58 "make-executable.vala"
MakeExecutable* make_executable_new (ValideProject* project) {
#line 58 "make-executable.vala"
	return make_executable_construct (TYPE_MAKE_EXECUTABLE, project);
#line 135 "make-executable.c"
}


static const char* make_executable_real_get_executable (ValideExecutable* base) {
	const char* result;
	MakeExecutable* self;
	self = (MakeExecutable*) base;
	result = MAKE_EXECUTABLE_EXECUTABLE_PATH;
	return result;
}


static void make_executable_real_set_executable (ValideExecutable* base, const char* value) {
	MakeExecutable* self;
	self = (MakeExecutable*) base;
	g_object_notify ((GObject *) self, "executable");
}


static void make_executable_class_init (MakeExecutableClass * klass) {
	make_executable_parent_class = g_type_class_peek_parent (klass);
	VALIDE_EXECUTABLE_CLASS (klass)->get_color_line = make_executable_real_get_color_line;
	VALIDE_EXECUTABLE_CLASS (klass)->get_executable = make_executable_real_get_executable;
	VALIDE_EXECUTABLE_CLASS (klass)->set_executable = make_executable_real_set_executable;
	G_OBJECT_CLASS (klass)->get_property = make_executable_get_property;
	G_OBJECT_CLASS (klass)->set_property = make_executable_set_property;
	g_object_class_override_property (G_OBJECT_CLASS (klass), MAKE_EXECUTABLE_EXECUTABLE, "executable");
}


static void make_executable_instance_init (MakeExecutable * self) {
}


GType make_executable_get_type (void) {
	static GType make_executable_type_id = 0;
	if (make_executable_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (MakeExecutableClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) make_executable_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (MakeExecutable), 0, (GInstanceInitFunc) make_executable_instance_init, NULL };
		make_executable_type_id = g_type_register_static (VALIDE_TYPE_COMPILER, "MakeExecutable", &g_define_type_info, 0);
	}
	return make_executable_type_id;
}


static void make_executable_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	MakeExecutable * self;
	gpointer boxed;
	self = MAKE_EXECUTABLE (object);
	switch (property_id) {
		case MAKE_EXECUTABLE_EXECUTABLE:
		g_value_set_string (value, valide_executable_get_executable ((ValideExecutable*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void make_executable_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	MakeExecutable * self;
	self = MAKE_EXECUTABLE (object);
	switch (property_id) {
		case MAKE_EXECUTABLE_EXECUTABLE:
		valide_executable_set_executable ((ValideExecutable*) self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}




