/* make-options.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <valide-0.0.h>
#include <stdlib.h>
#include <string.h>


#define VALIDE_TYPE_MAKE_BUILDER_OPTIONS (valide_make_builder_options_get_type ())
#define VALIDE_MAKE_BUILDER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_MAKE_BUILDER_OPTIONS, ValideMakeBuilderOptions))
#define VALIDE_MAKE_BUILDER_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_MAKE_BUILDER_OPTIONS, ValideMakeBuilderOptionsClass))
#define VALIDE_IS_MAKE_BUILDER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_MAKE_BUILDER_OPTIONS))
#define VALIDE_IS_MAKE_BUILDER_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_MAKE_BUILDER_OPTIONS))
#define VALIDE_MAKE_BUILDER_OPTIONS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_MAKE_BUILDER_OPTIONS, ValideMakeBuilderOptionsClass))

typedef struct _ValideMakeBuilderOptions ValideMakeBuilderOptions;
typedef struct _ValideMakeBuilderOptionsClass ValideMakeBuilderOptionsClass;
typedef struct _ValideMakeBuilderOptionsPrivate ValideMakeBuilderOptionsPrivate;

struct _ValideMakeBuilderOptions {
	GObject parent_instance;
	ValideMakeBuilderOptionsPrivate * priv;
};

struct _ValideMakeBuilderOptionsClass {
	GObjectClass parent_class;
};

struct _ValideMakeBuilderOptionsPrivate {
	char* _prefix;
	char* _configure_options;
	char* _build_options;
	char* _install_options;
	char* _dist_options;
	char* _clean_options;
	char* _distclean_options;
	char* _uninstall_options;
};


static gpointer valide_make_builder_options_parent_class = NULL;
static ValideBuilderOptionsIface* valide_make_builder_options_valide_builder_options_parent_iface = NULL;
static GYAMLBuildableIface* valide_make_builder_options_g_yaml_buildable_parent_iface = NULL;

GType valide_make_builder_options_get_type (void);
#define VALIDE_MAKE_BUILDER_OPTIONS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_MAKE_BUILDER_OPTIONS, ValideMakeBuilderOptionsPrivate))
enum  {
	VALIDE_MAKE_BUILDER_OPTIONS_DUMMY_PROPERTY,
	VALIDE_MAKE_BUILDER_OPTIONS_PREFIX,
	VALIDE_MAKE_BUILDER_OPTIONS_CONFIGURE_OPTIONS,
	VALIDE_MAKE_BUILDER_OPTIONS_BUILD_OPTIONS,
	VALIDE_MAKE_BUILDER_OPTIONS_INSTALL_OPTIONS,
	VALIDE_MAKE_BUILDER_OPTIONS_DIST_OPTIONS,
	VALIDE_MAKE_BUILDER_OPTIONS_CLEAN_OPTIONS,
	VALIDE_MAKE_BUILDER_OPTIONS_DISTCLEAN_OPTIONS,
	VALIDE_MAKE_BUILDER_OPTIONS_UNINSTALL_OPTIONS
};
const char* valide_make_builder_options_get_prefix (ValideMakeBuilderOptions* self);
const char* valide_make_builder_options_get_configure_options (ValideMakeBuilderOptions* self);
const char* valide_make_builder_options_get_build_options (ValideMakeBuilderOptions* self);
const char* valide_make_builder_options_get_install_options (ValideMakeBuilderOptions* self);
const char* valide_make_builder_options_get_dist_options (ValideMakeBuilderOptions* self);
const char* valide_make_builder_options_get_clean_options (ValideMakeBuilderOptions* self);
const char* valide_make_builder_options_get_distclean_options (ValideMakeBuilderOptions* self);
const char* valide_make_builder_options_get_uninstall_options (ValideMakeBuilderOptions* self);
char* valide_make_builder_options_to_string (ValideMakeBuilderOptions* self, BuildingStep step);
ValideMakeBuilderOptions* valide_make_builder_options_new (void);
ValideMakeBuilderOptions* valide_make_builder_options_construct (GType object_type);
void valide_make_builder_options_set_prefix (ValideMakeBuilderOptions* self, const char* value);
void valide_make_builder_options_set_configure_options (ValideMakeBuilderOptions* self, const char* value);
void valide_make_builder_options_set_build_options (ValideMakeBuilderOptions* self, const char* value);
void valide_make_builder_options_set_install_options (ValideMakeBuilderOptions* self, const char* value);
void valide_make_builder_options_set_dist_options (ValideMakeBuilderOptions* self, const char* value);
void valide_make_builder_options_set_clean_options (ValideMakeBuilderOptions* self, const char* value);
void valide_make_builder_options_set_distclean_options (ValideMakeBuilderOptions* self, const char* value);
void valide_make_builder_options_set_uninstall_options (ValideMakeBuilderOptions* self, const char* value);
static void valide_make_builder_options_finalize (GObject* obj);
static void valide_make_builder_options_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_make_builder_options_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static int _vala_strcmp0 (const char * str1, const char * str2);



#line 38 "make-options.vala"
char* valide_make_builder_options_to_string (ValideMakeBuilderOptions* self, BuildingStep step) {
#line 107 "make-options.c"
	char* result;
	GString* sb;
	GString* _tmp0_;
	const char* _tmp2_;
#line 38 "make-options.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 114 "make-options.c"
	sb = NULL;
	_tmp0_ = NULL;
#line 42 "make-options.vala"
	sb = (_tmp0_ = g_string_new (""), (sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL)), _tmp0_);
#line 43 "make-options.vala"
	switch (step) {
#line 121 "make-options.c"
		case BUILDING_STEP_CONFIGURE:
		{
			gboolean _tmp1_;
			_tmp1_ = FALSE;
#line 46 "make-options.vala"
			if (self->priv->_prefix != NULL) {
#line 46 "make-options.vala"
				_tmp1_ = _vala_strcmp0 (self->priv->_prefix, "") != 0;
#line 130 "make-options.c"
			} else {
#line 46 "make-options.vala"
				_tmp1_ = FALSE;
#line 134 "make-options.c"
			}
#line 46 "make-options.vala"
			if (_tmp1_) {
#line 48 "make-options.vala"
				g_string_append_printf (sb, "--prefix='%s' ", self->priv->_prefix);
#line 140 "make-options.c"
			}
#line 50 "make-options.vala"
			if (self->priv->_configure_options != NULL) {
#line 52 "make-options.vala"
				g_string_append (sb, self->priv->_configure_options);
#line 146 "make-options.c"
			}
#line 54 "make-options.vala"
			break;
#line 150 "make-options.c"
		}
		case BUILDING_STEP_BUILD:
		{
#line 56 "make-options.vala"
			if (self->priv->_build_options != NULL) {
#line 58 "make-options.vala"
				g_string_append (sb, self->priv->_build_options);
#line 158 "make-options.c"
			}
#line 60 "make-options.vala"
			break;
#line 162 "make-options.c"
		}
		case BUILDING_STEP_INSTALL:
		{
#line 62 "make-options.vala"
			g_string_append (sb, "install ");
#line 63 "make-options.vala"
			if (self->priv->_install_options != NULL) {
#line 65 "make-options.vala"
				g_string_append (sb, self->priv->_install_options);
#line 172 "make-options.c"
			}
#line 67 "make-options.vala"
			break;
#line 176 "make-options.c"
		}
		case BUILDING_STEP_DIST:
		{
#line 69 "make-options.vala"
			g_string_append (sb, "dist ");
#line 70 "make-options.vala"
			if (self->priv->_dist_options != NULL) {
#line 72 "make-options.vala"
				g_string_append (sb, self->priv->_dist_options);
#line 186 "make-options.c"
			}
#line 74 "make-options.vala"
			break;
#line 190 "make-options.c"
		}
		case BUILDING_STEP_CLEAN:
		{
#line 76 "make-options.vala"
			g_string_append (sb, "clean ");
#line 77 "make-options.vala"
			if (self->priv->_clean_options != NULL) {
#line 79 "make-options.vala"
				g_string_append (sb, self->priv->_clean_options);
#line 200 "make-options.c"
			}
#line 81 "make-options.vala"
			break;
#line 204 "make-options.c"
		}
		case BUILDING_STEP_DISTCLEAN:
		{
#line 83 "make-options.vala"
			g_string_append (sb, "disclean ");
#line 84 "make-options.vala"
			if (self->priv->_distclean_options != NULL) {
#line 86 "make-options.vala"
				g_string_append (sb, self->priv->_distclean_options);
#line 214 "make-options.c"
			}
#line 88 "make-options.vala"
			break;
#line 218 "make-options.c"
		}
		case BUILDING_STEP_UNINSTALL:
		{
#line 90 "make-options.vala"
			g_string_append (sb, "uninstall ");
#line 91 "make-options.vala"
			if (self->priv->_uninstall_options != NULL) {
#line 93 "make-options.vala"
				g_string_append (sb, self->priv->_uninstall_options);
#line 228 "make-options.c"
			}
#line 95 "make-options.vala"
			break;
#line 232 "make-options.c"
		}
	}
#line 97 "make-options.vala"
	_tmp2_ = NULL;
#line 237 "make-options.c"
	result = (_tmp2_ = sb->str, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_));
	(sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL));
	return result;
	(sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL));
}


#line 22 "make-options.vala"
ValideMakeBuilderOptions* valide_make_builder_options_construct (GType object_type) {
#line 247 "make-options.c"
	ValideMakeBuilderOptions * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


#line 22 "make-options.vala"
ValideMakeBuilderOptions* valide_make_builder_options_new (void) {
#line 22 "make-options.vala"
	return valide_make_builder_options_construct (VALIDE_TYPE_MAKE_BUILDER_OPTIONS);
#line 258 "make-options.c"
}


const char* valide_make_builder_options_get_prefix (ValideMakeBuilderOptions* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_prefix;
	return result;
}


void valide_make_builder_options_set_prefix (ValideMakeBuilderOptions* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 28 "make-options.vala"
	_tmp0_ = NULL;
#line 277 "make-options.c"
	self->priv->_prefix = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_prefix = (g_free (self->priv->_prefix), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "prefix");
}


const char* valide_make_builder_options_get_configure_options (ValideMakeBuilderOptions* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_configure_options;
	return result;
}


void valide_make_builder_options_set_configure_options (ValideMakeBuilderOptions* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 30 "make-options.vala"
	_tmp0_ = NULL;
#line 298 "make-options.c"
	self->priv->_configure_options = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_configure_options = (g_free (self->priv->_configure_options), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "configure-options");
}


const char* valide_make_builder_options_get_build_options (ValideMakeBuilderOptions* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_build_options;
	return result;
}


void valide_make_builder_options_set_build_options (ValideMakeBuilderOptions* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 31 "make-options.vala"
	_tmp0_ = NULL;
#line 319 "make-options.c"
	self->priv->_build_options = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_build_options = (g_free (self->priv->_build_options), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "build-options");
}


const char* valide_make_builder_options_get_install_options (ValideMakeBuilderOptions* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_install_options;
	return result;
}


void valide_make_builder_options_set_install_options (ValideMakeBuilderOptions* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 32 "make-options.vala"
	_tmp0_ = NULL;
#line 340 "make-options.c"
	self->priv->_install_options = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_install_options = (g_free (self->priv->_install_options), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "install-options");
}


const char* valide_make_builder_options_get_dist_options (ValideMakeBuilderOptions* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_dist_options;
	return result;
}


void valide_make_builder_options_set_dist_options (ValideMakeBuilderOptions* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 33 "make-options.vala"
	_tmp0_ = NULL;
#line 361 "make-options.c"
	self->priv->_dist_options = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_dist_options = (g_free (self->priv->_dist_options), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "dist-options");
}


const char* valide_make_builder_options_get_clean_options (ValideMakeBuilderOptions* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_clean_options;
	return result;
}


void valide_make_builder_options_set_clean_options (ValideMakeBuilderOptions* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 34 "make-options.vala"
	_tmp0_ = NULL;
#line 382 "make-options.c"
	self->priv->_clean_options = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_clean_options = (g_free (self->priv->_clean_options), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "clean-options");
}


const char* valide_make_builder_options_get_distclean_options (ValideMakeBuilderOptions* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_distclean_options;
	return result;
}


void valide_make_builder_options_set_distclean_options (ValideMakeBuilderOptions* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 35 "make-options.vala"
	_tmp0_ = NULL;
#line 403 "make-options.c"
	self->priv->_distclean_options = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_distclean_options = (g_free (self->priv->_distclean_options), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "distclean-options");
}


const char* valide_make_builder_options_get_uninstall_options (ValideMakeBuilderOptions* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_uninstall_options;
	return result;
}


void valide_make_builder_options_set_uninstall_options (ValideMakeBuilderOptions* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 36 "make-options.vala"
	_tmp0_ = NULL;
#line 424 "make-options.c"
	self->priv->_uninstall_options = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_uninstall_options = (g_free (self->priv->_uninstall_options), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "uninstall-options");
}


static void valide_make_builder_options_class_init (ValideMakeBuilderOptionsClass * klass) {
	valide_make_builder_options_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideMakeBuilderOptionsPrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_make_builder_options_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_make_builder_options_set_property;
	G_OBJECT_CLASS (klass)->finalize = valide_make_builder_options_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_MAKE_BUILDER_OPTIONS_PREFIX, g_param_spec_string ("prefix", "prefix", "prefix", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_MAKE_BUILDER_OPTIONS_CONFIGURE_OPTIONS, g_param_spec_string ("configure-options", "configure-options", "configure-options", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_MAKE_BUILDER_OPTIONS_BUILD_OPTIONS, g_param_spec_string ("build-options", "build-options", "build-options", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_MAKE_BUILDER_OPTIONS_INSTALL_OPTIONS, g_param_spec_string ("install-options", "install-options", "install-options", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_MAKE_BUILDER_OPTIONS_DIST_OPTIONS, g_param_spec_string ("dist-options", "dist-options", "dist-options", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_MAKE_BUILDER_OPTIONS_CLEAN_OPTIONS, g_param_spec_string ("clean-options", "clean-options", "clean-options", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_MAKE_BUILDER_OPTIONS_DISTCLEAN_OPTIONS, g_param_spec_string ("distclean-options", "distclean-options", "distclean-options", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_MAKE_BUILDER_OPTIONS_UNINSTALL_OPTIONS, g_param_spec_string ("uninstall-options", "uninstall-options", "uninstall-options", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void valide_make_builder_options_valide_builder_options_interface_init (ValideBuilderOptionsIface * iface) {
	valide_make_builder_options_valide_builder_options_parent_iface = g_type_interface_peek_parent (iface);
}


static void valide_make_builder_options_g_yaml_buildable_interface_init (GYAMLBuildableIface * iface) {
	valide_make_builder_options_g_yaml_buildable_parent_iface = g_type_interface_peek_parent (iface);
}


static void valide_make_builder_options_instance_init (ValideMakeBuilderOptions * self) {
	self->priv = VALIDE_MAKE_BUILDER_OPTIONS_GET_PRIVATE (self);
}


static void valide_make_builder_options_finalize (GObject* obj) {
	ValideMakeBuilderOptions * self;
	self = VALIDE_MAKE_BUILDER_OPTIONS (obj);
	self->priv->_prefix = (g_free (self->priv->_prefix), NULL);
	self->priv->_configure_options = (g_free (self->priv->_configure_options), NULL);
	self->priv->_build_options = (g_free (self->priv->_build_options), NULL);
	self->priv->_install_options = (g_free (self->priv->_install_options), NULL);
	self->priv->_dist_options = (g_free (self->priv->_dist_options), NULL);
	self->priv->_clean_options = (g_free (self->priv->_clean_options), NULL);
	self->priv->_distclean_options = (g_free (self->priv->_distclean_options), NULL);
	self->priv->_uninstall_options = (g_free (self->priv->_uninstall_options), NULL);
	G_OBJECT_CLASS (valide_make_builder_options_parent_class)->finalize (obj);
}


GType valide_make_builder_options_get_type (void) {
	static GType valide_make_builder_options_type_id = 0;
	if (valide_make_builder_options_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideMakeBuilderOptionsClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_make_builder_options_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideMakeBuilderOptions), 0, (GInstanceInitFunc) valide_make_builder_options_instance_init, NULL };
		static const GInterfaceInfo valide_builder_options_info = { (GInterfaceInitFunc) valide_make_builder_options_valide_builder_options_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo g_yaml_buildable_info = { (GInterfaceInitFunc) valide_make_builder_options_g_yaml_buildable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		valide_make_builder_options_type_id = g_type_register_static (G_TYPE_OBJECT, "ValideMakeBuilderOptions", &g_define_type_info, 0);
		g_type_add_interface_static (valide_make_builder_options_type_id, VALIDE_TYPE_BUILDER_OPTIONS, &valide_builder_options_info);
		g_type_add_interface_static (valide_make_builder_options_type_id, G_YAML_TYPE_BUILDABLE, &g_yaml_buildable_info);
	}
	return valide_make_builder_options_type_id;
}


static void valide_make_builder_options_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideMakeBuilderOptions * self;
	gpointer boxed;
	self = VALIDE_MAKE_BUILDER_OPTIONS (object);
	switch (property_id) {
		case VALIDE_MAKE_BUILDER_OPTIONS_PREFIX:
		g_value_set_string (value, valide_make_builder_options_get_prefix (self));
		break;
		case VALIDE_MAKE_BUILDER_OPTIONS_CONFIGURE_OPTIONS:
		g_value_set_string (value, valide_make_builder_options_get_configure_options (self));
		break;
		case VALIDE_MAKE_BUILDER_OPTIONS_BUILD_OPTIONS:
		g_value_set_string (value, valide_make_builder_options_get_build_options (self));
		break;
		case VALIDE_MAKE_BUILDER_OPTIONS_INSTALL_OPTIONS:
		g_value_set_string (value, valide_make_builder_options_get_install_options (self));
		break;
		case VALIDE_MAKE_BUILDER_OPTIONS_DIST_OPTIONS:
		g_value_set_string (value, valide_make_builder_options_get_dist_options (self));
		break;
		case VALIDE_MAKE_BUILDER_OPTIONS_CLEAN_OPTIONS:
		g_value_set_string (value, valide_make_builder_options_get_clean_options (self));
		break;
		case VALIDE_MAKE_BUILDER_OPTIONS_DISTCLEAN_OPTIONS:
		g_value_set_string (value, valide_make_builder_options_get_distclean_options (self));
		break;
		case VALIDE_MAKE_BUILDER_OPTIONS_UNINSTALL_OPTIONS:
		g_value_set_string (value, valide_make_builder_options_get_uninstall_options (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_make_builder_options_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideMakeBuilderOptions * self;
	self = VALIDE_MAKE_BUILDER_OPTIONS (object);
	switch (property_id) {
		case VALIDE_MAKE_BUILDER_OPTIONS_PREFIX:
		valide_make_builder_options_set_prefix (self, g_value_get_string (value));
		break;
		case VALIDE_MAKE_BUILDER_OPTIONS_CONFIGURE_OPTIONS:
		valide_make_builder_options_set_configure_options (self, g_value_get_string (value));
		break;
		case VALIDE_MAKE_BUILDER_OPTIONS_BUILD_OPTIONS:
		valide_make_builder_options_set_build_options (self, g_value_get_string (value));
		break;
		case VALIDE_MAKE_BUILDER_OPTIONS_INSTALL_OPTIONS:
		valide_make_builder_options_set_install_options (self, g_value_get_string (value));
		break;
		case VALIDE_MAKE_BUILDER_OPTIONS_DIST_OPTIONS:
		valide_make_builder_options_set_dist_options (self, g_value_get_string (value));
		break;
		case VALIDE_MAKE_BUILDER_OPTIONS_CLEAN_OPTIONS:
		valide_make_builder_options_set_clean_options (self, g_value_get_string (value));
		break;
		case VALIDE_MAKE_BUILDER_OPTIONS_DISTCLEAN_OPTIONS:
		valide_make_builder_options_set_distclean_options (self, g_value_get_string (value));
		break;
		case VALIDE_MAKE_BUILDER_OPTIONS_UNINSTALL_OPTIONS:
		valide_make_builder_options_set_uninstall_options (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




