/* make-preferences.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <valide-0.0.h>
#include <glib/gi18n-lib.h>
#include <stdlib.h>
#include <string.h>


#define TYPE_MAKE_BUILDER_PREFERENCES (make_builder_preferences_get_type ())
#define MAKE_BUILDER_PREFERENCES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MAKE_BUILDER_PREFERENCES, MakeBuilderPreferences))
#define MAKE_BUILDER_PREFERENCES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MAKE_BUILDER_PREFERENCES, MakeBuilderPreferencesClass))
#define IS_MAKE_BUILDER_PREFERENCES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MAKE_BUILDER_PREFERENCES))
#define IS_MAKE_BUILDER_PREFERENCES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MAKE_BUILDER_PREFERENCES))
#define MAKE_BUILDER_PREFERENCES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MAKE_BUILDER_PREFERENCES, MakeBuilderPreferencesClass))

typedef struct _MakeBuilderPreferences MakeBuilderPreferences;
typedef struct _MakeBuilderPreferencesClass MakeBuilderPreferencesClass;
typedef struct _MakeBuilderPreferencesPrivate MakeBuilderPreferencesPrivate;

#define VALIDE_TYPE_MAKE_BUILDER_OPTIONS (valide_make_builder_options_get_type ())
#define VALIDE_MAKE_BUILDER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_MAKE_BUILDER_OPTIONS, ValideMakeBuilderOptions))
#define VALIDE_MAKE_BUILDER_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_MAKE_BUILDER_OPTIONS, ValideMakeBuilderOptionsClass))
#define VALIDE_IS_MAKE_BUILDER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_MAKE_BUILDER_OPTIONS))
#define VALIDE_IS_MAKE_BUILDER_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_MAKE_BUILDER_OPTIONS))
#define VALIDE_MAKE_BUILDER_OPTIONS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_MAKE_BUILDER_OPTIONS, ValideMakeBuilderOptionsClass))

typedef struct _ValideMakeBuilderOptions ValideMakeBuilderOptions;
typedef struct _ValideMakeBuilderOptionsClass ValideMakeBuilderOptionsClass;

struct _MakeBuilderPreferences {
	GtkTable parent_instance;
	MakeBuilderPreferencesPrivate * priv;
};

struct _MakeBuilderPreferencesClass {
	GtkTableClass parent_class;
};

struct _MakeBuilderPreferencesPrivate {
	ValideMakeBuilderOptions* options;
	ValideProject* _project;
};


static gpointer make_builder_preferences_parent_class = NULL;

GType make_builder_preferences_get_type (void);
GType valide_make_builder_options_get_type (void);
#define MAKE_BUILDER_PREFERENCES_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_MAKE_BUILDER_PREFERENCES, MakeBuilderPreferencesPrivate))
enum  {
	MAKE_BUILDER_PREFERENCES_DUMMY_PROPERTY,
	MAKE_BUILDER_PREFERENCES_PROJECT
};
static void make_builder_preferences_set_project (MakeBuilderPreferences* self, ValideProject* value);
MakeBuilderPreferences* make_builder_preferences_new (ValideProject* project);
MakeBuilderPreferences* make_builder_preferences_construct (GType object_type, ValideProject* project);
ValideProject* make_builder_preferences_get_project (MakeBuilderPreferences* self);
const char* valide_make_builder_options_get_prefix (ValideMakeBuilderOptions* self);
void valide_make_builder_options_set_prefix (ValideMakeBuilderOptions* self, const char* value);
static void _lambda0_ (GtkEditable* s, MakeBuilderPreferences* self);
static void __lambda0__gtk_editable_changed (GtkEditable* _sender, gpointer self);
const char* valide_make_builder_options_get_configure_options (ValideMakeBuilderOptions* self);
void valide_make_builder_options_set_configure_options (ValideMakeBuilderOptions* self, const char* value);
static void _lambda1_ (GtkEditable* s, MakeBuilderPreferences* self);
static void __lambda1__gtk_editable_changed (GtkEditable* _sender, gpointer self);
const char* valide_make_builder_options_get_build_options (ValideMakeBuilderOptions* self);
void valide_make_builder_options_set_build_options (ValideMakeBuilderOptions* self, const char* value);
static void _lambda2_ (GtkEditable* s, MakeBuilderPreferences* self);
static void __lambda2__gtk_editable_changed (GtkEditable* _sender, gpointer self);
const char* valide_make_builder_options_get_install_options (ValideMakeBuilderOptions* self);
void valide_make_builder_options_set_install_options (ValideMakeBuilderOptions* self, const char* value);
static void _lambda3_ (GtkEditable* s, MakeBuilderPreferences* self);
static void __lambda3__gtk_editable_changed (GtkEditable* _sender, gpointer self);
const char* valide_make_builder_options_get_dist_options (ValideMakeBuilderOptions* self);
void valide_make_builder_options_set_dist_options (ValideMakeBuilderOptions* self, const char* value);
static void _lambda4_ (GtkEditable* s, MakeBuilderPreferences* self);
static void __lambda4__gtk_editable_changed (GtkEditable* _sender, gpointer self);
const char* valide_make_builder_options_get_clean_options (ValideMakeBuilderOptions* self);
void valide_make_builder_options_set_clean_options (ValideMakeBuilderOptions* self, const char* value);
static void _lambda5_ (GtkEditable* s, MakeBuilderPreferences* self);
static void __lambda5__gtk_editable_changed (GtkEditable* _sender, gpointer self);
const char* valide_make_builder_options_get_distclean_options (ValideMakeBuilderOptions* self);
void valide_make_builder_options_set_distclean_options (ValideMakeBuilderOptions* self, const char* value);
static void _lambda6_ (GtkEditable* s, MakeBuilderPreferences* self);
static void __lambda6__gtk_editable_changed (GtkEditable* _sender, gpointer self);
const char* valide_make_builder_options_get_uninstall_options (ValideMakeBuilderOptions* self);
void valide_make_builder_options_set_uninstall_options (ValideMakeBuilderOptions* self, const char* value);
static void _lambda7_ (GtkEditable* s, MakeBuilderPreferences* self);
static void __lambda7__gtk_editable_changed (GtkEditable* _sender, gpointer self);
static GObject * make_builder_preferences_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void make_builder_preferences_finalize (GObject* obj);
static void make_builder_preferences_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void make_builder_preferences_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);



#line 34 "make-preferences.vala"
MakeBuilderPreferences* make_builder_preferences_construct (GType object_type, ValideProject* project) {
#line 122 "make-preferences.c"
	GParameter * __params;
	GParameter * __params_it;
	MakeBuilderPreferences * self;
#line 34 "make-preferences.vala"
	g_return_val_if_fail (project != NULL, NULL);
#line 128 "make-preferences.c"
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
#line 36 "make-preferences.vala"
	__params_it->name = "project";
#line 36 "make-preferences.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_PROJECT);
#line 36 "make-preferences.vala"
	g_value_set_object (&__params_it->value, project);
#line 36 "make-preferences.vala"
	__params_it++;
#line 139 "make-preferences.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 34 "make-preferences.vala"
MakeBuilderPreferences* make_builder_preferences_new (ValideProject* project) {
#line 34 "make-preferences.vala"
	return make_builder_preferences_construct (TYPE_MAKE_BUILDER_PREFERENCES, project);
#line 154 "make-preferences.c"
}


ValideProject* make_builder_preferences_get_project (MakeBuilderPreferences* self) {
	ValideProject* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_project;
	return result;
}


static void make_builder_preferences_set_project (MakeBuilderPreferences* self, ValideProject* value) {
	ValideProject* _tmp1_;
	ValideProject* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 31 "make-preferences.vala"
	_tmp0_ = NULL;
#line 173 "make-preferences.c"
	self->priv->_project = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_project == NULL) ? NULL : (self->priv->_project = (g_object_unref (self->priv->_project), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "project");
}


static void _lambda0_ (GtkEditable* s, MakeBuilderPreferences* self) {
	GtkEditable* _tmp0_;
	g_return_if_fail (s != NULL);
#line 61 "make-preferences.vala"
	_tmp0_ = NULL;
#line 61 "make-preferences.vala"
	valide_make_builder_options_set_prefix (self->priv->options, gtk_entry_get_text ((_tmp0_ = s, GTK_IS_ENTRY (_tmp0_) ? ((GtkEntry*) _tmp0_) : NULL)));
#line 62 "make-preferences.vala"
	valide_project_save (self->priv->_project);
#line 188 "make-preferences.c"
}


static void __lambda0__gtk_editable_changed (GtkEditable* _sender, gpointer self) {
	_lambda0_ (_sender, self);
}


static void _lambda1_ (GtkEditable* s, MakeBuilderPreferences* self) {
	GtkEditable* _tmp0_;
	g_return_if_fail (s != NULL);
#line 77 "make-preferences.vala"
	_tmp0_ = NULL;
#line 77 "make-preferences.vala"
	valide_make_builder_options_set_configure_options (self->priv->options, gtk_entry_get_text ((_tmp0_ = s, GTK_IS_ENTRY (_tmp0_) ? ((GtkEntry*) _tmp0_) : NULL)));
#line 78 "make-preferences.vala"
	valide_project_save (self->priv->_project);
#line 206 "make-preferences.c"
}


static void __lambda1__gtk_editable_changed (GtkEditable* _sender, gpointer self) {
	_lambda1_ (_sender, self);
}


static void _lambda2_ (GtkEditable* s, MakeBuilderPreferences* self) {
	GtkEditable* _tmp0_;
	g_return_if_fail (s != NULL);
#line 93 "make-preferences.vala"
	_tmp0_ = NULL;
#line 93 "make-preferences.vala"
	valide_make_builder_options_set_build_options (self->priv->options, gtk_entry_get_text ((_tmp0_ = s, GTK_IS_ENTRY (_tmp0_) ? ((GtkEntry*) _tmp0_) : NULL)));
#line 94 "make-preferences.vala"
	valide_project_save (self->priv->_project);
#line 224 "make-preferences.c"
}


static void __lambda2__gtk_editable_changed (GtkEditable* _sender, gpointer self) {
	_lambda2_ (_sender, self);
}


static void _lambda3_ (GtkEditable* s, MakeBuilderPreferences* self) {
	GtkEditable* _tmp0_;
	g_return_if_fail (s != NULL);
#line 109 "make-preferences.vala"
	_tmp0_ = NULL;
#line 109 "make-preferences.vala"
	valide_make_builder_options_set_install_options (self->priv->options, gtk_entry_get_text ((_tmp0_ = s, GTK_IS_ENTRY (_tmp0_) ? ((GtkEntry*) _tmp0_) : NULL)));
#line 110 "make-preferences.vala"
	valide_project_save (self->priv->_project);
#line 242 "make-preferences.c"
}


static void __lambda3__gtk_editable_changed (GtkEditable* _sender, gpointer self) {
	_lambda3_ (_sender, self);
}


static void _lambda4_ (GtkEditable* s, MakeBuilderPreferences* self) {
	GtkEditable* _tmp0_;
	g_return_if_fail (s != NULL);
#line 125 "make-preferences.vala"
	_tmp0_ = NULL;
#line 125 "make-preferences.vala"
	valide_make_builder_options_set_dist_options (self->priv->options, gtk_entry_get_text ((_tmp0_ = s, GTK_IS_ENTRY (_tmp0_) ? ((GtkEntry*) _tmp0_) : NULL)));
#line 126 "make-preferences.vala"
	valide_project_save (self->priv->_project);
#line 260 "make-preferences.c"
}


static void __lambda4__gtk_editable_changed (GtkEditable* _sender, gpointer self) {
	_lambda4_ (_sender, self);
}


static void _lambda5_ (GtkEditable* s, MakeBuilderPreferences* self) {
	GtkEditable* _tmp0_;
	g_return_if_fail (s != NULL);
#line 141 "make-preferences.vala"
	_tmp0_ = NULL;
#line 141 "make-preferences.vala"
	valide_make_builder_options_set_clean_options (self->priv->options, gtk_entry_get_text ((_tmp0_ = s, GTK_IS_ENTRY (_tmp0_) ? ((GtkEntry*) _tmp0_) : NULL)));
#line 142 "make-preferences.vala"
	valide_project_save (self->priv->_project);
#line 278 "make-preferences.c"
}


static void __lambda5__gtk_editable_changed (GtkEditable* _sender, gpointer self) {
	_lambda5_ (_sender, self);
}


static void _lambda6_ (GtkEditable* s, MakeBuilderPreferences* self) {
	GtkEditable* _tmp0_;
	g_return_if_fail (s != NULL);
#line 157 "make-preferences.vala"
	_tmp0_ = NULL;
#line 157 "make-preferences.vala"
	valide_make_builder_options_set_distclean_options (self->priv->options, gtk_entry_get_text ((_tmp0_ = s, GTK_IS_ENTRY (_tmp0_) ? ((GtkEntry*) _tmp0_) : NULL)));
#line 158 "make-preferences.vala"
	valide_project_save (self->priv->_project);
#line 296 "make-preferences.c"
}


static void __lambda6__gtk_editable_changed (GtkEditable* _sender, gpointer self) {
	_lambda6_ (_sender, self);
}


static void _lambda7_ (GtkEditable* s, MakeBuilderPreferences* self) {
	GtkEditable* _tmp0_;
	g_return_if_fail (s != NULL);
#line 173 "make-preferences.vala"
	_tmp0_ = NULL;
#line 173 "make-preferences.vala"
	valide_make_builder_options_set_uninstall_options (self->priv->options, gtk_entry_get_text ((_tmp0_ = s, GTK_IS_ENTRY (_tmp0_) ? ((GtkEntry*) _tmp0_) : NULL)));
#line 174 "make-preferences.vala"
	valide_project_save (self->priv->_project);
#line 314 "make-preferences.c"
}


static void __lambda7__gtk_editable_changed (GtkEditable* _sender, gpointer self) {
	_lambda7_ (_sender, self);
}


static GObject * make_builder_preferences_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	MakeBuilderPreferencesClass * klass;
	GObjectClass * parent_class;
	MakeBuilderPreferences * self;
	klass = MAKE_BUILDER_PREFERENCES_CLASS (g_type_class_peek (TYPE_MAKE_BUILDER_PREFERENCES));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = MAKE_BUILDER_PREFERENCES (obj);
	{
		GtkLabel* label;
		GtkEntry* entry;
		ValideBuilderOptions* _tmp0_;
		GtkLabel* _tmp1_;
		GtkEntry* _tmp2_;
		GtkLabel* _tmp3_;
		GtkEntry* _tmp4_;
		GtkLabel* _tmp5_;
		GtkEntry* _tmp6_;
		GtkLabel* _tmp7_;
		GtkEntry* _tmp8_;
		GtkLabel* _tmp9_;
		GtkEntry* _tmp10_;
		GtkLabel* _tmp11_;
		GtkEntry* _tmp12_;
		GtkLabel* _tmp13_;
		GtkEntry* _tmp14_;
		GtkLabel* _tmp15_;
		GtkEntry* _tmp16_;
		label = NULL;
		entry = NULL;
#line 44 "make-preferences.vala"
		gtk_table_resize ((GtkTable*) self, (guint) 2, (guint) 8);
#line 45 "make-preferences.vala"
		gtk_container_set_border_width ((GtkContainer*) self, (guint) 5);
#line 46 "make-preferences.vala"
		gtk_table_set_col_spacings ((GtkTable*) self, (guint) 5);
#line 47 "make-preferences.vala"
		gtk_table_set_row_spacings ((GtkTable*) self, (guint) 10);
#line 48 "make-preferences.vala"
		_tmp0_ = NULL;
#line 48 "make-preferences.vala"
		self->priv->options = (_tmp0_ = valide_project_get_builder_options (self->priv->_project), VALIDE_IS_MAKE_BUILDER_OPTIONS (_tmp0_) ? ((ValideMakeBuilderOptions*) _tmp0_) : NULL);
#line 366 "make-preferences.c"
		/* prefix */
		_tmp1_ = NULL;
#line 51 "make-preferences.vala"
		label = (_tmp1_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("Prefix:"))), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp1_);
#line 52 "make-preferences.vala"
		gtk_misc_set_alignment ((GtkMisc*) label, 0.0f, 0.5f);
#line 53 "make-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) label, (guint) 0, (guint) 1, (guint) 0, (guint) 1, GTK_FILL, 0, (guint) 0, (guint) 0);
#line 375 "make-preferences.c"
		_tmp2_ = NULL;
#line 55 "make-preferences.vala"
		entry = (_tmp2_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL)), _tmp2_);
#line 56 "make-preferences.vala"
		if (valide_make_builder_options_get_prefix (self->priv->options) != NULL) {
#line 58 "make-preferences.vala"
			gtk_entry_set_text (entry, valide_make_builder_options_get_prefix (self->priv->options));
#line 383 "make-preferences.c"
		}
#line 60 "make-preferences.vala"
		g_signal_connect ((GtkEditable*) entry, "changed", (GCallback) __lambda0__gtk_editable_changed, self);
#line 64 "make-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) entry, (guint) 1, (guint) 2, (guint) 0, (guint) 1, GTK_EXPAND | GTK_FILL, 0, (guint) 0, (guint) 0);
#line 389 "make-preferences.c"
		/* configure_options */
		_tmp3_ = NULL;
#line 67 "make-preferences.vala"
		label = (_tmp3_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("Configure options:"))), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp3_);
#line 68 "make-preferences.vala"
		gtk_misc_set_alignment ((GtkMisc*) label, 0.0f, 0.5f);
#line 69 "make-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) label, (guint) 0, (guint) 1, (guint) 1, (guint) 2, GTK_FILL, 0, (guint) 0, (guint) 0);
#line 398 "make-preferences.c"
		_tmp4_ = NULL;
#line 71 "make-preferences.vala"
		entry = (_tmp4_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL)), _tmp4_);
#line 72 "make-preferences.vala"
		if (valide_make_builder_options_get_configure_options (self->priv->options) != NULL) {
#line 74 "make-preferences.vala"
			gtk_entry_set_text (entry, valide_make_builder_options_get_configure_options (self->priv->options));
#line 406 "make-preferences.c"
		}
#line 76 "make-preferences.vala"
		g_signal_connect ((GtkEditable*) entry, "changed", (GCallback) __lambda1__gtk_editable_changed, self);
#line 80 "make-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) entry, (guint) 1, (guint) 2, (guint) 1, (guint) 2, GTK_EXPAND | GTK_FILL, 0, (guint) 0, (guint) 0);
#line 412 "make-preferences.c"
		/* build_options */
		_tmp5_ = NULL;
#line 83 "make-preferences.vala"
		label = (_tmp5_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("Build options:"))), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp5_);
#line 84 "make-preferences.vala"
		gtk_misc_set_alignment ((GtkMisc*) label, 0.0f, 0.5f);
#line 85 "make-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) label, (guint) 0, (guint) 1, (guint) 2, (guint) 3, GTK_FILL, 0, (guint) 0, (guint) 0);
#line 421 "make-preferences.c"
		_tmp6_ = NULL;
#line 87 "make-preferences.vala"
		entry = (_tmp6_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL)), _tmp6_);
#line 88 "make-preferences.vala"
		if (valide_make_builder_options_get_build_options (self->priv->options) != NULL) {
#line 90 "make-preferences.vala"
			gtk_entry_set_text (entry, valide_make_builder_options_get_build_options (self->priv->options));
#line 429 "make-preferences.c"
		}
#line 92 "make-preferences.vala"
		g_signal_connect ((GtkEditable*) entry, "changed", (GCallback) __lambda2__gtk_editable_changed, self);
#line 96 "make-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) entry, (guint) 1, (guint) 2, (guint) 2, (guint) 3, GTK_EXPAND | GTK_FILL, 0, (guint) 0, (guint) 0);
#line 435 "make-preferences.c"
		/* install_options */
		_tmp7_ = NULL;
#line 99 "make-preferences.vala"
		label = (_tmp7_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("Install options:"))), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp7_);
#line 100 "make-preferences.vala"
		gtk_misc_set_alignment ((GtkMisc*) label, 0.0f, 0.5f);
#line 101 "make-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) label, (guint) 0, (guint) 1, (guint) 3, (guint) 4, GTK_FILL, 0, (guint) 0, (guint) 0);
#line 444 "make-preferences.c"
		_tmp8_ = NULL;
#line 103 "make-preferences.vala"
		entry = (_tmp8_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL)), _tmp8_);
#line 104 "make-preferences.vala"
		if (valide_make_builder_options_get_install_options (self->priv->options) != NULL) {
#line 106 "make-preferences.vala"
			gtk_entry_set_text (entry, valide_make_builder_options_get_install_options (self->priv->options));
#line 452 "make-preferences.c"
		}
#line 108 "make-preferences.vala"
		g_signal_connect ((GtkEditable*) entry, "changed", (GCallback) __lambda3__gtk_editable_changed, self);
#line 112 "make-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) entry, (guint) 1, (guint) 2, (guint) 3, (guint) 4, GTK_EXPAND | GTK_FILL, 0, (guint) 0, (guint) 0);
#line 458 "make-preferences.c"
		/* dist_options */
		_tmp9_ = NULL;
#line 115 "make-preferences.vala"
		label = (_tmp9_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("Dist options:"))), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp9_);
#line 116 "make-preferences.vala"
		gtk_misc_set_alignment ((GtkMisc*) label, 0.0f, 0.5f);
#line 117 "make-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) label, (guint) 0, (guint) 1, (guint) 4, (guint) 5, GTK_FILL, 0, (guint) 0, (guint) 0);
#line 467 "make-preferences.c"
		_tmp10_ = NULL;
#line 119 "make-preferences.vala"
		entry = (_tmp10_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL)), _tmp10_);
#line 120 "make-preferences.vala"
		if (valide_make_builder_options_get_dist_options (self->priv->options) != NULL) {
#line 122 "make-preferences.vala"
			gtk_entry_set_text (entry, valide_make_builder_options_get_dist_options (self->priv->options));
#line 475 "make-preferences.c"
		}
#line 124 "make-preferences.vala"
		g_signal_connect ((GtkEditable*) entry, "changed", (GCallback) __lambda4__gtk_editable_changed, self);
#line 128 "make-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) entry, (guint) 1, (guint) 2, (guint) 4, (guint) 5, GTK_EXPAND | GTK_FILL, 0, (guint) 0, (guint) 0);
#line 481 "make-preferences.c"
		/* clean_options */
		_tmp11_ = NULL;
#line 131 "make-preferences.vala"
		label = (_tmp11_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("Clean options:"))), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp11_);
#line 132 "make-preferences.vala"
		gtk_misc_set_alignment ((GtkMisc*) label, 0.0f, 0.5f);
#line 133 "make-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) label, (guint) 0, (guint) 1, (guint) 5, (guint) 6, GTK_FILL, 0, (guint) 0, (guint) 0);
#line 490 "make-preferences.c"
		_tmp12_ = NULL;
#line 135 "make-preferences.vala"
		entry = (_tmp12_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL)), _tmp12_);
#line 136 "make-preferences.vala"
		if (valide_make_builder_options_get_clean_options (self->priv->options) != NULL) {
#line 138 "make-preferences.vala"
			gtk_entry_set_text (entry, valide_make_builder_options_get_clean_options (self->priv->options));
#line 498 "make-preferences.c"
		}
#line 140 "make-preferences.vala"
		g_signal_connect ((GtkEditable*) entry, "changed", (GCallback) __lambda5__gtk_editable_changed, self);
#line 144 "make-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) entry, (guint) 1, (guint) 2, (guint) 5, (guint) 6, GTK_EXPAND | GTK_FILL, 0, (guint) 0, (guint) 0);
#line 504 "make-preferences.c"
		/* distclean_options */
		_tmp13_ = NULL;
#line 147 "make-preferences.vala"
		label = (_tmp13_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("Distclean options:"))), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp13_);
#line 148 "make-preferences.vala"
		gtk_misc_set_alignment ((GtkMisc*) label, 0.0f, 0.5f);
#line 149 "make-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) label, (guint) 0, (guint) 1, (guint) 6, (guint) 7, GTK_FILL, 0, (guint) 0, (guint) 0);
#line 513 "make-preferences.c"
		_tmp14_ = NULL;
#line 151 "make-preferences.vala"
		entry = (_tmp14_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL)), _tmp14_);
#line 152 "make-preferences.vala"
		if (valide_make_builder_options_get_distclean_options (self->priv->options) != NULL) {
#line 154 "make-preferences.vala"
			gtk_entry_set_text (entry, valide_make_builder_options_get_distclean_options (self->priv->options));
#line 521 "make-preferences.c"
		}
#line 156 "make-preferences.vala"
		g_signal_connect ((GtkEditable*) entry, "changed", (GCallback) __lambda6__gtk_editable_changed, self);
#line 160 "make-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) entry, (guint) 1, (guint) 2, (guint) 6, (guint) 7, GTK_EXPAND | GTK_FILL, 0, (guint) 0, (guint) 0);
#line 527 "make-preferences.c"
		/* uninstall_options */
		_tmp15_ = NULL;
#line 163 "make-preferences.vala"
		label = (_tmp15_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("Uninstall options:"))), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp15_);
#line 164 "make-preferences.vala"
		gtk_misc_set_alignment ((GtkMisc*) label, 0.0f, 0.5f);
#line 165 "make-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) label, (guint) 0, (guint) 1, (guint) 7, (guint) 8, GTK_FILL, 0, (guint) 0, (guint) 0);
#line 536 "make-preferences.c"
		_tmp16_ = NULL;
#line 167 "make-preferences.vala"
		entry = (_tmp16_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL)), _tmp16_);
#line 168 "make-preferences.vala"
		if (valide_make_builder_options_get_uninstall_options (self->priv->options) != NULL) {
#line 170 "make-preferences.vala"
			gtk_entry_set_text (entry, valide_make_builder_options_get_uninstall_options (self->priv->options));
#line 544 "make-preferences.c"
		}
#line 172 "make-preferences.vala"
		g_signal_connect ((GtkEditable*) entry, "changed", (GCallback) __lambda7__gtk_editable_changed, self);
#line 176 "make-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) entry, (guint) 1, (guint) 2, (guint) 7, (guint) 8, GTK_EXPAND | GTK_FILL, 0, (guint) 0, (guint) 0);
#line 550 "make-preferences.c"
		(label == NULL) ? NULL : (label = (g_object_unref (label), NULL));
		(entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL));
	}
	return obj;
}


static void make_builder_preferences_class_init (MakeBuilderPreferencesClass * klass) {
	make_builder_preferences_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (MakeBuilderPreferencesPrivate));
	G_OBJECT_CLASS (klass)->get_property = make_builder_preferences_get_property;
	G_OBJECT_CLASS (klass)->set_property = make_builder_preferences_set_property;
	G_OBJECT_CLASS (klass)->constructor = make_builder_preferences_constructor;
	G_OBJECT_CLASS (klass)->finalize = make_builder_preferences_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), MAKE_BUILDER_PREFERENCES_PROJECT, g_param_spec_object ("project", "project", "project", VALIDE_TYPE_PROJECT, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void make_builder_preferences_instance_init (MakeBuilderPreferences * self) {
	self->priv = MAKE_BUILDER_PREFERENCES_GET_PRIVATE (self);
}


static void make_builder_preferences_finalize (GObject* obj) {
	MakeBuilderPreferences * self;
	self = MAKE_BUILDER_PREFERENCES (obj);
	(self->priv->_project == NULL) ? NULL : (self->priv->_project = (g_object_unref (self->priv->_project), NULL));
	G_OBJECT_CLASS (make_builder_preferences_parent_class)->finalize (obj);
}


GType make_builder_preferences_get_type (void) {
	static GType make_builder_preferences_type_id = 0;
	if (make_builder_preferences_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (MakeBuilderPreferencesClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) make_builder_preferences_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (MakeBuilderPreferences), 0, (GInstanceInitFunc) make_builder_preferences_instance_init, NULL };
		make_builder_preferences_type_id = g_type_register_static (GTK_TYPE_TABLE, "MakeBuilderPreferences", &g_define_type_info, 0);
	}
	return make_builder_preferences_type_id;
}


static void make_builder_preferences_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	MakeBuilderPreferences * self;
	gpointer boxed;
	self = MAKE_BUILDER_PREFERENCES (object);
	switch (property_id) {
		case MAKE_BUILDER_PREFERENCES_PROJECT:
		g_value_set_object (value, make_builder_preferences_get_project (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void make_builder_preferences_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	MakeBuilderPreferences * self;
	self = MAKE_BUILDER_PREFERENCES (object);
	switch (property_id) {
		case MAKE_BUILDER_PREFERENCES_PROJECT:
		make_builder_preferences_set_project (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}




