/* opened-documents.vala
 *
 * Copyright (C) 2009 Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <valide-0.0.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <pango/pango.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <glib/gi18n-lib.h>


#define TYPE_OPENED_DOCUMENTS (opened_documents_get_type ())
#define OPENED_DOCUMENTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_OPENED_DOCUMENTS, OpenedDocuments))
#define OPENED_DOCUMENTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_OPENED_DOCUMENTS, OpenedDocumentsClass))
#define IS_OPENED_DOCUMENTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_OPENED_DOCUMENTS))
#define IS_OPENED_DOCUMENTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_OPENED_DOCUMENTS))
#define OPENED_DOCUMENTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_OPENED_DOCUMENTS, OpenedDocumentsClass))

typedef struct _OpenedDocuments OpenedDocuments;
typedef struct _OpenedDocumentsClass OpenedDocumentsClass;
typedef struct _OpenedDocumentsPrivate OpenedDocumentsPrivate;

#define OPENED_DOCUMENTS_TYPE_COL (opened_documents_col_get_type ())

struct _OpenedDocuments {
	GObject parent_instance;
	OpenedDocumentsPrivate * priv;
};

struct _OpenedDocumentsClass {
	GObjectClass parent_class;
};

struct _OpenedDocumentsPrivate {
	GtkVBox* box;
	GtkTreeView* tree_view;
	ValideDocumentManager* documents;
	char* _path;
	ValideWindow* _window;
};

/* enum for tree model columns, not all is currently used */
typedef enum  {
	OPENED_DOCUMENTS_COL_ICON,
	OPENED_DOCUMENTS_COL_NAME,
	OPENED_DOCUMENTS_COL_DOCUMENT,
	OPENED_DOCUMENTS_COL_STYLE,
	OPENED_DOCUMENTS_COL_TOOLTIP,
	OPENED_DOCUMENTS_COL_NB
} OpenedDocumentsCol;


static gpointer opened_documents_parent_class = NULL;
static ValidePluginIface* opened_documents_valide_plugin_parent_iface = NULL;

GType opened_documents_get_type (void);
#define OPENED_DOCUMENTS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_OPENED_DOCUMENTS, OpenedDocumentsPrivate))
enum  {
	OPENED_DOCUMENTS_DUMMY_PROPERTY,
	OPENED_DOCUMENTS_PATH,
	OPENED_DOCUMENTS_WINDOW
};
static GType opened_documents_col_get_type (void);
static void opened_documents_on_tab_changed (OpenedDocuments* self);
static gboolean opened_documents_populate_asc (OpenedDocuments* self);
static gboolean _opened_documents_populate_asc_gsource_func (gpointer self);
static void opened_documents_populate (OpenedDocuments* self);
static void opened_documents_on_cursor_change (OpenedDocuments* self, GtkTreeView* sender);
OpenedDocuments* opened_documents_new (void);
OpenedDocuments* opened_documents_construct (GType object_type);
static void _opened_documents_on_cursor_change_gtk_tree_view_cursor_changed (GtkTreeView* _sender, gpointer self);
static void _opened_documents_populate_valide_document_manager_tab_removed (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self);
static void _opened_documents_populate_valide_document_manager_tab_added (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self);
static void _opened_documents_populate_valide_document_manager_tab_state_changed (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self);
static void _opened_documents_on_tab_changed_valide_document_manager_tab_changed (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self);
static GObject * opened_documents_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void opened_documents_finalize (GObject* obj);
static void opened_documents_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void opened_documents_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType register_plugin (void);




static GType opened_documents_col_get_type (void) {
	static GType opened_documents_col_type_id = 0;
	if (G_UNLIKELY (opened_documents_col_type_id == 0)) {
		static const GEnumValue values[] = {{OPENED_DOCUMENTS_COL_ICON, "OPENED_DOCUMENTS_COL_ICON", "icon"}, {OPENED_DOCUMENTS_COL_NAME, "OPENED_DOCUMENTS_COL_NAME", "name"}, {OPENED_DOCUMENTS_COL_DOCUMENT, "OPENED_DOCUMENTS_COL_DOCUMENT", "document"}, {OPENED_DOCUMENTS_COL_STYLE, "OPENED_DOCUMENTS_COL_STYLE", "style"}, {OPENED_DOCUMENTS_COL_TOOLTIP, "OPENED_DOCUMENTS_COL_TOOLTIP", "tooltip"}, {OPENED_DOCUMENTS_COL_NB, "OPENED_DOCUMENTS_COL_NB", "nb"}, {0, NULL, NULL}};
		opened_documents_col_type_id = g_enum_register_static ("OpenedDocumentsCol", values);
	}
	return opened_documents_col_type_id;
}


#line 44 "opened-documents.vala"
static gboolean opened_documents_populate_asc (OpenedDocuments* self) {
#line 119 "opened-documents.c"
	gboolean result;
	GtkTreeIter iter = {0};
	GtkListStore* model;
	GtkListStore* _tmp2_;
	GtkListStore* _tmp1_;
	GtkTreeModel* _tmp0_;
#line 44 "opened-documents.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 128 "opened-documents.c"
	model = NULL;
	_tmp2_ = NULL;
#line 49 "opened-documents.vala"
	_tmp1_ = NULL;
#line 49 "opened-documents.vala"
	_tmp0_ = NULL;
#line 49 "opened-documents.vala"
	model = (_tmp2_ = (_tmp1_ = (_tmp0_ = gtk_tree_view_get_model (self->priv->tree_view), GTK_IS_LIST_STORE (_tmp0_) ? ((GtkListStore*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (model == NULL) ? NULL : (model = (g_object_unref (model), NULL)), _tmp2_);
#line 50 "opened-documents.vala"
	gtk_list_store_clear (model);
#line 139 "opened-documents.c"
	{
		GList* document_collection;
		GList* document_it;
#line 52 "opened-documents.vala"
		document_collection = valide_document_manager_get_documents (self->priv->documents);
#line 145 "opened-documents.c"
		for (document_it = document_collection; document_it != NULL; document_it = document_it->next) {
			ValideiDocument* _tmp5_;
			ValideiDocument* document;
#line 509 "valide-0.0.vapi"
			_tmp5_ = NULL;
#line 52 "opened-documents.vala"
			document = (_tmp5_ = (ValideiDocument*) document_it->data, (_tmp5_ == NULL) ? NULL : g_object_ref (_tmp5_));
#line 153 "opened-documents.c"
			{
				char* tooltip;
				PangoStyle style;
				char* _tmp3_;
				GdkPixbuf* _tmp4_;
				tooltip = NULL;
				style = 0;
#line 57 "opened-documents.vala"
				if (valide_idocument_get_is_save (document)) {
#line 59 "opened-documents.vala"
					style = PANGO_STYLE_NORMAL;
#line 165 "opened-documents.c"
				} else {
#line 63 "opened-documents.vala"
					style = PANGO_STYLE_ITALIC;
#line 169 "opened-documents.c"
				}
				_tmp3_ = NULL;
#line 65 "opened-documents.vala"
				tooltip = (_tmp3_ = valide_idocument_get_tooltips (document), tooltip = (g_free (tooltip), NULL), _tmp3_);
#line 66 "opened-documents.vala"
				gtk_list_store_append (model, &iter);
#line 1028 "valide-0.0.vapi"
				_tmp4_ = NULL;
#line 67 "opened-documents.vala"
				gtk_list_store_set (model, &iter, OPENED_DOCUMENTS_COL_ICON, _tmp4_ = valide_idocument_get_icon (document), OPENED_DOCUMENTS_COL_NAME, valide_idocument_get_filename (document), OPENED_DOCUMENTS_COL_DOCUMENT, document, OPENED_DOCUMENTS_COL_STYLE, style, OPENED_DOCUMENTS_COL_TOOLTIP, tooltip, -1);
#line 180 "opened-documents.c"
				(_tmp4_ == NULL) ? NULL : (_tmp4_ = (g_object_unref (_tmp4_), NULL));
				(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
				tooltip = (g_free (tooltip), NULL);
			}
		}
	}
#line 73 "opened-documents.vala"
	opened_documents_on_tab_changed (self);
#line 189 "opened-documents.c"
	result = FALSE;
	(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
	return result;
}


#line 44 "opened-documents.vala"
static gboolean _opened_documents_populate_asc_gsource_func (gpointer self) {
#line 198 "opened-documents.c"
	return opened_documents_populate_asc (self);
}


#line 77 "opened-documents.vala"
static void opened_documents_populate (OpenedDocuments* self) {
#line 77 "opened-documents.vala"
	g_return_if_fail (self != NULL);
#line 79 "opened-documents.vala"
	g_idle_add (_opened_documents_populate_asc_gsource_func, self);
#line 209 "opened-documents.c"
}


#line 82 "opened-documents.vala"
static void opened_documents_on_cursor_change (OpenedDocuments* self, GtkTreeView* sender) {
#line 215 "opened-documents.c"
	ValideiDocument* document;
	GtkTreeModel* model;
	GtkTreeIter iter = {0};
	GtkTreeSelection* selection;
	GtkTreeSelection* _tmp1_;
	GtkTreeSelection* _tmp0_;
	GtkTreeModel* _tmp5_;
	GtkTreeModel* _tmp4_;
	gboolean _tmp3_;
	GtkTreeModel* _tmp2_;
#line 82 "opened-documents.vala"
	g_return_if_fail (self != NULL);
#line 82 "opened-documents.vala"
	g_return_if_fail (sender != NULL);
#line 230 "opened-documents.c"
	document = NULL;
	model = NULL;
	selection = NULL;
	_tmp1_ = NULL;
#line 89 "opened-documents.vala"
	_tmp0_ = NULL;
#line 89 "opened-documents.vala"
	selection = (_tmp1_ = (_tmp0_ = gtk_tree_view_get_selection (self->priv->tree_view), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL)), _tmp1_);
#line 91 "opened-documents.vala"
	_tmp5_ = NULL;
#line 241 "opened-documents.c"
	_tmp4_ = NULL;
	_tmp2_ = NULL;
#line 91 "opened-documents.vala"
	if ((_tmp3_ = gtk_tree_selection_get_selected (selection, &_tmp2_, &iter), model = (_tmp4_ = (_tmp5_ = _tmp2_, (_tmp5_ == NULL) ? NULL : g_object_ref (_tmp5_)), (model == NULL) ? NULL : (model = (g_object_unref (model), NULL)), _tmp4_), _tmp3_)) {
#line 93 "opened-documents.vala"
		gtk_tree_model_get (model, &iter, OPENED_DOCUMENTS_COL_DOCUMENT, &document, -1);
#line 94 "opened-documents.vala"
		valide_document_manager_set_current (self->priv->documents, document);
#line 250 "opened-documents.c"
	}
	(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
	(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
	(selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL));
}


#line 98 "opened-documents.vala"
static void opened_documents_on_tab_changed (OpenedDocuments* self) {
#line 260 "opened-documents.c"
	GtkTreeIter iter = {0};
	ValideiDocument* document;
	GtkListStore* list_store;
	GtkListStore* _tmp2_;
	GtkListStore* _tmp1_;
	GtkTreeModel* _tmp0_;
#line 98 "opened-documents.vala"
	g_return_if_fail (self != NULL);
#line 269 "opened-documents.c"
	document = NULL;
	list_store = NULL;
	_tmp2_ = NULL;
#line 104 "opened-documents.vala"
	_tmp1_ = NULL;
#line 104 "opened-documents.vala"
	_tmp0_ = NULL;
#line 104 "opened-documents.vala"
	list_store = (_tmp2_ = (_tmp1_ = (_tmp0_ = gtk_tree_view_get_model (self->priv->tree_view), GTK_IS_LIST_STORE (_tmp0_) ? ((GtkListStore*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (list_store == NULL) ? NULL : (list_store = (g_object_unref (list_store), NULL)), _tmp2_);
#line 105 "opened-documents.vala"
	if (gtk_tree_model_get_iter_first ((GtkTreeModel*) list_store, &iter)) {
#line 281 "opened-documents.c"
		{
			gboolean _tmp3_;
			_tmp3_ = TRUE;
#line 107 "opened-documents.vala"
			while (TRUE) {
#line 107 "opened-documents.vala"
				if (!_tmp3_) {
#line 115 "opened-documents.vala"
					if (!gtk_tree_model_iter_next ((GtkTreeModel*) list_store, &iter)) {
#line 115 "opened-documents.vala"
						break;
#line 293 "opened-documents.c"
					}
				}
#line 107 "opened-documents.vala"
				_tmp3_ = FALSE;
#line 109 "opened-documents.vala"
				gtk_tree_model_get ((GtkTreeModel*) list_store, &iter, OPENED_DOCUMENTS_COL_DOCUMENT, &document, -1);
#line 110 "opened-documents.vala"
				if (document == valide_document_manager_get_current (self->priv->documents)) {
#line 112 "opened-documents.vala"
					gtk_tree_selection_select_iter (gtk_tree_view_get_selection (self->priv->tree_view), &iter);
#line 113 "opened-documents.vala"
					break;
#line 306 "opened-documents.c"
				}
			}
		}
	}
	(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
	(list_store == NULL) ? NULL : (list_store = (g_object_unref (list_store), NULL));
}


#line 24 "opened-documents.vala"
OpenedDocuments* opened_documents_construct (GType object_type) {
#line 318 "opened-documents.c"
	OpenedDocuments * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


#line 24 "opened-documents.vala"
OpenedDocuments* opened_documents_new (void) {
#line 24 "opened-documents.vala"
	return opened_documents_construct (TYPE_OPENED_DOCUMENTS);
#line 329 "opened-documents.c"
}


static const char* opened_documents_real_get_path (ValidePlugin* base) {
	const char* result;
	OpenedDocuments* self;
	self = (OpenedDocuments*) base;
	result = self->priv->_path;
	return result;
}


static void opened_documents_real_set_path (ValidePlugin* base, const char* value) {
	OpenedDocuments* self;
	char* _tmp1_;
	const char* _tmp0_;
	self = (OpenedDocuments*) base;
	_tmp1_ = NULL;
#line 41 "opened-documents.vala"
	_tmp0_ = NULL;
#line 350 "opened-documents.c"
	self->priv->_path = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_path = (g_free (self->priv->_path), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "path");
}


static ValideWindow* opened_documents_real_get_window (ValidePlugin* base) {
	ValideWindow* result;
	OpenedDocuments* self;
	self = (OpenedDocuments*) base;
	result = self->priv->_window;
	return result;
}


static void opened_documents_real_set_window (ValidePlugin* base, ValideWindow* value) {
	OpenedDocuments* self;
	ValideWindow* _tmp1_;
	ValideWindow* _tmp0_;
	self = (OpenedDocuments*) base;
	_tmp1_ = NULL;
#line 42 "opened-documents.vala"
	_tmp0_ = NULL;
#line 373 "opened-documents.c"
	self->priv->_window = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "window");
}


#line 82 "opened-documents.vala"
static void _opened_documents_on_cursor_change_gtk_tree_view_cursor_changed (GtkTreeView* _sender, gpointer self) {
#line 381 "opened-documents.c"
	opened_documents_on_cursor_change (self, _sender);
}


#line 77 "opened-documents.vala"
static void _opened_documents_populate_valide_document_manager_tab_removed (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self) {
#line 388 "opened-documents.c"
	opened_documents_populate (self);
}


#line 77 "opened-documents.vala"
static void _opened_documents_populate_valide_document_manager_tab_added (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self) {
#line 395 "opened-documents.c"
	opened_documents_populate (self);
}


#line 77 "opened-documents.vala"
static void _opened_documents_populate_valide_document_manager_tab_state_changed (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self) {
#line 402 "opened-documents.c"
	opened_documents_populate (self);
}


#line 98 "opened-documents.vala"
static void _opened_documents_on_tab_changed_valide_document_manager_tab_changed (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self) {
#line 409 "opened-documents.c"
	opened_documents_on_tab_changed (self);
}


static GObject * opened_documents_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	OpenedDocumentsClass * klass;
	GObjectClass * parent_class;
	OpenedDocuments * self;
	klass = OPENED_DOCUMENTS_CLASS (g_type_class_peek (TYPE_OPENED_DOCUMENTS));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = OPENED_DOCUMENTS (obj);
	{
		GtkImage* image;
		GtkListStore* store;
		GtkTreeViewColumn* col;
		GtkCellRenderer* renderer;
		GtkScrolledWindow* scrolled;
		GtkVBox* _tmp0_;
		GtkScrolledWindow* _tmp1_;
		GtkTreeView* _tmp2_;
		GtkListStore* _tmp3_;
		GtkTreeViewColumn* _tmp4_;
		GtkCellRenderer* _tmp5_;
		GtkCellRenderer* _tmp6_;
		ValideDocumentManager* _tmp8_;
		ValideDocumentManager* _tmp7_;
		GtkImage* _tmp10_;
		char* _tmp9_;
		image = NULL;
		store = NULL;
		col = NULL;
		renderer = NULL;
		scrolled = NULL;
		_tmp0_ = NULL;
#line 127 "opened-documents.vala"
		self->priv->box = (_tmp0_ = g_object_ref_sink ((GtkVBox*) gtk_vbox_new (FALSE, 0)), (self->priv->box == NULL) ? NULL : (self->priv->box = (g_object_unref (self->priv->box), NULL)), _tmp0_);
#line 448 "opened-documents.c"
		/* create some nice padding */
#line 130 "opened-documents.vala"
		gtk_box_set_spacing ((GtkBox*) self->priv->box, 2);
#line 131 "opened-documents.vala"
		gtk_container_set_border_width ((GtkContainer*) self->priv->box, (guint) 2);
#line 454 "opened-documents.c"
		/* create and pack tree view */
		_tmp1_ = NULL;
#line 134 "opened-documents.vala"
		scrolled = (_tmp1_ = g_object_ref_sink ((GtkScrolledWindow*) gtk_scrolled_window_new (NULL, NULL)), (scrolled == NULL) ? NULL : (scrolled = (g_object_unref (scrolled), NULL)), _tmp1_);
#line 135 "opened-documents.vala"
		gtk_scrolled_window_set_policy (scrolled, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
#line 136 "opened-documents.vala"
		gtk_scrolled_window_set_shadow_type (scrolled, GTK_SHADOW_IN);
#line 137 "opened-documents.vala"
		gtk_box_pack_start ((GtkBox*) self->priv->box, (GtkWidget*) scrolled, TRUE, TRUE, (guint) 0);
#line 465 "opened-documents.c"
		_tmp2_ = NULL;
#line 139 "opened-documents.vala"
		self->priv->tree_view = (_tmp2_ = g_object_ref_sink ((GtkTreeView*) gtk_tree_view_new ()), (self->priv->tree_view == NULL) ? NULL : (self->priv->tree_view = (g_object_unref (self->priv->tree_view), NULL)), _tmp2_);
#line 140 "opened-documents.vala"
		gtk_tree_view_set_rules_hint (self->priv->tree_view, FALSE);
#line 141 "opened-documents.vala"
		gtk_tree_view_set_headers_visible (self->priv->tree_view, FALSE);
#line 142 "opened-documents.vala"
		gtk_container_add ((GtkContainer*) scrolled, (GtkWidget*) self->priv->tree_view);
#line 475 "opened-documents.c"
		/* create model */
		_tmp3_ = NULL;
#line 145 "opened-documents.vala"
		store = (_tmp3_ = gtk_list_store_new ((gint) OPENED_DOCUMENTS_COL_NB, GDK_TYPE_PIXBUF, G_TYPE_STRING, VALIDE_TYPE_IDOCUMENT, PANGO_TYPE_STYLE, G_TYPE_STRING, NULL), (store == NULL) ? NULL : (store = (g_object_unref (store), NULL)), _tmp3_);
#line 148 "opened-documents.vala"
		gtk_tree_sortable_set_sort_column_id ((GtkTreeSortable*) store, (gint) OPENED_DOCUMENTS_COL_NAME, GTK_SORT_ASCENDING);
#line 482 "opened-documents.c"
		/* main column */
		_tmp4_ = NULL;
#line 151 "opened-documents.vala"
		col = (_tmp4_ = g_object_ref_sink (gtk_tree_view_column_new ()), (col == NULL) ? NULL : (col = (g_object_unref (col), NULL)), _tmp4_);
#line 152 "opened-documents.vala"
		gtk_tree_view_column_set_title (col, _ ("File"));
#line 489 "opened-documents.c"
		_tmp5_ = NULL;
#line 154 "opened-documents.vala"
		renderer = (_tmp5_ = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererPixbuf*) gtk_cell_renderer_pixbuf_new ()), (renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL)), _tmp5_);
#line 155 "opened-documents.vala"
		gtk_cell_layout_pack_start ((GtkCellLayout*) col, renderer, FALSE);
#line 156 "opened-documents.vala"
		gtk_tree_view_column_set_attributes (col, renderer, "pixbuf", OPENED_DOCUMENTS_COL_ICON, NULL);
#line 497 "opened-documents.c"
		_tmp6_ = NULL;
#line 158 "opened-documents.vala"
		renderer = (_tmp6_ = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL)), _tmp6_);
#line 159 "opened-documents.vala"
		gtk_cell_layout_pack_start ((GtkCellLayout*) col, renderer, FALSE);
#line 160 "opened-documents.vala"
		gtk_tree_view_column_set_attributes (col, renderer, "markup", OPENED_DOCUMENTS_COL_NAME, "style", OPENED_DOCUMENTS_COL_STYLE, NULL);
#line 162 "opened-documents.vala"
		gtk_tree_view_append_column (self->priv->tree_view, col);
#line 163 "opened-documents.vala"
		gtk_tree_view_set_model (self->priv->tree_view, (GtkTreeModel*) store);
#line 165 "opened-documents.vala"
		gtk_tree_view_set_tooltip_column (self->priv->tree_view, (gint) OPENED_DOCUMENTS_COL_TOOLTIP);
#line 511 "opened-documents.c"
		/* attach signal callbacks for tree view */
#line 168 "opened-documents.vala"
		g_signal_connect_object (self->priv->tree_view, "cursor-changed", (GCallback) _opened_documents_on_cursor_change_gtk_tree_view_cursor_changed, self, 0);
#line 515 "opened-documents.c"
		_tmp8_ = NULL;
#line 170 "opened-documents.vala"
		_tmp7_ = NULL;
#line 170 "opened-documents.vala"
		self->priv->documents = (_tmp8_ = (_tmp7_ = valide_window_get_documents (valide_plugin_get_window ((ValidePlugin*) self)), (_tmp7_ == NULL) ? NULL : g_object_ref (_tmp7_)), (self->priv->documents == NULL) ? NULL : (self->priv->documents = (g_object_unref (self->priv->documents), NULL)), _tmp8_);
#line 172 "opened-documents.vala"
		g_signal_connect_object (self->priv->documents, "tab-removed", (GCallback) _opened_documents_populate_valide_document_manager_tab_removed, self, 0);
#line 173 "opened-documents.vala"
		g_signal_connect_object (self->priv->documents, "tab-added", (GCallback) _opened_documents_populate_valide_document_manager_tab_added, self, 0);
#line 174 "opened-documents.vala"
		g_signal_connect_object (self->priv->documents, "tab-state-changed", (GCallback) _opened_documents_populate_valide_document_manager_tab_state_changed, self, 0);
#line 175 "opened-documents.vala"
		g_signal_connect_object (self->priv->documents, "tab-changed", (GCallback) _opened_documents_on_tab_changed_valide_document_manager_tab_changed, self, 0);
#line 529 "opened-documents.c"
		_tmp10_ = NULL;
#line 2034 "glib-2.0.vapi"
		_tmp9_ = NULL;
#line 177 "opened-documents.vala"
		image = (_tmp10_ = g_object_ref_sink ((GtkImage*) gtk_image_new_from_file (_tmp9_ = g_build_filename (valide_plugin_get_path ((ValidePlugin*) self), "opened-documents.png", NULL))), (image == NULL) ? NULL : (image = (g_object_unref (image), NULL)), _tmp10_);
#line 535 "opened-documents.c"
		_tmp9_ = (g_free (_tmp9_), NULL);
#line 179 "opened-documents.vala"
		valide_panel_add_item (valide_window_get_side_panel (valide_plugin_get_window ((ValidePlugin*) self)), (GtkWidget*) self->priv->box, _ ("Documents"), image);
#line 181 "opened-documents.vala"
		opened_documents_populate (self);
#line 182 "opened-documents.vala"
		gtk_widget_show_all ((GtkWidget*) self->priv->box);
#line 543 "opened-documents.c"
		(image == NULL) ? NULL : (image = (g_object_unref (image), NULL));
		(store == NULL) ? NULL : (store = (g_object_unref (store), NULL));
		(col == NULL) ? NULL : (col = (g_object_unref (col), NULL));
		(renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL));
		(scrolled == NULL) ? NULL : (scrolled = (g_object_unref (scrolled), NULL));
	}
	return obj;
}


static void opened_documents_class_init (OpenedDocumentsClass * klass) {
	opened_documents_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (OpenedDocumentsPrivate));
	G_OBJECT_CLASS (klass)->get_property = opened_documents_get_property;
	G_OBJECT_CLASS (klass)->set_property = opened_documents_set_property;
	G_OBJECT_CLASS (klass)->constructor = opened_documents_constructor;
	G_OBJECT_CLASS (klass)->finalize = opened_documents_finalize;
	g_object_class_override_property (G_OBJECT_CLASS (klass), OPENED_DOCUMENTS_PATH, "path");
	g_object_class_override_property (G_OBJECT_CLASS (klass), OPENED_DOCUMENTS_WINDOW, "window");
}


static void opened_documents_valide_plugin_interface_init (ValidePluginIface * iface) {
	opened_documents_valide_plugin_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_path = opened_documents_real_get_path;
	iface->set_path = opened_documents_real_set_path;
	iface->get_window = opened_documents_real_get_window;
	iface->set_window = opened_documents_real_set_window;
}


static void opened_documents_instance_init (OpenedDocuments * self) {
	self->priv = OPENED_DOCUMENTS_GET_PRIVATE (self);
}


static void opened_documents_finalize (GObject* obj) {
	OpenedDocuments * self;
	self = OPENED_DOCUMENTS (obj);
	{
		guint _tmp11_;
		guint _tmp12_;
		guint _tmp13_;
		guint _tmp14_;
#line 187 "opened-documents.vala"
		g_signal_handlers_disconnect_matched (self->priv->documents, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("tab-removed", VALIDE_TYPE_DOCUMENT_MANAGER, &_tmp11_, NULL, FALSE), _tmp11_), 0, NULL, (GCallback) _opened_documents_populate_valide_document_manager_tab_removed, self);
#line 188 "opened-documents.vala"
		g_signal_handlers_disconnect_matched (self->priv->documents, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("tab-added", VALIDE_TYPE_DOCUMENT_MANAGER, &_tmp12_, NULL, FALSE), _tmp12_), 0, NULL, (GCallback) _opened_documents_populate_valide_document_manager_tab_added, self);
#line 189 "opened-documents.vala"
		g_signal_handlers_disconnect_matched (self->priv->documents, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("tab-state-changed", VALIDE_TYPE_DOCUMENT_MANAGER, &_tmp13_, NULL, FALSE), _tmp13_), 0, NULL, (GCallback) _opened_documents_populate_valide_document_manager_tab_state_changed, self);
#line 190 "opened-documents.vala"
		g_signal_handlers_disconnect_matched (self->priv->documents, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("tab-changed", VALIDE_TYPE_DOCUMENT_MANAGER, &_tmp14_, NULL, FALSE), _tmp14_), 0, NULL, (GCallback) _opened_documents_on_tab_changed_valide_document_manager_tab_changed, self);
#line 191 "opened-documents.vala"
		valide_panel_remove_item (valide_window_get_side_panel (valide_plugin_get_window ((ValidePlugin*) self)), (GtkWidget*) self->priv->box);
#line 598 "opened-documents.c"
	}
	(self->priv->box == NULL) ? NULL : (self->priv->box = (g_object_unref (self->priv->box), NULL));
	(self->priv->tree_view == NULL) ? NULL : (self->priv->tree_view = (g_object_unref (self->priv->tree_view), NULL));
	(self->priv->documents == NULL) ? NULL : (self->priv->documents = (g_object_unref (self->priv->documents), NULL));
	self->priv->_path = (g_free (self->priv->_path), NULL);
	(self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL));
	G_OBJECT_CLASS (opened_documents_parent_class)->finalize (obj);
}


GType opened_documents_get_type (void) {
	static GType opened_documents_type_id = 0;
	if (opened_documents_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (OpenedDocumentsClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) opened_documents_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (OpenedDocuments), 0, (GInstanceInitFunc) opened_documents_instance_init, NULL };
		static const GInterfaceInfo valide_plugin_info = { (GInterfaceInitFunc) opened_documents_valide_plugin_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		opened_documents_type_id = g_type_register_static (G_TYPE_OBJECT, "OpenedDocuments", &g_define_type_info, 0);
		g_type_add_interface_static (opened_documents_type_id, VALIDE_TYPE_PLUGIN, &valide_plugin_info);
	}
	return opened_documents_type_id;
}


static void opened_documents_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	OpenedDocuments * self;
	gpointer boxed;
	self = OPENED_DOCUMENTS (object);
	switch (property_id) {
		case OPENED_DOCUMENTS_PATH:
		g_value_set_string (value, valide_plugin_get_path ((ValidePlugin*) self));
		break;
		case OPENED_DOCUMENTS_WINDOW:
		g_value_set_object (value, valide_plugin_get_window ((ValidePlugin*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void opened_documents_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	OpenedDocuments * self;
	self = OPENED_DOCUMENTS (object);
	switch (property_id) {
		case OPENED_DOCUMENTS_PATH:
		valide_plugin_set_path ((ValidePlugin*) self, g_value_get_string (value));
		break;
		case OPENED_DOCUMENTS_WINDOW:
		valide_plugin_set_window ((ValidePlugin*) self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


#line 196 "opened-documents.vala"
GType register_plugin (void) {
#line 658 "opened-documents.c"
	GType result;
	result = TYPE_OPENED_DOCUMENTS;
	return result;
}




