/* symbol-browser.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <valide-0.0.h>
#include <gtk/gtk.h>
#include <config.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <glib/gstdio.h>
#include <glib/gi18n-lib.h>
#include <gobject/gvaluecollector.h>


#define TYPE_SYMBOL (symbol_get_type ())
#define SYMBOL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SYMBOL, Symbol))
#define SYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SYMBOL, SymbolClass))
#define IS_SYMBOL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SYMBOL))
#define IS_SYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SYMBOL))
#define SYMBOL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SYMBOL, SymbolClass))

typedef struct _Symbol Symbol;
typedef struct _SymbolClass SymbolClass;
typedef struct _SymbolPrivate SymbolPrivate;
typedef struct _ParamSpecSymbol ParamSpecSymbol;

#define TYPE_SYMBOL_BROWSER (symbol_browser_get_type ())
#define SYMBOL_BROWSER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SYMBOL_BROWSER, SymbolBrowser))
#define SYMBOL_BROWSER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SYMBOL_BROWSER, SymbolBrowserClass))
#define IS_SYMBOL_BROWSER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SYMBOL_BROWSER))
#define IS_SYMBOL_BROWSER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SYMBOL_BROWSER))
#define SYMBOL_BROWSER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SYMBOL_BROWSER, SymbolBrowserClass))

typedef struct _SymbolBrowser SymbolBrowser;
typedef struct _SymbolBrowserClass SymbolBrowserClass;
typedef struct _SymbolBrowserPrivate SymbolBrowserPrivate;

#define SYMBOL_BROWSER_TYPE_COL (symbol_browser_col_get_type ())

/*
 * This code is re-writing in the vala language of GeditSymbolBrowserPlugin
 *
 * Copyright (C) 2007 - Micah Carrick
 
 just used to keep the population code readable */
struct _Symbol {
	GTypeInstance parent_instance;
	volatile int ref_count;
	SymbolPrivate * priv;
	char* name;
	char* full_name;
	char* signature;
	char** inherits;
	gint inherits_length1;
	gint inherits_size;
	char* typeref;
	char* access;
	char* implementation;
	char* type;
	char* file;
	char* language;
	gint line;
	Symbol* root;
	ValideAbstractTextDocument* document;
};

struct _SymbolClass {
	GTypeClass parent_class;
	void (*finalize) (Symbol *self);
};

struct _ParamSpecSymbol {
	GParamSpec parent_instance;
};

typedef enum  {
	SYMBOL_ERROR_KIND
} SymbolError;
#define SYMBOL_ERROR symbol_error_quark ()
struct _SymbolBrowser {
	GObject parent_instance;
	SymbolBrowserPrivate * priv;
};

struct _SymbolBrowserClass {
	GObjectClass parent_class;
};

struct _SymbolBrowserPrivate {
	GtkVBox* box;
	GtkTreeView* tree_view;
	GtkTreeStore* model;
	GtkLabel* label;
	GSList* expanded_rows;
	guint status_context;
	GtkTreeViewColumn* language_column;
	GtkTreeViewColumn* source_file_column;
	GtkTreeViewColumn* line_number_column;
	GHashTable* root_nodes;
	gint atom;
	char* _path;
	ValideWindow* _window;
};

/* enum for tree model columns, not all is currently used */
typedef enum  {
	SYMBOL_BROWSER_COL_ICON,
	SYMBOL_BROWSER_COL_NAME,
	SYMBOL_BROWSER_COL_FILE,
	SYMBOL_BROWSER_COL_LINE,
	SYMBOL_BROWSER_COL_LANGUAGE,
	SYMBOL_BROWSER_COL_NB
} SymbolBrowserCol;


static gpointer symbol_parent_class = NULL;
static gpointer symbol_browser_parent_class = NULL;
static ValidePluginIface* symbol_browser_valide_plugin_parent_iface = NULL;

gpointer symbol_ref (gpointer instance);
void symbol_unref (gpointer instance);
GParamSpec* param_spec_symbol (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_symbol (GValue* value, gpointer v_object);
gpointer value_get_symbol (const GValue* value);
GType symbol_get_type (void);
enum  {
	SYMBOL_DUMMY_PROPERTY
};
Symbol* symbol_new (void);
Symbol* symbol_construct (GType object_type);
static void symbol_finalize (Symbol* obj);
GQuark symbol_error_quark (void);
GType symbol_browser_get_type (void);
#define SYMBOL_BROWSER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_SYMBOL_BROWSER, SymbolBrowserPrivate))
enum  {
	SYMBOL_BROWSER_DUMMY_PROPERTY,
	SYMBOL_BROWSER_PATH,
	SYMBOL_BROWSER_WINDOW
};
static GType symbol_browser_col_get_type (void);
static void _g_slist_free_g_free (GSList* self);
static void symbol_browser_save_expanded_rows (SymbolBrowser* self);
static gboolean symbol_browser_is_expanded_row (SymbolBrowser* self, const char* name);
static void symbol_browser_add_expanded_row (SymbolBrowser* self, const char* name);
static void symbol_browser_remove_expanded_row (SymbolBrowser* self, const char* name);
static char* symbol_browser_exec_ctags (SymbolBrowser* self, const char* filename, GError** error);
static char** _vala_array_dup1 (char** self, int length);
static Symbol* symbol_browser_parse_line (SymbolBrowser* self, const char* line);
static void symbol_browser_clear_data (SymbolBrowser* self);
static gboolean symbol_browser_get_hierarchic_view (SymbolBrowser* self);
static GtkTreeIter* _gtk_tree_iter_dup (GtkTreeIter* self);
static gboolean symbol_browser_add_symbol (SymbolBrowser* self, Symbol* symbol);
static void _g_list_free_g_object_unref (GList* self);
static gboolean symbol_browser_get_only_load_active_tab (SymbolBrowser* self);
static gboolean symbol_browser_populate_symbols_asc (SymbolBrowser* self);
static gboolean _symbol_browser_populate_symbols_asc_gsource_func (gpointer self);
static void symbol_browser_populate_symbols (SymbolBrowser* self);
static void symbol_browser_restore_expanded_rows (SymbolBrowser* self);
static void symbol_browser_on_tab_changed (SymbolBrowser* self);
static void symbol_browser_on_row_activated (SymbolBrowser* self, GtkTreeView* sender, const GtkTreePath* path, GtkTreeViewColumn* column);
static void symbol_browser_on_cursor_changed (SymbolBrowser* self, GtkTreeView* sender);
static void symbol_browser_on_row_expanded (SymbolBrowser* self, GtkTreeView* sender, GtkTreeIter* iter, const GtkTreePath* path);
static void symbol_browser_on_row_collapsed (SymbolBrowser* self, GtkTreeView* sender, GtkTreeIter* iter, const GtkTreePath* path);
SymbolBrowser* symbol_browser_new (const char* path);
SymbolBrowser* symbol_browser_construct (GType object_type, const char* path);
static gboolean symbol_browser_get_show_language_column (SymbolBrowser* self);
static void symbol_browser_set_show_language_column (SymbolBrowser* self, gboolean value);
static void _lambda0_ (GtkToggleButton* s, SymbolBrowser* self);
static void __lambda0__gtk_toggle_button_toggled (GtkToggleButton* _sender, gpointer self);
static gboolean symbol_browser_get_show_source_file_column (SymbolBrowser* self);
static void symbol_browser_set_show_source_file_column (SymbolBrowser* self, gboolean value);
static void _lambda1_ (GtkToggleButton* s, SymbolBrowser* self);
static void __lambda1__gtk_toggle_button_toggled (GtkToggleButton* _sender, gpointer self);
static gboolean symbol_browser_get_show_line_number_column (SymbolBrowser* self);
static void symbol_browser_set_show_line_number_column (SymbolBrowser* self, gboolean value);
static void _lambda2_ (GtkToggleButton* s, SymbolBrowser* self);
static void __lambda2__gtk_toggle_button_toggled (GtkToggleButton* _sender, gpointer self);
static void symbol_browser_set_hierarchic_view (SymbolBrowser* self, gboolean value);
static void _lambda3_ (GtkToggleButton* s, SymbolBrowser* self);
static void __lambda3__gtk_toggle_button_toggled (GtkToggleButton* _sender, gpointer self);
GtkWidget* symbol_browser_create_configure_dialog (SymbolBrowser* self);
static void _symbol_browser_on_row_activated_gtk_tree_view_row_activated (GtkTreeView* _sender, const GtkTreePath* path, GtkTreeViewColumn* column, gpointer self);
static void _symbol_browser_on_cursor_changed_gtk_tree_view_cursor_changed (GtkTreeView* _sender, gpointer self);
static void _symbol_browser_on_row_expanded_gtk_tree_view_row_expanded (GtkTreeView* _sender, GtkTreeIter* iter, const GtkTreePath* path, gpointer self);
static void _symbol_browser_on_row_collapsed_gtk_tree_view_row_collapsed (GtkTreeView* _sender, GtkTreeIter* iter, const GtkTreePath* path, gpointer self);
static void _symbol_browser_populate_symbols_valide_document_manager_tab_removed (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self);
static void _symbol_browser_on_tab_changed_valide_document_manager_tab_changed (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self);
static void _symbol_browser_on_tab_changed_valide_document_manager_tab_saved (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self);
static GObject * symbol_browser_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void symbol_browser_finalize (GObject* obj);
static void symbol_browser_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void symbol_browser_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType register_plugin (void);
GtkWidget* create_configure_dialog (SymbolBrowser* _self_);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);



/*
 * This code is re-writing in the vala language of GeditSymbolBrowserPlugin
 *
 * Copyright (C) 2007 - Micah Carrick
 
 just used to keep the population code readable */
#line 31 "symbol.vala"
Symbol* symbol_construct (GType object_type) {
#line 231 "symbol.c"
	Symbol* self;
	self = (Symbol*) g_type_create_instance (object_type);
	return self;
}


#line 31 "symbol.vala"
Symbol* symbol_new (void) {
#line 31 "symbol.vala"
	return symbol_construct (TYPE_SYMBOL);
#line 242 "symbol.c"
}


static void value_symbol_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void value_symbol_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		symbol_unref (value->data[0].v_pointer);
	}
}


static void value_symbol_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = symbol_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer value_symbol_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* value_symbol_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		Symbol* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = symbol_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* value_symbol_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	Symbol** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = symbol_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* param_spec_symbol (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ParamSpecSymbol* spec;
	g_return_val_if_fail (g_type_is_a (object_type, TYPE_SYMBOL), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer value_get_symbol (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_SYMBOL), NULL);
	return value->data[0].v_pointer;
}


void value_set_symbol (GValue* value, gpointer v_object) {
	Symbol* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_SYMBOL));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TYPE_SYMBOL));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		symbol_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		symbol_unref (old);
	}
}


static void symbol_class_init (SymbolClass * klass) {
	symbol_parent_class = g_type_class_peek_parent (klass);
	SYMBOL_CLASS (klass)->finalize = symbol_finalize;
}


static void symbol_instance_init (Symbol * self) {
	self->ref_count = 1;
}


static void symbol_finalize (Symbol* obj) {
	Symbol * self;
	self = SYMBOL (obj);
	self->name = (g_free (self->name), NULL);
	self->full_name = (g_free (self->full_name), NULL);
	self->signature = (g_free (self->signature), NULL);
	self->inherits = (_vala_array_free (self->inherits, self->inherits_length1, (GDestroyNotify) g_free), NULL);
	self->typeref = (g_free (self->typeref), NULL);
	self->access = (g_free (self->access), NULL);
	self->implementation = (g_free (self->implementation), NULL);
	self->type = (g_free (self->type), NULL);
	self->file = (g_free (self->file), NULL);
	self->language = (g_free (self->language), NULL);
	(self->root == NULL) ? NULL : (self->root = (symbol_unref (self->root), NULL));
}


GType symbol_get_type (void) {
	static GType symbol_type_id = 0;
	if (symbol_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { value_symbol_init, value_symbol_free_value, value_symbol_copy_value, value_symbol_peek_pointer, "p", value_symbol_collect_value, "p", value_symbol_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (SymbolClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) symbol_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Symbol), 0, (GInstanceInitFunc) symbol_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		symbol_type_id = g_type_register_fundamental (g_type_fundamental_next (), "Symbol", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return symbol_type_id;
}


gpointer symbol_ref (gpointer instance) {
	Symbol* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void symbol_unref (gpointer instance) {
	Symbol* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		SYMBOL_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


GQuark symbol_error_quark (void) {
	return g_quark_from_static_string ("symbol_error-quark");
}



static GType symbol_browser_col_get_type (void) {
	static GType symbol_browser_col_type_id = 0;
	if (G_UNLIKELY (symbol_browser_col_type_id == 0)) {
		static const GEnumValue values[] = {{SYMBOL_BROWSER_COL_ICON, "SYMBOL_BROWSER_COL_ICON", "icon"}, {SYMBOL_BROWSER_COL_NAME, "SYMBOL_BROWSER_COL_NAME", "name"}, {SYMBOL_BROWSER_COL_FILE, "SYMBOL_BROWSER_COL_FILE", "file"}, {SYMBOL_BROWSER_COL_LINE, "SYMBOL_BROWSER_COL_LINE", "line"}, {SYMBOL_BROWSER_COL_LANGUAGE, "SYMBOL_BROWSER_COL_LANGUAGE", "language"}, {SYMBOL_BROWSER_COL_NB, "SYMBOL_BROWSER_COL_NB", "nb"}, {0, NULL, NULL}};
		symbol_browser_col_type_id = g_enum_register_static ("SymbolBrowserCol", values);
	}
	return symbol_browser_col_type_id;
}


static void _g_slist_free_g_free (GSList* self) {
	g_slist_foreach (self, (GFunc) g_free, NULL);
	g_slist_free (self);
}


#line 144 "symbol.vala"
static void symbol_browser_save_expanded_rows (SymbolBrowser* self) {
#line 144 "symbol.vala"
	g_return_if_fail (self != NULL);
#line 423 "symbol.c"
}


/*
        GConfClient             *client;
        GError                  *error=NULL;
        
        client = gconf_client_get_default ();
        
        gconf_client_set_list (client, 
                        GEDIT_SYMBOL_BROWSER_PLUGIN_GCONF_PATH"expanded-rows",
                        GCONF_VALUE_STRING, panel->priv->expanded_rows,
                        &error);
                        
        if (error != NULL)
        {
                g_warning ("Error saving gedit-symbol-browser-plugin setting: %s", 
                                error->message);
                                
                g_error_free (error);
        }

        g_object_unref (client);
*/
#line 169 "symbol.vala"
static gboolean symbol_browser_is_expanded_row (SymbolBrowser* self, const char* name) {
#line 450 "symbol.c"
	gboolean result;
	gboolean ret;
#line 169 "symbol.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 169 "symbol.vala"
	g_return_val_if_fail (name != NULL, FALSE);
#line 457 "symbol.c"
	ret = FALSE;
	{
		GSList* i_collection;
		GSList* i_it;
#line 173 "symbol.vala"
		i_collection = self->priv->expanded_rows;
#line 464 "symbol.c"
		for (i_it = i_collection; i_it != NULL; i_it = i_it->next) {
			const char* _tmp0_;
			char* i;
#line 70 "symbol.vala"
			_tmp0_ = NULL;
#line 173 "symbol.vala"
			i = (_tmp0_ = (const char*) i_it->data, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
#line 472 "symbol.c"
			{
#line 175 "symbol.vala"
				if (_vala_strcmp0 (i, name) == 0) {
#line 177 "symbol.vala"
					ret = TRUE;
#line 478 "symbol.c"
					i = (g_free (i), NULL);
#line 178 "symbol.vala"
					break;
#line 482 "symbol.c"
				}
				i = (g_free (i), NULL);
			}
		}
	}
	result = ret;
	return result;
}


#line 184 "symbol.vala"
static void symbol_browser_add_expanded_row (SymbolBrowser* self, const char* name) {
#line 184 "symbol.vala"
	g_return_if_fail (self != NULL);
#line 184 "symbol.vala"
	g_return_if_fail (name != NULL);
#line 186 "symbol.vala"
	if (!symbol_browser_is_expanded_row (self, name)) {
#line 501 "symbol.c"
		const char* _tmp0_;
#line 188 "symbol.vala"
		_tmp0_ = NULL;
#line 188 "symbol.vala"
		self->priv->expanded_rows = g_slist_prepend (self->priv->expanded_rows, (_tmp0_ = name, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)));
#line 189 "symbol.vala"
		symbol_browser_save_expanded_rows (self);
#line 509 "symbol.c"
	}
}


#line 193 "symbol.vala"
static void symbol_browser_remove_expanded_row (SymbolBrowser* self, const char* name) {
#line 516 "symbol.c"
	GSList* l;
#line 193 "symbol.vala"
	g_return_if_fail (self != NULL);
#line 193 "symbol.vala"
	g_return_if_fail (name != NULL);
#line 522 "symbol.c"
	l = NULL;
#line 197 "symbol.vala"
	if (symbol_browser_is_expanded_row (self, name) == TRUE) {
#line 526 "symbol.c"
		{
			gboolean _tmp0_;
#line 199 "symbol.vala"
			l = self->priv->expanded_rows;
#line 531 "symbol.c"
			_tmp0_ = TRUE;
#line 199 "symbol.vala"
			while (TRUE) {
#line 199 "symbol.vala"
				if (!_tmp0_) {
#line 199 "symbol.vala"
					l = l->next;
#line 539 "symbol.c"
				}
#line 199 "symbol.vala"
				_tmp0_ = FALSE;
#line 199 "symbol.vala"
				if (!(l != NULL)) {
#line 199 "symbol.vala"
					break;
#line 547 "symbol.c"
				}
#line 201 "symbol.vala"
				if (_vala_strcmp0 ((const char*) l->data, name) == 0) {
#line 203 "symbol.vala"
					self->priv->expanded_rows = g_slist_remove_all (self->priv->expanded_rows, (const char*) l->data);
#line 553 "symbol.c"
				}
			}
		}
#line 206 "symbol.vala"
		symbol_browser_save_expanded_rows (self);
#line 559 "symbol.c"
	}
}


/**
   * Execute the ctags command and return stdout.
   */
#line 213 "symbol.vala"
static char* symbol_browser_exec_ctags (SymbolBrowser* self, const char* filename, GError** error) {
#line 569 "symbol.c"
	char* result;
	GError * _inner_error_;
	char* output;
	char* command;
	char* _tmp0_;
	char* _tmp3_;
	gboolean _tmp2_;
	char* _tmp1_;
#line 213 "symbol.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 213 "symbol.vala"
	g_return_val_if_fail (filename != NULL, NULL);
#line 582 "symbol.c"
	_inner_error_ = NULL;
	output = NULL;
	command = NULL;
	/* build ctags command */
	_tmp0_ = NULL;
#line 219 "symbol.vala"
	command = (_tmp0_ = g_strdup_printf ("%s --extra=+q --fields=afmiKlnsStz -f - \"%s\"", VALIDE_CTAGS_EXEC, filename), command = (g_free (command), NULL), _tmp0_);
#line 590 "symbol.c"
	/* execute command */
	_tmp3_ = NULL;
	_tmp1_ = NULL;
#line 222 "symbol.vala"
	_tmp2_ = g_spawn_command_line_sync (command, &_tmp1_, NULL, NULL, &_inner_error_);
#line 222 "symbol.vala"
	output = (_tmp3_ = _tmp1_, output = (g_free (output), NULL), _tmp3_);
#line 222 "symbol.vala"
	_tmp2_;
#line 600 "symbol.c"
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		output = (g_free (output), NULL);
		command = (g_free (command), NULL);
		return NULL;
	}
	result = output;
	command = (g_free (command), NULL);
	return result;
}


static char** _vala_array_dup1 (char** self, int length) {
	char** result;
	int i;
	const char* _tmp86_;
	result = g_new0 (char*, length);
	for (i = 0; i < length; i++) {
		result[i] = (_tmp86_ = self[i], (_tmp86_ == NULL) ? NULL : g_strdup (_tmp86_));
	}
	return result;
}


/**
   * This function needs work. Used to be custom parsed for ex searches
   * which could potentially contain tabs, however, I've decided that isn't
   * really needed. But, it's still using Symbol. We could just pass back
   * **fields array.
   */
#line 233 "symbol.vala"
static Symbol* symbol_browser_parse_line (SymbolBrowser* self, const char* line) {
#line 633 "symbol.c"
	Symbol* result;
	gint fields_size;
	gint fields_length1;
	char** fields;
	Symbol* symbol;
	char** _tmp1_;
	char** _tmp0_;
#line 233 "symbol.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 233 "symbol.vala"
	g_return_val_if_fail (line != NULL, NULL);
#line 645 "symbol.c"
	/* (parent.)name file ex_cmd;" kind:[enumvalue|errorcode|errordomain|enum|struct|namespace] line:x language:x access:x 
	 (parent.)name file ex_cmd;" kind:field line:x language:x typeref:x access:x 
	 (parent.)name file ex_cmd;" kind:class line:x language:x inherits:x access:x implementation:x 
	 (parent.)name file ex_cmd;" kind:[delegate|method|signal] line:x language:x typeref:x access:x implementation:x signature:x 
	 (parent.)name file ex_cmd;" kind:interface line:x language:x inherits:x access:x 
	 (parent.)name file ex_cmd;" kind:property line:x language:x typeref:x access:x implementation:x 
	 name          fileex_cmd;" kind:function line:x language:x signature:x 
	 name          file ex_cmd;" kind:variable line:x language:x */
	fields = (fields_length1 = 0, NULL);
	symbol = NULL;
	_tmp1_ = NULL;
	_tmp0_ = NULL;
#line 247 "symbol.vala"
	fields = (_tmp1_ = _tmp0_ = g_strsplit (line, "\t", 0), fields = (_vala_array_free (fields, fields_length1, (GDestroyNotify) g_free), NULL), fields_length1 = _vala_array_length (_tmp0_), fields_size = fields_length1, _tmp1_);
#line 248 "symbol.vala"
	if (fields[0] != NULL) {
#line 662 "symbol.c"
		char* filename;
		char* name;
		char* full_name;
		char* kind;
		gint file_position;
		char* language;
		char* access;
		char** _tmp2_;
		gint inherits_size;
		gint inherits_length1;
		char** inherits;
		char* typeref;
		char* signature;
		char* implementation;
		gint splits_size;
		gint splits_length1;
		char** splits;
		Symbol* _tmp3_;
		char* _tmp5_;
		const char* _tmp4_;
		char* _tmp11_;
		const char* _tmp10_;
		char** _tmp13_;
		char** _tmp12_;
		char* _tmp15_;
		const char* _tmp14_;
		char** _tmp17_;
		char** _tmp16_;
		char** _tmp19_;
		char** _tmp18_;
		char* _tmp21_;
		const char* _tmp20_;
		GQuark _tmp77_;
		const char* _tmp76_;
		static GQuark _tmp77__label0 = 0;
		static GQuark _tmp77__label1 = 0;
		static GQuark _tmp77__label2 = 0;
		static GQuark _tmp77__label3 = 0;
		static GQuark _tmp77__label4 = 0;
		static GQuark _tmp77__label5 = 0;
		static GQuark _tmp77__label6 = 0;
		static GQuark _tmp77__label7 = 0;
		static GQuark _tmp77__label8 = 0;
		static GQuark _tmp77__label9 = 0;
		static GQuark _tmp77__label10 = 0;
		static GQuark _tmp77__label11 = 0;
		static GQuark _tmp77__label12 = 0;
		static GQuark _tmp77__label13 = 0;
		static GQuark _tmp77__label14 = 0;
		char* _tmp79_;
		const char* _tmp78_;
		char* _tmp81_;
		const char* _tmp80_;
		char* _tmp83_;
		const char* _tmp82_;
		char* _tmp85_;
		const char* _tmp84_;
		char** _tmp88_;
		char** _tmp87_;
		const char* _tmp86_;
		char* _tmp90_;
		const char* _tmp89_;
		Symbol* _tmp91_;
		filename = NULL;
		name = NULL;
		full_name = NULL;
		kind = NULL;
		file_position = 0;
		language = NULL;
		access = NULL;
#line 257 "symbol.vala"
		_tmp2_ = NULL;
#line 735 "symbol.c"
		inherits = (_tmp2_ = NULL, inherits_length1 = 0, inherits_size = inherits_length1, _tmp2_);
		typeref = NULL;
		signature = NULL;
		implementation = NULL;
		splits = (splits_length1 = 0, NULL);
		_tmp3_ = NULL;
#line 263 "symbol.vala"
		symbol = (_tmp3_ = symbol_new (), (symbol == NULL) ? NULL : (symbol = (symbol_unref (symbol), NULL)), _tmp3_);
#line 744 "symbol.c"
		_tmp5_ = NULL;
#line 264 "symbol.vala"
		_tmp4_ = NULL;
#line 264 "symbol.vala"
		full_name = (_tmp5_ = (_tmp4_ = fields[0], (_tmp4_ == NULL) ? NULL : g_strdup (_tmp4_)), full_name = (g_free (full_name), NULL), _tmp5_);
#line 265 "symbol.vala"
		if (g_utf8_strrchr (full_name, (gssize) (-1), (gunichar) '.') != NULL) {
#line 752 "symbol.c"
			char* _tmp7_;
			const char* _tmp6_;
			_tmp7_ = NULL;
#line 267 "symbol.vala"
			_tmp6_ = NULL;
#line 267 "symbol.vala"
			name = (_tmp7_ = (_tmp6_ = g_utf8_offset_to_pointer (g_utf8_strrchr (full_name, (gssize) (-1), (gunichar) '.'), (glong) 1), (_tmp6_ == NULL) ? NULL : g_strdup (_tmp6_)), name = (g_free (name), NULL), _tmp7_);
#line 760 "symbol.c"
		} else {
			char* _tmp9_;
			const char* _tmp8_;
			_tmp9_ = NULL;
#line 271 "symbol.vala"
			_tmp8_ = NULL;
#line 271 "symbol.vala"
			name = (_tmp9_ = (_tmp8_ = full_name, (_tmp8_ == NULL) ? NULL : g_strdup (_tmp8_)), name = (g_free (name), NULL), _tmp9_);
#line 769 "symbol.c"
		}
		_tmp11_ = NULL;
#line 273 "symbol.vala"
		_tmp10_ = NULL;
#line 273 "symbol.vala"
		filename = (_tmp11_ = (_tmp10_ = fields[1], (_tmp10_ == NULL) ? NULL : g_strdup (_tmp10_)), filename = (g_free (filename), NULL), _tmp11_);
#line 776 "symbol.c"
		_tmp13_ = NULL;
		_tmp12_ = NULL;
#line 274 "symbol.vala"
		splits = (_tmp13_ = _tmp12_ = g_strsplit (fields[3], ":", 0), splits = (_vala_array_free (splits, splits_length1, (GDestroyNotify) g_free), NULL), splits_length1 = _vala_array_length (_tmp12_), splits_size = splits_length1, _tmp13_);
#line 781 "symbol.c"
		_tmp15_ = NULL;
#line 275 "symbol.vala"
		_tmp14_ = NULL;
#line 275 "symbol.vala"
		kind = (_tmp15_ = (_tmp14_ = splits[1], (_tmp14_ == NULL) ? NULL : g_strdup (_tmp14_)), kind = (g_free (kind), NULL), _tmp15_);
#line 276 "symbol.vala"
		if (kind == NULL) {
#line 789 "symbol.c"
			result = NULL;
			filename = (g_free (filename), NULL);
			name = (g_free (name), NULL);
			full_name = (g_free (full_name), NULL);
			kind = (g_free (kind), NULL);
			language = (g_free (language), NULL);
			access = (g_free (access), NULL);
			inherits = (_vala_array_free (inherits, inherits_length1, (GDestroyNotify) g_free), NULL);
			typeref = (g_free (typeref), NULL);
			signature = (g_free (signature), NULL);
			implementation = (g_free (implementation), NULL);
			splits = (_vala_array_free (splits, splits_length1, (GDestroyNotify) g_free), NULL);
			fields = (_vala_array_free (fields, fields_length1, (GDestroyNotify) g_free), NULL);
			(symbol == NULL) ? NULL : (symbol = (symbol_unref (symbol), NULL));
			return result;
		}
		_tmp17_ = NULL;
		_tmp16_ = NULL;
#line 281 "symbol.vala"
		splits = (_tmp17_ = _tmp16_ = g_strsplit (fields[4], ":", 0), splits = (_vala_array_free (splits, splits_length1, (GDestroyNotify) g_free), NULL), splits_length1 = _vala_array_length (_tmp16_), splits_size = splits_length1, _tmp17_);
#line 282 "symbol.vala"
		file_position = atoi (splits[1]);
#line 812 "symbol.c"
		_tmp19_ = NULL;
		_tmp18_ = NULL;
#line 284 "symbol.vala"
		splits = (_tmp19_ = _tmp18_ = g_strsplit (fields[5], ":", 0), splits = (_vala_array_free (splits, splits_length1, (GDestroyNotify) g_free), NULL), splits_length1 = _vala_array_length (_tmp18_), splits_size = splits_length1, _tmp19_);
#line 817 "symbol.c"
		_tmp21_ = NULL;
#line 285 "symbol.vala"
		_tmp20_ = NULL;
#line 285 "symbol.vala"
		language = (_tmp21_ = (_tmp20_ = fields[1], (_tmp20_ == NULL) ? NULL : g_strdup (_tmp20_)), language = (g_free (language), NULL), _tmp21_);
#line 287 "symbol.vala"
		_tmp76_ = NULL;
#line 825 "symbol.c"
		_tmp76_ = kind;
		_tmp77_ = (NULL == _tmp76_) ? 0 : g_quark_from_string (_tmp76_);
		if ((((((_tmp77_ == ((0 != _tmp77__label0) ? _tmp77__label0 : (_tmp77__label0 = g_quark_from_static_string ("enumvalue")))) || (_tmp77_ == ((0 != _tmp77__label1) ? _tmp77__label1 : (_tmp77__label1 = g_quark_from_static_string ("errorcode"))))) || (_tmp77_ == ((0 != _tmp77__label2) ? _tmp77__label2 : (_tmp77__label2 = g_quark_from_static_string ("errordomain"))))) || (_tmp77_ == ((0 != _tmp77__label3) ? _tmp77__label3 : (_tmp77__label3 = g_quark_from_static_string ("enum"))))) || (_tmp77_ == ((0 != _tmp77__label4) ? _tmp77__label4 : (_tmp77__label4 = g_quark_from_static_string ("namespace"))))) || (_tmp77_ == ((0 != _tmp77__label5) ? _tmp77__label5 : (_tmp77__label5 = g_quark_from_static_string ("struct")))))
		do {
			char** _tmp23_;
			char** _tmp22_;
			char* _tmp25_;
			const char* _tmp24_;
			_tmp23_ = NULL;
			_tmp22_ = NULL;
#line 295 "symbol.vala"
			splits = (_tmp23_ = _tmp22_ = g_strsplit (fields[5], ":", 0), splits = (_vala_array_free (splits, splits_length1, (GDestroyNotify) g_free), NULL), splits_length1 = _vala_array_length (_tmp22_), splits_size = splits_length1, _tmp23_);
#line 838 "symbol.c"
			_tmp25_ = NULL;
#line 296 "symbol.vala"
			_tmp24_ = NULL;
#line 296 "symbol.vala"
			access = (_tmp25_ = (_tmp24_ = splits[1], (_tmp24_ == NULL) ? NULL : g_strdup (_tmp24_)), access = (g_free (access), NULL), _tmp25_);
#line 297 "symbol.vala"
			break;
#line 846 "symbol.c"
		} while (0); else if (_tmp77_ == ((0 != _tmp77__label6) ? _tmp77__label6 : (_tmp77__label6 = g_quark_from_static_string ("field"))))
		do {
			char** _tmp27_;
			char** _tmp26_;
			char* _tmp29_;
			const char* _tmp28_;
			_tmp27_ = NULL;
			_tmp26_ = NULL;
#line 299 "symbol.vala"
			splits = (_tmp27_ = _tmp26_ = g_strsplit (fields[6], ":", 0), splits = (_vala_array_free (splits, splits_length1, (GDestroyNotify) g_free), NULL), splits_length1 = _vala_array_length (_tmp26_), splits_size = splits_length1, _tmp27_);
#line 857 "symbol.c"
			_tmp29_ = NULL;
#line 300 "symbol.vala"
			_tmp28_ = NULL;
#line 300 "symbol.vala"
			access = (_tmp29_ = (_tmp28_ = splits[1], (_tmp28_ == NULL) ? NULL : g_strdup (_tmp28_)), access = (g_free (access), NULL), _tmp29_);
#line 301 "symbol.vala"
			break;
#line 865 "symbol.c"
		} while (0); else if (_tmp77_ == ((0 != _tmp77__label7) ? _tmp77__label7 : (_tmp77__label7 = g_quark_from_static_string ("class"))))
		do {
			char** _tmp31_;
			char** _tmp30_;
			gboolean _tmp32_;
			char** _tmp36_;
			char** _tmp35_;
			char* _tmp38_;
			const char* _tmp37_;
			char** _tmp40_;
			char** _tmp39_;
			char* _tmp42_;
			const char* _tmp41_;
			_tmp31_ = NULL;
			_tmp30_ = NULL;
#line 303 "symbol.vala"
			splits = (_tmp31_ = _tmp30_ = g_strsplit (fields[6], ":", 0), splits = (_vala_array_free (splits, splits_length1, (GDestroyNotify) g_free), NULL), splits_length1 = _vala_array_length (_tmp30_), splits_size = splits_length1, _tmp31_);
#line 883 "symbol.c"
			_tmp32_ = FALSE;
#line 304 "symbol.vala"
			if (splits != NULL) {
#line 304 "symbol.vala"
				_tmp32_ = splits[1] != NULL;
#line 889 "symbol.c"
			} else {
#line 304 "symbol.vala"
				_tmp32_ = FALSE;
#line 893 "symbol.c"
			}
#line 304 "symbol.vala"
			if (_tmp32_) {
#line 897 "symbol.c"
				char** _tmp34_;
				char** _tmp33_;
				_tmp34_ = NULL;
				_tmp33_ = NULL;
#line 306 "symbol.vala"
				inherits = (_tmp34_ = _tmp33_ = g_strsplit (splits[1], ",", 0), inherits = (_vala_array_free (inherits, inherits_length1, (GDestroyNotify) g_free), NULL), inherits_length1 = _vala_array_length (_tmp33_), inherits_size = inherits_length1, _tmp34_);
#line 904 "symbol.c"
			}
			_tmp36_ = NULL;
			_tmp35_ = NULL;
#line 308 "symbol.vala"
			splits = (_tmp36_ = _tmp35_ = g_strsplit (fields[7], ":", 0), splits = (_vala_array_free (splits, splits_length1, (GDestroyNotify) g_free), NULL), splits_length1 = _vala_array_length (_tmp35_), splits_size = splits_length1, _tmp36_);
#line 910 "symbol.c"
			_tmp38_ = NULL;
#line 309 "symbol.vala"
			_tmp37_ = NULL;
#line 309 "symbol.vala"
			access = (_tmp38_ = (_tmp37_ = splits[1], (_tmp37_ == NULL) ? NULL : g_strdup (_tmp37_)), access = (g_free (access), NULL), _tmp38_);
#line 916 "symbol.c"
			_tmp40_ = NULL;
			_tmp39_ = NULL;
#line 310 "symbol.vala"
			splits = (_tmp40_ = _tmp39_ = g_strsplit (fields[8], ":", 0), splits = (_vala_array_free (splits, splits_length1, (GDestroyNotify) g_free), NULL), splits_length1 = _vala_array_length (_tmp39_), splits_size = splits_length1, _tmp40_);
#line 921 "symbol.c"
			_tmp42_ = NULL;
#line 311 "symbol.vala"
			_tmp41_ = NULL;
#line 311 "symbol.vala"
			implementation = (_tmp42_ = (_tmp41_ = splits[1], (_tmp41_ == NULL) ? NULL : g_strdup (_tmp41_)), implementation = (g_free (implementation), NULL), _tmp42_);
#line 312 "symbol.vala"
			break;
#line 929 "symbol.c"
		} while (0); else if (((_tmp77_ == ((0 != _tmp77__label8) ? _tmp77__label8 : (_tmp77__label8 = g_quark_from_static_string ("delegate")))) || (_tmp77_ == ((0 != _tmp77__label9) ? _tmp77__label9 : (_tmp77__label9 = g_quark_from_static_string ("method"))))) || (_tmp77_ == ((0 != _tmp77__label10) ? _tmp77__label10 : (_tmp77__label10 = g_quark_from_static_string ("signal")))))
		do {
			char** _tmp44_;
			char** _tmp43_;
			char* _tmp46_;
			const char* _tmp45_;
			char** _tmp48_;
			char** _tmp47_;
			char* _tmp50_;
			const char* _tmp49_;
			char** _tmp52_;
			char** _tmp51_;
			char* _tmp54_;
			const char* _tmp53_;
			_tmp44_ = NULL;
			_tmp43_ = NULL;
#line 316 "symbol.vala"
			splits = (_tmp44_ = _tmp43_ = g_strsplit (fields[6], ":", 0), splits = (_vala_array_free (splits, splits_length1, (GDestroyNotify) g_free), NULL), splits_length1 = _vala_array_length (_tmp43_), splits_size = splits_length1, _tmp44_);
#line 948 "symbol.c"
			_tmp46_ = NULL;
#line 317 "symbol.vala"
			_tmp45_ = NULL;
#line 317 "symbol.vala"
			access = (_tmp46_ = (_tmp45_ = splits[1], (_tmp45_ == NULL) ? NULL : g_strdup (_tmp45_)), access = (g_free (access), NULL), _tmp46_);
#line 954 "symbol.c"
			_tmp48_ = NULL;
			_tmp47_ = NULL;
#line 318 "symbol.vala"
			splits = (_tmp48_ = _tmp47_ = g_strsplit (fields[7], ":", 0), splits = (_vala_array_free (splits, splits_length1, (GDestroyNotify) g_free), NULL), splits_length1 = _vala_array_length (_tmp47_), splits_size = splits_length1, _tmp48_);
#line 959 "symbol.c"
			_tmp50_ = NULL;
#line 319 "symbol.vala"
			_tmp49_ = NULL;
#line 319 "symbol.vala"
			implementation = (_tmp50_ = (_tmp49_ = splits[1], (_tmp49_ == NULL) ? NULL : g_strdup (_tmp49_)), implementation = (g_free (implementation), NULL), _tmp50_);
#line 965 "symbol.c"
			_tmp52_ = NULL;
			_tmp51_ = NULL;
#line 320 "symbol.vala"
			splits = (_tmp52_ = _tmp51_ = g_strsplit (fields[8], ":", 0), splits = (_vala_array_free (splits, splits_length1, (GDestroyNotify) g_free), NULL), splits_length1 = _vala_array_length (_tmp51_), splits_size = splits_length1, _tmp52_);
#line 970 "symbol.c"
			_tmp54_ = NULL;
#line 321 "symbol.vala"
			_tmp53_ = NULL;
#line 321 "symbol.vala"
			signature = (_tmp54_ = (_tmp53_ = splits[1], (_tmp53_ == NULL) ? NULL : g_strdup (_tmp53_)), signature = (g_free (signature), NULL), _tmp54_);
#line 322 "symbol.vala"
			break;
#line 978 "symbol.c"
		} while (0); else if (_tmp77_ == ((0 != _tmp77__label11) ? _tmp77__label11 : (_tmp77__label11 = g_quark_from_static_string ("interface"))))
		do {
			char** _tmp56_;
			char** _tmp55_;
			gboolean _tmp57_;
			char** _tmp61_;
			char** _tmp60_;
			char* _tmp63_;
			const char* _tmp62_;
			_tmp56_ = NULL;
			_tmp55_ = NULL;
#line 324 "symbol.vala"
			splits = (_tmp56_ = _tmp55_ = g_strsplit (fields[6], ":", 0), splits = (_vala_array_free (splits, splits_length1, (GDestroyNotify) g_free), NULL), splits_length1 = _vala_array_length (_tmp55_), splits_size = splits_length1, _tmp56_);
#line 992 "symbol.c"
			_tmp57_ = FALSE;
#line 325 "symbol.vala"
			if (splits != NULL) {
#line 325 "symbol.vala"
				_tmp57_ = splits[1] != NULL;
#line 998 "symbol.c"
			} else {
#line 325 "symbol.vala"
				_tmp57_ = FALSE;
#line 1002 "symbol.c"
			}
#line 325 "symbol.vala"
			if (_tmp57_) {
#line 1006 "symbol.c"
				char** _tmp59_;
				char** _tmp58_;
				_tmp59_ = NULL;
				_tmp58_ = NULL;
#line 327 "symbol.vala"
				inherits = (_tmp59_ = _tmp58_ = g_strsplit (splits[1], ",", 0), inherits = (_vala_array_free (inherits, inherits_length1, (GDestroyNotify) g_free), NULL), inherits_length1 = _vala_array_length (_tmp58_), inherits_size = inherits_length1, _tmp59_);
#line 1013 "symbol.c"
			}
			_tmp61_ = NULL;
			_tmp60_ = NULL;
#line 329 "symbol.vala"
			splits = (_tmp61_ = _tmp60_ = g_strsplit (fields[7], ":", 0), splits = (_vala_array_free (splits, splits_length1, (GDestroyNotify) g_free), NULL), splits_length1 = _vala_array_length (_tmp60_), splits_size = splits_length1, _tmp61_);
#line 1019 "symbol.c"
			_tmp63_ = NULL;
#line 330 "symbol.vala"
			_tmp62_ = NULL;
#line 330 "symbol.vala"
			access = (_tmp63_ = (_tmp62_ = splits[1], (_tmp62_ == NULL) ? NULL : g_strdup (_tmp62_)), access = (g_free (access), NULL), _tmp63_);
#line 331 "symbol.vala"
			break;
#line 1027 "symbol.c"
		} while (0); else if (_tmp77_ == ((0 != _tmp77__label12) ? _tmp77__label12 : (_tmp77__label12 = g_quark_from_static_string ("property"))))
		do {
			char** _tmp65_;
			char** _tmp64_;
			char* _tmp67_;
			const char* _tmp66_;
			char** _tmp69_;
			char** _tmp68_;
			char* _tmp71_;
			const char* _tmp70_;
			_tmp65_ = NULL;
			_tmp64_ = NULL;
#line 333 "symbol.vala"
			splits = (_tmp65_ = _tmp64_ = g_strsplit (fields[6], ":", 0), splits = (_vala_array_free (splits, splits_length1, (GDestroyNotify) g_free), NULL), splits_length1 = _vala_array_length (_tmp64_), splits_size = splits_length1, _tmp65_);
#line 1042 "symbol.c"
			_tmp67_ = NULL;
#line 334 "symbol.vala"
			_tmp66_ = NULL;
#line 334 "symbol.vala"
			access = (_tmp67_ = (_tmp66_ = splits[1], (_tmp66_ == NULL) ? NULL : g_strdup (_tmp66_)), access = (g_free (access), NULL), _tmp67_);
#line 1048 "symbol.c"
			_tmp69_ = NULL;
			_tmp68_ = NULL;
#line 335 "symbol.vala"
			splits = (_tmp69_ = _tmp68_ = g_strsplit (fields[7], ":", 0), splits = (_vala_array_free (splits, splits_length1, (GDestroyNotify) g_free), NULL), splits_length1 = _vala_array_length (_tmp68_), splits_size = splits_length1, _tmp69_);
#line 1053 "symbol.c"
			_tmp71_ = NULL;
#line 336 "symbol.vala"
			_tmp70_ = NULL;
#line 336 "symbol.vala"
			implementation = (_tmp71_ = (_tmp70_ = splits[1], (_tmp70_ == NULL) ? NULL : g_strdup (_tmp70_)), implementation = (g_free (implementation), NULL), _tmp71_);
#line 337 "symbol.vala"
			break;
#line 1061 "symbol.c"
		} while (0); else if (_tmp77_ == ((0 != _tmp77__label13) ? _tmp77__label13 : (_tmp77__label13 = g_quark_from_static_string ("function"))))
		do {
			char** _tmp73_;
			char** _tmp72_;
			char* _tmp75_;
			const char* _tmp74_;
			_tmp73_ = NULL;
			_tmp72_ = NULL;
#line 339 "symbol.vala"
			splits = (_tmp73_ = _tmp72_ = g_strsplit (fields[5], ":", 0), splits = (_vala_array_free (splits, splits_length1, (GDestroyNotify) g_free), NULL), splits_length1 = _vala_array_length (_tmp72_), splits_size = splits_length1, _tmp73_);
#line 1072 "symbol.c"
			_tmp75_ = NULL;
#line 340 "symbol.vala"
			_tmp74_ = NULL;
#line 340 "symbol.vala"
			signature = (_tmp75_ = (_tmp74_ = splits[1], (_tmp74_ == NULL) ? NULL : g_strdup (_tmp74_)), signature = (g_free (signature), NULL), _tmp75_);
#line 341 "symbol.vala"
			break;
#line 1080 "symbol.c"
		} while (0); else if (_tmp77_ == ((0 != _tmp77__label14) ? _tmp77__label14 : (_tmp77__label14 = g_quark_from_static_string ("variable"))))
		do {
#line 343 "symbol.vala"
			break;
#line 1085 "symbol.c"
		} while (0); else
		do {
#line 345 "symbol.vala"
			g_debug ("symbol.vala:345: Unknow kind '%s' for symbol '%s'", kind, name);
#line 346 "symbol.vala"
			break;
#line 1092 "symbol.c"
		} while (0);
		_tmp79_ = NULL;
#line 349 "symbol.vala"
		_tmp78_ = NULL;
#line 349 "symbol.vala"
		symbol->name = (_tmp79_ = (_tmp78_ = name, (_tmp78_ == NULL) ? NULL : g_strdup (_tmp78_)), symbol->name = (g_free (symbol->name), NULL), _tmp79_);
#line 1099 "symbol.c"
		_tmp81_ = NULL;
#line 350 "symbol.vala"
		_tmp80_ = NULL;
#line 350 "symbol.vala"
		symbol->full_name = (_tmp81_ = (_tmp80_ = full_name, (_tmp80_ == NULL) ? NULL : g_strdup (_tmp80_)), symbol->full_name = (g_free (symbol->full_name), NULL), _tmp81_);
#line 1105 "symbol.c"
		_tmp83_ = NULL;
#line 351 "symbol.vala"
		_tmp82_ = NULL;
#line 351 "symbol.vala"
		symbol->file = (_tmp83_ = (_tmp82_ = filename, (_tmp82_ == NULL) ? NULL : g_strdup (_tmp82_)), symbol->file = (g_free (symbol->file), NULL), _tmp83_);
#line 352 "symbol.vala"
		symbol->line = file_position;
#line 1113 "symbol.c"
		_tmp85_ = NULL;
#line 353 "symbol.vala"
		_tmp84_ = NULL;
#line 353 "symbol.vala"
		symbol->signature = (_tmp85_ = (_tmp84_ = signature, (_tmp84_ == NULL) ? NULL : g_strdup (_tmp84_)), symbol->signature = (g_free (symbol->signature), NULL), _tmp85_);
#line 1119 "symbol.c"
		_tmp88_ = NULL;
#line 354 "symbol.vala"
		_tmp87_ = NULL;
#line 36 "symbol.vala"
		_tmp86_ = NULL;
#line 354 "symbol.vala"
		symbol->inherits = (_tmp88_ = (_tmp87_ = inherits, (_tmp87_ == NULL) ? ((gpointer) _tmp87_) : _vala_array_dup1 (_tmp87_, inherits_length1)), symbol->inherits = (_vala_array_free (symbol->inherits, symbol->inherits_length1, (GDestroyNotify) g_free), NULL), symbol->inherits_length1 = inherits_length1, symbol->inherits_size = symbol->inherits_length1, _tmp88_);
#line 1127 "symbol.c"
		_tmp90_ = NULL;
#line 355 "symbol.vala"
		_tmp89_ = NULL;
#line 355 "symbol.vala"
		symbol->typeref = (_tmp90_ = (_tmp89_ = typeref, (_tmp89_ == NULL) ? NULL : g_strdup (_tmp89_)), symbol->typeref = (g_free (symbol->typeref), NULL), _tmp90_);
#line 1133 "symbol.c"
		_tmp91_ = NULL;
#line 356 "symbol.vala"
		symbol->root = (_tmp91_ = NULL, (symbol->root == NULL) ? NULL : (symbol->root = (symbol_unref (symbol->root), NULL)), _tmp91_);
#line 357 "symbol.vala"
		if (kind != NULL) {
#line 1139 "symbol.c"
			char* _tmp93_;
			const char* _tmp92_;
			_tmp93_ = NULL;
#line 359 "symbol.vala"
			_tmp92_ = NULL;
#line 359 "symbol.vala"
			symbol->type = (_tmp93_ = (_tmp92_ = kind, (_tmp92_ == NULL) ? NULL : g_strdup (_tmp92_)), symbol->type = (g_free (symbol->type), NULL), _tmp93_);
#line 1147 "symbol.c"
		}
#line 361 "symbol.vala"
		if (access != NULL) {
#line 1151 "symbol.c"
			char* _tmp95_;
			const char* _tmp94_;
			_tmp95_ = NULL;
#line 363 "symbol.vala"
			_tmp94_ = NULL;
#line 363 "symbol.vala"
			symbol->access = (_tmp95_ = (_tmp94_ = access, (_tmp94_ == NULL) ? NULL : g_strdup (_tmp94_)), symbol->access = (g_free (symbol->access), NULL), _tmp95_);
#line 1159 "symbol.c"
		}
#line 365 "symbol.vala"
		if (implementation != NULL) {
#line 1163 "symbol.c"
			char* _tmp97_;
			const char* _tmp96_;
			_tmp97_ = NULL;
#line 367 "symbol.vala"
			_tmp96_ = NULL;
#line 367 "symbol.vala"
			symbol->implementation = (_tmp97_ = (_tmp96_ = implementation, (_tmp96_ == NULL) ? NULL : g_strdup (_tmp96_)), symbol->implementation = (g_free (symbol->implementation), NULL), _tmp97_);
#line 1171 "symbol.c"
		}
		filename = (g_free (filename), NULL);
		name = (g_free (name), NULL);
		full_name = (g_free (full_name), NULL);
		kind = (g_free (kind), NULL);
		language = (g_free (language), NULL);
		access = (g_free (access), NULL);
		inherits = (_vala_array_free (inherits, inherits_length1, (GDestroyNotify) g_free), NULL);
		typeref = (g_free (typeref), NULL);
		signature = (g_free (signature), NULL);
		implementation = (g_free (implementation), NULL);
		splits = (_vala_array_free (splits, splits_length1, (GDestroyNotify) g_free), NULL);
	}
	result = symbol;
	fields = (_vala_array_free (fields, fields_length1, (GDestroyNotify) g_free), NULL);
	return result;
}


#line 373 "symbol.vala"
static void symbol_browser_clear_data (SymbolBrowser* self) {
#line 1193 "symbol.c"
	GHashTable* _tmp0_;
#line 373 "symbol.vala"
	g_return_if_fail (self != NULL);
#line 375 "symbol.vala"
	gtk_tree_store_clear (self->priv->model);
#line 1199 "symbol.c"
	_tmp0_ = NULL;
#line 376 "symbol.vala"
	self->priv->root_nodes = (_tmp0_ = g_hash_table_new (g_str_hash, g_str_equal), (self->priv->root_nodes == NULL) ? NULL : (self->priv->root_nodes = (g_hash_table_unref (self->priv->root_nodes), NULL)), _tmp0_);
#line 377 "symbol.vala"
	g_object_set ((GtkWidget*) self->priv->box, "sensitive", FALSE, NULL);
#line 378 "symbol.vala"
	gtk_label_set_text (self->priv->label, "");
#line 1207 "symbol.c"
}


#line 942 "glib-2.0.vapi"
static char* string_replace (const char* self, const char* old, const char* replacement) {
#line 1213 "symbol.c"
	char* result;
	GError * _inner_error_;
#line 942 "glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 942 "glib-2.0.vapi"
	g_return_val_if_fail (old != NULL, NULL);
#line 942 "glib-2.0.vapi"
	g_return_val_if_fail (replacement != NULL, NULL);
#line 1222 "symbol.c"
	_inner_error_ = NULL;
	{
		char* _tmp0_;
		GRegex* _tmp1_;
		GRegex* regex;
		char* _tmp2_;
#line 2667 "glib-2.0.vapi"
		_tmp0_ = NULL;
#line 944 "glib-2.0.vapi"
		_tmp1_ = NULL;
#line 1233 "symbol.c"
		regex = (_tmp1_ = g_regex_new (_tmp0_ = g_regex_escape_string (old, -1), 0, 0, &_inner_error_), _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_REGEX_ERROR) {
				goto __catch0_g_regex_error;
			}
			goto __finally0;
		}
		_tmp2_ = g_regex_replace_literal (regex, self, (glong) (-1), 0, replacement, 0, &_inner_error_);
		if (_inner_error_ != NULL) {
			(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
			if (_inner_error_->domain == G_REGEX_ERROR) {
				goto __catch0_g_regex_error;
			}
			goto __finally0;
		}
		result = _tmp2_;
		(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
		return result;
	}
	goto __finally0;
	__catch0_g_regex_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 947 "glib-2.0.vapi"
			g_assert_not_reached ();
#line 1262 "symbol.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally0:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
}


static GtkTreeIter* _gtk_tree_iter_dup (GtkTreeIter* self) {
	GtkTreeIter* dup;
	dup = g_new0 (GtkTreeIter, 1);
	memcpy (dup, self, sizeof (GtkTreeIter));
	return dup;
}


/**
   * Add a single symbol to the treeview. Parents rows are added as the type
   * whenever they aren't already present.
   */
#line 385 "symbol.vala"
static gboolean symbol_browser_add_symbol (SymbolBrowser* self, Symbol* symbol) {
#line 1289 "symbol.c"
	gboolean result;
	char* markup;
	GdkPixbuf* pixbuf;
	GtkTreeIter iter = {0};
	GtkTreeIter* parent;
	char* _tmp0_;
	char* _tmp1_;
	GdkPixbuf* _tmp2_;
#line 385 "symbol.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 385 "symbol.vala"
	g_return_val_if_fail (symbol != NULL, FALSE);
#line 1302 "symbol.c"
	markup = NULL;
	pixbuf = NULL;
	parent = NULL;
#line 392 "symbol.vala"
	valide_utils_process_gtk_events ();
#line 1308 "symbol.c"
	_tmp0_ = NULL;
#line 393 "symbol.vala"
	symbol->name = (_tmp0_ = string_replace (symbol->name, "<", "&lt;"), symbol->name = (g_free (symbol->name), NULL), _tmp0_);
#line 1312 "symbol.c"
	_tmp1_ = NULL;
#line 394 "symbol.vala"
	symbol->name = (_tmp1_ = string_replace (symbol->name, ">", "&gt;"), symbol->name = (g_free (symbol->name), NULL), _tmp1_);
#line 1316 "symbol.c"
	_tmp2_ = NULL;
#line 396 "symbol.vala"
	pixbuf = (_tmp2_ = valide_utils_get_symbol_pixbuf (symbol->type), (pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)), _tmp2_);
#line 398 "symbol.vala"
	if (!symbol_browser_get_hierarchic_view (self)) {
#line 1322 "symbol.c"
		char* _tmp3_;
		GtkTreeIter* _tmp5_;
		GtkTreeIter* _tmp4_;
		/* we could do bold here */
		_tmp3_ = NULL;
#line 401 "symbol.vala"
		markup = (_tmp3_ = g_strdup_printf ("<b>%s</b>", symbol->name), markup = (g_free (markup), NULL), _tmp3_);
#line 1330 "symbol.c"
		/* check for existing parent iter */
		_tmp5_ = NULL;
#line 404 "symbol.vala"
		_tmp4_ = NULL;
#line 404 "symbol.vala"
		parent = (_tmp5_ = (_tmp4_ = (GtkTreeIter*) g_hash_table_lookup (self->priv->root_nodes, markup), (_tmp4_ == NULL) ? NULL : _gtk_tree_iter_dup (_tmp4_)), (parent == NULL) ? NULL : (parent = (g_free (parent), NULL)), _tmp5_);
#line 1337 "symbol.c"
		/* create the parent if it wasn't already found */
#line 406 "symbol.vala"
		if (parent == NULL) {
#line 1341 "symbol.c"
			GtkTreeIter tmp = {0};
			GtkTreeIter* _tmp7_;
			GtkTreeIter* _tmp6_;
			GtkTreeIter* _tmp9_;
			const char* _tmp8_;
#line 410 "symbol.vala"
			gtk_tree_store_append (self->priv->model, &tmp, NULL);
#line 1349 "symbol.c"
			_tmp7_ = NULL;
#line 411 "symbol.vala"
			_tmp6_ = NULL;
#line 411 "symbol.vala"
			parent = (_tmp7_ = (_tmp6_ = &tmp, (_tmp6_ == NULL) ? NULL : _gtk_tree_iter_dup (_tmp6_)), (parent == NULL) ? NULL : (parent = (g_free (parent), NULL)), _tmp7_);
#line 412 "symbol.vala"
			gtk_tree_store_set (self->priv->model, parent, SYMBOL_BROWSER_COL_ICON, pixbuf, SYMBOL_BROWSER_COL_NAME, markup, -1);
#line 413 "symbol.vala"
			_tmp9_ = NULL;
#line 413 "symbol.vala"
			_tmp8_ = NULL;
#line 413 "symbol.vala"
			g_hash_table_insert (self->priv->root_nodes, (_tmp8_ = markup, (_tmp8_ == NULL) ? NULL : g_strdup (_tmp8_)), (_tmp9_ = parent, (_tmp9_ == NULL) ? NULL : _gtk_tree_iter_dup (_tmp9_)));
#line 1363 "symbol.c"
		}
#line 416 "symbol.vala"
		gtk_tree_store_append (self->priv->model, &iter, parent);
#line 1367 "symbol.c"
		/* add the new symbol */
#line 418 "symbol.vala"
		gtk_tree_store_set (self->priv->model, &iter, SYMBOL_BROWSER_COL_ICON, pixbuf, SYMBOL_BROWSER_COL_NAME, symbol->name, SYMBOL_BROWSER_COL_FILE, symbol->file, SYMBOL_BROWSER_COL_LINE, symbol->line, SYMBOL_BROWSER_COL_LANGUAGE, symbol->language, -1);
#line 1371 "symbol.c"
	} else {
		GtkTreeIter* _tmp15_;
		const char* _tmp14_;
#line 426 "symbol.vala"
		if (symbol->root != NULL) {
#line 1377 "symbol.c"
			GtkTreeIter* _tmp11_;
			GtkTreeIter* _tmp10_;
			_tmp11_ = NULL;
#line 428 "symbol.vala"
			_tmp10_ = NULL;
#line 428 "symbol.vala"
			parent = (_tmp11_ = (_tmp10_ = (GtkTreeIter*) g_hash_table_lookup (self->priv->root_nodes, symbol->root->full_name), (_tmp10_ == NULL) ? NULL : _gtk_tree_iter_dup (_tmp10_)), (parent == NULL) ? NULL : (parent = (g_free (parent), NULL)), _tmp11_);
#line 429 "symbol.vala"
			gtk_tree_store_append (self->priv->model, &iter, parent);
#line 1387 "symbol.c"
		} else {
			GtkTreeIter* _tmp13_;
			GtkTreeIter* _tmp12_;
#line 433 "symbol.vala"
			gtk_tree_store_append (self->priv->model, &iter, NULL);
#line 1393 "symbol.c"
			_tmp13_ = NULL;
#line 434 "symbol.vala"
			_tmp12_ = NULL;
#line 434 "symbol.vala"
			parent = (_tmp13_ = (_tmp12_ = &iter, (_tmp12_ == NULL) ? NULL : _gtk_tree_iter_dup (_tmp12_)), (parent == NULL) ? NULL : (parent = (g_free (parent), NULL)), _tmp13_);
#line 1399 "symbol.c"
		}
#line 436 "symbol.vala"
		gtk_tree_store_set (self->priv->model, &iter, SYMBOL_BROWSER_COL_ICON, pixbuf, SYMBOL_BROWSER_COL_NAME, symbol->name, SYMBOL_BROWSER_COL_FILE, symbol->file, SYMBOL_BROWSER_COL_LINE, symbol->line, SYMBOL_BROWSER_COL_LANGUAGE, symbol->language, -1);
#line 441 "symbol.vala"
		_tmp15_ = NULL;
#line 441 "symbol.vala"
		_tmp14_ = NULL;
#line 441 "symbol.vala"
		g_hash_table_insert (self->priv->root_nodes, (_tmp14_ = symbol->full_name, (_tmp14_ == NULL) ? NULL : g_strdup (_tmp14_)), (_tmp15_ = &iter, (_tmp15_ == NULL) ? NULL : _gtk_tree_iter_dup (_tmp15_)));
#line 1409 "symbol.c"
	}
	result = TRUE;
	markup = (g_free (markup), NULL);
	(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
	(parent == NULL) ? NULL : (parent = (g_free (parent), NULL));
	return result;
}


static void _g_list_free_g_object_unref (GList* self) {
	g_list_foreach (self, (GFunc) g_object_unref, NULL);
	g_list_free (self);
}


#line 446 "symbol.vala"
static gboolean symbol_browser_populate_symbols_asc (SymbolBrowser* self) {
#line 1427 "symbol.c"
	gboolean result;
	GError * _inner_error_;
#line 446 "symbol.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1432 "symbol.c"
	_inner_error_ = NULL;
#line 448 "symbol.vala"
	if (g_atomic_int_compare_and_exchange (&self->priv->atom, 0, 1)) {
#line 1436 "symbol.c"
		GtkTreeStore* _tmp2_;
		GtkTreeStore* _tmp1_;
		GtkTreeModel* _tmp0_;
		_tmp2_ = NULL;
#line 450 "symbol.vala"
		_tmp1_ = NULL;
#line 450 "symbol.vala"
		_tmp0_ = NULL;
#line 450 "symbol.vala"
		self->priv->model = (_tmp2_ = (_tmp1_ = (_tmp0_ = gtk_tree_view_get_model (self->priv->tree_view), GTK_IS_TREE_STORE (_tmp0_) ? ((GtkTreeStore*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (self->priv->model == NULL) ? NULL : (self->priv->model = (g_object_unref (self->priv->model), NULL)), _tmp2_);
#line 451 "symbol.vala"
		gtk_tree_view_set_model (self->priv->tree_view, NULL);
#line 452 "symbol.vala"
		gtk_widget_freeze_child_notify ((GtkWidget*) self->priv->tree_view);
#line 453 "symbol.vala"
		if (valide_document_manager_get_current (valide_window_get_documents (valide_plugin_get_window ((ValidePlugin*) self))) != NULL) {
#line 1453 "symbol.c"
			GList* doc;
			GList* _tmp3_;
			doc = NULL;
			_tmp3_ = NULL;
#line 457 "symbol.vala"
			doc = (_tmp3_ = NULL, (doc == NULL) ? NULL : (doc = (_g_list_free_g_object_unref (doc), NULL)), _tmp3_);
#line 1460 "symbol.c"
			/* find out if we want to load all symbols or just active tab */
#line 459 "symbol.vala"
			if (symbol_browser_get_only_load_active_tab (self)) {
#line 1464 "symbol.c"
				ValideiDocument* _tmp4_;
#line 461 "symbol.vala"
				_tmp4_ = NULL;
#line 461 "symbol.vala"
				doc = g_list_prepend (doc, (_tmp4_ = valide_document_manager_get_current (valide_window_get_documents (valide_plugin_get_window ((ValidePlugin*) self))), (_tmp4_ == NULL) ? NULL : g_object_ref (_tmp4_)));
#line 1470 "symbol.c"
			} else {
				GList* _tmp5_;
				_tmp5_ = NULL;
#line 465 "symbol.vala"
				doc = (_tmp5_ = g_list_copy (valide_document_manager_get_documents (valide_window_get_documents (valide_plugin_get_window ((ValidePlugin*) self)))), (doc == NULL) ? NULL : (doc = (_g_list_free_g_object_unref (doc), NULL)), _tmp5_);
#line 1476 "symbol.c"
			}
#line 468 "symbol.vala"
			symbol_browser_clear_data (self);
#line 1480 "symbol.c"
			/* run ctags on each document in the list and add symbols */
			{
				GList* d_collection;
				GList* d_it;
#line 470 "symbol.vala"
				d_collection = doc;
#line 1487 "symbol.c"
				for (d_it = d_collection; d_it != NULL; d_it = d_it->next) {
					ValideiDocument* _tmp25_;
					ValideiDocument* d;
#line 455 "symbol.vala"
					_tmp25_ = NULL;
#line 470 "symbol.vala"
					d = (_tmp25_ = (ValideiDocument*) d_it->data, (_tmp25_ == NULL) ? NULL : g_object_ref (_tmp25_));
#line 1495 "symbol.c"
					{
						gboolean _tmp6_;
						gboolean _tmp7_;
						_tmp6_ = FALSE;
						_tmp7_ = FALSE;
#line 472 "symbol.vala"
						if (VALIDE_IS_ABSTRACT_TEXT_DOCUMENT (d)) {
#line 472 "symbol.vala"
							_tmp7_ = valide_idocument_get_path (d) != NULL;
#line 1505 "symbol.c"
						} else {
#line 472 "symbol.vala"
							_tmp7_ = FALSE;
#line 1509 "symbol.c"
						}
#line 472 "symbol.vala"
						if (_tmp7_) {
#line 473 "symbol.vala"
							_tmp6_ = g_file_test (valide_idocument_get_path (d), G_FILE_TEST_EXISTS);
#line 1515 "symbol.c"
						} else {
#line 472 "symbol.vala"
							_tmp6_ = FALSE;
#line 1519 "symbol.c"
						}
#line 472 "symbol.vala"
						if (_tmp6_) {
#line 1523 "symbol.c"
							{
								char* output;
								Symbol* root;
								char** _tmp9_;
								gint lines_size;
								gint lines_length1;
								char** _tmp8_;
								char** lines;
								output = symbol_browser_exec_ctags (self, valide_idocument_get_path (d), &_inner_error_);
								if (_inner_error_ != NULL) {
									goto __catch1_g_error;
									goto __finally1;
								}
								root = NULL;
#line 480 "symbol.vala"
								_tmp9_ = NULL;
#line 1540 "symbol.c"
								_tmp8_ = NULL;
								lines = (_tmp9_ = _tmp8_ = g_strsplit (output, "\n", 0), lines_length1 = _vala_array_length (_tmp8_), lines_size = lines_length1, _tmp9_);
								{
									gint i;
									i = 0;
									{
										gboolean _tmp10_;
										_tmp10_ = TRUE;
#line 481 "symbol.vala"
										while (TRUE) {
#line 481 "symbol.vala"
											if (!_tmp10_) {
#line 481 "symbol.vala"
												i++;
#line 1555 "symbol.c"
											}
#line 481 "symbol.vala"
											_tmp10_ = FALSE;
#line 481 "symbol.vala"
											if (!(lines[i] != NULL)) {
#line 481 "symbol.vala"
												break;
#line 1563 "symbol.c"
											}
#line 483 "symbol.vala"
											if (_vala_strcmp0 (lines[i], "") != 0) {
#line 1567 "symbol.c"
												Symbol* symbol;
												Symbol* _tmp11_;
												symbol = NULL;
												_tmp11_ = NULL;
#line 487 "symbol.vala"
												symbol = (_tmp11_ = symbol_browser_parse_line (self, lines[i]), (symbol == NULL) ? NULL : (symbol = (symbol_unref (symbol), NULL)), _tmp11_);
#line 488 "symbol.vala"
												if (symbol != NULL) {
#line 1576 "symbol.c"
													Symbol* _tmp20_;
													Symbol* _tmp19_;
													gboolean _tmp21_;
													gboolean _tmp22_;
#line 490 "symbol.vala"
													symbol->document = VALIDE_ABSTRACT_TEXT_DOCUMENT (d);
#line 491 "symbol.vala"
													if (_vala_strcmp0 (symbol->type, "class") == 0) {
#line 1585 "symbol.c"
														Symbol* _tmp12_;
														_tmp12_ = NULL;
#line 493 "symbol.vala"
														root = (_tmp12_ = NULL, (root == NULL) ? NULL : (root = (symbol_unref (root), NULL)), _tmp12_);
#line 1590 "symbol.c"
													} else {
														gboolean _tmp13_;
														_tmp13_ = FALSE;
#line 495 "symbol.vala"
														if (root != NULL) {
#line 1596 "symbol.c"
															gboolean _tmp14_;
															gboolean _tmp15_;
															_tmp14_ = FALSE;
															_tmp15_ = FALSE;
#line 496 "symbol.vala"
															if (_vala_strcmp0 (root->type, "enum") == 0) {
#line 496 "symbol.vala"
																_tmp15_ = _vala_strcmp0 (symbol->type, "enumvalue") != 0;
#line 1605 "symbol.c"
															} else {
#line 496 "symbol.vala"
																_tmp15_ = FALSE;
#line 1609 "symbol.c"
															}
#line 496 "symbol.vala"
															if (_tmp15_) {
#line 496 "symbol.vala"
																_tmp14_ = TRUE;
#line 1615 "symbol.c"
															} else {
																gboolean _tmp16_;
																_tmp16_ = FALSE;
#line 497 "symbol.vala"
																if (_vala_strcmp0 (root->type, "errordomain") == 0) {
#line 497 "symbol.vala"
																	_tmp16_ = _vala_strcmp0 (symbol->type, "errorcode") != 0;
#line 1623 "symbol.c"
																} else {
#line 497 "symbol.vala"
																	_tmp16_ = FALSE;
#line 1627 "symbol.c"
																}
#line 497 "symbol.vala"
																_tmp14_ = _tmp16_;
#line 1631 "symbol.c"
															}
#line 496 "symbol.vala"
															_tmp13_ = _tmp14_;
#line 1635 "symbol.c"
														} else {
#line 495 "symbol.vala"
															_tmp13_ = FALSE;
#line 1639 "symbol.c"
														}
#line 495 "symbol.vala"
														if (_tmp13_) {
#line 1643 "symbol.c"
															Symbol* _tmp18_;
															Symbol* _tmp17_;
															_tmp18_ = NULL;
#line 499 "symbol.vala"
															_tmp17_ = NULL;
#line 499 "symbol.vala"
															root = (_tmp18_ = (_tmp17_ = root->root, (_tmp17_ == NULL) ? NULL : symbol_ref (_tmp17_)), (root == NULL) ? NULL : (root = (symbol_unref (root), NULL)), _tmp18_);
#line 1651 "symbol.c"
														}
													}
													_tmp20_ = NULL;
#line 501 "symbol.vala"
													_tmp19_ = NULL;
#line 501 "symbol.vala"
													symbol->root = (_tmp20_ = (_tmp19_ = root, (_tmp19_ == NULL) ? NULL : symbol_ref (_tmp19_)), (symbol->root == NULL) ? NULL : (symbol->root = (symbol_unref (symbol->root), NULL)), _tmp20_);
#line 502 "symbol.vala"
													symbol_browser_add_symbol (self, symbol);
#line 1661 "symbol.c"
													_tmp21_ = FALSE;
													_tmp22_ = FALSE;
#line 503 "symbol.vala"
													if (_vala_strcmp0 (symbol->type, "class") == 0) {
#line 503 "symbol.vala"
														_tmp22_ = TRUE;
#line 1668 "symbol.c"
													} else {
#line 503 "symbol.vala"
														_tmp22_ = _vala_strcmp0 (symbol->type, "enum") == 0;
#line 1672 "symbol.c"
													}
#line 503 "symbol.vala"
													if (_tmp22_) {
#line 503 "symbol.vala"
														_tmp21_ = TRUE;
#line 1678 "symbol.c"
													} else {
#line 504 "symbol.vala"
														_tmp21_ = _vala_strcmp0 (symbol->type, "errordomain") == 0;
#line 1682 "symbol.c"
													}
#line 503 "symbol.vala"
													if (_tmp21_) {
#line 1686 "symbol.c"
														Symbol* _tmp24_;
														Symbol* _tmp23_;
														_tmp24_ = NULL;
#line 506 "symbol.vala"
														_tmp23_ = NULL;
#line 506 "symbol.vala"
														root = (_tmp24_ = (_tmp23_ = symbol, (_tmp23_ == NULL) ? NULL : symbol_ref (_tmp23_)), (root == NULL) ? NULL : (root = (symbol_unref (root), NULL)), _tmp24_);
#line 1694 "symbol.c"
													}
												}
												(symbol == NULL) ? NULL : (symbol = (symbol_unref (symbol), NULL));
											}
										}
									}
								}
								output = (g_free (output), NULL);
								(root == NULL) ? NULL : (root = (symbol_unref (root), NULL));
								lines = (_vala_array_free (lines, lines_length1, (GDestroyNotify) g_free), NULL);
							}
							goto __finally1;
							__catch1_g_error:
							{
								GError * e;
								e = _inner_error_;
								_inner_error_ = NULL;
								{
#line 514 "symbol.vala"
									g_debug (_ ("Could not execute ctags: %s\n"), e->message);
#line 1715 "symbol.c"
									(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
								}
							}
							__finally1:
							if (_inner_error_ != NULL) {
								(d == NULL) ? NULL : (d = (g_object_unref (d), NULL));
								(doc == NULL) ? NULL : (doc = (_g_list_free_g_object_unref (doc), NULL));
								g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
								g_clear_error (&_inner_error_);
								return FALSE;
							}
						}
						(d == NULL) ? NULL : (d = (g_object_unref (d), NULL));
					}
				}
			}
#line 518 "symbol.vala"
			if (gtk_tree_model_iter_n_children ((GtkTreeModel*) self->priv->model, NULL) > 0) {
#line 520 "symbol.vala"
				g_object_set ((GtkWidget*) self->priv->box, "sensitive", TRUE, NULL);
#line 1736 "symbol.c"
			}
			(doc == NULL) ? NULL : (doc = (_g_list_free_g_object_unref (doc), NULL));
		} else {
#line 525 "symbol.vala"
			symbol_browser_clear_data (self);
#line 1742 "symbol.c"
		}
#line 527 "symbol.vala"
		gtk_tree_view_set_model (self->priv->tree_view, (GtkTreeModel*) self->priv->model);
#line 528 "symbol.vala"
		gtk_widget_thaw_child_notify ((GtkWidget*) self->priv->tree_view);
#line 1748 "symbol.c"
		/*      this.restore_expanded_rows ();*/
#line 530 "symbol.vala"
		gtk_tree_view_expand_all (self->priv->tree_view);
#line 531 "symbol.vala"
		g_atomic_int_set (&self->priv->atom, 0);
#line 1754 "symbol.c"
	}
	result = FALSE;
	return result;
}


#line 446 "symbol.vala"
static gboolean _symbol_browser_populate_symbols_asc_gsource_func (gpointer self) {
#line 1763 "symbol.c"
	return symbol_browser_populate_symbols_asc (self);
}


/**
   * Call to re-populate tree view.
   */
#line 539 "symbol.vala"
static void symbol_browser_populate_symbols (SymbolBrowser* self) {
#line 539 "symbol.vala"
	g_return_if_fail (self != NULL);
#line 541 "symbol.vala"
	g_idle_add (_symbol_browser_populate_symbols_asc_gsource_func, self);
#line 1777 "symbol.c"
}


/**
   * Re-expand any row when the list is repopulated if that row was previously
   * expanded.
   */
#line 548 "symbol.vala"
static void symbol_browser_restore_expanded_rows (SymbolBrowser* self) {
#line 1787 "symbol.c"
	gboolean _tmp0_;
#line 548 "symbol.vala"
	g_return_if_fail (self != NULL);
#line 1791 "symbol.c"
	/* is this going to fix the crash? */
#line 551 "symbol.vala"
	if ((g_object_get ((GtkWidget*) valide_window_get_documents (valide_plugin_get_window ((ValidePlugin*) self)), "sensitive", &_tmp0_, NULL), _tmp0_) == TRUE) {
#line 1795 "symbol.c"
		GtkTreeIter iter = {0};
		GtkTreeModel* _tmp1_;
		GtkTreeModel* model;
#line 554 "symbol.vala"
		_tmp1_ = NULL;
#line 1801 "symbol.c"
		model = (_tmp1_ = gtk_tree_view_get_model (self->priv->tree_view), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_));
#line 556 "symbol.vala"
		if (gtk_tree_model_get_iter_first (model, &iter) == TRUE) {
#line 1805 "symbol.c"
			{
				gboolean r;
				r = gtk_tree_model_get_iter_first (model, &iter);
				{
					gboolean _tmp2_;
					_tmp2_ = TRUE;
#line 558 "symbol.vala"
					while (TRUE) {
#line 1814 "symbol.c"
						char* name;
#line 558 "symbol.vala"
						if (!_tmp2_) {
#line 560 "symbol.vala"
							r = gtk_tree_model_iter_next (model, &iter);
#line 1820 "symbol.c"
						}
#line 558 "symbol.vala"
						_tmp2_ = FALSE;
#line 559 "symbol.vala"
						if (!(r == TRUE)) {
#line 559 "symbol.vala"
							break;
#line 1828 "symbol.c"
						}
						name = NULL;
#line 564 "symbol.vala"
						gtk_tree_model_get (model, &iter, SYMBOL_BROWSER_COL_NAME, &name, -1, -1);
#line 565 "symbol.vala"
						if (symbol_browser_is_expanded_row (self, name)) {
#line 1835 "symbol.c"
							GtkTreePath* path;
							path = gtk_tree_model_get_path (model, &iter);
#line 568 "symbol.vala"
							gtk_tree_view_expand_row (self->priv->tree_view, path, FALSE);
#line 1840 "symbol.c"
							(path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL));
						}
						name = (g_free (name), NULL);
					}
				}
			}
		}
		(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
	}
}


#line 575 "symbol.vala"
static void symbol_browser_on_tab_changed (SymbolBrowser* self) {
#line 575 "symbol.vala"
	g_return_if_fail (self != NULL);
#line 1857 "symbol.c"
	/* only needed in single-file mode  */
#line 578 "symbol.vala"
	if (symbol_browser_get_only_load_active_tab (self)) {
#line 580 "symbol.vala"
		symbol_browser_populate_symbols (self);
#line 1863 "symbol.c"
	}
}


#line 584 "symbol.vala"
static void symbol_browser_on_row_activated (SymbolBrowser* self, GtkTreeView* sender, const GtkTreePath* path, GtkTreeViewColumn* column) {
#line 1870 "symbol.c"
	GError * _inner_error_;
	GtkTreeIter iter = {0};
	GtkTreeModel* _tmp0_;
	GtkTreeModel* model;
	gboolean _tmp1_;
#line 584 "symbol.vala"
	g_return_if_fail (self != NULL);
#line 584 "symbol.vala"
	g_return_if_fail (sender != NULL);
#line 584 "symbol.vala"
	g_return_if_fail (path != NULL);
#line 584 "symbol.vala"
	g_return_if_fail (column != NULL);
#line 1884 "symbol.c"
	_inner_error_ = NULL;
#line 589 "symbol.vala"
	_tmp0_ = NULL;
#line 1888 "symbol.c"
	model = (_tmp0_ = gtk_tree_view_get_model (self->priv->tree_view), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
#line 590 "symbol.vala"
	gtk_tree_model_get_iter (model, &iter, path);
#line 1892 "symbol.c"
	_tmp1_ = FALSE;
#line 593 "symbol.vala"
	if (!symbol_browser_get_hierarchic_view (self)) {
#line 593 "symbol.vala"
		_tmp1_ = gtk_tree_model_iter_has_child (model, &iter);
#line 1898 "symbol.c"
	} else {
#line 593 "symbol.vala"
		_tmp1_ = FALSE;
#line 1902 "symbol.c"
	}
	/* expand/collapse row if it has children (and return) */
#line 593 "symbol.vala"
	if (_tmp1_) {
#line 595 "symbol.vala"
		if (gtk_tree_view_row_expanded (self->priv->tree_view, path)) {
#line 597 "symbol.vala"
			gtk_tree_view_collapse_row (self->priv->tree_view, path);
#line 1911 "symbol.c"
		} else {
#line 601 "symbol.vala"
			gtk_tree_view_expand_row (self->priv->tree_view, path, FALSE);
#line 1915 "symbol.c"
		}
	} else {
		gint line;
		char* name;
		char* filename;
		/* otherwise, show the file name and line number and flash statusbar */
		line = 0;
		name = NULL;
		filename = NULL;
#line 611 "symbol.vala"
		gtk_tree_model_get (model, &iter, SYMBOL_BROWSER_COL_NAME, &name, SYMBOL_BROWSER_COL_LINE, &line, SYMBOL_BROWSER_COL_FILE, &filename, -1);
#line 1927 "symbol.c"
		{
			ValideiDocument* document;
			ValideiDocument* _tmp2_;
			ValideiDocument* _tmp3_;
			document = NULL;
			_tmp2_ = valide_document_manager_create (valide_window_get_documents (valide_plugin_get_window ((ValidePlugin*) self)), filename, &_inner_error_);
			if (_inner_error_ != NULL) {
				(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
				goto __catch2_g_error;
				goto __finally2;
			}
			_tmp3_ = NULL;
#line 617 "symbol.vala"
			document = (_tmp3_ = _tmp2_, (document == NULL) ? NULL : (document = (g_object_unref (document), NULL)), _tmp3_);
#line 618 "symbol.vala"
			if (VALIDE_IS_ABSTRACT_TEXT_DOCUMENT (document)) {
#line 1944 "symbol.c"
				ValideiDocument* _tmp4_;
#line 620 "symbol.vala"
				_tmp4_ = NULL;
#line 620 "symbol.vala"
				valide_source_view_goto_line (valide_abstract_text_document_get_text_view ((_tmp4_ = document, VALIDE_IS_ABSTRACT_TEXT_DOCUMENT (_tmp4_) ? ((ValideAbstractTextDocument*) _tmp4_) : NULL)), line - 1);
#line 1950 "symbol.c"
			}
#line 622 "symbol.vala"
			gtk_widget_grab_focus ((GtkWidget*) document);
#line 1954 "symbol.c"
			(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
		}
		goto __finally2;
		__catch2_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
#line 626 "symbol.vala"
				g_warning (e->message);
#line 1966 "symbol.c"
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally2:
		if (_inner_error_ != NULL) {
			name = (g_free (name), NULL);
			filename = (g_free (filename), NULL);
			(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
		name = (g_free (name), NULL);
		filename = (g_free (filename), NULL);
	}
	(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
}


#line 631 "symbol.vala"
static void symbol_browser_on_cursor_changed (SymbolBrowser* self, GtkTreeView* sender) {
#line 1988 "symbol.c"
	GtkTreeIter iter = {0};
	GtkTreeModel* model;
	GtkTreeSelection* selection;
	GtkTreeModel* _tmp1_;
	GtkTreeModel* _tmp0_;
	GtkTreeSelection* _tmp3_;
	GtkTreeSelection* _tmp2_;
#line 631 "symbol.vala"
	g_return_if_fail (self != NULL);
#line 631 "symbol.vala"
	g_return_if_fail (sender != NULL);
#line 2000 "symbol.c"
	model = NULL;
	selection = NULL;
	_tmp1_ = NULL;
#line 637 "symbol.vala"
	_tmp0_ = NULL;
#line 637 "symbol.vala"
	model = (_tmp1_ = (_tmp0_ = gtk_tree_view_get_model (self->priv->tree_view), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (model == NULL) ? NULL : (model = (g_object_unref (model), NULL)), _tmp1_);
#line 2008 "symbol.c"
	_tmp3_ = NULL;
#line 638 "symbol.vala"
	_tmp2_ = NULL;
#line 638 "symbol.vala"
	selection = (_tmp3_ = (_tmp2_ = gtk_tree_view_get_selection (self->priv->tree_view), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL)), _tmp3_);
#line 639 "symbol.vala"
	if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
#line 2016 "symbol.c"
		gint line;
		char* name;
		char* filename;
		line = 0;
		name = NULL;
		filename = NULL;
#line 645 "symbol.vala"
		gtk_tree_model_get (model, &iter, SYMBOL_BROWSER_COL_NAME, &name, SYMBOL_BROWSER_COL_FILE, &filename, SYMBOL_BROWSER_COL_LINE, &line, -1);
#line 648 "symbol.vala"
		if (filename != NULL) {
#line 2027 "symbol.c"
			char* label_text;
			char* status_message;
			char* _tmp5_;
			char* _tmp4_;
			char* _tmp6_;
			label_text = NULL;
			status_message = NULL;
			_tmp5_ = NULL;
#line 2031 "glib-2.0.vapi"
			_tmp4_ = NULL;
#line 653 "symbol.vala"
			label_text = (_tmp5_ = g_strdup_printf ("%s Line:%d", _tmp4_ = g_path_get_basename (filename), line), label_text = (g_free (label_text), NULL), _tmp5_);
#line 2040 "symbol.c"
			_tmp4_ = (g_free (_tmp4_), NULL);
#line 654 "symbol.vala"
			gtk_label_set_text (self->priv->label, label_text);
#line 2044 "symbol.c"
			_tmp6_ = NULL;
#line 656 "symbol.vala"
			status_message = (_tmp6_ = g_strdup_printf ("Double-click to jump to '%s'", name), status_message = (g_free (status_message), NULL), _tmp6_);
#line 657 "symbol.vala"
			valide_statusbar_flash_message (valide_window_get_statusbar (valide_plugin_get_window ((ValidePlugin*) self)), self->priv->status_context, status_message);
#line 2050 "symbol.c"
			label_text = (g_free (label_text), NULL);
			status_message = (g_free (status_message), NULL);
		}
		name = (g_free (name), NULL);
		filename = (g_free (filename), NULL);
	}
	(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
	(selection == NULL) ? NULL : (selection = (g_object_unref (selection), NULL));
}


#line 662 "symbol.vala"
static void symbol_browser_on_row_expanded (SymbolBrowser* self, GtkTreeView* sender, GtkTreeIter* iter, const GtkTreePath* path) {
#line 2064 "symbol.c"
	char* name;
	GtkTreeModel* _tmp0_;
	GtkTreeModel* model;
#line 662 "symbol.vala"
	g_return_if_fail (self != NULL);
#line 662 "symbol.vala"
	g_return_if_fail (sender != NULL);
#line 662 "symbol.vala"
	g_return_if_fail (path != NULL);
#line 2074 "symbol.c"
	name = NULL;
#line 667 "symbol.vala"
	_tmp0_ = NULL;
#line 2078 "symbol.c"
	model = (_tmp0_ = gtk_tree_view_get_model (self->priv->tree_view), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
#line 668 "symbol.vala"
	gtk_tree_model_get (model, &(*iter), SYMBOL_BROWSER_COL_NAME, &name, -1, -1);
#line 669 "symbol.vala"
	g_return_if_fail (name != NULL);
#line 671 "symbol.vala"
	symbol_browser_add_expanded_row (self, name);
#line 2086 "symbol.c"
	name = (g_free (name), NULL);
	(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
}


#line 674 "symbol.vala"
static void symbol_browser_on_row_collapsed (SymbolBrowser* self, GtkTreeView* sender, GtkTreeIter* iter, const GtkTreePath* path) {
#line 2094 "symbol.c"
	char* name;
	GtkTreeModel* _tmp0_;
	GtkTreeModel* model;
#line 674 "symbol.vala"
	g_return_if_fail (self != NULL);
#line 674 "symbol.vala"
	g_return_if_fail (sender != NULL);
#line 674 "symbol.vala"
	g_return_if_fail (path != NULL);
#line 2104 "symbol.c"
	name = NULL;
#line 679 "symbol.vala"
	_tmp0_ = NULL;
#line 2108 "symbol.c"
	model = (_tmp0_ = gtk_tree_view_get_model (self->priv->tree_view), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
#line 680 "symbol.vala"
	gtk_tree_model_get (model, &(*iter), SYMBOL_BROWSER_COL_NAME, &name, -1, -1);
#line 681 "symbol.vala"
	g_return_if_fail (name != NULL);
#line 683 "symbol.vala"
	symbol_browser_remove_expanded_row (self, name);
#line 2116 "symbol.c"
	name = (g_free (name), NULL);
	(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
}


#line 789 "symbol.vala"
SymbolBrowser* symbol_browser_construct (GType object_type, const char* path) {
#line 2124 "symbol.c"
	GParameter * __params;
	GParameter * __params_it;
	SymbolBrowser * self;
#line 789 "symbol.vala"
	g_return_val_if_fail (path != NULL, NULL);
#line 2130 "symbol.c"
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
#line 791 "symbol.vala"
	__params_it->name = "path";
#line 791 "symbol.vala"
	g_value_init (&__params_it->value, G_TYPE_STRING);
#line 791 "symbol.vala"
	g_value_set_string (&__params_it->value, path);
#line 791 "symbol.vala"
	__params_it++;
#line 2141 "symbol.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 789 "symbol.vala"
SymbolBrowser* symbol_browser_new (const char* path) {
#line 789 "symbol.vala"
	return symbol_browser_construct (TYPE_SYMBOL_BROWSER, path);
#line 2156 "symbol.c"
}


static void _lambda0_ (GtkToggleButton* s, SymbolBrowser* self) {
	g_return_if_fail (s != NULL);
#line 816 "symbol.vala"
	symbol_browser_set_show_language_column (self, gtk_toggle_button_get_active (s));
#line 2164 "symbol.c"
}


static void __lambda0__gtk_toggle_button_toggled (GtkToggleButton* _sender, gpointer self) {
	_lambda0_ (_sender, self);
}


static void _lambda1_ (GtkToggleButton* s, SymbolBrowser* self) {
	g_return_if_fail (s != NULL);
#line 823 "symbol.vala"
	symbol_browser_set_show_source_file_column (self, gtk_toggle_button_get_active (s));
#line 2177 "symbol.c"
}


static void __lambda1__gtk_toggle_button_toggled (GtkToggleButton* _sender, gpointer self) {
	_lambda1_ (_sender, self);
}


static void _lambda2_ (GtkToggleButton* s, SymbolBrowser* self) {
	g_return_if_fail (s != NULL);
#line 830 "symbol.vala"
	symbol_browser_set_show_line_number_column (self, gtk_toggle_button_get_active (s));
#line 2190 "symbol.c"
}


static void __lambda2__gtk_toggle_button_toggled (GtkToggleButton* _sender, gpointer self) {
	_lambda2_ (_sender, self);
}


static void _lambda3_ (GtkToggleButton* s, SymbolBrowser* self) {
	g_return_if_fail (s != NULL);
#line 837 "symbol.vala"
	symbol_browser_set_hierarchic_view (self, gtk_toggle_button_get_active (s));
#line 2203 "symbol.c"
}


static void __lambda3__gtk_toggle_button_toggled (GtkToggleButton* _sender, gpointer self) {
	_lambda3_ (_sender, self);
}


#line 802 "symbol.vala"
GtkWidget* symbol_browser_create_configure_dialog (SymbolBrowser* self) {
#line 2214 "symbol.c"
	GtkWidget* result;
	GtkFrame* frame;
	GtkVBox* vbox;
	GtkCheckButton* check;
	GtkFrame* _tmp0_;
	GtkVBox* _tmp1_;
	GtkCheckButton* _tmp2_;
	GtkCheckButton* _tmp3_;
	GtkCheckButton* _tmp4_;
	GtkCheckButton* _tmp5_;
#line 802 "symbol.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 2227 "symbol.c"
	frame = NULL;
	vbox = NULL;
	check = NULL;
	_tmp0_ = NULL;
#line 808 "symbol.vala"
	frame = (_tmp0_ = g_object_ref_sink ((GtkFrame*) gtk_frame_new (_ ("Additionnal columns"))), (frame == NULL) ? NULL : (frame = (g_object_unref (frame), NULL)), _tmp0_);
#line 2234 "symbol.c"
	_tmp1_ = NULL;
#line 810 "symbol.vala"
	vbox = (_tmp1_ = g_object_ref_sink ((GtkVBox*) gtk_vbox_new (TRUE, 0)), (vbox == NULL) ? NULL : (vbox = (g_object_unref (vbox), NULL)), _tmp1_);
#line 811 "symbol.vala"
	gtk_container_add ((GtkContainer*) frame, (GtkWidget*) vbox);
#line 2240 "symbol.c"
	_tmp2_ = NULL;
#line 813 "symbol.vala"
	check = (_tmp2_ = g_object_ref_sink ((GtkCheckButton*) gtk_check_button_new_with_label (_ ("Programming language"))), (check == NULL) ? NULL : (check = (g_object_unref (check), NULL)), _tmp2_);
#line 814 "symbol.vala"
	gtk_toggle_button_set_active ((GtkToggleButton*) check, symbol_browser_get_show_language_column (self));
#line 815 "symbol.vala"
	g_signal_connect ((GtkToggleButton*) check, "toggled", (GCallback) __lambda0__gtk_toggle_button_toggled, self);
#line 818 "symbol.vala"
	gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) check, TRUE, TRUE, (guint) 0);
#line 2250 "symbol.c"
	_tmp3_ = NULL;
#line 820 "symbol.vala"
	check = (_tmp3_ = g_object_ref_sink ((GtkCheckButton*) gtk_check_button_new_with_label (_ ("Source file"))), (check == NULL) ? NULL : (check = (g_object_unref (check), NULL)), _tmp3_);
#line 821 "symbol.vala"
	gtk_toggle_button_set_active ((GtkToggleButton*) check, symbol_browser_get_show_source_file_column (self));
#line 822 "symbol.vala"
	g_signal_connect ((GtkToggleButton*) check, "toggled", (GCallback) __lambda1__gtk_toggle_button_toggled, self);
#line 825 "symbol.vala"
	gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) check, TRUE, TRUE, (guint) 0);
#line 2260 "symbol.c"
	_tmp4_ = NULL;
#line 827 "symbol.vala"
	check = (_tmp4_ = g_object_ref_sink ((GtkCheckButton*) gtk_check_button_new_with_label (_ ("Line number"))), (check == NULL) ? NULL : (check = (g_object_unref (check), NULL)), _tmp4_);
#line 828 "symbol.vala"
	gtk_toggle_button_set_active ((GtkToggleButton*) check, symbol_browser_get_show_line_number_column (self));
#line 829 "symbol.vala"
	g_signal_connect ((GtkToggleButton*) check, "toggled", (GCallback) __lambda2__gtk_toggle_button_toggled, self);
#line 832 "symbol.vala"
	gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) check, TRUE, TRUE, (guint) 0);
#line 2270 "symbol.c"
	_tmp5_ = NULL;
#line 834 "symbol.vala"
	check = (_tmp5_ = g_object_ref_sink ((GtkCheckButton*) gtk_check_button_new_with_label (_ ("Hierarchic view"))), (check == NULL) ? NULL : (check = (g_object_unref (check), NULL)), _tmp5_);
#line 835 "symbol.vala"
	gtk_toggle_button_set_active ((GtkToggleButton*) check, symbol_browser_get_hierarchic_view (self));
#line 836 "symbol.vala"
	g_signal_connect ((GtkToggleButton*) check, "toggled", (GCallback) __lambda3__gtk_toggle_button_toggled, self);
#line 839 "symbol.vala"
	gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) check, TRUE, TRUE, (guint) 0);
#line 2280 "symbol.c"
	result = (GtkWidget*) frame;
	(vbox == NULL) ? NULL : (vbox = (g_object_unref (vbox), NULL));
	(check == NULL) ? NULL : (check = (g_object_unref (check), NULL));
	return result;
}


static const char* symbol_browser_real_get_path (ValidePlugin* base) {
	const char* result;
	SymbolBrowser* self;
	self = (SymbolBrowser*) base;
	result = self->priv->_path;
	return result;
}


static void symbol_browser_real_set_path (ValidePlugin* base, const char* value) {
	SymbolBrowser* self;
	char* _tmp1_;
	const char* _tmp0_;
	self = (SymbolBrowser*) base;
	_tmp1_ = NULL;
#line 78 "symbol.vala"
	_tmp0_ = NULL;
#line 2305 "symbol.c"
	self->priv->_path = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_path = (g_free (self->priv->_path), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "path");
}


static ValideWindow* symbol_browser_real_get_window (ValidePlugin* base) {
	ValideWindow* result;
	SymbolBrowser* self;
	self = (SymbolBrowser*) base;
	result = self->priv->_window;
	return result;
}


static void symbol_browser_real_set_window (ValidePlugin* base, ValideWindow* value) {
	SymbolBrowser* self;
	ValideWindow* _tmp1_;
	ValideWindow* _tmp0_;
	self = (SymbolBrowser*) base;
	_tmp1_ = NULL;
#line 79 "symbol.vala"
	_tmp0_ = NULL;
#line 2328 "symbol.c"
	self->priv->_window = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "window");
}


static gboolean symbol_browser_get_show_language_column (SymbolBrowser* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = valide_abstract_config_manager_get_boolean ((ValideAbstractConfigManager*) valide_window_get_config_manager (valide_plugin_get_window ((ValidePlugin*) self)), "SymbolBrowser", "show-language");
	return result;
}


static void symbol_browser_set_show_language_column (SymbolBrowser* self, gboolean value) {
	g_return_if_fail (self != NULL);
#line 89 "symbol.vala"
	valide_abstract_config_manager_set_boolean ((ValideAbstractConfigManager*) valide_window_get_config_manager (valide_plugin_get_window ((ValidePlugin*) self)), "SymbolBrowser", "show-language", value);
#line 91 "symbol.vala"
	gtk_tree_view_column_set_visible (self->priv->language_column, value);
#line 2348 "symbol.c"
}


static gboolean symbol_browser_get_show_source_file_column (SymbolBrowser* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = valide_abstract_config_manager_get_boolean ((ValideAbstractConfigManager*) valide_window_get_config_manager (valide_plugin_get_window ((ValidePlugin*) self)), "SymbolBrowser", "show-source-file");
	return result;
}


static void symbol_browser_set_show_source_file_column (SymbolBrowser* self, gboolean value) {
	g_return_if_fail (self != NULL);
#line 103 "symbol.vala"
	valide_abstract_config_manager_set_boolean ((ValideAbstractConfigManager*) valide_window_get_config_manager (valide_plugin_get_window ((ValidePlugin*) self)), "SymbolBrowser", "show-source-file", value);
#line 105 "symbol.vala"
	gtk_tree_view_column_set_visible (self->priv->source_file_column, value);
#line 2366 "symbol.c"
}


static gboolean symbol_browser_get_show_line_number_column (SymbolBrowser* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = valide_abstract_config_manager_get_boolean ((ValideAbstractConfigManager*) valide_window_get_config_manager (valide_plugin_get_window ((ValidePlugin*) self)), "SymbolBrowser", "show-line-number");
	return result;
}


static void symbol_browser_set_show_line_number_column (SymbolBrowser* self, gboolean value) {
	g_return_if_fail (self != NULL);
#line 117 "symbol.vala"
	valide_abstract_config_manager_set_boolean ((ValideAbstractConfigManager*) valide_window_get_config_manager (valide_plugin_get_window ((ValidePlugin*) self)), "SymbolBrowser", "show-line-number", value);
#line 119 "symbol.vala"
	gtk_tree_view_column_set_visible (self->priv->line_number_column, value);
#line 2384 "symbol.c"
}


static gboolean symbol_browser_get_only_load_active_tab (SymbolBrowser* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = TRUE;
	return result;
}


static gboolean symbol_browser_get_hierarchic_view (SymbolBrowser* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = valide_abstract_config_manager_get_boolean ((ValideAbstractConfigManager*) valide_window_get_config_manager (valide_plugin_get_window ((ValidePlugin*) self)), "SymbolBrowser", "hierarchic-view");
	return result;
}


static void symbol_browser_set_hierarchic_view (SymbolBrowser* self, gboolean value) {
	g_return_if_fail (self != NULL);
#line 139 "symbol.vala"
	valide_abstract_config_manager_set_boolean ((ValideAbstractConfigManager*) valide_window_get_config_manager (valide_plugin_get_window ((ValidePlugin*) self)), "SymbolBrowser", "hierarchic-view", value);
#line 140 "symbol.vala"
	symbol_browser_populate_symbols (self);
#line 2410 "symbol.c"
}


#line 584 "symbol.vala"
static void _symbol_browser_on_row_activated_gtk_tree_view_row_activated (GtkTreeView* _sender, const GtkTreePath* path, GtkTreeViewColumn* column, gpointer self) {
#line 2416 "symbol.c"
	symbol_browser_on_row_activated (self, _sender, path, column);
}


#line 631 "symbol.vala"
static void _symbol_browser_on_cursor_changed_gtk_tree_view_cursor_changed (GtkTreeView* _sender, gpointer self) {
#line 2423 "symbol.c"
	symbol_browser_on_cursor_changed (self, _sender);
}


#line 662 "symbol.vala"
static void _symbol_browser_on_row_expanded_gtk_tree_view_row_expanded (GtkTreeView* _sender, GtkTreeIter* iter, const GtkTreePath* path, gpointer self) {
#line 2430 "symbol.c"
	symbol_browser_on_row_expanded (self, _sender, iter, path);
}


#line 674 "symbol.vala"
static void _symbol_browser_on_row_collapsed_gtk_tree_view_row_collapsed (GtkTreeView* _sender, GtkTreeIter* iter, const GtkTreePath* path, gpointer self) {
#line 2437 "symbol.c"
	symbol_browser_on_row_collapsed (self, _sender, iter, path);
}


#line 539 "symbol.vala"
static void _symbol_browser_populate_symbols_valide_document_manager_tab_removed (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self) {
#line 2444 "symbol.c"
	symbol_browser_populate_symbols (self);
}


#line 575 "symbol.vala"
static void _symbol_browser_on_tab_changed_valide_document_manager_tab_changed (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self) {
#line 2451 "symbol.c"
	symbol_browser_on_tab_changed (self);
}


#line 575 "symbol.vala"
static void _symbol_browser_on_tab_changed_valide_document_manager_tab_saved (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self) {
#line 2458 "symbol.c"
	symbol_browser_on_tab_changed (self);
}


static GObject * symbol_browser_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	SymbolBrowserClass * klass;
	GObjectClass * parent_class;
	SymbolBrowser * self;
	klass = SYMBOL_BROWSER_CLASS (g_type_class_peek (TYPE_SYMBOL_BROWSER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = SYMBOL_BROWSER (obj);
	{
		GtkScrolledWindow* scrolled;
		GtkTreeStore* store;
		GtkTreeViewColumn* col;
		GtkCellRendererText* renderer;
		GtkCellRendererPixbuf* renderer_pixbuf;
		GtkImage* image;
		GtkVBox* _tmp0_;
		GtkScrolledWindow* _tmp1_;
		GtkTreeView* _tmp2_;
		GtkLabel* _tmp3_;
		GtkTreeStore* _tmp4_;
		GtkTreeViewColumn* _tmp5_;
		GtkCellRendererPixbuf* _tmp6_;
		GtkCellRendererText* _tmp7_;
		GtkTreeViewColumn* _tmp8_;
		GtkCellRendererText* _tmp9_;
		GtkTreeViewColumn* _tmp10_;
		GtkCellRendererText* _tmp11_;
		GtkTreeViewColumn* _tmp12_;
		GtkCellRendererText* _tmp13_;
		GtkImage* _tmp15_;
		char* _tmp14_;
		scrolled = NULL;
		store = NULL;
		col = NULL;
		renderer = NULL;
		renderer_pixbuf = NULL;
		image = NULL;
#line 695 "symbol.vala"
		g_atomic_int_set (&self->priv->atom, 0);
#line 2503 "symbol.c"
		_tmp0_ = NULL;
#line 697 "symbol.vala"
		self->priv->box = (_tmp0_ = g_object_ref_sink ((GtkVBox*) gtk_vbox_new (FALSE, 0)), (self->priv->box == NULL) ? NULL : (self->priv->box = (g_object_unref (self->priv->box), NULL)), _tmp0_);
#line 2507 "symbol.c"
		/* create some nice padding */
#line 700 "symbol.vala"
		gtk_box_set_spacing ((GtkBox*) self->priv->box, 2);
#line 701 "symbol.vala"
		gtk_container_set_border_width ((GtkContainer*) self->priv->box, (guint) 2);
#line 2513 "symbol.c"
		/* create and pack tree view */
		_tmp1_ = NULL;
#line 704 "symbol.vala"
		scrolled = (_tmp1_ = g_object_ref_sink ((GtkScrolledWindow*) gtk_scrolled_window_new (NULL, NULL)), (scrolled == NULL) ? NULL : (scrolled = (g_object_unref (scrolled), NULL)), _tmp1_);
#line 705 "symbol.vala"
		gtk_scrolled_window_set_policy (scrolled, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
#line 706 "symbol.vala"
		gtk_scrolled_window_set_shadow_type (scrolled, GTK_SHADOW_IN);
#line 707 "symbol.vala"
		gtk_box_pack_start ((GtkBox*) self->priv->box, (GtkWidget*) scrolled, TRUE, TRUE, (guint) 0);
#line 2524 "symbol.c"
		_tmp2_ = NULL;
#line 709 "symbol.vala"
		self->priv->tree_view = (_tmp2_ = g_object_ref_sink ((GtkTreeView*) gtk_tree_view_new ()), (self->priv->tree_view == NULL) ? NULL : (self->priv->tree_view = (g_object_unref (self->priv->tree_view), NULL)), _tmp2_);
#line 710 "symbol.vala"
		gtk_tree_view_set_rules_hint (self->priv->tree_view, FALSE);
#line 711 "symbol.vala"
		gtk_tree_view_set_headers_visible (self->priv->tree_view, TRUE);
#line 712 "symbol.vala"
		gtk_tree_view_set_headers_clickable (self->priv->tree_view, TRUE);
#line 713 "symbol.vala"
		gtk_container_add ((GtkContainer*) scrolled, (GtkWidget*) self->priv->tree_view);
#line 2536 "symbol.c"
		/* create and pack label */
		_tmp3_ = NULL;
#line 716 "symbol.vala"
		self->priv->label = (_tmp3_ = g_object_ref_sink ((GtkLabel*) gtk_label_new ("")), (self->priv->label == NULL) ? NULL : (self->priv->label = (g_object_unref (self->priv->label), NULL)), _tmp3_);
#line 717 "symbol.vala"
		gtk_misc_set_alignment ((GtkMisc*) self->priv->label, (float) 0.0, (float) 0.5);
#line 718 "symbol.vala"
		gtk_box_pack_start ((GtkBox*) self->priv->box, (GtkWidget*) self->priv->label, FALSE, TRUE, (guint) 0);
#line 2545 "symbol.c"
		/* create model */
		_tmp4_ = NULL;
#line 721 "symbol.vala"
		store = (_tmp4_ = gtk_tree_store_new ((gint) SYMBOL_BROWSER_COL_NB, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING, G_TYPE_OBJECT, NULL), (store == NULL) ? NULL : (store = (g_object_unref (store), NULL)), _tmp4_);
#line 2550 "symbol.c"
		/*store.set_sort_column_id (1, Gtk.SortType.ASCENDING);
		 main column */
		_tmp5_ = NULL;
#line 727 "symbol.vala"
		col = (_tmp5_ = g_object_ref_sink (gtk_tree_view_column_new ()), (col == NULL) ? NULL : (col = (g_object_unref (col), NULL)), _tmp5_);
#line 728 "symbol.vala"
		gtk_tree_view_column_set_title (col, _ ("Symbol"));
#line 2558 "symbol.c"
		_tmp6_ = NULL;
#line 730 "symbol.vala"
		renderer_pixbuf = (_tmp6_ = g_object_ref_sink ((GtkCellRendererPixbuf*) gtk_cell_renderer_pixbuf_new ()), (renderer_pixbuf == NULL) ? NULL : (renderer_pixbuf = (g_object_unref (renderer_pixbuf), NULL)), _tmp6_);
#line 731 "symbol.vala"
		gtk_cell_layout_pack_start ((GtkCellLayout*) col, (GtkCellRenderer*) renderer_pixbuf, FALSE);
#line 732 "symbol.vala"
		gtk_tree_view_column_set_attributes (col, (GtkCellRenderer*) renderer_pixbuf, "pixbuf", SYMBOL_BROWSER_COL_ICON, NULL);
#line 2566 "symbol.c"
		_tmp7_ = NULL;
#line 734 "symbol.vala"
		renderer = (_tmp7_ = g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL)), _tmp7_);
#line 735 "symbol.vala"
		gtk_cell_layout_pack_start ((GtkCellLayout*) col, (GtkCellRenderer*) renderer, FALSE);
#line 736 "symbol.vala"
		gtk_tree_view_column_set_attributes (col, (GtkCellRenderer*) renderer, "markup", SYMBOL_BROWSER_COL_NAME, NULL);
#line 737 "symbol.vala"
		gtk_tree_view_append_column (self->priv->tree_view, col);
#line 2576 "symbol.c"
		/* additional columns */
		_tmp8_ = NULL;
#line 741 "symbol.vala"
		self->priv->language_column = (_tmp8_ = g_object_ref_sink (gtk_tree_view_column_new ()), (self->priv->language_column == NULL) ? NULL : (self->priv->language_column = (g_object_unref (self->priv->language_column), NULL)), _tmp8_);
#line 742 "symbol.vala"
		symbol_browser_set_show_language_column (self, valide_abstract_config_manager_get_boolean ((ValideAbstractConfigManager*) valide_window_get_config_manager (valide_plugin_get_window ((ValidePlugin*) self)), "SymbolBrowser", "show-language"));
#line 743 "symbol.vala"
		gtk_tree_view_column_set_title (self->priv->language_column, _ ("Language"));
#line 2585 "symbol.c"
		_tmp9_ = NULL;
#line 744 "symbol.vala"
		renderer = (_tmp9_ = g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL)), _tmp9_);
#line 745 "symbol.vala"
		gtk_cell_layout_pack_start ((GtkCellLayout*) self->priv->language_column, (GtkCellRenderer*) renderer, TRUE);
#line 746 "symbol.vala"
		gtk_tree_view_column_set_attributes (self->priv->language_column, (GtkCellRenderer*) renderer, "text", SYMBOL_BROWSER_COL_LANGUAGE, NULL);
#line 747 "symbol.vala"
		gtk_tree_view_append_column (self->priv->tree_view, self->priv->language_column);
#line 2595 "symbol.c"
		_tmp10_ = NULL;
#line 749 "symbol.vala"
		self->priv->source_file_column = (_tmp10_ = g_object_ref_sink (gtk_tree_view_column_new ()), (self->priv->source_file_column == NULL) ? NULL : (self->priv->source_file_column = (g_object_unref (self->priv->source_file_column), NULL)), _tmp10_);
#line 750 "symbol.vala"
		symbol_browser_set_show_source_file_column (self, valide_abstract_config_manager_get_boolean ((ValideAbstractConfigManager*) valide_window_get_config_manager (valide_plugin_get_window ((ValidePlugin*) self)), "SymbolBrowser", "show-source-file"));
#line 751 "symbol.vala"
		gtk_tree_view_column_set_title (self->priv->source_file_column, _ ("File"));
#line 2603 "symbol.c"
		_tmp11_ = NULL;
#line 752 "symbol.vala"
		renderer = (_tmp11_ = g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL)), _tmp11_);
#line 753 "symbol.vala"
		gtk_cell_layout_pack_start ((GtkCellLayout*) self->priv->source_file_column, (GtkCellRenderer*) renderer, TRUE);
#line 754 "symbol.vala"
		gtk_tree_view_column_set_attributes (self->priv->source_file_column, (GtkCellRenderer*) renderer, "text", SYMBOL_BROWSER_COL_FILE, NULL);
#line 755 "symbol.vala"
		gtk_tree_view_append_column (self->priv->tree_view, self->priv->source_file_column);
#line 2613 "symbol.c"
		_tmp12_ = NULL;
#line 757 "symbol.vala"
		self->priv->line_number_column = (_tmp12_ = g_object_ref_sink (gtk_tree_view_column_new ()), (self->priv->line_number_column == NULL) ? NULL : (self->priv->line_number_column = (g_object_unref (self->priv->line_number_column), NULL)), _tmp12_);
#line 758 "symbol.vala"
		symbol_browser_set_show_line_number_column (self, valide_abstract_config_manager_get_boolean ((ValideAbstractConfigManager*) valide_window_get_config_manager (valide_plugin_get_window ((ValidePlugin*) self)), "SymbolBrowser", "show-line-number"));
#line 759 "symbol.vala"
		gtk_tree_view_column_set_title (self->priv->line_number_column, _ ("Line"));
#line 2621 "symbol.c"
		_tmp13_ = NULL;
#line 760 "symbol.vala"
		renderer = (_tmp13_ = g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL)), _tmp13_);
#line 761 "symbol.vala"
		gtk_cell_layout_pack_start ((GtkCellLayout*) self->priv->line_number_column, (GtkCellRenderer*) renderer, TRUE);
#line 762 "symbol.vala"
		gtk_tree_view_column_set_attributes (self->priv->line_number_column, (GtkCellRenderer*) renderer, "text", SYMBOL_BROWSER_COL_LINE, NULL);
#line 763 "symbol.vala"
		gtk_tree_view_append_column (self->priv->tree_view, self->priv->line_number_column);
#line 765 "symbol.vala"
		gtk_tree_view_set_model (self->priv->tree_view, (GtkTreeModel*) store);
#line 2633 "symbol.c"
		/* attach signal callbacks for tree view */
#line 768 "symbol.vala"
		g_signal_connect_object (self->priv->tree_view, "row-activated", (GCallback) _symbol_browser_on_row_activated_gtk_tree_view_row_activated, self, 0);
#line 769 "symbol.vala"
		g_signal_connect_object (self->priv->tree_view, "cursor-changed", (GCallback) _symbol_browser_on_cursor_changed_gtk_tree_view_cursor_changed, self, 0);
#line 770 "symbol.vala"
		g_signal_connect_object (self->priv->tree_view, "row-expanded", (GCallback) _symbol_browser_on_row_expanded_gtk_tree_view_row_expanded, self, 0);
#line 771 "symbol.vala"
		g_signal_connect_object (self->priv->tree_view, "row-collapsed", (GCallback) _symbol_browser_on_row_collapsed_gtk_tree_view_row_collapsed, self, 0);
#line 773 "symbol.vala"
		self->priv->status_context = gtk_statusbar_get_context_id ((GtkStatusbar*) valide_window_get_statusbar (valide_plugin_get_window ((ValidePlugin*) self)), "valide_symbol_browser");
#line 775 "symbol.vala"
		g_object_set ((GtkWidget*) self->priv->box, "sensitive", FALSE, NULL);
#line 777 "symbol.vala"
		g_signal_connect_object (valide_window_get_documents (valide_plugin_get_window ((ValidePlugin*) self)), "tab-removed", (GCallback) _symbol_browser_populate_symbols_valide_document_manager_tab_removed, self, 0);
#line 778 "symbol.vala"
		g_signal_connect_object (valide_window_get_documents (valide_plugin_get_window ((ValidePlugin*) self)), "tab-changed", (GCallback) _symbol_browser_on_tab_changed_valide_document_manager_tab_changed, self, 0);
#line 779 "symbol.vala"
		g_signal_connect_object (valide_window_get_documents (valide_plugin_get_window ((ValidePlugin*) self)), "tab-saved", (GCallback) _symbol_browser_on_tab_changed_valide_document_manager_tab_saved, self, 0);
#line 2653 "symbol.c"
		_tmp15_ = NULL;
#line 2034 "glib-2.0.vapi"
		_tmp14_ = NULL;
#line 781 "symbol.vala"
		image = (_tmp15_ = g_object_ref_sink ((GtkImage*) gtk_image_new_from_file (_tmp14_ = g_build_filename (valide_plugin_get_path ((ValidePlugin*) self), "symbol-browser.png", NULL))), (image == NULL) ? NULL : (image = (g_object_unref (image), NULL)), _tmp15_);
#line 2659 "symbol.c"
		_tmp14_ = (g_free (_tmp14_), NULL);
#line 784 "symbol.vala"
		valide_panel_add_item (valide_window_get_side_panel (valide_plugin_get_window ((ValidePlugin*) self)), (GtkWidget*) self->priv->box, "Symbol Browser", image);
#line 786 "symbol.vala"
		gtk_widget_show_all ((GtkWidget*) self->priv->box);
#line 2665 "symbol.c"
		(scrolled == NULL) ? NULL : (scrolled = (g_object_unref (scrolled), NULL));
		(store == NULL) ? NULL : (store = (g_object_unref (store), NULL));
		(col == NULL) ? NULL : (col = (g_object_unref (col), NULL));
		(renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL));
		(renderer_pixbuf == NULL) ? NULL : (renderer_pixbuf = (g_object_unref (renderer_pixbuf), NULL));
		(image == NULL) ? NULL : (image = (g_object_unref (image), NULL));
	}
	return obj;
}


static void symbol_browser_class_init (SymbolBrowserClass * klass) {
	symbol_browser_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (SymbolBrowserPrivate));
	G_OBJECT_CLASS (klass)->get_property = symbol_browser_get_property;
	G_OBJECT_CLASS (klass)->set_property = symbol_browser_set_property;
	G_OBJECT_CLASS (klass)->constructor = symbol_browser_constructor;
	G_OBJECT_CLASS (klass)->finalize = symbol_browser_finalize;
	g_object_class_override_property (G_OBJECT_CLASS (klass), SYMBOL_BROWSER_PATH, "path");
	g_object_class_override_property (G_OBJECT_CLASS (klass), SYMBOL_BROWSER_WINDOW, "window");
}


static void symbol_browser_valide_plugin_interface_init (ValidePluginIface * iface) {
	symbol_browser_valide_plugin_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_path = symbol_browser_real_get_path;
	iface->set_path = symbol_browser_real_set_path;
	iface->get_window = symbol_browser_real_get_window;
	iface->set_window = symbol_browser_real_set_window;
}


static void symbol_browser_instance_init (SymbolBrowser * self) {
	self->priv = SYMBOL_BROWSER_GET_PRIVATE (self);
}


static void symbol_browser_finalize (GObject* obj) {
	SymbolBrowser * self;
	self = SYMBOL_BROWSER (obj);
	{
		guint _tmp16_;
		guint _tmp17_;
#line 796 "symbol.vala"
		g_signal_handlers_disconnect_matched (valide_window_get_documents (valide_plugin_get_window ((ValidePlugin*) self)), G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("tab-removed", VALIDE_TYPE_DOCUMENT_MANAGER, &_tmp16_, NULL, FALSE), _tmp16_), 0, NULL, (GCallback) _symbol_browser_populate_symbols_valide_document_manager_tab_removed, self);
#line 797 "symbol.vala"
		g_signal_handlers_disconnect_matched (valide_window_get_documents (valide_plugin_get_window ((ValidePlugin*) self)), G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("tab-changed", VALIDE_TYPE_DOCUMENT_MANAGER, &_tmp17_, NULL, FALSE), _tmp17_), 0, NULL, (GCallback) _symbol_browser_on_tab_changed_valide_document_manager_tab_changed, self);
#line 798 "symbol.vala"
		g_signal_connect_object (valide_window_get_documents (valide_plugin_get_window ((ValidePlugin*) self)), "tab-saved", (GCallback) _symbol_browser_on_tab_changed_valide_document_manager_tab_saved, self, 0);
#line 799 "symbol.vala"
		valide_panel_remove_item (valide_window_get_side_panel (valide_plugin_get_window ((ValidePlugin*) self)), (GtkWidget*) self->priv->box);
#line 2717 "symbol.c"
	}
	(self->priv->box == NULL) ? NULL : (self->priv->box = (g_object_unref (self->priv->box), NULL));
	(self->priv->tree_view == NULL) ? NULL : (self->priv->tree_view = (g_object_unref (self->priv->tree_view), NULL));
	(self->priv->model == NULL) ? NULL : (self->priv->model = (g_object_unref (self->priv->model), NULL));
	(self->priv->label == NULL) ? NULL : (self->priv->label = (g_object_unref (self->priv->label), NULL));
	(self->priv->expanded_rows == NULL) ? NULL : (self->priv->expanded_rows = (_g_slist_free_g_free (self->priv->expanded_rows), NULL));
	(self->priv->language_column == NULL) ? NULL : (self->priv->language_column = (g_object_unref (self->priv->language_column), NULL));
	(self->priv->source_file_column == NULL) ? NULL : (self->priv->source_file_column = (g_object_unref (self->priv->source_file_column), NULL));
	(self->priv->line_number_column == NULL) ? NULL : (self->priv->line_number_column = (g_object_unref (self->priv->line_number_column), NULL));
	(self->priv->root_nodes == NULL) ? NULL : (self->priv->root_nodes = (g_hash_table_unref (self->priv->root_nodes), NULL));
	self->priv->_path = (g_free (self->priv->_path), NULL);
	(self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL));
	G_OBJECT_CLASS (symbol_browser_parent_class)->finalize (obj);
}


GType symbol_browser_get_type (void) {
	static GType symbol_browser_type_id = 0;
	if (symbol_browser_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (SymbolBrowserClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) symbol_browser_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SymbolBrowser), 0, (GInstanceInitFunc) symbol_browser_instance_init, NULL };
		static const GInterfaceInfo valide_plugin_info = { (GInterfaceInitFunc) symbol_browser_valide_plugin_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		symbol_browser_type_id = g_type_register_static (G_TYPE_OBJECT, "SymbolBrowser", &g_define_type_info, 0);
		g_type_add_interface_static (symbol_browser_type_id, VALIDE_TYPE_PLUGIN, &valide_plugin_info);
	}
	return symbol_browser_type_id;
}


static void symbol_browser_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SymbolBrowser * self;
	gpointer boxed;
	self = SYMBOL_BROWSER (object);
	switch (property_id) {
		case SYMBOL_BROWSER_PATH:
		g_value_set_string (value, valide_plugin_get_path ((ValidePlugin*) self));
		break;
		case SYMBOL_BROWSER_WINDOW:
		g_value_set_object (value, valide_plugin_get_window ((ValidePlugin*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void symbol_browser_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SymbolBrowser * self;
	self = SYMBOL_BROWSER (object);
	switch (property_id) {
		case SYMBOL_BROWSER_PATH:
		valide_plugin_set_path ((ValidePlugin*) self, g_value_get_string (value));
		break;
		case SYMBOL_BROWSER_WINDOW:
		valide_plugin_set_window ((ValidePlugin*) self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


#line 846 "symbol.vala"
GType register_plugin (void) {
#line 2783 "symbol.c"
	GType result;
	result = TYPE_SYMBOL_BROWSER;
	return result;
}


#line 851 "symbol.vala"
GtkWidget* create_configure_dialog (SymbolBrowser* _self_) {
#line 2792 "symbol.c"
	GtkWidget* result;
#line 851 "symbol.vala"
	g_return_val_if_fail (_self_ != NULL, NULL);
#line 2796 "symbol.c"
	result = symbol_browser_create_configure_dialog (_self_);
	return result;
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




