/* todo-manager.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <valide-0.0.h>
#include <gtk/gtk.h>
#include <glib/gi18n-lib.h>
#include <gobject/gvaluecollector.h>


#define TYPE_TODO_ITEM (todo_item_get_type ())
#define TODO_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_TODO_ITEM, TodoItem))
#define TODO_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_TODO_ITEM, TodoItemClass))
#define IS_TODO_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_TODO_ITEM))
#define IS_TODO_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_TODO_ITEM))
#define TODO_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_TODO_ITEM, TodoItemClass))

typedef struct _TodoItem TodoItem;
typedef struct _TodoItemClass TodoItemClass;
typedef struct _TodoItemPrivate TodoItemPrivate;
typedef struct _ParamSpecTodoItem ParamSpecTodoItem;

#define TYPE_TODO (todo_get_type ())
#define TODO(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_TODO, Todo))
#define TODO_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_TODO, TodoClass))
#define IS_TODO(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_TODO))
#define IS_TODO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_TODO))
#define TODO_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_TODO, TodoClass))

typedef struct _Todo Todo;
typedef struct _TodoClass TodoClass;
typedef struct _TodoPrivate TodoPrivate;

#define TODO_TYPE_COL (todo_col_get_type ())

struct _TodoItem {
	GTypeInstance parent_instance;
	volatile int ref_count;
	TodoItemPrivate * priv;
	char* type;
	char* text;
	char* file;
	gint line;
	ValideAbstractTextDocument* document;
};

struct _TodoItemClass {
	GTypeClass parent_class;
	void (*finalize) (TodoItem *self);
};

struct _ParamSpecTodoItem {
	GParamSpec parent_instance;
};

/**
 * @todo Add support for author, like C::B
 */
struct _Todo {
	GObject parent_instance;
	TodoPrivate * priv;
};

struct _TodoClass {
	GObjectClass parent_class;
};

struct _TodoPrivate {
	GtkTreeView* tree_view;
	ValideDocumentManager* documents;
	GtkScrolledWindow* scrolled_window;
	char* _path;
	ValideWindow* _window;
};

typedef enum  {
	TODO_COL_TYPE,
	TODO_COL_TEXT,
	TODO_COL_FILE,
	TODO_COL_LINE,
	TODO_COL_DOCUMENT,
	TODO_COL_NB
} TodoCol;


static gpointer todo_item_parent_class = NULL;
static gpointer todo_parent_class = NULL;
static ValidePluginIface* todo_valide_plugin_parent_iface = NULL;

gpointer todo_item_ref (gpointer instance);
void todo_item_unref (gpointer instance);
GParamSpec* param_spec_todo_item (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void value_set_todo_item (GValue* value, gpointer v_object);
gpointer value_get_todo_item (const GValue* value);
GType todo_item_get_type (void);
enum  {
	TODO_ITEM_DUMMY_PROPERTY
};
TodoItem* todo_item_new (void);
TodoItem* todo_item_construct (GType object_type);
static void todo_item_finalize (TodoItem* obj);
GType todo_get_type (void);
#define TODO_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_TODO, TodoPrivate))
enum  {
	TODO_DUMMY_PROPERTY,
	TODO_PATH,
	TODO_WINDOW
};
static GType todo_col_get_type (void);
#define TODO_help_text _ ("Mark todo with `@(TODO|FIXME|CHANGED|NOTE|OPTIMIZE|IMPROVE|BUG) :'")
static void todo_on_row_activated (Todo* self, GtkTreeView* sender, const GtkTreePath* path, GtkTreeViewColumn* column);
static gboolean todo_get_only_load_active_tab (Todo* self);
static void _g_list_free_g_object_unref (GList* self);
static GList* todo_parse (Todo* self, ValideAbstractTextDocument* d);
static void _g_list_free_todo_item_unref (GList* self);
static void todo_add_line (Todo* self, TodoItem* item);
static void todo_populate_list (Todo* self);
static void todo_on_tab_changed (Todo* self);
Todo* todo_new (void);
Todo* todo_construct (GType object_type);
static void _todo_on_row_activated_gtk_tree_view_row_activated (GtkTreeView* _sender, const GtkTreePath* path, GtkTreeViewColumn* column, gpointer self);
static void _todo_populate_list_valide_document_manager_tab_removed (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self);
static void _todo_on_tab_changed_valide_document_manager_tab_changed (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self);
static void _todo_populate_list_valide_document_manager_tab_saved (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self);
static GObject * todo_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void todo_finalize (GObject* obj);
static void todo_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void todo_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType register_plugin (void);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);



#line 24 "todo.vala"
TodoItem* todo_item_construct (GType object_type) {
#line 160 "todo.c"
	TodoItem* self;
	self = (TodoItem*) g_type_create_instance (object_type);
	return self;
}


#line 24 "todo.vala"
TodoItem* todo_item_new (void) {
#line 24 "todo.vala"
	return todo_item_construct (TYPE_TODO_ITEM);
#line 171 "todo.c"
}


static void value_todo_item_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void value_todo_item_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		todo_item_unref (value->data[0].v_pointer);
	}
}


static void value_todo_item_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = todo_item_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer value_todo_item_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* value_todo_item_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		TodoItem* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = todo_item_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* value_todo_item_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	TodoItem** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = todo_item_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* param_spec_todo_item (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ParamSpecTodoItem* spec;
	g_return_val_if_fail (g_type_is_a (object_type, TYPE_TODO_ITEM), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer value_get_todo_item (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_TODO_ITEM), NULL);
	return value->data[0].v_pointer;
}


void value_set_todo_item (GValue* value, gpointer v_object) {
	TodoItem* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_TODO_ITEM));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TYPE_TODO_ITEM));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		todo_item_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		todo_item_unref (old);
	}
}


static void todo_item_class_init (TodoItemClass * klass) {
	todo_item_parent_class = g_type_class_peek_parent (klass);
	TODO_ITEM_CLASS (klass)->finalize = todo_item_finalize;
}


static void todo_item_instance_init (TodoItem * self) {
	self->ref_count = 1;
}


static void todo_item_finalize (TodoItem* obj) {
	TodoItem * self;
	self = TODO_ITEM (obj);
	self->type = (g_free (self->type), NULL);
	self->text = (g_free (self->text), NULL);
	self->file = (g_free (self->file), NULL);
}


GType todo_item_get_type (void) {
	static GType todo_item_type_id = 0;
	if (todo_item_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { value_todo_item_init, value_todo_item_free_value, value_todo_item_copy_value, value_todo_item_peek_pointer, "p", value_todo_item_collect_value, "p", value_todo_item_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (TodoItemClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) todo_item_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (TodoItem), 0, (GInstanceInitFunc) todo_item_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		todo_item_type_id = g_type_register_fundamental (g_type_fundamental_next (), "TodoItem", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return todo_item_type_id;
}


gpointer todo_item_ref (gpointer instance) {
	TodoItem* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void todo_item_unref (gpointer instance) {
	TodoItem* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		TODO_ITEM_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}



static GType todo_col_get_type (void) {
	static GType todo_col_type_id = 0;
	if (G_UNLIKELY (todo_col_type_id == 0)) {
		static const GEnumValue values[] = {{TODO_COL_TYPE, "TODO_COL_TYPE", "type"}, {TODO_COL_TEXT, "TODO_COL_TEXT", "text"}, {TODO_COL_FILE, "TODO_COL_FILE", "file"}, {TODO_COL_LINE, "TODO_COL_LINE", "line"}, {TODO_COL_DOCUMENT, "TODO_COL_DOCUMENT", "document"}, {TODO_COL_NB, "TODO_COL_NB", "nb"}, {0, NULL, NULL}};
		todo_col_type_id = g_enum_register_static ("TodoCol", values);
	}
	return todo_col_type_id;
}


#line 64 "todo.vala"
static void todo_on_row_activated (Todo* self, GtkTreeView* sender, const GtkTreePath* path, GtkTreeViewColumn* column) {
#line 331 "todo.c"
	GtkTreeIter iter = {0};
	GtkTreeModel* _tmp0_;
	GtkTreeModel* model;
	ValideAbstractTextDocument* document;
	gint line;
#line 64 "todo.vala"
	g_return_if_fail (self != NULL);
#line 64 "todo.vala"
	g_return_if_fail (sender != NULL);
#line 64 "todo.vala"
	g_return_if_fail (path != NULL);
#line 64 "todo.vala"
	g_return_if_fail (column != NULL);
#line 68 "todo.vala"
	_tmp0_ = NULL;
#line 347 "todo.c"
	model = (_tmp0_ = gtk_tree_view_get_model (self->priv->tree_view), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	document = NULL;
	line = 0;
#line 72 "todo.vala"
	gtk_tree_model_get_iter (model, &iter, path);
#line 73 "todo.vala"
	gtk_tree_model_get (model, &iter, TODO_COL_LINE, &line, TODO_COL_DOCUMENT, &document, -1, -1);
#line 75 "todo.vala"
	valide_document_manager_set_current (self->priv->documents, (ValideiDocument*) document);
#line 76 "todo.vala"
	valide_source_view_goto_line (valide_abstract_text_document_get_text_view (document), line - 1);
#line 77 "todo.vala"
	valide_abstract_text_document_grab_focus (document);
#line 361 "todo.c"
	(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
	(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
}


static void _g_list_free_g_object_unref (GList* self) {
	g_list_foreach (self, (GFunc) g_object_unref, NULL);
	g_list_free (self);
}


static void _g_list_free_todo_item_unref (GList* self) {
	g_list_foreach (self, (GFunc) todo_item_unref, NULL);
	g_list_free (self);
}


#line 80 "todo.vala"
static void todo_populate_list (Todo* self) {
#line 381 "todo.c"
	GtkListStore* model;
	GtkListStore* _tmp2_;
	GtkListStore* _tmp1_;
	GtkTreeModel* _tmp0_;
#line 80 "todo.vala"
	g_return_if_fail (self != NULL);
#line 388 "todo.c"
	model = NULL;
#line 84 "todo.vala"
	g_object_set ((GtkWidget*) self->priv->tree_view, "sensitive", FALSE, NULL);
#line 392 "todo.c"
	/* clear data */
	_tmp2_ = NULL;
#line 87 "todo.vala"
	_tmp1_ = NULL;
#line 87 "todo.vala"
	_tmp0_ = NULL;
#line 87 "todo.vala"
	model = (_tmp2_ = (_tmp1_ = (_tmp0_ = gtk_tree_view_get_model (self->priv->tree_view), GTK_IS_LIST_STORE (_tmp0_) ? ((GtkListStore*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (model == NULL) ? NULL : (model = (g_object_unref (model), NULL)), _tmp2_);
#line 88 "todo.vala"
	gtk_list_store_clear (model);
#line 90 "todo.vala"
	if (valide_document_manager_get_current (self->priv->documents) != NULL) {
#line 405 "todo.c"
		gint nb_doc;
		gint nb_task;
		GList* doc;
		nb_doc = 0;
		nb_task = 0;
		doc = NULL;
		/* find out if we want to load all symbols or just active tab */
#line 97 "todo.vala"
		if (todo_get_only_load_active_tab (self)) {
#line 415 "todo.c"
			ValideiDocument* _tmp3_;
#line 99 "todo.vala"
			_tmp3_ = NULL;
#line 99 "todo.vala"
			doc = g_list_prepend (doc, (_tmp3_ = valide_document_manager_get_current (self->priv->documents), (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)));
#line 421 "todo.c"
		} else {
			GList* _tmp4_;
			_tmp4_ = NULL;
#line 103 "todo.vala"
			doc = (_tmp4_ = g_list_copy (valide_document_manager_get_documents (self->priv->documents)), (doc == NULL) ? NULL : (doc = (_g_list_free_g_object_unref (doc), NULL)), _tmp4_);
#line 427 "todo.c"
		}
		{
			GList* document_collection;
			GList* document_it;
#line 106 "todo.vala"
			document_collection = doc;
#line 434 "todo.c"
			for (document_it = document_collection; document_it != NULL; document_it = document_it->next) {
				ValideiDocument* _tmp7_;
				ValideiDocument* document;
#line 94 "todo.vala"
				_tmp7_ = NULL;
#line 106 "todo.vala"
				document = (_tmp7_ = (ValideiDocument*) document_it->data, (_tmp7_ == NULL) ? NULL : g_object_ref (_tmp7_));
#line 442 "todo.c"
				{
#line 108 "todo.vala"
					if (VALIDE_IS_ABSTRACT_TEXT_DOCUMENT (document)) {
#line 446 "todo.c"
						GList* items;
						GList* _tmp5_;
						items = NULL;
#line 112 "todo.vala"
						nb_doc++;
#line 452 "todo.c"
						_tmp5_ = NULL;
#line 113 "todo.vala"
						items = (_tmp5_ = todo_parse (self, VALIDE_ABSTRACT_TEXT_DOCUMENT (document)), (items == NULL) ? NULL : (items = (_g_list_free_todo_item_unref (items), NULL)), _tmp5_);
#line 114 "todo.vala"
						if (items != NULL) {
#line 458 "todo.c"
							{
								GList* item_collection;
								GList* item_it;
#line 116 "todo.vala"
								item_collection = items;
#line 464 "todo.c"
								for (item_it = item_collection; item_it != NULL; item_it = item_it->next) {
									TodoItem* _tmp6_;
									TodoItem* item;
#line 110 "todo.vala"
									_tmp6_ = NULL;
#line 116 "todo.vala"
									item = (_tmp6_ = (TodoItem*) item_it->data, (_tmp6_ == NULL) ? NULL : todo_item_ref (_tmp6_));
#line 472 "todo.c"
									{
#line 118 "todo.vala"
										todo_add_line (self, item);
#line 119 "todo.vala"
										nb_task++;
#line 478 "todo.c"
										(item == NULL) ? NULL : (item = (todo_item_unref (item), NULL));
									}
								}
							}
						}
						(items == NULL) ? NULL : (items = (_g_list_free_todo_item_unref (items), NULL));
					}
					(document == NULL) ? NULL : (document = (g_object_unref (document), NULL));
				}
			}
		}
#line 124 "todo.vala"
		if (nb_doc > 0) {
#line 126 "todo.vala"
			g_object_set ((GtkWidget*) self->priv->tree_view, "sensitive", TRUE, NULL);
#line 127 "todo.vala"
			if (nb_task > 0) {
#line 129 "todo.vala"
				gtk_widget_set_tooltip_text ((GtkWidget*) self->priv->tree_view, NULL);
#line 498 "todo.c"
			} else {
#line 133 "todo.vala"
				gtk_widget_set_tooltip_text ((GtkWidget*) self->priv->tree_view, TODO_help_text);
#line 502 "todo.c"
			}
		} else {
#line 138 "todo.vala"
			g_object_set ((GtkWidget*) self->priv->tree_view, "sensitive", FALSE, NULL);
#line 507 "todo.c"
		}
		(doc == NULL) ? NULL : (doc = (_g_list_free_g_object_unref (doc), NULL));
	}
	(model == NULL) ? NULL : (model = (g_object_unref (model), NULL));
}


#line 143 "todo.vala"
static void todo_on_tab_changed (Todo* self) {
#line 143 "todo.vala"
	g_return_if_fail (self != NULL);
#line 519 "todo.c"
	/* only needed in single-file mode  */
#line 146 "todo.vala"
	if (todo_get_only_load_active_tab (self)) {
#line 148 "todo.vala"
		todo_populate_list (self);
#line 525 "todo.c"
	}
}


#line 152 "todo.vala"
static void todo_add_line (Todo* self, TodoItem* item) {
#line 532 "todo.c"
	GtkTreeIter iter = {0};
	GtkListStore* _tmp0_;
	GtkListStore* list_store;
#line 152 "todo.vala"
	g_return_if_fail (self != NULL);
#line 152 "todo.vala"
	g_return_if_fail (item != NULL);
#line 155 "todo.vala"
	_tmp0_ = NULL;
#line 542 "todo.c"
	list_store = (_tmp0_ = GTK_LIST_STORE (gtk_tree_view_get_model (self->priv->tree_view)), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
#line 157 "todo.vala"
	gtk_list_store_append (list_store, &iter);
#line 158 "todo.vala"
	gtk_list_store_set (list_store, &iter, TODO_COL_TYPE, item->type, TODO_COL_TEXT, item->text, TODO_COL_FILE, item->file, TODO_COL_LINE, item->line, TODO_COL_DOCUMENT, item->document, -1);
#line 548 "todo.c"
	(list_store == NULL) ? NULL : (list_store = (g_object_unref (list_store), NULL));
}


#line 922 "glib-2.0.vapi"
static char* string_substring (const char* self, glong offset, glong len) {
#line 555 "todo.c"
	char* result;
	glong string_length;
	const char* start;
#line 922 "glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 561 "todo.c"
	string_length = g_utf8_strlen (self, -1);
#line 924 "glib-2.0.vapi"
	if (offset < 0) {
#line 925 "glib-2.0.vapi"
		offset = string_length + offset;
#line 926 "glib-2.0.vapi"
		g_return_val_if_fail (offset >= 0, NULL);
#line 569 "todo.c"
	} else {
#line 928 "glib-2.0.vapi"
		g_return_val_if_fail (offset <= string_length, NULL);
#line 573 "todo.c"
	}
#line 930 "glib-2.0.vapi"
	if (len < 0) {
#line 931 "glib-2.0.vapi"
		len = string_length - offset;
#line 579 "todo.c"
	}
#line 933 "glib-2.0.vapi"
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
#line 583 "todo.c"
	start = g_utf8_offset_to_pointer (self, offset);
	result = g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
	return result;
}


#line 163 "todo.vala"
static GList* todo_parse (Todo* self, ValideAbstractTextDocument* d) {
#line 592 "todo.c"
	GList* result;
	GError * _inner_error_;
	GList* items;
	GtkTextBuffer* _tmp0_;
	GtkTextBuffer* buffer;
	GtkTextIter start = {0};
	GtkTextIter end = {0};
	const char* _tmp1_;
	char* text;
	char** _tmp3_;
	gint lines_size;
	gint lines_length1;
	char** _tmp2_;
	char** lines;
#line 163 "todo.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 163 "todo.vala"
	g_return_val_if_fail (d != NULL, NULL);
#line 611 "todo.c"
	_inner_error_ = NULL;
	items = NULL;
#line 166 "todo.vala"
	_tmp0_ = NULL;
#line 616 "todo.c"
	buffer = (_tmp0_ = (GtkTextBuffer*) valide_abstract_text_document_get_buffer (d), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
#line 169 "todo.vala"
	gtk_text_buffer_get_start_iter (buffer, &start);
#line 170 "todo.vala"
	gtk_text_buffer_get_end_iter (buffer, &end);
#line 171 "todo.vala"
	_tmp1_ = NULL;
#line 624 "todo.c"
	text = (_tmp1_ = gtk_text_buffer_get_text (buffer, &start, &end, FALSE), (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_));
#line 172 "todo.vala"
	_tmp3_ = NULL;
#line 628 "todo.c"
	_tmp2_ = NULL;
	lines = (_tmp3_ = _tmp2_ = g_strsplit (text, "\n", 0), lines_length1 = _vala_array_length (_tmp2_), lines_size = lines_length1, _tmp3_);
	{
		GRegex* regex;
		regex = g_regex_new ("@(TODO|FIXME|CHANGED|NOTE|OPTIMIZE|IMPROVE|BUG)[:]?[\\s]*(.*)", G_REGEX_CASELESS, 0, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch0_g_error;
			goto __finally0;
		}
		{
			gint i;
			i = 0;
			{
				gboolean _tmp4_;
				_tmp4_ = TRUE;
#line 179 "todo.vala"
				while (TRUE) {
#line 179 "todo.vala"
					if (!_tmp4_) {
#line 179 "todo.vala"
						i++;
#line 650 "todo.c"
					}
#line 179 "todo.vala"
					_tmp4_ = FALSE;
#line 179 "todo.vala"
					if (!(lines[i] != NULL)) {
#line 179 "todo.vala"
						break;
#line 658 "todo.c"
					}
					{
						GMatchInfo* match_info;
						GMatchInfo* _tmp7_;
						gboolean _tmp6_;
						GMatchInfo* _tmp5_;
						gboolean _tmp8_;
						match_info = NULL;
						_tmp7_ = NULL;
						_tmp5_ = NULL;
						_tmp8_ = (_tmp6_ = g_regex_match_full (regex, lines[i], (glong) (-1), 0, 0, &_tmp5_, &_inner_error_), match_info = (_tmp7_ = _tmp5_, (match_info == NULL) ? NULL : (match_info = (g_match_info_free (match_info), NULL)), _tmp7_), _tmp6_);
						if (_inner_error_ != NULL) {
							(match_info == NULL) ? NULL : (match_info = (g_match_info_free (match_info), NULL));
							goto __catch1_g_error;
							goto __finally1;
						}
#line 184 "todo.vala"
						if (_tmp8_) {
#line 186 "todo.vala"
							while (TRUE) {
#line 679 "todo.c"
								TodoItem* item;
								char* _tmp10_;
								char* _tmp9_;
								char* _tmp11_;
								char* _tmp14_;
								const char* _tmp13_;
								TodoItem* _tmp15_;
#line 186 "todo.vala"
								if (!g_match_info_matches (match_info)) {
#line 186 "todo.vala"
									break;
#line 691 "todo.c"
								}
								item = todo_item_new ();
								_tmp10_ = NULL;
#line 2698 "glib-2.0.vapi"
								_tmp9_ = NULL;
#line 190 "todo.vala"
								item->type = (_tmp10_ = g_utf8_strup (_tmp9_ = g_match_info_fetch (match_info, 1), -1), item->type = (g_free (item->type), NULL), _tmp10_);
#line 699 "todo.c"
								_tmp9_ = (g_free (_tmp9_), NULL);
								_tmp11_ = NULL;
#line 191 "todo.vala"
								item->text = (_tmp11_ = g_match_info_fetch (match_info, 2), item->text = (g_free (item->text), NULL), _tmp11_);
#line 192 "todo.vala"
								if (g_str_has_suffix (item->text, "*/")) {
#line 706 "todo.c"
									char* _tmp12_;
									_tmp12_ = NULL;
#line 194 "todo.vala"
									item->text = (_tmp12_ = string_substring (item->text, (glong) 0, g_utf8_strlen (item->text, -1) - 2), item->text = (g_free (item->text), NULL), _tmp12_);
#line 711 "todo.c"
								}
#line 196 "todo.vala"
								item->document = d;
#line 715 "todo.c"
								_tmp14_ = NULL;
#line 197 "todo.vala"
								_tmp13_ = NULL;
#line 197 "todo.vala"
								item->file = (_tmp14_ = (_tmp13_ = valide_idocument_get_filename ((ValideiDocument*) d), (_tmp13_ == NULL) ? NULL : g_strdup (_tmp13_)), item->file = (g_free (item->file), NULL), _tmp14_);
#line 198 "todo.vala"
								item->line = i + 1;
#line 199 "todo.vala"
								_tmp15_ = NULL;
#line 199 "todo.vala"
								items = g_list_append (items, (_tmp15_ = item, item = NULL, _tmp15_));
#line 200 "todo.vala"
								g_match_info_next (match_info, &_inner_error_);
#line 729 "todo.c"
								if (_inner_error_ != NULL) {
									(item == NULL) ? NULL : (item = (todo_item_unref (item), NULL));
									(match_info == NULL) ? NULL : (match_info = (g_match_info_free (match_info), NULL));
									goto __catch1_g_error;
									goto __finally1;
								}
								(item == NULL) ? NULL : (item = (todo_item_unref (item), NULL));
							}
						}
						(match_info == NULL) ? NULL : (match_info = (g_match_info_free (match_info), NULL));
					}
					goto __finally1;
					__catch1_g_error:
					{
						GError * e;
						e = _inner_error_;
						_inner_error_ = NULL;
						{
#line 206 "todo.vala"
							g_warning (e->message);
#line 750 "todo.c"
							(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
						}
					}
					__finally1:
					if (_inner_error_ != NULL) {
						(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
						goto __catch0_g_error;
						goto __finally0;
					}
				}
			}
		}
		(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
	}
	goto __finally0;
	__catch0_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 212 "todo.vala"
			g_debug (e->message);
#line 774 "todo.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally0:
	if (_inner_error_ != NULL) {
		(items == NULL) ? NULL : (items = (_g_list_free_todo_item_unref (items), NULL));
		(buffer == NULL) ? NULL : (buffer = (g_object_unref (buffer), NULL));
		text = (g_free (text), NULL);
		lines = (_vala_array_free (lines, lines_length1, (GDestroyNotify) g_free), NULL);
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
	result = items;
	(buffer == NULL) ? NULL : (buffer = (g_object_unref (buffer), NULL));
	text = (g_free (text), NULL);
	lines = (_vala_array_free (lines, lines_length1, (GDestroyNotify) g_free), NULL);
	return result;
}


/**
 * @todo Add support for author, like C::B
 */
#line 36 "todo.vala"
Todo* todo_construct (GType object_type) {
#line 801 "todo.c"
	Todo * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


#line 36 "todo.vala"
Todo* todo_new (void) {
#line 36 "todo.vala"
	return todo_construct (TYPE_TODO);
#line 812 "todo.c"
}


static const char* todo_real_get_path (ValidePlugin* base) {
	const char* result;
	Todo* self;
	self = (Todo*) base;
	result = self->priv->_path;
	return result;
}


static void todo_real_set_path (ValidePlugin* base, const char* value) {
	Todo* self;
	char* _tmp1_;
	const char* _tmp0_;
	self = (Todo*) base;
	_tmp1_ = NULL;
#line 53 "todo.vala"
	_tmp0_ = NULL;
#line 833 "todo.c"
	self->priv->_path = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_path = (g_free (self->priv->_path), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "path");
}


static ValideWindow* todo_real_get_window (ValidePlugin* base) {
	ValideWindow* result;
	Todo* self;
	self = (Todo*) base;
	result = self->priv->_window;
	return result;
}


static void todo_real_set_window (ValidePlugin* base, ValideWindow* value) {
	Todo* self;
	ValideWindow* _tmp1_;
	ValideWindow* _tmp0_;
	self = (Todo*) base;
	_tmp1_ = NULL;
#line 54 "todo.vala"
	_tmp0_ = NULL;
#line 856 "todo.c"
	self->priv->_window = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "window");
}


static gboolean todo_get_only_load_active_tab (Todo* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = TRUE;
	return result;
}


#line 64 "todo.vala"
static void _todo_on_row_activated_gtk_tree_view_row_activated (GtkTreeView* _sender, const GtkTreePath* path, GtkTreeViewColumn* column, gpointer self) {
#line 872 "todo.c"
	todo_on_row_activated (self, _sender, path, column);
}


#line 80 "todo.vala"
static void _todo_populate_list_valide_document_manager_tab_removed (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self) {
#line 879 "todo.c"
	todo_populate_list (self);
}


#line 143 "todo.vala"
static void _todo_on_tab_changed_valide_document_manager_tab_changed (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self) {
#line 886 "todo.c"
	todo_on_tab_changed (self);
}


#line 80 "todo.vala"
static void _todo_populate_list_valide_document_manager_tab_saved (ValideDocumentManager* _sender, ValideiDocument* document, gpointer self) {
#line 893 "todo.c"
	todo_populate_list (self);
}


static GObject * todo_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	TodoClass * klass;
	GObjectClass * parent_class;
	Todo * self;
	klass = TODO_CLASS (g_type_class_peek (TYPE_TODO));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = TODO (obj);
	{
		GtkListStore* list_store;
		GtkTreeViewColumn* column;
		GtkCellRendererText* render;
		GtkScrolledWindow* _tmp0_;
		GtkListStore* _tmp1_;
		GtkTreeView* _tmp2_;
		GtkCellRendererText* _tmp3_;
		GtkTreeViewColumn* _tmp4_;
		GtkCellRendererText* _tmp5_;
		GtkTreeViewColumn* _tmp6_;
		GtkCellRendererText* _tmp7_;
		GtkTreeViewColumn* _tmp8_;
		GtkCellRendererText* _tmp9_;
		GtkTreeViewColumn* _tmp10_;
		ValideDocumentManager* _tmp12_;
		ValideDocumentManager* _tmp11_;
		list_store = NULL;
		column = NULL;
		render = NULL;
		_tmp0_ = NULL;
#line 223 "todo.vala"
		self->priv->scrolled_window = (_tmp0_ = g_object_ref_sink ((GtkScrolledWindow*) gtk_scrolled_window_new (NULL, NULL)), (self->priv->scrolled_window == NULL) ? NULL : (self->priv->scrolled_window = (g_object_unref (self->priv->scrolled_window), NULL)), _tmp0_);
#line 225 "todo.vala"
		gtk_scrolled_window_set_policy (self->priv->scrolled_window, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
#line 932 "todo.c"
		_tmp1_ = NULL;
#line 228 "todo.vala"
		list_store = (_tmp1_ = gtk_list_store_new ((gint) TODO_COL_NB, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_OBJECT, NULL), (list_store == NULL) ? NULL : (list_store = (g_object_unref (list_store), NULL)), _tmp1_);
#line 936 "todo.c"
		_tmp2_ = NULL;
#line 232 "todo.vala"
		self->priv->tree_view = (_tmp2_ = g_object_ref_sink ((GtkTreeView*) gtk_tree_view_new_with_model ((GtkTreeModel*) list_store)), (self->priv->tree_view == NULL) ? NULL : (self->priv->tree_view = (g_object_unref (self->priv->tree_view), NULL)), _tmp2_);
#line 233 "todo.vala"
		g_signal_connect_object (self->priv->tree_view, "row-activated", (GCallback) _todo_on_row_activated_gtk_tree_view_row_activated, self, 0);
#line 234 "todo.vala"
		gtk_container_add ((GtkContainer*) self->priv->scrolled_window, (GtkWidget*) self->priv->tree_view);
#line 944 "todo.c"
		_tmp3_ = NULL;
#line 236 "todo.vala"
		render = (_tmp3_ = g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (render == NULL) ? NULL : (render = (g_object_unref (render), NULL)), _tmp3_);
#line 948 "todo.c"
		_tmp4_ = NULL;
#line 237 "todo.vala"
		column = (_tmp4_ = g_object_ref_sink (gtk_tree_view_column_new_with_attributes (_ ("Type"), (GtkCellRenderer*) render, "text", TODO_COL_TYPE, NULL, NULL)), (column == NULL) ? NULL : (column = (g_object_unref (column), NULL)), _tmp4_);
#line 238 "todo.vala"
		gtk_tree_view_append_column (self->priv->tree_view, column);
#line 954 "todo.c"
		_tmp5_ = NULL;
#line 239 "todo.vala"
		render = (_tmp5_ = g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (render == NULL) ? NULL : (render = (g_object_unref (render), NULL)), _tmp5_);
#line 958 "todo.c"
		_tmp6_ = NULL;
#line 240 "todo.vala"
		column = (_tmp6_ = g_object_ref_sink (gtk_tree_view_column_new_with_attributes (_ ("Text"), (GtkCellRenderer*) render, "text", TODO_COL_TEXT, NULL, NULL)), (column == NULL) ? NULL : (column = (g_object_unref (column), NULL)), _tmp6_);
#line 241 "todo.vala"
		gtk_tree_view_append_column (self->priv->tree_view, column);
#line 964 "todo.c"
		_tmp7_ = NULL;
#line 242 "todo.vala"
		render = (_tmp7_ = g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (render == NULL) ? NULL : (render = (g_object_unref (render), NULL)), _tmp7_);
#line 968 "todo.c"
		_tmp8_ = NULL;
#line 243 "todo.vala"
		column = (_tmp8_ = g_object_ref_sink (gtk_tree_view_column_new_with_attributes (_ ("File"), (GtkCellRenderer*) render, "text", TODO_COL_FILE, NULL, NULL)), (column == NULL) ? NULL : (column = (g_object_unref (column), NULL)), _tmp8_);
#line 244 "todo.vala"
		gtk_tree_view_append_column (self->priv->tree_view, column);
#line 974 "todo.c"
		_tmp9_ = NULL;
#line 245 "todo.vala"
		render = (_tmp9_ = g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (render == NULL) ? NULL : (render = (g_object_unref (render), NULL)), _tmp9_);
#line 978 "todo.c"
		_tmp10_ = NULL;
#line 246 "todo.vala"
		column = (_tmp10_ = g_object_ref_sink (gtk_tree_view_column_new_with_attributes (_ ("Line"), (GtkCellRenderer*) render, "text", TODO_COL_LINE, NULL, NULL)), (column == NULL) ? NULL : (column = (g_object_unref (column), NULL)), _tmp10_);
#line 247 "todo.vala"
		gtk_tree_view_append_column (self->priv->tree_view, column);
#line 984 "todo.c"
		_tmp12_ = NULL;
#line 249 "todo.vala"
		_tmp11_ = NULL;
#line 249 "todo.vala"
		self->priv->documents = (_tmp12_ = (_tmp11_ = valide_window_get_documents (valide_plugin_get_window ((ValidePlugin*) self)), (_tmp11_ == NULL) ? NULL : g_object_ref (_tmp11_)), (self->priv->documents == NULL) ? NULL : (self->priv->documents = (g_object_unref (self->priv->documents), NULL)), _tmp12_);
#line 251 "todo.vala"
		g_signal_connect_object (self->priv->documents, "tab-removed", (GCallback) _todo_populate_list_valide_document_manager_tab_removed, self, 0);
#line 252 "todo.vala"
		g_signal_connect_object (self->priv->documents, "tab-changed", (GCallback) _todo_on_tab_changed_valide_document_manager_tab_changed, self, 0);
#line 253 "todo.vala"
		g_signal_connect_object (self->priv->documents, "tab-saved", (GCallback) _todo_populate_list_valide_document_manager_tab_saved, self, 0);
#line 255 "todo.vala"
		g_object_set ((GtkWidget*) self->priv->tree_view, "sensitive", FALSE, NULL);
#line 256 "todo.vala"
		valide_panel_add_item_with_stock_icon (valide_window_get_bottom_panel (valide_plugin_get_window ((ValidePlugin*) self)), (GtkWidget*) self->priv->scrolled_window, "To-Do list", GTK_STOCK_EDIT);
#line 258 "todo.vala"
		gtk_widget_show_all ((GtkWidget*) self->priv->scrolled_window);
#line 1002 "todo.c"
		(list_store == NULL) ? NULL : (list_store = (g_object_unref (list_store), NULL));
		(column == NULL) ? NULL : (column = (g_object_unref (column), NULL));
		(render == NULL) ? NULL : (render = (g_object_unref (render), NULL));
	}
	return obj;
}


static void todo_class_init (TodoClass * klass) {
	todo_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (TodoPrivate));
	G_OBJECT_CLASS (klass)->get_property = todo_get_property;
	G_OBJECT_CLASS (klass)->set_property = todo_set_property;
	G_OBJECT_CLASS (klass)->constructor = todo_constructor;
	G_OBJECT_CLASS (klass)->finalize = todo_finalize;
	g_object_class_override_property (G_OBJECT_CLASS (klass), TODO_PATH, "path");
	g_object_class_override_property (G_OBJECT_CLASS (klass), TODO_WINDOW, "window");
}


static void todo_valide_plugin_interface_init (ValidePluginIface * iface) {
	todo_valide_plugin_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_path = todo_real_get_path;
	iface->set_path = todo_real_set_path;
	iface->get_window = todo_real_get_window;
	iface->set_window = todo_real_set_window;
}


static void todo_instance_init (Todo * self) {
	self->priv = TODO_GET_PRIVATE (self);
}


static void todo_finalize (GObject* obj) {
	Todo * self;
	self = TODO (obj);
	{
		guint _tmp13_;
		guint _tmp14_;
		guint _tmp15_;
		guint _tmp16_;
#line 263 "todo.vala"
		g_signal_handlers_disconnect_matched (self->priv->documents, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("tab-removed", VALIDE_TYPE_DOCUMENT_MANAGER, &_tmp13_, NULL, FALSE), _tmp13_), 0, NULL, (GCallback) _todo_populate_list_valide_document_manager_tab_removed, self);
#line 264 "todo.vala"
		g_signal_handlers_disconnect_matched (self->priv->documents, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("tab-changed", VALIDE_TYPE_DOCUMENT_MANAGER, &_tmp14_, NULL, FALSE), _tmp14_), 0, NULL, (GCallback) _todo_on_tab_changed_valide_document_manager_tab_changed, self);
#line 265 "todo.vala"
		g_signal_handlers_disconnect_matched (self->priv->documents, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("tab-saved", VALIDE_TYPE_DOCUMENT_MANAGER, &_tmp15_, NULL, FALSE), _tmp15_), 0, NULL, (GCallback) _todo_populate_list_valide_document_manager_tab_saved, self);
#line 266 "todo.vala"
		g_signal_handlers_disconnect_matched (self->priv->tree_view, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("row-activated", GTK_TYPE_TREE_VIEW, &_tmp16_, NULL, FALSE), _tmp16_), 0, NULL, (GCallback) _todo_on_row_activated_gtk_tree_view_row_activated, self);
#line 267 "todo.vala"
		valide_panel_remove_item (valide_window_get_bottom_panel (valide_plugin_get_window ((ValidePlugin*) self)), (GtkWidget*) self->priv->scrolled_window);
#line 1055 "todo.c"
	}
	(self->priv->tree_view == NULL) ? NULL : (self->priv->tree_view = (g_object_unref (self->priv->tree_view), NULL));
	(self->priv->documents == NULL) ? NULL : (self->priv->documents = (g_object_unref (self->priv->documents), NULL));
	(self->priv->scrolled_window == NULL) ? NULL : (self->priv->scrolled_window = (g_object_unref (self->priv->scrolled_window), NULL));
	self->priv->_path = (g_free (self->priv->_path), NULL);
	(self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL));
	G_OBJECT_CLASS (todo_parent_class)->finalize (obj);
}


GType todo_get_type (void) {
	static GType todo_type_id = 0;
	if (todo_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (TodoClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) todo_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Todo), 0, (GInstanceInitFunc) todo_instance_init, NULL };
		static const GInterfaceInfo valide_plugin_info = { (GInterfaceInitFunc) todo_valide_plugin_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		todo_type_id = g_type_register_static (G_TYPE_OBJECT, "Todo", &g_define_type_info, 0);
		g_type_add_interface_static (todo_type_id, VALIDE_TYPE_PLUGIN, &valide_plugin_info);
	}
	return todo_type_id;
}


static void todo_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	Todo * self;
	gpointer boxed;
	self = TODO (object);
	switch (property_id) {
		case TODO_PATH:
		g_value_set_string (value, valide_plugin_get_path ((ValidePlugin*) self));
		break;
		case TODO_WINDOW:
		g_value_set_object (value, valide_plugin_get_window ((ValidePlugin*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void todo_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	Todo * self;
	self = TODO (object);
	switch (property_id) {
		case TODO_PATH:
		valide_plugin_set_path ((ValidePlugin*) self, g_value_get_string (value));
		break;
		case TODO_WINDOW:
		valide_plugin_set_window ((ValidePlugin*) self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


#line 272 "todo.vala"
GType register_plugin (void) {
#line 1115 "todo.c"
	GType result;
	result = TYPE_TODO;
	return result;
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}




