/* waf-builder.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <valide-0.0.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>


#define TYPE_WAF_BUILDER (waf_builder_get_type ())
#define WAF_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_WAF_BUILDER, WafBuilder))
#define WAF_BUILDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_WAF_BUILDER, WafBuilderClass))
#define IS_WAF_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_WAF_BUILDER))
#define IS_WAF_BUILDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_WAF_BUILDER))
#define WAF_BUILDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_WAF_BUILDER, WafBuilderClass))

typedef struct _WafBuilder WafBuilder;
typedef struct _WafBuilderClass WafBuilderClass;
typedef struct _WafBuilderPrivate WafBuilderPrivate;

#define TYPE_WAF_EXECUTABLE (waf_executable_get_type ())
#define WAF_EXECUTABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_WAF_EXECUTABLE, WafExecutable))
#define WAF_EXECUTABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_WAF_EXECUTABLE, WafExecutableClass))
#define IS_WAF_EXECUTABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_WAF_EXECUTABLE))
#define IS_WAF_EXECUTABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_WAF_EXECUTABLE))
#define WAF_EXECUTABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_WAF_EXECUTABLE, WafExecutableClass))

typedef struct _WafExecutable WafExecutable;
typedef struct _WafExecutableClass WafExecutableClass;

#define VALIDE_TYPE_WAF_BUILDER_OPTIONS (valide_waf_builder_options_get_type ())
#define VALIDE_WAF_BUILDER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_WAF_BUILDER_OPTIONS, ValideWafBuilderOptions))
#define VALIDE_WAF_BUILDER_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_WAF_BUILDER_OPTIONS, ValideWafBuilderOptionsClass))
#define VALIDE_IS_WAF_BUILDER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_WAF_BUILDER_OPTIONS))
#define VALIDE_IS_WAF_BUILDER_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_WAF_BUILDER_OPTIONS))
#define VALIDE_WAF_BUILDER_OPTIONS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_WAF_BUILDER_OPTIONS, ValideWafBuilderOptionsClass))

typedef struct _ValideWafBuilderOptions ValideWafBuilderOptions;
typedef struct _ValideWafBuilderOptionsClass ValideWafBuilderOptionsClass;
typedef struct _WafExecutablePrivate WafExecutablePrivate;

#define TYPE_WAF_BUILDER_PREFERENCES (waf_builder_preferences_get_type ())
#define WAF_BUILDER_PREFERENCES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_WAF_BUILDER_PREFERENCES, WafBuilderPreferences))
#define WAF_BUILDER_PREFERENCES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_WAF_BUILDER_PREFERENCES, WafBuilderPreferencesClass))
#define IS_WAF_BUILDER_PREFERENCES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_WAF_BUILDER_PREFERENCES))
#define IS_WAF_BUILDER_PREFERENCES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_WAF_BUILDER_PREFERENCES))
#define WAF_BUILDER_PREFERENCES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_WAF_BUILDER_PREFERENCES, WafBuilderPreferencesClass))

typedef struct _WafBuilderPreferences WafBuilderPreferences;
typedef struct _WafBuilderPreferencesClass WafBuilderPreferencesClass;

/**
 * Builder base on waf: http://code.google.com/p/waf/
 */
struct _WafBuilder {
	GObject parent_instance;
	WafBuilderPrivate * priv;
};

struct _WafBuilderClass {
	GObjectClass parent_class;
};

struct _WafBuilderPrivate {
	ValideProject* _project;
	ValideExecutableManager* _executables;
	char* _path;
	ValideWindow* _window;
};

struct _WafExecutable {
	ValideCompiler parent_instance;
	WafExecutablePrivate * priv;
	BuildingStep step;
};

struct _WafExecutableClass {
	ValideCompilerClass parent_class;
};


static gpointer waf_builder_parent_class = NULL;
static ValidePluginIface* waf_builder_valide_plugin_parent_iface = NULL;
static ValideBuilderIface* waf_builder_valide_builder_parent_iface = NULL;

GType waf_builder_get_type (void);
#define WAF_BUILDER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_WAF_BUILDER, WafBuilderPrivate))
enum  {
	WAF_BUILDER_DUMMY_PROPERTY,
	WAF_BUILDER_PROJECT,
	WAF_BUILDER_EXECUTABLES,
	WAF_BUILDER_PATH,
	WAF_BUILDER_WINDOW
};
GType waf_executable_get_type (void);
GType valide_waf_builder_options_get_type (void);
char* valide_waf_builder_options_to_string (ValideWafBuilderOptions* self, BuildingStep step);
WafExecutable* waf_executable_new (ValideProject* project);
WafExecutable* waf_executable_construct (GType object_type, ValideProject* project);
static void waf_builder_run_waf (WafBuilder* self, BuildingStep step);
WafBuilder* waf_builder_new (ValideExecutableManager* executables, ValideProject* project);
WafBuilder* waf_builder_construct (GType object_type, ValideExecutableManager* executables, ValideProject* project);
static void waf_builder_real_configure (ValideBuilder* base);
static void waf_builder_real_build (ValideBuilder* base);
static void waf_builder_real_install (ValideBuilder* base);
static void waf_builder_real_dist (ValideBuilder* base);
static void waf_builder_real_clean (ValideBuilder* base);
static void waf_builder_real_distclean (ValideBuilder* base);
static void waf_builder_real_uninstall (ValideBuilder* base);
WafBuilderPreferences* waf_builder_preferences_new (ValideProject* project);
WafBuilderPreferences* waf_builder_preferences_construct (GType object_type, ValideProject* project);
GType waf_builder_preferences_get_type (void);
static GtkWidget* waf_builder_real_preferences (ValideBuilder* base);
static GObject * waf_builder_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void waf_builder_finalize (GObject* obj);
static void waf_builder_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void waf_builder_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType register_plugin (void);



#line 49 "waf-builder.vala"
static void waf_builder_run_waf (WafBuilder* self, BuildingStep step) {
#line 145 "waf-builder.c"
	WafExecutable* waf;
	ValideWafBuilderOptions* options;
	ValideExecutableOptions* exec_options;
	ValideWafBuilderOptions* _tmp2_;
	ValideWafBuilderOptions* _tmp1_;
	ValideBuilderOptions* _tmp0_;
	ValideExecutableOptions* _tmp3_;
	char* _tmp4_;
	WafExecutable* _tmp5_;
#line 49 "waf-builder.vala"
	g_return_if_fail (self != NULL);
#line 157 "waf-builder.c"
	waf = NULL;
	options = NULL;
	exec_options = NULL;
	_tmp2_ = NULL;
#line 55 "waf-builder.vala"
	_tmp1_ = NULL;
#line 55 "waf-builder.vala"
	_tmp0_ = NULL;
#line 55 "waf-builder.vala"
	options = (_tmp2_ = (_tmp1_ = (_tmp0_ = valide_project_get_builder_options (valide_builder_get_project ((ValideBuilder*) self)), VALIDE_IS_WAF_BUILDER_OPTIONS (_tmp0_) ? ((ValideWafBuilderOptions*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (options == NULL) ? NULL : (options = (g_object_unref (options), NULL)), _tmp2_);
#line 168 "waf-builder.c"
	_tmp3_ = NULL;
#line 57 "waf-builder.vala"
	exec_options = (_tmp3_ = valide_executable_options_new (), (exec_options == NULL) ? NULL : (exec_options = (g_object_unref (exec_options), NULL)), _tmp3_);
#line 43 "waf-options.vala"
	_tmp4_ = NULL;
#line 58 "waf-builder.vala"
	valide_executable_options_set_arguments (exec_options, _tmp4_ = valide_waf_builder_options_to_string (options, step));
#line 176 "waf-builder.c"
	_tmp4_ = (g_free (_tmp4_), NULL);
#line 59 "waf-builder.vala"
	valide_executable_options_set_working_dir (exec_options, valide_project_get_path (valide_builder_get_project ((ValideBuilder*) self)));
#line 180 "waf-builder.c"
	_tmp5_ = NULL;
#line 61 "waf-builder.vala"
	waf = (_tmp5_ = waf_executable_new (valide_builder_get_project ((ValideBuilder*) self)), (waf == NULL) ? NULL : (waf = (g_object_unref (waf), NULL)), _tmp5_);
#line 62 "waf-builder.vala"
	waf->step = step;
#line 63 "waf-builder.vala"
	valide_executable_manager_run (valide_builder_get_executables ((ValideBuilder*) self), (ValideExecutable*) waf, exec_options);
#line 188 "waf-builder.c"
	(waf == NULL) ? NULL : (waf = (g_object_unref (waf), NULL));
	(options == NULL) ? NULL : (options = (g_object_unref (options), NULL));
	(exec_options == NULL) ? NULL : (exec_options = (g_object_unref (exec_options), NULL));
}


#line 66 "waf-builder.vala"
WafBuilder* waf_builder_construct (GType object_type, ValideExecutableManager* executables, ValideProject* project) {
#line 197 "waf-builder.c"
	GParameter * __params;
	GParameter * __params_it;
	WafBuilder * self;
#line 66 "waf-builder.vala"
	g_return_val_if_fail (executables != NULL, NULL);
#line 66 "waf-builder.vala"
	g_return_val_if_fail (project != NULL, NULL);
#line 205 "waf-builder.c"
	__params = g_new0 (GParameter, 2);
	__params_it = __params;
#line 68 "waf-builder.vala"
	__params_it->name = "project";
#line 68 "waf-builder.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_PROJECT);
#line 68 "waf-builder.vala"
	g_value_set_object (&__params_it->value, project);
#line 68 "waf-builder.vala"
	__params_it++;
#line 69 "waf-builder.vala"
	__params_it->name = "executables";
#line 69 "waf-builder.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_EXECUTABLE_MANAGER);
#line 69 "waf-builder.vala"
	g_value_set_object (&__params_it->value, executables);
#line 69 "waf-builder.vala"
	__params_it++;
#line 224 "waf-builder.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 66 "waf-builder.vala"
WafBuilder* waf_builder_new (ValideExecutableManager* executables, ValideProject* project) {
#line 66 "waf-builder.vala"
	return waf_builder_construct (TYPE_WAF_BUILDER, executables, project);
#line 239 "waf-builder.c"
}


/**
   * @see Valide.Builder.configure
   */
#line 81 "waf-builder.vala"
static void waf_builder_real_configure (ValideBuilder* base) {
#line 248 "waf-builder.c"
	WafBuilder * self;
	self = (WafBuilder*) base;
#line 83 "waf-builder.vala"
	waf_builder_run_waf (self, BUILDING_STEP_CONFIGURE);
#line 253 "waf-builder.c"
}


/**
   * @see Valide.Builder.build
   */
#line 89 "waf-builder.vala"
static void waf_builder_real_build (ValideBuilder* base) {
#line 262 "waf-builder.c"
	WafBuilder * self;
	self = (WafBuilder*) base;
#line 91 "waf-builder.vala"
	waf_builder_run_waf (self, BUILDING_STEP_BUILD);
#line 267 "waf-builder.c"
}


/**
   * @see Valide.Builder.install
   */
#line 97 "waf-builder.vala"
static void waf_builder_real_install (ValideBuilder* base) {
#line 276 "waf-builder.c"
	WafBuilder * self;
	self = (WafBuilder*) base;
#line 99 "waf-builder.vala"
	waf_builder_run_waf (self, BUILDING_STEP_INSTALL);
#line 281 "waf-builder.c"
}


/**
   * @see Valide.Builder.dist
   */
#line 105 "waf-builder.vala"
static void waf_builder_real_dist (ValideBuilder* base) {
#line 290 "waf-builder.c"
	WafBuilder * self;
	self = (WafBuilder*) base;
#line 107 "waf-builder.vala"
	waf_builder_run_waf (self, BUILDING_STEP_DIST);
#line 295 "waf-builder.c"
}


/**
   * @see Valide.Builder.clean
   */
#line 113 "waf-builder.vala"
static void waf_builder_real_clean (ValideBuilder* base) {
#line 304 "waf-builder.c"
	WafBuilder * self;
	self = (WafBuilder*) base;
#line 115 "waf-builder.vala"
	waf_builder_run_waf (self, BUILDING_STEP_CLEAN);
#line 309 "waf-builder.c"
}


/**
   * @see Valide.Builder.distclean
   */
#line 121 "waf-builder.vala"
static void waf_builder_real_distclean (ValideBuilder* base) {
#line 318 "waf-builder.c"
	WafBuilder * self;
	self = (WafBuilder*) base;
#line 123 "waf-builder.vala"
	waf_builder_run_waf (self, BUILDING_STEP_DISTCLEAN);
#line 323 "waf-builder.c"
}


/**
   * @see Valide.Builder.uninstall
   */
#line 129 "waf-builder.vala"
static void waf_builder_real_uninstall (ValideBuilder* base) {
#line 332 "waf-builder.c"
	WafBuilder * self;
	self = (WafBuilder*) base;
#line 131 "waf-builder.vala"
	waf_builder_run_waf (self, BUILDING_STEP_UNINSTALL);
#line 337 "waf-builder.c"
}


/**
   * @see Valide.Builder.preferences
   */
#line 137 "waf-builder.vala"
static GtkWidget* waf_builder_real_preferences (ValideBuilder* base) {
#line 346 "waf-builder.c"
	WafBuilder * self;
	GtkWidget* result;
	self = (WafBuilder*) base;
	result = (GtkWidget*) g_object_ref_sink (waf_builder_preferences_new (valide_builder_get_project ((ValideBuilder*) self)));
	return result;
}


static ValideProject* waf_builder_real_get_project (ValideBuilder* base) {
	ValideProject* result;
	WafBuilder* self;
	self = (WafBuilder*) base;
	result = self->priv->_project;
	return result;
}


static void waf_builder_real_set_project (ValideBuilder* base, ValideProject* value) {
	WafBuilder* self;
	ValideProject* _tmp1_;
	ValideProject* _tmp0_;
	self = (WafBuilder*) base;
	_tmp1_ = NULL;
#line 32 "waf-builder.vala"
	_tmp0_ = NULL;
#line 372 "waf-builder.c"
	self->priv->_project = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_project == NULL) ? NULL : (self->priv->_project = (g_object_unref (self->priv->_project), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "project");
}


static ValideExecutableManager* waf_builder_real_get_executables (ValideBuilder* base) {
	ValideExecutableManager* result;
	WafBuilder* self;
	self = (WafBuilder*) base;
	result = self->priv->_executables;
	return result;
}


static void waf_builder_real_set_executables (ValideBuilder* base, ValideExecutableManager* value) {
	WafBuilder* self;
	ValideExecutableManager* _tmp1_;
	ValideExecutableManager* _tmp0_;
	self = (WafBuilder*) base;
	_tmp1_ = NULL;
#line 37 "waf-builder.vala"
	_tmp0_ = NULL;
#line 395 "waf-builder.c"
	self->priv->_executables = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_executables == NULL) ? NULL : (self->priv->_executables = (g_object_unref (self->priv->_executables), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "executables");
}


static const char* waf_builder_real_get_path (ValidePlugin* base) {
	const char* result;
	WafBuilder* self;
	self = (WafBuilder*) base;
	result = self->priv->_path;
	return result;
}


static void waf_builder_real_set_path (ValidePlugin* base, const char* value) {
	WafBuilder* self;
	char* _tmp1_;
	const char* _tmp0_;
	self = (WafBuilder*) base;
	_tmp1_ = NULL;
#line 42 "waf-builder.vala"
	_tmp0_ = NULL;
#line 418 "waf-builder.c"
	self->priv->_path = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_path = (g_free (self->priv->_path), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "path");
}


static ValideWindow* waf_builder_real_get_window (ValidePlugin* base) {
	ValideWindow* result;
	WafBuilder* self;
	self = (WafBuilder*) base;
	result = self->priv->_window;
	return result;
}


static void waf_builder_real_set_window (ValidePlugin* base, ValideWindow* value) {
	WafBuilder* self;
	ValideWindow* _tmp1_;
	ValideWindow* _tmp0_;
	self = (WafBuilder*) base;
	_tmp1_ = NULL;
#line 47 "waf-builder.vala"
	_tmp0_ = NULL;
#line 441 "waf-builder.c"
	self->priv->_window = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "window");
}


static GObject * waf_builder_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	WafBuilderClass * klass;
	GObjectClass * parent_class;
	WafBuilder * self;
	klass = WAF_BUILDER_CLASS (g_type_class_peek (TYPE_WAF_BUILDER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = WAF_BUILDER (obj);
	{
		ValideBuilderManager* _tmp0_;
#line 465 "valide-0.0.vapi"
		_tmp0_ = NULL;
#line 74 "waf-builder.vala"
		valide_builder_manager_register_builder (_tmp0_ = valide_builder_manager_get_instance (), G_TYPE_FROM_INSTANCE ((GObject*) self), VALIDE_TYPE_WAF_BUILDER_OPTIONS);
#line 462 "waf-builder.c"
		(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
	}
	return obj;
}


static void waf_builder_class_init (WafBuilderClass * klass) {
	waf_builder_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (WafBuilderPrivate));
	G_OBJECT_CLASS (klass)->get_property = waf_builder_get_property;
	G_OBJECT_CLASS (klass)->set_property = waf_builder_set_property;
	G_OBJECT_CLASS (klass)->constructor = waf_builder_constructor;
	G_OBJECT_CLASS (klass)->finalize = waf_builder_finalize;
	g_object_class_override_property (G_OBJECT_CLASS (klass), WAF_BUILDER_PROJECT, "project");
	g_object_class_override_property (G_OBJECT_CLASS (klass), WAF_BUILDER_EXECUTABLES, "executables");
	g_object_class_override_property (G_OBJECT_CLASS (klass), WAF_BUILDER_PATH, "path");
	g_object_class_override_property (G_OBJECT_CLASS (klass), WAF_BUILDER_WINDOW, "window");
}


static void waf_builder_valide_plugin_interface_init (ValidePluginIface * iface) {
	waf_builder_valide_plugin_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_path = waf_builder_real_get_path;
	iface->set_path = waf_builder_real_set_path;
	iface->get_window = waf_builder_real_get_window;
	iface->set_window = waf_builder_real_set_window;
}


static void waf_builder_valide_builder_interface_init (ValideBuilderIface * iface) {
	waf_builder_valide_builder_parent_iface = g_type_interface_peek_parent (iface);
	iface->configure = waf_builder_real_configure;
	iface->build = waf_builder_real_build;
	iface->install = waf_builder_real_install;
	iface->dist = waf_builder_real_dist;
	iface->clean = waf_builder_real_clean;
	iface->distclean = waf_builder_real_distclean;
	iface->uninstall = waf_builder_real_uninstall;
	iface->preferences = waf_builder_real_preferences;
	iface->get_project = waf_builder_real_get_project;
	iface->set_project = waf_builder_real_set_project;
	iface->get_executables = waf_builder_real_get_executables;
	iface->set_executables = waf_builder_real_set_executables;
}


static void waf_builder_instance_init (WafBuilder * self) {
	self->priv = WAF_BUILDER_GET_PRIVATE (self);
}


static void waf_builder_finalize (GObject* obj) {
	WafBuilder * self;
	self = WAF_BUILDER (obj);
	(self->priv->_project == NULL) ? NULL : (self->priv->_project = (g_object_unref (self->priv->_project), NULL));
	(self->priv->_executables == NULL) ? NULL : (self->priv->_executables = (g_object_unref (self->priv->_executables), NULL));
	self->priv->_path = (g_free (self->priv->_path), NULL);
	(self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL));
	G_OBJECT_CLASS (waf_builder_parent_class)->finalize (obj);
}


GType waf_builder_get_type (void) {
	static GType waf_builder_type_id = 0;
	if (waf_builder_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (WafBuilderClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) waf_builder_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (WafBuilder), 0, (GInstanceInitFunc) waf_builder_instance_init, NULL };
		static const GInterfaceInfo valide_plugin_info = { (GInterfaceInitFunc) waf_builder_valide_plugin_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo valide_builder_info = { (GInterfaceInitFunc) waf_builder_valide_builder_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		waf_builder_type_id = g_type_register_static (G_TYPE_OBJECT, "WafBuilder", &g_define_type_info, 0);
		g_type_add_interface_static (waf_builder_type_id, VALIDE_TYPE_PLUGIN, &valide_plugin_info);
		g_type_add_interface_static (waf_builder_type_id, VALIDE_TYPE_BUILDER, &valide_builder_info);
	}
	return waf_builder_type_id;
}


static void waf_builder_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	WafBuilder * self;
	gpointer boxed;
	self = WAF_BUILDER (object);
	switch (property_id) {
		case WAF_BUILDER_PROJECT:
		g_value_set_object (value, valide_builder_get_project ((ValideBuilder*) self));
		break;
		case WAF_BUILDER_EXECUTABLES:
		g_value_set_object (value, valide_builder_get_executables ((ValideBuilder*) self));
		break;
		case WAF_BUILDER_PATH:
		g_value_set_string (value, valide_plugin_get_path ((ValidePlugin*) self));
		break;
		case WAF_BUILDER_WINDOW:
		g_value_set_object (value, valide_plugin_get_window ((ValidePlugin*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void waf_builder_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	WafBuilder * self;
	self = WAF_BUILDER (object);
	switch (property_id) {
		case WAF_BUILDER_PROJECT:
		valide_builder_set_project ((ValideBuilder*) self, g_value_get_object (value));
		break;
		case WAF_BUILDER_EXECUTABLES:
		valide_builder_set_executables ((ValideBuilder*) self, g_value_get_object (value));
		break;
		case WAF_BUILDER_PATH:
		valide_plugin_set_path ((ValidePlugin*) self, g_value_get_string (value));
		break;
		case WAF_BUILDER_WINDOW:
		valide_plugin_set_window ((ValidePlugin*) self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


#line 144 "waf-builder.vala"
GType register_plugin (void) {
#line 588 "waf-builder.c"
	GType result;
	result = TYPE_WAF_BUILDER;
	return result;
}




