/* waf-executable.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <valide-0.0.h>
#include <stdlib.h>
#include <string.h>
#include <glib/gstdio.h>
#include <gdk/gdk.h>


#define TYPE_WAF_EXECUTABLE (waf_executable_get_type ())
#define WAF_EXECUTABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_WAF_EXECUTABLE, WafExecutable))
#define WAF_EXECUTABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_WAF_EXECUTABLE, WafExecutableClass))
#define IS_WAF_EXECUTABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_WAF_EXECUTABLE))
#define IS_WAF_EXECUTABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_WAF_EXECUTABLE))
#define WAF_EXECUTABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_WAF_EXECUTABLE, WafExecutableClass))

typedef struct _WafExecutable WafExecutable;
typedef struct _WafExecutableClass WafExecutableClass;
typedef struct _WafExecutablePrivate WafExecutablePrivate;

#define VALIDE_TYPE_WAF_BUILDER_OPTIONS (valide_waf_builder_options_get_type ())
#define VALIDE_WAF_BUILDER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_WAF_BUILDER_OPTIONS, ValideWafBuilderOptions))
#define VALIDE_WAF_BUILDER_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_WAF_BUILDER_OPTIONS, ValideWafBuilderOptionsClass))
#define VALIDE_IS_WAF_BUILDER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_WAF_BUILDER_OPTIONS))
#define VALIDE_IS_WAF_BUILDER_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_WAF_BUILDER_OPTIONS))
#define VALIDE_WAF_BUILDER_OPTIONS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_WAF_BUILDER_OPTIONS, ValideWafBuilderOptionsClass))

typedef struct _ValideWafBuilderOptions ValideWafBuilderOptions;
typedef struct _ValideWafBuilderOptionsClass ValideWafBuilderOptionsClass;

struct _WafExecutable {
	ValideCompiler parent_instance;
	WafExecutablePrivate * priv;
	BuildingStep step;
};

struct _WafExecutableClass {
	ValideCompilerClass parent_class;
};

struct _WafExecutablePrivate {
	guint _nb_task;
	guint _current_task;
};


static gpointer waf_executable_parent_class = NULL;

GType waf_executable_get_type (void);
#define WAF_EXECUTABLE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_WAF_EXECUTABLE, WafExecutablePrivate))
enum  {
	WAF_EXECUTABLE_DUMMY_PROPERTY,
	WAF_EXECUTABLE_EXECUTABLE,
	WAF_EXECUTABLE_NB_TASK,
	WAF_EXECUTABLE_CURRENT_TASK
};
#define WAF_EXECUTABLE_EXECUTABLE_PATH "python ./waf -pp"
GType valide_waf_builder_options_get_type (void);
const char* valide_waf_builder_options_get_build_dir (ValideWafBuilderOptions* self);
static char* waf_executable_real_get_file (ValideCompiler* base, const char* line, GError** error);
void waf_executable_set_nb_task (WafExecutable* self, guint value);
void waf_executable_set_current_task (WafExecutable* self, guint value);
static GdkColor waf_executable_parse_build_line (WafExecutable* self, char** line);
static GdkColor waf_executable_real_get_color_line (ValideExecutable* base, char** line);
WafExecutable* waf_executable_new (ValideProject* project);
WafExecutable* waf_executable_construct (GType object_type, ValideProject* project);
guint waf_executable_get_nb_task (WafExecutable* self);
guint waf_executable_get_current_task (WafExecutable* self);
static void waf_executable_finalize (GObject* obj);
static void waf_executable_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void waf_executable_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);



#line 38 "waf-executable.vala"
static char* waf_executable_real_get_file (ValideCompiler* base, const char* line, GError** error) {
#line 103 "waf-executable.c"
	WafExecutable * self;
	char* result;
	GError * _inner_error_;
	char* filename;
	char* _tmp0_;
	char* _tmp1_;
	self = (WafExecutable*) base;
#line 38 "waf-executable.vala"
	g_return_val_if_fail (line != NULL, NULL);
#line 113 "waf-executable.c"
	_inner_error_ = NULL;
	filename = NULL;
	_tmp0_ = VALIDE_COMPILER_CLASS (waf_executable_parent_class)->get_file (VALIDE_COMPILER (self), line, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		filename = (g_free (filename), NULL);
		return NULL;
	}
	_tmp1_ = NULL;
#line 42 "waf-executable.vala"
	filename = (_tmp1_ = _tmp0_, filename = (g_free (filename), NULL), _tmp1_);
#line 43 "waf-executable.vala"
	if (!g_file_test (filename, G_FILE_TEST_EXISTS)) {
#line 127 "waf-executable.c"
		ValideWafBuilderOptions* options;
		ValideBuilderOptions* _tmp2_;
		char* _tmp3_;
		options = NULL;
#line 47 "waf-executable.vala"
		_tmp2_ = NULL;
#line 47 "waf-executable.vala"
		options = (_tmp2_ = valide_project_get_builder_options (valide_compiler_get_project ((ValideCompiler*) self)), VALIDE_IS_WAF_BUILDER_OPTIONS (_tmp2_) ? ((ValideWafBuilderOptions*) _tmp2_) : NULL);
#line 136 "waf-executable.c"
		_tmp3_ = NULL;
#line 48 "waf-executable.vala"
		filename = (_tmp3_ = g_build_filename (valide_waf_builder_options_get_build_dir (options), filename, NULL), filename = (g_free (filename), NULL), _tmp3_);
#line 140 "waf-executable.c"
	}
	result = filename;
	return result;
}


#line 53 "waf-executable.vala"
static GdkColor waf_executable_parse_build_line (WafExecutable* self, char** line) {
#line 149 "waf-executable.c"
	GdkColor result;
	GdkColor color = {0};
#line 57 "waf-executable.vala"
	gdk_color_parse (VALIDE_UTILS_COLOR_DEFAULT, &color);
#line 58 "waf-executable.vala"
	if (g_str_has_prefix ((*line), "|")) {
#line 156 "waf-executable.c"
		gint tmp_size;
		gint tmp_length1;
		char** tmp;
		const char* _tmp0_;
		char* new_line;
		char* _tmp10_;
		const char* _tmp9_;
		tmp = (tmp_length1 = 0, NULL);
#line 61 "waf-executable.vala"
		_tmp0_ = NULL;
#line 167 "waf-executable.c"
		new_line = (_tmp0_ = (*line), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
		{
			char** _tmp1_;
			char** item_collection;
			int item_collection_length1;
			int item_it;
			_tmp1_ = NULL;
#line 63 "waf-executable.vala"
			item_collection = _tmp1_ = g_strsplit ((*line), "|", 0);
#line 177 "waf-executable.c"
			item_collection_length1 = _vala_array_length (_tmp1_);
			for (item_it = 0; item_it < _vala_array_length (_tmp1_); item_it = item_it + 1) {
				const char* _tmp8_;
				char* item;
#line 831 "glib-2.0.vapi"
				_tmp8_ = NULL;
#line 184 "waf-executable.c"
				item = (_tmp8_ = item_collection[item_it], (_tmp8_ == NULL) ? NULL : g_strdup (_tmp8_));
				{
					char** _tmp3_;
					char** _tmp2_;
					_tmp3_ = NULL;
					_tmp2_ = NULL;
#line 65 "waf-executable.vala"
					tmp = (_tmp3_ = _tmp2_ = g_strsplit (item, " ", 0), tmp = (_vala_array_free (tmp, tmp_length1, (GDestroyNotify) g_free), NULL), tmp_length1 = _vala_array_length (_tmp2_), tmp_size = tmp_length1, _tmp3_);
#line 66 "waf-executable.vala"
					if (_vala_strcmp0 (tmp[0], "Total") == 0) {
#line 68 "waf-executable.vala"
						waf_executable_set_nb_task (self, (guint) atoi (tmp[1]));
#line 197 "waf-executable.c"
					} else {
#line 70 "waf-executable.vala"
						if (_vala_strcmp0 (tmp[0], "Current") == 0) {
#line 72 "waf-executable.vala"
							waf_executable_set_current_task (self, (guint) atoi (tmp[1]));
#line 203 "waf-executable.c"
						} else {
#line 74 "waf-executable.vala"
							if (_vala_strcmp0 (tmp[0], "Inputs") == 0) {
#line 207 "waf-executable.c"
								char* _tmp5_;
								const char* _tmp4_;
								_tmp5_ = NULL;
#line 76 "waf-executable.vala"
								_tmp4_ = NULL;
#line 76 "waf-executable.vala"
								new_line = (_tmp5_ = (_tmp4_ = tmp[1], (_tmp4_ == NULL) ? NULL : g_strdup (_tmp4_)), new_line = (g_free (new_line), NULL), _tmp5_);
#line 215 "waf-executable.c"
							} else {
#line 78 "waf-executable.vala"
								if (_vala_strcmp0 (tmp[0], "Outputs") == 0) {
#line 219 "waf-executable.c"
									char* _tmp7_;
									char* _tmp6_;
									_tmp7_ = NULL;
									_tmp6_ = NULL;
#line 80 "waf-executable.vala"
									new_line = (_tmp7_ = g_strconcat (new_line, _tmp6_ = g_strconcat (" -> ", tmp[1], NULL), NULL), new_line = (g_free (new_line), NULL), _tmp7_);
#line 226 "waf-executable.c"
									_tmp6_ = (g_free (_tmp6_), NULL);
								}
							}
						}
					}
					item = (g_free (item), NULL);
				}
			}
#line 63 "waf-executable.vala"
			item_collection = (_vala_array_free (item_collection, item_collection_length1, (GDestroyNotify) g_free), NULL);
#line 237 "waf-executable.c"
		}
		_tmp10_ = NULL;
#line 83 "waf-executable.vala"
		_tmp9_ = NULL;
#line 83 "waf-executable.vala"
		(*line) = (_tmp10_ = (_tmp9_ = new_line, (_tmp9_ == NULL) ? NULL : g_strdup (_tmp9_)), (*line) = (g_free ((*line)), NULL), _tmp10_);
#line 244 "waf-executable.c"
		tmp = (_vala_array_free (tmp, tmp_length1, (GDestroyNotify) g_free), NULL);
		new_line = (g_free (new_line), NULL);
	} else {
#line 85 "waf-executable.vala"
		if (g_str_has_prefix ((*line), "Waf:")) {
#line 87 "waf-executable.vala"
			gdk_color_parse (VALIDE_UTILS_COLOR_INFO, &color);
#line 252 "waf-executable.c"
		} else {
#line 89 "waf-executable.vala"
			if (strstr ((*line), "warning:") != NULL) {
#line 91 "waf-executable.vala"
				gdk_color_parse (VALIDE_UTILS_COLOR_WARNING, &color);
#line 258 "waf-executable.c"
			} else {
#line 93 "waf-executable.vala"
				if (strstr ((*line), "error:") != NULL) {
#line 95 "waf-executable.vala"
					gdk_color_parse (VALIDE_UTILS_COLOR_ERROR, &color);
#line 264 "waf-executable.c"
				}
			}
		}
	}
	result = color;
	return result;
}


/**
   * @see Valide.Executable.get_color_line
   */
#line 103 "waf-executable.vala"
static GdkColor waf_executable_real_get_color_line (ValideExecutable* base, char** line) {
#line 279 "waf-executable.c"
	WafExecutable * self;
	GdkColor result;
	GdkColor color = {0};
	self = (WafExecutable*) base;
#line 107 "waf-executable.vala"
	color = VALIDE_EXECUTABLE_CLASS (waf_executable_parent_class)->get_color_line ((ValideExecutable*) VALIDE_COMPILER (self), &(*line));
#line 108 "waf-executable.vala"
	if (g_str_has_suffix ((*line), "finished successfully")) {
#line 110 "waf-executable.vala"
		gdk_color_parse (VALIDE_UTILS_COLOR_SUCCESS, &color);
#line 290 "waf-executable.c"
	} else {
#line 114 "waf-executable.vala"
		switch (self->step) {
#line 294 "waf-executable.c"
			case BUILDING_STEP_CONFIGURE:
			{
#line 117 "waf-executable.vala"
				if (strstr ((*line), ": ok") != NULL) {
#line 119 "waf-executable.vala"
					gdk_color_parse (VALIDE_UTILS_COLOR_SUCCESS, &color);
#line 301 "waf-executable.c"
				} else {
#line 121 "waf-executable.vala"
					if (strstr ((*line), ": fail") != NULL) {
#line 123 "waf-executable.vala"
						gdk_color_parse (VALIDE_UTILS_COLOR_WARNING, &color);
#line 307 "waf-executable.c"
					} else {
						gboolean _tmp0_;
						_tmp0_ = FALSE;
#line 125 "waf-executable.vala"
						if (g_str_has_prefix ((*line), " error: the configuration failed")) {
#line 125 "waf-executable.vala"
							_tmp0_ = TRUE;
#line 315 "waf-executable.c"
						} else {
#line 126 "waf-executable.vala"
							_tmp0_ = g_str_has_prefix ((*line), " error: the configuration failed");
#line 319 "waf-executable.c"
						}
#line 125 "waf-executable.vala"
						if (_tmp0_) {
#line 128 "waf-executable.vala"
							gdk_color_parse (VALIDE_UTILS_COLOR_ERROR, &color);
#line 325 "waf-executable.c"
						}
					}
				}
#line 130 "waf-executable.vala"
				break;
#line 331 "waf-executable.c"
			}
			case BUILDING_STEP_BUILD:
			case BUILDING_STEP_INSTALL:
			case BUILDING_STEP_DIST:
			{
#line 134 "waf-executable.vala"
				color = waf_executable_parse_build_line (self, &(*line));
#line 135 "waf-executable.vala"
				break;
#line 341 "waf-executable.c"
			}
		}
	}
	result = color;
	return result;
}


/**
   * Create a new WafExecutable
   *
   * @param project A project
   */
#line 146 "waf-executable.vala"
WafExecutable* waf_executable_construct (GType object_type, ValideProject* project) {
#line 357 "waf-executable.c"
	GParameter * __params;
	GParameter * __params_it;
	WafExecutable * self;
#line 146 "waf-executable.vala"
	g_return_val_if_fail (project != NULL, NULL);
#line 363 "waf-executable.c"
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
#line 148 "waf-executable.vala"
	__params_it->name = "project";
#line 148 "waf-executable.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_PROJECT);
#line 148 "waf-executable.vala"
	g_value_set_object (&__params_it->value, project);
#line 148 "waf-executable.vala"
	__params_it++;
#line 374 "waf-executable.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 146 "waf-executable.vala"
WafExecutable* waf_executable_new (ValideProject* project) {
#line 146 "waf-executable.vala"
	return waf_executable_construct (TYPE_WAF_EXECUTABLE, project);
#line 389 "waf-executable.c"
}


static const char* waf_executable_real_get_executable (ValideExecutable* base) {
	const char* result;
	WafExecutable* self;
	self = (WafExecutable*) base;
	result = WAF_EXECUTABLE_EXECUTABLE_PATH;
	return result;
}


static void waf_executable_real_set_executable (ValideExecutable* base, const char* value) {
	WafExecutable* self;
	self = (WafExecutable*) base;
	g_object_notify ((GObject *) self, "executable");
}


guint waf_executable_get_nb_task (WafExecutable* self) {
	guint result;
	g_return_val_if_fail (self != NULL, 0U);
	result = self->priv->_nb_task;
	return result;
}


void waf_executable_set_nb_task (WafExecutable* self, guint value) {
	g_return_if_fail (self != NULL);
	self->priv->_nb_task = value;
	g_object_notify ((GObject *) self, "nb-task");
}


guint waf_executable_get_current_task (WafExecutable* self) {
	guint result;
	g_return_val_if_fail (self != NULL, 0U);
	result = self->priv->_current_task;
	return result;
}


void waf_executable_set_current_task (WafExecutable* self, guint value) {
	g_return_if_fail (self != NULL);
	self->priv->_current_task = value;
	g_object_notify ((GObject *) self, "current-task");
}


static void waf_executable_class_init (WafExecutableClass * klass) {
	waf_executable_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (WafExecutablePrivate));
	VALIDE_COMPILER_CLASS (klass)->get_file = waf_executable_real_get_file;
	VALIDE_EXECUTABLE_CLASS (klass)->get_color_line = waf_executable_real_get_color_line;
	VALIDE_EXECUTABLE_CLASS (klass)->get_executable = waf_executable_real_get_executable;
	VALIDE_EXECUTABLE_CLASS (klass)->set_executable = waf_executable_real_set_executable;
	G_OBJECT_CLASS (klass)->get_property = waf_executable_get_property;
	G_OBJECT_CLASS (klass)->set_property = waf_executable_set_property;
	G_OBJECT_CLASS (klass)->finalize = waf_executable_finalize;
	g_object_class_override_property (G_OBJECT_CLASS (klass), WAF_EXECUTABLE_EXECUTABLE, "executable");
	g_object_class_install_property (G_OBJECT_CLASS (klass), WAF_EXECUTABLE_NB_TASK, g_param_spec_uint ("nb-task", "nb-task", "nb-task", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), WAF_EXECUTABLE_CURRENT_TASK, g_param_spec_uint ("current-task", "current-task", "current-task", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void waf_executable_instance_init (WafExecutable * self) {
	self->priv = WAF_EXECUTABLE_GET_PRIVATE (self);
}


static void waf_executable_finalize (GObject* obj) {
	WafExecutable * self;
	self = WAF_EXECUTABLE (obj);
	G_OBJECT_CLASS (waf_executable_parent_class)->finalize (obj);
}


GType waf_executable_get_type (void) {
	static GType waf_executable_type_id = 0;
	if (waf_executable_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (WafExecutableClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) waf_executable_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (WafExecutable), 0, (GInstanceInitFunc) waf_executable_instance_init, NULL };
		waf_executable_type_id = g_type_register_static (VALIDE_TYPE_COMPILER, "WafExecutable", &g_define_type_info, 0);
	}
	return waf_executable_type_id;
}


static void waf_executable_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	WafExecutable * self;
	gpointer boxed;
	self = WAF_EXECUTABLE (object);
	switch (property_id) {
		case WAF_EXECUTABLE_EXECUTABLE:
		g_value_set_string (value, valide_executable_get_executable ((ValideExecutable*) self));
		break;
		case WAF_EXECUTABLE_NB_TASK:
		g_value_set_uint (value, waf_executable_get_nb_task (self));
		break;
		case WAF_EXECUTABLE_CURRENT_TASK:
		g_value_set_uint (value, waf_executable_get_current_task (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void waf_executable_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	WafExecutable * self;
	self = WAF_EXECUTABLE (object);
	switch (property_id) {
		case WAF_EXECUTABLE_EXECUTABLE:
		valide_executable_set_executable ((ValideExecutable*) self, g_value_get_string (value));
		break;
		case WAF_EXECUTABLE_NB_TASK:
		waf_executable_set_nb_task (self, g_value_get_uint (value));
		break;
		case WAF_EXECUTABLE_CURRENT_TASK:
		waf_executable_set_current_task (self, g_value_get_uint (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




