/* waf-options.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <valide-0.0.h>
#include <stdlib.h>
#include <string.h>


#define VALIDE_TYPE_WAF_BUILDER_OPTIONS (valide_waf_builder_options_get_type ())
#define VALIDE_WAF_BUILDER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_WAF_BUILDER_OPTIONS, ValideWafBuilderOptions))
#define VALIDE_WAF_BUILDER_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_WAF_BUILDER_OPTIONS, ValideWafBuilderOptionsClass))
#define VALIDE_IS_WAF_BUILDER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_WAF_BUILDER_OPTIONS))
#define VALIDE_IS_WAF_BUILDER_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_WAF_BUILDER_OPTIONS))
#define VALIDE_WAF_BUILDER_OPTIONS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_WAF_BUILDER_OPTIONS, ValideWafBuilderOptionsClass))

typedef struct _ValideWafBuilderOptions ValideWafBuilderOptions;
typedef struct _ValideWafBuilderOptionsClass ValideWafBuilderOptionsClass;
typedef struct _ValideWafBuilderOptionsPrivate ValideWafBuilderOptionsPrivate;

struct _ValideWafBuilderOptions {
	GObject parent_instance;
	ValideWafBuilderOptionsPrivate * priv;
};

struct _ValideWafBuilderOptionsClass {
	GObjectClass parent_class;
};

struct _ValideWafBuilderOptionsPrivate {
	char* _build_dir;
	char* _prefix;
	char* _configure_options;
	char* _build_options;
	char* _install_options;
	char* _dist_options;
	char* _clean_options;
	char* _distclean_options;
	char* _uninstall_options;
};


static gpointer valide_waf_builder_options_parent_class = NULL;
static ValideBuilderOptionsIface* valide_waf_builder_options_valide_builder_options_parent_iface = NULL;
static GYAMLBuildableIface* valide_waf_builder_options_g_yaml_buildable_parent_iface = NULL;

GType valide_waf_builder_options_get_type (void);
#define VALIDE_WAF_BUILDER_OPTIONS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_WAF_BUILDER_OPTIONS, ValideWafBuilderOptionsPrivate))
enum  {
	VALIDE_WAF_BUILDER_OPTIONS_DUMMY_PROPERTY,
	VALIDE_WAF_BUILDER_OPTIONS_BUILD_DIR,
	VALIDE_WAF_BUILDER_OPTIONS_PREFIX,
	VALIDE_WAF_BUILDER_OPTIONS_CONFIGURE_OPTIONS,
	VALIDE_WAF_BUILDER_OPTIONS_BUILD_OPTIONS,
	VALIDE_WAF_BUILDER_OPTIONS_INSTALL_OPTIONS,
	VALIDE_WAF_BUILDER_OPTIONS_DIST_OPTIONS,
	VALIDE_WAF_BUILDER_OPTIONS_CLEAN_OPTIONS,
	VALIDE_WAF_BUILDER_OPTIONS_DISTCLEAN_OPTIONS,
	VALIDE_WAF_BUILDER_OPTIONS_UNINSTALL_OPTIONS
};
const char* valide_waf_builder_options_get_prefix (ValideWafBuilderOptions* self);
const char* valide_waf_builder_options_get_configure_options (ValideWafBuilderOptions* self);
const char* valide_waf_builder_options_get_build_options (ValideWafBuilderOptions* self);
const char* valide_waf_builder_options_get_install_options (ValideWafBuilderOptions* self);
const char* valide_waf_builder_options_get_dist_options (ValideWafBuilderOptions* self);
const char* valide_waf_builder_options_get_clean_options (ValideWafBuilderOptions* self);
const char* valide_waf_builder_options_get_distclean_options (ValideWafBuilderOptions* self);
const char* valide_waf_builder_options_get_uninstall_options (ValideWafBuilderOptions* self);
char* valide_waf_builder_options_to_string (ValideWafBuilderOptions* self, BuildingStep step);
ValideWafBuilderOptions* valide_waf_builder_options_new (void);
ValideWafBuilderOptions* valide_waf_builder_options_construct (GType object_type);
const char* valide_waf_builder_options_get_build_dir (ValideWafBuilderOptions* self);
void valide_waf_builder_options_set_build_dir (ValideWafBuilderOptions* self, const char* value);
void valide_waf_builder_options_set_prefix (ValideWafBuilderOptions* self, const char* value);
void valide_waf_builder_options_set_configure_options (ValideWafBuilderOptions* self, const char* value);
void valide_waf_builder_options_set_build_options (ValideWafBuilderOptions* self, const char* value);
void valide_waf_builder_options_set_install_options (ValideWafBuilderOptions* self, const char* value);
void valide_waf_builder_options_set_dist_options (ValideWafBuilderOptions* self, const char* value);
void valide_waf_builder_options_set_clean_options (ValideWafBuilderOptions* self, const char* value);
void valide_waf_builder_options_set_distclean_options (ValideWafBuilderOptions* self, const char* value);
void valide_waf_builder_options_set_uninstall_options (ValideWafBuilderOptions* self, const char* value);
static void valide_waf_builder_options_finalize (GObject* obj);
static void valide_waf_builder_options_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void valide_waf_builder_options_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static int _vala_strcmp0 (const char * str1, const char * str2);



#line 43 "waf-options.vala"
char* valide_waf_builder_options_to_string (ValideWafBuilderOptions* self, BuildingStep step) {
#line 111 "waf-options.c"
	char* result;
	GString* sb;
	GString* _tmp0_;
	const char* _tmp2_;
#line 43 "waf-options.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 118 "waf-options.c"
	sb = NULL;
	_tmp0_ = NULL;
#line 47 "waf-options.vala"
	sb = (_tmp0_ = g_string_new (""), (sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL)), _tmp0_);
#line 48 "waf-options.vala"
	switch (step) {
#line 125 "waf-options.c"
		case BUILDING_STEP_CONFIGURE:
		{
			gboolean _tmp1_;
#line 51 "waf-options.vala"
			g_string_append (sb, "configure ");
#line 131 "waf-options.c"
			_tmp1_ = FALSE;
#line 52 "waf-options.vala"
			if (self->priv->_prefix != NULL) {
#line 52 "waf-options.vala"
				_tmp1_ = _vala_strcmp0 (self->priv->_prefix, "") != 0;
#line 137 "waf-options.c"
			} else {
#line 52 "waf-options.vala"
				_tmp1_ = FALSE;
#line 141 "waf-options.c"
			}
#line 52 "waf-options.vala"
			if (_tmp1_) {
#line 54 "waf-options.vala"
				g_string_append_printf (sb, "--prefix='%s' ", self->priv->_prefix);
#line 147 "waf-options.c"
			}
#line 56 "waf-options.vala"
			if (self->priv->_configure_options != NULL) {
#line 58 "waf-options.vala"
				g_string_append (sb, self->priv->_configure_options);
#line 153 "waf-options.c"
			}
#line 60 "waf-options.vala"
			break;
#line 157 "waf-options.c"
		}
		case BUILDING_STEP_BUILD:
		{
#line 62 "waf-options.vala"
			g_string_append (sb, "build ");
#line 63 "waf-options.vala"
			if (self->priv->_build_options != NULL) {
#line 65 "waf-options.vala"
				g_string_append (sb, self->priv->_build_options);
#line 167 "waf-options.c"
			}
#line 67 "waf-options.vala"
			break;
#line 171 "waf-options.c"
		}
		case BUILDING_STEP_INSTALL:
		{
#line 69 "waf-options.vala"
			g_string_append (sb, "install ");
#line 70 "waf-options.vala"
			if (self->priv->_install_options != NULL) {
#line 72 "waf-options.vala"
				g_string_append (sb, self->priv->_install_options);
#line 181 "waf-options.c"
			}
#line 74 "waf-options.vala"
			break;
#line 185 "waf-options.c"
		}
		case BUILDING_STEP_DIST:
		{
#line 76 "waf-options.vala"
			g_string_append (sb, "dist ");
#line 77 "waf-options.vala"
			if (self->priv->_dist_options != NULL) {
#line 79 "waf-options.vala"
				g_string_append (sb, self->priv->_dist_options);
#line 195 "waf-options.c"
			}
#line 81 "waf-options.vala"
			break;
#line 199 "waf-options.c"
		}
		case BUILDING_STEP_CLEAN:
		{
#line 83 "waf-options.vala"
			g_string_append (sb, "clean ");
#line 84 "waf-options.vala"
			if (self->priv->_clean_options != NULL) {
#line 86 "waf-options.vala"
				g_string_append (sb, self->priv->_clean_options);
#line 209 "waf-options.c"
			}
#line 88 "waf-options.vala"
			break;
#line 213 "waf-options.c"
		}
		case BUILDING_STEP_DISTCLEAN:
		{
#line 90 "waf-options.vala"
			g_string_append (sb, "disclean ");
#line 91 "waf-options.vala"
			if (self->priv->_distclean_options != NULL) {
#line 93 "waf-options.vala"
				g_string_append (sb, self->priv->_distclean_options);
#line 223 "waf-options.c"
			}
#line 95 "waf-options.vala"
			break;
#line 227 "waf-options.c"
		}
		case BUILDING_STEP_UNINSTALL:
		{
#line 97 "waf-options.vala"
			g_string_append (sb, "uninstall ");
#line 98 "waf-options.vala"
			if (self->priv->_uninstall_options != NULL) {
#line 100 "waf-options.vala"
				g_string_append (sb, self->priv->_uninstall_options);
#line 237 "waf-options.c"
			}
#line 102 "waf-options.vala"
			break;
#line 241 "waf-options.c"
		}
	}
#line 104 "waf-options.vala"
	_tmp2_ = NULL;
#line 246 "waf-options.c"
	result = (_tmp2_ = sb->str, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_));
	(sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL));
	return result;
	(sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL));
}


#line 22 "waf-options.vala"
ValideWafBuilderOptions* valide_waf_builder_options_construct (GType object_type) {
#line 256 "waf-options.c"
	ValideWafBuilderOptions * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


#line 22 "waf-options.vala"
ValideWafBuilderOptions* valide_waf_builder_options_new (void) {
#line 22 "waf-options.vala"
	return valide_waf_builder_options_construct (VALIDE_TYPE_WAF_BUILDER_OPTIONS);
#line 267 "waf-options.c"
}


const char* valide_waf_builder_options_get_build_dir (ValideWafBuilderOptions* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_build_dir;
	return result;
}


void valide_waf_builder_options_set_build_dir (ValideWafBuilderOptions* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 28 "waf-options.vala"
	_tmp0_ = NULL;
#line 286 "waf-options.c"
	self->priv->_build_dir = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_build_dir = (g_free (self->priv->_build_dir), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "build-dir");
}


const char* valide_waf_builder_options_get_prefix (ValideWafBuilderOptions* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_prefix;
	return result;
}


void valide_waf_builder_options_set_prefix (ValideWafBuilderOptions* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 33 "waf-options.vala"
	_tmp0_ = NULL;
#line 307 "waf-options.c"
	self->priv->_prefix = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_prefix = (g_free (self->priv->_prefix), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "prefix");
}


const char* valide_waf_builder_options_get_configure_options (ValideWafBuilderOptions* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_configure_options;
	return result;
}


void valide_waf_builder_options_set_configure_options (ValideWafBuilderOptions* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 35 "waf-options.vala"
	_tmp0_ = NULL;
#line 328 "waf-options.c"
	self->priv->_configure_options = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_configure_options = (g_free (self->priv->_configure_options), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "configure-options");
}


const char* valide_waf_builder_options_get_build_options (ValideWafBuilderOptions* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_build_options;
	return result;
}


void valide_waf_builder_options_set_build_options (ValideWafBuilderOptions* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 36 "waf-options.vala"
	_tmp0_ = NULL;
#line 349 "waf-options.c"
	self->priv->_build_options = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_build_options = (g_free (self->priv->_build_options), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "build-options");
}


const char* valide_waf_builder_options_get_install_options (ValideWafBuilderOptions* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_install_options;
	return result;
}


void valide_waf_builder_options_set_install_options (ValideWafBuilderOptions* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 37 "waf-options.vala"
	_tmp0_ = NULL;
#line 370 "waf-options.c"
	self->priv->_install_options = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_install_options = (g_free (self->priv->_install_options), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "install-options");
}


const char* valide_waf_builder_options_get_dist_options (ValideWafBuilderOptions* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_dist_options;
	return result;
}


void valide_waf_builder_options_set_dist_options (ValideWafBuilderOptions* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 38 "waf-options.vala"
	_tmp0_ = NULL;
#line 391 "waf-options.c"
	self->priv->_dist_options = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_dist_options = (g_free (self->priv->_dist_options), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "dist-options");
}


const char* valide_waf_builder_options_get_clean_options (ValideWafBuilderOptions* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_clean_options;
	return result;
}


void valide_waf_builder_options_set_clean_options (ValideWafBuilderOptions* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 39 "waf-options.vala"
	_tmp0_ = NULL;
#line 412 "waf-options.c"
	self->priv->_clean_options = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_clean_options = (g_free (self->priv->_clean_options), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "clean-options");
}


const char* valide_waf_builder_options_get_distclean_options (ValideWafBuilderOptions* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_distclean_options;
	return result;
}


void valide_waf_builder_options_set_distclean_options (ValideWafBuilderOptions* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 40 "waf-options.vala"
	_tmp0_ = NULL;
#line 433 "waf-options.c"
	self->priv->_distclean_options = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_distclean_options = (g_free (self->priv->_distclean_options), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "distclean-options");
}


const char* valide_waf_builder_options_get_uninstall_options (ValideWafBuilderOptions* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_uninstall_options;
	return result;
}


void valide_waf_builder_options_set_uninstall_options (ValideWafBuilderOptions* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 41 "waf-options.vala"
	_tmp0_ = NULL;
#line 454 "waf-options.c"
	self->priv->_uninstall_options = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_uninstall_options = (g_free (self->priv->_uninstall_options), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "uninstall-options");
}


static void valide_waf_builder_options_class_init (ValideWafBuilderOptionsClass * klass) {
	valide_waf_builder_options_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideWafBuilderOptionsPrivate));
	G_OBJECT_CLASS (klass)->get_property = valide_waf_builder_options_get_property;
	G_OBJECT_CLASS (klass)->set_property = valide_waf_builder_options_set_property;
	G_OBJECT_CLASS (klass)->finalize = valide_waf_builder_options_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_WAF_BUILDER_OPTIONS_BUILD_DIR, g_param_spec_string ("build-dir", "build-dir", "build-dir", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_WAF_BUILDER_OPTIONS_PREFIX, g_param_spec_string ("prefix", "prefix", "prefix", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_WAF_BUILDER_OPTIONS_CONFIGURE_OPTIONS, g_param_spec_string ("configure-options", "configure-options", "configure-options", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_WAF_BUILDER_OPTIONS_BUILD_OPTIONS, g_param_spec_string ("build-options", "build-options", "build-options", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_WAF_BUILDER_OPTIONS_INSTALL_OPTIONS, g_param_spec_string ("install-options", "install-options", "install-options", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_WAF_BUILDER_OPTIONS_DIST_OPTIONS, g_param_spec_string ("dist-options", "dist-options", "dist-options", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_WAF_BUILDER_OPTIONS_CLEAN_OPTIONS, g_param_spec_string ("clean-options", "clean-options", "clean-options", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_WAF_BUILDER_OPTIONS_DISTCLEAN_OPTIONS, g_param_spec_string ("distclean-options", "distclean-options", "distclean-options", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALIDE_WAF_BUILDER_OPTIONS_UNINSTALL_OPTIONS, g_param_spec_string ("uninstall-options", "uninstall-options", "uninstall-options", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void valide_waf_builder_options_valide_builder_options_interface_init (ValideBuilderOptionsIface * iface) {
	valide_waf_builder_options_valide_builder_options_parent_iface = g_type_interface_peek_parent (iface);
}


static void valide_waf_builder_options_g_yaml_buildable_interface_init (GYAMLBuildableIface * iface) {
	valide_waf_builder_options_g_yaml_buildable_parent_iface = g_type_interface_peek_parent (iface);
}


static void valide_waf_builder_options_instance_init (ValideWafBuilderOptions * self) {
	self->priv = VALIDE_WAF_BUILDER_OPTIONS_GET_PRIVATE (self);
}


static void valide_waf_builder_options_finalize (GObject* obj) {
	ValideWafBuilderOptions * self;
	self = VALIDE_WAF_BUILDER_OPTIONS (obj);
	self->priv->_build_dir = (g_free (self->priv->_build_dir), NULL);
	self->priv->_prefix = (g_free (self->priv->_prefix), NULL);
	self->priv->_configure_options = (g_free (self->priv->_configure_options), NULL);
	self->priv->_build_options = (g_free (self->priv->_build_options), NULL);
	self->priv->_install_options = (g_free (self->priv->_install_options), NULL);
	self->priv->_dist_options = (g_free (self->priv->_dist_options), NULL);
	self->priv->_clean_options = (g_free (self->priv->_clean_options), NULL);
	self->priv->_distclean_options = (g_free (self->priv->_distclean_options), NULL);
	self->priv->_uninstall_options = (g_free (self->priv->_uninstall_options), NULL);
	G_OBJECT_CLASS (valide_waf_builder_options_parent_class)->finalize (obj);
}


GType valide_waf_builder_options_get_type (void) {
	static GType valide_waf_builder_options_type_id = 0;
	if (valide_waf_builder_options_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideWafBuilderOptionsClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_waf_builder_options_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideWafBuilderOptions), 0, (GInstanceInitFunc) valide_waf_builder_options_instance_init, NULL };
		static const GInterfaceInfo valide_builder_options_info = { (GInterfaceInitFunc) valide_waf_builder_options_valide_builder_options_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo g_yaml_buildable_info = { (GInterfaceInitFunc) valide_waf_builder_options_g_yaml_buildable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		valide_waf_builder_options_type_id = g_type_register_static (G_TYPE_OBJECT, "ValideWafBuilderOptions", &g_define_type_info, 0);
		g_type_add_interface_static (valide_waf_builder_options_type_id, VALIDE_TYPE_BUILDER_OPTIONS, &valide_builder_options_info);
		g_type_add_interface_static (valide_waf_builder_options_type_id, G_YAML_TYPE_BUILDABLE, &g_yaml_buildable_info);
	}
	return valide_waf_builder_options_type_id;
}


static void valide_waf_builder_options_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValideWafBuilderOptions * self;
	gpointer boxed;
	self = VALIDE_WAF_BUILDER_OPTIONS (object);
	switch (property_id) {
		case VALIDE_WAF_BUILDER_OPTIONS_BUILD_DIR:
		g_value_set_string (value, valide_waf_builder_options_get_build_dir (self));
		break;
		case VALIDE_WAF_BUILDER_OPTIONS_PREFIX:
		g_value_set_string (value, valide_waf_builder_options_get_prefix (self));
		break;
		case VALIDE_WAF_BUILDER_OPTIONS_CONFIGURE_OPTIONS:
		g_value_set_string (value, valide_waf_builder_options_get_configure_options (self));
		break;
		case VALIDE_WAF_BUILDER_OPTIONS_BUILD_OPTIONS:
		g_value_set_string (value, valide_waf_builder_options_get_build_options (self));
		break;
		case VALIDE_WAF_BUILDER_OPTIONS_INSTALL_OPTIONS:
		g_value_set_string (value, valide_waf_builder_options_get_install_options (self));
		break;
		case VALIDE_WAF_BUILDER_OPTIONS_DIST_OPTIONS:
		g_value_set_string (value, valide_waf_builder_options_get_dist_options (self));
		break;
		case VALIDE_WAF_BUILDER_OPTIONS_CLEAN_OPTIONS:
		g_value_set_string (value, valide_waf_builder_options_get_clean_options (self));
		break;
		case VALIDE_WAF_BUILDER_OPTIONS_DISTCLEAN_OPTIONS:
		g_value_set_string (value, valide_waf_builder_options_get_distclean_options (self));
		break;
		case VALIDE_WAF_BUILDER_OPTIONS_UNINSTALL_OPTIONS:
		g_value_set_string (value, valide_waf_builder_options_get_uninstall_options (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void valide_waf_builder_options_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValideWafBuilderOptions * self;
	self = VALIDE_WAF_BUILDER_OPTIONS (object);
	switch (property_id) {
		case VALIDE_WAF_BUILDER_OPTIONS_BUILD_DIR:
		valide_waf_builder_options_set_build_dir (self, g_value_get_string (value));
		break;
		case VALIDE_WAF_BUILDER_OPTIONS_PREFIX:
		valide_waf_builder_options_set_prefix (self, g_value_get_string (value));
		break;
		case VALIDE_WAF_BUILDER_OPTIONS_CONFIGURE_OPTIONS:
		valide_waf_builder_options_set_configure_options (self, g_value_get_string (value));
		break;
		case VALIDE_WAF_BUILDER_OPTIONS_BUILD_OPTIONS:
		valide_waf_builder_options_set_build_options (self, g_value_get_string (value));
		break;
		case VALIDE_WAF_BUILDER_OPTIONS_INSTALL_OPTIONS:
		valide_waf_builder_options_set_install_options (self, g_value_get_string (value));
		break;
		case VALIDE_WAF_BUILDER_OPTIONS_DIST_OPTIONS:
		valide_waf_builder_options_set_dist_options (self, g_value_get_string (value));
		break;
		case VALIDE_WAF_BUILDER_OPTIONS_CLEAN_OPTIONS:
		valide_waf_builder_options_set_clean_options (self, g_value_get_string (value));
		break;
		case VALIDE_WAF_BUILDER_OPTIONS_DISTCLEAN_OPTIONS:
		valide_waf_builder_options_set_distclean_options (self, g_value_get_string (value));
		break;
		case VALIDE_WAF_BUILDER_OPTIONS_UNINSTALL_OPTIONS:
		valide_waf_builder_options_set_uninstall_options (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




