/* waf-preferences.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <valide-0.0.h>
#include <glib/gi18n-lib.h>
#include <stdlib.h>
#include <string.h>


#define TYPE_WAF_BUILDER_PREFERENCES (waf_builder_preferences_get_type ())
#define WAF_BUILDER_PREFERENCES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_WAF_BUILDER_PREFERENCES, WafBuilderPreferences))
#define WAF_BUILDER_PREFERENCES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_WAF_BUILDER_PREFERENCES, WafBuilderPreferencesClass))
#define IS_WAF_BUILDER_PREFERENCES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_WAF_BUILDER_PREFERENCES))
#define IS_WAF_BUILDER_PREFERENCES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_WAF_BUILDER_PREFERENCES))
#define WAF_BUILDER_PREFERENCES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_WAF_BUILDER_PREFERENCES, WafBuilderPreferencesClass))

typedef struct _WafBuilderPreferences WafBuilderPreferences;
typedef struct _WafBuilderPreferencesClass WafBuilderPreferencesClass;
typedef struct _WafBuilderPreferencesPrivate WafBuilderPreferencesPrivate;

#define VALIDE_TYPE_WAF_BUILDER_OPTIONS (valide_waf_builder_options_get_type ())
#define VALIDE_WAF_BUILDER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_WAF_BUILDER_OPTIONS, ValideWafBuilderOptions))
#define VALIDE_WAF_BUILDER_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_WAF_BUILDER_OPTIONS, ValideWafBuilderOptionsClass))
#define VALIDE_IS_WAF_BUILDER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_WAF_BUILDER_OPTIONS))
#define VALIDE_IS_WAF_BUILDER_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_WAF_BUILDER_OPTIONS))
#define VALIDE_WAF_BUILDER_OPTIONS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_WAF_BUILDER_OPTIONS, ValideWafBuilderOptionsClass))

typedef struct _ValideWafBuilderOptions ValideWafBuilderOptions;
typedef struct _ValideWafBuilderOptionsClass ValideWafBuilderOptionsClass;

struct _WafBuilderPreferences {
	GtkTable parent_instance;
	WafBuilderPreferencesPrivate * priv;
};

struct _WafBuilderPreferencesClass {
	GtkTableClass parent_class;
};

struct _WafBuilderPreferencesPrivate {
	ValideWafBuilderOptions* options;
	ValideProject* _project;
};


static gpointer waf_builder_preferences_parent_class = NULL;

GType waf_builder_preferences_get_type (void);
GType valide_waf_builder_options_get_type (void);
#define WAF_BUILDER_PREFERENCES_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_WAF_BUILDER_PREFERENCES, WafBuilderPreferencesPrivate))
enum  {
	WAF_BUILDER_PREFERENCES_DUMMY_PROPERTY,
	WAF_BUILDER_PREFERENCES_PROJECT
};
static void waf_builder_preferences_set_project (WafBuilderPreferences* self, ValideProject* value);
WafBuilderPreferences* waf_builder_preferences_new (ValideProject* project);
WafBuilderPreferences* waf_builder_preferences_construct (GType object_type, ValideProject* project);
ValideProject* waf_builder_preferences_get_project (WafBuilderPreferences* self);
const char* valide_waf_builder_options_get_build_dir (ValideWafBuilderOptions* self);
void valide_waf_builder_options_set_build_dir (ValideWafBuilderOptions* self, const char* value);
static void _lambda0_ (GtkEditable* s, WafBuilderPreferences* self);
static void __lambda0__gtk_editable_changed (GtkEditable* _sender, gpointer self);
const char* valide_waf_builder_options_get_prefix (ValideWafBuilderOptions* self);
void valide_waf_builder_options_set_prefix (ValideWafBuilderOptions* self, const char* value);
static void _lambda1_ (GtkEditable* s, WafBuilderPreferences* self);
static void __lambda1__gtk_editable_changed (GtkEditable* _sender, gpointer self);
const char* valide_waf_builder_options_get_configure_options (ValideWafBuilderOptions* self);
void valide_waf_builder_options_set_configure_options (ValideWafBuilderOptions* self, const char* value);
static void _lambda2_ (GtkEditable* s, WafBuilderPreferences* self);
static void __lambda2__gtk_editable_changed (GtkEditable* _sender, gpointer self);
const char* valide_waf_builder_options_get_build_options (ValideWafBuilderOptions* self);
void valide_waf_builder_options_set_build_options (ValideWafBuilderOptions* self, const char* value);
static void _lambda3_ (GtkEditable* s, WafBuilderPreferences* self);
static void __lambda3__gtk_editable_changed (GtkEditable* _sender, gpointer self);
const char* valide_waf_builder_options_get_install_options (ValideWafBuilderOptions* self);
void valide_waf_builder_options_set_install_options (ValideWafBuilderOptions* self, const char* value);
static void _lambda4_ (GtkEditable* s, WafBuilderPreferences* self);
static void __lambda4__gtk_editable_changed (GtkEditable* _sender, gpointer self);
const char* valide_waf_builder_options_get_dist_options (ValideWafBuilderOptions* self);
void valide_waf_builder_options_set_dist_options (ValideWafBuilderOptions* self, const char* value);
static void _lambda5_ (GtkEditable* s, WafBuilderPreferences* self);
static void __lambda5__gtk_editable_changed (GtkEditable* _sender, gpointer self);
const char* valide_waf_builder_options_get_clean_options (ValideWafBuilderOptions* self);
void valide_waf_builder_options_set_clean_options (ValideWafBuilderOptions* self, const char* value);
static void _lambda6_ (GtkEditable* s, WafBuilderPreferences* self);
static void __lambda6__gtk_editable_changed (GtkEditable* _sender, gpointer self);
const char* valide_waf_builder_options_get_distclean_options (ValideWafBuilderOptions* self);
void valide_waf_builder_options_set_distclean_options (ValideWafBuilderOptions* self, const char* value);
static void _lambda7_ (GtkEditable* s, WafBuilderPreferences* self);
static void __lambda7__gtk_editable_changed (GtkEditable* _sender, gpointer self);
const char* valide_waf_builder_options_get_uninstall_options (ValideWafBuilderOptions* self);
void valide_waf_builder_options_set_uninstall_options (ValideWafBuilderOptions* self, const char* value);
static void _lambda8_ (GtkEditable* s, WafBuilderPreferences* self);
static void __lambda8__gtk_editable_changed (GtkEditable* _sender, gpointer self);
static GObject * waf_builder_preferences_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void waf_builder_preferences_finalize (GObject* obj);
static void waf_builder_preferences_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void waf_builder_preferences_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);



#line 34 "waf-preferences.vala"
WafBuilderPreferences* waf_builder_preferences_construct (GType object_type, ValideProject* project) {
#line 126 "waf-preferences.c"
	GParameter * __params;
	GParameter * __params_it;
	WafBuilderPreferences * self;
#line 34 "waf-preferences.vala"
	g_return_val_if_fail (project != NULL, NULL);
#line 132 "waf-preferences.c"
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
#line 36 "waf-preferences.vala"
	__params_it->name = "project";
#line 36 "waf-preferences.vala"
	g_value_init (&__params_it->value, VALIDE_TYPE_PROJECT);
#line 36 "waf-preferences.vala"
	g_value_set_object (&__params_it->value, project);
#line 36 "waf-preferences.vala"
	__params_it++;
#line 143 "waf-preferences.c"
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


#line 34 "waf-preferences.vala"
WafBuilderPreferences* waf_builder_preferences_new (ValideProject* project) {
#line 34 "waf-preferences.vala"
	return waf_builder_preferences_construct (TYPE_WAF_BUILDER_PREFERENCES, project);
#line 158 "waf-preferences.c"
}


ValideProject* waf_builder_preferences_get_project (WafBuilderPreferences* self) {
	ValideProject* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_project;
	return result;
}


static void waf_builder_preferences_set_project (WafBuilderPreferences* self, ValideProject* value) {
	ValideProject* _tmp1_;
	ValideProject* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
#line 31 "waf-preferences.vala"
	_tmp0_ = NULL;
#line 177 "waf-preferences.c"
	self->priv->_project = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_project == NULL) ? NULL : (self->priv->_project = (g_object_unref (self->priv->_project), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "project");
}


static void _lambda0_ (GtkEditable* s, WafBuilderPreferences* self) {
	GtkEditable* _tmp0_;
	g_return_if_fail (s != NULL);
#line 61 "waf-preferences.vala"
	_tmp0_ = NULL;
#line 61 "waf-preferences.vala"
	valide_waf_builder_options_set_build_dir (self->priv->options, gtk_entry_get_text ((_tmp0_ = s, GTK_IS_ENTRY (_tmp0_) ? ((GtkEntry*) _tmp0_) : NULL)));
#line 62 "waf-preferences.vala"
	valide_project_save (self->priv->_project);
#line 192 "waf-preferences.c"
}


static void __lambda0__gtk_editable_changed (GtkEditable* _sender, gpointer self) {
	_lambda0_ (_sender, self);
}


static void _lambda1_ (GtkEditable* s, WafBuilderPreferences* self) {
	GtkEditable* _tmp0_;
	g_return_if_fail (s != NULL);
#line 77 "waf-preferences.vala"
	_tmp0_ = NULL;
#line 77 "waf-preferences.vala"
	valide_waf_builder_options_set_prefix (self->priv->options, gtk_entry_get_text ((_tmp0_ = s, GTK_IS_ENTRY (_tmp0_) ? ((GtkEntry*) _tmp0_) : NULL)));
#line 78 "waf-preferences.vala"
	valide_project_save (self->priv->_project);
#line 210 "waf-preferences.c"
}


static void __lambda1__gtk_editable_changed (GtkEditable* _sender, gpointer self) {
	_lambda1_ (_sender, self);
}


static void _lambda2_ (GtkEditable* s, WafBuilderPreferences* self) {
	GtkEditable* _tmp0_;
	g_return_if_fail (s != NULL);
#line 93 "waf-preferences.vala"
	_tmp0_ = NULL;
#line 93 "waf-preferences.vala"
	valide_waf_builder_options_set_configure_options (self->priv->options, gtk_entry_get_text ((_tmp0_ = s, GTK_IS_ENTRY (_tmp0_) ? ((GtkEntry*) _tmp0_) : NULL)));
#line 94 "waf-preferences.vala"
	valide_project_save (self->priv->_project);
#line 228 "waf-preferences.c"
}


static void __lambda2__gtk_editable_changed (GtkEditable* _sender, gpointer self) {
	_lambda2_ (_sender, self);
}


static void _lambda3_ (GtkEditable* s, WafBuilderPreferences* self) {
	GtkEditable* _tmp0_;
	g_return_if_fail (s != NULL);
#line 109 "waf-preferences.vala"
	_tmp0_ = NULL;
#line 109 "waf-preferences.vala"
	valide_waf_builder_options_set_build_options (self->priv->options, gtk_entry_get_text ((_tmp0_ = s, GTK_IS_ENTRY (_tmp0_) ? ((GtkEntry*) _tmp0_) : NULL)));
#line 110 "waf-preferences.vala"
	valide_project_save (self->priv->_project);
#line 246 "waf-preferences.c"
}


static void __lambda3__gtk_editable_changed (GtkEditable* _sender, gpointer self) {
	_lambda3_ (_sender, self);
}


static void _lambda4_ (GtkEditable* s, WafBuilderPreferences* self) {
	GtkEditable* _tmp0_;
	g_return_if_fail (s != NULL);
#line 125 "waf-preferences.vala"
	_tmp0_ = NULL;
#line 125 "waf-preferences.vala"
	valide_waf_builder_options_set_install_options (self->priv->options, gtk_entry_get_text ((_tmp0_ = s, GTK_IS_ENTRY (_tmp0_) ? ((GtkEntry*) _tmp0_) : NULL)));
#line 126 "waf-preferences.vala"
	valide_project_save (self->priv->_project);
#line 264 "waf-preferences.c"
}


static void __lambda4__gtk_editable_changed (GtkEditable* _sender, gpointer self) {
	_lambda4_ (_sender, self);
}


static void _lambda5_ (GtkEditable* s, WafBuilderPreferences* self) {
	GtkEditable* _tmp0_;
	g_return_if_fail (s != NULL);
#line 141 "waf-preferences.vala"
	_tmp0_ = NULL;
#line 141 "waf-preferences.vala"
	valide_waf_builder_options_set_dist_options (self->priv->options, gtk_entry_get_text ((_tmp0_ = s, GTK_IS_ENTRY (_tmp0_) ? ((GtkEntry*) _tmp0_) : NULL)));
#line 142 "waf-preferences.vala"
	valide_project_save (self->priv->_project);
#line 282 "waf-preferences.c"
}


static void __lambda5__gtk_editable_changed (GtkEditable* _sender, gpointer self) {
	_lambda5_ (_sender, self);
}


static void _lambda6_ (GtkEditable* s, WafBuilderPreferences* self) {
	GtkEditable* _tmp0_;
	g_return_if_fail (s != NULL);
#line 157 "waf-preferences.vala"
	_tmp0_ = NULL;
#line 157 "waf-preferences.vala"
	valide_waf_builder_options_set_clean_options (self->priv->options, gtk_entry_get_text ((_tmp0_ = s, GTK_IS_ENTRY (_tmp0_) ? ((GtkEntry*) _tmp0_) : NULL)));
#line 158 "waf-preferences.vala"
	valide_project_save (self->priv->_project);
#line 300 "waf-preferences.c"
}


static void __lambda6__gtk_editable_changed (GtkEditable* _sender, gpointer self) {
	_lambda6_ (_sender, self);
}


static void _lambda7_ (GtkEditable* s, WafBuilderPreferences* self) {
	GtkEditable* _tmp0_;
	g_return_if_fail (s != NULL);
#line 173 "waf-preferences.vala"
	_tmp0_ = NULL;
#line 173 "waf-preferences.vala"
	valide_waf_builder_options_set_distclean_options (self->priv->options, gtk_entry_get_text ((_tmp0_ = s, GTK_IS_ENTRY (_tmp0_) ? ((GtkEntry*) _tmp0_) : NULL)));
#line 174 "waf-preferences.vala"
	valide_project_save (self->priv->_project);
#line 318 "waf-preferences.c"
}


static void __lambda7__gtk_editable_changed (GtkEditable* _sender, gpointer self) {
	_lambda7_ (_sender, self);
}


static void _lambda8_ (GtkEditable* s, WafBuilderPreferences* self) {
	GtkEditable* _tmp0_;
	g_return_if_fail (s != NULL);
#line 189 "waf-preferences.vala"
	_tmp0_ = NULL;
#line 189 "waf-preferences.vala"
	valide_waf_builder_options_set_uninstall_options (self->priv->options, gtk_entry_get_text ((_tmp0_ = s, GTK_IS_ENTRY (_tmp0_) ? ((GtkEntry*) _tmp0_) : NULL)));
#line 190 "waf-preferences.vala"
	valide_project_save (self->priv->_project);
#line 336 "waf-preferences.c"
}


static void __lambda8__gtk_editable_changed (GtkEditable* _sender, gpointer self) {
	_lambda8_ (_sender, self);
}


static GObject * waf_builder_preferences_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	WafBuilderPreferencesClass * klass;
	GObjectClass * parent_class;
	WafBuilderPreferences * self;
	klass = WAF_BUILDER_PREFERENCES_CLASS (g_type_class_peek (TYPE_WAF_BUILDER_PREFERENCES));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = WAF_BUILDER_PREFERENCES (obj);
	{
		GtkLabel* label;
		GtkEntry* entry;
		ValideBuilderOptions* _tmp0_;
		GtkLabel* _tmp1_;
		GtkEntry* _tmp2_;
		GtkLabel* _tmp3_;
		GtkEntry* _tmp4_;
		GtkLabel* _tmp5_;
		GtkEntry* _tmp6_;
		GtkLabel* _tmp7_;
		GtkEntry* _tmp8_;
		GtkLabel* _tmp9_;
		GtkEntry* _tmp10_;
		GtkLabel* _tmp11_;
		GtkEntry* _tmp12_;
		GtkLabel* _tmp13_;
		GtkEntry* _tmp14_;
		GtkLabel* _tmp15_;
		GtkEntry* _tmp16_;
		GtkLabel* _tmp17_;
		GtkEntry* _tmp18_;
		label = NULL;
		entry = NULL;
#line 44 "waf-preferences.vala"
		gtk_table_resize ((GtkTable*) self, (guint) 2, (guint) 9);
#line 45 "waf-preferences.vala"
		gtk_container_set_border_width ((GtkContainer*) self, (guint) 5);
#line 46 "waf-preferences.vala"
		gtk_table_set_col_spacings ((GtkTable*) self, (guint) 5);
#line 47 "waf-preferences.vala"
		gtk_table_set_row_spacings ((GtkTable*) self, (guint) 10);
#line 48 "waf-preferences.vala"
		_tmp0_ = NULL;
#line 48 "waf-preferences.vala"
		self->priv->options = (_tmp0_ = valide_project_get_builder_options (self->priv->_project), VALIDE_IS_WAF_BUILDER_OPTIONS (_tmp0_) ? ((ValideWafBuilderOptions*) _tmp0_) : NULL);
#line 390 "waf-preferences.c"
		/* building dir */
		_tmp1_ = NULL;
#line 51 "waf-preferences.vala"
		label = (_tmp1_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("Building directory:"))), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp1_);
#line 52 "waf-preferences.vala"
		gtk_misc_set_alignment ((GtkMisc*) label, 0.0f, 0.5f);
#line 53 "waf-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) label, (guint) 0, (guint) 1, (guint) 0, (guint) 1, GTK_FILL, 0, (guint) 0, (guint) 0);
#line 399 "waf-preferences.c"
		_tmp2_ = NULL;
#line 55 "waf-preferences.vala"
		entry = (_tmp2_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL)), _tmp2_);
#line 56 "waf-preferences.vala"
		if (valide_waf_builder_options_get_build_dir (self->priv->options) != NULL) {
#line 58 "waf-preferences.vala"
			gtk_entry_set_text (entry, valide_waf_builder_options_get_build_dir (self->priv->options));
#line 407 "waf-preferences.c"
		}
#line 60 "waf-preferences.vala"
		g_signal_connect ((GtkEditable*) entry, "changed", (GCallback) __lambda0__gtk_editable_changed, self);
#line 64 "waf-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) entry, (guint) 1, (guint) 2, (guint) 0, (guint) 1, GTK_EXPAND | GTK_FILL, 0, (guint) 0, (guint) 0);
#line 413 "waf-preferences.c"
		/* prefix */
		_tmp3_ = NULL;
#line 67 "waf-preferences.vala"
		label = (_tmp3_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("Prefix:"))), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp3_);
#line 68 "waf-preferences.vala"
		gtk_misc_set_alignment ((GtkMisc*) label, 0.0f, 0.5f);
#line 69 "waf-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) label, (guint) 0, (guint) 1, (guint) 1, (guint) 2, GTK_FILL, 0, (guint) 0, (guint) 0);
#line 422 "waf-preferences.c"
		_tmp4_ = NULL;
#line 71 "waf-preferences.vala"
		entry = (_tmp4_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL)), _tmp4_);
#line 72 "waf-preferences.vala"
		if (valide_waf_builder_options_get_prefix (self->priv->options) != NULL) {
#line 74 "waf-preferences.vala"
			gtk_entry_set_text (entry, valide_waf_builder_options_get_prefix (self->priv->options));
#line 430 "waf-preferences.c"
		}
#line 76 "waf-preferences.vala"
		g_signal_connect ((GtkEditable*) entry, "changed", (GCallback) __lambda1__gtk_editable_changed, self);
#line 80 "waf-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) entry, (guint) 1, (guint) 2, (guint) 1, (guint) 2, GTK_EXPAND | GTK_FILL, 0, (guint) 0, (guint) 0);
#line 436 "waf-preferences.c"
		/* configure_options */
		_tmp5_ = NULL;
#line 83 "waf-preferences.vala"
		label = (_tmp5_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("Configure options:"))), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp5_);
#line 84 "waf-preferences.vala"
		gtk_misc_set_alignment ((GtkMisc*) label, 0.0f, 0.5f);
#line 85 "waf-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) label, (guint) 0, (guint) 1, (guint) 2, (guint) 3, GTK_FILL, 0, (guint) 0, (guint) 0);
#line 445 "waf-preferences.c"
		_tmp6_ = NULL;
#line 87 "waf-preferences.vala"
		entry = (_tmp6_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL)), _tmp6_);
#line 88 "waf-preferences.vala"
		if (valide_waf_builder_options_get_configure_options (self->priv->options) != NULL) {
#line 90 "waf-preferences.vala"
			gtk_entry_set_text (entry, valide_waf_builder_options_get_configure_options (self->priv->options));
#line 453 "waf-preferences.c"
		}
#line 92 "waf-preferences.vala"
		g_signal_connect ((GtkEditable*) entry, "changed", (GCallback) __lambda2__gtk_editable_changed, self);
#line 96 "waf-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) entry, (guint) 1, (guint) 2, (guint) 2, (guint) 3, GTK_EXPAND | GTK_FILL, 0, (guint) 0, (guint) 0);
#line 459 "waf-preferences.c"
		/* build_options */
		_tmp7_ = NULL;
#line 99 "waf-preferences.vala"
		label = (_tmp7_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("Build options:"))), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp7_);
#line 100 "waf-preferences.vala"
		gtk_misc_set_alignment ((GtkMisc*) label, 0.0f, 0.5f);
#line 101 "waf-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) label, (guint) 0, (guint) 1, (guint) 3, (guint) 4, GTK_FILL, 0, (guint) 0, (guint) 0);
#line 468 "waf-preferences.c"
		_tmp8_ = NULL;
#line 103 "waf-preferences.vala"
		entry = (_tmp8_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL)), _tmp8_);
#line 104 "waf-preferences.vala"
		if (valide_waf_builder_options_get_build_options (self->priv->options) != NULL) {
#line 106 "waf-preferences.vala"
			gtk_entry_set_text (entry, valide_waf_builder_options_get_build_options (self->priv->options));
#line 476 "waf-preferences.c"
		}
#line 108 "waf-preferences.vala"
		g_signal_connect ((GtkEditable*) entry, "changed", (GCallback) __lambda3__gtk_editable_changed, self);
#line 112 "waf-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) entry, (guint) 1, (guint) 2, (guint) 3, (guint) 4, GTK_EXPAND | GTK_FILL, 0, (guint) 0, (guint) 0);
#line 482 "waf-preferences.c"
		/* install_options */
		_tmp9_ = NULL;
#line 115 "waf-preferences.vala"
		label = (_tmp9_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("Install options:"))), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp9_);
#line 116 "waf-preferences.vala"
		gtk_misc_set_alignment ((GtkMisc*) label, 0.0f, 0.5f);
#line 117 "waf-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) label, (guint) 0, (guint) 1, (guint) 4, (guint) 5, GTK_FILL, 0, (guint) 0, (guint) 0);
#line 491 "waf-preferences.c"
		_tmp10_ = NULL;
#line 119 "waf-preferences.vala"
		entry = (_tmp10_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL)), _tmp10_);
#line 120 "waf-preferences.vala"
		if (valide_waf_builder_options_get_install_options (self->priv->options) != NULL) {
#line 122 "waf-preferences.vala"
			gtk_entry_set_text (entry, valide_waf_builder_options_get_install_options (self->priv->options));
#line 499 "waf-preferences.c"
		}
#line 124 "waf-preferences.vala"
		g_signal_connect ((GtkEditable*) entry, "changed", (GCallback) __lambda4__gtk_editable_changed, self);
#line 128 "waf-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) entry, (guint) 1, (guint) 2, (guint) 4, (guint) 5, GTK_EXPAND | GTK_FILL, 0, (guint) 0, (guint) 0);
#line 505 "waf-preferences.c"
		/* dist_options */
		_tmp11_ = NULL;
#line 131 "waf-preferences.vala"
		label = (_tmp11_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("Dist options:"))), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp11_);
#line 132 "waf-preferences.vala"
		gtk_misc_set_alignment ((GtkMisc*) label, 0.0f, 0.5f);
#line 133 "waf-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) label, (guint) 0, (guint) 1, (guint) 5, (guint) 6, GTK_FILL, 0, (guint) 0, (guint) 0);
#line 514 "waf-preferences.c"
		_tmp12_ = NULL;
#line 135 "waf-preferences.vala"
		entry = (_tmp12_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL)), _tmp12_);
#line 136 "waf-preferences.vala"
		if (valide_waf_builder_options_get_dist_options (self->priv->options) != NULL) {
#line 138 "waf-preferences.vala"
			gtk_entry_set_text (entry, valide_waf_builder_options_get_dist_options (self->priv->options));
#line 522 "waf-preferences.c"
		}
#line 140 "waf-preferences.vala"
		g_signal_connect ((GtkEditable*) entry, "changed", (GCallback) __lambda5__gtk_editable_changed, self);
#line 144 "waf-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) entry, (guint) 1, (guint) 2, (guint) 5, (guint) 6, GTK_EXPAND | GTK_FILL, 0, (guint) 0, (guint) 0);
#line 528 "waf-preferences.c"
		/* clean_options */
		_tmp13_ = NULL;
#line 147 "waf-preferences.vala"
		label = (_tmp13_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("Clean options:"))), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp13_);
#line 148 "waf-preferences.vala"
		gtk_misc_set_alignment ((GtkMisc*) label, 0.0f, 0.5f);
#line 149 "waf-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) label, (guint) 0, (guint) 1, (guint) 6, (guint) 7, GTK_FILL, 0, (guint) 0, (guint) 0);
#line 537 "waf-preferences.c"
		_tmp14_ = NULL;
#line 151 "waf-preferences.vala"
		entry = (_tmp14_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL)), _tmp14_);
#line 152 "waf-preferences.vala"
		if (valide_waf_builder_options_get_clean_options (self->priv->options) != NULL) {
#line 154 "waf-preferences.vala"
			gtk_entry_set_text (entry, valide_waf_builder_options_get_clean_options (self->priv->options));
#line 545 "waf-preferences.c"
		}
#line 156 "waf-preferences.vala"
		g_signal_connect ((GtkEditable*) entry, "changed", (GCallback) __lambda6__gtk_editable_changed, self);
#line 160 "waf-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) entry, (guint) 1, (guint) 2, (guint) 6, (guint) 7, GTK_EXPAND | GTK_FILL, 0, (guint) 0, (guint) 0);
#line 551 "waf-preferences.c"
		/* distclean_options */
		_tmp15_ = NULL;
#line 163 "waf-preferences.vala"
		label = (_tmp15_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("Distclean options:"))), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp15_);
#line 164 "waf-preferences.vala"
		gtk_misc_set_alignment ((GtkMisc*) label, 0.0f, 0.5f);
#line 165 "waf-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) label, (guint) 0, (guint) 1, (guint) 7, (guint) 8, GTK_FILL, 0, (guint) 0, (guint) 0);
#line 560 "waf-preferences.c"
		_tmp16_ = NULL;
#line 167 "waf-preferences.vala"
		entry = (_tmp16_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL)), _tmp16_);
#line 168 "waf-preferences.vala"
		if (valide_waf_builder_options_get_distclean_options (self->priv->options) != NULL) {
#line 170 "waf-preferences.vala"
			gtk_entry_set_text (entry, valide_waf_builder_options_get_distclean_options (self->priv->options));
#line 568 "waf-preferences.c"
		}
#line 172 "waf-preferences.vala"
		g_signal_connect ((GtkEditable*) entry, "changed", (GCallback) __lambda7__gtk_editable_changed, self);
#line 176 "waf-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) entry, (guint) 1, (guint) 2, (guint) 7, (guint) 8, GTK_EXPAND | GTK_FILL, 0, (guint) 0, (guint) 0);
#line 574 "waf-preferences.c"
		/* uninstall_options */
		_tmp17_ = NULL;
#line 179 "waf-preferences.vala"
		label = (_tmp17_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (_ ("Uninstall options:"))), (label == NULL) ? NULL : (label = (g_object_unref (label), NULL)), _tmp17_);
#line 180 "waf-preferences.vala"
		gtk_misc_set_alignment ((GtkMisc*) label, 0.0f, 0.5f);
#line 181 "waf-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) label, (guint) 0, (guint) 1, (guint) 8, (guint) 9, GTK_FILL, 0, (guint) 0, (guint) 0);
#line 583 "waf-preferences.c"
		_tmp18_ = NULL;
#line 183 "waf-preferences.vala"
		entry = (_tmp18_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL)), _tmp18_);
#line 184 "waf-preferences.vala"
		if (valide_waf_builder_options_get_uninstall_options (self->priv->options) != NULL) {
#line 186 "waf-preferences.vala"
			gtk_entry_set_text (entry, valide_waf_builder_options_get_uninstall_options (self->priv->options));
#line 591 "waf-preferences.c"
		}
#line 188 "waf-preferences.vala"
		g_signal_connect ((GtkEditable*) entry, "changed", (GCallback) __lambda8__gtk_editable_changed, self);
#line 192 "waf-preferences.vala"
		gtk_table_attach ((GtkTable*) self, (GtkWidget*) entry, (guint) 1, (guint) 2, (guint) 8, (guint) 9, GTK_EXPAND | GTK_FILL, 0, (guint) 0, (guint) 0);
#line 597 "waf-preferences.c"
		(label == NULL) ? NULL : (label = (g_object_unref (label), NULL));
		(entry == NULL) ? NULL : (entry = (g_object_unref (entry), NULL));
	}
	return obj;
}


static void waf_builder_preferences_class_init (WafBuilderPreferencesClass * klass) {
	waf_builder_preferences_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (WafBuilderPreferencesPrivate));
	G_OBJECT_CLASS (klass)->get_property = waf_builder_preferences_get_property;
	G_OBJECT_CLASS (klass)->set_property = waf_builder_preferences_set_property;
	G_OBJECT_CLASS (klass)->constructor = waf_builder_preferences_constructor;
	G_OBJECT_CLASS (klass)->finalize = waf_builder_preferences_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), WAF_BUILDER_PREFERENCES_PROJECT, g_param_spec_object ("project", "project", "project", VALIDE_TYPE_PROJECT, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void waf_builder_preferences_instance_init (WafBuilderPreferences * self) {
	self->priv = WAF_BUILDER_PREFERENCES_GET_PRIVATE (self);
}


static void waf_builder_preferences_finalize (GObject* obj) {
	WafBuilderPreferences * self;
	self = WAF_BUILDER_PREFERENCES (obj);
	(self->priv->_project == NULL) ? NULL : (self->priv->_project = (g_object_unref (self->priv->_project), NULL));
	G_OBJECT_CLASS (waf_builder_preferences_parent_class)->finalize (obj);
}


GType waf_builder_preferences_get_type (void) {
	static GType waf_builder_preferences_type_id = 0;
	if (waf_builder_preferences_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (WafBuilderPreferencesClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) waf_builder_preferences_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (WafBuilderPreferences), 0, (GInstanceInitFunc) waf_builder_preferences_instance_init, NULL };
		waf_builder_preferences_type_id = g_type_register_static (GTK_TYPE_TABLE, "WafBuilderPreferences", &g_define_type_info, 0);
	}
	return waf_builder_preferences_type_id;
}


static void waf_builder_preferences_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	WafBuilderPreferences * self;
	gpointer boxed;
	self = WAF_BUILDER_PREFERENCES (object);
	switch (property_id) {
		case WAF_BUILDER_PREFERENCES_PROJECT:
		g_value_set_object (value, waf_builder_preferences_get_project (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void waf_builder_preferences_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	WafBuilderPreferences * self;
	self = WAF_BUILDER_PREFERENCES (object);
	switch (property_id) {
		case WAF_BUILDER_PREFERENCES_PROJECT:
		waf_builder_preferences_set_project (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}




