/* main.vala
 *
 * Copyright (C) 2008-2009 Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <glib/gi18n-lib.h>
#include <valide-0.0.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <unique/unique.h>
#include <config.h>


#define VALIDE_TYPE_APPLICATION (valide_application_get_type ())
#define VALIDE_APPLICATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALIDE_TYPE_APPLICATION, ValideApplication))
#define VALIDE_APPLICATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALIDE_TYPE_APPLICATION, ValideApplicationClass))
#define VALIDE_IS_APPLICATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALIDE_TYPE_APPLICATION))
#define VALIDE_IS_APPLICATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALIDE_TYPE_APPLICATION))
#define VALIDE_APPLICATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALIDE_TYPE_APPLICATION, ValideApplicationClass))

typedef struct _ValideApplication ValideApplication;
typedef struct _ValideApplicationClass ValideApplicationClass;
typedef struct _ValideApplicationPrivate ValideApplicationPrivate;

#define VALIDE_APPLICATION_TYPE_ACTION (valide_application_action_get_type ())

/* { "encoding", '\0', 0, OptionArg.STRING, ref encoding_charset, N_("Set the character encoding to be used to open the files listed on the command line"), N_("ENCODING")}, 
{ "new-window", '\0', 0, OptionArg.NONE, ref new_window_option, N_("Create a new toplevel window in an existing instance of valide"), null },
 "" = G_OPTION_REMAINING 
 collects file arguments */
struct _ValideApplication {
	GObject parent_instance;
	ValideApplicationPrivate * priv;
};

struct _ValideApplicationClass {
	GObjectClass parent_class;
};

struct _ValideApplicationPrivate {
	gboolean show_version;
	gboolean new_document;
	char** file_list;
	gint file_list_length1;
	gint file_list_size;
	GHashTable* windows;
};

typedef enum  {
	VALIDE_APPLICATION_ACTION_NEW,
	VALIDE_APPLICATION_ACTION_OPEN,
	VALIDE_APPLICATION_ACTION_ACTIVATE,
	VALIDE_APPLICATION_ACTION_INVALIDE
} ValideApplicationAction;


extern gboolean new_document_option;
gboolean new_document_option = FALSE;
extern gboolean show_version;
gboolean show_version = FALSE;
extern char** remaining_args;
extern gint remaining_args_length1;
char** remaining_args = NULL;
gint remaining_args_length1 = 0;
static gint remaining_args_size = 0;
static char* valide_application_app_id;
static char* valide_application_app_id = NULL;
static gpointer valide_application_parent_class = NULL;

GType valide_application_get_type (void);
#define VALIDE_APPLICATION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALIDE_TYPE_APPLICATION, ValideApplicationPrivate))
enum  {
	VALIDE_APPLICATION_DUMMY_PROPERTY
};
static GType valide_application_action_get_type (void);
static ValideWindow* valide_application_create_new_window (ValideApplication* self, ValideApplicationAction action, char** uris, int uris_length1, GdkScreen* screen);
static void _lambda0_ (GtkObject* s, ValideApplication* self);
static void __lambda0__gtk_object_destroy (GtkObject* _sender, gpointer self);
static UniqueResponse valide_application_message (ValideApplication* self, UniqueApp* sender, gint command, UniqueMessageData* data, guint time_);
static void _vala_array_add1 (char*** array, int* length, int* size, char* value);
ValideApplication* valide_application_new (char** args, int args_length1);
ValideApplication* valide_application_construct (GType object_type, char** args, int args_length1);
static UniqueResponse _valide_application_message_unique_app_message_received (UniqueApp* _sender, gint command, UniqueMessageData* message_data, guint time_, gpointer self);
gint valide_application_run (ValideApplication* self);
static void valide_application_finalize (GObject* obj);
gint _main (char** args, int args_length1);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);

static const GOptionEntry options[] = {{"version", 'V', 0, G_OPTION_ARG_NONE, &show_version, N_ ("Show the application's version"), NULL}, {"new-document", '\0', 0, G_OPTION_ARG_NONE, &new_document_option, N_ ("Create a new document in an existing instance of valide"), NULL}, {"", '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, &remaining_args, NULL, N_ ("[FILE...]")}, {NULL}};



static GType valide_application_action_get_type (void) {
	static GType valide_application_action_type_id = 0;
	if (G_UNLIKELY (valide_application_action_type_id == 0)) {
		static const GEnumValue values[] = {{VALIDE_APPLICATION_ACTION_NEW, "VALIDE_APPLICATION_ACTION_NEW", "new"}, {VALIDE_APPLICATION_ACTION_OPEN, "VALIDE_APPLICATION_ACTION_OPEN", "open"}, {VALIDE_APPLICATION_ACTION_ACTIVATE, "VALIDE_APPLICATION_ACTION_ACTIVATE", "activate"}, {VALIDE_APPLICATION_ACTION_INVALIDE, "VALIDE_APPLICATION_ACTION_INVALIDE", "invalide"}, {0, NULL, NULL}};
		valide_application_action_type_id = g_enum_register_static ("ValideApplicationAction", values);
	}
	return valide_application_action_type_id;
}


#line 54 "main.vala"
static ValideWindow* valide_application_create_new_window (ValideApplication* self, ValideApplicationAction action, char** uris, int uris_length1, GdkScreen* screen) {
#line 127 "main.c"
	ValideWindow* result;
	GError * _inner_error_;
	ValideWindow* window;
	gboolean opened;
	ValideWindow* _tmp0_;
#line 54 "main.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 135 "main.c"
	_inner_error_ = NULL;
	window = NULL;
	opened = FALSE;
	_tmp0_ = NULL;
#line 60 "main.vala"
	window = (_tmp0_ = g_object_ref_sink (valide_window_new ()), (window == NULL) ? NULL : (window = (g_object_unref (window), NULL)), _tmp0_);
#line 61 "main.vala"
	switch (action) {
#line 144 "main.c"
		case VALIDE_APPLICATION_ACTION_ACTIVATE:
		{
#line 64 "main.vala"
			if (screen != NULL) {
#line 66 "main.vala"
				gtk_window_set_screen ((GtkWindow*) window, screen);
#line 151 "main.c"
			}
#line 68 "main.vala"
			gtk_window_present ((GtkWindow*) window);
#line 69 "main.vala"
			break;
#line 157 "main.c"
		}
		case VALIDE_APPLICATION_ACTION_NEW:
		{
			ValideiDocument* _tmp1_;
#line 499 "valide-0.0.vapi"
			_tmp1_ = NULL;
#line 71 "main.vala"
			_tmp1_ = valide_document_manager_create (valide_window_get_documents (window), NULL, &_inner_error_);
#line 166 "main.c"
			if (_inner_error_ != NULL) {
				(window == NULL) ? NULL : (window = (g_object_unref (window), NULL));
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return NULL;
			}
			(_tmp1_ == NULL) ? NULL : (_tmp1_ = (g_object_unref (_tmp1_), NULL));
#line 72 "main.vala"
			opened = TRUE;
#line 73 "main.vala"
			break;
#line 178 "main.c"
		}
		case VALIDE_APPLICATION_ACTION_OPEN:
		{
#line 75 "main.vala"
			if (uris != NULL) {
#line 184 "main.c"
				{
					gint i;
					i = 0;
					{
						gboolean _tmp2_;
						_tmp2_ = TRUE;
#line 77 "main.vala"
						while (TRUE) {
#line 77 "main.vala"
							if (!_tmp2_) {
#line 77 "main.vala"
								i++;
#line 197 "main.c"
							}
#line 77 "main.vala"
							_tmp2_ = FALSE;
#line 77 "main.vala"
							if (!(uris[i] != NULL)) {
#line 77 "main.vala"
								break;
#line 205 "main.c"
							}
							{
#line 81 "main.vala"
								if (valide_project_manager_accept_file (valide_window_get_projects (window), uris[i])) {
#line 83 "main.vala"
									valide_project_manager_open (valide_window_get_projects (window), uris[i], &_inner_error_);
#line 212 "main.c"
									if (_inner_error_ != NULL) {
										goto __catch0_g_error;
										goto __finally0;
									}
								} else {
									ValideiDocument* _tmp3_;
#line 499 "valide-0.0.vapi"
									_tmp3_ = NULL;
#line 87 "main.vala"
									_tmp3_ = valide_document_manager_create (valide_window_get_documents (window), uris[i], &_inner_error_);
#line 223 "main.c"
									if (_inner_error_ != NULL) {
										goto __catch0_g_error;
										goto __finally0;
									}
									(_tmp3_ == NULL) ? NULL : (_tmp3_ = (g_object_unref (_tmp3_), NULL));
								}
#line 89 "main.vala"
								opened = TRUE;
#line 232 "main.c"
							}
							goto __finally0;
							__catch0_g_error:
							{
								GError * e;
								e = _inner_error_;
								_inner_error_ = NULL;
								{
#line 93 "main.vala"
									g_debug (e->message);
#line 243 "main.c"
									(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
								}
							}
							__finally0:
							if (_inner_error_ != NULL) {
								(window == NULL) ? NULL : (window = (g_object_unref (window), NULL));
								g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
								g_clear_error (&_inner_error_);
								return NULL;
							}
						}
					}
				}
			}
#line 97 "main.vala"
			break;
#line 260 "main.c"
		}
	}
#line 100 "main.vala"
	valide_window_run (window);
#line 102 "main.vala"
	if (opened) {
#line 104 "main.vala"
		valide_window_hide_start_page (window);
#line 269 "main.c"
	}
	result = window;
	return result;
}


static void _lambda0_ (GtkObject* s, ValideApplication* self) {
	char* ws_s;
	char* _tmp1_;
	GtkObject* _tmp0_;
	g_return_if_fail (s != NULL);
	ws_s = NULL;
	_tmp1_ = NULL;
#line 157 "main.vala"
	_tmp0_ = NULL;
#line 157 "main.vala"
	ws_s = (_tmp1_ = g_strdup_printf ("%u", (_tmp0_ = s, VALIDE_IS_WINDOW (_tmp0_) ? ((ValideWindow*) _tmp0_) : NULL)->workspace), ws_s = (g_free (ws_s), NULL), _tmp1_);
#line 158 "main.vala"
	g_hash_table_remove (self->priv->windows, ws_s);
#line 159 "main.vala"
	if (g_hash_table_size (self->priv->windows) == 0) {
#line 161 "main.vala"
		gtk_main_quit ();
#line 293 "main.c"
	}
	ws_s = (g_free (ws_s), NULL);
}


static void __lambda0__gtk_object_destroy (GtkObject* _sender, gpointer self) {
	_lambda0_ (_sender, self);
}


#line 110 "main.vala"
static UniqueResponse valide_application_message (ValideApplication* self, UniqueApp* sender, gint command, UniqueMessageData* data, guint time_) {
#line 306 "main.c"
	UniqueResponse result;
	char* ws;
	ValideWindow* window;
	guint workspace;
	UniqueResponse res;
	char* _tmp0_;
	ValideWindow* _tmp2_;
	ValideWindow* _tmp1_;
#line 110 "main.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 110 "main.vala"
	g_return_val_if_fail (sender != NULL, 0);
#line 110 "main.vala"
	g_return_val_if_fail (data != NULL, 0);
#line 321 "main.c"
	ws = NULL;
	window = NULL;
	workspace = 0U;
	res = UNIQUE_RESPONSE_INVALID;
#line 118 "main.vala"
	workspace = unique_message_data_get_workspace (data);
#line 328 "main.c"
	/* The first instance doesn't set the workspace */
#line 120 "main.vala"
	if (workspace == 0) {
#line 122 "main.vala"
		workspace = unique_backend_get_workspace (unique_backend_create ());
#line 334 "main.c"
	}
	_tmp0_ = NULL;
#line 124 "main.vala"
	ws = (_tmp0_ = g_strdup_printf ("%u", workspace), ws = (g_free (ws), NULL), _tmp0_);
#line 339 "main.c"
	_tmp2_ = NULL;
#line 125 "main.vala"
	_tmp1_ = NULL;
#line 125 "main.vala"
	window = (_tmp2_ = (_tmp1_ = (ValideWindow*) g_hash_table_lookup (self->priv->windows, ws), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (window == NULL) ? NULL : (window = (g_object_unref (window), NULL)), _tmp2_);
#line 126 "main.vala"
	if (window == NULL) {
#line 347 "main.c"
		ValideApplicationAction action;
		char** _tmp3_;
		gint uris_size;
		gint uris_length1;
		char** uris;
		action = 0;
#line 129 "main.vala"
		_tmp3_ = NULL;
#line 356 "main.c"
		uris = (_tmp3_ = NULL, uris_length1 = 0, uris_size = uris_length1, _tmp3_);
#line 131 "main.vala"
		switch (command) {
#line 360 "main.c"
			case UNIQUE_ACTIVATE:
			{
#line 134 "main.vala"
				action = VALIDE_APPLICATION_ACTION_ACTIVATE;
#line 135 "main.vala"
				res = UNIQUE_RESPONSE_OK;
#line 136 "main.vala"
				break;
#line 369 "main.c"
			}
			case UNIQUE_NEW:
			{
#line 138 "main.vala"
				action = VALIDE_APPLICATION_ACTION_NEW;
#line 139 "main.vala"
				res = UNIQUE_RESPONSE_OK;
#line 140 "main.vala"
				break;
#line 379 "main.c"
			}
			case UNIQUE_OPEN:
			{
				char** _tmp4_;
#line 142 "main.vala"
				action = VALIDE_APPLICATION_ACTION_ACTIVATE;
#line 386 "main.c"
				_tmp4_ = NULL;
#line 143 "main.vala"
				uris = (_tmp4_ = unique_message_data_get_uris (data), uris = (_vala_array_free (uris, uris_length1, (GDestroyNotify) g_free), NULL), uris_length1 = -1, uris_size = uris_length1, _tmp4_);
#line 144 "main.vala"
				res = UNIQUE_RESPONSE_OK;
#line 145 "main.vala"
				break;
#line 394 "main.c"
			}
			default:
			{
#line 147 "main.vala"
				action = VALIDE_APPLICATION_ACTION_INVALIDE;
#line 148 "main.vala"
				break;
#line 402 "main.c"
			}
		}
#line 151 "main.vala"
		if (action != VALIDE_APPLICATION_ACTION_INVALIDE) {
#line 407 "main.c"
			ValideWindow* _tmp5_;
			ValideWindow* _tmp7_;
			const char* _tmp6_;
			_tmp5_ = NULL;
#line 153 "main.vala"
			window = (_tmp5_ = valide_application_create_new_window (self, action, uris, uris_length1, unique_message_data_get_screen (data)), (window == NULL) ? NULL : (window = (g_object_unref (window), NULL)), _tmp5_);
#line 154 "main.vala"
			g_signal_connect ((GtkObject*) window, "destroy", (GCallback) __lambda0__gtk_object_destroy, self);
#line 164 "main.vala"
			_tmp7_ = NULL;
#line 164 "main.vala"
			_tmp6_ = NULL;
#line 164 "main.vala"
			g_hash_table_insert (self->priv->windows, (_tmp6_ = ws, (_tmp6_ == NULL) ? NULL : g_strdup (_tmp6_)), (_tmp7_ = window, (_tmp7_ == NULL) ? NULL : g_object_ref (_tmp7_)));
#line 422 "main.c"
		}
		uris = (_vala_array_free (uris, uris_length1, (GDestroyNotify) g_free), NULL);
	}
	result = res;
	ws = (g_free (ws), NULL);
	(window == NULL) ? NULL : (window = (g_object_unref (window), NULL));
	return result;
}


static void _vala_array_add1 (char*** array, int* length, int* size, char* value) {
	if ((*length) == (*size)) {
		*size = (*size) ? (2 * (*size)) : 4;
		*array = g_renew (char*, *array, (*size) + 1);
	}
	(*array)[(*length)++] = value;
	(*array)[*length] = NULL;
}


#line 171 "main.vala"
ValideApplication* valide_application_construct (GType object_type, char** args, int args_length1) {
#line 445 "main.c"
	GError * _inner_error_;
	ValideApplication * self;
	GOptionContext* context;
	GHashTable* _tmp0_;
	GOptionContext* _tmp1_;
	_inner_error_ = NULL;
	self = g_object_newv (object_type, 0, NULL);
	context = NULL;
#line 175 "main.vala"
	gtk_init (&args_length1, &args);
#line 176 "main.vala"
	bindtextdomain (GETTEXT_PACKAGE, LOCALE_DIR);
#line 177 "main.vala"
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
#line 460 "main.c"
	_tmp0_ = NULL;
#line 179 "main.vala"
	self->priv->windows = (_tmp0_ = g_hash_table_new (g_str_hash, g_str_equal), (self->priv->windows == NULL) ? NULL : (self->priv->windows = (g_hash_table_unref (self->priv->windows), NULL)), _tmp0_);
#line 464 "main.c"
	_tmp1_ = NULL;
#line 182 "main.vala"
	context = (_tmp1_ = g_option_context_new (_ ("- Integrated Development Environment for Vala")), (context == NULL) ? NULL : (context = (g_option_context_free (context), NULL)), _tmp1_);
#line 183 "main.vala"
	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
#line 184 "main.vala"
	g_option_context_add_group (context, gtk_get_option_group (FALSE));
#line 472 "main.c"
	{
#line 188 "main.vala"
		g_option_context_parse (context, &args_length1, &args, &_inner_error_);
#line 476 "main.c"
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_OPTION_ERROR) {
				goto __catch1_g_option_error;
			}
			goto __finally1;
		}
#line 189 "main.vala"
		if (remaining_args != NULL) {
#line 485 "main.c"
			{
				gint i;
				i = 0;
				{
					gboolean _tmp2_;
					_tmp2_ = TRUE;
#line 191 "main.vala"
					while (TRUE) {
#line 494 "main.c"
						const char* _tmp3_;
#line 191 "main.vala"
						if (!_tmp2_) {
#line 191 "main.vala"
							i++;
#line 500 "main.c"
						}
#line 191 "main.vala"
						_tmp2_ = FALSE;
#line 191 "main.vala"
						if (!(remaining_args[i] != NULL)) {
#line 191 "main.vala"
							break;
#line 508 "main.c"
						}
#line 193 "main.vala"
						_tmp3_ = NULL;
#line 193 "main.vala"
						_vala_array_add1 (&self->priv->file_list, &self->priv->file_list_length1, &self->priv->file_list_size, (_tmp3_ = remaining_args[i], (_tmp3_ == NULL) ? NULL : g_strdup (_tmp3_)));
#line 514 "main.c"
					}
				}
			}
		}
#line 196 "main.vala"
		self->priv->show_version = self->priv->show_version;
#line 197 "main.vala"
		self->priv->new_document = self->priv->new_document;
#line 523 "main.c"
	}
	goto __finally1;
	__catch1_g_option_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
#line 201 "main.vala"
			g_print (_ ("%s\nRun '%s --help' to see a full list of available command line options.\n"), e->message, args[0]);
#line 534 "main.c"
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally1:
	if (_inner_error_ != NULL) {
		(context == NULL) ? NULL : (context = (g_option_context_free (context), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
	(context == NULL) ? NULL : (context = (g_option_context_free (context), NULL));
	return self;
}


#line 171 "main.vala"
ValideApplication* valide_application_new (char** args, int args_length1) {
#line 171 "main.vala"
	return valide_application_construct (VALIDE_TYPE_APPLICATION, args, args_length1);
#line 554 "main.c"
}


#line 110 "main.vala"
static UniqueResponse _valide_application_message_unique_app_message_received (UniqueApp* _sender, gint command, UniqueMessageData* message_data, guint time_, gpointer self) {
#line 560 "main.c"
	return valide_application_message (self, _sender, command, message_data, time_);
}


#line 240 "main.vala"
gint valide_application_run (ValideApplication* self) {
#line 567 "main.c"
	gint result;
	UniqueApp* app;
#line 240 "main.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 572 "main.c"
	app = NULL;
#line 244 "main.vala"
	if (self->priv->show_version) {
#line 246 "main.vala"
		g_print ("%s - Version %s\n", g_get_application_name (), VERSION);
#line 578 "main.c"
	} else {
		UniqueApp* _tmp1_;
		gboolean _tmp2_;
#line 251 "main.vala"
		if (DEBUG) {
#line 584 "main.c"
			char* _tmp0_;
			_tmp0_ = NULL;
#line 253 "main.vala"
			valide_application_app_id = (_tmp0_ = g_strconcat (valide_application_app_id, "-debug", NULL), valide_application_app_id = (g_free (valide_application_app_id), NULL), _tmp0_);
#line 589 "main.c"
		}
		_tmp1_ = NULL;
#line 256 "main.vala"
		app = (_tmp1_ = unique_app_new (valide_application_app_id, NULL), (app == NULL) ? NULL : (app = (g_object_unref (app), NULL)), _tmp1_);
#line 257 "main.vala"
		if ((g_object_get (app, "is-running", &_tmp2_, NULL), _tmp2_)) {
#line 596 "main.c"
			UniqueCommand command;
			UniqueMessageData* data;
			UniqueMessageData* _tmp3_;
			command = 0;
			data = NULL;
			_tmp3_ = NULL;
#line 262 "main.vala"
			data = (_tmp3_ = unique_message_data_new (), (data == NULL) ? NULL : (data = (unique_message_data_free (data), NULL)), _tmp3_);
#line 263 "main.vala"
			if (self->priv->new_document) {
#line 265 "main.vala"
				command = UNIQUE_NEW;
#line 609 "main.c"
			} else {
#line 267 "main.vala"
				if (self->priv->file_list != NULL) {
#line 269 "main.vala"
					command = UNIQUE_OPEN;
#line 270 "main.vala"
					unique_message_data_set_uris (data, self->priv->file_list);
#line 617 "main.c"
				} else {
#line 274 "main.vala"
					command = UNIQUE_ACTIVATE;
#line 621 "main.c"
				}
			}
#line 276 "main.vala"
			unique_app_send_message (app, (gint) command, data);
#line 277 "main.vala"
			gdk_notify_startup_complete ();
#line 628 "main.c"
			(data == NULL) ? NULL : (data = (unique_message_data_free (data), NULL));
		} else {
			UniqueCommand command;
			UniqueMessageData* data;
			UniqueMessageData* _tmp4_;
			command = 0;
			data = NULL;
			_tmp4_ = NULL;
#line 284 "main.vala"
			data = (_tmp4_ = unique_message_data_new (), (data == NULL) ? NULL : (data = (unique_message_data_free (data), NULL)), _tmp4_);
#line 285 "main.vala"
			if (self->priv->new_document) {
#line 287 "main.vala"
				command = UNIQUE_NEW;
#line 643 "main.c"
			} else {
#line 289 "main.vala"
				if (self->priv->file_list != NULL) {
#line 291 "main.vala"
					command = UNIQUE_OPEN;
#line 292 "main.vala"
					unique_message_data_set_uris (data, self->priv->file_list);
#line 651 "main.c"
				} else {
#line 296 "main.vala"
					command = UNIQUE_ACTIVATE;
#line 655 "main.c"
				}
			}
#line 298 "main.vala"
			valide_application_message (self, app, (gint) command, data, (guint) 0);
#line 299 "main.vala"
			g_signal_connect_object (app, "message-received", (GCallback) _valide_application_message_unique_app_message_received, self, 0);
#line 300 "main.vala"
			gtk_main ();
#line 664 "main.c"
			(data == NULL) ? NULL : (data = (unique_message_data_free (data), NULL));
		}
	}
	result = 0;
	(app == NULL) ? NULL : (app = (g_object_unref (app), NULL));
	return result;
}


static void valide_application_class_init (ValideApplicationClass * klass) {
	valide_application_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValideApplicationPrivate));
	G_OBJECT_CLASS (klass)->finalize = valide_application_finalize;
	valide_application_app_id = g_strdup ("org.valide.valaide");
}


static void valide_application_instance_init (ValideApplication * self) {
	self->priv = VALIDE_APPLICATION_GET_PRIVATE (self);
}


static void valide_application_finalize (GObject* obj) {
	ValideApplication * self;
	self = VALIDE_APPLICATION (obj);
	self->priv->file_list = (_vala_array_free (self->priv->file_list, self->priv->file_list_length1, (GDestroyNotify) g_free), NULL);
	(self->priv->windows == NULL) ? NULL : (self->priv->windows = (g_hash_table_unref (self->priv->windows), NULL));
	G_OBJECT_CLASS (valide_application_parent_class)->finalize (obj);
}


GType valide_application_get_type (void) {
	static GType valide_application_type_id = 0;
	if (valide_application_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValideApplicationClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) valide_application_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValideApplication), 0, (GInstanceInitFunc) valide_application_instance_init, NULL };
		valide_application_type_id = g_type_register_static (G_TYPE_OBJECT, "ValideApplication", &g_define_type_info, 0);
	}
	return valide_application_type_id;
}


#line 308 "main.vala"
gint _main (char** args, int args_length1) {
#line 708 "main.c"
	gint result;
	ValideApplication* _tmp0_;
	gint _tmp1_;
	_tmp0_ = NULL;
	result = (_tmp1_ = valide_application_run (_tmp0_ = valide_application_new (args, args_length1)), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL)), _tmp1_);
	return result;
}


#line 308 "main.vala"
int main (int argc, char ** argv) {
#line 308 "main.vala"
	g_type_init ();
#line 308 "main.vala"
	return _main (argv, argc);
#line 724 "main.c"
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}




